/*!
********************************************************************************
**
** \file        fapex/abst_error.h
**
** \brief       System error codes.
**
** This file contains the definition of common ERROR codes which can be used by
** most of the drivers. In addition this file contains module bases for each
** FAPEX driver and in-house developed modules.
** The previously mentioned common ERROR codes shall be added to the module
** base if used within a module.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
********************************************************************************
*/

#ifndef FAPEX_ABST_ERROR_H
    #define FAPEX_ABST_ERROR_H

/*
********************************************************************************
** Common error codes (reserved from 0 to -99)
**
** Each error code shall be decreased in steps of 1.
********************************************************************************
*/
    //! Indication of success.
    #define FAPEX_OK                                        0

    //! Bad parameter passed.
    #define FAPEX_ERR_BAD_PARAMETER                        -1
    #define FAPEX_NG                                       FAPEX_ERR_BAD_PARAMETER

    //! Memory allocation failed.
    #define FAPEX_ERR_OUT_OF_MEMORY                        -2

    //! Device already initialised.
    #define FAPEX_ERR_ALREADY_INITIALIZED                  -3

    //! Device not initialised.
    #define FAPEX_ERR_NOT_INITIALIZED                      -4

    //! Feature or function is not available.
    #define FAPEX_ERR_FEATURE_NOT_SUPPORTED                -5

    //! Timeout occured.
    #define FAPEX_ERR_TIMEOUT                              -6

    //! The device is busy, try again later.
    #define FAPEX_ERR_DEVICE_BUSY                          -7

    //! Invalid handle was passed.
    #define FAPEX_ERR_INVALID_HANDLE                       -8

    //! Semaphore could not be created.
    #define FAPEX_ERR_SEMAPHORE_CREATE                     -9

    //! The driver's used version is not supported.
    #define FAPEX_ERR_UNSUPPORTED_VERSION                 -10

    //! Mailqueue coult not be created.
    #define FAPEX_ERR_MAILQUEUE_CREATE                    -11

    //! Device/handle is not open.
    #define FAPEX_ERR_NOT_OPEN                            -12

    //! Device/handle is already open.
    #define FAPEX_ERR_ALREADY_OPEN                        -13

/*
********************************************************************************
**
** FAPEX module specific bases (range from -1,000 to -999,000)
**
** Each module base shall be decreased in steps of 1,000.
**
** The module specific error codes shall range from -100 to -999.
********************************************************************************
*/
    //! ATA module base
    #define FAPEX_ATA_MODULE_BASE                       -1000

    //! AUDEC module base
    #define FAPEX_AUDEC_MODULE_BASE                     -2000

    //! AUOUT module base
    #define FAPEX_AUOUT_MODULE_BASE                     -3000

    //! BOOT module base
    #define FAPEX_BOOT_MODULE_BASE                      -4000

    //! BM module base
    #define FAPEX_BM_MODULE_BASE                        -5000

    //! CLKPWR module base
    #define FAPEX_CLKPWR_MODULE_BASE                    -6000

    //! DESC module base
    #define FAPEX_DESC_MODULE_BASE                      -7000

    //! DMA module base
    #define FAPEX_DMA_MODULE_BASE                       -8000

    //! ETH module base
    #define FAPEX_ETH_MODULE_BASE                       -9000

    //! FLASH module base
    #define FAPEX_FLASH_MODULE_BASE                    -10000

    //! FPC module base
    #define FAPEX_FPC_MODULE_BASE                      -11000

    //! GPIO module base
    #define FAPEX_GPIO_MODULE_BASE                     -12000

    //! GPREG module base
    #define FAPEX_GPREG_MODULE_BASE                    -13000

    //! HDMI module base
    #define FAPEX_HDMI_MODULE_BASE                     -14000

    //! I2C module base
    #define FAPEX_I2C_MODULE_BASE                      -15000

    //! ICC module base
    #define FAPEX_ICC_MODULE_BASE                      -16000

    //! INTR module base
    #define FAPEX_INTR_MODULE_BASE                     -17000

    //! IR module base
    #define FAPEX_IR_MODULE_BASE                       -18000

    //! MMU module base
    #define FAPEX_MMU_MODULE_BASE                      -19000

    //! SSP module base
    #define FAPEX_SSP_MODULE_BASE                      -20000

    //! SYNC module base
    #define FAPEX_SYNC_MODULE_BASE                     -21000

    //! TIMER module base
    #define FAPEX_TIMER_MODULE_BASE                    -22000

    //! TSD module base
    #define FAPEX_TSD_MODULE_BASE                      -23000

    //! TTX module base
    #define FAPEX_TTX_MODULE_BASE                      -24000

    //! UART module base
    #define FAPEX_UART_MODULE_BASE                     -25000

    //! UPI module base
    #define FAPEX_UPI_MODULE_BASE                      -26000

    //! USB module base
    #define FAPEX_USB_MODULE_BASE                      -27000

    //! VIDEC module base
    #define FAPEX_VIDEC_MODULE_BASE                    -28000

    //! VIENC module base
    #define FAPEX_VIENC_MODULE_BASE                    -29000

    //! VISCALE module base
    #define FAPEX_VISCALE_MODULE_BASE                  -30000

    //! BOARDIFC module base
    #define FAPEX_BOARDIFC_MODULE_BASE                  -31000

    //! FAPex common error code offset
    #define FAPEX_RET_ERR_BASE                    -1000000000

#if 0
/*
********************************************************************************
** Other module specific bases (range from -1,000,000 to -10,000,000,000)
**
** Each module base shall be decreased in steps of 100,000.
********************************************************************************
*/
    /*
    ****************************************************************************
    ** RTOS
    ****************************************************************************
    */

    //! RTOS module base
    #define RTOS_SHARED_MODULE_BASE                 -1000000

    //! FAMOS module base
    #define RTOS_FAMOS_MODULE_BASE                  -1100000

    //! STUB module base
    #define RTOS_STUB_MODULE_BASE                   -1200000


    /*
    ****************************************************************************
    ** Subsystems
    ****************************************************************************
    */

    //! FE module base
    #define SUBSYS_FE_MODULE_BASE                   -2000000

    //! FE (CE6353) module base
    #define SUBSYS_FE_CE6353_MODULE_BASE            -2100000

    //! FE (DSF89000) module base
    #define SUBSYS_FE_DSF89000_MODULE_BASE          -2200000

    //! FE (MB86668) module base
    #define SUBSYS_FE_MB86668_MODULE_BASE           -2300000

    //! PAINT module base
    #define SUBSYS_PAINT_MODULE_BASE                -2400000

    //! PAINT (DRAW) module base
    #define SUBSYS_PAINT_DRAW_MODULE_BASE           -2500000

    //! PAINT (FONT) module base
    #define SUBSYS_PAINT_FONT_MODULE_BASE           -2600000

    //! PAINT (OSDHANDLER) module base
    #define SUBSYS_PAINT_OSDHANDLER_MODULE_BASE     -2700000

    //! STORAGE module base
    #define SUBSYS_STORAGE_MODULE_BASE              -2800000

    //! STORAGE (MMCSD) module base
    #define SUBSYS_STORAGE_MMCSD_MODULE_BASE        -2900000

    //! AUDIO (MIXER) module base
    #define SUBSYS_AUDIO_MIXER_MODULE_BASE          -3000000


    /*
    ****************************************************************************
    ** Middleware
    ****************************************************************************
    */

    //! PVR module base
    #define MIDDLEWARE_PVR_MODULE_BASE              -5000000


    /*
    ****************************************************************************
    ** Applications
    ****************************************************************************
    */

    //! SmartGo module base
    #define APPLICATION_SMARTGO_MODULE_BASE        -10000000
#endif

#endif
