/*!
*************************************************************************
**
** \file    fapex/abst_libflash.h
**
** \brief   abstraction library header file (FLASH driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBFLASH_H
#define _FAPEX_ABST_LIBFLASH_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/
typedef struct {
    uint32_t    address;
    uint32_t    byteCount;
    void        *bufferPtr;
}FAPEX_FLASH_RWFuncStrT;

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex BOARDIFC error code base value */
#define FAPEX_FLASH_ERR_BASE             (FAPEX_FLASH_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_FLASH_ERR_BAD_PARAMETER    (FAPEX_FLASH_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_FLASH_ERR_OUT_OF_MEMORY    (FAPEX_FLASH_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*!
*************************************************************************
** \brief ioctl specific command for the BOARDIFC driver
*************************************************************************/
#define FAPEX_IOCCOM_FLASH_OPEN                   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 0,  uint32_t)
//#define FAPEX_IOCCOM_FLASH_SETENABLE             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 1, uint32_t)
//#define FAPEX_IOCCOM_FLASH_SETUPIINDEX          _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 2, uint32_t)
#define FAPEX_IOCCOM_FLASH_ERASE_CHIP             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 3, void *)
#define FAPEX_IOCCOM_FLASH_ERASE_SECTOR           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 4, uint32_t)
#define FAPEX_IOCCOM_FLASH_WRITE_BYTES            _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 5, FAPEX_FLASH_RWFuncStrT)
#define FAPEX_IOCCOM_FLASH_READ_BYTES             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 6, FAPEX_FLASH_RWFuncStrT)
//#define FAPEX_IOCCOM_FLASH_FLUSH                _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 7, uint32_t)
#define FAPEX_IOCCOM_FLASH_GET_DEVICE_SIZE        _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 8, int32_t)
#define FAPEX_IOCCOM_FLASH_GET_DEVICE_ID          _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 9, int32_t)
//#define FAPEX_IOCCOM_FLASH_GET_MANU_FACTURER_ID _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 10, uint32_t)
//#define FAPEX_IOCCOM_FLASH_DEVICE_NAME          _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 11, uint32_t)
//#define FAPEX_IOCCOM_FLASH_MANU_FACTURE_NAME    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 12, uint32_t)
#define FAPEX_IOCCOM_FLASH_GET_SECTOR_COUNT       _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 13, void *)
//#define FAPEX_IOCCOM_FLASH_GET_SECTOR_INDEX     _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 14, uint32_t)
#define FAPEX_IOCCOM_FLASH_GET_SECTOR_BYTES       _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 15, uint32_t)
//#define FAPEX_IOCCOM_FLASH_IS_SECTOR_BYTES      _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 16, void*)
//#define FAPEX_IOCCOM_FLASH_IS_CHIP_EMPTY        _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 17, void*)
//#define FAPEX_IOCCOM_FLASH_IS_SECTOR_PROTECTED  _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 18, void*)
//#define FAPEX_IOCCOM_FLASH_PROTECT_SECTOR       _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_FLASH + 19, void*)

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
}
#endif

FAPEXOPENRET_T FAPEX_FLASH_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_FLASH_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_FLASH_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif

#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBFLASH_H */

