/*!
*************************************************************************
**
** \file    fapex/abst_types.h
**
** \brief   abstraction library types header file
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_TYPES_H
#define _FAPEX_ABST_TYPES_H

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>

#include "fapex/abst_error.h"
#include "fapi/sys_driver.h"

#ifndef _LINUX_FAPI
#include "fapi/sys_services.h"
#include "fapi/drv_timer.h"
#include "rtos/rtos.h"
#else /* !_LINUX_FAPI */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdchar.h>
#include <semaphore.h>
#include <pthread.h>
#include <dirent.h>
#include "fapi/sys_error.h"
#endif /* !_LINUX_FAPI */

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex sys error code base value */

/*! Feature or function is not available. */
#define FAPEX_SYS_ERR_FEATURE_NOT_SUPPORTED (FAPEX_RET_ERR_BASE + FAPEX_ERR_FEATURE_NOT_SUPPORTED)

/*! Invalid status. */
#define FAPEX_SYS_ERR_INVALID_STATUS        (FAPEX_RET_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)

/*
*************************************************************************
** translation macro for system environment
*************************************************************************/
#ifdef _LINUX_FAPI

#define FAPI_NG     (-1)

#ifndef FAPI_SYS_MALLOC
#define FAPI_SYS_MALLOC         malloc
#endif
#ifndef FAPI_SYS_MALLOC_CACHED
#define FAPI_SYS_MALLOC_CACHED  malloc
#endif
#ifndef FAPI_SYS_REALLOC
#define FAPI_SYS_REALLOC        realloc
#endif
#ifndef FAPI_SYS_FREE
#define FAPI_SYS_FREE           free
#endif

#ifndef FAPI_SYS_SLEEP
#define FAPI_SYS_SLEEP(t)       usleep(1000*t)
#endif

#define FAPI_MMU_Free( x, ptr)             FAPI_SYS_FREE( ptr )
#define FAPI_MMU_Malloc( x, bytes)         FAPI_SYS_MALLOC( bytes )
#define FAPI_MMU_Realloc( x, ptr, bytes )  FAPI_SYS_REALLOC( ptr, bytes )

#define FAPI_SYS_CalculateCrc32            FAPEX_SYS_CalculateCrc32

/* definition of filename */
#define FAPEX_SYS_FMAXNAME  256     /* 256 byte name  */
#define FAPEX_SYS_FMAXEXT   256     /* 256 byte extension  */
#define FAPEX_SYS_FMAXPATH  512     /* maximum allowed filename or pathname */

/* dummy define */
#define FAPEX_SYS_FDRIVERINIT  int
#define FAPEX_SYS_FDRIVER      int
#define FAPEX_SYS_FPosStrT     unsigned long
#define FAPEX_SYS_FPARTITION   int

/* attribute file/directory bitpattern definitions */
#define FAPEX_SYS_FATTR_ARC         0x20
#define FAPEX_SYS_FATTR_DIR         0x10
#define FAPEX_SYS_FATTR_VOLUME      0x08
#define FAPEX_SYS_FATTR_SYSTEM      0x04
#define FAPEX_SYS_FATTR_HIDDEN      0x02
#define FAPEX_SYS_FATTR_READONLY    0x01

/*  definitions for ctime  */
#define FAPEX_SYS_FCTIME_SEC_SHIFT       0
#define FAPEX_SYS_FCTIME_SEC_MASK   0x001f  /* 0-30 in 2seconds */
#define FAPEX_SYS_FCTIME_MIN_SHIFT       5
#define FAPEX_SYS_FCTIME_MIN_MASK   0x07e0  /* 0-59  */
#define FAPEX_SYS_FCTIME_HOUR_SHIFT     11
#define FAPEX_SYS_FCTIME_HOUR_MASK  0xf800  /* 0-23 */

/*  definitions for cdate  */
#define FAPEX_SYS_FCDATE_DAY_SHIFT       0
#define FAPEX_SYS_FCDATE_DAY_MASK   0x001f  /* 0-31 */
#define FAPEX_SYS_FCDATE_MONTH_SHIFT     5
#define FAPEX_SYS_FCDATE_MONTH_MASK 0x01e0  /* 1-12 */
#define FAPEX_SYS_FCDATE_YEAR_SHIFT      9
#define FAPEX_SYS_FCDATE_YEAR_MASK  0xfe00  /* 0-119 (1980+value) */

//! Advise the system \b not to suspend the current thread.
#define FAPI_SYS_NO_SUSPEND         0x00000000UL

//! Advise the system to suspend the current thread.
#define FAPI_SYS_SUSPEND            0xFFFFFFFFUL

typedef sem_t*  FAPEX_SYS_SemaphoreT;

#define FAPI_SYS_SemaphoreT    FAPEX_SYS_SemaphoreT

#ifndef FAPI_SYS_CREATE_SEMAPHORE
#define FAPI_SYS_CREATE_SEMAPHORE(value)    FAPEX_SYS_CreateSemaphore(value)
#endif
#ifndef FAPI_SYS_DESTROY_SEMAPHORE
#define FAPI_SYS_DESTROY_SEMAPHORE(sem)     FAPEX_SYS_DestroySemaphore(sem)
#endif
#ifndef FAPI_SYS_GET_SEMAPHORE
#define FAPI_SYS_GET_SEMAPHORE(sem, type)   FAPEX_SYS_GetSemaphore(sem, type)
#endif
#ifndef FAPI_SYS_SET_SEMAPHORE
#define FAPI_SYS_SET_SEMAPHORE(sem, type)   FAPEX_SYS_SetSemaphore(sem, type)
#endif

#ifndef FAPI_SYS_PRINT_MSG
#define FAPI_SYS_PRINT_MSG(fmt,...)  printf(fmt,##__VA_ARGS__)
#endif
#if DEBUG > 0
#ifndef FAPI_SYS_PRINT_DEBUG
#define FAPI_SYS_PRINT_DEBUG(level,fmt,...)             \
            if( DEBUG >= (level) ) {                    \
                FAPI_SYS_PRINT_MSG(fmt,##__VA_ARGS__);  \
            }
#endif
#else
#define FAPI_SYS_PRINT_DEBUG(level,...)
#endif /* (DEBUG > 0) */

#define FAPEX_SYS_PRIORITY_LOWEST   1
#define FAPEX_SYS_PRIORITY_LOW      25
#define FAPEX_SYS_PRIORITY_MEDIUM   50
#define FAPEX_SYS_PRIORITY_HIGH     75
#define FAPEX_SYS_PRIORITY_HIGHEST  99
#else
#define FAPEX_SYS_PRIORITY_LOWEST   RTOS_PRIORITY_LOWEST
#define FAPEX_SYS_PRIORITY_LOW      RTOS_PRIORITY_LOW
#define FAPEX_SYS_PRIORITY_MEDIUM   RTOS_PRIORITY_MEDIUM
#define FAPEX_SYS_PRIORITY_HIGH     RTOS_PRIORITY_HIGH
#define FAPEX_SYS_PRIORITY_HIGHEST  RTOS_PRIORITY_HIGHEST
#endif /* _LINUX_FAPI */

#ifndef FAPI_SYS_PRINT_ERR
#define FAPI_SYS_PRINT_ERR(fmt,err)     \
            if(err == -1) {             \
                err = -errno;           \
            }                           \
            FAPI_SYS_PRINT_MSG(fmt,err);
#endif

#ifndef _LINUX_FAPI
#define FAPEX_TIMER_GetTimeStamp    FAPI_TIMER_GetTimeStamp
#endif

#define FAPEX_TIMER_RESOLUTION_1_USEC   1UL          //!< timer resolution 1 usec
#define FAPEX_TIMER_RESOLUTION_10_USEC  10UL         //!< timer resolution 10 usec
#define FAPEX_TIMER_RESOLUTION_100_USEC 100UL        //!< timer resolution 100 usec
#define FAPEX_TIMER_RESOLUTION_1_MSEC   1000UL       //!< timer resolution 1 msec
#define FAPEX_TIMER_RESOLUTION_10_MSEC  10000UL      //!< timer resolution 10 msec
#define FAPEX_TIMER_RESOLUTION_100_MSEC 100000UL     //!< timer resolution 100 msec
#define FAPEX_TIMER_RESOLUTION_1_SEC    1000000UL    //!< timer resolution 1 sec
#define FAPEX_TIMER_RESOLUTION_1_MIN    60000000UL   //!< timer resolution 1 minute
#define FAPEX_TIMER_RESOLUTION_1_HOUR   3600000000UL //!< timer resolution 1 hour

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

typedef void*       FAPEX_SYS_HandleT;
typedef void*       FAPEX_SYS_MailboxT;
typedef void        (*FAPEX_SYS_ThreadFunctionT)(void*);
typedef void*       FAPEX_SYS_ThreadT;
typedef void*       FAPEX_SYS_MailqueueT;
typedef void*       FAPEX_SYS_EventT;

/* open abstruct function */
typedef int32_t     FAPEXOPENRET_T;
typedef int32_t     FAPEXOPENARG1_T;
typedef int32_t     FAPEXOPENARG2_T;
typedef void*       FAPEXOPENARG3_T;

/* close abstruct function */
typedef int32_t     FAPEXRELRET_T;
typedef int32_t     FAPEXRELARG1_T;

/* read abstruct function */
typedef int32_t     FAPEXREADRET_T;
typedef int32_t     FAPEXREADARG1_T;
typedef void*       FAPEXREADARG2_T;
typedef int32_t     FAPEXREADARG3_T;

/* write abstruct function */
typedef int32_t     FAPEXWRTRET_T;
typedef int32_t     FAPEXWRTARG1_T;
typedef const void* FAPEXWRTARG2_T;
typedef int32_t     FAPEXWRTARG3_T;

/* ioctl abstruct function */
typedef int32_t     FAPEXIOCRET_T;
typedef int32_t     FAPEXIOCARG1_T;
typedef uint32_t    FAPEXIOCARG2_T;
typedef uint32_t    FAPEXIOCARG3_T;

/* mmap/munmap abstruct function */
typedef void*       FAPEXMMPRET_T;
typedef int32_t     FAPEXMUNMPRET_T;
typedef void*       FAPEXMMPARG1_T;
typedef size_t      FAPEXMMPARG2_T;
typedef int32_t     FAPEXMMPARG3_T;
typedef int32_t     FAPEXMMPARG4_T;
typedef int32_t     FAPEXMMPARG5_T;
#ifndef _LINUX_FAPI
typedef int32_t     FAPEXMMPARG6_T;
#else
typedef off_t       FAPEXMMPARG6_T;
#endif /* !_LINUX_FAPI */

#ifndef _LINUX_FAPI
typedef uint32_t    FAPEX_SYS_SLCTSTAT_T;
typedef struct {
    int32_t     tv_sec;
    int32_t     tv_usec;
} FAPEX_SYS_TIMEVAL_T;
#else
typedef fd_set          FAPEX_SYS_SLCTSTAT_T;
typedef struct timeval  FAPEX_SYS_TIMEVAL_T;
#endif /* !_LINUX_FAPI */

/*!
*************************************************************************
** \brief select structure
**
** \sa FAPEX_SYS_SelectInit, FAPEX_SYS_SelectSetID, FAPEX_SYS_SelectClearID,
** FAPEX_SYS_SelectSetParam, FAPEX_SYS_SelectCall, FAPEX_SYS_SelectCheck
*************************************************************************/
typedef struct {
    /*!
        status (read)
    */
    FAPEX_SYS_SLCTSTAT_T *rstat;
    /*!
        status (write)
    */
    FAPEX_SYS_SLCTSTAT_T *wstat;
    /*!
        status (exception)
    */
    FAPEX_SYS_SLCTSTAT_T *exstat;

    /*!
        original status (read)
    */
    FAPEX_SYS_SLCTSTAT_T *rstat_org;
    /*!
        original status (write)
    */
    FAPEX_SYS_SLCTSTAT_T *wstat_org;
    /*!
        original status (exception)
    */
    FAPEX_SYS_SLCTSTAT_T *exstat_org;

    /*!
        maximum id value
    */
    int32_t             maxid;
    /*!
        timeout parameter
    */
    FAPEX_SYS_TIMEVAL_T  *tout;

} FAPEX_SYS_SelectParamStrT;

#ifdef _LINUX_FAPI
/*!
*************************************************************************
** \brief find name structure
*************************************************************************/
typedef struct
{
    char_t  name[FAPEX_SYS_FMAXNAME];   /*!< not supported. file name */
    char_t  ext [FAPEX_SYS_FMAXNAME];   /*!< not supported. file extension */

} FAPEX_SYS_FNameStrT;

/*!
*************************************************************************
** \brief find structure
**
** \sa FAPEX_SYS_FFindFirst, FAPEX_SYS_FFindNext
*************************************************************************/
typedef struct
{
    char_t  filename[FAPEX_SYS_FMAXPATH];   /*!< file name+ext */
    char_t  filepath[FAPEX_SYS_FMAXPATH];   /*!< file path */
    char_t  name    [FAPEX_SYS_FMAXNAME];   /*!< not supported. file name */
    char_t  ext     [FAPEX_SYS_FMAXNAME];   /*!< not supported. file extension */
    uint8_t attr;                           /*!< attribute of the file */

    uint16_t        ctime;      /*!< creation time */
    uint16_t        cdate;      /*!< creation date */
    uint32_t        filesize;   /*!< length of file */

    uint32_t            cluster;    /*!< not supported. current file starting position */
    FAPEX_SYS_FNameStrT findfsname; /*!< not supported. find properties */
    FAPEX_SYS_FPosStrT  pos;        /*!< not supported. position of the current list */
    DIR                 *dir;       /*!< pointer to directry (opendir) */

} FAPEX_SYS_FFindStrT;
#endif /* _LINUX_FAPI */

/*!
*************************************************************************
** wrapper definitions for RTOS/Linux environment
*************************************************************************/
#ifndef _LINUX_FAPI

/* RTOS wrapper definitions */
#define FAPEX_SYS_SUSPEND               RTOS_SUSPEND
#define FAPEX_SYS_NO_SUSPEND            RTOS_NO_SUSPEND
#define FAPEX_SYS_MSG_IS_POINTER        RTOS_MSG_IS_POINTER

#define FAPEX_SYS_SemaphoreT            RTOS_SemaphoreT
#define FAPEX_SYS_CreateSemaphore       RTOS_CreateSemaphore

#define FAPEX_SYS_MailboxT              RTOS_MailboxT
#define FAPEX_SYS_CreateMailbox         RTOS_CreateMailbox

#define FAPEX_SYS_ThreadT               RTOS_ThreadT
#define FAPEX_SYS_ThreadFunctionT       RTOS_ThreadFunctionT
#define FAPEX_SYS_CreateThread          RTOS_CreateThread
#define FAPEX_SYS_GetThread             RTOS_GetThread
#define FAPEX_SYS_GetThreadName         RTOS_GetThreadName
#define FAPEX_SYS_ReportThreadTable     RTOS_ReportThreadTable

#define FAPEX_SYS_ExitApplication       RTOS_ExitApplication
#define FAPEX_SYS_InitServices          RTOS_InitServices
#define FAPEX_SYS_GetIdentity           RTOS_GetIdentity

#define FAPEX_SYS_StopPrintThread       RTOS_StopPrintThread
#define FAPEX_SYS_StartMultitasking     RTOS_StartMultitasking

#define FAPEX_SYS_EnterCriticalSection  RTOS_EnterCriticalSection
#define FAPEX_SYS_EnterCritical         RTOS_EnterCritical
#define FAPEX_SYS_LeaveCritical         RTOS_LeaveCritical

#define FAPEX_SYS_GetLastError          RTOS_GetLastError

#define FAPEX_SYS_FSEEK_SET             FN_SEEK_SET
#define FAPEX_SYS_FSEEK_CUR             FN_SEEK_CUR
#define FAPEX_SYS_FSEEK_END             FN_SEEK_END
#define FAPEX_SYS_FFILE                 FN_FILE
#define FAPEX_SYS_FFind                 FN_FIND
#define FAPEX_SYS_FOpen                 fm_open
#define FAPEX_SYS_FClose                fm_close
#define FAPEX_SYS_FSeek                 fm_seek
#define FAPEX_SYS_FTell                 fm_tell
#define FAPEX_SYS_FGetc                 fm_getc
#define FAPEX_SYS_FPutc                 fm_putc
#define FAPEX_SYS_FRewind               fm_rewind
#define FAPEX_SYS_FEof                  fm_eof
#define FAPEX_SYS_FRead                 fm_read
#define FAPEX_SYS_FWrite                fm_write
#define FAPEX_SYS_FFindFirst            fm_findfirst
#define FAPEX_SYS_FFindNext             fm_findnext

#else /* !_LINUX_FAPI */

/* Linux wrapper definitions */
#define FAPEX_SYS_SUSPEND            FAPI_SYS_SUSPEND
#define FAPEX_SYS_NO_SUSPEND         FAPI_SYS_NO_SUSPEND
#define FAPEX_SYS_MSG_IS_POINTER     0xFFFFFFFFUL //!< the message to transport is a generic pointer only

#define FAPEX_SYS_EnterCritical(status_reg) status_reg=FAPEX_SYS_EnterCriticalSection()
#define FAPEX_SYS_LeaveCritical(status_reg) FAPEX_SYS_LeaveCriticalSection(status_reg)

#define FAPEX_SYS_FSEEK_SET             SEEK_SET
#define FAPEX_SYS_FSEEK_CUR             SEEK_CUR
#define FAPEX_SYS_FSEEK_END             SEEK_END
#define FAPEX_SYS_FFILE                 FILE    //!< refer to FILE structure.
#define FAPEX_SYS_FOpen                 fopen   //!< refer to fopen().
#define FAPEX_SYS_FClose                fclose  //!< refer to fclose().
#define FAPEX_SYS_FSeek                 fseek   //!< refer to fseek().
#define FAPEX_SYS_FTell                 ftell   //!< refer to ftell().
#define FAPEX_SYS_FGetc                 fgetc   //!< refer to fgetc().
#define FAPEX_SYS_FPutc                 fputc   //!< refer to fputc().
#define FAPEX_SYS_FRewind               rewind  //!< refer to rewind().
#define FAPEX_SYS_FEof                  feof    //!< refer to feof().
#define FAPEX_SYS_FRead                 fread   //!< refer to fread().
#define FAPEX_SYS_FWrite                fwrite  //!< refer to fwrite().

#endif /* !_LINUX_FAPI */

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif

// API functions related to SELECT
int32_t     FAPEX_SYS_SelectGetSize( uint32_t mode );
int32_t     FAPEX_SYS_SelectInit( FAPEX_SYS_SelectParamStrT *psel,
                                  void* pbase, uint32_t mode );
int32_t     FAPEX_SYS_SelectSetID( FAPEX_SYS_SelectParamStrT *psel,
                                   int32_t id, uint32_t mode );
int32_t     FAPEX_SYS_SelectClearID( FAPEX_SYS_SelectParamStrT *psel,
                                     int32_t id, uint32_t mode );
int32_t     FAPEX_SYS_SelectSetParam( FAPEX_SYS_SelectParamStrT *psel,
                                      int32_t maxid,
                                      FAPEX_SYS_TIMEVAL_T *pTimeout );
int32_t     FAPEX_SYS_SelectCall( FAPEX_SYS_SelectParamStrT *psel );
int32_t     FAPEX_SYS_SelectCheck( FAPEX_SYS_SelectParamStrT *psel,
                                   int32_t id, uint32_t mode );

// API functions related to SEMAPHORE
#ifdef _LINUX_FAPI
FAPEX_SYS_SemaphoreT    FAPEX_SYS_CreateSemaphore( uint32_t value );
#endif
int32_t                 FAPEX_SYS_DestroySemaphore( FAPEX_SYS_SemaphoreT sem);
int32_t                 FAPEX_SYS_GetSemaphore( FAPEX_SYS_SemaphoreT sem,
                                                int32_t timeout );
int32_t                 FAPEX_SYS_SetSemaphore( FAPEX_SYS_SemaphoreT sem,
                                                int32_t timeout );

// API functions related to THREAD
#ifdef _LINUX_FAPI
FAPEX_SYS_ThreadT FAPEX_SYS_CreateThread( uint32_t* stackBuffer,
                                          uint32_t stackSize,
                                          uint32_t priority,
                                          FAPEX_SYS_ThreadFunctionT function,
                                          void* optArg, void* optData,
                                          const char_t* optName );
#endif
uint32_t          FAPEX_SYS_DestroyThread( FAPEX_SYS_ThreadT threadHandle );
#ifdef _LINUX_FAPI
FAPEX_SYS_ThreadT FAPEX_SYS_GetThread( void );
uint32_t          FAPEX_SYS_GetThreadState( FAPEX_SYS_ThreadT threadHandle );
const char_t*     FAPEX_SYS_GetThreadName( FAPEX_SYS_ThreadT threadHandle );
#endif
uint32_t          FAPEX_SYS_SuspendThread( FAPEX_SYS_ThreadT threadHandle );
uint32_t          FAPEX_SYS_WakeupThread( FAPEX_SYS_ThreadT threadHandle );
int32_t           FAPEX_SYS_WaitEndThread( FAPEX_SYS_ThreadT threadHandle );
void              FAPEX_SYS_WaitEndAllThread( void );

// API functions related to KERNEL
#ifdef _LINUX_FAPI
void            FAPEX_SYS_InitServices( void );
uint32_t        FAPEX_SYS_InitKernel( void );
void            FAPEX_SYS_ExitApplication( int32_t status );
const char_t*   FAPEX_SYS_GetIdentity( void );
void            FAPEX_SYS_StartMultitasking( void );
#endif

// API functions related to ERROR CHECKING
#ifdef _LINUX_FAPI
void        FAPEX_SYS_StartPrintThread( void );
void        FAPEX_SYS_StopPrintThread( void );
int32_t     FAPEX_SYS_GetLastError( void );
#endif

// API functions related to CRITICAL SECTION
#ifdef _LINUX_FAPI
uint32_t    FAPEX_SYS_CriticalSection_Init( void );
uint32_t    FAPEX_SYS_EnterCriticalSection( void );
void        FAPEX_SYS_LeaveCriticalSection( uint32_t cpuStatus );
#endif

// API functions related to MAILBOX
#ifdef _LINUX_FAPI
FAPEX_SYS_MailboxT  FAPEX_SYS_CreateMailbox( uint32_t messageBytes);
#endif
uint32_t            FAPEX_SYS_DestroyMailbox(FAPEX_SYS_MailboxT mailboxHandle);
uint32_t            FAPEX_SYS_SetMailbox(FAPEX_SYS_MailboxT mailboxHandle,
                                         void* messagePtr,
                                         uint32_t msecsTimeout);
uint32_t            FAPEX_SYS_GetMailbox(FAPEX_SYS_MailboxT mailboxHandle,
                                         void* resultBuffer,
                                         uint32_t msecsTimeout);

// API functions related to MAILQUEUE
FAPEX_SYS_MailqueueT FAPEX_SYS_CreateMailqueue( uint32_t queueElements,
                                                uint32_t elementBytes );
uint32_t             FAPEX_SYS_DestroyMailqueue( FAPEX_SYS_MailqueueT mailqueuePtr );
uint32_t             FAPEX_SYS_GetMailqueue( FAPEX_SYS_MailqueueT mailqueueHandle,
                                             void* resultBuffer,
                                             uint32_t msecsTimeout );
uint32_t             FAPEX_SYS_SetMailqueue( FAPEX_SYS_MailqueueT mailqueueHandle,
                                             void* messagePtr, uint32_t toFront,
                                             uint32_t msecsTimeout );

// API functions related to File System
#ifdef _LINUX_FAPI
int32_t     FAPEX_SYS_FFindFirst( const char *filename, FAPEX_SYS_FFindStrT *find );
int32_t     FAPEX_SYS_FFindNext( FAPEX_SYS_FFindStrT *find );
long        FAPEX_SYS_FFileLength(const char *filename);
#endif

// other API functions
#ifdef _LINUX_FAPI
int32_t     FAPEX_SYS_KEYIN( char_t *keycode, int32_t *length );

uint32_t    FAPEX_SYS_CalculateCrc32(const uint8_t* dataPtr,
                                     const uint32_t dataLength,
                                     const uint32_t* const crcTable);
uint64_t    FAPEX_TIMER_GetTimeStamp(uint32_t resolution);
#endif /* _LINUX_FAPI */

#ifdef __cplusplus
}
#endif
#endif /* !_FAPEX_ABST_TYPES_H */

