//#define FACS_DEBUG

#include "facs.h"
#include <errno.h>

#ifdef FACS_DEBUG
char_t _func_name[256];
#endif

#ifdef FACS_TESTMODE

void *
FACS_MALLOC( size_t size )
{
    return malloc( size );
}

void
FACS_FREE( void *ptr )
{
    free( ptr );
}

#endif



void
facs_set_lasterror( FACS_CONTROL *facs_hdl, int32_t code )
{
    facs_hdl->lasterror = code;
}

uint32_t
facs_sector_to_address( FACS_CONTROL *facs_hdl, int32_t sector_no )
{
    return ( facs_hdl->top_addr + sector_no * facs_hdl->sector_size );
}

int32_t
facs_address_to_sector( FACS_CONTROL *facs_hdl, uint32_t addr )
{
    int32_t sector_no;
    int32_t flag = 0;
    
    if ( addr < facs_hdl->top_addr || facs_hdl->end_addr < addr ) return -1;
    
    for( sector_no=0; sector_no < facs_hdl->sector_maxnum; sector_no++ )
    {
        if ( addr < facs_sector_to_address( facs_hdl, sector_no + 1) )
        {
            flag = 1;
            break;
        }
    }
    
    if ( flag != 1 )    return facs_hdl->sector_maxnum;

    return sector_no;
}


char_t *
facs_createfilename( FACS_CONTROL *facs_hdl, int32_t sector_no )
{
    char_t    work_sector_no[10];
    
    strcpy( facs_hdl->file_name, facs_hdl->base_fname );
    sprintf( work_sector_no, "%05d", sector_no );
    strcat( facs_hdl->file_name, work_sector_no );
    strcat( facs_hdl->file_name, facs_hdl->ext_fname );
    
    return facs_hdl->file_name;
}

int32_t
facs_is_addressrange( FACS_CONTROL *facs_hdl, uint32_t addr )
{
    if ( addr < facs_hdl->top_addr || facs_hdl->end_addr < addr )
    {
        return FACS_NG;
    }
    else
    {
        return FACS_OK;
    }
}
int32_t
facs_is_sectorrange( FACS_CONTROL *facs_hdl, int32_t sector_no )
{
    if ( sector_no < 0 || facs_hdl->sector_maxnum <= sector_no )
    {
        return FACS_NG;
    }
    else
    {
        return FACS_OK;
    }
    
}

uint32_t
facs_get_sectorbyte( FACS_CONTROL *facs_hdl, int32_t sector_no )
{
    /* to do ... */
    if ( sector_no < 0 || facs_hdl->sector_maxnum <= sector_no ) return 0;
    
    return facs_hdl->sector_size;
}

uint32_t
facs_get_vrflash_size( FACS_CONTROL *facs_hdl )
{
    int32_t i;
    uint32_t byte = 0;
    
    for(i=0;i<facs_hdl->sector_maxnum;i++)
    {
        byte += facs_get_sectorbyte( facs_hdl, i );
    }
    
    return byte;
}
int32_t
facs_get_sectorcount( FACS_CONTROL *facs_hdl )
{
    return facs_hdl->sector_maxnum;
}

FACS_CONTROL *  
facs_open( FACS_DEVICE_INFO *paramD )
{
    FACS_CONTROL    *facs_hdl;

    facs_hdl =  (FACS_CONTROL *)FACS_MALLOC( sizeof(FACS_CONTROL) );
    if ( facs_hdl == NULL )
    {
        return facs_hdl;
    }
    
    facs_hdl->lasterror         = 0;
    
    if ( paramD == NULL )
    {
        strcpy( facs_hdl->base_fname, FACS_BASE_FNAME );
        strcpy( facs_hdl->ext_fname, FACS_EXT_FNAME );
        strcpy( facs_hdl->deviceID,  FACS_DEVICE_ID );
        facs_hdl->sector_maxnum = FACS_SECTOR_MAXNUM;
        facs_hdl->sector_size   = FACS_SECTOR_SIZE;
        facs_hdl->max_size      = facs_get_vrflash_size( facs_hdl );
        facs_hdl->top_addr      = FACS_TOP_ADDR;    /* address of sector 0 */
        facs_hdl->end_addr      = facs_hdl->top_addr + facs_hdl->max_size - 1;
        facs_hdl->defVal        = FACS_DEFAULT_VAL;
    }
    else
    {
        strcpy( facs_hdl->base_fname, paramD->base_fname );
        strcpy( facs_hdl->ext_fname, paramD->ext_fname );
        strcpy( facs_hdl->deviceID,  paramD->deviceID );
        facs_hdl->sector_maxnum = paramD->sector_maxnum;
        facs_hdl->sector_size   = paramD->sector_size;
        facs_hdl->max_size      = facs_get_vrflash_size( facs_hdl );
        facs_hdl->top_addr      = paramD->top_addr;    /* address of sector 0 */
        facs_hdl->end_addr      = facs_hdl->top_addr + facs_hdl->max_size - 1;
        facs_hdl->defVal        = paramD->defVal;
    }
    facs_hdl->cur_sector_no     = FACS_NO_CACHE;

    facs_hdl->buff          = (char_t *)FACS_MALLOC( facs_hdl->sector_size ); /* to do need get 1sector max size */
    if ( facs_hdl->buff == NULL )
    {
        FACS_FREE( facs_hdl );
        return (FACS_CONTROL *)0;
    }
    facs_hdl->fp = NULL;
    
    return facs_hdl; 

}

int32_t
facs_close( FACS_CONTROL *facs_hdl )
{
    FACS_FREE( facs_hdl->buff );
    FACS_FREE( facs_hdl );
    
    return FACS_OK;
}

int32_t
facs_cachesectordata( FACS_CONTROL *facs_hdl, int32_t sector_no )
{
    char_t    *filename;
#ifdef FACS_DEBUG
    strcpy( _func_name, "facs_cachesectordata" );
#endif
    if ( facs_is_sectorrange( facs_hdl, sector_no ) != FACS_OK )
    {
        facs_set_lasterror( facs_hdl, 1 );
        return FACS_NG;
    }
    
    filename = facs_createfilename( facs_hdl, sector_no );
    
    if ( facs_hdl->fp != NULL )
    {
        facs_set_lasterror( facs_hdl, 2 );
        return FACS_NG;
    }
    
    if (( facs_hdl->fp = fopen( filename, "r+" )) == NULL )
    {
        if (( facs_hdl->fp = fopen( filename, "w+" )) == NULL )
        {
            facs_set_lasterror( facs_hdl, 3 );
            return FACS_NG;
        }
    }

    fseek( facs_hdl->fp, 0, SEEK_END );
    facs_hdl->file_size = ftell( facs_hdl->fp );

#ifdef FACS_DEBUG
    printf("%s() facs_get_sectorbyte()=%d, sector_no=%d\n", _func_name, facs_get_sectorbyte( facs_hdl, sector_no ), sector_no );
#endif
    
    memset( facs_hdl->buff, facs_hdl->defVal, facs_get_sectorbyte( facs_hdl, sector_no ));
    fseek( facs_hdl->fp, 0, SEEK_SET );
    fread( facs_hdl->buff, 1, facs_get_sectorbyte( facs_hdl, sector_no ), facs_hdl->fp );
    facs_hdl->cur_sector_no = sector_no;
    
    fclose( facs_hdl->fp );
    
    facs_hdl->fp = NULL;
    
    return FACS_NG;
}



int32_t
facs_flushsectordata( FACS_CONTROL *facs_hdl, int32_t sector_no )
{
        char_t    *filename;
    
    if ( facs_is_sectorrange( facs_hdl, sector_no ) != FACS_OK )
    {
        facs_set_lasterror( facs_hdl, 4 );
        return FACS_NG;
    }
    
    filename = facs_createfilename( facs_hdl, sector_no );
    
    if ( facs_hdl->fp != NULL )
    {
#ifdef FACS_DEBUG
        printf("facs_hdl->fp = %d\n", facs_hdl->fp );
#endif
        facs_set_lasterror( facs_hdl, 5 );
        return FACS_NG;
    }
    
    if (( facs_hdl->fp = fopen( filename, "r+" )) == NULL )
    {
        if (( facs_hdl->fp = fopen( filename, "w+" )) == NULL )
        {
            facs_set_lasterror( facs_hdl, 6 );
            return FACS_NG;
        }
    }

    fseek( facs_hdl->fp, 0, SEEK_END );
    facs_hdl->file_size = ftell( facs_hdl->fp );

    fseek( facs_hdl->fp, 0, SEEK_SET );
    fwrite( facs_hdl->buff, 1, facs_get_sectorbyte( facs_hdl, sector_no ), facs_hdl->fp );
    
    fclose( facs_hdl->fp );
    facs_hdl->fp = NULL;

    return FACS_OK;

}

int32_t
facs_erase_sector( FACS_CONTROL *facs_hdl, int32_t sector_no  )
{
    uint32_t work_size;
    
    if ( facs_is_sectorrange( facs_hdl, sector_no ) != FACS_OK )
    {
        return FACS_NG;
    }
    work_size = facs_get_sectorbyte( facs_hdl, sector_no );
    
    memset( facs_hdl->buff, facs_hdl->defVal, work_size );
    
    facs_flushsectordata( facs_hdl, sector_no );
    
    return FACS_OK;
}

int32_t
facs_read_byte( FACS_CONTROL *facs_hdl, uint32_t addr, int32_t byte, char_t *buf )
{
    uint32_t cur_addr;
    uint32_t end_addr;
    uint32_t cur_byte;
    int32_t           work_cur_sector_no;
    uint32_t cur_sector_addr;
    uint32_t cur_sector_byte;
    uint32_t cur_read_size;
    
    if ( byte == 0 ) return FACS_OK;
    
    cur_addr = addr;
    end_addr = addr + byte - 1;
    cur_byte = byte;
    
    if ( facs_is_addressrange( facs_hdl, cur_addr ) != FACS_OK )
    {
        facs_set_lasterror( facs_hdl, 10 );
        return FACS_NG;
    }
    if ( facs_is_addressrange( facs_hdl, end_addr ) != FACS_OK )
    {
        facs_set_lasterror( facs_hdl, 11 );
        return FACS_NG;
    }
    
    for(;;)
    {
        if ( cur_addr > end_addr ) break;
        
        work_cur_sector_no = facs_address_to_sector( facs_hdl, cur_addr );
        if ( facs_hdl->cur_sector_no != work_cur_sector_no )
        {
            facs_cachesectordata( facs_hdl, work_cur_sector_no );
        }
        
        cur_sector_addr = ( cur_addr - facs_sector_to_address( facs_hdl, work_cur_sector_no ) );
        cur_sector_byte = facs_get_sectorbyte( facs_hdl, work_cur_sector_no ) - cur_sector_addr;
        if ( cur_byte < cur_sector_byte )
        {
            memcpy( buf, (facs_hdl->buff + cur_sector_addr), cur_byte );
            cur_read_size = cur_byte;
        }
        else
        {
            memcpy( buf, (facs_hdl->buff + cur_sector_addr), cur_sector_byte );
            cur_read_size = cur_sector_byte;
        }
        
        cur_byte -= cur_read_size;
        buf      += cur_read_size;
        cur_addr += cur_read_size;
    }
    
    return FACS_OK;
}
int32_t
facs_write_byte( FACS_CONTROL *facs_hdl, uint32_t addr, int32_t byte, char_t *buf )
{
    uint32_t cur_addr;
    uint32_t end_addr;
    uint32_t cur_byte;
    uint32_t cur_sector_byte;
    uint32_t cur_written_size;
    int32_t           work_cur_sector_no;
    uint32_t cur_sector_addr;
#ifdef FACS_DEBUG
    strcpy( _func_name, "facs_write_byte");
#endif
    
    if ( byte == 0 ) return FACS_OK;
    
    cur_addr = addr;
    end_addr = addr + byte - 1;
    cur_byte = byte;

#ifdef FACS_DEBUG
    printf("%s() local cur_addr=%d, end_addr=%d\n", _func_name, cur_addr, end_addr );
    printf("%s() facs  top_addr=%d, end_addr=%d\n", _func_name, facs_hdl->top_addr, facs_hdl->end_addr );
#endif

    if ( facs_is_addressrange( facs_hdl, cur_addr ) != FACS_OK )
    {
        facs_set_lasterror( facs_hdl, 20 );
        return FACS_NG;
    }
    if ( facs_is_addressrange( facs_hdl, end_addr ) != FACS_OK )
    {
        facs_set_lasterror( facs_hdl, 21 );
        return FACS_NG;
    }
    
    for(;;)
    {
        if ( cur_addr > end_addr ) break;
        work_cur_sector_no = facs_address_to_sector( facs_hdl, cur_addr );
        if ( facs_hdl->cur_sector_no != work_cur_sector_no )
        {
            facs_cachesectordata( facs_hdl, work_cur_sector_no );
        }
        cur_sector_addr = ( cur_addr - facs_sector_to_address( facs_hdl, work_cur_sector_no ) );
        cur_sector_byte = facs_get_sectorbyte( facs_hdl, work_cur_sector_no ) - cur_sector_addr;
/* to do cur_byte and byte is NG  and raed byte */
        if ( cur_byte < cur_sector_byte )
        {
            memcpy( (facs_hdl->buff + cur_sector_addr), buf, cur_byte );
            cur_written_size = cur_byte;
        }
        else
        {
            memcpy( (facs_hdl->buff + cur_sector_addr), buf, cur_sector_byte );
            cur_written_size = cur_sector_byte;
        }
#ifdef FACS_DEBUG
    printf("%s() local cur_sector_addr=%d, cur_byte=%d, cur_addr=%d, end_addr=%d\n", _func_name, cur_sector_addr, cur_byte, cur_addr, end_addr );
    printf("%s() local cur_sector_byte=%d, work_cur_sector_no=%d, buf=%x    \n", _func_name, cur_sector_byte, work_cur_sector_no, buf);
#endif
        facs_flushsectordata( facs_hdl, work_cur_sector_no );

        cur_byte -= cur_written_size;
        buf      += cur_written_size;
        cur_addr += cur_written_size;
        
    }
    
    return FACS_OK;
}

int32_t
facs_flashFlushBuffer( FACS_CONTROL *facs_hdl )
{
    if ( facs_hdl->cur_sector_no == FACS_NO_CACHE )
    {
        return FACS_NG;
    }
    else
    {
        return facs_flushsectordata( facs_hdl, facs_hdl->cur_sector_no );
    }
}

int32_t
facs_get_lasterror( FACS_CONTROL *facs_hdl )
{
    return facs_hdl->lasterror;
}

