#ifndef __FACS_H
#define __FACS_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//#define FACS_VALTYPE
//#define FACS_TESTMODE

#ifdef FACS_VALTYPE
typedef int int32_t;
typedef unsigned int uint32_t;
typedef char char_t;
#endif /* FACS_VALTYPE */

#define FACS_OK 0
#define FACS_NG 1

#define FACS_NO_CACHE      (-1)

#define FACS_NAME_SIZE     (256)
#define FACS_BASE_FNAME     "./vrflash"
#define FACS_EXT_FNAME      ".dat"
#define FACS_DEVICEID_SIZE (256)
#define FACS_DEVICE_ID     "00000"
//#define FACS_SECTOR_MAXNUM (10)
//#define FACS_SECTOR_SIZE   (128)
#define FACS_SECTOR_MAXNUM (128)
#define FACS_SECTOR_SIZE   (64*1024)
#define FACS_TOP_ADDR      (0x00000000)
#define FACS_DEFAULT_VAL   (0xff)

typedef struct _facs_control
{
    char_t  file_name[FACS_NAME_SIZE];
    char_t  base_fname[FACS_NAME_SIZE];
    char_t  ext_fname[FACS_NAME_SIZE];
    char_t  deviceID[FACS_DEVICEID_SIZE];
    int32_t  sector_maxnum;
    int32_t  sector_size;
    uint32_t top_addr;
    uint32_t end_addr;
    int32_t  cur_sector_no;
    FILE    *fp;
    int32_t  defVal;
    int32_t  file_size;
    int32_t  max_size;
    char_t *buff;
    int32_t  lasterror;
} FACS_CONTROL;

typedef struct _facs_device_info
{
    char_t *base_fname;
    char_t *ext_fname;
    char_t *deviceID;
    int32_t  sector_maxnum;
    int32_t  sector_size;
    int32_t  top_addr;
    int32_t  defVal;
} FACS_DEVICE_INFO;

FACS_CONTROL *facs_open( FACS_DEVICE_INFO *paramD );
int32_t  facs_close( FACS_CONTROL *fac_hdl );
int32_t  facs_erase_sector( FACS_CONTROL *facs_hdl, int32_t sector_no  );
int32_t  facs_read_byte( FACS_CONTROL *facs_hdl, uint32_t addr, int32_t byte, char_t *buf );
int32_t  facs_write_byte( FACS_CONTROL *facs_hdl, uint32_t addr, int32_t byte, char_t *buf );
int32_t  facs_get_lasterror( FACS_CONTROL *facs_hdl );
int32_t  facs_address_to_sector( FACS_CONTROL *facs_hdl, uint32_t addr );
uint32_t facs_sector_to_address( FACS_CONTROL *facs_hdl, int32_t sector_no );
uint32_t facs_get_sectorbyte( FACS_CONTROL *facs_hdl, int32_t sector_no );
uint32_t facs_get_vrflash_size( FACS_CONTROL *facs_hdl );
int32_t  facs_get_sectorcount( FACS_CONTROL *facs_hdl );
int32_t  facs_is_addressrange( FACS_CONTROL *facs_hdl, uint32_t addr );
int32_t  facs_is_sectorrange( FACS_CONTROL *facs_hdl, int32_t sector_no );
int32_t  facs_flashFlushBuffer( FACS_CONTROL *facs_hdl );

#endif /* __FACS_H */

