/*
*****************************************************************************
**
** \file        ./sample_application/fapex_av_demo/src/audio.c
**
** \version     $Id$
**
** \brief      AUDIO Processor.
**
** This file contains the implementation of the AUDIO Processor API. It
** initializes and configures the audio decoder(s) and audio output unit
** including all buffers. Furthermore it provides a simple API to start/stop
** and run the audio decoder.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/
//#define DESCRAMBLE_TEST

#include <fapex/abst_types.h>
#include <fapex/abst_libauout.h>
#include <fapex/abst_libaudec.h>
#include <fapex/abst_libaudec_mpeg.h>
#include <fapex/abst_libaudec_ac3.h>
#include <fapex/abst_libaudec_ddpdcv.h>
#include <fapex/abst_libtsd.h>
#include <fapex/sys_config.h>
#include "audio.h"
#include "tsd.h"
#include "servicelist.h"

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************
#define AUDIO_MAX_VOLUME      0xfe
#define AUDIO_TSD_CHANNEL     8
#define DRV_ID_TSD0_AUDIO_CH  (FAPEXDEV_TSD0_SUB_MINOR_FIRST + AUDIO_TSD_CHANNEL)
#define DRV_ID_AUDEC_0        (FAPEXDEV_AUDEC_MINOR_FIRST)
#define DRV_ID_AUDEC_1        (FAPEXDEV_AUDEC_MINOR_FIRST + 1)
#define DRV_ID_AUDEC_2        (FAPEXDEV_AUDEC_MINOR_FIRST + 2)
#define DRV_ID_AUDEC_3        (FAPEXDEV_AUDEC_MINOR_FIRST + 3)
#define DRV_ID_AUDEC_4        (FAPEXDEV_AUDEC_MINOR_FIRST + 4)
#define DRV_ID_AUOUT_0        (FAPEXDEV_AUOUT_MINOR_FIRST)

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************
static int32_t fd_TSD0_AUDIO  = -1;
static int32_t fd_AUDEC0      = -1;
static int32_t fd_AUDEC_MPEG  = -1;
static int32_t fd_AUDEC_AAC   = -1;
static int32_t fd_AUDEC_AC3   = -1;
static int32_t fd_AUDEC_HEAAC = -1;
static int32_t fd_AUDEC_DDP   = -1;
static int32_t fd_AUOUT0      = -1;

//***************************************************************************
//***************************************************************************
//** External Functions Declaration
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************
extern int32_t fd_TSD[];


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*
*******************************************************************************
** Open TSD0/BM0 channel8.
*******************************************************************************
*/
static int32_t audioTsdOpen(void)
{
    FAPEX_TSD_OpenSubIdParamStrT pidChParam;
    FAPEX_TSD_PesParamStrT       pesParam;
    int32_t result;

    pidChParam.channelNo = AUDIO_TSD_CHANNEL;
    pidChParam.sub_id    = DRV_ID_TSD0_AUDIO_CH;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                              FAPEX_IOCCOM_TSD_OPEN_SUBID,
                              (FAPEXIOCARG3_T)&pidChParam);
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Open Audio TSD channel. %d\n", result);
        return result;
    }
    fd_TSD0_AUDIO = pidChParam.sub_id;

    /* set Audio PES paser */
    pesParam.pid   = 0x1FFF;  /* dummy */
    pesParam.flags = FAPEX_TSD_FLTFLAGS_RM_PES_HEADER | FAPEX_TSD_FLTFLAGS_PTS |
                     FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP | FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE;
    pesParam.bufferSize       = FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES;
    pesParam.bufferThresholdH = pesParam.bufferSize;
    pesParam.bufferThresholdL = 0;
    pesParam.streamType       = FAPEX_TSD_STREAMTYPE_AUDIO;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_TSD_SET_PES_FILTER,
                              (FAPEXIOCARG3_T)&pesParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Set Audio PES filter (code %d).\n", result);
        return result;
    }

    return 0;
}

/*
*******************************************************************************
** Close TSD0/BM0 channel8.
*******************************************************************************
*/
static int32_t audioTsdClose(void)
{
    int32_t result = 0;

    if(fd_TSD0_AUDIO >= 0) {
        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                                  FAPEX_IOCCOM_CLOSE_SUBID,
                                  (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close Audio TSD channel. %d\n", result);
        }
        fd_TSD0_AUDIO = -1;
    }
    return result;
}

/*
*******************************************************************************
** Open Audio Output/Decoder
*******************************************************************************
*/
static int32_t audioOutOpen(void)
{
    FAPEX_AUDEC_OpenParamStrT          audecOpenParam;
    FAPEX_AUOUT_OpenParamStrT          auoutOpenParam;
    FAPEX_AUDEC_OutputParamStrT        audecOutParam;
    FAPEX_AUDEC_CodecSpecificParamStrT codecParam;
    int32_t  result;

    /* open Audio Output */
    auoutOpenParam.enableFlag  = FAPEX_AUOUT_ENABLE_I2S | FAPEX_AUOUT_ENABLE_SPDIF |
                                 FAPEX_AUOUT_ENABLE_HDMI;
    auoutOpenParam.channelFlag = FAPEX_AUOUT_CHANNEL_STEREO;
    fd_AUOUT0 = FAPEX_AUOUT_Open( DRV_ID_AUOUT_0,
                                  O_RDWR,
                                  (FAPEXOPENARG3_T)&auoutOpenParam );
    if(fd_AUOUT0 < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Open AUOUT driver. %d\n", fd_AUOUT0);
        return fd_AUOUT0;
    }

    /* open Audio Decoder */
    //MPEG
    audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
    audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_MPEG;
    fd_AUDEC_MPEG = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audecOpenParam );
    if(fd_AUDEC_MPEG < 0) {
        FAPI_SYS_PRINT_ERR("ERROR: Cannot open MPEG audio decoder (code %d).\n", fd_AUDEC_MPEG);
        return fd_AUDEC_MPEG;
    }

    //HE-AAC
    audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_HEAAC;
    fd_AUDEC_HEAAC = FAPEX_AUDEC_Open( DRV_ID_AUDEC_3,
                                       O_RDWR,
                                       (FAPEXOPENARG3_T)&audecOpenParam );
    if(fd_AUDEC_HEAAC < 0) {
        FAPI_SYS_PRINT_ERR("WARNING: Cannot open HE-AAC audio decoder (code %d).\n", fd_AUDEC_HEAAC);
    }
#if 0
    //AAC-LC
    audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_AAC;
    fd_AUDEC_AAC = FAPEX_AUDEC_Open( DRV_ID_AUDEC_1,
                                     O_RDWR,
                                     (FAPEXOPENARG3_T)&audecOpenParam );
    if(fd_AUDEC_AAC < 0) {
        FAPI_SYS_PRINT_ERR("WARNING: Cannot open AAC-LC audio decoder (code %d).\n", fd_AUDEC_AAC);
    }
#endif
    //DD+
    audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_DDPDCV;
    fd_AUDEC_DDP = FAPEX_AUDEC_Open( DRV_ID_AUDEC_4,
                                     O_RDWR,
                                     (FAPEXOPENARG3_T)&audecOpenParam );
    if(fd_AUDEC_DDP < 0) {
        FAPI_SYS_PRINT_ERR("WARNING: Cannot open DD+ audio decoder converter (code %d).\n", fd_AUDEC_DDP);
    }

    //AC-3
    audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_AC3;
    fd_AUDEC_AC3 = FAPEX_AUDEC_Open( DRV_ID_AUDEC_2,
                                     O_RDWR,
                                     (FAPEXOPENARG3_T)&audecOpenParam );
    if(fd_AUDEC_AC3 < 0) {
        FAPI_SYS_PRINT_ERR("WARNING: Cannot open AC3 audio decoder (code %d).\n", fd_AUDEC_AC3);
    }

    /* set Audio Decoder Output parameter */
    audecOutParam.outputId   = DRV_ID_AUOUT_0;
    audecOutParam.sampleBits = 16;
    result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_MPEG,
                                FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                (FAPEXIOCARG3_T)&audecOutParam);
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR setup MPEG audio decoder (code %d).\n", result);
        return result;
    }
    else {
        FAPEX_AUDEC_MpegParamStrT  mpegParams;

        mpegParams.decoder               = FAPEX_AUDEC_CODEC_TYPE_MPEG;
        mpegParams.muteDuration          = 1200;
        mpegParams.startInsyncFrameCount = 10;

        codecParam.params = &mpegParams;
        codecParam.length = sizeof(FAPEX_AUDEC_MpegParamStrT);

        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_MPEG,
                                    FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS,
                                    (FAPEXIOCARG3_T)&codecParam );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR: MPEG decoder parameter setting failed (ret=%d).\n",
                               result);
        }
    }

    if(fd_AUDEC_HEAAC >= 0) {
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_HEAAC,
                                    FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                    (FAPEXIOCARG3_T)&audecOutParam);
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("WARNING: setup HE-AAC audio decoder (code %d).\n", result);
        }
    }

    if(fd_AUDEC_AAC >= 0) {
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_AAC,
                                    FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                    (FAPEXIOCARG3_T)&audecOutParam);
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("WARNING: setup AAC-LC audio decoder (code %d).\n", result);
        }
    }

    if(fd_AUDEC_DDP >= 0) {
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_DDP,
                                    FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                    (FAPEXIOCARG3_T)&audecOutParam);
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("WARNING: setup DD+ audio decoder converter (code %d).\n",
                               result);
        }
        else {
            FAPEX_AUDEC_DdpdcvParamStrT  ddpdcvParams;

            memset( &ddpdcvParams, 0, sizeof(FAPEX_AUDEC_DdpdcvParamStrT));
            ddpdcvParams.decoder       = FAPEX_AUDEC_CODEC_TYPE_DDPDCV;
            ddpdcvParams.compMode      = 2;
            ddpdcvParams.dynsclh       = 1.0;
            ddpdcvParams.dynscll       = 1.0;
            ddpdcvParams.pcmscl        = 1.0;
            ddpdcvParams.outChannelCfg = -1;

            codecParam.params = &ddpdcvParams;
            codecParam.length = sizeof(FAPEX_AUDEC_DdpdcvParamStrT);

            result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_DDP,
                                        FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS,
                                        (FAPEXIOCARG3_T)&codecParam );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("ERROR: DDPDCV decoder parameter setting failed (ret=%d).\n",
                                   result);
            }
        }
    }

    if(fd_AUDEC_AC3 >= 0) {
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_AC3,
                                    FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                    (FAPEXIOCARG3_T)&audecOutParam);
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("WARNING: setup AC3 audio decoder (code %d).\n", result);
        }
        else {
            FAPEX_AUDEC_Ac3ParamStrT ac3Params;

            memset( &ac3Params, 0, sizeof(FAPEX_AUDEC_Ac3ParamStrT));
            ac3Params.decoder       = FAPEX_AUDEC_CODEC_TYPE_AC3;
            ac3Params.compMode      = 2;
            ac3Params.dynsclh       = 1.0;
            ac3Params.dynscll       = 1.0;
            ac3Params.pcmscl        = 1.0;
            ac3Params.outChannelCfg = -1;

            codecParam.params = &ac3Params;
            codecParam.length = sizeof(FAPEX_AUDEC_Ac3ParamStrT);

            result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_AC3,
                                        FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS,
                                        (FAPEXIOCARG3_T)&codecParam );
            if(result < 0) {
                FAPI_SYS_PRINT_ERR("ERROR: AC3 decoder parameter setting failed (ret=%d).\n",
                                   result);
            }
        }
    }
    return 0;
}

/*
*******************************************************************************
** Close AUDEC and AUOUT.
*******************************************************************************
*/
static int32_t audioOutClose(void)
{
    int32_t result = 0;

    if(fd_AUOUT0 >= 0) {
        result = FAPEX_AUOUT_Close( fd_AUOUT0 );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close Audio Output (code %d).\n", result);
            return result;
        }
        fd_AUOUT0 = -1;
    }

    fd_AUDEC0 = -1;

    if(fd_AUDEC_MPEG >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_MPEG );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close MPEG audio decoder (code %d).\n", result);
            return result;
        }
        fd_AUDEC_MPEG = -1;
    }

    if(fd_AUDEC_AAC >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_AAC );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close AAC-LC audio decoder (code %d).\n", result);
            return result;
        }
        fd_AUDEC_AAC = -1;
    }

    if(fd_AUDEC_AC3 >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_AC3 );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close AC3 audio decoder (code %d).\n", result);
            return result;
        }
        fd_AUDEC_AC3 = -1;
    }

    if(fd_AUDEC_HEAAC >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_HEAAC );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close HE-AAC audio decoder (code %d).\n", result);
            return result;
        }
        fd_AUDEC_HEAAC = -1;
    }

    if(fd_AUDEC_DDP >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_DDP );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close DD+ audio decoder converter (code %d).\n", result);
            return result;
        }
        fd_AUDEC_DDP = -1;
    }
    return result;
}

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** Open AUDIO Processor.
**
** This function opens the audio decoder, configures the buffer manager
** and setup the TSD.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t AUDIO_Open(void)
{
    int32_t result;

    result = audioTsdOpen();

    if(result == 0) {
        result = audioOutOpen();

        if(result < 0) {
            audioTsdClose();
        }
    }
    return( result );
}

/*!
*******************************************************************************
**
** Close AUDIO Processor.
**
** This function closes the audio decoder.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t AUDIO_Close(void)
{
    int32_t result;

    result = audioOutClose();

    if(result == 0) {
        result = audioTsdClose();
    }
    return result;
}

/*!
*******************************************************************************
** Starts AUDIO Processor.
**
** This function starts the audio decoding process for the given PID.
**
** \param pid The audio PID to decode.
** \param pcr_flag
** \param audio_stream_type The audio stream type.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t AUDIO_Start(uint32_t pid, uint32_t pcr_flag, uint32_t audio_stream_type)
{
    FAPEX_TSD_PidParamStrT  pidParam;
    int32_t result;

#ifdef DESCRAMBLE_TEST
    FAPEX_TSD_DescKeysParamStrT  descParams;
    memset(&descParams, 0 , sizeof(FAPEX_TSD_DescKeysParamStrT));
    descParams.mode = FAPEX_DESC_DVB_CSA2_DECRYPT;
    descParams.evenKeyLength = 8;

    descParams.evenKey[0] = 0x07;
    descParams.evenKey[1] = 0xE0;
    descParams.evenKey[2] = 0x1B;
    descParams.evenKey[3] = 0x02;
    descParams.evenKey[4] = 0xC9;
    descParams.evenKey[5] = 0xE0;
    descParams.evenKey[6] = 0x45;
    descParams.evenKey[7] = 0xEE;
#endif


    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, AUDIO_Start() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    /* set Audio PES PID */
    pidParam.pid   = pid;
    pidParam.flags = (pcr_flag == 0) ? 0 : FAPEX_TSD_FLTFLAGS_PCR;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                              (FAPEXIOCARG3_T)&pidParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Set Audio Pid (code %d).\n", result);
        return result;
    }

#ifdef DESCRAMBLE_TEST 
         result = FAPEX_TSD_Ioctl(fd_TSD0_AUDIO,
                                    FAPEX_IOCCOM_TSD_SET_KEYS,
                                    (FAPEXIOCARG3_T)&descParams);
         if( result != FAPI_OK)  
             return result;

#endif


    /* start TSD channel */
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_START,
                              (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Start Audio TSD channel (code %d).\n", result);
        return result;
    }

    /* select decoder */
    switch(audio_stream_type) {
        case AUD_AC3_ATSC_A52:
        case USER_DEF_AC_3:
            if(fd_AUDEC_AC3 >= 0) {
                fd_AUDEC0 = fd_AUDEC_AC3;
            }
            else {
                fd_AUDEC0 = fd_AUDEC_DDP;
            }
            break;
        case AUD_AAC_MPEG2:
            if(fd_AUDEC_HEAAC >= 0) {
                fd_AUDEC0 = fd_AUDEC_HEAAC;
            }
            else {
                fd_AUDEC0 = fd_AUDEC_AAC;
            }
            break;
        case AUD_AAC_MPEG4:
            fd_AUDEC0 = fd_AUDEC_HEAAC;
            break;
        case USER_DEF_EAC_3:
            fd_AUDEC0 = fd_AUDEC_DDP;
            break;
        default:
            fd_AUDEC0 = fd_AUDEC_MPEG;
    }

    if ( fd_AUDEC0 < 0 ) {
        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                                  FAPEX_IOCCOM_STOP,
                                  (FAPEXIOCARG3_T)NULL );
        FAPI_SYS_PRINT_MSG( "ERROR: audio stream type 0x%x not supported. (%d)\n",
                            audio_stream_type, result );
        return fd_AUDEC0;
    }

    /* start Audio Decoder */
    result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC0,
                                FAPEX_IOCCOM_START,
                                (FAPEXIOCARG3_T)NULL );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Start AUDEC driver (code %d).\n", result);
    }
    else {
        FAPI_SYS_PRINT_MSG( "Audio decoder start with status %d.\n", result );
        AUDIO_Volume(100);
    }
    return result;
}

/*!
*******************************************************************************
** Stops AUDIO Processor.
**
** This function stops the audio decoding process.
*******************************************************************************
*/
int32_t AUDIO_Stop(void)
{
    int32_t result = 0;

    /* stop Audio Output */
    if(fd_AUOUT0 >= 0) {
        result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                    FAPEX_IOCCOM_STOP,
                                    (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Stop AUOUT driver (code %d).\n", result);
            return result;
        }
    }

   /* stop TSD channel */
    if(fd_TSD0_AUDIO >= 0) {
        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                                  FAPEX_IOCCOM_STOP,
                                  (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Stop Audio TSD channel (code %d).\n", result);
        }
    }


    /* stop Audio Decoder */
    if(fd_AUDEC0 >= 0) {
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC0,
                                    FAPEX_IOCCOM_STOP,
                                    (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Stop AUDEC driver (code %d).\n", result);
            return result;
        }
    }
#if 0
    /* stop TSD channel */
    if(fd_TSD0_AUDIO >= 0) {
        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                                  FAPEX_IOCCOM_STOP,
                                  (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Stop Audio TSD channel (code %d).\n", result);
        }
    }
#endif
    return result;
}

/*!
*******************************************************************************
** Mute/Unmute audio output.
*******************************************************************************
*/
void AUDIO_Mute(void)
{
    FAPEX_AUOUT_MuteParamStrT  muteParam;
    static uint32_t toggle = 1;

    muteParam.channelFlag = FAPEX_AUOUT_CHANNEL_ALL;
    if ( toggle ) {
        muteParam.muteFlag = 0;
        FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                           FAPEX_IOCCOM_AUOUT_SET_MUTE,
                           (FAPEXIOCARG3_T)&muteParam );
        toggle = 0;
    } else {
        muteParam.muteFlag = 1;
        FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                           FAPEX_IOCCOM_AUOUT_SET_MUTE,
                           (FAPEXIOCARG3_T)&muteParam );
        toggle = 1;
    }
}

/*!
*******************************************************************************
** Volume up/down audio output.
*******************************************************************************
*/
void AUDIO_Volume(int32_t percent)
{
    FAPEX_AUOUT_VolumeParamStrT  volumeParam;
    int32_t result;

    /* get volume value */
    volumeParam.channelFlag = FAPEX_AUOUT_CHANNEL_LEFT;
    result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                FAPEX_IOCCOM_AUOUT_GET_VOLUME,
                                (FAPEXIOCARG3_T)&volumeParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Get AUOUT volume. %d\n", result);
        return;
    }

    volumeParam.volume += ((AUDIO_MAX_VOLUME * percent) / 100);

    if ( volumeParam.volume < 0 )                volumeParam.volume = 0;
    if ( volumeParam.volume > AUDIO_MAX_VOLUME ) volumeParam.volume = AUDIO_MAX_VOLUME;

    /* set volume value */
    volumeParam.channelFlag = FAPEX_AUOUT_CHANNEL_ALL;
    result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                FAPEX_IOCCOM_AUOUT_SET_VOLUME,
                                (FAPEXIOCARG3_T)&volumeParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Set AUOUT volume. %d\n", result);
        return;
    }
}

/*!
*******************************************************************************
** Set delay for audio decoder.
*******************************************************************************
*/
void AUDIO_SetOutputDelay(int32_t delay)
{
    int32_t result;

    result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                FAPEX_IOCCOM_AUDEC_SET_OUTPUT_DELAY,
                                (FAPEXIOCARG3_T)delay );
    if(result < 0) {
        FAPI_SYS_PRINT_MSG("[AUDIO] Error upon setting delay (code: %d).\n", result);
    }
}

/*!
*******************************************************************************
** \brief   Return buffer size of audio input buffer.
**
** \return  The size of the audio input buffer.
*******************************************************************************
*/
int32_t AUDIO_GetBufferSize(void)
{
    FAPEX_TSD_ChannelInfoStrT  ch_info;
    int32_t  result;

    if((fd_TSD0_AUDIO < 0) || (fd_AUDEC0 < 0)) {
        return 0;
    }

    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO,
                              (FAPEXIOCARG3_T)&ch_info );
    if(result == 0) {
        return ch_info.bufferSize;
    }
    return result;
}

/*!
*******************************************************************************
** \brief   Return current filling level of audio input buffer.
**
** \return  The filling level of the audio input buffer.
*******************************************************************************
*/
int32_t AUDIO_GetBufferLevel(void)
{
    uint32_t buffer_level;
    int32_t  result;

    if((fd_TSD0_AUDIO < 0) || (fd_AUDEC0 < 0)) {
        return 0;
    }

    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                              (FAPEXIOCARG3_T)&buffer_level );
    if(result == 0) {
        return (int32_t)buffer_level;
    }
    return result;
}

/*!
*******************************************************************************
** \brief   Get current AUDEC driver id.
**
** \return  Possible return codes:
**          - Driver id (may be -1)
*******************************************************************************
*/
int32_t AUDIO_GetIdAudec(void)
{
    return fd_AUDEC0;
}

/*!
*******************************************************************************
** \brief   Get current AUOUT driver id.
**
** \return  Possible return codes:
**          - Driver id (may be -1)
*******************************************************************************
*/
int32_t AUDIO_GetIdAuout(void)
{
    return fd_AUOUT0;
}
