/*
*****************************************************************************
**
** \file        ./sample_application/fapex_av_demo/src/osd.c
**
** \version     $Id$
**
** \brief       OSD/CP related function implementations.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

#include <fapex/abst_types.h>
#include <val/val.h>
#include <osdhandler/osdhandler.h>
#include <fontserver/fontserver.h>

#include "osd.h"

#if OSD_USE_FREETYPE == 1
#include <fontserver/fontserver_engine_ft.h>
#include <ftf/ftf.h>
#else
#include <fontserver/fontserver_engine_fbf.h>
#include <fbf/fbf_zwo_bold.h>
#endif // OSD_USE_FREETYPE

#if OSD_DRAW_ROUND_CORNERS
#include <draw/draw.h>
#endif // OSD_DRAW_ROUND_CORNERS

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#define APP_FULL_STRING         0
#define APP_OSD_IDX             5
#define APP_BG_COLOUR           1
#define APP_TXT_COLOUR          2
#define APP_ENCODING            FONTSERVER_ENCODING_8_BIT
#define APP_FONT_INDEX_SMALL    0
#define APP_FONT_INDEX_MEDIUM   1
#define APP_FONT_INDEX_LARGE    2

//******************************************************************************
//******************************************************************************
//** thread related defines and macros
//******************************************************************************
//******************************************************************************

#define APP_FADE_THREAD_STACKSIZE          1024
#define APP_FADE_THREAD_PRIORITY           FAPEX_SYS_PRIORITY_MEDIUM
#define APP_FADE_THREAD_NAME               "app:osd"

//***************************************************************************
//***************************************************************************
//** Local Types
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************

//! Indicates whether the Cursor Plane is currently showing a program.
static uint32_t             displayMode = 0;

//! Indicates whether OSD1 or OSD2 is covering the Video layer.
static uint32_t             displayBlack = 0;

// Resolution of the system mode (will be set up later):
static uint32_t out_h_res = 0;
static uint32_t out_v_res = 0;

// Switch that decides whether to read the source as a frame or field:
static uint32_t readFrame = 0;

// coordinates for the display (OSD/CP with text description):
//static uint32_t display__x = 0;
//static uint32_t display__y = 0;

#if ENABLE_GUI_WINDOW == 0
static FAPI_SYS_HandleT     appFontserverHandle;

// font size
static uint32_t  appFontSize = 0;

// font index
static uint32_t  appFontIdx  = 0;

#if OSD_USE_FREETYPE == 1
static uint32_t  appTtfFontIndex = 0;
#else
static uint32_t  appFbfFontIndexPtr[3] = {0, 0, 0};
#endif // OSD_USE_FREETYPE
#endif

static uint32_t defaultClut[] =
{
    0x00000000, /* fully transparent */
    0x800080FF, /* black */   0x80FF80FF, /* white */
    0xFF0080FF, /* blue  */   0x8000FFFF, /* red   */
    0x00FF00FF, /* green */
    0x00000000, /* transparent in non-global-alpha mode */
    0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000,
    0x00000000,  0x00000000,  0x00000000,  0x00000000
};

static FAPEX_SYS_ThreadT    appFadeThreadHandle;

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

#if ENABLE_GUI_WINDOW == 0
/*!
*****************************************************************************
** \brief OSD Fadeout thread
*****************************************************************************
*/
static void appFadeThreadFunction(void* optArg)
{
    uint32_t  *interval;
    int32_t   result;

    /* hide OSD */
    if(displayMode == 1) {
        interval = (uint32_t *)optArg;

        /* wait for display */
        FAPI_SYS_SLEEP(*interval);

        /**** setup Cursor Plane Fading ****/
        result = OSDHANDLER_PlaneFading(APP_OSD_IDX, OSDHANDLER_PARAMETER_OFF, NULL);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("[OSD] Can't fade out: %d\n", result);
        }

        /* wait for fading */
        FAPI_SYS_SLEEP(*(interval+1));

        /**** stop Cursor Plane Fading ****/
        result = OSDHANDLER_StopLayer(APP_OSD_IDX, NULL);
        if(result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("[OSD] Error while stopping the Cursor Plane. %d\n", result);
        }
        displayMode = 0;
    }
}

/*!
*****************************************************************************
** \brief OSD Fadeout control
*****************************************************************************
*/
static void osdFadeOut( uint32_t milisecDisplay, uint32_t milisecFadeout )
{
    static uint32_t  interval[2];

    interval[0] = milisecDisplay;
    interval[1] = milisecFadeout;

    appFadeThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_FADE_THREAD_STACKSIZE,
            FAPEX_SYS_PRIORITY_MEDIUM,
            appFadeThreadFunction, interval, NULL,
            APP_FADE_THREAD_NAME);
    if(appFadeThreadHandle == NULL) {
        FAPI_SYS_PRINT_MSG( "ERROR:[OSD] creating thread \"%s\" failed\n",
                            APP_FADE_THREAD_NAME );
        exit(1);
    }
    return;
}
#endif

/*!
*****************************************************************************
** \brief stop OSD layer
*****************************************************************************
*/
static void osdStopOSD(void)
{
    int32_t  result;

#if ENABLE_GUI_WINDOW == 0
    result = OSDHANDLER_StopLayer(1, NULL);
    if(result < 0) {
        FAPI_SYS_PRINT_MSG("[OSD] Error while stopping the OSD1. %d\n", result);
    }
#endif

    result = OSDHANDLER_StopLayer(2, NULL);
    if(result < 0) {
        FAPI_SYS_PRINT_MSG("[OSD] Error while stopping the OSD2. %d\n", result);
    }
}

/*
*****************************************************************************
** \brief stop Cursor Plane layer
*****************************************************************************
*/
static void osdStopCP(void)
{
    int32_t  result;

    /* if OSD is already displayed we stop the timer and the current display */
    if(appFadeThreadHandle != NULL) {
        FAPEX_SYS_DestroyThread( appFadeThreadHandle );
        appFadeThreadHandle = NULL;
    }

    result = OSDHANDLER_StopLayer(APP_OSD_IDX, NULL);
    if(result < 0) {
        FAPI_SYS_PRINT_MSG("[OSD] Error while stopping the Cursor Plane. %d\n", result);
    }
}

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************


/*!
*****************************************************************************
** \brief   Initialize the OSDHANDLER and the FONT driver.
**
** \return
**        - #FAPI_OK if initialization was successful
**        - #FAPI_ERR_ALREADY_INITIALIZED
**          - if the OSDHANDLER has already been initialized
**          - if the FONT driver has already been initialized
**        - #FAPI_ERR_NOT_INITIALIZED if the OSDHANDLER (that is needed in
**          order to use the FONT driver) has not been initialized.
**        - #FAPI_ERR_BAD_PARAMETER if the initialization structure for the
**          FONT driver was bad.
*****************************************************************************
*/
int32_t OSD_Open(VAL_HdEncoderT systemType)
{
    OSDHANDLER_InitParamsT  handler_params;
    int32_t                 result              = 0; // return value
    uint32_t                error_code_u        = 0; // unsigned error code
#if ENABLE_GUI_WINDOW == 0
    FONTSERVER_InitParamsT  fontserver_init_params;
    FONTSERVER_OpenParamsT  fontserver_open_params;
    uint32_t                font_engine_data[1] = {0};

    memset(&handler_params, 0, sizeof(OSDHANDLER_InitParamsT));
    memset(&fontserver_init_params, 0, sizeof(FONTSERVER_InitParamsT));
#endif

    // init the OSDHANDLER:
#if ENABLE_GUI_WINDOW == 0
    handler_params.osd1InitParams.inUse          = 1;
#else
    handler_params.osd1InitParams.inUse          = 0;
#endif
    handler_params.osd1InitParams.globalAlpha    = 0xFF;
    handler_params.osd1InitParams.useGlobalAlpha = 1;
    handler_params.osd1InitParams.colourMode     = OSDHANDLER_OSD_MODE_CLUT_8BIT;
    handler_params.osd1InitParams.osdLayout      = OSDHANDLER_OSD_LAYOUT_CbYCrA;
    handler_params.osd1InitParams.defaultClutPtr = defaultClut;

    handler_params.osd2InitParams.inUse          = 1;
    handler_params.osd2InitParams.colourMode     = OSDHANDLER_OSD_MODE_CLUT_4BIT;
    handler_params.osd2InitParams.osdLayout      = OSDHANDLER_OSD_LAYOUT_CbYCrA;
    handler_params.osd2InitParams.defaultClutPtr = defaultClut;
    handler_params.osd2InitParams.useGlobalAlpha = 0;
    handler_params.osd2InitParams.globalAlpha    = 0xFF;

    handler_params.cpInitParams.inUse            = 1;
    handler_params.cpInitParams.colourMode       = OSDHANDLER_CURSOR_MODE_2BIT;
    handler_params.cpInitParams.osdLayout        = OSDHANDLER_OSD_LAYOUT_CbYCrA;
    handler_params.cpInitParams.defaultClutPtr   = defaultClut;
#if OSD_DRAW_ROUND_CORNERS
    handler_params.cpInitParams.useGlobalAlpha   = 0;
    handler_params.cpInitParams.globalAlpha      = 0x00;
#else // OSD_DRAW_ROUND_CORNERS
    handler_params.cpInitParams.useGlobalAlpha   = 1;
    handler_params.cpInitParams.globalAlpha      = 0xFF;
#endif // OSD_DRAW_ROUND_CORNERS

    result = OSDHANDLER_Init(&handler_params, &error_code_u);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while initializing the OSDHANDLER: %d.\n", result);
        return(result);
    }
    if(error_code_u) FAPI_SYS_PRINT_MSG("[OSD] error code: 0x%x\n", error_code_u);

    result = OSDHANDLER_PlaneOrder(OSDHANDLER_PARAMETER_SET, 5, 1, 2, 0, 3, 4, &error_code_u);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error set order: %d\n", result);
    }

    switch(systemType)
    {
        case(VAL_1280x720x2397P):
        case(VAL_1280x720x24P):
        case(VAL_1280x720x25P):
        case(VAL_1280x720x2997P):
        case(VAL_1280x720x30P):
        case(VAL_1280x720x50P):
        case(VAL_1280x720x5994P):
        case(VAL_1280x720x60P):
            out_h_res  = 1280;
            out_v_res  =  720;
            readFrame  = 1;
#if ENABLE_GUI_WINDOW == 0
            appFontSize  = 32;
            appFontIdx   = APP_FONT_INDEX_MEDIUM;
#endif
            //display__x = 140;
            //display__y = 50;
            break;
        case(VAL_720x576x50I):
            out_h_res  =  720;
            out_v_res  =  288;
            readFrame  = 0;
#if ENABLE_GUI_WINDOW == 0
            appFontSize  = 24;
            appFontIdx   = APP_FONT_INDEX_SMALL;
#endif
            //display__x = 80;
            //display__y = 24;
            break;
        case(VAL_720x576x50P):
            out_h_res  =  720;
            out_v_res  =  576;
            readFrame  = 1;
#if ENABLE_GUI_WINDOW == 0
            appFontSize  = 24;
            appFontIdx   = APP_FONT_INDEX_SMALL;
#endif
            //display__x = 80;
            //display__y = 48;
            break;
        case(VAL_720x480x5994I):
            out_h_res  =  720;
            out_v_res  =  280;
            readFrame  = 0;
#if ENABLE_GUI_WINDOW == 0
            appFontSize  = 24;
            appFontIdx   = APP_FONT_INDEX_SMALL;
#endif
            //display__x = 80;
            //display__y = 20;
            break;
        case(VAL_720x480x5994P):
            out_h_res  =  720;
            out_v_res  =  480;
            readFrame  = 1;
#if ENABLE_GUI_WINDOW == 0
            appFontSize  = 24;
            appFontIdx   = APP_FONT_INDEX_SMALL;
#endif
            //display__x = 80;
            //display__y = 40;
            break;
        case(VAL_1920x1080x2397P):
        case(VAL_1920x1080x24P):
        case(VAL_1920x1080x25P):
        case(VAL_1920x1080x2997P):
        case(VAL_1920x1080x30P):
            out_h_res  =  1920;
            out_v_res  =  1080;
            readFrame  = 1;
#if ENABLE_GUI_WINDOW == 0
            appFontSize  = 40;
            appFontIdx   = APP_FONT_INDEX_LARGE;
#endif
            //display__x = 80;
            //display__y = 40;
            break;
        case(VAL_1920x1080x50I):
        case(VAL_1920x1080x5994I):
        case(VAL_1920x1080x60I):
        default:
            out_h_res  = 1920;
            out_v_res  =  540;
            readFrame  = 0;
#if ENABLE_GUI_WINDOW == 0
            appFontSize  = 40;
            appFontIdx   = APP_FONT_INDEX_LARGE;
#endif
            //display__x = 188;
            //display__y = 40;
            break;
    }

    // init the FONTSERVER:

#if ENABLE_GUI_WINDOW == 0
#if OSD_USE_FREETYPE == 1
    font_engine_data[0] = (uint32_t)FONTSERVER_ENGINE_FT_INTERFACE_PTR;
#else
    font_engine_data[0] = (uint32_t)FONTSERVER_ENGINE_FBF_INTERFACE_PTR;
#endif //OSD_USE_FREETYPE
    fontserver_init_params.numEngines      = 1;
    fontserver_init_params.enginesPtr      = font_engine_data;
    FAPI_SYS_PRINT_MSG("[OSD] Initializing FONTSERVER... ");
    result = FONTSERVER_Init(&fontserver_init_params);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("ok\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("error: %d\n", result);
        return(result);
    }

    // register fonts with the FONTSERVER:

#if OSD_USE_FREETYPE == 1
    appTtfFontIndex = FONTSERVER_RegisterFont((uint32_t*)Fujitsu_Zwo_Bold_DATA, \
                                              Fujitsu_Zwo_Bold_SIZE,            \
                                              &result);
    if(result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while registering TTF font: %d\n", result);
    }

#else // OSD_USE_FREETYPE
    appFbfFontIndexPtr[APP_FONT_INDEX_SMALL] = FONTSERVER_RegisterFont  \
                            ((uint32_t*)Fujitsu_Zwo_Bold_FBF_24_DATA,   \
                             Fujitsu_Zwo_Bold_FBF_24_SIZE,              \
                             &result);
    if(result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while registering FBF 24 font: %d\n", result);
    }

    appFbfFontIndexPtr[APP_FONT_INDEX_MEDIUM] = FONTSERVER_RegisterFont \
                            ((uint32_t*)Fujitsu_Zwo_Bold_FBF_32_DATA,   \
                             Fujitsu_Zwo_Bold_FBF_32_SIZE,              \
                             &result);
    if(result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while registering FBF 32 font: %d\n", result);
    }

    appFbfFontIndexPtr[APP_FONT_INDEX_LARGE] = FONTSERVER_RegisterFont  \
                            ((uint32_t*)Fujitsu_Zwo_Bold_FBF_40_DATA,   \
                             Fujitsu_Zwo_Bold_FBF_40_SIZE,              \
                             &result);
    if(result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while registering FBF 40 font: %d\n", result);
    }
#endif // OSD_USE_FREETYPE

    // open a FONTSERVER handle:

    fontserver_open_params.version = FONTSERVER_VERSION;
    FAPI_SYS_PRINT_MSG("Opening FONTSERVER handle... ");
    appFontserverHandle = FONTSERVER_Open(&fontserver_open_params, &result);
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("ok\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("error: %d\n", result);
    }

    // set up FONTSERVER configuration:
    result = 0;
#if OSD_USE_FREETYPE
    result |= FONTSERVER_SetFont(appFontserverHandle, appTtfFontIndex);
#else
    result |= FONTSERVER_SetFont(appFontserverHandle, appFbfFontIndexPtr[appFontIdx]);
#endif // OSD_USE_FREETYPE
    result |= FONTSERVER_SetSize(appFontserverHandle, appFontSize);
    result |= FONTSERVER_SetColour(appFontserverHandle, APP_TXT_COLOUR);
    result |= FONTSERVER_SetLayerId(appFontserverHandle, APP_OSD_IDX);
    result |= FONTSERVER_SetEncoding(appFontserverHandle, APP_ENCODING);
    result |= FONTSERVER_SetStyle(appFontserverHandle, FONTSERVER_STYLE_NORMAL);
    result |= FONTSERVER_SetAntiAliasing(appFontserverHandle, FONTSERVER_ANTI_ALIAS_OFF);
    if(result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("Error while setting up fontserver configuration.\n");
    }

#if OSD_DRAW_ROUND_CORNERS
    FAPI_SYS_PRINT_MSG("[OSD] Initializing DRAW... ");
    result = DRAW_Init();
    if(result == FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("ok\n");
    }
    else
    {
        FAPI_SYS_PRINT_MSG("error: %d\n", result);
        return(result);
    }
#endif // OSD_DRAW_ROUND_CORNERS
#endif
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Shutdown the OSD subsystem.
*****************************************************************************
*/
int32_t OSD_Close(void)
{
    uint32_t  result;

    /* if OSD is already displayed we stop the timer and the current display */
    if(appFadeThreadHandle != NULL) {
        FAPEX_SYS_DestroyThread( appFadeThreadHandle );
        appFadeThreadHandle = NULL;
    }

#if ENABLE_GUI_WINDOW == 0
    FONTSERVER_Exit();
#endif
    result = OSDHANDLER_Exit();
    if(result != 0) {
        FAPI_SYS_PRINT_MSG("[OSD] ERROR close. %d\n", result);
    }
    return 0;
}

/*!
*****************************************************************************
** \brief   Show the program name on the Cursor Plane for a short time.
**
** \param   programInfo the name of the program that will be shown on the
**                      video output.
**
** \return
**        - #FAPI_OK on success
**        - #FAPI_ERR_NOT_INITIALIZED
**          - if the OSDHANDLER is not initialized
**          - if the FONT driver is not initialized
**        - #OSDHANDLER_ERR_NO_OSD_HANDLE if the OSDHANDLER did not allocate
**          the corresponding handle of the CP at initialization.
**        - #OSDHANDLER_ERR_OUT_OF_SIZE
**          if the dimensions of the Cursor Plane would allocate more than
**          16 KiB.
**        - #FAPI_ERR_OUT_OF_MEMORY if there was not enough memory available
**          for allocation in order to drive the CP at the given resolution
**          and in the current colour mode.
**        - #FONT_ERR_OUT_OF_OSD_SIZE
**          - if the given coordinates are out of range of the CP
**          - if the driver ran out of range of the CP layer while printing
**            the characters of the string
*****************************************************************************
*/
int32_t OSD_ShowProgram(uint8_t* programInfo)
{
#if ENABLE_GUI_WINDOW == 0
    OSDHANDLER_StartParamsT start_params;
    OSDHANDLER_LayerParamsT layer_params;
    FONTSERVER_InformationT font_info;
    int32_t                 result          = 0; // return value
    int32_t                 error_code_s    = 0; // signed error code
    int32_t                 string_width    = 0;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, OSD_ShowProgram() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    memset(&start_params, 0, sizeof(start_params));
    memset(&layer_params, 0, sizeof(layer_params));
    memset(&font_info,    0, sizeof(FONTSERVER_InformationT));

    if(programInfo == NULL)
    {
        FAPI_SYS_PRINT_MSG("[OSD] given string is NULL.\n");
        return(FAPI_ERR_BAD_PARAMETER);
    }

    if(displayMode == 1) {
        osdStopCP();
        displayMode = 0;
    }
    if(displayBlack)
    {
        osdStopOSD();
        displayBlack = 0;
    }

    string_width = FONTSERVER_GetStringPixelWidth \
                    (appFontserverHandle, programInfo, APP_FULL_STRING, NULL);
    (void)FONTSERVER_GetFontInfo(appFontserverHandle, &font_info);

    // start the Cursor Plane (only internally; does not display it yet):
    start_params.inputWidth    = (uint32_t)string_width + (font_info.maxCharWidth * 2);
    start_params.inputHeight   = (uint32_t)font_info.maxCharHeight + 30UL;
    start_params.outputWidth   = 0;
    start_params.outputHeight  = 0;
    start_params.xPos          = (out_h_res - start_params.inputWidth) / 2;
    start_params.yPos          = ((out_v_res * 95) / 100) - start_params.inputHeight;
    start_params.frameBased    = readFrame;

    /**** setup Cursor Plane Parameter ****/
    result = OSDHANDLER_StartLayer(APP_OSD_IDX, &start_params, NULL);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while starting the Cursor Plane: %d\n", result);
        return(result);
    }

    /**** get Cursor Plane Parameter ****/
    result = OSDHANDLER_GetLayerParameters(APP_OSD_IDX, &layer_params);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while requesting layer parameters: %d\n", result);
        return(result);
    }

#if OSD_DRAW_ROUND_CORNERS
    result = DRAW_RoundRectangle(APP_OSD_IDX, 0, 0,
                                 layer_params.xRes,
                                 layer_params.yRes,
                                 ((layer_params.xRes > layer_params.yRes) ?
                                 layer_params.yRes : layer_params.xRes) / 4,
                                 APP_BG_COLOUR);
    if(result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while drawing round corners: %d\n", result);
    }
#else // OSD_DRAW_ROUND_CORNERS
    result = OSDHANDLER_SetFillColour(APP_OSD_IDX, APP_BG_COLOUR);
    if(result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while filling plane: %d\n", result);
    }
#endif // OSD_DRAW_ROUND_CORNERS

    // print the string:
    result = FONTSERVER_PrintString(appFontserverHandle,    \
                                    programInfo,            \
                                    APP_FULL_STRING,        \
                                    (layer_params.xRes - (uint32_t)string_width) / 2,
#if OSD_USE_FREETYPE == 1
                              (start_params.inputHeight - 30),
#else
                              12,
#endif // OSD_USE_FREETYPE
                              &error_code_s);
    if (error_code_s != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while printing the string: %d.\n", error_code_s);
        return(result);
    }

    // show the cursor for 5 seconds:
    result = OSDHANDLER_ShowLayer(APP_OSD_IDX, NULL);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while displaying the Cursor Plane: %d\n", result);
        return(result);
    }

    displayMode = 1; // indicate that display is active

    osdFadeOut(5000, 2000);

#endif
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Stop the OSD program display immediately.
**
** \return
**          - #FAPI_OK on success.
**          - #OSDHANDLER_ERR_NOT_INITIALIZED if the OSDHANDLER is not
**              initialized.
**          - #OSDHANDLER_ERR_NO_OSD_HANDLE
**              if the OSDHANDLER has not allocated the corresponding handle
**              of the OSD/CP at initialization.
*****************************************************************************
*/
int32_t OSD_Stop(void)
{
    int32_t result;

    result = OSDHANDLER_StopLayer(APP_OSD_IDX, NULL);

    return(result);
}

/*!
*****************************************************************************
** \brief   Draws a curtains from the left and the right side
**          over the picture.
**
** \param   milliseconds
**              Duration for drawing the curtain.
**
** \return
**        - #FAPI_OK on success
**        - #FAPI_ERR_NOT_INITIALIZED
**          if the OSDHANDLER is not initialized.
**        - #OSDHANDLER_ERR_NO_OSD_HANDLE if the OSDHANDLER did not allocate
**          the corresponding handle at initialization.
**        - #FAPI_ERR_OUT_OF_MEMORY if there was not enough memory available
**          for allocation.
**        - #OSDHANDLER_ERR_LAYER_NOT_STARTED
**          if the layer could not be started.
*****************************************************************************
*/
int32_t OSD_DrawCurtain(uint32_t milliseconds)
{
    uint32_t v_res      = 0;
    int32_t  result     = 0;
    uint32_t count      = 0;
    uint32_t left_pos   = 0;
    uint32_t right_pos  = 0;
    uint32_t delay      = 0;
    OSDHANDLER_StartParamsT start_params;

    memset(&start_params, 0, sizeof(start_params));

    if(displayMode == 1) {
        osdStopCP();
        displayMode = 0;
    }

    if(displayBlack)
    {
        osdStopOSD();
        displayBlack = 0;
    }

    v_res = readFrame ? out_v_res : (out_v_res << 1);
    count = out_h_res / 8; // 4-pixel steps on 2 sides
    right_pos = (out_h_res >> 1) - 3;
    delay = milliseconds / count;

    start_params.xPos          = 0;
    start_params.yPos          = 0;
    start_params.inputWidth    = out_h_res >> 1;
    start_params.inputHeight   = v_res >> 4;
    start_params.outputWidth   = out_h_res;
    start_params.outputHeight  = v_res;
    start_params.frameBased    = readFrame;

    /**** setup OSD2 Parameter ****/
    result = OSDHANDLER_StartLayer(2, &start_params, NULL);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while starting OSD2. %d\n", result);
        return(result);
    }

    result = OSDHANDLER_SetFillColour(2, 6);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error when filling OSD2 transparent. %d\n", result);
        return(result);
    }

    /**** start OSD2 layer ****/
    result = OSDHANDLER_ShowLayer(2, NULL);

    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Can not show OSD2. %d\n", result);
        return(result);
    }

    v_res >>= 4;
    while(count--)
    {
        result = OSDHANDLER_BitBlitFill(2, left_pos,  0, 2, v_res, 1);
        if (result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("[OSD] Error draw rectangle: %d\n", result);
            return result;
        }

        result = OSDHANDLER_BitBlitFill(2, right_pos, 0, 2, v_res, 1);
        if (result != FAPI_OK) {
            FAPI_SYS_PRINT_MSG("[OSD] Error draw rectangle: %d\n", result);
            return result;
        }
        FAPI_SYS_SLEEP(delay);

        left_pos  += 2;
        right_pos -= 2;
    }
    displayBlack = 1;
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Fades the video output black by blending a black OSD that
**          covers the whole screen.
**
** \return
**        - #FAPI_OK on success
**        - #FAPI_ERR_NOT_INITIALIZED
**          if the OSDHANDLER is not initialized.
**        - #OSDHANDLER_ERR_NO_OSD_HANDLE if the OSDHANDLER did not allocate
**          the corresponding handle at initialization.
**        - #FAPI_ERR_OUT_OF_MEMORY if there was not enough memory available
**          for allocation.
**        - #OSDHANDLER_ERR_LAYER_NOT_STARTED
**          if the layer could not be started.
*****************************************************************************
*/
int32_t OSD_FadeBlack(void)
{
#if ENABLE_GUI_WINDOW == 0
    uint32_t v_res      = 0;
    int32_t  result     = 0;
    OSDHANDLER_StartParamsT start_params;

    memset(&start_params, 0, sizeof(start_params));
#endif

    if(displayMode == 1) {
        osdStopCP();
        displayMode = 0;
    }

    if(displayBlack)
    {
        osdStopOSD();
        displayBlack = 0;
    }

#if ENABLE_GUI_WINDOW == 0
    v_res = readFrame ? out_v_res : (out_v_res << 1);

    start_params.xPos          = 0;
    start_params.yPos          = 0;
    start_params.inputWidth    = out_h_res;
    start_params.inputHeight   = v_res;
    start_params.outputWidth   = 0;
    start_params.outputHeight  = 0;
    start_params.frameBased    = readFrame;

    /**** setup OSD1 Parameter ****/
    result = OSDHANDLER_StartLayer(1, &start_params, NULL);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while starting OSD1. %d\n", result);
        return(result);
    }

    result = OSDHANDLER_SetFillColour(1, 0);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error when filling OSD1 black. %d\n", result);
        return(result);
    }
    result = OSDHANDLER_PlaneFading(1, OSDHANDLER_PARAMETER_ON, NULL);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while fading OSD1: %d\n", result);
        return(result);
    }
    result = OSDHANDLER_ShowLayer(1, NULL);
    if (result != FAPI_OK)
    {
        FAPI_SYS_PRINT_MSG("[OSD] Error while showing OSD1: %d\n", result);
        return(result);
    }
#endif

    displayBlack = 1;
    return(FAPI_OK);
}
