/*!
*****************************************************************************
**
** \file        ./sample_applications/fapex_av_demo/src/servicelist.h
**
** \version     $Id$
**
** \brief       service list with tuning informaton.
**
** This file contains the declaration of the service list API. It initializes
** and configures the service list. Furthermore it provides a simple API to
** fill and retrieve the service list.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

#ifndef SERVICELIST_H
    #define SERVICELIST_H

    #include <stdint.h>
    #include <fapex/abst_types.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************
    #define SERVICELIST_MAX_NUM_SERVICES       1001
    #define SERVICELIST_MAX_SERVICE_NAME_LEN   31
    #define SERVICELIST_MAX_AUDIO_TRACK_NUMBER 20

    /* Stream type as defined in MPEG-2/DVB. */
    #define VID_MPEG1                          1
    #define VID_MPEG2                          2
    #define AUD_MPEG1                          3
    #define AUD_MPEG2                          4
    #define PRIV_PES                           6
    #define AUD_AAC_MPEG2                      15
    #define AUD_AAC_MPEG4                      17
    #define VID_H264                           27
    /* Stream type as defined in ATSC A52. */
    #define AUD_AC3_ATSC_A52                   129
    /* Special custom stream types */
    #define USER_DEF_AC_3                      0x6a06
    #define USER_DEF_EAC_3                     0x7a06

    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************
    typedef struct
    {
        uint8_t   name[SERVICELIST_MAX_SERVICE_NAME_LEN]; //!< service name
        uint32_t  videoPid;             //!< video PID
        uint32_t  videoType;            //!< stream type as defined in MPEG/DVB
        uint32_t  audioNumberOfTracks;  //!< number of audio track
        uint32_t  audioPid[SERVICELIST_MAX_AUDIO_TRACK_NUMBER]; //!< audio PID
        uint32_t  audioType[SERVICELIST_MAX_AUDIO_TRACK_NUMBER]; //!< stream type as defined in MPEG/DVB
        uint32_t  audioLanguage[SERVICELIST_MAX_AUDIO_TRACK_NUMBER]; //!< stream type as defined in MPEG/DVB
        uint32_t  audioDescriptionPid;  //!< audio Description PID (e.g. for UK)
        uint32_t  audioDescriptionLanguage; //!< audio Description PID (e.g. for UK)
        uint32_t  pcrPid;               //!< PCR PID if 0 then video PID used
        uint32_t  teletextPid;          //!< teletext PID
        uint32_t  dvbSubtitlePid;       //!< Subtitle PID
        uint32_t  serviceId;            //!< service ID
        uint32_t  serviceType;          //!< just informational e.g TV, radio..
        uint32_t  caFlag;               //!< CA flag - service is scrambled
        int32_t   trpListIndex;         //!< tuning parameter
    } SERVICELIST_ServiceInfo_T;


    //***********************************************************************
    //***********************************************************************
    //** Global Data
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** API Functions
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif
    int32_t                     SERVICELIST_AddService(SERVICELIST_ServiceInfo_T* service);
    void                        SERVICELIST_ClearList(void);
    int32_t                     SERVICELIST_DeleteAllCaServices(void);
    int32_t                     SERVICELIST_DeleteServiceById(uint32_t id);
    uint32_t                    SERVICELIST_GetSize(void);
    SERVICELIST_ServiceInfo_T*  SERVICELIST_GetService(uint32_t index);
    void                        SERVICELIST_Init(void);
    void                        SERVICELIST_LoadTestServices(void);
    int32_t                     SERVICELIST_UpdateService(SERVICELIST_ServiceInfo_T* service);

    #ifdef __cplusplus
    }
    #endif
#endif
