/*
********************************************************************************
********************************************************************************
**
** \file        ./sample_application/fapex_fe_demo/src/main.c
**
** \version     $Id$
**
** \brief       Demo Application for the FE subsystem
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2009-2010 by Fujitsu Semiconductor WSC
**
********************************************************************************
********************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <fapex/abst_types.h>
#include <fapex/abst_libir.h>
#include <fapex/abst_libfpc.h>

#include "fapex/sys_config.h"
#include <vcl/video.h>
#ifndef _LINUX_FAPI
#include <fapi/sys_shell.h>
#else
#include "shell.h"
#endif /* !_LINUX_FAPI */


#define DRV_ID_FPC0     (FAPEXDEV_FPC_MINOR_FIRST)

#define APP_VERSION      "1.0.0"

#define APP_REMOTE_MODE     FAPEX_IR_MODE_NEC   // FAPEX_IR_MODE_NEC or FAPEX_IR_MODE_RC5

#define APP_SHELL_THREAD_STACKSIZE         (4 * 1024)
#define APP_SHELL_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_MEDIUM
#define APP_SHELL_THREAD_NAME              "app:shell"

#define APP_START_THREAD_STACKSIZE         8*1024
#define APP_START_THREAD_PRIORITY          FAPEX_SYS_PRIORITY_HIGHEST
#define APP_START_THREAD_NAME              "app:fe_test"

#define APP_CommandStringSize 64

#define APP_IR_THREAD_STACKSIZE            1024
#define APP_IR_THREAD_PRIORITY             FAPEX_SYS_PRIORITY_MEDIUM
#define APP_IR_THREAD_NAME                 "app:ir"
#define APP_FPC_THREAD_STACKSIZE           1024
#define APP_FPC_THREAD_PRIORITY            FAPEX_SYS_PRIORITY_MEDIUM
#define APP_FPC_THREAD_NAME                "app:fpc"

#define DRV_ID_IR0      (FAPEXDEV_IR_MINOR_FIRST)


static VIDEO_HdEncoderT  appHdSystemType = VIDEO_1920x1080x50I;
static VIDEO_SdEncoderT  appSdSystemType = VIDEO_PAL;
static FAPEX_SYS_ThreadT  appStartThreadHandle;
static FAPEX_SYS_ThreadT  appShellThreadHandle;

static FAPEX_SYS_ThreadT    appIrThreadHandle;
static FAPEX_SYS_ThreadT    appFpcThreadHandle;

static int32_t fd_FPC0     = -1;
static int32_t fd_IR0      = -1;


//static void appStartThreadFunction(void* optArg);
static void appShellThreadFunction(void* optArg);
static void appIrThreadFunction(void* optArg);
static void appFpcThreadFunction(void* optArg);

static int32_t appRegisterCommand( char_t*  command,
                            uint32_t (*function)(int32_t, char_t**),
                            char_t*  brief,
                            char_t*  description );


#ifdef _LINUX_FAPI
//***************************************************************************
static void print_usage(char *name)
{
    FAPI_SYS_PRINT_MSG("usage: %s [-s hd_mode (sd_mode)]\n", name);
}
#endif

//***************************************************************************
//***************************************************************************
//** Main Program
//***************************************************************************
//***************************************************************************
int32_t main( int32_t argc, char_t* argv[] )
{
    int32_t  result = 0;

    FAPI_SYS_PRINT_MSG("\n%lld, main() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    /* initial input settings */
    if(argc > 1) {
        int32_t  i;

        for(i=1; i<argc; i++) {
            if(strcmp("-s", argv[i]) == 0) {
                i ++;
                appHdSystemType = atoi(argv[i]);
                if(argc > (i+1)) {
                    if( (strlen(argv[i+1]) == 1)
                     && (isdigit(argv[i+1])) ) {
                        i ++;
                        appSdSystemType = atoi(argv[i]);
                    }
                }
            }
            else if(*argv[i] == '-') {
                print_usage(argv[0]);
                exit(1);
            }
        }
    }

    FAPI_SYS_PRINT_MSG("\033[2J\n\n");
    FAPI_SYS_PRINT_MSG("#################################################################\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# paint demo application (FAPex version)\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# program version: %s\n", APP_VERSION);
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("# (c) Copyright 2007-2010 by Fujitsu Semiconductor Europe GmbH\n");
    FAPI_SYS_PRINT_MSG("# (c) Copyright 2008-2010 by Fujitsu Semiconductor Limited\n");
    FAPI_SYS_PRINT_MSG("#\n");
    FAPI_SYS_PRINT_MSG("#################################################################\n");
    FAPI_SYS_PRINT_MSG("\n");
    #if 0
    appStartThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_START_THREAD_STACKSIZE,
            APP_START_THREAD_PRIORITY,
            appStartThreadFunction,
            0, 0, APP_START_THREAD_NAME );
    if( appStartThreadHandle == NULL )
    {
        FAPI_SYS_PRINT_MSG( "ERROR: creating thread \"%s\" failed\n",
                            APP_START_THREAD_NAME );
        exit(1);
    }
    #endif
    appShellThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_SHELL_THREAD_STACKSIZE,
            APP_SHELL_THREAD_PRIORITY,
            appShellThreadFunction,
            0, 0, APP_SHELL_THREAD_NAME );
    if(appShellThreadHandle == NULL)
    {
        FAPI_SYS_PRINT_MSG( "Error: FAPEX_SYS_CreateThread (shell)\n" );
        exit(1);
    }

    FAPI_SYS_PRINT_MSG("\n%lld, thread start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    FAPEX_SYS_StopPrintThread();
    FAPEX_SYS_StartMultitasking();

    //FAPEX_SYS_WaitEndThread( appStartThreadHandle );
    FAPEX_SYS_WaitEndThread( appShellThreadHandle );

    exit( result );

}

static uint32_t fpc_cmd_set_integer_value( int32_t argc, char_t* argv[] )
{
    static int32_t  IntegerValue = -1;

    if( argc == 2 )
    {
        sscanf(argv[1], "%d", &IntegerValue);

        if(fd_FPC0 > 0)
        {
            int32_t  result;

            result = FAPEX_FPC_Ioctl( fd_FPC0,
                                      FAPEX_IOCCOM_FPC_SET_INTEGER_VALUE,
                                      (FAPEXIOCARG3_T)IntegerValue );
            if( result != 0 )
            {
                FAPI_SYS_PRINT_MSG("ERROR:[main.c] error FPC set value (code: %d).\n", result);
            }
        }
    }
    return 1;

}

static uint32_t fpc_cmd_set_brightness( int32_t argc, char_t* argv[] )
{
    static int32_t  Brightness = -1;

    if( argc == 2 )
    {
        sscanf(argv[1], "%d", &Brightness);

        if(fd_FPC0 > 0)
        {
            int32_t  result;

            result = FAPEX_FPC_Ioctl( fd_FPC0,
                                      FAPEX_IOCCOM_FPC_SET_BRIGHTNESS,
                                      (FAPEXIOCARG3_T)Brightness );
            if( result != 0 )
            {
                FAPI_SYS_PRINT_MSG("ERROR:[main.c] error FPC set value (code: %d).\n", result);
            }
        }
    }
    return 1;
}

static uint32_t fpc_cmd_set_scroll_text( int32_t argc, char_t* argv[] )
{
    static int32_t  text = -1;

    if( argc == 2 )
    {
        sscanf(argv[1], "%d", &text);

        if(fd_FPC0 > 0)
        {
            int32_t  result;

            result = FAPEX_FPC_Ioctl( fd_FPC0,
                                      FAPEX_IOCCOM_FPC_SET_SCROLL_TEXT,
                                      (FAPEXIOCARG3_T)text );
            if( result != 0 )
            {
                FAPI_SYS_PRINT_MSG("ERROR:[main.c] error FPC set value (code: %d).\n", result);
            }
        }
    }
    return 1;
}

static uint32_t fpc_cmd_set_dot_toggle( int32_t argc, char_t* argv[] )
{
    static int32_t  dot = -1;

    if( argc == 2 )
    {
        sscanf(argv[1], "%d", &dot);

        if(fd_FPC0 > 0)
        {
            int32_t  result;

            result = FAPEX_FPC_Ioctl( fd_FPC0,
                                      FAPEX_IOCCOM_FPC_SET_DOT_TOGGLE,
                                      (FAPEXIOCARG3_T)dot );
            if( result != 0 )
            {
                FAPI_SYS_PRINT_MSG("ERROR:[main.c] error FPC set value (code: %d).\n", result);
            }
        }
    }
    return 1;
}
static uint32_t fpc_cmd_set_display_segment( int32_t argc, char_t* argv[] )
{
    FAPEX_FPC_DispSegParamStrT segment ;

    if( argc == 5 )
    {
        sscanf(argv[1], "%c", &segment.data[3]);
        sscanf(argv[2], "%c", &segment.data[2]);
        sscanf(argv[3], "%c", &segment.data[1]);
        sscanf(argv[4], "%c", &segment.data[0]);
        segment.dot = 0x0f;//test
        segment.mask =0x0f;//test

        FAPI_SYS_PRINT_MSG("[main]%d,%d,%d,%d",segment.data[0],segment.data[1],segment.data[2],segment.data[3]);

        if(fd_FPC0 > 0)
        {
            int32_t  result;

            result = FAPEX_FPC_Ioctl( fd_FPC0,
                                      FAPEX_IOCCOM_FPC_SET_DISPLAY_SEGMENT,
                                      (FAPEXIOCARG3_T)(&segment) );
            if( result != 0 )
            {
                FAPI_SYS_PRINT_MSG("ERROR:[main.c] error FPC set value (code: %d).\n", result);
            }
        }
    }
    return 1;
}

static uint32_t fpc_cmd_set_digit( int32_t argc, char_t* argv[] )
{
    FAPEX_FPC_DigitParamStrT digit = {0};

    if( argc == 4 )
    {
        sscanf(argv[1], "%d", &digit.position);
        sscanf(argv[2], "%d", &digit.value);
        sscanf(argv[3], "%d", &digit.carry);

        digit.carry = 0;//test
        if(fd_FPC0 > 0)
        {
            int32_t  result;

            result = FAPEX_FPC_Ioctl( fd_FPC0,
                                      FAPEX_IOCCOM_FPC_SET_DIGIT,
                                      (FAPEXIOCARG3_T)(&digit) );
            if( result != 0 )
            {
                FAPI_SYS_PRINT_MSG("ERROR:[main.c] error FPC set value (code: %d).\n", result);
            }
        }
    }
    return 1;
}


/*
*****************************************************************************
**
** \brief IR thread
**
** This function monitors the IR event.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appIrThreadFunction(void* optArg)
{
    FAPEX_IR_KeyParamStrT       keyParams;
    int32_t  result;
#if 0 /* now not supported */
    int32_t  argc = 2;
    char_t*  argv[3];

    argv[0] = "";
#endif

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appIrThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    while(1) {
        result = FAPEX_IR_Ioctl( fd_IR0,
                                 FAPEX_IOCCOM_IR_GET_KEY,
                                 (FAPEXIOCARG3_T)&keyParams );
        if( result != 0 ) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] Get IR Key. %d\n", result);
        }
        switch( keyParams.rcmCode ) {
            case FAPEX_IR_KEY_MUTE:

                break;

            case FAPEX_IR_KEY_VOL_UP:

                break;

            case FAPEX_IR_KEY_VOL_DOWN:

                break;

            case FAPEX_IR_KEY_CHANNEL_UP:

                break;

            case FAPEX_IR_KEY_CHANNEL_DOWN:

                break;

            case FAPEX_IR_KEY_STANDBY:

                break;

#if 0 /* now not supported */
            case FAPEX_IR_KEY_1:
                argv[1] = "0";
                SERVICELIST_ClearList();
                (void)APP_Tune(argc, argv);
                trpListIndex = 0;
                appCurrentServiceIndex = -1;
                tsScan = 1;
                SCAN_Start(trpListIndex);
                break;

            case FAPEX_IR_KEY_2:
                argv[1] = "0";
                argv[2] = "0";
                FAPI_SYS_PRINT_MSG("Toneburst OFF, DiseqC OFF\n");
                (void)APP_ConfigSat( 3, argv );
                break;

            case FAPEX_IR_KEY_3:
                argv[1] = "0";
                argv[2] = "1";
                FAPI_SYS_PRINT_MSG("Toneburst OFF, DiseqC ON\n");
                (void)APP_ConfigSat( 3, argv );
                break;
#endif
            default:
                FAPI_SYS_PRINT_MSG( "Received key: 0x%x\n", keyParams.rcmCode );
        }
    }
}

/*
*****************************************************************************
**
** \brief FPC thread
**
** This function monitors the FPC's event.
**
** \param optArgPtr Optional argument pointer, ignored here!
**
*****************************************************************************
*/
static void appFpcThreadFunction(void* optArg)
{
    uint32_t buttonFlag;
    int32_t  result;

    FAPI_SYS_PRINT_DEBUG(1, "\n%lld, appFpcThreadFunction() start\n",
                            FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

    while(1) {
        result = FAPEX_FPC_Ioctl( fd_FPC0,
                                  FAPEX_IOCCOM_FPC_GET_BUTTON_NOTIFY,
                                  (FAPEXIOCARG3_T)&buttonFlag );
        if( result != 0 ) {
            FAPI_SYS_PRINT_ERR("ERROR:[main.c] error FPC get notify (code: %d).\n", result);
            break;
        }
        FAPI_SYS_PRINT_MSG( "Received button: 0x%x\n", buttonFlag );
    }
}


static int32_t appRegisterCommand( char_t*  command,
                            uint32_t (*function)(int32_t, char_t**),
                            char_t*  brief,
                            char_t*  description )
{
#ifndef _LINUX_FAPI
    return FAPI_SYS_ShellRegisterCommand(appShellInputHandler, command, function, brief, description);
#else
    return SHELL_RegisterCommand(command, function, brief, description);
#endif
}


static void appShellThreadFunction(void* optArg)
{
#ifndef _LINUX_FAPI
    appShellInputHandler = FAPI_SYS_ShellCreateInputHandler( APP_SHELL_PROMPT,
                                                             500,
                                                             NULL );
    if( !appShellInputHandler ) {
        exit(1);
    }
#else
    char_t    args_buff[APP_CommandStringSize];
    int32_t   i, c;
    FAPEX_IR_OpenParamStrT irOpenParams;

    // Open IR
    irOpenParams.mode = APP_REMOTE_MODE;
    fd_IR0 = FAPEX_IR_Open( DRV_ID_IR0,
                            O_RDWR,
                            (FAPEXOPENARG3_T)&irOpenParams );
    if(fd_IR0 < 0) {
        FAPI_SYS_PRINT_MSG("ERROR:[main.c] Open IR driver.\n");
    }
    else {
#if APP_REMOTE_MODE == FAPEX_IR_MODE_RC5
        FAPEX_IR_MapTableParamStrT  mapping_table;
        FAPEX_IR_KeycodeMapStrT     key_codes[] =
        {
            { 0x0C, FAPEX_IR_KEY_STANDBY },   { 0x57, FAPEX_IR_KEY_OK},
            { 0x50, FAPEX_IR_KEY_UP},         { 0x51, FAPEX_IR_KEY_DOWN},
            { 0x55, FAPEX_IR_KEY_LEFT},       { 0x56, FAPEX_IR_KEY_RIGHT},
            { 0x10, FAPEX_IR_KEY_VOL_UP},     { 0x11, FAPEX_IR_KEY_VOL_DOWN},
            { 0x20, FAPEX_IR_KEY_CHANNEL_UP}, { 0x21, FAPEX_IR_KEY_CHANNEL_DOWN},
            { 0x00, FAPEX_IR_KEY_0 },         { 0x01, FAPEX_IR_KEY_1 },
            { 0x02, FAPEX_IR_KEY_2 },         { 0x03, FAPEX_IR_KEY_3 },
            { 0x04, FAPEX_IR_KEY_4 },         { 0x05, FAPEX_IR_KEY_5 },
            { 0x06, FAPEX_IR_KEY_6 },         { 0x07, FAPEX_IR_KEY_7 },
            { 0x08, FAPEX_IR_KEY_8 },         { 0x09, FAPEX_IR_KEY_9 },
            { 0x0A, FAPEX_IR_KEY_MUTE },      { 0x52, FAPEX_IR_KEY_MENU},
            { 0x0F, FAPEX_IR_KEY_INFO},       { 0x6B, FAPEX_IR_KEY_RED },
            { 0x6C, FAPEX_IR_KEY_GREEN },     { 0x6D, FAPEX_IR_KEY_YELLOW },
            { 0x6E, FAPEX_IR_KEY_BLUE },      { 0x00, FAPEX_IR_KEY_UNDEFINED}
        };
        mapping_table.deviceId      = 0;
        mapping_table.pKeycodeTable = key_codes;
        mapping_table.keycodeNum    = 28;
        (void)FAPEX_IR_Ioctl( fd_IR0,
                              FAPEX_IOCCOM_IR_SET_MAPTABLE,
                              &mapping_table );
#endif
        appIrThreadHandle = FAPEX_SYS_CreateThread(
                NULL,
                APP_IR_THREAD_STACKSIZE,
                APP_IR_THREAD_PRIORITY,
                appIrThreadFunction,
                0, 0, APP_IR_THREAD_NAME );
        if( appIrThreadHandle == NULL )
        {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                                APP_IR_THREAD_NAME );
            FAPEX_IR_Close( fd_IR0 );
            exit(1);
        }
    }

    // Open FPC
    fd_FPC0 = FAPEX_FPC_Open( DRV_ID_FPC0,
                              O_RDWR,
                              (FAPEXOPENARG3_T)NULL );
    if(fd_FPC0 < 0) {
        FAPI_SYS_PRINT_MSG("ERROR:[main.c] Open FPC driver. %d\n", fd_FPC0);
    }
    else {
        (void)FAPEX_FPC_Ioctl( fd_FPC0,
                               FAPEX_IOCCOM_FPC_SET_INTEGER_VALUE,
                               (FAPEXIOCARG3_T)1 );

        appFpcThreadHandle = FAPEX_SYS_CreateThread(
                NULL,
                APP_FPC_THREAD_STACKSIZE,
                APP_FPC_THREAD_PRIORITY,
                appFpcThreadFunction,
                0, 0, APP_FPC_THREAD_NAME );
        if( appFpcThreadHandle == NULL )
        {
            FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                                APP_FPC_THREAD_NAME );
            FAPEX_FPC_Close( fd_FPC0 );
            exit(1);
        }
    }


    (void)appRegisterCommand(
        "?",
        SHELL_HelpCommand,
        "display all available commands",
        "To get more help about a specific command simply pass the command\n"   \
        "as optional argument to ?\n"                                           \
        "Example: ?       displays all available commands, one per line\n"     \
        "Example: ? !     displays a more detailed help for the ! command\n" \
    );

    (void)appRegisterCommand(
        "h",
        SHELL_HistoryCommand,
        "display the command history",
        NULL
    );

    (void)appRegisterCommand(
        "!",
        SHELL_RedoCommand,
        "redo a previous command",
        "This functions re-calls a previous command identified by number.\n" \
        "To find out the index of a previous command use the \"history\".\n" \
        "Example: !4     re-calls the command listed as #4 in the history output\n" \
        "Example: !!     re-calls the last command\n" \
    );
#endif

    (void)appRegisterCommand(
        "Setinteger",
        fpc_cmd_set_integer_value,
        "Set IntegerValue. Usage: input 1-9999.",
        "Usage: input 1-9999\n"
    );

    (void)appRegisterCommand(
        "Setbrightness",
        fpc_cmd_set_brightness,
        "Set IntegerValue. Usage: input 1-100.",
        "Usage: input 1-100\n"
    );

    (void)appRegisterCommand(
        "Settext",
        fpc_cmd_set_scroll_text,
        "Set IntegerValue. Usage:.",
        "Usage: \n"
    );

    (void)appRegisterCommand(
        "Setdot",
        fpc_cmd_set_dot_toggle,
        "Set IntegerValue. Usage:input 0-3.",
        "Usage: input 0-3\n"
    );

    (void)appRegisterCommand(
        "Setsegment",
        fpc_cmd_set_display_segment,
        "Set IntegerValue. Usage:input four character,each one separate by space.Example:Setsegment_A_3_B_2,'_'means space.",
        "Usage:input four character,each one separate by space.Example:Setsegment_A_3_B_2,'_'means space."
    );

    (void)appRegisterCommand(
        "Setdigit",
        fpc_cmd_set_digit,
        "Set IntegerValue. Usage:input postion,value,carry.Example:Setdigit_2_5_0.it means display a num at 2nd LED",
        "Usage:input postion,value,carry.Example:Setdigit_2_5_0.it means display 5 at 3rd(2+1) LED"
    );

#if 0
    appStartThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_START_THREAD_STACKSIZE,
            APP_START_THREAD_PRIORITY,
            appStartThreadFunction,
            0, 0, APP_START_THREAD_NAME );
    if( appStartThreadHandle == NULL )
    {
        FAPI_SYS_PRINT_MSG( "ERROR: creating thread \"%s\" failed\n",
                            APP_START_THREAD_NAME );
        exit(1);
    }
#endif
#ifndef _LINUX_FAPI
    FAPI_SYS_ShellStartInputHandler( appShellInputHandler );
#else
    strcpy( args_buff, "?" );
    SHELL_DoCommand( args_buff );

    while(1) {
    //while(appDrawEnable >= 0){
        i = 0;

        /* get command string */
        while( !feof(stdin) ) {
            c = getchar();
            args_buff[i] = c;
            if(c == '\n') {
                args_buff[i] = '\0';
                break;
            }
            else if( (c == '\b') && (i > 0) ) {
                i --;
            }
            else if(i < (APP_CommandStringSize - 1)) {
                i ++;
            }
        }
        SHELL_DoCommand( args_buff );

        FAPI_SYS_SLEEP(100);
    }
#endif /* !_LINUX_FAPI */

    FAPEX_SYS_WaitEndThread( appStartThreadHandle );
}



