/* 
*******************************************************************************
**
** \file        ./subsystems/audio/aal/src/aal.c
**
**              $Id: aal.c 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       Audio abstraction layer.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*******************************************************************************
*/

//#define APT_VOL_CTRL
//#define APT_11DB_ATTENUATE
//#define APT_SIMPLE_SRC

/*
*******************************************************************************
**
** \note The following LINT statements have been suppressed:
- Warning 641: Converting enum to int.
- Info    717: do ... while(0);
- Info    788: enum constant 'anyConst' not used within defaulted switch
**
*******************************************************************************
*/
/*lint -save -e641 -e717 -e788 */

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#define DEBUG 1

#include <stdint.h>
#include <string.h>

#ifdef _LINUX_FAPI

#include "fapex/abst_types.h"
#include "fapex/abst_libaudec.h"
#include "fapex/abst_libauout.h"
#include "fapex/abst_libtsd.h"
#include "fapex/abst_libhdmi.h"
#include "fapex/sys_config_mb86h61_devkit.h"


#define RTOS_SemaphoreT        FAPEX_SYS_SemaphoreT
#define RTOS_CreateSemaphore   FAPEX_SYS_CreateSemaphore
#define RTOS_GetSemaphore      FAPEX_SYS_GetSemaphore
#define RTOS_SetSemaphore      FAPEX_SYS_SetSemaphore
#define RTOS_DestroySemaphore  FAPEX_SYS_DestroySemaphore

#define AUDIO_MAX_VOLUME      0xfe
#define AUDIO_TSD_CHANNEL     8
#define DRV_ID_TSD0_AUDIO_CH  (FAPEXDEV_TSD0_SUB_MINOR_FIRST + AUDIO_TSD_CHANNEL)
#define DRV_ID_AUDEC_0        (FAPEXDEV_AUDEC_MINOR_FIRST)
#define DRV_ID_AUDEC_1        (FAPEXDEV_AUDEC_MINOR_FIRST + 1)
#define DRV_ID_AUDEC_2        (FAPEXDEV_AUDEC_MINOR_FIRST + 2)
#define DRV_ID_AUDEC_3        (FAPEXDEV_AUDEC_MINOR_FIRST + 3)
#define DRV_ID_AUDEC_4        (FAPEXDEV_AUDEC_MINOR_FIRST + 4)
#define DRV_ID_AUOUT_0        (FAPEXDEV_AUOUT_MINOR_FIRST)


static int32_t fd_TSD0_AUDIO  = -1;
static int32_t fd_AUDEC0      = -1;
static int32_t fd_AUDEC_MPEG  = -1;
static int32_t fd_AUDEC_AAC   = -1;
static int32_t fd_AUDEC_AC3   = -1;
static int32_t fd_AUDEC_HEAAC = -1;
static int32_t fd_AUDEC_DD    = -1;
static int32_t fd_AUDEC_DDP   = -1;
static int32_t fd_AUDEC_WMA   = -1;
static int32_t fd_AUOUT0      = -1;

extern int32_t fd_TSD[];
static void* audio_mmapbuf = NULL;

#else

#include <fapi/sys_services.h>
#include <fapi/drv_auout.h>
#include <fapi/drv_audec.h>
#include <fapi/drv_audec_ac3.h>
#include <fapi/drv_audec_ddpdcv.h>
#include <fapi/drv_audec_heaac.h>
#include <fapi/drv_audec_mpeg.h>
#include <fapi/drv_bm.h>
#include <fapi/drv_tsd.h>

#endif

#include "aal.h"


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define AAL_VOLUMELEVEL_MAX     0xFE    //!< NOTE: Avoid setting of level 0xFF!

#if defined(__GNUC__)

#define AAL_PARAMSBUF_SIZE      128

#endif /* defined(__GNUC__) */

#define AAL_DOLBY_LINE_MODE     2
#define AAL_DOLBY_RF_MODE       3

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef enum
{
    AAL_PATH_MAIN = 0,
    AAL_PATH_SPDIF,
    AAL_PATH_NUMS

} AAL_AudioPathT;

typedef struct
{
    FAPI_SYS_HandleT        handleBm;
    FAPI_SYS_HandleT        handleTsd;
    FAPI_SYS_HandleT        handleOut;
    FAPI_SYS_HandleT        handleDec;

    uint8_t                 started;
    AAL_AudioFormatT        format;
    AAL_AUDEC_DecoderEnumT decoder;
    uint32_t                pid;

    /* parameters valid in 'started' mode/ */
    uint32_t                bmSize;
    uint8_t                 enableSpdif;


} AAL_AudioPathOpenT;

/*!
*******************************************************************************
**
**  \brief  AAL Open Parameter structure.
**
*******************************************************************************
 */
typedef struct
{
    AAL_OpenParamsT         params;
    uint8_t                 open;

    AAL_AudioPathOpenT      path[AAL_PATH_NUMS];

    /* Parameters for main audio decoding. */

    uint8_t                 audecRunning;
    uint8_t                 audecRestart;
    uint8_t                 hdmiHaveSampleFreq;
    uint8_t                 hdmiRunning;

    /* Parameters for SPDIF output. */
    AAL_SPDIFModeT          spdifMode;

} AAL_OpenT;

typedef struct
{
    uint8_t                 muted;

} AAL_AudioPathDataT;

/*!
*******************************************************************************
**
**  \brief  AAL Data structure.
**
*******************************************************************************
 */
typedef struct
{
    RTOS_SemaphoreT         mutexHandle;

    AAL_AUDEC_DecoderEnumT targetDecoder[AAL_AUDIOFORMAT_NUMS];
                            //!< holds corresponding decoder for each audio
                            //!< format.
    AAL_AudioPathDataT      path[AAL_PATH_NUMS];

    AAL_StereoModeT         stereoMode;
    uint8_t                 lvlLeft;
    uint8_t                 lvlRight;

    uint8_t                 hdmiEnabled;

    AAL_InitParamsT         initParams;
    AAL_OpenT               openData;

    #if defined(__GNUC__)
    uint8_t                 paramsBuf[AAL_PARAMSBUF_SIZE];
    #endif /* defined(__GNUC__) */

} AAL_DataT;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static AAL_DataT* audioDatPtr = NULL;
static uint32_t   audioDolbyCompMode = AAL_DOLBY_LINE_MODE;

#ifdef APT_VOL_CTRL
#include <fapi/drv_auout_apt_vol.h>
static AAL_AUOUT_AptVolParamsT *audioAptVolParamsPtr = NULL;
static FAPI_SYS_HandleT audioAptVolHandle  = NULL;
#endif

#ifdef APT_11DB_ATTENUATE
static FAPI_SYS_HandleT audioApt11dbHandle  = NULL;
#endif

#ifdef APT_SIMPLE_SRC
#include <fapi/drv_auout_apt_ssrc.h>
static AAL_AUOUT_AptSsrcParamsT *audioAptSsrcParamsPtr = NULL;
static FAPI_SYS_HandleT audioAptSsrcHandle  = NULL;
#endif

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/


#ifndef _LINUX_FAPI

static AAL_OpenT*       aalCheckHandle      (FAPI_SYS_HandleT handle,
                                             int32_t* errCodePtr);
static int32_t          aalStartHdmi        (AAL_DataT* inst,
                                             AAL_OpenT* openPtr);
static int32_t          aalStopHdmi         (AAL_OpenT* openPtr);
static FAPI_SYS_HandleT aalOpenDecoder      (AAL_DataT* inst,
                                             AAL_AudioFormatT format,
                                             uint8_t ac3ToSpdif,
                                             uint8_t suppressErrorRecognition,
                                             FAPI_SYS_HandleT handleBm,
                                             FAPI_SYS_HandleT handleOut,
                                             AAL_AUDEC_DecoderEnumT*
                                             decoderPtr,
                                             int32_t* errCodePtr);
static FAPI_SYS_HandleT aalOpenBm           (AAL_DataT* inst,
                                             AAL_AudioPathT path,
                                             uint32_t* bmSizePtr,
                                             int32_t* errCodePtr);
static FAPI_SYS_HandleT aalOpenTsd          (AAL_DataT* inst,
                                             FAPI_SYS_HandleT handleBm,
                                             int32_t* errCodePtr);
static int32_t          aalIntStart         (AAL_DataT* inst,
                                             AAL_OpenT* openPtr,
                                             AAL_AudioFormatT format,
                                             uint32_t pid);
static int32_t          aalIntStartSPDIF    (AAL_DataT* inst,
                                             AAL_OpenT* openPtr,
                                             AAL_AudioFormatT format,
                                             uint32_t pid);
static int32_t          aalIntStop          (AAL_OpenT* openPtr,
                                             AAL_AudioPathT path);
static int32_t          aalIntStopModules   (AAL_OpenT* openPtr,
                                             AAL_AudioPathT path);
static int32_t          aalIntSetVolAndStereoMode (AAL_DataT* inst,
                                             AAL_OpenT* openPtr);

#if defined(__GNUC__)

static void             aalSwapLongwords    (uint8_t* bufPtr);

#endif /* defined(__GNUC__) */

#else

static AAL_OpenT* aalCheckHandle (FAPI_SYS_HandleT handle, int32_t* errCodePtr);

static int32_t audioTsdOpen(void);
static int32_t audioTsdClose(void);
static int32_t audioOutOpen(void);
static int32_t audioOutClose(void);
static int32_t aalStartHdmi (AAL_DataT* inst, AAL_OpenT* openPtr);
static int32_t aalStopHdmi (AAL_OpenT* openPtr);

#endif

//*****************************************************************************
//*****************************************************************************
//** API Functions
//*****************************************************************************
//*****************************************************************************

/*!
*******************************************************************************
** \brief Initialize AAL module.
**
** \param   paramsPtr   Pointer to init parameters.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \note    This function shall be called once at system startup.
**
** \sa      AAL_Exit
*******************************************************************************
*/
int32_t AAL_Init (const AAL_InitParamsT* paramsPtr)
{
    int32_t                 err_code = FAPI_OK;

#ifndef _LINUX_FAPI    
    AAL_AUOUT_OpenParamsT  auout_open_params;
    AAL_AUDEC_OpenParamsT  audec_open_params;
#else
    FAPEX_AUDEC_OpenParamStrT   audecOpenParam;
    FAPEX_AUOUT_OpenParamStrT   auoutOpenParam;
    FAPEX_AUDEC_OutputParamStrT audecOutParam;
    int32_t                     result = 0;
#endif

    AAL_AudioPathT          pathCnt;
    AAL_AudioFormatT        formatCnt;
    FAPI_SYS_HandleT        handleDec;
    FAPI_SYS_HandleT        handleBm  = NULL;
    FAPI_SYS_HandleT        handleOut = NULL;

    if ( audioDatPtr != NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_ALREADY_INITIALIZED;

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Init: err %i \n", err_code);

        return err_code;
    }

    do
    {
        if ( paramsPtr == NULL )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
            break;
        }

        audioDatPtr = (AAL_DataT*)FAPI_SYS_MALLOC_CACHED(sizeof(AAL_DataT));

        if ( audioDatPtr == NULL )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_OUT_OF_MEMORY;
            break;
        }

        memset(audioDatPtr, 0, sizeof(AAL_DataT));

        audioDatPtr->mutexHandle = FAPEX_SYS_CreateSemaphore(1);      
        if ( audioDatPtr->mutexHandle == NULL )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_SEMAPHORE_CREATE;
            break;
        }

        audioDatPtr->hdmiEnabled                 = 1;
        audioDatPtr->initParams                  = *paramsPtr;

        audioDatPtr->openData.open               = 0;
        audioDatPtr->openData.audecRunning       = 0;
        audioDatPtr->openData.hdmiHaveSampleFreq = 0;
        audioDatPtr->openData.hdmiRunning        = 0;

        /* set default volume levels */
        audioDatPtr->stereoMode                  = AAL_STEREO_MODE_STEREO;
        audioDatPtr->lvlLeft                     = 0x80;
        audioDatPtr->lvlRight                    = 0x80;

        for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS; pathCnt++ )
        {
            /* set default mute status */
            audioDatPtr->path[pathCnt].muted = 0;
        }

        for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS; pathCnt++ )
        {
            audioDatPtr->openData.path[pathCnt].started = 0;
        }

        /* ----- Get list of available decoders ---------------------------- */

        for ( formatCnt = (AAL_AudioFormatT)0; formatCnt < AAL_AUDIOFORMAT_NUMS; formatCnt++ )
        {
            audioDatPtr->targetDecoder[formatCnt] = AAL_AUDEC_DECODER_UNDEF;
        }


#ifndef _LINUX_FAPI

        /* open dummy BM and audio output to satisfy decoders that will be temporarily opened. */
        handleBm = aalOpenBm (audioDatPtr, AAL_PATH_MAIN, NULL, &err_code);
        if ( handleBm == NULL )
        {
            break;
        }     

        memset(&auout_open_params, 0, sizeof(auout_open_params));

        auout_open_params.version       = AAL_AUOUT_VERSION;
        auout_open_params.enableSPDIF   = 0;
        auout_open_params.speakerArr[0] = AAL_AUOUT_SPEAKER_LEFT;
        auout_open_params.speakerArr[1] = AAL_AUOUT_SPEAKER_RIGHT;
        auout_open_params.speakerArr[2] = AAL_AUOUT_SPEAKER_UNDEF;//end of list
        auout_open_params.pcmOutBuffer  = NULL;        

        handleOut = AAL_AUOUT_Open (&auout_open_params, &err_code);
        if ( handleOut == NULL )
        {
            break;
        }

#endif

       

#ifndef _LINUX_FAPI

        memset(&audec_open_params, 0, sizeof(audec_open_params));

        audec_open_params.version           = AAL_AUDEC_VERSION;
        audec_open_params.sampleBits        = AAL_AUDEC_SAMPLE_16_BIT;

        audec_open_params.inputStreamHandle = handleBm;
        audec_open_params.audioOutHandle    = handleOut;

        /* ----- MPEG ----- */        
        audec_open_params.decoder           = AAL_AUDEC_DECODER_MPEG;
        
        handleDec = AAL_AUDEC_Open(&audec_open_params, &err_code);


        FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening MPEG decoder:   ");

        if ( handleDec == NULL )
        {
            /* mandatory decoder, opening must not fail! */

            FAPI_SYS_PRINT_DEBUG(1, "failed: %i\n", err_code);
            break;
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "OK!\n");

            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_MPEG] = audec_open_params.decoder;

            (void)AAL_AUDEC_Close(handleDec);
        }
#else
        
        memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_MPEG;
        fd_AUDEC_MPEG = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audecOpenParam );
        if(fd_AUDEC_MPEG < 0) 
        {
            FAPI_SYS_PRINT_ERR("ERROR: Cannot open MPEG audio decoder (code %d).\n", fd_AUDEC_MPEG);
            break;
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "[aal][AAL_Init] Open MPEG audio decode OK!\n");

            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_MPEG] = AAL_AUDEC_DECODER_MPEG;

            FAPEX_AUDEC_Close(fd_AUDEC_MPEG);
        }

#endif


#ifndef _LINUX_FAPI

        /* ----- HE-AAC ----- */
        audec_open_params.decoder = AAL_AUDEC_DECODER_HEAAC;

        handleDec = AAL_AUDEC_Open(&audec_open_params, &err_code);

        FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening HE-AAC decoder: ");

        if ( handleDec == NULL )
        {
            if ( err_code == AAL_AUDEC_ERR_CODEC_NOT_SUPPORTED )
            {
                FAPI_SYS_PRINT_DEBUG(1, "NOT supported!\n");
                err_code = FAPI_OK; /* This is a warning only! */
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(1, "failed: %i\n", err_code);
                break;
            }
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "OK!\n");

            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_HEAAC] = audec_open_params.decoder;

            (void)AAL_AUDEC_Close(handleDec);
        }

#else
    
        memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_HEAAC;
        fd_AUDEC_HEAAC = FAPEX_AUDEC_Open( DRV_ID_AUDEC_1,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audecOpenParam );
        if(fd_AUDEC_HEAAC < 0) 
        {
            FAPI_SYS_PRINT_ERR("ERROR: Cannot open HEAAC audio decoder (code %d).\n", fd_AUDEC_HEAAC);
            //break;
        }
        else
        {
            //FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
            FAPI_SYS_PRINT_DEBUG(1, "[aal][AAL_Init] Open HEAAC audio decode OK!\n");

            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_HEAAC] = AAL_AUDEC_DECODER_HEAAC;

            FAPEX_AUDEC_Close(fd_AUDEC_HEAAC);
        }

#endif


#ifndef _LINUX_FAPI

        /* ----- AAC ----- */
        if ( audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_HEAAC]
             != AAL_AUDEC_DECODER_UNDEF )
        {
            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AAC] = AAL_AUDEC_DECODER_HEAAC;
        }
#else
       /* ----- AAC ----- */
        if ( audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_HEAAC]
             != AAL_AUDEC_DECODER_UNDEF )
        {
            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AAC] = AAL_AUDEC_DECODER_HEAAC;
        }

#endif


#ifndef _LINUX_FAPI
       /*--------DD--------*/
        audec_open_params.decoder = AAL_AUDEC_DECODER_DD;

        handleDec = AAL_AUDEC_Open(&audec_open_params, &err_code);

        FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening DD decoder:    ");

        if ( handleDec == NULL )
        {
            if ( err_code == AAL_AUDEC_ERR_CODEC_NOT_SUPPORTED )
            {
                FAPI_SYS_PRINT_DEBUG(1, "NOT supported!\n");
                err_code = FAPI_OK; /* This is a warning only! */
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(1, "failed: %i\n", err_code);
                break;
            }
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "OK!\n");

            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DD] = audec_open_params.decoder;
            (void)AAL_AUDEC_Close(handleDec);
        }
#else
        memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_DD;
        fd_AUDEC_DD = FAPEX_AUDEC_Open( DRV_ID_AUDEC_2,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audecOpenParam );
        if(fd_AUDEC_DD < 0)
        {
            FAPI_SYS_PRINT_ERR("ERROR: Cannot open DD audio decoder (code %d).\n", fd_AUDEC_DD);
            //break;
        }
        else
        {
            //FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
            FAPI_SYS_PRINT_DEBUG(1, "[aal][AAL_Init] Open DD audio decode OK!\n");
            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DD] = AAL_AUDEC_DECODER_DD;
            FAPEX_AUDEC_Close(fd_AUDEC_DD);
         }
#endif

#ifndef _LINUX_FAPI

        /* ----- DD+ ----- */

        audec_open_params.decoder = AAL_AUDEC_DECODER_DDPDCV;

        handleDec = AAL_AUDEC_Open(&audec_open_params, &err_code);

        FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening DD+ decoder:    ");

        if ( handleDec == NULL )
        {
            if ( err_code == AAL_AUDEC_ERR_CODEC_NOT_SUPPORTED )
            {
                FAPI_SYS_PRINT_DEBUG(1, "NOT supported!\n");
                err_code = FAPI_OK; /* This is a warning only! */
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(1, "failed: %i\n", err_code);
                break;
            }
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "OK!\n");

            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DDP] = audec_open_params.decoder;
            (void)AAL_AUDEC_Close(handleDec);
        }

#else
        memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_DDPDCV;
        fd_AUDEC_DDP = FAPEX_AUDEC_Open( DRV_ID_AUDEC_3,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audecOpenParam );
        if(fd_AUDEC_DDP < 0) 
        {
            FAPI_SYS_PRINT_ERR("ERROR: Cannot open DDP audio decoder (code %d).\n", fd_AUDEC_DDP);
            //break;
        }
        else
        {
            //FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
            FAPI_SYS_PRINT_DEBUG(1, "[aal][AAL_Init] Open DDP audio decode OK!\n");

            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DDP] = AAL_AUDEC_DECODER_DDPDCV;

            FAPEX_AUDEC_Close(fd_AUDEC_DDP);
        }
        
#endif

#ifndef _LINUX_FAPI

        /* ----- AC-3 ----- */

        if ( (audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DDP] == AAL_AUDEC_DECODER_UNDEF )
            &&(audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DD] == AAL_AUDEC_DECODER_UNDEF )) 
        {
            audec_open_params.decoder = AAL_AUDEC_DECODER_AC3;

            handleDec = AAL_AUDEC_Open(&audec_open_params, &err_code);

            FAPI_SYS_PRINT_DEBUG(1, "[AAL] Opening AC-3 decoder:   ");

            if ( handleDec == NULL )
            {
                if ( err_code == AAL_AUDEC_ERR_CODEC_NOT_SUPPORTED )
                {
                    FAPI_SYS_PRINT_DEBUG(1, "NOT supported!\n");
                    err_code = FAPI_OK; /* This is a warning only! */

                    /* DDP-decoder is also able to decode AC3 format. */
                    audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3] = audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DDP];
                }
                else
                {
                    FAPI_SYS_PRINT_DEBUG(1, "failed: %i\n", err_code);
                    break;
                }
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(1, "OK!\n");

                audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3]
                                                    = audec_open_params.decoder;
                (void)AAL_AUDEC_Close(handleDec);
            }
        }
        else
        {
            if( audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DD] != AAL_AUDEC_DECODER_UNDEF )
                audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3] = AAL_AUDEC_DECODER_DD;
            else
                audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3] = AAL_AUDEC_DECODER_DDPDCV;

        }
#else
        if ( (audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DDP] == AAL_AUDEC_DECODER_UNDEF )
            &&(audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DD] == AAL_AUDEC_DECODER_UNDEF ))
        {
            memset(&audecOpenParam, 0, sizeof(audecOpenParam));

            audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
            audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_AC3;
            fd_AUDEC_AC3 = FAPEX_AUDEC_Open( DRV_ID_AUDEC_4,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audecOpenParam );
            if(fd_AUDEC_AC3 < 0) 
            {
                FAPI_SYS_PRINT_ERR("ERROR: Cannot open AC3 audio decoder (code %d).\n", fd_AUDEC_AC3);
                //break;
            }
            else
            {
                //FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
                FAPI_SYS_PRINT_DEBUG(1, "[aal][AAL_Init] Open AC3 audio decode OK!\n");
                audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3] = AAL_AUDEC_DECODER_AC3;

                FAPEX_AUDEC_Close(fd_AUDEC_AC3);
            }
        }
        else
        {
            if( audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_DD] != AAL_AUDEC_DECODER_UNDEF )
                audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3] = AAL_AUDEC_DECODER_DD;
            else
                audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_AC3] = AAL_AUDEC_DECODER_DDPDCV;

        }

#endif

#if 0
#ifdef _LINUX_FAPI

        memset(&audecOpenParam, 0, sizeof(audecOpenParam));

        audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
        audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_WMA;
        fd_AUDEC_WMA = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audecOpenParam );
        if(fd_AUDEC_WMA < 0) 
        {
            FAPI_SYS_PRINT_ERR("ERROR: Cannot open WMA audio decoder (code %d).\n", fd_AUDEC_WMA);
            //break;
        }
        else
        {
            //FAPI_SYS_PRINT_DEBUG(1, "OK!\n");
            FAPI_SYS_PRINT_DEBUG(1, "[aal][AAL_Init] Open WMA audio decode OK!\n");
            audioDatPtr->targetDecoder[AAL_AUDIOFORMAT_WMA] = AAL_AUDEC_DECODER_WMA;

            FAPEX_AUDEC_Close(fd_AUDEC_WMA);
        }


#endif
#endif
        
    }
    while ( 0 );

#ifndef _LINUX_FAPI

    if ( handleOut != NULL )
    {
        (void)AAL_AUOUT_Close (handleOut);
    }

    if ( handleBm != NULL )
    {
        (void)FAPI_BM_Close (handleBm);
    }

#endif


    if ( err_code != FAPI_OK )
    {
        (void)AAL_Exit();
    }

    FAPI_SYS_PRINT_DEBUG(1, "[AAL] AAL_Init: err %i \n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Exit/deinitialize AAL module.
**
** \param   -
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \note    This function is never needed if AAL_Init is called only
**          once at system startup, as recommended.
**
** \sa      AAL_Init
*******************************************************************************
*/
int32_t AAL_Exit (void)
{
    if ( audioDatPtr != NULL )
    {
        (void)AAL_Close(&(audioDatPtr->openData));

        if ( audioDatPtr->mutexHandle != NULL )
        {
            (void)FAPEX_SYS_DestroySemaphore(audioDatPtr->mutexHandle);
        }

        FAPI_SYS_FREE(audioDatPtr);

        audioDatPtr = NULL;
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Exit: OK \n");

    return FAPI_OK;
}

/*!
*******************************************************************************
** \brief Audio format check.
**
** This function checks if the given audio format is decodable.
**
** \param format        Audio format that needs to be checked
** \param resultPtr     Pointer to the result if the given format is decodable;
**                      1: decodable, 0: not decodable.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_FormatIsDecodable (AAL_AudioFormatT format, uint8_t* resultPtr)
{
    int32_t          err_code = FAPI_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( (resultPtr == NULL)
           || (((int32_t)format) < 0)
           || (format >= AAL_AUDIOFORMAT_NUMS) )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else
    {
        if ( audioDatPtr->targetDecoder[format] == AAL_AUDEC_DECODER_UNDEF )
        {
            *resultPtr = 0;
        }
        else
        {
            *resultPtr = 1;
        }
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Open AAL module.
**
** \param   paramsPtr   Pointer to open parameters.
** \param   errCodePtr  Optionally returns error_status. Set to NULL if not
**                      needed.
**
** \return  NULL in case of failure, handle for further usage on success.
**
** \note    This shall be called before any audio decoding (for live or one of
**          the some other playback modes like MP3) is started. The returned
**          pointer is used for all further operations related to decoding
**          until the module is closed by AAL_Close.
**
** \sa      AAL_Close
*******************************************************************************
*/
FAPI_SYS_HandleT AAL_Open (const AAL_OpenParamsT* paramsPtr, int32_t* errCodePtr)
{
    int32_t             err_code = FAPI_OK;
    AAL_OpenT*          openPtr  = NULL;
    AAL_AudioPathT      pathCnt;
    AAL_AudioPathOpenT* pathPtr;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( paramsPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else if ( paramsPtr->numSpeakers > AAL_AUOUT_SPEAKER_ALL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else
    {        
    
        #ifndef _LINUX_FAPI

        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        if ( audioDatPtr->openData.open )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_ALREADY_OPEN;
        }
        else
        {
            openPtr = &(audioDatPtr->openData);

            openPtr->params = *paramsPtr;

            /* set default SPDIF mode */
            openPtr->spdifMode = AAL_SPDIFMODE_PCM;

            for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS;
                  pathCnt++ )
            {
                pathPtr = openPtr->path + pathCnt;

                pathPtr->started = 0;
                pathPtr->format  = AAL_AUDIOFORMAT_NONE;
                pathPtr->decoder = AAL_AUDEC_DECODER_UNDEF;
                pathPtr->pid     = 0;

                pathPtr->handleBm = aalOpenBm (audioDatPtr, pathCnt, &(pathPtr->bmSize), &err_code);
                if ( pathPtr->handleBm == NULL )
                {
                    break;
                }

            }

            if ( err_code != FAPI_OK )
            {
                for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS;
                      pathCnt++ )
                {
                    pathPtr = openPtr->path + pathCnt;

                    if (pathPtr->handleBm != NULL )
                    {
                        (void)FAPI_BM_Close(pathPtr->handleBm);

                        pathPtr->handleBm = NULL;
                        pathPtr->bmSize   = 0;
                    }
                }
            }
            else
            {
                openPtr->open = 1;
            }
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);   

        #else
        

        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);
        

        if ( audioDatPtr->openData.open )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_ALREADY_OPEN;
        }
        else
        {
            openPtr = &(audioDatPtr->openData);

            openPtr->params = *paramsPtr;

            /* set default SPDIF mode */
            openPtr->spdifMode = AAL_SPDIFMODE_PCM;
            
            if(audioTsdOpen() == 0) 
            {
                 if (0 == audioOutOpen())
                 {
                    openPtr->open = 1;
                 }
                 else
                 {
                    err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
                 }
              
            }
            else
            {
                err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
            }
        }
        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
        #endif

    }

    FAPI_SYS_PRINT_DEBUG(1, "[AAL] AAL_Open: err %i \n", err_code);

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    return (FAPI_SYS_HandleT)openPtr;
}

/*!
*******************************************************************************
** \brief   Close AAL module.
**
** \param   handle     Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      AAL_Open
*******************************************************************************
*/
int32_t AAL_Close (FAPI_SYS_HandleT handle)
{
    int32_t             err_code = FAPI_OK;
    AAL_OpenT*          openPtr;
    AAL_AudioPathT      pathCnt;
    AAL_AudioPathOpenT* pathPtr;

#ifndef _LINUX_FAPI

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        (void)aalIntStop (openPtr, AAL_PATH_MAIN);
        (void)aalIntStop (openPtr, AAL_PATH_SPDIF);
        for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS;
              pathCnt++ )
        {
            pathPtr = openPtr->path + pathCnt;

            if ( pathPtr->handleBm != NULL )
            {
            
                (void)FAPI_BM_Close (pathPtr->handleBm);
                
                pathPtr->handleBm = NULL;
                pathPtr->bmSize   = 0;
            }
        }

        openPtr->open = 0;

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }
    
#else
    

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL)
    {
        (void)FAPEX_SYS_GetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_SUSPEND);
        
        if(audioOutClose() == 0) 
        {
            if (0 == audioTsdClose())
            {
                openPtr->open = 0;
            }
            else
            {
                err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
            }
           
        }
        else
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
        }
        (void)FAPEX_SYS_SetSemaphore(audioDatPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }
        
#endif

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Close: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Start audio decoding and output for main (I2S) output path.
**
** \param handle    Audio handle returned by AAL_Open.
** \param format    Format of given audio stream.
** \param pid       Audio PID to decode.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Start (FAPI_SYS_HandleT handle, AAL_AudioFormatT format, uint32_t pid)
{
    int32_t     err_code = FAPI_OK;
    
#ifndef _LINUX_FAPI

    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {


        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        err_code = aalIntStart(audioDatPtr, openPtr, format, pid);

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }

#else

    {
         FAPEX_TSD_PidParamStrT  pidParam;
         int32_t result;

         FAPI_SYS_PRINT_DEBUG(1, "\n%lld AUDIO_Start() start\n",
                        FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));

         /* set Audio PES PID */
         pidParam.pid   = pid;
         pidParam.flags = FAPEX_TSD_FLTFLAGS_PCR;
         result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                          FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                          (FAPEXIOCARG3_T)&pidParam );
        if(result < 0) 
        {
            FAPI_SYS_PRINT_ERR("ERROR Set Audio Pid (code %d).\n", result);
            return result;
        }

        /* start TSD channel */
        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                          FAPEX_IOCCOM_START,
                          (FAPEXIOCARG3_T)NULL );
        if(result < 0) 
        {
            FAPI_SYS_PRINT_ERR("ERROR Start Audio TSD channel (code %d).\n", result);
            return result;
        }

        /* select decoder */
        switch(format) 
        {
            case AAL_AUDIOFORMAT_AC3:
            if(fd_AUDEC_AC3 >= 0) 
            {
                fd_AUDEC0 = fd_AUDEC_AC3;
            }
            else if(fd_AUDEC_DD >= 0)
            {
                fd_AUDEC0 = fd_AUDEC_DD;
            }
            else 
            {
                fd_AUDEC0 = fd_AUDEC_DDP;
            }
            break;
            
            case AAL_AUDIOFORMAT_AAC:
            if(fd_AUDEC_HEAAC >= 0) 
            {
                fd_AUDEC0 = fd_AUDEC_HEAAC;
            }
            break;

            case AAL_AUDIOFORMAT_HEAAC:
            fd_AUDEC0 = fd_AUDEC_HEAAC;
            break;
            
            case AAL_AUDIOFORMAT_DDP:
            fd_AUDEC0 = fd_AUDEC_DDP;
            break;
         
            case AAL_AUDIOFORMAT_DD:
            fd_AUDEC0 = fd_AUDEC_DD;

            case AAL_AUDIOFORMAT_WMA:
            fd_AUDEC0 = fd_AUDEC_WMA;
            break;  
            default:
            fd_AUDEC0 = fd_AUDEC_MPEG;
            break;
         }

         if ( fd_AUDEC0 < 0 ) 
         {
            result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_STOP,
                              (FAPEXIOCARG3_T)NULL );
            FAPI_SYS_PRINT_MSG( "ERROR: audio stream type 0x%x not supported. (%d)\n",
                        format, result );
            return fd_AUDEC0;
         }

         /* start Audio Decoder */
         result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC0,
                              FAPEX_IOCCOM_START,
                              (FAPEXIOCARG3_T)NULL );
         if(result < 0) 
         {
            FAPI_SYS_PRINT_ERR("ERROR Start AUDEC driver (code %d).\n", result);
         }
         else
         {
            FAPI_SYS_PRINT_MSG( "Audio decoder start with status %d.\n", result );
            result = AAL_SetVolume(100,100);
            printf("[AAL] AAL_SetVolume >> ret:[%d]\n",result);
         }
    }

#endif

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Start(format %i pid %i): %i\n",
                         format, pid, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Stop audio decoding and output for main (I2S) output path.
**
** \param handle    Audio handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Stop (FAPI_SYS_HandleT handle)
{
    int32_t             err_code   = FAPI_OK;
    AAL_OpenT*  openPtr = NULL;
#ifndef _LINUX_FAPI

    AAL_AudioFormatT    currFormat = AAL_AUDIOFORMAT_NONE;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        if ( (openPtr->path[AAL_PATH_MAIN].started)
          && (openPtr->spdifMode == AAL_SPDIFMODE_PASSTHROUGH)
          && (!(openPtr->path[AAL_PATH_SPDIF].started))
          && (openPtr->path[AAL_PATH_MAIN].pid
          == openPtr->path[AAL_PATH_SPDIF].pid) )
        {
            currFormat = openPtr->path[AAL_PATH_MAIN].format;
        }

        err_code = aalIntStop (openPtr, AAL_PATH_MAIN);

        if ( currFormat != AAL_AUDIOFORMAT_NONE )
        {
            (void)aalIntStartSPDIF (audioDatPtr, openPtr, currFormat,
                                     openPtr->path[AAL_PATH_SPDIF].pid);
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }

#else

    {
        int32_t result = 0;
        openPtr = aalCheckHandle(handle, &err_code);

        /* stop Auout hdmi*/
        if(NULL != openPtr)
        {
            (void)aalStopHdmi (openPtr);
        }
 
        /* stop Audio Output */
        if(fd_AUOUT0 >= 0) 
        {
            result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                      FAPEX_IOCCOM_STOP,
                                     (FAPEXIOCARG3_T)NULL );
            if(result < 0) 
            {
                FAPI_SYS_PRINT_ERR("ERROR Stop AUOUT driver (code %d).\n", result);
                return result;
            }
        }

        /* stop Audio Decoder */
        if(fd_AUDEC0 >= 0) 
        {
            result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC0,
                                      FAPEX_IOCCOM_STOP,
                                     (FAPEXIOCARG3_T)NULL );
            if(result < 0) 
            {
               FAPI_SYS_PRINT_ERR("ERROR Stop AUDEC driver (code %d).\n", result);
               return result;
            }
        }

        /* stop TSD channel */
        if(fd_TSD0_AUDIO >= 0) 
        {
            result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                                 FAPEX_IOCCOM_STOP,
                                (FAPEXIOCARG3_T)NULL );
            if(result < 0) 
            {
                FAPI_SYS_PRINT_ERR("ERROR Stop Audio TSD channel (code %d).\n", result);
            }
        }
    }

#endif


    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Stop: %i\n", err_code);

    return err_code;
}




/*!
*******************************************************************************
** \brief Set SPDIF operation mode.
**
** \param handle    Audio handle returned by AAL_Open.
** \param mode      Sets SPDIF output either to off, passthrough mode or I2S
**                  mode.
** \param format    Format of passthrough stream, only needed for mode
**                  AAL_SPDIFMODE_PASSTHROUGH.
** \param pid       PID of passthrough stream, only needed for mode
**                  AAL_SPDIFMODE_PASSTHROUGH.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_SetSPDIFMode (FAPI_SYS_HandleT handle, AAL_SPDIFModeT mode, AAL_AudioFormatT format, uint32_t pid)
{
    int32_t             err_code = FAPI_OK;

#ifndef _LINUX_FAPI

    AAL_OpenT*          openPtr;
    AAL_AudioPathOpenT* pathMainPtr;
    AAL_AudioPathOpenT* pathSpdifPtr;
    AAL_AudioFormatT    currFormat;
    uint32_t            currPid;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        pathMainPtr  = openPtr->path + AAL_PATH_MAIN;
        pathSpdifPtr = openPtr->path + AAL_PATH_SPDIF;

        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        switch ( mode )
        {
            case AAL_SPDIFMODE_OFF:
                err_code = aalIntStop (openPtr, AAL_PATH_SPDIF);

                if ( err_code == FAPI_OK )
                {
                    openPtr->spdifMode = mode;
                }
                break;

            case AAL_SPDIFMODE_PASSTHROUGH:

                if ( pathSpdifPtr->started )
                {
                    /* SPDIF audio is already running in passthrough mode. */
                    if ( pathSpdifPtr->pid != pid )
                    {
                        err_code = aalIntStop (openPtr, AAL_PATH_SPDIF);

                        if ( err_code != FAPI_OK )
                        {
                            break;
                        }
                    }
                    else
                    {
                        /* nothing to do */
                        break;
                    }
                }

                if ( (pathMainPtr->started)
                  && (pathMainPtr->pid == pid) )
                {
                    if ( (pathMainPtr->format != AAL_AUDIOFORMAT_AC3)
                      && (pathMainPtr->format != AAL_AUDIOFORMAT_DDP) )
                    {
                        err_code = SUBSYS_AUDIO_AAL_ERR_FEATURE_NOT_SUPPORTED;
                    }
                    else
                    {
                        openPtr->spdifMode   = mode;

                        pathSpdifPtr->format = format;
                        pathSpdifPtr->pid    = pid;

                        currFormat = pathMainPtr->format;
                        currPid    = pathMainPtr->pid;

                        if ( aalIntStop (openPtr, AAL_PATH_MAIN) == FAPI_OK )
                        {
                            err_code = aalIntStart(audioDatPtr, openPtr,
                                                    currFormat, currPid);
                        }
                    }
                }
                else
                {
                    if ( format != AAL_AUDIOFORMAT_AC3 )
                    {
                        err_code = SUBSYS_AUDIO_AAL_ERR_FEATURE_NOT_SUPPORTED;
                    }
                    else
                    {
                        openPtr->spdifMode   = mode;

                        pathSpdifPtr->format = format;
                        pathSpdifPtr->pid    = pid;

                        if ( (pathMainPtr->started)
                          && (pathMainPtr->enableSpdif) )
                        {
                            currFormat = pathMainPtr->format;
                            currPid    = pathMainPtr->pid;

                            if ( aalIntStop  (openPtr, AAL_PATH_MAIN)
                                 == FAPI_OK )
                            {
                                err_code = aalIntStart (audioDatPtr, openPtr,
                                                         currFormat, currPid);
                            }
                        }

                        err_code = aalIntStartSPDIF (audioDatPtr, openPtr,
                                                      format, pid);
                    }
                }
                break;

            case AAL_SPDIFMODE_PCM:

                if ( openPtr->spdifMode != mode )
                {
                    /* Stop passthrough SPDIF audio. */
                    err_code = aalIntStop (openPtr, AAL_PATH_SPDIF);

                    if ( err_code != FAPI_OK )
                    {
                        break;
                    }

                    openPtr->spdifMode   = mode;

                    pathSpdifPtr->format = AAL_AUDIOFORMAT_NONE;
                    pathSpdifPtr->pid    = 0;

                    if ( pathMainPtr->started )
                    {
                        currFormat = pathMainPtr->format;
                        currPid    = pathMainPtr->pid;

                        (void)aalIntStop  (openPtr, AAL_PATH_MAIN);
                        (void)aalIntStart (audioDatPtr, openPtr,
                                            currFormat, currPid);
                    }
                }
                break;

            default:
                err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
                break;
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }

#else

#endif


    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_SetSPDIFMode(mode %i format %i pid %i: %i\n",
                         mode, format, pid, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Set stereo mode.
**
** \param mode      Requested stereo mode.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_SetStereoMode (AAL_StereoModeT mode)
{
    int32_t err_code = FAPI_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( mode >= AAL_STEREO_MODE_NUMS )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else
    {
    
#ifndef _LINUX_FAPI

    
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        audioDatPtr->stereoMode = mode;

        if ( (audioDatPtr->openData.open)
          && (audioDatPtr->openData.path[AAL_PATH_MAIN].started) )
        {
            (void)aalIntSetVolAndStereoMode (audioDatPtr,
                                              &(audioDatPtr->openData));
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);

#else

{
    int32_t result;
    FAPEX_AUOUT_DualMonoParamStrT DualMonoParam;

    switch (mode)
    {
        default:
        case AAL_STEREO_MODE_STEREO:
            DualMonoParam.channelFlag = FAPEX_AUOUT_CHANNEL_ALL;
            break;

        case AAL_STEREO_MODE_MONO_LEFT:
            DualMonoParam.channelFlag = FAPEX_AUOUT_CHANNEL_LEFT;
            break;

        case AAL_STEREO_MODE_MONO_RIGHT:
            DualMonoParam.channelFlag = FAPEX_AUOUT_CHANNEL_RIGHT;
            break;

        case AAL_STEREO_MODE_MONO_LEFT_ONLY:
            DualMonoParam.channelFlag = FAPEX_AUOUT_CHANNEL_LEFT;
            break;

        case AAL_STEREO_MODE_MONO_RIGHT_ONLY:
            DualMonoParam.channelFlag = FAPEX_AUOUT_CHANNEL_RIGHT;
            break;
    }
    
    DualMonoParam.enableFlag = 1;
    FAPI_SYS_PRINT_DEBUG(1, "[aal][AAL_SetStereoMode] fd_AUOUT0 = %d\n", fd_AUOUT0);
    result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                FAPEX_IOCCOM_AUOUT_SET_DUALMONO,
                                (FAPEXIOCARG3_T)&DualMonoParam );
     
    if(result < 0) {
         FAPI_SYS_PRINT_ERR("ERROR Set AUOUT DualMono. %d\n", result);
         return;
    }

}    

#endif
        
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_SetStereoMode(%i): %i\n",
                         mode, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Feed the input buffer for main audio (I2S) decoding.
**
** \param   handle      Handle returned by AAL_Open.
** \param   bufferPtr   The pointer points to the data buffer feeding to
**                      the audio buffer.
** \param   bytes       The size of the audio data buffer in bytes.
**
** \note This function is only needed for feed mode AAL_FEEDMODE_BM.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_FeedBuffer (FAPI_SYS_HandleT handle, const void* bufferPtr, int32_t bytes)
{
    int32_t     err_code = FAPI_OK;
    int32_t     retVal;

#ifndef _LINUX_FAPI

    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        if ( (openPtr->params.feedMode == AAL_FEEDMODE_BM)
          && (bytes >= 0) )
        {
            retVal = FAPI_BM_Write(openPtr->path[AAL_PATH_MAIN].handleBm,
                                   (void*)bufferPtr,
                                   (uint32_t)bytes, 1);
            if ( retVal != bytes )
            {
                err_code = retVal;
            }
        }
        else
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
        }
    }

#else
   #if 1
    FAPEX_TSD_BmWriteParmStrT  bmParam;

    memset(&bmParam, 0, sizeof(FAPEX_TSD_BmWriteParmStrT));

    bmParam.bufferLen = bytes;
    bmParam.bufferPtr = bufferPtr;
    bmParam.timeout   = 500;

    /* write Audio BM buffer */
    retVal = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_TSD_BM_WRITE,
                              (FAPEXIOCARG3_T)&bmParam );
    
    if(retVal < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Write Audio BM buffer (code %d).\n", retVal);
        return retVal;
    }
   #else
    FAPEX_TSD_WriteInfoStrT writeInfo;
    uint32_t start_time,stop_time;
    uint32_t bytes_written;
    volatile uint8_t*   data8_ptr          = NULL;
    volatile uint32_t*  data32_ptr         = NULL;
    uint32_t time_difference = 0;
    uint32_t  write_ptr_position = 0;
    uint32_t  fillSize = 0;
    uint32_t timeout = 500;
    /*check param */
    if((NULL == audio_mmapbuf) || (0 == bytes))
    {
        return -1;
    }
    
    retVal = FAPEX_TSD_Ioctl((FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO,
                              (FAPEXIOCARG3_T)&writeInfo );
    if(retVal < 0) {
       FAPI_SYS_PRINT_ERR("ERROR Get Audio BM buffer Write info (code %d).\n", retVal);
        return retVal; 
    }
    
    write_ptr_position = writeInfo.writeOffset;
    fillSize = writeInfo.fillSize;
    
    if(write_ptr_position >= FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES)
        return FAPEX_TSD_ERR_OUT_OF_MEMORY;
    
    start_time = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
    
    while(((FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES - fillSize) < (bytes+1)) &&
             (time_difference < timeout))
    {
        stop_time = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
        time_difference = (int32_t)(stop_time - start_time);
        if(start_time > stop_time)
            time_difference = ((time_difference < 0L) ? -time_difference : time_difference); /* C-LIB code for labs() */
     
        retVal = FAPEX_TSD_Ioctl((FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_TSD_GET_BUFFER_WRITEINFO,
                              (FAPEXIOCARG3_T)&writeInfo );
        if(retVal < 0) {
           FAPI_SYS_PRINT_ERR("ERROR Get Audio BM buffer Write info (code %d).\n", retVal);
            return retVal; 
        }
        fillSize = writeInfo.fillSize;
    
    }

    /* timeout occurred */
    if( time_difference >= timeout)
        return(FAPEX_TSD_ERR_TIMEOUT);

    
     /* get address to write to */
    data8_ptr = (volatile uint8_t*)audio_mmapbuf;
    data8_ptr += write_ptr_position;

    bytes_written = 0;
    /* check alignment */
    if ( !((uint32_t)data8_ptr & 0x3) && !((uint32_t)bufferPtr & 0x3))
    {
        data32_ptr = (volatile uint32_t*)(void*)data8_ptr;

        /* read bytes from buffer (4 bytes at a time) */
        while((bytes_written + 4) <= bytes)
        {
            /* divide amount of bytes by 4 for 32 bit index */
            *data32_ptr++ = ((uint32_t*)bufferPtr)[(bytes_written >> 2)];
            /* increase data8_ptr for later usage */
            data8_ptr += 4;
            write_ptr_position += 4;
            bytes_written += 4;

            /* reset if end of buffer is reached */
            if(write_ptr_position ==FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES )
            {
                write_ptr_position = 0;
                data8_ptr = (volatile uint8_t*)audio_mmapbuf;

               /* check alignment */
                if(!((uint32_t)data8_ptr & 0x3))
                    /* is aligned */
                    data32_ptr = (volatile uint32_t*)(void*)data8_ptr;
                else
                    break;
            }
        }
    }

    /* write remaining bytes in buffer (1 byte at a time) */
    for(; bytes_written < bytes; bytes_written++)
    {
        *data8_ptr++ = ((uint8_t*)bufferPtr)[bytes_written];
        write_ptr_position++;

        /* reset if end of buffer is reached */
        if(write_ptr_position == FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES)
        {
            write_ptr_position = 0;
            data8_ptr = (volatile uint8_t*)audio_mmapbuf;
        }
    }
    
    retVal = FAPEX_TSD_Ioctl((FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_TSD_SET_BUFFER_WRITEPOINTEROFFSET,
                              (FAPEXIOCARG3_T)write_ptr_position );
    if(retVal < 0) {
       FAPI_SYS_PRINT_ERR("ERROR Set Audio BM buffer Write Pointer (code %d).\n", retVal);
       return retVal; 
    }
   #endif

#endif

    return err_code;
}

/*!
*******************************************************************************
** \brief   Clear the input buffer for main audio (I2S) decoding.
**
** \param   handle      Handle returned by AAL_Open.
**
** \note This function is only needed for feed mode AAL_FEEDMODE_BM.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_ClearBuffer (FAPI_SYS_HandleT handle)
{
    int32_t     err_code = FAPI_OK;

#ifndef _LINUX_FAPI

    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)FAPI_BM_SetBufferReadPointer  (openPtr->path[AAL_PATH_MAIN].
                                             handleBm, 0, 0);
        (void)FAPI_BM_SetBufferWritePointer (openPtr->path[AAL_PATH_MAIN].
                                             handleBm, 0, 0);
    }
    else
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }

#else


#endif

    return err_code;
}

/*!
*****************************************************************************
** \brief Get status of audio decoder.
**
** This function provides the status of the main audio (I2S) decoding.
**
** \param handle    Audio handle returned by AAL_Open.
** \param statusPtr Pointer to the current decoder status.
**
** \return  Possible return codes:
**          - FAPI_OK                             Successful completion.
**          - SUBSYS_AUDIO_AAL_DATA_NOT_AVAILABLE Decoding is inactive.
*****************************************************************************
*/
int32_t AAL_GetDecStatus (FAPI_SYS_HandleT handle, AAL_AUDEC_StatusEnumT* statusPtr)
{
    int32_t     err_code = FAPI_OK;


#ifndef _LINUX_FAPI
    
    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        if ( openPtr->path[AAL_PATH_MAIN].handleDec == NULL )
        {
            err_code = SUBSYS_AUDIO_AAL_DATA_NOT_AVAILABLE;
        }
        else if ( statusPtr != NULL )
        {
            *statusPtr = AAL_AUDEC_GetStatus (openPtr->path[AAL_PATH_MAIN].
                                               handleDec);
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }

#else

    {
        int32_t  result;
        FAPEX_AUDEC_StatusStrT status;
    
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC0,
                                    FAPEX_IOCCOM_AUDEC_GET_STATUS,
                                    (FAPEXIOCARG3_T)&status);

        if (result < 0)
        {
            err_code = SUBSYS_AUDIO_AAL_DATA_NOT_AVAILABLE;
        }
        
        *statusPtr = status.firmStatus;
    }

#endif

    return err_code;
}

/*!
*****************************************************************************
** \brief Get stream information on currently decoded main audio (I2S) stream.
**
** \param handle    Audio handle returned by AAL_Open.
** \param infoPtr   Pointer to the current decoder info.
**
** \return  Possible return codes:
**          - FAPI_OK                             Successful completion.
**          - SUBSYS_AUDIO_AAL_DATA_NOT_AVAILABLE Decoding has not been started.
*****************************************************************************
*/
int32_t AAL_GetStreamInfo (FAPI_SYS_HandleT handle, AAL_AUOUT_StreamInfoT* infoPtr)
{
    int32_t                 err_code = FAPI_OK;
    AAL_AUOUT_StreamInfoT  streamInfo;


#ifndef _LINUX_FAPI
    
    AAL_OpenT*              openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        if ( openPtr->path[AAL_PATH_MAIN].started )
        {
            streamInfo = AAL_AUOUT_GetStreamInfo (openPtr->path[AAL_PATH_MAIN].
                                                   handleOut);

            if ( infoPtr != NULL )
            {
                *infoPtr = streamInfo;
            }
        }
        else
        {
            err_code = SUBSYS_AUDIO_AAL_DATA_NOT_AVAILABLE;
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }

#else
{
    int32_t result;
    int32_t stream_info;
    FAPEX_AUOUT_PcmInfoStrT pcmInfoParam;
    
    result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                FAPEX_IOCCOM_AUOUT_GET_STREAM_TYPE,
                                (FAPEXIOCARG3_T)&stream_info );

    result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                FAPEX_IOCCOM_AUOUT_GET_PCM_INFO,
                                (FAPEXIOCARG3_T)&pcmInfoParam );

    streamInfo.sampleBits = pcmInfoParam.sampleBits;
    streamInfo.sampleFreq = pcmInfoParam.sampleFreq;

    switch (stream_info)
    {
        case FAPEX_AUOUT_STREAM_UNDEF:
        streamInfo.type = AAL_AUOUT_STREAM_UNDEF;
            break;
        case FAPEX_AUOUT_STREAM_AAC:
        streamInfo.type = AAL_AUOUT_STREAM_AAC;
            break;
        case FAPEX_AUOUT_STREAM_AC3:
        streamInfo.type = AAL_AUOUT_STREAM_AC3;
            break;
        case FAPEX_AUOUT_STREAM_MPEG_L1:
        streamInfo.type = AAL_AUOUT_STREAM_MPEG_L1;
            break;
        case FAPEX_AUOUT_STREAM_MPEG_L2:
        streamInfo.type = AAL_AUOUT_STREAM_MPEG_L2;
            break;
        case FAPEX_AUOUT_STREAM_MPEG_L3:
        streamInfo.type = AAL_AUOUT_STREAM_MPEG_L3;
            break;
        case FAPEX_AUOUT_STREAM_WMA:
        streamInfo.type = AAL_AUOUT_STREAM_WMA;
            break;

        default:
        streamInfo.type = AAL_AUOUT_STREAM_UNDEF;
            break;
    }
    
    if(result < 0) {
         FAPI_SYS_PRINT_ERR("ERROR Get Stream Info. %d\n", result);
    }
    else
    {
        if ( infoPtr != NULL )
        {
            *infoPtr = streamInfo;
        }
    }
}
    

#endif

    return err_code;
}

/*!
*******************************************************************************
** \brief   Function that handles audio-related tasks in application context.
**
** Should be regularly called by application, handles starting audec and HDMI
** audio check
**
** \param   handle           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Task (FAPI_SYS_HandleT handle)
{
    int32_t     err_code = FAPI_OK;

#ifndef _LINUX_FAPI
    
    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        if ( openPtr->path[AAL_PATH_MAIN].started )
        {
            if ( (!(openPtr->audecRunning))
                && (openPtr->audecRestart) )
            {
                err_code = AAL_AUDEC_Start (openPtr->path[AAL_PATH_MAIN].
                                             handleDec);

                if ( err_code == FAPI_OK )
                {
                    openPtr->audecRunning = 1;
                    openPtr->audecRestart = 0;
                }
            }

            if (   (openPtr->audecRunning)
              && (!(openPtr->hdmiRunning)) )
            {
                (void)aalStartHdmi(audioDatPtr, openPtr);
            }
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }

#else

    AAL_OpenT*  openPtr;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        if ( openPtr->path[AAL_PATH_MAIN].started )
        {
#if 0
            if ( (!(openPtr->audecRunning))
                && (openPtr->audecRestart) )
            {
                err_code = AAL_AUDEC_Start (openPtr->path[AAL_PATH_MAIN].
                                             handleDec);

                if ( err_code == FAPI_OK )
                {
                    openPtr->audecRunning = 1;
                    openPtr->audecRestart = 0;
                }
            }
#endif
            if (   (openPtr->audecRunning)
              && (!(openPtr->hdmiRunning)) )
            {
                (void)aalStartHdmi(audioDatPtr, openPtr);
            }
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }
#endif
    return err_code;
}

/*!
*******************************************************************************
** \brief   Start/Pause audio decoding.
**
** \param   handle            Handle returned by AAL_Open.
** \param   enterOrExit       The command to play or pause audio decoding.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Pause (FAPI_SYS_HandleT handle, uint8_t enterOrExit)
{
    int32_t             err_code = FAPI_OK;

#ifndef _LINUX_FAPI

    AAL_AudioPathOpenT* pathPtr;
    AAL_OpenT*          openPtr;
    FAPI_SYS_HandleT    spdifBufferHandle = NULL;

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr != NULL )
    {
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        /* handle main audio */
        pathPtr = openPtr->path + AAL_PATH_MAIN;

        if ( pathPtr->started )
        {
            spdifBufferHandle = AAL_AUOUT_GetSpdifBuffer (pathPtr->handleOut);

            if ( enterOrExit ) /* enter pause */
            {
                AAL_AUOUT_Stop(pathPtr->handleOut);

                /* stop SPDIF correctly */
                if ( spdifBufferHandle != NULL )
                {
                    (void)FAPI_BM_SetReadEnable (spdifBufferHandle, 0);
                }
                (void)aalStopHdmi(openPtr);
            }
            else /* exit pause */
            {
                (void)AAL_AUDEC_InvalidatePts(pathPtr->handleDec);

                err_code = AAL_AUOUT_Start(pathPtr->handleOut);

                if ( err_code == FAPI_OK )
                {
                    /* start SPDIF correctly */
                    if ( spdifBufferHandle != NULL )
                    {
                        (void)FAPI_BM_SetReadEnable (spdifBufferHandle, 1);
                    }

                    err_code = aalStartHdmi (audioDatPtr, openPtr);
                }
            }
        }

        /* handle SPDIF output */
        pathPtr = openPtr->path + AAL_PATH_SPDIF;

        if ( pathPtr->started )
        {
            spdifBufferHandle = AAL_AUOUT_GetSpdifBuffer (pathPtr->handleOut);

            if ( enterOrExit ) /* enter pause */
            {
                AAL_AUOUT_Stop(pathPtr->handleOut);

                /* stop SPDIF correctly */
                if ( spdifBufferHandle != NULL )
                {
                    (void)FAPI_BM_SetReadEnable (spdifBufferHandle, 0);
                }
            }
            else /* exit pause */
            {
                (void)AAL_AUDEC_InvalidatePts(pathPtr->handleDec);

                err_code = AAL_AUOUT_Start(pathPtr->handleOut);

                if ( err_code == FAPI_OK )
                {
                    /* start SPDIF correctly */
                    if ( spdifBufferHandle != NULL )
                    {
                        (void)FAPI_BM_SetReadEnable (spdifBufferHandle, 1);
                    }
                }
            }
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }

#else

{
    int32_t result;

    if(enterOrExit)
    {
        result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                      FAPEX_IOCCOM_PAUSE,
                                     (FAPEXIOCARG3_T)NULL );
    }
    else
    {
        result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                      FAPEX_IOCCOM_RESET,
                                      (FAPEXIOCARG3_T)NULL );
    }

    if (result < 0)
    {
        return FAPI_ERR_INVALID_HANDLE;
    }
}

#endif

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Pause: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Mute/Unmute audio output.
**
** This function mutes or unmutes the audio output.
**
** \param mute    Mute or unmute command.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_Mute (uint8_t mute)
{
    int32_t         err_code = FAPI_OK;

#ifndef _LINUX_FAPI
    
    AAL_AudioPathT  pathCnt;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        for ( pathCnt = (AAL_AudioPathT)0; pathCnt < AAL_PATH_NUMS;
              pathCnt++ )
        {
            audioDatPtr->path[pathCnt].muted = mute;

            if ( (audioDatPtr->openData.open)
              && (audioDatPtr->openData.path[pathCnt].started) )
            {
                if ( mute )
                {
                    AAL_AUOUT_Mute (audioDatPtr->openData.path[pathCnt].
                                     handleOut, AAL_AUOUT_SPEAKER_ALL);
                }
                else
                {
                    AAL_AUOUT_UnMute (audioDatPtr->openData.path[pathCnt].
                                       handleOut, AAL_AUOUT_SPEAKER_ALL);
                }
            }
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }

#else

{
    int32_t result;
    FAPEX_AUOUT_MuteParamStrT muteParam;
        
    if ( mute )
    {
        muteParam.muteFlag = 0;
    }
    else
    {
        muteParam.muteFlag = 1;
    }

    muteParam.channelFlag = FAPEX_AUOUT_CHANNEL_ALL;
    
    result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                FAPEX_IOCCOM_AUOUT_SET_MUTE,
                                (FAPEXIOCARG3_T)&muteParam );
     
    if(result < 0) {
         FAPI_SYS_PRINT_ERR("ERROR Set AUOUT volume. %d\n", result);
         return;
    }

}

#endif

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_Mute(%i): %i\n", mute, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Enable/Disable HDMI output.
**
** This function enables or disables the HDMI output.
**
** \param enable    The command to enable or disable the HDMI output.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_EnableHdmi (uint8_t enable)
{
    int32_t err_code = FAPI_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        //#ifndef _LINUX_FAPI
        
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        if ( audioDatPtr->hdmiEnabled != enable )
        {
            audioDatPtr->hdmiEnabled = enable;

            if ( (audioDatPtr->openData.open)
              && (audioDatPtr->openData.path[AAL_PATH_MAIN].started) )
            {
                if ( enable )
                {
                    err_code = aalStartHdmi (audioDatPtr,
                                              &(audioDatPtr->openData));
                }
                else
                {
                    err_code = aalStopHdmi (&(audioDatPtr->openData));
                }
            }
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);

        //#else

        

        //#endif
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_EnableHdmi(%i): %i\n", enable,
                         err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief Set audio volume.
**
** This function sets the audio volume on the left and right speaker.
**
** \param lvlLeft    The volume level on the left speaker.
** \param lvlRight   The volume level on the right speaker.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
int32_t AAL_SetVolume (uint8_t lvlLeft, uint8_t lvlRight)
{
    int32_t err_code = FAPI_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        #ifndef _LINUX_FAPI
        
        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        /* Avoid 0xFF being passed to the audio output. */
        if ( lvlLeft > AAL_VOLUMELEVEL_MAX )
        {
            lvlLeft = AAL_VOLUMELEVEL_MAX;
        }

        if ( lvlRight > AAL_VOLUMELEVEL_MAX )
        {
            lvlRight = AAL_VOLUMELEVEL_MAX;
        }

        audioDatPtr->lvlLeft  = lvlLeft;
        audioDatPtr->lvlRight = lvlRight;

        if ( (audioDatPtr->openData.open)
          && (audioDatPtr->openData.path[AAL_PATH_MAIN].started) )
        {
            (void)aalIntSetVolAndStereoMode (audioDatPtr,
                                              &(audioDatPtr->openData));
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);

        #else
        {
            FAPEX_AUOUT_VolumeParamStrT  volumeParam;
            int32_t result;

            /* get volume value */
            volumeParam.channelFlag = FAPEX_AUOUT_CHANNEL_LEFT;
            result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                FAPEX_IOCCOM_AUOUT_GET_VOLUME,
                                (FAPEXIOCARG3_T)&volumeParam );
            if(result < 0) {
                 FAPI_SYS_PRINT_ERR("ERROR Get AUOUT volume. %d\n", result);
                 return;
            }
            printf("[AAL] FAPEX_IOCCOM_AUOUT_GET_VOLUME %d\n", volumeParam.volume);
            volumeParam.volume  = lvlLeft;

            if ( volumeParam.volume < 0 )                volumeParam.volume = 0;
            if ( volumeParam.volume > 0xfe ) volumeParam.volume = 0xfe;

            /* set volume value */
            volumeParam.channelFlag = FAPEX_AUOUT_CHANNEL_ALL;
            result = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                      FAPEX_IOCCOM_AUOUT_SET_VOLUME,
                                      (FAPEXIOCARG3_T)&volumeParam );
            if(result < 0) {
                 FAPI_SYS_PRINT_ERR("ERROR Set AUOUT volume. %d\n", result);
                 return;
            }
            printf("[AAL] FAPEX_IOCCOM_AUOUT_SET_VOLUME %d\n", volumeParam.volume);
       }
    
        #endif
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_SetVolume(%i, %i): %i\n",
                         lvlLeft, lvlRight, err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Return buffer size of audio input buffer.
**
** \param   handle            Handle returned by AAL_Open.
**
** \return  The size of the audio input buffer.
**
** \sa      AAL_GetBufferLevel
*******************************************************************************
*/
uint32_t AAL_GetBufferSize (FAPI_SYS_HandleT handle)
{
    uint32_t    bufSize;
    AAL_OpenT*  openPtr;


#ifndef _LINUX_FAPI

    openPtr = aalCheckHandle(handle, NULL);

    if ( (openPtr != NULL) && (openPtr->path[AAL_PATH_MAIN].started) )
    {
        bufSize = openPtr->path[AAL_PATH_MAIN].bmSize;
    }
    else
    {
        bufSize = 0;
    }

#else

    {
        FAPEX_TSD_ChannelInfoStrT  ch_info;
        int32_t  result;

        if((fd_TSD0_AUDIO < 0) || (fd_AUDEC0 < 0)) {
            return 0;
        }

        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                                  FAPEX_IOCCOM_TSD_GET_CHANNEL_INFO,
                                  (FAPEXIOCARG3_T)&ch_info );
        if(result == 0) {
             return ch_info.bufferSize;
        }
     }

#endif

    return bufSize;
}

/*!
*******************************************************************************
** \brief   Return current filling level of audio input buffer.
**
** \param   handle            Handle returned by AAL_Open.
**
** \return  The filling level of the audio input buffer.
**
** \sa      AAL_GetBufferSize
*******************************************************************************
*/
uint32_t AAL_GetBufferLevel (FAPI_SYS_HandleT handle)
{
    int32_t     result;
    AAL_OpenT*  openPtr;
    uint32_t    level = 0;


    #ifndef _LINUX_FAPI

    openPtr = aalCheckHandle(handle, NULL);

    if ( (openPtr != NULL) && (openPtr->path[AAL_PATH_MAIN].started) )
    {
        AAL_Assert (openPtr->path[AAL_PATH_MAIN].handleBm != NULL);

        result = FAPI_BM_GetBufferFillLevel (openPtr->path[AAL_PATH_MAIN].handleBm);

        if ( result >= 0 )
        {
            level = (uint32_t)result;
        }
    }

    #else
    
    {
        //FAPEX_TSD_ReadInfoStrT  buffer_info;
        int32_t  result;
        uint32_t level = 0;

        if((fd_TSD0_AUDIO < 0) || (fd_AUDEC0 < 0)) {
            return 0;
        }

        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                                  FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                 (FAPEXIOCARG3_T)&level );
        if(result != 0) {
            return 0x1FFFFFFF;
        }
    }
    
    #endif

    return level;
}

/*!
*******************************************************************************
** \brief   Return buffer size of SPDIF input buffer.
**
** \param   handle            Handle returned by AAL_Open.
**
** \return  The size of the SPDIF input buffer.
**
** \sa      AAL_GetBufferLevel
*******************************************************************************
*/
uint32_t AAL_GetSPDIFBufferSize (FAPI_SYS_HandleT handle)
{
    uint32_t    bufSize = 0;
    AAL_OpenT*  openPtr;

#ifndef _LINUX_FAPI

    openPtr = aalCheckHandle(handle, NULL);

    if ( openPtr != NULL )
    {    
        switch ( openPtr->spdifMode )
        {
            default:
            case AAL_SPDIFMODE_OFF:
                bufSize = 0;
                break;

            case AAL_SPDIFMODE_PASSTHROUGH:

                if ( openPtr->path[AAL_PATH_SPDIF].started )
                {
                    bufSize = openPtr->path[AAL_PATH_SPDIF].bmSize;
                }
                else if ( (openPtr->path[AAL_PATH_MAIN].started)
                       && (openPtr->path[AAL_PATH_MAIN].enableSpdif) )
                {
                    bufSize = openPtr->path[AAL_PATH_MAIN].bmSize;
                }
                break;

            case AAL_SPDIFMODE_PCM:
                if ( openPtr->path[AAL_PATH_MAIN].started )
                {
                    bufSize = openPtr->path[AAL_PATH_MAIN].bmSize;
                }
                break;
        }
    }

#else
       
    
#endif
    

    return bufSize;
}

/*!
*******************************************************************************
** \brief   Return current filling level of SPDIF input buffer.
**
** \param   handle            Handle returned by AAL_Open.
**
** \return  The filling level of the SPDIF input buffer.
**
** \sa      AAL_GetBufferSize
*******************************************************************************
*/
uint32_t AAL_GetSPDIFBufferLevel (FAPI_SYS_HandleT handle)
{
    int32_t             result;
    AAL_OpenT*          openPtr;
    uint32_t            level    = 0;
    FAPI_SYS_HandleT    handleBm = NULL;

#ifndef _LINUX_FAPI

    openPtr = aalCheckHandle(handle, NULL);

    if ( openPtr != NULL )
    {    
        switch ( openPtr->spdifMode )
        {
            default:
            case AAL_SPDIFMODE_OFF:
                /* do nothing */
                break;

            case AAL_SPDIFMODE_PASSTHROUGH:

                if ( openPtr->path[AAL_PATH_SPDIF].started )
                {
                    handleBm = openPtr->path[AAL_PATH_SPDIF].handleBm;
                }
                else if ( (openPtr->path[AAL_PATH_MAIN].started)
                       && (openPtr->path[AAL_PATH_MAIN].enableSpdif) )
                {
                    handleBm = openPtr->path[AAL_PATH_MAIN].handleBm;
                }
                break;

            case AAL_SPDIFMODE_PCM:
                if ( openPtr->path[AAL_PATH_MAIN].started )
                {
                    handleBm = openPtr->path[AAL_PATH_MAIN].handleBm;
                }
                break;
        }

        if ( handleBm != NULL )
        {
            result = FAPI_BM_GetBufferFillLevel (handleBm);

            if ( result >= 0 )
            {
                level = (uint32_t)result;
            }
        }
    
    }

#else

#endif
    

    return level;
}

/*!
*******************************************************************************
** \brief   Set the delay in audio.
**
** \param   handle            Handle returned by AAL_Open.
** \param   outputDelayMsec   Requested the delay in milliseconds
**
** \return  Possible return codes:
**          - FAPI_OK                              Successful completion.
**          - SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE  NULL-pointer passed as
**                                                 openPtr.
*******************************************************************************
*/
int32_t AAL_SetOutputDelay (FAPI_SYS_HandleT handle, int32_t outputDelayMsec)
{
    int32_t     err_code = FAPI_OK;
    AAL_OpenT*  openPtr;

#ifndef _LINUX_FAPI

    openPtr = aalCheckHandle(handle, &err_code);

    if ( openPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE;
    }
    else
    {

        (void)RTOS_GetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_SUSPEND);

        if ( openPtr->path[AAL_PATH_MAIN].handleDec == NULL )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE;
        }
        else
        {           
            err_code = AAL_AUDEC_SetOutputDelay (openPtr->path[AAL_PATH_MAIN].handleDec, 
                                                  outputDelayMsec);
        }

        (void)RTOS_SetSemaphore(audioDatPtr->mutexHandle, FAPI_SYS_NO_SUSPEND);
    }

#else
    
                
                
#endif


    return err_code;
}

int32_t AAL_RebootAudec(void)
{
    int32_t err_code = FAPI_OK;

    if (fd_AUDEC0 == -1)
    {
        err_code == SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
        FAPI_SYS_PRINT_MSG("fd_AUDEC0 is not initialized\n");
    }
    else
    {
        err_code = FAPEX_AUDEC_Ioctl(fd_AUDEC0,
                                    FAPEX_IOCCOM_RESET,
                                    (FAPEXIOCARG3_T)NULL);
        if (err_code != FAPI_OK)
        {
            FAPI_SYS_PRINT_ERR("ERROR Reset AUDEC, err_code %d\n", err_code);
        }
        else
        {
             FAPI_SYS_PRINT_MSG("RebootAudec success\n");
        }
    }

    return err_code;
}


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************


#ifndef _LINUX_FAPI


/*
*******************************************************************************
** \brief   Internal function, checks validity of AAL handle.
**
** \param   handle           Pointer to be checked.
** \param   errCodePtr       Possible error codes:
**          - SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED  AAL is not initialized.
**          - SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE   Handle is a NULL pointer.
            - SUBSYS_AUDIO_AAL_ERR_NOT_OPEN         AAL is not open.
**
** \return  Possible return codes:
            - Driver handle (may be NULL)
**
*******************************************************************************
*/
static AAL_OpenT* aalCheckHandle (FAPI_SYS_HandleT handle, int32_t* errCodePtr)
{
    AAL_OpenT*  openPtr  = (AAL_OpenT*)handle;
    int32_t     err_code = FAPI_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( (openPtr == NULL)
           || (openPtr != &(audioDatPtr->openData)) )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE;
    }
    else if ( !(openPtr->open) )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    if ( err_code != FAPI_OK )
    {
        openPtr = NULL;
    }

    return openPtr;
}



/*
*******************************************************************************
** \brief Internal function, starts audio decoding and output.
**
** This function starts the audio decoding process for the given PID.
**
** \param openPtr   Handle returned by AAL_Open.
** \param format    Format of given audio stream.
** \param pid       Audio PID to decode.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
static int32_t aalIntStart (AAL_DataT* inst, AAL_OpenT* openPtr,
                             AAL_AudioFormatT format, uint32_t pid)
{
    AAL_AUOUT_OpenParamsT  auout_open_params;
    int32_t                 err_code        = FAPI_OK;
    AAL_AUDEC_DecoderEnumT decoder         = AAL_AUDEC_DECODER_UNDEF;
    AAL_AudioPathOpenT*     pathMainPtr     = openPtr->path + AAL_PATH_MAIN;
    AAL_AudioPathOpenT*     pathSpdifPtr    = openPtr->path + AAL_PATH_SPDIF;
    uint8_t                 enableSpdif     = 0;
    uint8_t                 ac3ToSpdif      = 0;
    AAL_AudioFormatT        currSpdifFormat = AAL_AUDIOFORMAT_NONE;
    uint32_t                currSpdifPid    = 0;
    uint8_t                 cnt;

    do
    {
        /* already started? */
        if ( pathMainPtr->started )
        {
            /* do not return an error code */
            break;
        }

        openPtr->audecRunning       = 0;
        openPtr->audecRestart       = 0;
        openPtr->hdmiHaveSampleFreq = 0;
        openPtr->hdmiRunning        = 0;

        switch ( openPtr->spdifMode )
        {
            default:
            case AAL_SPDIFMODE_OFF:
                enableSpdif = 0;
                ac3ToSpdif  = 0;
                break;

            case AAL_SPDIFMODE_PASSTHROUGH:
                if ( pathSpdifPtr->pid == pid )
                {
                    /* Stop an independently running SPDIF passthrough decoder.
                    */
                    if ( pathSpdifPtr->started )
                    {
                        currSpdifFormat = pathSpdifPtr->format;
                        currSpdifPid    = pathSpdifPtr->pid;

                        (void)aalIntStop (openPtr, AAL_PATH_SPDIF);
                    }

                    enableSpdif = 1;

                    if ( (format == AAL_AUDIOFORMAT_AC3)
                      || (format == AAL_AUDIOFORMAT_DDP) )
                    {
                        ac3ToSpdif = 1;
                    }
                    else
                    {
                        /* not a format compliant for passthrough, use PCM
                           mode as fallback. */
                        ac3ToSpdif = 0;
                    }
                }
                else
                {
                    enableSpdif = 0;
                    ac3ToSpdif  = 0;
                }
                break;

            case AAL_SPDIFMODE_PCM:
                enableSpdif = 1;
                ac3ToSpdif  = 0;
                break;
        }

        /* ----- open modules ---------------------------------------------- */

        pathMainPtr->handleTsd = aalOpenTsd (inst, pathMainPtr->handleBm,
                                              &err_code);
        if ( pathMainPtr->handleTsd == NULL )
        {
            break;
        }

        memset(&auout_open_params, 0, sizeof(auout_open_params));

        auout_open_params.version       = AAL_AUOUT_VERSION;
        auout_open_params.enableSPDIF   = enableSpdif;
        auout_open_params.pcmOutBuffer  = NULL;

        for ( cnt = 0; cnt < openPtr->params.numSpeakers; cnt++ )
        {
            /* NOTE: Security check: should never occur! speakerArr has
                     (AAL_AUOUT_SPEAKER_ALL + 1) entries */
            if ( cnt >= AAL_AUOUT_SPEAKER_ALL )
            {
                break;
            }

            auout_open_params.speakerArr[cnt] = openPtr->params.speaker[cnt];
        }

        /* mark end of speaker list */
        auout_open_params.speakerArr[cnt] = AAL_AUOUT_SPEAKER_UNDEF;

        pathMainPtr->handleOut = AAL_AUOUT_Open (&auout_open_params,
                                                  &err_code);

        if ( pathMainPtr->handleOut == NULL )
        {
            break;
        }

        pathMainPtr->enableSpdif = enableSpdif;

#ifdef APT_VOL_CTRL
        if ( (openPtr->spdifMode == AAL_SPDIFMODE_PCM)
             && (openPtr->params.numSpeakers > 0)
             && (audioAptVolHandle == NULL) )
        {
            if ( !audioAptVolParamsPtr )
            {
                audioAptVolParamsPtr = (AAL_AUOUT_AptVolParamsT*)FAPI_SYS_MALLOC_UNCACHED(
                                                                 sizeof(AAL_AUOUT_AptVolParamsT));
                if ( audioAptVolParamsPtr == NULL )
                {
                    break;
                }
            }
            memset( audioAptVolParamsPtr, 0, sizeof(AAL_AUOUT_AptVolParamsT));
            audioAptVolHandle = AAL_AUOUT_AddApt( pathMainPtr->handleOut,
                                                   AAL_AUOUT_APT_VOLUME_CONTROL,
                                                   audioAptVolParamsPtr,
                                                   sizeof(AAL_AUOUT_AptVolParamsT),
                                                   AAL_AUOUT_SPDIF_MASK,
                                                   &err_code
                                                 );
            if ( audioAptVolHandle == NULL )
            {
                break;
            }
        }
#endif
#ifdef APT_SIMPLE_SRC
        if ( audioAptSsrcHandle == NULL)
        {
            if ( !audioAptSsrcParamsPtr )
            {
                audioAptSsrcParamsPtr = (AAL_AUOUT_AptSsrcParamsT*)FAPI_SYS_MALLOC_UNCACHED(
                                                                 sizeof(AAL_AUOUT_AptSsrcParamsT));
                if ( audioAptSsrcParamsPtr == NULL )
                {
                    break;
                }
            }
            memset( audioAptSsrcParamsPtr, 0, sizeof(AAL_AUOUT_AptSsrcParamsT));

            audioAptSsrcParamsPtr->minTargetFreq  = AAL_AUOUT_32000_HZ;
            audioAptSsrcParamsPtr->maxTargetFreq  = AAL_AUOUT_48000_HZ;

            audioAptSsrcHandle = AAL_AUOUT_AddApt( pathMainPtr->handleOut,
                                                   AAL_AUOUT_APT_SIMPLE_SRC,
                                                   audioAptSsrcParamsPtr,
                                                   sizeof(AAL_AUOUT_AptSsrcParamsT),
                                                   AAL_AUOUT_ALLON_MASK,
                                                   &err_code
                                                 );
            if ( audioAptSsrcHandle == NULL )
            {
                break;
            }
        }
#endif
        /* set current volume and mute settings */
        if ( inst->path[AAL_PATH_MAIN].muted )
        {
            AAL_AUOUT_Mute (pathMainPtr->handleOut, AAL_AUOUT_SPEAKER_ALL);
        }

        (void)aalIntSetVolAndStereoMode (inst, openPtr);

        /* open decoder */
        pathMainPtr->handleDec = aalOpenDecoder (inst, format,
                                                  ac3ToSpdif,
                                                  openPtr->params.
                                                  suppressErrorRecognition,
                                                  pathMainPtr->handleBm,
                                                  pathMainPtr->handleOut,
                                                  &decoder, &err_code);
        if ( pathMainPtr->handleDec == NULL )
        {
            break;
        }

        if ( openPtr->params.feedMode == AAL_FEEDMODE_TSD )
        {
            err_code = FAPI_TSD_SetChannelPid (pathMainPtr->handleTsd, pid,
                                               0, 1);
            if ( err_code != FAPI_OK )
            {
                break;
            }

            err_code = FAPI_TSD_StartChannel (pathMainPtr->handleTsd);
            if ( err_code != FAPI_OK )
            {
                break;
            }
        }

        err_code = AAL_AUDEC_Start (pathMainPtr->handleDec);

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] Decoder start with status %d.\n",
                             err_code);

        if ( err_code != FAPI_OK )
        {
            if ( err_code == AAL_AUDEC_ERR_HEADER_NOT_FOUND )
            {
                openPtr->audecRestart = 1;

                err_code = FAPI_OK;
            }
        }
        else
        {
            openPtr->audecRunning = 1;

            (void)aalStartHdmi(inst, openPtr);
        }

        pathMainPtr->format  = format;
        pathMainPtr->decoder = decoder;
        pathMainPtr->pid     = pid;
        pathMainPtr->started = 1;

        if ( ac3ToSpdif )
        {
            pathSpdifPtr->format = format;
            pathSpdifPtr->pid    = pid;
        }
    }
    while ( 0 );

    if ( err_code != FAPI_OK )
    {
        (void)aalIntStopModules (openPtr, AAL_PATH_MAIN);

        /* Restart independently running SPDIF passthrough decoder if it was
           running before. */
        if ( currSpdifFormat != AAL_AUDIOFORMAT_NONE )
        {
            (void)aalIntStartSPDIF (inst, openPtr, currSpdifFormat,
                                     currSpdifPid);
        }
    }

    return err_code;
}

static int32_t aalIntStartSPDIF (AAL_DataT* inst, AAL_OpenT* openPtr,
                                  AAL_AudioFormatT format, uint32_t pid)
{
    AAL_AUOUT_OpenParamsT  auout_open_params;
    AAL_AUDEC_OpenParamsT  audec_open_params;
    int32_t                 err_code     = FAPI_OK;
    AAL_AudioPathOpenT*     pathSpdifPtr = openPtr->path + AAL_PATH_SPDIF;

    do
    {
        /* already started? */
        if ( pathSpdifPtr->started )
        {
            err_code = SUBSYS_AUDIO_AAL_ALREADY_STARTED;
            break;
        }
        else if ( openPtr->spdifMode != AAL_SPDIFMODE_PASSTHROUGH )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
            break;
        }
        else if ( format != AAL_AUDIOFORMAT_AC3 )
        {
            err_code = SUBSYS_AUDIO_AAL_ERR_FEATURE_NOT_SUPPORTED;
            break;
        }

        /* ----- open modules ---------------------------------------------- */

        pathSpdifPtr->handleTsd = aalOpenTsd (inst, pathSpdifPtr->handleBm,
                                               &err_code);
        if ( pathSpdifPtr->handleTsd == NULL )
        {
            break;
        }

        memset(&auout_open_params, 0, sizeof(auout_open_params));

        auout_open_params.version       = AAL_AUOUT_VERSION;
        auout_open_params.enableSPDIF   = 1;
        auout_open_params.speakerArr[0] = AAL_AUOUT_SPEAKER_UNDEF;
        auout_open_params.pcmOutBuffer  = NULL;

        pathSpdifPtr->handleOut = AAL_AUOUT_Open (&auout_open_params,
                                                   &err_code);

        if ( pathSpdifPtr->handleOut == NULL )
        {
            break;
        }

        pathSpdifPtr->enableSpdif = 1;

        if ( inst->path[AAL_PATH_SPDIF].muted )
        {
            AAL_AUOUT_Mute (pathSpdifPtr->handleOut, AAL_AUOUT_SPEAKER_ALL);
        }

        /* ----- open decoder ---------------------------------------------- */

        memset(&audec_open_params, 0, sizeof(audec_open_params));

        audec_open_params.version           = AAL_AUDEC_VERSION;
        audec_open_params.decoder           = AAL_AUDEC_DECODER_AC3SPDIF;
        audec_open_params.sampleBits        = AAL_AUDEC_SAMPLE_16_BIT;
        audec_open_params.inputStreamHandle = pathSpdifPtr->handleBm;
        audec_open_params.audioOutHandle    = pathSpdifPtr->handleOut;

        pathSpdifPtr->handleDec = AAL_AUDEC_Open (&audec_open_params,
                                                   &err_code);

        if ( pathSpdifPtr->handleDec == NULL )
        {
            break;
        }

        if ( openPtr->params.feedMode == AAL_FEEDMODE_TSD )
        {
            err_code = FAPI_TSD_SetChannelPid (pathSpdifPtr->handleTsd, pid,
                                               0, 1);
            if ( err_code != FAPI_OK )
            {
                break;
            }

            err_code = FAPI_TSD_StartChannel (pathSpdifPtr->handleTsd);
            if ( err_code != FAPI_OK )
            {
                break;
            }
        }

        err_code = AAL_AUDEC_Start (pathSpdifPtr->handleDec);

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] SPDIF Decoder start with status %d.\n",
                             err_code);

        if ( err_code != FAPI_OK )
        {
            break;
        }

        pathSpdifPtr->format  = format;
        pathSpdifPtr->decoder = audec_open_params.decoder;
        pathSpdifPtr->pid     = pid;
        pathSpdifPtr->started = 1;
    }
    while ( 0 );

    if ( err_code != FAPI_OK )
    {
        (void)aalIntStopModules (openPtr, AAL_PATH_SPDIF);
    }

    return err_code;
}

/*
*******************************************************************************
** \brief   Internal function, stop audio decoding and output.
**
** \param   openPtr           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      AAL_Stop
*******************************************************************************
*/
static int32_t aalIntStop (AAL_OpenT* openPtr, AAL_AudioPathT path)
{
    AAL_AudioPathOpenT* pathPtr = openPtr->path + path;

    if ( pathPtr->started )
    {
        (void)aalIntStopModules (openPtr, path);

        pathPtr->started = 0;
        pathPtr->format  = AAL_AUDIOFORMAT_NONE;
        pathPtr->decoder = AAL_AUDEC_DECODER_UNDEF;
        pathPtr->pid     = 0;
    }

    return FAPI_OK;
}

static int32_t aalIntStopModules (AAL_OpenT* openPtr, AAL_AudioPathT path)
{
    AAL_AudioPathOpenT* pathPtr = openPtr->path + path;

    if ( path == AAL_PATH_MAIN )
    {
        (void)aalStopHdmi (openPtr);
    }

    if ( pathPtr->handleDec != NULL )
    {
        (void)AAL_AUDEC_Stop  (pathPtr->handleDec);
        (void)AAL_AUDEC_Close (pathPtr->handleDec);

        pathPtr->handleDec = NULL;
    }

    if ( pathPtr->handleOut != NULL )
    {
#ifdef APT_11DB_ATTENUATE
        if ( (path == AAL_PATH_MAIN) && (audioApt11dbHandle != NULL) )
        {
            (void)AAL_AUOUT_RemoveApt( audioApt11dbHandle );
            audioApt11dbHandle = NULL;
        }
#endif
#ifdef APT_VOL_CTRL
        if ( (path == AAL_PATH_MAIN) && (audioAptVolHandle != NULL) )
        {
            (void)AAL_AUOUT_RemoveApt( audioAptVolHandle );
            audioAptVolHandle = NULL;
        }
#endif
#ifdef APT_SIMPLE_SRC
        if ( (path == AAL_PATH_MAIN) && (audioAptSsrcHandle != NULL) )
        {
            (void)AAL_AUOUT_RemoveApt( audioAptSsrcHandle );
            audioAptSsrcHandle = NULL;
        }
#endif
        (void)AAL_AUOUT_Close (pathPtr->handleOut);

        pathPtr->handleOut   = NULL;
        pathPtr->enableSpdif = 0;
    }

    if ( pathPtr->handleTsd != NULL )
    {
        (void)FAPI_TSD_StopChannel (pathPtr->handleTsd);
        (void)FAPI_TSD_Close       (pathPtr->handleTsd);

        pathPtr->handleTsd = NULL;
    }

    if ( pathPtr->handleBm != NULL )
    {
        (void)FAPI_BM_SetBufferWritePointer (pathPtr->handleBm, 0, 0);
        (void)FAPI_BM_SetBufferReadPointer  (pathPtr->handleBm, 0, 0);
    }

    if ( path == AAL_PATH_MAIN )
    {
        openPtr->audecRunning       = 0;
        openPtr->hdmiHaveSampleFreq = 0;
    }

    return FAPI_OK;
}

static int32_t aalIntSetVolAndStereoMode (AAL_DataT* inst, AAL_OpenT* openPtr)
{
    FAPI_SYS_HandleT handleOut;
    uint8_t          lvlLeft;
    uint8_t          lvlRight;
    uint8_t          cnt;

    handleOut = openPtr->path[AAL_PATH_MAIN].handleOut;

    if ( handleOut != NULL )
    {
        lvlLeft  = inst->lvlLeft;
        lvlRight = inst->lvlRight;

        switch ( inst->stereoMode )
        {
            default:
            case AAL_STEREO_MODE_STEREO:
                AAL_AUOUT_DisableDualMono(handleOut, AAL_AUOUT_SPEAKER_ALL);
                break;

            case AAL_STEREO_MODE_MONO_LEFT:
                /* NOTE: Assumes left and right speakers to be assigned on
                         a single I2S output. This setting is not suitable for
                         multi-channel-audio. */
                AAL_AUOUT_EnableDualMono(handleOut, AAL_AUOUT_SPEAKER_LEFT);
                break;

            case AAL_STEREO_MODE_MONO_RIGHT:
                /* NOTE: Assumes left and right speakers to be assigned on
                         a single I2S output. This setting is not suitable for
                         multi-channel-audio. */
                AAL_AUOUT_EnableDualMono(handleOut, AAL_AUOUT_SPEAKER_RIGHT);
                break;

            case AAL_STEREO_MODE_MONO_LEFT_ONLY:
                /* NOTE: Assumes left and right speakers to be assigned on
                         a single I2S output. This setting is not suitable for
                         multi-channel-audio. */
                AAL_AUOUT_EnableDualMono(handleOut, AAL_AUOUT_SPEAKER_LEFT);
                lvlRight = 0;
                break;

            case AAL_STEREO_MODE_MONO_RIGHT_ONLY:
                /* NOTE: Assumes left and right speakers to be assigned on
                         a single I2S output. This setting is not suitable for
                         multi-channel-audio. */
                AAL_AUOUT_EnableDualMono(handleOut, AAL_AUOUT_SPEAKER_RIGHT);
                lvlLeft = 0;
                break;
        }

        /* ---- set volume for all active speakers ------------------------- */

        for ( cnt = 0; cnt < openPtr->params.numSpeakers; cnt++ )
        {
            switch ( openPtr->params.speaker[cnt] )
            {
                case AAL_AUOUT_SPEAKER_LEFT:
                case AAL_AUOUT_SPEAKER_SUR_LEFT:
                case AAL_AUOUT_SPEAKER_BACK_LEFT:
#ifdef APT_VOL_CTRL
                    if ( audioAptVolHandle != NULL )
                    {
                        audioAptVolParamsPtr->targetVolume[openPtr->params.speaker[cnt]] = lvlLeft;
                    }
#endif
                    AAL_AUOUT_SetVolume (handleOut,
                                          openPtr->params.speaker[cnt],
                                          lvlLeft);
                    break;

                case AAL_AUOUT_SPEAKER_RIGHT:
                case AAL_AUOUT_SPEAKER_SUR_RIGHT:
                case AAL_AUOUT_SPEAKER_BACK_RIGHT:
#ifdef APT_VOL_CTRL
                    if ( audioAptVolHandle != NULL )
                    {
                        audioAptVolParamsPtr->targetVolume[openPtr->params.speaker[cnt]] = lvlRight;
                    }
#endif

                    AAL_AUOUT_SetVolume (handleOut,
                                          openPtr->params.speaker[cnt],
                                          lvlRight);
                    break;

                case AAL_AUOUT_SPEAKER_CENTER:
                case AAL_AUOUT_SPEAKER_LFE:

                    /* NOTE: For speakers not assigned to left or right side,
                             an average volume level is set. */
#ifdef APT_VOL_CTRL
                    if ( audioAptVolHandle != NULL )
                    {
                        audioAptVolParamsPtr->targetVolume[openPtr->params.speaker[cnt]] =
                                                                          (lvlLeft + lvlRight) / 2;
                    }
#endif
                    AAL_AUOUT_SetVolume (handleOut,
                                          openPtr->params.speaker[cnt],
                                          ((uint32_t)lvlLeft
                                         + (uint32_t)lvlRight) / 2);
                    break;

                default:
                    /* do nothing */
                    break;
            }
        }
#ifdef APT_VOL_CTRL
        if ( audioAptVolHandle != NULL )
        {
            (void)AAL_AUOUT_SetAptParams( audioAptVolHandle,
                                           audioAptVolParamsPtr,
                                           sizeof(AAL_AUOUT_AptVolParamsT));
        }
#endif
    }

    return FAPI_OK;
}

/*
*******************************************************************************
** \brief   Internal function, start HDMI output.
**
** \param   openPtr           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      AAL_Start
** \sa      AAL_EnableHdmi
*******************************************************************************
*/
static int32_t aalStartHdmi (AAL_DataT* inst, AAL_OpenT* openPtr)
{
    AAL_AUOUT_StreamInfoT  streamInfo;
    int32_t                 err_code;
    uint8_t                 blocked = 0;

    if ( (inst->hdmiEnabled)
      && (openPtr->audecRunning)
      && (!(openPtr->hdmiRunning)) )
    {
        if ( !(openPtr->hdmiHaveSampleFreq) )
        {
            blocked = 1;

            streamInfo = AAL_AUOUT_GetStreamInfo (openPtr->path[AAL_PATH_MAIN].
                                                   handleOut);

            if ( (streamInfo.channels != AAL_AUOUT_CHANNEL_UNDEF)
                  && (streamInfo.sampleFreq != AAL_AUOUT_UNDEF_HZ)
            )
            {
                openPtr->hdmiHaveSampleFreq = 1;
                blocked = 0;
            }
        }

        if ( blocked == 0 )
        {
            err_code = AAL_AUOUT_StartHdmi (openPtr->path[AAL_PATH_MAIN].
                                             handleOut);

            if ( err_code == FAPI_OK )
            {
                openPtr->hdmiRunning = 1;
            }

            FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_AUOUT_StartHdmi: %i\n",
                                 err_code);
        }
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalStartHdmi.\n");

    return FAPI_OK;
}

/*
*******************************************************************************
** \brief   Internal function, stop HDMI output.
**
** \param   openPtr           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      AAL_Pause
** \sa      AAL_EnableHdmi
*******************************************************************************
*/
static int32_t aalStopHdmi (AAL_OpenT* openPtr)
{
    if ( openPtr->hdmiRunning )
    {
        AAL_AUOUT_StopHdmi();

        openPtr->hdmiRunning = 0;

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_AUOUT_StopHdmi.\n");
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalStopHdmi.\n");

    return FAPI_OK;
}

static FAPI_SYS_HandleT aalOpenBm (AAL_DataT* inst, AAL_AudioPathT path,
                                    uint32_t* bmSizePtr, int32_t* errCodePtr)
{    
    FAPI_BM_OpenParamsT bm_open_params;
    FAPI_SYS_HandleT    handleBm;
    int32_t             err_code = FAPI_OK;
    FAPI_BM_PurposeT    purpose;

    memset (&bm_open_params, 0, sizeof(bm_open_params));

    if ( path == AAL_PATH_MAIN )
    {
        purpose = FAPI_BM_AUDIO_STREAM_BUFFER;
    }
    else
    {
        purpose = FAPI_BM_SPDIF_STREAM_BUFFER;
    }

    bm_open_params.version    = FAPI_BM_VERSION;
    bm_open_params.blockIndex = inst->initParams.blockIndex;
    bm_open_params.channelId  = FAPI_BM_CHANNEL_AUTODETECT;
    bm_open_params.address    = FAPI_BM_GetAddressByBufferPurpose(purpose);
    bm_open_params.size       = FAPI_BM_GetSizeByBufferPurpose(purpose);
    bm_open_params.swapWrite  = FAPI_BM_SWAP;
    bm_open_params.swapRead   = FAPI_BM_NO_SWAP;
    bm_open_params.swapMode   = FAPI_BM_SWAP_4_BYTES;

    handleBm = FAPI_BM_Open (&bm_open_params, &err_code);

    if ( handleBm != NULL )
    {
        if ( bmSizePtr != NULL )
        {
            *bmSizePtr = bm_open_params.size;
        }
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }
    
    return handleBm;

}

static FAPI_SYS_HandleT aalOpenTsd (AAL_DataT* inst, FAPI_SYS_HandleT handleBm,
                                     int32_t* errCodePtr)
{
    FAPI_TSD_OpenParamsT    tsd_open_params;
    FAPI_BM_InfoT           bm_info;
    FAPI_SYS_HandleT        handleTsd;
    int32_t                 err_code = FAPI_OK;

    memset (&tsd_open_params, 0, sizeof(tsd_open_params));

    tsd_open_params.version    = FAPI_TSD_VERSION;
    tsd_open_params.blockIndex = inst->initParams.blockIndex;
    tsd_open_params.channelId  = FAPI_TSD_CHANNEL_AUTODETECT;

    do
    {
        handleTsd = FAPI_TSD_Open (&tsd_open_params, &err_code);

        if ( handleTsd == NULL )
        {
            break;
        }

        err_code = FAPI_TSD_SetPesParser (handleTsd, 1, 1, 1);
        if ( err_code < 0 ) /* NOTE: special return value! */
        {
            break;
        }

        err_code = FAPI_BM_GetInfo (handleBm, &bm_info);
        if ( err_code != FAPI_OK )
        {
            break;
        }

        err_code = FAPI_TSD_SetBuffer (handleTsd, bm_info);
        if ( err_code != FAPI_OK )
        {
            break;
        }

        /* register PTS notifier */
        err_code = FAPI_TSD_SetPtsNotifier (handleTsd, AAL_AUDEC_PtsNotifier,
                                            handleBm, 1);
        if ( err_code != FAPI_OK )
        {
            break;
        }
    }
    while ( 0 );

    if ( err_code != FAPI_OK )
    {
        (void)FAPI_TSD_Close (handleTsd);

        handleTsd = NULL;
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    return handleTsd;
}

static FAPI_SYS_HandleT aalOpenDecoder (AAL_DataT* inst,
                                         AAL_AudioFormatT format,
                                         uint8_t ac3ToSpdif,
                                         uint8_t suppressErrorRecognition,
                                         FAPI_SYS_HandleT handleBm,
                                         FAPI_SYS_HandleT handleOut,
                                         AAL_AUDEC_DecoderEnumT* decoderPtr,
                                         int32_t* errCodePtr)
{
    AAL_AUDEC_OpenParamsT      audec_open_params;
    AAL_AUDEC_MpegParamsT      mpegParams;
    AAL_AUDEC_Ac3ParamsT       ac3Params;
    AAL_AUDEC_DdpdcvParamsT    ddpdcvParams;
    AAL_AUDEC_HeaacParamsT     heaacParams;
    FAPI_SYS_HandleT            handleDec  = NULL;
    int32_t                     err_code   = FAPI_OK;
    uint8_t*                    paramsPtr  = NULL;
    uint32_t                    paramsSize = 0;
    uint8_t                     swap       = 0;

    if ( (format < 0) || (format >= AAL_AUDIOFORMAT_NUMS) )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER;
    }
    else
    {
        memset (&audec_open_params, 0, sizeof(audec_open_params));

        audec_open_params.version           = AAL_AUDEC_VERSION;
        audec_open_params.decoder           = inst->targetDecoder[format];
        audec_open_params.sampleBits        = AAL_AUDEC_SAMPLE_16_BIT;
        audec_open_params.inputStreamHandle = handleBm;
        audec_open_params.audioOutHandle    = handleOut;

        handleDec = AAL_AUDEC_Open(&audec_open_params, &err_code);

        if ( handleDec != NULL )
        {
            switch ( audec_open_params.decoder )
            {
                case AAL_AUDEC_DECODER_MPEG:
                    memset (&mpegParams, 0, sizeof(mpegParams));

                    mpegParams.decoder               = AAL_AUDEC_DECODER_MPEG;
                    mpegParams.muteDuration          = 1200;

                    if ( suppressErrorRecognition )
                    {
                        mpegParams.startInsyncFrameCount = 0;
                    }
                    else
                    {
                        mpegParams.startInsyncFrameCount = 10;
                    }

                    paramsPtr  = (uint8_t*)(&(mpegParams));
                    paramsSize = sizeof(mpegParams);
#ifdef APT_11DB_ATTENUATE
                    if ( (audioApt11dbHandle == NULL )
                        && ((inst->targetDecoder[AAL_AUDIOFORMAT_AC3] != AAL_AUDEC_DECODER_UNDEF)
                           || (inst->targetDecoder[AAL_AUDIOFORMAT_DDP] != AAL_AUDEC_DECODER_UNDEF))
                    ) {
                        uint8_t ifc_mask;

                        if ( audioDolbyCompMode == AAL_DOLBY_LINE_MODE )
                        {
                            ifc_mask = AAL_AUOUT_ALLON_MASK;
                        }
                        else
                        {
                            ifc_mask = AAL_AUOUT_SPDIF_MASK;
                        }

                        audioApt11dbHandle = AAL_AUOUT_AddApt( handleOut,
                                                                AAL_AUOUT_APT_11DB_ATTENUATION,
                                                                NULL,
                                                                0,
                                                                ifc_mask,
                                                                &err_code
                                                              );
                    } else if ( audioApt11dbHandle == NULL ) {
                        // because of AC-3 passthrough
                        audioApt11dbHandle = AAL_AUOUT_AddApt( handleOut,
                                                                AAL_AUOUT_APT_11DB_ATTENUATION,
                                                                NULL,
                                                                0,
                                                                AAL_AUOUT_SPDIF_MASK,
                                                                &err_code
                                                              );
                    }
#endif
                    break;

                case AAL_AUDEC_DECODER_AC3:
                    memset (&ac3Params, 0, sizeof(ac3Params));

                    ac3Params.decoder       = AAL_AUDEC_DECODER_AC3;
                    ac3Params.compMode      = audioDolbyCompMode;
                    ac3Params.dynsclh       = 1.0;
                    ac3Params.dynscll       = 1.0;
                    ac3Params.pcmscl        = 1.0;
                    ac3Params.outChannelCfg = -1;
                    ac3Params.ac3ToSpdif    = ac3ToSpdif;

                    paramsPtr  = (uint8_t*)(&(ac3Params));
                    paramsSize = sizeof(ac3Params);
                    swap       = 1;
#ifdef APT_11DB_ATTENUATE
                    if ( (audioApt11dbHandle == NULL)
                          && (audioDolbyCompMode == AAL_DOLBY_RF_MODE)
                    ) {
                        audioApt11dbHandle = AAL_AUOUT_AddApt( handleOut,
                                                                AAL_AUOUT_APT_11DB_ATTENUATION,
                                                                NULL,
                                                                0,
                                                                AAL_AUOUT_SPDIF_MASK,
                                                                &err_code
                                                              );
                    }
#endif
                    break;

                case AAL_AUDEC_DECODER_DDPDCV:
                    memset (&ddpdcvParams, 0, sizeof(ddpdcvParams));

                    ddpdcvParams.decoder       = AAL_AUDEC_DECODER_DDPDCV;
                    ddpdcvParams.compMode      = audioDolbyCompMode;
                    ddpdcvParams.dynsclh       = 1.0;
                    ddpdcvParams.dynscll       = 1.0;
                    ddpdcvParams.pcmscl        = 1.0;
                    ddpdcvParams.outChannelCfg = -1;
                    ddpdcvParams.ac3ToSpdif    = ac3ToSpdif;

                    paramsPtr  = (uint8_t*)(&(ddpdcvParams));
                    paramsSize = sizeof(ddpdcvParams);
                    swap       = 1;
#ifdef APT_11DB_ATTENUATE
                    if ( (audioApt11dbHandle == NULL)
                          && (audioDolbyCompMode == AAL_DOLBY_RF_MODE)
                    ) {
                        audioApt11dbHandle = AAL_AUOUT_AddApt( handleOut,
                                                                AAL_AUOUT_APT_11DB_ATTENUATION,
                                                                NULL,
                                                                0,
                                                                AAL_AUOUT_SPDIF_MASK,
                                                                &err_code
                                                              );
                    }
#endif
                    break;

                case AAL_AUDEC_DECODER_HEAAC:
                    memset (&heaacParams, 0, sizeof(heaacParams));

                    /* NOTE: All values except startInsyncFrameCount are set
                             to default. */

                    heaacParams.decoder              = AAL_AUDEC_DECODER_HEAAC;
                    heaacParams.extSamplingRate      = -1;
                    heaacParams.maxOutSamplingRate   = 96000;
                    heaacParams.enableDSSBR          = 0;
                    heaacParams.enableDMSBR          = 0;
                    heaacParams.sbrMode              = 2;
                    heaacParams.enableDOWNMIX        = 1;
                    heaacParams.downmixMode          = 0;
                    heaacParams.enableDRC            = 1;
                    heaacParams.drcTargetRefLevel    = 127;
                    heaacParams.fadeOutFrameNum      = 5;
                    heaacParams.fadeInFrameNum       = 5;
                    heaacParams.muteFrameCount       = 5;

                    if ( suppressErrorRecognition )
                    {
                        heaacParams.startInsyncFrameCount = 0;
                    }
                    else
                    {
                        heaacParams.startInsyncFrameCount = 10;
                    }

                    paramsPtr  = (uint8_t*)(&(heaacParams));
                    paramsSize = sizeof(heaacParams);
                    break;

                default:
                    break;
            }

            if ( paramsPtr != NULL )
            {
                if ( swap )
                {
                    #if defined(__GNUC__)

                    /* NOTE: Temporary workaround, ARC/Metaware interprets
                             'double' entries in codec structure different than
                             ARM/GCC (ARM/Realview is OK). */

                    AAL_Assert(paramsSize <= AAL_PARAMSBUF_SIZE);

                    /*lint -save -e669 -e670 */
                    memcpy (inst->paramsBuf, paramsPtr, paramsSize);
                    /*lint -restore */

                    paramsPtr = inst->paramsBuf;

                    /* swap 'double' entries in structure (last three entries,
                       8 bytes each) */

                    /* pcmscl */
                    aalSwapLongwords(paramsPtr + paramsSize - 8);

                    /* dynscll */
                    aalSwapLongwords(paramsPtr + paramsSize - 16);

                    /* dynsclh */
                    aalSwapLongwords(paramsPtr + paramsSize - 24);

                    #endif /* defined(__GNUC__) */
                }

                err_code = AAL_AUDEC_SetCodecParams (handleDec, paramsPtr,
                                                      paramsSize);
            }

            if ( err_code != FAPI_OK )
            {
                (void)AAL_AUDEC_Close(handleDec);

                handleDec = NULL;
            }
            else
            {
                if ( decoderPtr != NULL )
                {
                    *decoderPtr = inst->targetDecoder[format];
                }
            }
        }
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    return handleDec;
}

#if defined(__GNUC__)

static void aalSwapLongwords (uint8_t* bufPtr)
{
    uint32_t    cnt;
    uint8_t     tmpByte;

    for ( cnt = 0; cnt < 4; cnt++ )
    {
        tmpByte = bufPtr[cnt];

        bufPtr[cnt]     = bufPtr[cnt + 4];
        bufPtr[cnt + 4] = tmpByte;
    }
}

#endif /* defined(__GNUC__) */


#else
/*
*******************************************************************************
** \brief   Internal function, checks validity of AAL handle.
**
** \param   handle           Pointer to be checked.
** \param   errCodePtr       Possible error codes:
**          - SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED  AAL is not initialized.
**          - SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE   Handle is a NULL pointer.
            - SUBSYS_AUDIO_AAL_ERR_NOT_OPEN         AAL is not open.
**
** \return  Possible return codes:
            - Driver handle (may be NULL)
**
*******************************************************************************
*/
static AAL_OpenT* aalCheckHandle (FAPI_SYS_HandleT handle, int32_t* errCodePtr)
{
    AAL_OpenT*  openPtr  = (AAL_OpenT*)handle;
    int32_t     err_code = FAPI_OK;

    if ( audioDatPtr == NULL )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED;
    }
    else if ( (openPtr == NULL)
           || (openPtr != &(audioDatPtr->openData)) )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE;
    }
    else if ( !(openPtr->open) )
    {
        err_code = SUBSYS_AUDIO_AAL_ERR_NOT_OPEN;
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    if ( err_code != FAPI_OK )
    {
        openPtr = NULL;
    }

    return openPtr;
}


/*
*******************************************************************************
** Open TSD0/BM0 channel8.
*******************************************************************************
*/
static int32_t audioTsdOpen(void)
{
    FAPEX_TSD_OpenSubIdParamStrT pidChParam;
    FAPEX_TSD_PesParamStrT       pesParam;
    int32_t result;
 
    pidChParam.channelNo = AUDIO_TSD_CHANNEL;
    pidChParam.sub_id    = DRV_ID_TSD0_AUDIO_CH;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                              FAPEX_IOCCOM_TSD_OPEN_SUBID,
                              (FAPEXIOCARG3_T)&pidChParam);
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Open Audio TSD channel. %d\n", result);
        return result;
    }
    fd_TSD0_AUDIO = pidChParam.sub_id;
    
    /* set Audio PES paser */
    pesParam.pid   = 0x1FFF;  /* dummy */
    pesParam.flags = FAPEX_TSD_FLTFLAGS_RM_PES_HEADER | FAPEX_TSD_FLTFLAGS_PTS |
                     FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP | FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE;
    pesParam.bufferSize       = FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES;
    pesParam.bufferThresholdH = pesParam.bufferSize;
    pesParam.bufferThresholdL = 0;
    pesParam.streamType       = FAPEX_TSD_STREAMTYPE_AUDIO;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                              FAPEX_IOCCOM_TSD_SET_PES_FILTER,
                              (FAPEXIOCARG3_T)&pesParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Set Audio PES filter (code %d).\n", result);
        return result;
    }
    
    audio_mmapbuf = (uint8_t*)FAPEX_TSD_Mmap( NULL, FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES,
                                             (PROT_READ | PROT_WRITE), MAP_SHARED,
                                              fd_TSD0_AUDIO, 0 );
    if (audio_mmapbuf == NULL){
        FAPI_SYS_PRINT_DEBUG(2, "[AAL] Error: TSD audio bm buffer mmap failed. \n");
        return -1;
    }
    printf("[AAL]  TSD audio bm buffer mmap ret (0x%x)\n",audio_mmapbuf);
    //printf("DRV_ID_TSD0_AUDIO_CH:[%d][%d][%d]\n",DRV_ID_TSD0_AUDIO_CH,pidChParam.sub_id,fd_TSD0_AUDIO);
    return 0;
}

/*
*******************************************************************************
** Close TSD0/BM0 channel8.
*******************************************************************************
*/
static int32_t audioTsdClose(void)
{
    int32_t result = 0;

    if(fd_TSD0_AUDIO >= 0) {
        if(audio_mmapbuf != NULL)
            FAPEX_TSD_Munmap(audio_mmapbuf,FAPEX_SYS_MMAP_BM_AUDIO_STREAM_BYTES);
        
        result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_AUDIO,
                                  FAPEX_IOCCOM_CLOSE_SUBID,
                                  (FAPEXIOCARG3_T)NULL );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close Audio TSD channel. %d\n", result);
        }
        fd_TSD0_AUDIO = -1;
    }
    return result;
}

/*
*******************************************************************************
** Open Audio Output/Decoder
*******************************************************************************
*/
static int32_t audioOutOpen(void)
{
    FAPEX_AUDEC_OpenParamStrT   audecOpenParam;
    FAPEX_AUOUT_OpenParamStrT   auoutOpenParam;
    FAPEX_AUDEC_OutputParamStrT audecOutParam;
    int32_t  result;

    /* open Audio Output */
    auoutOpenParam.enableFlag  = FAPEX_AUOUT_ENABLE_I2S | FAPEX_AUOUT_ENABLE_SPDIF |
                                 FAPEX_AUOUT_ENABLE_HDMI;
    auoutOpenParam.channelFlag = FAPEX_AUOUT_CHANNEL_STEREO;
    fd_AUOUT0 = FAPEX_AUOUT_Open( DRV_ID_AUOUT_0,
                                  O_RDWR,
                                  (FAPEXOPENARG3_T)&auoutOpenParam );
    if(fd_AUOUT0 < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Open AUOUT driver. %d\n", fd_AUOUT0);
        return fd_AUOUT0;
    }
    FAPI_SYS_PRINT_DEBUG(1, "[aal][audioOutOpen] Open AUOUT success (fd_aout = %d)\n", fd_AUOUT0);
    printf("DRV_ID_AUOUT_0:[%d]\n",DRV_ID_AUOUT_0);
    /* open Audio Decoder */
    audecOpenParam.inputId   = DRV_ID_TSD0_AUDIO_CH;
    audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_MPEG;
    fd_AUDEC_MPEG = FAPEX_AUDEC_Open( DRV_ID_AUDEC_0,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)&audecOpenParam );
    if(fd_AUDEC_MPEG < 0) {
        FAPI_SYS_PRINT_ERR("ERROR: Cannot open MPEG audio decoder (code %d).\n", fd_AUDEC_MPEG);
        return fd_AUDEC_MPEG;
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1, "[aal][audioOutOpen] Open AUDEC_0(MPEG) success (fd_AUDEC_MPEG = %d)\n", fd_AUDEC_MPEG);
    }
    printf("DRV_ID_AUDEC_0:[%d]\n",DRV_ID_AUDEC_0);
    
    audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_HEAAC;
    fd_AUDEC_HEAAC = FAPEX_AUDEC_Open( DRV_ID_AUDEC_1,
                                       O_RDWR,
                                       (FAPEXOPENARG3_T)&audecOpenParam );
    if(fd_AUDEC_HEAAC < 0) {
        FAPI_SYS_PRINT_ERR("WARNING: Cannot open HE-AAC audio decoder (code %d).\n", fd_AUDEC_HEAAC);
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1, "[aal][audioOutOpen] Open AUDEC_1(HEAAC) success (fd_AUDEC_HEAAC = %d)\n", fd_AUDEC_HEAAC);
    }
    printf("DRV_ID_AUDEC_1:[%d]\n",DRV_ID_AUDEC_1);

    audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_DD;
    fd_AUDEC_DD = FAPEX_AUDEC_Open( DRV_ID_AUDEC_2,
                                     O_RDWR,
                                     (FAPEXOPENARG3_T)&audecOpenParam );
    if(fd_AUDEC_DD < 0) {
        FAPI_SYS_PRINT_ERR("WARNING: Cannot open DD audio decoder converter (code %d).\n", fd_AUDEC_DD);
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1, "[aal][audioOutOpen] Open AUDEC_2(DD) success (fd_AUDEC_DD = %d)\n", fd_AUDEC_DD);
    }
    printf("DRV_ID_AUDEC_2:[%d]\n",DRV_ID_AUDEC_2);

    audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_DDPDCV;
    fd_AUDEC_DDP = FAPEX_AUDEC_Open( DRV_ID_AUDEC_3,
                                     O_RDWR,
                                     (FAPEXOPENARG3_T)&audecOpenParam );
    if(fd_AUDEC_DDP < 0) {
        FAPI_SYS_PRINT_ERR("WARNING: Cannot open DD+ audio decoder converter (code %d).\n", fd_AUDEC_DDP);
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1, "[aal][audioOutOpen] Open AUDEC_3(DD+) success (fd_AUDEC_DDP = %d)\n", fd_AUDEC_DDP);
    }
    printf("DRV_ID_AUDEC_3:[%d]\n",DRV_ID_AUDEC_3);


    if( fd_AUDEC_DD < 0 && fd_AUDEC_DDP < 0 ) {
        audecOpenParam.codecType = FAPEX_AUDEC_CODEC_TYPE_AC3;
        fd_AUDEC_AC3 = FAPEX_AUDEC_Open( DRV_ID_AUDEC_4,
                                         O_RDWR,
                                         (FAPEXOPENARG3_T)&audecOpenParam );
        if(fd_AUDEC_AC3 < 0) {
            FAPI_SYS_PRINT_ERR("WARNING: Cannot open AC3 audio decoder (code %d).\n", fd_AUDEC_AC3);
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "[aal][audioOutOpen] Open AUDEC_4(AC3) success (fd_AUDEC_AC3 = %d)\n", fd_AUDEC_AC3);
        }
        printf("DRV_ID_AUDEC_4:[%d]\n",DRV_ID_AUDEC_4);
    }
    
    /* set Audio Decoder Output parameter */
    audecOutParam.outputId   = DRV_ID_AUOUT_0;
    audecOutParam.sampleBits = 16;
    result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_MPEG,
                                FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                (FAPEXIOCARG3_T)&audecOutParam);
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR setup MPEG audio decoder (code %d).\n", result);
        return result;
    }

    if(fd_AUDEC_HEAAC >= 0) {
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_HEAAC,
                                    FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                    (FAPEXIOCARG3_T)&audecOutParam);
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("WARNING: setup HE-AAC audio decoder (code %d).\n", result);
        }
    }

    if(fd_AUDEC_DD >= 0) {
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_DD,
                                    FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                    (FAPEXIOCARG3_T)&audecOutParam);
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("WARNING: setup DD audio decoder converter (code %d).\n", result);
        }
    }

    if(fd_AUDEC_DDP >= 0) {
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_DDP,
                                    FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                    (FAPEXIOCARG3_T)&audecOutParam);
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("WARNING: setup DD+ audio decoder converter (code %d).\n", result);
        }
    }
    
    if(fd_AUDEC_AC3 >= 0) {
        result = FAPEX_AUDEC_Ioctl( (FAPEXIOCARG1_T)fd_AUDEC_AC3,
                                    FAPEX_IOCCOM_AUDEC_SET_OUTPUT,
                                    (FAPEXIOCARG3_T)&audecOutParam);
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("WARNING: setup DD+ audio decoder converter (code %d).\n", result);
        }

    }
    return 0;
}

/*
*******************************************************************************
** Close AUDEC and AUOUT.
*******************************************************************************
*/
static int32_t audioOutClose(void)
{
    int32_t result = 0;

    if(fd_AUOUT0 >= 0) {
        result = FAPEX_AUOUT_Close( fd_AUOUT0 );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close Audio Output (code %d).\n", result);
            return result;
        }
        fd_AUOUT0 = -1;
    }

    fd_AUDEC0 = -1;

    if(fd_AUDEC_MPEG >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_MPEG );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close MPEG audio decoder (code %d).\n", result);
            return result;
        }
        fd_AUDEC_MPEG = -1;
    }

    if(fd_AUDEC_AAC >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_AAC );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close AAC-LC audio decoder (code %d).\n", result);
            return result;
        }
        fd_AUDEC_AAC = -1;
    }

    if(fd_AUDEC_AC3 >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_AC3 );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close AC3 audio decoder (code %d).\n", result);
            return result;
        }
        fd_AUDEC_AC3 = -1;
    }

    if(fd_AUDEC_HEAAC >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_HEAAC );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close HE-AAC audio decoder (code %d).\n", result);
            return result;
        }
        fd_AUDEC_HEAAC = -1;
    }

    if(fd_AUDEC_DDP >= 0) {
        result = FAPEX_AUDEC_Close( fd_AUDEC_DDP );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("ERROR Close DD+ audio decoder converter (code %d).\n", result);
            return result;
        }
        fd_AUDEC_DDP = -1;
    }
    return result;
}

/*
*******************************************************************************
** \brief   Internal function, start HDMI output.
**
** \param   openPtr           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      AAL_Start
** \sa      AAL_EnableHdmi
*******************************************************************************
*/
static int32_t aalStartHdmi (AAL_DataT* inst, AAL_OpenT* openPtr)
{

    uint8_t                 blocked = 0;
    int32_t                 err_code= 0;
    FAPEX_AUOUT_PcmInfoStrT pcmInfo;

    if ( (inst->hdmiEnabled)
      && (openPtr->audecRunning)
      && (!(openPtr->hdmiRunning)) )
    {
        if ( !(openPtr->hdmiHaveSampleFreq) )
        {
            blocked = 1;

            memset(&pcmInfo, 0, sizeof(FAPEX_AUOUT_PcmInfoStrT));
            err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                        FAPEX_IOCCOM_AUOUT_GET_PCM_INFO,
                                        (FAPEXIOCARG3_T)&pcmInfo );


            if ( (0 == err_code) &&(pcmInfo.channelCount != AAL_AUOUT_CHANNEL_UNDEF)
                  && (pcmInfo.sampleFreq != AAL_AUOUT_UNDEF_HZ)
            )
            {
                openPtr->hdmiHaveSampleFreq = 1;
                blocked = 0;
            }
        }

        if ( blocked == 0 )
        {
            err_code = FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                                        FAPEX_IOCCOM_AUOUT_START_HDMI,
                                        (FAPEXIOCARG3_T)NULL);

            if ( err_code == FAPI_OK )
            {
                openPtr->hdmiRunning = 1;
            }

            FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_AUOUT_StartHdmi: %i\n",
                                 err_code);
        }
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalStartHdmi.\n");

    return FAPI_OK;
}

/*
*******************************************************************************
** \brief   Internal function, stop HDMI output.
**
** \param   openPtr           Handle returned by AAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      AAL_Pause
** \sa      AAL_EnableHdmi
*******************************************************************************
*/
static int32_t aalStopHdmi (AAL_OpenT* openPtr)
{
    if ( openPtr->hdmiRunning )
    {
        (void)FAPEX_AUOUT_Ioctl( (FAPEXIOCARG1_T)fd_AUOUT0,
                            FAPEX_IOCCOM_AUOUT_STOP_HDMI,
                            (FAPEXIOCARG3_T)NULL);

        openPtr->hdmiRunning = 0;

        FAPI_SYS_PRINT_DEBUG(2, "[AAL] AAL_AUOUT_StopHdmi.\n");
    }

    FAPI_SYS_PRINT_DEBUG(2, "[AAL] aalStopHdmi.\n");

    return FAPI_OK;
}

#endif

/*****************************************************************************/
/*lint -restore */
/* END OF FILE */
