/******************************************************************************
**
** \file      ./subsystems/fe/ce6353/src/ce6353_priv.h
**
** \brief     CE6353 OFDM Demodulator include file.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: ce6353_priv.h 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
#ifndef _OFDM_CE6353_PRIV_H_
#define _OFDM_CE6353_PRIV_H_

#include "ce6353.h"

/****************************************************************************************
*                       CE6353 register definitions                                     *
*****************************************************************************************/

#define CE6353_IRQMAIN                 0x00        /*4  IRQ regs 0 to 3 */
#define CE6353_IRQ4                    0x04        /*1  IRQ reg 4 */
#define CE6353_IRQ5                    0x05        /*1  IRQ reg 5 (TPS flags cannot be enabled to IRQ pin)*/
#define CE6353_STATUS                  0x06        /*4  the 4 status registers */
#define CE6353_AGCGAIN                 0x0A        /*2  14 bit AGC total gain value*/
#define CE6353_AGCDIG                  0x0C        /*2  16 bit Digital AGC */
#define CE6353_RFADCLEVEL              0x0E        /*1  7 bit ADC reading RF AGC */
#define CE6353_SNR                     0x10        /*1  SNR in 1/8 dB steps. */

#define CE6353_RSBERCNT                0x11        /*3  Reed-Solomon error count */
#define CE6353_RSUBC                   0x14        /*2  Reed-Solomon uncorrected block count */
#define CE6353_ITBFREQOFFSET           0x16        /*2  ITB Freq Offset */
#define CE6353_FREQOFFSET              0x18        /*3 Frequency offset calculated from this */

#define CE6353_TPSRECEIVED             0x1D        /*2  TPS received register */
#define CE6353_TPSCURRENT              0x1F        /*2  TPS current register */
#define CE6353_TPSCELLID               0x21        /*2  CELL ID*/
#define CE6353_TPSMISCDATA             0x23        /*3 TPS misc data */
#define CE6353_VITERRCNT               0x2F        /*3  Viterbi error count */
#define CE6353_FIRSTRW                 0x4C        /*   location of first non read only register  */
#define CE6353_ENABLE                  0x50        /*1  Power control /enable register*/
#define CE6353_CLKCTL0                 0x51        /*1 clock control */
#define CE6353_CLKCTL1                 0x52        /*1 clock control */
#define CE6353_PLLCTL0                 0x53        /*1 PLL control */
#define CE6353_PLLCTL1                 0x54        /*1 PLL control */
#define CE6353_RESET                   0x55        /*1 RESET reg*/
#define CE6353_AGCTARGET               0x56        /*1 AGC target and rev agc control*/
#define CE6353_OUTCTL0                 0x5A        /*1 MPEG output control */
#define CE6353_MCLKCTL                 0x5C        /*1 MPEG clock trim */
#define CE6353_ACQCTL                  0x5E        /* acquisition control */
#define CE6353_CAPTURE                 0x5F        /*1 capture range register*/
#define CE6353_RSERRPER                0x60        /*2  Reed-Solomon Error period*/
#define CE6353_SCANCTL                 0x62        /*1  contains I2C bus master/passthrough switches etc*/
#define CE6353_GPPCTL                  0x63        /*1  GPP controls */
#define CE6353_BWCTRL                  0x64        /*1 BW control and filter write control */
#define CE6353_TRLRATE                 0x65        /*2  TRL nominal rate */
#define CE6353_TUNADDRESS              0x67        /*1  Address of tuner */
#define CE6353_TUNERDATA               0x68        /*4  Tuner data (bytes 2,3,4,5) */
#define CE6353_ITBFREQ                 0x6C        /*2 ITB mixer frequency */
#define CE6353_TPSGIVEN                0x6E        /*2 TPS GIVEN register */
#define CE6353_TUNERGO                 0x70        /*1 Acq start */
#define CE6353_FSMGO                   0x71        /*1 Acq start */
#define CE6353_IRQMAINENABLE           0x72        /*4 IRQ enable regs 0 to 3 */
#define CE6353_IRQ4ENABLE              0x76        /*1 IRQ 4 enables*/
#define CE6353_AFCCTRL                 0x7C        /*1 */
#define CE6353_CHIPID                  0x7F        /*1 Read Only register for Chip ID */
#define CE6353_COEFF0                  0x9A        /*2 filter coefficient registers */
#define CE6353_IMPCTRL                 0x9C        /*1 */
#define CE6353_TRSWEEPEN               0xBA        /*1 */
#define CE6353_VITERRPER               0xD6        /*1 Viterbi error measurement period */
#define CE6353_BWCTRL6MHZ              0xDC        /*2 bandwidth control register for 6MHz */
#define CE6353_ADCCTL0                 0xEA        /*1 ADC recalibrate register */


/****************************************************************************************
*                       CE6353 register bits                                            *
*****************************************************************************************/

/*CE6353_STATUS         0x06 */
#define CE6353_STATUS_FULL_LOCK  0x40
#define CE6353_STATUS_FEC_LOCK   0x20
#define CE6353_STATUS_TPS_LOCK   0x08
#define CE6353_STATUS_OFDM_FOUND 0x04
#define CE6353_STATUS_SYM_LOCK   0x02
#define CE6353_STATUS_AGC_LOCK   0x01

/* CE6353_TPSRECEIVED   0x1D */
/* CE6353_TPSGIVEN    0x6E*/
#define CE6353_TPSGIVEN_LPSEL                 0x8000


/* CE6353_ENABLE    0x50*/
#define CE6353_ENABLE_DVRPORTSEL              0x20
#define CE6353_ENABLE_DVROUTEN                0x10
#define CE6353_ENABLE_DVRPORTDIS              0x08
#define CE6353_ENABLE_MPEGDIS                 0x04
#define CE6353_ENABLE_ADCEN                   0x02
#define CE6353_ENABLE_CHIPEN                  0x01


/* CE6353_CLKCTL0   0x51*/
#define CE6353_CLKCTL0_HIGHSAMPLE             0x04
#define CE6353_CLKCTL0_SERIALOUT              0x20
#define CE6353_CLKCTL0_MCLKINVERT             0x40
/* CE6353_CLKCTL1   0x52*/
#define CE6353_CLKCTL1_ADCXTAL                0x01
/*CE6353_RESET        0x55*/
#define CE6353_RESET_FULL                     0x80
#define CE6353_RESET_PART                     0x40
/*CE6353_AGCTARGET    0x56*/
#define REG_AGCTARGET_REVAGC                  0x80
    
/* CE6353_OUTCTL0    0x5A*/
#define CE6353_OUTCTL0_BG12_5B                0x01
#define CE6353_OUTCTL0_MDO_SWP                0x02
#define CE6353_OUTCTL0_BKERR_INV              0x04
#define CE6353_OUTCTL0_EN_TEI                 0x08
#define CE6353_OUTCTL0_MCLKIN_EN              0x10
#define CE6353_OUTCTL0_MCLKRAT_EN             0x20
#define CE6353_OUTCTL0_AGC2_DIS               0x40
#define CE6353_OUTCTL0_RFADC_EN               0x80

/*CE6353_ACQCTL        0x5E*/
#define CE6353_ACQCTL_FORCEMODE               0x01
#define CE6353_ACQCTL_FORCEGUARD              0x02

/*CE6353_SCANCTL       0x62*/
#define CE6353_SCANCTL_TUNR_DIV_0             0x20
#define CE6353_SCANCTL_TUNR_DIV_1             0x40
#define CE6353_SCANCTL_BUSMASTER              0x08
#define CE6353_SCANCTL_PASSTHRO               0x10

/*CE6353_GPPCTL        0x63*/
#define CE6353_GPPCTL_GPP0                    0x01
#define CE6353_GPPCTL_GPP1                    0x02
#define CE6353_GPPCTL_GPP2                    0x04
#define CE6353_GPPCTL_GPP3                    0x08
#define CE6353_GPPCTL_GPP0_OUT                0x10
#define CE6353_GPPCTL_GPP1_OUT                0x20
#define CE6353_GPPCTL_GPP2_OUT                0x40
#define CE6353_GPPCTL_GPP3_OUT                0x80

/*CE6353_TUNERGO      0x70*/
#define CE6353_TUNERGO_GO                     0x01
/*CE6353_FSMGO        0x71*/
#define CE6353_FSMGO_GO                       0x01

/* CE6353_ADCCTL0     0xEA*/
#define CE6353_ADCCTL0_CAL                    0x01
/*********************************************************************/


/*!
*******************************************************************************
**
** \name FE_OFDM_SYSTEM_PARAMS_S
** \brief Data structure to store during run time all hardware information.
**
*******************************************************************************
*/
typedef struct
{
    uint8_t                  mod_type;           //!< index of modulator used
    FE_SYSTEM_PARAMS_S sysParams;
    FE_STATUS_E        feStatus;
    uint8_t                  feInitialized;
    uint32_t                 frequency;          //!< real frequency tuner is locked to
    FE_BANDWIDTH_E     bandwidth;
    uint32_t                 demodADCFreq;       //!< ADC sampling frequency in kHz
    uint32_t                 demodFECFreq;       //!< FEC clock in kHz
    uint8_t                  antPower;           //!< FTRUE = Power On, FFALSE = Power Off
} FE_OFDM_SYSTEM_PARAMS_S;

typedef int32_t (*FE_TUNER_INIT_FCT_T) (uint8_t index);
typedef int32_t (*FE_TUNER_OPEN_FCT_T) (uint8_t index, uint32_t frequency, uint32_t bandwidth);
typedef int32_t (*FE_TUNER_CLOSE_FCT_T)(uint8_t index);

/* ------- public CE6353 API functions -------------------------------- */
int32_t CE6353_ConnectToTuner(uint8_t index);
int32_t CE6353_DisconnectTuner(uint8_t index);
int32_t CE6353_Geti2cId(uint8_t index);
int32_t CE6353_ReadFromTuner(uint8_t index, uint8_t *wData, uint8_t wLen, uint8_t *rData, uint8_t rLen);
int32_t CE6353_ReadDirectFromTuner(uint8_t index, uint8_t *data, uint8_t length);
int32_t CE6353_SetTunerStatus(uint8_t index, FE_STATUS_E status);
int32_t CE6353_StoreTunerFrequency(uint8_t index, uint32_t freq);
int32_t CE6353_WriteDirectToTuner(uint8_t index, uint8_t *data, uint8_t length);
int32_t CE6353_WriteToTuner(uint8_t index, uint8_t *data, uint8_t length);
int32_t CE6353_WaitTunerLock (uint8_t index, uint8_t wrLength);

#ifdef __cplusplus
}
#endif
#endif  /* _OFDM_CE6353_PRIV_H_ */
