/******************************************************************************
**
** \file      ./subsystems/fe/fedirect/src/fedirect.c
**
** \brief     front-end device driver for direct input.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: fedirect.c 3528 2009-05-13 13:42:53Z sgehre $
**
******************************************************************************/

#include "../src/fe_priv.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static FE_SYSTEM_PARAMS_S sysParams[2];
/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/
int32_t FE_DIRECT_Open(uint8_t index, FE_HANDLE *pDeviceData);
int32_t FE_DIRECT_Close(uint8_t index);
int32_t FE_DIRECT_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t FE_DIRECT_GetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t FE_DIRECT_Reset(uint8_t index);
int32_t FE_DIRECT_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams);
int32_t FE_DIRECT_GetStatus(uint8_t index, FE_STATUS_E* pStatus);
int32_t FE_DIRECT_GetAFC(uint8_t index, int32_t* pAFC);
int32_t FE_DIRECT_GetBER(uint8_t index, uint32_t* pBER);
int32_t FE_DIRECT_GetSignalQuality(uint8_t index, uint16_t* pQuality);
int32_t FE_DIRECT_GetSignalStrength(uint8_t index, uint16_t* pStrength);
int32_t FE_DIRECT_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams);
int32_t FE_DIRECT_SendDiseqcCommand(uint8_t  index, 
                                 uint8_t  length, 
                                 uint8_t* pCommands, 
                                 uint8_t* pReturnLength, 
                                 uint8_t* pReturnBuffer);
int32_t FE_DIRECT_Scan(uint8_t index, FE_SCAN_PARAMS_S* pScanParams);

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* call back functions                                                       */
/*---------------------------------------------------------------------------*/

int32_t FE_DIRECT_Detect( uint8_t slot, feHandleT *hp)
{
    if (slot >= FE_MAX_NUM)
    {
        return FE_ERR_BAD_PARAMETER;
    }
    if(slot)
    {
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    hp->systemType             = FE_SYSTEM_TYPE_DIRECT;
    hp->feOpenFct              = FE_DIRECT_Open;
    hp->feCloseFct             = FE_DIRECT_Close;
    hp->feSetSystemParamsFct   = FE_DIRECT_SetSystemParams;
    hp->feGetSystemParamsFct   = FE_DIRECT_GetSystemParams;
    hp->feResetFct             = FE_DIRECT_Reset;
    hp->feTuneFct              = FE_DIRECT_Tune;
    hp->feScanFct              = NULL;
    hp->feSendDiseqcCommandFct = NULL;
    hp->feGetStatusFct         = FE_DIRECT_GetStatus;
    hp->feGetFECFct            = NULL;
    hp->feGetAFCFct            = FE_DIRECT_GetAFC;
    hp->feGetBERFct            = FE_DIRECT_GetBER;
    hp->feGetSignalQualityFct  = FE_DIRECT_GetSignalQuality;
    hp->feGetSignalStrengthFct = FE_DIRECT_GetSignalStrength;
    hp->feGetTuneParamsFct     = NULL;
    hp->feSetAntennaPowerFct   = NULL;
    hp->isPresent              = 1;
    return FE_OK;
}


int32_t FE_DIRECT_Open(uint8_t index, FE_HANDLE* pDeviceData)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    return FE_OK;
}

int32_t FE_DIRECT_Close(uint8_t index)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    return FE_OK;
}

int32_t FE_DIRECT_SetSystemParams(uint8_t index,
                                    FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pSystemParams == NULL) return FE_ERR_BAD_PARAMETER;
    sysParams[index] = *pSystemParams;
    return FE_OK;
}

int32_t FE_DIRECT_GetSystemParams(uint8_t index,
                                    FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pSystemParams == NULL) return FE_ERR_BAD_PARAMETER;
    *pSystemParams = sysParams[index];
    return FE_OK;
}

int32_t FE_DIRECT_Reset(uint8_t index)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    return FE_OK;
}

int32_t FE_DIRECT_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    return FE_OK;
}

int32_t FE_DIRECT_GetStatus(uint8_t index, FE_STATUS_E* pStatus)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pStatus == NULL) return FE_ERR_BAD_PARAMETER;
    *pStatus = FE_STATUS_LOCKED;
    return FE_OK;
}

int32_t FE_DIRECT_GetAFC(uint8_t index, int32_t* pAFC)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pAFC == NULL) return FE_ERR_BAD_PARAMETER;
    *pAFC = 1;
    return FE_OK;
}

int32_t FE_DIRECT_GetBER(uint8_t index, uint32_t* pBER)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pBER == NULL) return FE_ERR_BAD_PARAMETER;
    *pBER = 1;
    return FE_OK;
}

int32_t FE_DIRECT_GetSignalQuality(uint8_t index, uint16_t* pQuality)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pQuality == NULL) return FE_ERR_BAD_PARAMETER;
    *pQuality = 64000;
    return FE_OK;
}

int32_t FE_DIRECT_GetSignalStrength(uint8_t index, uint16_t* pStrength)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pStrength == NULL) return FE_ERR_BAD_PARAMETER;
    *pStrength = 64000;
    return FE_OK;
}

/* Version History */
/******************************************************************************

$Log: fedirect.c,v $
Revision 1.1  2008/08/19 16:20:54  sgehre
Added a dummy frontend for direct TS feeding.
It is disabled by default.
If enabled, frontend detection will never fail,
but will fall back to the dummy frontend instead.


******************************************************************************/
