/***************************************************************************************
*   Copyright (c) 2008, MONTAGE PROPRIETARY AND CONFIDENTIAL
*   Montage Technology (Shanghai) Inc.
*   All Rights Reserved
*------------------------------------
*
* FILE: mt_fe_dmd_ds3000.c
*
* VERSION:    2.06.01
*
* DESCRIPTION:
*    Define the interfaces to initialize and get the channel info from DS3000.
*
* History:
* Description     Version       Date            Author
*---------------------------------------------------------------------------
* File Create     1.00.00      2008.08.21        Rick    
* Modify            1.01.00     2008.10.13        Rick
* Modify            1.10.00     2008.11.17        Rick
* Modify            1.10.01     2008.12.23        Rick
* Modify            2.00.00        2009.03.19        Rick
* Modify            2.01.00        2009.04.29        Rick
* Modify            2.02.00        2009.05.20        Rick
* Modify            2.03.00        2009.07.14        Rick
* Modify            2.04.00        2009.07.28        Rick
* Modify            2.05.00        2009.08.25        Rick
* Modify            2.06.00        2009.09.07        Rick
* Modify            2.06.01        2009.11.10        Rick
***************************************************************************************/

#include "stdio.h"
#include "../src/fe_priv.h"
#include "mt_fe_def.h"
#include "mt_fe_i2c.h"
#include "mt_fe_dmd_ds3000_fw.h"
#include "mt_fe_dmd_ds3000_reg_tbl.h"
#ifndef _LINUX_FAPI
#include <fapi/sys_services.h>
#endif


/************************
****external function****
*************************/

void _mt_sleep(U32 ms)
{
    FAPI_SYS_SLEEP(ms);
}


/************************
****internal function****
*************************/
MT_FE_RET _mt_fe_dmd_ds3k_set_demod(U32 sym_rate_KSs, MT_FE_TYPE type);
MT_FE_RET _mt_fe_dmd_ds3k_init_reg(const U8 (*p_reg_tabl)[2], S32 size);
MT_FE_RET _mt_fe_dmd_ds3k_download_fw(const U8* p_fw);
MT_FE_RET _mt_fe_dmd_ds3k_set_ts_out_mode(U8 mode);
MT_FE_RET _mt_fe_dmd_ds3k_set_CCI(MT_FE_ON_OFF is_cci_on);
MT_FE_RET _mt_fe_dmd_ds3k_check_CCI(MT_FE_LOCK_STATE *p_state);
MT_FE_RET _mt_fe_dmd_ds3k_set_clock_ratio(void);
MT_FE_RET _mt_fe_dmd_ds3k_set_sym_rate(U32 sym_rate_KSs);
MT_FE_RET _mt_fe_dmd_ds3k_get_sym_rate(U32 *sym_rate_KSs);
MT_FE_RET _mt_fe_dmd_ds3k_set_carrier_offset(S32 carrier_offset_KHz);
MT_FE_RET _mt_fe_dmd_ds3k_get_carrier_offset(S32 *carrier_offset_KHz);
MT_FE_RET _mt_fe_dmd_ds3k_get_total_carrier_offset(S32 *carrier_offset_KHz);
MT_FE_RET _mt_fe_dmd_ds3k_get_fec(MT_FE_TP_INFO *p_info, MT_FE_TYPE tp_type);

/* function pointer */
S32 (*mt_fe_tn_get_offset)(void);
MT_FE_RET (*mt_fe_tn_set_tuner)(U32 freq_KHz, U32 sym_rate_KSs, S16 lpf_offset_KHz);
MT_FE_RET (*mt_fe_tn_cal_offset)(U32 freqKHz, U32 *realfreqKHz);
MT_FE_RET (*mt_fe_tn_sleep)(void);
MT_FE_RET (*mt_fe_tn_wake_up)(void);


/* function for blind scan*/
MT_FE_RET _mt_fe_dmd_ds3k_bs_set_reg(U8 bs_times);
MT_FE_RET _mt_fe_dmd_ds3k_bs_set_demod(U32 freq_KHz, U32 sym_rate_KSs, MT_FE_TYPE type);
MT_FE_RET _mt_fe_dmd_ds3k_bs_remove_unlocked_TP(MT_FE_BS_TP_INFO *p_bs_info, U32 compare_freq_KHz);
MT_FE_RET _mt_fe_dmd_ds3k_bs_process_scanned_TP(MT_FE_BS_TP_INFO *p_bs_info,
                                                       U32 cur_scan_freq_KHz,
                                                       U8  bs_times);
MT_FE_RET _mt_fe_dmd_ds3k_bs_save_scanned_TP(MT_FE_BS_TP_INFO *p_bs_info,
                                                    U32 freq_KHz,
                                                    U32 symbol_rate_KSs,
                                                    U8  bs_times);
MT_FE_RET _mt_fe_dmd_ds3k_bs_TP_scan(U32 start_freq_KHz,
                                            U32 end_freq_KHz,
                                            MT_FE_BS_TP_INFO *p_bs_info,
                                            U16 *scanned_tp,
                                            U8 bs_times);
MT_FE_RET _mt_fe_dmd_ds3k_bs_A(U32 start_freq_KHz,
                                      U32 end_freq_KHz,
                                      MT_FE_BS_TP_INFO *p_bs_info,
                                      U8 bs_times);
MT_FE_RET _mt_fe_dmd_ds3k_bs_B(U32 start_freq_KHz,
                                      U32 end_freq_KHz,
                                      MT_FE_BS_TP_INFO *p_bs_info,
                                      U8 bs_times);
MT_FE_RET _mt_fe_dmd_ds3k_bs_connect(MT_FE_BS_TP_INFO *p_bs_info,
                                            U16 start_index,
                                            U16 scanned_tp);


void mt_fe_dummy_function(MT_FE_MSG msg, void* p_param)
{
    UNUSED_PARAMETER(msg);
    UNUSED_PARAMETER(p_param);
}
void (*mt_fe_bs_notify)(MT_FE_MSG msg, void *p_tp_info) = mt_fe_dummy_function;



/**********************
****global variable****
***********************/
typedef struct {
    U32 g_connect_sym_rate;
    U32 g_auto_tune_cnt;
    U32 g_cci_detect;
    MT_FE_TYPE g_current_type;
    MT_FE_TYPE g_connect_type;
    U32 g_check_TS_clock;

    /*blind scan*/
    U8  last_tpA_flag;
    U16 g_tp_index;
    U16 g_locked_tp_cnt;
    U16 g_scanned_tp_cnt;
    U32  g_cur_compare_TPA_KHz;
    MT_FE_BOOL  g_blindscan_cancel;
   U32  time_total;
   U32  g_bs_time_total;
} ds3000Ctx;


static ds3000Ctx feState[2] = {
    { 0, 0, 0, MtFeType_DvbS, MtFeType_DvbS, 0, 0, 0, 0, 0, 0, MtFe_False, 0, 0 },
    { 0, 0, 0, MtFeType_DvbS, MtFeType_DvbS, 0, 0, 0, 0, 0, 0, MtFe_False, 0, 0 }
};
extern U8 feIndex;

/************************************************************************
** Function: mt_fe_dmd_ds3k_get_driver_version
**
**
** Discription: This function is used to get the driver version in number
**              mode.
**
**
** Inputs:   None
**
**
** Outputs:
**
**    Parameter         Type        Description
**  ----------------------------------------------------------------------
**   p_version          U32*        version number pointer
**
*************************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_get_driver_version(U32* p_version)
{
    *p_version = 20500;    /* v2.05.00 */

    return MtFeErr_Ok;
}


/************************************************************************
** Function: mt_fe_dmd_ds3k_soft_reset
**
**
** Discription: This function is used to do software reset.
**
**
** Inputs:   None
**
**
** Outputs:  None
**
*************************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_soft_reset(void)
{

    feState[feIndex].g_cci_detect        = 1;
    feState[feIndex].g_auto_tune_cnt     = 0;

    _mt_fe_dmd_set_reg(0xb2, 0x01);
    _mt_fe_dmd_set_reg(0x00, 0x01);
    _mt_sleep(1);
    _mt_fe_dmd_set_reg(0x00, 0x00);
    _mt_fe_dmd_set_reg(0xb2, 0x00);


    return MtFeErr_Ok;
}



/******************************************************************
** Function: mt_fe_dmd_ds3k_hard_reset
**
**
** Discription: This function is used to do chip reset by GPIO pin
**
**
** Inputs:   None
**
**
** Outputs:  None
**
*******************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_hard_reset(void)
{
    U8 val;

    #if (MT_FE_CHECK_CARRIER_OFFSET != 0)
        U8 val_reg00, val_regb2;
    #endif

    /*
        TODO:
             do the hardware reset by the GPIO pin.
    */


    #if (MT_FE_CHECK_CARRIER_OFFSET != 0)
        _mt_fe_dmd_get_reg(0xb2, &val_regb2);
        _mt_fe_dmd_get_reg(0x00, &val_reg00);
        if(((val_reg00&(~0xe0)) == 0x01)&&(val_regb2 == 0x01))
        {
            _mt_fe_dmd_set_reg(0x00, 0x00);
            _mt_fe_dmd_set_reg(0xb2, 0x00);
        }
    #endif

    _mt_fe_dmd_set_reg(0x07, 0x80);
    _mt_fe_dmd_set_reg(0x07, 0x00);
    _mt_sleep(1);

    _mt_fe_dmd_get_reg(0x08, &val);
    val |= 0x01;
    _mt_fe_dmd_set_reg(0x08, val);


    /* reset global defines*/
    feState[feIndex].g_current_type      = MtFeType_DvbS;
    feState[feIndex].g_connect_type      = MtFeType_DvbS;
    feState[feIndex].g_connect_sym_rate  = 0;
    feState[feIndex].g_auto_tune_cnt     = 0;
    feState[feIndex].g_cci_detect        = 0;

    feState[feIndex].g_check_TS_clock = 0;

    _mt_sleep(1);

    return MtFeErr_Ok;
}




/******************************************************************************
** Function: mt_fe_dmd_ds3k_init
**
**
** Discription:    This function is used to initial ds3k. Only used
**              once after power on.
**
**
** Note:
**
**      You can change the TS output mode with the define "MT_FE_TS_OUPUT_MODE"
**  in "Mt_Fe_def.h"
**
**
** Inputs:   None
**
** Outputs:  None
**
*******************************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_init(U8 tunerType)
{
    mt_fe_dmd_ds3k_hard_reset();

    if(tunerType == 0) {
#if (MT_FE_TN_SUPPORT_TS2020 > 0)
        mt_fe_tn_set_tuner = mt_fe_tn_set_freq_ts2020;
        mt_fe_tn_get_offset = mt_fe_tn_get_tuner_freq_offset_ts2020;
        mt_fe_tn_sleep      = mt_fe_tn_sleep_ts2020;
        mt_fe_tn_wake_up    = mt_fe_tn_wake_up_ts2020;
        
        mt_fe_tn_init_ts2020();
#endif
    } else if(tunerType == 1) {
#if (MT_FE_TN_SUPPORT_SHARP6306 > 0)
        mt_fe_tn_set_tuner = mt_fe_tn_set_freq_sharp6306;
        mt_fe_tn_get_offset = mt_fe_tn_get_tuner_freq_offset_sharp6306;
        mt_fe_tn_sleep      = NULL;
        mt_fe_tn_wake_up    = NULL;
#endif
    }
    _mt_fe_dmd_ds3k_download_fw(m88ds3k_fm);

    _mt_fe_dmd_ds3k_set_ts_out_mode(MT_FE_TS_OUTPUT_MODE);


    return MtFeErr_Ok;
}


/************************************************************************
** Function: mt_fe_dmd_ds3k_sleep
**
**
** Discription:    This function is used to sleep ds300x.
** 
**
** Inputs:   None
**
**
** Outputs:  None
**
*************************************************************************/ 
MT_FE_RET mt_fe_dmd_ds3k_sleep(void)
{
    U8 val;


    if(mt_fe_tn_sleep != NULL)
        mt_fe_tn_sleep();

    _mt_fe_dmd_get_reg(0x08, &val);
    val &= ~0x01;
    _mt_fe_dmd_set_reg(0x08, val);

    _mt_fe_dmd_get_reg(0x04, &val);
    val |= 0x01;
    _mt_fe_dmd_set_reg(0x04, val);

    _mt_fe_dmd_get_reg(0x23, &val);
    val |= 0x10;
    _mt_fe_dmd_set_reg(0x23, val);


    return MtFeErr_Ok;
}



/************************************************************************
** Function: mt_fe_dmd_ds3k_wake_up
**
**
** Discription:    This function is used to wake up ds300x.
** 
**
** Inputs:   None
**
**
** Outputs:  None
**
*************************************************************************/ 
MT_FE_RET mt_fe_dmd_ds3k_wake_up(void)
{
    U8 val;

    _mt_fe_dmd_get_reg(0x08, &val);
    val |= 0x01;
    _mt_fe_dmd_set_reg(0x08, val);

    _mt_fe_dmd_get_reg(0x04, &val);
    val &= ~0x01;
    _mt_fe_dmd_set_reg(0x04, val);


    _mt_fe_dmd_get_reg(0x23, &val);
    val &= ~0x10;
    _mt_fe_dmd_set_reg(0x23, val);


    if(mt_fe_tn_wake_up != NULL)
        mt_fe_tn_wake_up();

    return MtFeErr_Ok;
}



/**********************************************************************************************
** Function: mt_fe_dmd_ds3k_connect
**
**
** Discription: This function is used to lock the specified TP.
**
**
** Inputs:
**
**      Parameter       Type            Description
**      ----------------------------------------------------------------------
**      freq_MHz        U32         TP's frequency(MHz) passed by app level
**      sym_rate_KSs    U32         TP's symbol rate(KSs) passed by app level
**      dvbs_type       MT_FE_TYPE  DVBS or DVBS2 mode  passed by app level
**
**
**  Note:
**
**      The dvbs_type passed by app level may be 3 mode.
**      "MtFeType_DvbS" or "MtFeType_DvbS2" or "MtFeType_DvbS_Unknown".
**
**      If the type is  "MtFeType_DvbS_Unknown", driver will try DVBS and DVBS2
**  automatically.
**
**
**
** Outputs:  None
**
**********************************************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_connect(U32 freq_MHz, U32 sym_rate_KSs, MT_FE_TYPE dvbs_type)
{
    S16 lpf_offset_KHz;
    U32 freq_tmp_KHz;

    #if (MT_FE_CHECK_CARRIER_OFFSET != 0)
    U8  val_regb2, val_reg00;
    #endif

    if((mt_fe_tn_set_tuner == NULL)||(mt_fe_tn_get_offset == NULL))
        return MtFeErr_NullPointer;

    feState[feIndex].g_connect_type     = dvbs_type;
    feState[feIndex].g_connect_sym_rate = sym_rate_KSs;
    feState[feIndex].g_auto_tune_cnt    = 0;
    feState[feIndex].g_cci_detect       = 1;

    #if (MT_FE_TS_CLOCK_AUTO_SET_FOR_CI_MODE != 0)
    feState[feIndex].g_check_TS_clock   = 1;
    #endif

    #if (MT_FE_CHECK_CARRIER_OFFSET != 0)
    _mt_fe_dmd_get_reg(0xb2, &val_regb2);
    _mt_fe_dmd_get_reg(0x00, &val_reg00);
    if(((val_reg00&(~0xe0)) == 0x01)&&(val_regb2 == 0x01))
    {
        _mt_fe_dmd_set_reg(0x00, 0x00);
        _mt_fe_dmd_set_reg(0xb2, 0x00);
    }
    #endif

    if (dvbs_type == MtFeType_DvbS_Unknown)
        feState[feIndex].g_current_type = MtFeType_DvbS;
    else
        feState[feIndex].g_current_type = dvbs_type;


    freq_tmp_KHz    =  freq_MHz*1000;
    lpf_offset_KHz  =  0;

#if ((MT_FE_TN_SUPPORT_TS2020 != 0)||(MT_FE_TN_SUPPORT_SHARP6306 != 0))
    if (sym_rate_KSs < 5000)
    {
        lpf_offset_KHz = FREQ_OFFSET_AT_SMALL_SYM_RATE_KHz;
        freq_tmp_KHz  += FREQ_OFFSET_AT_SMALL_SYM_RATE_KHz;
    }
#endif

    /*set tuner*/
    mt_fe_tn_set_tuner(freq_tmp_KHz, sym_rate_KSs, lpf_offset_KHz);

    _mt_fe_dmd_set_reg(0xb2, 0x01);
    _mt_fe_dmd_set_reg(0x00, 0x01);


    /*set demod registers*/
    _mt_fe_dmd_ds3k_set_demod(sym_rate_KSs, feState[feIndex].g_current_type);


    /*set carrier offset*/
#if MT_FE_FOR_GMI_HW_V10_APP
    _mt_fe_dmd_ds3k_set_carrier_offset(-(S32)mt_fe_tn_get_offset() - (freq_tmp_KHz - freq_MHz*1000));
#else
    _mt_fe_dmd_ds3k_set_carrier_offset((S32)mt_fe_tn_get_offset() + (freq_tmp_KHz - freq_MHz*1000));
#endif

    _mt_fe_dmd_set_reg(0x00, 0x00);
    _mt_fe_dmd_set_reg(0xb2, 0x00);

    return MtFeErr_Ok;
}



/******************************************************************
** Function: mt_fe_dmd_ds3k_get_lock_state
**
**
** Discription: This function is called by app level to get the
**          lock status of ds3k.
**
**
** Inputs:   None
**
**
** Outputs:
**
**      Parameter           Type        Description
**      -----------------------------------------------------------
**      p_state     MT_FE_LOCK_STATE*   lock status of ds3k
**
**
*******************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_get_lock_state(MT_FE_LOCK_STATE *p_state)
{
    U8  tmp;

#if (MT_FE_CHECK_CARRIER_OFFSET != 0)
    U8 val_reg00, val_regb2;
    S32 carrier_offset_KHz;
#endif
    if (p_state == 0)
        return MtFeErr_Param;


    *p_state = MtFeLockState_Waiting;

#if (MT_FE_CHECK_CARRIER_OFFSET != 0)
    _mt_fe_dmd_get_reg(0xb2, &val_regb2);
    _mt_fe_dmd_get_reg(0x00, &val_reg00);
    if(((val_reg00&(~0xe0)) == 0x01)&&(val_regb2 == 0x01))
    {
        *p_state = MtFeLockState_Unlocked;

        return MtFeErr_Ok;
    }
#endif
    if(feState[feIndex].g_cci_detect == 1)
    {
        _mt_fe_dmd_ds3k_check_CCI(p_state);

        if(*p_state == MtFeLockState_Unlocked)
            return MtFeErr_Ok;

        feState[feIndex].g_cci_detect = 0;
    }


    if (feState[feIndex].g_current_type == MtFeType_DvbS2)
    {
        _mt_fe_dmd_get_reg(0x0d, &tmp);

        if ((tmp & 0x8f) == 0x8f)
            *p_state = MtFeLockState_Locked;
    }
    else
    {
        _mt_fe_dmd_get_reg(0xd1, &tmp);

        if ((tmp & 0x07) == 0x07)
            *p_state = MtFeLockState_Locked;
    }


    if (*p_state == MtFeLockState_Locked)
    {
    #if MT_FE_CHECK_CARRIER_OFFSET
        carrier_offset_KHz = 0;
        _mt_fe_dmd_ds3k_get_total_carrier_offset(&carrier_offset_KHz);

        if((carrier_offset_KHz >= MT_FE_CARRIER_OFFSET_KHz)||
           (carrier_offset_KHz <= - MT_FE_CARRIER_OFFSET_KHz))
        {
            _mt_fe_dmd_set_reg(0xb2, 0x01);
            _mt_fe_dmd_set_reg(0x00, 0x01);

            *p_state = MtFeLockState_Unlocked;
        }
    #endif

    #if (MT_FE_TS_CLOCK_AUTO_SET_FOR_CI_MODE != 0)
        if(feState[feIndex].g_check_TS_clock == 1)
        {
            feState[feIndex].g_check_TS_clock = 0;
            _mt_fe_dmd_ds3k_set_clock_ratio();
        }
    #endif

        return MtFeErr_Ok;
    }
    else
    {
    #if (MT_FE_TS_CLOCK_AUTO_SET_FOR_CI_MODE != 0)
        feState[feIndex].g_check_TS_clock = 1;
    #endif
    }


    _mt_fe_dmd_get_reg(0xb3, &tmp);
    if(feState[feIndex].g_connect_type != MtFeType_DvbS_Unknown)
    {
        if ((tmp & 0xc0) != 0)
        {
            *p_state = MtFeLockState_Unlocked;
            return MtFeErr_Ok;
        }
    }
    else
    {
        if((tmp & 0x80) == 0x80)
        {
            *p_state = MtFeLockState_Unlocked;
            return MtFeErr_Ok;
        }


        if((tmp & 0x40) == 0x40)
        {
            if(feState[feIndex].g_auto_tune_cnt < 2)
            {
                feState[feIndex].g_current_type = (feState[feIndex].g_current_type == MtFeType_DvbS) ? MtFeType_DvbS2 : MtFeType_DvbS;
                _mt_fe_dmd_ds3k_set_demod(feState[feIndex].g_connect_sym_rate, feState[feIndex].g_current_type);
            }
            else
            {
                *p_state = MtFeLockState_Unlocked;
                return MtFeErr_Ok;
            }
        }
    }


    return MtFeErr_Ok;
}


/*********************************************************************
** Function: mt_fe_dmd_ds3k_get_per
**
**
** Discription: Get the two package counter value to calculate the per at
**          app level.
**
**
**
** Inputs:   None
**
**
** Outputs:
**
**    Parameter         Type            Description
**  ---------------------------------------------------------------
**    p_total_packags   U32*            total packags
**    p_err_packags     U32*            error packags
**
**
** NOTE:    This function outputs error and total packages in DVBS2 mode,
**      and error and total bits in DVBS mode. You can calculate the PER
**      and BER in APP level.
**
**********************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_get_per(U32 *p_total_packags, U32 *p_err_packags)
{
    #define TOTAL_PACKET_S  3000

    U8      tmp1, tmp2, tmp3;
    U8      val_0xF6, val_0xF7, val_0xF8;
    U32     code_rate_fac, ldpc_frame_cnt;
    static  U32 pre_err_packags   = 0;
    static  U32 pre_total_packags = 0xffff;
    MT_FE_CHAN_INFO_DVBS2 p_info;


    *p_err_packags   = pre_err_packags;
    *p_total_packags = pre_total_packags;


    pre_err_packags = 0;


    if (feState[feIndex].g_current_type == MtFeType_DvbS2)
    {
        mt_fe_dmd_ds3k_get_channel_info(&p_info);
        switch (p_info.code_rate)
        {
            case MtFeCodeRate_1_4:  code_rate_fac = 16008 - 80; break;
            case MtFeCodeRate_1_3:  code_rate_fac = 21408 - 80; break;
            case MtFeCodeRate_2_5:  code_rate_fac = 25728 - 80; break;
            case MtFeCodeRate_1_2:  code_rate_fac = 32208 - 80; break;
            case MtFeCodeRate_3_5:  code_rate_fac = 38688 - 80; break;
            case MtFeCodeRate_2_3:  code_rate_fac = 43040 - 80; break;
            case MtFeCodeRate_3_4:  code_rate_fac = 48408 - 80; break;
            case MtFeCodeRate_4_5:  code_rate_fac = 51648 - 80; break;
            case MtFeCodeRate_5_6:  code_rate_fac = 53840 - 80; break;
            case MtFeCodeRate_8_9:  code_rate_fac = 57472 - 80; break;
            case MtFeCodeRate_9_10: code_rate_fac = 58192 - 80; break;
            default:    break;
        }

        _mt_fe_dmd_get_reg(0xd7, &tmp1);
        _mt_fe_dmd_get_reg(0xd6, &tmp2);
        _mt_fe_dmd_get_reg(0xd5, &tmp3);
        ldpc_frame_cnt = (tmp1 << 16) | (tmp2 << 8) | tmp3;


        _mt_fe_dmd_get_reg(0xf8, &tmp1);
        _mt_fe_dmd_get_reg(0xf7, &tmp2);
        pre_err_packags = tmp1<<8 | tmp2;


        pre_total_packags = code_rate_fac * ldpc_frame_cnt / (188*8);


        if (ldpc_frame_cnt > TOTAL_PACKET_S)
        {
            _mt_fe_dmd_set_reg(0xd1, 0x01);
            _mt_fe_dmd_set_reg(0xf9, 0x01);
            _mt_fe_dmd_set_reg(0xf9, 0x00);
            _mt_fe_dmd_set_reg(0xd1, 0x00);

            *p_err_packags    = pre_err_packags;
            *p_total_packags  = pre_total_packags;
        }
    }
    else
    {
        _mt_fe_dmd_set_reg(0xf9, 0x04);
        _mt_fe_dmd_get_reg(0xf8, &val_0xF8);

        if((val_0xF8&0x10) == 0)
        {
            _mt_fe_dmd_get_reg(0xf6, &val_0xF6);
            _mt_fe_dmd_get_reg(0xf7, &val_0xF7);

            pre_err_packags = (val_0xF7<<8) | val_0xF6;

            val_0xF8 |= 0x10;
            _mt_fe_dmd_set_reg(0xf8, val_0xF8);
        }

        *p_err_packags   = pre_err_packags;

        if((val_0xF8&0x08) == 0)
            *p_total_packags = 8388608;
        else
            *p_total_packags = 1048576;
    }


    return MtFeErr_Ok;
}



/******************************************************************
** Function: mt_fe_dmd_ds3k_get_snr
**
**
** Discription: get snr value, unit dB
**
**
** Inputs:   None
**
**
** Outputs:
**
**    Parameter     Type        Description
**  ---------------------------------------------------------------
**    p_snr         S8*         snr with dB unit
**
*******************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_get_snr(S8 *p_snr)
{
    const U16 mes_log10[] =
    {
        0,      3010,   4771,   6021,   6990,   7781,   8451,   9031,   9542,   10000,
        10414,  10792,  11139,  11461,  11761,  12041,  12304,  12553,  12788,  13010,
        13222,  13424,  13617,  13802,  13979,  14150,  14314,  14472,  14624,  14771,
        14914,  15052,  15185,  15315,  15441,  15563,  15682,  15798,  15911,  16021,
        16128,  16232,  16335,  16435,  16532,  16628,  16721,  16812,  16902,  16990,
        17076,  17160,  17243,  17324,  17404,  17482,  17559,  17634,  17709,  17782,
        17853,  17924,  17993,  18062,  18129,  18195,  18261,  18325,  18388,  18451,
        18513,  18573,  18633,  18692,  18751,  18808,  18865,  18921,  18976,  19031
    };


    const U16 mes_loge[] =
    {
        0,      6931,   10986,  13863,  16094,  17918,  19459,  20794,  21972,  23026,
        23979,  24849,  25649,  26391,  27081,  27726,  28332,  28904,  29444,  29957
    };



    static S8 snr;

    U8      val, npow1, npow2, spow1, cnt;
    U16     tmp;
    U32     npow, spow, snr_total;


    *p_snr = 0;
    if (feState[feIndex].g_current_type == MtFeType_DvbS2)
    {
        cnt  = 10;
        npow = 0;
        spow = 0;

        while(cnt >0)
        {
            _mt_fe_dmd_get_reg(0x8c, &npow1);
            _mt_fe_dmd_get_reg(0x8d, &npow2);
            npow += (((npow1 & 0x3F) + (U16)(npow2 << 6)) >> 2);

            _mt_fe_dmd_get_reg(0x8e, &spow1);
            spow += ((spow1 * spow1) >> 1);

            cnt--;
        }

        npow /= 10;
        spow /= 10;

        if(spow == 0)
        {
            snr = 0;
        }
        else if(npow == 0)
        {
            snr = 19;
        }
        else
        {
            if(spow > npow)
            {
                tmp = (U16)((spow) / npow);
                if (tmp > 80)
                    tmp = 80;

                snr = (S8)(mes_log10[tmp - 1] / 1000);
            }
            else
            {
                tmp = (U16)(npow / spow);
                if (tmp > 80)
                    tmp = 80;

                snr = (S8)(-(mes_log10[tmp] / 1000));
            }
        }
    }
    else
    {
        cnt = 10;
        snr_total = 0;

        while(cnt > 0)
        {
            _mt_fe_dmd_get_reg(0xff, &val);
            snr_total += val;

            cnt--;
        }

        tmp = (U16)(snr_total/80);
        if(tmp > 0)
        {
            if (tmp > 20)
                tmp = 20;

            snr =  (S8)((mes_loge[tmp - 1] * 10) / 23026);
        }
        else
        {
            snr = 0;
        }
    }



    *p_snr = snr;


    return MtFeErr_Ok;
}



/******************************************************************
** Function: mt_fe_dmd_ds3k_get_strength
**
**
** Discription: get signal strength
**
**
** Inputs:   None
**
**
** Outputs:
**
**    Parameter         Type        Description
**  ---------------------------------------------------------------
**    p_strength        S8*         signal strength value(0----100)
**
*******************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_get_strength(S8 *p_strength)
{
#if MT_FE_TN_SUPPORT_TS2020
    mt_fe_tn_get_strength_ts2020(p_strength);
#elif MT_FE_TN_SUPPORT_TS2022
    mt_fe_tn_get_strength_ts2022(p_strength);
#elif MT_FE_TN_SUPPORT_SHARP6306
    mt_fe_tn_get_strength_sharp6306(p_strength);
#elif MT_FE_TN_SUPPORT_SERIT_SP761VB
    mt_fe_tn_get_strength_serit761vb(p_strength);
#endif

    return MtFeErr_Ok;
}



/******************************************************************
** Function: mt_fe_dmd_ds3k_get_channel_info
**
**
** Discription: get channel info
**
**
** Inputs:   None
**
**
** Outputs:
**
**    Parameter         Type            Description
**  ---------------------------------------------------------------
**    p_info    MT_FE_CHAN_INFO_DVBS2*  get the channel info.
**
*******************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_get_channel_info(MT_FE_CHAN_INFO_DVBS2 *p_info)
{
    U8  tmp, val_0x7E;


    p_info->type = feState[feIndex].g_current_type;

    if(feState[feIndex].g_current_type == MtFeType_DvbS2)
    {
        _mt_fe_dmd_get_reg(0x7e, &val_0x7E);
        tmp = (U8)((val_0x7E&0xC0) >> 6);
        switch(tmp)
        {
            case 0:  p_info->mod_mode = MtFeModMode_Qpsk;   break;
            case 1:  p_info->mod_mode = MtFeModMode_8psk;   break;
            case 2:  p_info->mod_mode = MtFeModMode_16Apsk; break;
            case 3:  p_info->mod_mode = MtFeModMode_32Apsk; break;
            default: p_info->mod_mode = MtFeModMode_Undef;  break;
        }


        p_info->is_pilot_on = ((val_0x7E&0x20) != 0) ? MtFe_True : MtFe_False;


        tmp = val_0x7E&0x0f;
        switch(tmp)
        {
            case 0:  p_info->code_rate = MtFeCodeRate_1_4;      break;
            case 1:  p_info->code_rate = MtFeCodeRate_1_3;      break;
            case 2:  p_info->code_rate = MtFeCodeRate_2_5;      break;
            case 3:  p_info->code_rate = MtFeCodeRate_1_2;      break;
            case 4:  p_info->code_rate = MtFeCodeRate_3_5;      break;
            case 5:  p_info->code_rate = MtFeCodeRate_2_3;      break;
            case 6:  p_info->code_rate = MtFeCodeRate_3_4;      break;
            case 7:  p_info->code_rate = MtFeCodeRate_4_5;      break;
            case 8:  p_info->code_rate = MtFeCodeRate_5_6;      break;
            case 9:  p_info->code_rate = MtFeCodeRate_8_9;      break;
            case 10: p_info->code_rate = MtFeCodeRate_9_10;     break;
            default: p_info->code_rate = MtFeCodeRate_Undef;    break;
        }


        _mt_fe_dmd_get_reg(0x89, &tmp);
        p_info->is_spectrum_inv = ((tmp&0x80) != 0) ? MtFeSpectrum_Inversion : MtFeSpectrum_Normal;


        _mt_fe_dmd_get_reg(0xf2, &tmp);
        tmp &= 0x03;
        switch(tmp)
        {
            case 0:  p_info->roll_off = MtFeRollOff_0p35;   break;
            case 1:  p_info->roll_off = MtFeRollOff_0p25;   break;
            case 2:  p_info->roll_off = MtFeRollOff_0p20;   break;
            default: p_info->roll_off = MtFeRollOff_Undef;  break;
        }
    }
    else
    {
        p_info->mod_mode = MtFeModMode_Qpsk;


        _mt_fe_dmd_get_reg(0xe6, &tmp);
        tmp = (U8)(tmp >> 5);
        switch(tmp)
        {
            case 0:  p_info->code_rate = MtFeCodeRate_7_8;      break;
            case 1:  p_info->code_rate = MtFeCodeRate_5_6;      break;
            case 2:  p_info->code_rate = MtFeCodeRate_3_4;      break;
            case 3:  p_info->code_rate = MtFeCodeRate_2_3;      break;
            case 4:  p_info->code_rate = MtFeCodeRate_1_2;      break;
            default: p_info->code_rate = MtFeCodeRate_Undef;    break;
        }


        p_info->is_pilot_on = MtFe_False;


        _mt_fe_dmd_get_reg(0xe0, &tmp);
        p_info->is_spectrum_inv = ((tmp&0x04) != 0) ? MtFeSpectrum_Inversion : MtFeSpectrum_Normal;


        p_info->roll_off = MtFeRollOff_0p35;
    }


    return MtFeErr_Ok;
}




/*********************************************************************************
** Function: mt_fe_dmd_ds3k_set_LNB
**
**
** Discription: set LNB status.
**
**
** Inputs:
**
**      Parameter           Type            Description
**  ---------------------------------------------------------------
**      is_LNB_enable   MT_FE_BOOL          Enable or Disable
**      is_22k_enable   MT_FE_BOOL          Enable or Disable
**      mode            MT_FE_LNB_VOLTAGE   13V or 18V
**
**
** Note:
**
**      Here the LNB setting should cooperate with the hardware circuit.
**
**      You should set the define "LNB_ENABLE_WHEN_LNB_EN_HIGH",
**  "LNB_13V_WHEN_VSEL_HIGH", "LNB_VSEL_STANDBY_HIGH" and
**  "LNB_DISEQC_OUT_STANDBY_HIGH" first according to the circuit designed.
**
**
**
** Outputs:     none
**
**
***********************************************************************************/
MT_FE_RET  mt_fe_dmd_ds3k_set_LNB(MT_FE_BOOL is_LNB_enable, MT_FE_BOOL is_22k_enable, MT_FE_LNB_VOLTAGE voltage_type)
{
    U8    val_0x29, val_0xa1, val_0xa2;


    _mt_fe_dmd_get_reg(0x29, &val_0x29);
    _mt_fe_dmd_get_reg(0xa1, &val_0xa1);
    _mt_fe_dmd_get_reg(0xa2, &val_0xa2);


    if(is_LNB_enable != MtFe_True)
    {
        val_0xa1 |= 0x40;

        /*set LNB_EN pin HIGH or LOW */
        #if (LNB_ENABLE_WHEN_LNB_EN_HIGH)
            val_0xa2 &= ~0x02;
        #else
            val_0xa2 |= 0x02;
        #endif


        /*set V_SEL Pin mode*/
        #if (LNB_VSEL_STANDBY_HIGH)
            val_0xa2 |= 0x01;
        #else
            val_0xa2 &= ~0x01;
        #endif


        /*set DiseQc_OUT mode*/
        #if (LNB_DISEQC_OUT_FORCE_HIGH)
            val_0xa2 |= 0xc0;
        #else
            val_0xa2 &= ~0xc0;
            val_0xa2 |= 0x80;
        #endif
    }
    else
    {
        #if (LNB_ENABLE_WHEN_LNB_EN_HIGH)
            val_0xa2 |= 0x02;
        #else
            val_0xa2 &= ~0x02;
        #endif


        #if (LNB_13V_WHEN_VSEL_HIGH)
            if (voltage_type == MtFeLNB_13V)
                val_0xa2 |= 0x01;
            else
                val_0xa2 &= ~0x01;
        #else
            if (voltage_type == MtFeLNB_13V)
                val_0xa2 &= ~0x01;
            else
                val_0xa2 |= 0x01;
        #endif

        #if (LNB_13V_WHEN_GPO_HIGH)
            if (voltage_type == MtFeLNB_13V)
                val_0x29 |= 0x0C;
            else
                val_0xa2 &= ~0x0C;
        #else
            if (voltage_type == MtFeLNB_13V)
                val_0x29 &= ~0x0C;
            else
                val_0x29 |= 0x0C;
        #endif

        if (is_22k_enable == MtFe_True)
        {
            val_0xa1 |= 0x04;
            val_0xa1 &= ~0x03;
            val_0xa1 &= ~0x40;
            val_0xa2 &= ~0xc0;
        }
        else
        {
            #if (LNB_DISEQC_OUT_FORCE_HIGH)
                val_0xa2 |= 0xc0;
            #else
                val_0xa2 &= ~0xc0;
                val_0xa2 |= 0x80;
            #endif
        }
    }


    _mt_fe_dmd_set_reg(0xa2, val_0xa2);
    _mt_fe_dmd_set_reg(0xa1, val_0xa1);
    _mt_fe_dmd_set_reg(0x29, val_0x29);


    return MtFeErr_Ok;
}




/*************************************************************************************
** Function: mt_fe_dmd_ds3k_DiSEqC_send_tone_burst
**
**
** Discription: send DiSEqC message in tone burst mode
**              1) modulated tone burst
**              2) unmodulated tone burst
**
**
** Inputs:
**
**      Parameter           Type                        Description
**  ---------------------------------------------------------------------------------
**      mode            MT_FE_DiSEqC_TONE_BURST     modulated/unmodulated tone burst
**
**
** Outputs:     none
**
**
**************************************************************************************/
MT_FE_RET  mt_fe_dmd_ds3k_DiSEqC_send_tone_burst(MT_FE_DiSEqC_TONE_BURST mode)
{
    U8  val;
    S32 time_out;


    _mt_fe_dmd_get_reg(0xa2, &val);
    val &= ~0xc0;
    _mt_fe_dmd_set_reg(0xa2, val);


    if (mode == MtFeDiSEqCToneBurst_Moulated)
        _mt_fe_dmd_set_reg(0xa1, 0x01);
    else
        _mt_fe_dmd_set_reg(0xa1, 0x02);



    _mt_sleep(13);



    time_out = 5;
    do
    {
        _mt_fe_dmd_get_reg(0xa1, &val);
        if ((val & 0x40) == 0)
            break;

        _mt_sleep(1);
        time_out --;

    } while (time_out > 0);


    _mt_fe_dmd_get_reg(0xa2, &val);
    #if LNB_DISEQC_OUT_FORCE_HIGH
        val |= 0xc0;
    #else
        val &= ~0xc0;
        val |= 0x80;
    #endif
    _mt_fe_dmd_set_reg(0xa2, val);




    return MtFeErr_Ok;
}




/*****************************************************************************
** Function: mt_fe_dmd_ds3k_DiSEqC_send_receive_msg
**
**
** Discription: send data with reply message by device in DiSEqC mode.
**
**
** Inputs:
**
**    Parameter         Type                Description
**  ----------------------------------------------------------------------
**    msg               MT_FE_DiSEqC_MSG    DiSEqC message pointer
**
**
** Outputs:     none
**
**
**
*******************************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_DiSEqC_send_receive_msg(MT_FE_DiSEqC_MSG msg)
{
    U8  i;
    U8  tmp, tmp1;
    S32 time_out;



    if (msg.size_send > 8)
        return MtFeErr_Param;



    _mt_fe_dmd_get_reg(0xa2, &tmp);
    tmp &= ~0xc0;
    _mt_fe_dmd_set_reg(0xa2, tmp);
#if 0
    printf("DiSEqC cmd 0x%02X %02X %02X %02X",
           msg.data_send[0],msg.data_send[1],
           msg.data_send[2],msg.data_send[3]);
    if(msg.size_send > 4) printf(" %02X",msg.data_send[4]);
    if(msg.size_send > 5) printf(" %02X",msg.data_send[5]);
    if(msg.size_send > 6) printf(" %02X",msg.data_send[6]);
    if(msg.size_send > 7) printf(" %02X",msg.data_send[7]);
    printf("\n");
#endif
    for (i = 0; i < msg.size_send; i ++)
    {
        _mt_fe_dmd_set_reg((U8)(0xa3 + i), msg.data_send[i]);
    }

    _mt_fe_dmd_get_reg(0xa1, &tmp);
    tmp &= ~0x38;
    tmp &= ~0x40;
    tmp |= ((msg.size_send-1) << 3) | 0x07;
    if (msg.is_enable_receive)
        tmp |= 0x80;
    else
        tmp &= ~0x80;

    _mt_fe_dmd_set_reg(0xa1, tmp);


    /*  1.5 * 9 * 8 = 108ms */
    time_out = 150;
    while (time_out > 0)
    {
        _mt_sleep(10);
        time_out -= 10;

        _mt_fe_dmd_get_reg(0xa1, &tmp);
        if ((tmp & 0x40) == 0)
            break;
    }

    if (time_out <= 0)
    {
        _mt_fe_dmd_get_reg(0xa1, &tmp);
        tmp &= ~0x80;
        tmp |= 0x40;
        _mt_fe_dmd_set_reg(0xa1, tmp);


        _mt_fe_dmd_get_reg(0xa2, &tmp);
        #if LNB_DISEQC_OUT_FORCE_HIGH
            tmp |= 0xc0;
        #else
            tmp &= ~0xc0;
            tmp |= 0x80;
        #endif
        _mt_fe_dmd_set_reg(0xa2, tmp);

        return MtFeErr_TimeOut;
    }


    if (!(msg.is_enable_receive))
    {
        _mt_fe_dmd_get_reg(0xa2, &tmp);
        #if LNB_DISEQC_OUT_FORCE_HIGH
            tmp |= 0xc0;
        #else
            tmp &= ~0xc0;
            tmp |= 0x80;
        #endif
        _mt_fe_dmd_set_reg(0xa2, tmp);

        return MtFeErr_Ok;
    }



    time_out = 170;
    while (time_out > 0)
    {
        _mt_fe_dmd_get_reg(0xa1, &tmp);
        tmp =(U8)((tmp >> 3) & 0x07);
        if (tmp != 0)
            break;

        _mt_sleep(10);
        time_out -= 10;
    }

    if (time_out <= 0)
    {
        _mt_fe_dmd_get_reg(0xa1, &tmp);
        tmp &= ~0x80;
        tmp |= 0x40;
        _mt_fe_dmd_set_reg(0xa1, tmp);


        _mt_fe_dmd_get_reg(0xa2, &tmp);
        #if LNB_DISEQC_OUT_FORCE_HIGH
            tmp |= 0xc0;
        #else
            tmp &= ~0xc0;
            tmp |= 0x80;
        #endif
        _mt_fe_dmd_set_reg(0xa2, tmp);

        return MtFeErr_TimeOut;
    }

    /*  1.5 * 9 * 8 = 108ms */
    time_out = 150;
    tmp1     = tmp;
    while (time_out > 0)
    {
        _mt_sleep(15);
        time_out -= 15;

        _mt_fe_dmd_get_reg(0xa1, &tmp);
        tmp =(U8)((tmp >> 3) & 0x07);
        if (tmp == tmp1 || tmp == 7)
            break;

        tmp1 = tmp;
    }

    if (time_out <= 0)
    {
        _mt_fe_dmd_get_reg(0xa1, &tmp);
        tmp &= ~0x80;
        tmp |= 0x40;
        _mt_fe_dmd_set_reg(0xa1, tmp);


        _mt_fe_dmd_get_reg(0xa2, &tmp);
        #if LNB_DISEQC_OUT_FORCE_HIGH
            tmp |= 0xc0;
        #else
            tmp &= ~0xc0;
            tmp |= 0x80;
        #endif
        _mt_fe_dmd_set_reg(0xa2, tmp);

        return MtFeErr_TimeOut;
    }


    msg.size_receive = tmp;
    _mt_fe_dmd_set_reg(0xa1, 0x40);

    for (i = 0; i < msg.size_receive; i ++)
    {
        _mt_fe_dmd_get_reg((U8)(0xa3 + i), &(msg.data_receive[i]));
    }



    /*  Indicates the parity error occurs   */
    _mt_fe_dmd_get_reg(0xab, &tmp);
    if (tmp != 0)
    {
        _mt_fe_dmd_get_reg(0xa1, &tmp);
        tmp &= ~0x80;
        tmp |= 0x40;
        _mt_fe_dmd_set_reg(0xa1, tmp);


        _mt_fe_dmd_get_reg(0xa2, &tmp);
        #if LNB_DISEQC_OUT_FORCE_HIGH
            tmp |= 0xc0;
        #else
            tmp &= ~0xc0;
            tmp |= 0x80;
        #endif
        _mt_fe_dmd_set_reg(0xa2, tmp);


        return MtFeErr_Fail;
    }


    _mt_fe_dmd_get_reg(0xa1, &tmp);
    tmp &= ~0x80;
    tmp |= 0x40;
    _mt_fe_dmd_set_reg(0xa1, tmp);

    _mt_fe_dmd_get_reg(0xa2, &tmp);
    #if LNB_DISEQC_OUT_FORCE_HIGH
        tmp |= 0xc0;
    #else
        tmp &= ~0xc0;
        tmp |= 0x80;
    #endif
    _mt_fe_dmd_set_reg(0xa2, tmp);


    return MtFeErr_Ok;
}


/**********************************************************************
** Function: mt_fe_ds3k_blindscan_abort
**
**
** Discription: cancel blind scan process
**
**
** Inputs:
**
**      Parameter       type                    discription
**  -------------------------------------------------------------------
**      bool            MT_FE_BOOL              MtFe_True: cancel
**
**
** Outputs:     none
**
**
************************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_blindscan_abort(MT_FE_BOOL bs_abort)
{

    feState[feIndex].g_blindscan_cancel = bs_abort;

    return MtFeErr_Ok;
}



/*********************************************************************************
** Function: mt_fe_dmd_ds3k_register_blindscan_callback
**
**
** Discription: register the blindscan callback function
**
**
** Inputs:
**
**      Parameter       type                    discription
**  -----------------------------------------------------------------------------
**      callback        MT_FE_RET (*)()         callback function pointer
**
**
** Outputs:     none
**
**
**********************************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_register_notify(void (*callback)(MT_FE_MSG msg, void *p_tp_info))
{
    if(callback == NULL)
        return MtFeErr_NullPointer;

    mt_fe_bs_notify = callback;

    return MtFeErr_Ok;
}



/******************************************************************************************
** Function: mt_fe_dmd_ds3k_blindscan
**
**
** Discription: blind scan satellite from the start freq. to the end freq.
**
**
** Inputs:
**
**      Parameter           type                                discription
**  ---------------------------------------------------------------------------------------
**      begin_freq_MHz      U32                                 blind scan start freq.
**      end_freq_MHz        U32                                 blind scan stop freq.
**
**
** Outputs:
**
**      Paramet         type                                discription
**  ---------------------------------------------------------------------------------------
**      p_bs_info           MT_FE_BS_TP_INFO*   blind scan result pointer
**
**
********************************************************************************************/
MT_FE_RET mt_fe_dmd_ds3k_blindscan(U32 begin_freq_MHz, U32 end_freq_MHz, MT_FE_BS_TP_INFO *p_bs_info)
{
    U8 times;
    U32 begin_freq_KHz, end_freq_KHz;

    if((p_bs_info->p_tp_info == NULL)||(p_bs_info->tp_max_num == 0))
        return MtFeErr_Param;

    mt_fe_bs_notify(MtFeMsg_BSStart, 0);

    feState[feIndex].g_scanned_tp_cnt    = 0;
    feState[feIndex].g_locked_tp_cnt        = 0;
    feState[feIndex].g_blindscan_cancel     = MtFe_False;
    feState[feIndex].g_bs_time_total= p_bs_info->bs_times;

    begin_freq_KHz = begin_freq_MHz*1000;
    end_freq_KHz   = end_freq_MHz*1000;

    p_bs_info->tp_num = 0;

    for(times = MT_FE_BS_TIMES_1ST; times < (p_bs_info->bs_times); times++)
    {
        if(p_bs_info->bs_algorithm == MT_FE_BS_ALGORITHM_A)
        {
            _mt_fe_dmd_ds3k_bs_A(begin_freq_KHz, end_freq_KHz, p_bs_info, times);
        }
        else if(p_bs_info->bs_algorithm == MT_FE_BS_ALGORITHM_B)
        {
            _mt_fe_dmd_ds3k_bs_B(begin_freq_KHz, end_freq_KHz, p_bs_info, times);
        }
        else
        {
            mt_fe_bs_notify(MtFeMsg_BSFinish, 0);
            return MtFeErr_Fail;
        }

        if(feState[feIndex].g_blindscan_cancel)
        {
            feState[feIndex].g_blindscan_cancel = FALSE;
            break;
        }
    }


    _mt_fe_dmd_ds3k_bs_remove_unlocked_TP(p_bs_info, (U32)FREQ_MAX_KHz);


    mt_fe_bs_notify(MtFeMsg_BSFinish, 0);

    return MtFeErr_Ok;
}



/******************************************************************
** Function: _mt_fe_dmd_ds3k_set_clock_ratio
**
**
** Discription: set TS Clock Ratio
**
**
** Inputs:      None
**
**
** Outputs:     None
**
**
*******************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_set_clock_ratio(void)
{
    U8    val, mod_fac, tmp1, tmp2;
    U8 sym_low, sym_high;
    U32 input_datarate, locked_sym_rate_KSs;
    MT_FE_CHAN_INFO_DVBS2 p_info;


    sym_low  = 0;
    sym_high = 0;
    locked_sym_rate_KSs = 0;
    _mt_fe_dmd_get_reg(0x6d, &sym_low);
    _mt_fe_dmd_get_reg(0x6e, &sym_high);
    locked_sym_rate_KSs = (U32)((((U32)(sym_high<<8) + sym_low)*(MT_FE_MCLK_KHZ/1000))/(65536/1000));


    mt_fe_dmd_ds3k_get_channel_info(&p_info);

    if (feState[feIndex].g_current_type == MtFeType_DvbS2)
    {
        switch(p_info.mod_mode)
        {
            case MtFeModMode_8psk:      mod_fac = 3; break;
            case MtFeModMode_16Apsk:    mod_fac = 4; break;
            case MtFeModMode_32Apsk:    mod_fac = 5; break;
            case MtFeModMode_Qpsk:
            default:                    mod_fac = 2; break;
        }

        switch(p_info.code_rate)
        {
            case MtFeCodeRate_1_4:  input_datarate = locked_sym_rate_KSs*mod_fac/8/4;   break;
            case MtFeCodeRate_1_3:  input_datarate = locked_sym_rate_KSs*mod_fac/8/3;   break;
            case MtFeCodeRate_2_5:  input_datarate = locked_sym_rate_KSs*mod_fac*2/8/5; break;
            case MtFeCodeRate_1_2:  input_datarate = locked_sym_rate_KSs*mod_fac/8/2;   break;
            case MtFeCodeRate_3_5:  input_datarate = locked_sym_rate_KSs*mod_fac*3/8/5; break;
            case MtFeCodeRate_2_3:  input_datarate = locked_sym_rate_KSs*mod_fac*2/8/3; break;
            case MtFeCodeRate_3_4:  input_datarate = locked_sym_rate_KSs*mod_fac*3/8/4; break;
            case MtFeCodeRate_4_5:  input_datarate = locked_sym_rate_KSs*mod_fac*4/8/5; break;
            case MtFeCodeRate_5_6:  input_datarate = locked_sym_rate_KSs*mod_fac*5/8/6; break;
            case MtFeCodeRate_8_9:  input_datarate = locked_sym_rate_KSs*mod_fac*8/8/9; break;
            case MtFeCodeRate_9_10: input_datarate = locked_sym_rate_KSs*mod_fac*9/8/10;break;
            default:                input_datarate = locked_sym_rate_KSs*mod_fac*2/8/3; break;
        }


        input_datarate = input_datarate*115/100;

        if(input_datarate < 6000)       {tmp1 = 12;tmp2 = 12;} //6MHz      TS clock
        else if(input_datarate < 6260)  {tmp1 = 11;tmp2 = 12;} //6.26MHz   TS clock
        else if(input_datarate < 6545)  {tmp1 = 11;tmp2 = 11;} //6.545MHz  TS clock
        else if(input_datarate < 6857)  {tmp1 = 10;tmp2 = 11;} //6.857MHz  TS clock
        else if(input_datarate < 7200)  {tmp1 = 10;tmp2 = 10;} //7.2MHz    TS clock
        else if(input_datarate < 7578)  {tmp1 = 9;tmp2 = 10;}  //7.578MHz  TS clock
        else if(input_datarate < 8000)    {tmp1 = 9;tmp2 = 9;}   //8MHz      TS clock
        else if(input_datarate < 8470)    {tmp1 = 8;tmp2 = 9;}   //8.47MHz   TS clock
        else if(input_datarate < 9000)    {tmp1 = 8;tmp2 = 8;}   //9MHz      TS clock
        else if(input_datarate < 9600)    {tmp1 = 7;tmp2 = 8;}   //9.6MHz    TS clock
        else if(input_datarate < 10285)    {tmp1 = 7;tmp2 = 7;}   //10.285MHz TS clock
    #if 1
        else if(input_datarate < 12000)    {tmp1 = 6;tmp2 = 6;}   //12MHz     TS clock
        else if(input_datarate < 14400)    {tmp1 = 5;tmp2 = 5;}   //14.4MHz   TS clock
        else if(input_datarate < 18000)    {tmp1 = 4;tmp2 = 4;}   //18MHz     TS clock
        else                               {tmp1 = 3;tmp2 = 3;}   //24MHz     TS clock
    #else
        else if(input_datarate < 11076)    {tmp1 = 6;tmp2 = 7;}   //11.076MHz TS clock
        else if(input_datarate < 12000)    {tmp1 = 6;tmp2 = 6;}   //12MHz     TS clock
        else if(input_datarate < 13090)    {tmp1 = 5;tmp2 = 6;}   //13.09MHz  TS clock
        else if(input_datarate < 14400)    {tmp1 = 5;tmp2 = 5;}   //14.4MHz   TS clock
        else if(input_datarate < 16000)    {tmp1 = 4;tmp2 = 5;}   //16MHz     TS clock
        else if(input_datarate < 18000)    {tmp1 = 4;tmp2 = 4;}   //18MHz     TS clock
        else if(input_datarate < 20571)    {tmp1 = 3;tmp2 = 4;}   //20.571MHz TS clock
        else                               {tmp1 = 3;tmp2 = 3;}   //24MHz     TS clock
    #endif

        val = (U8)((tmp1<<4) + tmp2);
        _mt_fe_dmd_set_reg(0xfe, val);
    }
    else
    {
        mod_fac = 2;

        switch(p_info.code_rate)
        {
            case MtFeCodeRate_1_2:  input_datarate = locked_sym_rate_KSs*mod_fac/2/8;   break;
            case MtFeCodeRate_2_3:  input_datarate = locked_sym_rate_KSs*mod_fac*2/3/8; break;
            case MtFeCodeRate_3_4:  input_datarate = locked_sym_rate_KSs*mod_fac*3/4/8; break;
            case MtFeCodeRate_5_6:  input_datarate = locked_sym_rate_KSs*mod_fac*5/6/8; break;
            case MtFeCodeRate_7_8:  input_datarate = locked_sym_rate_KSs*mod_fac*7/8/8; break;
            default:                input_datarate = locked_sym_rate_KSs*mod_fac*3/4/8; break;
        }


        input_datarate = input_datarate*115/100;

    #if 1
        if(input_datarate < 6857)        {tmp1 = 7;tmp2 = 7;} //6.857MHz     TS clock
        else if(input_datarate < 8000)    {tmp1 = 6;tmp2 = 6;} //8MHz         TS clock
        else if(input_datarate < 9600)    {tmp1 = 5;tmp2 = 5;} //9.6MHz        TS clock
        else if(input_datarate < 12000)    {tmp1 = 4;tmp2 = 4;} //12MHz          TS clock
        else if(input_datarate < 16000)    {tmp1 = 3;tmp2 = 3;} //16MHz         TS clock
        else                             {tmp1 = 2;tmp2 = 2;} //24MHz         TS clock
    #else
        if(input_datarate < 6857)        {tmp1 = 7;tmp2 = 7;} //6.857MHz     TS clock
        else if(input_datarate < 7384)    {tmp1 = 6;tmp2 = 7;} //7.384MHz     TS clock        
        else if(input_datarate < 8000)    {tmp1 = 6;tmp2 = 6;} //8MHz         TS clock
        else if(input_datarate < 8727)    {tmp1 = 5;tmp2 = 6;} //8.727MHz     TS clock
        else if(input_datarate < 9600)    {tmp1 = 5;tmp2 = 5;} //9.6MHz        TS clock
        else if(input_datarate < 10666)    {tmp1 = 4;tmp2 = 5;} //10.666MHz     TS clock
        else if(input_datarate < 12000)    {tmp1 = 4;tmp2 = 4;} //12MHz          TS clock
        else if(input_datarate < 13714)    {tmp1 = 3;tmp2 = 4;} //13.714MHz     TS clock
        else if(input_datarate < 16000)    {tmp1 = 3;tmp2 = 3;} //16MHz         TS clock
        else if(input_datarate < 19200)    {tmp1 = 2;tmp2 = 3;} //19.2MHz       TS clock
        else                             {tmp1 = 2;tmp2 = 2;} //24MHz         TS clock
    #endif

        _mt_fe_dmd_get_reg(0xfe, &val);
        val &= 0xc0;
        val |= ((U8)((tmp1<<3) + tmp2));
        _mt_fe_dmd_set_reg(0xfe, val);
    }


    return MtFeErr_Ok;
}


/******************************************************************
** Function: _mt_fe_dmd_ds3k_set_ts_out_mode
**
**
** Discription: set the TS ouput mode
**
**
** Inputs:
**
**    Parameter       Type          Description
**  ----------------------------------------------------------
**    mode      MT_FE_TS_OUT_MODE   passed by up level
**
**
** Outputs:     none
**
**
*******************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_set_ts_out_mode(U8 mode)
{
    U8    tmp, tmp1, tmp2, val, val_0x08, val_0x27;


    _mt_fe_dmd_get_reg(0x08, &val_0x08);

    _mt_fe_dmd_get_reg(0x27, &val_0x27);
    val_0x27 &= ~0x01;
    _mt_fe_dmd_set_reg(0x27, val_0x27);

    /*set DVBS mode*/
    tmp = (U8)(val_0x08 & (~0x04));
    _mt_fe_dmd_set_reg(0x08, tmp);
    #if(MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Common)
        //8MHz TS clock output
        tmp1 = 6;
        tmp2 = 6;
    #elif (MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Parallel)
        #if (MtFeTSOut_Max_Clock_12_MHz)
            tmp1 = 4;
            tmp2 = 4;
        #elif (MtFeTSOut_Max_Clock_16_MHz)
            tmp1 = 3;
            tmp2 = 3;
        #elif (MtFeTSOut_Max_Clock_19_p_2_MHz)
            tmp1 = 3;
            tmp2 = 2;
        #elif (MtFeTSOut_Max_Clock_24_MHz)
            tmp1 = 2;
            tmp2 = 2;
        #endif
    #endif

    _mt_fe_dmd_get_reg(0xfe, &val);
    val &= 0xc0;
    val |= ((U8)((tmp1<<3) + tmp2));
    _mt_fe_dmd_set_reg(0xfe, val);


    _mt_fe_dmd_get_reg(0xfd, &tmp);
    if (mode == MtFeTsOutMode_Parallel)
    {
        tmp &= ~0x80;
        tmp &= ~0x40;
    }
    else if (mode == MtFeTsOutMode_Serial)
    {
        tmp &= ~0x80;
        tmp |= 0x40;
    }
    else
    {
        tmp |= 0x80;
        tmp &= ~0x40;
    }
    tmp |= 0x20;
    tmp &= ~0x1f;
    _mt_fe_dmd_set_reg(0xfd, tmp);



    /*set DVBS2 mode*/
    tmp = (U8)(val_0x08|0x04);
    _mt_fe_dmd_set_reg(0x08, tmp);
    #if(MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Common)
        //8.47MHz TS clock output
        tmp1 = 8;
        tmp2 = 9;
    #elif (MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Parallel)
        #if (MtFeTSOut_Max_Clock_12_MHz)
            tmp1 = 6;
            tmp2 = 6;
        #elif (MtFeTSOut_Max_Clock_16_MHz)
            tmp1 = 5;
            tmp2 = 4;
        #elif (MtFeTSOut_Max_Clock_19_p_2_MHz)
            /*actually 18MHz in DVBS2 mode*/
            tmp1 = 4;
            tmp2 = 4;
        #elif (MtFeTSOut_Max_Clock_24_MHz)
            tmp1 = 3;
            tmp2 = 3;
        #endif
    #endif

    val = (U8)((tmp1<<4) + tmp2);
    _mt_fe_dmd_set_reg(0xfe, val);


    _mt_fe_dmd_get_reg(0xfd, &tmp);
    if (mode == MtFeTsOutMode_Parallel)
    {
        tmp &= ~0x01;
        tmp &= ~0x04;
    }
    else if (mode == MtFeTsOutMode_Serial)
    {
        tmp &= ~0x01;
        tmp |= 0x04;
    }
    else
    {
        tmp |= 0x01;
        tmp &= ~0x04;
    }
    tmp &= ~0xb8;
    tmp |= 0x42;
    _mt_fe_dmd_set_reg(0xfd, tmp);



    _mt_fe_dmd_set_reg(0x08, val_0x08);

    _mt_fe_dmd_get_reg(0x27, &val_0x27);
    val_0x27 |= 0x01;
    _mt_fe_dmd_set_reg(0x27, val_0x27);

    return MtFeErr_Ok;
}



/*****************************************************************
** Function: _mt_fe_dmd_ds3k_set_CCI
**
**
** Discription: set CCI on/off
**
**
** Inputs:
**
**    Parameter         Type        Description
**  ----------------------------------------------------------
**    is_cci_on     MT_FE_ON_OFF    cci on or off
**
** Outputs:
**
**
*****************************************************************/
MT_FE_RET  _mt_fe_dmd_ds3k_set_CCI(MT_FE_ON_OFF is_cci_on)
{
    U8 tmp;


    _mt_fe_dmd_get_reg(0x56, &tmp);
    if (is_cci_on == MtFe_On)
        tmp &= ~0x01;
    else
        tmp |= 0x01;
    _mt_fe_dmd_set_reg(0x56, tmp);


    return MtFeErr_Ok;
}



/*****************************************************************
** Function: _mt_fe_dmd_ds3k_check_CCI
**
**
** Discription: check CCI whether it needs to turn off
**
**
** Inputs:  None
**
**
** Outputs: None
**
**
*****************************************************************/
MT_FE_RET  _mt_fe_dmd_ds3k_check_CCI(MT_FE_LOCK_STATE *p_state)
{
    U8 i, cnt, reg_0xb3, reg_0x59, CCI_on_flag_cnt;
    S8 val_cci;

    cnt = 0;

    _mt_fe_dmd_get_reg(0xb3, &reg_0xb3);
    while((reg_0xb3 == 0x38)||(reg_0xb3 <= 3))
    {
        cnt ++;

        if (cnt > 10)
        {
            *p_state = MtFeLockState_Unlocked;
            return MtFeErr_Ok;
        }

        _mt_sleep(10);

        _mt_fe_dmd_get_reg(0xb3, &reg_0xb3);
    }


    CCI_on_flag_cnt = 0;
    for(i = 0; i < 15; i++)
    {
        _mt_fe_dmd_get_reg(0x59, &reg_0x59);
        val_cci = (S8)reg_0x59;

        if(val_cci < 0)
        {
            CCI_on_flag_cnt = 0;
            break;
        }

        if(val_cci > MT_FE_CCI_THRESHOLD)
            CCI_on_flag_cnt++;
    }


    if (CCI_on_flag_cnt < 10)
        _mt_fe_dmd_ds3k_set_CCI(MtFe_Off);


    _mt_sleep(1);


    return MtFeErr_Ok;
}


/*****************************************************************************
** Function: _mt_fe_dmd_ds3k_set_demod
**
**
** Discription: set some registers according the param.
**
**
** Inputs:
**
**  Parameter       Type        Description
**  ----------------------------------------------------------------------
**  sym_rate_KSs    U8          symbol rate passed by app level
**  type            MT_FE_TYPE  passed by up level
**
**
** Outputs:
**
**
******************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_set_demod(U32 sym_rate_KSs, MT_FE_TYPE type)
{
    MT_FE_RET   ret;
    U8          val, val_0xc3, val_0xc8, tmp, tmp1, tmp2;


    feState[feIndex].g_auto_tune_cnt++;


    _mt_fe_dmd_get_reg(0x08, &val);
    if (type == MtFeType_DvbS2)
    {
        val |= 0x04;
        _mt_fe_dmd_set_reg(0x08, val);
        ret = _mt_fe_dmd_ds3k_init_reg(ds3k_reg_tbl_dvbs2_def, sizeof(ds3k_reg_tbl_dvbs2_def)/2);

        #if(MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Common)
            //8.47MHz TS clock output
            tmp1 = 8;
            tmp2 = 9;
        #elif (MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Parallel)
        #if (MtFeTSOut_Max_Clock_12_MHz)
            tmp1 = 6;
            tmp2 = 6;
        #elif (MtFeTSOut_Max_Clock_16_MHz)
            tmp1 = 5;
            tmp2 = 4;
        #elif (MtFeTSOut_Max_Clock_19_p_2_MHz)
            /*actually 18MHz in DVBS2 mode*/
            tmp1 = 4;
            tmp2 = 4;
        #elif (MtFeTSOut_Max_Clock_24_MHz)
            tmp1 = 3;
            tmp2 = 3;
        #endif
        #endif

        val = (U8)((tmp1<<4) + tmp2);
        _mt_fe_dmd_set_reg(0xfe, val);
        if(sym_rate_KSs <= 5000)
        {
            _mt_fe_dmd_set_reg(0xc0, 0x04);
            _mt_fe_dmd_set_reg(0x8a, 0x09);
            _mt_fe_dmd_set_reg(0x8b, 0x22);
            _mt_fe_dmd_set_reg(0x8c, 0x88);
        }
    }
    else
    {
        val &= ~0x04;
        _mt_fe_dmd_set_reg(0x08, val);
        ret = _mt_fe_dmd_ds3k_init_reg(ds3k_reg_tbl_dvbs_def, sizeof(ds3k_reg_tbl_dvbs_def)/2);

        #if(MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Common)
            //8MHz TS clock output
            tmp1 = 6;
            tmp2 = 6;
        #elif (MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Parallel)
        #if (MtFeTSOut_Max_Clock_12_MHz)
            tmp1 = 4;
            tmp2 = 4;
        #elif (MtFeTSOut_Max_Clock_16_MHz)
            tmp1 = 3;
            tmp2 = 3;
        #elif (MtFeTSOut_Max_Clock_19_p_2_MHz)
            tmp1 = 3;
            tmp2 = 2;
        #elif (MtFeTSOut_Max_Clock_24_MHz)
            tmp1 = 2;
            tmp2 = 2;
            #endif
        #endif

        _mt_fe_dmd_get_reg(0xfe, &val);
        val &= 0xc0;
        val |= ((U8)((tmp1<<3) + tmp2));
        _mt_fe_dmd_set_reg(0xfe, val);
    }

#if MT_FE_TN_SUPPORT_SHARP6306
    _mt_fe_dmd_set_reg(0x33, 0x38);
#else
    _mt_fe_dmd_set_reg(0x33, 0x35);
#endif

#if MT_FE_TN_SUPPORT_SHARP6306
    _mt_fe_dmd_set_reg(0x36, 0x10);
    _mt_fe_dmd_set_reg(0x39, 0x00);
#elif MT_FE_TN_SUPPORT_SERIT_SP761VB
    _mt_fe_dmd_set_reg(0x36, 0x07);
    _mt_fe_dmd_set_reg(0x39, 0x00);
#endif

#if MT_FE_ENABLE_27MHZ_CLOCK_OUT
    _mt_fe_dmd_get_reg(0x29, &tmp);
    tmp &= ~0x80;
    _mt_fe_dmd_set_reg(0x29, tmp);
#else
    _mt_fe_dmd_get_reg(0x29, &tmp);
    tmp |= 0x80;
    _mt_fe_dmd_set_reg(0x29, tmp);
#endif

#if MT_FE_ENABLE_13_P_5_MHZ_CLOCK_OUT
    _mt_fe_dmd_get_reg(0x29, &tmp);
    tmp |= 0x10;
    _mt_fe_dmd_set_reg(0x29, tmp);
#else
    _mt_fe_dmd_get_reg(0x29, &tmp);
    tmp &= ~0x10;
    _mt_fe_dmd_set_reg(0x29, tmp);
#endif

#if MT_FE_ENABLE_AC_COUPLING
    _mt_fe_dmd_get_reg(0x25, &tmp);
    tmp |= 0x08;
    _mt_fe_dmd_set_reg(0x25, tmp);
#endif


#if (MT_FE_ENHANCE_PERFORMANCE != 0)
    if(sym_rate_KSs <= 5000)
    {
        val_0xc8 = (U8)(29777/sym_rate_KSs + 1);

        if(val_0xc8%2 != 0)
            val_0xc8 += 1;

        _mt_fe_dmd_set_reg(0xc3, 0x0d);
        _mt_fe_dmd_set_reg(0xc8, val_0xc8);
        _mt_fe_dmd_set_reg(0xc4, 0x08);
        _mt_fe_dmd_set_reg(0xc7, 0x0e);
    }
    else if(sym_rate_KSs <= 10000)
    {
        val_0xc8 = (U8)(92166/sym_rate_KSs + 1);

        if(val_0xc8%2 != 0)
            val_0xc8 += 1;

        _mt_fe_dmd_set_reg(0xc3, 0x07);
        _mt_fe_dmd_set_reg(0xc8, val_0xc8);
        _mt_fe_dmd_set_reg(0xc4, 0x08);
        _mt_fe_dmd_set_reg(0xc7, 0x0a);
    }
    else if(sym_rate_KSs <= 20000)
    {
        val_0xc3 = (U8)(64516/sym_rate_KSs + 1);

        _mt_fe_dmd_set_reg(0xc3, 0x10);
        _mt_fe_dmd_set_reg(0xc8, 0x3e);
        _mt_fe_dmd_set_reg(0xc4, 0x04);
        _mt_fe_dmd_set_reg(0xc7, 0x14);
    }
    else
    {
        val_0xc3 = (U8)(129032/sym_rate_KSs + 1);

        _mt_fe_dmd_set_reg(0xc3, val_0xc3);
        _mt_fe_dmd_set_reg(0xc8, 0x0a);
        _mt_fe_dmd_set_reg(0xc4, 0x05);
        _mt_fe_dmd_set_reg(0xc7, 0x12);
    }
#endif

    _mt_fe_dmd_ds3k_set_sym_rate(sym_rate_KSs);

    _mt_fe_dmd_ds3k_set_CCI(MtFe_On);
    _mt_fe_dmd_get_reg(0x76, &val);
    val &= ~0x80;
    _mt_fe_dmd_set_reg(0x76, val);


    return ret;
}



/*****************************************************************************
** Function: _mt_fe_dmd_ds3k_set_sym_rate
**
**
** Discription: set symbol rate
**
**
** Inputs:
**
**  Parameter       Type        Description
**  ----------------------------------------------------------------------
**  sym_rate_KSs    U8      symbol rate passed by app level
**
**
** Outputs:
**
**
*******************************************************************************/
MT_FE_RET  _mt_fe_dmd_ds3k_set_sym_rate(U32 sym_rate_KSs)
{
    U32 tmp;
    U8  reg_0x61, reg_0x62;


    tmp = (U32)(((sym_rate_KSs << 15) + (MT_FE_MCLK_KHZ / 4)) / (MT_FE_MCLK_KHZ / 2));


    reg_0x61 = (U8)(tmp & 0x00FF);
    reg_0x62 = (U8)((tmp & 0xFF00) >> 8);

    _mt_fe_dmd_set_reg(0x61, reg_0x61);
    _mt_fe_dmd_set_reg(0x62, reg_0x62);


    return MtFeErr_Ok;
}



/*****************************************************************************
** Function: _mt_fe_dmd_ds3k_get_sym_rate
**
**
** Discription: get symbol rate from register
**
**
** Inputs:  none
**
**
**
** Outputs:
**
**  Parameter       Type        Description
**  ----------------------------------------------------------------------
**  sym_rate_KSs    U32*        symbol rate read from register
**
**
*******************************************************************************/
 MT_FE_RET _mt_fe_dmd_ds3k_get_sym_rate(U32 *sym_rate_KSs)
{
    U16 tmp;
    U32 sym_rate_tmp;
    U8  val_0x6d, val_0x6e;


    _mt_fe_dmd_get_reg(0x6d, &val_0x6d);
    _mt_fe_dmd_get_reg(0x6e, &val_0x6e);

    tmp = (U16)((val_0x6e<<8) | val_0x6d);


    sym_rate_tmp = (U32)(tmp * MT_FE_MCLK_KHZ);
    sym_rate_tmp = (U32)(sym_rate_tmp / (1<<16));

    *sym_rate_KSs = sym_rate_tmp;


    return MtFeErr_Ok;
}



/*************************************************************************
** Function:  _mt_fe_dmd_ds3k_set_carrier_offset
**
**
** Discription: set blind scan registers
**
**
** Inputs:
**
**  Parameter               Type        Description
**  ----------------------------------------------------------------------
**  carrier_offset_KHz      U32         carrier offset
**
**
** Outputs:     none
**
**
*************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_set_carrier_offset(S32 carrier_offset_KHz)
{
    S32 tmp;


    tmp = carrier_offset_KHz;
    tmp *= 65536;

    tmp = (S32)((2*tmp + MT_FE_MCLK_KHZ)/(2*MT_FE_MCLK_KHZ));

    if(tmp < 0)
        tmp += 65536;


    _mt_fe_dmd_set_reg(0x5f, (U8)(tmp>>8));
    _mt_fe_dmd_set_reg(0x5e, (U8)(tmp&0xFF));

    return MtFeErr_Ok;
}



/***********************************************************************
** Function: _mt_fe_dmd_ds3k_bs_get_carrier_offset
**
**
** Discription: get the carrier offset
**
**
** Inputs:  none
**
**
** Outputs:
**
**  Parameter               Type        Description
**  --------------------------------------------------------------------
**  carrier_offset_KHz      S32*        carrier offset value
**
**
*************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_get_carrier_offset(S32 *carrier_offset_KHz)
{
    S32 val;
    U8  tmp, val_0x5e, val_0x5f;


    _mt_fe_dmd_get_reg(0x5d, &tmp);
    tmp &= 0xf8;
    _mt_fe_dmd_set_reg(0x5d, tmp);

    _mt_fe_dmd_get_reg(0x5e, &val_0x5e);
    _mt_fe_dmd_get_reg(0x5f, &val_0x5f);
    val = (val_0x5f<<8) | val_0x5e;


    if((val&0x8000) == 0x8000)
        val -= (1<<16);


    *carrier_offset_KHz = (S32)((val * MT_FE_MCLK_KHZ) / (1<<16));

    return MtFeErr_Ok;
}


/***********************************************************************
** Function: _mt_fe_dmd_ds3k_get_total_carrier_offset
**
**
** Discription:    get the total carrier offset
**
**
** Inputs:    none
**
**
** Outputs:
**
**    Parameter                Type        Description
**    --------------------------------------------------------------------
**    carrier_offset_KHz         S32*        carrier offset value
**
**
*************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_get_total_carrier_offset(S32 *carrier_offset_KHz)
{
    U8    tmp, val_0x5e, val_0x5f;
    S32 val_0x5e_0x5f_1, val_0x5e_0x5f_2, nval_1, nval_2;    
    
    _mt_fe_dmd_get_reg(0x5d, &tmp);
    tmp &= 0xf8;
    _mt_fe_dmd_set_reg(0x5d, tmp);
    
    _mt_fe_dmd_get_reg(0x5e, &val_0x5e);
    _mt_fe_dmd_get_reg(0x5f, &val_0x5f);
    val_0x5e_0x5f_1 = (val_0x5f<<8) | val_0x5e;
    
    tmp |= 0x06;
    _mt_fe_dmd_set_reg(0x5d, tmp);

    _mt_fe_dmd_get_reg(0x5e, &val_0x5e);
    _mt_fe_dmd_get_reg(0x5f, &val_0x5f);
    val_0x5e_0x5f_2 = (val_0x5f<<8) | val_0x5e;
    
    
    if(((val_0x5e_0x5f_1>>15)&0x01) == 0x01)
        nval_1 = val_0x5e_0x5f_1 - (1<<16);
    else
        nval_1 = val_0x5e_0x5f_1;
    
    if(((val_0x5e_0x5f_2>>15)&0x01) == 0x01)
        nval_2 = val_0x5e_0x5f_2 - (1<<16);
    else
        nval_2 = val_0x5e_0x5f_2;

#if MT_FE_FOR_GMI_HW_V10_APP
    *carrier_offset_KHz = (nval_2 - nval_1) * MT_FE_MCLK_KHZ / (1<<16);
#else
    *carrier_offset_KHz = (nval_1 - nval_2) * MT_FE_MCLK_KHZ / (1<<16);
#endif
    
    return MtFeErr_Ok;
}
/*************************************************************************
** Function:  _mt_fe_dmd_ds3k_get_fec
**
**
** Discription: get code rate
**
**
** Inputs:
**
**  Parameter           Type                Description
**  ----------------------------------------------------------------------
**  tp_type             MT_FE_TYPE          DVBS/DVBS2
**
**
** Outputs:
**
**  Parameter           Type                Description
**  ----------------------------------------------------------------------
**  p_info              MT_FE_TP_INFO*      structure pointer
**
**
*************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_get_fec(MT_FE_TP_INFO *p_info, MT_FE_TYPE tp_type)
{
    U8 tmp, val_0x7E;


    if(tp_type == MtFeType_DvbS2)
    {
        _mt_fe_dmd_get_reg(0x7e, &val_0x7E);
        tmp = val_0x7E&0x0f;
        switch(tmp)
        {
            case 0:  p_info->code_rate = MtFeCodeRate_1_4;      break;
            case 1:  p_info->code_rate = MtFeCodeRate_1_3;      break;
            case 2:  p_info->code_rate = MtFeCodeRate_2_5;      break;
            case 3:  p_info->code_rate = MtFeCodeRate_1_2;      break;
            case 4:  p_info->code_rate = MtFeCodeRate_3_5;      break;
            case 5:  p_info->code_rate = MtFeCodeRate_2_3;      break;
            case 6:  p_info->code_rate = MtFeCodeRate_3_4;      break;
            case 7:  p_info->code_rate = MtFeCodeRate_4_5;      break;
            case 8:  p_info->code_rate = MtFeCodeRate_5_6;      break;
            case 9:  p_info->code_rate = MtFeCodeRate_8_9;      break;
            case 10: p_info->code_rate = MtFeCodeRate_9_10;     break;
            default: p_info->code_rate = MtFeCodeRate_Undef;    break;
        }
    }
    else
    {
        _mt_fe_dmd_get_reg(0xe6, &tmp);
        tmp = (U8)(tmp >> 5);
        switch(tmp)
        {
            case 0:  p_info->code_rate = MtFeCodeRate_7_8;      break;
            case 1:  p_info->code_rate = MtFeCodeRate_5_6;      break;
            case 2:  p_info->code_rate = MtFeCodeRate_3_4;      break;
            case 3:  p_info->code_rate = MtFeCodeRate_2_3;      break;
            case 4:  p_info->code_rate = MtFeCodeRate_1_2;      break;
            default: p_info->code_rate = MtFeCodeRate_Undef;    break;
        }
    }


    return MtFeErr_Ok;
}

/**********************************************************************
** Function: _mt_fe_dmd_ds3k_init_reg
**
**
** Discription: initialize the register of ds3k
**
**
** Inputs:
*
**  Parameter        Type           Description
**  ---------------------------------------------------------------
**  p_reg_tabl  const U8 *[2]       pointer to the array
**  size        S32                 the array size
**
** Outputs:
**
**
***********************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_init_reg(const U8 (*p_reg_tabl)[2], S32 size)
{
    MT_FE_RET   ret;
    U32         i;


    for (i=0; i < (U32)size; i++)
    {
        ret = _mt_fe_dmd_set_reg(p_reg_tabl[i][0], p_reg_tabl[i][1]);

        if (ret != MtFeErr_Ok)
            return ret;
    }


    return MtFeErr_Ok;
}



/*******************************************************************
** Function: _mt_fe_dmd_ds3k_download_fw
**
**
** Discription: download the mcu code to ds3k
**
**
** Inputs:
**
**  Parameter   Type        Description
**  ------------------------------------------------------------
**  p_fw        const U8*   pointer of the mcu array
**
** Outputs:
**
**
********************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_download_fw(const U8* p_fw)
{
    U32 i;
    MT_FE_RET   ret = MtFeErr_Ok;

    _mt_fe_dmd_set_reg(0xb2, 0x01);

    for(i = 0; i < 128; i++)
    {
        ret = _mt_fe_write_fw(0xb0, &(((U8*)p_fw)[64*i]), 64);
        if (ret != MtFeErr_Ok)
        {
            break;
        }
    }
    FE_DPrintf("[m88ds3k] _mt_fe_dmd_ds3k_download_fw (i=%d) err(%d)\n",i,ret);

    _mt_fe_dmd_set_reg(0xb2, 0x00);

    if (ret != MtFeErr_Ok)
        return MtFeErr_I2cErr;


    return MtFeErr_Ok;
}


/*************************************************************************
** Function: _mt_fe_dmd_ds3k_bs_set_reg
**
**
** Discription: set blind scan registers
**
**
** Inputs:
**
**  Parameter       type                discription
**  ---------------------------------------------------------------------------
**  bs_times        U8                  blind scan times
**
**
** Outputs:     none
**
**
*************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_bs_set_reg(U8 bs_times)
{
    U8 tmp, sm_buf;


    switch(bs_times)
    {
        case MT_FE_BS_TIMES_1ST:        sm_buf = (feState[feIndex].g_bs_time_total > 1)? SM_BUF_1ST: ONE_LOOP_SM_BUF; break;
        case MT_FE_BS_TIMES_2ND:        sm_buf = SM_BUF_2ND;    break;
        case MT_FE_BS_TIMES_3RD:        sm_buf = SM_BUF_3RD;    break;
        default:                        sm_buf = ONE_LOOP_SM_BUF; break;
    }


    _mt_fe_dmd_set_reg(0xB2, 0x01);
    _mt_fe_dmd_set_reg(0x00, 0x01);


    _mt_fe_dmd_set_reg(0x4A, 0x00);


    _mt_fe_dmd_get_reg(0x4D, &tmp);
    tmp |= 0x91;
    _mt_fe_dmd_set_reg(0x4D, tmp);


    _mt_fe_dmd_get_reg(0x90, &tmp);
    tmp |= TMP1;
    _mt_fe_dmd_set_reg(0x90, tmp);


#if MT_FE_TN_SUPPORT_TS2020
    if(sm_buf == SM_BUF_1ST)
        tmp = 128 + TMP2;
    else
        tmp = TMP2;
#elif (MT_FE_TN_SUPPORT_SHARP6306||MT_FE_TN_SUPPORT_SERIT_SP761VB)
    tmp = TMP2;
#endif
    _mt_fe_dmd_set_reg(0x91, tmp);

    tmp = (TMP3 + sm_buf - 1);
    _mt_fe_dmd_set_reg(0x92, tmp);


    _mt_fe_dmd_get_reg(0x93, &tmp);
    tmp |= TMP4;
    _mt_fe_dmd_set_reg(0x93, tmp);



    _mt_fe_dmd_get_reg(0x94, &tmp);
    if(sm_buf == SM_BUF_1ST)            tmp |= FIND_NOTCH_STEP*16+THR_FACTOR_1ST;
    else if(sm_buf == SM_BUF_2ND)       tmp |= FIND_NOTCH_STEP*16+THR_FACTOR_2ND;
    else if(sm_buf == SM_BUF_3RD)       tmp |= FIND_NOTCH_STEP*16+THR_FACTOR_3RD;
    else if(sm_buf == ONE_LOOP_SM_BUF)  tmp |= FIND_NOTCH_STEP*16+THR_FACTOR_ONE_LOOP;
    _mt_fe_dmd_set_reg(0x94, tmp);


    if(sm_buf == SM_BUF_1ST)            tmp = (64+ERR_BOUND_FACTOR_1ST);
    else if(sm_buf == SM_BUF_2ND)       tmp = (64+ERR_BOUND_FACTOR_2ND);
    else if(sm_buf == SM_BUF_3RD)       tmp = (64+ERR_BOUND_FACTOR_3RD);
    else if(sm_buf == ONE_LOOP_SM_BUF)  tmp = (64+ERR_BOUND_FACTOR_ONE_LOOP);
    _mt_fe_dmd_set_reg(0x95, tmp);


    _mt_fe_dmd_get_reg(0x97, &tmp);
    if(sm_buf == SM_BUF_1ST)            tmp = (TUNER_SLIP_STEP*16 + SNR_THR_1ST/16 - 1);
    else if(sm_buf == SM_BUF_2ND)       tmp = (TUNER_SLIP_STEP*16 + SNR_THR_2ND/16 - 1);
    else if(sm_buf == SM_BUF_3RD)       tmp = (TUNER_SLIP_STEP*16 + SNR_THR_3RD/16 - 1);
    else if(sm_buf == ONE_LOOP_SM_BUF)  tmp = (TUNER_SLIP_STEP*16 + SNR_THR_ONE_LOOP/16 - 1);
    _mt_fe_dmd_set_reg(0x97, tmp);


    if(sm_buf == ONE_LOOP_SM_BUF)
        tmp = FLAT_THR_ONE_LOOP;
    else if(sm_buf == SM_BUF_1ST)
        tmp = FLAT_THR_1ST;
    else if(sm_buf == SM_BUF_2ND)
        tmp = FLAT_THR_2ND;
    else if(sm_buf == SM_BUF_3RD)
        tmp = FLAT_THR_3RD;
    _mt_fe_dmd_set_reg(0x99, tmp);


    _mt_fe_dmd_set_reg(0x30, 0x08);
    _mt_fe_dmd_set_reg(0x32, 0x44);
#if MT_FE_TN_SUPPORT_SHARP6306
    _mt_fe_dmd_set_reg(0x33, 0x38);
#else
    _mt_fe_dmd_set_reg(0x33, 0x35);
#endif
    _mt_fe_dmd_set_reg(0x35, 0x7F);
    _mt_fe_dmd_set_reg(0x4B, 0x04);
    _mt_fe_dmd_set_reg(0x56, 0x01);
    _mt_fe_dmd_set_reg(0xA0, 0x44);
    _mt_fe_dmd_set_reg(0x08, 0x83);

#if MT_FE_ENABLE_AC_COUPLING
    _mt_fe_dmd_get_reg(0x25, &tmp);
    tmp |= 0x08;
    _mt_fe_dmd_set_reg(0x25, tmp);
#endif

    _mt_fe_dmd_set_reg(0x00, 0x00);
    _mt_fe_dmd_set_reg(0xB2, 0x00);

    return MtFeErr_Ok;
}



/*****************************************************************************
** Function: _mt_fe_dmd_ds3k_bs_set_demod
**
**
** Discription: set
**
**
** Inputs:
**
**  Parameter               type                discription
**  ---------------------------------------------------------------------------
**  sym_rate_KSs            U32                 symbol rate passed by app level
**  type                    MT_FE_TYPE          passed by up level
**
**
** Outputs:
**
**
******************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_bs_set_demod(U32 freq_KHz, U32 sym_rate_KSs, MT_FE_TYPE type)
{
    U8            tmp1, tmp2, val, val_0xc3, val_0xc8, val_0xc4 , val_0xc7;
    MT_FE_RET   ret = MtFeErr_Ok;


    if(type == MtFeType_DvbS2)
    {
        _mt_fe_dmd_ds3k_init_reg(ds3k_reg_tbl_dvbs2_def_blindscan, sizeof(ds3k_reg_tbl_dvbs2_def_blindscan)/2);
        _mt_fe_dmd_set_reg(0xc6, 0x40);
        _mt_fe_dmd_get_reg(0x08, &val);
        val |= 0x04;
        _mt_fe_dmd_set_reg(0x08, val);

        #if(MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Common)
            //8.47MHz TS clock output
            tmp1 = 8;
            tmp2 = 9;
        #elif (MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Parallel)
            #if (MtFeTSOut_Max_Clock_12_MHz)
                tmp1 = 6;
                tmp2 = 6;
            #elif (MtFeTSOut_Max_Clock_16_MHz)
                tmp1 = 5;
                tmp2 = 4;
            #elif (MtFeTSOut_Max_Clock_19_p_2_MHz)
                /*actually 18MHz in DVBS2 mode*/
                tmp1 = 4;
                tmp2 = 4;
            #elif (MtFeTSOut_Max_Clock_24_MHz)
                tmp1 = 3;
                tmp2 = 3;
            #endif
        #endif

        val = (U8)((tmp1<<4) + tmp2);
        _mt_fe_dmd_set_reg(0xfe, val);
    }
    else
    {
        _mt_fe_dmd_ds3k_init_reg(ds3k_reg_tbl_dvbs_def_blindscan, sizeof(ds3k_reg_tbl_dvbs_def_blindscan)/2);

        _mt_fe_dmd_get_reg(0x08, &val);
        val &= ~0x04;
        _mt_fe_dmd_set_reg(0x08, val);
        
        #if(MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Common)
            //8MHz TS clock output
            tmp1 = 6;
            tmp2 = 6;
        #elif (MT_FE_TS_OUTPUT_MODE == MtFeTsOutMode_Parallel)
            #if (MtFeTSOut_Max_Clock_12_MHz)
                tmp1 = 4;
                tmp2 = 4;
            #elif (MtFeTSOut_Max_Clock_16_MHz)
                tmp1 = 3;
                tmp2 = 3;
            #elif (MtFeTSOut_Max_Clock_19_p_2_MHz)
                tmp1 = 3;
                tmp2 = 2;
            #elif (MtFeTSOut_Max_Clock_24_MHz)
                tmp1 = 2;
                tmp2 = 2;
            #endif
        #endif

        _mt_fe_dmd_get_reg(0xfe, &val);
        val &= 0xc0;
        val |= ((U8)((tmp1<<3) + tmp2));
        _mt_fe_dmd_set_reg(0xfe, val);
    }


#if MT_FE_ENABLE_AC_COUPLING
    _mt_fe_dmd_get_reg(0x25, &val);
    val |= 0x08;
    _mt_fe_dmd_set_reg(0x25, val);
#endif

#if (MT_FE_ENHANCE_PERFORMANCE != 0)
    #if MT_FE_TN_SUPPORT_TS2022
    if(sym_rate_KSs <= 5000)
    {
        _mt_fe_dmd_set_reg(0xc3, 0x07);
        _mt_fe_dmd_set_reg(0xc8, 0x0c);
        _mt_fe_dmd_set_reg(0xc4, 0x08);
        _mt_fe_dmd_set_reg(0xc7, 0x12);
    }
    else if(sym_rate_KSs <= 20000)
    {
        _mt_fe_dmd_set_reg(0xc3, 0x07);
        _mt_fe_dmd_set_reg(0xc8, 0x0c);
        _mt_fe_dmd_set_reg(0xc4, 0x08);
        _mt_fe_dmd_set_reg(0xc7, 0x0c);
    }
    else
    {
        val_0xc3 = (U8)((2*129032 + sym_rate_KSs)/(2*sym_rate_KSs));
            _mt_fe_dmd_set_reg(0xc3, val_0xc3);
            _mt_fe_dmd_set_reg(0xc8, 0x08);
            _mt_fe_dmd_set_reg(0xc4, 0x08);
            _mt_fe_dmd_set_reg(0xc7, 0x12);
        }
    #else//#if MT_FE_TN_SUPPORT_TS2022
    
        if(sym_rate_KSs <= 5000)
        {
            _mt_fe_dmd_set_reg(0xc3, 0x07);
            _mt_fe_dmd_set_reg(0xc8, 0x0c);
            _mt_fe_dmd_set_reg(0xc4, 0x08);
            _mt_fe_dmd_set_reg(0xc7, 0x12);
        }
        else if(sym_rate_KSs <= 20000)
        {
            _mt_fe_dmd_set_reg(0xc3, 0x07);
            _mt_fe_dmd_set_reg(0xc8, 0x0c);
            _mt_fe_dmd_set_reg(0xc4, 0x08);
            _mt_fe_dmd_set_reg(0xc7, 0x0c);
        }
        else
        {
            val_0xc3 = (U8)((2*129032 + sym_rate_KSs)/(2*sym_rate_KSs));
        _mt_fe_dmd_set_reg(0xc3, val_0xc3);
        _mt_fe_dmd_set_reg(0xc8, 0x08);
        _mt_fe_dmd_set_reg(0xc4, 0x08);
        _mt_fe_dmd_set_reg(0xc7, 0x0c);
    }
    #endif//#if MT_FE_TN_SUPPORT_TS2022
#endif

    _mt_fe_dmd_ds3k_set_sym_rate(sym_rate_KSs);

    _mt_fe_dmd_ds3k_set_CCI(MtFe_Off);

    _mt_fe_dmd_get_reg(0x76, &val);
    val &= ~0x80;
    _mt_fe_dmd_set_reg(0x76, val);


    return ret;
}



/******************************************************************************************
** Function: _mt_fe_dmd_ds3k_bs_save_scanned_TP
**
**
** Discription: save the scanned TPs
**
**
** Inputs:
**
**      Parameter           type        discription
**  ---------------------------------------------------------------------------------------
**      freq_KHz            U32         freq. of the TP need to save
**      symbol_rate_KSs     U32         S.R. of the TP need to save
**      bs_times            U8          blind scan times
**
**
** Outputs:
**
**      Parameter       type                        discription
**  ---------------------------------------------------------------------------------------
**      p_bs_info       MT_FE_BS_TP_INFO *          blind scan struture pointer
**
**
*******************************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_bs_save_scanned_TP(MT_FE_BS_TP_INFO *p_bs_info,
                                            U32 freq_KHz,
                                            U32 symbol_rate_KSs,
                                            U8  bs_times)
{
    U16 i, j;
    MT_FE_TP_INFO *tmp = NULL;

    tmp = p_bs_info->p_tp_info;


    if(feState[feIndex].g_scanned_tp_cnt > (p_bs_info->tp_max_num))
    {
        return MtFeErr_NoMemory;
    }


    if((bs_times == MT_FE_BS_TIMES_1ST)||(feState[feIndex].g_bs_time_total == 1))
    {
        tmp[feState[feIndex].g_scanned_tp_cnt-1].freq_KHz        = freq_KHz;
        tmp[feState[feIndex].g_scanned_tp_cnt-1].sym_rate_KSs    = symbol_rate_KSs;
        tmp[feState[feIndex].g_scanned_tp_cnt-1].dvb_type        = MtFeType_DvbS_Unknown;
        tmp[feState[feIndex].g_scanned_tp_cnt-1].code_rate        = MtFeCodeRate_Undef;
    }
    else
    {
        for(i = 0; i < feState[feIndex].g_scanned_tp_cnt; i++)
        {
            if(freq_KHz <= tmp[i].freq_KHz)
            {
                for(j = feState[feIndex].g_scanned_tp_cnt; j > i; j--)
                {
                    tmp[j].freq_KHz     = tmp[j-1].freq_KHz;
                    tmp[j].sym_rate_KSs = tmp[j-1].sym_rate_KSs;
                    tmp[j].dvb_type     = tmp[j-1].dvb_type;
                    tmp[j].code_rate    = tmp[j-1].code_rate;
                }

                tmp[j].freq_KHz     = freq_KHz;
                tmp[j].sym_rate_KSs = symbol_rate_KSs;
                tmp[j].dvb_type     = MtFeType_DvbS_Unknown;
                tmp[j].code_rate    = MtFeCodeRate_Undef;

                break;
            }
        }


        if(i == feState[feIndex].g_scanned_tp_cnt)
        {
            tmp[feState[feIndex].g_scanned_tp_cnt-1].freq_KHz    = freq_KHz;
            tmp[feState[feIndex].g_scanned_tp_cnt-1].sym_rate_KSs= symbol_rate_KSs;
            tmp[feState[feIndex].g_scanned_tp_cnt-1].dvb_type    = MtFeType_DvbS_Unknown;
            tmp[feState[feIndex].g_scanned_tp_cnt-1].code_rate    = MtFeCodeRate_Undef;
        }
    }



    return MtFeErr_Ok;
}



/********************************************************************************************
** Function: _mt_fe_dmd_ds3k_bs_remove_unlocked_TP
**
**
** Discription: remove the unlocked TP if the TP's freq. is lower than the compared freq.
**
**
** Inputs:
**
**  Parameter           type                                    discription
**  -----------------------------------------------------------------------------------------
**  p_bs_info           MT_FE_BS_TP_INFO *                      blind scan struture pointer
**  compare_freq_MHz    U32                                     the campared TP freq.
**
** Outputs:     none
**
**
*********************************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_bs_remove_unlocked_TP(MT_FE_BS_TP_INFO *p_bs_info, U32 compare_freq_KHz)
{
    U8   i;
    U16  scanned_TP_num, index, move_steps, sym_tmp;
    U32  fre_tmp;

    scanned_TP_num = feState[feIndex].g_scanned_tp_cnt;

    for(i = 0; i < scanned_TP_num; i++)
    {
        if(p_bs_info->p_tp_info[i].freq_KHz >= compare_freq_KHz)
        {
            index = i;
            break;
        }
    }


    if(i == scanned_TP_num)
        index = scanned_TP_num;


    i = 0;
    move_steps = 0;

    while(i <= index)
    {
        if(p_bs_info->p_tp_info[i].dvb_type != MtFeType_DvbS_Checked)
        {
            i++;
            continue;
        }

        while((p_bs_info->p_tp_info[i].dvb_type == MtFeType_DvbS_Checked)&&(i <= index))
        {
            move_steps++;
            i++;
        }

        while((p_bs_info->p_tp_info[i].dvb_type != MtFeType_DvbS_Checked)&&(i <= index))
        {
            fre_tmp = p_bs_info->p_tp_info[i-move_steps].freq_KHz;
            sym_tmp = p_bs_info->p_tp_info[i-move_steps].sym_rate_KSs;

            p_bs_info->p_tp_info[i-move_steps].freq_KHz     = p_bs_info->p_tp_info[i].freq_KHz;
            p_bs_info->p_tp_info[i-move_steps].sym_rate_KSs = p_bs_info->p_tp_info[i].sym_rate_KSs;
            p_bs_info->p_tp_info[i-move_steps].dvb_type     = p_bs_info->p_tp_info[i].dvb_type;
            p_bs_info->p_tp_info[i-move_steps].code_rate    = p_bs_info->p_tp_info[i].code_rate;

            p_bs_info->p_tp_info[i].freq_KHz    = fre_tmp;
            p_bs_info->p_tp_info[i].sym_rate_KSs= sym_tmp;
            p_bs_info->p_tp_info[i].dvb_type    = MtFeType_DvbS_Checked;
            p_bs_info->p_tp_info[i].code_rate   = MtFeCodeRate_Undef;

            i++;
        }
    }


    return MtFeErr_Ok;
}



/********************************************************************************************
** Function: _mt_fe_dmd_ds3k_bs_process_scanned_TP
**
**
** Discription: process the scanned new TP in each blind scan bandwidth
**
**
** Inputs:
**
**      Parameter           type                    discription
**  ---------------------------------------------------------------------------------------
**      cur_scan_freq_KHz   U32             start frequency in each blind scan bandwidth
**      bs_times            U8                     blind scan times
**
**
** Outputs:
**
**      Parameter           type                                discription
**  ------------------------------------------------------------------------------------
**      p_bs_info           MT_FE_BS_TP_INFO *                  tp structure pointer
**
**
********************************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_bs_compare_deal_TP(MT_FE_BS_TP_INFO *p_bs_info,
                                            U32 cur_scan_freq_KHz,
                                            U8  bs_times)
{
    MT_FE_RET   ret = MtFeErr_Ok;

    U8  tmp1, tmp2, tmp3, tmp4, covered_ratio;
    U16 i, last_check_index;
    U32 centerfreqKHz_tpA, symrateKSs_tpA;
    U32 centerfreqKHz_tpB, symrateKSs_tpB;
    U32 left_boundary, right_boundary;
    S32 s_value, delta_freq_KHz, delta_symbol_rate_KSs, freq_delta_KHz;
    S32 similar_symbol_rate_KSs, freq_offset_tmp, symrate_offset_tmp;

#if (MT_FE_DEBUG != 0)
    MT_FE_TP_INFO       tp_info;
#endif

    s_value = 0;
    covered_ratio = 0;

    centerfreqKHz_tpA = 0;
    centerfreqKHz_tpB = 0;
    symrateKSs_tpA    = 0;
    symrateKSs_tpB    = 0;

    _mt_fe_dmd_set_reg(0x9A, 0x20);
    _mt_fe_dmd_get_reg(0x9B, &tmp1);
    _mt_fe_dmd_get_reg(0x9B, &tmp2);
    _mt_fe_dmd_get_reg(0x9B, &tmp3);
    _mt_fe_dmd_get_reg(0x9B, &tmp4);


    symrateKSs_tpB = (U32)((((tmp4&0x03)<<8) | tmp3)*MT_FE_MCLK_KHZ/ FFT_LENGTH);

    s_value = (S32)(((tmp2&0x03)<<8) | tmp1);
    if(s_value >= FFT_LENGTH)
        s_value -= 1024;

    centerfreqKHz_tpB = (U32)(cur_scan_freq_KHz + s_value*(MT_FE_MCLK_KHZ/FFT_LENGTH));
    centerfreqKHz_tpA = p_bs_info->p_tp_info[feState[feIndex].g_tp_index].freq_KHz;
    symrateKSs_tpA    = p_bs_info->p_tp_info[feState[feIndex].g_tp_index].sym_rate_KSs;


    if(centerfreqKHz_tpB < 950000)
        return MtFeErr_Ok;
    if((symrateKSs_tpB > 48000)||(symrateKSs_tpB < 1000))
        return MtFeErr_Ok;
    if((symrateKSs_tpB < 10000)&&((bs_times == MT_FE_BS_TIMES_1ST)&&(feState[feIndex].g_bs_time_total > 1)))
        return MtFeErr_Ok;
    if((symrateKSs_tpB > 15000)&&(bs_times == MT_FE_BS_TIMES_3RD))
        return MtFeErr_Ok;


#if (MT_FE_DEBUG != 0)
    tp_info.freq_KHz     = centerfreqKHz_tpB;
    tp_info.sym_rate_KSs = symrateKSs_tpB;
    mt_fe_bs_notify(MtFeMsg_BSTpFind, &tp_info);
#endif


    if((bs_times == MT_FE_BS_TIMES_1ST)||(feState[feIndex].g_bs_time_total == 1))
    {
        feState[feIndex].g_scanned_tp_cnt++;
        ret = _mt_fe_dmd_ds3k_bs_save_scanned_TP(p_bs_info, centerfreqKHz_tpB, symrateKSs_tpB, bs_times);
        if(ret != MtFeErr_Ok)
            return ret;

        return MtFeErr_Ok;
    }


    while(!feState[feIndex].g_blindscan_cancel)
    {
        right_boundary = centerfreqKHz_tpB + symrateKSs_tpB/2;
        left_boundary  = centerfreqKHz_tpA - symrateKSs_tpA/2;


        if(right_boundary <= left_boundary)
        {
            if(feState[feIndex].g_tp_index > 0)
                feState[feIndex].g_tp_index--;
            else
                feState[feIndex].g_tp_index = 0;

            centerfreqKHz_tpA = p_bs_info->p_tp_info[feState[feIndex].g_tp_index].freq_KHz;

            feState[feIndex].g_scanned_tp_cnt++;
            ret = _mt_fe_dmd_ds3k_bs_save_scanned_TP(p_bs_info, centerfreqKHz_tpB, symrateKSs_tpB, bs_times);
            if(ret != MtFeErr_Ok)
                return ret;

            /*update tpA index*/
            for(i = 0; i < feState[feIndex].g_scanned_tp_cnt; i++)
            {
                if(feState[feIndex].g_blindscan_cancel) break;
                
                if(centerfreqKHz_tpA == (p_bs_info->p_tp_info[i].freq_KHz))
                {
                    feState[feIndex].g_tp_index = i;
                    break;
                }
            }

            break;
        }



        if((p_bs_info->p_tp_info[feState[feIndex].g_tp_index].dvb_type == MtFeType_DvbS)||
           (p_bs_info->p_tp_info[feState[feIndex].g_tp_index].dvb_type == MtFeType_DvbS2))
        {
            if(symrateKSs_tpB > 6000)   covered_ratio = 2;
            else                        covered_ratio = 3;

            if(symrateKSs_tpB > 25000)      freq_delta_KHz = 7000;
            else if(symrateKSs_tpB > 10000) freq_delta_KHz = 4000;
            else if(symrateKSs_tpB > 5000)  freq_delta_KHz = 3000;
            else if(symrateKSs_tpB > 3000)  freq_delta_KHz = 2000;
            else                            freq_delta_KHz = 1000;

            if(centerfreqKHz_tpB >= centerfreqKHz_tpA)
            {
                delta_freq_KHz = centerfreqKHz_tpB - centerfreqKHz_tpA;
                delta_symbol_rate_KSs = (centerfreqKHz_tpA + symrateKSs_tpA/2) - (centerfreqKHz_tpB - symrateKSs_tpB/2);
            }
            else
            {
                delta_freq_KHz = centerfreqKHz_tpA - centerfreqKHz_tpB;
                delta_symbol_rate_KSs = (centerfreqKHz_tpB + symrateKSs_tpB/2) - (centerfreqKHz_tpA - symrateKSs_tpA/2);
            }


            if((delta_symbol_rate_KSs >= (S32)(symrateKSs_tpB/covered_ratio))&&
               (delta_freq_KHz <= (S32)(freq_delta_KHz)))
            {
                mt_fe_print(("\n\tOverlape Skip TP!!!!!!!   Freq. == %d\n", centerfreqKHz_tpB));
                break;
            }
        }
        else
        {
            if(symrateKSs_tpB > 25000)
            {
                freq_offset_tmp     = 5000;
                symrate_offset_tmp  = 800;//5000;
            }
            else if(symrateKSs_tpB > 10000)
            {
                freq_offset_tmp     = 3000;
                symrate_offset_tmp  = 500;//2000;
            }
            else if(symrateKSs_tpB > 5000)
            {
                freq_offset_tmp     = 2100;
                symrate_offset_tmp  = 350;//500;
            }
            else if(symrateKSs_tpB > 3000)
            {
                freq_offset_tmp     = 1000;
                symrate_offset_tmp  = 250;
            }
            else if(symrateKSs_tpB > 1000)
            {
                freq_offset_tmp     = 500;
                symrate_offset_tmp  = 100;
            }


            if(symrateKSs_tpB >= symrateKSs_tpA)
                similar_symbol_rate_KSs = symrateKSs_tpB - symrateKSs_tpA;
            else
                similar_symbol_rate_KSs = symrateKSs_tpA - symrateKSs_tpB;


            if(centerfreqKHz_tpB >= centerfreqKHz_tpA)
                delta_freq_KHz = centerfreqKHz_tpB - centerfreqKHz_tpA;
            else
                delta_freq_KHz = centerfreqKHz_tpA - centerfreqKHz_tpB;

            if((similar_symbol_rate_KSs <= symrate_offset_tmp)&&
               (delta_freq_KHz <= freq_offset_tmp))
            {
                mt_fe_print(("\n\tSimilar Skip TP!!!!!!!  Freq. == %d\n", centerfreqKHz_tpB));
                break;
            }
        }


        last_check_index = feState[feIndex].g_tp_index;
        feState[feIndex].g_tp_index++;

        /*find next tpA*/
        for(i = feState[feIndex].g_tp_index; ; i++)
        {
            if(feState[feIndex].g_blindscan_cancel) break;
            
            if(feState[feIndex].g_tp_index >= feState[feIndex].g_scanned_tp_cnt)
            {
                feState[feIndex].last_tpA_flag = 1;
                feState[feIndex].g_tp_index = last_check_index;
                                    
                break;
            }

            if(p_bs_info->p_tp_info[i].dvb_type != MtFeType_DvbS_Unknown)
              {
                feState[feIndex].g_tp_index = i;
                  centerfreqKHz_tpA = p_bs_info->p_tp_info[feState[feIndex].g_tp_index].freq_KHz;
                symrateKSs_tpA    = p_bs_info->p_tp_info[feState[feIndex].g_tp_index].sym_rate_KSs;

                break;
            }
        }


        if(feState[feIndex].last_tpA_flag == 1)
        {
            feState[feIndex].g_tp_index = last_check_index;
            centerfreqKHz_tpA = p_bs_info->p_tp_info[feState[feIndex].g_tp_index].freq_KHz;

            feState[feIndex].g_scanned_tp_cnt++;
            ret = _mt_fe_dmd_ds3k_bs_save_scanned_TP(p_bs_info, centerfreqKHz_tpB, symrateKSs_tpB, bs_times);
            if(ret != MtFeErr_Ok)
                return ret;

            break;
        }
    }

    feState[feIndex].g_cur_compare_TPA_KHz = centerfreqKHz_tpA;


    return MtFeErr_Ok;
}


/********************************************************************************************
** Function: _mt_fe_dmd_ds3k_bs_process_scanned_TP
**
**
** Discription: process the scanned new TP in each blind scan bandwidth
**
**
** Inputs:
**
**      Parameter           type                    discription
**  ---------------------------------------------------------------------------------------
**      cur_scan_freq_KHz   U32                     start frequency in each blind scan bandwidth
**      bs_times            U8                      blind scan times
**
**
** Outputs:
**
**      Parameter           type                                discription
**  ------------------------------------------------------------------------------------
**      p_bs_info           MT_FE_BS_TP_INFO *                  tp structure pointer
**
**
********************************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_bs_process_scanned_TP(MT_FE_BS_TP_INFO *p_bs_info,
                                               U32 cur_scan_freq_KHz,
                                               U8  bs_times)
{
    MT_FE_BOOL  b_analog_locked, b_fftdone;
    MT_FE_RET   ret = MtFeErr_Ok;

    U8  tmp, cnt;
    U16 i, totaltpnum;

    for(i = 0; i < 2; i++)
    {
        b_fftdone       = MtFe_False;
        b_analog_locked = MtFe_False;

        cnt = 10;
        do
        {
            _mt_fe_dmd_get_reg(0x0d, &tmp);
            b_analog_locked = ((tmp&0x01) == 0x01) ? MtFe_True : MtFe_False;
            _mt_sleep(50);
            cnt--;
        }while((b_analog_locked == MtFe_False)&&(cnt > 0)&&(!feState[feIndex].g_blindscan_cancel));

        if(cnt == 0)
            break;

        _mt_fe_dmd_set_reg(0x9a, 0x80);

        cnt = 50;
        do
        {
            _mt_fe_dmd_get_reg(0x9a, &tmp);
            b_fftdone = ((tmp&0x80) == 0x00) ? MtFe_True : MtFe_False;
            _mt_sleep(10);
            cnt--;
        }while((b_fftdone == MtFe_False)&&(cnt > 0)&&(!feState[feIndex].g_blindscan_cancel));


        if(b_fftdone || (1 == i))
        {
            break;
        }
        else
        {
            _mt_fe_dmd_set_reg(0x5f, 0x00);
            _mt_fe_dmd_set_reg(0x5e, 0x70);
        }
    }

    _mt_fe_dmd_get_reg(0x9a, &tmp);
    totaltpnum = tmp & 0x1F;


    feState[feIndex].g_tp_index       = 0;
    feState[feIndex].last_tpA_flag = 0;
    while((totaltpnum > 0)&&(!feState[feIndex].g_blindscan_cancel))
    {
        totaltpnum--;

        _mt_fe_dmd_ds3k_bs_compare_deal_TP(p_bs_info, cur_scan_freq_KHz, bs_times);
    }


    if(bs_times == MT_FE_BS_TIMES_3RD)
    {
        _mt_fe_dmd_ds3k_bs_remove_unlocked_TP(p_bs_info, feState[feIndex].g_cur_compare_TPA_KHz);

        for(i = 0; i < p_bs_info->tp_max_num; i++)
        {
            if(feState[feIndex].g_cur_compare_TPA_KHz == (p_bs_info->p_tp_info[i].freq_KHz))
            {
                feState[feIndex].g_tp_index = i;
                break;
            }
        }
    }


    return MtFeErr_Ok;
}



/******************************************************************************************
** Function: _mt_fe_dmd_ds3k_bs_A
**
**
** Discription: scan the TP from the start_freq_KHz to end_freq_KHz
**
**
** Inputs:
**
**      Parameter           type                                discription
**  ---------------------------------------------------------------------------------------
**      start_freq_KHz      U32                                 blind scan start freq.
**      end_freq_KHz        U32                                 blind scan stop freq.
**      bs_times            U8                                  blind scan times
**
** Outputs:
**
**      Parameter           type                                discription
**  ---------------------------------------------------------------------------------------
**      p_bs_info           MT_FE_BS_TP_INFO*                   blind scan result pointer
**
**
*******************************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_bs_A(U32 start_freq_KHz,
                              U32 end_freq_KHz,
                              MT_FE_BS_TP_INFO *p_bs_info,
                              U8 bs_times)
{
    U8  val_0x95, val_0x96;
    U32 freq_KHz, tmp_freq_KHz, tmp_tp_scanned;
    MT_FE_RET   ret = MtFeErr_Ok;


    if(mt_fe_tn_set_tuner == NULL)
        return MtFeErr_NullPointer;

    tmp_tp_scanned = 0;
    freq_KHz  = start_freq_KHz;

    while((freq_KHz < end_freq_KHz)&&(!feState[feIndex].g_blindscan_cancel))
    {
        _mt_fe_dmd_ds3k_bs_set_reg(bs_times);
        mt_fe_tn_set_tuner(freq_KHz, BLINDSCAN_SYMRATEKSs, 0);

    #if (MT_FE_TN_SUPPORT_TS2020||MT_FE_TN_SUPPORT_SERIT_SP761VB||MT_FE_TN_SUPPORT_TS2022)
        _mt_fe_dmd_ds3k_set_carrier_offset((S32)(mt_fe_tn_get_offset()));
    #elif MT_FE_TN_SUPPORT_SHARP6306
        _mt_fe_dmd_ds3k_set_carrier_offset((S32)(-mt_fe_tn_get_offset()));
    #endif

        ret = _mt_fe_dmd_ds3k_bs_process_scanned_TP(p_bs_info, freq_KHz, bs_times);
        if(ret != MtFeErr_Ok)
            return ret;

        _mt_fe_dmd_get_reg(0x95, &val_0x95);
        _mt_fe_dmd_get_reg(0x96, &val_0x96);

        tmp_freq_KHz = (U32)((((((val_0x95&0xc0)>>6) << 8) | val_0x96)*MT_FE_MCLK_KHZ)/FFT_LENGTH);
        freq_KHz = (U32)(freq_KHz + tmp_freq_KHz);


    #if (MT_FE_TN_SUPPORT_TS2020||MT_FE_TN_SUPPORT_SERIT_SP761VB||MT_FE_TN_SUPPORT_TS2022)
        if(tmp_freq_KHz == 0)
        {
            freq_KHz += 36000;
        }
    #elif MT_FE_TN_SUPPORT_SHARP6306
        if(tmp_freq_KHz == 0)
        {
            freq_KHz += 30000;
        }
    #endif


        if(tmp_tp_scanned < feState[feIndex].g_scanned_tp_cnt)
        {
            tmp_tp_scanned = feState[feIndex].g_scanned_tp_cnt;
            
            _mt_fe_dmd_ds3k_bs_connect(p_bs_info, 0, feState[feIndex].g_scanned_tp_cnt);
        }

        mt_fe_bs_notify(MtFeMsg_BSOneWinFinish, &(freq_KHz));
    }

    feState[feIndex].time_total = 0;

    return MtFeErr_Ok;
}



/******************************************************************************************
** Function: _mt_fe_dmd_ds3k_bs_B
**
**
** Discription: scan the TP from the start_freq_KHz to end_freq_KHz
**
**
** Inputs:
**
**      Parameter           type                                discription
**  ---------------------------------------------------------------------------------------
**      start_freq_KHz      U32                                 blind scan start freq.
**      end_freq_KHz        U32                                 blind scan stop freq.
**      bs_times            U8                                  blind scan times
**
**
** Outputs:
**
**      Parameter           type                                discription
**  ---------------------------------------------------------------------------------------
**      p_bs_info           MT_FE_BS_TP_INFO*                   blind scan result pointer
**
**
*******************************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_bs_B(U32 start_freq_KHz,
                              U32 end_freq_KHz,
                              MT_FE_BS_TP_INFO *p_bs_info,
                              U8 bs_times)
{
    U8  val_0x95, val_0x96;
    U32 freq_KHz, tmp_freq_KHz;
    MT_FE_RET   ret = MtFeErr_Ok;


    if(mt_fe_tn_set_tuner == NULL)
        return MtFeErr_NullPointer;

    freq_KHz = start_freq_KHz;

    _mt_fe_dmd_ds3k_bs_set_reg(bs_times);

    while((freq_KHz < end_freq_KHz)&&(!feState[feIndex].g_blindscan_cancel))
    {
        mt_fe_tn_set_tuner(freq_KHz, BLINDSCAN_SYMRATEKSs, BLINDSCAN_LPF_OFFSET_KHz);
    #if (MT_FE_TN_SUPPORT_TS2020||MT_FE_TN_SUPPORT_SERIT_SP761VB||MT_FE_TN_SUPPORT_TS2022)
        _mt_fe_dmd_ds3k_set_carrier_offset((S32)(mt_fe_tn_get_offset()));
    #elif MT_FE_TN_SUPPORT_SHARP6306
        _mt_fe_dmd_ds3k_set_carrier_offset((S32)(-mt_fe_tn_get_offset()));
    #endif


        ret = _mt_fe_dmd_ds3k_bs_process_scanned_TP(p_bs_info, freq_KHz, bs_times);
        if(ret != MtFeErr_Ok)
            return ret;

        _mt_fe_dmd_get_reg(0x95, &val_0x95);
        _mt_fe_dmd_get_reg(0x96, &val_0x96);


        tmp_freq_KHz = (U32)((((((val_0x95&0xc0)>>6) << 8) | val_0x96)*MT_FE_MCLK_KHZ)/FFT_LENGTH);
        freq_KHz = (U32)(freq_KHz + tmp_freq_KHz);


    #if (MT_FE_TN_SUPPORT_TS2020||MT_FE_TN_SUPPORT_SERIT_SP761VB||MT_FE_TN_SUPPORT_TS2022)
        if(tmp_freq_KHz == 0)
        {
            printf("Error Error !!!");
            freq_KHz += 36000;
        }
    #elif MT_FE_TN_SUPPORT_SHARP6306
        if(tmp_freq_KHz == 0)
        {
            printf("Error Error !!!");
            freq_KHz += 33000;
        }
    #endif


        mt_fe_bs_notify(MtFeMsg_BSOneWinFinish, &(freq_KHz));
    }


    _mt_fe_dmd_ds3k_bs_connect(p_bs_info, 0, feState[feIndex].g_scanned_tp_cnt);

    feState[feIndex].time_total = 0;

    return MtFeErr_Ok;
}



/************************************************************************************
** Function: _mt_fe_dmd_ds3k_bs_connect
**
**
** Discription: try to lock the scanned TPs according to the start TP index and the
**              TP numbers need to lock
**
**
** Inputs:
**
**      Parameter       type        discription
**  --------------------------------------------------------------------
**      start_index     U16         the start TP index in memory
**      scanned_tp      U16         TP num need to try lock
**
** Outputs:
**
**      Parameter       type                    discription
**  ---------------------------------------------------------------------------------
**      p_bs_info       MT_FE_BS_TP_INFO *      the blindscan TP pointer
**
**
*************************************************************************************/
MT_FE_RET _mt_fe_dmd_ds3k_bs_connect(MT_FE_BS_TP_INFO *p_bs_info,
                                    U16 start_index,
                                    U16 scanned_tp)
{
    U16     index, timing_thr, crl_thr;
    U8      j, count, timing_fail, crl_fail, tmp, same_locked_TP_flag;
    U32     sym_rate_KSs, centerfreq_KHz, truefreq_KHz;
    S16     i, lpf_offset_KHz;
    S32     carrieroffset_KHz, delta_freq_KHz, delta_smy_rate_KSs;

    MT_FE_TYPE          current_mode;
    MT_FE_LOCK_STATE    lockstate;
    MT_FE_TP_INFO       *p_scanned_tp_info = (MT_FE_TP_INFO *)(p_bs_info->p_tp_info);



    if((mt_fe_tn_set_tuner == NULL)||(mt_fe_tn_get_offset == NULL))
        return MtFeErr_NullPointer;

    for(index = start_index; index < scanned_tp; index++)
    {
        if(feState[feIndex].g_blindscan_cancel) break;

        if(p_scanned_tp_info[index].dvb_type != MtFeType_DvbS_Unknown)
            continue;

        centerfreq_KHz  = p_scanned_tp_info[index].freq_KHz;
        sym_rate_KSs    = p_scanned_tp_info[index].sym_rate_KSs;

        lpf_offset_KHz = 0;
        truefreq_KHz   = 0;

    #if ((MT_FE_TN_SUPPORT_TS2020 != 0)||(MT_FE_TN_SUPPORT_TS2022 != 0))
        if (sym_rate_KSs < 5000)
        {
            lpf_offset_KHz  =  BLINDSCAN_LPF_OFFSET_KHz;
            centerfreq_KHz  += BLINDSCAN_LPF_OFFSET_KHz;
        }
    #endif

        for(j = 0; j < 2; j++)
        {
            if(feState[feIndex].g_blindscan_cancel) break;

            current_mode = (j == 0) ? MtFeType_DvbS : MtFeType_DvbS2;

            mt_fe_tn_set_tuner(centerfreq_KHz, sym_rate_KSs, lpf_offset_KHz);

            _mt_fe_dmd_set_reg(0xb2, 0x01);
            _mt_fe_dmd_set_reg(0x00, 0x01);

            _mt_fe_dmd_ds3k_bs_set_demod(centerfreq_KHz, sym_rate_KSs, current_mode);

            if((sym_rate_KSs <= 5000)&&(current_mode == MtFeType_DvbS2))
            {
                _mt_fe_dmd_set_reg(0xc0, 0x04);
                _mt_fe_dmd_set_reg(0x8a, 0x09);
                _mt_fe_dmd_set_reg(0x8b, 0x22);
                _mt_fe_dmd_set_reg(0x8c, 0x88);
            }
        #if MT_FE_TN_SUPPORT_TS2022
            _mt_fe_dmd_get_reg(0x0c, &tmp);
            tmp |= 0x04;
            _mt_fe_dmd_set_reg(0x0c, tmp);
        #endif


             _mt_fe_dmd_ds3k_set_carrier_offset((S32)mt_fe_tn_get_offset()+lpf_offset_KHz);
 
            _mt_fe_dmd_set_reg(0x00, 0x00);
            _mt_fe_dmd_set_reg(0xb2, 0x00);

    #if 1
            if(sym_rate_KSs >= 10000)
            {
                count = 30;
                timing_thr = 2;
                crl_thr = 1;
            }
            else if(sym_rate_KSs >= 4000)
            {
                count = 50;
                timing_thr = 2;
                crl_thr = 1;
            }
            else/*only care the time to lock DVBS signal*/
            {
                count = 40;
                timing_thr = 2;
                crl_thr = 1;
            }
    #else
            if(sym_rate_KSs >= 10000)
            {
                count = 25;
                timing_thr = 2;
                crl_thr = 1;
            }
            else if(sym_rate_KSs >= 4000)
            {
                count = 28;
                timing_thr = 2;
                crl_thr = 1;
            }
            else if(sym_rate_KSs >= 2000)
            {
                count = 35;
                timing_thr = 2;
                crl_thr = 1;
            }
            else
            {
                count = 40;
                timing_thr = 2;
                crl_thr = 1;
            }
    #endif

            timing_fail = 0;
            crl_fail    = 0;
            lockstate   = MtFeLockState_Unlocked;

            while((count > 0)&&(!feState[feIndex].g_blindscan_cancel))
            {
                if (current_mode == MtFeType_DvbS2)
                {
                    _mt_fe_dmd_get_reg(0x0d, &tmp);
                    if ((tmp & 0x8f) == 0x8f)
                        lockstate = MtFeLockState_Locked;
                }
                else
                {
                    _mt_fe_dmd_get_reg(0xd1, &tmp);
                    if ((tmp & 0x07) == 0x07)
                        lockstate = MtFeLockState_Locked;
                }


                if(lockstate == MtFeLockState_Locked)
                {
                    break;
                }
                else
                {
                    _mt_fe_dmd_get_reg(0xb3, &tmp);
                    if((tmp & 0x80) == 0x80)
                    {
                        timing_fail++;
                        if(timing_fail >= timing_thr)
                        {
                            timing_fail = 1;
                            break;
                        }
                        mt_fe_dmd_ds3k_soft_reset();
                        count = 30;
                    }
                    else if(((tmp & 0x40) == 0x40)&&((tmp & 0x80) != 0x80))
                    {
                        crl_fail++;
                        if(crl_fail >= crl_thr)
                        {
                            crl_fail = 1;
                            break;
                        }
                        mt_fe_dmd_ds3k_soft_reset();
                        count = 50;
                    }
                }

                _mt_sleep(50);
                count--;
            }


            if(current_mode == MtFeType_DvbS2)
            {
                printf(" Try S2 time_total == %dms\n", feState[feIndex].time_total);
            }


            if(count == 0)
            {
                mt_fe_bs_notify(MtFeMsg_BSTpUnlock, &(p_bs_info->p_tp_info[index]));
                p_bs_info->p_tp_info[index].dvb_type = MtFeType_DvbS_Checked;
                break;
            }


            if(timing_fail == 1)
            {
                mt_fe_bs_notify(MtFeMsg_BSTpUnlock, &(p_bs_info->p_tp_info[index]));
                p_bs_info->p_tp_info[index].dvb_type = MtFeType_DvbS_Checked;
                break;
            }
            else if(crl_fail == 1)
            {
                if(current_mode == MtFeType_DvbS)
                {
                    continue;
                }
                else
                {
                    mt_fe_bs_notify(MtFeMsg_BSTpUnlock, &(p_bs_info->p_tp_info[index]));
                    p_bs_info->p_tp_info[index].dvb_type = MtFeType_DvbS_Checked;
                    break;
                }
            }

            if(lockstate == MtFeLockState_Locked)
            {
                _mt_fe_dmd_ds3k_get_carrier_offset(&carrieroffset_KHz);

                #if (MT_FE_TN_SUPPORT_TS2020||MT_FE_TN_SUPPORT_SERIT_SP761VB||MT_FE_TN_SUPPORT_TS2022)
                    truefreq_KHz = centerfreq_KHz + (S32)mt_fe_tn_get_offset() - carrieroffset_KHz;
                #elif MT_FE_TN_SUPPORT_SHARP6306
                    truefreq_KHz = centerfreq_KHz - (S32)mt_fe_tn_get_offset() - carrieroffset_KHz;
                #endif

                _mt_fe_dmd_ds3k_get_sym_rate(&sym_rate_KSs);

                same_locked_TP_flag = 0;
                for(i = feState[feIndex].g_scanned_tp_cnt-1; i >= 0; i--)
                {
                    if(feState[feIndex].g_blindscan_cancel) break;

                    if(p_bs_info->p_tp_info[i].freq_KHz > truefreq_KHz)
                        delta_freq_KHz = p_bs_info->p_tp_info[i].freq_KHz - truefreq_KHz;
                    else
                        delta_freq_KHz = truefreq_KHz - p_bs_info->p_tp_info[i].freq_KHz;

                    if(p_bs_info->p_tp_info[i].sym_rate_KSs > sym_rate_KSs)
                        delta_smy_rate_KSs =  p_bs_info->p_tp_info[i].sym_rate_KSs - sym_rate_KSs;
                    else
                        delta_smy_rate_KSs =  sym_rate_KSs - p_bs_info->p_tp_info[i].sym_rate_KSs;


                    if(((delta_smy_rate_KSs < 10)&&(delta_freq_KHz < 3000))&&
                       (p_bs_info->p_tp_info[i].dvb_type == current_mode))
                    {
                        same_locked_TP_flag = 1;

                        break;
                    }
                }

                if(same_locked_TP_flag == 1)
                    break;

                feState[feIndex].g_locked_tp_cnt++;
                p_bs_info->p_tp_info[index].freq_KHz      = truefreq_KHz;
                p_bs_info->p_tp_info[index].sym_rate_KSs = sym_rate_KSs;
                p_bs_info->p_tp_info[index].dvb_type      = current_mode;
                _mt_fe_dmd_ds3k_get_fec(&(p_bs_info->p_tp_info[index]), current_mode);
                
                p_bs_info->tp_num = feState[feIndex].g_locked_tp_cnt;

            #if (MT_FE_TS_CLOCK_AUTO_SET_FOR_CI_MODE != 0)
                _mt_fe_dmd_ds3k_set_clock_ratio();
            #endif

                mt_fe_bs_notify(MtFeMsg_BSTpLocked, &(p_bs_info->p_tp_info[index]));

                break;
            }
            else
            {
                if(current_mode == MtFeType_DvbS2)
                    p_bs_info->p_tp_info[index].dvb_type = MtFeType_DvbS_Checked;

                mt_fe_bs_notify(MtFeMsg_BSTpUnlock, &(p_bs_info->p_tp_info[index]));
            }
        }
    }


    return MtFeErr_Ok;
}
