/****************************************************************************
* MONTAGE PROPRIETARY AND CONFIDENTIAL
* Montage Technology (Shanghai) Inc.
* All Rights Reserved
* --------------------------------------------------------------------------
*
* File:                mt_fe_i2c.c
*
* Current version:    2.01.00
*
* Description:        Define all i2c function for FE module.
*
* Log:    Description            Version        Date        Author
*        ---------------------------------------------------------------------
*        Create                1.00.00        2008.08.21    Rick
*        Modify                1.01.00        2008.10.13    Rick
*        Modify                1.10.00        2008.11.17    Rick
*        Modify                2.01.00        2009.04.29  Rick
****************************************************************************/
#include "mt_fe_def.h"
#include "../src/fe_priv.h"
#ifdef _LINUX_FAPI
#include <fapex/abst_types.h>
#else
#include <fapi/drv_i2c.h>
#endif

int32_t i2cHandle = 0;//feHandleArray[0].i2cHandle;
uint8_t   i2cAddr   = 0;
uint8_t   feIndex   = 0;

/*****************************************************************
** Function: _mt_fe_dmd_set_reg
**
**
** Discription: write data to demod register
**
**
** Inputs:
**
**    Parameter        Type        Description
**  ----------------------------------------------------------
**  reg_index   U8          register index
**  data        U8          value to write
**
**
** Outputs:
**
**
*****************************************************************/
MT_FE_RET _mt_fe_dmd_set_reg(U8 reg_index, U8 data)
{
    U8  buffer[2];
    S32 err;
    buffer[0] = reg_index;
    buffer[1] = data;
    /*
        TODO:
            Obtain the i2c mutex
    */



    /*
        TODO:
            write data to demodulator register
    */

    err = FAPI_I2C_Write(i2cHandle,i2cAddr,buffer,2);
    if(err !=MtFeErr_Ok)
    {
        FAPI_SYS_PRINT_MSG("[dmd set] 0x%x error %d.\n",reg_index,err);
        return (MT_FE_RET)err;
    }


    /*
        TODO:
            Release the i2c mutex
    */


    
    return MtFeErr_Ok;
}



/*****************************************************************
** Function: _mt_fe_dmd_get_reg
**
**
** Discription:    read data from demod register
**
**
** Inputs:
**
**    Parameter        Type        Description
**    ----------------------------------------------------------
**    reg_index        U8            register index
**
**
**    Parameter        Type        Description
**    ----------------------------------------------------------
**    p_buf            U8*            register data
**
**
*****************************************************************/
MT_FE_RET _mt_fe_dmd_get_reg(U8 reg_index, U8 *p_buf)
{
    
    U8 buffer[1];
    S32 err;
    buffer[0] = reg_index;    
    /*
        TODO:
            Obtain the i2c mutex
    */

    

    /*
        TODO:
            read demodulator register value 
    */

    
    err = FAPI_I2C_Read(i2cHandle,i2cAddr,buffer,1,p_buf,1);
    if(err !=MtFeErr_Ok)
    {
        FAPI_SYS_PRINT_MSG("[dmd get] 0x%x error %d.\n",reg_index,err);
        return (MT_FE_RET)err;
    }
    /*
        TODO:
            Release the i2c mutex
    */

    
    return MtFeErr_Ok;
}


/*****************************************************************
** Function: _mt_fe_tn_set_reg
**
**
** Discription: write data to tuner register
**
**
** Inputs:
**
**    Parameter        Type        Description
**  ----------------------------------------------------------
**  reg_index   U8          register index
**  data        U8          value to write
**
**
** Outputs:
**
**
*****************************************************************/
MT_FE_RET _mt_fe_tn_set_reg(U8 reg_index, U8 data)
{
    U8 val, ret;
    U8 buffer[2];
    S32 err=0;

    buffer[0] = reg_index;
    buffer[1] = data;

    /*
        TODO:
            Obtain the i2c mutex
    */
    /*open I2C repeater*/
    /*Do not care to close the I2C repeater, it will close by itself*/
#if(LOCK_LED_WHEN_LOCK_HIGH)
    val = 0x11;
#else
    val = 0x31;
#endif
    ret = _mt_fe_dmd_set_reg(0x03, val);
    if (ret != MtFeErr_Ok)
        return ret;
    
    
    /*Do not sleep any time after I2C repeater is opened.*/
    /*please set tuner register at once.*/
    /*
        TODO:
            write value to tuner register
    */

    err = FAPI_I2C_Write(i2cHandle,MT_FE_TN_I2C_ADDR,buffer,2);
    if(err !=MtFeErr_Ok)
    {
        FAPI_SYS_PRINT_MSG("[tn set] error %d.\n",err);
        return (MT_FE_RET)err;
    }
    /*
        TODO:
            Release the i2c mutex
    */

    return MtFeErr_Ok;
}




/*****************************************************************
** Function: _mt_fe_tn_get_reg
**
**
** Discription: get tuner register data
**
**
** Inputs:
**
**  Parametre   type        discription
**  ----------------------------------------------------------
**  register    U8          register address
**
**
** Outputs:
**
**  Parametre   type        discription
**  ----------------------------------------------------------
**  p_buf       U8*         register data
**
**
*****************************************************************/
MT_FE_RET _mt_fe_tn_get_reg(U8 reg_index, U8 *p_buf)
{
      U8 val, ret;
    U8 buffer[1];
      S32 err;

    buffer[0] = reg_index;
    /*
        TODO:
            Obtain the i2c mutex
    */
    /*open I2C repeater*/
    /*Do not care to close the I2C repeater, it will close by itself*/
#if(LOCK_LED_WHEN_LOCK_HIGH)
    val = 0x12;
#else
    val = 0x32;
#endif
    ret = _mt_fe_dmd_set_reg(0x03, val);
      if (ret != MtFeErr_Ok)
            return ret;

    /*Do not sleep any time after I2C repeater is opened.*/
    /*please read tuner register at once.*/

    /*
        TODO:
            read tuner register value
    */
    err = FAPI_I2C_Read(i2cHandle,MT_FE_TN_I2C_ADDR,buffer,1,p_buf,1);

    if(err !=MtFeErr_Ok)
    {
        FAPI_SYS_PRINT_MSG("[tn get] 0x%x error %d.\n",reg_index,err);
        return (MT_FE_RET)err;
    }

    /*
        TODO:
            Release the i2c mutex
    */
    return MtFeErr_Ok;
}

MT_FE_RET _mt_fe_tn_write(U8 *p_buf, U16 n_byte)
{
    U8  val, ret;
    S32 err;

    /*
        TODO:
            Obtain the i2c mutex
    */


    /*open I2C repeater*/
    /*Do not care to close the I2C repeater, it will close by itself*/
#if(LOCK_LED_WHEN_LOCK_HIGH)
    val = 0x11;
#else
    val = 0x31;
#endif

    ret = _mt_fe_dmd_set_reg(0x03, val);
    if (ret != MtFeErr_Ok)
        return ret;

    /*Do not sleep any time after I2C repeater is opened.*/
    /*please write N bytes to register at once.*/

    /*
        TODO:
            write N bytes to tuner
    */

    err = FAPI_I2C_Write(i2cHandle,MT_FE_TN_I2C_ADDR,p_buf,n_byte);
    if(err !=MtFeErr_Ok)
    {
        FAPI_SYS_PRINT_MSG("[tn set] error %d.\n",err);
        return (MT_FE_RET)err;
    }

    /*
        TODO:
            Release the i2c mutex
    */

    return MtFeErr_Ok;
}


/*****************************************************************
** Function: _mt_fe_iic_write
**
**
** Discription: write n bytes via iic to device
**
**
** Inputs:
**
**
**  Parametre   type        discription
**  ----------------------------------------------------------
**  p_buf       U8*         pointer of the tuner data
**  n_byte      U16         the data length
**
**
** Outputs:     none
**
**
**
*****************************************************************/
MT_FE_RET _mt_fe_write_fw(U8 reg_index, const U8 *p_buf, U16 n_byte)
{

    S32 err;
    U8 buffer[65];
    buffer[0] = reg_index;
    memcpy(&(buffer[1]),p_buf,n_byte);

    /*
        TODO:
            Obtain the i2c mutex
    */

    /*
        TODO:
            write n bytes to demodulator
    */
    err = FAPI_I2C_Write(i2cHandle,i2cAddr,&(buffer[0]),n_byte+1);
    if(err != FAPI_OK)
    {
        return MtFeErr_I2cErr;        
    }



    /*
        TODO:
            Release the i2c mutex
    */

    return MtFeErr_Ok;
}

#if 0
/*****************************************************************
** Function: _mt_fe_iic_read
**
**
** Discription: read n bytes via iic
**
**
** Inputs:
**
**
**  Parametre   type        discription
**  ----------------------------------------------------------
**  p_buf       U8*         pointer of the tuner data
**  n_byte      U16         the data length
**
**
** Outputs:     none
**
**
**
*****************************************************************/
MT_FE_RET _mt_fe_iic_read(U8 chip_addr, U8 reg_index, U8 *p_buf, U16 n_byte)
{



    /*
        TODO:
            Obtain the i2c mutex
    */




    /*
        TODO:
            write n bytes to tuner
    */



    /*
        TODO:
            Release the i2c mutex
    */

    return MtFeErr_Ok;
}
#endif


