/******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Customise.h
**
** \brief     Application dependent functions for MB86667 driver.
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: MB86667_Customise.h 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
/*-------- compilation control switches ---------------------------- */
#ifndef MB86667_CUSTOMISE_H
#define MB86667_CUSTOMISE_H

/* ------- required includes --------------------------------------- */
#include "MB86667_Api.h"
#include "MB86667_Tuner.h"

/* ------- defines configuring debug outputs ----------------------- */
/* define FAPI_FE_MB86667_DEBUG01 if you want to get debug messages */
// #define FAPI_FE_MB86667_DEBUG01
/* define FAPI_FE_MB86667_DEBUG02 if you want to get additional test debug messages */
// #define FAPI_FE_MB86667_DEBUG02

#if defined (FAPI_FE_MB86667_DEBUG01) || defined (FAPI_FE_MB86667_DEBUG02)
   #include <stdio.h>
   extern char PrintHelp[];  /* temporary string used for printing messages */
#endif

/* ------- defines configuring the hardware ------------------------ */
/* Configures the used tuner device enable selected */
// #define TUNER_MAXIM_2118 // currently not supported
#define TUNER_SHARP_BS2F7VZ0302
#define TUNER_SHARP_BS2S7HZ1402

/* ------- used in MB86667_Init to set ports and TS output ------------- */
/* MB86667_TS_POL defines if the transport data are valid at the rising of
   falling edge of TCLK */
#define MB86667_TS_POL  1 /* 0 = falling, 1 = rising edge */
/* MB86667_TS_ERROR defines if the error flag in the packet header should be
   set in the case of uncorrected bit errors */
#define MB86667_TS_ERROR 0 /* 0 = error flag is used, 2 = error flag is always 0 */
/* MB86667_TS_ORDER defines whether the serial TS stream is available at pin
   TS0 or TS7. In parallel mode it swaps the order of MSB and LSB */
#define MB86667_TS_ORDER 0 /* 0 = MSB at pin TS7
                            4 = MSB at pin TS0 */
/* MB86667_TS_MODE defines if TS is output in serial or parallel mode */
#define MB86667_TS_SERIAL 0 /* 0 = parallel mode, 8 = serial mode */
#define MB86667_DIG_OUTPUTS_ENABLE_BY_I2C  0x20  /* controls output mode of
                                                    digital pins by i2c and 
                                                    sets pins to normal operation */
#define MB86667_DIG_OUTPUTS_DISABLE_BY_I2C 0x30  /* controls output mode of
                                                    digital pins by i2c and 
                                                    sets pins to highZ */
/* MB86667_FR_LOCK defines number of sync bytes in series until lock is detected */
#define MB86667_FR_LOCK   0x02 /* 0x02 ... 0x0F */
/* MB86667_FR_UNLOCK defines number of missing sync bytes in series until lock is lost */
#define MB86667_FR_UNLOCK 0x0F /* 0x02 ... 0x0F */

/* ------- defines, configuring the API -----------------------------*/
/* defines for timer flags */
#define TIM_SYN_TIM 1
#define TIM_RES_TIM 2
#define TIM_GET_TIM 4
/* defines the timer resolution in ms, needed for rounding */
#define TIM_RESOLUTION 1

/* the following values normally don't need to be changed, only if you want to
   add more timer calls */
/* defines for timer usage */
#define TIM_CS    0 /* timer for carrier search */
#define TIM_LOCK  1 /* timer for complete lock */
#define TIM_SLEEP 2 /* timer for quality read out */

#define MB86667_MAX_USER_TIMERS  10

#ifdef  __cplusplus
extern "C" {
#endif

/********************************************************
    Definitions for MB86667
*********************************************************/
void MB86667_SetContext(uint8_t index, FAPI_SYS_HandleT _ghwIndex, int32_t _i2cId, uint8_t _i2cDevAddr, uint8_t _i2cTunAddr,
                        FE_MB86667_RF_INI_FCT_T iniFct, FE_MB86667_RF_SET_FCT_T setFct);
void MB86667_SaveContext(uint8_t index);
void MB86667_ChangeFlags(uint8_t index, FAPI_MB86667_DIS_FLAGS_E flags, uint8_t comRep, uint8_t uncRep);

int32_t MB86667_I2C_Bypass_Read(uint8_t reg, uint8_t* rvalue);
int32_t MB86667_I2C_Bypass_Write(uint8_t* value, uint8_t length);

/* the following functions, located in MB86667_Customise.c, need to be adopted to your environment */
int32_t MB86667_I2C_Read(uint8_t ucSubAddress, uint8_t ucNrOfBytes, uint8_t *pucData);
int32_t MB86667_I2C_Write(uint8_t ucSubAddress, uint8_t ucNrOfBytes, uint8_t *pucData);
int32_t TUNER_I2C_Read(uint8_t ucReg, uint8_t ucNrOfBytes, uint8_t *pucData);
int32_t TUNER_I2C_Write(uint8_t ucNrOfBytes, uint8_t *pucData);
int32_t MB86667_HardReset(void);
int32_t MB86667_User_Timers(uint8_t ucFlag, uint8_t ucIndex, uint32_t *pulDiffTime);
int32_t MB86667_User_Sleep(uint32_t ulTime);
#if defined (FAPI_FE_MB86667_DEBUG01) || defined (FAPI_FE_MB86667_DEBUG02)
    int32_t MB86667_User_Print(char *pString);
#endif

#ifdef  __cplusplus
}
#endif


#endif /* MB86667_CUSTOMISE_H */
