/*******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Demod.h
**
** \brief     Demod Driver for MB86667.
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   $Id: MB86667_Demod.h 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
#ifndef _MB86667_DEMOD_H_
#define _MB86667_DEMOD_H_

#include "MB86667_Api.h"

/*TP SETTING*/
typedef struct
{
  uint32_t frequency;
  uint16_t symbolrate;
  uint8_t  fec;
  int32_t freqerr;
}MB86667_SET_PARAM;

/* BLIND SCAN SETTING */
typedef struct
{
  uint32_t fre_min;
  uint32_t fre_max;
  uint16_t sr_min;
  uint16_t sr_max;
}MB86667_BLIND_PARAM;

/* TP RESULT */
typedef struct
{
  uint32_t frequency;
  
  uint16_t symbolrate;
  uint8_t  fec;
  uint8_t  iq;
}MB86667_TP_RESULT;

enum
{
  MB86667_RECEIVE    = 0x33,
  MB86667_BLIND_SCAN = 0x34
};


/*******************************************************************************
**
** constants
**
******************************************************************************/
#define   FCP                 3000  //capture range of carrier recovery[kHz]
#define   RF_FRE_STEP         500   //freq_step=500kHz
#define   SIG1MIN             17
#define   TUNER_POLARITY      -1
#define   TUNER_BYPASS_ADR    0x25
#define   TUNER_BYPASS_DATA   0x02
#define   TP_NUM              60
#define   START_ADDRESS       0xEA1000
#define   TIMEOUT             0x1000000


#ifdef __cplusplus
extern "C" {
#endif
/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/
int32_t  MB86667_Abs(int32_t a);
int32_t MB86667_Acfex_Data_Set(uint8_t afcex_l, uint8_t afcex_h);
void MB86667_Afcex_Info_Get(int32_t clkmst, int32_t afcex_freq, uint8_t *afcex_l, uint8_t *afcex_h);
int32_t MB86667_Afcexen_Set(uint8_t afcexen,uint16_t smrt);
int32_t MB86667_Afcofs_Data_Set(uint8_t afcex_l, uint8_t afcex_h);
int32_t MB86667_AGC_SET(uint8_t agcw);
int32_t MB86667_Cntm_Set(uint8_t timint1, uint8_t timint2,uint8_t cnext);
int32_t MB86667_Dagc_Data_Set(uint8_t dagca, uint8_t dagcw);
int32_t  MB86667_DoTune(MB86667_SET_PARAM* tp_param, uint8_t mode, FBOOL quicklock);
int32_t MB86667_En_Set(uint8_t cren, uint8_t afcen);
int32_t MB86667_Eq_Set(uint8_t en, uint8_t gain);
int32_t MB86667_Fec_Srst(void);
int32_t MB86667_Freqerr_Chk(uint32_t fTP, int32_t fOSC_10, uint16_t smrt, uint8_t unit, int32_t *freqerr );
uint32_t  MB86667_GetStatus(void);
int32_t MB86667_I2C_READ(uint8_t reg, uint8_t *rvalue);
int32_t MB86667_I2C_WRITE(uint8_t Reg, uint8_t Value);
int32_t MB86667_Initial_All(void);
int32_t MB86667_Initial_Set(void);
int32_t MB86667_Iq_Vt_Set(uint8_t iqinv);
int32_t MB86667_S01t_Set(uint8_t s1t, uint8_t s0t );
int32_t MB86667_S2t_Set(uint8_t s2t );
int32_t MB86667_S45t_Set(uint8_t s4t, uint8_t s5t );
int32_t MB86667_Seq_Set(uint8_t Loop);
int32_t MB86667_Signal_Det(uint16_t smrt, uint8_t *deci, uint8_t *csel, uint8_t *rsel, int32_t *clkmst, uint8_t *sig, FBOOL *ret);
void MB86667_Smrt_Info_Get(uint16_t sr, uint8_t *deci,uint8_t *csel,uint8_t *rsel,int32_t *clkmst );
int32_t MB86667_Smrt_Set(uint16_t sr, uint8_t deci, uint8_t csel, uint8_t rsel, int32_t clkmst);
int32_t MB86667_Srst(void);
int32_t MB86667_Stlp_Set(uint8_t stras,uint8_t strbs);
int32_t MB86667_Stlp_Set2(uint8_t stra,uint8_t strb);
int32_t  MB86667_Swp_Freq_Calculation( int32_t i, int8_t v, int16_t *va,  int8_t vmax, int8_t vmin,
                                   uint8_t sigmin, int32_t fOSC_10, int32_t afcex_freq, int32_t swp_ofs, uint8_t *sig1);
void MB86667_Swp_Info_Get(int32_t fOSC_start, int32_t clkmst, int8_t v,
                          int16_t rf_freq_step, int32_t swp_ofs, int32_t *fOSC_10,
                          int32_t *afcex_freq, uint8_t *afcex_l, uint8_t *afcex_h);
void MB86667_Swp_Info_Get2(int16_t rf_freq_step,int32_t swp_freq, int32_t clkmst, int32_t *afcex_freq,
                           int32_t *fOSC_10, uint8_t *afcex_l, uint8_t *afcex_h);
int32_t MB86667_Sync_Chk2(uint8_t *virm, uint8_t *sync);
int32_t MB86667_Sync_Chk1(uint8_t *sync);
int32_t MB86667_Vi_Set(uint8_t eth, uint8_t via);
void MB86667_WAIT(int32_t time);


#ifdef __cplusplus
}
#endif


#endif /* _MB86667_DEMOD_H_ */
/* end of MB86667_Demod.h */
