/******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Priv.h
**
** \brief     Private functions for MB86667 driver.
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: MB86667_Priv.h 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
/*-------- compilation control switches -----------------------------*/
#ifndef MB86667_PRIVATE_H
#define MB86667_PRIVATE_H

/* ------- defines --------------------------------------------------*/
#define MB86667_FEC_MASK 0x1F

typedef enum
{   TF_UNKNOWN    = 0x04, /* unknown state, due to failed tune process */
    TF_ACQ_START  = 0x03, /* acquisition just started */
    TF_CS_START   = 0x02, /* carrier search started */
    TF_CS_END     = 0x01, /* carrier search ended */
    TF_ACQ_END    = 0x00  /* acquisition ended */
} FAPI_MB86667_TUNE_STATE_E;

typedef enum
{   DAGC_LOCK     = 0, /* setting for normal operation */
    DAGC_SPECTRUM,     /* setting for spectrum scan */
    DAGC_DISTORTION    /* setting for distortion check */
} FD_MB86667_DAGC_MODE;

#ifdef __cplusplus
extern "C" {
#endif
/*-------- local type definitions -----------------------------------*/

/*-------- local function definitions -------------------------------*/
uint8_t                      mb86667_clock_fac(FAPI_MB86667_GETSET_E dir,  uint8_t ucClockFac);
uint8_t                      mb86667_com_sw(FAPI_MB86667_GETSET_E dir, uint8_t ucRep);
uint32_t                     mb86667_cs_time(FAPI_MB86667_GETSET_E dir,  uint32_t ulTime);
FAPI_MB86667_DIS_FLAGS_E  mb86667_flags(FAPI_MB86667_GETSET_E dir,  FAPI_MB86667_DIS_FLAGS_E DisFlags);
int32_t                    mb86667_lnb_control(uint32_t ulFreq, uint16_t usTrPar, uint16_t usSatID, FAPI_MB86667_BAND_E bFreqB);
uint32_t                     mb86667_lock_time(FAPI_MB86667_GETSET_E dir,  uint32_t ulTime);
uint16_t                     mb86667_sym_rate(FAPI_MB86667_GETSET_E dir,  uint16_t uwSymRate);
FAPI_MB86667_TUNE_STATE_E mb86667_tune_states(FAPI_MB86667_GETSET_E dir, FAPI_MB86667_TUNE_STATE_E tuneState);
uint8_t                      mb86667_unc_sw(FAPI_MB86667_GETSET_E dir, uint8_t ucRep);

#ifdef __cplusplus
}
#endif

#endif /* MB86667_PRIVATE_H */
