/******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Regs.h
**
** \brief     Generic front-end driver.
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: MB86667_Regs.h 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
/*-------- compilation control switches -----------------------------*/
#ifndef MB86667_REGS_H
#define MB86667_REGS_H

//#define ORIGINAL_FUJ_EVALBOARD

/* ------- register addresses -------------------------------------- */
#define MB86667_REG_TSOUT        0x00
#define MB86667_REG_FEC          0x01
#define MB86667_REG_FAGC         0x02
    #define AGCR                    0x01 //bit 0: output rate: 0=slow 1=fast
    #define AGCP                    0x02 //bit 1: polarity: 0=proportional 1=invers  

    #define SHIFT_AGCW_0_3          0x02
    #define AGMD                    0x40 //bit 7: automatic: 0=off 1=on; orig= 0x00

#define MB86667_REG_RATE_ROUGH   0x03
    #define MB86667_BIT_RATE_ROUGH_SHIFT_CSEL  0x01
    #define MB86667_BIT_RATE_ROUGH_SHIFT_DECI2 0x02
#define MB86667_REG_RATE_ACCUR1  0x04
#define MB86667_REG_RATE_ACCUR2  0x05
#define MB86667_REG_IQ_AUTODET   0x06
    #define IQRES                   0x80
    #define IQAT                    0x40
    #define IQINV                   0x20
    #define VIR4                    0x10
    #define VIR3                    0x08
    #define VIR2                    0x04
    #define VIR1                    0x02
    #define VIR0                    0x01
    #define SHIFT_IQINV             0x05

#define MB86667_REG_SYNC         0x07
#define MB86667_REG_CR_L_FLT1    0x08
#define MB86667_REG_STRF1        0x0A
    #define SHIFT_STRBS_0_2         0x03
#define MB86667_REG_STRF2        0x0B

#define MB86667_REG_RESET        0x0C
    #define MB86667_BIT_RESET_QP    0x04
    #define MB86667_BIT_RESET_FEC   0x02
    #define MB86667_BIT_RESET_LSI   0x01
#define MB86667_REG_STAT         0x0D
#define MB86667_REG_AFCML        0x0E
#define MB86667_REG_AFCMH        0x0F
#define MB86667_REG_BER          0x10
#define MB86667_REG_BERLS        0x12
#define MB86667_REG_BERMI        0x13
#define MB86667_REG_BERMS        0x14
#define MB86667_REG_AGC_MONITOR  0x15
#define MB86667_REG_DICR         0x16
#define MB86667_REG_DIST         0x17
#define MB86667_REG_DIB0         0x18
#define MB86667_REG_DIB1         0x19
#define MB86667_REG_DIB2         0x1A
#define MB86667_REG_DIB3         0x1B
#define MB86667_REG_DIB4         0x1C
#define MB86667_REG_DIB5         0x1D
#define MB86667_REG_DIEN         0x1E
#define MB86667_REG_TONE2        0x20
  #define MB86667_BIT_TONE2_TMD_TONE  0x00; /* bit definition to switch output to tone function */
  #define MB86667_BIT_TONE2_TMD_DISEQ 0x04; /* bit definition to switch output to DiSEqC function */
  #define MB86667_BIT_TONE2_TMD_LOW   0x08; /* bit definition to switch output to low */
  #define MB86667_BIT_TONE2_TMD_HIGH  0x0C; /* bit definition to switch output to high */
#define MB86667_REG_TUSET        0x21
#define MB86667_REG_FRQ1         0x22
#define MB86667_REG_FRQ2         0x23
#define MB86667_REG_FRQ3         0x24
#define MB86667_REG_I2C2         0x25
#define MB86667_REG_CN_MONITOR   0x26
#define MB86667_REG_PORT0        0x27
#define MB86667_REG_PORT1        0x28
#define MB86667_REG_AFC_SET      0x2A
    #define SHIFT_AFCA_0_2           0x02
    #define SHIFT_AFCEXEN            0x05
#define MB86667_REG_AFCIL        0x2B
#define MB86667_REG_AFCIH        0x2C
#define MB86667_REG_DIGITAL_AGC  0x2D
    #define SHIFT_DAGCA_0_1          0x03
#define MB86667_REG_STR_WIN1_2    0x30
#define MB86667_REG_STR_WIN2_2    0x31
#define MB86667_REG_CR_WIN1_2    0x2F
#define MB86667_REG_SEQ_MODE_SET 0x32
    #define IQ_DET_BY_FRAME_SYNC    0x01
    #define SLOCK1                  0x02
    #define SHIFT_SLOCK1            0x02
#define MB86667_REG_S0_S1_TIM    0x33
    #define SHIFT_S1T_0_3            0x03
#define MB86667_REG_S2_S3_TIM    0x34
#define MB86667_REG_S4_S5_TIM    0x35
    #define SHIFT_S5T_0_4            0x04
#define MB86667_REG_CN_TIMER     0x36
    #define SHIFT_TIMINT1_0_1        0x04
    #define SHIFT_TIMINT2_0_1        0x02
#define MB86667_REG_SIG_1        0x37
#define MB86667_REG_SIG_2        0x38
#define MB86667_REG_VIT_GAIN     0x39
#define MB86667_REG_VIT_ERR_OFFS 0x3B
#define MB86667_REG_VIT_PROT_VAL 0x3C
#define MB86667_REG_FRONTAGC1    0x3D
#define MB86667_REG_FRONTAGC2    0x3E
#define MB86667_REG_FRONTAGC3    0x3F
#define MB86667_REG_FRONTAGC4    0x40
#define MB86667_REG_FRONTAGC5    0x41
#define MB86667_REG_CRM          0x43
#define MB86667_REG_STRM         0x44
#define MB86667_REG_AGCI         0x45
#define MB86667_REG_AGCML        0x45
#define MB86667_REG_AGCMH        0x46
#define MB86667_REG_THROUGH_MODE 0x49
    #define MB86667_BIT_DRCEN       0x02
    #define MB86667_BIT_AFCEN       0x04
    #define MB86667_BIT_INTPEN      0x08
    #define MB86667_BIT_DECIEN      0x10
    #define MB86667_BIT_DAGCEN      0x20
    #define MB86667_BIT_NFEN        0x40
    #define MB86667_BIT_CREN        0x80
    #define MB86667_BIT_SHIFT_AFCEN 0x02
    #define MB86667_BIT_SHIFT_CREN  0x07

#define MB86667_REG_STR_F_MIN    0x50
#define MB86667_REG_STR_F_MAX    0x51
#define MB86667_REG_SIG_DIST     0x52
#define MB86667_REG_SEQ_STATE    0x53
#define MB86667_REG_AFCTH_1_L    0x54
#define MB86667_REG_AFCCOFS_L    0x58
#define MB86667_REG_AFCCOFS_H    0x59
#define MB86667_REG_AFC_3_0      0x5E
#define MB86667_REG_EQL          0x61
    #define SHIFT_ESHIFT_0_3         0x02

#define MB86667_REG_CLOCK        0x6C
#define MB86667_REG_VER          0x7F

/* bit definitions */
#define MB86667_BIT_GPIO   0x30
#ifdef ORIGINAL_FUJ_EVALBOARD
   #define MB86667_BIT_LNB18V 0x04
   #define MB86667_BIT_LNB13V 0x00
   #define MB86667_BIT_LNB    0x01
   #define MB86667_BIT_LNBOFF 0x01
   #define MB86667_BIT_LNBON  0x00
#else
   #define MB86667_BIT_LNB18V 0x01
   #define MB86667_BIT_LNB13V 0x00
   #define MB86667_BIT_LNB    0x04
   #define MB86667_BIT_LNBOFF 0x04
   #define MB86667_BIT_LNBON  0x00
#endif


/* ------- bit values ---------------------------------------------- */


#endif  /* MB86667_REGS_H */
