/*******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Tuner.c
**
** \brief     RF functions of MB86667 driver.
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   $Id: MB86667_Tuner.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/

#include "MB86667_Api.h"
#include "MB86667_Customise.h"
#include "MB86667_Tuner.h"
#include "fapex/abst_types.h"

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

#ifdef  __cplusplus
extern "C" {
#endif


#if (defined (TUNER_SHARP_BS2S7HZ1402) || defined (TUNER_SHARP_BS2F7VZ0302))
/*----------------------------------------------------------------------------
Name:        MB86667_Rf_Val_Set_Ini_S7HZ1402  (Tuner_Sharp)
             used for SHARP BS2F7VZ0302 and SHARP BS2S7HZ1402
Description: Set RF chip Initializtion
-------------------------------------------------------------------------------*/
int32_t MB86667_Rf_Val_Set_Ini_S7HZ1402(uint32_t f10, uint16_t smrt)
{
  uint8_t  TU_adr = 0xc0;
  uint8_t  ucExitTry = 20;
  uint32_t N, A ;
  uint8_t  tu_val[5] ;
  uint8_t  pd5, pd4, pd3, pd2 ;
  uint8_t  pd ;
  uint8_t  BA, DIV ;
  uint8_t  TM ;
  uint8_t  Ref=1,PSC=0;

  N = f10 / 160 ;

  A = f10*2/10 - 32*N ;

  if(      f10<10650 ) { DIV=1 ; BA=6 ; }
  else if( f10<11700 ) { DIV=1 ; BA=7 ; }
  else if( f10<13000 ) { DIV=0 ; BA=1 ; }
  else if( f10<14450 ) { DIV=0 ; BA=2 ; }
  else if( f10<16070 ) { DIV=0 ; BA=3 ; }
  else if( f10<17780 ) { DIV=0 ; BA=4 ; }
  else if( f10<19420 ) { DIV=0 ; BA=5 ; }
  else                 { DIV=0 ; BA=6 ; }

  tu_val[0] = TU_adr ;
  tu_val[1] = (uint8_t)(N >> 3) ;
  tu_val[2] = (uint8_t)(((N & 0x0007) << 5) + A );


  TM = 0 ;
  pd5 = 0 ;
  pd4 = 0 ;
  pd3 = 0 ;
  pd2 = 0 ;
  tu_val[3] = (uint8_t)(0xe0 | (pd5<<4) | (pd4<<3) | (TM<<2) | (Ref<<0));    // Byte 4
  tu_val[4] = (uint8_t)(0x00 | (BA<<5) | (pd3<<3) | (pd2<<2) | (DIV<<1) | (PSC<<4));    // Byte 5

  //Tuner setting 1
  if (MB86667_I2C_Bypass_Write(tu_val, 5) != FE_OK)
  {
FAPI_SYS_PRINT_MSG("MB86667_I2C_Bypass_Write failed\n");
    return FE_ERR_FE_I2C;
  }


  //tuner setting 2
  TM = 1 ;
  pd5 = 0 ;
  pd4 = 0 ;
  tu_val[1] = (uint8_t)(0xe0 | (pd5<<4) | (pd4<<3) | (TM<<2) | (Ref<<0)); // byte 4

  if (MB86667_I2C_Bypass_Write(tu_val, 2) != FE_OK)
  {
FAPI_SYS_PRINT_MSG("MB86667_I2C_Bypass_Write failed\n");
    return FE_ERR_FE_I2C;
  }

  if (MB86667_User_Sleep( 6 ) != FE_OK)  //wait 6ms
    return FE_ERR_FE_I2C;

  //tuner setting 3
  TM = 1 ;
  if(      smrt>40500 ) pd = 13 ;
  else if( smrt>39000 ) pd = 12 ;
  else if( smrt>33250 ) pd = 11 ;
  else if( smrt>31500 ) pd = 10 ;
  else if( smrt>30250 ) pd =  9 ;
  else if( smrt>26500 ) pd =  8 ;
  else if( smrt>25250 ) pd =  7 ;
  else if( smrt>20750 ) pd =  6 ;
  else if( smrt>16500 ) pd =  5 ;
  else if( smrt>15500 ) pd =  4 ;
  else                  pd =  3 ;


  if(      pd== 3 ) { pd2=0 ; pd3=0 ; pd4=1 ; pd5=1 ; }  //fc=10.0MHz
  else if( pd== 4 ) { pd2=0 ; pd3=1 ; pd4=0 ; pd5=0 ; }  //fc=12.0MHz
  else if( pd== 5 ) { pd2=0 ; pd3=1 ; pd4=0 ; pd5=1 ; }  //fc=14.0MHz
  else if( pd== 6 ) { pd2=0 ; pd3=1 ; pd4=1 ; pd5=0 ; }  //fc=16.0MHz
  else if( pd== 7 ) { pd2=0 ; pd3=1 ; pd4=1 ; pd5=1 ; }  //fc=18.0MHz
  else if( pd== 8 ) { pd2=1 ; pd3=0 ; pd4=0 ; pd5=0 ; }  //fc=20.0MHz
  else if( pd== 9 ) { pd2=1 ; pd3=0 ; pd4=0 ; pd5=1 ; }  //fc=22.0MHz
  else if( pd==10 ) { pd2=1 ; pd3=0 ; pd4=1 ; pd5=0 ; }  //fc=24.0MHz
  else if( pd==11 ) { pd2=1 ; pd3=0 ; pd4=1 ; pd5=1 ; }  //fc=26.0MHz
  else if( pd==12 ) { pd2=1 ; pd3=1 ; pd4=0 ; pd5=0 ; }  //fc=28.0MHz
  else if( pd==13 ) { pd2=1 ; pd3=1 ; pd4=0 ; pd5=1 ; }  //fc=30.0MHz
  tu_val[1] = (uint8_t)(0xe0 | (pd5<<4) | (pd4<<3) | (TM<<2) | (Ref<<0));  // byte 4
  tu_val[2] = (uint8_t)(0x00 | (BA<<5) | (PSC<<4) | (pd3<<3) | (pd2<<2) | (DIV<<1)) ; // byte 5

  if (MB86667_I2C_Bypass_Write(tu_val, 3) != FE_OK)
  {
FAPI_SYS_PRINT_MSG("MB86667_I2C_Bypass_Write failed\n");
    return FE_ERR_FE_I2C;
  }

  /* tuner lock check */
  do
  {
     uint8_t val;
     if(MB86667_I2C_Bypass_Read(255,&val)!=FE_OK)
     {
FAPI_SYS_PRINT_MSG("MB86667_I2C_Bypass_Read failed\n");
       return FE_ERR_FE_I2C;
     }
     if((val&0x40)!=0) /* locked */
     {
        #ifdef FAPI_FE_MB86667_DEBUG01
           MB86667_User_Print("Tuner locked\n");
        #endif
        return FE_OK;
     }
     MB86667_User_Sleep(10); /* wait 10 msec and test again */
     ucExitTry--;
  }
  while (ucExitTry > 0);

  #ifdef FAPI_FE_MB86667_DEBUG01
     MB86667_User_Print("Tuner not locked\n");
  #endif
FAPI_SYS_PRINT_MSG("Tuner not locked\n");
  return FE_ERR_FE_OTHER;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Rf_Val_Set  (Tuner_Sharp)
Description: Set RF chip again
-------------------------------------------------------------------------------*/
int32_t MB86667_Rf_Val_Set_S7HZ1402(uint32_t f10, uint16_t smrt)
{
  uint8_t  TU_adr = 0xc0;
  uint8_t  ucExitTry = 20;
  uint8_t  tu_val[5] ;
  uint8_t  pd5, pd4, pd3, pd2 ;
  uint8_t  pd ;
  uint32_t N, A ;

  uint8_t  BA, DIV ;
  uint8_t  TM ;
  uint8_t  Ref=1,PSC=0;

  N = f10 / 160 ;

  A = f10*2/10 - 32*N ;

  if(      f10<10650 ) { DIV=1 ; BA=6 ; }
  else if( f10<11700 ) { DIV=1 ; BA=7 ; }
  else if( f10<13000 ) { DIV=0 ; BA=1 ; }
  else if( f10<14450 ) { DIV=0 ; BA=2 ; }
  else if( f10<16070 ) { DIV=0 ; BA=3 ; }
  else if( f10<17780 ) { DIV=0 ; BA=4 ; }
  else if( f10<19420 ) { DIV=0 ; BA=5 ; }
  else                 { DIV=0 ; BA=6 ; }

  tu_val[0] = TU_adr ;
  tu_val[1] = (uint8_t)(N>>3) ;
  tu_val[2] = (uint8_t)(((N&0x0007)<<5) + A) ;

  TM = 1 ;
  if(      smrt>40500 ) pd = 13 ;
  else if( smrt>39000 ) pd = 12 ;
  else if( smrt>33250 ) pd = 11 ;
  else if( smrt>31500 ) pd = 10 ;
  else if( smrt>30250 ) pd =  9 ;
  else if( smrt>26500 ) pd =  8 ;
  else if( smrt>25250 ) pd =  7 ;
  else if( smrt>20750 ) pd =  6 ;
  else if( smrt>16500 ) pd =  5 ;
  else if( smrt>15500 ) pd =  4 ;
  else                  pd =  3 ;

  if(      pd== 3 ) { pd2=0 ; pd3=0 ; pd4=1 ; pd5=1 ; }  //fc=10.0MHz
  else if( pd== 4 ) { pd2=0 ; pd3=1 ; pd4=0 ; pd5=0 ; }  //fc=12.0MHz
  else if( pd== 5 ) { pd2=0 ; pd3=1 ; pd4=0 ; pd5=1 ; }  //fc=14.0MHz
  else if( pd== 6 ) { pd2=0 ; pd3=1 ; pd4=1 ; pd5=0 ; }  //fc=16.0MHz
  else if( pd== 7 ) { pd2=0 ; pd3=1 ; pd4=1 ; pd5=1 ; }  //fc=18.0MHz
  else if( pd== 8 ) { pd2=1 ; pd3=0 ; pd4=0 ; pd5=0 ; }  //fc=20.0MHz
  else if( pd== 9 ) { pd2=1 ; pd3=0 ; pd4=0 ; pd5=1 ; }  //fc=22.0MHz
  else if( pd==10 ) { pd2=1 ; pd3=0 ; pd4=1 ; pd5=0 ; }  //fc=24.0MHz
  else if( pd==11 ) { pd2=1 ; pd3=0 ; pd4=1 ; pd5=1 ; }  //fc=26.0MHz
  else if( pd==12 ) { pd2=1 ; pd3=1 ; pd4=0 ; pd5=0 ; }  //fc=28.0MHz
  else if( pd==13 ) { pd2=1 ; pd3=1 ; pd4=0 ; pd5=1 ; }  //fc=30.0MHz
  else { pd2=1 ; pd3=1 ; pd4=0 ; pd5=1 ; }  //default is highest cut off freq.

  tu_val[3] = (uint8_t)(0xe0 | (pd5<<4) | (pd4<<3) | (TM<<2) | (Ref<<0));
  tu_val[4] = (uint8_t)(0x00 | (BA<<5) | (PSC<<4) | (pd3<<3) | (pd2<<2) | (DIV<<1));

  //Tuner setting 1

  if (MB86667_I2C_Bypass_Write(tu_val, 5) != FE_OK)
  {
FAPI_SYS_PRINT_MSG("MB86667_I2C_Bypass_Write failed\n");
    return FE_ERR_FE_I2C;
  }
  /* tuner lock check */
  do
  {
     uint8_t val;
     if(MB86667_I2C_Bypass_Read(255,&val)!=FE_OK)
     {
FAPI_SYS_PRINT_MSG("MB86667_I2C_Bypass_Read failed\n");
       return FE_ERR_FE_I2C;
     }
     if((val&0x40)!=0) /* locked */
     {
        #ifdef FAPI_FE_MB86667_DEBUG01
           MB86667_User_Print("Tuner locked\n");
        #endif
        return FE_OK;
     }
     MB86667_User_Sleep(10); /* wait 10 msec and test again */
     ucExitTry--;
  }
  while (ucExitTry>0);

  #ifdef FAPI_FE_MB86667_DEBUG01
     MB86667_User_Print("Tuner not locked\n");
  #endif
FAPI_SYS_PRINT_MSG("Tuner not locked\n");
  return FE_ERR_FE_OTHER;
}
#endif

#ifdef TUNER_MAXIM_2118
/*----------------------------------------------------------------------------
Name:        MB86667_Rf_Val_Set_Ini_MAX2118  (for MAXIM_2118)
Description: Set RF chip Initializtion
-------------------------------------------------------------------------------*/
int32_t MB86667_Rf_Val_Set_Ini_MAX2118(uint32_t f10, uint16_t smrt)
{
  uint8_t tu_val[7];
  uint8_t DIV   = 0;    /* VCO frequency divider */
  uint8_t R     = 0x02;  /* Reference divider bits 2^( 4*R(2)+2*R(1)+R(0) ) dafault 0x01 -> 4 */
  uint8_t CP    = 0x03;  /* Charge pump current  dafault 0x03 -> 400uA */
  uint8_t VCO   = 0x05;  /* VCO selection default 0x05 ->  */
  uint8_t FDAC  = 0x7F;    /* I/Q Filter DAC -> 2^7xf(7)+2^6*F(6)+...+2^0F(0) dafault 0x7F = 127 */
  uint8_t ADL   = 0;    /* VCO ADC latch-enable bit */
  uint8_t ADE   = 0;    /* VCO tuner voltage DAC read */
  uint8_t DL    = 1;    /* Set differential output drive level for MAX2118 */
  uint8_t M     = 0x02;  /* the value of M Counter of lowpass filter BW control */
  uint8_t D     = 0;    /* control diagnostci feature */
  /* 041118 Change Tuner Base Band Gain : 0x00 --> 0x07 */
  uint8_t G     = 0x07;    /* controls the gain of the baseand VGA */
  /* controls the gain of the baseand VGA */
  uint8_t ADC   = 0x00;
  //unsigned char PRE_AGC=0x00;  //PreAgc Value
  uint32_t Fsym_lpf=0;
  uint32_t N14;    //Register Byte0~1 [N14~N0] Value
  uint32_t Freq = f10/10;
  uint8_t tunerstatus = 0;

  N14=Freq*2;
  Fsym_lpf = (S32)smrt*100*3/4;  //CutoffFrequency *(2/3) = Symbolrate/2

  if         (Fsym_lpf/100000>29) M=2;
  else if    (Fsym_lpf/100000>22) M=3;
  else if    (Fsym_lpf/100000>17) M=4;
  else if    (Fsym_lpf/100000>14) M=5;
  else if    (Fsym_lpf/100000>12) M=6;
  else if    (Fsym_lpf/100000>10) M=7;
  else if    (Fsym_lpf/100000>8)  M=8;
  else if    (Fsym_lpf/100000>6)  M=11;
  else if    (Fsym_lpf/100000>4)  M=14;
  else                            M=22;

  FDAC=((uint8_t)((Fsym_lpf*M-1600000)/58000+1))&0x7F;  //F(-3dB)=Xtal/M*(4+0.145*FDAC)

  if ( ( 931   < Freq) && (Freq<=1035   ) ) {DIV = 0;VCO = 5;};
  if ( ( 1035  < Freq) && (Freq<=1125   ) ) {DIV = 0;VCO = 6;};
  if ( ( 1125  < Freq) && (Freq<=1216   ) ) {DIV = 1;VCO = 0;};
  if ( ( 1216  < Freq) && (Freq<=1355   ) ) {DIV = 1;VCO = 1;};
  if ( ( 1355  < Freq) && (Freq<=1512   ) ) {DIV = 1;VCO = 2;};
  if ( ( 1512  < Freq) && (Freq<=1670   ) ) {DIV = 1;VCO = 3;};
  if ( ( 1670  < Freq) && (Freq<=1863   ) ) {DIV = 1;VCO = 4;};
  if ( ( 1863  < Freq) && (Freq<=2071   ) ) {DIV = 1;VCO = 5;};
  if ( ( 2071  < Freq) && (Freq<=2246   ) ) {DIV = 1;VCO = 6;};

  /* tuner bypass setting */
  /* Tuner setting 1st */
    tu_val[0] =  0x00;
    tu_val[1] = (DIV<<7) |(((uint32_t)N14)>>8);
    tu_val[2] = ((uint32_t)(N14))&0xff;
    tu_val[3] = (R<<5) |(3<<3)|VCO;
    tu_val[4] =  FDAC;
    tu_val[5] = (ADL<<7) | (ADE<<6) | (DL<<5) | M;
    tu_val[6] = (D<<5) | G;
    if (MB86667_I2C_Bypass_Write(tu_val, 7) != FE_OK)
      return FE_ERR_FE_I2C;

  /* tuner bypass setting */
  uint8_t reg = 0x02;
  if (MB86667_I2C_Bypass_Read(reg, &tunerstatus) != FE_OK)
    return FE_ERR_FE_I2C;

  /* tuner bypass setting */
  /* Tuner setting 2
     Charge Pump Current Select */
  ADC=(tunerstatus>>2)&0x07;
  if(     (ADC==0x01)||(ADC==0x02))  CP=0x01;
  else if ((ADC==0x03)||(ADC==0x04)) CP=0x02;
  else                               CP=0x03;
  uint8_t tu_val2[2];
  tu_val2[0] = reg;
  tu_val2[1] = (tu_val[3]&0xe7) | (CP<<3);
  if (MB86667_I2C_Bypass_Write(tu_val2, 2) != FE_OK)
    return FE_ERR_FE_I2C;
  return FE_OK;
}

/*----------------------------------------------------------------------------
Name:        MB86667_Rf_Val_Set (for MAXIM_2118)
Description: Set RF chip again
-------------------------------------------------------------------------------*/
int32_t MB86667_Rf_Val_Set_MAX2118(uint32_t f10, uint16_t smrt)
{
  uint8_t tu_val[7];
  uint8_t DIV   = 0;    /* VCO frequency divider */
  uint8_t R     = 0x02;  /* Reference divider bits 2^( 4*R(2)+2*R(1)+R(0) ) dafault 0x01 -> 4 */
  uint8_t CP    = 0x03;  /* Charge pump current  dafault 0x03 -> 400uA */
  uint8_t VCO   = 0x05;  /* VCO selection default 0x05 ->  */
  uint8_t FDAC  = 0x7F;  /* I/Q Filter DAC -> 2^7xf(7)+2^6*F(6)+...+2^0F(0) dafault 0x7F = 127 */
  uint8_t ADL   = 0;    /* VCO ADC latch-enable bit */
  uint8_t ADE   = 0;    /* VCO tuner voltage DAC read */
  uint8_t DL    = 1;    /* Set differential output drive level for MAX2118 */
  uint8_t M     = 0x02;  /* the value of M Counter of lowpass filter BW control */
  uint8_t D     = 0;    /* control diagnostci feature */
  /* 041118 Change Tuner Base Band Gain : 0x00 --> 0x07 */
  uint8_t G     = 0x07;    /* controls the gain of the baseand VGA */
  /* controls the gain of the baseand VGA */
  uint8_t ADC   = 0x00;
  //unsigned char PRE_AGC=0x00;  //PreAgc Value
  uint32_t Fsym_lpf=0;
  uint32_t N14;    //Register Byte0~1 [N14~N0] Value

  uint32_t Freq = f10/10;
  uint8_t tunerstatus = 0;

  N14=Freq*2;               //N14~N0 = Freq*(1/Step)
  Fsym_lpf = (S32)smrt*100*3/4;  //CutoffFrequency *(2/3) = Symbolrate/2

  if         (Fsym_lpf/100000>29) M=2;
  else if    (Fsym_lpf/100000>22) M=3;
  else if    (Fsym_lpf/100000>17) M=4;
  else if    (Fsym_lpf/100000>14) M=5;
  else if    (Fsym_lpf/100000>12) M=6;
  else if    (Fsym_lpf/100000>10) M=7;
  else if    (Fsym_lpf/100000>8)  M=8;
  else if    (Fsym_lpf/100000>6)  M=11;
  else if    (Fsym_lpf/100000>4)  M=14;
  else                            M=22;

  FDAC=((uint8_t)((Fsym_lpf*M-1600000)/58000+1))&0x7F;  //F(-3dB)=Xtal/M*(4+0.145*FDAC)

     if ( ( 931   < Freq) && (Freq<=1035   ) ) {DIV = 0;VCO = 5;};
  if ( ( 1035  < Freq) && (Freq<=1125   ) ) {DIV = 0;VCO = 6;};
  if ( ( 1125  < Freq) && (Freq<=1216   ) ) {DIV = 1;VCO = 0;};
  if ( ( 1216  < Freq) && (Freq<=1355   ) ) {DIV = 1;VCO = 1;};
  if ( ( 1355  < Freq) && (Freq<=1512   ) ) {DIV = 1;VCO = 2;};
  if ( ( 1512  < Freq) && (Freq<=1670   ) ) {DIV = 1;VCO = 3;};
  if ( ( 1670  < Freq) && (Freq<=1863   ) ) {DIV = 1;VCO = 4;};
  if ( ( 1863  < Freq) && (Freq<=2071   ) ) {DIV = 1;VCO = 5;};
  if ( ( 2071  < Freq) && (Freq<=2246   ) ) {DIV = 1;VCO = 6;};

  /* tuner bypass setting */
  /* Tuner setting 1st */
  tu_val[0] = 0x00;
  tu_val[1] = (DIV<<7) |(((uint32_t)N14)>>8);
  tu_val[2] = ((uint32_t)(N14))&0xff;
  tu_val[3] = (R<<5) | (3<<3) | VCO;
  tu_val[4] =  FDAC;
  tu_val[5] = (ADL<<7) | (ADE<<6) | (DL<<5) | M;
  tu_val[6] = (D<<5) | G;
  if (MB86667_I2C_Bypass_Write(tu_val, 6) != FE_OK)
    return FE_ERR_FE_I2C;

  /* tuner bypass setting */
  /*Tuner setting 2
   Charge Pump Current Select */
  ADC=(tunerstatus>>2)&0x07;
  if(     (ADC==0x01)||(ADC==0x02))  CP=0x01;
  else if ((ADC==0x03)||(ADC==0x04)) CP=0x02;
  else                               CP=0x03;
  uint8_t tu_val2[2];
  tu_val2[0] = 0x02;
  tu_val2[1] = (tu_val[3]&0xe7) | (CP<<3); //Sean
  if (MB86667_I2C_Bypass_Write(tu_val2, 2) != FE_OK)
    return FE_ERR_FE_I2C;
  return FE_OK;
}
#endif

#ifdef  __cplusplus
}
#endif

/* Version History */
/*****************************************************************************

$Log: MB86667_Tuner.c,v $
Revision 1.1  2008/07/11 13:36:32  sgehre
- DSF89000 activated by default
- added support for MB86667 DVB-S frontend
- use SUBSYS_FE_MODULE_BASE for errors

Revision 1.3  2008/02/24 21:11:52  kraft
version Fuj 4.3 implemented

Revision 1.2  2007/11/19 09:11:32  kraft
fuj sat fe with blind scan

Revision 1.3  2007/08/29 11:47:14  skau
Supporting confidential files and front end detection now.

Revision 1.2  2007/07/04 10:40:45  skau
Finally made compatible to the windows monitor software.

Revision 1.1  2007/06/25 12:26:50  skau
New file structure.

Revision 1.3  2007/01/24 10:08:00  skau
Compiles without warnings, all CVS headers updated and
ready for first customer release.

Revision 1.2  2006/11/30 09:50:19  skau
First working version.

Revision 1.1  2006/11/03 15:11:02  skau
Driver for MB86667.This is the first (non-working) version.


*****************************************************************************/
