/******************************************************************************
**
** \file      ./subsystems/fe/mb86668/src/mb86668_demod.h
**
** \brief     Demodulator driver functions for MB86668.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86668_demod.h 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/

#ifndef _FD_MB86668_DEMOD_H_
#define _FD_MB86668_DEMOD_H_

/* for previous lock (during scan)*/
#define NO_LOCK           0
#define LOCK_NO_OFF       1
#define LOCK_RI_OFF       2
#define LOCK_LE_OFF       3

typedef enum
{
        QAM16,
        QAM32,
        QAM64,
        QAM128,
        QAM256,
} FE_86668_QamVal_t;


typedef struct
{
        uint8_t                  Status;
    uint32_t                 Agc_val;/* Agc1 on MSB */
        uint32_t                 Ber;
        uint16_t                 Cn;
} FE_86668_SearchResult_t;

typedef struct
{
    uint32_t                 Frequency;  // in kHz
    FE_86668_QamVal_t   Qam;
    uint32_t                 SymbolRate; // in kHz
    uint32_t                 Clock; // in kHz
    uint32_t                 If_frequency; // in kHz
    uint32_t                 D_decimation; // from symbolRateSet
    uint32_t                 S_intpo_nco; // from symbolRateSet
    uint8_t                  OutputSettingsVal;
}FE_86668_InternalParams_t;

typedef struct
{
    uint32_t                 Frequency;  // in kHz
    FE_86668_QamVal_t   Qam;
    uint32_t                 SymbolRate; // in kHz
} FE_86668_SearchParams_t;

// accessable demodulator types
typedef enum
{
   NONE_DEM=0,
   MB86668,
} DEM_types;

typedef struct
{
   DEM_types Type;
   uint8_t I2cAddress;
   uint8_t WrSize;
   uint8_t WrBuffer[8];
   uint8_t RdSize;
   uint8_t RdBuffer[8];

// demod private data
   FE_86668_InternalParams_t  params;
   FE_86668_SearchResult_t    result;
} DEM_device;

// error coding in demod
#define DEM_ERR_BASE -2000

#define CARRIER_RECOVERY_LIMITER_FREQ (170*1)  //in kHz       170
#define SYMBOL_TIMING_RECOVERY_LIMITER_FREQ 200  //in ppm  200
#define USED_DEMOD_CLOCK 28800 // in kHz     28800

typedef enum
{
  DEM_I2C_ERR =    DEM_ERR_BASE -1,
  DEM_WRONG_PARAMETER = DEM_ERR_BASE -2,
  DEM_TUNING_ERROR = DEM_ERR_BASE -3,
  DEM_NOT_LOCKED = DEM_ERR_BASE -4,
  DEM_SEARCH_FAILED = DEM_ERR_BASE -5,
  DEM_NOT_FOUND = DEM_ERR_BASE -6,
} DEM_err;

#endif // _FD_MB86668_DEMOD_H_

