/******************************************************************************
**
** \file      ./subsystems/fe/mb86668/src/mb86668_fe.h
**
** \brief     FE Api declarations.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86668_fe.h 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/

#ifndef _FD_MB86668_FE_H_
#define _FD_MB86668_FE_H_


//-------------------FE API-------------------
typedef enum
{
   DVB_S = 0,
   DVB_C,
   DVB_T
} fe_type_t;

typedef enum
{
   NO_DEMOD = 0,
   C_MB86668,
} demod_type_t;

typedef enum
{
   NO_TUNER = 0,
   C_CD1316L_3,
   C_CD1600L_3,
   C_DCT70602,
   C_DCT70701,
   LAST_TUNER,
} tuner_type_t;

typedef enum
{
   NO_LNB = 0,
} lnb_type_t;

typedef struct
{
   fe_type_t    devicetype;
   uint8_t           demod_i2c_adr;
   demod_type_t demod_type;
   uint8_t           tuner_i2c_adr;
   tuner_type_t tuner_type;
   uint8_t           lnb_i2c_adr;
   lnb_type_t   lnb_type;
   int8_t           ghwIndex;
   void *       internal_data;     // here is the encapsulated device handle with private date
}  FE_Device;

typedef enum fe_modulation
{
   QPSK,
   QAM_16,
   QAM_32,
   QAM_64,
   QAM_128,
   QAM_256,
   QAM_AUTO
} fe_modulation_t;

struct dvb_qam_parameters
{
   uint32_t symbol_rate; /* symbol rate in Symbols per second */
   fe_modulation_t modulation; /* modulation type (see above) */
};

typedef enum
{
   DEFAULT_PRIORITY=0,
   HIGH_PRIORITY,
   LOW_PRIORITY
} fe_tps_t;

typedef enum fe_spectral_inversion
{
   INVERSION_OFF,
   INVERSION_ON,
   INVERSION_AUTO
} fe_spectral_inversion_t;

struct dvb_frontend_parameters
{
   uint32_t frequency; /* (absolute) frequency in Hz for QAM/OFDM */
                       /* intermediate frequency in kHz for QPSK */
   fe_spectral_inversion_t inversion;
   struct dvb_qam_parameters qam;
};

typedef struct dvb_frontend_parameters T_TuningParam;


/* DVB cable api */
FE_Device *FEC_Create(demod_type_t dem_type, uint8_t dem_i2c,
                      tuner_type_t tun_type, uint8_t tun_i2c);
void FEC_Destroy(FE_Device *handle);
int32_t FEC_TestDevice(FE_Device * handle);
int32_t FEC_Init( FE_Device *handle);
int32_t FEC_Tune( FE_Device *handle, T_TuningParam *param);
int32_t FEC_GetStatus( FE_Device *handle, uint8_t *status);
int32_t FEC_GetAgc( FE_Device *handle, uint8_t *agcVal);
int32_t FEC_GetCN( FE_Device *handle, uint16_t *cnVal);
int32_t FEC_GetBer(FE_Device *handle, uint32_t *ber);
int32_t FEC_GetTuningParam(FE_Device *handle, T_TuningParam *param);
int32_t   FEC_Reset(feHandleT *handle);

#endif // _FD_MB86668_FE_H_



