/*
*****************************************************************************
**
** \file        ./subsystems/paint/draw/src/draw_scale.c
**
** \version     $Id: draw_scale.c 4984 2009-10-14 08:45:46Z rseng $
**
** \brief       DRAW Scale Driver
**
** This file contains the implementation of the DRAW driver's core API for
** picture scaling.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2008-2009 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2009      by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/


#include "draw.h"
#include "draw_priv.h"
#include <setjmp.h>

#if DRAW_ENABLE_RESIZE

#include <osdhandler/osdhandler.h>
#include <fapex/abst_types.h>

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************
#define DRAW_MIN_TRANSPARENCY 0x10

/* Set the precision level here. (more precise is slower and vice-versa) */
#define CALCULATION_DOUBLE_PRECISION 0

#if CALCULATION_DOUBLE_PRECISION
    /* Set to "float" or "double" depending on desired precision. */
    typedef double CALCULATION_TYPE;
#else
    typedef float  CALCULATION_TYPE;
#endif

//***************************************************************************
//***************************************************************************
//** Local Structures
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************
static uint8_t transparentColour = 0;

//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************

    static uint32_t  drawFloor          (CALCULATION_TYPE x);
    static uint32_t  drawSquared        (uint32_t x);
    static uint8_t   drawGetPixelAlpha  (uint32_t *bitmapData,
                                         uint32_t  bitmapWidth,
                                         uint32_t  xVal,
                                         uint32_t  yVal);
    static uint8_t   drawGetPixelRed    (uint32_t *bitmapData,
                                         uint32_t  bitmapWidth,
                                         uint32_t  xVal,
                                         uint32_t  yVal);
    static uint8_t   drawGetPixelGreen  (uint32_t *bitmapData,
                                         uint32_t  bitmapWidth,
                                         uint32_t  xVal,
                                         uint32_t  yVal);
    static uint8_t   drawGetPixelBlue   (uint32_t *bitmapData,
                                         uint32_t  bitmapWidth,
                                         uint32_t  xVal,
                                         uint32_t  yVal);
    static uint32_t  drawGetPixelAll    (uint32_t *bitmapData,
                                         uint32_t  bitmapWidth,
                                         uint32_t  xVal,
                                         uint32_t  yVal);
    /* Returns the pointer within the bitmap data, instead of the actual value. */
    static uint32_t* drawGetPixelPtr    (uint32_t *bitmapData,
                                         uint32_t  bitmapWidth,
                                         uint32_t  xVal,
                                         uint32_t  yVal);
    static uint32_t drawGetClutValue    (const DRAW_PicItemT* picPtr,
                                         uint8_t ClutIdx,
                                         uint32_t* clutToUse);
    static uint8_t  drawGetBestColor    (uint32_t  colorValue,
                                         const uint32_t* clutPtr,
                                         uint8_t clutStart,
                                         uint8_t clutEnd);

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************


#define GET_RED(pixelval)       ((uint8_t)(((pixelval) & 0xFF000000) >> 24))
#define GET_GREEN(pixelval)     ((uint8_t)(((pixelval) & 0x00FF0000) >> 16))
#define GET_BLUE(pixelval)      ((uint8_t)(((pixelval) & 0x0000FF00) >> 8))
#define GET_ALPHA(pixelval)     ((uint8_t)((pixelval) & 0x000000FF))


static uint32_t drawFloor (CALCULATION_TYPE x)
{
    uint32_t val;

    val = (uint32_t) x;

    return val;
}

static uint32_t drawSquared (uint32_t x)
{
    return (x * x);
}


static uint8_t drawGetPixelAlpha(uint32_t* bitmapData,
                      uint32_t  bitmapWidth,
                      uint32_t  xVal,
                      uint32_t  yVal)
{
    uint32_t    pixel_val;
    uint8_t     final;

    if (bitmapData == NULL) return 0;

    pixel_val = (uint32_t)*(bitmapData + ((yVal * bitmapWidth) + xVal));
    final = GET_ALPHA(pixel_val);

    return final;
}

static uint8_t drawGetPixelRed(uint32_t* bitmapData,
                           uint32_t  bitmapWidth,
                           uint32_t  xVal,
                           uint32_t  yVal)
{
    uint32_t    pixel_val;
    uint8_t     final;

    if (bitmapData == NULL) return 0;

    pixel_val = (uint32_t)*(bitmapData + ((yVal * bitmapWidth) + xVal));
    final = GET_RED(pixel_val);

    return final;
}

static uint8_t drawGetPixelGreen(uint32_t* bitmapData,
                             uint32_t  bitmapWidth,
                             uint32_t  xVal,
                             uint32_t  yVal)
{
    uint32_t    pixel_val;
    uint8_t     final;

    if (bitmapData == NULL) return 0;

    pixel_val = (uint32_t)*(bitmapData + ((yVal * bitmapWidth) + xVal));
    final = GET_GREEN(pixel_val);

    return final;
}

static uint8_t drawGetPixelBlue(uint32_t* bitmapData,
                            uint32_t  bitmapWidth,
                            uint32_t  xVal,
                            uint32_t  yVal)
{
    uint32_t    pixel_val;
    uint8_t     final;

    if (bitmapData == NULL) return 0;

    pixel_val = (uint32_t)*(bitmapData + ((yVal * bitmapWidth) + xVal));
    final = GET_BLUE(pixel_val);

    return final;
}

static uint32_t drawGetPixelAll(uint32_t *bitmapData,
                                uint32_t  bitmapWidth,
                                uint32_t  xVal,
                                uint32_t  yVal)
{
    uint32_t    pixel_val;

    if (bitmapData == NULL) return 0;

    pixel_val = (uint32_t)*(bitmapData + ((yVal * bitmapWidth) + xVal));

    return pixel_val;
}

    /* Returns the pointer within the bitmap data, instead of the actual value. */
static uint32_t* drawGetPixelPtr(uint32_t* bitmapData,
                                 uint32_t  bitmapWidth,
                                 uint32_t  xVal,
                                 uint32_t  yVal)
{
    if (bitmapData == NULL) return 0;

    return (bitmapData + ((yVal * bitmapWidth) + xVal));
}


static uint32_t drawGetClutValue (const DRAW_PicItemT* picPtr,
                                  uint8_t clutIdx,
                                  uint32_t* clutToUse)
{
    uint16_t    index_to_find;
    uint32_t*   p_ptr;

    if (picPtr == NULL) return 0;

    if ((clutIdx < picPtr->clutStartIdx) || (clutIdx > picPtr->clutEndIdx))
    {
        index_to_find = 0;
    }
    else
    {
        index_to_find = clutIdx;
    }

    p_ptr = clutToUse + index_to_find;

    return (*p_ptr);

}


static uint8_t drawGetBestColor(uint32_t colorValue,
                                const uint32_t* clutPtr,
                                uint8_t clutStart,
                                uint8_t clutEnd)
{
    uint16_t  x;
    uint32_t  lowest_colour_diff = 0xFFFFFFFF;
    uint32_t  current_colour_diff;
    uint8_t   best_colour_idx = 0;
    uint32_t* curr_ptr;

    if (clutPtr == NULL)
    {
        return 0;
    }

    curr_ptr = (uint32_t*) clutPtr;

    if( ( colorValue & 0x000000FF) <= DRAW_MIN_TRANSPARENCY)
    {
        best_colour_idx = transparentColour;
    }
    else
    {
        for (x = clutStart; x < clutEnd; x++)
        {
            current_colour_diff = drawSquared((uint32_t)(((colorValue & 0xFF000000) >> 24) -
                (((*curr_ptr) & (0xFF000000)) >> 24))) +
                               drawSquared((uint32_t)(((colorValue & 0x00FF0000) >> 16) -
                (((*curr_ptr) & (0x00FF0000)) >> 16))) +
                               drawSquared((uint32_t)(((colorValue & 0x0000FF00) >> 8) -
                (((*curr_ptr) & (0x0000FF00)) >>  8))) +
                               drawSquared((uint32_t)(((colorValue & 0x000000FF)) -
                (((*curr_ptr) & (0x000000FF)) )));

            if (current_colour_diff <= lowest_colour_diff)
            {
                best_colour_idx = (uint8_t)x;
                lowest_colour_diff = current_colour_diff;
            }

            curr_ptr++;
        }
    }

    return (best_colour_idx);

}

/*!
*****************************************************************************
** \brief   Rescale a FBM picture.
**
**          The picPtr parameters specify how the source picture will be read.
**          For CLUT colour modes only:
**          If a picPtr->clutId is specified, the CLUT of the OSD with layerId
**          'clutId' will be used for decoding the source picture. If the
**          picPtr->clutId is set to 0, the specified picPtr->clutPtr will
**          be used. The picPtr->clutStartIdx and picPtr->clutEndIdx define
**          the range of colours that are considered to be 'valid' when decoding
**          the picture. If there exist pixels with colour values out of this
**          range, they will be replaced by CLUT entry 0.
**          If the selected colour mode of the source picture is
**          not a CLUT colour mode, the picPtr->clutPtr may be NULL while
**          the clutId is 0.
**
**          The convPicPtr specifies how the picture will be converted.
**          It is also possible to store the converted picture in a
**          different colour format as the source picture.
**          The convPicPtr->fbmHeaderPtr specifies the colour mode of the
**          converted picture. If the fbmHeaderPtr is set to NULL, it will be
**          automatically allocated and the colour mode will be the same as
**          that from the source picture. However, if the fbmHeaderPtr is
**          given, the colour mode of the converted picture is specified
**          by convPicPtr->fbmHeaderPtr->colourMode.
**          If the convPicPtr->clutId is unequal 0, the converted picture will
**          use the CLUT of the OSD with layerId 'clutId' and there will no
**          CLUT explicitly be stored within the picture. Otherwise, if the
**          clutId is 0, the convPicPtr->clutPtr is the CLUT that will define
**          the available colours of the converted picture. In both cases
**          convPicPtr->clutStartIdx and convPicPtr->clutEndIdx indicate the
**          range of the colours in the destination CLUT that will be used for
**          mapping colours. Thus the convPicPtr->dataPtr field will only contain
**          values between these two parameters if the picture is stored in a
**          CLUT colour format. If the convPicPtr->clutPtr is NULL while
**          convPicPtr->clutId is 0, the clutPtr will be allocated automatically
**          and the CLUT will be copied from the source picture CLUT. In this
**          case, the clutStartIdx and clutEndIdx will also be the same as those
**          from the source picture. In all other cases, convPicPtr->clutPtr,
**          convPicPtr->clutStartIdx and convPicPtr->clutEndIdx must be specified
**          by the user.
**          If convPicPtr->fbmHeaderPtr->colourMode specifies a colour mode
**          that is not a CLUT colour mode, the clutId, clutPtr, clutStartIdx
**          and clutEndIdx parameters will not affect the conversion of
**          the picture at all.
**          If the dataPtr of the converted picture is NULL, it will also
**          be automatically allocated.
**
** \attention
**          If any of the pointer-type members of the converted picture
**          is automatically allocated during conversion, you will have
**          to deallocate this memory when the picture is not needed any more.
**          Use DRAW_FreeFbm(convPicPtr) in order to release allocated resources.
**
** \param picPtr
**              Points to the source picture data which will be rescaled.
** \param convPicPtr
**              Points to a structure that will receive the rescaled picture.
** \param newWidth
**              Width of the converted picture.
** \param newHeight
**              Height of the converted picture.
**
** \return
**        - #FAPI_OK on success
**        - #DRAW_ERR_NOT_INITIALIZED if the DRAW driver is not initialized
**        - #DRAW_ERR_BAD_PARAMETER if
**          - picPtr is NULL
**          - convPicPtr is NULL
**          - for pictures in a CLUT colour mode the following condition holds:
**              (picPtr->clutId == 0) && (picPtr->clutPtr == NULL)
**        - #DRAW_ERR_WRONG_COLOUR_MODE if the source picture is not stored in
**          a RGB colour format or if the converted picture defines a colour
**          mode different from RGB colour format.
**        - #DRAW_ERR_OUT_OF_MEMORY if there was not enough memory available
**          for allocation in order to store the transcoded FBM picture file.
**
** \sa
**          - DRAW_FreeFbm()
**
*****************************************************************************
*/
int32_t DRAW_ScalePicture(const DRAW_PicItemT* picPtr,
                          DRAW_PicItemT* convPicPtr,
                          const uint32_t newWidth,
                          const uint32_t newHeight)
{
    int32_t             result = 0;
    uint32_t            new_width = 0;
    uint32_t            new_height = 0;
    uint32_t            I = 0;
    uint32_t            J = 0;
    uint32_t            i = 0;
    uint32_t            j = 0;
    uint32_t            old_width = 0;
    uint32_t            old_height = 0;
    uint32_t            previous_I = 0xFFFFFFFFUL; // initialize to (uint32_t)-1UL
    uint32_t*           bitmap_src_ptr = NULL;
    uint32_t*           bitmap_dest_ptr = NULL;
    uint32_t*           temp_dest_ptr = NULL;
    uint32_t            R, G, B, A, colour_value;
    uint32_t            colour_00, colour_10, colour_01, colour_11;
    uint32_t*           u32_ptr = NULL;
    uint16_t*           u16_ptr = NULL;
    uint8_t*            u08_ptr = NULL;
    int32_t             ret_val = 0;
    uint8_t             u08_temp = 0;
    uint32_t            x, y;
    uint32_t            cnt = 0;
    uint32_t            clut_array[256];
    uint32_t*           clut_out_ptr = NULL;
    uint8_t             clut_start_idx = 0;
    uint8_t             clut_end_idx = 0;
    uint32_t            new_alloc_info = 0;
    CALCULATION_TYPE    a;
    CALCULATION_TYPE    b;
    CALCULATION_TYPE    c;
    CALCULATION_TYPE    d;
    CALCULATION_TYPE    multiplier;
    CALCULATION_TYPE    multiplier2;
    CALCULATION_TYPE    multiplier3;
    CALCULATION_TYPE    multiplier4;
    CALCULATION_TYPE    multiplier5;
    CALCULATION_TYPE    multiplier6;
    CALCULATION_TYPE    multiplier7;
    CALCULATION_TYPE    theta_I;
    CALCULATION_TYPE    theta_J;
    jmp_buf jmp_env;

    if ( ! drawInitialized )
    {
        return(DRAW_ERR_NOT_INITIALIZED);
    }
    if ((picPtr == NULL) || (convPicPtr == NULL))
    {
        return DRAW_ERR_BAD_PARAMETER;
    }

    if ((picPtr->dataPtr == NULL) || (picPtr->fbmHeaderPtr == NULL))
    {
        return DRAW_ERR_BAD_PARAMETER;
    }

    if((picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_1BIT)
    || (picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_2BIT)
    || (picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_4BIT)
    || (picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_8BIT)
    || (picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_24BIT)
    || (picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_32BIT))
    {
        return(DRAW_ERR_WRONG_COLOUR_MODE);
    }
    if(convPicPtr->fbmHeaderPtr &&
        ( (convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_1BIT)
        ||(convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_2BIT)
        ||(convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_4BIT)
        ||(convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_8BIT)
        ||(convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_24BIT)
        ||(convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_32BIT)
        )
      )
    {
        return(DRAW_ERR_WRONG_COLOUR_MODE);
    }

    if ((picPtr->clutId == 0) && (picPtr->clutPtr == NULL) &&
        ( (picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_1BIT)
        ||(picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_2BIT)
        ||(picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_4BIT)
        ||(picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_8BIT)
        ||(picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_1BIT_RGB)
        ||(picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_2BIT_RGB)
        ||(picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_4BIT_RGB)
        ||(picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_8BIT_RGB)
        )
       )
    {
        return DRAW_ERR_BAD_PARAMETER;
    }

    memset(clut_array, 0, sizeof(clut_array));

    if (picPtr->clutId != 0)
    {
        ret_val = OSDHANDLER_GetClut(picPtr->clutId, clut_array);
        if (ret_val != FAPI_OK)
        {
            return ret_val;
        }
    }
    else
    {
        switch (picPtr->fbmHeaderPtr->colourMode)
        {
                case DRAW_COLOURMODE_1BIT_RGB:
                    memcpy(clut_array, picPtr->clutPtr, 2*sizeof(uint32_t));
                    break;
                case DRAW_COLOURMODE_2BIT_RGB:
                    memcpy(clut_array, picPtr->clutPtr, 4*sizeof(uint32_t));
                    break;
                case DRAW_COLOURMODE_4BIT_RGB:
                    memcpy(clut_array, picPtr->clutPtr, 16*sizeof(uint32_t));
                    break;
                case DRAW_COLOURMODE_8BIT_RGB:
                    memcpy(clut_array, picPtr->clutPtr, 256*sizeof(uint32_t));
                    break;

                case DRAW_COLOURMODE_1BIT:
                case DRAW_COLOURMODE_2BIT:
                case DRAW_COLOURMODE_4BIT:
                case DRAW_COLOURMODE_8BIT:
                case DRAW_COLOURMODE_24BIT:
                case DRAW_COLOURMODE_32BIT:
                case DRAW_COLOURMODE_16BIT_RGB:
                case DRAW_COLOURMODE_24BIT_RGB:
                case DRAW_COLOURMODE_32BIT_RGB:
                default:
                    break;
        }
    }
    /* find transparent colour */
    for(cnt = picPtr->clutStartIdx; cnt <= picPtr->clutEndIdx; cnt++)
    {
        if( (clut_array[cnt] & 0x000000FF) == 0)
            break;
    }
    transparentColour = (uint8_t)cnt;

    old_width = picPtr->fbmHeaderPtr->width;
    old_height = picPtr->fbmHeaderPtr->height;
    new_width = newWidth;
    new_height = newHeight;

    if (new_width < 1)
        new_width = 1;

    if (new_height < 1)
        new_height = 1;

    // exception handling:
    /*lint -save -e774 -e586 */
    result = setjmp(jmp_env);
    if(result)
    {
        if(new_alloc_info & 0x0100) FAPI_SYS_FREE(bitmap_src_ptr);
        if(new_alloc_info & 0x0200) FAPI_SYS_FREE(bitmap_dest_ptr);
        if(new_alloc_info & 0x0004) FAPI_SYS_FREE(convPicPtr->clutPtr);
        if(new_alloc_info & 0x0002) FAPI_SYS_FREE(convPicPtr->dataPtr);
        if(new_alloc_info & 0x0001) FAPI_SYS_FREE(convPicPtr->fbmHeaderPtr);
        // conv_pic_ptr->alloc_information = (new_alloc_info & 0x00FF);
        return(result);
    }
    /*lint -restore*/

    // allocate the bitmap_src_ptr:
    if (picPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_32BIT_RGB)
    {
        bitmap_src_ptr = picPtr->dataPtr;
    }
    else
    {
        bitmap_src_ptr = (uint32_t *)FAPI_SYS_MALLOC(( (old_width)*(old_height)) * sizeof(uint32_t));
        if (bitmap_src_ptr == NULL)
        {
            return DRAW_ERR_OUT_OF_MEMORY;
        }
        new_alloc_info |= 0x0100;
    }

    // allocate the convPicPtr header:
    if (convPicPtr->fbmHeaderPtr == NULL)
    {
        convPicPtr->fbmHeaderPtr = (DRAW_FbmHeaderT*) FAPI_SYS_MALLOC(sizeof(DRAW_FbmHeaderT));
        if (convPicPtr->fbmHeaderPtr == NULL)
        {
            /*lint -save -e586 */
            longjmp(jmp_env, DRAW_ERR_OUT_OF_MEMORY);
            /*lint -restore */
        }
        new_alloc_info |= 0x0001;
        memcpy(convPicPtr->fbmHeaderPtr, picPtr->fbmHeaderPtr, sizeof(DRAW_FbmHeaderT));
    }

    // Deallocate previously automatically allocated data member:
    if (convPicPtr->allocInformation & 0x0002)
    {
        FAPI_SYS_FREE(convPicPtr->dataPtr);
        convPicPtr->allocInformation &= ~0x0002;
    }

    // Allocate data member automatically if not given by the application:
    if (convPicPtr->dataPtr == NULL)
    {
        x = new_width;
        switch (convPicPtr->fbmHeaderPtr->colourMode)
        {
            case DRAW_COLOURMODE_1BIT_RGB:
                if(x % 8) // align to a multiple of eight
                {
                    x += 8 - (x % 8);
                }
                convPicPtr->dataPtr = (uint32_t *)FAPI_SYS_MALLOC(((x / 8) * new_height));
                break;
            case DRAW_COLOURMODE_2BIT_RGB:
                if(x % 4) // align to a multiple of four
                {
                    x += 4 - (x % 4);
                }
                convPicPtr->dataPtr = (uint32_t *)FAPI_SYS_MALLOC(((x / 4) * new_height));
                break;
            case DRAW_COLOURMODE_4BIT_RGB:
                if(x % 2) // align to a multiple of two
                {
                    x++;
                }
                convPicPtr->dataPtr = (uint32_t *)FAPI_SYS_MALLOC(((x / 2) * new_height));
                break;
            case DRAW_COLOURMODE_8BIT_RGB:
                convPicPtr->dataPtr = (uint32_t *)FAPI_SYS_MALLOC(((x) * new_height));
                break;
            case DRAW_COLOURMODE_16BIT_RGB:
                convPicPtr->dataPtr = (uint32_t *)FAPI_SYS_MALLOC(((x * 2) * new_height));
                break;
            case DRAW_COLOURMODE_24BIT_RGB:
                convPicPtr->dataPtr = (uint32_t *)FAPI_SYS_MALLOC(((x * 3) * new_height));
                break;
            case DRAW_COLOURMODE_32BIT_RGB:
                convPicPtr->dataPtr = (uint32_t *)FAPI_SYS_MALLOC(( (new_width)*(new_height)) * sizeof(uint32_t));
                break;
            case DRAW_COLOURMODE_1BIT:
            case DRAW_COLOURMODE_2BIT:
            case DRAW_COLOURMODE_4BIT:
            case DRAW_COLOURMODE_8BIT:
            case DRAW_COLOURMODE_24BIT:
            case DRAW_COLOURMODE_32BIT:
            default:
                break;
        }
        if (convPicPtr->dataPtr == NULL)
        {
            /*lint -save -e586 */
            longjmp(jmp_env, DRAW_ERR_OUT_OF_MEMORY);
            /*lint -restore */
        }
        new_alloc_info |= 0x0002;
    }

    if(convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_32BIT_RGB)
    {
        bitmap_dest_ptr = (uint32_t *)(convPicPtr->dataPtr);
    }
    else
    {
        bitmap_dest_ptr = (uint32_t *)FAPI_SYS_MALLOC(( (new_width)*(new_height)) * sizeof(uint32_t));
        if (bitmap_dest_ptr == NULL)
        {
            /*lint -save -e586 */
            longjmp(jmp_env, DRAW_ERR_OUT_OF_MEMORY);
            /*lint -restore */
        }
        new_alloc_info |= 0x0200;
    }

    /* allocate memory for convPic clut */
    if((convPicPtr->clutId == 0)
    && (convPicPtr->clutPtr == NULL)
    && ( (convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_1BIT_RGB)
       ||(convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_2BIT_RGB)
       ||(convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_4BIT_RGB)
       ||(convPicPtr->fbmHeaderPtr->colourMode == DRAW_COLOURMODE_8BIT_RGB)
       )
      )
    {
        convPicPtr->clutPtr = (uint32_t *)FAPI_SYS_MALLOC(256*sizeof(uint32_t));
        if (convPicPtr->clutPtr == NULL)
        {
            /*lint -save -e586 */
            longjmp(jmp_env, DRAW_ERR_OUT_OF_MEMORY);
            /*lint -restore */
        }
        new_alloc_info |= 0x0004;
        memcpy(convPicPtr->clutPtr, picPtr->clutPtr, 256*sizeof(uint32_t));
        convPicPtr->clutStartIdx = picPtr->clutStartIdx;
        convPicPtr->clutEndIdx = picPtr->clutEndIdx;
    }

    /* Read in the bitmap source:
    ** "bitmap_src_ptr" will contain the bitmap source data in 32-bit RGBA colour mode.
    ** For 32-bit RGB mode, it is already assigned correctly, for other modes convert
    ** the input picture picPtr to 32-bit and store it at bitmap_src_ptr.
    */
    temp_dest_ptr = bitmap_dest_ptr;
    u32_ptr = bitmap_src_ptr;
    switch (picPtr->fbmHeaderPtr->colourMode)
    {
        case DRAW_COLOURMODE_1BIT_RGB:
            u08_ptr = (uint8_t *)(picPtr->dataPtr);
            for (y = 0; y < old_height; y++)
            {
                for (x = 0; x < old_width; x++)
                {
                    switch (x % 8)
                    {
                        case 0: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x80) >> 7), clut_array); break;
                        case 1: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x40) >> 6), clut_array); break;
                        case 2: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x20) >> 5), clut_array); break;
                        case 3: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x10) >> 4), clut_array); break;
                        case 4: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x08) >> 3), clut_array); break;
                        case 5: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x04) >> 2), clut_array); break;
                        case 6: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x02) >> 1), clut_array); break;
                        case 7: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x01)), clut_array); break;
                        default:
                            break;
                    }
                    u32_ptr++;
                    if ((x >= (old_width - 1)) || ((x % 8) == 7))
                    {
                        u08_ptr++;
                    }
                }
            }
            break;
        case DRAW_COLOURMODE_2BIT_RGB:
            u08_ptr = (uint8_t *)(picPtr->dataPtr);
            for (y = 0; y < old_height; y++)
            {
                for (x = 0; x < old_width; x++)
                {
                    switch (x % 4)
                    {
                        case 0: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0xC0) >> 6), clut_array); break;
                        case 1: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x30) >> 4), clut_array); break;
                        case 2: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x0C) >> 2), clut_array); break;
                        case 3: *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0x03)), clut_array); break;
                        default:
                            break;
                    }
                    u32_ptr++;
                    if ((x >= (old_width - 1)) || ((x % 4) == 3))
                    {
                        u08_ptr++;
                    }
                }
            }
            break;
        case DRAW_COLOURMODE_4BIT_RGB:
            u08_ptr = (uint8_t *)(picPtr->dataPtr);
            for (y = 0; y < old_height; y++)
            {
                x = 0;
                while (x < old_width)
                {
                    *u32_ptr = drawGetClutValue(picPtr, (((*u08_ptr) & 0xF0) >> 4), clut_array);
                    x++;
                    u32_ptr++;
                    if (x < old_width)
                    {
                        *u32_ptr = drawGetClutValue(picPtr, ((*u08_ptr) & 0x0F), clut_array);
                        u32_ptr++;
                    }
                    x++;
                    u08_ptr++;
                }
            }
            break;
        case DRAW_COLOURMODE_8BIT_RGB:
            u08_ptr = (uint8_t *)(picPtr->dataPtr);
            for (y = 0; y < old_height; y++)
            {
                for (x = 0; x < old_width; x++)
                {
                    *u32_ptr = drawGetClutValue(picPtr, (*u08_ptr), clut_array);
                    u32_ptr++;
                    u08_ptr++;
                }
            }
            break;
        case DRAW_COLOURMODE_16BIT_RGB:
            u16_ptr = (uint16_t *)(void*)(picPtr->dataPtr);
            for (y = 0; y < old_height; y++)
            {
                for (x = 0; x < old_width; x++)
                {
                    // formula is based on getting the appropriate color, multiplying by 8 to get
                    // the 8-bit component value and finally shifting it to the correct location
                    // in the RGBA word
                    *u32_ptr = (((*u16_ptr) & 0xF800) << 16) +
                               (((*u16_ptr) & 0x07C0) << 13) +
                               (((*u16_ptr) & 0x003E) << 10) +
                               ((picPtr->globalAlpha & 0x0100) ? 0xFF : (picPtr->globalAlpha & 0xFF));
                    u32_ptr++;
                    u16_ptr++;
                }
            }
            break;
        case DRAW_COLOURMODE_24BIT_RGB:
            u08_ptr = (uint8_t *)(picPtr->dataPtr);
            for (y = 0; y < old_height; y++)
            {
                for (x = 0; x < old_width; x++)
                {
                    *u32_ptr = ((*(u08_ptr + 2)) << 24) |
                               ((*(u08_ptr + 1)) << 16) |
                               ((*(u08_ptr)) << 8) |
                               ((picPtr->globalAlpha & 0x0100) ? 0xFF : (picPtr->globalAlpha & 0xFF));
                    u32_ptr++;
                    u08_ptr+= 3;
                }
            }

            break;
        case DRAW_COLOURMODE_32BIT_RGB:
            /* no more conversion necessary. */
            break;

        // dead code below:
        case DRAW_COLOURMODE_1BIT:
        case DRAW_COLOURMODE_2BIT:
        case DRAW_COLOURMODE_4BIT:
        case DRAW_COLOURMODE_8BIT:
        case DRAW_COLOURMODE_24BIT:
        case DRAW_COLOURMODE_32BIT:
        default:
            /*lint -save -e586 */
            longjmp(jmp_env, DRAW_ERR_BAD_PARAMETER);
            /*lint -restore */
    }

    // use the global alpha value of the fbm file if it is enabled:
    if(picPtr->globalAlpha < 0x0100)
    {
        u32_ptr = bitmap_src_ptr;
        for(y = 0; y < old_height; y++)
        {
            for(x = 0; x < old_width; x++)
            {
                *u32_ptr = (*u32_ptr & 0xFFFFFF00) | picPtr->globalAlpha;
                u32_ptr++;
            }
        }
    }

// ========================== RESCALING ALGORITHM START ========================

#if CALCULATION_DOUBLE_PRECISION
    a = old_height - 1.0d;
    b = new_height - 1.0d;
    c = old_width - 1.0d;
    d = new_width - 1.0d;
#else
    a = old_height - 1.0f;
    b = new_height - 1.0f;
    c = old_width - 1.0f;
    d = new_width - 1.0f;
#endif

    multiplier = -a;

    /* Optimization: Multiply by the reciprocal rather than dividing of the dividend (for
    the multiplication operations below. */
#if CALCULATION_DOUBLE_PRECISION
    b = 1.0d/b;
    d = 1.0d/d;
#else
    b = 1.0f/b;
    d = 1.0f/d;
#endif

    for( j=0; j < (new_height-1) ; j++ )
    {
        multiplier += a;
        theta_J = multiplier * b; // = (i * oldHeight) / newHeight
        J    = drawFloor( theta_J ); // = floor((i * oldHeight) / newHeight)
        theta_J -= J;
        multiplier2 = -c;

        /* Initialize 2x2 Colour Grid */
        colour_00 = 0; // current pixel
        colour_01 = 0; // pixel below
        colour_10 = 0; // pixel on the right
        colour_11 = 0; // pixel below on the right

        /* Initialize to -1 */
        previous_I = 0xFFFFFFFFUL;

        // all 2x2 Colour Grids
        for( i=0; i < (new_width-1) ; i++ )
        {
            multiplier2 += c;
            theta_I = multiplier2 * d; // = (i * oldWidth) / newWidth
            I = drawFloor( theta_I ); // = floor((i * oldWidth) / newWidth)
            theta_I -= I;

            if (previous_I != I)
            {
                u32_ptr = drawGetPixelPtr(bitmap_src_ptr, old_width, I, J);
                colour_00 = *u32_ptr;
                u32_ptr++;
                colour_10 = *u32_ptr;
                u32_ptr = drawGetPixelPtr(bitmap_src_ptr, old_width, I, J+1);
                colour_01 = *u32_ptr;
                u32_ptr++;
                colour_11 = *u32_ptr;
                previous_I = I;
            }
            else
            {
                /* If previous I is the same, then there is no need to update the color grid. */
            }
            multiplier3 = theta_I * theta_J;
            multiplier4 = (1.0 - theta_I) - theta_J;
            multiplier5 = multiplier4 + multiplier3;
            multiplier6 = theta_I-multiplier3;
            multiplier7 = theta_J-multiplier3;


            R = (uint8_t)
                          ( ((multiplier5)*(GET_RED(colour_00)))
                           +((multiplier6)*(GET_RED(colour_10)))
                           +((multiplier7)*(GET_RED(colour_01)))
                           +((multiplier3)*(GET_RED(colour_11))) );
            G = (uint8_t)
                          ( ((multiplier5)*(GET_GREEN(colour_00)))
                           +((multiplier6)*(GET_GREEN(colour_10)))
                           +((multiplier7)*(GET_GREEN(colour_01)))
                           +((multiplier3)*(GET_GREEN(colour_11))) );
            B = (uint8_t)
                          ( ((multiplier5)*(GET_BLUE(colour_00)))
                           +((multiplier6)*(GET_BLUE(colour_10)))
                           +((multiplier7)*(GET_BLUE(colour_01)))
                           +((multiplier3)*(GET_BLUE(colour_11))) );

            A = (uint8_t)
                          ( ((multiplier5)*(GET_ALPHA(colour_00)))
                           +((multiplier6)*(GET_ALPHA(colour_10)))
                           +((multiplier7)*(GET_ALPHA(colour_01)))
                           +((multiplier3)*(GET_ALPHA(colour_11))) );

            colour_value = (R << 24) | (G << 16) | (B << 8) | (A);
            *temp_dest_ptr = colour_value;

            temp_dest_ptr++;
        }
        // process the last column (no pixels on the right):

        // not precise (~~rk):
        //colour_00 = drawGetPixelAll(bitmap_src_ptr, old_width, (old_width-1), J);
        //colour_01 = drawGetPixelAll(bitmap_src_ptr, old_width, (old_width-1), J+1);

        multiplier2 += c;
        theta_I = multiplier2 * d;
        I = drawFloor( theta_I );

        colour_00 = drawGetPixelAll(bitmap_src_ptr, old_width, I, J);
        colour_01 = drawGetPixelAll(bitmap_src_ptr, old_width, I, J+1);

        R = (uint8_t)( ((1.0-theta_J)*(GET_RED(colour_00)))
                          + (theta_J*(GET_RED(colour_01))) );
        G = (uint8_t)( ((1.0-theta_J)*(GET_GREEN(colour_00)))
                          + (theta_J*(GET_GREEN(colour_01))) );
        B = (uint8_t)( ((1.0-theta_J)*(GET_BLUE(colour_00)))
                          + (theta_J*(GET_BLUE(colour_01))) );
        A = (uint8_t)( ((1.0-theta_J)*(GET_ALPHA(colour_00)))
                          + (theta_J*(GET_ALPHA(colour_01))) );

        colour_value = (R << 24) | (G << 16) | (B << 8) | (A);
        *temp_dest_ptr = colour_value;
        temp_dest_ptr++;
    }

    for( i = 0 ; i < (new_width-1) ; i++ )
    {
        #if CALCULATION_DOUBLE_PRECISION
        // double precision:
        theta_I = (CALCULATION_TYPE)(i*(old_width-1.0d)) / (CALCULATION_TYPE)(new_width-1.0d);
        #else
        // single precision:
        theta_I = (CALCULATION_TYPE)(i*(old_width-1.0f)) / (CALCULATION_TYPE)(new_width-1.0f);
        #endif

        I = drawFloor( theta_I );
        theta_I -= I;
        R = (uint8_t)  ( ((1.0-theta_I)*(drawGetPixelRed(bitmap_src_ptr, old_width, I, (old_height-1))))
                          + (theta_I*(drawGetPixelRed(bitmap_src_ptr, old_width, I, (old_height-1)))) );
        G = (uint8_t)  ( ((1.0-theta_I)*(drawGetPixelGreen(bitmap_src_ptr, old_width, I, (old_height-1))))
                          + (theta_I*(drawGetPixelGreen(bitmap_src_ptr, old_width, I, (old_height-1)))) );
        B = (uint8_t)  ( ((1.0-theta_I)*(drawGetPixelBlue(bitmap_src_ptr, old_width, I, (old_height-1))))
                          + (theta_I*(drawGetPixelBlue(bitmap_src_ptr, old_width, I, (old_height-1)))) );
        A = (uint8_t)  ( ((1.0-theta_I)*(drawGetPixelAlpha(bitmap_src_ptr, old_width, I, (old_height-1))))
                          + (theta_I*(drawGetPixelAlpha(bitmap_src_ptr, old_width, I, (old_height-1)))) );

        colour_value = (R << 24) | (G << 16) | (B << 8) | (A);
        *temp_dest_ptr = colour_value;
        temp_dest_ptr++;
    }

    *temp_dest_ptr = drawGetPixelAll(bitmap_src_ptr, old_width, (old_width - 1), (old_height - 1));

// ========================== RESCALING ALGORITHM END ==========================


    /* Finally update the type, width, and height information. */

    // first define the clut_out_ptr
    if(convPicPtr->clutId == 0)
    {
        clut_out_ptr = convPicPtr->clutPtr;
    }
    else
    {
        ret_val = OSDHANDLER_GetClut(convPicPtr->clutId, clut_array);
        if (ret_val != FAPI_OK)
        {
            /*lint -save -e586 */
            longjmp(jmp_env, ret_val);
            /*lint -restore */
        }
        clut_out_ptr = clut_array;
    }
    clut_start_idx = (uint8_t)convPicPtr->clutStartIdx;
    clut_end_idx = (uint8_t)convPicPtr->clutEndIdx;

    convPicPtr->fbmHeaderPtr->width = new_width;
    convPicPtr->fbmHeaderPtr->height = new_height;
    switch (convPicPtr->fbmHeaderPtr->colourMode)
    {
        case DRAW_COLOURMODE_1BIT_RGB:
            u08_ptr = (uint8_t*)(convPicPtr->dataPtr);
            u32_ptr = bitmap_dest_ptr;
            for (y = 0; y < new_height; y++)
            {
                for (x = 0; x < new_width; x++)
                {
                    switch (x % 8)
                    {
                        case 0: *u08_ptr =               (uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 7) ; break;
                        case 1: *u08_ptr = (*u08_ptr) | ((uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 6)); break;
                        case 2: *u08_ptr = (*u08_ptr) | ((uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 5)); break;
                        case 3: *u08_ptr = (*u08_ptr) | ((uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 4)); break;
                        case 4: *u08_ptr = (*u08_ptr) | ((uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 3)); break;
                        case 5: *u08_ptr = (*u08_ptr) | ((uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 2)); break;
                        case 6: *u08_ptr = (*u08_ptr) | ((uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 1)); break;
                        case 7: *u08_ptr = (*u08_ptr) |            drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx); break;
                        default:
                            break;
                    }
                    u32_ptr++;
                    if ((x >= (new_width - 1)) || ((x % 8) == 7))
                    {
                        u08_ptr++;
                    }
                }
            }
            break;
        case DRAW_COLOURMODE_2BIT_RGB:
            u08_ptr = (uint8_t*)(convPicPtr->dataPtr);
            u32_ptr = bitmap_dest_ptr;
            for (y = 0; y < new_height; y++)
            {
                for (x = 0; x < new_width; x++)
                {
                    switch (x % 4)
                    {
                        case 0: *u08_ptr =              (uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 6); break;
                        case 1: *u08_ptr = (*u08_ptr) | (uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 4); break;
                        case 2: *u08_ptr = (*u08_ptr) | (uint8_t)(drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx) << 2); break;
                        case 3: *u08_ptr = (*u08_ptr) | (uint8_t) drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx); break;
                        default:
                            break;
                    }
                    u32_ptr++;
                    if ((x >= (new_width - 1)) || ((x % 4) == 3))
                    {
                        u08_ptr++;
                    }
                }
            }
            break;
        case DRAW_COLOURMODE_4BIT_RGB:
            u08_ptr = (uint8_t*)(convPicPtr->dataPtr);
            u32_ptr = bitmap_dest_ptr;
            for (y = 0; y < new_height; y++)
            {
                x = 0;
                while ( x < new_width )
                {
                    u08_temp = drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx);
                    u08_temp = (uint8_t)(u08_temp << 4);
                    x++;
                    u32_ptr++;
                    if (x < new_width)
                    {
                        u08_temp = (u08_temp) | (drawGetBestColor(*u32_ptr, clut_out_ptr, clut_start_idx, clut_end_idx));
                        u32_ptr++;
                    }
                    *u08_ptr = u08_temp;
                    x++;
                    u08_ptr++;
                }
            }
            break;
        case DRAW_COLOURMODE_8BIT_RGB:
            u08_ptr = (uint8_t*)(convPicPtr->dataPtr);
            u32_ptr = bitmap_dest_ptr;
            for (y = 0; y < new_height; y++)
            {
                for (x = 0; x < new_width; x++)
                {
                    *u08_ptr = drawGetBestColor(*u32_ptr,
                                                 clut_out_ptr,
                                                 clut_start_idx,
                                                 clut_end_idx);
                    u32_ptr++;
                    u08_ptr++;
                }

            }
            break;
        case DRAW_COLOURMODE_16BIT_RGB:
            u16_ptr = (uint16_t*)(void*)(convPicPtr->dataPtr);
            u32_ptr = bitmap_dest_ptr;
            convPicPtr->clutId = 0x00000000;
            convPicPtr->clutStartIdx = 0;
            convPicPtr->clutEndIdx = 0;
            for (y = 0; y < new_height; y++)
            {
                for (x = 0; x < new_width; x++)
                {
                    // Must perform successive shifts (rather than only one),
                    // to drop the unnecessary bits during this conversion process
                    *u16_ptr = ((uint16_t)(((((*u32_ptr) & (0xFF000000)) >> 24) >> 3) << 11)) +
                        ((uint16_t)(((((*u32_ptr) & (0x00FF0000)) >> 16) >> 3) << 6)) +
                        ((uint16_t)(((((*u32_ptr) & (0x0000FF00)) >> 8) >> 3) << 1));
                    if ((*u32_ptr) & 0x000000FF)
                    {
                        *u16_ptr += 1;
                    }
                    u16_ptr++;
                    u32_ptr++;
                }
            }
            break;
        case DRAW_COLOURMODE_24BIT_RGB:
            u08_ptr = (uint8_t*)(convPicPtr->dataPtr);
            u32_ptr = bitmap_dest_ptr;
            convPicPtr->clutId = 0x00000000;
            convPicPtr->clutStartIdx = 0;
            convPicPtr->clutEndIdx = 0;
            for (y = 0; y < new_height; y++)
            {
                for (x = 0; x < new_width; x++)
                {
                    *u08_ptr = (uint8_t)(((*u32_ptr) & 0x0000FF00) >> 8);  u08_ptr++;
                    *u08_ptr = (uint8_t)(((*u32_ptr) & 0x00FF0000) >> 16); u08_ptr++;
                    *u08_ptr = (uint8_t)(((*u32_ptr) & 0xFF000000) >> 24); u08_ptr++;
                    u32_ptr++;
                }
            }
            break;
        case DRAW_COLOURMODE_32BIT_RGB:
            convPicPtr->clutId = 0x00000000;
            convPicPtr->clutStartIdx = 0;
            convPicPtr->clutEndIdx = 0;
            break;

        case DRAW_COLOURMODE_1BIT:
        case DRAW_COLOURMODE_2BIT:
        case DRAW_COLOURMODE_4BIT:
        case DRAW_COLOURMODE_8BIT:
        case DRAW_COLOURMODE_24BIT:
        case DRAW_COLOURMODE_32BIT:
        default:
            break;
    }

    if (bitmap_src_ptr != picPtr->dataPtr)
    {
        FAPI_SYS_FREE(bitmap_src_ptr);
    }
    if (bitmap_dest_ptr != convPicPtr->dataPtr)
    {
        FAPI_SYS_FREE(bitmap_dest_ptr);
    }
    convPicPtr->allocInformation |= (new_alloc_info & 0x00FF);
    return FAPI_OK;
}

#endif /* DRAW_ENABLE_RESIZE */
