
#include <jpeg/JPEG_api.h>
#include <fapex/abst_types.h>


static void JPEG_ScaleLine(U16 left, U16 numcomps, U16 scanline, U16 outwidth, JSAMPIMAGE input_buf, JDIMENSION input_row, JPEG_scaler_t * scalerP);



void JPEG_BuildTables(JPEG_t* jpegP, JPEG_scaler_t* scalerP)
{    long            i, n, r;
    long            base= sizeof(struct jpeg_decompress_struct) + sizeof(JPEG_scaler_t);
    unsigned char*    divP;

    for(i= 1; i<10; i++)
    {    scalerP->div[i]= &((unsigned char*)jpegP->env.staticP)[base];
        base= base + i*256;
    }
    
    for(i= 1; i<10; i++)
    {    divP= scalerP->div[i];
        for(n= 0; n<256; n++)
            for(r= 0; r < i; r++)
                *divP++= n;
    }            
}


short JPEG_ReadHeader(JPEG_t* jpegP)
{    j_decompress_ptr                cinfoP;
    short                            error;
    short                            i;
    
    cinfoP= jpegP->env.staticP;
    cinfoP->used[JPOOL_PERMANENT]= sizeof(struct jpeg_decompress_struct) + sizeof(JPEG_scaler_t) + 45 * 256;
    cinfoP->used[JPOOL_IMAGE]= 0;
    cinfoP->pool[JPOOL_PERMANENT]= jpegP->env.staticP;
    jpeg_CreateDecompress(cinfoP);
    jpeg_stdio_src(cinfoP, 0);
    cinfoP->src->userP= jpegP->env.userDataP;
    cinfoP->src->loadFct= jpegP->env.loadFct;
    error= jpeg_read_header(cinfoP);
    
    if(cinfoP->arith_code)
        return JERR_ARITH_NOTIMPL;

    if(error != JPEG_HEADER_OK)
        return error;

    jpegP->info.sizeX= cinfoP->image_width;
    jpegP->info.sizeY= cinfoP->image_height;
    if(cinfoP->saw_JFIF_marker)
    {    jpegP->info.dpiX= cinfoP->X_density;
        jpegP->info.dpiY= cinfoP->Y_density;
    }
    else
    {    jpegP->info.dpiX= 0;
        jpegP->info.dpiY= 0;
    }
    jpegP->info.color= (cinfoP->jpeg_color_space != JCS_GRAYSCALE);
    jpegP->info.progressive= cinfoP->progressive_mode;

    jpegP->env.temporalbytes= 0;
    if(cinfoP->progressive_mode)
    {    for(i= 0; i<cinfoP->num_components; i++)
            jpegP->env.temporalbytes= jpegP->env.temporalbytes + cinfoP->cur_comp_info[i]->downsampled_width * cinfoP->cur_comp_info[i]->downsampled_height * 2;
        jpegP->env.temporalbytes= jpegP->env.temporalbytes + 20 * cinfoP->image_width + ((3 * cinfoP->image_height) >> 1) + 8196;
    }
    
    for(i= 0; i<cinfoP->num_components; i++)
        jpegP->env.temporalbytes= jpegP->env.temporalbytes + cinfoP->cur_comp_info[i]->downsampled_width * cinfoP->cur_comp_info[i]->v_samp_factor * 10;
    jpegP->env.temporalbytes= jpegP->env.temporalbytes + 10 * cinfoP->image_width + 21000;

    jpegP->image.left= 0;
    jpegP->image.top= 0;
    jpegP->image.right= jpegP->info.sizeX;
    jpegP->image.bottom= jpegP->info.sizeY;
    jpegP->image.orient= ROT_0;
    if(jpegP->env.screen != SCREEN_CUSTOM)
    {    jpegP->dest.left= 0;
        jpegP->dest.top= 0;
        jpegP->dest.right= 720;
        switch(jpegP->env.screen)
        {    case SCREEN_NTSC43:
            case SCREEN_NTSC169:
                jpegP->dest.bottom= 480;
                break;
            case SCREEN_PAL43:
            case SCREEN_PAL169:
                jpegP->dest.bottom= 576;
                break;
            case SCREEN_LCD15:
                jpegP->dest.right= 1024;
                jpegP->dest.bottom= 768;
                break;
            case SCREEN_LCD4:
                jpegP->dest.right= 320;
                jpegP->dest.bottom= 240;
                break;
            case SCREEN_LCD7W:
                jpegP->dest.right= 480;
                jpegP->dest.bottom= 270;
                break;
            case SCREEN_HD:
                jpegP->dest.right= 1900;
                jpegP->dest.bottom= 1080;
                break;
        }        
    }
    return 0;
}


short JPEG_Decode(JPEG_t* jpegP)
{    short        error;
    char        again;

    error= JPEG_StartDecode(jpegP);
    if(error)
        return error;

    do
    {    again= JPEG_Continue(jpegP);
    }while(again);

    JPEG_FinishDecode(jpegP);

    return 0;
}


char JPEG_Continue(JPEG_t* jpegP)
{    struct jpeg_decompress_struct *    cinfoP= jpegP->env.staticP;
    JPEG_scaler_t *                    scalerP= (JPEG_scaler_t*)&((char*)jpegP->env.staticP)[sizeof(struct jpeg_decompress_struct)];
    
    return jpeg_read_scanlines(cinfoP, scalerP);
}
   
   
void JPEG_FinishDecode(JPEG_t* jpegP)    
{    struct jpeg_decompress_struct *    cinfoP= jpegP->env.staticP;
    
    jpeg_finish_decompress(cinfoP);

#ifdef DEMO_VERSION
    {    unsigned long                x, y;
        volatile unsigned long*        screen1P;
        unsigned long                demo[11]= {0xFCFD833E,
                                                 0xFEFD837F,
                                                  0xC6C18363,
                                               0xC6C1C763,
                                               0xC6C1EF63,
                                               0xC6F1EF63,
                                               0xC6F1BB63,
                                               0xC6C1BB63,
                                               0xC6C19363,
                                               0xFEFD937F,
                                               0xFCFD933E};
        
        screen1P= (unsigned long*)(&jpegP->dest.screenP[(((jpegP->dest.left+24) & 0xFFFE)*2+4) + (jpegP->dest.top+16)*jpegP->dest.rowlongs*4]);

        for(y= 0; y<11; y++)
        {    for(x= 0; x<32; x++)
            {    if(demo[y] & (0x80000000>>x))
                {    screen1P[x]= 0x80E080E0;
                    screen1P[x+jpegP->dest.rowlongs]= 0x80E080E0;
                }
            }
            screen1P= &screen1P[jpegP->dest.rowlongs*2];
        }
    }
#endif
}


short JPEG_StartDecode(JPEG_t* jpegP)
{    struct jpeg_decompress_struct *    cinfoP= jpegP->env.staticP;
    short                            error;
    long                            ratioV, ratioCV, ratioCH, temp;
    long                            sizeH, sizeV;
    long                            sourceH, sourceV;
    JPEG_scaler_t *                    scalerP= (JPEG_scaler_t*)&((char*)jpegP->env.staticP)[sizeof(struct jpeg_decompress_struct)];
    short                            i, n;
    static const long                aspect[]= {15360, 11520, 18432, 13824, 16384, 16384, 12288, 16384};
    static const long                iaspect[]= {17476, 23301, 14563, 19418, 16384, 16384, 21845, 16384};
    unsigned short                    realleft, realtop;
    volatile unsigned long *        curtainP;

    JPEG_BuildTables(jpegP, scalerP);

    cinfoP->pool[JPOOL_IMAGE]= jpegP->env.temporalP;

    if(jpegP->image.fast)
    {    cinfoP->do_fancy_upsampling= FALSE;
        cinfoP->dct_method= JDCT_IFAST;
    }
    else 
    {    cinfoP->do_fancy_upsampling= FALSE;//TRUE; /* Does not work with optimizer > level 3 */
        cinfoP->dct_method= JDCT_ISLOW;
    }

    if((jpegP->image.orient == ROT_0) || (jpegP->image.orient == ROT_180))
    {    sourceH= jpegP->image.right-jpegP->image.left;
        sourceV= jpegP->image.bottom-jpegP->image.top;
    }
    else
    {    sourceV= jpegP->image.right-jpegP->image.left;
        sourceH= jpegP->image.bottom-jpegP->image.top;
    }
    
    scalerP->blowup= 1;

    if(jpegP->env.screen != SCREEN_CUSTOM)
    {    ratioV= ((jpegP->dest.bottom-jpegP->dest.top) << 16) / sourceV;
        if(ratioV < 0x20000)
            ratioCH= ratioV * aspect[jpegP->env.screen] >> 14;
        else
            ratioCH= (ratioV >> 10) * aspect[jpegP->env.screen] >> 4;    
        
        if(((sourceH * ratioCH) >> 16) <= (jpegP->dest.right-jpegP->dest.left))
            ratioCV= ratioV;
        else
        {    ratioCH= ((jpegP->dest.right-jpegP->dest.left) << 16) / sourceH;
            if(ratioCH < 0x10000)
                ratioCV= (ratioCH >> 2) * iaspect[jpegP->env.screen] >> 12;
            else
                ratioCV= (ratioCH >> 4) * (iaspect[jpegP->env.screen] >> 2) >> 8;    
        }
        while(ratioCV > 0x10000)
        {
            ratioCV= ratioCV >> 1;
            ratioCH= ratioCH >> 1;
            sourceH= sourceH << 1;
            sourceV= sourceV << 1;
            scalerP->blowup= scalerP->blowup << 1;    
        }
    }
    else
    {    ratioCV= ((jpegP->dest.bottom-jpegP->dest.top) << 16) / sourceV;
        ratioCH= ((jpegP->dest.right-jpegP->dest.left) << 16) / sourceH;
        
        if((ratioCV > 0x10000) || (ratioCH > 0x10000))
            return JERR_BAD_SCALING; 
    }
    
    if((ratioCV < 911) || ((ratioCH < 911)))
        return JERR_BAD_SCALING;
    
    scalerP->accuY= 0;
    scalerP->countY= 0;
    scalerP->firstsourceY= jpegP->image.top;
    scalerP->leftsource= jpegP->image.left;
    scalerP->ratioY= ratioCV;
    scalerP->ratioX= ratioCH;
                
    for(i= 0; i<3; i++)
    {    if((scalerP->ratioY <= 0x8000) && (scalerP->ratioX <= 0x8000))
        {    cinfoP->scale_denom= cinfoP->scale_denom << 1;
            scalerP->ratioY= scalerP->ratioY << 1;
            scalerP->ratioX= scalerP->ratioX << 1;
            scalerP->firstsourceY= scalerP->firstsourceY >> 1;
            scalerP->leftsource= scalerP->leftsource >> 1;
        }
        else
            break;
    }
    
    scalerP->ratioX= scalerP->ratioX + 10;        // Avoid rounding problems
    scalerP->ratioY= scalerP->ratioY + 10;
    
    //ratio check 
    if((scalerP->ratioX < 0x2000) || ((scalerP->ratioY < 0x2000)))
        return JERR_BAD_SCALING;  //KK: 0x2000 ~= 0xFFFF/10
        
    error= jpeg_start_decompress(cinfoP);
    if(error)
        return error;

    for(i= 0; i<1900; i++)
    {    scalerP->Y[i]= 0;
        scalerP->U[i]= 0;
        scalerP->V[i]= 0;
    }
    
    sizeH= ((sourceH * ratioCH) >> 16) & 0xFFFE;
    sizeV= ((sourceV * ratioCV) >> 16) & 0xFFFE;
    scalerP->sizeX= sizeH;
    if(sizeV > (jpegP->dest.bottom-jpegP->dest.top))
        sizeV= jpegP->dest.bottom-jpegP->dest.top;
    scalerP->sizeY= sizeV;
    
    if((jpegP->image.orient != ROT_0) && (jpegP->image.orient != ROT_180))
    {    temp= scalerP->ratioY;
        scalerP->ratioY= scalerP->ratioX;
        scalerP->ratioX= temp;
        scalerP->sizeX= sizeV;
        scalerP->sizeY= sizeH;
    }    
    scalerP->rowlongs= jpegP->dest.rowlongs;
    scalerP->orient= jpegP->image.orient;
    realleft= (jpegP->dest.left + (((jpegP->dest.right-jpegP->dest.left) - sizeH) >> 1)) & 0xFFFE;
    realtop= jpegP->dest.top + (((jpegP->dest.bottom-jpegP->dest.top) - sizeV) >> 1);
    switch(jpegP->image.orient)
    {    case ROT_0:
            scalerP->screenP= (unsigned long*)&jpegP->dest.screenP[(realtop * jpegP->dest.rowlongs + (realleft >> 1)) << 2];
            break;
        case ROT_90:
            scalerP->screenP= (unsigned long*)&jpegP->dest.screenP[(realtop * jpegP->dest.rowlongs + ((realleft + sizeH - 1) >> 1)) << 2];
            break;
        case ROT_180:
            scalerP->screenP= (unsigned long*)&jpegP->dest.screenP[((realtop + sizeV-1) * jpegP->dest.rowlongs + ((realleft + sizeH - 1) >> 1)) << 2];
            break;
        case ROT_270:
            scalerP->screenP= (unsigned long*)&jpegP->dest.screenP[((realtop + sizeV - 1) * jpegP->dest.rowlongs + (realleft >> 1)) << 2];
            break;
    }
    
    if(jpegP->dest.bgcolor)
    {    if(realleft != jpegP->dest.left)
        {    curtainP= (unsigned long*)&jpegP->dest.screenP[(realtop * jpegP->dest.rowlongs) << 2];
            for(i= 0; i < sizeV; i++)
            {    for(n= jpegP->dest.left>>1; n < (realleft>>1); n++)
                    curtainP[n]= jpegP->dest.bgcolor;
                for(n= (realleft+sizeH)>>1; n < (jpegP->dest.right>>1); n++)    
                    curtainP[n]= jpegP->dest.bgcolor;
                curtainP= &curtainP[scalerP->rowlongs];
            }
        }
        if(realtop != jpegP->dest.top)
        {    curtainP= (unsigned long*)&jpegP->dest.screenP[(jpegP->dest.top * jpegP->dest.rowlongs) << 2];
            for(i= 0; i < (realtop-jpegP->dest.top); i++)
            {    for(n= jpegP->dest.left>>1; n < (jpegP->dest.right >> 1); n++)
                    curtainP[n]= jpegP->dest.bgcolor;
                curtainP= &curtainP[scalerP->rowlongs];
            }
            curtainP= &curtainP[scalerP->rowlongs * sizeV];
            for(i= 0; i < (realtop-jpegP->dest.top); i++)
            {    for(n= jpegP->dest.left>>1; n < (jpegP->dest.right >> 1); n++)
                    curtainP[n]= jpegP->dest.bgcolor;
                curtainP= &curtainP[scalerP->rowlongs];
            }
        }
    }
    return 0;
}


void JPEG_Scale(j_decompress_ptr cinfoP, JSAMPIMAGE input_buf, JDIMENSION input_row, JPEG_scaler_t * scalerP)
{
    U16            numcomps= cinfoP->num_components;
    U16            outwidth= cinfoP->output_width;
    U16            left= scalerP->leftsource;
    U16            scanline= cinfoP->output_scanline;
    U32            size= outwidth-left;
    U32            i, c, b, n;
     JSAMPLE        Ybuffer[3800];
    JSAMPLE        Ubuffer[3800];
    JSAMPLE        Vbuffer[3800];
    JSAMPROW    Yline= Ybuffer;
    JSAMPROW    Uline= Ubuffer;
    JSAMPROW    Vline= Vbuffer;
    JSAMPARRAY    image[3]= {0, 0, 0};

    image[0] = &Yline;
    image[1] = &Uline;
    image[2] = &Vline;
    
    if(scalerP->blowup == 1)
        JPEG_ScaleLine(left, numcomps, scanline, outwidth, input_buf, input_row, scalerP);
    else
    {
         if((size * scalerP->blowup) > 3800)
            size= 3800 / scalerP->blowup;
         
         for(c= 0; c<numcomps; c++)
        {
            n= 0;
            for(i= 0; i<size; i++)
            {
                for(b= 0; b<scalerP->blowup; b++)
                {
                    image[c][0][n]= input_buf[c][input_row][left+i];
                    n++;
                }    
            }
        }
        
        for(i= 0; i<scalerP->blowup; i++)
        {
            JPEG_ScaleLine(0, numcomps, scanline, (outwidth-left)*scalerP->blowup, image, 0, scalerP);
        }
    }
}


static void JPEG_ScaleLine(U16 left, U16 numcomps, U16 scanline, U16 outwidth, JSAMPIMAGE input_buf, JDIMENSION input_row, JPEG_scaler_t * scalerP)
{
    long                        Y= 0;
    long                        U= 0;
    long                        V= 0;
    unsigned long                accuH= 0;
    long                        countH= 0;
    unsigned long                iVbuf= 0;
    unsigned long                i;
    volatile unsigned long *    tempP;
    unsigned char *                divP;
    
    
    if((scanline < scalerP->firstsourceY) || (scalerP->sizeY <= 0)) //KK: sizeY==0-> ROT_0 overwrite
        return;
    
    i= left;
    if(numcomps == 3)
    {    while(i < outwidth)
        {    Y= Y + input_buf[0][input_row][i];
            U= U + input_buf[1][input_row][i];
            V= V + input_buf[2][input_row][i];
            i++;
            countH++;
            accuH= accuH + scalerP->ratioX;
            if((accuH > 0x0FFFF) || (i == outwidth))
            {    divP= scalerP->div[countH];      
                scalerP->Y[iVbuf]= scalerP->Y[iVbuf] + divP[Y];
                scalerP->U[iVbuf]= scalerP->U[iVbuf] + divP[U];
                scalerP->V[iVbuf]= scalerP->V[iVbuf] + divP[V];
                Y= 0;
                U= 0;
                V= 0;
                iVbuf++;
                accuH= accuH - 0x10000;
                countH= 0;
                if(iVbuf >= scalerP->sizeX)
                    break;
            }
        }
    }
    else
    {    while(i < outwidth)
        {    Y= Y + input_buf[0][input_row][i];
            i++;
            countH++;
            accuH= accuH + scalerP->ratioX;
            if((accuH > 0x0FFFF) || (i == outwidth))
            {    divP= scalerP->div[countH];      
                scalerP->Y[iVbuf]= scalerP->Y[iVbuf] + divP[Y];
                scalerP->U[iVbuf]= scalerP->U[iVbuf] + 0x80;
                scalerP->V[iVbuf]= scalerP->V[iVbuf] + 0x80;
                Y= 0;
                iVbuf++;
                accuH= accuH - 0x10000;
                countH= 0;
                if(iVbuf >= scalerP->sizeX)
                    break;
            }
        }
    }
    
    scalerP->countY++;
    scalerP->accuY= scalerP->accuY + scalerP->ratioY;
    if((scalerP->accuY > 0x0FFFF) || (scalerP->sizeY == 0))
    {    divP= scalerP->div[scalerP->countY];
        switch(scalerP->orient)
        {    case ROT_0:
                  for(i= 0; i<(scalerP->sizeX>>1); i++)
                {    scalerP->screenP[i]= (((((divP[scalerP->U[i*2]] << 8) | divP[scalerP->Y[i*2]]) << 8) | divP[scalerP->V[i*2]]) << 8) | divP[scalerP->Y[i*2+1]];
                    scalerP->U[i*2]= 0;
                    scalerP->V[i*2]= 0;
                    scalerP->Y[i*2]= 0;
                    scalerP->Y[i*2+1]= 0;
                }
                scalerP->screenP= &scalerP->screenP[scalerP->rowlongs];
                break;

            case ROT_90:
                tempP= scalerP->screenP;
                for(i= 0; i<scalerP->sizeX; i++)
                {    if(scalerP->sizeY & 1)
                        *tempP= (((((divP[scalerP->U[i]] << 8) | divP[scalerP->Y[i]]) << 8) | divP[scalerP->V[i]]) << 8) | scalerP->Y2[i];
                    else 
                        scalerP->Y2[i]= divP[scalerP->Y[i]];
                        scalerP->Y[i]= 0;
                        scalerP->V[i]= 0;
                        scalerP->U[i]= 0;
                        tempP= &tempP[scalerP->rowlongs];
                }
                  if(scalerP->sizeY & 1)
                    scalerP->screenP= &scalerP->screenP[-1];
                break;

            case ROT_180:
                  for(i= 0; i<(scalerP->sizeX>>1); i++)
                {    scalerP->screenP[-i]= (((((divP[scalerP->U[i*2]] << 8) | divP[scalerP->Y[i*2+1]]) << 8) | divP[scalerP->V[i*2]]) << 8) | divP[scalerP->Y[i*2]];
                    scalerP->U[i*2]= 0;
                    scalerP->V[i*2]= 0;
                    scalerP->Y[i*2]= 0;
                    scalerP->Y[i*2+1]= 0;
                }
                scalerP->screenP= &scalerP->screenP[-scalerP->rowlongs];
                break;

            case ROT_270:
                tempP= scalerP->screenP;
                for(i= 0; i<scalerP->sizeX; i++)
                {    if(scalerP->sizeY & 1)
                        *tempP= (((((divP[scalerP->U[i]] << 8) | scalerP->Y2[i]) << 8) | divP[scalerP->V[i]]) << 8) | divP[scalerP->Y[i]];
                    else 
                        scalerP->Y2[i]= divP[scalerP->Y[i]];
                        scalerP->Y[i]= 0;
                        scalerP->V[i]= 0;
                        scalerP->U[i]= 0;
                        tempP= &tempP[-scalerP->rowlongs];
                }
                  if(scalerP->sizeY & 1)
                    scalerP->screenP= &scalerP->screenP[1];
                break;
        }
        scalerP->countY= 0;
        scalerP->accuY= scalerP->accuY - 0x10000;
        scalerP->sizeY--; 
    }
}

