/*!
*****************************************************************************
**
** \file        ./subsystems/paint/osdhandler/src/osdhandler_asm.h
**
** \version     $Id: osdhandler_asm.h 3750 2009-06-04 15:21:43Z rklose $
**
** \brief       Assembly functions header file.
**
** This file contains the declaration of assembly functions with optimized
** cache strategy for the OSDHANDLER.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2007 - 2009 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/

#ifndef OSDHANDLER_ASM_H
    #define OSDHANDLER_ASM_H

    #include <stdio.h>
    #include <stdint.h>
    #include <fapi/sys_error.h>

    //***********************************************************************
    //***********************************************************************
    //** Private Defines and Macros
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Private Data types
    //***********************************************************************
    //***********************************************************************

    typedef struct
    {
        uint16_t x;
        uint16_t y;
        uint16_t w;
        uint16_t h;
        uint32_t c;
    } OSDHANDLER_RectT;


    //***********************************************************************
    //***********************************************************************
    //** Assembly functions with optimized cache strategy
    //***********************************************************************
    //***********************************************************************

    void     OSD_Fill8bpp (uint8_t  *layerStart,
                           OSDHANDLER_RectT *dst,
                           uint32_t layerStride);
    void     OSD_Fill16bpp(uint16_t *layerStart,
                           OSDHANDLER_RectT *dst,
                           uint32_t layerStride);
    void     OSD_Fill32bpp(uint32_t *layerStart,
                           OSDHANDLER_RectT *dst,
                           uint32_t layerStride);
    void     OSD_Blit1bppBEto8bpp(uint8_t *layerStart,
                                  OSDHANDLER_RectT *dst,
                                  uint32_t layerStride,
                                  uint32_t *src);
    void     OSD_Blit1bppLEto8bpp(uint8_t *layerStart,
                                  OSDHANDLER_RectT *dst,
                                  uint32_t layerStride,
                                  uint32_t *src);
    void     OSD_Blit1bppBEto16bpp(uint16_t *layerStart,
                                   OSDHANDLER_RectT *dst,
                                   uint32_t layerStride,
                                   uint32_t *src);
    void     OSD_Blit1bppLEto16bpp(uint16_t *layerStart,
                                   OSDHANDLER_RectT *dst,
                                   uint32_t layerStride,
                                   uint32_t *src);
    void     OSD_Blit1bppBEto32bpp(uint32_t *layerStart,
                                   OSDHANDLER_RectT *dst,
                                   uint32_t layerStride,
                                   uint32_t *src);
    void     OSD_Blit1bppLEto32bpp(uint32_t *layerStart,
                                   OSDHANDLER_RectT *dst,
                                   uint32_t layerStride,
                                   uint32_t *src);

#endif
