/****************************************************************************
 * MODULE     : DsubDisplayDriver
 * FILE       : DsubDisplayDriver.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 ****************************************************************************/

#ifndef DSUBDISPLAYDRIVER_H
#define DSUBDISPLAYDRIVER_H


#include "DsubTDisplayDriver.h"
#include "DsubStddef.h"

/****************************************************************************
 * TYPE_: DsubTDisplayDriver
 ****************************************************************************/

typedef struct DsubTDisplayDriverStruct * DsubTDisplayDriver;



#ifdef __cplusplus
  extern "C" {
#endif



/****************************************************************************
 * FUNCTION: DsubDisplayDriverCreate
 *
 * DESCRIPTION:
 *   This function creates a new DisplayDriver. 
 *
 * RETURN:
 *   A new DispalyDriver or DSUB_NULL if an error occurs.
 *
 * SEE ALSO:
 *   DsubDisplayDriverDestroy  
 ****************************************************************************/

PUBLIC DsubTDisplayDriver 
DsubDisplayDriverCreate (int OsdLayerIdx, int StrechFactor, 
                         DsubTRect newScreenBounds);


/****************************************************************************
 * FUNCTION: DsubDisplayDriverDestroy
 *
 * DESCRIPTION:
 *   This function destroys the DisplayDriver.
 * 
 * PARAMETERS:
 *   This - The DisplayDriver to be destroyed.
 ****************************************************************************/

PUBLIC void 
DsubDisplayDriverDestroy (
  DsubTDisplayDriver This );


/****************************************************************************
 * FUNCTION: DsubDisplayDriverDraw
 *
 * DESCRIPTION:
 *   This function paints a passed bitmap onto the screen.
 *
 * PARAMETERS:
 *   aUserData - The DisplayDriver.
 *   aBitmap   - The bitmap containing a rectangular area to be redrawn.
 *   aSrcRect  - The area within the bitmap to be redrawn.
 *   aDestRect - The destination position of the rect.
 ****************************************************************************/

PUBLIC void
DsubDisplayDriverDraw (
  void *       aUserData,
  DsubTBitmap  aBitmap,
  DsubTRect    aSrcRect,
  DsubTRect    aDestRect );

/****************************************************************************
 * FUNCTION: DsubDisplayDriverSetDisplayBounds
 *
 * DESCRIPTION:
 *   Returns the screen bounds of the DisplayDriver.
 ****************************************************************************/


PUBLIC void 
DsubDisplayDriverSetDisplayBounds (
  void *     aUserData,
  DsubTRect  aDisplayBounds );

/****************************************************************************
 * FUNCTION: DsubDisplayDriverGetScreenBounds
 *
 * DESCRIPTION:
 *   Returns the screen bounds of the DisplayDriver.
 ****************************************************************************/

PUBLIC DsubTRect
DsubDisplayDriverGetScreenBounds ( void );

/****************************************************************************
 * FUNCTION: DsubDisplayDriverSetScreenBounds
 *
 * DESCRIPTION:
 *   Sets the screen bounds of the DisplayDriver.
 ****************************************************************************/

PUBLIC void
DsubDisplayDriverSetScreenBounds ( DsubTRect givenScreenBounds );

/****************************************************************************
 * FUNCTION: DsubDisplayDriverClear
 *
 * DESCRIPTION:
 *   This function must be registered at the DsubDecoder. It clears a 
 *   rectangular area of the window.
 *
 * PARAMETERS:
 *   aUserData - The DisplayDriver.
 *   aRect     - The bounds of the rectangle to be cleared.
 ****************************************************************************/

PUBLIC void 
DsubDisplayDriverClear (
  void *     aUserData,
  DsubTRect  aRect );


/****************************************************************************
 * FUNCTION: DsubDisplayDriverRepaint
 *
 * DESCRIPTION:
 *   This function must be called by the DisplayDriver task. It repaints the
 *   window of the DisplayDriver.
 *
 * PARAMETERS:
 *   This - The DisplayDriver.
 ****************************************************************************/

PUBLIC void 
DsubDisplayDriverRepaint( 
  DsubTDisplayDriver This );


/****************************************************************************
 * FUNCTION: DsubDisplayDriverGetDefaultClut
 *
 * DESCRIPTION:
 *   Returns the default CLUT.
 ****************************************************************************/

PUBLIC const DsubTClutEntry*
DsubDisplayDriverGetDefaultClut( void );

PUBLIC const DsubTClutEntry*
DsubDisplayDriverGetHardwareClut( void );

#ifdef __cplusplus
  }
#endif

#endif /* DSUBDISPLAYDRIVER_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
