/************************************************************************
 * MODULE      : TT_Key
 * FILE        : TT_Key.h
 * COPYRIGHT   : TARA Systems GmbH
 * DESCRIPTION : 
 *   Key definitions for the Teletext decoder
 ***********************************************************************/
#ifndef _TT_KEY_H_
#define _TT_KEY_H_

/************ decoder controlling *****************/
#define TT_DEC_TTX_ON                     0x0100u
#define TT_DEC_TTX_OFF                    0x0101u
#define TT_DEC_RESET                      0x0102u
#define TT_DEC_REDRAW_PAGE                0x0103u
#define TT_DEC_FREEZE_DISPLAY             0x1004u
  /* parameters: OFF/ON */

/*************** decoder settings *****************/
#define TT_DEC_SUBPAGE_OVEWRITE_MODE      0x0200u
  /* parameters: OFF/ON/TOGGLE */

#define TT_DEC_SUBPAGE_MANUAL_MODE        0x0201u
  /* parameters: OFF/ON/TOGGLE */

#define TT_DEC_LANGUAGE_GROUP             0x0202u
  /* parameters: WEST/EAST/RUSSIAN/ARABIC/FARSI */

#define TT_DEC_TELETEXT_LEVEL             0x0203u
  /* parameters: LEVEL15/LEVEL25/LEVEL25_SIDEPANEL */

/* TOP MAY BE PATENT RELATED */
#define TT_DEC_COLOR_KEY_MODE             0x0204u
  /* parameters: NONE/TOP_ENA/FLOF_ENA/TOP_FLOF_ENA */
  /* default: FLOF_ENA */

#define TT_DEC_BEYOND_MAGAZINE            0x2005u
  /* parameters: OFF/ON */
  /* default: ON */

#define TT_DEC_SET_TIMER                  0x2006u
  /* parameter 1: timer type */
  /* parameter 2: timer value in milli seconds (16bit) */
  /* If the highest bit is set the value is interpreted 
     as seconds instead of milli seconds. */

#define TT_DEC_SUBPAGE_BAR_TYPE           0x2007u
  /* parameter 1: subpage bar type */

#define TT_DEC_SUBPAGE_BAR_COLOR          0x2008u
  /* parameter 1: color type */
  /* parameter 2: color value as color index */

#define TT_DEC_ZOOM_TYPE                  0x2009u
  /* parameter 1: zoom type */


/**************** standard keys *******************/
#define TT_DEC_KEY_NONE                   0x0001u
#define TT_DEC_KEY_LEFT                   0x0008u
#define TT_DEC_KEY_RIGHT                  0x0009u
#define TT_DEC_KEY_DOWN                   0x000Au
#define TT_DEC_KEY_UP                     0x000Bu
#define TT_DEC_KEY_OK                     0x000Du
#define TT_DEC_KEY_CUSTOM1                0x0010u
#define TT_DEC_KEY_CUSTOM2                0x0011u
#define TT_DEC_KEY_RED                    0x001Cu
#define TT_DEC_KEY_GREEN                  0x001Du
#define TT_DEC_KEY_CYAN                   0x001Eu
#define TT_DEC_KEY_YELLOW                 0x001Fu
#define TT_DEC_KEY_NUM_0                  0x0030u
#define TT_DEC_KEY_NUM_1                  0x0031u
#define TT_DEC_KEY_NUM_2                  0x0032u
#define TT_DEC_KEY_NUM_3                  0x0033u
#define TT_DEC_KEY_NUM_4                  0x0034u
#define TT_DEC_KEY_NUM_5                  0x0035u
#define TT_DEC_KEY_NUM_6                  0x0036u
#define TT_DEC_KEY_NUM_7                  0x0037u
#define TT_DEC_KEY_NUM_8                  0x0038u
#define TT_DEC_KEY_NUM_9                  0x0039u
#define TT_DEC_DIGIT                      TT_DEC_KEY_NUM_0
#define TT_DEC_KEY_INDEX                  0x0040u

/************** decoder navigation ****************/
#define TT_DEC_DIGIT_CANCEL               0x0300u
#define TT_DEC_SUBPAGE_MANUAL_ON          0x0301u
#define TT_DEC_SUBPAGE_MANUAL_OFF         0x0302u
#define TT_DEC_SUBPAGE_PREVIOUS           0x0303u
#define TT_DEC_SUBPAGE_NEXT               0x0304u
#define TT_DEC_PAGE_PREVIOUS              0x0305u
#define TT_DEC_PAGE_NEXT                  0x0306u

#define TT_DEC_HOLD                       0x0307u
  /* parameters: OFF/ON/TOGGLE */

#define TT_DEC_REVEAL                     0x0308u
  /* parameters: OFF/ON/TOGGLE */

#define TT_DEC_CANCEL                     0x0309u
  /* parameters: OFF/ON/TOGGLE */

/* MAY BE PATENT RELATED */
#define TT_DEC_LIST                       0x030Au
  /* parameters: OFF/ON/TOGGLE */

#define TT_DEC_MIX                        0x030Bu
  /* parameters: OFF/ON/TOGGLE */

#define TT_DEC_CLOCK                      0x030Cu
  /* parameters: OFF/ON/TOGGLE */

#define TT_DEC_SIZE                       0x030Du
  /* parameters: TOGGLE/NORMAL/UPPER/LOWER */

#define TT_DEC_SUBTITLE_MODE              0x030Eu
  /* parameters: none */

#define TT_DEC_DISPLAY_PAGE               0x030Fu
  /* parameter 1: page no */
  /* parameter 2: subpage no (0xFFFF for don't care) */
  /* PageProcesses of TT_DEC_STATUS is set to 1 */

#define TT_DEC_REQUEST_PAGE               0x0310u
  /* parameter 1: page no */
  /* parameter 2: subpage no (0xFFFF for don't care) */
  /* content is written to page request buffer. */
  /* TT_DecoderSetPageRequestBuffer() to set page request buffer. */
  /* RequestedPageAvailable of TT_DEC_STATUS is set to 1. */
  /* buffer format: each line has 42 bytes */
  /* - 2 bytes packet address (already hamming encrypted) */
  /* - 40 bytes data (with paritiy information */

#define TT_DEC_CANCEL_ROLLING_HEADER      0x0311u
  /* parameters: none */

#define TT_DEC_SUBPAGE_INPUT              0x0312u
  /* parameters: OFF/ON/TOGGLE */

#define TT_DEC_PAGE_BACK                  0x0313u
  /* parameters: none */
  /* goes to previously selected page */

#define TT_DEC_STATUS_DISPLAY             0x0314u
  /* parameters: OFF/ON/TOGGLE */

#define TT_DEC_DEBUG_DISPLAY              0x0315u
  /* parameters: OFF/ON/TOGGLE */


/****************** page catching *****************/
/* MAY BE PATENT RELATED */
#define TT_DEC_PAGE_CATCH_OFF             0x0400u
#define TT_DEC_PAGE_CATCH_ON              0x0401u
#define TT_DEC_PAGE_CATCH_UP              0x0402u
#define TT_DEC_PAGE_CATCH_DOWN            0x0403u
#define TT_DEC_PAGE_CATCH_LEFT            0x0404u
#define TT_DEC_PAGE_CATCH_RIGHT           0x0405u
#define TT_DEC_PAGE_CATCH_SELECT          0x0406u

/******************** top menu ********************/
/* MAY BE PATENT RELATED */
#define TT_DEC_TOP_MENU_OFF               0x0500u
#define TT_DEC_TOP_MENU_ON                0x0501u
#define TT_DEC_TOP_MENU_UP                0x0502u
#define TT_DEC_TOP_MENU_DOWN              0x0503u
#define TT_DEC_TOP_MENU_LEFT              0x0504u
#define TT_DEC_TOP_MENU_RIGHT             0x0505u
#define TT_DEC_TOP_MENU_PREVIOUS          0x0506u
#define TT_DEC_TOP_MENU_NEXT              0x0507u
#define TT_DEC_TOP_MENU_SELECT            0x0508u

/********* ACI (auto channel installation) *******/
/* MAY BE PATENT RELATED */
#define TT_DEC_ACI_START                  0x0600u
#define TT_DEC_ACI_STOP                   0x0601u
#define TT_DEC_ACI_UPDATE                 0x0602u
#define TT_DEC_SET_ACI_LANGUAGE           0x0603u

/*************************************************/
/* parameter definitions                         */
/*************************************************/

/***************** display modes *****************/
#define TT_DEC_OFF                        0x0000u
#define TT_DEC_ON                         0x0001u
#define TT_DEC_TOGGLE                     0x0002u


/******************* zoom mode *******************/
#define TT_DEC_SIZE_TOGGLE                0x0000u
#define TT_DEC_SIZE_NORMAL                0x0001u
#define TT_DEC_SIZE_UPPER                 0x0002u
#define TT_DEC_SIZE_LOWER                 0x0003u


/*************** TTX runtime level ***************/
#define TT_DEC_LEVEL10                    0x0000u
#define TT_DEC_LEVEL15                    0x0001u
#define TT_DEC_LEVEL25                    0x0002u
#define TT_DEC_LEVEL25_SIDEPANEL          0x0003u


/**************** navigation mode ****************/
#define TT_DEC_NONE                       0x0000u
#define TT_DEC_TOP_ENA                    0x0001u
#define TT_DEC_FLOF_ENA                   0x0002u
#define TT_DEC_TOP_FLOF_ENA               0x0003u
#define TT_DEC_FLOF_TOP_ENA               0x0007u


/**************** language group *****************/
#define TT_DEC_LG_WEST                    0x0000u
#define TT_DEC_LG_EAST                    0x0001u
#define TT_DEC_LG_RUSSIAN                 0x0002u
#define TT_DEC_LG_ARABIC                  0x0003u
#define TT_DEC_LG_FARSI                   0x0004u


/**************** timer type *********************/
#define TT_DEC_TIMER_HEADER_DISPLAY       0x0000u
#define TT_DEC_TIMER_USER_INPUT           0x0001u
#define TT_DEC_TIMER_DIGIT_INPUT          0x0002u
#define TT_DEC_TIMER_PAGE_INFO_VISIBLE    0x0003u
#define TT_DEC_TIMER_HEX_PAGE_CHECK       0x0004u
#define TT_DEC_TIMER_DATABASE_CHECK       0x0005u


/************ subpage bar type *******************/
#define TT_DEC_SUBPAGEBAR_DYNAMIC_DISPLAY 0x0001u
#define TT_DEC_SUBPAGEBAR_FIXED_DISPLAY   0x0002u


/************ subpage bar color type *************/
#define TT_DEC_SUBPAGEBAR_BG              0x0000u
#define TT_DEC_SUBPAGEBAR_FG              0x0001u
#define TT_DEC_SUBPAGEBAR_FG_RECEIVED     0x0002u
#define TT_DEC_SUBPAGEBAR_FG_NOT_RECEIVED 0x0003u
#define TT_DEC_SUBPAGEBAR_BG_CURSOR       0x0004u
#define TT_DEC_SUBPAGEBAR_FG_CURSOR       0x0005u


/*************** zoom type ***********************/
#define TT_DEC_ZOOM_TYPE_0_12             0x0000u
#define TT_DEC_ZOOM_TYPE_0_11             0x0001u



#endif /* _TT_KEY_H_ */

