/*
*****************************************************************************
**
** \file        ./subsystems/video/val/src/val.c
**
** \version     $Id: val.c 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       Video abstraction interface suitable for live and PVR playback.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

/*
*******************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    716: while ( 1 )
** - Info    717: do ... while(0);
** - Info    788: enum constant 'anyConst' not used within defaulted switch
**
*******************************************************************************
*/
/*lint -save -e716 -e717 -e788 */
//#define     DESCRAMBLE_TEST
#define DEBUG 0

#define VAL_FASTMODE_SPEEDUP

#include <stdio.h>

#include <fapex/abst_types.h>
#include <fapex/abst_libvienc.h>
#include <fapex/abst_libviout.h>
#include <fapex/abst_libvidec.h>
#include <fapex/abst_libtsd.h>
#include <fapex/abst_libhdmi.h>

#include "val.h"

//*****************************************************************************
//*****************************************************************************
//** Defines and macros
//*****************************************************************************
//*****************************************************************************

#define VAL_THREAD_STACKSIZE                8192        //!< Stacksize in bytes.
#define VAL_THREAD_NAME                     "VAL:HDMI"  //!< Thread name.
#define VAL_THREAD_SLEEP_TIME               50          //!< In ms.
#define VAL_THREAD_WAIT_SLEEP_TIME          10          //!< In ms.
#define VAL_THREAD_POLL_CONNECTED           200         //!< In ms.
#define VAL_THREAD_POLL_HDCP_AUTHENTICATE   2000        //!< In ms.
#define VAL_THREAD_POLL_DISCONNECTED        200         //!< In ms.

#define VAL_TASK_H264_AUD_TIMEOUT           1000        //!< In ms.

#define VAL_DEFAULT_HDSYS               VAL_1920x1080x50I
#define VAL_DEFAULT_SDSYS               VAL_SD_PAL
#define VAL_DEFAULT_AR_SD               FAPEX_VIOUT_DISPLAY_AR_NONE
#define VAL_DEFAULT_AR_HD               FAPEX_VIOUT_DISPLAY_AR_16_9
#define VAL_DEFAULT_SM_HD               FAPEX_VIOUT_SCALER_MODE_PLBOX
#define VAL_DEFAULT_ENABLE_HDCP         0

#define VAL_HDMI_HPD_STATUS_MASK        0x04
#define VAL_HDMI_HPD_STATUS_CONNECTED   0x04


//*****************************************************************************
//*****************************************************************************
//** Global functions
//*****************************************************************************
//*****************************************************************************

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************
int32_t        fd_VIOUT = -1;
extern int32_t fd_TSD[];

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************
#define VIDEO_TSD_CHANNEL     9
#define DRV_ID_TSD0_VIDEO_CH  (FAPEXDEV_TSD0_SUB_MINOR_FIRST + VIDEO_TSD_CHANNEL)
#define DRV_ID_VIOUT          (FAPEXDEV_VIOUT_CMN_MINOR)
#define DRV_ID_VIOUT_HD       (FAPEXDEV_VIOUT_SUB_MINOR_FIRST + 0)
#define DRV_ID_VIOUT_SD       (FAPEXDEV_VIOUT_SUB_MINOR_FIRST + 1)
#define DRV_ID_VIDEC          (FAPEXDEV_VIDEC_MINOR_FIRST)
#define DRV_ID_VIENC          (FAPEXDEV_VIENC_CMN_MINOR)
#define DRV_ID_VIENC_HD       (FAPEXDEV_VIENC_SUB_MINOR_FIRST + 0)
#define DRV_ID_VIENC_SD       (FAPEXDEV_VIENC_SUB_MINOR_FIRST + 1)
#define DRV_ID_HDMI           (FAPEXDEV_HDMI_MINOR_FIRST)

//*****************************************************************************
//*****************************************************************************
//** Local data
//*****************************************************************************
//*****************************************************************************

static VAL_OpenT    openDat;
static VAL_DataT*   gnrDataPtr  = NULL;

//*****************************************************************************
//*****************************************************************************
//** Local Functions Declaration
//*****************************************************************************
//*****************************************************************************

static int32_t valIntGoUp         (VAL_DataT* dataPtr,
                                   VAL_HdEncoderT hdSystemType,
                                   VAL_SdEncoderT sdSystemType);
static int32_t valIntGoDown       (VAL_DataT* dataPtr);
static int32_t valIntOpen         (VAL_OpenT* openPtr,
                                   const VAL_OpenParamsT* paramsPtr);
static int32_t valIntClose        (VAL_OpenT* openPtr);
static int32_t valIntStart        (VAL_OpenT* openPtr, uint32_t pid,
                                   VAL_PlayModeEnumT mode);
static int32_t valIntStop         (VAL_OpenT* openPtr, uint8_t freeze);
static int32_t valIntReboot       (VAL_OpenT* openPtr,
                                   const VAL_DecoderTypeT* decoderPtr);
static int32_t valArgCheckOpen    (const VAL_OpenT* openPtr);

static int32_t valViencInitClocks (VAL_HdEncoderT hdSys);
static int32_t valViencOpen       (VAL_DataT* dataPtr,
                                   VAL_HdEncoderT hdSys,
                                   VAL_SdEncoderT sdSys);
static int32_t valViencClose      (VAL_DataT* dataPtr);

static int32_t valHdmiOpen        (VAL_DataT* dataPtr);
static int32_t valHdmiClose       (VAL_DataT* dataPtr);
static int32_t valHdmiUpdateStatus(VAL_DataT* dataPtr, uint8_t* connectedPtr,
                                   uint8_t* changePtr);
static int32_t valHdmiHandleHDCP  (VAL_DataT* dataPtr);
static int32_t valHdmiGetEDIDInfo (VAL_DataT* dataPtr);

static int32_t valViscaleOpen     (VAL_DataT* dataPtr,
                                   VAL_HdEncoderT hdSys);
static int32_t valViscaleClose    (VAL_DataT* dataPtr);
static void    valViscaleParamChgCb(void* paramPtr,
                                    void* optData);

static int32_t valChannelOpen  (VAL_OpenT* openPtr);
static int32_t valChannelClose (VAL_OpenT* openPtr);
static int32_t valChannelStart (VAL_OpenT* openPtr, uint16_t pid);
static int32_t valChannelStop  (VAL_OpenT* openPtr);

static int32_t valVidecOpen       (VAL_OpenT* openPtr,
                                   const VAL_DecoderTypeT* decoderPtr,
                                   uint8_t enableAfd,
                                   VAL_EnvironmentT environment,
                                   int32_t delimiterType);
static int32_t valVidecClose      (VAL_OpenT* openPtr);
static int32_t valVidecStart      (VAL_OpenT* openPtr,
                                   int32_t mode,
                                   uint8_t feedViaTsd);
static int32_t valVidecStop       (VAL_OpenT* openPtr, uint8_t freeze);
static void    valVidecFrameAvlCb (void* optData);
static void    valVidecFrameDecCb (void* optData);
static void    valVidecFrameInfoCb(void* optData);
static void    valVidecParamChgCb (void* optData);

//*****************************************************************************
//*****************************************************************************
//** Local Functions
//*****************************************************************************
//*****************************************************************************

/*!
*******************************************************************************
** \brief   Initialize video interface library.
**
** \param   paramsPtr   Initialization parameters.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_ALREADY_INITIALIZED  Already initialized.
**
** \note    This function shall be called once at system startup. Use VAL_GoUp
**          and VAL_GoDown to perform transitions between active and standby
**          mode.
**
** \sa      VAL_GoUp, VAL_GoDown
*******************************************************************************
*/
int32_t VAL_Init (const VAL_InitParamsT* paramsPtr)
{
    int32_t err_code;

    if ( gnrDataPtr != NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_ALREADY_INITIALIZED;
    }
#if 0 /* non support VAL_InitParamsT */
    else if ( paramsPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
    }
#endif
    else
    {
        err_code = FAPI_OK;

        do
        {
            gnrDataPtr = (VAL_DataT*)FAPI_SYS_MALLOC_CACHED(sizeof(VAL_DataT));

            if ( gnrDataPtr == NULL )
            {
                err_code = SUBSYS_VIDEO_VAL_ERR_OUT_OF_MEMORY;
                break;
            }

            memset (gnrDataPtr, 0, sizeof(VAL_DataT));

            /* create semaphore for mutual exclusion purposes */
            gnrDataPtr->mutexHandle = FAPEX_SYS_CreateSemaphore (1);
            if ( gnrDataPtr->mutexHandle == NULL )
            {
                err_code = SUBSYS_VIDEO_VAL_ERR_SEMAPHORE_CREATE;
                break;
            }

            /* create semaphore for communication with VAL thread */
            gnrDataPtr->triggerHandle = FAPEX_SYS_CreateSemaphore (0);
            if ( gnrDataPtr->triggerHandle == NULL )
            {
                err_code = SUBSYS_VIDEO_VAL_ERR_SEMAPHORE_CREATE;
                break;
            }

            gnrDataPtr->threadSleepReq           = 0;
            gnrDataPtr->threadSleepAck           = 0;

            gnrDataPtr->hdmiId                   = -1;
            gnrDataPtr->viencId                  = -1;
            gnrDataPtr->viencSubId[VAL_VOUT_SD]  = -1;
            gnrDataPtr->viencSubId[VAL_VOUT_HD]  = -1;
            gnrDataPtr->vioutId                  = -1;
            gnrDataPtr->vioutSubId[VAL_VOUT_SD]  = -1;
            gnrDataPtr->vioutSubId[VAL_VOUT_HD]  = -1;

            gnrDataPtr->appHdSystemType          = VAL_DEFAULT_HDSYS;
            gnrDataPtr->appSdSystemType          = VAL_DEFAULT_SDSYS;
            gnrDataPtr->viencHdEncoder           = VAL_DEFAULT_HDSYS;
            gnrDataPtr->viencSdEncoder           = VAL_DEFAULT_SDSYS;
            gnrDataPtr->vioutDispAr[VAL_VOUT_SD] = VAL_DEFAULT_AR_SD;
            gnrDataPtr->vioutDispAr[VAL_VOUT_HD] = VAL_DEFAULT_AR_HD;
            gnrDataPtr->vioutHdScalingMethod     = VAL_DEFAULT_SM_HD;
            gnrDataPtr->hdmiHDCPEnabled          = VAL_DEFAULT_ENABLE_HDCP;

        }
        while ( 0 );

        if ( err_code != FAPI_OK )
        {
            if ( gnrDataPtr != NULL )
            {
                if ( gnrDataPtr->triggerHandle != NULL )
                {
                    (void)FAPEX_SYS_DestroySemaphore (gnrDataPtr->triggerHandle);
                }

                if ( gnrDataPtr->mutexHandle != NULL )
                {
                    (void)FAPEX_SYS_DestroySemaphore (gnrDataPtr->mutexHandle);
                }

                FAPI_SYS_FREE(gnrDataPtr);

                gnrDataPtr = NULL;
            }
        }
    }

    FAPI_SYS_PRINT_DEBUG(2, "[VAL] VAL_Init: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Initialize and open video output.
**
** \param   paramsPtr   Default video parameters.
** \param   errCodePtr  Optionally returns error_status. Set to NULL if not
**                      needed.
**
** \return  NULL in case of failure, handle for further usage on success.
**
** \note    This shall be called after returning from standby. The returned
**          pointer is used for general video output operations and is passed
**          to VAL_Open.
**
** \sa      VAL_GoDown, VAL_Open
*******************************************************************************
*/
VAL_DataT* VAL_GoUp (const VAL_UpParamsT* paramsPtr, int32_t* errCodePtr)
{
    VAL_DataT*  dataPtr = gnrDataPtr;
    int32_t     err_code = FAPI_OK;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_NOT_INITIALIZED;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( !(dataPtr->up) )
        {
            if ( paramsPtr != NULL )
            {
                dataPtr->appHdSystemType = paramsPtr->hdSystemType;
                dataPtr->appSdSystemType = paramsPtr->sdSystemType;
                dataPtr->viencHdEncoder  = paramsPtr->hdSystemType;
                dataPtr->viencSdEncoder  = paramsPtr->sdSystemType;
                dataPtr->vioutDispAr[VAL_VOUT_SD] = paramsPtr->displayAspectRatio[VAL_VOUT_SD];
                dataPtr->vioutDispAr[VAL_VOUT_HD] = paramsPtr->displayAspectRatio[VAL_VOUT_HD];
                dataPtr->vioutHdScalingMethod     = paramsPtr->scalingMethod;
                dataPtr->hdmiHDCPEnabled          = paramsPtr->enableHDCP;
                dataPtr->hdmiPictureAr            = FAPEX_VIOUT_DISPLAY_AR_NONE;
                dataPtr->hdmiActiveFormat         = 0;
            }

            dataPtr->videoBlanked                = 0;
            dataPtr->hdmiConnectParams.connected = 0;
            dataPtr->hdmiConnectParams.haveEDID  = 0;

            dataPtr->hdmiHDCPState   = VAL_HDMI_HDCPSTATE_AUTH_INIT;

            err_code = valIntGoUp (dataPtr, dataPtr->appHdSystemType,
                                    dataPtr->appSdSystemType);

            if ( err_code == FAPI_OK )
            {
                dataPtr->up      = 1;
                dataPtr->openPtr = NULL;
            }
        }

        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    FAPI_SYS_PRINT_DEBUG(2, "[VAL] VAL_GoUp: %i\n", err_code);

    if ( err_code != FAPI_OK )
    {
        dataPtr = NULL;
    }

    return dataPtr;
}

/*!
*******************************************************************************
** \brief   Close video output.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  Invalid handle.
**
** \note    This shall be called before entering standby.
**
** \sa      VAL_GoUp
*******************************************************************************
*/
int32_t VAL_GoDown (VAL_DataT* dataPtr)
{
    int32_t err_code = FAPI_OK;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( dataPtr->up )
        {
            /* close if still open */
            if ( dataPtr->openPtr != NULL )
            {
                err_code = valIntClose(dataPtr->openPtr);

                /* mark instance as closed */
                dataPtr->openPtr = NULL;
            }
            if(err_code == FAPI_OK) {
                err_code = valIntGoDown(dataPtr);

                dataPtr->up = 0;
            }
        }

        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_GoDown: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Check HDMI connection and proceed HDCP authentication protocol
**
** This function shall only be used if HDMI handling in a separate thread has
** been disabled by setting threadPriority in VAL_InitParamsT to 0.
**
** It basically does the same handling, also callbacks set by
** VAL_SetHDMIConnectionCb will be called in the context of this function.
**
** A calling interval of ca. 100 ms is needed as long as an HDMI connection
** is active and HDCP handling is requested. Please note that this module
** only handles HDMI video. HDMI audio must be restarted on connection as well
** (see AAL layer). Use the callback provided by VAL_SetHDMIConnectionCb in
** order to properly restart HDMI audio on connection.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   connectedPtr    Returns HDMI connection information if a non-NULL
**                          pointer is passed (1: connected, 0: not connected).
** \param   changePtr       Returns HDMI connection change indication if a
**                          non-NULL pointer is passed (1: connection status
**                          has changed since last calling of this function,
**                          0: not changed).
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  Invalid handle.
**
*******************************************************************************
*/
int32_t VAL_UpdateHDMIConnectionStatus (VAL_DataT* dataPtr,
                                        uint8_t* connectedPtr,
                                        uint8_t* changePtr)
{
    int32_t err_code = FAPI_OK;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else if ( dataPtr->vioutId < 0 )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        err_code = valHdmiUpdateStatus(dataPtr, connectedPtr, changePtr);

        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Return current HDMI connection status and EDID availability status.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   paramsPtr       Returns information if a non-NULL pointer is passed.
**
** \return  Possible return codes:
**          - FAPI_OK                              Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  NULL-pointer passed as
**                                                 dataPtr.
*******************************************************************************
*/
int32_t VAL_GetHDMIConnectionStatus (VAL_DataT* dataPtr,
                                     VAL_HDMIConnectParamsT* paramsPtr)
{
    int32_t err_code;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        if ( paramsPtr != NULL )
        {
            *paramsPtr = dataPtr->hdmiConnectParams;
        }

        err_code = FAPI_OK;
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Set notification function to be informed about changes in HDMI
**          connection and EDID availability status.
**
** \note Please note that this module only handles HDMI video. HDMI audio
**       must be restarted on connection as well (see AAL layer). Use the
**       callback provided by this function in order to properly restart
**       HDMI audio on connection.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   cb              Notification function, NULL disables notification.
** \param   cbArg           Optional parameter passed with callback function.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
*******************************************************************************
*/
int32_t VAL_SetHDMIConnectionCb (VAL_DataT* dataPtr,
                                 VAL_HDMIConnectCallbackT cb, void* cbArg)
{
    int32_t err_code;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        dataPtr->hdmiConnectCb    = cb;
        dataPtr->hdmiConnectCbArg = cbArg;

        (void)FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

        err_code = FAPI_OK;
    }

    FAPI_SYS_PRINT_DEBUG(2, "[VAL] VAL_SetHDMIConnectionCb: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Return current HDMI EDID information.
**
** \param   dataPtr    Handle returned by VAL_GoUp.
** \param   edidPtr    Returns EDID information if a non-NULL pointer
**                     is passed (see HDMI driver for details).
**
** \return  Possible return codes:
**          - FAPI_OK                                 Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_DATA_NOT_AVAILABLE EDID infmation not available.
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE     NULL-pointer passed as dataPtr.
*******************************************************************************
*/
int32_t VAL_GetHDMI_EDID (VAL_DataT* dataPtr, VAL_HDMI_EdidInfoStrT* edidPtr)
{
    int32_t err_code;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( !(dataPtr->hdmiConnectParams.haveEDID) )
        {
            err_code = SUBSYS_VIDEO_VAL_ERR_DATA_NOT_AVAILABLE;
        }
        else
        {
            if ( edidPtr != NULL )
            {
                *edidPtr = dataPtr->hdmiEdid;
            }

            err_code = FAPI_OK;
        }

        (void)FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Modifies SD and HD video output system.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   hdSystemType    Requested HD system type.
** \param   sdSystemType    Requested SD system type.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  dataPtr is invalid.
*******************************************************************************
*/
int32_t VAL_SetOutputMode (VAL_DataT* dataPtr, VAL_HdEncoderT hdSystemType,
                           VAL_SdEncoderT sdSystemType)
{
    VAL_OpenT*          openPtr       = NULL;
    int32_t             err_code      = FAPI_OK;
    VAL_OpEnumT         currOperation = VAL_OP_NONE;
    uint32_t            currPid       = 0;
    VAL_PlayModeEnumT   currMode      = VAL_PLAYMODE_NONE;
    VAL_OpenParamsT     openParams;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        memset(&openParams, 0, sizeof(openParams)); /* satisfy MISRA */

        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        do
        {
            if ( dataPtr->openPtr != NULL )
            {
                openPtr = dataPtr->openPtr;

                currOperation = openPtr->operation;
                currPid       = openPtr->pid;
                currMode      = openPtr->mode;
                openParams    = openPtr->openParams;

                do
                {
                    /* Stop potentially running decoder. */
                    err_code = valIntStop(openPtr, 0);
                    if ( err_code != FAPI_OK )
                    {
                        break;
                    }

                    err_code = valIntClose(openPtr);
                    if ( err_code != FAPI_OK )
                    {
                        break;
                    }
                }
                while ( 0 );

                if ( err_code != FAPI_OK )
                {
                    openPtr->operation = VAL_OP_ERROR;
                    break;
                }
            }

            err_code = valIntGoDown(dataPtr);
            if(err_code != FAPI_OK) {
                break;
            }

            err_code = valIntGoUp  (dataPtr, hdSystemType, sdSystemType);
            if(err_code != FAPI_OK) {
                break;
            }

            dataPtr->appHdSystemType = hdSystemType;
            dataPtr->appSdSystemType = sdSystemType;

            if ( openPtr != NULL )
            {
                do
                {
                    err_code = valIntOpen(openPtr, &openParams);
                    if ( err_code != FAPI_OK )
                    {
                        break;
                    }

                    if ( (currOperation == VAL_OP_LIVE)
                      || (currOperation == VAL_OP_PLAY)
              || (currOperation == VAL_OP_STREAMING) )
                    {
                        /* Restart video with parameters from previous start. */

                        err_code = valIntStart (openPtr, currPid, currMode);

                        if ( err_code != FAPI_OK )
                        {
                            break;
                        }
                    }
                }
                while ( 0 );

                if ( err_code != FAPI_OK )
                {
                    openPtr->operation = VAL_OP_ERROR;
                    break;
                }
            }

        } while ( 0 );

        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_SetOutputMode: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Reads the SD and HD video output system.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   hdSystemType    Pointer to store the HD system type (optional.
** \param   sdSystemType    Pointer to store the SD system type (optional).
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  dataPtr is invalid.
*******************************************************************************
*/
int32_t VAL_GetOutputMode (VAL_DataT* dataPtr,
                           VAL_HdEncoderT* hdSystemType,
                           VAL_SdEncoderT* sdSystemType)
{
    if ( dataPtr == NULL )
    {
        return SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }

    if ( hdSystemType != NULL)
    {
        *hdSystemType = dataPtr->appHdSystemType;
    }
    if ( sdSystemType != NULL)
    {
        *sdSystemType = dataPtr->appSdSystemType;
    }
    return FAPI_OK;
}

/*!
*******************************************************************************
** \brief   Set the delay in video display.
**
** \param   dataPtr           Handle returned by VAL_GoUp.
** \param   outputDelayMsec   Requested the delay in milliseconds
**
** \return  Possible return codes:
**          - FAPI_OK                              Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  NULL-pointer passed as
**                                                 openPtr.
*******************************************************************************
*/
int32_t VAL_SetOutputDelay (VAL_DataT* dataPtr, int32_t outputDelayMsec)
{
    int32_t err_code = FAPI_OK;
    VAL_OpenT*  openPtr;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        openPtr = (VAL_OpenT*)(dataPtr->openPtr);

        if ( openPtr->videcId >= 0 )
        {
            err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                         FAPEX_IOCCOM_VIDEC_SET_OUTPUT_DELAY,
                                         (FAPEXIOCARG3_T)outputDelayMsec);
        }
        else
        {
            err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
        }

        (void)FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[VAL] VAL_SetOutputDelay: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Set user-defined display aspect ratio.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
** \param   output      Video output to set aspect ratio for.
** \param   displayAr   Requested display aspect ratio.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  dataPtr is invalid.
*******************************************************************************
*/
int32_t VAL_SetDisplayAspectRatio (VAL_DataT* dataPtr,
                                   VAL_VideoOutputT output,
                                   int32_t displayAr)
{
    int32_t err_code;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else if ( output >= VAL_VOUT_NUMS )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        dataPtr->vioutDispAr[output] = displayAr;

        err_code = FAPEX_VIOUT_Ioctl(dataPtr->vioutSubId[output],
                                     FAPEX_IOCCOM_VIOUT_SET_ASPECT_RATIO,
                                     (FAPEXIOCARG3_T)dataPtr->vioutDispAr[output]);

        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

        err_code = FAPI_OK;
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_SetDisplayAspectRatio: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Get user-defined display aspect ratio.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
** \param   output      Video output to set aspect ratio for.
** \param   displayAr   Pointer to store the display aspect ratio.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  dataPtr is invalid.
*******************************************************************************
*/
int32_t VAL_GetDisplayAspectRatio (VAL_DataT* dataPtr,
                                   VAL_VideoOutputT output,
                                   int32_t* displayAr)
{
    if(dataPtr == NULL )
    {
        return SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else if ( output >= VAL_VOUT_NUMS )
    {
        return SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
    }
    else if(displayAr == NULL)
    {
        return SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
    }
    *displayAr = dataPtr->vioutDispAr[output];

    return FAPI_OK;
}

/*!
*******************************************************************************
** \brief   Set scaling method of HD scaler.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   scalingMethod   Requested scaling method.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  dataPtr is invalid.
*******************************************************************************
*/
int32_t VAL_SetHdScalingMethod (VAL_DataT* dataPtr,
                                int32_t scalingMethod)
{
    FAPEX_VIOUT_ScalerParamStrT  *spPtr;
    int32_t err_code;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        spPtr = dataPtr->vioutParams + (int32_t)VAL_VOUT_HD;

        spPtr->mode               = scalingMethod;
        spPtr->osdByteOrder       = FAPEX_VIOUT_OSD_LAYOUT_UNKNOWN;
        spPtr->cpTileNumX         = 0;
        spPtr->cpTileNumY         = 0;
        spPtr->flagFrameField     = 0;
        spPtr->inputWindowX0      = 0;
        spPtr->inputWindowY0      = 0;
        spPtr->inputWindowWidth   = 0;
        spPtr->inputWindowHeight  = 0;
        spPtr->outputWindowX0     = 0;
        spPtr->outputWindowY0     = 0;
        spPtr->outputWindowWidth  = 0;
        spPtr->outputWindowHeight = 0;

        err_code = FAPEX_VIOUT_Ioctl(dataPtr->vioutSubId[VAL_VOUT_HD],
                                     FAPEX_IOCCOM_VIOUT_SET_SCALER_PARAMS,
                                     (FAPEXIOCARG3_T)spPtr);
        if ( err_code == FAPI_OK )
        {
            dataPtr->vioutHdScalingMethod = scalingMethod;
        }

        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_SetHdScalingMethod: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Get scaling method of HD scaler.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   scalingMethod   Pointer to store the scaling method.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  dataPtr is invalid.
*******************************************************************************
*/
int32_t VAL_GetHdScalingMethod (VAL_DataT* dataPtr,
                                int32_t* scalingMethod)
{
    if(dataPtr == NULL )
    {
        return SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else if(scalingMethod == NULL)
    {
        return SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
    }
    *scalingMethod = dataPtr->vioutHdScalingMethod;

    return FAPI_OK;
}

/*!
*******************************************************************************
** \brief   Set parameters for video output window.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
** \param   x0          Horizontal start position in pixels.
** \param   y0          Vertical start position in pixels.
** \param   width       Width in pixels.
** \param   height      Height in pixels.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  dataPtr is invalid.
**
** \sa      VAL_Crop
*******************************************************************************
*/
int32_t VAL_Resize (VAL_DataT* dataPtr, uint32_t x0, uint32_t y0,
                    uint32_t width, uint32_t height)
{
    int32_t                      err_code;
    FAPEX_VIOUT_ScalerParamStrT* spPtr;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        spPtr = dataPtr->vioutParams + (int32_t)VAL_VOUT_HD;

        spPtr->outputWindowX0     = (uint32_t)x0;
        spPtr->outputWindowY0     = (uint32_t)y0;
        spPtr->outputWindowWidth  = (uint32_t)width;
        spPtr->outputWindowHeight = (uint32_t)height;

        err_code = FAPEX_VIOUT_Ioctl(dataPtr->vioutSubId[VAL_VOUT_HD],
                                     FAPEX_IOCCOM_VIOUT_SET_RESIZE,
                                     (FAPEXIOCARG3_T)spPtr);

        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_Resize: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Set video window to be taken as input for scaling.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
** \param   x0          Horizontal start position in pixels.
** \param   y0          Vertical start position in pixels.
** \param   width       Width in pixels.
** \param   height      Height in pixels.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  dataPtr is invalid.
**
** \sa      VAL_Resize
*******************************************************************************
*/
int32_t VAL_Crop (VAL_DataT* dataPtr, uint32_t x0, uint32_t y0,
                  uint32_t width, uint32_t height)
{
    int32_t                      err_code;
    FAPEX_VIOUT_ScalerParamStrT* spPtr;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        spPtr = dataPtr->vioutParams + (int32_t)VAL_VOUT_HD;

        spPtr->inputWindowX0     = (uint32_t)x0;
        spPtr->inputWindowY0     = (uint32_t)y0;
        spPtr->inputWindowWidth  = (uint32_t)width;
        spPtr->inputWindowHeight = (uint32_t)height;

        err_code = FAPEX_VIOUT_Ioctl(dataPtr->vioutSubId[VAL_VOUT_HD],
                                     FAPEX_IOCCOM_VIOUT_SET_RESIZE,
                                     (FAPEXIOCARG3_T)spPtr);
        
        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_Crop: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Blank or unblank video output.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   blankOrUnblank  true (1): blank, false (0): unblank video output
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  dataPtr is invalid.
*******************************************************************************
*/
int32_t VAL_Blank (VAL_DataT* dataPtr, uint8_t blankOrUnblank)
{
    int32_t     err_code = FAPI_OK;
    VAL_OpenT*  openPtr;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( dataPtr->videoBlanked != blankOrUnblank )
        {
            dataPtr->videoBlanked = blankOrUnblank;

            openPtr = (VAL_OpenT*)(dataPtr->openPtr);

            if ( (openPtr != NULL) &&
                 ( (openPtr->decoderStatus == VAL_STATUS_RUNNING) ||
                   (openPtr->operation     == VAL_OP_IFRAME) ) )
             {
                if ( blankOrUnblank )
                {
                    err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                                 FAPEX_IOCCOM_VIDEC_SET_DISPOFF,
                                                 (FAPEXIOCARG3_T)NULL);
                }
                else
                {
                    err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                                 FAPEX_IOCCOM_VIDEC_SET_DISPON,
                                                 (FAPEXIOCARG3_T)FAPEX_VIDEC_ENABLE_FORCE_ON);
                }
            }
        }

        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_Blank(%i): %i\n", blankOrUnblank,
                            err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Set notification function to be informed about changes in parameters
**          related to video scaling.
**
** \param   dataPtr           Handle returned by VAL_GoUp.
** \param   paramChangeCb     Notification function, NULL disables notification.
** \param   paramChangeCbArg  Optional parameter passed with paramChangeCb.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
*******************************************************************************
*/
int32_t VAL_SetScalingParamChangeCb(VAL_DataT* dataPtr,
                                    void (*paramChangeCb)(void *, void *),
                                    void* paramChangeCbArg)
{
    int32_t err_code;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        dataPtr->vioutParamChangeCb    = paramChangeCb;
        dataPtr->vioutParamChangeCbArg = paramChangeCbArg;

        (void)FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

        err_code = FAPI_OK;
    }

    FAPI_SYS_PRINT_DEBUG(2, "[VAL] VAL_SetScalingParamChangeCb: %i\n",
                         err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Update HDMI AVI InfoFrame for aspect ratio handling.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   pictureAr       Picture aspect ratio
** \param   activeFormat    Active format codes. It refers to AFD in the DVB
**                          specification.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      VAL_HdmiSetVideoParams(), VAL_HdmiSetup()
*******************************************************************************
*/
int32_t VAL_HdmiUpdateAviInfoFrame (VAL_DataT* dataPtr, 
                                    int32_t pictureAr,
                                    uint8_t activeFormat)
{
    int32_t err_code  = FAPI_OK;
    uint8_t paramsChg = 0;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else  if ( dataPtr->hdmiId < 0 )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        dataPtr->hdmiPictureAr      = pictureAr;
        dataPtr->hdmiActiveFormat   = activeFormat;

        if ( dataPtr->hdmiConnectParams.connected )
        {
            FAPEX_VIOUT_HdmiParamStrT  hdmi_video_params;

            hdmi_video_params.inputType    = FAPEX_HDMI_YUV444;
            hdmi_video_params.outputType   = FAPEX_HDMI_RGB;
            hdmi_video_params.scanInfo     = FAPEX_HDMI_SCAN_NO_DATA;
            hdmi_video_params.activeFormat = activeFormat;
            hdmi_video_params.pictureAr    = pictureAr;

            err_code = FAPEX_VIOUT_Ioctl( dataPtr->vioutId,
                                          FAPEX_IOCCOM_VIOUT_UPDATE_AVI_INFO,
                                          (FAPEXIOCARG3_T)&hdmi_video_params );
        }

        (void)FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Dynamic ON/OFF HDCP function.
**
** \param   dataPtr    Handle returned by VAL_GoUp.
** \param   onOffFlag  enable/disable flag
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      VAL_HdmiUpdataStatus(), valHdmiHandleHDCP()
*******************************************************************************
*/
int32_t VAL_HdmiHdcpEnable (VAL_DataT* dataPtr, uint32_t onOffFlag)
{
    int32_t err_code  = FAPI_OK;
    uint8_t hdcpStateChanged = 0;

    if (( dataPtr == NULL ) || ( dataPtr->hdmiId < 0 ))
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if( dataPtr->hdmiHDCPEnabled != onOffFlag )
        {
            hdcpStateChanged = 1;
            dataPtr->hdmiHDCPEnabled = (uint8_t)onOffFlag;
            if(onOffFlag)
            {
                dataPtr->hdmiHDCPState = VAL_HDMI_HDCPSTATE_AUTH_INIT;
            }
            else
            {
                dataPtr->hdmiHDCPState = VAL_HDMI_HDCPSTATE_OFF;
            }
            err_code = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)dataPtr->hdmiId,
                                         FAPEX_IOCCOM_HDMI_SET_DYNAMIC_HDCP,
                                         (FAPEXIOCARG3_T)onOffFlag );
        }

        if( hdcpStateChanged )
        {
            if ( dataPtr->hdmiHDCPStateCb != NULL )
            {
                dataPtr->hdmiHDCPStateCb(dataPtr->hdmiHDCPState,
                                         dataPtr->hdmiHDCPStateCbArg );
            }
            FAPI_SYS_PRINT_DEBUG(1, "[VAL] HDCP state=%i\n",
                                        dataPtr->hdmiHDCPState);
        }

        (void)FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Return current HDMI HDCP ON/OFF status.
**
** \param   dataPtr    Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - HDCP Status                               Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  NULL-pointer passed as
**                                                 dataPtr.
*******************************************************************************
*/
int32_t VAL_GetHdmiHdcpOnOffStatus (VAL_DataT* dataPtr)
{
    int32_t ret_code;
//    int32_t ret_val;

    if ( dataPtr == NULL )
    {
        ret_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        if(dataPtr->hdmiHDCPState == VAL_HDMI_HDCPSTATE_OFF)
        {
            ret_code = FAPEX_HDMI_HDCP_DYNAMIC_OFF;
        }
        else
        {
            ret_code = FAPEX_HDMI_HDCP_DYNAMIC_ON;
        }
    }

    return ret_code;
}

/*!
*******************************************************************************
** \brief   Set notification function to be informed about state changes in
**          HDCP authentication of HDMI.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   cb              Notification function, NULL disables notification.
** \param   cbArg           Optional arg pointer for notification function.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
*******************************************************************************
*/
int32_t VAL_SetHdmiHdcpStateCb(VAL_DataT* dataPtr,
                               VAL_HdmiHdcpStateCallbackT cb, void* cbArg)
{
    int32_t err_code;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        dataPtr->hdmiHDCPStateCb    = cb;
        dataPtr->hdmiHDCPStateCbArg = cbArg;

        (void)FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

        err_code = FAPI_OK;
    }

    FAPI_SYS_PRINT_DEBUG(2, "[VAL] VAL_SetHdmiHdcpStateCb: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Get current HDMI driver handle.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - Driver handle (may be -1)
*******************************************************************************
*/
int32_t VAL_GetIdHdmi (VAL_DataT* dataPtr)
{
    if(dataPtr == NULL) {
        return -1;
    }
    return dataPtr->hdmiId;
}

/*!
*******************************************************************************
** \brief   Get current SD video encoder driver handle.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - Driver handle (may be -1)
*******************************************************************************
*/
int32_t VAL_GetIdViencSd (VAL_DataT* dataPtr)
{
    if(dataPtr == NULL) {
        return -1;
    }
    return dataPtr->viencSubId[VAL_VOUT_SD];
}

/*!
*******************************************************************************
** \brief   Get current HD video encoder driver handle.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - Driver handle (may be -1)
*******************************************************************************
*/
int32_t VAL_GetIdViencHd (VAL_DataT* dataPtr)
{
    if(dataPtr == NULL) {
        return -1;
    }
    return dataPtr->viencSubId[VAL_VOUT_HD];
}

/*!
*******************************************************************************
** \brief   Get current SD video scaler driver handle.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - Driver handle (may be -1)
*******************************************************************************
*/
int32_t VAL_GetIdViscaleSd (VAL_DataT* dataPtr)
{
    if(dataPtr == NULL) {
        return -1;
    }
    return dataPtr->vioutSubId[VAL_VOUT_SD];
}

/*!
*******************************************************************************
** \brief   Get current HD video scaler driver handle.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - Driver handle (may be -1)
*******************************************************************************
*/
int32_t VAL_GetIdViscaleHd (VAL_DataT* dataPtr)
{
    if(dataPtr == NULL) {
        return -1;
    }
    return dataPtr->vioutSubId[VAL_VOUT_HD];
}

/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************
** \brief   Open video for decoding.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
** \param   paramsPtr   Open parameters.
** \param   errCodePtr  Optionally returns error_status. Set to NULL if not
**                      needed.
**
** \return  NULL in case of failure, handle for further usage on success.
**
** \note    This shall be called before any video decoding (for live or one of
**          the PVR playback modes) is started. The returned pointer is
**          pointer is used for all further operations related to decoding
**          until the module is closed by VAL_Close.
**
** \sa      VAL_Close
*******************************************************************************
*/
VAL_OpenT* VAL_Open (VAL_DataT* dataPtr, const VAL_OpenParamsT* paramsPtr,
                     int32_t* errCodePtr)
{
    int32_t    err_code = FAPI_OK;
    VAL_OpenT* openPtr  = &openDat;

    if ( dataPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else if ( paramsPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

        if ( dataPtr->openPtr != NULL )
        {
            err_code = SUBSYS_VIDEO_VAL_ERR_ALREADY_OPEN;
        }
        else
        {
            memset(openPtr, 0, sizeof(VAL_OpenT));

            openPtr->tsdId   = -1;
            openPtr->videcId = -1;

            switch( paramsPtr->h264AudType )
            {
                default:
                case VAL_H264_AUD_TYPE_AUTO:
                case VAL_H264_AUD_TYPE_AUD:
                    openPtr->delimiterType = FAPEX_VIDEC_DELIMITER_AUD_HEADER;
                    break;
                case VAL_H264_AUD_TYPE_NO_AUD:
                    openPtr->delimiterType = FAPEX_VIDEC_DELIMITER_PES_HEADER;
            }

            err_code = valIntOpen (openPtr, paramsPtr);

            if ( err_code == FAPI_OK )
            {
                /* successfully opened: link structures */
                openPtr->dataPtr = dataPtr;
                dataPtr->openPtr = openPtr;
            }
        }

        FAPEX_SYS_SetSemaphore(dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    }

    if ( errCodePtr != NULL )
    {
        *errCodePtr = err_code;
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_Open: %i\n", err_code);

    if ( err_code != FAPI_OK )
    {
        return NULL;
    }
    else
    {
        return openPtr;
    }
}

/*!
*******************************************************************************
** \brief   Close video decoding.
**
** \param   openPtr     Handle returned by VAL_Open.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  Invalid handle.
**
** \note    This shall be called after decoding of a stream is finished, e.g.
**          on service change or stopping of a PVR playback (i.e. any time the
**          source TS potentially changes). After the function return with
**          FAPI_OK, 'openPtr' is no longer valid.
**
** \sa      VAL_Open
*******************************************************************************
*/
int32_t VAL_Close (VAL_OpenT* openPtr)
{
    int32_t err_code = FAPI_OK;

    if ( openPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        if ( openPtr->dataPtr->openPtr == openPtr )
        {
            (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle,
                                         FAPEX_SYS_SUSPEND);

            err_code = valIntClose(openPtr);

            /* mark instance as closed */
            openPtr->dataPtr->openPtr = NULL;

            FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle,
                                   FAPEX_SYS_NO_SUSPEND);
        }
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_Close: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Start video decoding/presentation.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   pid             PID of video stream to be decoded.
** \param   mode            Initial playback mode (for environment
**                          VAL_ENV_PLAY only).
** \param   pcrFlag         0 : PCR's pid is not equal to pid.
**                          1 : PCR's pid is equal to pid.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      VAL_Stop
*******************************************************************************
*/
int32_t VAL_Start (VAL_OpenT* openPtr, uint32_t pid, VAL_PlayModeEnumT mode, uint32_t pcrFlag)
{
    int32_t err_code;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code == FAPI_OK )
    {
        (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle,
                                     FAPEX_SYS_SUSPEND);

        openPtr->pcrFlag = pcrFlag;

        err_code = valIntStart (openPtr, pid, mode);

        FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle,
                               FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_Start: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Stop video decoding and output.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   freeze            Video stream PID.
**
** \return  Possible return codes:
**
** \sa      VAL_Start
*******************************************************************************
*/
int32_t VAL_Stop (VAL_OpenT* openPtr, uint8_t freeze)
{
    int32_t err_code;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code == FAPI_OK )
    {
        (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle,
                                     FAPEX_SYS_SUSPEND);

        err_code = valIntStop(openPtr, freeze);

        FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle,
                               FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_Stop: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Freeze or unfreeze video.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   freezeOn          If true, freeze video, else unfreeze it.
**
** \return  Possible return codes:
**
** \sa      VAL_Stop, VAL_Start
*******************************************************************************
*/
int32_t VAL_Freeze (VAL_OpenT* openPtr, uint8_t freezeOn)
{
    int32_t  err_code;
    int32_t  freezeSwitch;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code == FAPI_OK )
    {
        if ( freezeOn ) freezeSwitch = FAPEX_VIDEC_FREEZE_ON;
        else            freezeSwitch = FAPEX_VIDEC_FREEZE_OFF;

        (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle,
                                     FAPEX_SYS_SUSPEND);

        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                     FAPEX_IOCCOM_VIDEC_SET_FREEZE_MODE,
                                     (FAPEXIOCARG3_T)freezeSwitch);

        FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle,
                               FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_Freeze: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Function that handles video-related tasks in application context.
**
** Should be regularly called by application, handles booting of correct
** video decoder for H.264.
**
** \param   openPtr           Handle returned by VAL_Open.
**
** \return  Possible return codes:
**
*******************************************************************************
*/
int32_t VAL_Task (VAL_OpenT* openPtr)
{
    /* NOTE: There are currently no jobs to be performed by this function.
             However, the interface will be kept for runtime handling that
             might become necessary in the future. */
    
    return FAPI_OK;
}
/*lint -restore */

/*!
*******************************************************************************
** \brief   Set notification function to be called once the first video frame
**          has been decoded and displayed.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   frameAvlCb        Notification function, NULL disables notification.
** \param   frameAvlCbArg     Optional parameter passed with frameAvlCb.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
*******************************************************************************
*/
int32_t VAL_SetFrameAvailableCb(VAL_OpenT* openPtr,
                                void (*frameAvlCb)(void* optData),
                                void* frameAvlCbArg)
{
    int32_t err_code;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code == FAPI_OK )
    {
        openPtr->frameAvlCb    = frameAvlCb;
        openPtr->frameAvlCbArg = frameAvlCbArg;
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_SetFrameAvailableCb: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Set notification function to be informed about parameter changes
**          in the current video stream.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   paramChangeCb     Notification function, NULL disables notification.
** \param   paramChangeCbArg  Optional parameter passed with paramChangeCb.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
*******************************************************************************
*/
int32_t VAL_SetParamChangeCb (VAL_OpenT* openPtr,
                              void (*paramChangeCb)(VAL_VIDEC_DecFrameInfoStrT *, void*),
                              void* paramChangeCbArg)
{
    int32_t err_code;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code == FAPI_OK )
    {
        openPtr->paramChangeCb    = paramChangeCb;
        openPtr->paramChangeCbArg = paramChangeCbArg;
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_SetParamChangeCb: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Allows to reboot video decoder for new decoder type at runtime.
**
** Current video operation will be maintained, i.e. restarting of video
** after reboot is not necessary.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   decoderPtr        Pointer to structure describing requested decoder
**                            type.
**
** \return  Possible return codes:
**
** \sa      VAL_Stop, VAL_Start
*******************************************************************************
*/
int32_t VAL_Reboot (VAL_OpenT* openPtr, const VAL_DecoderTypeT* decoderPtr)
{
    int32_t err_code;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code != FAPI_OK )
    {
        /* do nothing */
    }
    else if ( decoderPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle,
                                     FAPEX_SYS_SUSPEND);

        err_code = valIntReboot(openPtr, decoderPtr);

        FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle,
                               FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_Reboot: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Perform video decoder recovery.
**
** \param   openPtr           Handle returned by VAL_Open.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE Invalid (NULL) handle passed.
**          - SUBSYS_VIDEO_VAL_ERR_NOT_OPEN       VAL is not open.
**
*******************************************************************************
*/
int32_t VAL_Recover (VAL_OpenT* openPtr)
{
    int32_t err_code;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code == FAPI_OK )
    {
        (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle,
                                     FAPEX_SYS_SUSPEND);

        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                     FAPEX_IOCCOM_VIDEC_RECOVER,
                                     (FAPEXIOCARG3_T)NULL);

        FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle,
                               FAPEX_SYS_NO_SUSPEND);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Return buffer size of video ES input buffer.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   sizePtr           Returns buffer size if a non-NULL pointer is
**                            passed.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      VAL_GetBufferLevel
*******************************************************************************
*/
int32_t VAL_GetBufferSize (VAL_OpenT* openPtr, uint32_t* sizePtr)
{
    if ( sizePtr != NULL )
    {
        if ( openPtr != NULL )
        {
            *sizePtr = openPtr->bufferSize;
        }
        else
        {
            *sizePtr = 0;
        }
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
** \brief   Return current filling level of video ES input buffer.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   levelPtr          Returns filling level if a non-NULL pointer is
**                            passed.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      VAL_GetBufferSize
*******************************************************************************
*/
int32_t VAL_GetBufferLevel (VAL_OpenT* openPtr, uint32_t* levelPtr)
{
    int32_t  err_code = FAPI_OK;
    uint32_t level    = 0;

    if ( (openPtr != NULL) && (openPtr->videcId >= 0) )
    {
        err_code = FAPEX_TSD_Ioctl(openPtr->tsdId,
                                   FAPEX_IOCCOM_TSD_GET_BUFFER_LEVEL,
                                   (FAPEXIOCARG3_T)&level);
    }

    if ( levelPtr != NULL )
    {
        *levelPtr = level;
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Informs if first frame after video startup has been displayed..
**
** \note Frame availability is reset in valIntStart, VAL_PlayChangeMode and
**       VAL_PlayPrepareNextFrame.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   avlPtr          Returns frame availability status (true, false)
**                          if a non-NULL pointer is passed.
**
** \return  Possible return codes:
**          - FAPI_OK                               Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE   NULL-pointer passed as
**                                                  openPtr.
*******************************************************************************
*/
int32_t VAL_IsFrameAvailable (VAL_OpenT* openPtr, uint8_t* avlPtr)
{
    int32_t err_code;

    if ( openPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        if ( avlPtr != NULL )
        {
            *avlPtr = openPtr->frameDecoded;
        }

        err_code = FAPI_OK;
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Return current VAL module status.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   statusPtr         Returns status information if a non-NULL pointer
**                            is passed.
**
** \return  Possible return codes:
**          - FAPI_OK                              Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE  NULL-pointer passed as
**                                                 openPtr.
*******************************************************************************
*/
int32_t VAL_GetStatus (VAL_OpenT* openPtr, VAL_StatusT* statusPtr)
{
    int32_t err_code;

    if ( openPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else
    {
        if ( statusPtr != NULL )
        {
            memset(statusPtr, 0, sizeof (VAL_StatusT));

            statusPtr->environment   = openPtr->openParams.environment;
            statusPtr->operation     = openPtr->operation;
            statusPtr->mode          = openPtr->mode;
            statusPtr->decoderStatus = openPtr->decoderStatus;
            statusPtr->decoder       = openPtr->decoder;
        }

        err_code = FAPI_OK;
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Get detailed information on currently displayed video frame.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   videcInfoPtr      Returns video frame information if a non-NULL
**                            pointer is passed and return value is FAPI_OK.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_DATA_NOT_AVAILABLE No frame info available (yet).
*******************************************************************************
*/
int32_t VAL_GetVideoFrameInfo (VAL_OpenT* openPtr,
                               VAL_VIDEC_DecFrameInfoStrT* videcInfoPtr)
{
    int32_t err_code;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code == FAPI_OK )
    {
        if ( openPtr->videcFrameInfoAvailable == 0 )
        {
            err_code = SUBSYS_VIDEO_VAL_ERR_DATA_NOT_AVAILABLE;
        }
        else if ( videcInfoPtr != NULL )
        {
            *videcInfoPtr = openPtr->videcFrameInfo;
        }
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Get current TSD driver handle.
**
** \param   openPtr           Handle returned by VAL_Open.
**
** \return  Possible return codes:
**          - Driver handle (may be -1)
*******************************************************************************
*/
int32_t VAL_GetIdTsd (VAL_OpenT* openPtr)
{
    if(openPtr == NULL) {
        return -1;
    }
    return openPtr->tsdId;
}

/*!
*******************************************************************************
** \brief   Get current VIDEC driver handle.
**
** \param   openPtr           Handle returned by VAL_Open.
**
** \return  Possible return codes:
**          - Driver handle (may be -1)
*******************************************************************************
*/
int32_t VAL_GetIdVidec (VAL_OpenT* openPtr)
{
    if(openPtr == NULL) {
        return -1;
    }
    return openPtr->videcId;
}

/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************
** \brief   Decode an MPEG-I-Frame.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   dataPtr         Source data (MPEG 1/2 ES), endianess is detected
**                          automatically.
** \param   sizeInBytes     Data size in bytes.
**
** \return  Possible return codes:
**          - FAPI_OK                                    Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_ALREADY_STARTED       Decoding is already active.
**          - SUBSYS_VIDEO_VAL_ERR_FEATURE_NOT_SUPPORTED Currently selected
**                                                       decoder is not an MPEG decoder.
**
** \sa      VAL_IFrameDisplay
*******************************************************************************
*/
int32_t VAL_IFrameDecode (VAL_OpenT* openPtr, const uint32_t* dataPtr,
                          uint32_t sizeInBytes)
{
    FAPEX_VIDEC_ESdataParamStrT  esDataParam;
    int32_t err_code = FAPI_OK;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code == FAPI_OK )
    {
        (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle,
                                     FAPEX_SYS_SUSPEND);

        if ( openPtr->operation != VAL_OP_NONE )
        {
            err_code = SUBSYS_VIDEO_VAL_ERR_ALREADY_STARTED;
        }
        else if ( ( openPtr->decoder.system != FAPEX_VIDEC_CODEC_TYPE_MPEG2 ) &&
                  ( openPtr->decoder.system != FAPEX_VIDEC_CODEC_TYPE_H264 ) )
        {
            err_code = SUBSYS_VIDEO_VAL_ERR_FEATURE_NOT_SUPPORTED;
        }
        else
        {
            do
            {
                esDataParam.data = (uint32_t*)dataPtr;
                esDataParam.size = sizeInBytes;

                err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                             FAPEX_IOCCOM_VIDEC_DEC_SINGLE_I_FRAME,
                                             (FAPEXIOCARG3_T)&esDataParam);
                if(err_code != FAPI_OK) {
                    break;
                }

                err_code = FAPEX_VIOUT_Ioctl(openPtr->dataPtr->vioutSubId[VAL_VOUT_HD],
                                             FAPEX_IOCCOM_VIOUT_SET_ASPECT_RATIO,
                                             (FAPEXIOCARG3_T)openPtr->dataPtr->vioutDispAr[VAL_VOUT_HD]);
                if(err_code != FAPI_OK) {
                    break;
                }

                err_code = FAPEX_VIOUT_Ioctl(openPtr->dataPtr->vioutSubId[VAL_VOUT_SD],
                                             FAPEX_IOCCOM_VIOUT_SET_ASPECT_RATIO,
                                             (FAPEXIOCARG3_T)openPtr->dataPtr->vioutDispAr[VAL_VOUT_SD]);
                if(err_code != FAPI_OK) {
                    break;
                }
                openPtr->operation = VAL_OP_IFRAME;
            }
            while ( 0 );
        }

        FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle,
                               FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_IFrameDecode: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Display an MPEG-I-Frame decoded with VAL_IFrameDecode.
**
** \param   openPtr         Handle returned by VAL_Open.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_BAD_MODE   No I-Frame has been decoded.
**
** \sa      VAL_IFrameDecode
*******************************************************************************
*/
int32_t VAL_IFrameDisplay (VAL_OpenT* openPtr)
{
    int32_t err_code;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code == FAPI_OK )
    {
        (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle,
                                     FAPEX_SYS_SUSPEND);

        if ( openPtr->operation != VAL_OP_IFRAME )
        {
            err_code = SUBSYS_VIDEO_VAL_ERR_BAD_MODE;
        }
        else
        {
            err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                         FAPEX_IOCCOM_VIDEC_DISP_SINGLE_I_FRAME,
                                         (FAPEXIOCARG3_T)NULL);
        }

        FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle,
                               FAPEX_SYS_NO_SUSPEND);
    }

    FAPI_SYS_PRINT_DEBUG(3, "[VAL] VAL_IFrameDisplay: %i\n", err_code);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Change mode of playback operation.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   mode            Requested playback mode.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_BAD_MODE Module is not in PVR playback operation.
**
** \sa      VAL_Start, VAL_Stop
*******************************************************************************
*/
int32_t VAL_PlayChangeMode (VAL_OpenT* openPtr, VAL_PlayModeEnumT mode)
{
    int32_t                         err_code      = FAPI_OK;
    uint8_t                         unfreezeVideo = 0;
    int32_t                         videcMode;
    uint8_t                         restart       = 0;
    FAPEX_VIDEC_StartParamStrT      decStartParam;
    
    FAPI_SYS_PRINT_DEBUG(3, "[VAL] Change playback mode: %i -> %i\n",
                            openPtr->mode, mode);
    
    if ( openPtr == NULL )
    {
        return SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else if ( openPtr->operation != VAL_OP_PLAY )
    {
        return SUBSYS_VIDEO_VAL_ERR_BAD_MODE;
    }
    else if ( mode >= VAL_PLAYMODE_NUMS )
    {
        return SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
    }
    else if ( mode == openPtr->mode )
    {
        return FAPI_OK;
    }

    (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

    /* ----- freeze/reset video -------------------------------------------- */

    /* Handle all non-seamless transitions. */

    if ( (mode          == VAL_PLAYMODE_FAST_DEC_IONLY)
      || (mode          == VAL_PLAYMODE_FAST_DEC_IP)
      || (mode          == VAL_PLAYMODE_FAST_LIB_IONLY)
      || (mode          == VAL_PLAYMODE_FAST_CONTROLLEDFEED)
      || (openPtr->mode == VAL_PLAYMODE_FAST_DEC_IONLY)
      || (openPtr->mode == VAL_PLAYMODE_FAST_DEC_IP)
      || (openPtr->mode == VAL_PLAYMODE_FAST_LIB_IONLY)
      || (openPtr->mode == VAL_PLAYMODE_FAST_CONTROLLEDFEED) )
    {
        err_code = valChannelStop (openPtr);
        if(err_code != FAPI_OK) {
            goto VAL_PlayChangeMode_end;
        }

        /* Handle special cases where simply freezing and unfreezing of the
           video is not sufficient. */

        if ( (mode          == VAL_PLAYMODE_FAST_DEC_IONLY)
          || (mode          == VAL_PLAYMODE_FAST_CONTROLLEDFEED)
          || (openPtr->mode == VAL_PLAYMODE_FAST_DEC_IONLY)
          || (openPtr->mode == VAL_PLAYMODE_FAST_CONTROLLEDFEED) )
        {
            /* Both of these modes use the I_ONLY mode provided by the video
               firmware. Resetting the video decoder is necessary when entering
               or leaving this mode */
            err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                         FAPEX_IOCCOM_RESET,
                                         (FAPEXIOCARG3_T)FAPEX_VIDEC_HARD_RESET);
            if(err_code != FAPI_OK) {
                goto VAL_PlayChangeMode_end;
            }
            restart = 1;
        }
        else if ( openPtr->mode == VAL_PLAYMODE_PAUSE )
        {
            if ( mode == VAL_PLAYMODE_FAST_LIB_IONLY )
            {
                // TODO: add error handling!

                err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                             FAPEX_IOCCOM_RESET,
                                             (FAPEXIOCARG3_T)FAPEX_VIDEC_HARD_RESET);
                if(err_code != FAPI_OK) {
                    goto VAL_PlayChangeMode_end;
                }

                err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                             FAPEX_IOCCOM_VIDEC_SET_PLAYBACK_MODE,
                                             (FAPEXIOCARG3_T)FAPEX_VIDEC_PLAYBACK_MODE_I_ONLY);
                if(err_code != FAPI_OK) {
                    goto VAL_PlayChangeMode_end;
                }
            }
            restart = 1;
        }
        else
        {
            err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                         FAPEX_IOCCOM_VIDEC_SET_FREEZE_MODE,
                                         (FAPEXIOCARG3_T)FAPEX_VIDEC_FREEZE_ON);
            if(err_code != FAPI_OK) {
                goto VAL_PlayChangeMode_end;
            }
            unfreezeVideo = 1;
        }

        if ( mode != VAL_PLAYMODE_FAST_LIB_IONLY )
        {
            err_code = valChannelStart (openPtr, openPtr->pid);
            if(err_code != FAPI_OK) {
                goto VAL_PlayChangeMode_end;
            }
        }
    }

    /* ----- change decoder mode ------------------------------------------- */

    switch ( mode )
    {
        default:
        case VAL_PLAYMODE_FAST_LIB_IONLY:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_NORMAL;
            break;

        case VAL_PLAYMODE_NORMAL:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_NORMAL;
            break;

        case VAL_PLAYMODE_PAUSE:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_PAUSE;
            break;

        case VAL_PLAYMODE_FAST_DEC_IONLY:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_I_ONLY;
            break;

        case VAL_PLAYMODE_FAST_DEC_IP:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_I_AND_P_ONLY;
            break;

        case VAL_PLAYMODE_FAST_CONTROLLEDFEED:
            #ifdef VAL_FASTMODE_SPEEDUP
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_I_ONLY;
            #else
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_FIRST_I_ONLY;
            #endif /* VAL_FASTMODE_SPEEDUP */
            break;

        case VAL_PLAYMODE_SLOW_2X:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_2X;
            break;

        case VAL_PLAYMODE_SLOW_4X:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_4X;
            break;

        case VAL_PLAYMODE_SLOW_8X:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_8X;
            break;
    }

    if ( videcMode != FAPEX_VIDEC_PLAYBACK_MODE_UNKNOWN )
    {
        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                     FAPEX_IOCCOM_VIDEC_SET_PLAYBACK_MODE,
                                     (FAPEXIOCARG3_T)videcMode);
        if(err_code != FAPI_OK) {
            goto VAL_PlayChangeMode_end;
        }
    }

    /* ----- unfreeze video ------------------------------------------------ */

    openPtr->frameAvailable = 0;
    openPtr->frameDecoded   = 0;

    openPtr->startFlags = FAPEX_VIDEC_NOTIFY_FLAGS_DEC_AVAILABLE |
                          FAPEX_VIDEC_NOTIFY_FLAGS_DISP_AVAILABLE;

    if ( unfreezeVideo )
    {
        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                     FAPEX_IOCCOM_VIDEC_SET_FREEZE_MODE,
                                     (FAPEXIOCARG3_T)FAPEX_VIDEC_FREEZE_OFF);
    }
    else if ( restart )
    {
        decStartParam.outputMode   = FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_NO_SUSPEND;
        decStartParam.playbackMode = videcMode;
        decStartParam.flags        = openPtr->startFlags;

        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                     FAPEX_IOCCOM_VIDEC_START,
                                     (FAPEXIOCARG3_T)&decStartParam);
    }

    if ( err_code == FAPI_OK )
    {
        openPtr->mode  = mode;
    }

VAL_PlayChangeMode_end:

    FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Decode and display an I-frame as part of a series (e.g. fast mode).
**
** This function is to be used in playback mode VAL_PLAYMODE_FAST_LIB_IONLY
** only. As long as the playback mode is not changed, it may be called multiple
** times in order to show a sequence of I-frames without blanking in between.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   dataPtr         Source data (MPEG 1/2 ES), endianess is detected
**                          automatically.
** \param   dataBytes       Data size in bytes.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_BAD_MODE Module is not in PVR playback operation.
*******************************************************************************
*/
int32_t VAL_PlayShowIFrame (VAL_OpenT* openPtr, const uint32_t* dataPtr,
                            uint32_t dataBytes)
{
    FAPEX_VIDEC_ESdataParamStrT  esDataParam;
    int32_t err_code;

    // FIXME: add openPtr check?

    (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

    if ( openPtr->operation != VAL_OP_PLAY )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_BAD_MODE;
    }
    else
    {
        //FAPI_SYS_PRINT_MSG("SHOW I: %p, %i bytes\n", dataPtr, dataBytes);

        // TODO: check if in correct decoder mode! (use also for live?)

        esDataParam.data = (uint32_t*)dataPtr;
        esDataParam.size = dataBytes;

        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                     FAPEX_IOCCOM_VIDEC_DEC_SEQUENCE_I_FRAME,
                                     (FAPEXIOCARG3_T)&esDataParam);
        if ( err_code == FAPI_OK )
        {
            err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                         FAPEX_IOCCOM_VIDEC_DISP_SEQUENCE_I_FRAME,
                                         (FAPEXIOCARG3_T)NULL);
        }
    }

    FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Prepare decoding of next I-frame for special playback mode.
**
** This function is to be used in playback mode
** VAL_PLAYMODE_FAST_CONTROLLEDFEED only. It should be called before the first
** data is fed via TSD. In order to realise a sequence of I-frames, it should
** be called periodically AFTER the decoder has indicated that a frame has been
** displayed and BEFORE data for the new frame has been fed.
**
** \param   openPtr         Handle returned by VAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**          - SUBSYS_VIDEO_VAL_ERR_BAD_MODE Module is not in correct playback mode
**                                          for this operation.
*******************************************************************************
*/
int32_t VAL_PlayPrepareNextFrame (VAL_OpenT* openPtr)
{
    #ifndef VAL_FASTMODE_SPEEDUP
    FAPEX_VIDEC_StartParamStrT  decStartParam;
    #endif
    int32_t err_code = FAPI_OK;
    
    // FIXME: add openPtr check?
    
    (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);
    
    do
    {
        if ( openPtr->operation != VAL_OP_PLAY )
        {
            err_code = SUBSYS_VIDEO_VAL_ERR_BAD_MODE;
            break;
        }
        else if ( openPtr->mode != VAL_PLAYMODE_FAST_CONTROLLEDFEED )
        {
            err_code = SUBSYS_VIDEO_VAL_ERR_BAD_MODE;
            break;
        }
        
        #ifndef VAL_FASTMODE_SPEEDUP
        
        err_code = valChannelStop (openPtr);
        if(err_code != FAPI_OK) {
            break;
        }

        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                     FAPEX_IOCCOM_RESET,
                                     (FAPEXIOCARG3_T)FAPEX_VIDEC_HARD_RESET);
        if(err_code != FAPI_OK) {
            break;
        }

        err_code = valChannelStart (openPtr, openPtr->pid);
        if ( err_code != FAPI_OK )
        {
            break;
        }

        #endif /* VAL_FASTMODE_SPEEDUP */
        
        #ifndef VAL_FASTMODE_SPEEDUP
        
        decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_FIRST_I_ONLY;
        
        #endif /* VAL_FASTMODE_SPEEDUP */
        
        if ( err_code != FAPI_OK )
        {
            break;
        }

        openPtr->frameAvailable = 0;
        openPtr->frameDecoded   = 0;
        
        openPtr->startFlags = FAPEX_VIDEC_NOTIFY_FLAGS_DEC_AVAILABLE |
                              FAPEX_VIDEC_NOTIFY_FLAGS_DISP_AVAILABLE;
        
        #ifndef VAL_FASTMODE_SPEEDUP
        
        if ( !(openPtr->dataPtr->videoBlanked) )
        {
            decStartParam.outputMode = FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_NO_SUSPEND;
        }
        else {
            decStartParam.outputMode = FAPEX_VIDEC_ENABLE_FORCE_OFF;
        }
        decStartParam.flags = openPtr->startFlags;
        
        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                     FAPEX_IOCCOM_VIDEC_START,
                                     (FAPEXIOCARG3_T)&decStartParam);
        
        #endif /* VAL_FASTMODE_SPEEDUP */
    }
    while ( 0 );

    FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    
    return err_code;
}

/*!
 ********************************************************************************
 ** \brief   API to intimate the driver the stream segment type.
 **
 ** This function is to be used after opening the video decoder and before starting
 ** it. Though it can be called any time after open but for a smooth playback before
 ** start is recommended
 **
 ** \param   openPtr         Handle returned by VAL_Open.
 ** \param   segmentType     Qualifier for segment type
 **
 ** \return  Possible return codes:
 **          - FAPI_OK  Successful completion.
 **          - SUBSYS_VIDEO_VAL_ERR_BAD_MODE
 ********************************************************************************/
int32_t VAL_SetStreamSegment(VAL_OpenT* openPtr, VAL_StreamSegmentTypeT segmentType)
{
    int32_t err_code = FAPI_OK;
    (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);
   
    do{
        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                               FAPEX_IOCCOM_VIDEC_SET_STREAMSEGMENTMODE,
                               (FAPEXIOCARG3_T)segmentType);
        if(err_code != FAPI_OK){
            break;
    }
    }
    while(0);
    FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    return err_code;
} 

/*!
 *********************************************************************************
 ** \brief   API to set the brightness.
 **
 ** This function is to be used after opening the video encoder and scaler
 **
 ** \param   openPtr         Handle returned by VAL_Open.
 ** \param   level          Level for Video graphics equaliser.
 **
 ** \return  Possible return codes:
 **          - FAPI_OK  Successful completion.
 ********************************************************************************/
int32_t VAL_SetBrightness(VAL_OpenT* openPtr, int32_t level)
{
    int32_t err_code = FAPI_OK;
    (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

    do{
        err_code = FAPEX_VIDEC_Ioctl(fd_VIOUT,
                               FAPEX_IOCCOM_VIOUT_SET_BRIGHTNESS,
                               (FAPEXIOCARG3_T)level);
        if(err_code != FAPI_OK){
            break;
        }
    }
    while(0);
    FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    return err_code;
}

/*!
 *********************************************************************************
 ** \brief   API to set the contrast.
 **
 ** This function is to be used after opening the video encoder and scaler
 **
 ** \param   openPtr         Handle returned by VAL_Open.
 ** \param   level          Level for Video graphics equaliser.
 **
 ** \return  Possible return codes:
 **          - FAPI_OK  Successful completion.
 ********************************************************************************/
int32_t VAL_SetContrast(VAL_OpenT* openPtr, int32_t level)
{
    int32_t err_code = FAPI_OK;
    (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

    do{
        err_code = FAPEX_VIDEC_Ioctl(fd_VIOUT,
                               FAPEX_IOCCOM_VIOUT_SET_CONTRAST,
                               (FAPEXIOCARG3_T)level);
        if(err_code != FAPI_OK){
            break;
        }
    }
    while(0);
    FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    return err_code;
}

/*!
 *********************************************************************************
 ** \brief   API to set the saturation.
 **
 ** This function is to be used after opening the video encoder and scaler
 **
 ** \param   openPtr         Handle returned by VAL_Open.
 ** \param   level          Level for Video graphics equaliser.
 **
 ** \return  Possible return codes:
 **          - FAPI_OK  Successful completion.
 ********************************************************************************/
int32_t VAL_SetSaturation(VAL_OpenT* openPtr, int32_t level)
{
    int32_t err_code = FAPI_OK;
    (void)FAPEX_SYS_GetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_SUSPEND);

    do{
        err_code = FAPEX_VIDEC_Ioctl(fd_VIOUT,
                               FAPEX_IOCCOM_VIOUT_SET_SATURATION,
                               (FAPEXIOCARG3_T)level);
        if(err_code != FAPI_OK){
            break;
        }
    }
    while(0);
    FAPEX_SYS_SetSemaphore(openPtr->dataPtr->mutexHandle, FAPEX_SYS_NO_SUSPEND);
    return err_code;
}


/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************
** \brief   Internal function, wakeup VAL module.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   hdSystemType    Requested HD system type.
** \param   sdSystemType    Requested SD system type.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valIntGoDown
*******************************************************************************
*/
static int32_t valIntGoUp (VAL_DataT* dataPtr, VAL_HdEncoderT hdSystemType,
                            VAL_SdEncoderT sdSystemType)
{
    int32_t err_code = FAPI_OK;
    
    do
    {
        err_code = valViencInitClocks (hdSystemType);
        
        if ( err_code < 0 )
        {
            err_code = -errno;
            FAPI_SYS_PRINT_DEBUG(2, "[VAL] vienc init error: %d\n", err_code);
            break;
        }
        dataPtr->viencId = err_code;

        err_code = valViencOpen(dataPtr, hdSystemType, sdSystemType);
        if ( err_code != FAPI_OK )
        {
            break;
        }

        err_code = valViscaleOpen (dataPtr, hdSystemType);
        if ( err_code != FAPI_OK )
        {
           
            break;
        }
        err_code = valHdmiOpen(dataPtr);
        if ( err_code != FAPI_OK )
        {

            break;
        }
    }
    while ( 0 );

    if ( err_code != FAPI_OK )
    { 
        (void)valHdmiClose    (dataPtr);
        (void)valViscaleClose (dataPtr);
        (void)valViencClose   (dataPtr);
    }
    
    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, shutdown VAL module.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valIntGoUp
*******************************************************************************
*/
static int32_t valIntGoDown (VAL_DataT* dataPtr)
{
    int32_t err_code;

    err_code = valHdmiClose(dataPtr);
    if(err_code != FAPI_OK) {
        return err_code;
    }

    err_code = valViscaleClose(dataPtr);
    if(err_code != FAPI_OK) {
        return err_code;
    }

    err_code = valViencClose(dataPtr);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, open VAL module.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   paramsPtr       Open parameters.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valIntClose
*******************************************************************************
*/
static int32_t valIntOpen (VAL_OpenT* openPtr,
                            const VAL_OpenParamsT* paramsPtr)
{
    int32_t err_code = FAPI_OK;

    openPtr->openParams              = *paramsPtr;
    openPtr->bufferSize              = 0;
    openPtr->operation               = VAL_OP_NONE;
    openPtr->pid                     = 0;
    openPtr->mode                    = VAL_PLAYMODE_NONE;
    openPtr->decoder                 = paramsPtr->decoder;
    openPtr->decoderStatus           = VAL_STATUS_STOPPED;
    openPtr->videcFrameInfoAvailable = 0;
    openPtr->checkThread             = NULL;
    openPtr->h264AudTimeOut          = 0;

    err_code = valChannelOpen (openPtr);

    if ( err_code == FAPI_OK )
    {
        err_code = valVidecOpen (openPtr, &(paramsPtr->decoder),
                                  paramsPtr->enableAfd,
                                  paramsPtr->environment,
                                  openPtr->delimiterType);
        if ( err_code != FAPI_OK )
        {
            (void)valChannelClose (openPtr);
        }
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, close VAL module.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   paramsPtr       Open parameters.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valIntClose
*******************************************************************************
*/
static int32_t valIntClose (VAL_OpenT* openPtr)
{
    int32_t  err_code;

    err_code = valIntStop(openPtr, 0);
    if(err_code != FAPI_OK) {
        return err_code;
    }

    err_code = valVidecClose(openPtr);
    if(err_code != FAPI_OK) {
        return err_code;
    }

    err_code = valChannelClose(openPtr);

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function for starting video decoding/output.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   pid               Video stream PID.
** \param   operation         Requested operation.
** \param   mode              Requested playback mode.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_ALREADY_STARTED  Video already started.
**          - SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER    Invalid playback mode
**                                                  requested.
** \sa      VAL_Stop
*******************************************************************************
*/
static int32_t valIntStart (VAL_OpenT* openPtr, uint32_t pid,
                             VAL_PlayModeEnumT mode)
{
    int32_t                         err_code;
    int32_t                         videcMode;
    uint8_t                         feedViaTsd = 1;
    VAL_OpEnumT                     operation;

    err_code = valArgCheckOpen(openPtr);

    if ( err_code != FAPI_OK )
    {
        return err_code;
    }
    else if ( openPtr->operation != VAL_OP_NONE )
    {
        return SUBSYS_VIDEO_VAL_ERR_ALREADY_STARTED;
    }

    switch ( openPtr->openParams.environment )
    {
        case VAL_ENV_LIVE:
            operation = VAL_OP_LIVE;
            break;

        case VAL_ENV_PLAY:
            operation = VAL_OP_PLAY;
            break;
        
        case VAL_ENV_STREAMING:
            operation = VAL_OP_STREAMING;
            break;

        default:
            return SUBSYS_VIDEO_VAL_ERR_INTERNAL;
    }

    switch ( mode )
    {
        case VAL_PLAYMODE_NORMAL:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_NORMAL;
            break;

        case VAL_PLAYMODE_FAST_LIB_IONLY:
            videcMode  = FAPEX_VIDEC_PLAYBACK_MODE_NORMAL;
            feedViaTsd = 0;
            break;

        case VAL_PLAYMODE_PAUSE:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_PAUSE;
            break;

        case VAL_PLAYMODE_FAST_DEC_IONLY:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_I_ONLY;
            break;

        case VAL_PLAYMODE_FAST_DEC_IP:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_I_AND_P_ONLY;
            break;

        case VAL_PLAYMODE_FAST_CONTROLLEDFEED:
            #ifdef VAL_FASTMODE_SPEEDUP
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_I_ONLY;
            #else
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_FIRST_I_ONLY;
            #endif /* VAL_FASTMODE_SPEEDUP */
            break;

        case VAL_PLAYMODE_SLOW_2X:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_2X;
            break;

        case VAL_PLAYMODE_SLOW_4X:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_4X;
            break;

        case VAL_PLAYMODE_SLOW_8X:
            videcMode = FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_8X;
            break;

        default:
            return SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
    }

    do
    {
        err_code = valChannelStart (openPtr, pid);
        if ( err_code != FAPI_OK ) break;

        err_code = valVidecStart (openPtr, videcMode,
                                   feedViaTsd);
        if ( err_code != FAPI_OK ) break;

        openPtr->operation = operation;
        openPtr->pid       = pid;
        openPtr->mode      = mode;
    }
    while ( 0 );

    if ( err_code != FAPI_OK )
    {
        (void)valVidecStop   (openPtr, 0);
        (void)valChannelStop (openPtr);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, stop video decoding and output.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   freeze            Video stream PID.
**
** \return  Possible return codes:
**
** \sa      VAL_Start
*******************************************************************************
*/
static int32_t valIntStop (VAL_OpenT* openPtr, uint8_t freeze)
{
    int32_t err_code = FAPI_OK;
    
    if ( openPtr->operation == VAL_OP_ERROR )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_STATE;
    }
    else if ( openPtr->operation != VAL_OP_NONE )
    {
        err_code = valChannelStop(openPtr);
        if(err_code != FAPI_OK) {
            return err_code;
        }
        err_code = valVidecStop(openPtr, freeze);
        if(err_code == FAPI_OK) {
            openPtr->operation = VAL_OP_NONE;
            openPtr->pid       = 0;
            openPtr->mode      = VAL_PLAYMODE_NONE;
        }
    }
    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, reboots video decoder for new decoder type at 
**          runtime.
**
** Current video operation will be maintained, i.e. restarting of video
** after reboot is not necessary.
**
** \param   openPtr           Handle returned by VAL_Open.
** \param   decoderPtr        Pointer to structure describing requested decoder
**                            type.
**
** \return  Possible return codes:
**
*******************************************************************************
*/
static int32_t valIntReboot (VAL_OpenT* openPtr,
                              const VAL_DecoderTypeT* decoderPtr)
{
    int32_t             err_code = FAPI_OK;
    VAL_OpenParamsT     openParams;
    VAL_OpEnumT         currOperation;
    uint32_t            currPid;
    VAL_PlayModeEnumT   currMode;
    
    do
    {
        currOperation = openPtr->operation;
        currPid       = openPtr->pid;
        currMode      = openPtr->mode;
        
        openParams         = openPtr->openParams;
        openParams.decoder = *decoderPtr;
        
        /* Stop potentially running decoder. */
        err_code = valIntStop(openPtr, 0);
        if ( err_code != FAPI_OK )
        {
            break;
        }
        
        err_code = valIntClose(openPtr);
        if ( err_code != FAPI_OK )
        {
            break;
        }
        
        err_code = valIntOpen(openPtr, &openParams);
        if ( err_code != FAPI_OK )
        {
            break;
        }
        
        if ( (currOperation == VAL_OP_LIVE)
          || (currOperation == VAL_OP_PLAY)
          || (currOperation == VAL_OP_STREAMING) )
        {
            /* Restart video with parameters from previous start. */
            
            err_code = valIntStart (openPtr, currPid, currMode);
            if ( err_code != FAPI_OK )
            {
                break;
            }
        }
    }
    while ( 0 );
    
    if ( err_code != FAPI_OK )
    {
        openPtr->operation = VAL_OP_ERROR;
    }
    
    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, open video input data channel.
**
** \param   openPtr         Handle returned by VAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valChannelClose
*******************************************************************************
*/
static int32_t valChannelOpen (VAL_OpenT* openPtr)
{
    FAPEX_TSD_OpenSubIdParamStrT  pidChParam;
    FAPEX_TSD_PesParamStrT        pesParam;
    int32_t  err_code = FAPI_OK;

    if (openPtr->tsdId >= 0)
    {
        return SUBSYS_VIDEO_VAL_ERR_ALREADY_OPEN;
    }

    do
    {
        /* open Video TSD channel */
        pidChParam.channelNo = VIDEO_TSD_CHANNEL;
        pidChParam.sub_id    = DRV_ID_TSD0_VIDEO_CH;

        err_code = FAPEX_TSD_Ioctl((FAPEXIOCARG1_T)fd_TSD[0],
                                   FAPEX_IOCCOM_TSD_OPEN_SUBID,
                                   (FAPEXIOCARG3_T)&pidChParam);
        if ( err_code != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(2, "[VAL] tsd channel open error: %d\n", err_code);
            break;
        }
        openPtr->tsdId = pidChParam.sub_id;

        pesParam.pid              = openPtr->pid;
        pesParam.flags            = FAPEX_TSD_FLTFLAGS_RM_TS_HEADER | FAPEX_TSD_FLTFLAGS_PTS |
                                    FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP | FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE;
        pesParam.bufferSize       = 0x007FFC00;
        pesParam.bufferThresholdH = pesParam.bufferSize;
        pesParam.bufferThresholdL = 0;
        pesParam.streamType       = FAPEX_TSD_STREAMTYPE_VIDEO;

        if( (openPtr->decoder.system != FAPEX_VIDEC_CODEC_TYPE_H264)
         || (openPtr->delimiterType != FAPEX_VIDEC_DELIMITER_PES_HEADER) ) {
            pesParam.flags |= FAPEX_TSD_FLTFLAGS_RM_PES_HEADER;
        }

        err_code = FAPEX_TSD_Ioctl(openPtr->tsdId,
                                   FAPEX_IOCCOM_TSD_SET_PES_FILTER,
                                   (FAPEXIOCARG3_T)&pesParam);
        if ( err_code != FAPI_OK )
        {
            break;
        }
        openPtr->bufferSize = pesParam.bufferSize;
    }
    while ( 0 );

    if ( err_code != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[VAL] valChannelOpen failed: %i\n", err_code);

        if ( openPtr->tsdId >= 0 )
        {
            (void)FAPEX_TSD_Ioctl((FAPEXIOCARG1_T)openPtr->tsdId,
                                   FAPEX_IOCCOM_CLOSE_SUBID,
                                   (FAPEXIOCARG3_T)NULL);
            openPtr->tsdId = -1;
        }
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, close video input data channel.
**
** \param   openPtr         Handle returned by VAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valChannelOpen
*******************************************************************************
*/
static int32_t valChannelClose (VAL_OpenT* openPtr)
{
    int32_t err_code = FAPI_OK;

    if ( openPtr->tsdId >= 0 )
    {
        err_code = FAPEX_TSD_Ioctl(openPtr->tsdId,
                                   FAPEX_IOCCOM_CLOSE_SUBID,
                                   (FAPEXIOCARG3_T)NULL);
    }

    if ( err_code == FAPI_OK )
    {
        openPtr->tsdId = -1;
    }

    openPtr->bufferSize = 0;

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, start video input data channel.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   pid             PID of stream to be decoded.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valChannelStop
*******************************************************************************
*/
static int32_t valChannelStart (VAL_OpenT* openPtr, uint16_t pid)
{
    FAPEX_TSD_PidParamStrT  pidParam;
    int32_t err_code = FAPI_OK;

#ifdef DESCRAMBLE_TEST
    FAPEX_TSD_DescKeysParamStrT  descParams;
    memset(&descParams, 0 , sizeof(FAPEX_TSD_DescKeysParamStrT));
    descParams.mode = FAPEX_DESC_DVB_CSA2_DECRYPT;
    descParams.oddKeyLength = 8;
   
    descParams.oddKey[0] = 0xBC;    
    descParams.oddKey[1] = 0xFB;    
    descParams.oddKey[2] = 0xB2;    
    descParams.oddKey[3] = 0x69;    
    descParams.oddKey[4] = 0x13;    
    descParams.oddKey[5] = 0xBA;    
    descParams.oddKey[6] = 0xBE;    
    descParams.oddKey[7] = 0x8B;    
#endif 
 

    if ( (openPtr->tsdId   < 0)
      || (openPtr->videcId < 0) )
    {
        return SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }

    do
    {
        pidParam.pid   = pid;
        pidParam.flags = (openPtr->pcrFlag != 0) ? FAPEX_TSD_FLTFLAGS_PCR : 0;

        err_code = FAPEX_TSD_Ioctl(openPtr->tsdId,
                                   FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                                   (FAPEXIOCARG3_T)&pidParam);

        if ( err_code != FAPI_OK ) break;

#ifdef DESCRAMBLE_TEST 
         err_code = FAPEX_TSD_Ioctl(openPtr->tsdId,
                                    FAPEX_IOCCOM_TSD_SET_KEYS,
                                    (FAPEXIOCARG3_T)&descParams);
         if( err_code != FAPI_OK)  break;

#endif

        err_code = FAPEX_TSD_Ioctl(openPtr->tsdId,
                                   FAPEX_IOCCOM_START,
                                   (FAPEXIOCARG3_T)NULL);
        if ( err_code != FAPI_OK ) break;
    }
    while ( 0 );

    if ( err_code != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[VAL] valChannelStart (pid %i) failed: %i\n",
                                pid, err_code);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, stop video input data channel.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   pid             PID of stream to be decoded.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valChannelStart
*******************************************************************************
*/
static int32_t valChannelStop (VAL_OpenT* openPtr)
{
    int32_t err_code = FAPI_OK;

    if (openPtr->tsdId >= 0)
    {
        err_code = FAPEX_TSD_Ioctl(openPtr->tsdId,
                                   FAPEX_IOCCOM_STOP,
                                   (FAPEXIOCARG3_T)NULL);
    }

    if ( err_code != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[VAL] valChannelStop failed: %i\n", err_code);
    }

    return err_code;
}

/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************
** \brief   Internal function, performs pre-initialisation of clocks.
**
** \param   hdSys           Requested HD video output system.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
*******************************************************************************
*/
static int32_t valViencInitClocks (VAL_HdEncoderT hdSys)
{
    int32_t  clkSys;

    switch ( hdSys )
    {
        case VAL_1920x1080x2397P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x2397P;
            break;

        case VAL_1920x1080x24P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x24P;
            break;

        case VAL_1920x1080x25P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x25P;
            break;

        case VAL_1920x1080x2997P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x2997P;
            break;

        case VAL_1920x1080x30P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x30P;
            break;

        case VAL_1920x1080x50I:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x50I;
            break;

        case VAL_1920x1080x50P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x50P;
            break;

        case VAL_1920x1080x5994I:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x5994I;
            break;

        case VAL_1920x1080x5994P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x5994P;
            break;

        case VAL_1920x1080x60I:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x60I;
            break;

        case VAL_1920x1080x60P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1920x1080x60P;
            break;

        case VAL_1280x720x2397P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1280x720x2397P;
            break;

        case VAL_1280x720x24P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1280x720x24P;
            break;

        case VAL_1280x720x25P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1280x720x25P;
            break;

        case VAL_1280x720x2997P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1280x720x2997P;
            break;

        case VAL_1280x720x30P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1280x720x30P;
            break;

        case VAL_1280x720x50P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1280x720x50P;
            break;

        case VAL_1280x720x5994P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1280x720x5994P;
            break;

        case VAL_1280x720x60P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1280x720x60P;
            break;

        case VAL_720x576x50I:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_720x576x50I;
            break;

        case VAL_720x576x50P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_720x576x50P;
            break;

        case VAL_720x480x5994I:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_720x480x5994I;
            break;

        case VAL_720x480x5994P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_720x480x5994P;
            break;

        case VAL_VGA_640x480x5994P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_640x480x5994P;
            break;

        case VAL_VGA_800x600x5994P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_800x600x5994P;
            break;

        case VAL_VGA_1024x768x5994P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1024x768x5994P;
            break;

        case VAL_VGA_1360x768x5994P:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_1360x768x5994P;
            break;

        default:
            clkSys = FAPEX_VIENC_SYSTEM_MODE_UNKNOWN;
            break;
    }

    return FAPEX_VIENC_Open(DRV_ID_VIENC, O_RDWR, (FAPEXOPENARG3_T)clkSys);
}

/*!
*******************************************************************************
** \brief   Internal function, open and start VIENC driver.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
** \param   hdSys       Requested HD video output system.
** \param   sdSys       Requested SD video output system.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valViencClose
*******************************************************************************
*/
static int32_t valViencOpen (VAL_DataT* dataPtr, VAL_HdEncoderT hdSys,
                              VAL_SdEncoderT sdSys)
{
    int32_t                         err_code = FAPI_OK;
    FAPEX_VIENC_OpenSubIDParamStrT  viencOpenParam;
    
    if ( (dataPtr->viencSubId[VAL_VOUT_SD] >= 0)
      || (dataPtr->viencSubId[VAL_VOUT_HD] >= 0) )
    {
        return SUBSYS_VIDEO_VAL_ERR_ALREADY_OPEN;
    }
    
    do
    {
        dataPtr->viencHdEncoder = hdSys;
        dataPtr->viencSdEncoder = sdSys;

        // HD Encoder
        viencOpenParam.sub_id                 = DRV_ID_VIENC_HD;
        viencOpenParam.encoderType            = FAPEX_VIENC_ENCODER_HD;
        viencOpenParam.colourSystem           = FAPEX_VIENC_COLOUR_SYSTEM_UNKNOWN;
        viencOpenParam.colourComponent        = FAPEX_VIENC_CCOMPONENT_YPrPb;
        viencOpenParam.crossChrominanceFilter = FAPEX_VIENC_CROSS_CHROMIANACE_FILTER_UNKNOWN;

        if(hdSys == VAL_1920x1080x2397P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x2397P;
        }
        else if(hdSys == VAL_1920x1080x24P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x24P;
        }
        else if(hdSys == VAL_1920x1080x25P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x25P;
        }
        else if(hdSys == VAL_1920x1080x2997P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x2997P;
        }
        else if(hdSys == VAL_1920x1080x30P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x30P;
        }
        else if(hdSys == VAL_1920x1080x50I){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x50I;
        }
        else if(hdSys == VAL_1920x1080x50P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x50P;
        }
        else if(hdSys == VAL_1920x1080x5994I){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x5994I;
        }
        else if(hdSys == VAL_1920x1080x5994P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x5994P;
        }
        else if(hdSys == VAL_1920x1080x60I){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x60I;
        }
        else if(hdSys == VAL_1920x1080x60P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x60P;
        }
        else if(hdSys == VAL_1280x720x2397P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1280x720x2397P;
        }
        else if(hdSys == VAL_1280x720x24P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1280x720x24P;
        }
        else if(hdSys == VAL_1280x720x25P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1280x720x25P;
        }
        else if(hdSys == VAL_1280x720x2997P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1280x720x2997P;
        }
        else if(hdSys == VAL_1280x720x30P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1280x720x30P;
        }
        else if(hdSys == VAL_1280x720x50P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1280x720x50P;
        }
        else if(hdSys == VAL_1280x720x5994P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1280x720x5994P;
        }
        else if(hdSys == VAL_1280x720x60P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1280x720x60P;
        }
        else if(hdSys == VAL_720x576x50I){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_720x576x50I;
        }
        else if(hdSys == VAL_720x576x50P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_720x576x50P;
        }
        else if(hdSys == VAL_720x480x5994I){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_720x480x5994I;
        }
        else if(hdSys == VAL_720x480x5994P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_720x480x5994P;
        }
        else if(hdSys == VAL_VGA_640x480x5994P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_640x480x5994P;
            viencOpenParam.colourComponent      = FAPEX_VIENC_CCOMPONENT_RGB;
        }
        else if(hdSys == VAL_VGA_800x600x5994P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_800x600x5994P;
            viencOpenParam.colourComponent      = FAPEX_VIENC_CCOMPONENT_RGB;
        }
        else if(hdSys == VAL_VGA_1024x768x5994P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1024x768x5994P;
            viencOpenParam.colourComponent      = FAPEX_VIENC_CCOMPONENT_RGB;
        }
        else if(hdSys == VAL_VGA_1360x768x5994P){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1360x768x5994P;
            viencOpenParam.colourComponent      = FAPEX_VIENC_CCOMPONENT_RGB;
        }
        else{
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_1920x1080x50I;
        }

        err_code = FAPEX_VIENC_Ioctl( dataPtr->viencId,
                                      FAPEX_IOCCOM_VIENC_OPEN_SUBID,
                                      (FAPEXIOCARG3_T)&viencOpenParam );
        if ( err_code != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(2, "[VAL] vienc-HD open error: %d\n", err_code);
            break;
        }
        dataPtr->viencSubId[VAL_VOUT_HD] = viencOpenParam.sub_id;

        err_code = FAPEX_VIENC_Ioctl( dataPtr->viencSubId[VAL_VOUT_HD],
                                      FAPEX_IOCCOM_START,
                                      (FAPEXIOCARG3_T)NULL );
        if ( err_code != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(2, "[VAL] vienc-HD start error: %d\n", err_code);
            break;
        }

        // SD Encoder
        viencOpenParam.sub_id                 = DRV_ID_VIENC_SD;
        viencOpenParam.encoderType            = FAPEX_VIENC_ENCODER_SD;
        viencOpenParam.colourComponent        = FAPEX_VIENC_CCOMPONENT_CVBS_YC;
        viencOpenParam.crossChrominanceFilter = FAPEX_VIENC_CROSS_CHROMIANACE_FILTER_OFF;

        if(sdSys == VAL_SD_SECAM){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_SECAM;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_SECAM;
        }
        else if(sdSys == VAL_SD_NTSC){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_NTSC;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_NTSC;
        }
        else if(sdSys == VAL_SD_NTSC_4_43){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_NTSC;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_NTSC_4_43;
        }
        else if(sdSys == VAL_SD_PAL){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_PAL;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_PAL;
        }
        else if(sdSys == VAL_SD_PAL60){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_NTSC;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_PAL_60;
        }
        else if(sdSys == VAL_SD_PAL_M){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_NTSC;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_PAL_M;
        }
        else if(sdSys == VAL_SD_PAL_N){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_PAL;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_PAL_N;
        }
        else if(sdSys == VAL_SD_PAL_N_ARG){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_PAL;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_PAL_N_ARG;
        }
        else if(sdSys == VAL_SD_OFF){
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_NTSC;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_NTSC;
            viencOpenParam.colourComponent      = FAPEX_VIENC_CCOMPONENT_DAC_POWER_DOWN;
        }
        else{
            viencOpenParam.interlaceProgressive = FAPEX_VIENC_IP_MODE_NTSC;
            viencOpenParam.colourSystem         = FAPEX_VIENC_COLOUR_SYSTEM_NTSC;
            viencOpenParam.colourComponent      = FAPEX_VIENC_CCOMPONENT_DAC_POWER_DOWN;
        }

        err_code = FAPEX_VIENC_Ioctl( dataPtr->viencId,
                                      FAPEX_IOCCOM_VIENC_OPEN_SUBID,
                                      (FAPEXIOCARG3_T)&viencOpenParam );
        if ( err_code != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(2, "[VAL] vienc-SD open error: %d\n", err_code);
            break;
        }
        dataPtr->viencSubId[VAL_VOUT_SD] = viencOpenParam.sub_id;

        err_code = FAPEX_VIENC_Ioctl( dataPtr->viencSubId[VAL_VOUT_SD],
                                      FAPEX_IOCCOM_START,
                                      (FAPEXIOCARG3_T)NULL );
        if ( err_code != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(2, "[VAL] vienc-SD start error: %d\n", err_code);
        }
    }
    while ( 0 );

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, close VIENC driver.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valViencOpen
*******************************************************************************
*/
static int32_t valViencClose (VAL_DataT* dataPtr)
{
    int32_t             err_code = FAPI_OK;
    VAL_VideoOutputT    output;

    err_code = FAPEX_VIENC_Ioctl( dataPtr->viencId,
                                  FAPEX_IOCCOM_STOP,
                                  (FAPEXIOCARG3_T)NULL );
    if(err_code != FAPI_OK)
    {
        return err_code;
    }

    for ( output = VAL_VOUT_SD; output < VAL_VOUT_NUMS; output++ )
    {
        if ( dataPtr->viencSubId[output] >= 0 )
        {
            err_code = FAPEX_VIENC_Ioctl( dataPtr->viencSubId[output],
                                          FAPEX_IOCCOM_CLOSE_SUBID,
                                          (FAPEXIOCARG3_T)NULL );
            if ( err_code == FAPI_OK )
            {
                dataPtr->viencSubId[output] = -1;
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(2, "[VAL] Close dataPtr->viencSubId[%d] failed. %d\n",
                                        output, err_code);
                return err_code;
            }
        }
    }
    err_code = FAPEX_VIENC_Close(dataPtr->viencId);

    if ( err_code == FAPI_OK )
    {
        dataPtr->viencId = -1;
    }
    else {
        FAPI_SYS_PRINT_DEBUG(2, "[VAL] FAPEX_VIENC_Close failed. %d\n", err_code);
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, open and start VISCALE driver.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
** \param   hdSys       Requested HD video output system.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valViscaleClose
*******************************************************************************
*/
static int32_t valViscaleOpen (VAL_DataT* dataPtr, VAL_HdEncoderT hdSys)
{
    int32_t                       err_code = FAPI_OK;
    FAPEX_VIOUT_OpenCmnParamStrT  vioutOpenCmnParam;
    FAPEX_VIOUT_OpenSubParamStrT  vioutOpenSubParam;
    FAPEX_VIOUT_ScalerParamStrT   *spPtr;
    FAPEX_VIOUT_AlphaParamStrT    alphaParam;

    if ( (dataPtr->vioutSubId[VAL_VOUT_SD] >= 0)
      || (dataPtr->vioutSubId[VAL_VOUT_HD] >= 0) )
    {
        return SUBSYS_VIDEO_VAL_ERR_ALREADY_OPEN;
    }
    else if ( dataPtr->viencSubId[VAL_VOUT_HD] < 0 )
    {
        return SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }

    vioutOpenCmnParam.enableFlag              = FAPEX_VIOUT_ENABLE_HDMI;
    vioutOpenCmnParam.hdmiParams.inputType    = FAPEX_HDMI_YUV444;
    vioutOpenCmnParam.hdmiParams.outputType   = FAPEX_HDMI_RGB;
    vioutOpenCmnParam.hdmiParams.scanInfo     = FAPEX_HDMI_SCAN_NO_DATA;
    vioutOpenCmnParam.hdmiParams.activeFormat = 0x08;
    vioutOpenCmnParam.hdmiParams.pictureAr    = FAPEX_VIOUT_DISPLAY_AR_NONE;
    vioutOpenCmnParam.outputId                = DRV_ID_VIENC_HD;

    dataPtr->vioutId = FAPEX_VIOUT_Open(DRV_ID_VIOUT, O_RDWR, &vioutOpenCmnParam);
    if(dataPtr->vioutId < 0)
    {
        return -errno;
    }
    fd_VIOUT = dataPtr->vioutId;

    err_code = FAPEX_VIOUT_Ioctl( dataPtr->vioutId,
                                  FAPEX_IOCCOM_VIOUT_SET_BGCOLOUR,
                                  (FAPEXIOCARG3_T)0x00108080 );  // black
    if ( err_code < 0 )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[VAL] set bg colour failed. %d\n", err_code);
        return err_code;
    }

    // HD Scaler
    vioutOpenSubParam.sub_id       = DRV_ID_VIOUT_HD;
    vioutOpenSubParam.scalerType   = FAPEX_VIOUT_SCALER_VIDEO;
    vioutOpenSubParam.topologyType = FAPEX_VIOUT_VO_BYPASS_DISABLE;

    err_code = FAPEX_VIOUT_Ioctl( dataPtr->vioutId,
                                  FAPEX_IOCCOM_VIOUT_OPEN_SUBID,
                                  (FAPEXIOCARG3_T)&vioutOpenSubParam );
    if ( err_code < 0 )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[VAL] viout-HD open error: %d\n", err_code);
        return err_code;
    }
    dataPtr->vioutSubId[VAL_VOUT_HD] = vioutOpenSubParam.sub_id;

    spPtr = dataPtr->vioutParams + (int32_t)VAL_VOUT_HD;

    spPtr->mode               = dataPtr->vioutHdScalingMethod;
    spPtr->osdByteOrder       = FAPEX_VIOUT_OSD_LAYOUT_UNKNOWN;
    spPtr->cpTileNumX         = 0;
    spPtr->cpTileNumY         = 0;
    spPtr->flagFrameField     = 0;
    spPtr->inputWindowX0      = 0;
    spPtr->inputWindowY0      = 0;
    spPtr->inputWindowWidth   = 0;
    spPtr->inputWindowHeight  = 0;
    spPtr->outputWindowX0     = 0;
    spPtr->outputWindowY0     = 0;
    spPtr->outputWindowWidth  = 0;
    spPtr->outputWindowHeight = 0;

    // TODO: add error handling

    err_code = FAPEX_VIOUT_Ioctl( dataPtr->vioutSubId[VAL_VOUT_HD],
                                  FAPEX_IOCCOM_VIOUT_SET_SCALER_PARAMS,
                                  (FAPEXIOCARG3_T)spPtr );
    if ( err_code < 0 )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[VAL] viout-HD set param error: %d\n", err_code);
        return err_code;
    }

    err_code = FAPEX_VIOUT_Ioctl( dataPtr->vioutSubId[VAL_VOUT_HD],
                                  FAPEX_IOCCOM_VIOUT_SET_ASPECT_RATIO,
                                  (FAPEXIOCARG3_T)dataPtr->vioutDispAr[VAL_VOUT_HD] );
    if ( err_code < 0 )
    {
        return err_code;
    }

    // SD Scaler
    vioutOpenSubParam.sub_id     = DRV_ID_VIOUT_SD;
    vioutOpenSubParam.scalerType = FAPEX_VIOUT_SCALER_SD;

    err_code = FAPEX_VIOUT_Ioctl( dataPtr->vioutId,
                                  FAPEX_IOCCOM_VIOUT_OPEN_SUBID,
                                  (FAPEXIOCARG3_T)&vioutOpenSubParam );
    if ( err_code < 0 )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[VAL] viout-SD open error: %d\n", err_code);
        return err_code;
    }
    dataPtr->vioutSubId[VAL_VOUT_SD] = vioutOpenSubParam.sub_id;

    spPtr = dataPtr->vioutParams + (int32_t)VAL_VOUT_SD;

    err_code = FAPEX_VIOUT_Ioctl( dataPtr->vioutSubId[VAL_VOUT_SD],
                                  FAPEX_IOCCOM_VIOUT_SET_SCALER_PARAMS,
                                  (FAPEXIOCARG3_T)spPtr );
    if ( err_code < 0 )
    {
        FAPI_SYS_PRINT_DEBUG(2, "[VAL] viout-SD set param error: %d\n", err_code);
        return err_code;
    }

    err_code = FAPEX_VIOUT_Ioctl( dataPtr->vioutSubId[VAL_VOUT_SD],
                                  FAPEX_IOCCOM_VIOUT_SET_ASPECT_RATIO,
                                  (FAPEXIOCARG3_T)dataPtr->vioutDispAr[VAL_VOUT_SD] );
    if ( err_code < 0 )
    {
        return err_code;
    }

#if 0
    /**** start Video Output (HD Scaler & SD Scaler) ****/
    err_code = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)dataPtr->vioutId,
                                  FAPEX_IOCCOM_START,
                                  (FAPEXIOCARG3_T)NULL );
    if(err_code < 0) {
        FAPI_SYS_PRINT_ERR("ERROR Start VIOUT driver. %d\n", err_code);
        return err_code;
    }
#else
    /**** start Video Output (HD Scaler) ****/
    err_code = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)dataPtr->vioutSubId[VAL_VOUT_HD],
                                  FAPEX_IOCCOM_START,
                                  (FAPEXIOCARG3_T)NULL );
    if ( err_code < 0 )
    {
        FAPI_SYS_PRINT_ERR("ERROR Start VIOUT[HD] driver. %d\n", err_code);
        return err_code;
    }

    /**** start Video Output (SD Scaler) ****/
    err_code = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)dataPtr->vioutSubId[VAL_VOUT_SD],
                                  FAPEX_IOCCOM_START,
                                  (FAPEXIOCARG3_T)NULL );
    if ( err_code < 0 )
    {
        FAPI_SYS_PRINT_ERR("ERROR Start VIOUT[SD] driver. %d\n", err_code);
        return err_code;
    }
#endif

    /**** set Video Output Alpha (HD Scaler) ****/
    alphaParam.mode  = FAPEX_VIOUT_PARAMETER_SET;
    alphaParam.alpha = 255;
    err_code = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)dataPtr->vioutSubId[VAL_VOUT_HD],
                                  FAPEX_IOCCOM_VIOUT_SET_ALPHA,
                                  (FAPEXIOCARG3_T)&alphaParam );
    if ( err_code < 0 )
    {
        FAPI_SYS_PRINT_ERR("ERROR Set Aplha VIOUT[HD] driver. %d\n", err_code);
    }
    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, close VISCALE driver.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valViscaleOpen
*******************************************************************************
*/
static int32_t valViscaleClose (VAL_DataT* dataPtr)
{
    int32_t             err_code = FAPI_OK;
    VAL_VideoOutputT    output;

    err_code = FAPEX_VIOUT_Ioctl( dataPtr->vioutId,
                                  FAPEX_IOCCOM_STOP,
                                  (FAPEXIOCARG3_T)NULL );
    if(err_code != FAPI_OK)
    {
        return err_code;
    }

    for ( output = VAL_VOUT_SD; output < VAL_VOUT_NUMS; output++ )
    {
        if ( dataPtr->vioutSubId[output] >= 0 )
        {
            err_code = FAPEX_VIOUT_Ioctl(dataPtr->vioutSubId[output],
                                         FAPEX_IOCCOM_CLOSE_SUBID,
                                         (FAPEXIOCARG3_T)NULL );
            if ( err_code == FAPI_OK )
            {
                dataPtr->vioutSubId[output] = -1;
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(2, "[VAL] Close dataPtr->vioutSubId[%d] failed. %d\n",
                                        output, err_code);
                break;
            }
        }
    }

    if ( err_code == FAPI_OK )
    {
        err_code = FAPEX_VIOUT_Close (dataPtr->vioutId);

        if(err_code == FAPI_OK) {
            dataPtr->vioutId = -1;
        }
        else {
            FAPI_SYS_PRINT_DEBUG(2, "[VAL] FAPEX_VIOUT_Close failed. %d\n", err_code);
        }
    }

    return err_code;
}

/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************
** \brief   Internal function, open HDMI driver.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      VAL_HdmiSetup, valHdmiClose
*******************************************************************************
*/
static int32_t valHdmiOpen (VAL_DataT* dataPtr)
{
    if ( dataPtr->hdmiId >= 0 )
    {
        return SUBSYS_VIDEO_VAL_ERR_ALREADY_OPEN;
    }
    
    dataPtr->hdmiId = FAPEX_HDMI_Open(DRV_ID_HDMI,
                                      O_RDWR,
                                      (FAPEXOPENARG3_T)NULL);
    if(dataPtr->hdmiId >= 0) {
        /* read EDID information if HDMI sink is already connected. */
        (void)valHdmiGetEDIDInfo (dataPtr);
        return 0;
    }
    FAPI_SYS_PRINT_DEBUG(2, "[VAL] valHdmiOpen failed. %d\n", dataPtr->hdmiId);
    return dataPtr->hdmiId;
}

/*!
*******************************************************************************
** \brief   Internal function, close HDMI driver.
**
** \param   dataPtr     Handle returned by VAL_GoUp.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      VAL_HdmiSetup, valHdmiOpen
*******************************************************************************
*/
static int32_t valHdmiClose (VAL_DataT* dataPtr)
{
    int32_t err_code = FAPI_OK;

    if ( dataPtr->hdmiId >= 0 )
    {
        err_code = FAPEX_HDMI_Close (dataPtr->hdmiId);

        if ( err_code == FAPI_OK )
        {
            dataPtr->hdmiId = -1;
        }
        else {
            FAPI_SYS_PRINT_DEBUG(2, "[VAL] valHdmiClose failed. %d\n", err_code);
        }
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, update HDMI connection status.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** \param   connectedPtr    Returns HDMI connection information if a non-NULL
**                          pointer is passed (1: connected, 0: not connected).
** \param   changePtr       Returns HDMI connection change indication if a 
**                          non-NULL pointer is passed (1: connection status
**                          has changed since last calling of this function, 
**                          0: not changed).
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valHdmiOpen, valHdmiClose
*******************************************************************************
*/
static int32_t valHdmiUpdateStatus (VAL_DataT* dataPtr, uint8_t* connectedPtr,
                                     uint8_t* changePtr)
{
    int32_t                 hpd_status;
    uint8_t                 statusChanged = 0;
    VAL_HDMIConnectParamsT  currParams;
    uint8_t                 hdcpStateChanged = 0;

    if(dataPtr->hdmiId < 0) {
        return 0;
    }
    currParams = dataPtr->hdmiConnectParams;

    FAPEX_HDMI_Ioctl(dataPtr->hdmiId,
                     FAPEX_IOCCOM_HDMI_CHECK_HPD,
                     (FAPEXIOCARG3_T)&hpd_status );

    //FAPI_SYS_PRINT_MSG("FAPI_HDMI_CheckHPD: %X\n", hpd_status);

    //HPD connected
    if ( (hpd_status & VAL_HDMI_HPD_STATUS_MASK) 
          == VAL_HDMI_HPD_STATUS_CONNECTED )
    {
        //state change from disconnected to connected
        if ( !(dataPtr->hdmiConnectParams.connected) )
        {
            FAPI_SYS_PRINT_DEBUG(3,"[VAL] HDMI CONNECTION DETECTED\n");

            dataPtr->hdmiConnectParams.connected = 1;

            statusChanged = 1;

            if(dataPtr->hdmiHDCPEnabled)
            {
                //update HDCP state
                dataPtr->hdmiHDCPState = VAL_HDMI_HDCPSTATE_AUTH_INIT;
                hdcpStateChanged =1;
            }
        }

        //if no haveEDID, get EDIDInfo
        if (   (dataPtr->hdmiConnectParams.connected)
          && (!(dataPtr->hdmiConnectParams.haveEDID)) )
        {
            if ( valHdmiGetEDIDInfo (dataPtr) == FAPI_OK )
            {
                /* restart HDMI video */ //because VID could be different
                (void)FAPEX_VIOUT_Ioctl( dataPtr->vioutId,
                                         FAPEX_IOCCOM_VIOUT_HDMI_RESTART,
                                         (FAPEXIOCARG3_T)NULL );
            }
        }
    }
    else //HPD fails
    {
        if ( dataPtr->hdmiConnectParams.connected )
        {
            FAPI_SYS_PRINT_DEBUG(3,"[VAL] HDMI UNCONNECTED\n");
            
            statusChanged = 1;
        }
        
        dataPtr->hdmiConnectParams.connected = 0;
        dataPtr->hdmiConnectParams.haveEDID  = 0;
    }

    if ( connectedPtr != NULL )
    {
        *connectedPtr = dataPtr->hdmiConnectParams.connected;
    }

    if ( changePtr != NULL )
    {
        *changePtr = statusChanged;
    }

    if ( dataPtr->hdmiConnectCb != NULL )
    {
        if ( (currParams.connected != dataPtr->hdmiConnectParams.connected)
          || (currParams.haveEDID  != dataPtr->hdmiConnectParams.haveEDID) )
        {
            dataPtr->hdmiConnectCb (&(dataPtr->hdmiConnectParams),
                                      dataPtr->hdmiConnectCbArg);
        }
    }

    if( hdcpStateChanged )
    {
        if ( dataPtr->hdmiHDCPStateCb != NULL )
        {
            dataPtr->hdmiHDCPStateCb(dataPtr->hdmiHDCPState,
                                     dataPtr->hdmiHDCPStateCbArg );
        }
        FAPI_SYS_PRINT_DEBUG(1, "[VAL] HDCP state=%i\n",
                                 dataPtr->hdmiHDCPState);
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
** \brief   Internal function, read EDID structure via HDMI if not available
**          yet.
**
** \param   dataPtr         Handle returned by VAL_GoUp.
** 
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
*******************************************************************************
*/
static int32_t valHdmiGetEDIDInfo (VAL_DataT* dataPtr)
{
    int32_t err_code = FAPI_OK;
    
    /* get EDID info */
    if (   (dataPtr->hdmiConnectParams.connected)
      && (!(dataPtr->hdmiConnectParams.haveEDID)) )
    {
        int32_t  status;
        
        err_code = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)dataPtr->hdmiId,
                                     FAPEX_IOCCOM_HDMI_READ_EDID,
                                     (FAPEXIOCARG3_T)&status );
        if ( status != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(3, "FAPI_HDMI_ReadEDID failed!\n");
        }
        else
        {
            err_code = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)dataPtr->hdmiId,
                                       FAPEX_IOCCOM_HDMI_GET_EDID_INFO,
                                       (FAPEXIOCARG3_T)&(dataPtr->hdmiEdid) );
            if ( err_code != FAPI_OK )
            {
                FAPI_SYS_PRINT_DEBUG(3, "FAPI_HDMI_GetEDID_Info failed!\n");
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(3, "[VAL] EDID info read!\n");
                
                dataPtr->hdmiConnectParams.haveEDID = 1; 
            }
        }
    }
    
    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, perform HDCP protocol/authentication handling.
** 
** This function is periodically called as long as an HDMI sink is connected.
** 
** \param   dataPtr         Handle returned by VAL_GoUp.
** 
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
*******************************************************************************
*/
static int32_t valHdmiHandleHDCP (VAL_DataT* dataPtr)
{
    int32_t              retVal = FAPI_OK;
    int32_t              status;
    VAL_HdmiHdcpStateT   state  = VAL_HDMI_HDCPSTATE_AUTH_INIT;

    if (   (dataPtr->hdmiId < 0)
      &&   (dataPtr->hdmiHDCPEnabled)
      &&   (dataPtr->hdmiConnectParams.connected)
      &&   (dataPtr->hdmiConnectParams.haveEDID) )
    {
        retVal = FAPEX_HDMI_Ioctl( (FAPEXIOCARG1_T)dataPtr->hdmiId,
                                   FAPEX_IOCCOM_HDMI_LINK_HDCP,
                                   (FAPEXIOCARG3_T)&status );

        if ( (retVal == 0) && (status != FAPEX_HDMI_STATUS_OK) )
        {
            state = VAL_HDMI_HDCPSTATE_AUTH_FAIL;

            FAPI_SYS_PRINT_DEBUG(1, "[VAL] FAPI_HDMI_LinkHDCP failed: %i\n",
                                 retVal);

            if ( retVal == FAPEX_HDMI_STATUS_ERR_HDCP_NOT_SUPPORT )
            {
                FAPI_SYS_PRINT_DEBUG(1,
                        "[VAL] HDCP requested but not supported by chip!\n");
            }
        }
        else
        {
            state = VAL_HDMI_HDCPSTATE_AUTH_SUCC;
        }
    }

    if(state != dataPtr->hdmiHDCPState)
    {
        dataPtr->hdmiHDCPState = state;
        if ( dataPtr->hdmiHDCPStateCb != NULL )
        {
            dataPtr->hdmiHDCPStateCb(state, dataPtr->hdmiHDCPStateCbArg );
        }
        FAPI_SYS_PRINT_DEBUG(1, "[VAL] HDCP state=%i\n", state);
    }

    return retVal;
}

/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************
** \brief   Internal function, check task for frame availability and information.
**
** \param   optData           Used to pass openPtr.
**
** \return  -
**
*******************************************************************************
*/
static void VAL_VidecCheckTask(void *optData)
{
    VAL_OpenT* openPtr = (VAL_OpenT*)optData;
    int32_t  ret;

    while(1)
    {
        ret = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                FAPEX_IOCCOM_VIDEC_GET_DEC_FRAME_INFO,
                                (FAPEXIOCARG3_T)&openPtr->videcFrameInfo);
        if(ret == 0) {
            if(openPtr->videcFrameInfo.enableFlag & FAPEX_VIDEC_INFO_DEC_AVAILABLE) {
                valVidecFrameAvlCb(optData);
            }
            if(openPtr->videcFrameInfo.enableFlag & FAPEX_VIDEC_INFO_DISP_ENABLE) {
                valVidecFrameInfoCb(optData);
            }
            if(openPtr->videcFrameInfo.enableFlag & FAPEX_VIDEC_INFO_DISP_AVAILABLE) {
                valVidecFrameDecCb(optData);
            }
            if(openPtr->videcFrameInfo.enableFlag & FAPEX_VIDEC_INFO_DISP_CHANGED) {
                valVidecParamChgCb(optData);
            }
            if(openPtr->videcFrameInfo.enableFlag & FAPEX_VIDEC_INFO_STOPPED) {
                break;
            }
        }
        else {
            break;
        }
    }
    openPtr->checkThread = NULL;
}

/*!
*******************************************************************************
** \brief   Internal function, open video decoder driver.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   decoderPtr      Requested decoder.
** \param   enableAfd       Set to true if AFD handling shall be enabled.
** \param   environment     Requested environment (live or playback).
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valVidecClose
*******************************************************************************
*/
static int32_t valVidecOpen (VAL_OpenT* openPtr,
                              const VAL_DecoderTypeT* decoderPtr,
                              uint8_t enableAfd,
                              VAL_EnvironmentT environment,
                              int32_t delimiterType)
{
    int32_t                    err_code = FAPI_OK;
    FAPEX_VIDEC_OpenParamStrT  params;
    uint32_t  output_dev;

    if ( openPtr->videcId >= 0)
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_ALREADY_OPEN;
    }
    else
    {
        params.inputId   = DRV_ID_TSD0_VIDEO_CH;
        params.codecType = decoderPtr->system;

        if ( decoderPtr->system == FAPEX_VIDEC_CODEC_TYPE_H264 )
        {
            params.delimiterType = delimiterType;
        }
        else
        {
            params.delimiterType = FAPEX_VIDEC_DELIMITER_AUD_HEADER;
        }
        
        if ( enableAfd )
        {
            params.afdFlag = FAPEX_VIDEC_AFD_ENABLE_AUTO;
        }
        else
        {
            params.afdFlag = FAPEX_VIDEC_AFD_DISABLE;
        }

        switch ( environment )
        {
            case VAL_ENV_LIVE:
                params.playbackEnv = FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_LIVE;
                break;

            case VAL_ENV_PLAY:
                params.playbackEnv = FAPEX_VIDEC_PLAYBACK_ENVIRONMENT_PVR;
                break;
        
            case VAL_ENV_STREAMING:
                params.playbackEnv = FAPEX_VIDEC_PALYBACK_ENVIRONMENT_STREAMING;
                break;

            default:
                err_code = SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER;
                break;
        }

        if ( err_code == FAPI_OK )
        {
            openPtr->videcId = FAPEX_VIDEC_Open(DRV_ID_VIDEC, O_RDWR, (FAPEXOPENARG3_T)&params);
            if(openPtr->videcId < 0)
            {
                err_code = openPtr->videcId;
                FAPI_SYS_PRINT_DEBUG(2, "[VAL] FAPEX_VIDEC_Open error. %d\n", err_code);
            }
            else {
                /* set output to Video Decoder */
                output_dev = DRV_ID_VIOUT;
                err_code = FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)openPtr->videcId,
                                              FAPEX_IOCCOM_VIDEC_SET_OUTPUT,
                                              (FAPEXIOCARG3_T)&output_dev );
                if(err_code < 0) {
                    FAPI_SYS_PRINT_ERR("ERROR VIDEC set output. %d\n", err_code);
                }

                if(openPtr->checkThread == NULL) {
                    openPtr->checkThread = FAPEX_SYS_CreateThread(
                                                NULL, 1024, 100,
                                                VAL_VidecCheckTask,
                                                openPtr, NULL, "checkThread" );
                }
            }
        }
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, close video decoder driver.
**
** \param   openPtr         Handle returned by VAL_Open.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valVidecOpen
*******************************************************************************
*/
static int32_t valVidecClose (VAL_OpenT* openPtr)
{
    int32_t err_code = FAPI_OK;

    if ( openPtr->videcId >= 0 )
    {
        err_code = FAPEX_VIDEC_Close(openPtr->videcId);

        if(openPtr->checkThread != NULL) {
            FAPEX_SYS_DestroyThread( openPtr->checkThread );
        }
        openPtr->videcId = -1;
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, start video decoder driver.
**
** \param   openPtr             Handle returned by VAL_Open.
** \param   operation           Requested operation.
** \param   mode                Requested playback mode.
** \param   feedViaTsd          Set to true to indicate data feeding via TSD
**                              (normal operation), only special playback modes
**                              require direct feeding of data.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valVidecStop
*******************************************************************************
*/
static int32_t valVidecStart (VAL_OpenT* openPtr,
                               int32_t mode,
                               uint8_t feedViaTsd)
{
    int32_t                         err_code = FAPI_OK;
    FAPEX_VIDEC_StartParamStrT      decStartParam;
    FAPEX_VIOUT_ResizeParamStrT     resizeParam;

    if ( openPtr->videcId < 0 )
    {
        return SUBSYS_VIDEO_VAL_ERR_INTERNAL;
    }

    do
    {
        // ToDo for feedViaTsd

        openPtr->frameAvailable          = 0;
        openPtr->frameDecoded            = 0;
        openPtr->videcFrameInfoAvailable = 0;
        openPtr->startFlags = FAPEX_VIDEC_NOTIFY_FLAGS_DEC_AVAILABLE |
                              FAPEX_VIDEC_NOTIFY_FLAGS_DISP_AVAILABLE |
                              FAPEX_VIDEC_NOTIFY_FLAGS_FRAME_INFO |
                              FAPEX_VIDEC_NOTIFY_FLAGS_PARAM_CHANGE;
        switch(mode) {
            case VAL_PLAYMODE_NORMAL:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_NORMAL;
                break;
            case VAL_PLAYMODE_FAST_LIB_IONLY:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_NORMAL;
                break;
            case VAL_PLAYMODE_PAUSE:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_PAUSE;
                break;
            case VAL_PLAYMODE_FAST_DEC_IONLY:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_I_ONLY;
                break;
            case VAL_PLAYMODE_FAST_DEC_IP:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_I_AND_P_ONLY;
                break;
            case VAL_PLAYMODE_FAST_CONTROLLEDFEED:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_FIRST_I_ONLY;
                break;
            case VAL_PLAYMODE_SLOW_2X:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_2X;
                break;
            case VAL_PLAYMODE_SLOW_4X:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_4X;
                break;
            case VAL_PLAYMODE_SLOW_8X:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_SLOW_FORWARD_8X;
                break;
            default:
                decStartParam.playbackMode = FAPEX_VIDEC_PLAYBACK_MODE_UNKNOWN;
        }
        decStartParam.outputMode = FAPEX_VIDEC_ENABLE_SYNCHRONOUSLY_NO_SUSPEND;
        decStartParam.flags      = openPtr->startFlags;

        err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                     FAPEX_IOCCOM_VIDEC_START,
                                     (FAPEXIOCARG3_T)&decStartParam);
        if ( err_code != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(2, "[VAL] valVidecStart: Videc start error! %d\n", err_code);
            break;
        }

        openPtr->decoderStatus  = VAL_STATUS_RUNNING;
    }
    while ( 0 );

    if ( err_code != FAPI_OK )
    {
        (void)FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                FAPEX_IOCCOM_STOP,
                                (FAPEXIOCARG3_T)NULL);
        
        openPtr->decoderStatus = VAL_STATUS_STOPPED;
    }

    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, stop video decoder driver.
**
** \param   openPtr         Handle returned by VAL_Open.
** \param   freeze          If true, freeze last picture instead of blanking
**                          video.
**
** \return  Possible return codes:
**          - FAPI_OK  Successful completion.
**
** \sa      valVidecStart
*******************************************************************************
*/
static int32_t valVidecStop (VAL_OpenT* openPtr, uint8_t freeze)
{
    int32_t err_code = FAPI_OK;
    
    if ( openPtr->videcId >= 0 )
    {
        if ( freeze )
        {
            err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                         FAPEX_IOCCOM_VIDEC_SET_FREEZE_MODE,
                                         (FAPEXIOCARG3_T)FAPEX_VIDEC_FREEZE_ON);
        }
        else
        {
            err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                         FAPEX_IOCCOM_VIDEC_SET_DISPOFF,
                                         (FAPEXIOCARG3_T)NULL);
        }
        
        if ( err_code == FAPI_OK )
        {
            err_code = FAPEX_VIDEC_Ioctl(openPtr->videcId,
                                         FAPEX_IOCCOM_STOP,
                                         (FAPEXIOCARG3_T)NULL);
        }
        
        openPtr->decoderStatus = VAL_STATUS_STOPPED;
    }
    
    return err_code;
}

/*!
*******************************************************************************
** \brief   Internal function, callback indicating that frame has been decoded.
**
** \param   optData           Used to pass openPtr.
**
** \return  -
**
*******************************************************************************
*/
static void valVidecFrameDecCb (void* optData)
{
    VAL_OpenT* openPtr = (VAL_OpenT*)optData;
    
    FAPI_SYS_PRINT_DEBUG(4, "[VAL] Callback: Video frame decoded!\n");
    
    openPtr->frameDecoded = 1;
}

/*!
*******************************************************************************
** \brief   Internal function, frame availability callback.
**
** \param   optData           Used to pass openPtr.
**
** \return  -
**
*******************************************************************************
*/
static void valVidecFrameAvlCb (void* optData)
{
    VAL_OpenT* openPtr = (VAL_OpenT*)optData;

    FAPI_SYS_PRINT_DEBUG(4, "[VAL] Callback: Video frame available!\n");

    openPtr->frameAvailable = 1;

    if ( openPtr->frameAvlCb != NULL )
    {
        openPtr->frameAvlCb(openPtr->frameAvlCbArg);
    }
}

/*!
*******************************************************************************
** \brief   Internal function, frame information callback.
**
** \param   optData           Used to pass openPtr.
**
** \return  -
**
*******************************************************************************
*/
static void valVidecFrameInfoCb (void* optData)
{
    VAL_OpenT* openPtr = (VAL_OpenT*)optData;
    
    openPtr->videcFrameInfoAvailable = 1;
}

static void valVidecParamChgCb (void* optData)
{
    VAL_OpenT* openPtr = (VAL_OpenT*)optData;
    
    FAPI_SYS_PRINT_DEBUG(4, "[VAL] Callback: Video parameters changed!\n");
    
    if ( openPtr->paramChangeCb != NULL )
    {
        openPtr->paramChangeCb(&openPtr->videcFrameInfo,
                               openPtr->paramChangeCbArg);
    }
}

/*!
*******************************************************************************
** \brief   Internal function, checks validity of openPtr handle.
**
** \param   openPtr           Pointer to be checked.
**
** \return  Possible return codes:
**          - SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE   Handle is a NULL pointer.
**          - SUBSYS_VIDEO_VAL_ERR_STATE            Module is in an error state,
**                                                  needs to be closed by
**                                                  using VAL_Close.
**          - SUBSYS_VIDEO_VAL_ERR_NOT_OPEN         VAL is not open.
**
*******************************************************************************
*/
static int32_t valArgCheckOpen (const VAL_OpenT* openPtr)
{
    int32_t err_code;

    if ( openPtr == NULL )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE;
    }
    else if ( (openPtr->dataPtr == NULL) || (openPtr->dataPtr->openPtr != openPtr) )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_NOT_OPEN;
    }
    else if ( openPtr->operation == VAL_OP_ERROR )
    {
        err_code = SUBSYS_VIDEO_VAL_ERR_STATE;
    }
    else
    {
        err_code = FAPI_OK;
    }

    return err_code;
}

/*lint -restore */
