/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/inc/app_lang.h
**
**              $Id: app_lang.h 5866 2010-03-02 12:09:31Z jmorha $ 
**
** \brief       OSD language handling.
** 
** In this file are all string ID's defined. To add a new string you have to add
** the string ID first. The string it self is place in
** /app_ref/res/all_language.xls, this file contains all strings for all 
** languages. The first collumn contains the String ID defined here. 
** (The ID-Name MUST be identical and case sensitv). The next collumns contains
** the string in different languages. The first cell of each collumn defines the
** language file name.
**
** Detail description. [\ref app_smartgo_app_ref_language]
** 
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _APP_LANG_H_
#define _APP_LANG_H_

#include "sys_api.h"

/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/
#define OSDLANG_MAX          4        // max. number of languages
#define OSDLANG_NAME_LEN     32       // max. length of language name string

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

/*! 
 * \brief OSD string definitions
 */
typedef enum
{
    STR_EMPTYSTRING = 0,
    
    STR_TITLE_MAIN,
    STR_TITLE_REC,
    STR_TITLE_TIMERPROG,
    STR_TITLE_TIMEREDIT,
    STR_TITLE_MOSAIC,
    STR_TITLE_STRSEL,
    STR_TITLE_SYSTEMSETUP,
    STR_TITLE_INFORMATION,
    STR_TITLE_LANGUAGE,
    STR_TITLE_TIME,
    STR_TITLE_ANTENNA,
    STR_TITLE_ADVD,
    STR_TITLE_OSD,
    STR_TITLE_PASSWORD,
    STR_TITLE_UTILITIES,
    STR_TITLE_SCAN_SAT,
    STR_TITLE_SCAN_TERR,
    STR_TITLE_SCAN_CABLE,
    STR_TITLE_SCAN_DIRECT,
    STR_TITLE_SCANNING,
    STR_TITLE_SCAN_ABORTED,
    STR_TITLE_SCAN_FINISHED,
    STR_TITLE_STREAMINF,
    STR_TITLE_ANTPROG,
    STR_TITLE_SATINSTTYPE,
    STR_TITLE_SATLIST,
    STR_TITLE_TPLIST,
    STR_TITLE_SERVLIST,
    STR_TITLE_EXPMOTOR,
    STR_TITLE_TUNER,
    STR_TITLE_TUNER_SAT,
    STR_TITLE_TUNER_TERR,
    STR_TITLE_TUNER_CAB,
    STR_TITLE_TUNER_LNB,
    STR_TITLE_TUNER_DVBT,
    STR_TITLE_WELCOME,
    STR_TITLE_STOPREC,
    STR_TITLE_PBLIST,
    STR_TITLE_EDITREC,
    STR_TITLE_VIEWER,
    STR_TITLE_MEDIA_DIALOG,
    STR_TITLE_MP3,
    STR_TITLE_CI,
    STR_TITLE_CA,
    STR_TITLE_UPDATE,
    STR_TITLE_UPDATING,
    STR_TITLE_BUSY,
    STR_TITLE_DEBUG,
    STR_TITLE_EDITSTRING,
    STR_TITLE_SCALERDEMO,
    STR_TITLE_NETWORK,
    STR_TITLE_NETWORK_IP,
    STR_TITLE_NETWORK_INFO,
    STR_TITLE_NETWORK_DNS,
    STR_TITLE_OAD_SELTP,
    
    /* Main Menu */
    STR_ENTRY_MAIN_VIEWER,
    STR_ENTRY_MAIN_MEDIA,
    STR_ENTRY_MAIN_RECORD,
    STR_ENTRY_MAIN_TIMER,
    STR_ENTRY_MAIN_MOSAIC,
    STR_ENTRY_MAIN_STRSEL,
    STR_ENTRY_MAIN_SETUP,
    STR_ENTRY_MAIN_INFO,

    /* Recordings */
    STR_ENTRY_PLIST_DATE,
    STR_ENTRY_PLIST_TIME,
    STR_ENTRY_PLIST_TITLE,
    STR_ENTRY_PLIST_TYPE,
    STR_ENTRY_PLIST_DURATION,
    STR_ENTRY_PLIST_DRIVE,
    STR_ENTRY_PLIST_TV,
    STR_ENTRY_PLIST_RADIO,
    STR_ENTRY_PLIST_REC,

    /* System setup */
    STR_ENTRY_SYSTEMSETUP_LANG,
    STR_ENTRY_SYSTEMSETUP_TIME,
    STR_ENTRY_SYSTEMSETUP_ANTENNA,
    STR_ENTRY_SYSTEMSETUP_ADVD,
    STR_ENTRY_SYSTEMSETUP_OSD,
    STR_ENTRY_SYSTEMSETUP_UTILITIES,
    STR_ENTRY_SYSTEMSETUP_PASSWORD,
    STR_ENTRY_SYSTEMSETUP_FACTORY,
    STR_ENTRY_SYSTEMSETUP_POWER,
    STR_ENTRY_SYSTEMSETUP_CI,
    STR_ENTRY_SYSTEMSETUP_CA,
    STR_ENTRY_SYSTEMSETUP_CICA,
    STR_ENTRY_SYSTEMSETUP_UPDATE,
    STR_ENTRY_SYSTEMSETUP_NETWORK,

    /* Information */
    STR_ENTRY_INFORMATION_FRONTEND,
    STR_ENTRY_INFORMATION_CI,
    STR_ENTRY_INFORMATION_MEMAVAIL,
    STR_ENTRY_INFORMATION_HDD,
    STR_ENTRY_INFORMATION_USB,
    STR_ENTRY_INFORMATION_INFOTXT,
    STR_ENTRY_INFORMATION_CURRSAT,
    STR_ENTRY_INFORMATION_CURRSYS,
    STR_ENTRY_INFORMATION_CURRTP,
    STR_ENTRY_INFORMATION_CURRCH,
    STR_ENTRY_INFORMATION_STRENGTH,
    STR_ENTRY_INFORMATION_QUALITY,
    STR_ENTRY_INFORMATION_STREAM,
    STR_ENTRY_INFORMATION_DEBUG,
    STR_ENTRY_INFORMATION_DEV_NOT_AVL,
    STR_ENTRY_INFORMATION_DEV_EXT_ACCESS,
    STR_ENTRY_INFORMATION_DEV_NOT_FORMAT,

    /* Timer programming */
    STR_ENTRY_TIMERPROG_DATE,
    STR_ENTRY_TIMERPROG_TIME,
    STR_ENTRY_TIMERPROG_ACTIVE,
    STR_ENTRY_TIMERPROG_PROG,
    STR_ENTRY_TIMERPROG_TITLE,
    STR_ENTRY_TIMERPROG_MODE,
    STR_ENTRY_TIMERPROG_AFTER,
    STR_ENTRY_TIMERPROG_RECTO,

    /* Timer Edit */
    STR_ENTRY_TIMEREDIT_NAME,
    STR_ENTRY_TIMEREDIT_DATE,
    STR_ENTRY_TIMEREDIT_STIME,
    STR_ENTRY_TIMEREDIT_ETIME,
    STR_ENTRY_TIMEREDIT_PROG,
    STR_ENTRY_TIMEREDIT_MODE,
    STR_ENTRY_TIMEREDIT_PWRMODE,
    STR_ENTRY_TIMEREDIT_RECTO,

    /* Language settings */
    STR_ENTRY_LANGUAGE_OSD,
    STR_ENTRY_LANGUAGE_AUDIO1,
    STR_ENTRY_LANGUAGE_AUDIO2,
    STR_ENTRY_LANGUAGE_SUBT1,
    STR_ENTRY_LANGUAGE_SUBT2,
    STR_ENTRY_LANGUAGE_MHEG,
    
    /* Time Settings */
    STR_ENTRY_TIME_REGION,
    STR_ENTRY_TIME_GMT,
    STR_ENTRY_TIME_DST,
    STR_ENTRY_TIME_DATE,
    STR_ENTRY_TIME_TIME,
    STR_ENTRY_TIME_AUTOTIME,
    STR_VALUE_TIME_AUTOTIME_ENABLE,
    STR_VALUE_TIME_AUTOTIME_DISABLE,
    STR_ENTRY_TIME_AUTOZONE,
    STR_VALUE_TIME_AUTOZONE_ENABLE,
    STR_VALUE_TIME_AUTOZONE_DISABLE,
    STR_TIME_MSG_INFO,
    STR_TIME_MSG_GRIDEPG,
    STR_TIME_MSG_TIMER_REC,
    STR_TIME_MSG_INSTANT_REC,
    STR_TIME_MSG_TIMERPROG,

    /* Network */
    STR_ENTRY_NETWORK_IPADDR,
    STR_ENTRY_NETWORK_DNS,
    STR_ENTRY_NETWORK_INFO,

    /* Network IP/Info  */
    STR_ENTRY_IP_STATUS,
    STR_ENTRY_IP_SPEED,
    STR_ENTRY_IP_DHCP,
    STR_ENTRY_IP_ADDR,
    STR_ENTRY_IP_NETMASK,
    STR_ENTRY_IP_GATEWAY,
    STR_ENTRY_IP_MAC,
    
    /* Network DNS/Info */
    STR_ENTRY_DNS_MODE,
    STR_ENTRY_DNS_PREV,
    STR_ENTRY_DNS_ALT,

    /* Antenna and program setup */
    STR_ENTRY_ANTPROG_GUID,
    STR_ENTRY_ANTPROG_SATINST,
    STR_ENTRY_ANTPROG_SATLIST,
    STR_ENTRY_ANTPROG_TUNER,
    STR_ENTRY_ANTPROG_TPLIST,
    STR_ENTRY_ANTPROG_SCAN_SAT,
    STR_ENTRY_ANTPROG_SCAN_TERR,
    STR_ENTRY_ANTPROG_SCAN_CABLE,
    STR_ENTRY_ANTPROG_SCAN_DIRECT,
    STR_ENTRY_ANTPROG_SERVLIST,
    STR_ENTRY_ANTPROG_ENDOFGUIDED,

    /* Type of satellite instalation */
    STR_ENTRY_SATINSTTYPE_HEADER,
    STR_ENTRY_SATINSTTYPE_INFO,
    STR_ENTRY_SATINSTTYPE_INFOTXT,
    STR_ENTRY_SATINSTTYPE_TUNERLOOP,
    STR_ENTRY_SATINSTTYPE_2LNB1DISH,
    STR_ENTRY_SATINSTTYPE_2DISH,
    STR_ENTRY_SATINSTTYPE_1TSGL,
    STR_ENTRY_SATINSTTYPE_1TMUX,
    STR_ENTRY_SATINSTTYPE_1TMOT,
    STR_ENTRY_SATINSTTYPE_SAMTV,
    STR_ENTRY_SATINSTTYPE_PRE1,
    STR_ENTRY_SATINSTTYPE_PRE2,
    STR_ENTRY_SATINSTTYPE_PRE3,
    STR_ENTRY_SATINSTTYPE_PRE4,
    STR_ENTRY_SATINSTTYPE_NOSAT,

    /* Satellite list */
    STR_ENTRY_SATLIST_HEADER,
    STR_ENTRY_SATLIST_SELSAT,
    STR_ENTRY_SATLIST_NAME,
    STR_ENTRY_SATLIST_ORBPOS,
    STR_ENTRY_SATLIST_WEFLAG,

    /* Tuner Configuration */
    STR_ENTRY_TUNER_CONTROLMETHOD,
    STR_ENTRY_TUNER_SELSAT,
    STR_ENTRY_TUNER_HIGHFREQ,
    STR_ENTRY_TUNER_LOWFREQ,
    STR_ENTRY_TUNER_CLASSICID,
    STR_ENTRY_TUNER_TONEBURSTID,
    STR_ENTRY_TUNER_DISEQC10ID,
    STR_ENTRY_TUNER_DISEQC11ID,
    STR_ENTRY_TUNER_DISEQC11ID_SMATV,
    STR_ENTRY_TUNER_DISEQC11REP,
    STR_ENTRY_TUNER_DISEQC12POS,
    STR_ENTRY_TUNER_APPLY,
    STR_ENTRY_TUNER_GOTOPOS,
    STR_ENTRY_TUNER_MOVESTYLE,
    STR_ENTRY_TUNER_MOVELR,
    STR_ENTRY_TUNER_STORESAT,
    STR_ENTRY_TUNER_IF_FREQUNCY,
    STR_ENTRY_TUNER_EXPMOTOR,
    STR_ENTRY_TUNER_SELTP,
    STR_ENTRY_TUNER_LOCK,
    STR_ENTRY_TUNER_LONGITUDE,
    STR_ENTRY_TUNER_LATITUDE,
    STR_ENTRY_TUNER_TURNMOTOR,
    STR_ENTRY_TUNER_GOTO,
    STR_ENTRY_TUNER_GOTO_NEXT,
    STR_ENTRY_TUNER_ANTENNA_POWER,

    /* Scan Menu */
    STR_ENTRY_SCAN_SATELLITE,
    STR_ENTRY_SCAN_METHOD,
    STR_ENTRY_SCAN_MODE,
    STR_ENTRY_SCAN_TVRADIO,
    STR_ENTRY_SCAN_EDITTP,
    STR_ENTRY_SCAN_EDITCH_TERR,
    STR_ENTRY_SCAN_EDITCH,
    STR_ENTRY_SCAN_REGION,
    STR_ENTRY_SCAN_NETWORK,
    STR_ENTRY_SCAN_STARTSCAN,
    STR_ENTRY_SCAN_SAT_GOTO,
    STR_ENTRY_SCAN_TERR_GOTO,
    STR_ENTRY_SCAN_CABLE_GOTO,
    STR_ENTRY_SCAN_DIRECT_GOTO,
    STR_ENTRY_SCAN_FREQUENCY,
    STR_ENTRY_SCAN_SYMBOL,
    STR_ENTRY_SCAN_MODULATION,

    /* Scanning Menu */
    STR_ENTRY_SCANNING_SCANSTATUS,
    STR_ENTRY_SCANNING_NEWTP,
    STR_ENTRY_SCANNING_NEWSERV,
    STR_ENTRY_SCANNING_TVLIST,
    STR_ENTRY_SCANNING_RADIOLIST,

    /* Service list */
    STR_ENTRY_SERVLIST_GLOBTV,
    STR_ENTRY_SERVLIST_GLOBRADIO,
    STR_ENTRY_SERVLIST_FAV,
    STR_ENTRY_SERVLIST_GOTO,

    /* Expert motor settings */
    STR_ENTRY_EXPMOTOR_GOTO0,
    STR_ENTRY_EXPMOTOR_RECALC,
    STR_ENTRY_EXPMOTOR_LIMITS,
    STR_ENTRY_EXPMOTOR_EASTLIMIT,
    STR_ENTRY_EXPMOTOR_WESTLIMIT,

    /* Audio/Video setup */
    STR_ENTRY_ADVD_VOUT,
    STR_ENTRY_ADVD_FORMAT,
    STR_ENTRY_ADVD_SCALING,
    STR_ENTRY_ADVD_HD,
    STR_ENTRY_ADVD_SD,
    STR_ENTRY_ADVD_SCREENSHOT,
    STR_ENTRY_ADVD_SCREENSHOTTO,
    STR_ENTRY_ADVD_STARTMODE,
    STR_ENTRY_ADVD_SWITCHMODE,
    STR_ENTRY_ADVD_VIDEOFADE,
    STR_ENTRY_ADVD_AOUT,
    STR_ENTRY_ADVD_RFMODCH,
    STR_ENTRY_ADVD_RFTVSTD,
    STR_ENTRY_ADVD_LIPSYNCMODE,
    STR_ENTRY_ADVD_LIPSYNCLEVEL,

    /* OSD setup */
    STR_ENTRY_OSD_DISPLAYTIME,
    STR_ENTRY_OSD_TRANSPARENCY,
    STR_ENTRY_OSD_COLORSCHEME,
    STR_ENTRY_OSD_HORALIGN,
    STR_ENTRY_OSD_VERALIGN,

    /* Password and parental settings */
    STR_ENTRY_PASSWORD_SETUP,
    STR_ENTRY_PASSWORD_LOCKEDCH,
    STR_ENTRY_PASSWORD_MASTERPW,
    STR_ENTRY_PASSWORD_AGE,
    STR_ENTRY_PASSWORD_INFOSTREAM,
    STR_MSG_PASSWORD_ENTER_PIN,
    STR_MSG_PASSWORD_CURR_PIN,
    STR_MSG_PASSWORD_NEW_PIN,
    STR_MSG_PASSWORD_CONFIRM_PIN,
    STR_MSG_PASSWORD_PIN_CHANGED,
    STR_MSG_PASSWORD_ERROR,
    STR_GNR_PIN_DIGITSET,
    STR_GNR_PIN_DIGITFREE,
    
    /* HDD utilities */
    STR_ENTRY_HDD_DRIVE,
    STR_ENTRY_HDD_INFO,
    STR_ENTRY_HDD_FORMAT,
    STR_ENTRY_HDD_POWER,
    STR_ENTRY_HDD_USBRECONNECT,
    STR_ENTRY_HDD_USBAUTOCON,
    STR_ENTRY_HDD_USBFORMAT,
    STR_ENTRY_HDD_JUMPTIME,

    /* Stream Information */
    STR_ENTRY_STREAMINFO_VIDEOPID,
    STR_ENTRY_STREAMINFO_VIDEOPIC,
    STR_ENTRY_STREAMINFO_VIDEOFRATE,
    STR_ENTRY_STREAMINFO_VIDEOFORMAT,
    STR_ENTRY_STREAMINFO_VIDEOMAXBITRATE,
    STR_ENTRY_STREAMINFO_VIDOEREALBITRATE,
    STR_ENTRY_STREAMINFO_AUDIOPID,
    STR_ENTRY_STREAMINFO_AUDIOFORMAT,
    STR_ENTRY_STREAMINFO_AUDIOMODE,
    STR_ENTRY_STREAMINFO_AUDIOSAMPLE,
    STR_ENTRY_STREAMINFO_AUDIOBITRATE,
    STR_ENTRY_STREAMINFO_PCRPID,
    STR_ENTRY_STREAMINFO_TTXPID,
    STR_ENTRY_STREAMINFO_AUDIOPIDSPDIF,

    /* Stream selection */
    STR_ENTRY_STREAMSEL_SUBTTSTR,
    STR_ENTRY_STREAMSEL_AUDIOSTR,
    STR_ENTRY_STREAMSEL_SPDIFSTR,
    STR_ENTRY_STREAMSEL_TTX_ADDON,
    STR_ENTRY_STREAMSEL_DISP_4_3,
    STR_ENTRY_STREAMSEL_DISP_16_9,
    STR_ENTRY_STREAMSEL_DISP_2_21_1,
    STR_ENTRY_STREAMSEL_DISP_HD,
    STR_ENTRY_STREAMSEL_HEARINGIMPAIRED,
    STR_ENTRY_STREAMSEL_VISUALIMPAIRED,
    STR_ENTRY_STREAMSEL_EFFECTS,
    STR_ENTRY_STREAMSEL_AUDIOFORMAT_AC3,
    STR_ENTRY_STREAMSEL_AUDIOFORMAT_AAC,
    STR_ENTRY_STREAMSEL_AUDIOFORMAT_HEAAC,
    STR_ENTRY_STREAMSEL_AUDIOFORMAT_DDPDCV,

    /* Transponder list */
    STR_ENTRY_TPLIST_SELSAT,
    STR_ENTRY_TPLIST_SELTP,
    STR_ENTRY_TPLIST_FREQ,
    STR_ENTRY_TPLIST_SYMBRATE,
    STR_ENTRY_TPLIST_POL,
    STR_ENTRY_TPLIST_MODULATION,
    STR_ENTRY_TPLIST_SELCABLE,

    /* debug menu */
    STR_ENTRY_DEBUG_STREAMINF,
    STR_ENTRY_DEBUG_FORMATFFS,
    STR_ENTRY_DEBUG_FORMATHDD,

    /* special help entries */
    /* has to replaced by fujitsu symbol font */
    STR_ENTRY_OKDOWN,
    STR_ENTRY_LOKDOWNR,
    STR_ENTRY_09LR,
    STR_ENTRY_L,
    STR_ENTRY_R,
    STR_ENTRY_U,
    STR_ENTRY_D,
    STR_ENTRY_UD,
    STR_ENTRY_LOKR,
    STR_ENTRY_OK,
    STR_ENTRY_LR,
    STR_ENTRY_09,

    /* Message strings */
    STR_MSG_RECSTOPPED_DISKFULL,
    STR_MSG_RECSTOPPED_PERFORMANCE,
    STR_MSG_RECSTART_FAILED,
    STR_MSG_RECSTART_FAILED_HDD_FULL,
    STR_MSG_RECSTART_FAILED_USB_FULL,
    STR_MSG_PLEASEWAIT,
    STR_MSG_RECSTARTED,
    STR_MSG_RECSTOPPED,
    STR_MSG_NOHDD,
    STR_MSG_NORECDEVICE,
    STR_MSG_NOSERVICES,
    STR_MSG_FORMATFFS,
    STR_MSG_FORMATHDD,
    STR_MSG_NOTLOCKED,
    STR_MSG_PROHIBDURINGRECORD,
    STR_MSG_OUTOFMEMORY,
    STR_MSG_NOMOSAIC,
    STR_MSG_NORECPOSSIBLE,
    STR_MSG_NOINSTRECPOSSIBLE,
    STR_MSG_NOMORESATPOSSIBLE,
    STR_MSG_FAILURERETURNED,
    STR_MSG_NOTIMPLEMENTEDYET,
    STR_MSG_NOMOREMARKER,
    STR_MSG_MARKERSETSUCCESS,
    STR_MSG_SERVICE_NOT_RUNNING,
    STR_MSG_SCREENSAVED_HDD,
    STR_MSG_SCREENSAVED_USB,
    STR_MSG_SCREENFAILED,
    STR_MSG_HARDWARE_ERROR,
    STR_MSG_USBMSC_DISCONNECT,
    STR_MSG_USBMSC_CONNECT_ACCESS,
    STR_MSG_USBMSC_CONNECT_BLOCKED,
    STR_MSG_FS_DEVICE_NOT_FORMATTED,
    STR_MSG_MP3_FORMAT_NOT_SUPPORT,
    STR_MSG_FS_HDD_FORMAT_FAILED,
    STR_MSG_HDD_SPINUP,
    STR_MSG_HDD_NOT_ENOUGHT_SPACE,
    STR_MSG_HDD_DISKFULL,
    STR_MSG_USB_NOT_ENOUGHT_SPACE,
    STR_MSG_USB_DISKFULL,
    STR_MSG_MOTOR_MOVES,
    STR_MSG_PIC_NOT_SUPPORT,
    STR_MSG_HDCP_AUTH_PROGRESS,
    STR_MSG_HDCP_AUTH_FAILED,
    STR_MSG_INVALIDSTRING,
    
    /* File viewer menu */
    STR_ENTRY_VIEWER_VOLUME,
    STR_ENTRY_VIEWER_JPEG_SIZE,
    STR_ENTRY_VIEWER_JPEG_DIM,
    STR_ENTRY_VIEWER_MP3_ARTIST,
    STR_ENTRY_VIEWER_MP3_TITLE,
    STR_ENTRY_VIEWER_MP3_ALBUM,
    STR_ENTRY_VIEWER_MP3_UNKNOWN,
    STR_ENTRY_VIEWER_DELETE,
    STR_ENTRY_VIEWER_COPY,
    STR_ENTRY_VIEWER_DELETE_FILE,
    STR_ENTRY_VIEWER_DELETE_FOLDER,
    STR_ENTRY_VIEWER_DELETE_RECORDING,
    STR_ENTRY_VIEWER_DELETE_OK,
    STR_ENTRY_VIEWER_DELETE_CANCEL,
    STR_ENTRY_VIEWER_OVERWRITE_FILE,
    STR_ENTRY_VIEWER_OVERWRITE_FOLDER,
    STR_ENTRY_VIEWER_OVERWRITE_OK,
    STR_ENTRY_VIEWER_OVERWRITE_ABORT,
    STR_ENTRY_VIEWER_NEW_FOLDER,
    STR_ENTRY_VIEWER_RENAME,
    STR_ENTRY_VIEWER_OPTIONS,
    STR_ENTRY_VIEWER_MOVE,
    STR_ENTRY_VIEWER_ABORTDELETE,
    STR_ENTRY_VIEWER_ABORTMOVE,
    STR_ENTRY_VIEWER_ABORTCOPY,
    STR_ENTRY_VIEWER_COPYTO,
    STR_ENTRY_VIEWER_MOVETO,
    STR_ENTRY_VIEWER_DESTINATION,
    STR_ENTRY_VIEWER_CANCELRENAME,
    STR_ENTRY_VIEWER_UPDATENAME,
    STR_ENTRY_VIEWER_UPDATESUPPLIER,
    STR_ENTRY_VIEWER_UPDATEVERSION,
    STR_ENTRY_VIEWER_INVALID_DIMENSION,
    STR_ENTRY_VIEWER_INVALID_FORMAT,
    
    /* Media Menu */
    STR_ENTRY_MEDIA_DIALOG_OPENPIC,
    STR_ENTRY_MEDIA_DIALOG_OPENMUSIC,
    STR_ENTRY_MEDIA_DIALOG_LEAVE,
    STR_ENTRY_MEDIA_PIC_NONESELECTED,
    STR_ENTRY_MEDIA_MUSIC_NONESELECTED,
    STR_ENTRY_MEDIA_SLIDESHOW_ACTIVE,
    STR_ENTRY_MEDIA_SHUFFLE_ACTIVE,
    STR_ENTRY_MEDIA_HELP_PIC_NEXT,
    STR_ENTRY_MEDIA_HELP_PIC_PREV,
    STR_ENTRY_MEDIA_HELP_PIC_SLIDESHOW_START,
    STR_ENTRY_MEDIA_HELP_PIC_SLIDESHOW_STOP,
    STR_ENTRY_MEDIA_HELP_SHUFFLE_ON,
    STR_ENTRY_MEDIA_HELP_SHUFFLE_OFF,
    STR_ENTRY_MEDIA_HELP_MUSIC_NEXT,
    STR_ENTRY_MEDIA_HELP_MUSIC_PREV,
    STR_ENTRY_MEDIA_HELP_MUSIC_PLAY_START,
    STR_ENTRY_MEDIA_HELP_MUSIC_PLAY_RESUME,
    STR_ENTRY_MEDIA_HELP_MUSIC_STOP,
    STR_ENTRY_MEDIA_HELP_MUSIC_PAUSE,
    
    STR_ENTRY_JPEG_PLAY,
    STR_ENTRY_JPEG_NEXT,
    STR_ENTRY_JPEG_PREVIOUS,
    STR_ENTRY_JPEG_MENU,
    STR_ENTRY_JPEG_INFO,
    STR_ENTRY_JPEG_ZOOMIN,
    STR_ENTRY_JPEG_ZOOMOUT,
    STR_ENTRY_JPEG_ROTATE,
    STR_ENTRY_JPEG_FOLDER,
    STR_ENTRY_JPEG_SUBFOLDERS,
    STR_ENTRY_JPEG_SHUFFLE,
    STR_ENTRY_JPEG_PICTURES,
    STR_ENTRY_JPEG_STOP,
    STR_ENTRY_JPEG_HIDESHOW,
    STR_ENTRY_JPEG_VIEWER,

    STR_ENTRY_MP3_NOW,
    STR_ENTRY_MP3_NEXT,
    STR_ENTRY_MP3_SKIPTHIS,
    STR_ENTRY_MP3_SKIPNEXT,
    STR_ENTRY_MP3_REPEAT,
    STR_ENTRY_MP3_SHUFFLEON,
    STR_ENTRY_MP3_SHUFFLEOFF,
    STR_ENTRY_MP3_UNKNOWN_TITLE,
    STR_ENTRY_MP3_UNKNOWN_ARTIST,
    STR_ENTRY_MP3_UNKNOWN_ALBUM,
    
    STR_ENTRY_CI_SLOT,
    STR_ENTRY_CI_SLOTEMPTY,
    STR_ENTRY_CI_MODULEAVL,

    STR_ENTRY_UPDATE_FROM_FILE,
    STR_ENTRY_UPDATE_FROM_AIR,
    STR_ENTRY_UPDATE_REVERT,
    STR_ENTRY_UPDATE_CURRENT_VERSION,
    STR_ENTRY_UPDATE_PREV_VERSION,
    STR_ENTRY_UPDATE_UPDATE,
    STR_ENTRY_UPDATE_REPLACE,
    STR_ENTRY_UPDATE_BY,
    STR_ENTRY_UPDATE_CHOOSE,
    STR_ENTRY_UPDATE_DELETE,
    STR_ENTRY_UPDATE_HINT,
    STR_ENTRY_UPDATE_NORECORD,
    STR_ENTRY_UPDATE_ABORT,
    STR_ENTRY_UPDATE_ABORT_INFO,
    STR_ENTRY_UPDATE_ABORT_OK,
    STR_ENTRY_UPDATE_ABORT_DECLINE,
    STR_ENTRY_UPDATE_SAVESSU,
    STR_ENTRY_UPDATE_CONFIRM,
    STR_ENTRY_UPDATE_OAD_START,
    STR_ENTRY_UPDATE_INFO,
    
    STR_ENTRY_OAD_STARTUPDATE,
    STR_ENTRY_OAD_SATELLITE,
    STR_ENTRY_OAD_EDITTP,

    STR_INFO_UPDATE_SYSBUSY,
    STR_INFO_UPDATE_INVALID_DATA,
    STR_INFO_UPDATE_GET_DATA,
    STR_INFO_UPDATE_PROGRAMMING,
    STR_INFO_UPDATE_ABORT,
    STR_INFO_UPDATE_ERROR,
    STR_INFO_UPDATE_SUCCESS,

    STR_TTX_SELECT_TITLE,
    STR_TTX_SELECT_MHEG,
    STR_TTX_SELECT_TTX,
    /* General strings */
    STR_GNR_EMPTY,
    STR_GNR_ON,
    STR_GNR_OFF,
    STR_GNR_VOLUME,
    STR_GNR_REMOVEITEM,   // no string defined
    STR_GNR_SIGSTRENGTH,
    STR_GNR_SIGQUALITY,
    STR_GNR_ADDTOTIMERS,
    STR_GNR_EDITTIMER,
    STR_GNR_ACTIVATETIMER,
    STR_GNR_TOGGLELISTS,
    STR_GNR_MONDAY,
    STR_GNR_TUESDAY,
    STR_GNR_WEDNESDAY,
    STR_GNR_THURSDAY,
    STR_GNR_FRIDAY,
    STR_GNR_SATURDAY,
    STR_GNR_SUNDAY,
    STR_GNR_TODAY,
    STR_GNR_TOMORROW,
    STR_GNR_EVTCURR,
    STR_GNR_EVTNEXT,
    STR_GNR_SPDIFPCM,
    STR_GNR_DOLBYDIGITAL,
    STR_GNR_SAVEREQ,
    STR_GNR_SAVEOK,
    STR_GNR_SAVEABORT,
    STR_GNR_SAVEDISCARD,
    STR_GNR_NOEITCURR,
    STR_GNR_NOEITNEXT,
    STR_GNR_STREAMDISABLED,
    STR_GNR_STRINGCUT,
    STR_GNR_COMMA,
    STR_GNR_COLON,
    STR_GNR_BACKSLASH,
    STR_GNR_SPACE,
    STR_GNR_MINUS,
    STR_GNR_2_MINUS,
    STR_GNR_STAR,
    STR_GNR_BRAKES_OPEN,
    STR_GNR_BRAKES_CLOSE,
    STR_GNR_AUTO,
    STR_GNR_AGE,
    STR_GNR_N_A,

    /* single Keys */
    STR_KEY_BACK,
    STR_KEY_MENU,
    STR_KEY_EXIT,
    STR_KEY_EPG,
    STR_KEY_TV,
    STR_KEY_REW,
    STR_KEY_FF,
    STR_KEY_OK,
    STR_KEY_PLAY,
    STR_KEY_REC,
    STR_KEY_STOP,
    STR_KEY_PAUSE,
    STR_KEY_PRPLUS,
    STR_KEY_PRMINUS,
    STR_KEY_INFO,
    STR_KEY_APPL,
    STR_KEY_TEXT,
    STR_KEY_RIGHT,
    STR_KEY_F1,
    STR_KEY_F2,
    STR_KEY_F3,
    STR_KEY_F4,
    
    /* double keys */
    STR_KEYS_REWFF,

    /* Frontend-specific elements */
    STR_ENTRY_SCAN_LNB,
    STR_ENTRY_SCAN_TRANSPONDER,
    STR_ENTRY_SCAN_POL,
    STR_GNR_HORIZONTAL,
    STR_GNR_VERTICAL,
    STR_GNR_SCANTPFOUND,
    STR_GNR_SCANSTAT_HOR,
    STR_GNR_SCANSTAT_VER,
    STR_GNR_TRANSPONDER,

    STR_ENTRY_SCAN_CHANNEL,
    STR_GNR_CHANNEL,
    STR_GNR_SCANCHANFOUND,
    STR_GNR_GUARD_1_4,
    STR_GNR_GUARD_1_8,
    STR_GNR_GUARD_1_16,
    STR_GNR_GUARD_1_32,
    STR_GNR_BW_6MHZ,
    STR_GNR_BW_7MHZ,
    STR_GNR_BW_8MHZ,
    STR_GNR_MOD_QPSK,
    STR_GNR_MOD_QAM16,
    STR_GNR_MOD_QAM32,
    STR_GNR_MOD_QAM64,
    STR_GNR_MOD_QAM128,
    STR_GNR_MOD_QAM256,
    STR_GNR_MOD_AUTO,

    STR_VALUE_TIMERPROG_MODE_SINGLE,
    STR_VALUE_TIMERPROG_MODE_WEEKDAYS,
    STR_VALUE_TIMERPROG_MODE_WEEKLY,
    STR_VALUE_TIMERPROG_MODE_WEEKENDS,
    STR_VALUE_TIMERPROG_MODE_DAILY,

    STR_VALUE_TIMERPROG_ACTIVE_YES,
    STR_VALUE_TIMERPROG_ACTIVE_NO,

    STR_VALUE_TIMERPROG_AFTEREC_STANDBY,
    STR_VALUE_TIMERPROG_AFTEREC_PWRUP,

    STR_VALUE_TIMERPROG_RECTO_INTHD,
    STR_VALUE_TIMERPROG_RECTO_USB,

    STR_VALUE_TIMEREDIT_PWRMODE_STDBY,
    STR_VALUE_TIMEREDIT_PWRMODE_PWRUP,

    STR_VALUE_TIMEREDIT_MODE_SINGLE,
    STR_VALUE_TIMEREDIT_MODE_WEEKDAYS,
    STR_VALUE_TIMEREDIT_MODE_WEEKLY,
    STR_VALUE_TIMEREDIT_MODE_WEEKENDS,
    STR_VALUE_TIMEREDIT_MODE_DAILY,

    STR_VALUE_TIMEREDIT_RECTO_INTHD,
    STR_VALUE_TIMEREDIT_RECTO_USB,

    STR_VALUE_SYSTEMSETUP_STANDBY,
    STR_VALUE_SYSTEMSETUP_UP,

    STR_VALUE_TUNER_CONFMODE_CLASSIC,
    STR_VALUE_TUNER_CONFMODE_TONEBURST,
    STR_VALUE_TUNER_CONFMODE_DISEQC10,
    STR_VALUE_TUNER_CONFMODE_DISEQC11_DTH,
    STR_VALUE_TUNER_CONFMODE_DISEQC11_SMATV,
    STR_VALUE_TUNER_CONFMODE_DISEQC12,
    STR_VALUE_TUNER_CONFMODE_USALS,

    STR_VALUE_TUNER_DISEQC11REP_NONE,
    STR_VALUE_TUNER_DISEQC11REP_1,
    STR_VALUE_TUNER_DISEQC11REP_2,
    
    STR_VALUE_TUNER_MOVSTYLE_STEP,
    STR_VALUE_TUNER_MOVSTYLE_CONT,
    STR_VALUE_TUNER_MOVSTYLE_1SEC,

    STR_VALUE_TUNER_MOTLR_STOP,
    STR_VALUE_TUNER_MOTLR_LEFT,
    STR_VALUE_TUNER_MOTLR_RIGHT,

    STR_VALUE_TUNER_LOCK_LOCKED,
    STR_VALUE_TUNER_LOCK_UNLOCKED,

    STR_VALUE_TUNER_SATOFF,

    STR_VALUE_TUNER_CLASSICID_ON,
    STR_VALUE_TUNER_CLASSICID_OFF,

    /* Values Audio/Video setup */
    STR_VALUE_ADVD_VOUT_YCRCB,
    STR_VALUE_ADVD_VOUT_SVIDEO,
    STR_VALUE_ADVD_VOUT_YC,
    STR_VALUE_ADVD_VOUT_RGB,

    STR_VALUE_ADVD_VFORMAT_43,
    STR_VALUE_ADVD_VFORMAT_169,

    STR_VALUE_ADVD_METHOD_LETTERBOX,
    STR_VALUE_ADVD_METHOD_FULLSCREEN,
    STR_VALUE_ADVD_METHOD_CCO,
    STR_VALUE_ADVD_METHOD_TVSCALES,

    STR_VALUE_ADVD_TV_SD_AUTO,
    STR_VALUE_ADVD_TV_PAL,
    STR_VALUE_ADVD_TV_NTSC,
    STR_VALUE_ADVD_TV_SECAM,
    STR_VALUE_ADVD_TV_PAL_M,
    STR_VALUE_ADVD_TV_PAL_N,
    STR_VALUE_ADVD_TV_PAL_NAR,
    STR_VALUE_ADVD_TV_NTSC443,
    STR_VALUE_ADVD_TV_HD_AUTO,
    STR_VALUE_ADVD_TV_576P,
    STR_VALUE_ADVD_TV_576I,
    STR_VALUE_ADVD_TV_720P,
    STR_VALUE_ADVD_TV_1080I,
    
    STR_VALUE_ADVD_SCRNSHOT_VIDEO,
    STR_VALUE_ADVD_SCRNSHOT_OSDVIDEO,
    STR_VALUE_ADVD_SCRNSHOTTO_INTHD,
    STR_VALUE_ADVD_SCRNSHOTTO_USB,

    STR_VALUE_ADVD_START_FAST,
    STR_VALUE_ADVD_START_SYNCED,

    STR_VALUE_ADVD_SWITCH_FREEZE,
    STR_VALUE_ADVD_SWITCH_BLANK,

    STR_VALUE_ADVD_FADE_OFF,
    STR_VALUE_ADVD_FADE_SLOW,
    STR_VALUE_ADVD_FADE_FAST,

    STR_VALUE_ADVD_AOUT_PCMONLY,
    STR_VALUE_ADVD_AOUT_PREFERPASSTHROUGH,

    STR_VALUE_ADVD_RF_MOD_RF_MOD_OFF,
    STR_VALUE_ADVD_RF_MOD_RF_MOD_TEST_SIGNAL,
    STR_VALUE_ADVD_RF_MOD_RF_MOD_PALSEC_B,
    STR_VALUE_ADVD_RF_MOD_RF_MOD_PALSEC_BGH,
    STR_VALUE_ADVD_RF_MOD_RF_MOD_PAL_I,
    STR_VALUE_ADVD_RF_MOD_RF_MOD_PALSEC_DKL,
    STR_VALUE_ADVD_RF_MOD_RF_MOD_NTSC_MN,
    STR_VALUE_ADVD_RF_MOD_RF_MOD_NTSC_M,

    STR_VALUE_ADVD_LIPSYNC_AUTO,
    STR_VALUE_ADVD_LIPSYNC_MANUAL,
    STR_VALUE_ADVD_LIPSYNC_NOINFO,
    
    STR_VALUE_OSD_COLOR_BLUE,
    STR_VALUE_OSD_COLOR_GREEN,
    STR_VALUE_OSD_COLOR_RED,
    STR_VALUE_OSD_COLOR_BLACK,
    
    STR_VALUE_OSD_ALIGN_CENTER,
    STR_VALUE_OSD_ALIGN_RIGHT,
    STR_VALUE_OSD_ALIGN_LEFT,
    STR_VALUE_OSD_ALIGN_TOP,
    STR_VALUE_OSD_ALIGN_BOTTOM,

    STR_VALUE_SATLIST_WEFLAG_NONE,
    STR_VALUE_SATLIST_WEFLAG_WEST,
    STR_VALUE_SATLIST_WEFLAG_EAST,
    STR_VALUE_SATLIST_NEWSAT,
    STR_VALUE_SATLIST_NOSAT,

    STR_VALUE_TPLIST_MOD_QPSK,
    STR_VALUE_TPLIST_MOD_QPSK_S2,
    STR_VALUE_TPLIST_MOD_8PSK,

    STR_VALUE_TPLIST_POL_HOR,
    STR_VALUE_TPLIST_POL_VER,
    STR_VALUE_TPLIST_NEWTP,

    STR_VALUE_IP_LINKDOWN,
    STR_VALUE_IP_LINKUP,
    
    STR_VALUE_IP_HALF_DUPLEX,
    STR_VALUE_IP_FULL_DUPLEX,
    
    STR_VALUE_IP_DHCP_OFF,
    STR_VALUE_IP_DHCP_ON,

    STR_VALUE_DNS_AUTO,
    STR_VALUE_DNS_MAN,
    
    STR_VALUE_HDD_SPINDOWN_5MIN,
    STR_VALUE_HDD_SPINDOWN_15MIN,
    STR_VALUE_HDD_SPINDOWN_30MIN,
    STR_VALUE_HDD_SPINDOWN_1H,

    STR_VALUE_STREAMINFO_AUDIOFORMAT_AC3,
    STR_VALUE_STREAMINFO_AUDIOFORMAT_AAC,
    STR_VALUE_STREAMINFO_AUDIOFORMAT_WMA,
    STR_VALUE_STREAMINFO_AUDIOFORMAT_EC3,
    STR_VALUE_STREAMINFO_AUDIOFORMAT_I,
    STR_VALUE_STREAMINFO_AUDIOFORMAT_II,
    STR_VALUE_STREAMINFO_AUDIOFORMAT_III,
    STR_VALUE_STREAMINFO_AUDIOFORMAT_HEAAC,
    STR_VALUE_STREAMINFO_AUDIOFORMAT_INVALID,

    STR_VALUE_STREAMINFO_AUDIOMODE_MONO,
    STR_VALUE_STREAMINFO_AUDIOMODE_STEREO,
    STR_VALUE_STREAMINFO_AUDIOMODE_DUALMONO,
    STR_VALUE_STREAMINFO_AUDIOMODE_C,
    STR_VALUE_STREAMINFO_AUDIOMODE_L_C_R,
    STR_VALUE_STREAMINFO_AUDIOMODE__L_R_S,
    STR_VALUE_STREAMINFO_AUDIOMODE_L_C_R_S,
    STR_VALUE_STREAMINFO_AUDIOMODE_L_R_SL_SR,
    STR_VALUE_STREAMINFO_AUDIOMODE_L_C_R_SL_SR,
    STR_VALUE_STREAMINFO_AUDIOMODE_INVALID,

    STR_VALUE_STREAMINFO_AUDIOSAMPLE_INVALID,

    STR_VALUE_STREAMINFO_AUDIOBITRATE_INVALID,
    STR_VALUE_STREAMINFO_AUDIOBITRATE_FREE,

    STR_VALUE_STREAMINFO_VIDEOFORMAT_MPEG1,
    STR_VALUE_STREAMINFO_VIDEOFORMAT_MPEG2,
    STR_VALUE_STREAMINFO_VIDEOFORMAT_H264,
    STR_VALUE_STREAMINFO_VIDEOFORMAT_VC1,
    STR_VALUE_STREAMINFO_VIDEOFORMAT_UNKNOWN,

    STR_VALUE_STREAMINFO_VIDEOFRATE_INVALID,
    STR_VALUE_STREAMINFO_VIDEOPID_INVALID,

    STR_VALUE_SCAN_MODULATION_16,
    STR_VALUE_SCAN_MODULATION_32,
    STR_VALUE_SCAN_MODULATION_64,
    STR_VALUE_SCAN_MODULATION_128,
    STR_VALUE_SCAN_MODULATION_256,
    STR_VALUE_SCAN_MODULATION_AUTO,

    STR_VALUE_SCAN_METHOD_AUTO,
    STR_VALUE_SCAN_METHOD_CHANNEL,
    STR_VALUE_SCAN_METHOD_MANUAL,
    STR_VALUE_SCAN_METHOD_CURRTS,
    STR_VALUE_SCAN_METHOD_BLIND,

    STR_VALUE_SCAN_MODE_ALL,
    STR_VALUE_SCAN_MODE_FREEONLY,
    STR_VALUE_SCAN_MODE_CRYPTONLY,

    STR_VALUE_SCAN_CHOICE_ALL,
    STR_VALUE_SCAN_CHOICE_TV,
    STR_VALUE_SCAN_CHOICE_RADIO,

    STR_VALUE_SCAN_LISTFULL,

    STR_VALUE_SCAN_STAT_SINGLE,
    STR_VALUE_SCAN_STAT_CURRENTTS,

    STR_VALUE_SCAN_REGION_WESTEUROPE,
    STR_VALUE_SCAN_REGION_EASTEUROPE,
    STR_VALUE_SCAN_REGION_UK,
    STR_VALUE_SCAN_REGION_FRANCE,
    STR_VALUE_SCAN_REGION_ITALY,
    STR_VALUE_SCAN_REGION_IRELAND,
    STR_VALUE_SCAN_REGION_CHINA,
    STR_VALUE_SCAN_REGION_USA,
    STR_VALUE_SCAN_REGION_CANADA,
    STR_VALUE_SCAN_REGION_JAPAN,
    STR_VALUE_SCAN_REGION_AUSTRALIA,

    STR_VALUE_SCAN_NETWORK_RASTER,
    STR_VALUE_SCAN_NETWORK_CUSTOM,

    STR_VALUE_SCANNING_NEWSERV_TV,
    STR_VALUE_SCANNING_NEWSERV_RADIO,

    STR_VALUE_DAYLIGHT_SAVINGS_ON,
    STR_VALUE_DAYLIGHT_SAVINGS_OFF,

    STR_VALUE_SERVLIST_DELETED,

    /* Help Strings */
    STR_HELP_MENU_BACK,
    STR_HELP_MENU_EXIT,
    STR_HELP_MENU_OPEN,

    STR_EPG_HELP_ADDTOTIMERS,
    STR_EPG_HELP_PREV_DAY,
    STR_EPG_HELP_NEXT_DAY,
    STR_EPG_HELP_SELECT_SERVICE,
    STR_EPG_HELP_EXTINFO,

    STR_HELP_SATLIST_ADDSAT,
    STR_HELP_SATLIST_REMOVESAT,

    STR_HELP_TIMERPROG_EDIT,
    STR_HELP_TIMERPROG_ACTIVATE,
    STR_HELP_TIMERPROG_REMOVE,

    STR_HELP_TIMEREDIT_REC,
    STR_HELP_TIMEREDIT_DELTHIS,
    STR_HELP_TIMEREDIT_DELPREV,
    STR_HELP_TIMEREDIT_INSBLANK,
    STR_HELP_TIMEREDIT_SAVEANDEXIT,

    STR_HELP_TPLIST_ADDTP,
    STR_HELP_TPLIST_REMOVETP,

    STR_HELP_SERVLIST_SWITCHLIST,
    STR_HELP_SERVLIST_TOGGLETVRA,
    STR_HELP_SERVLIST_IDXUPDOWN,
    STR_HELP_SERVLIST_ADDTOFAV,
    STR_HELP_SERVLIST_TUNE,
    STR_HELP_SERVLIST_SELECTLIST,
    STR_HELP_SERVLIST_REMOVEITEM,
    STR_HELP_SERVLIST_SORTBYNAME,
    STR_HELP_SERVLIST_SORTBYIDX,
    STR_HELP_SERVLIST_CLEARLISTS,
    STR_HELP_SERVLIST_LOCK,
    STR_HELP_SERVLIST_RENAME_SERVICE,

    STR_HELP_SERVBOX_TV,
    STR_HELP_SERVBOX_RADIO,
    STR_HELP_SERVBOX_FAV,
    STR_HELP_SERVBOX_OPEN_MENU,
    
    STR_HELP_SCANNING_STOP,
    STR_HELP_SCANNING_TOGGLETVRA,

    STR_HELP_PLIST_PLAYBACK,
    STR_HELP_PLIST_EDIT,
    STR_HELP_PLIST_REMOVE,
    STR_HELP_PLIST_RENAME,

    STR_HELP_RECORDING_STOP,

    STR_HELP_EDITREC_EXPHELPBOT_HALF,
    STR_HELP_EDITREC_EXPHELPBOT_SMALL,
    STR_HELP_EDITREC_REMIDX,
    STR_HELP_EDITREC_GENERAL_IDX,
    STR_HELP_EDITREC_ABIDX,
    STR_HELP_EDITREC_CUTIDX,
    STR_HELP_EDITREC_SLOMOBACK,
    STR_HELP_EDITREC_PLUS5MIN,
    STR_HELP_EDITREC_MINUS5MIN,
    STR_HELP_EDITREC_SLOMOFORW,
    STR_HELP_EDITREC_STOP,
    STR_HELP_EDITREC_REWIND,
    STR_HELP_EDITREC_PLAY,
    STR_HELP_EDITREC_PAUSE,
    STR_HELP_EDITREC_FFORW,
    STR_HELP_EDITREC_GO2NEXTIDX,
    STR_HELP_EDITREC_GO2PREVIDX,
    STR_HELP_EDITREC_SYMBOL_STOP,
    STR_HELP_EDITREC_SYMBOL_PLAY,
    STR_HELP_EDITREC_SYMBOL_2FF,
    STR_HELP_EDITREC_SYMBOL_4FF,
    STR_HELP_EDITREC_SYMBOL_8FF,
    STR_HELP_EDITREC_SYMBOL_16FF,
    STR_HELP_EDITREC_SYMBOL_32FF,
    STR_HELP_EDITREC_SYMBOL_REW,
    STR_HELP_EDITREC_SYMBOL_2REW,
    STR_HELP_EDITREC_SYMBOL_4REW,
    STR_HELP_EDITREC_SYMBOL_8REW,
    STR_HELP_EDITREC_SYMBOL_16REW,
    STR_HELP_EDITREC_SYMBOL_32REW,
    STR_HELP_EDITREC_SYMBOL_1SLOMOF2,
    STR_HELP_EDITREC_SYMBOL_1SLOMOF4,
    STR_HELP_EDITREC_SYMBOL_1SLOMOF8,
    STR_HELP_EDITREC_SYMBOL_PAUSE,
    STR_HELP_EDITREC_SYMBOL_NEXTIDX,
    STR_HELP_EDITREC_SYMBOL_PREVIDX,
    STR_EDITREC_ELAPSED_TIME_DESC,
    STR_EDITREC_TOTAL_TIME_DESC,
    STR_EDITREC_INFO_BKGRREC_PLAYBACK,
    STR_EDITREC_INFO_OTHER_PLAYBACK,
    STR_HELP_EDITREC_JUMPPLUSONE,
    STR_HELP_EDITREC_JUMPMINUSONE,
    STR_HELP_EDITSTRING_SYMBOL_RED,
    STR_HELP_EDITSTRING_SYMBOL_GREEN,
    STR_HELP_EDITSTRING_SYMBOL_YELLOW,
    STR_HELP_EDITSTRING_KEY_ALL,
    STR_HELP_EDITSTRING_KEY_NUM,
    STR_HELP_EDITSTRING_KEY_CHAR,
    STR_HELP_EDITSTRING_KEY_UPPER,
    STR_HELP_EDITSTRING_KEY_LOWER,
    STR_HELP_EDITSTRING_KEY_SPECIAL,
    STR_HELP_EDITSTRING_MENU,
    STR_HELP_EDITSTRING_EXIT,
    STR_HELP_EDITSTRING_NUMKEY,
    STR_HELP_EDITSTRING_LEFTRIGHT,
    STR_HELP_EDITSTRING_UPDOWN,

    /* Update Menu */
    STR_HELP_UPDATE_CANCEL,

    /* SYMBOLS from FJ_SYMBOL_BOLD.ftf */
    STR_SYMBOL_RECORD,
    STR_SYMBOL_STOP,
    STR_SYMBOL_PAUSE,
    STR_SYMBOL_PLAY,
    STR_SYMBOL_REW,
    STR_SYMBOL_FF,
    STR_SYMBOL_PREV,
    STR_SYMBOL_NEXT,
    STR_SYMBOL_1_8,
    STR_SYMBOL_1_4,
    STR_SYMBOL_1_2,
    STR_SYMBOL_LEFT,
    STR_SYMBOL_UP,
    STR_SYMBOL_RIGHT,
    STR_SYMBOL_DOWN,
    STR_SYMBOL_CHECK,

    /* Confirm Dialog */
    STR_CONFDIALOG_SAVEOK,
    STR_CONFDIALOG_SAVEABORT,
    STR_CONFDIALOG_SAVEDISCARD,
    STR_CONFDIALOG_SERVLIST,
    STR_CONFDIALOG_SCANNING,
    STR_CONFDIALOG_PLAYLIST,
    STR_CONFDIALOG_RESTORFACTORY,
    STR_CONFDIALOG_FORMATHDD,
    STR_CONFDIALOG_FORMATUSB,
    STR_CONFDIALOG_FORMATINFO,
    STR_CONFDIALOG_FORMATOK,
    STR_CONFDIALOG_FORMATABORT,
    STR_CONFDIALOG_REM_REC,
    STR_CONFDIALOG_REM_REC_OK,
    STR_CONFDIALOG_REM_REC_ABORT,
    STR_CONFDIALOG_SAVE_TIMER,
    STR_CONFDIALOG_SAVE_TIMER_OK,
    STR_CONFDIALOG_SAVE_TIMER_ABORT,
    STR_CONFDIALOG_REM_SAT,
    STR_CONFDIALOG_REM_SAT_OK,
    STR_CONFDIALOG_REM_SAT_ABORT,
    STR_CONFDIALOG_REM_TP,
    STR_CONFDIALOG_REM_TP_OK,
    STR_CONFDIALOG_REM_TP_ABORT,
    STR_CONFDIALOG_SSS_PROBLEM,
    STR_CONFDIALOG_SSS_REPORT,
    STR_CONFDIALOG_SSS_PROBLEMEX,
    STR_CONFDIALOG_SSS_REPORTEX,
    STR_CONFDIALOG_PVR_PLAY_START_INFO,
    STR_CONFDIALOG_PVR_PLAY_START_ACTION,
    STR_CONFDIALOG_PVR_PLAY_LASTSTOP,
    STR_CONFDIALOG_PVR_PLAY_BEGINNING,
    STR_CONFDIALOG_REM_MARKERINFO,
    STR_CONFDIALOG_REM_MARKERTITLE,
    STR_CONFDIALOG_REM_MARKER_OK,
    STR_CONFDIALOG_REM_MARKER_ABORT,

    /* Units */
    STR_UNIT_MSEC,
    STR_UNIT_SECS,
    STR_UNIT_MIN,
    STR_UNIT_HOUR,
    STR_UNIT_DAY,
    STR_UNIT_MONTH,
    STR_UNIT_YEAR,
    STR_UNIT_DEG,
    STR_UNIT_PERCENT,
    STR_UNIT_BYTE,
    STR_UNIT_KBYTE,
    STR_UNIT_MBYTE,
    STR_UNIT_HZ,
    STR_UNIT_KHZ,
    STR_UNIT_MHZ,
    STR_UNIT_BITSEC,
    STR_UNIT_KBITSEC,
    STR_UNIT_MBITSEC,
    STR_UNIT_MBAUD,
    STR_UNIT_KBAUD,
    STR_UNIT_NORTH,
    STR_UNIT_SOUTH,
    STR_UNIT_WEST,
    STR_UNIT_EAST,
    STR_UNIT_HOR,
    STR_UNIT_VER,

    /* Languages: NOTE: Ascending values expected,
                        starting with STR_LANG_OTHER */

    STR_LANG_OTHER,
    STR_LANG_ALBANIAN,
    STR_LANG_BULGARIAN,
    STR_LANG_CZECH,
    STR_LANG_WELSH,
    STR_LANG_DANISH,
    STR_LANG_DUTCH,
    STR_LANG_ENGLISH,
    STR_LANG_ESTONIAN,
    STR_LANG_FRENCH,
    STR_LANG_GERMAN,
    STR_LANG_ICELANDIC,
    STR_LANG_ITALIAN,
    STR_LANG_SPANISH,
    STR_LANG_GREEK,
    STR_LANG_FINNISH,
    STR_LANG_IRISH,
    STR_LANG_CROATIAN,
    STR_LANG_HUNGARIAN,
    STR_LANG_KOREAN,
    STR_LANG_LETTISH,
    STR_LANG_NORWEGIAN,
    STR_LANG_POLISH,
    STR_LANG_PORTUGUESE,
    STR_LANG_ROMANIAN,
    STR_LANG_RUSSIAN,
    STR_LANG_SLOVAK,
    STR_LANG_SERBIAN,
    STR_LANG_SWEDISH,
    STR_LANG_TURKISH,
    STR_LANG_CHINESE,
    STR_LANG_ARABIC,
    STR_LANG_BRETON,
    STR_LANG_CATALAN,
    STR_LANG_CORSICAN,
    STR_LANG_BASQUE,
    STR_LANG_PERSIAN,
    STR_LANG_FAROESE,
    STR_LANG_SCOTS_GAELIC,
    STR_LANG_GALICIAN,
    STR_LANG_HEBREW,
    STR_LANG_JAPANESE,
    STR_LANG_YIDDISH,
    STR_LANG_GREENLANDIC,
    STR_LANG_KANNADA,
    STR_LANG_LITHUANIAN,
    STR_LANG_MACEDONIAN,
    STR_LANG_MOLDAVIAN,
    STR_LANG_MALTESE,
    STR_LANG_SLOVENIAN,
    STR_LANG_UKRAINIAN,
    STR_LANG_AFAR,
    STR_LANG_ABKHAZIAN,
    STR_LANG_AFRIKAANS,
    STR_LANG_AMHARIC,
    STR_LANG_ASSAMESE,
    STR_LANG_AYMARA,
    STR_LANG_AZERBAIJANI,
    STR_LANG_BASHKIR,
    STR_LANG_BYELORUSSIAN,
    STR_LANG_BIHARI,
    STR_LANG_BISLAMA,
    STR_LANG_BENGALI,
    STR_LANG_TIBETAN,
    STR_LANG_ESPERANTO,
    STR_LANG_FIJI,
    STR_LANG_FRISIAN,
    STR_LANG_GUARANI,
    STR_LANG_GUJARATI,
    STR_LANG_HAUSA,
    STR_LANG_HINDI,
    STR_LANG_ARMENIAN,
    STR_LANG_INTERLINGUA,
    STR_LANG_INTERLINGUE,
    STR_LANG_INUPIAK,
    STR_LANG_INDONESIAN,
    STR_LANG_JAVANESE,
    STR_LANG_GEORGIAN,
    STR_LANG_KAZAKH,
    STR_LANG_CAMBODIAN,
    STR_LANG_KASHMIRI,
    STR_LANG_KURDISH,
    STR_LANG_KIRGHIZ,
    STR_LANG_LATIN,
    STR_LANG_LINGALA,
    STR_LANG_LAOTHIAN,
    STR_LANG_MALAGASY,
    STR_LANG_MAORI,
    STR_LANG_MALAYALAM,
    STR_LANG_MONGOLIAN,
    STR_LANG_MARATHI,
    STR_LANG_MALAY,
    STR_LANG_BURMESE,
    STR_LANG_NAURU,
    STR_LANG_NEPALI,
    STR_LANG_OCCITAN,
    STR_LANG_AFAN_OROMO,
    STR_LANG_ORIYA,
    STR_LANG_PUNJABI,
    STR_LANG_PASHTO,
    STR_LANG_QUECHUA,
    STR_LANG_RHAETO_ROMANCE,
    STR_LANG_KURUNDI,
    STR_LANG_KINYARWANDA,
    STR_LANG_SANSKRIT,
    STR_LANG_SINDHI,
    STR_LANG_SANGHO,
    STR_LANG_SINGHALESE,
    STR_LANG_SAMOAN,
    STR_LANG_SHONA,
    STR_LANG_SOMALI,
    STR_LANG_SISWATI,
    STR_LANG_SESOTHO,
    STR_LANG_SUNDANESE,
    STR_LANG_SWAHILI,
    STR_LANG_TAMIL,
    STR_LANG_TELUGU,
    STR_LANG_TAJIK,
    STR_LANG_THAI,
    STR_LANG_TIGRINYA,
    STR_LANG_TURKMEN,
    STR_LANG_TAGALOG,
    STR_LANG_SETSWANA,
    STR_LANG_TONGA,
    STR_LANG_TSONGA,
    STR_LANG_TATAR,
    STR_LANG_TWI,
    STR_LANG_URDU,
    STR_LANG_UZBEK,
    STR_LANG_VIETNAMESE,
    STR_LANG_VOLAPUK,
    STR_LANG_WOLOF,
    STR_LANG_XHOSA,
    STR_LANG_YORUBA,
    STR_LANG_ZULU,
    STR_LANG_NONE,   //for subtitle preferred language setting
    
    STR_LANG_LAST, /* used internally only */
    
    STR_TIMEZONE_BAKER_ISLAND,
    STR_TIMEZONE_SAMOA,
    STR_TIMEZONE_HAWAIIAN,
    STR_TIMEZONE_ANCHORAGE,
    STR_TIMEZONE_LOSANGLES,
    STR_TIMEZONE_DENVER,
    STR_TIMEZONE_MEXICO,
    STR_TIMEZONE_NEWYORK,
    STR_TIMEZONE_MANAUS,
    STR_TIMEZONE_BRASILIA,
    STR_TIMEZONE_SOUTHGEORGIA,
    STR_TIMEZONE_AZORES,
    STR_TIMEZONE_LONDON,
    STR_TIMEZONE_BERLIN,
    STR_TIMEZONE_CAIRO,
    STR_TIMEZONE_MOSCOW,
    STR_TIMEZONE_DUBAI,
    STR_TIMEZONE_PERM,
    STR_TIMEZONE_OMSK,
    STR_TIMEZONE_LAOS,
    STR_TIMEZONE_SHANGHAI,
    STR_TIMEZONE_TOKYO,
    STR_TIMEZONE_SYDNEY,
    STR_TIMEZONE_MAGADAN,
    STR_TIMEZONE_FIJI,
    STR_TIMEZONE_TONGA,
    STR_TIMEZONE_KIRIBATI,
    /* half hour timezones */
    STR_TIMEZONE_ILES_MARQUISES,
    STR_TIMEZONE_NEWFOUNDLAND,
    STR_TIMEZONE_TEHRAN,
    STR_TIMEZONE_KABUL,
    STR_TIMEZONE_BOMBAY,
    STR_TIMEZONE_BURMA,
    STR_TIMEZONE_ALICE_SPRINGS,
    STR_TIMEZONE_LORD_HOWE,
    STR_TIMEZONE_NORFOLK,
    STR_TIMEZONE_VENEZUELA,

    STR_TIMEZONE_NEPAL,
    STR_TIMEZONE_CAIGUNA_EUCLA,
    STR_TIMEZOME_CHATHAM_ISLAND,

    /* IDs 0x760 to 0x7CF for embedded Conax, order must not be changed */
    STR_CONAX_NAME,
    STR_CONAX_MONTH1,
    STR_CONAX_MONTH2,
    STR_CONAX_MONTH3,
    STR_CONAX_MONTH4,
    STR_CONAX_MONTH5,
    STR_CONAX_MONTH6,
    STR_CONAX_MONTH7,
    STR_CONAX_MONTH8,
    STR_CONAX_MONTH9,
    STR_CONAX_MONTH10,
    STR_CONAX_MONTH11,
    STR_CONAX_MONTH12,
    STR_CONAX_WAIT,
    STR_CONAX_MAIN_TITLE,
    STR_CONAX_MAIN_PRODORDER,
    STR_CONAX_MAIN_STATUS_SUB,
    STR_CONAX_MAIN_STATUS_EVT,
    STR_CONAX_MAIN_STATUS_TOK,
    STR_CONAX_MAIN_CHANGE_CA,
    STR_CONAX_MAIN_CHANGE_SIGN,
    STR_CONAX_MAIN_CHANGE_MAT,
    STR_CONAX_MAIN_SET_MODEM,
    STR_CONAX_MAIN_ABOUT,
    STR_CONAX_ENT_STATUS_SUB,
    STR_CONAX_ENT_STATUS_EVT,
    STR_CONAX_ENT_STATUS_TOK,
    STR_CONAX_ENT_NAME,
    STR_CONAX_ENT_START,
    STR_CONAX_ENT_END,
    STR_CONAX_ENT_DATE,
    STR_CONAX_ENT_TIME,
    STR_CONAX_ENT_LEFT,
    STR_CONAX_ENT_PURSE,
    STR_CONAX_ENT_BALANCE,
    STR_CONAX_ENT_STATUS_DEB,
    STR_CONAX_ENT_TOKENS,
    STR_CONAX_ENT_STATUS_CRED,
    STR_CONAX_ENT_LABEL,
    STR_CONAX_CHANGE_CA,
    STR_CONAX_PRESENT_CA,
    STR_CONAX_NEW_CA,
    STR_CONAX_CONFIRM_CA,
    STR_CONAX_CHANGE_SIGN,
    STR_CONAX_PRESENT_SIGN,
    STR_CONAX_NEW_SIGN,
    STR_CONAX_CONFIRM_SIGN,
    STR_CONAX_WRONG_CA,
    STR_CONAX_WRONG_OLD_CA,
    STR_CONAX_WRONG_SIGN,
    STR_CONAX_NO_CONFIRM,
    STR_CONAX_SIGN_PRE,
    STR_CONAX_SIGN_SUF,
    STR_CONAX_SIGN_LOCK,
    STR_CONAX_SIGN_SERVICE,
    STR_CONAX_CHANGED_CA,
    STR_CONAX_FAILED_CA,
    STR_CONAX_CHANGED_SIGN,
    STR_CONAX_CHANGE_MAT,
    STR_CONAX_MAT_CURR,
    STR_CONAX_MAT_AVL,
    STR_CONAX_MAT_G,
    STR_CONAX_MAT_PG,
    STR_CONAX_MAT_A,
    STR_CONAX_MAT_X,
    STR_CONAX_MAT_CA,
    STR_CONAX_INFO_SW,
    STR_CONAX_INFO_IF,
    STR_CONAX_INFO_SERIAL,
    STR_CONAX_INFO_SESS,
    STR_CONAX_INFO_LANG,
    STR_CONAX_INFO_CAID,
    STR_CONAX_NOCARD,
    STR_CONAX_WRONG_CONAX_CARD,
    STR_CONAX_WRONG_CARD,
    STR_CONAX_CARDERR,
    STR_CONAX_BAD_COMM,
    STR_CONAX_NOACCESS,
    STR_CONAX_ORDERING,
    STR_CONAX_ORDERPIN,
    STR_CONAX_START,
    STR_CONAX_MINLEFT,
    STR_CONAX_STARTPIN,
    STR_CONAX_BLACKOUT,
    STR_CONAX_MAT_LOCK,
    STR_CONAX_MAT_CARDLEVEL,
    STR_CONAX_MAT_PROGLEVEL,
    STR_CONAX_MATPIN,
    STR_CONAX_NETW_BLOCK,
    STR_CONAX_TOK_ACCESS,
    STR_CONAX_TOK_PER_MIN,
    STR_CONAX_TOKPIN,
    STR_CONAX_ENT_PURSE_BAL,
    STR_CONAX_TOK_PRICE,
    STR_CONAX_TOK_INSUF,
    STR_CONAX_NODECODE,
    STR_CONAX_EMPTY,
    STR_CONAX_NO_ENTRY,
    STR_CONAX_UNAVAILABLE,
    
    STR_NUMS
} STR_IDX_S;

/*! 
 * \brief Number of "known" languages, subset of all languages
 */
#define KNOWN_LANG_NUMS           (((uint32_t)STR_LANG_LAST) - ((uint32_t)STR_LANG_OTHER))

#ifdef LANG_LOADFROMFLASH
typedef struct
{
    uint32_t      id;                          // file identification
    uint32_t      revision;                    // file structure revision
    SYS_LANGCODE  langId;                      // language identifier
    char_t        langName[OSDLANG_NAME_LEN];  // language name string

} OSDLANG_FILEHDR;
#endif

typedef struct
{
    int32_t   idx;
    char_t*   str;

} OSDLANG_ENTRY_S;

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

int32_t         OSD_LANG_Init                    (void);
uint32_t        OSD_LANG_GetNumberOfLangs        (void);
int32_t         OSD_LANG_GetLangInfo             (uint32_t langIdx, SYS_LANGCODE* id);
uint32_t        OSD_LANG_GetLangCurr             (void);
int32_t         OSD_LANG_Select                  (uint32_t langIdx);
SYS_STRING_S    OSD_LANG_GetLanguageString       (SYS_LANGCODE langCode);
SYS_LANGCODE    OSD_LANG_GetKnownLangCodeByIndex (uint16_t index);
uint16_t        OSD_LANG_GetKnownLangIndex       (SYS_LANGCODE langCode);
SYS_STRING_S    OSD_GetStr                       (uint32_t strIdx);
SYS_STRING_S    OSD_GetWeekDayStr                (SYS_TIME_WEEKDAY_E dayOfWeek);

#ifdef __cplusplus
}
#endif

#endif /* _APP_LANG_H_ */
/******************************************************************************/
/* END OF FILE */
