/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/inc/app_main.h
**
**              $Id: app_main.h 5866 2010-03-02 12:09:31Z jmorha $ 
**
** \brief       Main header file of reference application for SmartGo-HD.
**
** Detail description. [\ref app_smartgo_app_ref]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _APP_MAIN_H_
#define _APP_MAIN_H_

#include "fgs_box.h"
#include "fgs_list.h"
#include "fgs_navilist.h"

#include "app_osd.h"
#include "app_menu_items.h"
#include "menustack.h"

/*!
********************************************************************************
** Application specific error codes
**
** Each error code shall be decreased in steps of 1.
********************************************************************************
*/

#define SMARTGO_APPL_ERR_PROHIBITED         (APPL_SMARTGO_APPL_ERR_BASE - 1)
//!< Requested operation currently not possible.

#define SMARTGO_APPL_ERR_INTERNAL           (APPL_SMARTGO_APPL_ERR_BASE - 2)
//!< Internal error, should not occur!

#define SMARTGO_APPL_ERR_CAPACITY           (APPL_SMARTGO_APPL_ERR_BASE - 3)
//!< Requested operation requires space that is currently not available.

#define SMARTGO_APPL_ERR_ALREADY_OPEN       (APPL_SMARTGO_APPL_ERR_BASE - 4)
//!< Device/handle is already open.

#define SMARTGO_APPL_ERR_DATA_NOT_AVL       (APPL_SMARTGO_APPL_ERR_BASE - 5)
//!< Requested data is not available

/*----------------------------------------------------------------------------*/
/* Application-specific configuration                                         */
/*----------------------------------------------------------------------------*/

#define MAIN_LINEBUFCHARS      256
#define MAIN_LINEBUFSIZE       (MAIN_LINEBUFCHARS * 2)  //!< Unicode buffer!
#define MAIN_LINEBUFCHARSET     SYS_CHARSET_ISO_10646   //!< Character set used
                                                        //!< for common line 
                                                        //!< buffer.

#define RECORD_PRE_SECONDS     180 // record timer pre-interval
#define RECORD_POST_SECONDS    180 // record timer post-interval

#define SERVICESELECT_TIMEOUT  10  // timeout in deci-seconds

/* -------------------------------------------------------------------------- */

/* ----- default settings --------------------------------------------------- */

#define SETTINGS_DEFAULT_DISPTIME           5
#define SETTINGS_DEFAULT_VIDEOOUT           VIDEOOUT_RGB
#define SETTINGS_DEFAULT_HDTVSTANDARD       HDSYS_1080I
#define SETTINGS_DEFAULT_SDTVSTANDARD       SDSYS_PAL
#define SETTINGS_DEFAULT_STARTMODE          VIDEOSTART_FAST
#define SETTINGS_DEFAULT_UTCOFFSET          60                  //!< in minutes.
#define SETTINGS_DEFAULT_DSTACTIVE          FFALSE
#define SETTINGS_DEFAULT_POWERUPMODE        SYS_PWRMODE_UP
#define SETTINGS_DEFAULT_SWITCHMODE         FFALSE
#define SETTINGS_DEFAULT_DISPFORMAT         PICFORMAT_16_9
#define SETTINGS_DEFAULT_SCALEMETHOD        SCALEMETHOD_PILLAR_LETTERBOX
#define SETTINGS_DEFAULT_SPDIFMODE          SPDIFMODE_PCMONLY
#define SETTINGS_DEFAULT_TRANSPARENCY       OSD_TRANSPARENCY_30
#define SETTINGS_DEFAULT_COLSCHEME          OSD_COLSCHEME_BLUE

#define SETTINGS_DEFAULT_HOR_ALIGN          FGS_ALIGN_HOR_CENTER
#define SETTINGS_DEFAULT_VER_ALIGN          FGS_ALIGN_VER_CENTER
#define DEFAULT_RESET_FACTORY_FLAG          0x1
#define SETTINGS_DEFAULT_PVRJUMPTIME        2    // 2 Min

/* Language Defaults this parameter maight by change by appProfile selection */
#define SETTINGS_DEFAULT_AUD_LANG               LANG_ENGLISH
#define SETTINGS_DEFAULT_UI_LANG                LANG_ENGLISH
#define SETTINGS_DEFAULT_SUB_LANG               LANG_ENGLISH
#define SETTINGS_DEFAULT_AUTOTIME_OFFSET        FTRUE
#define SETTINGS_DEFAULT_AUTOTIME_OFFSET_CTRY   CTRY_GERMANY

#define PVR_DEFAULT_SPINDOWN_TIME           300  // 5 min

/* LCN Defaults this parameter maight by change by appProfile selection */
#define SLIST_DEFAULT_LCN_STATUS            FTRUE
#define SLIST_DEFAULT_LCN_VARIANT_START     800
#define SLIST_DEFAULT_LCN_VARIANT_LENGTH    100
#define SLIST_DEFAULT_LCN_USEVISIBILITY     FFALSE

/* Folder definitions for internal HDD */
#define MAIN_DIR_HDD_CAPTURE                "snapshot"  //!< HDD: Capture dir.
#define MAIN_DIR_HDD_PVR                    "pvr"       //!< HDD: PVR dir.
#define MAIN_DIR_OTH_PVR                    ""          //!< Others: PVR dir.

/* -------------------------------------------------------------------------- */

/* bitfield mapping for PCTRL parameter 'protectionArea' */
#define APP_PROTECTION_SETUP    0x00000001

/* -------------------------------------------------------------------------- */

typedef void  (*OSD_UPDATE_CB)       (void);

enum /* application-specific events */
{
    APP_EVT_MENU_REFRESH = EVT_NUMS,
    
    #ifdef APPL_MHEG_ENABLED
    
    APP_EVT_MHEG_START
    
    #endif /* APPL_MHEG_ENABLED */
};

enum
{
    OSD_TIMER_MENU = 0,
    OSD_TIMER_ICON,
    OSD_TIMER_SERVSELECT,
    OSD_TIMER_MOTOR_MSG
};

enum
{
    OSD_PERIODTIMER_SIGNAL = 0,
    OSD_PERIODTIMER_GENERAL,
    
};

enum /* unique FGS member item IDs */
{
    ITEM_SIGBOX_ID_LVLSTR = 64,
    ITEM_SIGBOX_ID_QUALSTR,
    ITEM_SIGBOX_ID_BARS,
    
    ITEM_HELP_KEY_MENU,
    ITEM_HELP_KEY_EXIT,
    ITEM_HELP_TEXT_BACK,
    ITEM_HELP_TEXT_CLOSEMENU,
};

typedef enum
{
    CHANREGION_WESTEUROPE = 0,
    CHANREGION_EASTEUROPE,
    CHANREGION_UK,
    CHANREGION_FRANCE,
    CHANREGION_ITALY,
    CHANREGION_IRELAND,
    CHANREGION_CHINA,
    CHANREGION_USA,
    CHANREGION_CANADA,
    CHANREGION_JAPAN,
    CHANREGION_AUSTRALIA,
    CHANREGION_NUMS
    
} CHANREGION_E;

typedef enum
{
    CABLEPRESET_RASTER,
    CABLEPRESET_CUSTOM,
    CABLEPRESET_NUMS
    
} CABLEPRESET_E;

typedef enum
{
    USERDAT_TRANSPARENCY = 0,
    USERDAT_COLORSCHEME,
    USERDAT_FACTORYFLAG,
    USERDAT_MENUALIGNMENT,
    USERDAT_PVRJUMPTIME,    
    USERDAT_NUMS
    
} USERDAT_ID_E;

typedef struct
{
//    uint8_t                 font;          //!< font to be used. JM replace by SCALER_GetFont(MAIN_FONT);
    uint32_t                colFocus;      //!< non-focus text color.
    uint32_t                colNonFocus;   //!< text color when item is focussed.
    uint32_t                colDisabled;   //!< text color when item is disabled.
    
} APP_TEXT_ATTR_S;

#ifdef APPL_EDITMARKER_ENABLED

typedef enum
{
    JUMP_MARKER_NEXT = 0,
    JUMP_MARKER_PREV

} APP_MARKER_JUMP_MODE_E;

typedef enum
{
    AB_REPEAT_MODE_NONE = 0,
    AB_REPEAT_MODE_A_POS,
    AB_REPEAT_MODE_B_POS,
    AB_REPEAT_MODE_NUM,

} APP_PLAYBACK_AB_REPEAT_MODE_E;

typedef struct
{
    fbool_t                         cutModeActive;
    APP_PLAYBACK_AB_REPEAT_MODE_E   abRepeatMode;
    
} APP_PLAYBACK_STATUS_S;

#endif /* APPL_EDITMARKER_ENABLED */

typedef struct
{
    /* FGS resources */
    FGS_PANEL_INST_S        pnlInst;      //!< Panel memory shared by all major menus
    FGS_LIST_INST_S         listInst;     //!< List memory
    FGS_LIST_INST_S         helpListInst; //!< Helplist memory
    FGS_LNAVI_INST_S        helpNaviInst; //!< Alternative helplist memory
    FGS_LNAVI_INST_S        lnaviInst;
    FGS_POS_S               rootPos;
    FGS_MEMBER_S*           pStartupMem;
    FGS_MEMBER_S*           pRootMem;
    
    /* OSD item instances */
    MENU_SELECTLIST_INST_S  selectLstInst;
    MENU_SERVBOX_INST_S     servBox;
    MAIN_SIGBOX_INST_S      sigBox;
    MENU_CONFDIALOG_INST_S  confDialog;
    VOLUMEBAR_INST_S        volBar;
    PROHIBITED_INST_S       prohib;
    MAIN_MSGBOX_INST_S      msgBox;
    MENU_PINDIALOG_INST_S   pinDialog;
    MENU_EDITSTRING_INST_S  editStringBox;
    
    const SYS_CONFIGDAT_S*  sysCfg;
    const SYS_HWCONFIG_S*   hwCfg;

    SYS_OSDLAYER_S          osdLayerParams;
    uint8_t*                osdBasePtr;
    uint32_t                osdBufSize;
    fbool_t                 menuHasLayer;
    FAPEX_SYS_SemaphoreT         displayMutex;
    
    /* OSD-related data */
    uint8_t                 schemeData[OSD_COLSCHEME_NUMS][OSD_SCHEMECOL_NUMS];
    
    /* menu stack data */
    MENUSTACK_INST_S        menuStack;
    
    SYS_HDTV_STANDARD       currentHdOutSys;
    SYS_SDTV_STANDARD       currentSdOutSys;
    OSD_UPDATE_CB           osdUpdateCb;
    void                    (*menuTimerHandler)(void* arg);
    void*                   menuTimerArg;
    
    fbool_t                 startedByTimer;

    /* display time parameters */
    SYS_TIME_S              timeCurr;
    FAPI_SYS_HandleT        timeBoxPnlHdl;
    int32_t                 timeBoxId;
    
    FGS_FORMAT_TIME_E       timeFormat;
    FGS_FORMAT_DATE_E       dataFormatNorm;
    FGS_FORMAT_DATE_E       dataFormatShort;
    
    FAPI_SYS_HandleT        numBoxPnlHdl;   //!< textbox containing service number
    int32_t                 numBoxId;
    
    FAPI_SYS_HandleT        nameBoxPnlHdl;  //!< textbox containing service name
    int32_t                 nameBoxId;
    int32_t                 nameBoxParent;
    
    FAPI_SYS_HandleT        symbBoxPnlHdl;
    int32_t                 symbBoxId;
    int32_t                 symbBoxParent;
    
    APP_TEXT_ATTR_S         txtAttr;    //!< menu text attribute constants
    
    fbool_t                 fcDataDirty;
    uint16_t                fcDataIdleCnt;
    
    #ifdef APPL_PVR_ENABLED
    
    PVRSYS_STATUS_S         pvrStatus;
    SYS_STREAM_INFO_S       playStreamInf;
    PVR_PlayInfoT           playInf;
    SLIST_ITEM_S            playTimeShiftService;

    #ifdef APPL_EDITMARKER_ENABLED
    
    /*
        the information in the structure below is only valid if
        pvrStatus shows an active playback.
    */
    APP_PLAYBACK_STATUS_S   playStatus;
    
    PVR_BLOCK_MARKER_ITEM_S blockMarkerData;  
    
    #endif /* APPL_EDITMARKER_ENABLED */
    
    #endif /* APPL_PVR_ENABLED */
    
    #ifdef APPL_PIP_ENABLED
    
    uint8_t                 pipPos;
    
    #endif /* APPL_PIP_ENABLED */
    
    /* frontend-related parameters */
    // TODO: to be improved!
    int8_t                  avaiTuner;
    uint8_t                 numSatFe;    // number of SAT FEs available
    uint8_t                 numTerrFe;   // number of Terrestrial FEs available
    uint8_t                 numCableFe;  // number of Cable FEs available
    uint8_t                 numDirectTs; // number of Direct TS available
    
    #ifdef APPL_RECTIMER_ENABLED
    
    RECTIMER_IDX            currActTimer;
    
    #endif /* APPL_RECTIMER_ENABLED */
    
    #ifdef APPL_PARENTAL_ENABLED
    
    /* Parental control related data */
    SYS_SERVLOC_S           currAuthServLoc;
    
    #endif /* APPL_PARENTAL_ENABLED */
    
    SYS_STRBUF_S            osdStringBuf;
    SYS_STRBUF_S            osdCutStrBuf;
    
    char_t*                 asciiBuf;
    
    /* font-related parameters */
    FAPI_SYS_HandleT        fontHdl;
    SCALER_FONT_E           fontIdCurr;
    int32_t                 fontCurr;
    int32_t                 fontSizeCurr;
    FONTSERVER_InformationT fontInfo[SCALER_FONT_NUMS];
    SYS_CHARSET             fontCs  [SCALER_FONT_NUMS];
    
    #ifdef APPL_DISEQC12_ENABLED
    
    fbool_t                 motorMsgShown;
    
    #endif /* APPL_DISEQC12_ENABLED */
    
    #ifdef APPL_FESAT_ENABLED
    
    /* satellite/carrier selection shared by different menu panels*/
    CFGSAT_IDX              currSatIdx;
    // TODO: add carrier index
    
    #endif /* APPL_FESAT_ENABLED */
    
    uint32_t                dynamicOnOffHDCP;
    fbool_t                 hdmiConnected;
    uint8_t                 hdcpState;
    
} APP_DATA_S;

typedef enum
{
    MENU_TIME_NEXT_EPG = 0,
    MENU_TIME_NEXT_START_TIMER_REC,
    MENU_TIME_NEXT_START_INSTANT_REC,
    MENU_TIME_NEXT_TIMERPROG_MENU,
    
} MENU_TIME_NEXT_E;

typedef struct
{
    uint32_t                strIdx; 
    fbool_t                 done;
    MENU_TIME_NEXT_E        next;
    
    void*                   privData;
    
} MENU_TIME_OPEN_S;


extern APP_DATA_S* appDat;

#ifdef __cplusplus
extern "C" {
#endif

#ifdef APPL_PVR_ENABLED

fbool_t        MAIN_PvrStartInstantRecording (void);
int32_t        MAIN_PvrStartTimerRecording   (RECTIMER_IDX idx);
int32_t        MAIN_PvrStopRecording         (int8_t recIdx);
fbool_t        MAIN_PvrKeyStop               (void);
fbool_t        MAIN_PvrKeyPause              (void);
fbool_t        MAIN_PvrKeyPlay               (void);
fbool_t        MAIN_PvrKeyForward            (void);
fbool_t        MAIN_PvrKeyRewind             (void);
fbool_t        MAIN_PvrKeyForwSlow           (void);
fbool_t        MAIN_PvrKeyRewSlow            (void);
fbool_t        MAIN_PvrKeyJump               (fbool_t jumpForward);
void           MAIN_PvrHandlePlayEvent       (PVR_PlayNotifyT event);
void           MAIN_PvrHandleRecEvent        (PVR_RecordNotifyT event,
                                              uint8_t recIdx);
int32_t        MAIN_PvrResumeLiveTv          (const PVRSYS_STATUS_S* pvrStatus);
SYS_DEVICE_E   MAIN_PvrGetDefaultRecDevice   (void);
int32_t        MAIN_PvrStartTimeShiftPlayback(int8_t recIdx,
                                              PVRSYS_TIMESHIFTPOS_E pos,
                                              PVRSYS_PLAY_STATUS_E mode,
                                              int16_t speedIndex);

#ifdef APPL_EDITMARKER_ENABLED

fbool_t        MAIN_PvrKeyABRepeat           (void);

fbool_t        MAIN_PvrCheckMarker           (FDVR_PLAY_MARKER_TYPE_E* pType);
int32_t        MAIN_PvrJumpMarker            (PVR_BLOCK_MARKER_ITEM_S* pMarker,
                                              uint8_t                       direction);
void           MAIN_PvrInitMarker            (void);
void           MAIN_PvrGetABRepeatStatus     (void);

#endif /* APPL_EDITMARKER_ENABLED */

#endif /* APPL_PVR_ENABLED */

void    MAIN_HandleZappingKeys     (fbool_t hasFocus);
void    MAIN_HandlePvrKeys         (fbool_t hasFocus);
void    MAIN_HandleExitKeys        (fbool_t hasFocus);
fbool_t MAIN_DirectMenuTrans       (FAPI_SYS_HandleT h);

fbool_t MAIN_NumberKey             (FAPI_SYS_HandleT h, uint8_t value);
fbool_t MAIN_VolumeUp              (FAPI_SYS_HandleT h);
fbool_t MAIN_VolumeDown            (FAPI_SYS_HandleT h);
fbool_t MAIN_VolumeMute            (FAPI_SYS_HandleT h);
fbool_t MAIN_ChannelUp             (FAPI_SYS_HandleT h);
fbool_t MAIN_ChannelDown           (FAPI_SYS_HandleT h);
int32_t MAIN_SelectService         (const SYS_SERVLOC_S* pServLoc,
                                    fbool_t quietMode);
int32_t MAIN_SelectNextService     (void);
int32_t MAIN_SelectPreviousService (void);
int32_t MAIN_StartInitialService   (void);
int32_t MAIN_RestartLastService    (void);
fbool_t MAIN_SetNextAudioStream    (FAPI_SYS_HandleT h);
fbool_t MAIN_SwapTvAndRadio        (FAPI_SYS_HandleT h);
fbool_t MAIN_SwapService           (FAPI_SYS_HandleT h);

fbool_t MAIN_DynamicOnOffHDCP      (FAPI_SYS_HandleT h);

fbool_t     MAIN_KeyPIP                (FAPI_SYS_HandleT h);
int32_t     MAIN_ServiceSwitchGetAvailability (const SYS_SERVLOC_S* pServLoc);
int32_t     MAIN_StopPresentation       (void);
int32_t     MAIN_RestartPresentation    (void);
void        MAIN_SaveSettingsIfChanged  (void);
void        MAIN_MsgFunctionReturnedFailure(int32_t ferr);

int32_t     MENU_InfoServiceNumBoxCreate    (FAPI_SYS_HandleT pnlHdl, int32_t id, int32_t parent);
void        MENU_InfoServiceNumBoxRemove    (void);
fbool_t     MENU_InfoServiceNumBoxInput     (FAPI_SYS_HandleT h, uint8_t number);
void        MENU_InfoServiceNumBoxInputAbort(void);
void        MENU_InfoServiceNumBoxUpdate    (void);

int32_t     MENU_InfoTimeBoxCreate          (FAPI_SYS_HandleT pnlHdl, int32_t id);
void        MENU_InfoTimeBoxRemove          (void);
void        MENU_InfoTimeBoxUpdate          (void);

int32_t     MENU_InfoListSymbolBoxCreate    (FAPI_SYS_HandleT pnlHdl, int32_t id, int32_t parent);
void        MENU_InfoListSymbolBoxRemove    (void);
void        MENU_InfoListSymbolBoxUpdate    (void);

int32_t     MENU_InfoServiceNameBoxCreate   (FAPI_SYS_HandleT pnlHdl, uint16_t width, int32_t id,
                                             int32_t parent);
void        MENU_InfoServiceNameBoxRemove   (void);
void        MENU_InfoServiceNameBoxUpdate   (void);

void        MAIN_FreqDataHasChanged     (void);

fbool_t     MAIN_PowerDown              (FAPI_SYS_HandleT h);
fbool_t     MAIN_PowerUp                (FAPI_SYS_HandleT h);

int32_t     MAIN_MenuTimerLaunch        (uint16_t timeInSeconds,
                                         void (*handler)(void* arg),
                                         void* arg);
void        MAIN_MenuTimerAbort         (void);

#ifdef APPL_FETERR_ENABLED
void        MAIN_GenerateTerrChannelList(CHANREGION_E region);
#endif /* #ifdef APPL_FETERR_ENABLED */

#ifdef APPL_FECABLE_ENABLED
void        MAIN_GenerateCableChannelList (CHANREGION_E region);
#endif /* #ifdef APPL_FECABLE_ENABLED */

#ifdef APPL_PIP_ENABLED

int32_t APP_PIP_Init                (void);
int32_t APP_PIP_AdjustSize          (fbool_t increase,
                                         const PIP_STATUS_S* pStatus);
int32_t APP_PIP_AdjustPosition      (fbool_t moveRight,
                                         const PIP_STATUS_S* pStatus);

#endif /* APPL_PIP_ENABLED */

int32_t MAIN_UserDataGet            (USERDAT_ID_E id, uint32_t* value);
int32_t MAIN_UserDataSet            (USERDAT_ID_E id, uint32_t  value);

#ifdef __cplusplus
}
#endif

#endif /* _APP_MAIN_H_ */
/******************************************************************************/
/* END OF FILE */
