/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/inc/app_menu_viewer.h
**
**              $Id: app_menu_viewer.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       File Viewer
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _APP_MENU_VIEWER_H_
#define _APP_MENU_VIEWER_H_

/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/

#define MAIN_VIEWER_PATHLEN         256 //!< Maximum path length in bytes.
#define MAIN_VIEWER_NAMELEN         30  //!< Maximum filename length in bytes.

#define MAIN_VIEWER_NUM_FILTER_TYPES       4

#define MAIN_VIEWER_FILTYPE_ALL     0x002A0000    //!< '*  '

#define MAIN_VIEWER_FILTYPE_JPG_LC  0x006A7067   //!< 'jpg' lower case
#define MAIN_VIEWER_FILTYPE_JPG_UC  0x004A5047   //!< 'JPG' upper case
#define MAIN_VIEWER_FILTYPE_BMP_LC  0x00626D70   //!< 'bmp' lower case
#define MAIN_VIEWER_FILTYPE_BMP_UC  0x00424D50   //!< 'BMP' upper case
#define MAIN_VIEWER_FILTYPE_PNG_LC  0x00706E67   //!< 'png' lower case
#define MAIN_VIEWER_FILTYPE_PNG_UC  0x00504E47   //!< 'PNG' upper case
#define MAIN_VIEWER_FILTYPE_MP3_LC  0x006D7033   //!< 'mp3' lower case
#define MAIN_VIEWER_FILTYPE_MP3_UC  0x004D5033   //!< 'MP3' upper case
#define MAIN_VIEWER_FILTYPE_SSU_LC  0x00737375   //!< 'ssu' lower case
#define MAIN_VIEWER_FILTYPE_SSU_UC  0x00535355   //!< 'SSU' upper case

#define MAIN_VIEWER_FILEOP_RENAME   0x01
#define MAIN_VIEWER_FILEOP_COPY     0x02
#define MAIN_VIEWER_FILEOP_MOVE     0x04
#define MAIN_VIEWER_FILEOP_DELETE   0x08
#define MAIN_VIEWER_FILEOP_ALL      0xFF

#define MAIN_VIEWER_OP_SHOWINFO     0x01
#define MAIN_VIEWER_OP_PREVIEW      0x02
#define MAIN_VIEWER_OP_ALL          0xFF

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

typedef enum
{
    MAIN_VIEWER_FILETYPE_FILTERED = 0, //!< File type filtered by 'filterType'
    MAIN_VIEWER_FILETYPE_PICTURE,
    MAIN_VIEWER_FILETYPE_MUSIC,
    MAIN_VIEWER_FILETYPE_MOVIE

} MAIN_VIEWER_FILETYPE;

typedef enum
{
    MAIN_VIEWER_GETFILE_FIRST = 0,  //!< Get first matching file.
    MAIN_VIEWER_GETFILE_LAST,       //!< Get last matching file.
    MAIN_VIEWER_GETFILE_NEXT,       //!< Get next matching file from ref. index.
    MAIN_VIEWER_GETFILE_PREVIOUS,   //!< Get previous matching file from ref.
                                    //!< index.
    MAIN_VIEWER_GETFILE_THIS        //!< Get file referenced by index if
                                    //!< matching.
} MAIN_VIEWER_GETFILE_E;

typedef struct
{
    SYS_STRING_S        titleString;
    fbool_t             allowStackedView;
    uint8_t             allowedFileOps; //!< Bitfield of MAIN_VIEWER_FILEOP_XXX.
    uint8_t             allowedPicOps;  //!< Bitfield of MAIN_VIEWER_OP_XXX.
    uint8_t             allowedMusicOps;//!< Bitfield of MAIN_VIEWER_OP_XXX.
    uint8_t             allowedMovieOps;//!< Bitfield of MAIN_VIEWER_OP_XXX.

    char_t              initialPath[MAIN_VIEWER_PATHLEN];

    uint32_t            filterType[MAIN_VIEWER_NUM_FILTER_TYPES];

    fbool_t             (*onBack)  (void* arg);
    fbool_t             (*onExit)  (void* arg);
    fbool_t             (*onSelect)(void* arg,
                                    MAIN_VIEWER_FILETYPE type,
                                    const char_t* path,
                                    uint16_t pathLenMax,
                                    const char_t* fileName,
                                    uint16_t nameLenMax,
                                    uint16_t index,
                                    uint16_t numItems);
    void*               cbArg;  //!< general argument passed with all
                                //!< callback functions in this structure.

} MAIN_VIEWER_CONFIG_S;

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

int32_t  MAIN_ViewerInit             (void);
fbool_t  MAIN_ViewerOpen             (MAIN_VIEWER_CONFIG_S* params);
fbool_t  MAIN_ViewerClose            (void);

void     MAIN_ViewerUpdateProgress   (FILEOP_INFO_S* infoP);
void     MAIN_ViewerDeviceUpdate     (void);
void     MAIN_ViewerHandleEvent      (SYS_EVTTYPE_E type, const void* data);

int32_t  MAIN_ViewerGetFile          (const char_t* path, uint16_t index,
                                      MAIN_VIEWER_GETFILE_E mode,
                                      fbool_t (*fileMatch)(void* arg,
                                                           const char_t* name,
                                                           uint16_t nameLenMax),
                                      void* fileMatchArg,
                                      uint16_t* pNewIndex,
                                      char_t* fileNameBuf,
                                      uint16_t fileNameLength);

#ifdef __cplusplus
}
#endif

#endif /* _APP_MENU_VIEWER_H_ */
/******************************************************************************/
/* END OF FILE */
