/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/inc/app_osd.h
**
**              $Id: app_osd.h 5866 2010-03-02 12:09:31Z jmorha $ 
**
** \brief       Common OSD functions used by all OSD modules.
**
** Detail description. [\ref app_smartgo_app_ref_osd]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _APP_OSD_H_
#define _APP_OSD_H_


#include "app_scaler.h"
#include "app_lang.h"

/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/

/* The diagonal and rectangle corner styles are based on an
   algorithm and are universal for all frame-widths */
#define OSD_CORNER_RECT                  0
#define OSD_CORNER_DIAG                  1
/* The user style is based on a bitmap. The bitmap is stored
   in osd.c and has to be defined for each framewidth individually */
#define OSD_CORNER_USER                  2


#define OSD_FRAMESTYLE                   OSD_CORNER_DIAG

#define OSD_FONT_DEFAULT_SIZE            34

/* ------------------------------------------------------------------------- */ 
/* changed by Jim.kang at 2010-7-29 */
//#define OSD_LAYER_MENU       2  //!< reserved for application OSD
#define OSD_LAYER_MENU       1  //!< reserved for application OSD

//#define OSD_LAYER_USER0      0x00010002  //!< for free usage
#define OSD_LAYER_MHEG       4  //!< used by system for displaying MHEG-5
#define OSD_LAYER_TTX_PIP    3  //!< used by system for displaying Teletext
#define OSD_LAYER_SUBTT      1  //!< used for subtitling if enabled, else free
#define OSD_LAYER_JPEG       3
#define OSD_LAYER_MOSAIC     3

/* NOTE: Layer 0 can not be switched off in the analogue video output! */

/* ------------------------------------------------------------------------- */

/* CLUT map assignment */
#define OSD_CLUT_RESERVED_SIZE          128    
#define OSD_CLUT_STANDARD_SIZE           64
#define OSD_CLUT_APPLSPEC_SIZE           32
#define OSD_CLUT_DYNAMIC_SIZE            32

#define OSD_CLUT_RESERVED_START           0
#define OSD_CLUT_STANDARD_START           (OSD_CLUT_RESERVED_START +  \
                                           OSD_CLUT_RESERVED_SIZE)
#define OSD_CLUT_APPLSPEC_START           (OSD_CLUT_STANDARD_START +  \
                                           OSD_CLUT_STANDARD_SIZE)
#define OSD_CLUT_DYNAMIC_START            (OSD_CLUT_APPLSPEC_START +  \
                                           OSD_CLUT_APPLSPEC_SIZE)
#define OSD_CLUT_END                      (OSD_CLUT_DYNAMIC_START  +  \
                                           OSD_CLUT_DYNAMIC_SIZE)

#if ( OSD_CLUT_END > 256 )
    #error Invalid CLUT map assignment
#endif

/* ------------------------------------------------------------------------- */


#define OSD_COL_BLACK                    0
#define OSD_COL_WHITE                    1
#define OSD_COL_RED                      2
#define OSD_COL_GREEN                    3
#define OSD_COL_BLUE                     4
#define OSD_COL_CYAN                     5
#define OSD_COL_YELLOW                   6
#define OSD_COL_MAGENTA                  7
#define OSD_COL_BLANK                    15

#define OSD_COL_TITLE_TEXT               13
#define OSD_COL_TEXT                     13
#define OSD_COL_TEXT_HIGHLIGHTED         14
#define OSD_COL_TEXT_DISABLED            11
#define OSD_COL_TEXT_TITLE               13
#define OSD_COL_BAR_INACTIVE             11

#define OSD_COL_HELP_TEXT                13
#define OSD_COL_HELP_BUTTXT              13

#define OSD_COL_LIST_TEXT                13
#define OSD_COL_LIST_TEXT_HIGHL          14

#define OSD_COL_NUMS                    216 //!< number of application-specific CLUT entries

#define OSD_COL_BACKGROUND_NON_FOCUS     11 //!< item: focus && list: non-focus
#define OSD_COL_BAR_ACTIVE               13 //!< active part of progress bars, e.g. timebar in info menu
#define OSD_COL_SCROLLB_ACTIVE           13 //!< active part of scrollbars
#define OSD_COL_ICON_FRAME               13 //!< can be used as frame for bmp icons, e.g. red, green, blue, yellow buttons
#define OSD_COL_FRAME                    13


typedef enum
{
    OSD_COLSCHEME_CURRENT = -1,
    OSD_COLSCHEME_BLUE = 0,
    OSD_COLSCHEME_GREEN,
    OSD_COLSCHEME_RED,
    OSD_COLSCHEME_BLACK,
    OSD_COLSCHEME_NUMS

} OSD_COLSCHEME_E;

typedef enum
{
    OSD_SCHEMECOL_MAIN_BKGR = 0,
    OSD_SCHEMECOL_MAIN_BKGR_HIGHL,
    OSD_SCHEMECOL_BLACK_BKGR,
    OSD_SCHEMECOL_HELP_BKGR,
    OSD_SCHEMECOL_TITLE_BKGR,
    OSD_SCHEMECOL_FRAME,
    OSD_SCHEMECOL_FRAME_SHADOW1,
    OSD_SCHEMECOL_FRAME_SHADOW2,
    OSD_SCHEMECOL_SCROLLBAR_INACT,
    OSD_SCHEMECOL_PROGRESSBAR_INACT,
    OSD_SCHEMECOL_NUMS
} OSD_SCHEMECOL_E;

/* ------------------------------------------------------------------------- */

typedef enum
{
    OSD_TRANSPARENCY_CURRENT = -1,
    OSD_TRANSPARENCY_0 = 0,   //!<  0  % (opaque)
    OSD_TRANSPARENCY_10,      //!< 10  %
    OSD_TRANSPARENCY_20,      //!< 20  %
    OSD_TRANSPARENCY_30,      //!< 30  %
    OSD_TRANSPARENCY_40,      //!< 40  %
    OSD_TRANSPARENCY_50,      //!< 50  %
    OSD_TRANSPARENCY_60,      //!< 60  %
    OSD_TRANSPARENCY_70,      //!< 70  %
    OSD_TRANSPARENCY_80,      //!< 80  %
    OSD_TRANSPARENCY_90,      //!< 90  %
    OSD_TRANSPARENCY_100,     //!< 100 %
    OSD_TRANSPARENCY_NUMS     //!< Number of transparency levels
    
} OSD_TRANSPARENCY_E;

enum
{
    FUJITSU_ZWO_BOLD,
    FUJITSU_SYMBOL_BOLD,
    FONT_NUMS
};

enum     /** Panel types */
{
    OSD_PNLTYPE_MENUWTITLE = 0,      // no Help
    OSD_PNLTYPE_MENUWTITLEWHELP0,    // 0 bottom help lines
    OSD_PNLTYPE_MENUWTITLEWHELP1,    // 1 bottom help lines
    OSD_PNLTYPE_MENUWTITLEWHELP2,    // 2 bottom help lines
    OSD_PNLTYPE_MENUWTITLEWHELP3,    // 3 bottom help lines
    OSD_PNLTYPE_MENUWTITLEWHELP4,    // 4 bottom help lines
    OSD_PNLTYPE_MENUWTITLEWHELP5,    // 5 bottom help lines
    OSD_PNLTYPE_MENUWTITLEWHELP6,    // 6 bottom help lines
    OSD_PNLTYPE_MENUWTITLEWBOTHELP0,    // 0 bottom help lines no Help list
    OSD_PNLTYPE_MENUWTITLEWBOTHELP1,    // 1 bottom help lines no Help list
    OSD_PNLTYPE_MENUWTITLEWBOTHELP2,    // 2 bottom help lines no Help list
    OSD_PNLTYPE_MENUWTITLEWBOTHELP3,    // 3 bottom help lines no Help list
    OSD_PNLTYPE_MENUWTITLEWBOTHELP4,    // 4 bottom help lines no Help list
    OSD_PNLTYPE_MENUWTITLEWBOTHELP5,    // 5 bottom help lines no Help list
    OSD_PNLTYPE_MENUWTITLEWBOTHELP6,    // 6 bottom help lines no Help list
    OSD_PNLTYPE_MENUWTITLEWBOTHELP7,    // 7 bottom help lines no Help list
    OSD_PNLTYPE_MENU,
    OSD_PNLTYPE_PROGGUIDE,
    OSD_PNLTYPE_EPGEXT,
    OSD_PNLTYPE_EPGEXT2,
    OSD_PNLTYPE_POPUP,
    OSD_PNLTYPE_CONFDIALOG,
    OSD_PNLTYPE_SELECTLST,
    OSD_PNLTYPE_TRANSPARENT,
    OSD_PNLTYPE_EDITSTRING,
    OSD_PNLTYPE_BLACKRECTANGLE,
    OSD_PNLTYPE_INFOBAR
};

#define OSD_PNLTYPEMASK_TYPE          0x000000FF  //!< OSD_PNLTYPE_XXX
#define OSD_PNLTYPEMASK_STACKEDFRAMES 0x0000FF00  //!< Number of stacked frames.
#define OSD_PNLTYPEMASK_HELPWIDTH     0xFFFF0000  //!< Help column width 
                                                  //!< in pixels.

/* ----- fgs item background types ----------------------------------------- */

#define OSD_TYPEMASK_BKGR       0x00FF
#define OSD_TYPEMASK_CORNERS    0x0F00

/** Corner identifiers */
#define OSD_CORNER_NONE         0x0000
#define OSD_CORNER_TOP_LEFT     0x0100
#define OSD_CORNER_TOP_RIGHT    0x0200
#define OSD_CORNER_BOT_LEFT     0x0400
#define OSD_CORNER_BOT_RIGHT    0x0800
#define OSD_CORNER_ALL          0x0F00

enum                            /** Box background types */
{
    OSD_BOX_BKGR_NORMAL = 0,    //!< Box is not part of menu title
    OSD_BOX_BKGR_TITLE,         //!< Box is part of menu title
    OSD_BOX_BKGR_POPUP,         //!< Box is part of popup box
    OSD_BOX_BKGR_HELP,          //!< Box is part of Help line
    OSD_BOX_BKGR_BLACK,         //!< Box has black background
    OSD_BOX_BKGR_NONE,          //!< Drawing of background not necessary
    OSD_BOX_BKGR_TRANSPARENT
};

enum
{
    OSD_LSTITEM_BKGR_NORMAL = 0,   //!< Standard list item type
    OSD_LSTITEM_BKGR_HELP,
    OSD_LSTITEM_BKGR_TITLE,

};

enum
{
    OSD_NAVIBKGR_NORMAL = 0,         //!< Standard navigation list
    OSD_NAVIBKGR_HELP,
    OSD_NAVIBKGR_NOSELECTION,
};


/* ----- Menu item dimensions ---------------------------------------------- */

#define OSD_GetPanelTitleHeight(type)    (SCALER_GetCurrVal(TITLE_LINE_HEIGHT) + SCALER_GetCurrVal(INNER_FRAME_WIDTH))
#define OSD_GetListHeight(numLines)      (numLines * SCALER_GetCurrVal(LIST_LINE_HEIGHT))

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

int32_t OSD_Init                (void);
int32_t OSD_Open                (SYS_HDTV_STANDARD hdSys,
                                 SYS_SDTV_STANDARD sdSys);
int32_t OSD_Close               (void);

int32_t OSD_GetOsdLayer         (void);
void    OSD_GiveOsdLayer        (void);
void OSD_SetColor            (uint16_t i, uint32_t rgba);

int32_t  OSD_SetColorScheme      (OSD_COLSCHEME_E scheme);
int32_t  OSD_GetColorScheme      (OSD_COLSCHEME_E* scheme);
uint8_t  OSD_GetSchemeColor      (OSD_SCHEMECOL_E id);

int32_t OSD_SetTransparency     (OSD_TRANSPARENCY_E transparency);
int32_t OSD_GetTransparency     (OSD_TRANSPARENCY_E* transparency);

void OSD_Fill                (OSDHANDLER_StartParamsT* reqLayer, uint8_t c);
void OSD_DrawRectangle       (uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint32_t c);
void OSD_DrawFilledRectangle (uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint32_t c);

void OSD_DrawSeparatorVertical   (uint16_t x, uint16_t y, uint16_t height);
void OSD_DrawSeparatorHorizontal (uint16_t x, uint16_t y, uint16_t width);

void OSD_DrawString    (uint16_t x, uint16_t y, uint32_t c, uint8_t font, SYS_STRING_S string);
void OSD_DrawStringLtd (uint16_t x, uint16_t y, uint32_t c, uint8_t font, int32_t numBytes,
                        uint16_t maxWidth, int32_t strWidth, SYS_STRING_S string,
                        fbool_t softCut);

uint32_t  OSD_GetStrWidth       (SYS_STRING_S string, int32_t numBytes, uint8_t font);
void OSD_GetFontPars            (uint8_t font, uint8_t* ascent, uint8_t* descent, int8_t* offsetY);

void OSD_DrawStatusBarVertical  (uint16_t x, uint16_t y, uint16_t width, uint16_t height,
                                 uint32_t lvlCurr, uint32_t lvlMax,
                                 uint32_t colBarActive, uint32_t colBarInactive, uint32_t colFrame);
void OSD_DrawStatusBarHorizontal(uint16_t x, uint16_t y, uint16_t width, uint16_t height,
                                 uint32_t lvlCurr, uint32_t lvlMax,
                                 uint32_t colBarActive, uint32_t colBarInactive, uint32_t colFrame);

const uint32_t* OSD_IconGet     (SCALER_PIC_ID_E id);

void OSD_DrawHorizontalLine     (uint16_t x1, uint16_t x2, uint16_t y,
                                 uint32_t c);
void OSD_DrawVerticalLine       (uint16_t x, uint16_t y1, uint16_t y2,
                                 uint32_t c);

#ifdef __cplusplus
}
#endif

#endif /* _APP_OSD_H_ */
/******************************************************************************/
/* END OF FILE */
