/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/inc/config.h
**
**              $Id: config.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Contains specific compile-time configuration options for
**              "app_ref" application.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _CONFIG_H_
#define _CONFIG_H_

/* ***************************************************************************
 *
 * System configuration options
 *
 ****************************************************************************/

#define SYS_AUDIO_STR_MAX      8  //!< Max. number of audio streams per service.

/* ***************************************************************************
 *
 * TTX configuration options */

/*!
 * Size of TTX Page cache in bytes (static allocation). Each page requires
 *  1400 byte and the TTX decoder requires 70k byte roughly. The page number
 *  affects directly the speed of displaying a next page.
 */
#define MY_TTX_NO_OF_PAGES   200

/* ***************************************************************************
 *
 * EIT database configuration options
 *
 ****************************************************************************/

/*!
 * Number of services to store curr/next events for.
 */
#define AV_EIT_CURRNEXT_SERV_MAX      7
/*!
 * Number of services to store sched events for.
 */
#define AV_EIT_SCHED_SERV_MAX         7

/*!
 * Max. entries stored for each service.
 */
#define AV_SCHED_ENTRIES_MAX        450

/*!
 * Entries starting later than this
 * day from now are discarded
 */
#define AV_SCHED_DAYS_MAX             7  //!< Max. stored days in EIT Schedule.
#define AV_SCHED_STORE_EXT            1  //!< if 1: store extended information

/*!
 * Max. stored length of event description (current/next).
 */
#define AV_EIT_EXTLEN_CURRNEXT     1024
/*!
 * Max. stored length of event description (schedule).
 */
#define AV_EIT_EXTLEN_SCHED         896

#define AV_EIT_NAMELEN_MAX           64   //!< Max. length: event name.
#define AV_EIT_DESCLEN_MAX          256   //!< Max. length: short event description.


/* ***************************************************************************
 *
 * Service list configuration
 *
 ****************************************************************************/

#define SLIST_USR_NUMLISTS      10 //!< Number of user service lists.

#define SLIST_MAX_TV          2000 //!< Max. entries in global TV list.
#define SLIST_MAX_RADIO       1000 //!< Max. entries in global radio list.
#define SLIST_MAX_INVISIBLE    100 //!< Max. entries in "invisible" list.
#define SLIST_USR_SIZE         100 //!< Max. entries in user list.

#define PVR_LIST_MAX_ITEMS     100 //!< Max. entries in list of recorded files

/* ***************************************************************************
 *
 * TP and Satellite configuration options
 *
 ****************************************************************************/

#define CFGSAT_SAT_MAX           32 //!< Max. number of supported satellites

#define TPLIST_CARRIERS_MAX     200 //!< Max. number of carriers per network.
#define CARRIER_SERVICE_MAX     200 //!< Max. number of services per carrier.

#define CHANREGION_DEFAULT       CHANREGION_WESTEUROPE
//!< default region setting: used for choosing of appropriate channel list

/* ***************************************************************************
 *
 * Miscellaneous options
 *
 ****************************************************************************/

/*!
 * Initial startup mode: only #SYS_PWRMODE_UP or #SYS_PWRMODE_DOWN is possible
 */
#define PWR_INITIAL_STARTUPMODE    SYS_PWRMODE_DOWN

/*!
 * Number of seconds prior to record timer expiration at which the first
 * rectimer notification is sent.
 */
#define RECTIMER_NOTIFY_TIME     40

/*!
 * Minimum time in deci-seconds that a service must be selected to become
 * target service for a swap operation (set to 0 to disable swap interval
 * feature).
 */
#define AV_MIN_SWAP_INTERVAL     10

/*!
 * Timeout for start-of-decoding of a selected service in deci-seconds. On
 * expiration #EVT_SERVICE_NO_DECODING will be sent.
 */
#define AV_SERVICE_DECODING_TIMEOUT 60

/*!
 * Number of entries in applicationevent queue.
 */
#define EVT_EVENTS_MAX           96

/*!
 * Maximum size of one application event.
 */
#define EVT_SIZE_MAX             128

#define AV_DEFAULTVOLUME_LEFT    160 //!< Initial audio level on left channel.
#define AV_DEFAULTVOLUME_RIGHT   160 //!< Initial audio level on right channel.

#define RECTIMER_MAX             48  //!< Max. number of recording timers.

/* ***************************************************************************
 *
 * Flash File System mapping
 *
 * These values define the minimum file size required for each file residing on
 * flash. Mapping on flash sectors will be done at runtime. Note that changing
 * configuration parameters like SLIST_MAX_TV might affect the minimum
 * required size for a corresponding file.
 *
 * NOTE: A change in these values might result in a modified file mapping on
 *       flash, invalidating at least some of the files already residing on
 *       flash.
 *
 ****************************************************************************/

#define FILESIZE_MIN_SATDAT                  32     //!< Min. size in kbytes.
#define FILESIZE_MIN_TPLIST_SAT              448    //!< Min. size in kbytes.
#define FILESIZE_MIN_TPLIST_TER              128    //!< Min. size in kbytes.
#define FILESIZE_MIN_TPLIST_CAB              128    //!< Min. size in kbytes.
#define FILESIZE_MIN_STARTUPPARAMS           8      //!< Min. size in kbytes.
#define FILESIZE_MIN_SERVICEDATA             1024   //!< Min. size in kbytes.
#define FILESIZE_MIN_FREQCHGDATA             8      //!< Min. size in kbytes.
#define FILESIZE_MIN_RECTIMER                16     //!< Min. size in kbytes.
#define FILESIZE_MIN_PCTRL                   8      //!< Min. size in kbytes.
#define FILESIZE_MIN_CADATA                  128    //!< Min. size in kbytes.
#define FILESIZE_MIN_PVRDAT                  8      //!< Min. size in kbytes.

/* ***************************************************************************
 *
 *  RTOS thread configuration
 *
 ****************************************************************************/
#define THREAD_NAME_START                 "Startup"
#define THREAD_NAME_APPLICATION           "Appl"
#define THREAD_NAME_AV                    "AV"
#define THREAD_NAME_INPOLL                "Input Poller"
#define THREAD_NAME_TTX                   "TTX"
#define THREAD_NAME_TTX_DEC               "TTX Decoder"
#define THREAD_NAME_SUBT                  "SUBT"
#define THREAD_NAME_TIMER                 "Timer"
#define THREAD_NAME_MMI                   "MMI"
#define THREAD_NAME_CI                    "CI"
#define THREAD_NAME_PIP                   "PIP"
#define THREAD_NAME_FILEOP                "FileOp"
#define THREAD_NAME_JPEG                  "JPEG"
#define THREAD_NAME_MP3                   "MP3"
#define THREAD_NAME_DEVICEOP              "DeviceOp"
#define THREAD_NAME_UPDATE                "Update"
#define THREAD_NAME_DOWNLOAD              "Download"
#define THREAD_NAME_USB                   "USB BSR"
#define THREAD_NAME_MHEG_AD_NOTIFY        "MHEG Audio"
#define THREAD_NAME_MHEG_TIMER_BSR        "MHEG Timer"
#define THREAD_NAME_MHEG_AC               "MHEG Audio Clip"
#define THREAD_NAME_MHEG_HTTP             "MHEG HTTP"
#define THREAD_NAME_TCP_IP                "TCP/IP"
#define THREAD_NAME_ETHERNET              "Ethernet"

#define THREAD_STACKSZ_START              ( 8 * 1024)
#define THREAD_STACKSZ_APPLICATION        (16 * 1024)
#define THREAD_STACKSZ_AV                 (16 * 1024)
#define THREAD_STACKSZ_INPOLL             ( 3 * 1024)
#define THREAD_STACKSZ_TTX                ( 4 * 1024)
#define THREAD_STACKSZ_TTX_DEC            ( 4 * 1024)
#define THREAD_STACKSZ_SUBT               ( 6 * 1024)
#define THREAD_STACKSZ_TIMER              ( 2 * 1024)
#define THREAD_STACKSZ_CI                 ( 8 * 1024)
#define THREAD_STACKSZ_MMI                ( 8 * 1024)
#define THREAD_STACKSZ_PIP                ( 2 * 1024)
#define THREAD_STACKSZ_FILEOP             (16 * 1024)
#define THREAD_STACKSZ_JPEG               (25 * 1024)
#define THREAD_STACKSZ_MP3                (16 * 1024)
#define THREAD_STACKSZ_DEVICEOP           ( 8 * 1024)
#define THREAD_STACKSZ_UPDATE             ( 2 * 1024)
#define THREAD_STACKSZ_FLASHFS            ( 4 * 1024)
#define THREAD_STACKSZ_DOWNLOAD           ( 2 * 1024)
#define THREAD_STACKSZ_USB                ( 2 * 1024)
#define THREAD_STACKSZ_MHEG_AD_NOTIFY     ( 2 * 1024)
#define THREAD_STACKSZ_MHEG_TIMER_BSR     ( 2 * 1024)
#define THREAD_STACKSZ_MHEG_AC            ( 4 * 1024)
#define THREAD_STACKSZ_MHEG_HTTP          (32 * 1024)
// TODO: check if stack sizes are sufficient
#define THREAD_STACKSZ_TCP_IP             ( 4 * 1024)
#define THREAD_STACKSZ_ETHERNET           ( 8 * 1024)


/*!
 * System thread priorities in ascending order (lowest first)
 *
 * - start thread must have the highest priority.
 * - RC polling should have a relative high priority.
 * - priority for recording should be higher than for playback
 * - control thread priority must be lower than recording and playback
 */
enum
{
    THREAD_PRIO_SFAL = 20,       //!< THREAD_PRIO_SFAL
    THREAD_PRIO_MHEG_AD_NOTIFY,  //!< THREAD_PRIO_MHEG_AD_NOTIFY
    THREAD_PRIO_MHEG_LOW,        //!< THREAD_PRIO_MHEG_LOW
    THREAD_PRIO_FLASHFS,         //!< THREAD_PRIO_FLASHFS
    THREAD_PRIO_DOWNLOAD,        //!< THREAD_PRIO_DOWNLOAD
    THREAD_PRIO_UPDATE,          //!< THREAD_PRIO_UPDATE
    THREAD_PRIO_MHEG_LOWER,      //!< THREAD_PRIO_MHEG_LOWER
    // TODO: find best-fitting prios for TCP/IP and Ethernet threads!
    THREAD_PRIO_TCP_IP,          //!< THREAD_PRIO_TCP_IP
    THREAD_PRIO_MHEG_HTTP,       //!< THREAD_PRIO_MHEG_HTTP
    THREAD_PRIO_ETHERNET,        //!< THREAD_PRIO_ETHERNET
    THREAD_PRIO_TTX,             //!< THREAD_PRIO_TTX
    THREAD_PRIO_TTX_DEC,         //!< THREAD_PRIO_TTX_DEC
    THREAD_PRIO_SUBT,            //!< THREAD_PRIO_SUBT
    THREAD_PRIO_MHEG_MAIN,       //!< THREAD_PRIO_MHEG_MAIN
    THREAD_PRIO_FILEOP,          //!< THREAD_PRIO_FILEOP
    THREAD_PRIO_DEVICEOP,        //!< THREAD_PRIO_DEVICEOP
    THREAD_PRIO_MHEG_NORMAL,     //!< THREAD_PRIO_MHEG_NORMAL
    THREAD_PRIO_MHEG_AC,         //!< THREAD_PRIO_MHEG_AC
    THREAD_PRIO_AV,              //!< THREAD_PRIO_AV
    THREAD_PRIO_JPEG,            //!< THREAD_PRIO_JPEG
    THREAD_PRIO_MP3,             //!< THREAD_PRIO_MP3
    THREAD_PRIO_MHEG_HIGH,       //!< THREAD_PRIO_MHEG_HIGH
    THREAD_PRIO_DSM_FLTR,        //!< THREAD_PRIO_DSM_FLTR
    THREAD_PRIO_CONAX,           //!< THREAD_PRIO_CONAX
    THREAD_PRIO_HDMI,            //!< THREAD_PRIO_HDMI
    THREAD_PRIO_INPOLL,          //!< THREAD_PRIO_INPOLL
    THREAD_PRIO_APPLICATION,     //!< THREAD_PRIO_APPLICATION
    THREAD_PRIO_MMI,             //!< THREAD_PRIO_MMI
    THREAD_PRIO_CI,              //!< THREAD_PRIO_CI
    THREAD_PRIO_PVR_PLAY_FEED,   //!< THREAD_PRIO_PVR_PLAY_FEED
    THREAD_PRIO_PVR_PLAY_READ,   //!< THREAD_PRIO_PVR_PLAY_READ
    THREAD_PRIO_PVR_REC_LO,      //!< THREAD_PRIO_PVR_REC_LO
    THREAD_PRIO_PVR_REC_HI,      //!< THREAD_PRIO_PVR_REC_HI
    THREAD_PRIO_TIMER,           //!< THREAD_PRIO_TIMER
    THREAD_PRIO_TIMER_CB,
    THREAD_PRIO_MHEG_TIMER_BSR,  //!< THREAD_PRIO_MHEG_TIMER_BSR
    THREAD_PRIO_MHEG_REALTIME,   //!< THREAD_PRIO_MHEG_REALTIME
    THREAD_PRIO_START,           //!< THREAD_PRIO_START
};

/*!
 * default flash type when booting over debug port choice is
 * #FAPI_FLASH_DEVICE_PFLASH or #FAPI_FLASH_DEVICE_SFLASH
 */
#ifndef _LINUX_FAPI
#define DEFAULT_FLASH_DEVICE   FAPI_FLASH_DEVICE_PFLASH
#else
#define DEFAULT_FLASH_DEVICE   FAPEX_VRFLASH_DEVICE_SFLASH
#endif

#endif /* _CONFIG_H_ */
/* ***************************************************************************/
/* END OF FILE */
