/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/inc/menustack.h
**
**              $Id: menustack.h 5866 2010-03-02 12:09:31Z jmorha $ 
**
** \brief       Menu stack implementation.
**
** Detail description. [\ref app_smartgo_app_ref_menustack]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _MENUSTACK_H_
#define _MENUSTACK_H_

/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/

#define MENU_STACKITEMS_MAX    16  // max. depth of menu hierarchy
#define MENU_EXTITEMS_MAX       8  // max. number of external items

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

typedef fbool_t (*MENUSTACK_ENTRY)   (FAPI_SYS_HandleT h, fbool_t test);
typedef fbool_t (*MENUSTACK_CLEANUP) (FAPI_SYS_HandleT h, fbool_t force);

typedef struct
{
    /* menu's entry point */
    MENUSTACK_ENTRY     entry;
    FAPI_SYS_HandleT    entryArg;

    /* optional cleanup function */
    MENUSTACK_CLEANUP   cleanup;
    FAPI_SYS_HandleT    cleanupArg;

    /* menu's general-purpose data */
    fbool_t             dataValid;
    uint32_t            data;

} MENUSTACK_ITEM_S;

typedef struct
{
    fbool_t             used;
    MENUSTACK_CLEANUP   cleanup;
    FAPI_SYS_HandleT    cleanupArg;

} MENUSTACK_EXTITEM_S;

typedef struct
{
    void (*lastMenuClosed) (void);
    void (*firstMenuToOpen)(void);
    
    MENUSTACK_ENTRY     fallbackMenu;
    FAPI_SYS_HandleT    fallbackMenuArg;
    
} MENUSTACK_INIT_S;

typedef struct
{
    MENUSTACK_INIT_S    pars;
    
    MENUSTACK_ITEM_S    menuStack[MENU_STACKITEMS_MAX];
    uint16_t            menuStackLvl;
    fbool_t             menuStackForceExit;
    
    MENUSTACK_EXTITEM_S menuStackExt[MENU_EXTITEMS_MAX];
    uint16_t            numExt;
    
    /* FGS panel focus handling */
    int32_t             focusClaimId;  //!< Panel ID claiming or having focus.
    int32_t             focusForceId;  //!< Panel ID having focus by force.
    
} MENUSTACK_INST_S;

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

int32_t  MENUSTACK_Init         (MENUSTACK_INIT_S* initPars);
fbool_t  MENUSTACK_Up           (MENUSTACK_ENTRY newEntry,
                                 FAPI_SYS_HandleT arg);
fbool_t  MENUSTACK_Switch       (MENUSTACK_ENTRY newEntry,
                                 FAPI_SYS_HandleT arg,
                                 int16_t targetLevel);
fbool_t  MENUSTACK_Back         (FAPI_SYS_HandleT h);
fbool_t  MENUSTACK_Exit         (FAPI_SYS_HandleT h);
uint16_t MENUSTACK_GetLevel     (void);
fbool_t  MENUSTACK_GetData      (uint32_t* data);
void     MENUSTACK_SetData      (uint32_t data);
void     MENUSTACK_SetEntry     (MENUSTACK_ENTRY newEntry,
                                 FAPI_SYS_HandleT arg);
void     MENUSTACK_SetCleanup   (MENUSTACK_CLEANUP newCleanup,
                                 FAPI_SYS_HandleT arg);
void     MENUSTACK_SetForceMode (void);

int32_t  MENUSTACK_AddExt       (MENUSTACK_CLEANUP newCleanup,
                                 FAPI_SYS_HandleT arg);
int32_t  MENUSTACK_RemoveExt    (int32_t index);
fbool_t  MENUSTACK_CloseAllExt  (fbool_t force);

int32_t  MENUSTACK_FocusClaim   (int32_t id);
int32_t  MENUSTACK_FocusRelease (int32_t id);
int32_t  MENUSTACK_FocusForce   (int32_t id);

#ifdef __cplusplus
}
#endif

#endif /* _MENUSTACK_H_ */
/******************************************************************************/
/* END OF FILE */
