/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_confdialog.c
**
**              $Id: app_confdialog.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Implementation of confirmation dialogue module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info 713: Loss of precision (assignment) (unsigned int to short).
** - Info 715: Symbol not referenced.
** - Info 737: Loss of sign in promotion from int to unsigned int.
** - Note 961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**   C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_CONFDIALOG_ID_DESC = 0,
    MENU_CONFDIALOG_HELP_INFOTEXT,
    MENU_CONFDIALOG_HELP_ID_CONFIRMKEY,
    MENU_CONFDIALOG_HELP_ID_CONFIRMTEXT,
    MENU_CONFDIALOG_HELP_ID_DECLINEKEY,
    MENU_CONFDIALOG_HELP_ID_DECLINETEXT,
    MENU_CONFDIALOG_HELP_ID_DISCARDKEY,
    MENU_CONFDIALOG_HELP_ID_DISCARDTEXT

};

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t MENU_ConfDialogKeys       (FAPI_SYS_HandleT h);
FSTATIC void   MENU_ConfDialogOnFocusSet  (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC fbool_t MENU_ConfDialogIntClose   (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC void   MENU_ConfDialogGetItem     (FAPI_SYS_HandleT h,
                                           int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);

int32_t MENU_ConfDialogInit (MENU_CONFDIALOG_INST_S* inst)
{
    memset(inst, 0, sizeof(MENU_CONFDIALOG_INST_S));

    return FAPI_OK;
}

fbool_t MENU_ConfDialogOpen (MENU_CONFDIALOG_INST_S* inst,
                           const MENU_CONFDIALOG_S* openParams)
{
    FGS_PANEL_OPEN_S        pnlDat;
    FGS_PANEL_MEMBER_S      member;
    FGS_BOX_OPEN_S          boxPars;
    int32_t                 retVal = FAPI_OK;
    uint16_t                numDescLines=0;
    uint16_t                numHelpLines=0;
    FAPI_SYS_HandleT        itemHdl;

    /* ensure that dialogue is not opened twice */
    if ( inst->pnlHdl != NULL ) return FFALSE;

    inst->data   = *openParams;
    inst->extIdx = -1;

    memset(&pnlDat,  0, sizeof(pnlDat));
    memset(&member,  0, sizeof(member));
    memset(&boxPars, 0, sizeof(boxPars));

    if ( !(STR_IsEmpty(inst->data.confirmStr)) ) numHelpLines++;
    if ( !(STR_IsEmpty(inst->data.declineStr)) ) numHelpLines++;
    if ( !(STR_IsEmpty(inst->data.discardStr)) ) numHelpLines++;

    do
    {
        pnlDat.bkgrType      = OSD_PNLTYPE_CONFDIALOG;
        if(openParams->pos == NULL)
        {
            if ( !(STR_IsEmpty(inst->data.infoStr)) )
            {
                numDescLines  = FGS_GetLinesFromString (inst->data.infoStr,
                                                        SCALER_GetCurrVal(CONFDIALOG_WIDTH),
                                                        MAIN_FONT, NULL);
            }

            pnlDat.pos.x         = 0;
            pnlDat.pos.y         = 0;
            pnlDat.pos.width     = SCALER_GetCurrVal(CONFDIALOG_WIDTH);
            pnlDat.pos.height    = (2*SCALER_GetCurrVal(OUTER_FRAME_ALL)) +
                                    SCALER_GetCurrVal(MAIN_TOP_PADDING) + SCALER_GetCurrVal(MAIN_BOT_PADDING) +
                                    ( inst->data.numDescLines        * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                                    ((inst->data.numDescLines - 1 )  * SCALER_GetCurrVal(LIST_LINE_PADDING)) +
                                    ( numDescLines    * SCALER_GetCurrVal(HELP_LINE_HEIGHT)) +
                                    ((numDescLines -1)* SCALER_GetCurrVal(HELP_LINE_PADDING)) +
                                    ( numHelpLines    * SCALER_GetCurrVal(HELP_LINE_HEIGHT)) +
                                    ((numHelpLines -1)* SCALER_GetCurrVal(HELP_LINE_PADDING)) +
                                    (2*SCALER_GetCurrVal(HELP_TB_PADDING)) + SCALER_GetCurrVal(INNER_FRAME_WIDTH);
            pnlDat.pos.alignment = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
        }
        else
        {
            if ( !(STR_IsEmpty(inst->data.infoStr)) )
            {
                numDescLines  = FGS_GetLinesFromString (inst->data.infoStr,
                                                        openParams->pos->width,
                                                        MAIN_FONT, NULL);
            }

            pnlDat.pos.x         = openParams->pos->x;
            pnlDat.pos.y         = openParams->pos->y;
            pnlDat.pos.width     = openParams->pos->width;
            pnlDat.pos.height    = openParams->pos->height;
            pnlDat.pos.alignment = openParams->pos->alignment;
        }

        pnlDat.onFocusSet    = MENU_ConfDialogOnFocusSet;
        pnlDat.onFocusSetArg = (FAPI_SYS_HandleT)inst;
        pnlDat.pInst         = &(inst->pnlInst);

        retVal = FGS_PANEL_Open(&pnlDat, MENUID_CONFDIALOG, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        boxPars.getItem         = MENU_ConfDialogGetItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        boxPars.pos.x           = SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                  SCALER_GetCurrVal(OUTER_FRAME_ALL);
        boxPars.pos.y           = SCALER_GetCurrVal(MAIN_TOP_PADDING);
        boxPars.pos.width       = pnlDat.pos.width -
                                  boxPars.pos.x -
                                  SCALER_GetCurrVal(MAIN_TOP_PADDING) -
                                  SCALER_GetCurrVal(OUTER_FRAME_ALL);
        boxPars.pos.height      = inst->data.numDescLines *
                                  SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_POPUP;

        retVal = FGS_BOX_Open (&boxPars, MENU_CONFDIALOG_ID_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Bottom help lines ------------------------------------------ */

        boxPars.pos.y               = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - 2*SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                      - numHelpLines*
                                        SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - (numHelpLines-1)*
                                        SCALER_GetCurrVal(HELP_LINE_PADDING)
                                      - numDescLines*
                                        SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - (numDescLines-1)*
                                        SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;

        /* -- Info Help line -- */
        if ( !(STR_IsEmpty(inst->data.infoStr)) )
        {
            boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
            boxPars.pos.width       = pnlDat.pos.width -
                                      boxPars.pos.x -
                                      SCALER_GetCurrVal(MAIN_TOP_PADDING) -
                                      SCALER_GetCurrVal(OUTER_FRAME_ALL);
            boxPars.pos.height      = numDescLines *
                                      SCALER_GetCurrVal(HELP_LINE_HEIGHT);

            retVal = FGS_BOX_Open (&boxPars, MENU_CONFDIALOG_HELP_INFOTEXT,
                                        inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            boxPars.pos.y += numDescLines *
                             SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                             (numDescLines-1) *
                             SCALER_GetCurrVal(HELP_LINE_PADDING);
        }

        /* -- 1. Bottom line -- */
        if ( !(STR_IsEmpty(inst->data.confirmStr)) )
        {
            boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
            boxPars.pos.width           = SCALER_GetCurrVal(CONFDIALOG_COL_WIDTH);
            boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);

            retVal = FGS_BOX_Open (&boxPars, MENU_CONFDIALOG_HELP_ID_CONFIRMKEY,
                                        inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            boxPars.pos.x              += SCALER_GetCurrVal(CONFDIALOG_COL_WIDTH) +
                                          SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
            boxPars.pos.width           = pnlDat.pos.width - boxPars.pos.x -
                                          SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

            retVal = FGS_BOX_Open (&boxPars, MENU_CONFDIALOG_HELP_ID_CONFIRMTEXT,
                                        inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;
            boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);
        }

        /* -- 2. Bottom line -- */
        if ( !(STR_IsEmpty(inst->data.declineStr)) )
        {
            boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
            boxPars.pos.width           = SCALER_GetCurrVal(CONFDIALOG_COL_WIDTH);

            retVal = FGS_BOX_Open (&boxPars, MENU_CONFDIALOG_HELP_ID_DECLINEKEY,
                                        inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            boxPars.pos.x              += SCALER_GetCurrVal(CONFDIALOG_COL_WIDTH) +
                                          SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
            boxPars.pos.width           = pnlDat.pos.width - boxPars.pos.x -
                                          SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

            retVal = FGS_BOX_Open (&boxPars, MENU_CONFDIALOG_HELP_ID_DECLINETEXT,
                                        inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;
            boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);
        }

        /* --3. Bottom line -- */
        if ( !(STR_IsEmpty(inst->data.discardStr)) )
        {
            boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
            boxPars.pos.width           = SCALER_GetCurrVal(CONFDIALOG_COL_WIDTH);

            retVal = FGS_BOX_Open (&boxPars, MENU_CONFDIALOG_HELP_ID_DISCARDKEY,
                                        inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            boxPars.pos.x              += SCALER_GetCurrVal(CONFDIALOG_COL_WIDTH) +
                                          SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
            boxPars.pos.width           = pnlDat.pos.width - boxPars.pos.x -
                                          SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

            retVal = FGS_BOX_Open (&boxPars, MENU_CONFDIALOG_HELP_ID_DISCARDTEXT,
                                        inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;
        }

        /* ----------------------------------------------------------------- */

        inst->extIdx = MENUSTACK_AddExt(MENU_ConfDialogIntClose,
                                       (FAPI_SYS_HandleT)inst);

        (void)MENUSTACK_FocusForce ((int32_t)MENUID_CONFDIALOG);
        (void)FGS_PANEL_Show       (inst->pnlHdl, FFALSE); /* turn visibility on */
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        inst->pnlHdl = NULL;
        return FFALSE;
    }

    return FTRUE;
}

fbool_t MENU_ConfDialogClose (MENU_CONFDIALOG_INST_S* inst)
{
    /* close dialog box */
    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
        inst->pnlHdl = NULL;

        (void)MENUSTACK_FocusRelease ((int32_t)MENUID_CONFDIALOG);
        (void)MENUSTACK_RemoveExt    (inst->extIdx);
    }

    return FTRUE;
}


FSTATIC void MENU_ConfDialogOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    MENU_CONFDIALOG_INST_S* inst    = (MENU_CONFDIALOG_INST_S*)h;

    if(inst == NULL) return;

    DBG_Assert(inst->pnlHdl != NULL);

    if ( hasFocus )
    {
        if ( !(STR_IsEmpty(inst->data.confirmStr)) )
        {
            FGS_SetAppKeyHandler(FGS_KEY_OK, MENU_ConfDialogKeys, h,
                                 FTRUE, FFALSE);
        }

        if ( !(STR_IsEmpty(inst->data.discardStr)) )
        {
            FGS_SetAppKeyHandler(FGS_KEY_EXIT, MENU_ConfDialogKeys, h,
                                 FTRUE, FFALSE);
        }

        if ( !(STR_IsEmpty(inst->data.declineStr)) )
        {
            FGS_SetAppKeyHandler(FGS_KEY_RED, MENU_ConfDialogKeys, h,
                                 FTRUE, FFALSE);
        }
    }
    else
    {
        if ( !(STR_IsEmpty(inst->data.confirmStr)) )
        {
            FGS_ClearAppKeyHandler(FGS_KEY_OK);
        }
        if ( !(STR_IsEmpty(inst->data.discardStr)) )
        {
            FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        }
        if ( !(STR_IsEmpty(inst->data.declineStr)) )
        {
            FGS_ClearAppKeyHandler(FGS_KEY_RED);
        }
    }
}

FSTATIC fbool_t MENU_ConfDialogKeys (FAPI_SYS_HandleT h)
{
    MENU_CONFDIALOG_INST_S* inst    = (MENU_CONFDIALOG_INST_S*)h;
    fbool_t                   keyUsed = FTRUE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_OK:
            if ( inst->data.onConfirm != NULL ) inst->data.onConfirm(inst->data.arg);
            break;

        case FGS_KEY_RED:
            if ( inst->data.onDecline != NULL ) inst->data.onDecline(inst->data.arg);
            break;

        case FGS_KEY_EXIT:
            if ( inst->data.onDiscard != NULL ) inst->data.onDiscard(inst->data.arg);
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    if ( keyUsed )
    {
        (void)MENU_ConfDialogClose(inst);
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_ConfDialogIntClose (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_CONFDIALOG_INST_S* inst = (MENU_CONFDIALOG_INST_S*)h;

    /* This dialog closes by force only */
    if ( force == FFALSE ) return FFALSE;

    return MENU_ConfDialogClose(inst);
}

FSTATIC void MENU_ConfDialogGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                     FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_CONFDIALOG_INST_S* inst    = (MENU_CONFDIALOG_INST_S*)h;

    switch ( id )
    {
        case MENU_CONFDIALOG_ID_DESC:
            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item->item.str.chr        = inst->data.titleStr;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            item->item.str.col        = OSD_COL_TITLE_TEXT;
            break;

        case MENU_CONFDIALOG_HELP_INFOTEXT:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            item->item.str.chr           = inst->data.infoStr;
            item->item.str.font          = HELP_FONT;
            item->item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
            item->item.str.col           = OSD_COL_HELP_TEXT;
            break;

        case MENU_CONFDIALOG_HELP_ID_CONFIRMKEY:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            item->item.str.chr           = OSD_GetStr(STR_KEY_OK);
            item->item.str.font          = HELP_FONT;
            item->item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
            item->item.str.col           = OSD_COL_HELP_TEXT;
            break;

        case MENU_CONFDIALOG_HELP_ID_CONFIRMTEXT:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            item->item.str.chr           = inst->data.confirmStr;
            item->item.str.font          = HELP_FONT;
            item->item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
            item->item.str.col           = OSD_COL_HELP_TEXT;
            break;

        case MENU_CONFDIALOG_HELP_ID_DECLINEKEY:
            MENU_COMMON_ITEM_BUTTON(item, ICON_BTNREDSMALL);
            break;

        case MENU_CONFDIALOG_HELP_ID_DECLINETEXT:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            item->item.str.chr           = inst->data.declineStr;
            item->item.str.font          = HELP_FONT;
            item->item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
            item->item.str.col           = OSD_COL_HELP_TEXT;
            break;

        case MENU_CONFDIALOG_HELP_ID_DISCARDKEY:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            item->item.str.chr           = OSD_GetStr(STR_KEY_EXIT);
            item->item.str.font          = HELP_FONT;
            item->item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
            item->item.str.col           = OSD_COL_HELP_TEXT;
            break;

        case MENU_CONFDIALOG_HELP_ID_DISCARDTEXT:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            item->item.str.chr           = inst->data.discardStr;
            item->item.str.font          = HELP_FONT;
            item->item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
            item->item.str.col           = OSD_COL_HELP_TEXT;
            break;

        default:
            break;
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
