/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_directfunctions.c
**
**              $Id: app_directfunctions.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Functions mapped directly on RC keys.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    715: Symbol not referenced.
** - Info    730: Boolean argument to function.
**
*****************************************************************************
*/
/*lint -save -e715 -e730 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
#ifdef APPL_PVR_ENABLED
FSTATIC int32_t  MAIN_ServiceSwitchHandling (void);
#endif

fbool_t MAIN_VolumeUp (FAPI_SYS_HandleT h)
{
    uint8_t     lvlLeft;
    uint8_t     lvlRight;
    uint8_t     lvlMax;
    uint16_t    lvlNew;
    fbool_t     isMuted;

    if ( AV_VolumeGet(&lvlLeft, &lvlRight, &lvlMax, &isMuted) != FAPI_OK )
    {
        return FFALSE;
    }

    /* unmute if currently muted */
    if ( isMuted )
    {
        (void)AV_Mute(FFALSE);

        if ( AV_VolumeGet(&lvlLeft, &lvlRight, &lvlMax, &isMuted) != FAPI_OK )
        {
            return FFALSE;
        }
    }

    lvlNew = lvlLeft + AV_VOLUME_STEP;

    lvlLeft  = (lvlNew >= lvlMax) ? lvlMax : (uint8_t)lvlNew;

    (void)AV_VolumeSet(lvlLeft, lvlLeft);

    /* show volume bar if possible */
    (void)MAIN_VolumeBarShow(&(appDat->volBar));
    MAIN_FreqDataHasChanged();
    return FTRUE;
}



fbool_t MAIN_VolumeDown(FAPI_SYS_HandleT h)
{
    uint8_t    lvlLeft;
    uint8_t    lvlRight;
    uint8_t    lvlMax;
    fbool_t isMuted;

    if ( AV_VolumeGet(&lvlLeft, &lvlRight, &lvlMax, &isMuted) != FAPI_OK )
    {
        return FFALSE;
    }

    /* unmute if currently muted */
    if ( isMuted )
    {
        (void)AV_Mute(FFALSE);

        if ( AV_VolumeGet(&lvlLeft, &lvlRight, &lvlMax, &isMuted) != FAPI_OK )
        {
            return FFALSE;
        }
    }

    lvlLeft = (lvlLeft >= AV_VOLUME_STEP) ? (lvlLeft - AV_VOLUME_STEP) : 0;

    (void)AV_VolumeSet(lvlLeft, lvlLeft);

    /* show volume bar if possible */
    (void)MAIN_VolumeBarShow(&(appDat->volBar));
    MAIN_FreqDataHasChanged();
    return FTRUE;
}


fbool_t MAIN_VolumeMute (FAPI_SYS_HandleT h)
{
    uint8_t    lvlLeft;
    uint8_t    lvlRight;
    uint8_t    lvlMax;
    fbool_t isMuted;

    if ( AV_VolumeGet(&lvlLeft, &lvlRight, &lvlMax, &isMuted) == FAPI_OK )
    {
        /* toggle mute status */
        isMuted = (isMuted == FTRUE) ? FFALSE : FTRUE;

        (void)AV_Mute(isMuted);

        /* show volume bar if possible */
        (void)MAIN_VolumeBarShow(&(appDat->volBar));
    }

    return FTRUE;
}


/* ------------------------------------------------------------------------- */

int32_t MAIN_StartInitialService (void)
{
    #ifdef APPL_PVR_ENABLED

    int32_t retVal;

    retVal = MAIN_ServiceSwitchHandling();
    if ( retVal != FAPI_OK ) return retVal;

    #endif /* APPL_PVR_ENABLED */

    return AV_StartInitialService();
}

int32_t MAIN_SelectService (const SYS_SERVLOC_S* pServLoc, fbool_t quietMode)
{
    uint32_t    ctrlInfo = 0;
    #ifdef APPL_PVR_ENABLED

    int32_t retVal;

    retVal = MAIN_ServiceSwitchHandling();
    if ( retVal != FAPI_OK ) return retVal;

    #endif /* APPL_PVR_ENABLED */

    ctrlInfo = quietMode ? SERVICE_CTRL_QUIET : SERVICE_CTRL_NORMAL;
    return AV_SelectService (pServLoc, NULL, ctrlInfo, FFALSE);
}

int32_t MAIN_RestartLastService (void)
{
    int32_t retVal;

    #ifdef APPL_PVR_ENABLED

    retVal = MAIN_ServiceSwitchHandling();
    if ( retVal != FAPI_OK ) return retVal;

    #endif /* APPL_PVR_ENABLED */

    retVal = AV_RestartService();

    if ( retVal != FAPI_OK )
    {
        retVal = AV_StartInitialService();
    }


    return retVal;
}


fbool_t MAIN_ChannelUp (FAPI_SYS_HandleT h)
{
    if (  MAIN_SelectNextService() == FAPI_OK ) return FTRUE;
    else                                      return FFALSE;
}


fbool_t MAIN_ChannelDown (FAPI_SYS_HandleT h)
{
    if (  MAIN_SelectPreviousService() == FAPI_OK ) return FTRUE;
    else                                          return FFALSE;
}


int32_t MAIN_SelectNextService (void)
{
    SYS_SERVLOC_S servLoc;
    int32_t          retVal;

    #ifdef APPL_PVR_ENABLED

    retVal = MAIN_ServiceSwitchHandling();
    if ( retVal != FAPI_OK ) return retVal;

    #endif /* APPL_PVR_ENABLED */

    retVal = AV_GetRequestedService(&servLoc);
    if ( retVal != FAPI_OK ) return retVal;

    if ( servLoc.listId == SLIST_INVISIBLE ) return SMARTGO_SYS_ERR_PROHIBITED;

    return AV_SelectNextService();
}

int32_t MAIN_SelectPreviousService (void)
{
    SYS_SERVLOC_S servLoc;
    int32_t          retVal;

    #ifdef APPL_PVR_ENABLED

    retVal = MAIN_ServiceSwitchHandling();
    if ( retVal != FAPI_OK ) return retVal;

    #endif /* APPL_PVR_ENABLED */

    retVal = AV_GetRequestedService(&servLoc);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    if ( servLoc.listId == SLIST_INVISIBLE )
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    return AV_SelectPreviousService();
}


fbool_t MAIN_SwapTvAndRadio (FAPI_SYS_HandleT h)
{
    SYS_SERVLOC_S servLoc;
    SYS_SERVLOC_S newLoc;

    #ifdef APPL_PVR_ENABLED

    if ( MAIN_ServiceSwitchHandling() != FAPI_OK )
    {
        return FFALSE;
    }

    #endif /* APPL_PVR_ENABLED */

    newLoc.listId = SLIST_NONE;

    (void) AV_GetRequestedService (&servLoc);

    switch ( servLoc.listId )
    {
        case SLIST_RADIO:
            /* -> Swap to TV */
            if ( SLIST_ListSizeGet(SLIST_TV) > 0 )
            {
                newLoc.listId = SLIST_TV;
            }
            break;
        default:
        case SLIST_NONE:
        case SLIST_TV:
            /* -> Swap to radio */
            if ( SLIST_ListSizeGet(SLIST_RADIO) > 0 )
            {
                newLoc.listId = SLIST_RADIO;
            }
            break;
    }

    if ( (newLoc.listId != SLIST_NONE) &&
         (AV_GetPrevServiceByList(newLoc.listId, &(newLoc.index)) == FAPI_OK) )
    {
        if ( (newLoc.index < 0) &&
             (SLIST_ListSizeGet(newLoc.listId) > 0) )
        {
            /* If list to be swapped to has not been selected before,
               choose first service in list. */
            newLoc.index = 0;
        }

        if ( newLoc.index >= 0 )
        {
            (void)MAIN_SelectService(&newLoc, FFALSE);
        }
    }

    return FTRUE;
}


fbool_t MAIN_SwapService (FAPI_SYS_HandleT h)
{
    #ifdef APPL_PVR_ENABLED

    if ( MAIN_ServiceSwitchHandling() != FAPI_OK ) return FFALSE;

    #endif /* APPL_PVR_ENABLED */

    (void)AV_SwapService();
    return FTRUE;
}

/* ------------------------------------------------------------------------- */

#if 0

/* currently unused */
fbool_t MAIN_SetNextAudioStream (FAPI_SYS_HandleT h)
{
    SLIST_ITEM_S    service;
    SYS_SERVLOC_S   servLoc;
    int8_t          audioStrCurr;
    uint8_t         newIndex;
    fbool_t         retVal = FFALSE;

    #ifdef APPL_PVR_ENABLED

    SYS_STREAM_INFO_S playStreamInf;

    if ( appDat->pvrStatus.play.status != PLAY_STATUS_INACTIVE )
    {
        if ( PVR_PlayGetStreamInfo (&playStreamInf) != FAPI_OK )
        {
            return FFALSE;
        }

        if ( (PVR_PlayGetAudioStream (&audioStrCurr) != FAPI_OK) ||
             (audioStrCurr < 0) )
        {
            return FFALSE;
        }

        newIndex = audioStrCurr + 1;

        while ( newIndex != audioStrCurr )
        {
            if ( newIndex >= playStreamInf.audio.numStreams )
            {
                newIndex = 0;
                continue;
            }

            if ( playStreamInf.audio.stream[newIndex].format == SYS_AUDIO_MPEG ) break;

            newIndex++;
        }

        if ( newIndex != audioStrCurr )
        {
            PVR_PlaySetAudioStream(newIndex);
            retVal = FTRUE;
        }
    }
    else
    {

    #endif /* APPL_PVR_ENABLED */

        (void)AV_GetRequestedService (&servLoc);

    if ( SLIST_EntryGet (&servLoc, &service) != FAPI_OK )
    {
        return FFALSE;
    }

    if ( (AV_GetAudioStream (&audioStrCurr) != FAPI_OK) ||
         (audioStrCurr < 0) )
    {
        return FFALSE;
    }

    newIndex = audioStrCurr + 1;

    while ( newIndex != audioStrCurr )
    {
        if ( newIndex >= service.strPid.audio.numStreams )
        {
            newIndex = 0;
            continue;
        }

        if ( service.strPid.audio.stream[newIndex].format == SYS_AUDIO_MPEG ) break;

        newIndex++;
    }

    if ( newIndex != audioStrCurr )
    {
        AV_SetAudioStream(newIndex);
        retVal = FTRUE;
    }

    #ifdef APPL_PVR_ENABLED
    }
    #endif /* APPL_PVR_ENABLED */

    return retVal;
}

#endif

/* ------------------------------------------------------------------------- */

#ifdef APPL_PVR_ENABLED

FSTATIC int32_t MAIN_ServiceSwitchHandling (void)
{
    switch ( appDat->pvrStatus.play.type )
    {
        case PLAY_TYPE_NORMAL:
            /* service switch prohibited during non-timeshift playback */
            return SMARTGO_SYS_ERR_PROHIBITED;

        case PLAY_TYPE_TIMESHIFT:
            /* NOTE: Timeshift playback is stopped in
                     MAIN_PvrStopRecording(). */
            break;

        default:
            break;
    }

    /* stop potentially running background recording
       (+ timeshift playback if active) */
    if ( appDat->pvrStatus.bkgrRecIdx >= 0 )
    {
        (void)MAIN_PvrStopRecording (appDat->pvrStatus.bkgrRecIdx);
    }

    return FAPI_OK;
}

#endif /* APPL_PVR_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
