/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_main.c
**
**              $Id: app_main.c 5871 2010-03-02 13:42:55Z jmorha $
**
** \brief       Root menu and system setup functions.
**
** Detail description. [\ref app_smartgo_app_ref]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    715: Symbol not referenced
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
** - Note    970: Usage of int (return value of int main(void)
**
*****************************************************************************
*/
/*lint -save -e641 -e715  -e961 */
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapex/abst_types.h>
#include <fapex/abst_libir.h>
//#include <voc/voc.h>

#include "app_menu.h"
#include "app_menu_scan.h"
#include "app_menu_info.h"

#ifdef APPL_FESAT_ENABLED
#include "app_satellites.h"
#endif

#ifdef APPL_FILEOP_ENABLED
#include "app_menu_viewer.h"
#endif

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
//#define APPL_ENABLE_SPECIAL_TEST_HOTKEYS 1 //

#define MAIN_FCDATA_STORE_DELAY  2   // in seconds
#define MAIN_STARTUP_DELAY       2   // in seconds

#ifdef APPL_ENABLE_SPECIAL_TEST_HOTKEYS
  #include "key_input.h"
#endif

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
#ifdef APPL_FETERR_ENABLED

typedef struct
{
    char_t* name;
    uint32_t   freq;

} TERR_CHAN_VHF;

#endif /* APPL_FETERR_ENABLED */


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC uint32_t bootFrameData[] = { /* MPEG I-Frame data */
#include "../res/fujitsu.fmv"
};

FSTATIC uint32_t radioFrameData[] = {
#include "../res/radiobkgr.fmv"
};

#ifdef APPL_MHEG_ENABLED

//FSTATIC uint32_t mhegSplashData[] = {
//#include "../res/loading.fmv"
//};

/*
    The hostString uniquely identifies the receiver via its make, model and
    version number. This shall be of the form "mmmcccvvv", where "mmm"
    uniquely identifies the manufacturer, "ccc" is a unique model code and
    "vvv" is the principal version of the complete receiver software build.
*/
FSTATIC uint8_t hostString[] = "fme001001";

#endif /* APPL_MHEG_ENABLED */

#ifdef PLAY_WELCOME_JINGLE

FSTATIC uint8_t welcomeJingleData[] = {
   // #include "../res/audioWelcome.dat"
};

#endif /* PLAY_WELCOME_JINGLE */

APP_DATA_S* appDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC int32_t MENU_Init (void);

FSTATIC void    MAIN_HandleEvtKey             (const void* pEvt);
FSTATIC void    MAIN_HandleEvtServiceSelected (const void* pEvt);
FSTATIC void    MAIN_HandleEvtRunstat         (const void* pEvt);
FSTATIC void    MAIN_HandleEvtTimeUpdate      (const void* pEvt);
FSTATIC void    MAIN_HandleEvtTimerExpired    (const void* pEvt);
FSTATIC void    MAIN_HandleEvtLockStatus      (const void* pEvt);
FSTATIC void    MAIN_HandleEvtDeviceStatus    (const void* pEvt);
FSTATIC void    MAIN_HandleEvtPmtReceived     (const void* pEvt);
FSTATIC void    MAIN_HandleEvtSdtReceived     (const void* pEvt);
FSTATIC void    MAIN_HandleEvtNitReceived     (const void* pEvt);

#ifdef APPL_DISEQC12_ENABLED
FSTATIC void    MAIN_HandleEvtMotor           (fbool_t moving);
FSTATIC void    MAIN_MotorMsgHide             (void);
#endif

#ifdef APPL_PVR_ENABLED

FSTATIC void    MAIN_HandleEvtPvrPlay         (const void* pEvt);
FSTATIC void    MAIN_HandleEvtPvrRec          (const void* pEvt);
FSTATIC void    MAIN_HandleEvtPlayEvt         (const void* pEvt);
FSTATIC void    MAIN_HandleEvtRecEvt          (const void* pEvt);

#endif /* APPL_PVR_ENABLED */

#ifdef APPL_USB_FUNC_MSC_ENABLED

FSTATIC void    MAIN_HandleEventUsbConnectStat(const void* pEvt);

#endif /* APPL_USB_FUNC_MSC_ENABLED */


#ifdef APPL_RECTIMER_ENABLED

FSTATIC void    MAIN_HandleEvtRectimerStatus  (const void* pEvt);
//FSTATIC void MAIN_HandleEvtSleepTimer  (const void* pEvt);

#ifdef APPL_PVR_ENABLED

FSTATIC int32_t MAIN_AttemptRecordStart (const RECTIMER_ITEM_S* pRectimer,
                                         RECTIMER_IDX idx);

#endif /* APPL_PVR_ENABLED */


#endif /* APPL_RECTIMER_ENABLED */

#ifdef APPL_HDCP_ENABLED
FSTATIC void MAIN_HandleEvtHDMIAuthenticate (SYS_EVTTYPE_E type,
                                                const void* data);
#endif /* APPL_HDCP_ENABLED */

FSTATIC fbool_t MENU_StandbyLeave       (FAPI_SYS_HandleT h, fbool_t force);

FSTATIC fbool_t MENU_StartupEntry       (FAPI_SYS_HandleT h, fbool_t test);
FSTATIC fbool_t MENU_StartupLeave       (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC int32_t MENU_StartupSetFocus    (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_StartupTimerExpired(void* arg);

FSTATIC fbool_t MENU_RootLeave          (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC int32_t MENU_RootSetFocus       (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC fbool_t MENU_RootKeys           (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_RootKeyUp          (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_RootKeyDown        (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_RootKeyLeft        (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_RootKeyRight       (FAPI_SYS_HandleT h);

FSTATIC void    MAIN_TvStandardOutputUpdate (SYS_HDTV_STANDARD newHdSystem,
                                           SYS_SDTV_STANDARD newSdSystem);

FSTATIC void    MAIN_AddStaticKeys    (void);
FSTATIC void    MAIN_RemoveStaticKeys (void);

#if (defined APPL_MHEG_ENABLED) || (defined APPL_TTXT_ENABLED)
FSTATIC fbool_t MAIN_GotoText (FAPI_SYS_HandleT h);
#endif

#if (defined APPL_MHEG_ENABLED) && (defined APPL_TTXT_ENABLED)
FSTATIC void    MAIN_GotoMheg (void* arg);
FSTATIC void    MAIN_GotoTtxt (void* arg);
#endif

#ifdef APPL_FESAT_ENABLED

FSTATIC void    MAIN_GenerateSatCarrierList (const SYS_NETWORK_S* network);

#endif /* APPL_FESAT_ENABLED */

#ifdef APPL_FETERR_ENABLED

FSTATIC void    MAIN_AddVHFChannels     (TPLIST_ITEM_S* pItem,
                                         TERR_CHAN_VHF* pList,
                                         FE_BANDWIDTH_E bandwidth);
FSTATIC void    MAIN_AddUHFChannels     (TPLIST_ITEM_S* pItem,
                                         uint32_t chanFirst,
                                         uint32_t chanLast,
                                         uint32_t startFrequency,
                                         FE_BANDWIDTH_E bandwidth);
#endif /* APPL_FETERR_ENABLED */

#ifdef APPL_FECABLE_ENABLED

FSTATIC void    MAIN_AddCableChannels   (TPLIST_ITEM_S* pItem,
                                         uint32_t chanFirst,
                                         uint32_t chanLast,
                                         uint32_t startFrequency,
                                         char_t* chanNamePrefix,
                                         FE_BANDWIDTH_E bandwidth);

#endif /* APPL_FECABLE_ENABLED */

#ifdef APPL_PVR_ENABLED

FSTATIC void    MAIN_SetPvrDefaults      (void);

#endif /* APPL_PVR_ENABLED */

#ifdef APPL_HDD_ENABLED

FSTATIC void    MAIN_SetHddDefaults      (void);

#endif /* APPL_HDD_ENABLED */

FSTATIC int32_t MAIN_SetupGuidedMode     (void);
FSTATIC void    MAIN_SetSatDefaults      (void);
FSTATIC void    MAIN_SetCarrierDefaults  (const SYS_NETWORK_S* network);
FSTATIC void    MAIN_SetServListDefaults (SLIST_TYPE listId);
FSTATIC void    MAIN_SetStartupDefaults  (void);
FSTATIC void    MAIN_SetUsalsDefaults    (void);
FSTATIC void    MAIN_SetRfModDefaults    (void);

FSTATIC SYS_PWRMODE_E MAIN_GetStartupOperationMode (void);
FSTATIC int32_t MAIN_Init                (void);
FSTATIC void    MAIN_EvtReceived         (SYS_EVTTYPE_E type, const void* data);
FSTATIC void    MAIN_LastMenuClosed      (void);
FSTATIC void    MAIN_FirstMenuToOpen     (void);

FSTATIC void    MAIN_MenuTimerExpired    (void);

FSTATIC int32_t MAIN_UserDataGetParams   (USERDAT_ID_E id, uint8_t* idx,
                                          uint32_t* mask, uint8_t* offset);

#ifdef APPL_PARENTAL_ENABLED

FSTATIC void    MAIN_SetPctrlDefaults     (void);
FSTATIC void    MAIN_PctrlHandleEvent     (SYS_EVTTYPE_E type,
                                           const void* data);
FSTATIC void    MAIN_PctrlOnPinEnter      (void* arg, const char_t* pin,
                                           uint8_t numChars);

#endif /* APPL_PARENTAL_ENABLED */

#ifdef APPL_SCRNSHOT_ENABLE

FSTATIC fbool_t MAIN_ScreenShot           (FAPI_SYS_HandleT h);
FSTATIC fbool_t MAIN_ScreenShotDevTest    (SYS_DEVICE_E device,
                                           STR_IDX_S* errMsg);

#endif /* APPL_SCRNSHOT_ENABLE */

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
/*!
*******************************************************************************
** \brief   main function.
**
** \return  allways 0.
*******************************************************************************
*/
/*lint -save -e970 */
int main (void)
/*lint -restore */
{
    SYS_CONFIGDAT_S cfgDat;

    FAPI_SYS_PRINT_MSG("Hello Smartgo\n");

#ifdef APPL_ENABLE_SPECIAL_TEST_HOTKEYS
    KEYIN_RegisterSpecialHotkeyCallback(TEST_TriggerCcir18TestPictureOutput);
//    KEYIN_RegisterSpecialHotkeyCallback(TEST_TriggerSmartCardStressTest);
#endif

    memset(&cfgDat, 0, sizeof(SYS_CONFIGDAT_S));

    /* ----- Application-specific system configuration --------------------- */
    //cfgDat.appProfile = (SYS_APP_PROFILE_E)APPL_PROFILE;

    /* --------------------------------------------------------------------- */
    cfgDat.lineBufSize  = MAIN_LINEBUFSIZE;
    cfgDat.asciiBufSize = MAIN_LINEBUFCHARS;

    /* --------------------------------------------------------------------- */

    cfgDat.bootScreen      = bootFrameData;
    cfgDat.bootScreenSize  = sizeof(bootFrameData);
    cfgDat.radioScreen     = radioFrameData;
    cfgDat.radioScreenSize = sizeof(radioFrameData);

    #ifdef PLAY_WELCOME_JINGLE

    cfgDat.welcomeJingle      = welcomeJingleData;
    cfgDat.welcomeJingleSize  = sizeof(welcomeJingleData);
    cfgDat.welcomeJingleDelay = 500;

    #endif /* PLAY_WELCOME_JINGLE */

    /* --------------------------------------------------------------------- */
    switch(cfgDat.appProfile )
    {
        case SYS_APP_PROFILE_UKDTT_T2:
            cfgDat.supportedAudioTypes  = SYS_AUDIO_TYPE_NORMAL;
            cfgDat.supportedAudioTypes |= SYS_AUDIO_TYPE_UNKNOWN;
            cfgDat.avPmtControlled     = FTRUE;
            cfgDat.altDefaultSiCharSet = SYS_CHARSET_TABLE0;
            break;
        case SYS_APP_PROFILE_AUSTRALIA_FREEVIEW:
            cfgDat.supportedAudioTypes  = SYS_AUDIO_TYPE_NORMAL;
            cfgDat.supportedAudioTypes |= SYS_AUDIO_TYPE_UNKNOWN;
            cfgDat.avPmtControlled     = FTRUE;
            cfgDat.altDefaultSiCharSet = SYS_CHARSET_TABLE0;
            break;
        case SYS_APP_PROFILE_UKDTT:
            cfgDat.supportedAudioTypes  = SYS_AUDIO_TYPE_NORMAL;
            cfgDat.supportedAudioTypes |= SYS_AUDIO_TYPE_EFFECTS;
            cfgDat.supportedAudioTypes |= SYS_AUDIO_TYPE_UNKNOWN;
            cfgDat.avPmtControlled      = FTRUE;
            cfgDat.altDefaultSiCharSet  = SYS_CHARSET_TABLE0;
            break;
        default:
            /**
             * \note SYS_AUDIO_TYPE_EFFECTS is used by many broadcasters though
             *       it should not describe a "normal" audio stream.
             *
             * \note SYS_AUDIO_TYPE_HEARING_IMPAIRED is used by some
             *       broadcasters though it should not describe a "normal" audio
             *       stream.
             */
            cfgDat.supportedAudioTypes = SYS_AUDIO_TYPE_ALL;
            cfgDat.avPmtControlled     = FFALSE;
            cfgDat.altDefaultSiCharSet = SYS_CHARSET_ASCII;
            break;
    }

    /* --------------------------------------------------------------------- */

    #ifdef APPL_TTXT_ENABLED

    cfgDat.ttx.layer      = OSD_LAYER_TTX_PIP;
    cfgDat.ttx.clutOffset = OSD_CLUT_DYNAMIC_START;

    #endif /* APPL_TTXT_ENABLED */

    #ifdef APPL_MHEG_ENABLED

    cfgDat.mheg.layer      = OSD_LAYER_MHEG;
    cfgDat.mheg.splashData = NULL;//mhegSplashData;
    cfgDat.mheg.splashSize = 0;//sizeof(mhegSplashData);
    cfgDat.mheg.os.eosPrio[0] = THREAD_PRIO_MHEG_LOW;
    cfgDat.mheg.os.eosPrio[1] = THREAD_PRIO_MHEG_LOWER;
    cfgDat.mheg.os.eosPrio[2] = THREAD_PRIO_MHEG_NORMAL;
    cfgDat.mheg.os.eosPrio[3] = THREAD_PRIO_MHEG_HIGH;
    cfgDat.mheg.os.eosPrio[4] = THREAD_PRIO_MHEG_REALTIME;
    cfgDat.mheg.os.timerBsrPrio  = THREAD_PRIO_MHEG_TIMER_BSR;
    cfgDat.mheg.os.timerBsrStackSz = THREAD_STACKSZ_MHEG_TIMER_BSR;
    (void) UTILS_strncpy((char_t*)cfgDat.mheg.os.timerBsrName,THREAD_NAME_MHEG_TIMER_BSR,15);
    (void) UTILS_strncpy((char_t*)cfgDat.mheg.hostString,(const char_t*)hostString,10);

    #if APPL_MHEG_PROFILE == 1
    cfgDat.mheg.profile = REDKEY_PROFILE_UK_SD;       /* United Kingdom SD profile 1.06 */
    #elif APPL_MHEG_PROFILE == 2
    cfgDat.mheg.profile = REDKEY_PROFILE_UK_HD;       /* United Kingdom HD profile 1.06 */
    #elif APPL_MHEG_PROFILE == 3
    cfgDat.mheg.profile = REDKEY_PROFILE_NZ;          /* New Zealand profile */
    #elif APPL_MHEG_PROFILE == 4
    cfgDat.mheg.profile = REDKEY_PROFILE_HK;          /* Hong Kong profile */
    #elif APPL_MHEG_PROFILE == 5
    cfgDat.mheg.profile = REDKEY_PROFILE_UK_FSAT;     /* United Kingdom Freesat profile */
    #elif APPL_MHEG_PROFILE == 6
    cfgDat.mheg.profile = REDKEY_PROFILE_CIPLUS;      /* CI Plus Only profile */
    #elif APPL_MHEG_PROFILE == 7
    cfgDat.mheg.profile = REDKEY_PROFILE_AUSTRALIA;   /* Australia Profile */
    #elif APPL_MHEG_PROFILE == 8
    cfgDat.mheg.profile = REDKEY_PROFILE_SOUTH_AFRICA; /* South Africa Profile */
    #endif
    //pass the key masks to app_mheg
    cfgDat.mheg.appKeyCallback = MENU_MhegSetKeyMask;

    #endif /* APPL_MHEG_ENABLED */

    #ifdef APPL_OAD_ENABLED

    cfgDat.oad.versionId      = 0x12345678;

    #endif /* APPL_OAD_ENABLED */

    #ifdef APPL_SUBTT_ENABLED

    cfgDat.subtt.layer      = OSD_LAYER_SUBTT;
    cfgDat.subtt.clutOffset = OSD_CLUT_DYNAMIC_START;
    cfgDat.subtt.numColors  = OSD_CLUT_DYNAMIC_SIZE;

    cfgDat.subtt.supportedSubtitleTypes = SYS_SUBTITLE_TYPE_NORMAL;
    if ( ( cfgDat.appProfile == SYS_APP_PROFILE_UKDTT_T2 ) ||
         ( cfgDat.appProfile == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW) )
    {
        cfgDat.subtt.supportedSubtitleTypes |= SYS_SUBTITLE_TYPE_HEARINGIMPAIRED;
    }

    #endif /* APPL_SUBTT_ENABLED */

    #ifdef APPL_PIP_ENABLED

    cfgDat.pip.layer        = OSD_LAYER_TTX_PIP;

    #endif /* APPL_PIP_ENABLED */

    #ifdef APPL_JPEG_ENABLED

    cfgDat.jpeg.layer        = OSD_LAYER_JPEG;

    #endif /* APPL_JPEG_ENABLED */

    #ifdef APPL_CI_ENABLED

    {
        uint32_t cnt = 0;

        cfgDat.ci.opLst[cnt++] = RSRC_OP_DESCRREC;
        cfgDat.ci.opLst[cnt++] = RSRC_OP_REC1;
        cfgDat.ci.opLst[cnt++] = RSRC_OP_REC2;
        cfgDat.ci.opLst[cnt++] = RSRC_OP_LIVE;
        cfgDat.ci.opLst[cnt++] = RSRC_OP_PLAY;

        DBG_Assert(cnt <= RSRC_OP_NUMS);
    }

    #endif /* APPL_CI_ENABLED */

    /* --------------------------------------------------------------------- */

    /* set callback functions for application thread and setting of default
       data */
    cfgDat.setStartupDefaults         = MAIN_SetStartupDefaults;
    cfgDat.setRfModDefaults           = MAIN_SetRfModDefaults;
    cfgDat.setUsalsDefaults           = MAIN_SetUsalsDefaults;
    cfgDat.setSatDefaults             = MAIN_SetSatDefaults;
    cfgDat.setCarrierDefaults         = MAIN_SetCarrierDefaults;
    cfgDat.setServListDefaults        = MAIN_SetServListDefaults;
    cfgDat.appInit                    = MAIN_Init;
    cfgDat.appEventReceived           = MAIN_EvtReceived;
    cfgDat.appGetStartupOperationMode = MAIN_GetStartupOperationMode;

    #ifdef APPL_HDD_ENABLED
    cfgDat.setHddDefaults          = MAIN_SetHddDefaults;
    #endif /* APPL_HDD_ENABLED */

    #ifdef APPL_PVR_ENABLED
    cfgDat.setPvrDefaults          = MAIN_SetPvrDefaults;
    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_PARENTAL_ENABLED
    cfgDat.setParentalCtrlDefaults = MAIN_SetPctrlDefaults;
    #endif /* APPL_PARENTAL_ENABLED */

    /* pass control to system */
    (void)SYS_Start(&cfgDat);
    return 0;
}

FSTATIC SYS_PWRMODE_E MAIN_GetStartupOperationMode (void)
{
    SYS_PWRMODE_E powerUpMode;

    if ( SETTINGS_GetPowerUpMode(&powerUpMode) != FAPI_OK )
    {
        powerUpMode = SYS_PWRMODE_DOWN;
    }

    return powerUpMode;
}

FSTATIC int32_t MAIN_Init (void)
{
    int32_t             retVal = FAPI_OK;
    MENUSTACK_INIT_S menustackInitPars;
    SLIST_PARAMS_S   slistParams;

    for(;;)
    {
        appDat = (APP_DATA_S*)UTILS_Malloc(sizeof(APP_DATA_S));

        if ( appDat == NULL )
        {
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
            break;
        }

        /* ----------------------------------------------------------------- */
        #if defined(APPL_MOSAIC_ENABLED) || defined(APPL_SCRNSHOT_ENABLE)

        /* init VOC for mosaic and screen capture*/
        retVal = VOC_Init();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_MOSAIC_ENABLED || APPL_SCRNSHOT_ENABLE */

        /* ----- init global application data ------------------------------ */

        memset(appDat, 0, sizeof(APP_DATA_S));

        appDat->sysCfg                 = SYS_GetConfig();
        appDat->hwCfg                  = SYS_GetHWConfig();

        appDat->osdUpdateCb            = NULL;
        appDat->menuTimerHandler       = NULL;
        appDat->menuTimerArg           = NULL;
        appDat->startedByTimer         = FFALSE;

        retVal = SETTINGS_OutputTvStandardGet(&(appDat->currentHdOutSys),
                                              &(appDat->currentSdOutSys));
        if ( retVal != FAPI_OK ) break;

        appDat->txtAttr.colFocus       = OSD_COL_TEXT_HIGHLIGHTED;
        appDat->txtAttr.colNonFocus    = OSD_COL_TEXT;
        appDat->txtAttr.colDisabled    = OSD_COL_TEXT_DISABLED;

        appDat->fcDataDirty            = FFALSE;

        appDat->asciiBuf               = STR_IntASCIIBuf();

        #ifdef APPL_FESAT_ENABLED

        /* TODO: search preferred default satellite by ID in list, get index
                 and assign it to currSatIdx! */
        appDat->currSatIdx             = 0;

        #endif /* APPL_FESAT_ENABLED */

        /* ----------------------------------------------------------------- */

        #ifdef APPL_PVR_ENABLED

        /* get default PVR status */
        (void)PVRSYS_GetStatus(&(appDat->pvrStatus));

        #endif /* APPL_PVR_ENABLED */

        #ifdef APPL_RECTIMER_ENABLED

        appDat->currActTimer = RECTIMER_INVALID;

        #endif /* APPL_RECTIMER_ENABLED */

        appDat->hdmiConnected = FFALSE;
        appDat->hdcpState     = 0;

        /* ----------------------------------------------------------------- */

        memset(&menustackInitPars, 0, sizeof(menustackInitPars));

        menustackInitPars.lastMenuClosed  = MAIN_LastMenuClosed;
        menustackInitPars.firstMenuToOpen = MAIN_FirstMenuToOpen;
        menustackInitPars.fallbackMenu    = MENU_RootEntry;
        menustackInitPars.fallbackMenuArg = NULL;

        retVal = MENUSTACK_Init(&menustackInitPars);
        if ( retVal != FAPI_OK ) break;

        retVal = OSD_LANG_Init();          /* init language module */
        if ( retVal != FAPI_OK ) break;

        retVal = OSD_Init();               /* init OSD module */
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_Init();              /* init OSD menus and items */
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_PIP_ENABLED

        retVal = APP_PIP_Init ();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_PIP_ENABLED */

        (void)KEYIN_SetRepeatInterval (200, 100);
        if( (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT) ||
            (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2) )
        {
            slistParams.sortByLcn          = FTRUE;
            slistParams.variantRangeStart  = 800;
            slistParams.variantRangeLength = 100;
            slistParams.useLcnVisibility   = FFALSE;
        }
        else if(SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW)
        {
            slistParams.sortByLcn          = FTRUE;
            slistParams.variantRangeStart  = 300;
            slistParams.variantRangeLength = 100;
            slistParams.useLcnVisibility   = FFALSE;
        }
        else
        {
            slistParams.sortByLcn          = SLIST_DEFAULT_LCN_STATUS;
            slistParams.variantRangeStart  = SLIST_DEFAULT_LCN_VARIANT_START;
            slistParams.variantRangeLength = SLIST_DEFAULT_LCN_VARIANT_LENGTH;
            slistParams.useLcnVisibility   = SLIST_DEFAULT_LCN_USEVISIBILITY;
        }

        (void)SLIST_SetParams (&slistParams);
        break; //end of for(;;)
    }


    switch ( retVal )
    {
        case APPL_SMARTGO_ERR_OUT_OF_MEMORY:
            FAPI_SYS_PRINT_DEBUG(1,"Application init failed: OUT OF MEMORY\n");
            break;

        case FAPI_OK:
            /* start according to initial power mode */
            switch ( SYS_GetOperationMode() )
            {
                case SYS_PWRMODE_UP:
                    (void)MAIN_PowerUp (NULL);
                    break;

                case SYS_PWRMODE_DOWN:
                    (void)MAIN_PowerDown (NULL);
                    break;

                default:
                    break;
            }
            break;

        default:
            FAPI_SYS_PRINT_DEBUG(1,"Application init failed: %ld\n", retVal);
            break;
    }

    return retVal;
}

FSTATIC void MAIN_EvtReceived (SYS_EVTTYPE_E type, const void* data)
{
    MENU_TIME_OPEN_S    timeMenuParams;

    switch ( type )
    {
        case EVT_KEY:
            MAIN_HandleEvtKey (data);
            break;

        case EVT_SERVICE_SELECTED:
            MAIN_HandleEvtServiceSelected (data);
            break;

        #ifdef APPL_CA_ENABLED
        case EVT_SERVICE_NO_DECODING:
            CONAX_CA_NoDecode();
            break;
        #endif /* APPL_CA_ENABLED */

        case EVT_SERVICE_RUNSTAT:
            MAIN_HandleEvtRunstat (data);
            break;

        case EVT_EIT_UPDATE:
            MENU_InfoEitUpdate(((SYS_EVT_EITEVT_S*)data)->servIdx);
            break;

        case EVT_EIT_SCHED_UPDATE:
            MENU_InfoUpdateSched (((SYS_EVT_EITEVT_S*)data)->servIdx);
            MENU_EpgGridUpdate   (((SYS_EVT_EITEVT_S*)data)->servIdx);
            break;

        case EVT_AUDIOSTREAM_CHANGED:
            MENU_InfoUpdateAudioLanguage();
            break;

        case EVT_SCAN_SERVICE_ADDED:
            MENU_ScanningServAdded((SYS_EVT_SERVICE_S*)data);
            break;

        case EVT_SCAN_CARRIER_ADDED:
            MENU_ScanningCarrierAdded((SYS_EVT_CARRIERADD_S*)data);
            break;

        case EVT_SCAN_PROGRESS:
            MENU_ScanningProgress((SYS_EVT_SCAN_S*)data);
            break;

        case EVT_TIME_UPDATE:
            MAIN_HandleEvtTimeUpdate(data);

            #ifdef APPL_JPEG_ENABLED
            MENU_JPEGHandleTime();
            #endif /* APPL_JPEG_ENABLED */

            #ifdef APPL_MP3_ENABLED
            MENU_MP3HandleTime();
            #endif /* APPL_MP3_ENABLED */

            #ifdef APPL_MOSAIC_ENABLED
            MENU_MosaicHandleTime();
            #endif /* APPL_MOSAIC_ENABLED */

            #ifdef APPL_FILEOP_ENABLED
            MENU_MediaHandleTime();
            #endif /* APPL_FILEOP_ENABLED */
            break;

        case EVT_TIMESYNC_TIMEOUT:
                timeMenuParams.strIdx = STR_TIME_MSG_TIMER_REC;
                timeMenuParams.done   = FFALSE;
                timeMenuParams.next   = MENU_TIME_NEXT_START_TIMER_REC;

                (void)MENU_TimeHandleEvt(&timeMenuParams);
            break;

        case EVT_TIMER_EXPIRED:
            MAIN_HandleEvtTimerExpired (data);
            break;

        case EVT_OUT_SYS_CHANGE:
            MAIN_TvStandardOutputUpdate (((SYS_EVT_TVSYS_S*)data)->hdTvSys,
                                         ((SYS_EVT_TVSYS_S*)data)->sdTvSys);
            break;

        case EVT_PIC_FORMAT_CHANGE:
            MENU_InfoUpdatePicFormat(((SYS_EVT_PICFORMAT_S*)data)->format);
            break;

        #ifdef APPL_PVR_ENABLED

        case EVT_PVR_PLAYBACK:
            MAIN_HandleEvtPvrPlay(data);
            break;

        case EVT_PVR_RECORD:
            MAIN_HandleEvtPvrRec(data);
            break;

        case EVT_PVR_PLAYEVENT:
            MAIN_HandleEvtPlayEvt(data);
            break;

        case EVT_PVR_RECEVENT:
            MAIN_HandleEvtRecEvt(data);
            break;

        #endif /* APPL_PVR_ENABLED */

        case EVT_LOCK_STATUS:
            MAIN_HandleEvtLockStatus(data);
            break;

        #ifdef APPL_RECTIMER_ENABLED

        case EVT_RECTIMER_LISTCHG:
            /* update timer menu if currently open */
            MENU_TimerProgUpdate();
            break;

        case EVT_RECTIMER_STATUS:
            MAIN_HandleEvtRectimerStatus (data);
            break;

        case EVT_SLEEPTIMER_EXPIRED:
            // TODO: MAIN_HandleEvtSleepTimer (data);
            break;

        #endif /* APPL_RECTIMER_ENABLED */

        case EVT_PMT_RECEIVED:
            MAIN_HandleEvtPmtReceived(data);
            break;

        case EVT_SDT_RECEIVED:
            MAIN_HandleEvtSdtReceived(data);
            break;

        case EVT_NIT_RECEIVED:
            MAIN_HandleEvtNitReceived(data);
            break;

        #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

        case EVT_CISYS_STATUSCHG:
            MENU_CiSlotStatusUpdate  ((const CISYS_EVT_STATUS_S*)data);
            MENU_MmiSlotStatusUpdate ((const CISYS_EVT_STATUS_S*)data);
            break;

        case EVT_CISYS_MMI:
            MENU_MmiMmiStatusUpdate ((const CISYS_EVT_MMI_S*)data);
            break;

        #endif /* APPL_CI/CA_ENABLED */

        #ifdef APPL_UPDATE_ENABLED

        case EVT_UPDATE_STATUS:
            MENU_UpdateProgress((const SYS_EVT_UPDATE_S*)data);
            break;

        #endif /* APPL_UPDATE_ENABLED */

        case EVT_DEVICE_STATUS:
            MAIN_HandleEvtDeviceStatus(data);
            break;

        // TODO: this workaround is to be replaced!
        case APP_EVT_MENU_REFRESH :
            ((const MENU_EVT_REFRESH_S*)data)->cb();
            break;

        #ifdef APPL_MHEG_ENABLED

        case EVT_MHEG_STATUS:
            (void) MENU_MhegStateChange(((MHEG_EVT_STATE_S*)data)->state);
            break;

        case APP_EVT_MHEG_START :
            if( (MENUSTACK_GetLevel() == 0) &&
                (MHEG_IsStarted() == FTRUE) )
            {
                (void)MENUSTACK_Switch (MENU_MhegEntry,NULL,0);
            }
            break;

        #endif /* APPL_MHEG_ENABLED */

        #ifdef APPL_FILEOP_ENABLED

        case EVT_FILEOP_STATUS:
            MAIN_ViewerUpdateProgress(&((FILEOP_EVT_S*)data)->data);
            break;

        #endif /* APPL_FILEOP_ENABLED */

        #ifdef APPL_JPEG_ENABLED

        case EVT_JPEG_FILEINFO:
            #ifdef APPL_FILEOP_ENABLED

            MAIN_ViewerHandleEvent(type, data);
            MENU_MediaHandleEvent (type, data);

            #endif /* APPL_FILEOP_ENABLED */

            MENU_JPEGHandleEvent(type, data);
            break;

        case  EVT_JPEG_OSDINFO:
            #ifdef APPL_FILEOP_ENABLED

            MAIN_ViewerHandleEvent(type, data);

            #endif /* APPL_FILEOP_ENABLED */
            break;

        case EVT_JPEG_DECODEINFO:
            #ifdef APPL_FILEOP_ENABLED

            MAIN_ViewerHandleEvent(type, data);

            #endif /* APPL_FILEOP_ENABLED */

            MENU_JPEGHandleEvent(type, data);
            break;

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        case EVT_MP3_STATUS:
            MENU_MP3HandleEvent ((MP3_EVT_STATUS_S*)data);

            #ifdef APPL_FILEOP_ENABLED
            MENU_MediaHandleEvent (type, data);
            #endif /* APPL_FILEOP_ENABLED */
            break;

        #endif /* APPL_MP3_ENABLED */

        #ifdef APPL_PARENTAL_ENABLED

        case EVT_PCTRL_NEEDAUTH:
        case EVT_PCTRL_NOMOREAUTH:
            MAIN_PctrlHandleEvent(type, data);
            break;

        #endif /* APPL_PARENTAL_ENABLED */

        #ifdef APPL_USB_FUNC_MSC_ENABLED

        case EVT_USB_CONNECT_STATUS:
            MAIN_HandleEventUsbConnectStat(data);
            break;

        #endif /* APPL_USB_FUNC_MSC_ENABLED */

        #ifdef APPL_DISEQC12_ENABLED

        case EVT_MOTOR_MOVES:
            MAIN_HandleEvtMotor(((SYS_EVT_MOTOR_S*)data)->moving);
            break;

        #endif

        #ifdef APPL_HDCP_ENABLED

        case EVT_HDMI_HDCP_STATE:
            MAIN_HandleEvtHDMIAuthenticate(type, data);
            break;

        #endif /* APPL_HDCP_ENABLED */

        /* ignored events */
        case EVT_IN_SYS_CHANGE:
        case EVT_SPDIFSTREAM_CHANGED:
        case EVT_HDMI_CONNECTION:
            break;

        #ifdef APPL_ETHERNET_ENABLED

        case EVT_ETH_LINK:
            MENU_NetworkIpUpdate();
            MENU_NetworkDnsUpdate();
            MENU_NetworkInfoUpdate();
            break;

        case EVT_ETH_DHCP:
            MENU_NetworkIpUpdate();
            MENU_NetworkDnsUpdate();
            MENU_NetworkInfoUpdate();
            break;

        #endif
        default:
            FAPI_SYS_PRINT_DEBUG(2,"unknown event: %i, discard!\n",
                    ((SYS_EVTHDR_S*)data)->type);
            break;
    }
}

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void MAIN_HandleEvtKey (const void* pEvt)
{
    const KEYIN_EVT_KEY_S* pKeyEvt = pEvt;
    FGS_KEY_E              keyCode = FGS_KEY_NONE;
    FGS_KEYTYPE_E          keyType = FGS_KEYTYPE_PRESSED;
    int8_t                 keyVal = -1;

    FAPI_SYS_PRINT_DEBUG(2,"[RCU] key: %i\n", pKeyEvt->val);

    switch ( pKeyEvt->type )
    {
        case KEYIN_KEY_PRESSED:
            keyType = FGS_KEYTYPE_PRESSED;
            break;

        case KEYIN_KEY_REPEATED:
            keyType = FGS_KEYTYPE_REPEATED;
            break;

        case KEYIN_KEY_RELEASED:
            /* ignore "key-released" events */
            return;
    }
    /* clear shutdown marker in case system has been started by
       timer */
    appDat->startedByTimer = FFALSE;

    /* ----- translate key to fgs world ------------------------------------ */

    switch ( pKeyEvt->val )
    {
        /* ----- known and managed keys -------------------------------------- */

        case FAPEX_IR_KEY_0:
        case FAPEX_IR_KEY_1:
        case FAPEX_IR_KEY_2:
        case FAPEX_IR_KEY_3:
        case FAPEX_IR_KEY_4:
        case FAPEX_IR_KEY_5:
        case FAPEX_IR_KEY_6:
        case FAPEX_IR_KEY_7:
        case FAPEX_IR_KEY_8:
        case FAPEX_IR_KEY_9:            keyCode  = FGS_KEY_NUMBER;
                                       keyVal   = (int8_t)(pKeyEvt->val - FAPEX_IR_KEY_0);
                                                                          break;
        case FAPEX_IR_KEY_RIGHT:        keyCode  = FGS_KEY_RIGHT;          break;
        case FAPEX_IR_KEY_LEFT:         keyCode  = FGS_KEY_LEFT;           break;
        case FAPEX_IR_KEY_UP:           keyCode  = FGS_KEY_UP;             break;
        case FAPEX_IR_KEY_DOWN:         keyCode  = FGS_KEY_DOWN;           break;
        case FAPEX_IR_KEY_OK:           keyCode  = FGS_KEY_OK;             break;
        case FAPEX_IR_KEY_APPL:         keyCode  = FGS_KEY_APPL;           break;
        case FAPEX_IR_KEY_MENU:         keyCode  = FGS_KEY_MENU;           break;
        case FAPEX_IR_KEY_NAVI:         keyCode  = FGS_KEY_NAVI;           break;
        case FAPEX_IR_KEY_INFO:         keyCode  = FGS_KEY_INFO;           break;
        case FAPEX_IR_KEY_HIDE:         keyCode  = FGS_KEY_HIDE;           break;
        case FAPEX_IR_KEY_EXIT:         keyCode  = FGS_KEY_EXIT;           break;
        case FAPEX_IR_KEY_RED:          keyCode  = FGS_KEY_RED;            break;
        case FAPEX_IR_KEY_GREEN:        keyCode  = FGS_KEY_GREEN;          break;
        case FAPEX_IR_KEY_YELLOW:       keyCode  = FGS_KEY_YELLOW;         break;
        case FAPEX_IR_KEY_BLUE:         keyCode  = FGS_KEY_BLUE;           break;
        case FAPEX_IR_KEY_STANDBY:      keyCode  = FGS_KEY_STANDBY;        break;
        case FAPEX_IR_KEY_CHANNEL_UP:   keyCode  = FGS_KEY_CHAN_UP;        break;
        case FAPEX_IR_KEY_CHANNEL_DOWN: keyCode  = FGS_KEY_CHAN_DOWN;      break;
        case FAPEX_IR_KEY_VOL_UP:       keyCode  = FGS_KEY_VOL_UP;         break;
        case FAPEX_IR_KEY_VOL_DOWN:     keyCode  = FGS_KEY_VOL_DOWN;       break;
        case FAPEX_IR_KEY_MUTE:         keyCode  = FGS_KEY_MUTE;           break;
        case FAPEX_IR_KEY_BACK:         keyCode  = FGS_KEY_BACK;           break;
        case FAPEX_IR_KEY_TEXT:         keyCode  = FGS_KEY_TEXT;           break;
        case FAPEX_IR_KEY_REC:          keyCode  = FGS_KEY_RECORD;         break;
        case FAPEX_IR_KEY_PLAY:         keyCode  = FGS_KEY_PLAY;           break;
        case FAPEX_IR_KEY_FORWARD:      keyCode  = FGS_KEY_FF;             break;
        case FAPEX_IR_KEY_REWIND:       keyCode  = FGS_KEY_REW;            break;
        case FAPEX_IR_KEY_FORWSLOW:     keyCode  = FGS_KEY_REWSLOW;        break;
        case FAPEX_IR_KEY_REWSLOW:      keyCode  = FGS_KEY_FFSLOW;         break;
        case FAPEX_IR_KEY_PAUSE:        keyCode  = FGS_KEY_PAUSE;          break;
        case FAPEX_IR_KEY_STOP:         keyCode  = FGS_KEY_STOP;           break;
        case FAPEX_IR_KEY_TV:           keyCode  = FGS_KEY_TV;             break;
        case FAPEX_IR_KEY_F1:           keyCode  = FGS_KEY_F1;             break;
        case FAPEX_IR_KEY_F2:           keyCode  = FGS_KEY_F2;             break;
        case FAPEX_IR_KEY_F3:           keyCode  = FGS_KEY_F3;             break;
        case FAPEX_IR_KEY_F4:           keyCode  = FGS_KEY_F4;             break;

        /* ----- known but not managed keys ---------------------------------- */

        case FAPEX_IR_KEY_VCR:
            //FM_Printf("[RCU] key ignored\n");
            break;

        /* ----- unknown keys ------------------------------------------------ */

        default:
            break;
    }

    if ( keyCode != FGS_KEY_NONE )
    {
        if ( FGS_DispatchKey(keyCode, keyType, keyVal) == FFALSE )
        {
            (void)MAIN_ProhibitedShow(&(appDat->prohib));
        }
    }
}

FSTATIC void MAIN_HandleEvtServiceSelected (const void* pEvt)
{
    const SYS_EVT_SERVICE_S* pEvtServ = pEvt;
    MENU_INFOBAR_PARAMS_S    infoParams;

    if ( SYS_GetOperationMode() == SYS_PWRMODE_DOWN ) return;

    switch ( pEvtServ->status )
    {
        case SERVICE_STAT_START:
            FAPI_SYS_PRINT_DEBUG(2,"SERVICE SELECT: Start\n");

            #ifdef APPL_PARENTAL_ENABLED

            /* close potentially opened PIN dialogue */
            (void)MENU_PinDialogClose(&(appDat->pinDialog));

            #endif /* APPL_PARENTAL_ENABLED */

            if ( ( (pEvtServ->ctrlInfo & SERVICE_CTRL_QUIET)        != SERVICE_CTRL_QUIET )  &&
                 ( ( (pEvtServ->ctrlInfo & SERVICE_CTRL_KEEP_DISPLAY) == SERVICE_CTRL_KEEP_DISPLAY ) ||
                   ( (pEvtServ->ctrlInfo & SERVICE_CTRL_DISPLAY_BANNER) == SERVICE_CTRL_DISPLAY_BANNER ) ) )
            {
                MENU_InfoUpdateService();
                MENU_ServSelectServiceUpdate(&(pEvtServ->servLoc));
            }

            /* show bar only if no menu is selected */
            if (( MENUSTACK_GetLevel() == 0 )  &&
                 ( (pEvtServ->ctrlInfo & SERVICE_CTRL_DISPLAY_BANNER) ==
                    SERVICE_CTRL_DISPLAY_BANNER ) )
            {
                infoParams.permanent   = FFALSE;
                infoParams.numberInput = -1;

                (void)MENUSTACK_Switch (MENU_InfobarEntry,
                                        (FAPI_SYS_HandleT)(&infoParams),
                                        1);
            }
            break;

        case SERVICE_STAT_SUCCESS:
            MENU_InfoUpdateAudioLanguage();

            /* show bar only if no menu is selected */
            if ( ( MENUSTACK_GetLevel() == 0 ) &&
                 ( (pEvtServ->ctrlInfo & SERVICE_CTRL_DISPLAY_BANNER) ==
                    SERVICE_CTRL_DISPLAY_BANNER ) )
            {
                infoParams.permanent   = FFALSE;
                infoParams.numberInput = -1;

                (void)MENUSTACK_Switch (MENU_InfobarEntry, (FAPI_SYS_HandleT)(&infoParams),
                                  1);
            }

            FAPI_SYS_PRINT_DEBUG(2,"SERVICE SELECT: Success\n");

            MAIN_FreqDataHasChanged();
            break;

        case SERVICE_STAT_STOPPED:
            // NOTE: Removed, this causes problems when service list is empty
            //MENU_InfoUpdateService();
            break;

        default:
            /* do nothing */
            break;
    }
    #ifdef APPL_MOSAIC_ENABLED

    /* pass to mosaic preview menu if currently open */
    MENU_MosaicProgress(pEvtServ);

    #endif /* APPL_MOSAIC_ENABLED */

    MENU_StrSelServSelEvent(pEvtServ);
}

FSTATIC void MAIN_HandleEvtRunstat (const void* pEvt)
{
    const SYS_EVT_RUNSTAT_S* pEvtRunstat = pEvt;

    #ifdef APPL_MHEG_ENABLED
    AV_STATUS_LIVE_S      avStatus;

    /* get current service running status */
    (void) AV_GetLiveStatus (&avStatus);

    switch (avStatus.runningStatus)
    {
        default:
        case SYS_RUNSTAT_UNKNOWN:
        case SYS_RUNSTAT_RUNNING:
            if( ( MHEG_IsOpened() == FTRUE ) &&
                ( MHEG_IsStarted() == FFALSE) )
            {
                FAPI_SYS_PRINT_DEBUG(2,"[MHEG] Started by Service Running status change\n");
                (void) MHEG_Start();
            }
            break;
        case SYS_RUNSTAT_NOTRUNNING:
            if( ( MHEG_IsOpened() == FTRUE ) &&
                ( MHEG_IsStarted() == FTRUE) )
            {
                (void) MHEG_Stop();
            }
            break;
    }

    #endif /* APPL_MHEG_ENABLED */

    if ( pEvtRunstat->status == SYS_RUNSTAT_NOTRUNNING )
    {
        // TODO be improved!
        MENU_CommonMsgShow(STR_MSG_SERVICE_NOT_RUNNING, FFALSE);
    }
}

FSTATIC void MAIN_HandleEvtLockStatus (const void* pEvt)
{
    const SYS_EVT_LOCKSTAT_S* pEvtLock = pEvt;

    if ( pEvtLock->feNum >= appDat->hwCfg->feNums ) return;

    /* inform potentially interested modules */
    MENU_InfoUpdateLockStatus  (pEvtLock->feNum, pEvtLock->status);
    MENU_ScanUpdateLockStatus  (pEvtLock->feNum, pEvtLock->status);
    MENU_TunerUpdateLockStatus (pEvtLock->feNum, pEvtLock->status);
}

FSTATIC void MAIN_HandleEvtTimeUpdate (const void* pEvt)
{
    const SYS_EVT_TIME_S* pEvtTime = pEvt;
    fbool_t               dateChange = FFALSE;
    fbool_t               dispChange = FFALSE;
    int16_t               utcOffset;
    fbool_t               dstIsActive;

    /* test for EIT cleanup condition */
    if ( appDat->timeCurr.hour != pEvtTime->time.hour )
    {
        (void)SETTINGS_TimeOffsetGet(&utcOffset, &dstIsActive);

        if ( dstIsActive ) utcOffset += 60;

        utcOffset /= 60; /* convert to hours */

        if ( (pEvtTime->time.hour == ((24 - utcOffset) % 24)) )
        {
            dateChange = FTRUE;
        }
    }

    /* test if update of time display is necessary */
    if ( (appDat->timeCurr.hour   != pEvtTime->time.hour) ||
         (appDat->timeCurr.minute != pEvtTime->time.minute) )
    {
        dispChange = FTRUE;
    }

    /* assign current values */
    appDat->timeCurr = pEvtTime->time;

    if ( dispChange == FTRUE )
    {
        /* display is only updated every minute */
        MENU_InfoTimeBoxUpdate();
        MENU_TimeUpdate();
    }

    /* At change of date, all outdated EIT schedule database entries are
       cleared. */

    if ( dateChange == FTRUE )
    {
        FAPI_SYS_PRINT_DEBUG(1,"### START CLEANUP AT DATE CHANGE (%02i:%02i)###\n",
        pEvtTime->time.hour,
        pEvtTime->time.minute);

        EIT_CleanupSched();
    }

    /* Test if saving of frequently-changing data is necessary */
    if ( appDat->fcDataDirty == FTRUE )
    {
        if ( appDat->fcDataIdleCnt >= MAIN_FCDATA_STORE_DELAY )
        {
            (void)SETTINGS_StoreFreqChgData();
            appDat->fcDataDirty = FFALSE;
        }
        else
        {
            appDat->fcDataIdleCnt++;
        }
    }

    MAIN_VolumeBarHandleExpiration (&(appDat->volBar));
    MAIN_MsgBoxHandleExpiration    (&(appDat->msgBox));
}

FSTATIC void MAIN_HandleEvtTimerExpired (const void* pEvt)
{
    const SYS_EVT_TIMER_S* pEvtTimer = pEvt;

    switch ( pEvtTimer->type )
    {
        case TIMER_SINGLE:
            switch ( pEvtTimer->num )
            {
                case OSD_TIMER_MENU:
                    MAIN_MenuTimerExpired();
                    break;

                case OSD_TIMER_ICON:
                    (void)MAIN_ProhibitedHide(&(appDat->prohib));
                    break;

                case OSD_TIMER_SERVSELECT:
                    MENU_InfoServSelectTimerExpired();
                    break;

                #ifdef APPL_DISEQC12_ENABLED
                case OSD_TIMER_MOTOR_MSG:
                    MAIN_MotorMsgHide();
                    break;
                #endif

                default:
                    break;
            }
            break;

        case TIMER_PERIODIC:
            switch ( pEvtTimer->num )
            {
                case OSD_PERIODTIMER_SIGNAL:
                    (void)MAIN_SignalBoxUpdate(&(appDat->sigBox));
                    break;

                case OSD_PERIODTIMER_GENERAL:
                    if ( appDat->osdUpdateCb != NULL ) appDat->osdUpdateCb();
                    break;

                default:
                    break;
            }
            break;

        default:
            break;
    }
}

FSTATIC void MAIN_HandleEvtDeviceStatus (const void* pEvt)
{
    #ifdef APPL_PVR_ENABLED

    const SYS_EVT_DEVSTAT_S* pEvtDevStat = pEvt;

    /* ----- Synchronize PVR list on connect and disconnect ---------------- */

    if ( pEvtDevStat->status.mountStatus != SYS_DEV_FS_MOUNTED )
    {
        (void)PVRSYS_DeviceRemoved (pEvtDevStat->device);
    }
    else
    {
        switch ( pEvtDevStat->device )
        {
            case SYS_DEV_HDD:
                (void)SYS_DeviceMkdir(SYS_DEV_HDD, MAIN_DIR_HDD_PVR);

                (void)PVRSYS_DeviceSyncDirectory (pEvtDevStat->device,
                                                  MAIN_DIR_HDD_PVR);
                break;

            default:
                (void)PVRSYS_DeviceSyncDirectory (pEvtDevStat->device,
                                                  MAIN_DIR_OTH_PVR);
                break;
        }
    }

    MENU_PlayListUpdate();

    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_FILEOP_ENABLED

    MAIN_ViewerDeviceUpdate();

    #endif /* APPL_FILEOP_ENABLED */

    #if (defined APPL_HDD_ENABLED) || (defined APPL_USB_ENABLED)

    MENU_HddDeviceUpdate ((const SYS_EVT_DEVSTAT_S*)pEvt);

    #endif /* (defined APPL_HDD_ENABLED) || (defined APPL_USB_ENABLED) */
}

FSTATIC void MAIN_HandleEvtPmtReceived (const void* pEvt)
{
    const SYS_EVT_PMT_S* pEvtPmt = pEvt;

    if ( (pEvtPmt->operation & (RSRC_OP_LIVE | RSRC_OP_PLAY)) != 0 )
    {
        MENU_StrSelPmtUpdate();
        MENU_InfoUpdateService();
    }
}

FSTATIC void MAIN_HandleEvtSdtReceived (const void* pEvt)
{
    const SYS_EVT_SDT_S*    pEvtSdt   = pEvt;
    SYS_PRESSTATE_E         presState;

    if ( pEvtSdt->servicesChanged )
    {
        /* SDT update has caused change in the service list. Infobar is
           updated and the service list is stored. */

        if ( (SYS_GetPresentationState (&presState) == FAPI_OK) &&
             (presState == SYS_PRESSTATE_LIVETV) )
        {
            MENU_InfoUpdateService ();
        }

        MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);
        (void)SLIST_Save (FFALSE);

        (void)MAIN_MsgBoxHide(&(appDat->msgBox));
    }
}

FSTATIC void MAIN_HandleEvtNitReceived (const void* pEvt)
{
    const SYS_EVT_NIT_S*    pEvtNIt   = pEvt;
    SYS_PRESSTATE_E         presState;

    if ( pEvtNIt->servicesChanged )
    {
        /* NIT update has caused change in the service list. Infobar is
           updated and the service list is stored. */

        if ( (SYS_GetPresentationState (&presState) == FAPI_OK) &&
             (presState == SYS_PRESSTATE_LIVETV) )
        {
            MENU_InfoUpdateService ();
            (void)MAIN_ServBoxUpdate(&(appDat->servBox));
        }

        MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);
        (void)SLIST_Save (FFALSE);

        (void)MAIN_MsgBoxHide(&(appDat->msgBox));
    }
}

#ifdef APPL_DISEQC12_ENABLED
FSTATIC void MAIN_HandleEvtMotor (fbool_t moving)
{
    if(moving)
    {
        if(!(appDat->motorMsgShown))
        {
            (void)TIME_SingleTimerLaunch(OSD_TIMER_MOTOR_MSG, 100); // 10 second
            MENU_CommonMsgShow(STR_MSG_MOTOR_MOVES,FTRUE);
            appDat->motorMsgShown = FTRUE;
        }
    }
    else
    {
        (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        (void)TIME_SingleTimerAbort(OSD_TIMER_MOTOR_MSG);
        appDat->motorMsgShown = FFALSE;
    }
}

FSTATIC void MAIN_MotorMsgHide(void)
{
        (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        appDat->motorMsgShown = FFALSE;
}
#endif

#ifdef APPL_RECTIMER_ENABLED

FSTATIC void MAIN_HandleEvtRectimerStatus (const void* pEvt)
{
    const SYS_EVT_RECTIMER_STATUS_S* pEvtRec = pEvt;
    RECTIMER_ITEM_S                  recTimer;
    SYS_SERVLOC_S                    servLoc;

    #ifdef APPL_PVR_ENABLED

    int32_t             retVal;
    PVRSYS_STATUS_S     pvrStatus;
    int8_t              cnt;

    if ( PVRSYS_GetStatus (&pvrStatus) != FAPI_OK ) return;

    #endif /* APPL_PVR_ENABLED */

    /* ----- Record timer start -------------------------------------------- */

    switch ( pEvtRec->status )
    {
        case RECTIMER_STATUS_START:

            FAPI_SYS_PRINT_DEBUG(1,"[APP] RECTIMER: START idx %i seconds %i\n",
                       pEvtRec->idx, pEvtRec->seconds);

            #ifdef APPL_PVR_ENABLED

            /* Ensure that given timer is not already being recorded. */
            for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
            {
                if ( (pvrStatus.rec[cnt].status   != REC_STATUS_INACTIVE) &&
                     (pvrStatus.rec[cnt].timerIdx == pEvtRec->idx) )
                {
                    return;
                }
            }

            #endif /* APPL_PVR_ENABLED */

            /* if currently in standby, power up box in any case  */
            if ( SYS_GetOperationMode() == SYS_PWRMODE_DOWN )
            {
                if ( MAIN_PowerUp(NULL) == FFALSE )
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[APP} MAIN_PowerUp() failed!\n");
                    return;
                }

                appDat->startedByTimer = FTRUE;
            }

            if ( RECTIMER_GetByIndex(pEvtRec->idx, &recTimer) != FAPI_OK )
            {
                return;
            }

            /* Timer is due to start: It is assumed that system is active
               by now... */
            if ( pEvtRec->seconds == 0 )
            {
                switch ( recTimer.operation )
                {
                    #ifdef APPL_PVR_ENABLED

                    case RECTIMER_OP_RECORD:
                        (void)MAIN_AttemptRecordStart (&recTimer, pEvtRec->idx);
                        break;

                    #endif /* APPL_PVR_ENABLED */

                    case RECTIMER_OP_SWITCH_SERVICE:
                        if ( appDat->currActTimer != pEvtRec->idx )
                        {
                            #ifdef APPL_PVR_ENABLED
                            /* Stop playback if currently active */
                            (void)PVRSYS_PlayStop();
                            #endif /* APPL_PVR_ENABLED */

                            servLoc = SLIST_GetListReference(recTimer.serviceIndex);

                            /* No harddisk recording: select service requested by timer */
                            (void)MAIN_SelectService (&servLoc, FFALSE);

                            appDat->currActTimer = pEvtRec->idx;
                        }
                        break;

                    default:
                        /* do nothing */
                        break;
                }
            }
            else /* timer does not have to be started yet */
            {
                #ifdef APPL_PVR_ENABLED

                if ( recTimer.operation == RECTIMER_OP_RECORD )
                {
                    retVal = RSRC_ServiceAvlForRecord(recTimer.serviceIndex,
                                                      RSRC_OP_ANY, NULL, NULL);
                    #ifndef APPL_DEBUG_MODE
                    retVal = retVal;
                    #endif
                    FAPI_SYS_PRINT_DEBUG(1,"[APP] TIMER TO BE STARTED: %s\n",
                              (retVal == FAPI_OK) ?
                              "OK" : "NOT POSSIBLE");
                }

                #endif /* APPL_PVR_ENABLED */
            }
            break;

        /* ----- Record timer stop ----------------------------------------- */

        case RECTIMER_STATUS_STOP:

            FAPI_SYS_PRINT_DEBUG(1,"[APP] RECTIMER: STOP idx %i\n", pEvtRec->idx);

            if ( RECTIMER_GetByIndex(pEvtRec->idx, &recTimer) != FAPI_OK )
            {
                break;
            }

            #ifdef APPL_PVR_ENABLED

            if ( (recTimer.operation == RECTIMER_OP_RECORD) ||
                 (recTimer.operation == RECTIMER_OP_DESCRAMBLE_REC) )
            {
                /* Search record with given timer ID */
                for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
                {
                    if ( (pvrStatus.rec[cnt].status   != REC_STATUS_INACTIVE) &&
                         (pvrStatus.rec[cnt].timerIdx == pEvtRec->idx) )
                    {
                        (void)MAIN_PvrStopRecording(cnt);
                        break;
                    }
                }
            }

            #endif /* APPL_PVR_ENABLED */

            if ( appDat->currActTimer == pEvtRec->idx )
            {
                appDat->currActTimer = RECTIMER_INVALID;
            }

            if ( pEvtRec->removeIt )
            {
                (void)RECTIMER_RemoveByIndex (pEvtRec->idx);
            }

            (void)RECTIMER_Save(FFALSE);

            if ( SYS_GetOperationMode() == SYS_PWRMODE_UP )
            {
                if ( appDat->startedByTimer || recTimer.autoStandby )
                {
                    /* shutdown system */
                    (void)MAIN_PowerDown(NULL);
                }
            }
            break;

        /* ----------------------------------------------------------------- */

        case RECTIMER_STATUS_REMOVED:

            FAPI_SYS_PRINT_DEBUG(1,"[APP] RECTIMER: REMOVED: idx %i \n", pEvtRec->idx);

            #ifdef APPL_PVR_ENABLED

            /* Search record with given timer ID */
            for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
            {
                if ( (pvrStatus.rec[cnt].status   != REC_STATUS_INACTIVE) &&
                     (pvrStatus.rec[cnt].timerIdx == pEvtRec->idx) )
                {
                    (void)MAIN_PvrStopRecording(cnt);
                    break;
                }
            }

            #endif /* APPL_PVR_ENABLED */

            if ( appDat->currActTimer == pEvtRec->idx )
            {
                appDat->currActTimer = RECTIMER_INVALID;
            }
            break;

        default:
            break;
    }

    MENU_TimerProgUpdate();
}

#ifdef APPL_PVR_ENABLED

FSTATIC int32_t MAIN_AttemptRecordStart (const RECTIMER_ITEM_S* pRectimer,
                                         RECTIMER_IDX idx)
{
    int32_t             retVal = FAPI_OK;
    SYS_DEV_STAT_S      devStat;
    SYS_PRESSTATE_E     presState;
    SYS_SERVLOC_S       globServLoc;
    SYS_SERVLOC_S       servLoc;
    STR_IDX_S           msg = STR_MSG_RECSTART_FAILED;
    RECTIMER_ITEM_S     recTimer;
    #ifdef APPL_HDD_ENABLED
    SYS_DEV_PWRMODE_E   hddPwrState;
    #endif /* APPL_HDD_ENABLED */

    retVal = SYS_DeviceGetStatus (pRectimer->targetDevice, &devStat);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    if ( (devStat.mountStatus != SYS_DEV_FS_MOUNTED) )
    {
        FAPI_SYS_PRINT_DEBUG(1,"RECORD START FAILED: Device not ready\n");
        return SMARTGO_APPL_ERR_PROHIBITED;
    }

    retVal = RSRC_ServiceAvlForRecord (pRectimer->serviceIndex,
                                       RSRC_OP_ANY, NULL, NULL);

    if ( retVal != FAPI_OK )
    {
        /* Record is not available: attempt to free some resources and try
           again. */
        #ifdef APPL_PIP_ENABLED

        /* Abort PIP presentation */
        PIP_Hide();

        #endif /* APPL_PIP_ENABLED */

        retVal = RSRC_ServiceAvlForRecord (pRectimer->serviceIndex,
                                           RSRC_OP_ANY, NULL, NULL);

        /* Switch live presentation to service to be recorded */
        if ( retVal != FAPI_OK )
        {
            if ( (SYS_GetPresentationState (&presState) == FAPI_OK) &&
                 (presState == SYS_PRESSTATE_LIVETV) )
            {
                /* attempt to find target service in currenty selected list */
                globServLoc = SLIST_GetListReference (pRectimer->serviceIndex);

                if ( AV_GetRequestedService(&servLoc) == FAPI_OK )
                {
                    if ( SLIST_EntryFindInList(&globServLoc, servLoc.listId,
                                               &(servLoc.index)) != FAPI_OK )
                    {
                        servLoc = globServLoc;
                    }
                }
                else
                {
                    servLoc = globServLoc;
                }

                (void)MAIN_SelectService (&servLoc, FFALSE);

                retVal = RSRC_ServiceAvlForRecord (pRectimer->serviceIndex,
                                                   RSRC_OP_ANY, NULL, NULL);
            }
        }
    }

    if ( retVal == FAPI_OK )
    {
        memset(&recTimer, 0, sizeof(recTimer));
        retVal = RECTIMER_GetByIndex(idx, &recTimer);
        if ( retVal != FAPI_OK ) return retVal;

        if ( SYS_DeviceGetFreekBytes(recTimer.targetDevice) == 0 )
        {
            switch ( recTimer.targetDevice )
            {
                #ifdef APPL_HDD_ENABLED

                case SYS_DEV_HDD:
                    msg = STR_MSG_RECSTART_FAILED_HDD_FULL;
                    break;

                #endif /* APPL_HDD_ENABLED */

                #ifdef APPL_USB_ENABLED

                case SYS_DEV_USB:
                    msg = STR_MSG_RECSTART_FAILED_USB_FULL;
                    break;

                #endif /* APPL_USB_ENABLED */

                default:
                    /* do nothing */
                    break;
            }

            (void)RECTIMER_Reinsert(idx);
            FAPI_SYS_PRINT_DEBUG(1,"Rec timer %i reinserted\n",idx);
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
        }
        else
        {
            #ifdef APPL_HDD_ENABLED
            retVal = SYS_DeviceGetPowerMode(SYS_DEV_HDD,&hddPwrState);
            if( (retVal                == FAPI_OK ) &&
                (recTimer.targetDevice == SYS_DEV_HDD) &&
                (hddPwrState           == SYS_DEV_PWRMODE_STANDBY) )
            {
                MENU_CommonMsgShow(STR_MSG_HDD_SPINUP,FTRUE);
            }
            #endif /* APPL_HDD_ENABLED */

            retVal = MAIN_PvrStartTimerRecording(idx);
        }
    }

    if ( retVal != FAPI_OK )
    {
        MENU_CommonMsgShow(msg,FFALSE);
        FAPI_SYS_PRINT_DEBUG(1,"RECORD START FAILED: Resource problem\n");
    }

    return retVal;
}

#endif /* APPL_PVR_ENABLED */

#if 0
// TODO: to be implemented!
FSTATIC void MAIN_HandleEvtSleepTimer (void* pEvt)
{
    #ifdef APPL_PVR_ENABLED

    fbool_t isSet;

    if ( PVRSYS_IsRecording(NULL) == FTRUE )
    {
        /* stop non-timer recording */
        if ( RECTIMER_GetRecordEnd (&isSet, NULL, NULL) != FAPI_OK ) return;

        if ( isSet == FFALSE )
        {
            (void)MAIN_PvrStopRecording(xx);
        }
        else
        {
            /* do not enter standby if a timed record is currently running */
            return;
        }
    }

    #endif /* APPL_PVR_ENABLED */

    MAIN_PowerDown (NULL);
}
#endif

#endif /* APPL_RECTIMER_ENABLED */

#ifdef APPL_PVR_ENABLED

FSTATIC void MAIN_HandleEvtPvrPlay (const void* pEvt)
{
    const SYS_EVT_PVRPLAY_S* pEvent = pEvt;
    PVRSYS_STATUS_S          pvrStatus;

    if ( PVRSYS_GetStatus(&pvrStatus) == FAPI_OK )
    {
        // TODO: split into separate calls for playback and recording
        MENU_InfoUpdatePvrStatus(&pvrStatus);
    }

    MENU_StrSelPlayEvent(pEvent);

    /* Restart live presentation in case of a playback stop not initiated by
       application. */
    if ( (pEvent->action == PVR_PLAY_STOP) && (!(pEvent->userInitiated)) )
    {
        /* resume previous live service */
        (void)AV_RestartService();
    }
}

FSTATIC void MAIN_HandleEvtPvrRec (const void* pEvt)
{
    //const SYS_EVT_PVRREC_S* pEvent = pEvt;
    PVRSYS_STATUS_S         pvrStatus;

    if ( PVRSYS_GetStatus(&pvrStatus) == FAPI_OK )
    {
        // TODO: split into separate calls for playback and recording
        MENU_InfoUpdatePvrStatus(&pvrStatus);

        (void)MAIN_ServBoxUpdate(&(appDat->servBox));

        /* update Recoding list if menu is open */
        MENU_PlayListUpdate();
    }
}

FSTATIC void MAIN_HandleEvtPlayEvt (const void* pEvt)
{
    PVRSYS_STATUS_S pvrStatus;

    MAIN_PvrHandlePlayEvent(((SYS_EVT_PLAYEVT_S*)pEvt)->event);

    if ( PVRSYS_GetStatus(&pvrStatus) == FAPI_OK )
    {
        // TODO: split into separate calls for playback and recording
        MENU_InfoUpdatePvrStatus(&pvrStatus);
    }
}

FSTATIC void MAIN_HandleEvtRecEvt (const void* pEvt)
{
    PVRSYS_STATUS_S pvrStatus;

    MAIN_PvrHandleRecEvent(((SYS_EVT_RECEVT_S*)pEvt)->event,
                           ((SYS_EVT_RECEVT_S*)pEvt)->recIdx);

    if ( PVRSYS_GetStatus(&pvrStatus) == FAPI_OK )
    {
        // TODO: split into separate calls for playback and recording
        MENU_InfoUpdatePvrStatus(&pvrStatus);

        (void)MAIN_ServBoxUpdate(&(appDat->servBox));
    }
}

#endif /* APPL_PVR_ENABLED */

#ifdef APPL_USB_FUNC_MSC_ENABLED

FSTATIC void MAIN_HandleEventUsbConnectStat (const void* pEvt)
{
    const SYS_EVT_USB_CONNECT_S* pMscEvt = pEvt;
    STR_IDX_S                    msgText = STR_EMPTYSTRING;

    if ( pMscEvt->status.devType == USB_DEVTYPE_HOST )
    {
        if ( pMscEvt->status.connected )
        {
            if ( pMscEvt->status.accessStatus == USB_MSC_ACCESS_GRANTED )
            {
                msgText = STR_MSG_USBMSC_CONNECT_ACCESS;
            }
            else if ( pMscEvt->status.accessStatus == USB_MSC_ACCESS_DENIED )
            {
                msgText = STR_MSG_USBMSC_CONNECT_BLOCKED;
            }
        }
        else
        {
            msgText = STR_MSG_USBMSC_DISCONNECT;
        }

        MENU_CommonMsgShow(msgText, FFALSE);
    }
}

#endif /* APPL_USB_FUNC_MSC_ENABLED */

#ifdef APPL_HDCP_ENABLED
FSTATIC void MAIN_HandleEvtHDMIAuthenticate (SYS_EVTTYPE_E type,
                                                const void* data)
{
    /* handle HDCP  */
    if ( type == EVT_HDMI_HDCP_STATE )
    {
        switch(((SYS_EVT_HDCPSTATE_S*)data)->state)
        {
            case VAL_HDMI_HDCPSTATE_AUTH_INIT:
                MENU_CommonMsgShow(STR_MSG_HDCP_AUTH_PROGRESS, FFALSE);
                break;
            case VAL_HDMI_HDCPSTATE_AUTH_FAIL:
                MENU_CommonMsgShow(STR_MSG_HDCP_AUTH_FAILED, FFALSE);
                break;
            case VAL_HDMI_HDCPSTATE_OFF:
            case VAL_HDMI_HDCPSTATE_AUTH_SUCC:
            default:
                /* do nothing */
                break;
        }
    }
}
#endif /* APPL_HDCP_ENABLED */

FSTATIC void MAIN_TvStandardOutputUpdate (SYS_HDTV_STANDARD newHdSystem,
                                          SYS_SDTV_STANDARD newSdSystem)
{
    MENU_EVT_REFRESH_S  evtDat;
    FGS_POS_S           fgsDim;

    if( (newHdSystem != appDat->currentHdOutSys) ||
        (newSdSystem != appDat->currentSdOutSys) )
    {
        appDat->currentHdOutSys = newHdSystem;
        appDat->currentSdOutSys = newSdSystem;

        (void)OSD_Close ();

        if ( OSD_Open (newHdSystem, newSdSystem) == FAPI_OK )
        {
            memset(&fgsDim, 0, sizeof(fgsDim));

            FGS_GetFrameDimensions (&fgsDim);

            fgsDim.x      = (int16_t)SCALER_GetCurrVal(OSD_POS_X);
            fgsDim.y      = (int16_t)SCALER_GetCurrVal(OSD_POS_Y);
            fgsDim.width  = SCALER_GetCurrVal(OSD_WIDTH);
            fgsDim.height = SCALER_GetCurrVal(OSD_HEIGHT);

            (void)FGS_SetFrameDimensions (&fgsDim);

            // TODO: this workaround is to be replaced!
            evtDat.hdr.type   = APP_EVT_MENU_REFRESH;
            evtDat.hdr.length = sizeof(evtDat);
            evtDat.cb         = MENU_AudioVideoRefresh;
            (void)EVT_Send (&evtDat,"Refresh Audio/Video Menu");

            // TODO: this workaround is to be replaced!
            evtDat.hdr.type   = APP_EVT_MENU_REFRESH;
            evtDat.hdr.length = sizeof(evtDat);
            evtDat.cb         = MENU_WelcomeRefresh;
            (void)EVT_Send (&evtDat,"Refresh Welcome Menu");
        }
    }
}

/*---------------------------------------------------------------------------*/

FSTATIC int32_t MENU_Init (void)
{
    int32_t retVal = FAPI_OK;

    do
    {
        retVal = MENU_ConfDialogInit(&(appDat->confDialog));
        if ( retVal != FAPI_OK ) break;

        retVal = MAIN_SignalBoxInit(&(appDat->sigBox));
        if ( retVal != FAPI_OK ) break;

        retVal = MAIN_VolumeBarInit(&(appDat->volBar));
        if ( retVal != FAPI_OK ) break;

        retVal = MAIN_ProhibitedInit(&(appDat->prohib));
        if ( retVal != FAPI_OK ) break;

        retVal = MAIN_MsgBoxInit(&(appDat->msgBox));
        if ( retVal != FAPI_OK ) break;

        retVal = MAIN_ServBoxInit(&(appDat->servBox));
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_FILEOP_ENABLED

        retVal = MAIN_ViewerInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_FILEOP_ENABLED */

        retVal = MENU_PinDialogInit(&(appDat->pinDialog));
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_GuidedInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_InfoInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_InfobarInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_MainInit();
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_RECTIMER_ENABLED

        retVal = MENU_TimerProgInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_TimerEditInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_RECTIMER_ENABLED */

        #ifdef APPL_MOSAIC_ENABLED

        retVal = MENU_MosaicInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_MOSAIC_ENABLED */

        retVal = MENU_SystemSetupInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_InformationInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_LanguageInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_TimeInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_AntProgInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_AudioVideoInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_OsdInit();
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_PARENTAL_ENABLED

        retVal = MENU_PasswordInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_PARENTAL_ENABLED */

        #ifdef APPL_MHEG_ENABLED

        retVal = MENU_MhegInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_MHEG_ENABLED */

        #ifdef APPL_UPDATE_ENABLED

        retVal = MENU_UpdateInit();
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_OAD_ENABLED

        retVal = MENU_OadInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_OAD_ENABLED */

        #endif /* APPL_UPDATE_ENABLED */

        retVal = MENU_StreamInfoInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_TunerInit();
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_FESAT_ENABLED

        retVal = MENU_SatInstTypeInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_SatListInit();
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_DISEQC12_ENABLED

        retVal = MENU_ExpMotorInit();
        if ( retVal != FAPI_OK ) break;

        #endif

        #endif /* APPL_FESAT_ENABLED */

        retVal = MENU_WelcomeInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_ScanInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_ScanningInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_StrSelInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_TpListInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_ServListInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_EpgGridInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_EpgExtInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_EpgSchedInit();
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_FILEOP_ENABLED

        retVal = MENU_ViewerInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_MediaInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_FILEOP_ENABLED */

        #ifdef APPL_JPEG_ENABLED

        retVal = MENU_JPEGInit();
        if ( retVal != FAPI_OK ) break;

        #endif

        #ifdef APPL_MP3_ENABLED

        retVal = MENU_MP3Init();
        if ( retVal != FAPI_OK ) break;

        #endif

        #ifdef APPL_ETHERNET_ENABLED

        retVal = MENU_NetworkInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_NetworkIpInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_NetworkDnsInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_NetworkInfoInit();
        if ( retVal != FAPI_OK ) break;

        #endif

        retVal = MENU_ServSelectInit();
        if ( retVal != FAPI_OK ) break;

        #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

        retVal = MENU_CiInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_MmiInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_CI/CA_ENABLED */

        #ifdef APPL_PVR_ENABLED

        retVal = MENU_PlayListInit();
        if ( retVal != FAPI_OK ) break;

        retVal = MENU_PvrInit();
        if ( retVal != FAPI_OK ) break;

        // TODO: to be ported!
        //retVal = MENU_EditRecInit();
        //if ( retVal != FAPI_OK ) break;

        #endif /* APPL_PVR_ENABLED */

        #if defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED)

        retVal = MENU_HddInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED) */

        #ifdef APPL_PIP_ENABLED

        retVal = MENU_PipSelectInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_PIP_ENABLED */

        retVal = MENU_EditStringInit(&(appDat->editStringBox));
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    return retVal;
}


FSTATIC void MAIN_AddStaticKeys (void)
{
    FGS_SetAppKeyHandler (FGS_KEY_VOL_UP,   MAIN_VolumeUp,   NULL, FFALSE, FFALSE);
    FGS_SetAppKeyHandler (FGS_KEY_VOL_DOWN, MAIN_VolumeDown, NULL, FFALSE, FFALSE);
    FGS_SetAppKeyHandler (FGS_KEY_MUTE,     MAIN_VolumeMute, NULL, FTRUE,  FFALSE);

    #ifdef APPL_SCRNSHOT_ENABLE
    FGS_SetAppKeyHandler (FGS_KEY_F1,       MAIN_ScreenShot, NULL, FTRUE,  FFALSE);
    #endif /* APPL_SCRNSHOT_ENABLE */
}

FSTATIC void MAIN_RemoveStaticKeys (void)
{
    FGS_ClearAppKeyHandler (FGS_KEY_VOL_UP);
    FGS_ClearAppKeyHandler (FGS_KEY_VOL_DOWN);
    FGS_ClearAppKeyHandler (FGS_KEY_MUTE);

    #ifdef APPL_SCRNSHOT_ENABLE
    FGS_ClearAppKeyHandler (FGS_KEY_F1);
    #endif /* APPL_SCRNSHOT_ENABLE */
}

/*-------------------------------------------------------------------------- */

void MAIN_HandleZappingKeys (fbool_t hasFocus)
{
    #ifdef APPL_PVR_ENABLED
    MAIN_HandlePvrKeys (hasFocus);
    #endif /* APPL_PVR_ENABLED */

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler (FGS_KEY_CHAN_UP,   MAIN_ChannelUp,          NULL, FFALSE,  FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_CHAN_DOWN, MAIN_ChannelDown,        NULL, FFALSE,  FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_TV,        MAIN_SwapTvAndRadio,     NULL, FTRUE,   FFALSE);

        #if (defined APPL_MHEG_ENABLED) || (defined APPL_TTXT_ENABLED)
        FGS_SetAppKeyHandler (FGS_KEY_TEXT,  MAIN_GotoText, NULL, FTRUE, FFALSE);

        #endif /* #if (defined APPL_MHEG_ENABLED) || (defined APPL_TTXT_ENABLED) */
    }
    else
    {
        FGS_ClearAppKeyHandler (FGS_KEY_CHAN_UP);
        FGS_ClearAppKeyHandler (FGS_KEY_CHAN_DOWN);
        FGS_ClearAppKeyHandler (FGS_KEY_TV);

        #if (defined APPL_MHEG_ENABLED) || (defined APPL_TTXT_ENABLED)

        /* disable TTX keys */
        FGS_ClearAppKeyHandler (FGS_KEY_TEXT);

        #endif /* #if (defined APPL_MHEG_ENABLED) || (defined APPL_TTXT_ENABLED) */
    }
}

#ifdef APPL_PVR_ENABLED
void MAIN_HandlePvrKeys (fbool_t hasFocus)
{
    if ( hasFocus )
    {
        FGS_SetAppKeyHandler (FGS_KEY_RECORD,  MENU_PvrKeys, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_PLAY,    MENU_PvrKeys, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_STOP,    MENU_PvrKeys, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_FF,      MENU_PvrKeys, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_REW,     MENU_PvrKeys, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_FFSLOW,  MENU_PvrKeys, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_REWSLOW, MENU_PvrKeys, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_PAUSE,   MENU_PvrKeys, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_F4,      MENU_PvrKeys, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_F3,      MENU_PvrKeys, NULL, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler (FGS_KEY_RECORD);
        FGS_ClearAppKeyHandler (FGS_KEY_PLAY);
        FGS_ClearAppKeyHandler (FGS_KEY_STOP);
        FGS_ClearAppKeyHandler (FGS_KEY_FF);
        FGS_ClearAppKeyHandler (FGS_KEY_REW);
        FGS_ClearAppKeyHandler (FGS_KEY_FFSLOW);
        FGS_ClearAppKeyHandler (FGS_KEY_REWSLOW);
        FGS_ClearAppKeyHandler (FGS_KEY_PAUSE);
        FGS_ClearAppKeyHandler (FGS_KEY_F4);
        FGS_ClearAppKeyHandler (FGS_KEY_F3);
    }
}
#endif /* APPL_PVR_ENABLED */

void MAIN_HandleExitKeys (fbool_t hasFocus)
{
    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_MENU, MENUSTACK_Back, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK, MENUSTACK_Back, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_EXIT, MENUSTACK_Exit, NULL, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
    }

}

/* General direct menu transitions */
fbool_t MAIN_DirectMenuTrans (FAPI_SYS_HandleT h)
{
    MENUSTACK_ENTRY       newEntry;
    FAPI_SYS_HandleT             arg = NULL;
    MENU_INFOBAR_PARAMS_S infoParams;
    MENU_EPGEXT_OPEN_S    epgExtParams;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:     newEntry = MENU_MainEntry;
                               break;

        case FGS_KEY_INFO:     infoParams.permanent   = FTRUE;
                               infoParams.numberInput = -1;

                               newEntry = MENU_InfobarEntry;
                               arg      = (FAPI_SYS_HandleT)(&infoParams);
                               break;

        case FGS_KEY_NAVI:     memset(&epgExtParams, 0,
                                              sizeof(epgExtParams));

                               epgExtParams.schedMode = FFALSE;
                               epgExtParams.currType  = EIT_TYPE_CURR;

                               newEntry = MENU_EpgExtEntry;
                               arg      = (FAPI_SYS_HandleT)(&epgExtParams);
                               break;

        case FGS_KEY_OK:       newEntry = MENU_ServSelectEntry;
                               break;

        default:
            return FFALSE;
    }

    return MENUSTACK_Switch(newEntry, arg, 1);
}

#if (defined APPL_MHEG_ENABLED) && (defined APPL_TTXT_ENABLED)

FSTATIC void MAIN_GotoMheg (void* arg)
{
    (void) MENUSTACK_Switch (MENU_MhegEntry, (FAPI_SYS_HandleT)FGS_KEY_TEXT,0);
}


FSTATIC void MAIN_GotoTtxt (void* arg)
{
    (void) MENUSTACK_Switch (MENU_TtxtEntry, NULL,1);
}

FSTATIC fbool_t MAIN_GotoText (FAPI_SYS_HandleT h)
{
    SLIST_ITEM_S       service;
    SYS_SERVLOC_S      servDesc;
    MENU_CONFDIALOG_S  confPars;

    memset(&confPars,   0, sizeof(confPars));

    (void)AV_GetRequestedService  (&servDesc);
    (void)SLIST_EntryGet(&servDesc, &service);

// TODO: Abfrage ndern: warum nicht TTX-Status abfragen statt PID?
//  -> funktioniert z.B. nicht fr Playback
    if ( (service.strPid.mhegPresent) &&
         (service.strPid.ttx == 0) )
    {
        return MENUSTACK_Switch (MENU_MhegEntry, NULL,0);
    }
    else if ( (service.strPid.mhegPresent == 0) &&
              (service.strPid.ttx != 0) )
    {
        return MENUSTACK_Up (MENU_TtxtEntry, NULL);
    }
    else if ( (service.strPid.mhegPresent) &&
              (service.strPid.ttx != 0) )
    {
        memset(&confPars, 0, sizeof(confPars));

        confPars.numDescLines = 1;
        confPars.titleStr     = OSD_GetStr(STR_TTX_SELECT_TITLE);
        confPars.confirmStr   = OSD_GetStr(STR_TTX_SELECT_TTX);
        confPars.declineStr   = OSD_GetStr(STR_TTX_SELECT_MHEG);
        confPars.onConfirm    = MAIN_GotoTtxt;
        confPars.onDecline    = MAIN_GotoMheg;
        confPars.arg          = NULL;

        return MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
    }
    return FFALSE;
}

#endif /* (defined APPL_MHEG_ENABLED) || (defined APPL_TTXT_ENABLED) */

#if (defined APPL_MHEG_ENABLED) && (!defined APPL_TTXT_ENABLED)

FSTATIC fbool_t MAIN_GotoText (FAPI_SYS_HandleT h)
{
    return MENUSTACK_Switch (MENU_MhegEntry, (FAPI_SYS_HandleT)FGS_KEY_TEXT,0);
}
#endif /* (defined APPL_MHEG_ENABLED) || (!defined APPL_TTXT_ENABLED) */

#if (!defined APPL_MHEG_ENABLED) && (defined APPL_TTXT_ENABLED)

FSTATIC fbool_t MAIN_GotoText (FAPI_SYS_HandleT h)
{
    return MENUSTACK_Switch (MENU_TtxtEntry, NULL,1);
}
#endif /* (!defined APPL_MHEG_ENABLED) || (defined APPL_TTXT_ENABLED) */

/* ------------------------------------------------------------------------- */

FSTATIC void MAIN_LastMenuClosed (void)
{
    FAPI_SYS_PRINT_DEBUG(3,"### MAIN_LastMenuClosed \n");

    #ifdef APPL_SUBTT_ENABLED
        SUBTT_ShowOsd();    /* re-enable DVB subtitles */
    #endif /* APPL_SUBTT_ENABLED */

    #ifdef APPL_TTXT_ENABLED
        TTXSUBTT_Restart(); /* re-enable TTX subtitles */
    #endif /* APPL_TTXT_ENABLED */

    #ifdef APPL_MHEG_ENABLED
        MHEG_Enable (FTRUE, SYS_SUSPENDSRC_USER);
    #endif
}

FSTATIC void MAIN_FirstMenuToOpen (void)
{
    FAPI_SYS_PRINT_DEBUG(3,"[%s l=%i] MAIN_FirstMenuToOpen() \n",__FILE__,__LINE__);
    /* ----- Disable some modules to avoid CLUT conflicts ------------------ */

    #ifdef APPL_SUBTT_ENABLED
    SUBTT_HideOsd(); /* hide DVB subtitles */
    #endif /* APPL_SUBTT_ENABLED */

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_Hide(); /* disable TTX subtitles */
    #endif /* APPL_TTXT_ENABLED */

    (void) OSD_GetOsdLayer();
}

/* ------------------------------------------------------------------------- */

FSTATIC fbool_t MENU_StartupEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    FGS_MEMBER_S        member;

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    DBG_Assert(appDat->pStartupMem == NULL);

    memset(&member, 0, sizeof(FGS_MEMBER_S));

    member.h         = NULL;
    member.pos       = &(appDat->rootPos); // position is all zeros!
    member.id        = (int32_t)MENUID_STARTUP;
    member.isVisible = FFALSE;
    member.inFront   = FFALSE;
    member.setFocus  = MENU_StartupSetFocus;
    member.refresh   = NULL;

    if ( FGS_AddMember(&member, &(appDat->pStartupMem)) != FAPI_OK )
    {
        return FFALSE;
    }

    (void)MENUSTACK_FocusForce ((int32_t)MENUID_STARTUP);
    MENUSTACK_SetCleanup (MENU_StartupLeave, NULL);

    (void)MAIN_MenuTimerLaunch(MAIN_STARTUP_DELAY, MENU_StartupTimerExpired, NULL);

    return FTRUE;
}

FSTATIC fbool_t MENU_StartupLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    DBG_Assert(appDat->pStartupMem != NULL);

    MAIN_MenuTimerAbort();

    (void)FGS_RemoveMember       (appDat->pStartupMem);
    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_STARTUP);

    appDat->pStartupMem = NULL;

    return FTRUE;
}

FSTATIC int32_t MENU_StartupSetFocus (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    if ( hasFocus )
    {
        FGS_SetAppKeyHandler   (FGS_KEY_STANDBY, MAIN_PowerDown, NULL, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler (FGS_KEY_STANDBY);
    }

    return FAPI_OK;
}

FSTATIC void MENU_StartupTimerExpired (void* arg)
{
    uint32_t              resetToFactSettings = 0;
    MENU_INFOBAR_PARAMS_S infoParams;

    if ( appDat->pStartupMem == NULL ) return;

    (void)MENUSTACK_Switch(MENU_RootEntry, NULL, 0); /* start in root menu*/

    /* set static keys: apply for all menues  */
    MAIN_AddStaticKeys();

    /* --------------------------------------------------------------------- */

    (void)AV_PowerupScreenHide(); /* Hide boot screen */

    /* attempt to start last active service */
    if ( MAIN_StartInitialService() != FAPI_OK )
    {
        if ( (MAIN_UserDataGet (USERDAT_FACTORYFLAG, &resetToFactSettings)
              == FAPI_OK) &&
             (resetToFactSettings != 0) )
        {
            (void)MENUSTACK_Up (MENU_WelcomeEntry, NULL);
        }
        else
        {
            infoParams.permanent   = FTRUE;
            infoParams.numberInput = -1;

            /* all lists seem to be empty: show infobar */
            (void)MENUSTACK_Up (MENU_InfobarEntry, (FAPI_SYS_HandleT)(&infoParams));
        }
    }
}

/* ------------------------------------------------------------------------- */

fbool_t MENU_RootEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    FGS_MEMBER_S        member;

    #ifdef APPL_MHEG_ENABLED
    MHEG_EVT_START_S    ctrlEvent;
    #endif /* APPL_MHEG_ENABLED */

    if ( test == FTRUE ) return FTRUE;

    memset(&member, 0, sizeof(FGS_MEMBER_S));

    member.h         = NULL;
    member.pos       = &(appDat->rootPos); // position is all zeros!
    member.id        = (int32_t)MENUID_ROOT;
    member.isVisible = FFALSE;
    member.inFront   = FFALSE;
    member.setFocus  = MENU_RootSetFocus;
    member.refresh   = NULL;

    if ( FGS_AddMember(&member, &(appDat->pRootMem)) != FAPI_OK ) return FFALSE;

    (void)MENUSTACK_FocusClaim ((int32_t)MENUID_ROOT);
    MENUSTACK_SetCleanup (MENU_RootLeave, NULL);

    #ifdef APPL_PVR_ENABLED
    if ( appDat->pvrStatus.play.status == PLAY_STATUS_INACTIVE )
    #endif /* APPL_PVR_ENABLED */
    {
        #ifdef APPL_MHEG_ENABLED
        MHEG_Enable (FTRUE, SYS_SUSPENDSRC_USER);
        ctrlEvent.hdr.type   = APP_EVT_MHEG_START;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);

        (void)EVT_Send(&ctrlEvent, "MHEG-5 Start");
        #endif /* APPL_MHEG_ENABLED */
    }
    return FTRUE;
}


FSTATIC fbool_t MENU_RootLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    (void)FGS_RemoveMember (appDat->pRootMem);
    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_ROOT);
    return FTRUE;
}

fbool_t MENU_HandleRootKeys(FGS_KEY_E key)
{
    switch(key)
    {
        case FGS_KEY_UP     : return MENU_RootKeyUp   (NULL);
        case FGS_KEY_DOWN   : return MENU_RootKeyDown (NULL);
        case FGS_KEY_LEFT   : return MENU_RootKeyLeft (NULL);
        case FGS_KEY_RIGHT  : return MENU_RootKeyRight(NULL);
        case FGS_KEY_INFO   : return MENU_RootKeys    (NULL);
        case FGS_KEY_NAVI   : return MENU_RootKeys    (NULL);
        case FGS_KEY_APPL   : return MENU_RootKeys    (NULL);
        case FGS_KEY_TV     : return MAIN_SwapTvAndRadio(NULL);
        case FGS_KEY_MENU   : return MENU_RootKeys    (NULL);
        case FGS_KEY_OK     : return MENU_RootKeys    (NULL);
        case FGS_KEY_FAV    : return MENU_RootKeys    (NULL);
        #ifdef APPL_PIP_ENABLED
        case FGS_KEY_PIP    : return MAIN_KeyPIP      (NULL);
        #endif
        case FGS_KEY_EXIT   : return MENU_RootKeys    (NULL);
        case FGS_KEY_ZOOM   : return MENU_RootKeys    (NULL);
        case FGS_KEY_STANDBY: return MAIN_PowerDown   (NULL);
        default :
            break;
    }
    return FFALSE;
}

FSTATIC int32_t MENU_RootSetFocus (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    MAIN_HandleZappingKeys (hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler   (FGS_KEY_UP,      MENU_RootKeyUp,    NULL, FFALSE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_DOWN,    MENU_RootKeyDown,  NULL, FFALSE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_LEFT,    MENU_RootKeyLeft,  NULL, FFALSE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_RIGHT,   MENU_RootKeyRight, NULL, FFALSE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_INFO,    MENU_RootKeys,     NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_NAVI,    MENU_RootKeys,     NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_MENU,    MENU_RootKeys,     NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_OK,      MENU_RootKeys,     NULL, FTRUE,  FFALSE);
        FGS_SetAppNumKeyHandler(FGS_KEY_NUMBER,  MAIN_NumberKey,    NULL, FTRUE,  FFALSE);
        #ifdef APPL_PIP_ENABLED
        FGS_SetAppKeyHandler   (FGS_KEY_BACK,    MAIN_KeyPIP,       NULL, FTRUE,  FFALSE);
        #endif
        FGS_SetAppKeyHandler   (FGS_KEY_EXIT,    MENU_RootKeys,     NULL, FTRUE,  FFALSE);

        FGS_SetAppKeyHandler   (FGS_KEY_STANDBY, MAIN_PowerDown,    NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_APPL,    MENU_RootKeys,    NULL, FTRUE,  FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler (FGS_KEY_UP);
        FGS_ClearAppKeyHandler (FGS_KEY_DOWN);
        FGS_ClearAppKeyHandler (FGS_KEY_LEFT);
        FGS_ClearAppKeyHandler (FGS_KEY_RIGHT);

        FGS_ClearAppKeyHandler (FGS_KEY_INFO);
        FGS_ClearAppKeyHandler (FGS_KEY_NAVI);
        FGS_ClearAppKeyHandler (FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler (FGS_KEY_MENU);
        FGS_ClearAppKeyHandler (FGS_KEY_OK);
        FGS_ClearAppKeyHandler (FGS_KEY_NUMBER);
        FGS_ClearAppKeyHandler (FGS_KEY_BACK);
        FGS_ClearAppKeyHandler (FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler (FGS_KEY_APPL);
    }

    return FAPI_OK;
}

FSTATIC fbool_t MENU_RootKeys (FAPI_SYS_HandleT h)
{
    MENUSTACK_ENTRY       newEntry = NULL;
    FAPI_SYS_HandleT      arg      = NULL;
    fbool_t               keyUsed  = FFALSE;
    MENU_INFOBAR_PARAMS_S infoParams;
    MENU_EPGEXT_OPEN_S    epgExtParams;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_INFO:     infoParams.permanent   = FTRUE;
                               infoParams.numberInput = -1;

                               newEntry = MENU_InfobarEntry;
                               arg      = (FAPI_SYS_HandleT)(&infoParams);
                               break;

        case FGS_KEY_NAVI:     memset(&epgExtParams, 0,
                                              sizeof(epgExtParams));

                               epgExtParams.schedMode = FFALSE;
                               epgExtParams.currType  = EIT_TYPE_CURR;

                               newEntry = MENU_EpgExtEntry;
                               arg      = (FAPI_SYS_HandleT)(&epgExtParams);
                               break;

        case FGS_KEY_MENU:     newEntry = MENU_MainEntry;
                               break;

        case FGS_KEY_OK:       newEntry = MENU_ServSelectEntry;
                               break;

        case FGS_KEY_BACK:
            /* FALLTHROUGHT */
        case FGS_KEY_EXIT:     keyUsed = (MAIN_VolumeBarHide(&(appDat->volBar))
                                          == FAPI_OK) ? FTRUE : FFALSE;
                               break;
        case FGS_KEY_APPL:     newEntry = MENU_StrSelEntry ;
                               break;

        default:               break;
    }
    if ( newEntry != NULL )
    {
        if(MENUSTACK_GetLevel()!= 0)
        {
            (void) MENUSTACK_Exit(NULL);
        }
        return MENUSTACK_Up(newEntry, arg);
    }
    else
    {
        return keyUsed;
    }
}

/* ------------------------------------------------------------------------- */

FSTATIC fbool_t MENU_RootKeyUp (FAPI_SYS_HandleT h)
{
    #ifdef APPL_PIP_ENABLED

    PIP_STATUS_S pipStatus;

    if ( (PIP_GetStatus(&pipStatus) == FAPI_OK) &&
         (pipStatus.active) )
    {
        APP_PIP_AdjustSize (FFALSE, &pipStatus);
        return FTRUE;
    }

    #endif /* APPL_PIP_ENABLED */

    return MAIN_ChannelUp(NULL);
}

FSTATIC fbool_t MENU_RootKeyDown (FAPI_SYS_HandleT h)
{
    #ifdef APPL_PIP_ENABLED

    PIP_STATUS_S pipStatus;

    if ( (PIP_GetStatus(&pipStatus) == FAPI_OK) &&
         (pipStatus.active) )
    {
        APP_PIP_AdjustSize (FTRUE, &pipStatus);
        return FTRUE;
    }

    #endif /* APPL_PIP_ENABLED */

    return MAIN_ChannelDown(NULL);
}

FSTATIC fbool_t MENU_RootKeyLeft (FAPI_SYS_HandleT h)
{
    #ifdef APPL_PIP_ENABLED

    PIP_STATUS_S pipStatus;

    if ( (PIP_GetStatus(&pipStatus) == FAPI_OK) &&
         (pipStatus.active) )
    {
        APP_PIP_AdjustPosition (FFALSE, &pipStatus);
        return FTRUE;
    }

    #endif /* APPL_PIP_ENABLED */

    return MAIN_VolumeDown(NULL);
}

FSTATIC fbool_t MENU_RootKeyRight (FAPI_SYS_HandleT h)
{
    #ifdef APPL_PIP_ENABLED

    PIP_STATUS_S pipStatus;

    if ( (PIP_GetStatus(&pipStatus) == FAPI_OK) &&
         (pipStatus.active) )
    {
        APP_PIP_AdjustPosition (FTRUE, &pipStatus);
        return FTRUE;
    }

    #endif /* APPL_PIP_ENABLED */

    return MAIN_VolumeUp(NULL);
}

fbool_t MAIN_KeyPIP (FAPI_SYS_HandleT h)
{
    #ifdef APPL_PIP_ENABLED

    PIP_STATUS_S pipStatus;

    if ( PIP_GetStatus(&pipStatus) != FAPI_OK ) return FFALSE;

    if ( pipStatus.active )
    {
        return ( PIP_Hide() == FAPI_OK ) ? FTRUE: FFALSE;
    }
    else
    {
        return MENUSTACK_Switch (MENU_PipSelectEntry, NULL, 1);
    }

    #else

    return FFALSE;

    #endif /* APPL_PIP_ENABLED */
}

#ifdef APPL_PVR_ENABLED

FSTATIC fbool_t MENU_PvrKeys (FAPI_SYS_HandleT h)
{
    fbool_t             keyUsed = FTRUE;
    PVRSYS_STATUS_S     pvrStatus;
    MENU_TIME_OPEN_S    timePars;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_RECORD:
            if ( PVRSYS_GetStatus(&pvrStatus) == FAPI_OK )
            {
                /* maximum number of parallel recordings already active? */
                if ( pvrStatus.numActiveRecs >= RSRC_REC_MAX )
                {
                    MENU_CommonMsgShow(STR_MSG_NORECPOSSIBLE, FFALSE);
                    keyUsed = FFALSE;
                }
                /* currently in playback ? */
                else if ( pvrStatus.play.status != PLAY_STATUS_INACTIVE )
                {
                    MENU_CommonMsgShow(STR_MSG_NOINSTRECPOSSIBLE, FFALSE);
                    keyUsed = FFALSE;
                }
                else
                {
                    if ( TIME_IsSync() )
                    {
                        keyUsed = MAIN_PvrStartInstantRecording();
                    }
                    else
                    {
                        timePars.strIdx       = STR_TIME_MSG_INSTANT_REC;
                        timePars.done         = FFALSE;
                        timePars.next         = MENU_TIME_NEXT_START_INSTANT_REC;
                        (void)MENUSTACK_Up(MENU_TimeEntry,(FAPI_SYS_HandleT)(&timePars));
                    }
                    //keyUsed = MAIN_PvrStartInstantRecording();
                }
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case FGS_KEY_PLAY:
            keyUsed = MAIN_PvrKeyPlay();
            break;

        case FGS_KEY_STOP:
            keyUsed = MAIN_PvrKeyStop();
            break;

        case FGS_KEY_FF:
            keyUsed = MAIN_PvrKeyForward();
            break;

        case FGS_KEY_REW:
            keyUsed = MAIN_PvrKeyRewind();
            break;

        case FGS_KEY_FFSLOW:
            keyUsed = MAIN_PvrKeyForwSlow();
            break;

        case FGS_KEY_REWSLOW:
            keyUsed = MAIN_PvrKeyRewSlow();
            break;

        case FGS_KEY_PAUSE:
            keyUsed = MAIN_PvrKeyPause();
            break;

        case FGS_KEY_F4:
            keyUsed = MAIN_PvrKeyJump(FTRUE);
            break;

        case FGS_KEY_F3:
            keyUsed = MAIN_PvrKeyJump(FFALSE);
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

#endif /* APPL_PVR_ENABLED */

int32_t MAIN_StopPresentation (void)
{
    SYS_PRESSTATE_E presState;
    int32_t            retVal;

    retVal = SYS_GetPresentationState(&presState);
    if ( retVal != FAPI_OK ) return retVal;

    switch ( presState )
    {
        case SYS_PRESSTATE_LIVETV:
            (void)AV_Stop();
            break;

        #ifdef APPL_PVR_ENABLED

        case SYS_PRESSTATE_PLAYBACK:
            (void)PVRSYS_PlayStop();
            break;

        #endif /* APPL_PVR_ENABLED */

        default:
            break;
    }

    #ifdef APPL_PIP_ENABLED
    PIP_Hide();
    #endif /* APPL_PIP_ENABLED */

    (void)AV_Nop();

    return FAPI_OK;
}

int32_t MAIN_RestartPresentation (void)
{
    /* NOTE: Currently live-TV is restarted regardless of the presentation
             state before stopping. This might be improved. */

    return MAIN_RestartLastService();
}

int32_t MAIN_ServiceSwitchGetAvailability (const SYS_SERVLOC_S* pServLoc)
{
    RSRC_OP op;

    op = RSRC_OP_ANY & (~RSRC_OP_LIVE);

    #ifdef APPL_PVR_ENABLED

    /* exclude potentially active background recording and timeshifted playback
       from calculation (will be stopped automatically at service switch) */

    if ( appDat->pvrStatus.bkgrRecIdx >= 0 )
    {
        if ( appDat->pvrStatus.bkgrRecIdx == 0 ) op &= (~RSRC_OP_REC1);
        else                                     op &= (~RSRC_OP_REC2);

        if ( appDat->pvrStatus.play.type == PLAY_TYPE_TIMESHIFT )
        {
            op &= (~RSRC_OP_PLAY);
        }
    }

    #endif /* APPL_PVR_ENABLED */

    return RSRC_ServiceAvlForLive (SLIST_GetServiceIndex(pServLoc), op, NULL,
                                   NULL);
}

/* ------------------------------------------------------------------------- */

fbool_t MENU_StandbyEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    if ( test == FTRUE ) return FTRUE;

    FGS_SetAppKeyHandler(FGS_KEY_STANDBY, MAIN_PowerUp, NULL, FTRUE, FFALSE);

    MENUSTACK_SetCleanup(MENU_StandbyLeave, NULL);
    return FTRUE;
}

FSTATIC fbool_t MENU_StandbyLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    return FTRUE;
}

/* ------------------------------------------------------------------------- */

fbool_t MAIN_NumberKey (FAPI_SYS_HandleT h, uint8_t value)
{
    MENU_INFOBAR_PARAMS_S infoParams;

    #ifdef APPL_PVR_ENABLED

    /* During PVR playback, service selection by number input is prohibited. */
    if ( appDat->pvrStatus.play.type == PLAY_TYPE_NORMAL )
    {
        return FFALSE;
    }

    #endif /* APPL_PVR_ENABLED */

    infoParams.permanent   = FFALSE;
    infoParams.numberInput = (int8_t)value;

    return MENUSTACK_Switch (MENU_InfobarEntry, (FAPI_SYS_HandleT)(&infoParams), 1);
}

/* ------------------------------------------------------------------------- */

void MAIN_SaveSettingsIfChanged (void)
{
    if ( SETTINGS_SavingIsNecessary() == FTRUE )
    {
        MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);
        (void)SETTINGS_Save();

        (void)MAIN_MsgBoxHide(&(appDat->msgBox));
    }
}

void MAIN_FreqDataHasChanged (void)
{
    appDat->fcDataDirty   = FTRUE;
    appDat->fcDataIdleCnt = 0;
}

void MAIN_MsgFunctionReturnedFailure(int32_t ferr)
{
    MAIN_MSGBOX_OPEN_S msgBoxPars;
    uint32_t          bufPos = 0;

    bufPos += STR_BufAppend (NULL, bufPos,
                             OSD_GetStr((uint32_t)STR_MSG_FAILURERETURNED),
                             MAIN_LINEBUFCHARSET);

    sprintf (appDat->asciiBuf, " 0x%lX", (uint32_t)ferr);

    bufPos += STR_BufAppend (NULL, bufPos,
                             STR_IntASCIIStr(),
                             MAIN_LINEBUFCHARSET);

    msgBoxPars.text      = STR_IntLineStr();
    msgBoxPars.permanent = FFALSE;
    msgBoxPars.numLines  = 1;
    msgBoxPars.width     = -1;

    (void)MAIN_MsgBoxShow(&(appDat->msgBox), &msgBoxPars);
}

/* ------------------------------------------------------------------------- */

int32_t MAIN_MenuTimerLaunch (uint16_t timeInSeconds, void (*handler)(void* arg),
                           void* arg)
{
    /* Launch or restart timer */
    (void)TIME_SingleTimerLaunch(OSD_TIMER_MENU, timeInSeconds * 10);

    appDat->menuTimerHandler       = handler;
    appDat->menuTimerArg           = arg;

    return FAPI_OK;
}

void MAIN_MenuTimerAbort (void)
{
    (void)TIME_SingleTimerAbort(OSD_TIMER_MENU);

    appDat->menuTimerHandler       = NULL;
    appDat->menuTimerArg           = NULL;
}

FSTATIC void MAIN_MenuTimerExpired (void)
{
    if ( appDat->menuTimerHandler != NULL )
    {
        appDat->menuTimerHandler(appDat->menuTimerArg);

        appDat->menuTimerHandler = NULL;
        appDat->menuTimerArg     = NULL;
    }
}

/* ------------------------------------------------------------------------- */

fbool_t MAIN_PowerDown (FAPI_SYS_HandleT h)
{
    EIT_PARAMS_S eitParams;

    FAPI_SYS_PRINT_DEBUG(1,"[APP] Power down...\n");

    #ifdef APPL_PIP_ENABLED
        PIP_Hide();
    #endif /* APPL_PIP_ENABLED */

// TODO: block in some cases if record is running...
#if 0
    #ifdef APPL_PVR_ENABLED

    if ( PVRSYS_IsRecording(NULL) == FTRUE )
    {
        /* to do: behaviour to be defined: prohibited in all cases or not
                  in case of sleep timer, ... */
        MENU_CommonMsgShow(STR_MSG_PROHIBDURINGRECORD, FFALSE);
        return FFALSE;
    }

    #endif /* APPL_PVR_ENABLED */
#endif


    /* close all open panels not related to a menu on menustack */
    (void)MENUSTACK_CloseAllExt(FTRUE);

    MENUSTACK_SetForceMode();

    /* go to (virtual) standby menu */
    if ( MENUSTACK_Switch(MENU_StandbyEntry, NULL, 0) == FFALSE ) return FFALSE;

    MAIN_RemoveStaticKeys();

    /* Test if saving of frequently-changing data is necessary */
    if ( appDat->fcDataDirty == FTRUE )
    {
        (void)SETTINGS_StoreFreqChgData();
        appDat->fcDataDirty = FFALSE;
    }

    eitParams.modeCurrNext = EIT_FILTERMODE_OFF;
    eitParams.modeSchedule = EIT_FILTERMODE_OFF;

    (void)EIT_SetParams(&eitParams);

    (void)OSD_Close();

    (void)SYS_SetOperationMode(SYS_PWRMODE_DOWN);  /* put system into standby mode */
    FAPI_SYS_PRINT_DEBUG(1,"[APP] Power down OK\n");
    return FTRUE;
}

fbool_t MAIN_PowerUp (FAPI_SYS_HandleT h)
{
    uint8_t             cnt;
    SYS_FE_STATUS_S     feStatus;
    EIT_PARAMS_S        eitParams;
    SYS_HDTV_STANDARD   hdSys = SETTINGS_DEFAULT_HDTVSTANDARD;
    SYS_SDTV_STANDARD   sdSys = SETTINGS_DEFAULT_SDTVSTANDARD;
    int32_t             retVal = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(1,"[app_main][MAIN_PowerUp] Power up... \n");

    (void)SYS_SetOperationMode(SYS_PWRMODE_UP);  /* put system into active mode */

    (void)SETTINGS_OutputTvStandardGet(&hdSys, &sdSys);
    FAPI_SYS_PRINT_DEBUG(1,"[app_main][MAIN_PowerUp] SETTINGS_OutputTvStandardGet... \n");

    retVal = OSD_Open(hdSys, sdSys);
    if(retVal != FAPI_OK)
    {
        return FFALSE;
    }

    FAPI_SYS_PRINT_DEBUG(1,"[app_main][MAIN_PowerUp] OSD_Open Success\n");
    eitParams.modeCurrNext = EIT_FILTERMODE_THISSERVICE;
    eitParams.modeSchedule = EIT_FILTERMODE_THISSERVICE;

    (void)EIT_SetParams(&eitParams);

    /* Cleanup key handlers */
    FGS_ClearAllAppKeyHandlers ();

    MAIN_LastMenuClosed();
    FAPI_SYS_PRINT_DEBUG(1,"[app_main][MAIN_PowerUp] MAIN_LastMenuClosed Success\n");

    /* --------------------------------------------------------------------- */

    FAPI_SYS_PRINT_DEBUG(1,"app_main][MAIN_PowerUp] appDat is %s\n", appDat == NULL ? "NULL" : "Valid");

    /* Determine available frontends */

    appDat->avaiTuner  = 0;
    appDat->numSatFe   = 0;
    appDat->numTerrFe  = 0;
    appDat->numCableFe = 0;
    FAPI_SYS_PRINT_DEBUG(1,"app_main][MAIN_PowerUp] appDat->hwCfg is %s\n", appDat->hwCfg == NULL ? "NULL" : "Valid");
    FAPI_SYS_PRINT_DEBUG(1,"[app_main][MAIN_PowerUp] feNum = %d\n", appDat->hwCfg->feNums);

    for( cnt = 0; cnt < appDat->hwCfg->feNums; cnt++ )
    {
        if ( ((AV_GetFeStatus((int8_t)cnt,&feStatus)) == FAPI_OK) &&
             (feStatus.use != SYS_FEUSAGE_NOTAVAILABLE) )
        {
            appDat->avaiTuner++;

            switch ( feStatus.type )
            {
                case FE_SYSTEM_TYPE_SATELLITE:   appDat->numSatFe++;    break;
                case FE_SYSTEM_TYPE_TERRESTRIAL: appDat->numTerrFe++;   break;
                case FE_SYSTEM_TYPE_CABLE:       appDat->numCableFe++;  break;
                case FE_SYSTEM_TYPE_DIRECT:      appDat->numDirectTs++; break;
                default:                                                break;
            }
        }
    }
    FAPI_SYS_PRINT_DEBUG(1,"[app_main][MAIN_PowerUp] AV_GetFeStatus Success \n");

    (void)MAIN_SetupGuidedMode();

    FAPI_SYS_PRINT_DEBUG(1,"[app_main][MAIN_PowerUp] MAIN_SetupGuidedMode Success \n");
    /* --------------------------------------------------------------------- */

    /* Enter virtual startup menu */
    (void)MENUSTACK_Switch(MENU_StartupEntry, NULL, 0);

    FAPI_SYS_PRINT_DEBUG(1,"[app_main][MAIN_PowerUp] MENUSTACK_Switch Success \n");

    FAPI_SYS_PRINT_DEBUG(1,"[APP] Power up OK \n");
    return FTRUE;
}

/* ------------------------------------------------------------------------- */
/*!
******************************************************************************
** \brief   TODO
**
** \param   id      TODO
** \param   value   TODO
**
** \return  TODO
**
******************************************************************************
*/
int32_t MAIN_UserDataGet (USERDAT_ID_E id, uint32_t* value)
{
    uint8_t   index = 0;
    uint32_t  mask = 0;
    uint8_t   offset = 0;
    uint32_t  data = 0;
    int32_t   retVal;

    retVal = MAIN_UserDataGetParams(id, &index, &mask, &offset);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SETTINGS_UserDatGet(index, &data);
    if ( retVal != FAPI_OK ) return retVal;

    *value = (data & mask) >> offset;
    return FAPI_OK;
}

int32_t MAIN_UserDataSet (USERDAT_ID_E id, uint32_t value)
{
    uint8_t   index = 0;
    uint32_t  mask = 0;
    uint8_t   offset = 0;
    uint32_t  data = 0;
    int32_t   retVal;

    retVal = MAIN_UserDataGetParams(id, &index, &mask, &offset);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SETTINGS_UserDatGet(index, &data);
    if ( retVal != FAPI_OK ) return retVal;

    value = value << offset;

    data = (data & (~mask)) | (value & mask);

    return SETTINGS_UserDatSet(index, data);
}

FSTATIC int32_t MAIN_UserDataGetParams (USERDAT_ID_E id, uint8_t* idx, uint32_t* mask,
                                     uint8_t* offset)
{
    switch ( id )
    {
        case USERDAT_TRANSPARENCY:  *idx = 0; *mask = 0xFF000000; *offset = 24; break;
        case USERDAT_COLORSCHEME:   *idx = 0; *mask = 0x00FF0000; *offset = 16; break;
        case USERDAT_MENUALIGNMENT: *idx = 0; *mask = 0x0000FF00; *offset =  8; break;
        case USERDAT_FACTORYFLAG:   *idx = 0; *mask = 0x00000001; *offset =  0; break;
        case USERDAT_PVRJUMPTIME:   *idx = 1; *mask = 0xFF000000; *offset = 24; break;
        default: return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    return FAPI_OK;
}

/* ------------------------------------------------------------------------- */

/* Determines order of menu dialogues in "guided installation" mode */
FSTATIC int32_t MAIN_SetupGuidedMode (void)
{
    int32_t           retVal = FAPI_OK;
    MENU_GUIDED_S     guidedParams;
    SYS_FE_STATUS_S   feStatus;
    uint8_t           feIdx;
    fbool_t           firstFeFound = FFALSE;

    for(;;)
    {
        retVal = MENU_GuidedClear();
        if ( retVal != FAPI_OK ) break;

        memset(&guidedParams, 0, sizeof(guidedParams));

        guidedParams.params.guided = FTRUE;

        #ifdef APPL_FESAT_ENABLED

        if ( appDat->numSatFe > 0 )
        {
            guidedParams.entry         = MENU_SatInstTypeEntry;
            guidedParams.params.arg1   = 0;
            guidedParams.strIdx        = STR_EMPTYSTRING;

            retVal = MENU_GuidedAdd(&guidedParams);
            if ( retVal != FAPI_OK ) break;

            guidedParams.entry         = MENU_SatListEntry;
            guidedParams.params.arg1   = 0;
            guidedParams.strIdx        = STR_EMPTYSTRING;

            retVal = MENU_GuidedAdd(&guidedParams);
            if ( retVal != FAPI_OK ) break;

            for ( feIdx = 0; feIdx < appDat->hwCfg->feNums; feIdx++ )
            {
                if ( (AV_GetFeStatus((int8_t)feIdx, &feStatus) == FAPI_OK) &&
                     (feStatus.type == FE_SYSTEM_TYPE_SATELLITE) )
                {
                    guidedParams.entry         = MENU_TunerEntry;
                    guidedParams.params.arg1   = feIdx; /* tuner/fe index */
                    guidedParams.params.arg2   = 0;     /* ignored for guided */

                    if ( firstFeFound == FFALSE )
                    {
                        guidedParams.strIdx = STR_ENTRY_TUNER_GOTO;
                        firstFeFound = FTRUE;
                    }
                    else
                    {
                        guidedParams.strIdx = STR_ENTRY_TUNER_GOTO_NEXT;
                    }

                    retVal = MENU_GuidedAdd(&guidedParams);
                    if ( retVal != FAPI_OK ) break;
                }
            }

        }
        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        if ( appDat->numTerrFe > 0 )
        {
            for ( feIdx = 0; feIdx < appDat->hwCfg->feNums; feIdx++ )
            {
                if ( (AV_GetFeStatus((int8_t)feIdx, &feStatus) == FAPI_OK) &&
                     (feStatus.type == FE_SYSTEM_TYPE_TERRESTRIAL) )
                {
                    guidedParams.entry         = MENU_TunerEntry;
                    guidedParams.params.arg1   = feIdx; /* tuner/fe index */
                    guidedParams.params.arg2   = 0;     /* ignored for guided */

                    if ( firstFeFound == FFALSE )
                    {
                        guidedParams.strIdx = STR_ENTRY_TUNER_GOTO;
                        firstFeFound = FTRUE;
                    }
                    else
                    {
                        guidedParams.strIdx = STR_ENTRY_TUNER_GOTO_NEXT;
                    }

                    retVal = MENU_GuidedAdd(&guidedParams);
                    if ( retVal != FAPI_OK ) break;
                }
            }
        }
        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED
        if ( appDat->numCableFe > 0 )
        {
            for ( feIdx = 0; feIdx < appDat->hwCfg->feNums; feIdx++ )
            {
                if ( (AV_GetFeStatus((int8_t)feIdx, &feStatus) == FAPI_OK) &&
                     (feStatus.type == FE_SYSTEM_TYPE_CABLE) )
                {
                    guidedParams.entry         = MENU_TunerEntry;
                    guidedParams.params.arg1   = feIdx; /* tuner/fe index */
                    guidedParams.params.arg2   = 0;     /* ignored for guided */

                    if ( firstFeFound == FFALSE )
                    {
                        guidedParams.strIdx = STR_ENTRY_TUNER_GOTO;
                        firstFeFound = FTRUE;
                    }
                    else
                    {
                        guidedParams.strIdx = STR_ENTRY_TUNER_GOTO_NEXT;
                    }

                    retVal = MENU_GuidedAdd(&guidedParams);
                    if ( retVal != FAPI_OK ) break;
                }
            }
        }
        #endif /* APPL_FECABLE_ENABLED */

        #ifdef APPL_FESAT_ENABLED
        if ( appDat->numSatFe > 0 )
        {
            guidedParams.entry         = MENU_ScanSatEntry;
            guidedParams.params.arg1   = 0;
            guidedParams.strIdx        = STR_ENTRY_SCAN_SAT_GOTO;

            retVal = MENU_GuidedAdd(&guidedParams);
            if ( retVal != FAPI_OK ) break;
        }
        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        if ( appDat->numTerrFe > 0 )
        {
            guidedParams.entry         = MENU_ScanTerrEntry;
            guidedParams.params.arg1   = 0;
            guidedParams.strIdx        = STR_ENTRY_SCAN_TERR_GOTO;

            retVal = MENU_GuidedAdd(&guidedParams);
            if ( retVal != FAPI_OK ) break;
        }

        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED

        if ( appDat->numCableFe > 0 )
        {
            guidedParams.entry         = MENU_ScanCableEntry;
            guidedParams.params.arg1   = 0;
            guidedParams.strIdx        = STR_ENTRY_SCAN_CABLE_GOTO;

            retVal = MENU_GuidedAdd(&guidedParams);
            if ( retVal != FAPI_OK ) break;
        }

        #endif /* APPL_FECABLE_ENABLED */

        #ifdef APPL_FEDIRECT_ENABLED

        if ( appDat->numDirectTs > 0 )
        {
            guidedParams.entry         = MENU_ScanDirectEntry;
            guidedParams.params.arg1   = 0;
            guidedParams.strIdx        = STR_ENTRY_SCAN_DIRECT_GOTO;

            retVal = MENU_GuidedAdd(&guidedParams);
            if ( retVal != FAPI_OK ) break;
        }

        #endif /* APPL_FEDIRECT_ENABLED */

        guidedParams.entry         = MENU_ServListEntry;
        guidedParams.params.arg1   = 0;
        guidedParams.strIdx        = STR_ENTRY_SERVLIST_GOTO;

        retVal = MENU_GuidedAdd(&guidedParams);
        if ( retVal != FAPI_OK ) break;

        break; // end of for(;;)
    }

    return retVal;
}

/* ------------------------------------------------------------------------- */

#ifdef APPL_FESAT_ENABLED

FSTATIC void MAIN_GenerateSatCarrierList (const SYS_NETWORK_S* network)
{
    TPLIST_ITEM_S       carrierItem;
    uint32_t            itemCnt;
    int16_t             tpIdx;
    CFGSAT_SAT_S        satInf;
    APP_SATPAR_S*       pSat;

    if ( CFGSAT_GetDataByIndex (network->satIdx, &satInf) != FAPI_OK ) return;

    /* known satellite ?*/
    if ( satInf.id >= (uint32_t)SATID_NUMS ) return;

    pSat    = satLst + satInf.id;
    itemCnt = 0;

    if ( pSat->pCarrier == NULL ) return;

    while ( pSat->pCarrier[itemCnt].frequency != 0 )
    {
        memset(&carrierItem, 0, sizeof(TPLIST_ITEM_S));

        carrierItem.tsId       = -1;
        carrierItem.networkId  = pSat->networkId;
        carrierItem.onId       = -1;

        carrierItem.pars.sat.frequency       = pSat->pCarrier[itemCnt].
                                               frequency;
        carrierItem.pars.sat.symbolRate      = pSat->pCarrier[itemCnt].
                                               symbol;
        carrierItem.pars.sat.fec             = FE_FEC_NONE;
        carrierItem.pars.sat.pol             = pSat->pCarrier[itemCnt].
                                               pol;
        carrierItem.pars.sat.modulation      = pSat->pCarrier[itemCnt].
                                               modulation;
        carrierItem.pars.sat.satID           = 0;
        carrierItem.pars.sat.orbitalPosition = 0;
        carrierItem.pars.sat.westEastFlag    = FE_WE_FLAG_NONE;
        carrierItem.pars.sat.iq              = FE_IQ_NONE;

        (void)TPLIST_AddCarrier (network, &carrierItem, SYS_OWNER_SYSTEM,
                           &tpIdx);

        itemCnt++;
    }
}

#endif /* APPL_FESAT_ENABLED */

/* ------------------------------------------------------------------------- */

#ifdef APPL_FETERR_ENABLED

FSTATIC TERR_CHAN_VHF vhfWesternEurope[] =
{
    /* NOTE: lower VHF channels are currently not accessable by tuner! */
    //{ "E 2",   50500 },
    //{ "E 3",   57500 },
    //{ "E 4",   64500 },
    { "E 5",  177500000 },
    { "E 6",  184500000 },
    { "E 7",  191500000 },
    { "E 8",  198500000 },
    { "E 9",  205500000 },
    { "E 10", 212500000 },
    { "E 11", 219500000 },
    { "E 12", 226500000 },
    { "",          0 }
};

FSTATIC TERR_CHAN_VHF vhfEasternEurope[] =
{
    /* NOTE: lower VHF channels are currently not accessable by tuner! */
    //{ "R I",     52500 },
    //{ "R II",    62000 },
    //{ "R III",   80000 },
    //{ "R IV",    88000 },
    //{ "R V",     96000 },
    { "R VI",   178000000 },
    { "R VII",  186000000 },
    { "R VIII", 194000000 },
    { "R IX",   202000000 },
    { "R X",    210000000 },
    { "R XI",   218000000 },
    { "R XII",  226000000 },
    { "",            0 }
};

FSTATIC TERR_CHAN_VHF vhfIreland[] =
{
    /* NOTE: lower VHF channels are currently not accessable by tuner! */
    //{ "I A",     48500 },
    //{ "I B",     56500 },
    //{ "I C",     64500 },
    { "I D",    178000000 },
    { "I E",    186000000 },
    { "I F",    194000000 },
    { "I G",    202000000 },
    { "I H",    210000000 },
    { "I I",    218000000 },
    { "I J",    226000000 },
    { "",            0 }
};

FSTATIC TERR_CHAN_VHF vhfItaly[] =
{
    /* NOTE: lower VHF channels are currently not accessable by tuner! */
    //{ "A",       56000 },
    //{ "B",       64500 },
    //{ "C",       84500 },
    { "D",      177500000 },
    { "E",      186000000 },
    { "F",      194500000 },
    { "G",      203500000 },
    { "H",      212500000 },
    { "H1",     219500000 },
    { "H2",     226500000 },
    { "",            0 }
};

FSTATIC TERR_CHAN_VHF vhfFrance[] =
{
    /* NOTE: lower VHF channels are currently not accessable by tuner! */
    //{ "A",       45000 },
    //{ "B",       53000 },
    //{ "C",       61000 },
    //{ "C1",      57750 },
    { "1",      178750000 },
    { "2",      186750000 },
    { "3",      194750000 },
    { "4",      202750000 },
    { "5",      210750000 },
    { "6",      218750000 },
    { "",            0 }
};

FSTATIC TERR_CHAN_VHF vhfAustralia[] =
{
    { "0",    48500000 },
    { "1",    52500000 },
    { "2",    59500000 },
    { "3",    88500000 },
    { "4",    97500000 },
    { "5",   104500000 },
    { "6",   177500000 },
    { "7",   184500000 },
    { "8",   191500000 },
    {"9a",   205500000 },
    {"10",   212500000 },
    {"11",   219500000 },
    { "",            0 }
};


void MAIN_GenerateTerrChannelList (CHANREGION_E region)
{
    TPLIST_ITEM_S chanItem;
    SYS_NETWORK_S network;

    network.type   = FE_SYSTEM_TYPE_TERRESTRIAL;
    network.satIdx = 0;

    /* Ensure that list is empty */
    (void)TPLIST_Clear(&network, SYS_OWNER_ALL);

    memset(&chanItem, 0, sizeof(chanItem));

    chanItem.tsId                    = -1;
    chanItem.networkId               = -1;
    chanItem.onId                    = -1;
    chanItem.pars.terr.frequency     = 0;
    chanItem.pars.terr.bandwidth     = FE_BANDWIDTH_NONE;
    chanItem.pars.terr.codeRateHP    = FE_FEC_NONE;
    chanItem.pars.terr.codeRateLP    = FE_FEC_NONE;
    chanItem.pars.terr.modulation    = FE_MOD_NONE;
    chanItem.pars.terr.transmission  = FE_TRANS_MODE_NONE;
    chanItem.pars.terr.guardInterval = FE_GUARD_INTERVAL_NONE;
    chanItem.pars.terr.hierarchy     = FE_HIERARCHY_NONE;
    chanItem.pars.terr.iq            = FE_IQ_NONE;

    /* ----- generate VHF channels ----------------------------------------- */

    switch ( region )
    {
        default:
        case CHANREGION_WESTEUROPE:
            MAIN_AddVHFChannels(&chanItem, vhfWesternEurope,
                                FE_BANDWIDTH_7_MHZ);
            break;

        case CHANREGION_EASTEUROPE:
            MAIN_AddVHFChannels(&chanItem, vhfEasternEurope,
                                FE_BANDWIDTH_8_MHZ);
            break;

        case CHANREGION_IRELAND:
            MAIN_AddVHFChannels(&chanItem, vhfIreland, FE_BANDWIDTH_8_MHZ);
            break;

        case CHANREGION_FRANCE:
            MAIN_AddVHFChannels(&chanItem, vhfFrance, FE_BANDWIDTH_8_MHZ);
            break;

        case CHANREGION_ITALY:
            MAIN_AddVHFChannels(&chanItem, vhfItaly, FE_BANDWIDTH_7_MHZ);
            break;

        case CHANREGION_UK:
        case CHANREGION_CHINA:
        case CHANREGION_USA:
        case CHANREGION_CANADA:
        case CHANREGION_JAPAN:
        case CHANREGION_NUMS:
            /* no VHF TV channels available! */
            break;

        case CHANREGION_AUSTRALIA:
            MAIN_AddVHFChannels(&chanItem, vhfAustralia, FE_BANDWIDTH_7_MHZ);
            break;
    }

    /* ----- generate UHF channels ----------------------------------------- */
    switch(region)
    {
        case CHANREGION_CHINA:
            MAIN_AddUHFChannels(&chanItem,13,24,474000000,FE_BANDWIDTH_8_MHZ);
            MAIN_AddUHFChannels(&chanItem,25,62,610000000,FE_BANDWIDTH_8_MHZ);
            break;
        case CHANREGION_USA:
        case CHANREGION_CANADA:
            MAIN_AddUHFChannels(&chanItem,14,83,473000000,FE_BANDWIDTH_6_MHZ);
            break;
        case CHANREGION_JAPAN:
            MAIN_AddUHFChannels(&chanItem,13,62,473000000,FE_BANDWIDTH_6_MHZ);
            break;
        case CHANREGION_AUSTRALIA:
            MAIN_AddUHFChannels(&chanItem,28,69,529500000,FE_BANDWIDTH_7_MHZ);
            break;
        default:
            MAIN_AddUHFChannels(&chanItem,21,69,474000000,FE_BANDWIDTH_8_MHZ);
            break;
    } /* end of switch(region) */
}

FSTATIC void MAIN_AddVHFChannels (TPLIST_ITEM_S* pItem, TERR_CHAN_VHF* pList,
                                  FE_BANDWIDTH_E bandwidth)
{
    int16_t           tpIdx;
    SYS_NETWORK_S network;

    network.type   = FE_SYSTEM_TYPE_TERRESTRIAL;
    network.satIdx = 0;

    pItem->pars.terr.bandwidth = bandwidth;

    while ( pList->freq != 0 )
    {
        pItem->pars.terr.frequency = pList->freq;
        pItem->nameCharSet = SYS_CHARSET_ASCII;
        (void)UTILS_strncpy(pItem->name, pList->name, TPLIST_CARRIERNAME_MAX);

        (void)TPLIST_AddCarrier(&network, pItem, SYS_OWNER_SYSTEM, &tpIdx);

        pList++;
    }
}

FSTATIC void MAIN_AddUHFChannels(TPLIST_ITEM_S* pItem,
                                 uint32_t chanFirst,
                                 uint32_t chanLast,
                                 uint32_t startFrequency,
                                 FE_BANDWIDTH_E bandwidth)
{
    uint32_t             chanCnt;
    uint32_t             frequencyStep;
    int16_t             tpIdx;
    SYS_NETWORK_S   network;

    network.type   = FE_SYSTEM_TYPE_TERRESTRIAL;
    network.satIdx = 0;

    chanCnt = chanFirst;
    pItem->pars.terr.bandwidth = bandwidth;
    pItem->pars.terr.frequency = startFrequency;

    switch(bandwidth)
    {
        case FE_BANDWIDTH_8_MHZ:
            frequencyStep = 8000000;
            break;
        case FE_BANDWIDTH_7_MHZ:
            frequencyStep = 7000000;
            break;
        case FE_BANDWIDTH_6_MHZ:
            frequencyStep = 6000000;
            break;
        case FE_BANDWIDTH_AUTO:
        case FE_BANDWIDTH_NONE:
        default:
            frequencyStep = 8000000;
            break;
    }
    while(chanCnt <= chanLast)
    {
        pItem->nameCharSet = SYS_CHARSET_ASCII;
        sprintf(pItem->name, "%-2ld", chanCnt);

        (void)TPLIST_AddCarrier(&network, pItem, SYS_OWNER_SYSTEM, &tpIdx);

        pItem->pars.terr.frequency += frequencyStep;
        chanCnt++;
    }
}

#endif /* APPL_FETERR_ENABLED */

#ifdef APPL_FECABLE_ENABLED

void MAIN_GenerateCableChannelList (CHANREGION_E region)
{
    TPLIST_ITEM_S chanItem;
    SYS_NETWORK_S network;
    char_t*         chanNamePrefix;

    network.type   = FE_SYSTEM_TYPE_CABLE;
    network.satIdx = 0;

    /* Ensure that list is empty */
    (void)TPLIST_Clear(&network, SYS_OWNER_ALL);

    memset(&chanItem, 0, sizeof(chanItem));

    chanItem.tsId                  = -1;
    chanItem.networkId             = -1;
    chanItem.onId                  = -1;
    chanItem.pars.cable.frequency  = 0;
    chanItem.pars.cable.symbolRate = 0;
    chanItem.pars.cable.modulation = FE_MOD_NONE;
    chanItem.pars.cable.fec        = FE_FEC_NONE;
    chanItem.pars.cable.iq         = FE_IQ_NONE;

    /* ----- generate VHF channels ----------------------------------------- */

    switch ( region )
    {
        case CHANREGION_WESTEUROPE:
            chanNamePrefix = "";
            MAIN_AddCableChannels(&chanItem,21,69,474000000,chanNamePrefix,FE_BANDWIDTH_8_MHZ);
            break;

        /* not implemented yet use defaults */
        case CHANREGION_AUSTRALIA:
        case CHANREGION_CANADA:
        case CHANREGION_CHINA:
        case CHANREGION_EASTEUROPE:
        case CHANREGION_FRANCE:
        case CHANREGION_IRELAND:
        case CHANREGION_ITALY:
        case CHANREGION_JAPAN:
        case CHANREGION_UK:
        case CHANREGION_USA:
        case CHANREGION_NUMS:
        default:
            chanNamePrefix = "";
            MAIN_AddCableChannels(&chanItem,21,69,474000000,chanNamePrefix,FE_BANDWIDTH_8_MHZ);
            break;
    } /* end of switch(region) */
}

FSTATIC void MAIN_AddCableChannels(TPLIST_ITEM_S* pItem,
                                   uint32_t chanFirst,
                                   uint32_t chanLast,
                                   uint32_t startFrequency,
                                   char_t* chanNamePrefix,
                                   FE_BANDWIDTH_E bandwidth)
{
    uint32_t             chanCnt;
    uint32_t             frequencyStep;
    int16_t             tpIdx;
    SYS_NETWORK_S   network;

    network.type   = FE_SYSTEM_TYPE_CABLE;
    network.satIdx = 0;

    chanCnt = chanFirst;
    pItem->pars.cable.symbolRate = 6875;
    pItem->pars.cable.modulation = FE_MOD_QAM_64;
    pItem->pars.cable.frequency = startFrequency;
    pItem->nameCharSet = SYS_CHARSET_ASCII;
    switch(bandwidth)
    {
        case FE_BANDWIDTH_8_MHZ:
            frequencyStep = 8000000;
            break;
        case FE_BANDWIDTH_7_MHZ:
            frequencyStep = 7000000;
            break;
        case FE_BANDWIDTH_6_MHZ:
            frequencyStep = 6000000;
            break;
        case FE_BANDWIDTH_AUTO:
        case FE_BANDWIDTH_NONE:
        default:
            frequencyStep = 8000000;
            break;
    }
    while(chanCnt <= chanLast)
    {
        sprintf(pItem->name, "%s%-2ld",chanNamePrefix, chanCnt);

        (void)TPLIST_AddCarrier(&network, pItem, SYS_OWNER_SYSTEM, &tpIdx);

        pItem->pars.cable.frequency += frequencyStep;
        chanCnt++;
    }
}

#endif /* APPL_FECABLE_ENABLED */

FSTATIC void MAIN_SetStartupDefaults (void)
{
    const SYS_HWCONFIG_S*   hwCfg = SYS_GetHWConfig();
    int8_t                  feIdx;
    uint8_t                 cnt;

    (void) SETTINGS_DisplayTimeSet      (SETTINGS_DEFAULT_DISPTIME);
    (void) SETTINGS_VideoOutputSet      (SETTINGS_DEFAULT_VIDEOOUT);
    (void) SETTINGS_OutputTvStandardSet (SETTINGS_DEFAULT_HDTVSTANDARD,
                                         SETTINGS_DEFAULT_SDTVSTANDARD);
    (void) SETTINGS_SetServiceStartMode (SETTINGS_DEFAULT_STARTMODE);
    (void) SETTINGS_SetServiceSwitchMode(SETTINGS_DEFAULT_SWITCHMODE);
    (void) SETTINGS_TimeOffsetSet       (SETTINGS_DEFAULT_UTCOFFSET,
                                         SETTINGS_DEFAULT_DSTACTIVE);
    (void) SETTINGS_SetPowerUpMode      (SETTINGS_DEFAULT_POWERUPMODE);
    (void) SETTINGS_DispFormatSet       (SETTINGS_DEFAULT_DISPFORMAT,
                                         SETTINGS_DEFAULT_SCALEMETHOD);
    (void) SETTINGS_DefaultSPDIFModeSet (SETTINGS_DEFAULT_SPDIFMODE);

    for ( feIdx = 0; feIdx < hwCfg->feNums; feIdx++ )
    {
        (void) SETTINGS_AntennaPowerSet(feIdx, FFALSE);
    }

    (void )SETTINGS_UserDatSet (0, ((uint8_t)SETTINGS_DEFAULT_TRANSPARENCY << 24) |
                                   ((uint8_t)SETTINGS_DEFAULT_COLSCHEME    << 16) |
                                   (SETTINGS_DEFAULT_VER_ALIGN             <<  8) |
                                   (SETTINGS_DEFAULT_HOR_ALIGN             <<  8) |
                                   (DEFAULT_RESET_FACTORY_FLAG));

    (void)SETTINGS_UserDatSet (1, (SETTINGS_DEFAULT_PVRJUMPTIME << 24));

    if ( ( SYS_GetConfig()->appProfile == SYS_APP_PROFILE_UKDTT) ||
         ( SYS_GetConfig()->appProfile == SYS_APP_PROFILE_UKDTT_T2 ) ||
         ( SYS_GetConfig()->appProfile == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW) )
    {
        (void) SETTINGS_AutoTimeOffsetSet(FTRUE, CTRY_UNITED_KINGDOM);
        for(cnt = 0; cnt < SETTINGS_AUDIOLANGPREF_MAX; cnt++)
        {
            (void) SETTINGS_AudioLangPrefSet(cnt, LANG_ENGLISH);
        }

        for ( cnt = 0; cnt < SETTINGS_UILANGPREF_MAX; cnt++ )
        {
            (void) SETTINGS_UILangPrefSet(cnt, LANG_ENGLISH);
        }

        for ( cnt = 0; cnt < SETTINGS_SUBTLANGPREF_MAX; cnt++ )
        {
            (void) SETTINGS_SubtLangPrefSet(cnt, LANG_ENGLISH);
        }
    }
    else
    {
        (void) SETTINGS_AutoTimeOffsetSet(SETTINGS_DEFAULT_AUTOTIME_OFFSET, SETTINGS_DEFAULT_AUTOTIME_OFFSET_CTRY);
        for(cnt = 0; cnt < SETTINGS_AUDIOLANGPREF_MAX; cnt++)
        {
            (void) SETTINGS_AudioLangPrefSet(cnt, SETTINGS_DEFAULT_AUD_LANG);
        }

        for ( cnt = 0; cnt < SETTINGS_UILANGPREF_MAX; cnt++ )
        {
            (void) SETTINGS_UILangPrefSet(cnt, SETTINGS_DEFAULT_UI_LANG);
        }

        for ( cnt = 0; cnt < SETTINGS_SUBTLANGPREF_MAX; cnt++ )
        {
            (void) SETTINGS_SubtLangPrefSet(cnt, SETTINGS_DEFAULT_SUB_LANG);
        }
    }
}

FSTATIC void MAIN_SetUsalsDefaults (void)
{
    #ifdef APPL_USALS_ENABLED

    SYS_GEO_POS_S localPos;

    localPos.longitude   = 85;  /* 8.5 deg. East */
    localPos.latitude    = 500; /* 50 deg. North */

    (void)USALS_SetGeoPosition(&localPos);

    #endif /* APPL_USALS_ENABLED */
}


FSTATIC void MAIN_SetRfModDefaults (void)
{
    #ifdef APPL_RFMOD_ENABLED
    RFMOD_EXCHG_PARAMS_S rfmodExchgParams;

    /* set user defined Parameters for one modulator*/
    /* for second modulator index would be 2 */
    /* at this time the modulator is not intialized and opened, therefor a special */
    /* indication is required */
    rfmodExchgParams.rfmod_index = 1;
    rfmodExchgParams.rfmod_TVSystem = RF_MOD_PALSEC_BGH;
    rfmodExchgParams.rfmod_channel = 33;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_PALSEC_BGH].rfmodTvStdStored = RF_MOD_PALSEC_BGH;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_PALSEC_BGH].rfmodChannelStored = 33;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_PALSEC_B].rfmodTvStdStored = RF_MOD_PALSEC_B;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_PALSEC_B].rfmodChannelStored = 33;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_PAL_I].rfmodTvStdStored = RF_MOD_PAL_I;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_PAL_I].rfmodChannelStored = 33;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_PALSEC_DKL].rfmodTvStdStored = RF_MOD_PALSEC_DKL;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_PALSEC_DKL].rfmodChannelStored = 33;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_NTSC_MN].rfmodTvStdStored = RF_MOD_NTSC_MN;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_NTSC_MN].rfmodChannelStored = 33;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_NTSC_M].rfmodTvStdStored = RF_MOD_NTSC_M;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_NTSC_M].rfmodChannelStored = 33;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_OFF].rfmodTvStdStored = RF_MOD_OFF;
    /* indicate with channel 255 that user wishes to set defaults */
    rfmodExchgParams.rfmod_std_ch[RF_MOD_OFF].rfmodChannelStored = 255;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_TEST_SIGNAL].rfmodTvStdStored = RF_MOD_TEST_SIGNAL;
    rfmodExchgParams.rfmod_std_ch[RF_MOD_TEST_SIGNAL].rfmodChannelStored = 33;

    /* transfer user default parameters */
    RFMOD_SetParams (&rfmodExchgParams);

    #endif /* APPL_RFMOD_ENABLED */
}

/* add default channel/transponder lists */
FSTATIC void MAIN_SetCarrierDefaults (const SYS_NETWORK_S* network)
{
    switch ( network->type )
    {
        #ifdef APPL_FESAT_ENABLED

        case FE_SYSTEM_TYPE_SATELLITE:
            MAIN_GenerateSatCarrierList(network);
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case FE_SYSTEM_TYPE_TERRESTRIAL:
            MAIN_GenerateTerrChannelList(CHANREGION_DEFAULT);
            //appDat->cfg->region = CHANREGION_DEFAULT;
            break;

        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED

        case FE_SYSTEM_TYPE_CABLE:
            MAIN_GenerateCableChannelList(CHANREGION_DEFAULT);
            //appDat->cfg->region = CHANREGION_DEFAULT;
            break;

        #endif /* APPL_FECABLE_ENABLED */

        case FE_SYSTEM_TYPE_DIRECT:
            /* no carrier */
            break;

        default:
            break;
    }
}

FSTATIC void MAIN_SetSatDefaults (void)
{
    #ifdef APPL_FESAT_ENABLED

    CFGSAT_SAT_S        satInf;
    CFGSAT_FEPARS_S     fePars;
    uint8_t             feCnt;
    int16_t             newPos;
    int16_t             satCnt;
    int16_t             numSats = 0;

    /* Set frontend configuration settings */
    memset(&fePars, 0, sizeof(fePars));

    fePars.installType = CFGSAT_INSTALLTYPE_INDEPENDENT;

    for ( feCnt = 0; feCnt < RSRC_FE_MAX; feCnt++ )
    {
        fePars.controlMode[feCnt] = CFGSAT_FEMODE_DISEQC10;
        fePars.ifFreq             = 1284;//default first IF freq.
    }

    (void)CFGSAT_SetFrontendSetup(&fePars);

    /* ----- Add all satellites -------------------------------------------- */

    for ( satCnt = 0; satCnt < (int16_t)SATID_NUMS; satCnt++ )
    {
        memset(&satInf, 0, sizeof(satInf));

        for ( feCnt = 0; feCnt < RSRC_FE_MAX; feCnt++ )
        {
            satInf.lnbCfg[feCnt].active = FFALSE;
        }

        satInf.id                      = (uint32_t)satCnt;
        satInf.nameCharSet             = SYS_CHARSET_ASCII;
        satInf.active                  = FTRUE;
        satInf.orbitalPos.degrees      = satLst[satCnt].orbitalPosition;
        satInf.orbitalPos.westEastFlag = satLst[satCnt].westEastFlag;

        (void)UTILS_strncpy(satInf.name, satLst[satCnt].name, CFGSAT_SATNAME_SZ_MAX);
        (void)CFGSAT_AddSat(&satInf, SYS_OWNER_SYSTEM, &newPos);
    }

    /* ----- Set default LNB configuration --------------------------------- */

    (void)CFGSAT_GetNumSats (&numSats);

    for ( satCnt = 0; satCnt < numSats; satCnt++ )
    {
        if ( CFGSAT_GetSatData (satCnt, &satInf) != FAPI_OK ) continue;

        /* default configuration */
        satInf.lnbCfg[0].active           = FTRUE;
        satInf.lnbCfg[0].freqLo           =  9750000;
        satInf.lnbCfg[0].freqHi           = 10600000;
        satInf.lnbCfg[0].toneburstId      = 0;
        satInf.lnbCfg[0].diseqc10Id       = 0;
        satInf.lnbCfg[0].diseqc11Id       = 0;
        satInf.lnbCfg[0].diseqc11Id_SMATV = 0;
        satInf.lnbCfg[0].diseqc11Rep      = 1;
        satInf.lnbCfg[0].diseqc12Pos      = 0;
        satInf.lnbCfg[0].usalsAvl         = FFALSE;

        switch ( satInf.id )
        {
            case SATID_HOTBIRD: // Ku-band
                satInf.lnbCfg[0].toneburstId      = 1;
                satInf.lnbCfg[0].diseqc10Id       = 1;
                satInf.lnbCfg[0].diseqc11Id       = 1;
                satInf.lnbCfg[0].diseqc11Id_SMATV = 1;
                satInf.lnbCfg[0].diseqc12Pos      = 0;
                satInf.lnbCfg[0].usalsAvl         = FFALSE;
                break;

            case SATID_ASTRA:           // Ku-band
            case SATID_EUTELSAT_W2:     // Ku-band
            case SATID_EUTELSAT_W1:     // Ku-band
            case SATID_EUTELSAT_W3A:    // Ku-band
            case SATID_SIRIUS:          // Ku-band
            case SATID_TELECOM_2C:      // C-band & Ku-band
            case SATID_INTELSAT_10_02:  // C-band & Ku-band
            case SATID_ATLANTIC_BIRD_3: // Ku-band
            case SATID_ATLANTIC_BIRD_2: // Ku-band
            case SATID_INTELSAT_907:    // C-band & Ku-band
            case SATID_HISPASAT_1C_1D:  // Ku-band
            case SATID_ASTRA_1D_3A:     // Ku-band
            case SATID_ARABSAT:         // C-band & Ku-band
            case SATID_EUROBIRD_1:      // Ku-band
            case SATID_ASTRA_2:         // Ku-band
            case SATID_ARABSAT_2B:      // C-band & Ku-band
            case SATID_EUROBIRD_3:      // Ku-band
            case SATID_TURKSAT_1C:      // Ku-band
            case SATID_EUTELSAT_SESAT:  // Ku-band
            case SATID_PAS_1_R:         // Ku-band
            case SATID_THOR:
            case SATID_NILESAT:         // Ku-band
            case SATID_ATLANTIC_BIRD_1: // Ku-band
                /* keep default settings */
                break;

            default:
                /* ignore unknown satellites */
                continue;
        }

        satInf.lnbCfg[1] = satInf.lnbCfg[0];

        (void)CFGSAT_ReplaceSat (satCnt, &satInf, SYS_OWNER_SYSTEM);
    }

    #endif /* APPL_FESAT_ENABLED */
}

FSTATIC void MAIN_SetServListDefaults (SLIST_TYPE listId)
{
    SLIST_PARAMS_S slistParams;

    switch ( listId )
    {
        case SLIST_TV:
            (void)SLIST_ListNameSet(listId, OSD_GetStr((uint32_t)STR_ENTRY_SERVLIST_GLOBTV));
            break;

        case SLIST_RADIO:
            (void)SLIST_ListNameSet(listId, OSD_GetStr((uint32_t)STR_ENTRY_SERVLIST_GLOBRADIO));
            break;

        case SLIST_INVISIBLE:
            /* do not assign a name*/
            break;

        default:
            if ( (int16_t)listId >= SLIST_OFFSET_USR )
            {
                sprintf (STR_IntASCIIBuf(), "Favorites %d",
                        (listId - SLIST_OFFSET_USR) + 1);

                (void)SLIST_ListNameSet (listId, STR_IntASCIIStr());
            }
            break;
    }

    slistParams.sortByLcn          = SLIST_DEFAULT_LCN_STATUS;
    slistParams.variantRangeStart  = SLIST_DEFAULT_LCN_VARIANT_START;
    slistParams.variantRangeLength = SLIST_DEFAULT_LCN_VARIANT_LENGTH;
    slistParams.useLcnVisibility   = SLIST_DEFAULT_LCN_USEVISIBILITY;

    (void)SLIST_SetParams (&slistParams);
}

#ifdef APPL_HDD_ENABLED

FSTATIC void MAIN_SetHddDefaults (void)
{
    (void)HDD_SetSpindownTime (PVR_DEFAULT_SPINDOWN_TIME);
}

#endif /* APPL_HDD_ENABLED */

#ifdef APPL_PVR_ENABLED

FSTATIC void MAIN_SetPvrDefaults (void)
{
    /* currently unused */
}

#endif /* APPL_PVR_ENABLED */

/* ------------------------------------------------------------------------- */

#ifdef APPL_PARENTAL_ENABLED

FSTATIC void MAIN_SetPctrlDefaults (void)
{
    char_t           pinMaster[] = { '1', '2', '3', '4' };
    char_t           pinSingle[] = { '0', '0', '0', '0' };
    PCTRL_PARAMS_S   pctrlParams;

    (void)PCTRL_SetPin (PCTRL_PIN_MASTER, pinMaster);
    (void)PCTRL_SetPin (PCTRL_PIN_SINGLE, pinSingle);

    if ( ( SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT ) ||
         ( SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2 ) )
    {
        pctrlParams.country = CTRY_UNITED_KINGDOM;
    }
    else if ( SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW )
    {
        pctrlParams.country = CTRY_AUSTRALIA;
    }
    else
    {
        pctrlParams.country = CTRY_GERMANY;
    }
    pctrlParams.useStreamRating = FFALSE;
    pctrlParams.viewerAge       = 18;
    pctrlParams.protectionArea  = 0;

    (void)PCTRL_SetParams(&pctrlParams);
}

FSTATIC void MAIN_PctrlHandleEvent (SYS_EVTTYPE_E type, const void* data)
{
    MENU_PINDIALOG_S openParams;

    MENU_InfoUpdateParentalLockStatus();

    switch ( type )
    {
        case EVT_PCTRL_NEEDAUTH:
            appDat->currAuthServLoc = ((PCTRL_EVT_NEEDAUTH_S*)data)->servLoc;

            memset(&openParams, 0, sizeof(openParams));

            openParams.titleStr       = OSD_GetStr((uint32_t)STR_MSG_PASSWORD_ENTER_PIN);
            openParams.numChars       = PCTRL_PIN_LEN;
            openParams.onPinEnter     = MAIN_PctrlOnPinEnter;
            openParams.arg            = NULL;
            openParams.keyHdlChanUp   = MAIN_ChannelUp;
            openParams.keyHdlChanDown = MAIN_ChannelDown;

            (void)MENU_PinDialogOpen(&(appDat->pinDialog), &openParams);
            break;

        case EVT_PCTRL_NOMOREAUTH:
            /* close potentially opened PIN dialogue */
            (void)MENU_PinDialogClose(&(appDat->pinDialog));
            break;

        default:
            break;
    }
}

void MAIN_PctrlOnPinEnter (void* arg, const char_t* pin, uint8_t numChars)
{
    int32_t retVal;

    retVal = PCTRL_AuthenticateService (&(appDat->currAuthServLoc), pin);

    if ( retVal == FAPI_OK )
    {
        (void)MENU_PinDialogClose(&(appDat->pinDialog));
    }
    else
    {
        (void)MENU_PinDialogReset(&(appDat->pinDialog));
    }
}

#endif /* APPL_PARENTAL_ENABLED */

#ifdef APPL_SCRNSHOT_ENABLE

FSTATIC fbool_t MAIN_ScreenShot (FAPI_SYS_HandleT h)
{
    fbool_t              spinUpShow = FFALSE;
    SCREENCAPTURE_MODE_E mode;
    SYS_SDTV_STANDARD    sdSys;
    SYS_DEVICE_E         device;
    STR_IDX_S            message = STR_EMPTYSTRING;
    fbool_t              permanent = FFALSE;

    do
    {
        if ( SETTINGS_ScreenShotToGet(&device) != FAPI_OK )
        {
            break;
        }

        if(!MAIN_ScreenShotDevTest(device, &message))
        {

            if(device == SYS_DEV_HDD)
            {
                MENU_CommonMsgShow(message, permanent);
                break;
            }
            /* change to HDD as a fallback device if the first device fails*/
            else
            {
                device = SYS_DEV_HDD;
                if(!MAIN_ScreenShotDevTest(device, NULL))
                {
                    /* show error message of the first device */
                    MENU_CommonMsgShow(message, permanent);
                    break;
                }
                else
                {
                    /* continue */
                }
            }
        }

        if (( SETTINGS_ScreenShotModeGet(&mode) != FAPI_OK ) ||
            ( SETTINGS_OutputTvStandardGet(NULL, &sdSys) != FAPI_OK ))
        {
            break;
        }

        (void)SYS_DevicePreAccess (device);

        if(spinUpShow == FTRUE)
        {
            (void)MAIN_MsgBoxHide(&(appDat->msgBox)); // hide spin up msg
            spinUpShow = FFALSE;
        }

        if ( SYS_CaptureScreen(device, MAIN_DIR_HDD_CAPTURE, mode, sdSys,
                               NULL, NULL) == FAPI_OK )
        {
            switch(device)
            {
                case SYS_DEV_HDD:
                    message = STR_MSG_SCREENSAVED_HDD;
                    break;
                case SYS_DEV_USB:
                    message = STR_MSG_SCREENSAVED_USB;
                    break;
                default:
                    break;
            }
        }
        else
        {
            message = STR_MSG_SCREENFAILED;
        }

        MENU_CommonMsgShow(message, FFALSE);

    } while ( 0 );

    /* hide spin up msg in case error occurs */
    if(spinUpShow == FTRUE)
    {
        (void)MAIN_MsgBoxHide(&(appDat->msgBox)); // hide spin up msg
    }

    return FTRUE;
}

FSTATIC fbool_t MAIN_ScreenShotDevTest(SYS_DEVICE_E device, STR_IDX_S* errMsg)
{
    fbool_t             retVal = FTRUE;
    int32_t             errCode;
    SYS_DEV_PWRMODE_E   devPwrState;
    SYS_DEV_STAT_S      devStat;
    STR_IDX_S           message = STR_EMPTYSTRING;

    do
    {
        /* USB has no power mode*/
        errCode = SYS_DeviceGetPowerMode(device,&devPwrState);
        if( (errCode     == FAPI_OK ) &&
        (devPwrState == SYS_DEV_PWRMODE_STANDBY) )
        {
            message   = STR_MSG_HDD_SPINUP;
        }

        /* test if device is available */
        if ( SYS_DeviceGetStatus (device, &devStat) != FAPI_OK )
        {
            retVal = FFALSE;

            switch(device)
            {
                case SYS_DEV_HDD:
                    message   = STR_MSG_NOHDD;
                    break;
                case SYS_DEV_USB:
                    message = STR_MSG_USBMSC_DISCONNECT;
                    break;
                default:
                    message = STR_EMPTYSTRING;
                    break;
            }
            break;
        }

        if ( devStat.mountStatus != SYS_DEV_FS_MOUNTED )
        {
            retVal = FFALSE;

            switch(device)
            {
                case SYS_DEV_HDD:
                    message   = STR_MSG_NOHDD;
                    break;
                case SYS_DEV_USB:
                    message = STR_MSG_USBMSC_DISCONNECT;
                    break;
                default:
                    message = STR_EMPTYSTRING;
                    break;
            }
            break;
        }

        if (SYS_DeviceGetFreekBytes(device) <= 1200)//min size for a screenshot
        {
            retVal = FFALSE;

            switch(device)
            {
                case SYS_DEV_HDD:
                    message   = STR_MSG_HDD_NOT_ENOUGHT_SPACE;
                    break;
                case SYS_DEV_USB:
                    message = STR_MSG_USB_NOT_ENOUGHT_SPACE;
                    break;
                default:
                    message = STR_EMPTYSTRING;
                    break;
            }
            break;
        }

    } while ( 0 );

    if(errMsg != NULL)
    {
        *errMsg = message;
    }

    return retVal;
}

#endif /* APPL_SCRNSHOT_ENABLE */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
