/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_antenna_program.c
**
**              $Id: app_menu_antenna_program.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD Antenna and Programm setup menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_ANTPROG_ID_TITLE = 0,
    MENU_ANTPROG_ID_LIST,
    MENU_ANTPROG_ID_HELPLIST
};

enum
{
    MENU_ANTPROG_ID_ENTRY_GUID = 0,

    /* empty line is needed though not referenced.
     * TODO need a way to switch off the MISRA warning.
     */
    MENU_ANTPROG_ID_ENTRY_EMPTY,

    #ifdef APPL_FESAT_ENABLED
    MENU_ANTPROG_ID_ENTRY_SATINST,
    MENU_ANTPROG_ID_ENTRY_SATLIST,
    #endif

    MENU_ANTPROG_ID_ENTRY_TUNER,
    MENU_ANTPROG_ID_ENTRY_TPLIST,

    #ifdef APPL_FESAT_ENABLED
    MENU_ANTPROG_ID_ENTRY_SCAN_SAT,
    #endif

    #ifdef APPL_FETERR_ENABLED
    MENU_ANTPROG_ID_ENTRY_SCAN_TERR,
    #endif

    #ifdef APPL_FECABLE_ENABLED
    MENU_ANTPROG_ID_ENTRY_SCAN_CABLE,
    #endif

    #ifdef APPL_FEDIRECT_ENABLED
    MENU_ANTPROG_ID_ENTRY_SCAN_DIRECT,
    #endif

    MENU_ANTPROG_ID_ENTRY_SERVLIST,
    MENU_ANTPROG_ID_ENTRY_NUMS
};

typedef struct
{
    FAPI_SYS_HandleT       pnlHdl;
    FAPI_SYS_HandleT       listHdl;
    FAPI_SYS_HandleT       listHelpHdl;

} MENU_ANTPROG_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC MENU_ANTPROG_INST_S*  menuAntProgDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_AntProgLeave            (FAPI_SYS_HandleT h,
                                              fbool_t force);
FSTATIC fbool_t MENU_AntProgOnSelect         (FAPI_SYS_HandleT h,
                                              int32_t itemId);
FSTATIC void    MENU_AntProgGetDescrItem     (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC fbool_t MENU_AntProgBuild            (FAPI_SYS_HandleT h);
FSTATIC void    MENU_AntProgOnFocusChg       (FAPI_SYS_HandleT h,
                                              uint8_t index,
                                              int32_t id);
FSTATIC void    MENU_AntProgOnFocusSet       (FAPI_SYS_HandleT h,
                                              fbool_t hasFocus);
FSTATIC void    MENU_AntProgGetHelpDescrItem (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
/*!
******************************************************************************
** \brief   TODO
**
** \return  one of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_ALREADY_INITIALIZED
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
******************************************************************************
*/
int32_t MENU_AntProgInit (void)
{
    if ( menuAntProgDat != NULL)
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    menuAntProgDat = UTILS_Malloc(sizeof(MENU_ANTPROG_INST_S));
    if ( menuAntProgDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(menuAntProgDat, 0, sizeof(MENU_ANTPROG_INST_S));

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   With this function you enters the Antenna and Programm setup menu.
**
** This function is called twice. First with \p test flag #FTRUE to check if
** the menu can be opend. If the first call was successful and return #FTRUE
** the second call will open the menu (\p test flag #FFALSE).
**
** \param   h       Handle of Menu Instants.
** \param   test    A flag to test if opening this menu is possible.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
fbool_t MENU_AntProgEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_ANTPROG_INST_S* inst = menuAntProgDat;
    uint32_t             lastFocusItem = MENU_ANTPROG_ID_ENTRY_EMPTY;

    // TODO: return FFALSE if no FEs have been found

    if ( test == FTRUE ) return FTRUE;

    (void)MENU_AntProgBuild((FAPI_SYS_HandleT)inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = MENU_ANTPROG_ID_ENTRY_GUID;
    }

    (void)MAIN_StopPresentation(); /* stop running presentation */

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, (int32_t)lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_ANTPROG_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_ANTPROG);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); // turn visibility on

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_AntProgLeave,
                         (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

FSTATIC void MENU_AntProgOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_ANTPROG_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);
}

FSTATIC void MENU_AntProgOnFocusChg (FAPI_SYS_HandleT h,
                                     uint8_t index,
                                     int32_t id)
{
    MENU_ANTPROG_INST_S*   inst = (MENU_ANTPROG_INST_S*)h;
    FGS_LNAVI_PARAMS_S     naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}

FSTATIC fbool_t MENU_AntProgLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_ANTPROG_INST_S*    inst = (MENU_ANTPROG_INST_S*)h;

    /* start last known service and clear AntProgParameters if antenna menu
     * is left.
     */
    if(FGS_GetCurrentKeyCode() != FGS_KEY_OK)
    {
        (void)MAIN_RestartPresentation();
    }

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_ANTPROG);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC fbool_t MENU_AntProgOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENUSTACK_ENTRY       newEntry = NULL;
    FAPI_SYS_HandleT      arg      = NULL;
    MENU_GUIDED_PARAMS_S  menuParams;
    MENU_TPLIST_PARAMS_S  tplistPars;
    SYS_FE_STATUS_S       feStatus;
    uint8_t               feIdx;

    memset(&menuParams, 0, sizeof(menuParams));

    menuParams.guided = FFALSE;

    switch ( itemId )
    {
        case MENU_ANTPROG_ID_ENTRY_GUID:
            return MENU_GuidedStart ();

        #ifdef APPL_FESAT_ENABLED

        case MENU_ANTPROG_ID_ENTRY_SATINST:
            arg        = (FAPI_SYS_HandleT)(&menuParams);
            newEntry   = MENU_SatInstTypeEntry;
            break;

        case MENU_ANTPROG_ID_ENTRY_SATLIST:
            arg        = (FAPI_SYS_HandleT)(&menuParams);
            newEntry   = MENU_SatListEntry;
            break;

        #endif /* APPL_FESAT_ENABLED */

        case MENU_ANTPROG_ID_ENTRY_TUNER:
            /* look for first FE that requires a tuner setup menu */
            for ( feIdx = 0; feIdx < appDat->hwCfg->feNums; feIdx++ )
            {
                if ( (AV_GetFeStatus(feIdx, &feStatus) == FAPI_OK) &&
                     ( (feStatus.type == FE_SYSTEM_TYPE_SATELLITE) ||
                       (feStatus.type == FE_SYSTEM_TYPE_TERRESTRIAL) ) )
                {
                    menuParams.arg1 = feIdx; /* select first frontend/tuner */
                    break;
                }
            }

            /* get last ID of FE that require tuner setup menu */
            feIdx = appDat->hwCfg->feNums;
            do
            {
                feIdx--;

                if ( (AV_GetFeStatus(feIdx, &feStatus) == FAPI_OK) &&
                     ( (feStatus.type == FE_SYSTEM_TYPE_SATELLITE) ||
                       (feStatus.type == FE_SYSTEM_TYPE_TERRESTRIAL) ) )
                {
                    menuParams.arg2 = feIdx; /* select last frontend/tuner */
                    break;
                }
            } while(feIdx > 0);

            arg        = (FAPI_SYS_HandleT)(&menuParams);
            newEntry   = MENU_TunerEntry;
            break;

        case MENU_ANTPROG_ID_ENTRY_TPLIST:
            memset(&tplistPars, 0, sizeof(tplistPars));

            tplistPars.system        = FE_SYSTEM_TYPE_SATELLITE;
            tplistPars.satLstPos     = 0;
            tplistPars.carrierLstPos = 0;
            tplistPars.lockSat       = FFALSE;

            arg        = (FAPI_SYS_HandleT)(&tplistPars);
            newEntry   = MENU_TpListEntry;
            break;

        #ifdef APPL_FESAT_ENABLED

        case MENU_ANTPROG_ID_ENTRY_SCAN_SAT:
            arg        = (FAPI_SYS_HandleT)(&menuParams);
            newEntry   = MENU_ScanSatEntry;
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case MENU_ANTPROG_ID_ENTRY_SCAN_TERR:
            arg        = (FAPI_SYS_HandleT)(&menuParams);
            newEntry   = MENU_ScanTerrEntry;
            break;

        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED

        case MENU_ANTPROG_ID_ENTRY_SCAN_CABLE:
            arg        = (FAPI_SYS_HandleT)(&menuParams);
            newEntry   = MENU_ScanCableEntry;
            break;

        #endif /* APPL_FECABLE_ENABLED */

        #ifdef APPL_FEDIRECT_ENABLED

        case MENU_ANTPROG_ID_ENTRY_SCAN_DIRECT:
            arg        = (FAPI_SYS_HandleT)(&menuParams);
            newEntry   = MENU_ScanDirectEntry;
            break;

        #endif /* APPL_FECABLE_ENABLED */

        case MENU_ANTPROG_ID_ENTRY_SERVLIST:
            arg        = (FAPI_SYS_HandleT)(&menuParams);
            newEntry   = MENU_ServListEntry;
            break;

        default:
            MENU_CommonMsgShow(STR_MSG_NOTIMPLEMENTEDYET, FFALSE);
            return FFALSE;
    }

    /* store last focus */
    MENUSTACK_SetData(itemId);

    if ( newEntry == NULL ) return FTRUE;
    else                    return MENUSTACK_Up(newEntry, arg);
}

FSTATIC void MENU_AntProgGetDescrItem (FAPI_SYS_HandleT h,
                                       int32_t id,
                                       uint32_t index,
                                       FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item)
{
    uint32_t    strIdx;

    switch ( id )
    {
        case MENU_ANTPROG_ID_ENTRY_GUID:
            strIdx = STR_ENTRY_ANTPROG_GUID;
            break;
        case MENU_ANTPROG_ID_ENTRY_TUNER:
            strIdx = STR_ENTRY_ANTPROG_TUNER;
            break;
        case MENU_ANTPROG_ID_ENTRY_TPLIST:
            strIdx = STR_ENTRY_ANTPROG_TPLIST;
            break;

        #ifdef APPL_FESAT_ENABLED
        case MENU_ANTPROG_ID_ENTRY_SATINST:
            strIdx = STR_ENTRY_ANTPROG_SATINST;
            break;
        case MENU_ANTPROG_ID_ENTRY_SATLIST:
            strIdx = STR_ENTRY_ANTPROG_SATLIST;
            break;
        case MENU_ANTPROG_ID_ENTRY_SCAN_SAT:
            strIdx = STR_ENTRY_ANTPROG_SCAN_SAT;
            break;
        #endif

        #ifdef APPL_FETERR_ENABLED
        case MENU_ANTPROG_ID_ENTRY_SCAN_TERR:
            strIdx = STR_ENTRY_ANTPROG_SCAN_TERR;
            break;
        #endif

        #ifdef APPL_FECABLE_ENABLED
        case MENU_ANTPROG_ID_ENTRY_SCAN_CABLE:
            strIdx = STR_ENTRY_ANTPROG_SCAN_CABLE;
            break;
        #endif

        #ifdef APPL_FEDIRECT_ENABLED
        case MENU_ANTPROG_ID_ENTRY_SCAN_DIRECT:
            strIdx = STR_ENTRY_ANTPROG_SCAN_DIRECT;
            break;
        #endif

        case MENU_ANTPROG_ID_ENTRY_SERVLIST:
            strIdx = STR_ENTRY_ANTPROG_SERVLIST;
            break;

        default:
            return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_AntProgGetHelpDescrItem (FAPI_SYS_HandleT h,
                                           int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        case MENU_ANTPROG_ID_ENTRY_GUID:
        case MENU_ANTPROG_ID_ENTRY_TUNER:
        case MENU_ANTPROG_ID_ENTRY_TPLIST:

        #ifdef APPL_FESAT_ENABLED
        case MENU_ANTPROG_ID_ENTRY_SATINST:
        case MENU_ANTPROG_ID_ENTRY_SATLIST:
        case MENU_ANTPROG_ID_ENTRY_SCAN_SAT:
        #endif

        #ifdef APPL_FETERR_ENABLED
        case MENU_ANTPROG_ID_ENTRY_SCAN_TERR:
        #endif

        #ifdef APPL_FECABLE_ENABLED
        case MENU_ANTPROG_ID_ENTRY_SCAN_CABLE:
        #endif

        #ifdef APPL_FEDIRECT_ENABLED
        case MENU_ANTPROG_ID_ENTRY_SCAN_DIRECT:
        #endif

        case MENU_ANTPROG_ID_ENTRY_SERVLIST:
            strIdx = STR_ENTRY_OK;
            break;

        default:
            return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC fbool_t MENU_AntProgBuild(FAPI_SYS_HandleT h)
{
    MENU_ANTPROG_INST_S*    inst = (MENU_ANTPROG_INST_S*)h;
    FGS_PANEL_OPEN_S        pnlPars;
    FGS_LIST_OPEN_S         listPars;
    FAPI_SYS_HandleT        itemHdl;
    FGS_ITEM_TXT_OPEN_S     txtPars;
    FGS_POS_S               itemPos;
    FGS_POS_S               descrPos;
    int32_t                 retVal;
    uint16_t                helpWidth;
    uint16_t                stackedFrameWidth;
    uint16_t                innerHeight;
    uint32_t                numLines = MENU_ANTPROG_ID_ENTRY_NUMS;

    memset(&txtPars,  0, sizeof(txtPars));

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_OK),
                                   -1,SYMBOL_FONT);
    innerHeight       = (numLines) * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                        SCALER_GetCurrVal(LIST_LINE_PADDING));
    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth,
                                        1, &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_AntProgOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_ANTPROG, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create box holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_ANTPROG_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_ANTPROG,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_AntProgGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_AntProgOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_AntProgOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = NULL;
        listPars.onValChgArg     = NULL;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_ANTPROG_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, NULL, 100,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- menu lines ------------------------------------------------ */

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.pos                = itemPos;

        /* --- guided installation ------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_GUID,
                                inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);
        // add an empty line
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        #ifdef APPL_FESAT_ENABLED

        /* --- Front end install ------------------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_SATINST,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* check if any Sat FE was detected */
        if ( appDat->numSatFe == 0 )
        {
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_ANTPROG_ID_ENTRY_SATINST,
                                            FFALSE);
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Sat list editor --------------------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_SATLIST,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* check if any Sat FE was detected */
        if ( appDat->numSatFe == 0 )
        {
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_ANTPROG_ID_ENTRY_SATLIST,
                                            FFALSE);
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_FESAT_ENABLED */

        /* --- Tuner ----------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_TUNER,
                                inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* check if any Sat or Terr FE was detected */
        if ( appDat->numSatFe == 0 && appDat->numTerrFe == 0 )
        {
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_ANTPROG_ID_ENTRY_TUNER,
                                            FFALSE);
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Transponder list --------------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_TPLIST,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* check if any Sat FE was detected */
        if ( appDat->numSatFe == 0 )
        {
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_ANTPROG_ID_ENTRY_TPLIST,
                                            FFALSE);
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        #ifdef APPL_FESAT_ENABLED

        /* ----- scan menu for satellite ---------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_SCAN_SAT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* check if any Sat FE was detected */
        if ( appDat->numSatFe == 0 )
        {
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_ANTPROG_ID_ENTRY_SCAN_SAT,
                                            FFALSE);
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        /* ----- scan menu for terrestrial -------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_SCAN_TERR,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* check if any Terrestrial FE was detected */
        if( appDat->numTerrFe == 0 )
        {
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_ANTPROG_ID_ENTRY_SCAN_TERR,
                                            FFALSE);
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);
        #endif

        #ifdef APPL_FECABLE_ENABLED
        /* ----- scan menu for cable -------------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_SCAN_CABLE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* check if any Cable FE was detected */
        if ( appDat->numCableFe == 0 )
        {
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_ANTPROG_ID_ENTRY_SCAN_CABLE,
                                            FFALSE);
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);
        #endif

        #ifdef APPL_FEDIRECT_ENABLED
        /* ----- scan menu for direct ts ------------------------------------ */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_SCAN_DIRECT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* check if any Cable FE was detected */
        if ( appDat->numDirectTs == 0 )
        {
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_ANTPROG_ID_ENTRY_SCAN_DIRECT,
                                            FFALSE);
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);
        #endif

        /* -----Service List editor --------------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_ANTPROG_ID_ENTRY_SERVLIST,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_ANTPROG_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH),
                                        listPars.pos.y,
                                        helpWidth,
                                        numLines,  MENU_AntProgGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(
                            stackedFrameWidth +
                            SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                            SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                            - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                            SCALER_GetCurrVal(HELP_TB_PADDING) -
                            SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                            pnlPars.pos.width -
                            stackedFrameWidth -
                            2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                            SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                            SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                            inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

        FAPI_SYS_FREE(inst);
        return FFALSE;
    }

    return FTRUE;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
