/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_audio_video.c
**
**              $Id: app_menu_audio_video.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD audio/video menu.
**
** Detail description. [\ref app_smartgo_app_ref_advd]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info 715: Symbol not referenced.
**
*****************************************************************************
*/
/*lint -save -e641 -e715 -e774 -e571*/
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#ifdef APPL_RFMOD_ENABLED
//    #define _RFMOD_REACT_ON_ERROR_    //use if menu should react on error return values
    #define _RFMOD_LOCK_MENU_ON_ERROR_  //use if menu should lock if error encountered
#endif

#define LIPSYNC_MIN_DELAY  -1000 // ms
#define LIPSYNC_MAX_DELAY  1000  // ms

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_ADVD_ID_TITLE = 0,
    MENU_ADVD_ID_LIST,
    MENU_ADVD_ID_REST,
    MENU_ADVD_ID_HELPLIST
};

enum
{
    MENU_ADVD_ID_ENTRY_FORMAT = 0,
    MENU_ADVD_ID_ENTRY_SCALING,
    MENU_ADVD_ID_ENTRY_HD,
    MENU_ADVD_ID_ENTRY_SD,
    MENU_ADVD_ID_ENTRY_VOUT,

    #ifdef APPL_SCRNSHOT_ENABLE
    MENU_ADVD_ID_ENTRY_SCREENSHOT,
    MENU_ADVD_ID_ENTRY_SCREENSHOTTO,
    #endif /* APPL_SCRNSHOT_ENABLE */

    //MENU_ADVD_ID_ENTRY_STARTMODE,
    //MENU_ADVD_ID_ENTRY_SWITCHMODE,
    //MENU_ADVD_ID_ENTRY_VIDEOFADE,
    MENU_ADVD_ID_ENTRY_AOUT,
    MENU_ADVD_ID_ENTRY_LIPSYNCMODE,
    MENU_ADVD_ID_ENTRY_LIPSYNCLEVEL,

    #ifdef APPL_RFMOD_ENABLED

    MENU_ADVD_ID_ENTRY_RFMODCH,
    MENU_ADVD_ID_ENTRY_RFTVSTD,

    #endif /* APPL_RFMOD_ENABLED */

    MENU_ADVD_ID_ENTRY_NUMS
};

enum
{
    MENU_ADVD_SCALE_PILLAR_LETTERBOX = 0,
    MENU_ADVD_SCALE_FULLSCREEN,
    MENU_ADVD_SCALE_CCO,
    MENU_ADVD_SCALE_TVSCALES
};

typedef struct
{
    SYS_DEVICE_E        device;
    fbool_t             available;

} MENU_ADVD_DEV_S;

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    listHdl;
    FAPI_SYS_HandleT    listHelpHdl;

    #ifdef APPL_RFMOD_ENABLED
    uint8_t           currRfCh;              //!< currently used RF Channel
    uint8_t           currTvStd;             //!< currently used RF TV Standard
    uint8_t           base_channel;          //!< lowest channel number in current TV Standard
    uint8_t           last_channel;          //!< last channel used in current TV Standard
    fbool_t        rfmodLockMenueAppl;    //!< flag indicating to lock menu on rfmod error

    #endif /* APPL_RFMOD_ENABLED */

    FGS_PANEL_MEMBER_S* pItem;
    FGS_POS_S           itemPos;

    uint32_t            lastFocusItem;

    #ifdef APPL_SCRNSHOT_ENABLE

    MENU_ADVD_DEV_S     devList[SYS_DEV_NUMS]; //!< List of (currently)
                                                //!< available devices.
    uint32_t            devNums;                //!< Number of devices in list.
    uint32_t            devFirstIdx;            //!< Initial list index.

    #endif /* APPL_SCRNSHOT_ENABLE */

}MENU_ADVD_INST_S;

FSTATIC MENU_ADVD_INST_S* menuAudioVideoDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

// MB: move below var into above structure

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_AudioVideoLeave        (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_AudioVideoOnSelect     (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void    MENU_AudioVideoValChg       (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal);
FSTATIC void    MENU_AudioVideoGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                             FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC void    MENU_AudioVideoGetHelpDescrItem
                                            (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                             FGS_ITEMSTATE_E state,
                                             FGS_ITEM_S* item);
FSTATIC void    MENU_AudioVideoGetItem      (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                             FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC fbool_t MENU_AudioVideoBuild        (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_AudioVideoDraw         (FAPI_SYS_HandleT h);
FSTATIC void    MENU_AudioVideoOnFocusSet   (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_AudioVideoOnFocusChg   (FAPI_SYS_HandleT h, uint8_t index, int32_t id);

int32_t MENU_AudioVideoInit (void)
{
    if ( menuAudioVideoDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuAudioVideoDat = UTILS_Malloc(sizeof(MENU_ADVD_INST_S));
    if ( menuAudioVideoDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuAudioVideoDat, 0, sizeof(MENU_ADVD_INST_S));

    return FAPI_OK;
}


fbool_t MENU_AudioVideoEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_ADVD_INST_S* inst = menuAudioVideoDat;
    uint32_t               lastFocusItem;

    #ifdef APPL_RFMOD_ENABLED

    RFMOD_EXCHG_PARAMS_S  rfmodParams;
    RF_TV_SYSTEMS_S       rfchParams;

    #endif

    #ifdef APPL_SCRNSHOT_ENABLE
    uint32_t                  cnt;
    SYS_DEV_STAT_S            devStat;
    SYS_DEVICE_E              target;
    SYS_DEVICE_E              devCnt;
    #endif /* APPL_SCRNSHOT_ENABLE */

    if ( test == FTRUE ) return FTRUE;

    #ifdef APPL_RFMOD_ENABLED

    /* setup and initialize structures to store basic RF Modulator
       data required by the AV-menue */
    memset(&rfmodParams, 0, sizeof(rfmodParams));
    memset(&rfchParams, 0, sizeof(rfchParams));

    inst->rfmodLockMenueAppl = FFALSE;
    rfmodParams.rfmod_index = 1;            /* only one modulator is used right now */
    if(RFMOD_GetParams(&rfmodParams)!=FAPI_OK)
        inst->rfmodLockMenueAppl = FTRUE   ;    /* read current RF settings, on error disable menue */
    RFMOD_GetData(&rfmodParams, &rfchParams);   /* read TVSystem parameters */

    inst->currRfCh = rfmodParams.rfmod_channel;
    inst->currTvStd = rfmodParams.rfmod_TVSystem;
    inst->base_channel = rfchParams.base_channel;
    inst->last_channel = rfchParams.last_channel;

    #endif /* APPL_RFMOD_ENABLED */

    /* ----- Create list of available devices ------------------------------ */

    #ifdef APPL_SCRNSHOT_ENABLE

    inst->devNums = 0;

    /* Populate device list with available devices. In order to also show a
       target device which is currently not available (but has been selected as
       target) it is added to the list in any case, but with a flag
       indicating its availability. */

    for ( devCnt = (SYS_DEVICE_E)0; devCnt < SYS_DEV_NUMS; devCnt++ )
    {
        if ( SYS_DeviceGetStatus (devCnt, &devStat) == FAPI_OK )
        {
            if ( devStat.mountStatus == SYS_DEV_FS_MOUNTED )
            {
                inst->devList[inst->devNums].device    = devCnt;
                inst->devList[inst->devNums].available = FTRUE;

                inst->devNums++;
            }
        }
    }

    /* choose initial selection index from requested device */

    (void)SETTINGS_ScreenShotToGet(&target);

    for ( cnt = 0; cnt < inst->devNums; cnt++ )
    {
        if ( inst->devList[cnt].device == target )
        {
            inst->devFirstIdx = cnt;
            break;
        }
    }

    if ( cnt >= inst->devNums )
    {
        /* requested device is not available, add it to the list! */

        inst->devList[inst->devNums].device    = target;
        inst->devList[inst->devNums].available = FFALSE;

        inst->devFirstIdx = inst->devNums;

        inst->devNums++;
    }

    #endif /* APPL_SCRNSHOT_ENABLE */

    (void)MENU_AudioVideoBuild((FAPI_SYS_HandleT)inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = MENU_ADVD_ID_ENTRY_FORMAT;
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, (int32_t)lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_ADVD_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_ADVD);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_AudioVideoLeave, (FAPI_SYS_HandleT)inst);
    return FTRUE;
}

FSTATIC void MENU_AudioVideoOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_ADVD_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);
}

FSTATIC void MENU_AudioVideoOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_ADVD_INST_S*   inst = (MENU_ADVD_INST_S*)h;
    FGS_LNAVI_PARAMS_S  naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = (uint16_t)id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
    inst->lastFocusItem = (uint32_t)FGS_LIST_GetMemberFocusID(inst->listHdl);
}


FSTATIC fbool_t MENU_AudioVideoLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_ADVD_INST_S* inst = (MENU_ADVD_INST_S*)h;


    if ( force == FFALSE )
    {
        MAIN_SaveSettingsIfChanged();

    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease((int32_t)MENUID_ADVD);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC fbool_t MENU_AudioVideoOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_ADVD_INST_S*        inst = (MENU_ADVD_INST_S*)h;
    MENUSTACK_ENTRY          newEntry = NULL;
    FAPI_SYS_HandleT                arg = NULL;
    uint8_t                       subLines = 0;
    MENU_SELECTLIST_OPEN_S   selectLst;
    FGS_ITEM_SEL_PARAMS_S    selParams;
    fbool_t                    keyUsed = FTRUE;
    FGS_LOC_S                memberLoc;
    fbool_t                    openSelectLst = FFALSE;

    switch ( itemId )
    {
         case MENU_ADVD_ID_ENTRY_VOUT :
            openSelectLst = FTRUE;
            subLines      = 2;
        break;
        case MENU_ADVD_ID_ENTRY_FORMAT :
            openSelectLst    = FTRUE;
            subLines         = 2;
        break;
        case MENU_ADVD_ID_ENTRY_SCALING :
            openSelectLst    = FTRUE;
            subLines         = 3;
        break;
        case MENU_ADVD_ID_ENTRY_HD :
            openSelectLst      = FTRUE;
            subLines           = 3;
        break;
        case MENU_ADVD_ID_ENTRY_SD :
            openSelectLst      = FTRUE;
            subLines           = 3;
        break;

        #ifdef APPL_SCRNSHOT_ENABLE
        case MENU_ADVD_ID_ENTRY_SCREENSHOT :
            openSelectLst      = FTRUE;
            subLines           = 2;
        break;

        case MENU_ADVD_ID_ENTRY_SCREENSHOTTO :
            openSelectLst      = FTRUE;
            subLines           = 2;
        break;
        #endif /* APPL_SCRNSHOT_ENABLE */

        /*
        case MENU_ADVD_ID_ENTRY_STARTMODE :
            openSelectLst      = FTRUE;
            subLines           = 2;
        break;
        case MENU_ADVD_ID_ENTRY_SWITCHMODE :
            openSelectLst       = FTRUE;
            subLines            = 2;
        break;
        case MENU_ADVD_ID_ENTRY_VIDEOFADE :
            openSelectLst       = FTRUE;
            subLines            = 3;
        break;
        */
        case MENU_ADVD_ID_ENTRY_AOUT :
            openSelectLst  = FTRUE;
            subLines       = 2;
        break;

        case MENU_ADVD_ID_ENTRY_LIPSYNCMODE :
            openSelectLst  = FTRUE;
            subLines       = 2;
        break;
        #ifdef APPL_RFMOD_ENABLED

        case MENU_ADVD_ID_ENTRY_RFMODCH :
            break;

        case MENU_ADVD_ID_ENTRY_RFTVSTD :
            openSelectLst  = FTRUE;
            subLines       = 2;
            break;

        #endif /* APPL_RFMOD_ENABLED */

        default:
            MENU_CommonMsgShow(STR_MSG_NOTIMPLEMENTEDYET, FFALSE);
            return FFALSE;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.width     = (4 * SCALER_GetCurrVal(SUB_LIST_WIDTH)) / 3;
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.x         = (memberLoc.x + (int16_t)SCALER_GetCurrVal(MAIN_WIDTH)) -
                                  (int16_t)selectLst.pos.width;
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_AudioVideoGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;

        keyUsed = MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);

        if ( keyUsed )
        {
            (void)MAIN_SignalBoxPause(&(appDat->sigBox));
        }

        return keyUsed;
    }
    else
    {
        /* store last focus */
        MENUSTACK_SetData((uint32_t)itemId);
        if(newEntry == NULL)
            return FFALSE;
        else
            return MENUSTACK_Up(newEntry, arg);
    }
}

FSTATIC void MENU_AudioVideoValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    MENU_ADVD_INST_S*     inst        = (MENU_ADVD_INST_S*)h;
    SYS_PICFORMAT_E       format      = PICFORMAT_4_3;
    SYS_SCALEMETHOD_E     scaleMethod = SCALEMETHOD_PILLAR_LETTERBOX;
    SYS_HDTV_STANDARD     hdTvSys;
    SYS_SDTV_STANDARD     sdTvSys;

    #ifdef APPL_SCRNSHOT_ENABLE
    SCREENCAPTURE_MODE_E  scrnShotMode;
    SYS_DEVICE_E          scrnShotTo;
    #endif /* APPL_SCRNSHOT_ENABLE */

    #ifdef APPL_RFMOD_ENABLED

    MENU_ADVD_INST_S*     inst = (MENU_ADVD_INST_S*)h;
    int32_t               ferr = FAPI_OK;
    RFMOD_EXCHG_PARAMS_S  rfmodParams;
    RF_TV_SYSTEMS_S       rfchParams;
    FGS_ITEM_NUM_PARAMS_S numParams;
    FGS_ITEM_SEL_PARAMS_S selParams;
    uint8_t         j;

    memset (&rfmodParams, 0, sizeof(rfmodParams));
    memset (&rfchParams,  0, sizeof(rfchParams));

    rfmodParams.rfmod_index = 1;
    RFMOD_GetParams(&rfmodParams); /* read current RF settings */

    #endif /* APPL_RFMOD_ENABLED */

    switch ( itemId )
    {
        case MENU_ADVD_ID_ENTRY_FORMAT:
            (void)SETTINGS_DispFormatGet (&format, &scaleMethod);
            (void)SETTINGS_DispFormatSet ((SYS_PICFORMAT_E)newVal.number, scaleMethod);
            break;

        case MENU_ADVD_ID_ENTRY_SCALING:
            (void)SETTINGS_DispFormatGet (&format, &scaleMethod);
            switch ( newVal.number )
            {
                case MENU_ADVD_SCALE_PILLAR_LETTERBOX:
                    scaleMethod = SCALEMETHOD_PILLAR_LETTERBOX;
                    break;

                case MENU_ADVD_SCALE_FULLSCREEN:
                    scaleMethod = SCALEMETHOD_FULLSCREEN;
                    break;

                case MENU_ADVD_SCALE_CCO:
                    scaleMethod = SCALEMETHOD_CCO;
                    break;

                case MENU_ADVD_SCALE_TVSCALES:
                    scaleMethod = SCALEMETHOD_TV_SCALES;
                    break;

                default:
                    break;
            }

            (void)SETTINGS_DispFormatSet (format, scaleMethod);
            break;

        case MENU_ADVD_ID_ENTRY_HD:
            (void)SETTINGS_OutputTvStandardGet (&hdTvSys,&sdTvSys);
            //take effect only when value changes
            if (hdTvSys!=(SYS_HDTV_STANDARD)newVal.number)
            {
                (void)FGS_PANEL_Hide(inst->pnlHdl);

                /* hide prohibited sign if visible */
                (void)MAIN_ProhibitedHide(&(appDat->prohib));

                hdTvSys = (SYS_HDTV_STANDARD)newVal.number;
                (void)SETTINGS_OutputTvStandardSet (hdTvSys,sdTvSys);
            }
            break;

        case MENU_ADVD_ID_ENTRY_SD:
            (void)SETTINGS_OutputTvStandardGet (&hdTvSys,&sdTvSys);
            //take effect setting only when value changes
            if (sdTvSys!=(SYS_SDTV_STANDARD)newVal.number)
            {
                (void)FGS_PANEL_Hide(inst->pnlHdl);

                /* hide prohibited sign if visible */
                (void)MAIN_ProhibitedHide(&(appDat->prohib));

                sdTvSys = (SYS_SDTV_STANDARD)newVal.number;
                (void)SETTINGS_OutputTvStandardSet (hdTvSys,sdTvSys);
            }
            break;

        case MENU_ADVD_ID_ENTRY_VOUT:
            (void)SETTINGS_VideoOutputSet ((SYS_VIDEO_OUT)newVal.number);
            break;

        #ifdef APPL_SCRNSHOT_ENABLE
        case MENU_ADVD_ID_ENTRY_SCREENSHOT:
            scrnShotMode = (SCREENCAPTURE_MODE_E) newVal.number;
            (void)SETTINGS_ScreenShotModeSet(scrnShotMode);
            break;

        case MENU_ADVD_ID_ENTRY_SCREENSHOTTO:
            if ( (newVal.number < (int32_t)inst->devNums)
              && (inst->devList[newVal.number].available) )
            {
                scrnShotTo = inst->devList[newVal.number].device;
                (void)SETTINGS_ScreenShotToSet(scrnShotTo);
            }
            break;
        #endif /* APPL_SCRNSHOT_ENABLE */

        /*
        case MENU_ADVD_ID_ENTRY_STARTMODE:
            (void)SETTINGS_SetServiceStartMode((SYS_VIDSTART_MODE_E)newVal.number);
            break;

        case MENU_ADVD_ID_ENTRY_SWITCHMODE:
            (void)SETTINGS_SetServiceSwitchMode((fbool_t)newVal.number);
            break;
        case MENU_ADVD_ID_ENTRY_VIDEOFADE:
            //(void)SETTINGS_SetServiceVideoFadeMode(newVal.number);
            break;
          */
        case MENU_ADVD_ID_ENTRY_AOUT:
            (void)SETTINGS_DefaultSPDIFModeSet((SYS_SPDIF_MODE_E)newVal.number);
            break;

        case MENU_ADVD_ID_ENTRY_LIPSYNCMODE:
            (void)SETTINGS_LipSyncModeSet((AV_LIPSYNC_MODE_E)newVal.number);

            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                       MENU_ADVD_ID_ENTRY_LIPSYNCLEVEL,
                                       (newVal.number == AV_LIPSYNC_MODE_LEVEL) ?
                                       FTRUE : FFALSE);
            (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_ADVD_ID_ENTRY_LIPSYNCLEVEL);
            break;

        case MENU_ADVD_ID_ENTRY_LIPSYNCLEVEL:
            (void)SETTINGS_LipSyncDelaySet(newVal.number);
            break;
        #ifdef APPL_RFMOD_ENABLED

        /* Change the RF Modulator channel. Value can be increased or decreased
           by left or right arrow key or can be entered directly with the number
           keys. If the value is out of the specified range the selection adopts
           automatically to the next in range value.
           The display is greyed either in the OFF state or if the modulator is not
           availble.
           On returned error the diplayed value stays as is and no change is accepted */
        case     MENU_ADVD_ID_ENTRY_RFMODCH :
            rfmodParams.rfmod_channel = newVal.number;
            rfmodParams.rfmod_TVSystem = inst->currTvStd;
            for(j=0;j < RF_MOD_LAST_SYSTEM;j++)
            {
                if(rfmodParams.rfmod_std_ch[j].rfmodTvStdStored == inst->currTvStd)
                    rfmodParams.rfmod_std_ch[j].rfmodChannelStored = newVal.number;
            }
            if((ferr=RFMOD_SetParams(&rfmodParams)) == FAPI_OK)
            {
                inst->currRfCh = newVal.number;
            }

            #ifdef _RFMOD_REACT_ON_ERROR_

            else /* leave the value as is and don't change it */
            {
                /* dependant upon failure code show a message box */
                if(((ferr >> 16) & 0x0000ffff) == FD_I2C_MODULE_ID)
                {
                    /* show a menue box with hardware failure message */
                    MENU_CommonMsgShow(STR_MSG_HARDWARE_ERROR, FFALSE);
                }
            }

            #else

            else
            {
                //inst->currRfCh = newVal.number;
            }

            #endif
            break;

        /* Change the TV System for the RF Modulator. At this point the boundaries
           for the selected TV Standard are also verified and set accordingly when
           out of range.
           In case of OFF selection the access to the channel change menue is locked
           and released again when another mode is selected.
           In case of returned error code the display selection stays as is. */
        case MENU_ADVD_ID_ENTRY_RFTVSTD:
            /* only do something if new value is different than current */
            if(inst->currTvStd == newVal.number) break;
            /* check channel boundaries against new TV standard */
            rfmodParams.rfmod_TVSystem = newVal.number;
            for(j=0;j < RF_MOD_LAST_SYSTEM;j++)
            {
                if(rfmodParams.rfmod_std_ch[j].rfmodTvStdStored == newVal.number)
                    rfmodParams.rfmod_channel = rfmodParams.rfmod_std_ch[j].rfmodChannelStored;
            }
            //rfmodParams.rfmod_channel = inst->currRfCh;
            RFMOD_GetData(&rfmodParams, &rfchParams);
            if(newVal.number < RF_MOD_OFF)
            {
                /* current channel is outside new boundaries */
                if((rfmodParams.rfmod_channel > rfchParams.last_channel) ||
                   (rfmodParams.rfmod_channel < rfchParams.base_channel))
                {
                    rfmodParams.rfmod_channel = rfchParams.base_channel;
                    if((ferr=RFMOD_SetParams(&rfmodParams)) == FAPI_OK)
                    {
                        /* if previously Power Off enable Channel selection again */
                        if(inst->currTvStd == RF_MOD_OFF)
                            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl, MENU_ADVD_ID_ENTRY_RFMODCH, FTRUE);
                        inst->currRfCh = rfmodParams.rfmod_channel;
                        inst->currTvStd = rfmodParams.rfmod_TVSystem;
                        inst->base_channel = rfchParams.base_channel;
                        inst->last_channel = rfchParams.last_channel;
                        numParams.minValue = inst->base_channel;
                        numParams.maxValue = inst->last_channel;
                        numParams.maxDigits = 2;
                        numParams.step = 1;
                        (void)FGS_LIST_SetMemberParams (inst->listHdl, MENU_ADVD_ID_ENTRY_RFMODCH, &numParams);
                        (void)FGS_LIST_RefreshMember   (inst->listHdl, MENU_ADVD_ID_ENTRY_RFMODCH);

                    }

                    #ifdef _RFMOD_REACT_ON_ERROR_
                    else /* leave the value as is and don't change it, initial version */
                    {
                    /* dependant upon failure code show a message box */
                        if(((ferr >> 16) & 0x0000ffff) == FD_I2C_MODULE_ID)
                        {
                            /* show a menue box with hardware failure message */
                            MENU_CommonMsgShow(STR_MSG_HARDWARE_ERROR, FFALSE);
                        }
                    }
                    #else
                    else /* don't care about error from FD_RFMOD, menu stays constant */
                    {
                        //inst->currRfCh = rfmodParams.rfmod_channel;
                        //inst->currTvStd = rfmodParams.rfmod_TVSystem;
                        //inst->base_channel = rfchParams.base_channel;
                        //inst->last_channel = rfchParams.last_channel;
                        //numParams.minValue = inst->base_channel;
                        //numParams.maxValue = inst->last_channel;
                        //numParams.maxDigits = 2;
                        //numParams.step = 1;
                        //(void)FGS_LIST_SetMemberParams  (inst->listHdl, MENU_ADVD_ID_RFMODCH, &numParams);
                        #ifdef _RFMOD_TEST_SIGNAL_
                        selParams.numItems = RF_MOD_LAST_SYSTEM;
                        #else
                        selParams.numItems = RF_MOD_LAST_SYSTEM - 1;
                        #endif
                        selParams.index = inst->currTvStd;
                        (void)FGS_LIST_SetMemberParams  (inst->listHdl, MENU_ADVD_ID_ENTRY_RFTVSTD, &selParams);
                    }
                    #endif

                }
                /* current channel is inside new boundaries */
                else
                {
                    if((ferr=RFMOD_SetParams(&rfmodParams)) == FAPI_OK)
                    {
                        if(inst->currTvStd == RF_MOD_OFF)
                            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl, MENU_ADVD_ID_ENTRY_RFMODCH, FTRUE);
                        inst->currRfCh = rfmodParams.rfmod_channel;
                        inst->currTvStd = rfmodParams.rfmod_TVSystem;
                        inst->base_channel = rfchParams.base_channel;
                        inst->last_channel = rfchParams.last_channel;
                        numParams.minValue = inst->base_channel;
                        numParams.maxValue = inst->last_channel;
                        numParams.maxDigits = 2;
                        numParams.step = 1;
                        (void)FGS_LIST_SetMemberParams  (inst->listHdl, MENU_ADVD_ID_ENTRY_RFMODCH, &numParams);
                        FGS_LIST_RefreshMember (inst->listHdl, MENU_ADVD_ID_ENTRY_RFMODCH);
                    }

                    #ifdef _RFMOD_REACT_ON_ERROR_

                    else /* leave the value as is and don't change it */
                    {
                    /* dependant upon failure code show a message box */
                        if(((ferr >> 16) & 0x0000ffff) == FD_I2C_MODULE_ID)
                        {
                            /* show a menue box with hardware failure message */
                            MENU_CommonMsgShow(STR_MSG_HARDWARE_ERROR, FFALSE);
                        }
                    }

                    #else

                    else
                    {
                        //inst->currTvStd = rfmodParams.rfmod_TVSystem;
                        //inst->base_channel = rfchParams.base_channel;
                        //inst->last_channel = rfchParams.last_channel;
                        //numParams.minValue = inst->base_channel;
                        //numParams.maxValue = inst->last_channel;
                        //numParams.maxDigits = 2;
                        //numParams.step = 1;
                        //(void)FGS_LIST_SetMemberParams  (inst->listHdl, MENU_ADVD_ID_RFMODCH, &numParams);
                        #ifdef _RFMOD_TEST_SIGNAL_
                        selParams.numItems = RF_MOD_LAST_SYSTEM;
                        #else
                        selParams.numItems = RF_MOD_LAST_SYSTEM - 1;
                        #endif
                        selParams.index = inst->currTvStd;
                        (void)FGS_LIST_SetMemberParams  (inst->listHdl, MENU_ADVD_ID_ENTRY_RFTVSTD, &selParams);
                    }

                    #endif
                }
            } /* end of (newVal.number < RF_MOD_OFF) */
            /* Test Signal or Power Save */
            else
            {
                if((ferr=RFMOD_SetParams(&rfmodParams)) == FAPI_OK)
                {
                    inst->currTvStd = rfmodParams.rfmod_TVSystem;
                    /* if Power Off disable Channel selection */
                    if(inst->currTvStd == RF_MOD_OFF)
                        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl, MENU_ADVD_ID_ENTRY_RFMODCH, FFALSE);
                }
                #ifdef _RFMOD_REACT_ON_ERROR_
                else /* leave the value as is and don't change it */
                {
                /* dependant upon failure code show a message box */
                    if(((ferr >> 16) & 0x0000ffff) == FD_I2C_MODULE_ID)
                    {
                        /* show a menue box with hardware failure message */
                        MENU_CommonMsgShow(STR_MSG_HARDWARE_ERROR, FFALSE);
                    }
                }
                #else
                else
                {
                    //inst->currTvStd = rfmodParams.rfmod_TVSystem;
                    #ifdef _RFMOD_TEST_SIGNAL_
                    selParams.numItems = RF_MOD_LAST_SYSTEM;
                    #else
                    selParams.numItems = RF_MOD_LAST_SYSTEM - 1;
                    #endif
                    selParams.index = inst->currTvStd;
                    (void)FGS_LIST_SetMemberParams  (inst->listHdl, MENU_ADVD_ID_ENTRY_RFTVSTD, &selParams);

                }
                #endif
            }
            break;

        #endif /* APPL_RFMOD_ENABLED */

        default:
            break;
    }

}

FSTATIC void MENU_AudioVideoGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_ADVD_ID_ENTRY_VOUT      : strIdx = STR_ENTRY_ADVD_VOUT;       break;
        case MENU_ADVD_ID_ENTRY_FORMAT    : strIdx = STR_ENTRY_ADVD_FORMAT;     break;
        case MENU_ADVD_ID_ENTRY_SCALING   : strIdx = STR_ENTRY_ADVD_SCALING;    break;
        case MENU_ADVD_ID_ENTRY_HD        : strIdx = STR_ENTRY_ADVD_HD;         break;
        case MENU_ADVD_ID_ENTRY_SD        : strIdx = STR_ENTRY_ADVD_SD;         break;

        #ifdef APPL_SCRNSHOT_ENABLE
        case MENU_ADVD_ID_ENTRY_SCREENSHOT   : strIdx = STR_ENTRY_ADVD_SCREENSHOT;   break;

        case MENU_ADVD_ID_ENTRY_SCREENSHOTTO : strIdx = STR_ENTRY_ADVD_SCREENSHOTTO; break;
        #endif /* APPL_SCRNSHOT_ENABLE */

        //case MENU_ADVD_ID_ENTRY_STARTMODE : strIdx = STR_ENTRY_ADVD_STARTMODE;  break;
        //case MENU_ADVD_ID_ENTRY_SWITCHMODE: strIdx = STR_ENTRY_ADVD_SWITCHMODE; break;
        //case MENU_ADVD_ID_ENTRY_VIDEOFADE : strIdx = STR_ENTRY_ADVD_VIDEOFADE;  break;
        case MENU_ADVD_ID_ENTRY_AOUT      : strIdx = STR_ENTRY_ADVD_AOUT;       break;

        case MENU_ADVD_ID_ENTRY_LIPSYNCMODE   : strIdx = STR_ENTRY_ADVD_LIPSYNCMODE;    break;
        case MENU_ADVD_ID_ENTRY_LIPSYNCLEVEL  : strIdx = STR_ENTRY_ADVD_LIPSYNCLEVEL;         break;
        #ifdef APPL_RFMOD_ENABLED

        case MENU_ADVD_ID_ENTRY_RFMODCH   : strIdx = STR_ENTRY_ADVD_RFMODCH;    break;
        case MENU_ADVD_ID_ENTRY_RFTVSTD   : strIdx = STR_ENTRY_ADVD_RFTVSTD;    break;

        #endif /* APPL_RFMOD_ENABLED */

        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_AudioVideoGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        case MENU_ADVD_ID_ENTRY_VOUT       :
        case MENU_ADVD_ID_ENTRY_FORMAT     :
        case MENU_ADVD_ID_ENTRY_SCALING    :
        case MENU_ADVD_ID_ENTRY_HD         :
        case MENU_ADVD_ID_ENTRY_SD         :

        #ifdef APPL_SCRNSHOT_ENABLE
        case MENU_ADVD_ID_ENTRY_SCREENSHOT   :

        case MENU_ADVD_ID_ENTRY_SCREENSHOTTO :
        #endif /* APPL_SCRNSHOT_ENABLE */

        //case MENU_ADVD_ID_ENTRY_STARTMODE  :
        //case MENU_ADVD_ID_ENTRY_SWITCHMODE :
        //case MENU_ADVD_ID_ENTRY_VIDEOFADE  :
        case MENU_ADVD_ID_ENTRY_AOUT       :

        case MENU_ADVD_ID_ENTRY_LIPSYNCMODE    :
        #ifdef APPL_RFMOD_ENABLED

        case MENU_ADVD_ID_ENTRY_RFMODCH    :
        case MENU_ADVD_ID_ENTRY_RFTVSTD    :

        #endif /* APPL_RFMOD_ENABLED */

            strIdx = STR_ENTRY_LOKR;
            break;

        case MENU_ADVD_ID_ENTRY_LIPSYNCLEVEL   :

            strIdx = STR_ENTRY_09LR;
            break;
        default:  return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC void MENU_AudioVideoGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t                strIdx = STR_EMPTYSTRING;
    int32_t                 lipSyncLevel = 0;
    int32_t                 hdmiDelay = 0;
    AV_LIPSYNC_MODE_E       lipSyncMode;

    #ifdef APPL_SCRNSHOT_ENABLE

    MENU_ADVD_INST_S* inst = (MENU_ADVD_INST_S*)h;

    #endif /* APPL_SCRNSHOT_ENABLE */


    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = (int32_t)MAIN_FONT;
    item->item.str.lineHeight = 0;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
    }

    switch ( id )
    {
        case MENU_ADVD_ID_ENTRY_VOUT:
            switch ( index )
            {
                //case VIDEOOUT_YCRCB  : strIdx = STR_VALUE_ADVD_VOUT_YCRCB;  break;
                case VIDEOOUT_SVIDEO : strIdx = STR_VALUE_ADVD_VOUT_YC; break;
                case VIDEOOUT_RGB    : strIdx = STR_VALUE_ADVD_VOUT_RGB;    break;
                default              : strIdx = STR_EMPTYSTRING;            break;
            }
            item->item.str.chr        = OSD_GetStr(strIdx);
            break;

        case MENU_ADVD_ID_ENTRY_FORMAT:
            switch ( index )
            {
                case PICFORMAT_4_3  : strIdx = STR_VALUE_ADVD_VFORMAT_43;  break;
                case PICFORMAT_16_9 : strIdx = STR_VALUE_ADVD_VFORMAT_169; break;
                default             : strIdx = STR_EMPTYSTRING;            break;
            }
            item->item.str.chr        = OSD_GetStr(strIdx);
            break;

        case MENU_ADVD_ID_ENTRY_SCALING:
            switch ( index )
            {
                case MENU_ADVD_SCALE_PILLAR_LETTERBOX:
                    strIdx = STR_VALUE_ADVD_METHOD_LETTERBOX;
                    break;

                case MENU_ADVD_SCALE_FULLSCREEN:
                    strIdx = STR_VALUE_ADVD_METHOD_FULLSCREEN;
                    break;

                case MENU_ADVD_SCALE_CCO:
                    strIdx = STR_VALUE_ADVD_METHOD_CCO;
                    break;

                case MENU_ADVD_SCALE_TVSCALES:
                    strIdx = STR_VALUE_ADVD_METHOD_TVSCALES;
                    break;

                default:
                    break;
            }

            item->item.str.chr        = OSD_GetStr(strIdx);
            break;

        case MENU_ADVD_ID_ENTRY_HD:
            switch ( index )
            {
                case HDSYS_576P      : strIdx = STR_VALUE_ADVD_TV_576P;   break;
                case HDSYS_576I      : strIdx = STR_VALUE_ADVD_TV_576I;   break;
                case HDSYS_720P      : strIdx = STR_VALUE_ADVD_TV_720P;   break;
                case HDSYS_1080I     : strIdx = STR_VALUE_ADVD_TV_1080I;  break;
                //case HDSYS_AUTO      : strIdx = STR_VALUE_ADVD_TV_HD_AUTO;break;
                default              : strIdx = STR_EMPTYSTRING;          break;
            }
            item->item.str.chr = OSD_GetStr(strIdx);
            break;

        case MENU_ADVD_ID_ENTRY_SD:
            switch ( index )
            {
                case SDSYS_PAL     : strIdx = STR_VALUE_ADVD_TV_PAL;     break;
                case SDSYS_NTSC    : strIdx = STR_VALUE_ADVD_TV_NTSC;    break;
                case SDSYS_SECAM   : strIdx = STR_VALUE_ADVD_TV_SECAM;   break;
                case SDSYS_PAL_M   : strIdx = STR_VALUE_ADVD_TV_PAL_M;   break;
                case SDSYS_PAL_N   : strIdx = STR_VALUE_ADVD_TV_PAL_N;   break;
                case SDSYS_PAL_N_ARG:
                                strIdx = STR_VALUE_ADVD_TV_PAL_NAR;  break;
                // SDSYS_AUTO_PAL_NTSC:
                //                strIdx = STR_VALUE_ADVD_TV_SD_AUTO;      break;
                default            : strIdx = STR_EMPTYSTRING;           break;
            }
            item->item.str.chr        = OSD_GetStr(strIdx);
            break;

        #ifdef APPL_SCRNSHOT_ENABLE
        case MENU_ADVD_ID_ENTRY_SCREENSHOT:
            switch ( index )
            {
                case SCREENCAPTURE_VIDEO_ONLY : strIdx = STR_VALUE_ADVD_SCRNSHOT_VIDEO;     break;
                case SCREENCAPTURE_VIDEO_OSD  : strIdx = STR_VALUE_ADVD_SCRNSHOT_OSDVIDEO;  break;
                default                       : strIdx = STR_EMPTYSTRING;           break;
            }
            item->item.str.chr        = OSD_GetStr(strIdx);
            break;

        case MENU_ADVD_ID_ENTRY_SCREENSHOTTO:
            if ( index >= inst->devNums )
            {
                item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            }
            else
            {
                switch ( inst->devList[index].device )
                {
                    #ifdef APPL_HDD_ENABLED

                    case SYS_DEV_HDD:
                        item->item.str.chr = OSD_GetStr(
                                             STR_VALUE_ADVD_SCRNSHOTTO_INTHD);
                        break;

                    #endif /* APPL_HDD_ENABLED */

                    #ifdef APPL_USB_ENABLED

                    case SYS_DEV_USB:
                        item->item.str.chr = OSD_GetStr(
                                             STR_VALUE_ADVD_SCRNSHOTTO_USB);
                        break;

                    #endif /* APPL_USB_ENABLED */

                    default:
                        item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                        break;
                }

                if  ( !(inst->devList[index].available) )
                {
                    item->item.str.col = appDat->txtAttr.colDisabled;
                }
            }

            break;
        #endif /* APPL_SCRNSHOT_ENABLE */

        /*
        case MENU_ADVD_ID_ENTRY_STARTMODE:
            switch ( index )
            {
                case VIDEOSTART_FAST : strIdx = STR_VALUE_ADVD_START_FAST;   break;
                case VIDEOSTART_SYNC : strIdx = STR_VALUE_ADVD_START_SYNCED; break;
                default:               strIdx = STR_EMPTYSTRING;             break;
            }
            item->item.str.chr = OSD_GetStr(strIdx);
            break;

        case MENU_ADVD_ID_ENTRY_SWITCHMODE:
            switch ( index )
            {
                case FTRUE  : strIdx = STR_VALUE_ADVD_SWITCH_FREEZE; break;
                case FFALSE : strIdx = STR_VALUE_ADVD_SWITCH_BLANK;  break;
                default     : strIdx = STR_EMPTYSTRING;              break;
            }
            item->item.str.chr = OSD_GetStr(strIdx);
            break;

        case MENU_ADVD_ID_ENTRY_VIDEOFADE:
            switch ( index )
            {
                case VIDEOFADE_OFF  : strIdx = STR_VALUE_ADVD_FADE_OFF;  break;
                case VIDEOFADE_SLOW : strIdx = STR_VALUE_ADVD_FADE_SLOW; break;
                case VIDEOFADE_FAST : strIdx = STR_VALUE_ADVD_FADE_FAST; break;
                default     : strIdx = STR_EMPTYSTRING;              break;
            }
            item->item.str.chr        = OSD_GetStr(strIdx);
            break;
        */
        case MENU_ADVD_ID_ENTRY_AOUT:
            switch ( index )
            {
                case SPDIFMODE_PCMONLY:
                    strIdx = STR_VALUE_ADVD_AOUT_PCMONLY;
                    break;

                case SPDIFMODE_PREFERPASSTHROUGH:
                    strIdx = STR_VALUE_ADVD_AOUT_PREFERPASSTHROUGH;
                    break;

                default:
                    strIdx = STR_EMPTYSTRING;
                    break;
            }

            item->item.str.chr = OSD_GetStr(strIdx);
            break;

        case MENU_ADVD_ID_ENTRY_LIPSYNCMODE:
            switch ( index )
            {
                case AV_LIPSYNC_MODE_AUTO  : strIdx = STR_VALUE_ADVD_LIPSYNC_AUTO;   break;
                case AV_LIPSYNC_MODE_LEVEL : strIdx = STR_VALUE_ADVD_LIPSYNC_MANUAL; break;
                default                 : strIdx = STR_EMPTYSTRING;               break;
            }
            item->item.str.chr = OSD_GetStr(strIdx);
            break;
        case MENU_ADVD_ID_ENTRY_LIPSYNCLEVEL:
            (void)SETTINGS_LipSyncModeGet(&lipSyncMode);

            if ( lipSyncMode == AV_LIPSYNC_MODE_AUTO )
            {
                if ( AV_HDMIGetEDIDLipSyncDelay(&hdmiDelay) )
                {
                    strIdx = STR_VALUE_ADVD_LIPSYNC_NOINFO;
                    item->item.str.chr        = OSD_GetStr(strIdx);
                }
                else
                {
                    item->type                      = FGS_ITEM_TYPE_NUMBER;
                    item->item.number.value         = hdmiDelay;
                    item->item.number.format        = FGS_FORMAT_NUM_NORM;
                    item->item.number.decimalPlaces = 0;
                    item->item.number.unitStr       = OSD_GetStr(STR_UNIT_MSEC);
                }
            }
            else
            {
                (void)SETTINGS_LipSyncDelayGet(&lipSyncLevel);

                item->type                      = FGS_ITEM_TYPE_NUMBER;
                item->item.number.value         = lipSyncLevel;
                item->item.number.format        = FGS_FORMAT_NUM_NORM;
                item->item.number.decimalPlaces = 0;
                item->item.number.unitStr       = OSD_GetStr(STR_UNIT_MSEC);
                item->item.number.font          = MAIN_FONT;
            }
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.number.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.number.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.number.col = appDat->txtAttr.colDisabled; break;
            }
            break;
        #ifdef APPL_RFMOD_ENABLED

        case MENU_ADVD_ID_ENTRY_RFMODCH:
        // get data move to inst->rfModCh
            item->type                      = FGS_ITEM_TYPE_NUMBER;
            item->item.number.value         = inst->currRfCh;
            item->item.number.format        = FGS_FORMAT_NUM_NORM;
            item->item.number.decimalPlaces = 0;
            item->item.number.unitStr       = OSD_GetStr(STR_EMPTYSTRING);
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.number.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.number.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.number.col = appDat->txtAttr.colDisabled; break;
            }
            break;

        case MENU_ADVD_ID_ENTRY_RFTVSTD:
            switch ( index )
            {
                case RF_MOD_OFF : strIdx = STR_VALUE_ADVD_RF_MOD_RF_MOD_OFF;          break;  //
                #ifdef _RFMOD_TEST_SIGNAL_
                case RF_MOD_TEST_SIGNAL : strIdx = STR_VALUE_ADVD_RF_MOD_RF_MOD_TEST_SIGNAL;  break;  //
                #endif
                case RF_MOD_PALSEC_B    : strIdx = STR_VALUE_ADVD_RF_MOD_RF_MOD_PALSEC_B;     break;  //
                case RF_MOD_PALSEC_BGH  : strIdx = STR_VALUE_ADVD_RF_MOD_RF_MOD_PALSEC_BGH;   break;  //
                case RF_MOD_PAL_I       : strIdx = STR_VALUE_ADVD_RF_MOD_RF_MOD_PAL_I;        break;  //
                case RF_MOD_PALSEC_DKL  : strIdx = STR_VALUE_ADVD_RF_MOD_RF_MOD_PALSEC_DKL;   break;  //
                case RF_MOD_NTSC_MN     : strIdx = STR_VALUE_ADVD_RF_MOD_RF_MOD_NTSC_MN;      break;  //
                case RF_MOD_NTSC_M      : strIdx = STR_VALUE_ADVD_RF_MOD_RF_MOD_NTSC_M;       break;  //
                default                 : strIdx = STR_EMPTYSTRING;            break;
            }
            item->item.str.chr        = OSD_GetStr(strIdx);
            break;

        #endif /* APPL_RFMOD_ENABLED */

        default:  return;
    }
}

FSTATIC fbool_t MENU_AudioVideoBuild(FAPI_SYS_HandleT h)
{
    MENU_ADVD_INST_S*     inst = (MENU_ADVD_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_LIST_OPEN_S       listPars;
    FAPI_SYS_HandleT      itemHdl;
    FGS_ITEM_NUM_OPEN_S   numPars;
    FGS_ITEM_SEL_OPEN_S   selPars;
    FGS_POS_S             itemPos;
    FGS_POS_S             valuePos;
    FGS_POS_S             descrPos;
    int32_t               retVal;
    uint16_t              helpWidth;
    uint16_t              stackedFrameWidth;
    uint16_t              innerHeight;
    uint32_t              numLines = MENU_ADVD_ID_ENTRY_NUMS;
    SYS_VIDEO_OUT         videoout;
    SYS_PICFORMAT_E       format = PICFORMAT_4_3;
    SYS_SCALEMETHOD_E     scaleMethod;
    SYS_HDTV_STANDARD     hdTvSys;
    SYS_SDTV_STANDARD     sdTvSys;
    SYS_SPDIF_MODE_E      spdifMode = SETTINGS_DEFAULT_SPDIFMODE;
    AV_LIPSYNC_MODE_E     lipSyncMode;
    /*SYS_VIDSTART_MODE_E   videoStartMode;*/
    /*fbool_t               videoSwitchMode;*/
    /*SYS_VIDFADE_MODE_E  uint8_t videoFadeMode;*/
    FGS_PANEL_MEMBER_S    member;

    #ifdef APPL_SCRNSHOT_ENABLE
    SCREENCAPTURE_MODE_E  scrnShotMode;
    SYS_DEVICE_E          scrnShotTo;
    #endif /* APPL_SCRNSHOT_ENABLE */

    memset(&numPars,  0, sizeof(numPars));
    memset(&selPars,  0, sizeof(selPars));
    memset(&member,   0, sizeof(member));

    helpWidth         = (2*SCALER_GetCurrVal(HELP_LR_PADDING)) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_09LR),
                                   -1, (uint8_t)SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                            SCALER_GetCurrVal(LIST_LINE_PADDING));

    for(;;)
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth,
                                        1, &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_AudioVideoOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, (int32_t)MENUID_ADVD, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_ADVD_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_ADVD,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_AudioVideoGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_AudioVideoGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_AudioVideoOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_AudioVideoOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_AudioVideoValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_ADVD_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- Display Format  ------------------------------------------- */

        (void)SETTINGS_DispFormatGet (&format, &scaleMethod);

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = (uint32_t)PICFORMAT_NUMS;
        selPars.params.index        = (uint32_t)format;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_FORMAT,
                                     inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                        SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ----- Scaling Method ----------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.params.numItems     = (uint32_t)SCALEMETHOD_NUMS;
        selPars.params.index        = (uint32_t)scaleMethod;
        /* NOTE: Parameters are sent in a separate function. */

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_SCALING,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                    SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ----- HDMI Signal ----------------------------------------------- */

        (void)SETTINGS_OutputTvStandardGet (&hdTvSys,&sdTvSys);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = (uint32_t)HDSYS_NUMS;
        selPars.params.index        = (uint32_t)hdTvSys;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_HD,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                    SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ----- SDTV standard --------------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.params.numItems     = (uint32_t)SDSYS_NUMS;
        selPars.params.index        = (uint32_t)sdTvSys;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_SD,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                    SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ----- Scart output ---------------------------------------------- */

        (void)SETTINGS_VideoOutputGet (&videoout);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = (uint32_t)VIDEOOUT_NUMS;
        selPars.params.index        = (uint32_t)videoout;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_VOUT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                    SCALER_GetCurrVal(LIST_LINE_PADDING));


        #ifdef APPL_SCRNSHOT_ENABLE
         /* ----- Screen Shot mode ---------------------------------------- */

        (void)SETTINGS_ScreenShotModeGet(&scrnShotMode);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = (uint32_t)SCREENCAPTURE_NUMS;
        selPars.params.index        = (uint32_t)scrnShotMode;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_SCREENSHOT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                    SCALER_GetCurrVal(LIST_LINE_PADDING));

         /* -----  Screen Shot Save To ----------------------------------- */

        (void)SETTINGS_ScreenShotToGet(&scrnShotTo);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = inst->devNums;
        selPars.params.index        = inst->devFirstIdx;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_SCREENSHOTTO,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                    SCALER_GetCurrVal(LIST_LINE_PADDING));

        #endif /* APPL_SCRNSHOT_ENABLE */

        /* ----- Service start mode ---------------------------------------- */

        /*(void)SETTINGS_GetServiceStartMode(&videoStartMode);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = (uint32_t)VIDEOSTART_NUMS;
        selPars.params.index        = (uint32_t)videoStartMode;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_STARTMODE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                        MENU_ADVD_ID_ENTRY_STARTMODE,
                                        FFALSE);
        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                SCALER_GetCurrVal(LIST_LINE_PADDING));
        */
        /* ----- Service switch mode --------------------------------------- */

        /*(void)SETTINGS_GetServiceSwitchMode(&videoSwitchMode);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = 2;
        selPars.params.index        = (uint32_t)videoSwitchMode;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_SWITCHMODE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                        MENU_ADVD_ID_ENTRY_SWITCHMODE,
                                        FFALSE);
        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                SCALER_GetCurrVal(LIST_LINE_PADDING));
        */
        /* ----- Video fade mode ------------------------------------------- */

        /* (void)SETTINGS_GetServiceFadeMode(&videoFadeMode);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = 3;
        selPars.params.index        = videoFadeMode;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_VIDEOFADE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        */
        /* ----- Audio output ---------------------------------------------- */

        (void)SETTINGS_DefaultSPDIFModeGet(&spdifMode);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = (int32_t)SPDIFMODE_NUMS;
        selPars.params.index        = (int32_t)spdifMode;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_AOUT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                             + SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ----- lip sync setting ------------------------------------------- */

        (void)SETTINGS_LipSyncModeGet(&lipSyncMode);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = (int32_t)AV_LIPSYNC_MODE_NUMS;
        selPars.params.index        = (int32_t)lipSyncMode;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_LIPSYNCMODE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ----- lip sync audio delay level -------------------------------- */

        numPars.pos                 = itemPos;
        numPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        numPars.descrPos            = descrPos;
        numPars.valuePos            = valuePos;
        numPars.type                = FGS_NITEM_TYPE_NUMBER;
        numPars.wraparound          = FTRUE;
        numPars.params.minValue     = LIPSYNC_MIN_DELAY;
        numPars.params.maxValue     = LIPSYNC_MAX_DELAY;
        numPars.params.maxDigits    = 3;
        numPars.params.step         = 20;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_ADVD_ID_ENTRY_LIPSYNCLEVEL,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* if AUTO lipSync is enabled, disable manual setting */
        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                              MENU_ADVD_ID_ENTRY_LIPSYNCLEVEL,
                                              (lipSyncMode == AV_LIPSYNC_MODE_LEVEL) ?
                                              FTRUE : FFALSE);

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                SCALER_GetCurrVal(LIST_LINE_PADDING));
        #ifdef APPL_RFMOD_ENABLED

        /* ----- RFMODulator channel --------------------------------------- */

        numPars.pos        = itemPos;
        numPars.bkgrType   = OSD_LSTITEM_BKGR_NORMAL;
        numPars.descrPos   = descrPos;
        numPars.valuePos   = valuePos;
        numPars.type       = FGS_NITEM_TYPE_NUMBER;
        numPars.wraparound = FTRUE;
        numPars.params.minValue   = inst->base_channel;
        numPars.params.maxValue   = inst->last_channel;
        numPars.params.maxDigits  = 2;
        numPars.params.step       = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_ADVD_ID_ENTRY_RFMODCH,
                                    inst->listHdl, &itemHdl);

        #ifdef _RFMOD_LOCK_MENU_ON_ERROR_
        if(inst->currTvStd == RF_MOD_OFF || inst->rfmodLockMenueAppl == FTRUE)
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl, MENU_ADVD_ID_ENTRY_RFMODCH, FFALSE);
        #else
        if(inst->currTvStd == RF_MOD_OFF)
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl, MENU_ADVD_ID_ENTRY_RFMODCH, FFALSE);
        #endif /* _RFMOD_LOCK_MENU_ON_ERROR_ */

        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* --- RF TV standard ------------------------------------*/

        selPars.pos                 = itemPos;
        #ifdef _RFMOD_TEST_SIGNAL_
        selPars.params.numItems = RF_MOD_LAST_SYSTEM;
        #else
        selPars.params.numItems = RF_MOD_LAST_SYSTEM - 1;
        #endif
        selPars.params.index               = inst->currTvStd;


        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_ADVD_ID_ENTRY_RFTVSTD,
                                    inst->listHdl, &itemHdl);

        #ifdef _RFMOD_LOCK_MENU_ON_ERROR_
        if(inst->rfmodLockMenueAppl == FTRUE)
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl, MENU_ADVD_ID_ENTRY_RFTVSTD, FFALSE);
        #endif

        if ( retVal != FAPI_OK ) break;
        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                SCALER_GetCurrVal(LIST_LINE_PADDING));

        #endif /* APPL_RFMOD_ENABLED */

        /* ----- Separator lines ------------------------------------------- */

        inst->itemPos.x         = listPars.pos.x;
        inst->itemPos.y         = listPars.pos.y;
        inst->itemPos.width     = SCALER_GetCurrVal(MAIN_WIDTH);
        inst->itemPos.height    = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(INNER_FRAME_WIDTH));
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemPos);
        member.id            = MENU_ADVD_ID_REST;
        member.isHidden      = FFALSE;
        member.show          = MENU_AudioVideoDraw;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pItem));
        if ( retVal != FAPI_OK ) break;

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_ADVD_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH), listPars.pos.y,
                                        helpWidth,
                                        numLines,
                                        MENU_AudioVideoGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine((int16_t)(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING)),
                                                -(int16_t)((( SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2) +
                                                SCALER_GetCurrVal(HELP_TB_PADDING)) +
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT)),
                                                (((pnlPars.pos.width -
                                                stackedFrameWidth) -
                                                (2 * SCALER_GetCurrVal(OUTER_FRAME_ALL))) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING)) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;
        break; /* end of for(;;) */
    }


    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    return FTRUE;
}

FSTATIC int32_t  MENU_AudioVideoDraw (FAPI_SYS_HandleT h)
{
    MENU_ADVD_INST_S* inst = (MENU_ADVD_INST_S*)h;

    #ifdef APPL_SCRNSHOT_ENABLE
    OSD_DrawSeparatorHorizontal((uint16_t)inst->pItem->abspos.x,
                                ((uint16_t)inst->pItem->abspos.y - SCALER_GetCurrVal(LIST_LINE_PADDING)) +
                                (7 *(SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING))),
                                SCALER_GetCurrVal(MAIN_WIDTH));
    #else
    OSD_DrawSeparatorHorizontal((uint16_t)inst->pItem->abspos.x,
                                ((uint16_t)inst->pItem->abspos.y - SCALER_GetCurrVal(LIST_LINE_PADDING)) +
                                (5 *(SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING))),
                                SCALER_GetCurrVal(MAIN_WIDTH));

    #endif /* APPL_SCRNSHOT_ENABLE */

    #ifdef APPL_RFMOD_ENABLED
    OSD_DrawSeparatorHorizontal((uint16_t)inst->pItem->abspos.x,
                                ((uint16_t)inst->pItem->abspos.y - SCALER_GetCurrVal(LIST_LINE_PADDING)) +
                                (8 *(SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING))),
                                SCALER_GetCurrVal(MAIN_WIDTH));
    #endif /* APPL_RFMOD_ENABLED */

    return FAPI_OK;
}


void MENU_AudioVideoRefresh(void)
{
    MENU_ADVD_INST_S* inst = (MENU_ADVD_INST_S*)menuAudioVideoDat;

    /* check if Menu is open */
    if ( ( inst == NULL ) || ( inst->pnlHdl == NULL ) ) return;

    /* ----- close menu panel ------------------------------------------ */
    (void)FGS_PANEL_Close(inst->pnlHdl);

    inst->pnlHdl = NULL;

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_ADVD);

    /* ----- re-open menu panel ---------------------------------------- */
    (void)MENU_AudioVideoBuild((FAPI_SYS_HandleT)inst);

    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, (int32_t)inst->lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_ADVD_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_ADVD);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
