/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_editstring.c
**
**              $Id: app_menu_editstring.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD edit string function.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 525: Negative indentation.
** - Warning 570: Loss of sign (assignment)(int to unsigned short).
** - Warning 641: Converting enum 'anyEnum' to int.
** - Warning 650: Constant '-28' out of range for operator 'case'.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    743: Negative character constant
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e525 -e570 -e641 -e650 -e713 -e715 -e737 -e743 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define MENU_EDITSTRING_BUFCHARS    128         //!< (Excluding termination)
#define MENU_EDITSTRING_BUFSIZE     (MENU_EDITSTRING_BUFCHARS * 2)
                                                //!< Unicode buffer!
#define MENU_EDITSTRING_BUFCHARSET  SYS_CHARSET_ISO_10646
                                                //!< Character set used
#define MENU_ES_BYTESPERCHAR        2

#define MENU_EDITSTRING_NUM_HELP_COLUMN     4

#define MENU_EDITSTRING_STRING_POS_X        (SCALER_GetCurrVal(MAIN_LEFT_PADDING))
#define MENU_EDITSTRING_STRING_POS_Y        (SCALER_GetCurrVal(OUTER_FRAME_ALL) + \
                                             SCALER_GetCurrVal(INNER_FRAME_WIDTH) + \
                                             SCALER_GetCurrVal(MAIN_TOP_PADDING) + \
                                             (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) + \
                                             SCALER_GetCurrVal(TITLE_LINE_HEIGHT))
#define MENU_EDITSTRING_STRING_WIDTH        (SCALER_GetCurrVal(EDIT_STRING_WIDTH) - \
                                             SCALER_GetCurrVal(MAIN_LEFT_PADDING) - \
                                             SCALER_GetCurrVal(MAIN_RIGHT_PADDING))
#define MENU_EDITSTRING_STRING_HEIGHT       (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + \
                                            (2*SCALER_GetCurrVal(LIST_LINE_PADDING)))


/* edit string function defines */
enum
{
    MENU_EDITSTRING_ID_TITLE = 0,
    MENU_EDITSTRING_ID_WINDOW,
    MENU_EDITSTRING_NUM_ID
};

/* other bottom help keys */
enum
{
    MENU_EDITSTRING_BOTTOM_HELP_ICON_LEFTRIGHT = MENU_EDITSTRING_NUM_ID,
    MENU_EDITSTRING_BOTTOM_HELP_TEXT_LEFTRIGHT,
    MENU_EDITSTRING_BOTTOM_HELP_ICON_MENU,
    MENU_EDITSTRING_BOTTOM_HELP_TEXT_MENU,
    MENU_EDITSTRING_BOTTOM_HELP_ICON_EXIT,
    MENU_EDITSTRING_BOTTOM_HELP_TEXT_EXIT,
    MENU_EDITSTRING_BOTTOM_HELP_ICON_UPDOWN,
    MENU_EDITSTRING_BOTTOM_HELP_TEXT_UPDOWN,
    MENU_EDITSTRING_BOTTOM_HELP_ICON_NUMKEY,
    MENU_EDITSTRING_BOTTOM_HELP_TEXT_NUMKEY,
    MENU_EDITSTRING_BOTTOM_HELP_ICON_RED,
    MENU_EDITSTRING_BOTTOM_HELP_TEXT_RED,
    MENU_EDITSTRING_BOTTOM_HELP_ICON_GREEN,
    MENU_EDITSTRING_BOTTOM_HELP_TEXT_GREEN,
    MENU_EDITSTRING_BOTTOM_HELP_ICON_YELLOW,
    MENU_EDITSTRING_BOTTOM_HELP_TEXT_YELLOW,
    MENU_EDITSTRING_BOTTOM_HELP_ICON_BLUE,
    MENU_EDITSTRING_BOTTOM_HELP_TEXT_BLUE
};


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC void MENU_EditStringOnFocusSet      (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC fbool_t MENU_EditStringClose          (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_EditStringNumKeys        (FAPI_SYS_HandleT h, uint8_t value);
FSTATIC fbool_t MENU_EditStringKeys           (FAPI_SYS_HandleT h);
FSTATIC void MENU_EditStringGetBoxItemText  (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                             FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC void MENU_EditStringGetBoxItemIcon  (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                             FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC fbool_t MENU_EditStringGetNextNumChar(SYS_STRBUF_S* pBuf, uint16_t charPos,
                                            MENU_EDITSTRING_KEY_STATUS status,
                                            uint8_t keyNum);
FSTATIC void MENU_EditStringGetNextChar    (SYS_STRBUF_S* pBuf, uint16_t charPos,
                                            MENU_EDITSTRING_KEY_STATUS status,
                                            fbool_t up);
FSTATIC void MENU_EditStringBackSpace      (MENU_EDITSTRING_INST_S* inst);
FSTATIC void MENU_EditStringDelete         (MENU_EDITSTRING_INST_S* inst);
FSTATIC void MENU_EditStringInsert         (MENU_EDITSTRING_INST_S* inst);



int32_t MENU_EditStringInit (MENU_EDITSTRING_INST_S* inst)
{
    int32_t retVal;

    memset(inst, 0, sizeof(MENU_EDITSTRING_INST_S));

    retVal = STR_BufOpen (&(inst->tmpBuf), MENU_EDITSTRING_BUFSIZE +
                          MENU_ES_BYTESPERCHAR, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = STR_BufOpen (&(inst->targetBuf), MENU_EDITSTRING_BUFSIZE +
                          MENU_ES_BYTESPERCHAR, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    return FAPI_OK;
}

fbool_t MENU_EditStringOpen (MENU_EDITSTRING_INST_S* inst,
                             MENU_EDITSTRING_OPEN_S* openParams)
{
    FGS_PANEL_OPEN_S    pnlDat;
    FGS_BOX_OPEN_S      boxPars;
    FAPI_SYS_HandleT           dummyHdl;
    int32_t                retVal = FAPI_OK;
    uint32_t                 bufPos;
    uint16_t                 menuHeight;
    uint16_t                 bottomHelpKeyWidth;
    uint16_t                 bottomHelpItemWidth;
    uint16_t                 bottomHelpDescWidth;
    uint8_t                  numLines = 1;
    uint8_t                  numHelpLines = 3;

    inst->pnlHdl        = NULL;
    inst->firstVisChar  = 0;
    inst->keyStatus     = MENU_EDITSTRING_KEY_ALL;
    inst->extIdx        = -1;
    inst->editPars      = *openParams;

    if ( inst->editPars.editData.maxChars > MENU_EDITSTRING_BUFCHARS )
    {
        inst->editPars.editData.maxChars = MENU_EDITSTRING_BUFCHARS;
    }
    else if ( inst->editPars.editData.maxChars == 0 )
    {
        inst->editPars.editData.maxChars = MENU_EDITSTRING_BUFCHARS;
    }

    (void)STR_BufClear(&(inst->tmpBuf));
    (void)STR_BufClear(&(inst->targetBuf));

    /* Copy string to internal buffer and convert it to common editing
       format. */
    bufPos = STR_BufAppend (&(inst->tmpBuf), 0, inst->editPars.editData.string,
                            MENU_EDITSTRING_BUFCHARSET);

    inst->strPos  = 0;
    inst->strSize = bufPos / MENU_ES_BYTESPERCHAR;

    memset(&pnlDat, 0, sizeof(FGS_PANEL_OPEN_S));
    memset(&boxPars,   0, sizeof(boxPars));

    menuHeight              = (2*SCALER_GetCurrVal(OUTER_FRAME_ALL)) +
                              (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) + SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                              SCALER_GetCurrVal(MAIN_TOP_PADDING) + SCALER_GetCurrVal(MAIN_BOT_PADDING) +
                              ( numLines        * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                              ((numLines - 1 )  * SCALER_GetCurrVal(LIST_LINE_PADDING)) +
                              ( numHelpLines    * SCALER_GetCurrVal(HELP_LINE_HEIGHT)) +
                              ((numHelpLines -1)* SCALER_GetCurrVal(HELP_LINE_PADDING)) +
                              (2*SCALER_GetCurrVal(HELP_TB_PADDING)) + (2*SCALER_GetCurrVal(INNER_FRAME_WIDTH));
    /* string "MENU" is reference width for required key box width */
    bottomHelpItemWidth     = (((SCALER_GetCurrVal(EDIT_STRING_WIDTH)
                               - (2*SCALER_GetCurrVal(OUTER_FRAME_WIDTH)))
                               - SCALER_GetCurrVal(MAIN_LEFT_PADDING))
                               - SCALER_GetCurrVal(MAIN_RIGHT_PADDING)
                              ) /  MENU_EDITSTRING_NUM_HELP_COLUMN;

    bottomHelpKeyWidth      = OSD_GetStrWidth(OSD_GetStr(STR_KEY_MENU),-1,HELP_FONT);
    bottomHelpDescWidth     = (bottomHelpItemWidth
                              - bottomHelpKeyWidth)
                              - (2*SCALER_GetCurrVal(MAIN_CENTER_PADDING));
    do
    {
        pnlDat.bkgrType     = OSD_PNLTYPE_EDITSTRING;
        pnlDat.pos          = inst->editPars.pos;
        pnlDat.pos.width    = SCALER_GetCurrVal(EDIT_STRING_WIDTH);
        pnlDat.pos.height   = menuHeight;
        pnlDat.inFront      = FTRUE;
        pnlDat.onFocusSet   = MENU_EditStringOnFocusSet;
        pnlDat.onFocusSetArg= (FAPI_SYS_HandleT)inst;
        pnlDat.pInst        = &(inst->pnlInst);

        retVal = FGS_PANEL_Open(&pnlDat, MENUID_STRINGEDIT, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_EDITSTRING_ID_TITLE,
                                     inst->pnlHdl, pnlDat.pos.width,
                                     0, STR_TITLE_EDITSTRING, NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* open edit window box */
        boxPars.pos.x               = MENU_EDITSTRING_STRING_POS_X;
        boxPars.pos.y               = MENU_EDITSTRING_STRING_POS_Y;
        boxPars.pos.width           = MENU_EDITSTRING_STRING_WIDTH;
        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                      (2*SCALER_GetCurrVal(LIST_LINE_PADDING));
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.getItemArg          = (FAPI_SYS_HandleT)inst;
        boxPars.getItem             = MENU_EditStringGetBoxItemText;

        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_ID_WINDOW,
                               inst->pnlHdl, &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* -----Bottom help lines ------------------------------------------ */
        /* -----1st line --------------------------------------------------- */
        /* Icon Red */
        boxPars.pos.x            = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_LEFT_PADDING);

        boxPars.pos.y            = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                   - SCALER_GetCurrVal(HELP_TB_PADDING)
                                   - numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                   - (numHelpLines-1)*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.height       = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width        = bottomHelpKeyWidth;
        boxPars.pos.alignment    = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType         = OSD_BOX_BKGR_HELP;
        boxPars.getItem          = MENU_EditStringGetBoxItemIcon;

        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_ICON_RED, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* Icon Green */
        boxPars.pos.x              += bottomHelpItemWidth;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_ICON_GREEN, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;
        /* Icon Yellow */
        boxPars.pos.x              += bottomHelpItemWidth;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_ICON_YELLOW, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;
        /* Icon Blue */
        boxPars.pos.x              += bottomHelpItemWidth;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_ICON_BLUE, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* -----2nd line --------------------------------------------------- */
        /* Icon Left Right */
        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  SCALER_GetCurrVal(MAIN_LEFT_PADDING);

        boxPars.pos.y          += SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.getItem         = MENU_EditStringGetBoxItemText;

        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_ICON_LEFTRIGHT, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* Icon Up/Down */
        boxPars.pos.x          += bottomHelpItemWidth;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_ICON_UPDOWN, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* Icon Number Keys */
        boxPars.pos.x          += bottomHelpItemWidth;
        boxPars.getItem         = MENU_EditStringGetBoxItemText;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_ICON_NUMKEY, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* -----3rd line --------------------------------------------------- */
        /* Icon MENU */
        boxPars.pos.x            = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y           += SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                                   SCALER_GetCurrVal(HELP_LINE_PADDING);

        boxPars.getItem          = MENU_EditStringGetBoxItemText;
        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_ICON_MENU, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* Icon Exit */
        boxPars.pos.x              += (3 * bottomHelpItemWidth);


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_ICON_EXIT, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* -----Bottom help lines ------------------------------------------ */
        /* -----1st line --------------------------------------------------- */
        /* Text Red */
        boxPars.pos.x            = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                   bottomHelpKeyWidth +
                                   SCALER_GetCurrVal(MAIN_CENTER_PADDING);
        boxPars.pos.y            = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                   - SCALER_GetCurrVal(HELP_TB_PADDING)
                                   - numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                   - (numHelpLines-1)*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.width        = bottomHelpDescWidth;
        boxPars.pos.height       = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.alignment    = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType         = OSD_BOX_BKGR_HELP;
        boxPars.getItem          = MENU_EditStringGetBoxItemText;

        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_TEXT_RED, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* TEXT Green */
        boxPars.pos.x              += bottomHelpItemWidth;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_TEXT_GREEN, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;
        /* TEXT Yellow */
        boxPars.pos.x              += bottomHelpItemWidth;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_TEXT_YELLOW, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;
        /* TEXT Blue */
        boxPars.pos.x              += bottomHelpItemWidth;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_TEXT_BLUE, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* -----2nd line --------------------------------------------------- */
        /* TEXT Left Right */
        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                   bottomHelpKeyWidth +
                                   SCALER_GetCurrVal(MAIN_CENTER_PADDING);
        boxPars.pos.y              += SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                                      SCALER_GetCurrVal(HELP_LINE_PADDING);

        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_TEXT_LEFTRIGHT, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* TEXT Up/Down */
        boxPars.pos.x              += bottomHelpItemWidth;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_TEXT_UPDOWN, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* TEXT Number Keys */
        boxPars.pos.x              += bottomHelpItemWidth;


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_TEXT_NUMKEY, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* -----3rd line --------------------------------------------------- */
        /* TEXT MENU */
        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                   bottomHelpKeyWidth +
                                   SCALER_GetCurrVal(MAIN_CENTER_PADDING);
        boxPars.pos.y              += SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                                      SCALER_GetCurrVal(HELP_LINE_PADDING);

        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_TEXT_MENU, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* TEXT Exit */
        boxPars.pos.x              += (3 * bottomHelpItemWidth);


        retVal = FGS_BOX_Open (&boxPars, MENU_EDITSTRING_BOTTOM_HELP_TEXT_EXIT, inst->pnlHdl,
                               &dummyHdl);
        if ( retVal != FAPI_OK )break;

        /* ----------------------------------------------------------------- */
        inst->extIdx = MENUSTACK_AddExt(MENU_EditStringClose, (FAPI_SYS_HandleT)inst);

        retVal = FGS_PANEL_SetMemberFocus(inst->pnlHdl, MENU_EDITSTRING_ID_WINDOW);
        if ( retVal != FAPI_OK ) break;
        (void)MENUSTACK_FocusForce    ((int32_t)MENUID_STRINGEDIT);

        retVal = FGS_PANEL_Show (inst->pnlHdl, FFALSE); /* turn visibility on */
        if ( retVal != FAPI_OK ) break;
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        inst->pnlHdl = NULL;
        return FFALSE;
    }

    return FTRUE;
}

FSTATIC void MENU_EditStringOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_EDITSTRING_INST_S*)h)->pnlHdl != NULL);

    if ( hasFocus )
    {
        FGS_SetAppNumKeyHandler(FGS_KEY_NUMBER, MENU_EditStringNumKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_MENU,   MENU_EditStringKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK,   MENU_EditStringKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_EXIT,   MENU_EditStringKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RED,    MENU_EditStringKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_GREEN,  MENU_EditStringKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_YELLOW, MENU_EditStringKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BLUE,   MENU_EditStringKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_LEFT,   MENU_EditStringKeys, h, FTRUE,  FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_RIGHT,  MENU_EditStringKeys, h, FTRUE,  FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_UP,     MENU_EditStringKeys, h, FFALSE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_DOWN,   MENU_EditStringKeys, h, FFALSE, FTRUE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_NUMBER);
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_GREEN);
        FGS_ClearAppKeyHandler(FGS_KEY_YELLOW);
        FGS_ClearAppKeyHandler(FGS_KEY_BLUE);
        FGS_ClearAppKeyHandler(FGS_KEY_LEFT);
        FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
        FGS_ClearAppKeyHandler(FGS_KEY_UP);
        FGS_ClearAppKeyHandler(FGS_KEY_DOWN);
    }

}


FSTATIC fbool_t MENU_EditStringClose (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_EDITSTRING_INST_S* inst    = (MENU_EDITSTRING_INST_S*)h;

    /* NOTE: 'force' argument is ignored. It is always possible to close the
             selection list dialogue.  */

    /* close dialog box */
    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
    inst->pnlHdl = NULL;

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_STRINGEDIT);
    (void)MENUSTACK_RemoveExt    (inst->extIdx);

    if ( inst->editPars.onExit != NULL )
    {
        (void)STR_BufAppend (&(inst->targetBuf), 0, inst->tmpBuf.string,
                       inst->editPars.editData.string.cs);

        inst->editPars.editData.string = inst->targetBuf.string;

        inst->editPars.onExit(&(inst->editPars.editData), inst->editPars.id);
    }

    return FTRUE;
}

FSTATIC fbool_t MENU_EditStringNumKeys (FAPI_SYS_HandleT h, uint8_t value)
{
    MENU_EDITSTRING_INST_S* inst = (MENU_EDITSTRING_INST_S*)h;
    fbool_t keyUsed = FTRUE;

    /* get next character up the pressed key */
    keyUsed = MENU_EditStringGetNextNumChar (&(inst->tmpBuf), inst->strPos,
                                             inst->keyStatus, value);

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_EDITSTRING_ID_WINDOW);

    return keyUsed;
}

FSTATIC fbool_t MENU_EditStringKeys(FAPI_SYS_HandleT h)
{
    MENU_EDITSTRING_INST_S* inst = (MENU_EDITSTRING_INST_S*)h;
    fbool_t                   keyUsed = FTRUE;
    SYS_STRING_S            string1;
    SYS_STRING_S            string2;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
            /* FALLTHROUGHT */
        case FGS_KEY_BACK:
            inst->editPars.editData.updated = FTRUE;
            (void)MENU_EditStringClose (h, FTRUE);
            break;
        case FGS_KEY_EXIT:
            inst->editPars.editData.updated = FFALSE;
            (void)MENU_EditStringClose (h, FTRUE);
            break;
        case FGS_KEY_RED:
            /* Delete back */
            if(inst->strPos == 0)
                keyUsed = FFALSE;
            else
            {
                MENU_EditStringBackSpace(inst);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_EDITSTRING_ID_WINDOW);
            }
            break;
        case FGS_KEY_GREEN:     /* Delete */
            if ( inst->strSize < 2 )
            {
                /* last character can not be deleted */
                keyUsed = FFALSE;
            }
            else
            {
                MENU_EditStringDelete(inst);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_EDITSTRING_ID_WINDOW);
            }
            break;
        case FGS_KEY_YELLOW:
            /* Insert */
            /* not more than maximum is allowed */
            if ( inst->strSize >= inst->editPars.editData.maxChars )
            {
                keyUsed = FFALSE;
            }
            else
            {
                MENU_EditStringInsert(inst);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_EDITSTRING_ID_WINDOW);
            }
            break;
        case FGS_KEY_BLUE:
            /* toggle key status */
            switch(inst->keyStatus)
            {
                 case MENU_EDITSTRING_KEY_ALL:
                    inst->keyStatus = MENU_EDITSTRING_KEY_NUM;
                    break;
                 case MENU_EDITSTRING_KEY_NUM:
                    inst->keyStatus = MENU_EDITSTRING_KEY_UPPER;
                    break;
                 case MENU_EDITSTRING_KEY_UPPER:
                    inst->keyStatus = MENU_EDITSTRING_KEY_LOWER;
                    break;
                 default:
                 case MENU_EDITSTRING_KEY_LOWER:
                    inst->keyStatus = MENU_EDITSTRING_KEY_SPECIAL;
                    break;
                 case MENU_EDITSTRING_KEY_SPECIAL:
                    inst->keyStatus = MENU_EDITSTRING_KEY_ALL;
                    break;
            }
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_EDITSTRING_BOTTOM_HELP_TEXT_BLUE);
            break;
        case FGS_KEY_LEFT:
            /* change one position left */
            if(inst->strPos == 0)
                keyUsed = FFALSE;
            else
            {
                inst->strPos--;
                if ( inst->strPos <= inst->firstVisChar )
                {
                    if ( inst->firstVisChar >= 8 )
                        inst->firstVisChar -= 8;
                    else
                        inst->firstVisChar = 0;

                }
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_EDITSTRING_ID_WINDOW);
            }
            break;
        case FGS_KEY_RIGHT:
            /* change one position right */
            if ( ((uint16_t)(inst->strPos + 1)) >= inst->strSize )
            {
                keyUsed = FFALSE;
            }
            else
            {
                inst->strPos++;

                string1 = STR_Build (inst->tmpBuf.string.cs,
                                     inst->tmpBuf.string.chr +
                                     (inst->firstVisChar *
                                     MENU_ES_BYTESPERCHAR));

                string2 = STR_Build (inst->tmpBuf.string.cs,
                                     inst->tmpBuf.string.chr +
                                     (inst->strPos *
                                     MENU_ES_BYTESPERCHAR));

                if ( (OSD_GetStrWidth (string1,
                                       (inst->strPos - inst->firstVisChar) *
                                       MENU_ES_BYTESPERCHAR,
                                       MAIN_FONT) +
                      OSD_GetStrWidth (string2, MENU_ES_BYTESPERCHAR,
                                       MAIN_FONT)) >=
                     MENU_EDITSTRING_STRING_WIDTH )
                {
                    if ( (uint16_t)(inst->strPos - inst->firstVisChar) >= 8)
                    {
                        inst->firstVisChar += 8;
                    }
                    else
                    {
                        inst->firstVisChar = inst->strPos;
                    }
                }

                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_EDITSTRING_ID_WINDOW);
            }
            break;
        case FGS_KEY_UP:
            /* get next character up the alphabet */
            MENU_EditStringGetNextChar (&(inst->tmpBuf), inst->strPos,
                                        inst->keyStatus, FTRUE);

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_EDITSTRING_ID_WINDOW);
            break;
        case FGS_KEY_DOWN:
            /* get next character down the alphabet */
            MENU_EditStringGetNextChar (&(inst->tmpBuf), inst->strPos,
                                        inst->keyStatus, FFALSE);

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_EDITSTRING_ID_WINDOW);
            break;
        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

FSTATIC void MENU_EditStringGetBoxItemText(FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                           FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_EDITSTRING_INST_S* inst = (MENU_EDITSTRING_INST_S*)h;

    uint16_t                     yPos, xPos;
    uint32_t                     prepos, lettersize;
    SYS_STRING_S            string1;
    SYS_STRING_S            string2;

    item->type                   = FGS_ITEM_TYPE_STRING;
    item->align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.font          = MAIN_FONT;
    item->item.str.lineHeight    = 0;
    item->item.str.col           = OSD_COL_LIST_TEXT;

    switch(id)
    {
        case MENU_EDITSTRING_ID_WINDOW:

            /* to do: this should be realized using the generic FGS item
               FGS_GENERIC_S! */

            item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);

            string1 = STR_Build (inst->tmpBuf.string.cs,
                                 inst->tmpBuf.string.chr +
                                 (inst->firstVisChar *
                                 MENU_ES_BYTESPERCHAR));

            string2 = STR_Build (inst->tmpBuf.string.cs,
                                 inst->tmpBuf.string.chr +
                                 (inst->strPos * MENU_ES_BYTESPERCHAR));

            prepos = OSD_GetStrWidth (string1,
                                      inst->strPos - inst->firstVisChar,
                                      MAIN_FONT);

            lettersize = OSD_GetStrWidth (string2, 1, MAIN_FONT);
            xPos = inst->pnlInst.pMem->abspos.x + MENU_EDITSTRING_STRING_POS_X;
            yPos = inst->pnlInst.pMem->abspos.y + MENU_EDITSTRING_STRING_POS_Y;

            /* clear window first */
            OSD_DrawFilledRectangle(xPos,
                                    yPos,
                                    xPos + MENU_EDITSTRING_STRING_WIDTH - 1,
                                    yPos + MENU_EDITSTRING_STRING_HEIGHT - 1,
                                    OSD_GetSchemeColor(OSD_SCHEMECOL_MAIN_BKGR));

            OSD_DrawFilledRectangle(xPos + prepos,
                                    yPos,
                                    xPos + prepos+lettersize,
                                    yPos + MENU_EDITSTRING_STRING_HEIGHT - 1,
                                    OSD_GetSchemeColor(OSD_SCHEMECOL_MAIN_BKGR_HIGHL));
            OSD_DrawStringLtd(xPos, yPos,
                              OSD_COL_TEXT,
                              MAIN_FONT, -1,
                              MENU_EDITSTRING_STRING_WIDTH, -1,
                              string1, FFALSE);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_TEXT_RED:
            item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_SYMBOL_RED);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_TEXT_GREEN:
            item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_SYMBOL_GREEN);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_TEXT_YELLOW:
            item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_SYMBOL_YELLOW);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_TEXT_BLUE:
            switch(inst->keyStatus)
            {
                default:
                case MENU_EDITSTRING_KEY_ALL:
                    item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_KEY_ALL);
                    break;
                case MENU_EDITSTRING_KEY_NUM:
                    item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_KEY_NUM);
                    break;
                case MENU_EDITSTRING_KEY_UPPER:
                    item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_KEY_UPPER);
                    break;
                case MENU_EDITSTRING_KEY_LOWER:
                    item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_KEY_LOWER);
                    break;
                case MENU_EDITSTRING_KEY_SPECIAL:
                    item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_KEY_SPECIAL);
                    break;
            }
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_ICON_MENU:
            item->item.str.chr = OSD_GetStr(STR_KEY_MENU);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_TEXT_MENU:
            item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_MENU);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_ICON_EXIT:
            item->item.str.chr = OSD_GetStr(STR_KEY_EXIT);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_TEXT_EXIT:
            item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_EXIT);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_ICON_NUMKEY:
            item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_KEY_NUM);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_TEXT_NUMKEY:
            item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_NUMKEY);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_ICON_LEFTRIGHT:
            item->align        = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
            item->item.str.font  = SYMBOL_FONT;
            item->item.str.chr = OSD_GetStr(STR_ENTRY_LR);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_TEXT_LEFTRIGHT:
            item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_LEFTRIGHT);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_ICON_UPDOWN:
            item->align        = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
            item->item.str.font= SYMBOL_FONT;
            item->item.str.chr = OSD_GetStr(STR_ENTRY_UD);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_TEXT_UPDOWN:
            item->item.str.chr = OSD_GetStr(STR_HELP_EDITSTRING_UPDOWN);
            break;
        default:
            break;
    } /* end of switch(id) */
}

FSTATIC void MENU_EditStringGetBoxItemIcon(FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                           FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
//    MENU_EDITSTRING_INST_S* inst = (MENU_EDITSTRING_INST_S*)h;

    item->type                   = FGS_ITEM_TYPE_ICON;
    item->align                  = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_TOP;
    item->item.icon.drawMode     = FGS_ICONMODE_OVRLAY;
    item->item.icon.bkgrCol      = OSD_COL_BLANK;
    item->item.icon.altBkgrCol   = OSD_COL_BLANK;

    switch(id)
    {
        case MENU_EDITSTRING_BOTTOM_HELP_ICON_RED:
            item->item.icon.addr = OSD_IconGet(ICON_BTNREDSMALL);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_ICON_GREEN:
            item->item.icon.addr = OSD_IconGet(ICON_BTNGREENSMALL);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_ICON_YELLOW:
            item->item.icon.addr = OSD_IconGet(ICON_BTNYELLOWSMALL);
            break;
        case MENU_EDITSTRING_BOTTOM_HELP_ICON_BLUE:
            item->item.icon.addr = OSD_IconGet(ICON_BTNBLUESMALL);
            break;
       default:
            break;
    } /* end of switch(id) */
}

/* Handles ASCII-subset in 16-Bit unicode character set only! */
FSTATIC fbool_t MENU_EditStringGetNextNumChar (SYS_STRBUF_S* pBuf, uint16_t charPos,
                                             MENU_EDITSTRING_KEY_STATUS
                                             status, uint8_t value)
{
    SYS_CHAR    currChar = 0;
    fbool_t     retVal   = FTRUE;

    if ( STR_CharGet (pBuf->string, charPos * MENU_ES_BYTESPERCHAR, &currChar)
                      == 0 ) return FFALSE;

    else if ( currChar == 0 ) return FFALSE;

    if ( status == MENU_EDITSTRING_KEY_NUM )
    {
        currChar = value + '0';
    }
    else
    {
        switch ( value )
        {
            case 0: /*  */
                switch ( status )
                {
                    default:
                    case MENU_EDITSTRING_KEY_LOWER:
                    case MENU_EDITSTRING_KEY_UPPER:
                        currChar = ' ';
                        break;
                    case MENU_EDITSTRING_KEY_ALL:
                        currChar = (currChar == '0') ? ' ' : '0';
                    break;
                }
                break;

            case 1: /*  */
                if      ((currChar >= ' ') && (currChar < '/')) currChar++;
                else if ((currChar >= ':') && (currChar < '@')) currChar++;
                else if ((currChar >= '[') && (currChar < 96 )) currChar++;
                else if ((currChar >= '{') && (currChar < '~')) currChar++;
                else if  (currChar == '/')                      currChar = ':';
                else if  (currChar == '@')                      currChar = '[';
                else if  (currChar ==  96)                      currChar = '{';
                else if  (currChar == '~')                      currChar = '1';
                else if  (currChar == '1')                      currChar = ' ';
                /* default */
                else                                            currChar = ' ';
                break;

            case 2: /* a b c */
                switch(status)
                {
                    default:
                    case MENU_EDITSTRING_KEY_LOWER:
                        switch( currChar )
                        {
                            case 'a' : currChar = 'b'; break;
                            case 'b' : currChar = 'c'; break;
                            default:
                            case 'c' : currChar = 'a'; break;
                        }
                        break;
                    case MENU_EDITSTRING_KEY_UPPER:
                        switch ( currChar )
                        {
                            case 'A' : currChar = 'B'; break;
                            case 'B' : currChar = 'C'; break;
                            default:
                            case 'C' : currChar = 'A'; break;
                        }
                        break;

                    case MENU_EDITSTRING_KEY_ALL:
                        switch ( currChar )
                        {
                            case 'a' : currChar = 'b'; break;
                            case 'b' : currChar = 'c'; break;
                            case 'c' : currChar = 'A'; break;
                            case 'A' : currChar = 'B'; break;
                            case 'B' : currChar = 'C'; break;
                            case 'C' : currChar = '2'; break;
                            default:
                            case '2' : currChar = 'a'; break;
                        }
                        break;
//          not usable for ISO
                    case MENU_EDITSTRING_KEY_SPECIAL:
                        switch(currChar)
                        {
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            default:
                            case '' : currChar = ''; break;
                        }
                        break;
                }
                break;

            case 3: /* d e f */
                switch(status)
                {
                    default:
                    case MENU_EDITSTRING_KEY_LOWER:
                            switch ( currChar )
                            {
                                case 'd' : currChar = 'e'; break;
                                case 'e' : currChar = 'f'; break;
                                default:
                                case 'f' : currChar = 'd'; break;
                            }
                            break;

                        case MENU_EDITSTRING_KEY_UPPER:
                            switch ( currChar )
                            {
                                case 'D' : currChar = 'E'; break;
                                case 'E' : currChar = 'F'; break;
                                default:
                                case 'F' : currChar = 'D'; break;
                        }
                        break;

                    case MENU_EDITSTRING_KEY_ALL:
                            switch ( currChar )
                            {
                                case 'd' : currChar = 'e'; break;
                                case 'e' : currChar = 'f'; break;
                                case 'f' : currChar = 'D'; break;
                                case 'D' : currChar = 'E'; break;
                                case 'E' : currChar = 'F'; break;
                                case 'F' : currChar = '3'; break;
                                default:
                                case '3' : currChar = 'd'; break;
                            }
                        break;

//          not usable for ISO
                    case MENU_EDITSTRING_KEY_SPECIAL:
                        switch(currChar)
                        {
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            case '' : currChar = ''; break;
                            default:
                            case '' : currChar = ''; break;
                        }
                        break;
            }
            break;

            case 4: /* g h i */
        switch(status)
            {
                default:
                case MENU_EDITSTRING_KEY_LOWER:
                        switch ( currChar )
                        {
                            case 'g' : currChar = 'h'; break;
                            case 'h' : currChar = 'i'; break;
                            default:
                            case 'i' : currChar = 'g'; break;
                        }
                        break;

                    case MENU_EDITSTRING_KEY_UPPER:
                        switch ( currChar )
                        {
                            case 'G' : currChar = 'H'; break;
                            case 'H' : currChar = 'I'; break;
                            default:
                            case 'I' : currChar = 'G'; break;
                    }
                    break;

                case MENU_EDITSTRING_KEY_ALL:
                        switch ( currChar )
                        {
                            case 'g' : currChar = 'h'; break;
                            case 'h' : currChar = 'i'; break;
                            case 'i' : currChar = 'G'; break;
                            case 'G' : currChar = 'H'; break;
                            case 'H' : currChar = 'I'; break;
                            case 'I' : currChar = '4'; break;
                            default:
                            case '4' : currChar = 'g'; break;
                        }
                    break;
            }
            break;

        case 5: /* j k l */
        switch(status)
            {
                default:
                    case MENU_EDITSTRING_KEY_LOWER:
                        switch ( currChar )
                        {
                            case 'j' : currChar = 'k'; break;
                            case 'k' : currChar = 'l'; break;
                            default:
                            case 'l' : currChar = 'j'; break;
                        }
                        break;

                    case MENU_EDITSTRING_KEY_UPPER:
                        switch ( currChar )
                        {
                            case 'J' : currChar = 'K'; break;
                            case 'K' : currChar = 'L'; break;
                            default:
                            case 'L' : currChar = 'J'; break;
                        }
                    break;

                    case MENU_EDITSTRING_KEY_ALL:
                        switch ( currChar )
                        {
                            case 'j' : currChar = 'k'; break;
                            case 'k' : currChar = 'l'; break;
                            case 'l' : currChar = 'J'; break;
                            case 'J' : currChar = 'K'; break;
                            case 'K' : currChar = 'L'; break;
                            case 'L' : currChar = '5'; break;
                            default:
                            case '5' : currChar = 'j'; break;
                        }
                        break;
                }
                break;

        case 6: /* m n o */
        switch(status)
            {
                default:
                case MENU_EDITSTRING_KEY_LOWER:
                        switch( currChar )
                        {
                            case 'm' : currChar = 'n'; break;
                            case 'n' : currChar = 'o'; break;
                            default:
                            case 'o' : currChar = 'm'; break;
                        }
                        break;

                    case MENU_EDITSTRING_KEY_UPPER:
                        switch ( currChar )
                        {
                            case 'M' : currChar = 'N'; break;
                            case 'N' : currChar = 'O'; break;
                            default:
                            case 'O' : currChar = 'M'; break;
                    }
                    break;

                case MENU_EDITSTRING_KEY_ALL:
                        switch ( currChar )
                        {
                            case 'm' : currChar = 'n'; break;
                            case 'n' : currChar = 'o'; break;
                            case 'o' : currChar = 'M'; break;
                            case 'M' : currChar = 'N'; break;
                            case 'N' : currChar = 'O'; break;
                            case 'O' : currChar = '6'; break;
                            default:
                            case '6' : currChar = 'm'; break;
                        }
                    break;
            }
            break;

        case 7: /* p q r s */
        switch(status)
            {
                default:
                case MENU_EDITSTRING_KEY_LOWER:
                        switch ( currChar )
                        {
                            case 'p' : currChar = 'q'; break;
                            case 'q' : currChar = 'r'; break;
                            case 'r' : currChar = 's'; break;
                            default:
                            case 's' : currChar = 'p'; break;
                        }
                        break;

                    case MENU_EDITSTRING_KEY_UPPER:
                        switch ( currChar )
                        {
                            case 'P' : currChar = 'Q'; break;
                            case 'Q' : currChar = 'R'; break;
                            case 'R' : currChar = 'S'; break;
                            default:
                            case 'S' : currChar = 'P'; break;
                    }
                    break;

                case MENU_EDITSTRING_KEY_ALL:
                        switch ( currChar )
                        {
                            case 'p' : currChar = 'q'; break;
                            case 'q' : currChar = 'r'; break;
                            case 'r' : currChar = 's'; break;
                            case 's' : currChar = 'P'; break;
                            case 'P' : currChar = 'Q'; break;
                            case 'Q' : currChar = 'R'; break;
                            case 'R' : currChar = 'S'; break;
                            case 'S' : currChar = '7'; break;
                            default:
                            case '7' : currChar = 'p'; break;
                    }
                    break;
            }
            break;
        case 8: /* t u v */
        switch(status)
            {
                default:
                case MENU_EDITSTRING_KEY_LOWER:
                        switch ( currChar )
                        {
                            case 't' : currChar = 'u'; break;
                            case 'u' : currChar = 'v'; break;
                            default:
                            case 'v' : currChar = 't'; break;
                        }
                        break;

                    case MENU_EDITSTRING_KEY_UPPER:
                        switch ( currChar )
                        {
                            case 'T' : currChar = 'U'; break;
                            case 'U' : currChar = 'V'; break;
                            default:
                            case 'V' : currChar = 'T'; break;
                    }
                    break;

                case MENU_EDITSTRING_KEY_ALL:
                        switch ( currChar )
                        {
                            case 't' : currChar = 'u'; break;
                            case 'u' : currChar = 'v'; break;
                            case 'v' : currChar = 'T'; break;
                            case 'T' : currChar = 'U'; break;
                            case 'U' : currChar = 'V'; break;
                            case 'V' : currChar = '8'; break;
                            default:
                            case '8' : currChar = 't'; break;
                    }
                    break;
            }
            break;

        case 9: /* w x y z */
            switch(status)
            {
                default:
                case MENU_EDITSTRING_KEY_LOWER:
                        switch ( currChar )
                        {
                            case 'w' : currChar = 'x'; break;
                            case 'x' : currChar = 'y'; break;
                            case 'y' : currChar = 'z'; break;
                            default:
                            case 'z' : currChar = 'w'; break;
                        }
                        break;

                    case MENU_EDITSTRING_KEY_UPPER:
                        switch ( currChar )
                        {
                            case 'W' : currChar = 'X'; break;
                            case 'X' : currChar = 'Y'; break;
                            case 'Y' : currChar = 'Z'; break;
                            default:
                            case 'Z' : currChar = 'W'; break;
                    }
                    break;

                    case MENU_EDITSTRING_KEY_ALL:
                            switch ( currChar )
                            {
                                case 'w' : currChar = 'x'; break;
                                case 'x' : currChar = 'y'; break;
                                case 'y' : currChar = 'z'; break;
                                case 'z' : currChar = 'W'; break;
                                case 'W' : currChar = 'X'; break;
                                case 'X' : currChar = 'Y'; break;
                                case 'Y' : currChar = 'Z'; break;
                                case 'Z' : currChar = '9'; break;
                                default:
                                case '9' : currChar = 'w'; break;
                            }
                            break;
                }
                break;
                default:
                    retVal = FFALSE;
                    break;
        }
    }

    if ( retVal == FTRUE )
    {
        (void)STR_CharSet (pBuf->string, charPos * MENU_ES_BYTESPERCHAR, currChar);
    }

    return retVal;
}

/* Handles ASCII-subset in 16-Bit unicode character set only! */
FSTATIC void MENU_EditStringGetNextChar (SYS_STRBUF_S* pBuf, uint16_t charPos,
                                         MENU_EDITSTRING_KEY_STATUS status,
                                         fbool_t up)
{                 // Order:  0-9 A-Z_.
    SYS_CHAR currChar = 0;

    if ( STR_CharGet (pBuf->string, charPos * MENU_ES_BYTESPERCHAR, &currChar)
                      == 0 ) return;

    else if ( currChar == 0 ) return;

    if(up)
    {
        switch(status)
        {
            default:
            case MENU_EDITSTRING_KEY_ALL:
                if      ((currChar >= 'A') && (currChar < 'Z')) currChar++;
                else if ((currChar >= 'a') && (currChar < 'z')) currChar++;
                else if ((currChar >= '0') && (currChar < '9')) currChar++;
                else if  (currChar == 'Z')                      currChar = 'a';
                else if  (currChar == 'z')                      currChar = '0';
                else                                            currChar = 'A';
                break;

            case MENU_EDITSTRING_KEY_NUM:
                if ((currChar >= '0') && (currChar < '9'))      currChar++;
                else                                            currChar = '0';
                break;

            case MENU_EDITSTRING_KEY_UPPER:
                if((currChar >= 'A') && (currChar < 'Z'))       currChar++;
                else                                            currChar = 'A';
                break;

            case MENU_EDITSTRING_KEY_LOWER:
                if((currChar >= 'a') && (currChar < 'z'))       currChar++;
                else                                            currChar = 'a';
                break;

            case MENU_EDITSTRING_KEY_SPECIAL:
                if     ((currChar >= ' ') && (currChar < '/'))  currChar++;
                else if((currChar >= ':') && (currChar < '@'))  currChar++;
                else if((currChar >= '[') && (currChar < 96))   currChar++;
                else if((currChar >= '{') && (currChar < '~'))  currChar++;
                else if(currChar == '/')                        currChar = ':';
                else if(currChar == '@')                        currChar = '[';
                else if(currChar == 96)                         currChar = '{';
                else if(currChar == '~')                        currChar = ' ';
                else                                            currChar = '~';
                break;
        }
    }
    else
    {
        switch(status)
        {
            default:
            case MENU_EDITSTRING_KEY_ALL:
                if     ((currChar > 'A') && (currChar <= 'Z'))  currChar--;
                else if((currChar > 'a') && (currChar <= 'z'))  currChar--;
                else if((currChar > '0') && (currChar <= '9'))  currChar--;
                else if(currChar == 'A')                        currChar = '9';
                else if(currChar == 'a')                        currChar = 'Z';
                else                                            currChar = 'z';
                break;

            case MENU_EDITSTRING_KEY_NUM:
                if((currChar > '0') && (currChar <= '9'))       currChar--;
                else                                            currChar = '9';
                break;

            case MENU_EDITSTRING_KEY_UPPER:
                if((currChar > 'A') && (currChar <= 'Z'))       currChar--;
                else                                            currChar = 'Z';
                break;

            case MENU_EDITSTRING_KEY_LOWER:
                if((currChar > 'a') && (currChar <= 'z'))       currChar--;
                else                                            currChar = 'z';
                break;

            case MENU_EDITSTRING_KEY_SPECIAL:
                if     ((currChar > ' ') && (currChar <= '/'))  currChar--;
                else if((currChar > ':') && (currChar <= '@'))  currChar--;
                else if((currChar > '[') && (currChar <= 96))   currChar--;
                else if((currChar > '{') && (currChar <= '~'))  currChar--;
                else if(currChar == ' ')                        currChar = '_';
                else if(currChar == ':')                        currChar = '/';
                else if(currChar == '[')                        currChar = '@';
                else if(currChar == '{')                        currChar = 96;
                else                                            currChar = '~';
                break;
        }
    }

    (void)STR_CharSet (pBuf->string, charPos * MENU_ES_BYTESPERCHAR, currChar);
}

/*
   delete character preceding actual string position. All succeeding characters
   are shifted to the left inclusive terminating '\0'
   it is expected that at entry there is still one character to the left because
   no consensous checking is done.
*/
FSTATIC void MENU_EditStringBackSpace (MENU_EDITSTRING_INST_S* inst)
{
    if ( (inst->strPos  >= 1) &&
         (inst->strSize >= 1) )
    {
        (void)STR_CharRemove (inst->tmpBuf.string,
                       (inst->strPos - 1) * MENU_ES_BYTESPERCHAR);
        inst->strSize--;
        inst->strPos--;
    }
}

/*
    delete current character and shift succeeding characters one position to
    the left.
    if position is last character, delete character and shift position one
    to the left.
    it is expected that at entry there are minimum two characters in the string because
    no consensous checking is done.
*/
FSTATIC void MENU_EditStringDelete (MENU_EDITSTRING_INST_S* inst)
{
    if ( inst->strPos < inst->strSize )
    {
        (void)STR_CharRemove (inst->tmpBuf.string,
                        inst->strPos * MENU_ES_BYTESPERCHAR);

        if ( (inst->strPos > 0) &&
             (((uint16_t)(inst->strPos + 1)) == inst->strSize) )
        {
            inst->strPos--;
        }

        inst->strSize--;
    }
}


/*
    insert at current position one character and shift all succeeding characters
    one position to the right.
    it is expected that there is still one position available to the right because
    no consensous checking is done
*/
FSTATIC void MENU_EditStringInsert (MENU_EDITSTRING_INST_S* inst)
{
    uint8_t newPos;

    if( (inst->strPos == 0) && (inst->strSize == 0) )
    {
        newPos = 0;
    } else {
        newPos = inst->strPos + 1;
    }
    if ( STR_BufInsertChar (&(inst->tmpBuf),
                            newPos * MENU_ES_BYTESPERCHAR,
                            '_', MENU_EDITSTRING_BUFCHARSET) > 0 )
    {
        inst->strSize++;
        inst->strPos = newPos;
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
