/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_epg_sched.c
**
**              $Id: app_menu_epg_sched.c 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       Schedule EPG menu implementation.
**
** Detail description. [\ref app_smartgo_app_ref_epg]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: loss of precision
** - Info    715: symbol 'anySymb' not referenced
** - Info    737: loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"
#include "app_menu_info.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define OSDINF_SCHED_LINES    10

/* start pixels: left-aligned items: relative to title box */
#define OSDINF_TITLE_X        (SCALER_GetCurrVal(MAIN_TEXT_PAD_X)+ \
                               SCALER_GetCurrVal(INFO_NUM_WIDTH) + \
                               SCALER_GetCurrVal(INNER_FRAME_WIDTH) + \
                               (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X)))

/* start pixels: right-aligned items: relative to title box end */
#define OSDINF_TIME_X         (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) + \
                               SCALER_GetCurrVal(INFO_TIME_WIDTH))

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum
{
    MENU_EPGSCHED_ID_LIST = 0,
    MENU_EPGSCHED_ID_TITLE,
    MENU_EPGSCHED_ID_SERVICENUM,
    MENU_EPGSCHED_ID_TIME,
    MENU_EPGSCHED_ID_REST,
    MENU_EPGSCHED_ID_REST2,
    MENU_EPGSCHED_ID_DESCBOX,
    MENU_EPGSCHED_ID_DAYBOX,
    MENU_EPGSCHED_ID_DATEBOX,
    MENU_EPGSCHED_ID_TODAYBOX,
    MENU_EPGSCHED_ID_HELPBOTTOM
};

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    lstHdl;
    FAPI_SYS_HandleT    descHdl;

    FGS_PANEL_MEMBER_S* pItem;
    FGS_POS_S           itemPos;

    FGS_PANEL_MEMBER_S* pItem2;
    FGS_POS_S           itemPos2;

    /* schedule list parameters */
    SYS_TIME_S          topTimeUtc;
    SYS_DATE_S          topDateUtc;
    SYS_DATE_S          topDate;
    uint16_t            dateOffset;
    EIT_POS_S           firstRef;
    uint32_t            numDateItems;
    SYS_SERVLOC_S*      pServLocCurr;

    EIT_EVT_S           schedItem;
    EIT_EVT_S           tmpItem;

} MENU_EPGSCHED_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC MENU_EPGSCHED_INST_S*  menuEpgSchedDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_EpgSchedLeave       (FAPI_SYS_HandleT h,
                                          fbool_t force);
FSTATIC void    MENU_EpgSchedOnFocusSet  (FAPI_SYS_HandleT h,
                                          fbool_t hasFocus);
FSTATIC fbool_t MENU_EpgSchedKeys        (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_EpgSchedTrans       (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_EpgSchedChgDate     (FAPI_SYS_HandleT h);
FSTATIC void    MENU_EpgSchedUpdateDesc  (FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint16_t newIdx);
FSTATIC int32_t MENU_EpgSchedDraw        (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_EpgSchedDrawDesc    (FAPI_SYS_HandleT h);
FSTATIC void    MENU_EpgSchedGetItem     (FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item);
FSTATIC void    MENU_EpgSchedGetDateItem (FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item);
#ifdef APPL_RECTIMER_ENABLED
FSTATIC fbool_t MENU_EpgSchedTimerAdd    (FAPI_SYS_HandleT h);
#endif /* APPL_RECTIMER_ENABLED */

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
/*!
******************************************************************************
** \brief   TODO
**
** \return  one of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_ALREADY_INITIALIZED
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
******************************************************************************
*/
int32_t MENU_EpgSchedInit (void)
{
    if ( menuEpgSchedDat != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    menuEpgSchedDat = UTILS_Malloc(sizeof(MENU_EPGSCHED_INST_S));
    if ( menuEpgSchedDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(menuEpgSchedDat, 0, sizeof(MENU_EPGSCHED_INST_S));

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle (unused).
** \param   test    TODO
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
fbool_t MENU_EpgSchedEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_PANEL_MEMBER_S    member;
    FGS_LNAVI_OPEN_S      scrollDat;
    FGS_BOX_OPEN_S        boxPars;
    MENU_EPGSCHED_INST_S* inst = menuEpgSchedDat;
    int32_t               retVal;
    int32_t               idCnt;
    FGS_ITEM_S            item;
    FAPI_SYS_HandleT      itemHdl;
    uint32_t              numLines = OSDINF_SCHED_LINES;
    uint8_t               numHelpLines=1;
    uint16_t              stackedFrameWidth;
    uint16_t              menuHeight;
    uint16_t              helpWidth;

    if ( ( inst         == NULL ) || // test if initialzed
         ( inst->pnlHdl != NULL ) )  // test if already open
    {
        return FFALSE;
    }
    if ( test == FTRUE )
    {
        return FTRUE;
    }

    memset(inst,       0, sizeof(MENU_EPGSCHED_INST_S));
    memset(&pnlPars,   0, sizeof(FGS_PANEL_OPEN_S));
    memset(&scrollDat, 0, sizeof(FGS_LNAVI_OPEN_S));
    memset(&member,    0, sizeof(member));
    memset(&item,      0, sizeof(item));

    helpWidth         = 0;
    stackedFrameWidth = (MENUSTACK_GetLevel() - 1) *
                        (SCALER_GetCurrVal(STACKED_FRAME_DIST) +
                                SCALER_GetCurrVal(OUTER_FRAME_ALL));
    menuHeight        = stackedFrameWidth + (2*SCALER_GetCurrVal(OUTER_FRAME_ALL)) +
                        (2*SCALER_GetCurrVal(TITLE_LINE_HEIGHT)) +
                        ( numLines        * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                        ( numHelpLines    * SCALER_GetCurrVal(HELP_LINE_HEIGHT)) +
                        ((numHelpLines -1)* SCALER_GetCurrVal(HELP_LINE_PADDING)) +
                        (2*SCALER_GetCurrVal(HELP_TB_PADDING)) +
                        (3*SCALER_GetCurrVal(INNER_FRAME_WIDTH)) +
                        (2*SCALER_GetCurrVal(LIST_LINE_HEIGHT));
    do
    {
        pnlPars.pos.width     = SCALER_GetCurrVal(EPG_SCHED_WIDTH) +
                                helpWidth +
                                stackedFrameWidth;
        pnlPars.pos.height    = menuHeight;
        pnlPars.bkgrType      = OSD_PNLTYPE_EPGEXT2;
        pnlPars.onFocusSet    = MENU_EpgSchedOnFocusSet;
        pnlPars.onFocusSetArg = (FAPI_SYS_HandleT)inst;
        pnlPars.pInst         = &(appDat->pnlInst);

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_EPGSCHED, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create scroll list ---------------------------------------- */

        scrollDat.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                  + stackedFrameWidth;
        scrollDat.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                  + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                  + (2*SCALER_GetCurrVal(TITLE_TB_PADDING))
                                  + (2*SCALER_GetCurrVal(TITLE_LINE_HEIGHT))
                                  + stackedFrameWidth;
        scrollDat.pos.width       = pnlPars.pos.width
                                  - SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                  - SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        scrollDat.pos.height      = OSDINF_SCHED_LINES *
                                    (SCALER_GetCurrVal(LIST_LINE_HEIGHT));
        scrollDat.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        scrollDat.bkgrType        = OSD_NAVIBKGR_NORMAL;
        scrollDat.type            = FGS_LNAVI_TYPE_SCROLL;
        scrollDat.rowHeight       = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        scrollDat.scrollbarWidth  = SCALER_GetCurrVal(SCROLLBAR_WIDTH);
        scrollDat.numVisLines     = OSDINF_SCHED_LINES;
        scrollDat.numColumns      = 2;
        scrollDat.params.numItems = 0;
        scrollDat.params.index    = 0;
        scrollDat.params.posInVis = 0;
        scrollDat.colStart[0]     = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        scrollDat.colWidth[0]     = SCALER_GetCurrVal(EPG_SCHED_TIME_WIDTH);
        scrollDat.colStart[1]     = scrollDat.colStart[0] +
                                    scrollDat.colWidth[0];
        scrollDat.colWidth[1]     = scrollDat.pos.width -
                                    scrollDat.colStart[1] -
                                    scrollDat.scrollbarWidth -
                                    SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        scrollDat.topDownOrient   = FTRUE;
        scrollDat.wraparound      = FTRUE;
        scrollDat.getItem         = MENU_EpgSchedGetItem;
        scrollDat.getItemArg      = (FAPI_SYS_HandleT)inst;
        scrollDat.onFocusChg      = MENU_EpgSchedUpdateDesc;
        scrollDat.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        scrollDat.pInst           = &(appDat->lnaviInst);

        retVal = FGS_LNAVI_Open (&scrollDat, MENU_EPGSCHED_ID_LIST,
                                 inst->pnlHdl,
                                &(inst->lstHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create service number textbox ----------------------------- */

        retVal = MENU_InfoServiceNumBoxCreate(inst->pnlHdl,
                                          MENU_EPGSCHED_ID_SERVICENUM,
                                          MENUID_EPGSCHED);
        if ( retVal != FAPI_OK ) break;

        /* ----- create service name textbox ------------------------------- */

        retVal = MENU_InfoServiceNameBoxCreate(inst->pnlHdl,
                                  pnlPars.pos.width -
                                  (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH)) -
                                  OSDINF_TITLE_X - OSDINF_TIME_X -
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH) -
                                  (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X)),
                                  MENU_EPGSCHED_ID_TITLE,
                                  MENUID_EPGSCHED);
        if ( retVal != FAPI_OK ) break;

        /* ----- create time textbox --------------------------------------- */

        retVal = MENU_InfoTimeBoxCreate(inst->pnlHdl, MENU_EPGSCHED_ID_TIME);
        if ( retVal != FAPI_OK ) break;

        /* ----- Create calendar date box ---------------------------------- */

        memset(&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X) +
                                  stackedFrameWidth;
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                  stackedFrameWidth;
        boxPars.pos.width       = SCALER_GetCurrVal(EPG_SCHED_DATE_WIDTH);
        boxPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_TITLE;
        boxPars.getItem         = MENU_EpgSchedGetDateItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPGSCHED_ID_DAYBOX,
                                inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x          += boxPars.pos.width;
        boxPars.pos.width       = SCALER_GetCurrVal(EPG_SCHED_DATE_WIDTH);
        retVal = FGS_BOX_Open (&boxPars, MENU_EPGSCHED_ID_DATEBOX,
                                inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x          += boxPars.pos.width;
        boxPars.pos.width       = SCALER_GetCurrVal(EPG_SCHED_TODAY_WIDTH);
        retVal = FGS_BOX_Open (&boxPars, MENU_EPGSCHED_ID_TODAYBOX,
                                inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- Create EIT description text box --------------------------- */

        memset(&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        boxPars.pos.y           = scrollDat.pos.y
                                + scrollDat.pos.height
                                + SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.width       = pnlPars.pos.width
                                - (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH))
                                - (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X));
        boxPars.pos.height      = (2 * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                                    SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;

        boxPars.item[FGS_BOX_IDX_NONFOCUS].type = FGS_ITEM_TYPE_NONE;
        boxPars.item[FGS_BOX_IDX_FOCUS].type    = FGS_ITEM_TYPE_NONE;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPGSCHED_ID_DESCBOX,
                               inst->pnlHdl, &(inst->descHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        inst->itemPos.x         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->itemPos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->itemPos.width     = pnlPars.pos.width -
                                  (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        inst->itemPos.height    = pnlPars.pos.height -
                                  (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h        = (FAPI_SYS_HandleT)inst;
        member.pos      = &(inst->itemPos);
        member.id       = MENU_EPGSCHED_ID_REST;
        member.isHidden = FFALSE;
        member.setFocus = NULL;
        member.show     = MENU_EpgSchedDraw;
        member.close    = NULL;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pItem));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */
        inst->itemPos2.x         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->itemPos2.y         = scrollDat.pos.y + scrollDat.pos.height;
        inst->itemPos2.width     = pnlPars.pos.width -
                                   (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        inst->itemPos2.height    = pnlPars.pos.height -
                                   (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        inst->itemPos2.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.pos      = &(inst->itemPos2);
        member.id       = MENU_EPGSCHED_ID_REST2;
        member.show     = MENU_EpgSchedDrawDesc;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pItem2));
        if ( retVal != FAPI_OK ) break;

        /* -----Bottom help lines ------------------------------------------ */

        idCnt = MENU_EPGSCHED_ID_HELPBOTTOM;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_EXIT);

        boxPars.pos.x               = stackedFrameWidth +
                                      SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = -((int16_t)((SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2)
                                      + SCALER_GetCurrVal(HELP_TB_PADDING)
                                      + (numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT))
                                      + ((numHelpLines-1)*SCALER_GetCurrVal(HELP_LINE_PADDING))));
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,
                                                      -1, HELP_FONT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_MENU_EXIT);
        boxPars.pos.x              += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,
                                                      -1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNBLUESMALL);

        boxPars.pos.x               = (((uint32_t)pnlPars.pos.width)/3)
                                    + SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_EPG_HELP_ADDTOTIMERS);

        boxPars.pos.x              += (int16_t)(SCALER_GetCurrVal(HELP_BOT_XOFF));
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,
                                                      -1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        MENU_InfoServiceNumBoxUpdate();

        MENU_EpgSchedUpdate();

        (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_EPGSCHED_ID_LIST);
        (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_EPGSCHED);
        (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);
        }

        inst->pnlHdl = NULL;
        return FFALSE;
    }

    MENUSTACK_SetCleanup (MENU_EpgSchedLeave, (FAPI_SYS_HandleT)inst);
    return FTRUE;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Schedule EPG Instants.
** \param   force   TODO
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgSchedLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_EPGSCHED_INST_S* inst = (MENU_EPGSCHED_INST_S*)h;

    DBG_Assert(inst->pnlHdl != 0);

    if ( inst == NULL )
    {
        return FFALSE;
    }

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
    }

    MENU_InfoTimeBoxRemove();
    MENU_InfoServiceNumBoxRemove();
    MENU_InfoListSymbolBoxRemove();
    MENU_InfoServiceNameBoxRemove();

    MENU_InfoServiceNumBoxInputAbort(); /* reset service number input mode */

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_EPGSCHED);

    inst->pnlHdl = NULL;
    return FTRUE;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Schedule EPG Instants.
**
** \return  one of the following status codes:
**          - #FAPI_OK always
**
******************************************************************************
*/
FSTATIC int32_t MENU_EpgSchedDraw (FAPI_SYS_HandleT h)
{
    MENU_EPGSCHED_INST_S* inst = (MENU_EPGSCHED_INST_S*)h;

    /* Vertical separators in top line */
    OSD_DrawSeparatorVertical((uint16_t)inst->pItem->abspos.x +
                              OSDINF_TITLE_X -
                              SCALER_GetCurrVal(MAIN_TEXT_PAD_X) -
                              SCALER_GetCurrVal(INNER_FRAME_WIDTH),
                              (uint16_t)inst->pItem->abspos.y,
                              SCALER_GetCurrVal(TITLE_LINE_HEIGHT));

    OSD_DrawSeparatorVertical((uint16_t)inst->pItem->abspos.x +
                              inst->itemPos.width -
                              OSDINF_TIME_X -
                              SCALER_GetCurrVal(MAIN_TEXT_PAD_X) -
                              SCALER_GetCurrVal(INNER_FRAME_WIDTH),
                              (uint16_t)inst->pItem->abspos.y,
                              SCALER_GetCurrVal(TITLE_LINE_HEIGHT));

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Schedule EPG Instants.
**
** \return  one of the following status codes:
**          - #FAPI_OK always
**
******************************************************************************
*/
FSTATIC int32_t MENU_EpgSchedDrawDesc (FAPI_SYS_HandleT h)
{
    MENU_EPGSCHED_INST_S* inst = (MENU_EPGSCHED_INST_S*)h;

    OSD_DrawSeparatorHorizontal((uint16_t)(inst->pItem2->abspos.x),
                                (uint16_t)(inst->pItem2->abspos.y),
                                           inst->itemPos2.width);

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h           Handle of the Schedule EPG Instants.
** \param   hasFocus    #FTRUE set the focus to this menu #FFALSE remove the
**                      focus.
**
******************************************************************************
*/
FSTATIC void MENU_EpgSchedOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_EPGSCHED_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleZappingKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler   (FGS_KEY_MENU,  MAIN_DirectMenuTrans,   h,    FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_BACK,  MENUSTACK_Exit,         NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_EXIT,  MENUSTACK_Exit,         NULL, FTRUE,  FFALSE);
        FGS_SetAppNumKeyHandler(FGS_KEY_NUMBER,MENU_InfoServiceNumBoxInput,NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_NAVI,  MENU_EpgSchedTrans,     h,    FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_INFO,  MAIN_DirectMenuTrans,   h,    FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_LEFT,  MENU_EpgSchedChgDate,   h,    FTRUE,  FTRUE);
        FGS_SetAppKeyHandler   (FGS_KEY_RIGHT, MENU_EpgSchedChgDate,   h,    FTRUE,  FTRUE);
        FGS_SetAppKeyHandler   (FGS_KEY_OK,    MENU_EpgSchedKeys,      h,    FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_BACK,  MAIN_KeyPIP,            NULL, FTRUE,  FFALSE);

        #ifdef APPL_RECTIMER_ENABLED
        FGS_SetAppKeyHandler   (FGS_KEY_BLUE,  MENU_EpgSchedTimerAdd,  h,    FTRUE,  FFALSE);
        #endif /* APPL_RECTIMER_ENABLED */
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler(FGS_KEY_NUMBER);
        FGS_ClearAppKeyHandler(FGS_KEY_NAVI);
        FGS_ClearAppKeyHandler(FGS_KEY_INFO);
        FGS_ClearAppKeyHandler(FGS_KEY_LEFT);
        FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
        FGS_ClearAppKeyHandler(FGS_KEY_OK);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);

        #ifdef APPL_RECTIMER_ENABLED
        FGS_ClearAppKeyHandler(FGS_KEY_BLUE);
        #endif /* APPL_RECTIMER_ENABLED */
    }
}

/*!
*******************************************************************************
**
** \brief   Handles the Keys adapted by this menu.
**
** This function is used as callback for the FGS_SetAppKeyHandler() and
** handles the following Keys adapted by the schedule EPG:
** - #FGS_KEY_OK
**
** \param   h   Address of schedule EPG handle.
**
** \return  One of the following status codes:
**          - #FTRUE    If the key was used by this menu.
**          - #FFALSE   If the key wasn't used.
**
** \sa
**          - FGS_SetAppKeyHandler()
**          - MENU_SchedEpgOnFocusSet()
**
*******************************************************************************-
*/
FSTATIC fbool_t MENU_EpgSchedKeys (FAPI_SYS_HandleT h)
{
    fbool_t keyUsed = FTRUE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_OK:
            if(MENU_InfoIsInputMode() == FTRUE)
            {
                MAIN_SelectServiceByInput();
                keyUsed = FTRUE;
            }
            else
            {
                keyUsed = MENU_EpgSchedTrans(h);
            }
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

/*!
******************************************************************************
**
** \brief   Handles the direct transfer between SmartGo-HD EPG's.
**
** This function is used as callback for the FGS_SetAppKeyHandler() and
** handles the following Keys adapted by the schedule EPG:
** - #FGS_KEY_OK    only if not in Service number input mode.
** - #FGS_KEY_NAVI
**
** \param   h   Address of schedule EPG handle.
**
** \return  One of the following status codes:
**          - #FTRUE    If the key was used by this menu.
**          - #FFALSE   If the key wasn't used.
**
** \sa
**          - FGS_SetAppKeyHandler()
**          - MENU_SchedEpgOnFocusSet()
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgSchedTrans (FAPI_SYS_HandleT h)
{
    MENU_EPGSCHED_INST_S* inst = (MENU_EPGSCHED_INST_S*)h;
    MENUSTACK_ENTRY       newEntry;
    FAPI_SYS_HandleT      arg = NULL;
    FGS_LNAVI_PARAMS_S    naviParams;
    MENU_EPGEXT_OPEN_S    epgExtParams;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_OK:
            memset (&epgExtParams, 0, sizeof(epgExtParams));

            /* if schedule data is not (yet) available, show current/next
             * events
             */
            if ( inst->numDateItems  == 0 )
            {
                epgExtParams.schedMode = FFALSE;
                epgExtParams.currType  = EIT_TYPE_CURR;
            }
            else
            {
                epgExtParams.schedMode = FTRUE;
                epgExtParams.currRef   = inst->firstRef;

                if ( ( FGS_LNAVI_GetParams(inst->lstHdl,
                                           &naviParams) == FAPI_OK) &&
                     (EIT_GetSchedByReference ((int16_t)(naviParams.index),
                                                NULL, NULL,
                                                &(epgExtParams.currRef))
                                              != FAPI_OK) )
                {
                    memset(&(epgExtParams.currRef), 0, sizeof(EIT_POS_S));
                }
            }

            newEntry = MENU_EpgExtEntry;
            arg      = (FAPI_SYS_HandleT)(&epgExtParams);
            break;

        case FGS_KEY_NAVI:
            newEntry = MENU_EpgGridEntry;
            arg      = NULL;
            break;

        default:
            return FFALSE;
    }

    return MENUSTACK_Switch(newEntry, arg, -1);
}

/*!
******************************************************************************
**
** \brief   Handles the date change in schedule EPG.
**
** This function is used as callback for the FGS_SetAppKeyHandler() and
** handles the following Keys adapted by the schedule EPG:
** - #FGS_KEY_LEFT
** - #FGS_KEY_RIGHT
**
** \param   h   Address of schedule EPG handle.
**
** \return  One of the following status codes:
**          - #FTRUE    If the key was used by this menu.
**          - #FFALSE   If the key wasn't used.
**
** \sa
**          - FGS_SetAppKeyHandler()
**          - MENU_SchedEpgOnFocusSet()
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgSchedChgDate (FAPI_SYS_HandleT h)
{
    MENU_EPGSCHED_INST_S* inst = (MENU_EPGSCHED_INST_S*)h;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_LEFT:
            if ( inst->dateOffset < 1 )
                 inst->dateOffset = AV_SCHED_DAYS_MAX - 1;
            else inst->dateOffset--;
            break;

        case FGS_KEY_RIGHT:
            if ( (uint16_t)(inst->dateOffset + 1) >= AV_SCHED_DAYS_MAX )
                  inst->dateOffset = 0;
            else  inst->dateOffset++;
            break;

        default:
           return FFALSE;
    }

    MENU_EpgSchedUpdate();
    return FTRUE;
}

#ifdef APPL_RECTIMER_ENABLED
/*!
******************************************************************************
**
** \brief   Handles adding timer on selected event in schedule EPG.
**
** This function is used as callback for the FGS_SetAppKeyHandler() and
** handles the following Keys adapted by the schedule EPG:
** - #FGS_KEY_BLUE
**
** \param   h   Address of schedule EPG handle.
**
** \return  One of the following status codes:
**          - #FTRUE    If the key was used by this menu.
**          - #FFALSE   If the key wasn't used.
**
** \sa
**          - FGS_SetAppKeyHandler()
**          - MENU_SchedEpgOnFocusSet()
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgSchedTimerAdd (FAPI_SYS_HandleT h)
{
    MENU_EPGSCHED_INST_S*   inst = (MENU_EPGSCHED_INST_S*)h;
    EIT_EVT_S               schedItem;
    RECTIMER_ITEM_S         timer;
    int32_t                 lengthInSecs;
    MENU_TIMEREDIT_PARAMS_S teditPars;
    EIT_POS_S               schedRef;
    FGS_LNAVI_PARAMS_S      naviParams;

    memset(&timer, 0, sizeof(timer));

    if ( inst->pServLocCurr == NULL ) return FFALSE;

    if ( FGS_LNAVI_GetParams(inst->lstHdl, &naviParams) != FAPI_OK )
    {
        return FFALSE;
    }

    timer.serviceIndex = SLIST_GetServiceIndex(inst->pServLocCurr);

    schedRef = inst->firstRef;
    if ( EIT_GetSchedByReference(naviParams.index,
                                 &schedItem,
                                 NULL,
                                 &schedRef) != FAPI_OK )
    {
        return FFALSE;
    }

    lengthInSecs = (int32_t)TIME_ConvToSeconds(&(schedItem.duration));

    (void)STR_Copy (STR_Build(schedItem.nameCharSet, schedItem.name),
                    timer.desc, &(timer.descCharSet),
                    schedItem.nameCharSet,
                    RECTIMER_DESC_LENGTH);

    (void)STR_Copy (STR_Build(schedItem.descCharSet, schedItem.desc),
                    timer.extDesc, &(timer.extDescCharSet),
                    schedItem.descCharSet,
                    RECTIMER_DESC_LENGTH);

    timer.type      = RECTIMER_TYPE_SINGLE;
    timer.enabled   = FTRUE;
    timer.startTime = schedItem.startTime;
    timer.startDate = schedItem.startDate;

    timer.startTime.second = 0; /* clear seconds: record at full minute only! */

    /* set pre-recording interval */
    TIME_AddSeconds (&(timer.startTime),
                     &(timer.startDate), -(RECORD_PRE_SECONDS));

    /* set duration + post-recording interval */
    TIME_AddSeconds (&(schedItem.startTime),
                     &(schedItem.startDate), lengthInSecs + RECORD_POST_SECONDS);

    timer.endTime     = schedItem.startTime;
    timer.endDate     = schedItem.startDate;
    timer.eventId     = schedItem.id;
    timer.autoStandby = FFALSE;

    #ifdef APPL_PVR_ENABLED

        timer.operation    = RECTIMER_OP_RECORD;
        timer.targetDevice = MAIN_PvrGetDefaultRecDevice();

        if ( timer.targetDevice == SYS_DEV_NULL )
        {
            /* no recording device found */
            MENU_CommonMsgShow(STR_MSG_NORECDEVICE, FFALSE);
            return FFALSE;
        }

    #else

        timer.operation    = RECTIMER_OP_SWITCH_SERVICE;
        timer.targetDevice = SYS_DEV_NULL;

    #endif /* APPL_PVR_ENABLED */

    teditPars.timerIndex = -1;
    teditPars.newDat     = &timer;
    teditPars.newService = *(inst->pServLocCurr);

    /* jump to Timer Edit Menu */
    return MENUSTACK_Up (MENU_TimerEditEntry, (FAPI_SYS_HandleT)(&teditPars));
}
#endif /* APPL_RECTIMER_ENABLED */

/*!
******************************************************************************
** \brief   TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgSchedUpdate (void)
{
    MENU_EPGSCHED_INST_S*   inst        = menuEpgSchedDat;
    int16_t                 utcOffset   = 0;
    fbool_t                 dstIsActive = FFALSE;
    SYS_TIME_S              currTime;
    SYS_DATE_S              currDate;

    if ( ( inst         == NULL ) ||
         ( inst->pnlHdl == NULL ) )
    {
        return;
    }

    inst->numDateItems = 0;

    (void)TIME_TimeAndDateGetUtc(&currTime, &currDate, NULL);
     /* ignore the has synced flag to have an valid date
      * instead of 0000-00-00
      */
    inst->topTimeUtc = currTime;
    inst->topDateUtc = currDate;

    (void)SETTINGS_TimeOffsetGet (&utcOffset, &dstIsActive);

    if ( dstIsActive )
    {
        utcOffset += 60;
    }

    /* ----- update date --------------------------------------------------- */
    /* add UTC offset */
    TIME_AddSeconds (&(inst->topTimeUtc), &(inst->topDateUtc),
                     (int32_t)utcOffset * 60);

    /* add day offset */
    TIME_AddDays(&inst->topDateUtc, (int32_t)inst->dateOffset);

    /* store UTC-corrected date for displaying */
    inst->topDate = inst->topDateUtc;

    memset(&inst->topTimeUtc, 0, sizeof(SYS_TIME_S));

    TIME_AddSeconds (&(inst->topTimeUtc), &(inst->topDateUtc),
                     -((int32_t)utcOffset * 60));

    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPGSCHED_ID_DAYBOX);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPGSCHED_ID_DATEBOX);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPGSCHED_ID_TODAYBOX);

    MENU_EpgSchedUpdateList (FTRUE);
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Schedule EPG Instants.
** \param   id      The item identifier.
** \param   index   TODO
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_EpgSchedGetDateItem (FAPI_SYS_HandleT h,
                                       int32_t id,
                                       uint32_t index,
                                       FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item)
{
    MENU_EPGSCHED_INST_S*   inst = (MENU_EPGSCHED_INST_S*)h;
    uint32_t                strIdx;

    switch(id)
    {
        case MENU_EPGSCHED_ID_DAYBOX:
            (void)STR_BufAppend (NULL,0,
                                OSD_GetWeekDayStr(inst->topDate.dayOfWeek),
                                 MAIN_LINEBUFCHARSET);
            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_LEFT |
                                        FGS_ALIGN_VER_CENTER;
            item->item.str.chr        = STR_IntLineStr();
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.col        = OSD_COL_TITLE_TEXT;
            break;
        case MENU_EPGSCHED_ID_DATEBOX:
            item->type             = FGS_ITEM_TYPE_DATE;
            item->align            = FGS_ALIGN_HOR_LEFT |
                                     FGS_ALIGN_VER_CENTER;
            item->item.date.value  = inst->topDate;
            item->item.date.format = appDat->dataFormatNorm;
            item->item.date.font   = MAIN_FONT;
            item->item.date.col    = OSD_COL_TITLE_TEXT;
            break;
        case MENU_EPGSCHED_ID_TODAYBOX:
            if ( inst->dateOffset == 0 )
            {
                strIdx = STR_GNR_TODAY;
            }
            else if ( inst->dateOffset == 1 )
            {
                strIdx = STR_GNR_TOMORROW;
            }
            else
            {
                strIdx = STR_EMPTYSTRING;
            }
            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_LEFT |
                                        FGS_ALIGN_VER_CENTER;
            item->item.str.chr        = OSD_GetStr(strIdx);
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.col        = OSD_COL_TITLE_TEXT;
            break;
        default:
            break;
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   resetIndex  TODO
**
******************************************************************************
*/
void MENU_EpgSchedUpdateList (fbool_t resetIndex)
{
    MENU_EPGSCHED_INST_S* inst          = menuEpgSchedDat;
    fbool_t               currIdxFound  = FFALSE;
    SYS_TIME_S            timeCalc      = inst->topTimeUtc;
    SYS_DATE_S            dateCalc      = inst->topDateUtc;
    uint16_t              currActiveIdx = 0;
    fbool_t               error         = FFALSE;
    SYS_TIME_S            timeCurr;
    SYS_DATE_S            dateCurr;
    EIT_EVT_S             schedEvt;
    EIT_POS_S             schedRef;
    fbool_t               hasSynced;
    SYS_PRESSTATE_E       presState;
    FGS_LNAVI_PARAMS_S    naviParams;

    if ( ( inst         == NULL ) ||
         ( inst->pnlHdl == NULL ) )
    {
        return;
    }
    inst->numDateItems = 0;

    do
    {
        (void)TIME_TimeAndDateGetUtc(&timeCurr, &dateCurr, &hasSynced);

        if ( hasSynced == FFALSE ) break;

        if ( SYS_GetPresentationState(&presState) != FAPI_OK ) break;

        switch ( presState )
        {
            default:
            case SYS_PRESSTATE_NONE:
                error = FTRUE;
                break;

            case SYS_PRESSTATE_LIVETV:
                inst->pServLocCurr = MENU_InfoGetCurrServLoc();
                break;

            case SYS_PRESSTATE_PLAYBACK:
                inst->pServLocCurr = NULL;
                break;
        }

        if ( error ) break;

        if ( EIT_GetSchedByDate(inst->pServLocCurr, &schedEvt, NULL, &schedRef,
                                &(inst->topTimeUtc), &(inst->topDateUtc),
                                FFALSE) < 0 )
        {
            break;
        }

        /* EIT_GetSchedByDate might provide an event with a start time before
           the specified start time. If this is the case, choose next event
           in chronological list. */
        if ( TIME_IsLaterThan(&inst->topTimeUtc, &inst->topDateUtc,
                              &(schedEvt.startTime),
                              &(schedEvt.startDate)) == FTRUE )
        {
            if ( EIT_GetSchedNext(&schedEvt, NULL, &schedRef) != FAPI_OK )
            {
                break;
            }
        }

        inst->firstRef = schedRef; /* store reference to first list item */

        /* get date's end time */
        TIME_AddDays   (&dateCalc, 1);
        TIME_AddSeconds(&timeCalc, &dateCalc, -1);

        while ( 1 )
        {
            /* test if event starts within date to be displayed */
            if ( TIME_IsLaterThan (&(schedEvt.startTime),
                                   &(schedEvt.startDate),
                                   &timeCalc, &dateCalc) == FTRUE )
            {
                break;
            }

            inst->numDateItems++;

            /* search for currently active item */
            if ( (inst->dateOffset == 0) &&
                 (currIdxFound     == FFALSE) )
            {
                if ( TIME_IsLaterThan (&(schedEvt.startTime),
                                       &(schedEvt.startDate),
                                       &timeCurr, &dateCurr) == FTRUE )
                {
                    if ( currActiveIdx > 0 ) currActiveIdx--;

                    currIdxFound = FTRUE;
                }
                else
                {
                    currActiveIdx++;
                }
            }

            if ( EIT_GetSchedNext(&schedEvt, NULL, &schedRef) != FAPI_OK )
            {
                /* end of list */
                break;
            }
        }
    }
    while ( 0 );

    if ( FGS_LNAVI_GetParams(inst->lstHdl, &naviParams) == FAPI_OK )
    {
        naviParams.numItems = inst->numDateItems;

        if ( resetIndex ) /* adjust index only if requested */
        {
            naviParams.index = currActiveIdx;
        }

        (void)FGS_LNAVI_SetParams(inst->lstHdl, &naviParams);
    }

    /* update description of focussed entry: get list parameters a second time,
       might have been adjusted internally at last call of
       (void)FGS_LNAVI_SetParams(). */
    if ( FGS_LNAVI_GetParams(inst->lstHdl, &naviParams) == FAPI_OK )
    {
        MENU_EpgSchedUpdateDesc((FAPI_SYS_HandleT)inst, 0, naviParams.index);
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h           Handle of the Schedule EPG Instants.
** \param   id          TODO
** \param   schedIdx    TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgSchedUpdateDesc (FAPI_SYS_HandleT h,
                                      int32_t id,
                                      uint16_t schedIdx)
{
    MENU_EPGSCHED_INST_S* inst = (MENU_EPGSCHED_INST_S*)h;
    FGS_ITEM_S            boxItem;
    EIT_POS_S             schedRef;

    schedRef = inst->firstRef;

    if ( (inst->numDateItems == 0) ||
         (EIT_GetSchedByReference ((int16_t)schedIdx, &(inst->schedItem), NULL,
                                   &schedRef) != FAPI_OK) )
    {
        (void)FGS_BOX_Clear(inst->descHdl);
    }
    else
    {
        boxItem.type                = FGS_ITEM_TYPE_STRING;
        boxItem.align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        boxItem.item.str.chr        = STR_Build (inst->schedItem.descCharSet,
                                                 inst->schedItem.desc);
        boxItem.item.str.font       = MAIN_FONT;
        boxItem.item.str.lineHeight = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxItem.item.str.col        = OSD_COL_TEXT;

        (void)FGS_BOX_SetNonFocusItem(inst->descHdl, &boxItem);
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Schedule EPG Instants.
** \param   id      The item identifier.
** \param   index   TODO
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_EpgSchedGetItem (FAPI_SYS_HandleT h,
                                   int32_t id,
                                   uint32_t index,
                                   FGS_ITEMSTATE_E state,
                                   FGS_ITEM_S* item)
{
    MENU_EPGSCHED_INST_S* inst = (MENU_EPGSCHED_INST_S*)h;
    EIT_POS_S             schedRef;

    if ( index < inst->numDateItems )
    {
        schedRef = inst->firstRef;

        if ( EIT_GetSchedByReference ((int16_t)index, &(inst->tmpItem), NULL,
                                      &schedRef) != FAPI_OK )
        {
            return;
        }

        (void)TIME_AddUtcOffset (&(inst->tmpItem.startTime),
                                 &(inst->tmpItem.startDate));

        item[0].type                = FGS_ITEM_TYPE_STRING;
        item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item[0].item.str.chr        = STR_IntASCIIStr();
        item[0].item.str.font       = MAIN_FONT;
        item[0].item.str.lineHeight = 0;

        switch ( state )
        {
            default:
            case FGS_NONFOCUS:
                item[0].item.str.col = OSD_COL_TEXT;
                break;
            case FGS_FOCUS:
                item[0].item.str.col = OSD_COL_TEXT_HIGHLIGHTED;
                break;
            case FGS_DISABLED:
                item[0].item.str.col = OSD_COL_TEXT_DISABLED;
                break;
        }

        item[1]                     = item[0];
        item[1].item.str.chr        = STR_Build (inst->tmpItem.nameCharSet,
                                                 inst->tmpItem.name);

        sprintf (appDat->asciiBuf, "%2d:%02d", inst->tmpItem.startTime.hour,
                                               inst->tmpItem.startTime.minute);
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
