/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_hdd.c
**
**              $Id: app_menu_hdd.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Harddisk/USB utility menu.
**
** Detail description. [\ref app_smartgo_app_ref_hdd]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbols not referenced
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Info    961: Dependence placed on C's operator precedence
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define MENU_HDD_FORMATTIMEOUT          60  // in seconds
#define MENU_HDD_FORMATFAILURETIMEOUT   5   // in seconds
#define MENU_HDD_PVRJUMPTIME_MIN        1   // in minutes
#define MENU_HDD_PVRJUMPTIME_MAX        255 // in minutes
#define MENU_HDD_PVRJUPMTIME_STEP       1   // in minutes

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_HDD_ID_TITLE = 0,
    MENU_HDD_ID_LIST,
    MENU_HDD_ID_HELPLIST
};

#ifdef APPL_HDD_ENABLED
enum
{
    MENU_HDD_SPINDOWN_5MIN = 0,
    MENU_HDD_SPINDOWN_15MIN,
    MENU_HDD_SPINDOWN_30MIN,
    MENU_HDD_SPINDOWN_60MIN,
    MENU_HDD_SPINDOWN_NUMITEMS
};
#endif

enum
{
    //MENU_HDD_ID_ENTRY_INFO,
    //MENU_HDD_ID_ENTRY_DRIVE,

    #ifdef APPL_HDD_ENABLED

    MENU_HDD_ID_ENTRY_FORMAT,
    MENU_HDD_ID_ENTRY_POWER,

    #endif /* APPL_HDD_ENABLED */

    #ifdef APPL_USB_ENABLED

    MENU_HDD_ID_ENTRY_AUTOCON,
    MENU_HDD_ID_ENTRY_RECON,
    MENU_HDD_ID_ENTRY_USBFORMAT,

    #endif /* APPL_USB_ENABLED */

    #ifdef APPL_PVR_ENABLED

    MENU_HDD_ID_ENTRY_JUMPTIME,

    #endif /* APPL_PVR_ENABLED */

    MENU_HDD_ID_NUMS
};

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    listHdl;
    FAPI_SYS_HandleT    listHelpHdl;

    SYS_DEVICE_E        formatDevice;
    fbool_t             formatMenuOpen;

} MENU_HDD_INST_S;

FSTATIC MENU_HDD_INST_S* menuHddDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_HddLeave         (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_HddOnSelect      (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void    MENU_HddOnFocusSet    (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_HddValChg        (FAPI_SYS_HandleT h, int32_t itemId,
                                       FGS_VAL_S newVal);
FSTATIC void    MENU_HddGetItem       (FAPI_SYS_HandleT h, int32_t id,
                                       uint32_t index, FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item);
FSTATIC fbool_t MENU_HddBuild         (MENU_HDD_INST_S* inst);
FSTATIC void    MENU_HddOnFocusChg    (FAPI_SYS_HandleT h, uint8_t index,
                                       int32_t id);
FSTATIC void    MENU_HddGetDescrItem  (FAPI_SYS_HandleT h, int32_t id,
                                       uint32_t index, FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item);
FSTATIC void    MENU_HddGetHelpDescrItem
                                      (FAPI_SYS_HandleT h, int32_t id,
                                       uint32_t index, FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item);

FSTATIC void    MENU_HddConfirmFormat (void* arg);

FSTATIC fbool_t MENU_HddFormatEntry   (FAPI_SYS_HandleT h, fbool_t test);
FSTATIC fbool_t MENU_HddFormatLeave   (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC void    MENU_HddFormatTimerExpired (void* arg);

int32_t MENU_HddInit (void)
{
    if ( menuHddDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuHddDat = UTILS_Malloc(sizeof(MENU_HDD_INST_S));
    if ( menuHddDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuHddDat, 0, sizeof(MENU_HDD_INST_S));

    return FAPI_OK;
}

fbool_t MENU_HddEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_HDD_INST_S* inst = menuHddDat;
    uint32_t              lastFocusItem;

    if ( test == FTRUE ) return FTRUE;

    if ( !(MENU_HddBuild(inst)) )
    {
        return FFALSE;
    }

    inst->formatMenuOpen = FFALSE;

    MENU_HddDeviceUpdate (NULL);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = 0;
    }

    /* set list focus */
    if ( FGS_LIST_SetMemberFocus  (inst->listHdl, (int32_t)lastFocusItem)
        != FAPI_OK )
    {
        (void)FGS_LIST_SetMemberFocusAuto(inst->listHdl, NULL);
    }

    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_HDD_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_HDD);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_HddLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

FSTATIC void MENU_HddOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_HDD_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);
}

FSTATIC void MENU_HddOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_HDD_INST_S*   inst = (MENU_HDD_INST_S*)h;
    FGS_LNAVI_PARAMS_S naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = (uint16_t)id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}

FSTATIC fbool_t MENU_HddLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_HDD_INST_S* inst = (MENU_HDD_INST_S*)h;

    if ( !force )
    {
        MAIN_SaveSettingsIfChanged();
    }

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_HDD);

    inst->pnlHdl  = NULL;
    inst->listHdl = NULL;

    return FTRUE;
}

FSTATIC fbool_t MENU_HddOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_HDD_INST_S*        inst = (MENU_HDD_INST_S*)h;
    MENUSTACK_ENTRY         newEntry = NULL;
    FAPI_SYS_HandleT        arg = NULL;
    uint8_t                 subLines = 0;
    MENU_SELECTLIST_OPEN_S  selectLst;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    fbool_t                 keyUsed = FTRUE;
    FGS_LOC_S               memberLoc;
    fbool_t                 openSelectLst = FFALSE;
    MENU_CONFDIALOG_S       confPars;

    switch ( itemId )
    {
        #ifdef APPL_HDD_ENABLED

        case MENU_HDD_ID_ENTRY_FORMAT:
            inst->formatDevice = SYS_DEV_HDD;

            memset(&confPars, 0, sizeof(confPars));

            confPars.numDescLines = 1;
            confPars.infoStr      = OSD_GetStr(STR_CONFDIALOG_FORMATINFO);
            confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_FORMATHDD);
            confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_FORMATOK);
            confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_FORMATABORT);
            confPars.onConfirm    = MENU_HddConfirmFormat;
            confPars.onDecline    = NULL;
            confPars.onDiscard    = NULL;
            confPars.arg          = inst;

            return MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);

        case MENU_HDD_ID_ENTRY_POWER:
            openSelectLst  = FTRUE;
            subLines       = 4;
            break;

        #endif /* APPL_HDD_ENABLED */

        #ifdef APPL_USB_ENABLED

        case MENU_HDD_ID_ENTRY_AUTOCON:
            openSelectLst  = FTRUE;
            subLines       = 2;
            break;

        case MENU_HDD_ID_ENTRY_RECON:
            USB_Reconnect();
            break;

        case MENU_HDD_ID_ENTRY_USBFORMAT:
            inst->formatDevice = SYS_DEV_USB;

            memset(&confPars, 0, sizeof(confPars));

            confPars.numDescLines = 1;
            confPars.infoStr      = OSD_GetStr(STR_CONFDIALOG_FORMATINFO);
            confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_FORMATUSB);
            confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_FORMATOK);
            confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_FORMATABORT);
            confPars.onConfirm    = MENU_HddConfirmFormat;
            confPars.onDecline    = NULL;
            confPars.onDiscard    = NULL;
            confPars.arg          = inst;

            return MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);

        #endif /* APPL_USB_ENABLED */

        default:
            return FFALSE;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.width     = SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.x         = memberLoc.x + SCALER_GetCurrVal(MAIN_WIDTH) -
                                  selectLst.pos.width;
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_HddGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;

        keyUsed = MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);

        if ( keyUsed )
        {
            (void)MAIN_SignalBoxPause(&(appDat->sigBox));
        }
        return keyUsed;
    } else {
        /* store last focus */
        MENUSTACK_SetData((uint32_t)itemId);
        if(newEntry == NULL)
            return keyUsed;
        else
            return MENUSTACK_Up(newEntry, arg);
    }
}

FSTATIC void MENU_HddValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    //MENU_HDD_INST_S* inst = (MENU_HDD_INST_S*)h;
    #ifdef APPL_HDD_ENABLED
    uint32_t time;
    #endif /* APPL_HDD_ENABLED */

    switch ( itemId )
    {
        #ifdef APPL_HDD_ENABLED

        case MENU_HDD_ID_ENTRY_POWER:

            switch ( newVal.number )
            {
                default:
                case  MENU_HDD_SPINDOWN_5MIN:  time =  5; break;
                case  MENU_HDD_SPINDOWN_15MIN: time = 15; break;
                case  MENU_HDD_SPINDOWN_30MIN: time = 30; break;
                case  MENU_HDD_SPINDOWN_60MIN: time = 60; break;
            }

            (void)HDD_SetSpindownTime(time * 60);
            break;

        #endif /* APPL_HDD_ENABLED */

        #ifdef APPL_USB_ENABLED

        case MENU_HDD_ID_ENTRY_AUTOCON:
            (void)SETTINGS_UsbAutoConnectSet((fbool_t)newVal.number);
            break;

        #endif /* APPL_USB_ENABLED */

        #ifdef APPL_PVR_ENABLED

        case MENU_HDD_ID_ENTRY_JUMPTIME:
            (void)MAIN_UserDataSet(USERDAT_PVRJUMPTIME,(uint32_t)newVal.number);
            break;

        #endif /* APPL_PVR_ENABLED */

        default:
            break;
    }
}

FSTATIC void MENU_HddGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                   FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        //case MENU_HDD_ID_ENTRY_DRIVE:  strIdx = STR_ENTRY_HDD_DRIVE;  break;
        //case MENU_HDD_ID_ENTRY_INFO:   strIdx = STR_ENTRY_HDD_INFO;   break;

        #ifdef APPL_HDD_ENABLED

        case MENU_HDD_ID_ENTRY_FORMAT:      strIdx = STR_ENTRY_HDD_FORMAT;
                                            break;

        case MENU_HDD_ID_ENTRY_POWER:       strIdx = STR_ENTRY_HDD_POWER;
                                            break;

        #endif /* APPL_HDD_ENABLED */

        #ifdef APPL_USB_ENABLED

        case MENU_HDD_ID_ENTRY_AUTOCON:     strIdx = STR_ENTRY_HDD_USBAUTOCON;
                                            break;

        case MENU_HDD_ID_ENTRY_RECON:       strIdx = STR_ENTRY_HDD_USBRECONNECT;
                                            break;

        case MENU_HDD_ID_ENTRY_USBFORMAT:   strIdx = STR_ENTRY_HDD_USBFORMAT;
                                            break;

        #endif /* APPL_USB_ENABLED */

        #ifdef APPL_PVR_ENABLED

        case MENU_HDD_ID_ENTRY_JUMPTIME:    strIdx = STR_ENTRY_HDD_JUMPTIME;
                                            break;

        #endif /* APPL_PVR_ENABLED */

        default:
            return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_HddGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                       FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        //case MENU_HDD_ID_ENTRY_INFO:

        #ifdef APPL_HDD_ENABLED

        //case MENU_HDD_ID_ENTRY_DRIVE:
        case MENU_HDD_ID_ENTRY_FORMAT:      strIdx = STR_ENTRY_OK;   break;
        case MENU_HDD_ID_ENTRY_POWER:       strIdx = STR_ENTRY_LOKR; break;

        #endif /* APPL_HDD_ENABLED */

        #ifdef APPL_USB_ENABLED

        case MENU_HDD_ID_ENTRY_AUTOCON:     strIdx = STR_ENTRY_LOKR; break;
        case MENU_HDD_ID_ENTRY_RECON:       strIdx = STR_ENTRY_OK;   break;
        case MENU_HDD_ID_ENTRY_USBFORMAT:   strIdx = STR_ENTRY_OK;   break;

        #endif /* APPL_USB_ENABLED */

        #ifdef APPL_PVR_ENABLED

        case MENU_HDD_ID_ENTRY_JUMPTIME:    strIdx = STR_ENTRY_09LR; break;

        #endif /* APPL_PVR_ENABLED */

        default: return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC void MENU_HddGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    #if defined(APPL_USB_ENABLED) || defined(APPL_HDD_ENABLED)
    uint32_t strIdx;
    #endif

    #ifdef APPL_PVR_ENABLED
    uint32_t jumpTime; // in minutes
    #endif /* APPL_PVR_ENABLED */

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    switch ( id )
    {
        #ifdef APPL_HDD_ENABLED

        case MENU_HDD_ID_ENTRY_POWER:
            switch ( index )
            {
                case MENU_HDD_SPINDOWN_5MIN:  strIdx = STR_VALUE_HDD_SPINDOWN_5MIN;
                                                       break;
                case MENU_HDD_SPINDOWN_15MIN: strIdx = STR_VALUE_HDD_SPINDOWN_15MIN;
                                                       break;
                case MENU_HDD_SPINDOWN_30MIN: strIdx = STR_VALUE_HDD_SPINDOWN_30MIN;
                                                       break;
                case MENU_HDD_SPINDOWN_60MIN: strIdx = STR_VALUE_HDD_SPINDOWN_1H;
                                                       break;
                default:                      strIdx = STR_EMPTYSTRING;
                                              break;
            }

            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.font       = (int32_t)MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.chr        = OSD_GetStr(strIdx);

            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
            }
            break;

        #endif /* APPL_HDD_ENABLED */

        #ifdef APPL_USB_ENABLED

        case MENU_HDD_ID_ENTRY_AUTOCON:
            switch ( index )
            {
                case 0:  strIdx = STR_GNR_OFF;     break;
                case 1:  strIdx = STR_GNR_ON;      break;
                default: strIdx = STR_EMPTYSTRING; break;
            }

            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.font       = (int32_t)MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.chr        = OSD_GetStr(strIdx);

            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
            }
            break;

        #endif /* APPL_USB_ENABLED */

        #ifdef APPL_PVR_ENABLED

        case MENU_HDD_ID_ENTRY_JUMPTIME:
            (void)MAIN_UserDataGet(USERDAT_PVRJUMPTIME, &jumpTime);

            item->type                      = FGS_ITEM_TYPE_NUMBER;
            item->item.number.value         = jumpTime;
            item->item.number.format        = FGS_FORMAT_NUM_NORM;
            item->item.number.decimalPlaces = 0;
            item->item.number.unitStr       = OSD_GetStr(STR_UNIT_MIN);
            item->item.number.font          = MAIN_FONT;
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.number.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.number.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.number.col = appDat->txtAttr.colDisabled; break;
            }
            break;

        #endif /* APPL_PVR_ENABLED */

        default:
            break;
    }
}

FSTATIC fbool_t MENU_HddBuild (MENU_HDD_INST_S* inst)
{
    FGS_PANEL_OPEN_S    pnlPars;
    FGS_LIST_OPEN_S     listPars;
    FGS_ITEM_TXT_OPEN_S txtPars;
    FGS_ITEM_SEL_OPEN_S selPars;
    FGS_ITEM_NUM_OPEN_S numPars;
    FGS_POS_S           itemPos;
    FGS_POS_S           valuePos;
    FGS_POS_S           descrPos;
    int32_t             retVal;
    uint16_t            helpWidth;
    uint16_t            stackedFrameWidth;
    uint16_t            innerHeight;
    uint32_t            numLines = MENU_HDD_ID_NUMS;

    #ifdef APPL_HDD_ENABLED
    uint32_t            spindownTime = 0;
    #endif /* APPL_HDD_ENABLED */

    #ifdef APPL_USB_ENABLED
    fbool_t             usbAutoConnect = FTRUE;
    #endif /* APPL_USB_ENABLED */

    #if defined(APPL_USB_ENABLED) || defined(APPL_HDD_ENABLED)
    FAPI_SYS_HandleT    itemHdl;
    #endif

    memset(&selPars,  0, sizeof(selPars));
    memset(&numPars,  0, sizeof(numPars));

    helpWidth   = (2 * SCALER_GetCurrVal(HELP_LR_PADDING))
                + OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_09LR), -1,
                                 (int32_t)SYMBOL_FONT);
    innerHeight = numLines
                * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                 + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth,
                                        1, &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_HddOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, (int32_t)MENUID_HDD, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_HDD_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_UTILITIES,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset(&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_HddGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_HddGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_HddOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_HddOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_HddValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_HDD_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 75,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- menu lines -------------------------------------------------*/

        memset(&txtPars, 0, sizeof(txtPars));

        txtPars.bkgrType = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos = descrPos;
        txtPars.valuePos = valuePos;
        txtPars.pos      = itemPos;

        /* ----- Select drive -----------------------------------------------*/

        #if 0

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_HDD_ID_ENTRY_DRIVE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif

        /* ----- HDD Info -------------------------------------------------- */

        #if 0

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_HDD_ID_ENTRY_INFO,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif

        /* ----- format HDD ------------------------------------------------ */

        #ifdef APPL_HDD_ENABLED

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_HDD_ID_ENTRY_FORMAT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- HDD power down--------------------------------------------- */

        (void)HDD_GetSpindownTime (&spindownTime);

        switch ( spindownTime )
        {
            case  300 : selPars.params.index = MENU_HDD_SPINDOWN_5MIN;  break;
            case  900 : selPars.params.index = MENU_HDD_SPINDOWN_15MIN; break;
            case 1800 : selPars.params.index = MENU_HDD_SPINDOWN_30MIN; break;
            case 3600 : selPars.params.index = MENU_HDD_SPINDOWN_60MIN; break;

            default:
                (void)HDD_SetSpindownTime(1800);
                selPars.params.index = MENU_HDD_SPINDOWN_30MIN;
        }

        selPars.pos                 = itemPos;
        selPars.bkgrType            = txtPars.bkgrType;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = MENU_HDD_SPINDOWN_NUMITEMS;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_HDD_ID_ENTRY_POWER,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_HDD_ENABLED */

        #ifdef APPL_USB_ENABLED

        /* ----- USB auto-reconnect ---------------------------------------- */

        (void)SETTINGS_UsbAutoConnectGet(&usbAutoConnect);

        selPars.params.index        = (uint32_t)usbAutoConnect;

        selPars.pos                 = itemPos;
        selPars.bkgrType            = txtPars.bkgrType;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = 2;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_HDD_ID_ENTRY_AUTOCON,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- USB reconnect --------------------------------------------- */

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_HDD_ID_ENTRY_RECON,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- USB format --------------------------------------------- */

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_HDD_ID_ENTRY_USBFORMAT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_USB_ENABLED */

        /* ----- PVR Jump time --------------------------------------------- */

        #ifdef APPL_PVR_ENABLED

        numPars.pos                 = itemPos;
        numPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        numPars.descrPos            = descrPos;
        numPars.valuePos            = valuePos;
        numPars.type                = FGS_NITEM_TYPE_NUMBER;
        numPars.wraparound          = FTRUE;
        numPars.params.minValue     = MENU_HDD_PVRJUMPTIME_MIN;
        numPars.params.maxValue     = MENU_HDD_PVRJUMPTIME_MAX;
        numPars.params.maxDigits    = 3;
        numPars.params.step         = MENU_HDD_PVRJUPMTIME_STEP;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_HDD_ID_ENTRY_JUMPTIME,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_PVR_ENABLED */

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_HDD_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH), listPars.pos.y,
                                        helpWidth, numLines,
                                        MENU_HddGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)(SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2) -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);

            inst->pnlHdl  = NULL;
            inst->listHdl = NULL;
        }

        return FFALSE;
    }

    return FTRUE;
}

void MENU_HddDeviceUpdate (const SYS_EVT_DEVSTAT_S* pEvt)
{
    MENU_HDD_INST_S*    inst = menuHddDat;
    SYS_DEV_STAT_S      devStatus;
    fbool_t             enable;

    /* check if menu is initialised */
    if ( inst == NULL )
    {
        return;
    }

    /* ----- test for end of formatting process ---------------------------- */

    if ( (pEvt != NULL) && (inst->formatMenuOpen) )
    {
        if ( (pEvt->device             == inst->formatDevice) &&
             (pEvt->status.mountStatus == SYS_DEV_FS_MOUNTED) )
        {
            /* Mount has successfully finished: Back to HDD menu */
            (void)MENUSTACK_Back(NULL);
        }

        /* NOTE: A failure during mount is currently not detected. Dialogue
                 will be closed if timeout occurs. */

        // TODO: detect failure by checking for status SYS_DEVFS_ERROR!
    }

    /* ----- handle device update with menu currently opened --------------- */

    if ( inst->listHdl != NULL )
    {
        #ifdef APPL_HDD_ENABLED

        if ( SYS_DeviceGetStatus (SYS_DEV_HDD, &devStatus) != FAPI_OK )
        {
            memset(&devStatus, 0, sizeof(devStatus));

            devStatus.avlStatus   = SYS_DEVAVL_NOTAVAILABLE;
            devStatus.mountStatus = SYS_DEV_FS_NONE;
            devStatus.access      = FFALSE;
        }

        /* NOTE: This menu item is disabled if HDD is either not available or
                 currently accessed by an external (USB host) device. */
        enable = (devStatus.avlStatus == SYS_DEVAVL_AVAILABLE) ? FTRUE : FFALSE;

        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                              MENU_HDD_ID_ENTRY_FORMAT, enable);

        /* NOTE: This menu item is enabled only if HDD is internally mounted. */
        enable = (devStatus.mountStatus == SYS_DEV_FS_MOUNTED) ? FTRUE : FFALSE;

        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                              MENU_HDD_ID_ENTRY_POWER, enable);

        #endif /* APPL_HDD_ENABLED */

        /* ----- handle USB device update -------------------------------------- */

        #ifdef APPL_USB_ENABLED

        if ( SYS_DeviceGetStatus (SYS_DEV_USB, &devStatus) != FAPI_OK )
        {
            enable = FFALSE;
        }
        else
        {
            enable = (devStatus.avlStatus == SYS_DEVAVL_AVAILABLE) ? FTRUE
                                                                   : FFALSE;
        }

        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                              MENU_HDD_ID_ENTRY_USBFORMAT, enable);

        #endif /* APPL_USB_ENABLED */
    }
}

FSTATIC void MENU_HddConfirmFormat (void* arg)
{
    (void)MENUSTACK_Up (MENU_HddFormatEntry, (FAPI_SYS_HandleT)arg);
}

/* ------------------------------------------------------------------------- */

FSTATIC fbool_t MENU_HddFormatEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_HDD_INST_S* inst = (MENU_HDD_INST_S*)h;
    int32_t          retVal;
    uint16_t         timeout;

    if ( test == FTRUE ) return FTRUE;

    /* start formatting of device */
    retVal = SYS_DeviceFormat (inst->formatDevice);

    if ( retVal == FAPI_OK )
    {
        timeout = MENU_HDD_FORMATTIMEOUT;
        MENU_CommonMsgShow(STR_MSG_FORMATHDD, FTRUE);
    }
    else
    {
        timeout = MENU_HDD_FORMATFAILURETIMEOUT;

        MENU_CommonMsgShow(STR_MSG_FS_HDD_FORMAT_FAILED, FTRUE);
    }

    /* launch timer for timeout */
    (void)MAIN_MenuTimerLaunch (timeout, MENU_HddFormatTimerExpired, inst);

    MENUSTACK_SetCleanup (MENU_HddFormatLeave, inst);

    inst->formatMenuOpen = FTRUE;

    return FTRUE;
}

FSTATIC fbool_t MENU_HddFormatLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    ((MENU_HDD_INST_S*)h)->formatMenuOpen = FFALSE;

    MAIN_MenuTimerAbort ();

    (void)MAIN_MsgBoxHide(&(appDat->msgBox));

    return FTRUE;
}

FSTATIC void MENU_HddFormatTimerExpired (void* arg)
{
    if ( ((MENU_HDD_INST_S*)arg)->formatMenuOpen )
    {
        (void)MENUSTACK_Back(NULL);
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
