/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_info.c
**
**              $Id: app_menu_info.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Implementation of infobar and related OSD components.
**
** Detail description.
** - [\ref app_smartgo_app_ref_info]
** - [\ref app_smartgo_app_ref_infobar]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: loss of precision
** - Info    715: symbol 'anySymb' not referenced
** - Info    737: loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"
#include "app_menu_info.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define MENU_INFO_TITLE_LENGTH     MAIN_LINEBUFSIZE
#define MENU_INFO_TITLE_CHARSET    MAIN_LINEBUFCHARSET

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
typedef struct
{
    SYS_SERVLOC_S    currServLoc;
    SLIST_SERVIDX    currServIdx;

    #ifdef APPL_SUBTT_ENABLED
    fbool_t          subtitleAvl;
    #endif /* APPL_SUBTT_ENABLED */
    #ifdef APPL_MHEG_ENABLED
    fbool_t          mhegAvl;
    #endif /* APPL_SUBTT_ENABLED */
    fbool_t          crypted;
    fbool_t          teletextAvl;
    fbool_t          is16_9;        //!< current video is 16:9?
    fbool_t          isHD;

    #ifdef APPL_PARENTAL_ENABLED
    fbool_t          isLocked;      //!< If FTRUE, output is currently disabled
                                    //!< due to parental control
    #endif /* APPL_PARENTAL_ENABLED */

    SYS_STRBUF_S     title;

    #ifdef APPL_PVR_ENABLED

    SYS_TIME_S       recTime[RSRC_REC_MAX];        //!< recording time
    SYS_TIME_S       playTime;                     //!< playback/timeshift time
    PVR_LIST_ITEM_S  tmpItem;

    #endif /* APPL_PVR_ENABLED */

    /* number input parameters */
    fbool_t          inInputMode;
    uint8_t          charIdx;
    uint16_t         servNumVal;

} OSD_INFO_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC OSD_INFO_S*            infoDat   = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
int32_t MENU_InfoInit (void)
{
    int32_t         retVal;
    SYS_PICFORMAT_E picFormat;

    if (infoDat != NULL)
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    infoDat = UTILS_Zalloc(sizeof(OSD_INFO_S));
    if ( infoDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    infoDat->currServLoc.listId    = SLIST_NONE;
    infoDat->currServLoc.index     = -1;
    infoDat->currServIdx           = -1;

    retVal = AV_PicFormatCurrGet(&picFormat);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    retVal = STR_BufOpen (&(infoDat->title), MENU_INFO_TITLE_LENGTH, NULL);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    #ifdef APPL_SUBTT_ENABLED
    infoDat->subtitleAvl = FFALSE;
    #endif /* APPL_SUBTT_ENABLED */
    infoDat->crypted     = FFALSE;
    infoDat->teletextAvl = FFALSE;
    infoDat->is16_9      = (picFormat == PICFORMAT_16_9) ? FTRUE : FFALSE;
    infoDat->isHD        = FFALSE;
    #ifdef APPL_PARENTAL_ENABLED
    infoDat->isLocked              = FFALSE;
    #endif /* APPL_PARENTAL_ENABLED */

    infoDat->inInputMode           = FFALSE;
    infoDat->charIdx               = 0;
    infoDat->servNumVal            = 0;

    return FAPI_OK;
}

/* ----- Service number handling ------------------------------------------- */

fbool_t MENU_InfoServiceNumBoxInput (FAPI_SYS_HandleT h, uint8_t number)
{
    /* update value */
    infoDat->inInputMode = FTRUE;
    infoDat->servNumVal  = (infoDat->servNumVal * 10) + number;
    infoDat->charIdx++;

    /* draw channel number name */
    MENU_InfoServiceNumBoxUpdate();

    if ( infoDat->charIdx >= 4 ) /* last input */
    {
        MAIN_SelectServiceByInput();
    }
    else
    {
        /* set timeout for service selection */
        (void)TIME_SingleTimerLaunch(OSD_TIMER_SERVSELECT,
                                     SERVICESELECT_TIMEOUT);
    }

    return FTRUE;
}

void MENU_InfoServiceNumBoxInputAbort(void)
{
    infoDat->inInputMode = FFALSE;
    infoDat->charIdx     = 0;
    infoDat->servNumVal  = 0;
}

void MAIN_SelectServiceByInput (void)
{
    int32_t         retVal = FAPI_OK;
    SYS_SERVLOC_S   servLoc;

    #ifdef APPL_LCN_ENABLED
    SLIST_PARAMS_S  slistParams;
    #endif /* APPL_LCN_ENABLED */

    infoDat->inInputMode = FFALSE;

    if ( infoDat->servNumVal > 0 )
    {
        servLoc.listId = infoDat->currServLoc.listId;
        servLoc.index  = (int16_t)(infoDat->servNumVal - 1);

        #ifdef APPL_LCN_ENABLED

        if ( (SLIST_GetParams(&slistParams) == FAPI_OK) &&
              slistParams.sortByLcn )
        {
            if ( SLIST_GetServLocByLcn (infoDat->servNumVal,
                                        infoDat->currServLoc.listId,
                                        &servLoc) != FAPI_OK )
            {
                retVal = SLIST_GetServLocByLcn (infoDat->servNumVal,
                                                SLIST_NONE, &servLoc);
            }
        }

        #endif /* APPL_LCN_ENABLED */

        if ( retVal == FAPI_OK )
        {
            retVal = MAIN_SelectService (&servLoc, FFALSE);
        }
    }
    else
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    infoDat->charIdx    = 0;
    infoDat->servNumVal = 0;

    if ( retVal != FAPI_OK )
    {
        /* Selection failed: redraw number of active service */
        MENU_InfoServiceNumBoxUpdate();
    }

    /* Restart display timer if infobar is in non-permanent mode */
    if ( MENU_InfobarIsVisible() == FTRUE )
    {
        MENU_InfobarUpdateDisplayTimer ();
    }
}

void MENU_InfoServiceNumBoxUpdate (void)
{
    if ( appDat->numBoxPnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember(appDat->numBoxPnlHdl, appDat->numBoxId);
    }
}

/* ------------------------------------------------------------------------- */


int32_t MENU_InfoServiceNumBoxCreate(FAPI_SYS_HandleT pnlHdl, int32_t id, int32_t parent)
{
    FGS_BOX_OPEN_S   boxPars;
    FAPI_SYS_HandleT itemHdl;
    int32_t          retVal;

    memset(&boxPars, 0, sizeof(boxPars));

    boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                            + SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
    boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
    boxPars.pos.width       = SCALER_GetCurrVal(INFO_NUM_WIDTH);
    boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
    boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
    boxPars.bkgrType        = OSD_BOX_BKGR_TITLE | OSD_CORNER_TOP_LEFT;
    boxPars.getItem         = MAIN_ServiceNumGetItem;
    boxPars.getItemArg      = NULL;

    retVal = FGS_BOX_Open (&boxPars, id, pnlHdl, &itemHdl);

    if ( retVal == FAPI_OK )
    {
        appDat->numBoxPnlHdl = pnlHdl;
        appDat->numBoxId     = id;
    }

    return retVal;
}

void MENU_InfoServiceNumBoxRemove (void)
{
    /* NOTE: This invalidates only the pointer, as the object itself is removed
             together with the panel it has been assigned to! */
    appDat->numBoxPnlHdl = NULL;
}

void MAIN_ServiceNumGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                     FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint16_t        servNum;

    #ifdef APPL_LCN_ENABLED
    SLIST_PARAMS_S  slistParams;
    #endif /* APPL_LCN_ENABLED */

    if ( appDat->numBoxPnlHdl == NULL ) return;

    #ifdef APPL_PVR_ENABLED

    /* special handling for infobar in PVR playback (not timeshift!) */
    if ( appDat->pvrStatus.play.type == PLAY_TYPE_NORMAL )
    {
        return;
    }

    #endif /* APPL_PVR_ENABLED */

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = TITLE_FONT;
    item->item.str.lineHeight = 0;
    item->item.str.col        = OSD_COL_TEXT_HIGHLIGHTED;
    item->item.str.chr        = STR_IntASCIIStr();

    switch ( infoDat->charIdx )
    {
        default:
        case 0:
            if ( infoDat->currServLoc.index < 0 )
            {
                sprintf(appDat->asciiBuf, "- - - -");
            }
            else
            {
                servNum = (uint16_t)(infoDat->currServLoc.index + 1);

                #ifdef APPL_LCN_ENABLED

                if ( (SLIST_GetParams(&slistParams) == FAPI_OK) &&
                      slistParams.sortByLcn )
                {
                    servNum = 0;

                    (void)SLIST_GetLcn (&(infoDat->currServLoc), &servNum);
                }

                #endif /* APPL_LCN_ENABLED */

                sprintf(appDat->asciiBuf, "%04d", servNum);
            }
            item->item.str.col = OSD_COL_TITLE_TEXT;
            break;

        case 1:
            sprintf(appDat->asciiBuf, "%01d.", infoDat->servNumVal);
            break;

        case 2:
            sprintf(appDat->asciiBuf, "%02d.", infoDat->servNumVal);
            break;

        case 3:
            sprintf(appDat->asciiBuf, "%03d.", infoDat->servNumVal);
            break;

        case 4:
            sprintf(appDat->asciiBuf, "%04d", infoDat->servNumVal);
            break;
    }
}

int32_t MENU_InfoTimeBoxCreate (FAPI_SYS_HandleT pnlHdl, int32_t id)
{
    FGS_BOX_OPEN_S   boxPars;
    FAPI_SYS_HandleT itemHdl;
    int32_t          retVal;

    memset(&boxPars, 0, sizeof(boxPars));

    boxPars.pos.x           = -((int16_t)SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                              + (int16_t)SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
                              + (int16_t)SCALER_GetCurrVal(INFO_TIME_WIDTH));
    boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
    boxPars.pos.width       = SCALER_GetCurrVal(INFO_TIME_WIDTH);
    boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
    boxPars.pos.alignment   = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_TOP;
    boxPars.bkgrType        = OSD_BOX_BKGR_TITLE | OSD_CORNER_TOP_RIGHT;
    boxPars.getItem         = MENU_InfoTimeBoxGetItem;
    boxPars.getItemArg      = NULL;

    retVal = FGS_BOX_Open (&boxPars, id, pnlHdl, &itemHdl);

    if ( retVal == FAPI_OK )
    {
        appDat->timeBoxPnlHdl = pnlHdl;
        appDat->timeBoxId     = id;
    }

    return retVal;
}

void MENU_InfoTimeBoxRemove (void)
{
    /* NOTE: This invalidates only the pointer, as the object itself is removed
             together with the panel it has been assigned to! */
    appDat->timeBoxPnlHdl = NULL;
}

void MENU_InfoTimeBoxUpdate (void)
{
    if ( appDat->timeBoxPnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember(appDat->timeBoxPnlHdl, appDat->timeBoxId);
    }
}

void MENU_InfoTimeBoxGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                  FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    item->type             = FGS_ITEM_TYPE_TIME;
    item->align            = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item->item.time.value  = appDat->timeCurr;
    item->item.time.format = appDat->timeFormat;
    item->item.time.font   = TITLE_FONT;
    item->item.time.col    = OSD_COL_TITLE_TEXT;
}


int32_t MENU_InfoListSymbolBoxCreate (FAPI_SYS_HandleT pnlHdl, int32_t id, int32_t parent)
{
    FGS_BOX_OPEN_S   boxPars;
    int32_t          retVal;
    FAPI_SYS_HandleT itemHdl;

    memset(&boxPars, 0, sizeof(boxPars));

    boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                            + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
                            + SCALER_GetCurrVal(INNER_FRAME_WIDTH);
    boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                              SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                              SCALER_GetCurrVal(INNER_FRAME_WIDTH);
    boxPars.pos.width       = SCALER_GetCurrVal(INFO_LISTID_WIDTH);
    boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
    boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
    boxPars.bkgrType        = OSD_BOX_BKGR_TITLE;
    boxPars.getItem         = MENU_InfoListSymbolBoxGetItem;
    boxPars.getItemArg      = NULL;

    retVal = FGS_BOX_Open (&boxPars, id, pnlHdl, &itemHdl);

    if ( retVal == FAPI_OK )
    {
        appDat->symbBoxPnlHdl = pnlHdl;
        appDat->symbBoxId     = id;
        appDat->symbBoxParent = parent;
    }

    return retVal;
}

void MENU_InfoListSymbolBoxRemove (void)
{
    /* NOTE: This invalidates only the pointer, as the object itself is removed
             together with the panel it has been assigned to! */
    appDat->symbBoxPnlHdl = NULL;
}

void MENU_InfoListSymbolBoxGetItem (FAPI_SYS_HandleT h, int32_t id,
                                     uint32_t index, FGS_ITEMSTATE_E state,
                                     FGS_ITEM_S* item)
{
    SYS_PRESSTATE_E presState;
    uint32_t*       pSymb = NULL;

    #ifdef APPL_PVR_ENABLED

    uint32_t        strIdx = STR_EMPTYSTRING;
    uint32_t        bufPos = 0;
    char_t          tmpStr[16];

    #endif /* APPL_PVR_ENABLED */

    if ( SYS_GetPresentationState(&presState) != FAPI_OK ) return;

    switch ( presState )
    {
        default:
        case SYS_PRESSTATE_NONE:
            break;

        case SYS_PRESSTATE_LIVETV:
            switch ( infoDat->currServLoc.listId )
            {
                case SLIST_TV:
                    pSymb = (uint32_t*)OSD_IconGet(ICON_TV);
                    break;

                case SLIST_RADIO:
                    pSymb = (uint32_t*)OSD_IconGet(ICON_RADIO);
                    break;

                case SLIST_INVISIBLE:
                    /* do not show a list symbol. */
                    break;

                default:
                    if ( ((int16_t)infoDat->currServLoc.listId >= SLIST_OFFSET_USR)
                         && (infoDat->currServLoc.listId < (SLIST_OFFSET_USR +
                                                     SLIST_USR_NUMLISTS)) )
                    {
                        pSymb = (uint32_t*)OSD_IconGet(ICON_USR);
                    }
                    break;
            }

            if ( pSymb != NULL )
            {
                item->type                 = FGS_ITEM_TYPE_ICON;
                item->align                = FGS_ALIGN_HOR_CENTER |
                                             FGS_ALIGN_VER_CENTER;
                item->item.icon.drawMode   = FGS_ICONMODE_OVRLAYPLAIN;
                item->item.icon.bkgrCol    = OSD_COL_BLANK;
                item->item.icon.altBkgrCol = OSD_GetSchemeColor(OSD_SCHEMECOL_TITLE_BKGR);
                item->item.icon.addr       = pSymb;
            }
            break;

        #ifdef APPL_PVR_ENABLED

        case SYS_PRESSTATE_PLAYBACK:
            switch ( appDat->pvrStatus.play.status )
            {
                case PLAY_STATUS_NORMAL:
                    item->item.str.chr = OSD_GetStr(STR_SYMBOL_PLAY);
                    break;

                case PLAY_STATUS_PAUSE:
                    item->item.str.chr = OSD_GetStr(STR_SYMBOL_PAUSE);
                    break;

                case PLAY_STATUS_FAST:

                    if ( appDat->pvrStatus.play.speedIndex >= 0 )
                    {
                        strIdx = STR_SYMBOL_FF;
                    }
                    else
                    {
                        strIdx = STR_SYMBOL_REW;
                    }

                    sprintf(tmpStr, "x%i", appDat->pvrStatus.play.speedFactor);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             OSD_GetStr(strIdx),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_GetASCII(tmpStr),
                                             MAIN_LINEBUFCHARSET);

                    item->item.str.chr = STR_IntLineStr();
                    break;

                case PLAY_STATUS_SLOW:

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             OSD_GetStr(STR_SYMBOL_FF),
                                             MAIN_LINEBUFCHARSET);

                    switch ( appDat->pvrStatus.play.speedIndex )
                    {
                        default:
                        case 0:
                            strIdx = STR_SYMBOL_1_2;
                            break;

                        case 1:
                            strIdx = STR_SYMBOL_1_4;
                            break;

                        case 2:
                            strIdx = STR_SYMBOL_1_8;
                            break;
                    }

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             OSD_GetStr(strIdx),
                                             MAIN_LINEBUFCHARSET);

                    item->item.str.chr = STR_IntLineStr();
                    break;

                default:
                    item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                    break;
            }

            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_CENTER
                                         | FGS_ALIGN_VER_CENTER;
            item->item.str.font          = SYMBOL_TITLE;
            item->item.str.lineHeight    = 0;
            item->item.str.col           = OSD_COL_TEXT_TITLE;
            break;

        #endif /* APPL_PVR_ENABLED */
    }
}

void MENU_InfoListSymbolBoxUpdate (void)
{
    if ( appDat->symbBoxPnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember(appDat->symbBoxPnlHdl, appDat->symbBoxId);
    }
}

int32_t MENU_InfoServiceNameBoxCreate (FAPI_SYS_HandleT pnlHdl, uint16_t width, int32_t id, int32_t parent)
{
    FGS_BOX_OPEN_S   boxPars;
    FAPI_SYS_HandleT itemHdl;
    int32_t          retVal;

    memset(&boxPars, 0, sizeof(boxPars));

    boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                              SCALER_GetCurrVal(MAIN_TEXT_PAD_X) +
                              SCALER_GetCurrVal(INFO_NUM_WIDTH) +
                              SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                              (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X));
    boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
    boxPars.pos.width       = width;
    boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
    boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
    boxPars.bkgrType        = OSD_BOX_BKGR_TITLE;
    boxPars.getItem         = MENU_InfoServiceNameBoxGetItem;
    boxPars.getItemArg      = NULL;

    retVal = FGS_BOX_Open (&boxPars, id, pnlHdl, &itemHdl);

    if ( retVal == FAPI_OK )
    {
        appDat->nameBoxPnlHdl = pnlHdl;
        appDat->nameBoxId     = id;
        appDat->nameBoxParent = parent;
    }

    return retVal;
}

void MENU_InfoServiceNameBoxRemove (void)
{
    /* NOTE: This invalidates only the pointer, as the object itself is removed
             together with the panel it has been assigned to! */
    appDat->nameBoxPnlHdl = NULL;
}

void MENU_InfoServiceNameBoxUpdate (void)
{
    if ( appDat->nameBoxPnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember(appDat->nameBoxPnlHdl, appDat->nameBoxId);
    }
}


void MENU_InfoServiceNameBoxGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    SYS_PRESSTATE_E  presState;
    SYS_STRING_S     string;

    memset(&string,0,sizeof(string));

    if ( SYS_GetPresentationState(&presState) != FAPI_OK )
    {
        return;
    }

    switch ( presState )
    {
        default:
        case SYS_PRESSTATE_NONE:
            break;

        case SYS_PRESSTATE_LIVETV:
            string = infoDat->title.string;
            break;

        #ifdef APPL_PVR_ENABLED

        case SYS_PRESSTATE_PLAYBACK:
            switch ( appDat->pvrStatus.play.type )
            {
                case PLAY_TYPE_NORMAL:
                    /* Normal playback: show recording name */
                    if ( PVR_LIST_GetItemByIndex(appDat->pvrStatus.play.lstIdx,
                                          &infoDat->tmpItem, NULL) != FAPI_OK ) return;

                    string = STR_Build(infoDat->tmpItem.nameCharSet,
                                       infoDat->tmpItem.name);
                    break;

                case PLAY_TYPE_TIMESHIFT:
                    /* Timeshift playback: show service name corresponding to
                       timeshift recording */
                    string = STR_Build(
                             appDat->playTimeShiftService.nameCharSet,
                             appDat->playTimeShiftService.name);
                    break;

                default:
                    break;
            }
            break;

        #endif /* APPL_PVR_ENABLED */
    }

    if ( string.chr != NULL )
    {
        item->type                = FGS_ITEM_TYPE_STRING;
        item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item->item.str.chr        = string;
        item->item.str.font       = TITLE_FONT;
        item->item.str.lineHeight = 0;
        item->item.str.col        = OSD_COL_TITLE_TEXT;
    }
}

fbool_t MENU_InfoIsInputMode (void)
{
    if( infoDat == NULL)
    {
        return FFALSE;
    }

    return infoDat->inInputMode;
}

void MENU_InfoServSelectTimerExpired (void)
{
    if ( infoDat->inInputMode == FTRUE )
    {
        MAIN_SelectServiceByInput();
    }
}

void MENU_InfoUpdateService (void)
{
    SLIST_ITEM_S       service;
    SYS_SERVLOC_S      servDesc;
    MENU_EPGEXT_OPEN_S epgExtParams;
    int32_t            retVal;

    servDesc.listId = SLIST_NONE;
    servDesc.index  = -1;

    /* ----- update service information ------------------------------------ */

    (void)AV_GetRequestedService  (&servDesc);

    infoDat->currServLoc = servDesc;
    infoDat->currServIdx = SLIST_GetServiceIndex(&servDesc);

    retVal = SLIST_EntryGet (&servDesc, &service);

    switch ( retVal )
    {
        case FAPI_OK:
            (void)STR_BufAppend (&(infoDat->title), 0,
                                 STR_Build(service.nameCharSet, service.name),
                                 MENU_INFO_TITLE_CHARSET);

            infoDat->crypted     = SLIST_FlagIsSet(&service, SLIST_FLAG_ALLFREE)
                                                   ? FFALSE : FTRUE;
            infoDat->teletextAvl = (service.strPid.ttx != 0) ? FTRUE : FFALSE;
            #ifdef APPL_MHEG_ENABLED
            infoDat->mhegAvl     = service.strPid.mhegPresent ? FTRUE : FFALSE;
            #endif

            #ifdef APPL_SUBTT_ENABLED
            infoDat->subtitleAvl = (SUBTT_NumStreamsGet() != 0) ? FTRUE : FFALSE;
            #endif /* APPL_SUBTT_ENABLED */

            if ( (service.serviceType == SERVICE_TYPE_DIGITAL_TV_HD) ||
                 (service.serviceType == SERVICE_TYPE_DIGITAL_TV_HD_ADVANCED))
            {
                infoDat->isHD = FTRUE;
            }
            else
            {
                infoDat->isHD = FFALSE;
            }
            if ( service.strPid.video == 0 )
            {
                infoDat->isHD = FFALSE;
                infoDat->is16_9 = FFALSE;
            }
            break;

        case SMARTGO_SYS_ERR_ITEM_EMPTY:
            (void)STR_BufAppend (&(infoDat->title), 0,
                                 OSD_GetStr(STR_GNR_EMPTY),
                                 MENU_INFO_TITLE_CHARSET);

            #ifdef APPL_SUBTT_ENABLED
            infoDat->subtitleAvl = FFALSE;
            #endif /* APPL_SUBTT_ENABLED */

            #ifdef APPL_MHEG_ENABLED
            infoDat->mhegAvl = FFALSE;
            #endif /* APPL_MHEG_ENABLED */

            infoDat->crypted     = FFALSE;
            infoDat->teletextAvl = FFALSE;
            infoDat->is16_9      = FFALSE;
            infoDat->isHD        = FFALSE;
            break;

        default:   /* invalid entry */
            (void)STR_BufClear (&(infoDat->title));

            #ifdef APPL_SUBTT_ENABLED
            infoDat->subtitleAvl = FFALSE;
            #endif /* APPL_SUBTT_ENABLED */

            #ifdef APPL_MHEG_ENABLED
            infoDat->mhegAvl = FFALSE;
            #endif /* APPL_MHEG_ENABLED */

            infoDat->crypted     = FFALSE;
            infoDat->teletextAvl = FFALSE;
            infoDat->is16_9      = FFALSE;
            infoDat->isHD        = FFALSE;
            break;
    }

    MENU_InfoUpdateParentalLockStatus();

    infoDat->charIdx         = 0;
    infoDat->servNumVal      = 0;

    /* ----- update open menus --------------------------------------------- */

    MENU_InfoServiceNumBoxUpdate();
    MENU_InfoServiceNameBoxUpdate();
    MENU_InfoListSymbolBoxUpdate();

    if ( MENU_InfobarIsVisible() == FTRUE )
    {
        MENU_InfobarUpdateDisplayTimer();
        MENU_InfobarUpdateCurrNext();
        MENU_InfobarUpdateWideSymbol();
        MENU_InfobarUpdateServiceSymbols();
        MENU_InfobarUpdateAudio();
        MENU_InfobarUpdateProgressbar();
    }

    #ifdef APPL_PVR_ENABLED
    // TODO: ndern: soll innerhalb der Funktionen abgefragt werden!
    if ( appDat->pvrStatus.play.status == PLAY_STATUS_INACTIVE )
    #endif /* APPL_PVR_ENABLED */
    {
        memset(&epgExtParams, 0, sizeof(epgExtParams));

        epgExtParams.schedMode = FFALSE;
        epgExtParams.currType  = EIT_TYPE_CURR;

        /* update service description */
        MENU_EpgExtUpdate(&epgExtParams);

        /* update sched list */
        MENU_EpgSchedUpdate();
    }
}

void MENU_InfoUpdateAudioLanguage (void)
{
    MENU_InfobarUpdateAudio();
}

void MENU_InfoUpdateParentalLockStatus (void)
{
    #ifdef APPL_PARENTAL_ENABLED

    PCTRL_STATUS_S pctrlStatus;

    if ( PCTRL_GetStatus(&pctrlStatus) == FAPI_OK )
    {
        infoDat->isLocked = pctrlStatus.outputLocked ? FTRUE : FFALSE;
    }
    else
    {
        infoDat->isLocked = FFALSE;
    }

    if ( MENU_InfobarIsVisible() == FTRUE )
    {
        MENU_InfobarUpdateLockedIcon();
    }

    #endif /* APPL_PARENTAL_ENABLED */
}

void MENU_InfoEitUpdate (SLIST_SERVIDX servIdx)
{
    SYS_PRESSTATE_E  presState = SYS_PRESSTATE_NONE;
    fbool_t          update    = FFALSE;

    /* ----- infobar update ------------------------------------------------ */

    if ( MENU_InfobarIsVisible() == FTRUE )
    {
        (void)SYS_GetPresentationState(&presState);

        switch ( presState )
        {
            case SYS_PRESSTATE_LIVETV:
                if ( servIdx == infoDat->currServIdx )
                {
                    update = FTRUE;
                }
                break;

            #ifdef APPL_PVR_ENABLED

            case SYS_PRESSTATE_PLAYBACK:
                if ( servIdx == SLIST_SERVIDX_PLAY )
                {
                    update = FTRUE;
                }
                break;

            #endif /* APPL_PVR_ENABLED */

            default:
                break;
        }

        if ( update )
        {
            /* update current/next text description */
            MENU_InfobarUpdateCurrNext();
            MENU_InfobarUpdateProgressbar();
        }
    }

    /* ----- ext. EPG view update ------------------------------------------ */

    /* update "current" description */
    MENU_EpgExtUpdate (NULL);
}

void MENU_InfoUpdateSched (SLIST_SERVIDX servIdx)
{
    SYS_PRESSTATE_E  presState = SYS_PRESSTATE_NONE;
    fbool_t          update    = FFALSE;

    (void)SYS_GetPresentationState(&presState);

    switch ( presState )
    {
        case SYS_PRESSTATE_LIVETV:
            if ( servIdx == infoDat->currServIdx )
            {
                update = FTRUE;
            }
            break;

        #ifdef APPL_PVR_ENABLED

        case SYS_PRESSTATE_PLAYBACK:
            if ( servIdx == SLIST_SERVIDX_PLAY )
            {
                update = FTRUE;
            }
            break;

        #endif /* APPL_PVR_ENABLED */

        default:
            break;
    }

    if ( update )
    {
        MENU_EpgSchedUpdateList (FFALSE);
    }
}

void MENU_InfoUpdatePicFormat (SYS_PICFORMAT_E newFormat)
{
    fbool_t newIs16_9 = (newFormat == PICFORMAT_16_9) ? FTRUE : FFALSE;

    if ( newIs16_9 != infoDat->is16_9 )
    {
        infoDat->is16_9 = newIs16_9;

        if ( MENU_InfobarIsVisible() == FTRUE )
        {
            MENU_InfobarUpdateWideSymbol();
        }
    }
}

#ifdef APPL_PVR_ENABLED

void MENU_InfoUpdatePvrStatus (PVRSYS_STATUS_S* newStatus)
{
    fbool_t               playbackStarted = FFALSE;
    MENU_INFOBAR_PARAMS_S infoParams;

    /* show infobar if a playback has just started */
    if ( (appDat->pvrStatus.play.status == PLAY_STATUS_INACTIVE) &&
         (newStatus->play.status        != PLAY_STATUS_INACTIVE) )
    {
        playbackStarted = FTRUE;
    }

    appDat->pvrStatus = *newStatus; /* store new PVR status */

    /* update playback information if playback is active */
    if ( appDat->pvrStatus.play.status != PLAY_STATUS_INACTIVE )
    {
        (void)PVRSYS_PlayGetInfo       (&(appDat->playInf));
        (void)PVRSYS_PlayGetStreamInfo (&(appDat->playStreamInf));

        /* In case of timeshift playback: store service currently played */
        if ( appDat->pvrStatus.play.type == PLAY_TYPE_TIMESHIFT )
        {
            memset(&(appDat->playTimeShiftService), 0,
                           sizeof(appDat->playTimeShiftService));

            if ( appDat->pvrStatus.play.recIdx >= 0 )
            {
                (void)SLIST_GetServByIndex(appDat->pvrStatus.rec
                                           [appDat->pvrStatus.play.recIdx].
                                           servIdx,
                                           &(appDat->playTimeShiftService));
            }
        }
    }

    if ( playbackStarted )
    {
        if ( MENUSTACK_GetLevel() == 0 ) /* show infobar if possible */
        {
            infoParams.permanent   = FFALSE;
            infoParams.numberInput = -1;

            (void)MENUSTACK_Switch (MENU_InfobarEntry,
                                   (FAPI_SYS_HandleT)(&infoParams), 1);
        }
        else if ( MENU_InfobarIsVisible() == FTRUE ) /* infobar already open */
        {
            MENU_InfobarUpdateProgressbar();
            MENU_InfobarUpdateRecStatus(FTRUE);
            MENU_InfobarUpdateCurrNext();
        }

        MENU_EpgExtUpdate(NULL);
        MENU_EpgSchedUpdate();
    }
    else if ( MENU_InfobarIsVisible() == FTRUE  ) /* update infobar elements */
    {
        if ( newStatus->play.status != PLAY_STATUS_INACTIVE ) /* playback active */
        {
            // Update Playback Speed
        }

        MENU_InfobarUpdateRecStatus(FFALSE);
    }

    MENU_InfoListSymbolBoxUpdate();
    MENU_InfoServiceNameBoxUpdate();
    MENU_InfobarUpdateAudio();
}

#endif /* APPL_PVR_ENABLED */

void MENU_InfoUpdateLockStatus (uint8_t feNum, SYS_FE_LOCK_E status)
{
    fbool_t          update = FFALSE;
    AV_STATUS_LIVE_S liveStatus;

    if ( AV_GetLiveStatus(&liveStatus) != FAPI_OK )
    {
        return;
    }

    /* test if frontend is used for live-TV */
    if ( (liveStatus.status == SERVICE_STAT_STOPPED) ||
         (liveStatus.feIdx  != feNum) )
    {
        return;
    }

    switch ( status )
    {
        case SYS_FELOCK_NOTLOCKED:
        case SYS_FELOCK_LOCKED:
            update = FTRUE;
            break;

        default:
            break;
    }

    if ( update &&
        (MENU_InfobarIsVisible() == FTRUE ) )
    {
        /* update infobar if currently opened */
        #ifdef APPL_PVR_ENABLED
        if ( appDat->pvrStatus.play.status == PLAY_STATUS_INACTIVE )
        {
        #endif /* APPL_PVR_ENABLED */

            MENU_InfobarUpdateCurrNext();

        #ifdef APPL_PVR_ENABLED
        }
        #endif /* APPL_PVR_ENABLED */
    }
}

SYS_SERVLOC_S* MENU_InfoGetCurrServLoc(void)
{
    if(infoDat == NULL)
    {
        return NULL;
    }
    return &infoDat->currServLoc;
}

#ifdef APPL_PVR_ENABLED
SYS_TIME_S* MENU_InfoGetPlayTime(void)
{
    if(infoDat == NULL)
    {
        return NULL;
    }
    return &infoDat->playTime;
}

SYS_TIME_S* MENU_InfoGetRecTime(int8_t recIdx)
{
    if(infoDat == NULL)
    {
        return NULL;
    }
    if(recIdx >= RSRC_REC_MAX)
    {
        return NULL;
    }
    return &infoDat->recTime[recIdx];
}
#endif /* #ifdef APPL_PVR_ENABLED */

#ifdef APPL_SUBTT_ENABLED
fbool_t MENU_InfoIsSubtitleAvl(void)
{
    if(infoDat == NULL)
    {
        return FFALSE;
    }
    return infoDat->subtitleAvl;
}
#endif

#ifdef APPL_MHEG_ENABLED
fbool_t MENU_InfoIsMhegAvl(void)
{
    if(infoDat == NULL)
    {
        return FFALSE;
    }
    return infoDat->mhegAvl;
}
#endif

fbool_t MENU_InfoIsTeletextAvl(void)
{
    if(infoDat == NULL)
    {
        return FFALSE;
    }
    return infoDat->teletextAvl;
}

fbool_t MENU_InfoIsCrypted(void)
{
    if(infoDat == NULL)
    {
        return FFALSE;
    }
    return infoDat->crypted;
}

fbool_t MENU_InfoIs16_9(void)
{
    if(infoDat == NULL)
    {
        return FFALSE;
    }
    return infoDat->is16_9;
}

fbool_t MENU_InfoIsHD(void)
{
    if(infoDat == NULL)
    {
        return FFALSE;
    }
    return infoDat->isHD;
}

#ifdef APPL_PARENTAL_ENABLED

fbool_t MENU_InfoIsLocked(void)
{
    if(infoDat == NULL)
    {
        return FFALSE;
    }
    return infoDat->isLocked;
}
#endif /* APPL_PARENTAL_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
