/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_info.c
**
**              $Id: app_menu_infobar.c 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       Implementation of infobar and related OSD components.
**
** Detail description.
** - [\ref app_smartgo_app_ref_info]
** - [\ref app_smartgo_app_ref_infobar]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: loss of precision
** - Info    715: symbol 'anySymb' not referenced
** - Info    737: loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e734 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"
#include "app_menu_info.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/* Info bar dimensions */

/* top line items */
#ifdef APPL_SUBTT_ENABLED
#define MENU_INFOBAR_SYMB_WIDTH   (4 * SCALER_GetCurrVal(MAIN_ICON_WIDTH))
#else
#define MENU_INFOBAR_SYMB_WIDTH   (3 * SCALER_GetCurrVal(MAIN_ICON_WIDTH))
#endif /* APPL_SUBTT_ENABLED */

#define MENU_INFOBAR_TITLE_HEIGHT (2*(SCALER_GetCurrVal(TITLE_LINE_HEIGHT) + SCALER_GetCurrVal(INNER_FRAME_WIDTH)))

/* start pixels: left-aligned items: relative to title box */
#define MENU_INFOBAR_TITLE_X   (SCALER_GetCurrVal(MAIN_TEXT_PAD_X)+ \
                                SCALER_GetCurrVal(INFO_NUM_WIDTH) + \
                                SCALER_GetCurrVal(INNER_FRAME_WIDTH) + \
                                (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X)))

/* start pixels: right-aligned items: relative to title box end */
#define MENU_INFOBAR_TIME_X    (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) + \
                                SCALER_GetCurrVal(INFO_TIME_WIDTH))
#define MENU_INFOBAR_PROG_X    (SCALER_GetCurrVal(INNER_FRAME_WIDTH) + \
                                SCALER_GetCurrVal(INFO_PROG_WIDTH)   + \
                                SCALER_GetCurrVal(INFO_PROGBAR_PAD))
#define MENU_INFOBAR_SYMB_X    (MENU_INFOBAR_PROG_X + \
                                MENU_INFOBAR_SYMB_WIDTH + \
                                (2*SCALER_GetCurrVal(MAIN_TEXT_PAD_X)))

#define MENU_INFOBAR_PROGUPDATE_INTERVAL 250

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_INFOBAR_ID_TITLE = 0,
    MENU_INFOBAR_ID_LISTSYMBOL,
    MENU_INFOBAR_ID_SERVICENUM,
    MENU_INFOBAR_ID_TIME,
    MENU_INFOBAR_ID_EITTIMECURR,
    MENU_INFOBAR_ID_EITTIMENEXT,
    MENU_INFOBAR_ID_EITCURRNAME,
    MENU_INFOBAR_ID_EITCURRDESC,
    MENU_INFOBAR_ID_EITNEXTNAME,
    MENU_INFOBAR_ID_EITNEXTDESC,
    MENU_INFOBAR_ID_AUDIOLANG,

    #ifdef APPL_PVR_ENABLED
    MENU_INFOBAR_ID_RECICON1,
    MENU_INFOBAR_ID_RECICON2,
    MENU_INFOBAR_ID_RECINFO1,
    MENU_INFOBAR_ID_RECINFO2,
    MENU_INFOBAR_ID_PLAYICON,
    MENU_INFOBAR_ID_PLAYINFO,
    #endif

    #ifdef APPL_PARENTAL_ENABLED
    MENU_INFOBAR_ID_LOCKEDICON,
    #endif /* APPL_PARENTAL_ENABLED */

    MENU_INFOBAR_ID_AGEICON,
    MENU_INFOBAR_ID_AGEINFO,
    MENU_INFOBAR_ID_ICONTTX,

    #ifdef APPL_SUBTT_ENABLED
    MENU_INFOBAR_ID_ICONSUBT,
    #endif /* APPL_SUBTT_ENABLED */

    MENU_INFOBAR_ID_ICONCRYPT,
    MENU_INFOBAR_ID_ICONWIDE,
    MENU_INFOBAR_ID_PROGRESSBAR,
    MENU_INFOBAR_ID_REST
};

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;

    FGS_ITEM_S          evtTime[EIT_TYPE_NUMS]; //!< Present/following events.
    FGS_ITEM_S          evtName[EIT_TYPE_NUMS]; //!< Present/following events.
    FGS_ITEM_S          evtDesc[EIT_TYPE_NUMS]; //!< Present/following events.

    FGS_PANEL_MEMBER_S* pProgBar;
    FGS_POS_S           progBarPos;
    uint64_t            lastPbUpdate;

    FGS_PANEL_MEMBER_S* pItem;
    FGS_POS_S           itemPos;

    fbool_t             inPermanentMode;

    /* Present/following events */
    EIT_EVT_S        eitEvt[EIT_TYPE_NUMS];

} MENU_INFOBAR_INST_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC MENU_INFOBAR_INST_S*   infobar   = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_InfobarLeave               (FAPI_SYS_HandleT h,
                                                 fbool_t force);
FSTATIC void    MENU_InfobarOnFocusSet          (FAPI_SYS_HandleT h,
                                                 fbool_t hasFocus);
FSTATIC void    MENU_InfobarUpdateTimerCb       (void);
FSTATIC int32_t MENU_InfobarDraw                (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_InfobarDrawProgBar         (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_InfobarGetCurrNextEvt      (MENU_INFOBAR_INST_S* inst,
                                                 const SYS_SERVLOC_S* pServLoc,
                                                 EIT_EVT_TYPE_E type,
                                                 FGS_ITEM_S* pTimeItem,
                                                 FGS_ITEM_S* pTextItem);
FSTATIC fbool_t MENU_InfobarKeys                (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_InfobarBuild               (MENU_INFOBAR_INST_S* inst);

FSTATIC void MENU_InfobarGetParentalItem        (FAPI_SYS_HandleT h,
                                                 int32_t id,
                                                 uint32_t index,
                                                 FGS_ITEMSTATE_E state,
                                                 FGS_ITEM_S* item);
FSTATIC void MENU_InfobarGetCurrNextItem        (FAPI_SYS_HandleT h,
                                                 int32_t id,
                                                 uint32_t index,
                                                 FGS_ITEMSTATE_E state,
                                                 FGS_ITEM_S* item);
FSTATIC void MENU_InfobarGetAudioItem           (FAPI_SYS_HandleT h,
                                                 int32_t id,
                                                 uint32_t index,
                                                 FGS_ITEMSTATE_E state,
                                                 FGS_ITEM_S* item);
FSTATIC void MENU_InfobarGetSymbolItem          (FAPI_SYS_HandleT h,
                                                 int32_t id,
                                                 uint32_t index,
                                                 FGS_ITEMSTATE_E state,
                                                 FGS_ITEM_S* item);
FSTATIC void MENU_InfobarDisplayTimerExpired    (void* arg);
#ifdef APPL_PVR_ENABLED
FSTATIC void MENU_InfobarGetRecItem             (FAPI_SYS_HandleT h,
                                                 int32_t id,
                                                 uint32_t index,
                                                 FGS_ITEMSTATE_E state,
                                                 FGS_ITEM_S* item);
FSTATIC void MENU_InfobarGetPlayItem            (FAPI_SYS_HandleT h,
                                                 int32_t id,
                                                 uint32_t index,
                                                 FGS_ITEMSTATE_E state,
                                                 FGS_ITEM_S* item);
#endif /* APPL_PVR_ENABLED */

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
int32_t MENU_InfobarInit (void)
{
    if (infobar != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    infobar = UTILS_Zalloc(sizeof(MENU_INFOBAR_INST_S));
    if ( infobar == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    return FAPI_OK;
}

fbool_t MENU_InfobarEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_INFOBAR_INST_S*    inst = infobar;
    uint8_t                 numInput = 0xF;
    MENU_INFOBAR_PARAMS_S*  params = (MENU_INFOBAR_PARAMS_S*)h;

    if ( inst->pnlHdl != NULL )
    {
        /* infobar already open: update permanency mode and set/relaunch timer
           accordingly */
        if ( params != NULL )
        {
            inst->inPermanentMode = params->permanent;
        }

        MENU_InfobarUpdateDisplayTimer();
        return FFALSE;
    }

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    memset(inst, 0, sizeof(MENU_INFOBAR_INST_S));

    if ( params != NULL )
    {
        inst->inPermanentMode = params->permanent;

        if ( (params->numberInput >= 0) &&
             (params->numberInput <= 9) )
        {
            numInput = (uint8_t)(params->numberInput);
        }
    }
    else
    {
        inst->inPermanentMode = FTRUE;
    }

    (void) MENU_InfobarBuild(inst);

    MENU_InfoServiceNumBoxUpdate();
    MENU_InfobarUpdateCurrNext();
    MENU_InfobarUpdateServiceSymbols();

    #ifdef APPL_PVR_ENABLED

    MENU_InfobarUpdatePlayTime (FTRUE);

    #endif /* APPL_PVR_ENABLED */

    if ( numInput <= 9 )
    {
        (void)MENU_InfoServiceNumBoxInput(NULL, numInput);
    }

    (void)MENUSTACK_FocusClaim ((int32_t)MENUID_INFOBAR);
    (void)FGS_PANEL_Show(inst->pnlHdl, FFALSE);

    /* Raise minimum key repeat interval for smoother zapping */
    (void)KEYIN_SetRepeatInterval (125, 125);

    MENU_InfobarUpdateDisplayTimer();

    /* start timer for periodic updating of playback position */
    appDat->osdUpdateCb = MENU_InfobarUpdateTimerCb;

    /* choose minimum period: 100 ms */
    (void)TIME_PeriodicTimerLaunch(OSD_PERIODTIMER_GENERAL, 1);

    MENUSTACK_SetCleanup(MENU_InfobarLeave, (FAPI_SYS_HandleT)inst);
    return FTRUE;
}

fbool_t MENU_InfobarIsVisible(void)
{
    if ( ( infobar == NULL ) ||
         ( infobar->pnlHdl == NULL ) )
    {
        return FFALSE;
    }
    return FTRUE;
}

FSTATIC fbool_t MENU_InfobarLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_INFOBAR_INST_S* inst = (MENU_INFOBAR_INST_S*)h;

    DBG_Assert(inst->pnlHdl != 0);

    /* Restore minimum key repeat interval */
    (void)KEYIN_SetRepeatInterval (200, 100);

    (void)TIME_PeriodicTimerAbort(OSD_PERIODTIMER_GENERAL);

    appDat->osdUpdateCb = NULL;

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
    }

    MENU_InfoTimeBoxRemove();
    MENU_InfoServiceNumBoxRemove();
    MENU_InfoListSymbolBoxRemove();
    MENU_InfoServiceNameBoxRemove();

    MENU_InfoServiceNumBoxInputAbort(); /* reset service number input mode */

    (void)MENUSTACK_FocusRelease((int32_t)MENUID_INFOBAR);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC void MENU_InfobarOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_INFOBAR_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleZappingKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler   (FGS_KEY_MENU,   MAIN_DirectMenuTrans,    NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_BACK,   MENUSTACK_Exit,          NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_EXIT,   MENUSTACK_Exit,          NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_UP,     MAIN_ChannelUp,          NULL, FFALSE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_DOWN,   MAIN_ChannelDown,        NULL, FFALSE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_LEFT,   MAIN_VolumeDown,         NULL, FFALSE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_RIGHT,  MAIN_VolumeUp,           NULL, FFALSE, FFALSE);
        FGS_SetAppNumKeyHandler(FGS_KEY_NUMBER, MENU_InfoServiceNumBoxInput, NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_INFO,   MENUSTACK_Exit,          NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_NAVI,   MAIN_DirectMenuTrans,    NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_OK,     MENU_InfobarKeys,        NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_APPL,   MENU_InfobarKeys,        NULL, FTRUE,  FFALSE);

        #ifdef APPL_PIP_ENABLED
        FGS_SetAppKeyHandler   (FGS_KEY_BACK,   MAIN_KeyPIP,             NULL, FTRUE,  FFALSE);
        #endif
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler(FGS_KEY_UP);
        FGS_ClearAppKeyHandler(FGS_KEY_DOWN);
        FGS_ClearAppKeyHandler(FGS_KEY_LEFT);
        FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
        FGS_ClearAppKeyHandler(FGS_KEY_NUMBER);
        FGS_ClearAppKeyHandler(FGS_KEY_INFO);
        FGS_ClearAppKeyHandler(FGS_KEY_NAVI);
        FGS_ClearAppKeyHandler(FGS_KEY_OK);
        FGS_ClearAppKeyHandler(FGS_KEY_APPL);

        #ifdef APPL_PIP_ENABLED
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        #endif
    }
}

/*!
*******************************************************************************
**
** \brief   Handles the Keys adapted by this menu.
**
** This function is used as callback for the FGS_SetAppKeyHandler() and
** handles the following Keys adapted by the Infobar:
** - #FGS_KEY_OK
**
** \param   h   Address of infobar handle.
**
** \return  One of the following status codes:
**          - #FTRUE    If the key was used by this menu.
**          - #FFALSE   If the key wasn't used.
**
** \sa
**          - FGS_SetAppKeyHandler()
**          - MENU_InfobarOnFocusSet()
**
******************************************************************************/
FSTATIC fbool_t MENU_InfobarKeys (FAPI_SYS_HandleT h)
{
    fbool_t keyUsed = FTRUE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_OK:
            if(MENU_InfoIsInputMode() == FTRUE)
            {
                MAIN_SelectServiceByInput();
                keyUsed = FTRUE;
            }
            else
            {
                keyUsed = MAIN_DirectMenuTrans(h);
            }
            break;

        case FGS_KEY_APPL:
            return MENUSTACK_Switch(MENU_StrSelEntry,NULL,1);
        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

FSTATIC void MENU_InfobarUpdateTimerCb (void)
{
    uint64_t timeCurr;

    if ( infobar->pnlHdl == NULL )
    {
        return; /* infobar is not visible */
    }

    /* ----- update progress bar -------------------------------------------- */

    timeCurr = UTILS_GetTimerStamp();

    /* once in a second is sufficient */
    if ( UTILS_GetTimerStampInterval(infobar->lastPbUpdate, timeCurr) >=
         MENU_INFOBAR_PROGUPDATE_INTERVAL )
    {
        (void)FGS_PANEL_RefreshMember(infobar->pnlHdl, MENU_INFOBAR_ID_PROGRESSBAR);

        infobar->lastPbUpdate = timeCurr;
    }

    /* ----- update recording/timeshift status ----------------------------- */

    #ifdef APPL_PVR_ENABLED

    MENU_InfobarUpdateRecStatus(FFALSE);
    MENU_InfobarUpdatePlayTime (FFALSE);

    #endif /* APPL_PVR_ENABLED */
}

FSTATIC int32_t MENU_InfobarDrawProgBar (FAPI_SYS_HandleT h)
{
    MENU_INFOBAR_INST_S* inst = (MENU_INFOBAR_INST_S*)h;
    SYS_TIME_S           timeStruct;
    SYS_TIME_S           timeCurr;
    fbool_t              hasSynced;
    EIT_EVT_S            event;
    uint32_t             progCurr = 0;
    uint32_t             progMax  = 1;
    uint32_t             lenCurr;
    uint16_t             posX;
    uint16_t             posY;

    #ifdef APPL_PVR_ENABLED

    PVR_PlayInfoT        playInf;

    // TODO: hier lieber nach presentation state abfragen!
    if ( appDat->pvrStatus.play.status != PLAY_STATUS_INACTIVE )
    {
        if ( PVRSYS_PlayGetInfo(&playInf) != FAPI_OK )
        {
            return FAPI_OK;
        }
        progCurr = playInf.time;
        progMax  = playInf.duration;
    }
    else
    #endif /* APPL_PVR_ENABLED */
    {
        /* draw progress bar: only if "current" EIT event is available */

        if ( EIT_GetEvent (MENU_InfoGetCurrServLoc(),
                           EIT_TYPE_CURR, &event) == FAPI_OK )
        {
            /* get length of current service in seconds */
            progMax = TIME_ConvToSeconds (&(event.duration));

            /* get current time */
            if ( TIME_TimeAndDateGetUtc (&timeCurr, NULL, &hasSynced) !=
                 FAPI_OK )
            {
                progCurr = 0;
            }
            else if ( hasSynced == FFALSE ) /* time is not (yet) in DVB-Sync */
            {
                progCurr = 0;
            }
            else
            {
                timeStruct = event.startTime;

                progCurr = TIME_GetLengthInSeconds (&(timeStruct), &(timeCurr));
            }
        }
        else
        {
            progCurr = 0;
            progMax  = 1;
        }
    }

    /* --------------------------------------------------------------------- */

    posX = (uint16_t)(inst->pProgBar->abspos.x);
    posY = (uint16_t)(inst->pProgBar->abspos.y);

    if ( progCurr >= progMax )
    {
        lenCurr = inst->progBarPos.width;
    }
    else
    {
        lenCurr = (progCurr * inst->progBarPos.width) / progMax;
    }

    if ( lenCurr > 0 )
    {
        OSD_DrawFilledRectangle (posX, posY,
                                 posX + lenCurr - 1,
                                 posY + inst->progBarPos.height - 1,
                                 OSD_COL_BAR_ACTIVE);
    }

    if ( lenCurr < inst->progBarPos.width )
    {
        OSD_DrawFilledRectangle(posX + lenCurr, posY,
                                posX + inst->progBarPos.width - 1,
                                posY + inst->progBarPos.height - 1,
                                OSD_GetSchemeColor(OSD_SCHEMECOL_PROGRESSBAR_INACT));
    }

    return FAPI_OK;
}

FSTATIC int32_t MENU_InfobarDraw (FAPI_SYS_HandleT h)
{
    MENU_INFOBAR_INST_S* inst = (MENU_INFOBAR_INST_S*)h;
    uint16_t         x, y, innerWidth;
    uint16_t         posX;

    innerWidth = inst->itemPos.width;
    x          = (uint16_t)inst->pItem->abspos.x;
    y          = (uint16_t)inst->pItem->abspos.y;

    /* ----- elements in title box ----------------------------------------- */

    /* Vertical separators in top line */
    //left of service name
    OSD_DrawSeparatorVertical(x + MENU_INFOBAR_TITLE_X
                              - SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
                              - SCALER_GetCurrVal(INNER_FRAME_WIDTH),
                              y, SCALER_GetCurrVal(TITLE_LINE_HEIGHT));

    //right of list id
    OSD_DrawSeparatorVertical(x + (2*SCALER_GetCurrVal(MAIN_TEXT_PAD_X))+
                               SCALER_GetCurrVal(INFO_LISTID_WIDTH) +
                               SCALER_GetCurrVal(INNER_FRAME_WIDTH) ,
                              y + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + SCALER_GetCurrVal(TITLE_LINE_HEIGHT),
                              SCALER_GetCurrVal(TITLE_LINE_HEIGHT));

    //left of symbol boxes
    OSD_DrawSeparatorVertical(x + innerWidth - MENU_INFOBAR_SYMB_X
                              - SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
                              - SCALER_GetCurrVal(INNER_FRAME_WIDTH),
                              y + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + SCALER_GetCurrVal(TITLE_LINE_HEIGHT),
                              SCALER_GetCurrVal(TITLE_LINE_HEIGHT));

    //left of time box
    /* Use same x pos for separator line as for progressbar
    OSD_DrawSeparatorVertical(x + innerWidth - MENU_INFOBAR_TIME_X
                              - SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
                              - SCALER_GetCurrVal(INNER_FRAME_WIDTH),
                              y, SCALER_GetCurrVal(TITLE_LINE_HEIGHT)); */
    OSD_DrawSeparatorVertical(x + innerWidth - MENU_INFOBAR_PROG_X
                              - SCALER_GetCurrVal(INFO_PROGBAR_PAD)
                              - SCALER_GetCurrVal(INNER_FRAME_WIDTH),
                              y, SCALER_GetCurrVal(TITLE_LINE_HEIGHT));

    //left of progress bar
    OSD_DrawSeparatorVertical(x + innerWidth - MENU_INFOBAR_PROG_X
                              - SCALER_GetCurrVal(INFO_PROGBAR_PAD)
                              - SCALER_GetCurrVal(INNER_FRAME_WIDTH),
                              y + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + SCALER_GetCurrVal(TITLE_LINE_HEIGHT),
                              SCALER_GetCurrVal(TITLE_LINE_HEIGHT));

    /*horizontal inside the title*/
    OSD_DrawSeparatorHorizontal (x, y + SCALER_GetCurrVal(TITLE_LINE_HEIGHT), innerWidth);

    /* ----- elements in inner box ----------------------------------------- */

    y += MENU_INFOBAR_TITLE_HEIGHT +
         (4 * SCALER_GetCurrVal(LIST_LINE_HEIGHT));

    OSD_DrawSeparatorHorizontal (x, y, innerWidth);

    y += SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    /* ----- elements in bottom line --------------------------------------- */

    /* left-aligned items */
    posX = (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) * 3)
         +  SCALER_GetCurrVal(INFO_RECICON_WIDTH)
         +  SCALER_GetCurrVal(INFO_REC_WIDTH);

    OSD_DrawSeparatorVertical(x + posX, y, SCALER_GetCurrVal(LIST_LINE_HEIGHT));

    posX +=  SCALER_GetCurrVal(INNER_FRAME_WIDTH)
          + (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) * 3)
          +  SCALER_GetCurrVal(INFO_RECICON_WIDTH)
          +  SCALER_GetCurrVal(INFO_REC_WIDTH);

    OSD_DrawSeparatorVertical(x + posX, y, SCALER_GetCurrVal(LIST_LINE_HEIGHT));

    /* right-aligned items */
    posX  = innerWidth
          - (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) * 2)
          -  SCALER_GetCurrVal(INFO_AUDIO_WIDTH)
          -  SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(x + posX, y, SCALER_GetCurrVal(LIST_LINE_HEIGHT));

    posX -=  (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) * 3)
          +  SCALER_GetCurrVal(INFO_AGEICON_WIDTH)
          +  SCALER_GetCurrVal(INFO_AGE_WIDTH)
          +  SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(x + posX, y, SCALER_GetCurrVal(LIST_LINE_HEIGHT));

    posX -=  (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) * 2)
          +  SCALER_GetCurrVal(INFO_LOCKEDICON_WIDTH)
          +  SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(x + posX, y, SCALER_GetCurrVal(LIST_LINE_HEIGHT));

    return FAPI_OK;
}

#ifdef APPL_PVR_ENABLED

void MENU_InfobarUpdatePlayTime (fbool_t force)
{
    MENU_INFOBAR_INST_S* inst = infobar;
    PVR_PlayInfoT   playInf;
    PVR_RecordInfoT recInf;
    SYS_TIME_S      newTime;
    fbool_t         draw = FFALSE;
    uint32_t        tempDuration;
    SYS_TIME_S*     playTime;

    memset(&newTime, 0, sizeof(newTime));

    switch ( appDat->pvrStatus.play.type )
    {
        case PLAY_TYPE_NONE:
            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                           MENU_INFOBAR_ID_PLAYICON);
            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                           MENU_INFOBAR_ID_PLAYINFO);
            break;

        case PLAY_TYPE_NORMAL:
            if ( PVRSYS_PlayGetInfo(&playInf) != FAPI_OK ) break;

            newTime = TIME_ConvToTime(playInf.time / 1000);
            draw    = FTRUE;
            break;

        case PLAY_TYPE_TIMESHIFT:

            /* display negative offset to live-TV */
            if ( PVRSYS_PlayGetInfo(&playInf) != FAPI_OK )
            {
                break;
            }
            if ( PVRSYS_RecordGetInfo(appDat->pvrStatus.play.recIdx,
                                      NULL, &recInf) != FAPI_OK )
            {
                break;
            }

            if ( recInf.duration > playInf.time )
            {
                recInf.duration -= playInf.time;

                tempDuration = recInf.duration / 1000;
                if ( (recInf.duration % 1000) >= 500 ) tempDuration += 1;
            }
            else
            {
                tempDuration = 0;
            }

            newTime = TIME_ConvToTime(tempDuration);
            draw    = FTRUE;
            break;
    }

    if ( draw )
    {
        playTime = MENU_InfoGetPlayTime();
        if(playTime != NULL)
        {
            if ( (force == FTRUE ) ||
                 (newTime.hour   != playTime->hour)   ||
                 (newTime.minute != playTime->minute) ||
                 (newTime.second != playTime->second) )
            {
                *playTime = newTime;

                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MENU_INFOBAR_ID_PLAYICON);
                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MENU_INFOBAR_ID_PLAYINFO);
            }
        }
    }
}

// NOTE: Assumes infobar being currently open
void MENU_InfobarUpdateRecStatus (fbool_t force)
{
    MENU_INFOBAR_INST_S* inst = infobar;
    PVR_RecordInfoT recInf;
    int8_t          cnt;
    SYS_TIME_S      newTime;
    SYS_TIME_S*     recTime;

    for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
    {
        switch ( appDat->pvrStatus.rec[cnt].status )
        {
            default:
            case REC_STATUS_INACTIVE:

                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MENU_INFOBAR_ID_RECICON1 + cnt);
                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MENU_INFOBAR_ID_RECINFO1 + cnt);
                break;

            case REC_STATUS_STOP:
            case REC_STATUS_ACTIVE:
                if ( PVRSYS_RecordGetInfo(cnt, NULL, &recInf) != FAPI_OK )
                {
                    break;
                }

                newTime = TIME_ConvToTime(recInf.duration / 1000);
                recTime = MENU_InfoGetRecTime(cnt);
                if(recTime != NULL)
                {
                    if ( force ||
                         (newTime.hour   != recTime->hour)   ||
                         (newTime.minute != recTime->minute) ||
                         (newTime.second != recTime->second) )
                    {
                        *recTime = newTime;

                        (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                      MENU_INFOBAR_ID_RECICON1 +
                                                      cnt);
                        (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                      MENU_INFOBAR_ID_RECINFO1 +
                                                      cnt);
                    }
                }
                break;
        }
    }
}

#endif /* APPL_PVR_ENABLED */

void MENU_InfobarUpdateWideSymbol ( void )
{
    MENU_INFOBAR_INST_S* inst = infobar;

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_ICONWIDE);
    }
}

void MENU_InfobarUpdateProgressbar ( void )
{
    MENU_INFOBAR_INST_S* inst = infobar;

    if ( inst->pnlHdl != NULL )
    {
        (void) FGS_PANEL_RefreshMember(infobar->pnlHdl,
                                       MENU_INFOBAR_ID_PROGRESSBAR);
    }
}
#ifdef APPL_PARENTAL_ENABLED
void MENU_InfobarUpdateLockedIcon ( void )
{
    MENU_INFOBAR_INST_S* inst = infobar;

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember(infobar->pnlHdl,
                                      MENU_INFOBAR_ID_LOCKEDICON);
    }
}
#endif

FSTATIC void MENU_InfobarUpdateServiceSymbols ( void )
{
    MENU_INFOBAR_INST_S* inst = infobar;

    if ( inst->pnlHdl != NULL )
    {
        #ifdef APPL_SUBTT_ENABLED
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_ICONSUBT);
        #endif /* APPL_SUBTT_ENABLED */

        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_ICONTTX);
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_ICONCRYPT);
    }
}

FSTATIC void MENU_InfobarGetCurrNextItem (FAPI_SYS_HandleT h, int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item)
{
    MENU_INFOBAR_INST_S* inst = (MENU_INFOBAR_INST_S*)h;

    switch ( id )
    {
        case MENU_INFOBAR_ID_EITTIMECURR: *item = inst->evtTime[EIT_TYPE_CURR];
                                          break;
        case MENU_INFOBAR_ID_EITTIMENEXT: *item = inst->evtTime[EIT_TYPE_NEXT];
                                          break;
        case MENU_INFOBAR_ID_EITCURRNAME: *item = inst->evtName[EIT_TYPE_CURR];
                                          break;
        case MENU_INFOBAR_ID_EITCURRDESC: *item = inst->evtDesc[EIT_TYPE_CURR];
                                          break;
        case MENU_INFOBAR_ID_EITNEXTNAME: *item = inst->evtName[EIT_TYPE_NEXT];
                                          break;
        case MENU_INFOBAR_ID_EITNEXTDESC: *item = inst->evtDesc[EIT_TYPE_NEXT];
                                          break;

        default:
            /* do nothing */
            break;
    }
}

void MENU_InfobarUpdateDisplayTimer ( void )
{
    uint16_t dispTime;

    if ( infobar->pnlHdl == NULL )
    {
        return;
    }

    if ( (!infobar->inPermanentMode) &&
         (SETTINGS_DisplayTimeGet(&dispTime) == FAPI_OK) )
    {
        /* Launch or restart timer */
        (void)MAIN_MenuTimerLaunch(dispTime, MENU_InfobarDisplayTimerExpired,
                                   NULL);
    }
    else
    {
        /* abort a potentially active expiration timer */
        MAIN_MenuTimerAbort();
    }
}

FSTATIC void MENU_InfobarGetAudioItem (FAPI_SYS_HandleT h, int32_t id,
                                       uint32_t index, FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item)
{
    SLIST_ITEM_S service;
    int32_t      retVal;
    int8_t       streamIdx;

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;
    item->item.str.col        = OSD_COL_TEXT;
    item->item.str.chr        = OSD_GetStr(STR_EMPTYSTRING); /* default string */

    #ifdef APPL_PVR_ENABLED

    if ( appDat->pvrStatus.play.status != PLAY_STATUS_INACTIVE )
    {
        retVal = PVRSYS_PlayGetAudioStream (&streamIdx);

        /* audio information available? */
        if ( (retVal == FAPI_OK) && (streamIdx >= 0) )
        {
            item->item.str.chr = OSD_LANG_GetLanguageString(
                                 appDat->playStreamInf.audio.stream[streamIdx].
                                 langCode);
        }
    }
    else
    #endif /* APPL_PVR_ENABLED */
    {
        if ( SLIST_EntryGet(MENU_InfoGetCurrServLoc(), &service) == FAPI_OK )
        {
            retVal = AV_GetAudioStream(&streamIdx);

            /* audio information available? */
            if ( (retVal == FAPI_OK) && (streamIdx >= 0) )
            {
                item->item.str.chr = OSD_LANG_GetLanguageString(
                                     service.strPid.audio.stream[streamIdx].
                                     langCode);
            }
        }
    }
}

FSTATIC void MENU_InfobarGetSymbolItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                        FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    SYS_PRESSTATE_E  presState;

    if ( SYS_GetPresentationState(&presState) != FAPI_OK ) return;

    switch ( id )
    {
        #ifdef APPL_SUBTT_ENABLED

        case MENU_INFOBAR_ID_ICONSUBT:
            if ( (presState != SYS_PRESSTATE_LIVETV) ||
                 (MENU_InfoIsSubtitleAvl() == FFALSE) )
            {
                return;
            }

            item->item.icon.addr = OSD_IconGet(ICON_SUBT);
            break;

        #endif /* APPL_SUBTT_ENABLED */

        case MENU_INFOBAR_ID_ICONTTX:
            if ( (presState == SYS_PRESSTATE_LIVETV) &&
                 ((MENU_InfoIsTeletextAvl() == FTRUE)
                 #ifdef APPL_MHEG_ENABLED
                  ||(MENU_InfoIsMhegAvl() == FTRUE)
                 #endif /* APPL_MHEG_ENABLED */
                 ) )
            {
                item->item.icon.addr = OSD_IconGet(ICON_TTXT);
            }
            break;

        case MENU_INFOBAR_ID_ICONCRYPT:
            if ( (presState != SYS_PRESSTATE_LIVETV) ||
                 (MENU_InfoIsCrypted() == FFALSE ) )
            {
                return;
            }

            item->item.icon.addr = OSD_IconGet(ICON_CRYPTED);
            break;

        case MENU_INFOBAR_ID_ICONWIDE:
            if (presState == SYS_PRESSTATE_NONE)
            {
                return;
            }
            if ( MENU_InfoIs16_9() == FTRUE )
            {
                item->item.icon.addr = OSD_IconGet(ICON_16_9);
            }
            if ( MENU_InfoIsHD() == FTRUE )
            {
                item->item.icon.addr = OSD_IconGet(ICON_HD);
            }
            break;

        default:
            return;
    }

    item->type                 = FGS_ITEM_TYPE_ICON;
    item->align                = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
    item->item.icon.drawMode   = FGS_ICONMODE_OVRLAYPLAIN;
    item->item.icon.bkgrCol    = OSD_COL_BLANK;
    item->item.icon.altBkgrCol = OSD_GetSchemeColor(OSD_SCHEMECOL_TITLE_BKGR);
}

FSTATIC void MENU_InfobarGetParentalItem (FAPI_SYS_HandleT h, int32_t id,
                                          uint32_t index, FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item)
{
    switch ( id )
    {
        #ifdef APPL_PARENTAL_ENABLED

        case MENU_INFOBAR_ID_LOCKEDICON:
            if ( MENU_InfoIsLocked() == FTRUE )
            {
                item->type                 = FGS_ITEM_TYPE_ICON;
                item->align                = FGS_ALIGN_HOR_CENTER
                                           | FGS_ALIGN_VER_CENTER;
                item->item.icon.addr       = OSD_IconGet(ICON_LOCKED);
                item->item.icon.drawMode   = FGS_ICONMODE_OVRLAYPLAIN;
                item->item.icon.bkgrCol    = OSD_COL_BLANK;
                item->item.icon.altBkgrCol = OSD_GetSchemeColor(
                                             OSD_SCHEMECOL_MAIN_BKGR);
            }
            break;

        #endif /* APPL_PARENTAL_ENABLED */

        case MENU_INFOBAR_ID_AGEICON:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_RIGHT
                                         | FGS_ALIGN_VER_CENTER;
            item->item.str.font          = MAIN_FONT;
            item->item.str.lineHeight    = 0;
            item->item.str.col           = OSD_COL_TEXT;
            item->item.str.chr           = OSD_GetStr(STR_GNR_AGE); // TODO: add symbol
            break;

        case MENU_INFOBAR_ID_AGEINFO:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_LEFT
                                         | FGS_ALIGN_VER_CENTER;
            item->item.str.font          = MAIN_FONT;
            item->item.str.lineHeight    = 0;
            item->item.str.col           = OSD_COL_TEXT;

            if( SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW )
            {
                if ( infobar->eitEvt[EIT_TYPE_CURR].minAge != 0 )
                {
                    switch(infobar->eitEvt[EIT_TYPE_CURR].minAge)
                    {
                        default:
                        case 0x00:
                        case 0x01: sprintf(appDat->asciiBuf, "%s","-"); break;
                        case 0x02:
                        case 0x03: sprintf(appDat->asciiBuf, "%s","P"); break;
                        case 0x04:
                        case 0x05: sprintf(appDat->asciiBuf, "%s","C"); break;
                        case 0x06:
                        case 0x07: sprintf(appDat->asciiBuf, "%s","G"); break;
                        case 0x08:
                        case 0x09: sprintf(appDat->asciiBuf, "%s","PG"); break;
                        case 0x0A:
                        case 0x0B: sprintf(appDat->asciiBuf, "%s","M"); break;
                        case 0x0C:
                        case 0x0D: sprintf(appDat->asciiBuf, "%s","MA 15+"); break;
                        case 0x0E: sprintf(appDat->asciiBuf, "%s","AV 15+"); break;
                        case 0x0F: sprintf(appDat->asciiBuf, "%s","R"); break;
                    }
                    item->item.str.chr = STR_IntASCIIStr();
                }
                else
                {
                    item->item.str.chr = OSD_GetStr(STR_GNR_N_A);
                }
            }
            else
            {
                if ( infobar->eitEvt[EIT_TYPE_CURR].minAge != 0 )
                {
                    sprintf(appDat->asciiBuf, "%i",
                            infobar->eitEvt[EIT_TYPE_CURR].minAge);
                    item->item.str.chr = STR_IntASCIIStr();
                }
                else
                {
                    item->item.str.chr = OSD_GetStr(STR_GNR_N_A);
                }
            }
            break;

        default:
            /* do nothing */
            break;
    }
}

#ifdef APPL_PVR_ENABLED

FSTATIC void MENU_InfobarGetRecItem (FAPI_SYS_HandleT h, int32_t id,
                                     uint32_t index, FGS_ITEMSTATE_E state,
                                     FGS_ITEM_S* item)
{
    SYS_TIME_S* recTime;
    int8_t     recIdx;

    if ( id <= MENU_INFOBAR_ID_RECICON2 )
    {
        recIdx = id - MENU_INFOBAR_ID_RECICON1;
    }
    else
    {
        recIdx = id - MENU_INFOBAR_ID_RECINFO1;
    }

    if ( (recIdx < 0) || (recIdx >= RSRC_REC_MAX) )
    {
        return;
    }

    switch ( appDat->pvrStatus.rec[recIdx].status )
    {
        default:
        case REC_STATUS_INACTIVE:
            /* box is empty */
            break;

        case REC_STATUS_STOP:
        case REC_STATUS_ACTIVE:

            switch ( id )
            {
                case MENU_INFOBAR_ID_RECICON1:
                case MENU_INFOBAR_ID_RECICON2:
                    item->type                   = FGS_ITEM_TYPE_STRING;
                    item->align                  = FGS_ALIGN_HOR_RIGHT
                                                 | FGS_ALIGN_VER_CENTER;
                    item->item.str.font          = SYMBOL_TITLE;
                    item->item.str.lineHeight    = 0;
                    item->item.str.col           = OSD_COL_RED;
                    item->item.str.chr           = OSD_GetStr(STR_SYMBOL_RECORD);
                    break;

                case MENU_INFOBAR_ID_RECINFO1:
                case MENU_INFOBAR_ID_RECINFO2:
                    recTime = MENU_InfoGetRecTime(recIdx);
                    if(recTime)
                    {
                        item->item.time.value  = *recTime;
                    }
                    item->type             = FGS_ITEM_TYPE_TIME;
                    item->align            = FGS_ALIGN_HOR_LEFT |
                                             FGS_ALIGN_VER_CENTER;
                    item->item.time.format = FGS_FORMAT_TIME_PLAYTIME;
                    item->item.time.font   = MAIN_FONT;
                    item->item.time.col    = OSD_COL_TEXT;
                    break;

                default:
                    /* do nothing */
                    break;
            }
            break;
    }
}

FSTATIC void MENU_InfobarGetPlayItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    fbool_t     isTs = FFALSE;
    SYS_TIME_S* playTime;

    switch ( appDat->pvrStatus.play.type )
    {
        default:
        case PLAY_TYPE_NONE:
            return;

        case PLAY_TYPE_NORMAL:
            /* do nothing */
            break;

        case PLAY_TYPE_TIMESHIFT:
            isTs = FTRUE;
            break;
    }

    switch ( id )
    {
        case MENU_INFOBAR_ID_PLAYICON:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_RIGHT
                                         | FGS_ALIGN_VER_CENTER;
            item->item.str.font          = SYMBOL_TITLE;
            item->item.str.lineHeight    = 0;
            item->item.str.col           = OSD_COL_TEXT;
            item->item.str.chr           = OSD_GetStr(STR_SYMBOL_PLAY);
            break;

        case MENU_INFOBAR_ID_PLAYINFO:
            playTime = MENU_InfoGetPlayTime();
            if(playTime != NULL)
            {
                item->item.time.value  = *playTime;
            }
            item->type             = FGS_ITEM_TYPE_TIME;
            item->align            = FGS_ALIGN_HOR_LEFT |
                                     FGS_ALIGN_VER_CENTER;
            item->item.time.font   = MAIN_FONT;
            item->item.time.col    = OSD_COL_TEXT;

            if ( isTs )
            {
                item->item.time.format = FGS_FORMAT_TIME_NEGPLAYTIME;
            }
            else
            {
                item->item.time.format = FGS_FORMAT_TIME_PLAYTIME;
            }
            break;

        default:
            /* do nothing */
            break;
    }
}

#endif /* APPL_PVR_ENABLED */

/* ------------------------------------------------------------------------- */

FSTATIC void MENU_InfobarDisplayTimerExpired (void* arg)
{
    if ( infobar->pnlHdl != NULL ) /* infobar open? */
    {
        /* do not close if currently in service number input mode */
        if ( (MENU_InfoIsInputMode()   == FFALSE) &&
             (infobar->inPermanentMode == FFALSE) )
        {
            (void)MENUSTACK_Exit(NULL);
        }
    }
}


void MENU_InfobarUpdateCurrNext ( void )
{
    MENU_INFOBAR_INST_S* inst = infobar;
    FGS_ITEM_S           textItem;
    FGS_ITEM_S           timeItem;
    AV_STATUS_LIVE_S     liveStatus;
    SYS_FE_STATUS_S      feStatus;
    SYS_PRESSTATE_E      presState;

    if ( inst->pnlHdl == NULL ) return;

    /* clear EIT area in inner box */
    inst->evtTime[EIT_TYPE_CURR].type = FGS_ITEM_TYPE_NONE;
    inst->evtTime[EIT_TYPE_NEXT].type = FGS_ITEM_TYPE_NONE;
    inst->evtName[EIT_TYPE_CURR].type = FGS_ITEM_TYPE_NONE;
    inst->evtName[EIT_TYPE_NEXT].type = FGS_ITEM_TYPE_NONE;
    inst->evtDesc[EIT_TYPE_CURR].type = FGS_ITEM_TYPE_NONE;
    inst->evtDesc[EIT_TYPE_NEXT].type = FGS_ITEM_TYPE_NONE;

    textItem.type                = FGS_ITEM_TYPE_STRING;
    textItem.align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    textItem.item.str.font       = MAIN_FONT;
    textItem.item.str.lineHeight = 0;
    textItem.item.str.col        = OSD_COL_TEXT;

    timeItem.type                = FGS_ITEM_TYPE_TIME;
    timeItem.align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    timeItem.item.time.format    = appDat->timeFormat;
    timeItem.item.time.font      = MAIN_FONT;
    timeItem.item.time.col       = OSD_COL_TEXT;

    if ( SYS_GetPresentationState(&presState) != FAPI_OK ) return;

    switch ( presState )
    {
        default:
        case SYS_PRESSTATE_NONE:
            /* Test if main service lists are empty */
            if ( (SLIST_ListSizeGet(SLIST_TV)    == 0) &&
                 (SLIST_ListSizeGet(SLIST_RADIO) == 0) )
            {
                textItem.item.str.chr = OSD_GetStr(STR_MSG_NOSERVICES);

                inst->evtName[EIT_TYPE_CURR] = textItem;
            }
            break;

        case SYS_PRESSTATE_LIVETV:
            if ( AV_GetLiveStatus(&liveStatus) != FAPI_OK ) break;

            else if ( liveStatus.status == SERVICE_STAT_STOPPED ) break;

            else if ( AV_GetFeStatus(liveStatus.feIdx, &feStatus) != FAPI_OK )
            {
                break;
            }

            if ( (feStatus.lock == SYS_FELOCK_NOTLOCKED) ||
                 (feStatus.lock == SYS_FELOCK_RECOVERYTUNING) )
            {
                textItem.item.str.chr = OSD_GetStr(STR_MSG_NOTLOCKED);

                inst->evtName[EIT_TYPE_CURR] = textItem;
            }
            else
            {
                /* draw "present" info */
                (void)MENU_InfobarGetCurrNextEvt (inst,
                                                  MENU_InfoGetCurrServLoc(),
                                                  EIT_TYPE_CURR,
                                                  &timeItem, &textItem);

                /* draw "following" info */
                (void)MENU_InfobarGetCurrNextEvt (inst,
                                                  MENU_InfoGetCurrServLoc(),
                                                  EIT_TYPE_NEXT,
                                                  &timeItem, &textItem);
            }
            break;

        case SYS_PRESSTATE_PLAYBACK:
            /* draw "present" info */
            (void)MENU_InfobarGetCurrNextEvt (inst, NULL, EIT_TYPE_CURR,
                                              &timeItem, &textItem);

            /* draw "following" info */
            (void)MENU_InfobarGetCurrNextEvt (inst, NULL, EIT_TYPE_NEXT,
                                              &timeItem, &textItem);
            break;
    }

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_EITTIMECURR);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_EITTIMENEXT);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_EITCURRNAME);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_EITCURRDESC);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_EITNEXTNAME);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_EITNEXTDESC);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_AGEINFO);
}

FSTATIC int32_t MENU_InfobarGetCurrNextEvt (MENU_INFOBAR_INST_S* inst,
                                            const SYS_SERVLOC_S* pServLoc,
                                            EIT_EVT_TYPE_E type,
                                            FGS_ITEM_S* pTimeItem,
                                            FGS_ITEM_S* pTextItem)
{
    EIT_EVT_S*      pEvt = infobar->eitEvt + type;
    int32_t         retVal;
    SYS_STRING_S    string;

    memset (pEvt, 0, sizeof(EIT_EVT_S));

    retVal = EIT_GetEvent (pServLoc, type, pEvt);

    switch ( retVal )
    {
        case FAPI_OK:
            pTimeItem->item.time.value = pEvt->startTime;

            (void)TIME_AddUtcOffset(&(pTimeItem->item.time.value), NULL);
            inst->evtTime[type] = *pTimeItem;

            pTextItem->item.str.chr = STR_Build (pEvt->nameCharSet, pEvt->name);
            inst->evtName[type] = *pTextItem;

            string = STR_Build (pEvt->descCharSet, pEvt->desc);

            pTextItem->item.str.chr = string;
            inst->evtDesc[type] = *pTextItem;

            /* Reformat event description: Line breaks are replaced by
               semicolons as there is only a single line for displaying the
               string! */
            STR_ReplaceLineBreaks(string, ';', AV_EIT_DESCLEN_MAX);
            break;

        case SMARTGO_SYS_ERR_DATA_NOT_PROVIDED:

            if ( type == EIT_TYPE_CURR )
            {
                pTextItem->item.str.chr = OSD_GetStr(STR_GNR_NOEITCURR);
            }
            else
            {
                pTextItem->item.str.chr = OSD_GetStr(STR_GNR_NOEITNEXT);
            }

            inst->evtName[type] = *pTextItem;
            break;

        default:
            /* data not yet available */
            break;
    }

    return retVal;
}


void MENU_InfobarUpdateAudio ( void )
{
    MENU_INFOBAR_INST_S* inst = infobar;

    if ( (inst         == NULL) ||
         (inst->pnlHdl == NULL) ) return;

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_INFOBAR_ID_AUDIOLANG);
}

/*!
*******************************************************************************
**
** \brief   Builds the menu pannel and its members.
**
** \param   inst    Menu handle pointer.
**
** \return
**          - #FTRUE
**          - #FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_InfobarBuild(MENU_INFOBAR_INST_S* inst)
{
    FGS_PANEL_OPEN_S        pnlPars;
    FGS_PANEL_MEMBER_S      member;
    FGS_BOX_OPEN_S          boxPars;
    int32_t                 retVal;
    FAPI_SYS_HandleT        itemHdl;
    #ifdef APPL_PVR_ENABLED
    uint32_t                cnt;
    #endif /* APPL_PVR_ENABLED */

    memset(&pnlPars, 0, sizeof(pnlPars));
    memset(&boxPars, 0, sizeof(boxPars));
    memset(&member,  0, sizeof(member));

    do
    {
        pnlPars.bkgrType      = OSD_PNLTYPE_INFOBAR;
        pnlPars.pos.width     = SCALER_GetCurrVal(INFO_WIDTH);
        pnlPars.pos.height    = (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH)) +
                                MENU_INFOBAR_TITLE_HEIGHT +
                                (5 * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                                SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        pnlPars.pos.x         = 0;
        pnlPars.pos.y         = -((int16_t)(SCALER_GetCurrVal(MAIN_PAD_Y)
                                  + pnlPars.pos.height));
        pnlPars.pos.alignment = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_BOTTOM;
        pnlPars.onFocusSet    = MENU_InfobarOnFocusSet;
        pnlPars.onFocusSetArg = (FAPI_SYS_HandleT)inst;
        pnlPars.pInst         = &(appDat->pnlInst);

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_INFOBAR, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create service number textbox ----------------------------- */
        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                                + SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        boxPars.pos.width       = SCALER_GetCurrVal(INFO_NUM_WIDTH);
        boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_TITLE | OSD_CORNER_TOP_LEFT;
        boxPars.getItem         = MAIN_ServiceNumGetItem;
        boxPars.getItemArg      = NULL;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_SERVICENUM,
                                inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        appDat->numBoxPnlHdl = inst->pnlHdl;
        appDat->numBoxId     = MENU_INFOBAR_ID_SERVICENUM;

        /* ----- create list symbol box ------------------------------------ */
        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X) +
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.width       = SCALER_GetCurrVal(INFO_LISTID_WIDTH);
        boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_TITLE;
        boxPars.getItem         = MENU_InfoListSymbolBoxGetItem;
        boxPars.getItemArg      = NULL;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_LISTSYMBOL,
                             inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        appDat->symbBoxPnlHdl = inst->pnlHdl;
        appDat->symbBoxId     = MENU_INFOBAR_ID_LISTSYMBOL;
        appDat->symbBoxParent = MENUID_INFOBAR;

        /* ----- create service name textbox ------------------------------- */
        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  MENU_INFOBAR_TITLE_X;
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        boxPars.pos.width       = pnlPars.pos.width -
                                  (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH)) -
                                  MENU_INFOBAR_TITLE_X - MENU_INFOBAR_TIME_X -
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH) -
                                  (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X));
        boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_TITLE;
        boxPars.getItem         = MENU_InfoServiceNameBoxGetItem;
        boxPars.getItemArg      = NULL;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_TITLE,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        appDat->nameBoxPnlHdl = inst->pnlHdl;
        appDat->nameBoxId     = MENU_INFOBAR_ID_TITLE;
        appDat->nameBoxParent = MENUID_INFOBAR;

        /* ----- create time textbox --------------------------------------- */
        boxPars.pos.x           = -((int16_t)SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                                  + (int16_t)SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
                                  + (int16_t)SCALER_GetCurrVal(INFO_TIME_WIDTH));
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        boxPars.pos.width       = SCALER_GetCurrVal(INFO_TIME_WIDTH);
        boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_TITLE | OSD_CORNER_TOP_RIGHT;
        boxPars.getItem         = MENU_InfoTimeBoxGetItem;
        boxPars.getItemArg      = NULL;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_TIME,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        appDat->timeBoxPnlHdl = inst->pnlHdl;
        appDat->timeBoxId     = MENU_INFOBAR_ID_TIME;

        /* ----- create EIT boxes ------------------------------------------ */
        /* time column */
        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  MENU_INFOBAR_TITLE_HEIGHT;
        boxPars.pos.width       = SCALER_GetCurrVal(INFO_STIME_WIDTH);
        boxPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem         = MENU_InfobarGetCurrNextItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_EITTIMECURR,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.y += 2 * SCALER_GetCurrVal(LIST_LINE_HEIGHT);

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_EITTIMENEXT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* text column */
        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X) +
                                  SCALER_GetCurrVal(INFO_STIME_WIDTH) +
                                  (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X));
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  MENU_INFOBAR_TITLE_HEIGHT;
        boxPars.pos.width       = pnlPars.pos.width -
                                  boxPars.pos.x -
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X) -
                                  SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        boxPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_EITCURRNAME,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT);

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_EITCURRDESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT);

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_EITNEXTNAME,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT);

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_EITNEXTDESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- create bottom line boxes ---------------------------------- */

        /* general box parameters */
        boxPars.pos.y           = -((int16_t)
                                   (SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                                  + SCALER_GetCurrVal(LIST_LINE_HEIGHT)));
        boxPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT);

        /* create audio language box */
        boxPars.pos.x           = -((int16_t)(SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                                  + SCALER_GetCurrVal(INFO_AUDIO_WIDTH)
                                  + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)));
        boxPars.pos.width       = SCALER_GetCurrVal(INFO_AUDIO_WIDTH);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL | OSD_CORNER_BOT_RIGHT;

        boxPars.getItem         = MENU_InfobarGetAudioItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_AUDIOLANG,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* create viewing age information boxes */

        boxPars.pos.x          -= SCALER_GetCurrVal(INFO_AGE_WIDTH)
                                + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                + (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) * 2);
        boxPars.pos.width       = SCALER_GetCurrVal(INFO_AGE_WIDTH);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem         = MENU_InfobarGetParentalItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_AGEINFO,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x          -= SCALER_GetCurrVal(INFO_AGEICON_WIDTH)
                                + SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        boxPars.pos.width       = SCALER_GetCurrVal(INFO_AGEICON_WIDTH);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem         = MENU_InfobarGetParentalItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_AGEICON,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_PARENTAL_ENABLED

        boxPars.pos.x          -=  SCALER_GetCurrVal(INFO_LOCKEDICON_WIDTH)
                                +  SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                + (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) * 2);
        boxPars.pos.width       = SCALER_GetCurrVal(INFO_LOCKEDICON_WIDTH);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem         = MENU_InfobarGetParentalItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_LOCKEDICON,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_PARENTAL_ENABLED */

        /* create recording info box(es) */

        #ifdef APPL_PVR_ENABLED

        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.getItem         = MENU_InfobarGetRecItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
        {
            boxPars.pos.width = SCALER_GetCurrVal(INFO_RECICON_WIDTH);
            boxPars.bkgrType  = OSD_BOX_BKGR_NORMAL;

            if ( cnt == 0 )
            {
                boxPars.bkgrType |= OSD_CORNER_BOT_LEFT;
            }

            retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_RECICON1 + cnt,
                                   inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            boxPars.pos.x    += boxPars.pos.width
                              + SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
            boxPars.pos.width = SCALER_GetCurrVal(INFO_REC_WIDTH);
            boxPars.bkgrType  = OSD_BOX_BKGR_NORMAL;

            retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_RECINFO1 + cnt,
                                   inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            boxPars.pos.x +=  boxPars.pos.width
                           +  SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                           + (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) * 2);
        }

        /* create playback info box */

        boxPars.pos.width  = SCALER_GetCurrVal(INFO_PLAYICON_WIDTH);
        boxPars.bkgrType   = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem    = MENU_InfobarGetPlayItem;
        boxPars.getItemArg = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_PLAYICON,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x     +=  boxPars.pos.width
                           +  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        boxPars.pos.width  =  SCALER_GetCurrVal(INFO_PLAY_WIDTH);

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_PLAYINFO,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_PVR_ENABLED */

        /* ----- create symbol icon items ---------------------------------- */

        boxPars.pos.x           = -((int16_t)
                                   (SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                    MENU_INFOBAR_SYMB_X));
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                    SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                    SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.width       = SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_TITLE;
        boxPars.getItem         = MENU_InfobarGetSymbolItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        #ifdef APPL_SUBTT_ENABLED

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_ICONSUBT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x          += SCALER_GetCurrVal(MAIN_ICON_WIDTH);

        #endif /* APPL_SUBTT_ENABLED */

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_ICONTTX,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x          += SCALER_GetCurrVal(MAIN_ICON_WIDTH);

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_ICONCRYPT,
                                inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x          += SCALER_GetCurrVal(MAIN_ICON_WIDTH);

        retVal = FGS_BOX_Open (&boxPars, MENU_INFOBAR_ID_ICONWIDE,
                                inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- create progress bar --------------------------------------- */

        inst->progBarPos.x         = -((int16_t)
                                      (SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                                     + MENU_INFOBAR_PROG_X));
        inst->progBarPos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                    SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                    SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                     SCALER_GetCurrVal(INFO_PROGBAR_PAD);
        inst->progBarPos.width     = SCALER_GetCurrVal(INFO_PROG_WIDTH);
        inst->progBarPos.height    = SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                                   - (2 * SCALER_GetCurrVal(INFO_PROGBAR_PAD));
        inst->progBarPos.alignment = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->progBarPos);
        member.id            = MENU_INFOBAR_ID_PROGRESSBAR;
        member.isHidden      = FFALSE;
        member.show          = MENU_InfobarDrawProgBar;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pProgBar));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */
        //title separate line
        inst->itemPos.x         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->itemPos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->itemPos.width     = pnlPars.pos.width -
                                  (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        inst->itemPos.height    = pnlPars.pos.height -
                                  (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemPos);
        member.id            = MENU_INFOBAR_ID_REST;
        member.isHidden      = FFALSE;
        member.show          = MENU_InfobarDraw;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pItem));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */
    } while (0);

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);
        }

        inst->pnlHdl = NULL;
        return FFALSE;
    }

    return FTRUE;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
