/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_information.c
**
**              $Id: app_menu_information.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD information menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: loss of precision.
** - Info    715: Symbol not referenced.
** - Info    737: loss of sign in promotion from int to unsigned int
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_INFORMATION_ID_TITLE,
    MENU_INFORMATION_ID_INFOLIST,
    MENU_INFORMATION_ID_LIST,
    MENU_INFORMATION_ID_HELPLIST
};

enum
{
    MENU_INFORMATION_ID_FRONTEND = 0,

    #ifdef APPL_CI_ENABLED
    MENU_INFORMATION_ID_CI,
    #endif /* APPL_CI_ENABLED */

    MENU_INFORMATION_ID_MEMAVAIL,

    #ifdef APPL_HDD_ENABLED
    MENU_INFORMATION_ID_HDD,
    #endif /* APPL_HDD_ENABLED */

    #ifdef APPL_USB_ENABLED
    MENU_INFORMATION_ID_USB,
    #endif /* APPL_USB_ENABLED */

    MENU_INFORMATION_ID_INFOTXT,
    MENU_INFORMATION_ID_SYSTEM,
    MENU_INFORMATION_ID_TP,
    MENU_INFORMATION_ID_STRENGTH,
    MENU_INFORMATION_ID_QUALITY,
    MENU_INFORMATION_ID_STREAM,

    MENU_INFORMATION_ID_NUMS,

    MENU_INFORMATION_HELP_ID_STREAM
};

typedef struct
{
    FGS_LIST_INST_S listInst;

    FAPI_SYS_HandleT        pnlHdl;
    FAPI_SYS_HandleT        infoListHdl;
    FAPI_SYS_HandleT        listHdl;
    FAPI_SYS_HandleT        listHelpHdl;

    int8_t                  feCurr;
    SYS_DEVICE_INFO_S       devInf[SYS_DEV_NUMS];

}MENU_INFORMATION_INST_S;

FSTATIC MENU_INFORMATION_INST_S* menuInformationDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_InformationLeave      (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_InformationOnSelect   (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void    MENU_InformationUpdate     (void);
FSTATIC fbool_t MENU_InformationBuild      (FAPI_SYS_HandleT h);
FSTATIC void    MENU_InformationOnFocusChg (FAPI_SYS_HandleT h, uint8_t index,
                                            int32_t id);
FSTATIC void    MENU_InformationOnFocusSet (FAPI_SYS_HandleT h,
                                            fbool_t hasFocus);
FSTATIC void    MENU_InformationGetItem    (FAPI_SYS_HandleT h, int32_t id,
                                            uint32_t index,
                                            FGS_ITEMSTATE_E state,
                                            FGS_ITEM_S* item);
FSTATIC void    MENU_InformationGetDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                            uint32_t index,
                                            FGS_ITEMSTATE_E state,
                                            FGS_ITEM_S* item);
FSTATIC SYS_STRING_S MENU_InformationGetDevInfoString (
                                            MENU_INFORMATION_INST_S* inst,
                                            SYS_DEVICE_E device);

int32_t MENU_InformationInit (void)
{
    if ( menuInformationDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuInformationDat = UTILS_Malloc(sizeof(MENU_INFORMATION_INST_S));
    if ( menuInformationDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuInformationDat, 0, sizeof(MENU_INFORMATION_INST_S));

    return FAPI_OK;
}


fbool_t MENU_InformationEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_INFORMATION_INST_S* inst = menuInformationDat;
    uint32_t                 lastFocusItem;
    AV_STATUS_LIVE_S         liveStatus;

    if ( test == FTRUE ) return FTRUE;

    memset(&liveStatus, 0, sizeof(liveStatus));

    (void)AV_GetLiveStatus(&liveStatus);

    inst->feCurr = liveStatus.feIdx;

    if ( !MENU_InformationBuild((FAPI_SYS_HandleT)inst) )
    {
        return FFALSE;
    }

    (void)MAIN_SignalBoxSetFeIdx(&(appDat->sigBox), inst->feCurr);

    MENU_InformationUpdate(); /* perform first update */

    /* start timer for periodic updating of dynamic items */
    appDat->osdUpdateCb = MENU_InformationUpdate;

    (void)TIME_PeriodicTimerLaunch(OSD_PERIODTIMER_GENERAL, 2); // period: 200 ms

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = MENU_INFORMATION_ID_STREAM;
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_INFORMATION_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_INFORMATION);   /* set global focus on menu panel */
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_InformationLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}


FSTATIC void MENU_InformationOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_INFORMATION_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        /* start signal bar update */
        (void)MAIN_SignalBoxResume(&(appDat->sigBox));
    }
    else
    {
        /* halt signal bar update */
        (void)MAIN_SignalBoxPause(&(appDat->sigBox));
    }
}


FSTATIC void MENU_InformationOnFocusChg (FAPI_SYS_HandleT h, uint8_t index,
                                         int32_t id)
{
    MENU_INFORMATION_INST_S* inst = (MENU_INFORMATION_INST_S*)h;

    (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl,
                                         MENU_INFORMATION_HELP_ID_STREAM,
                                         FFALSE);

    switch ( id )
    {
        case MENU_INFORMATION_ID_STREAM:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl,
                                                MENU_INFORMATION_HELP_ID_STREAM,
                                                FTRUE);
            break;

        default:
            /* do nothing */
            break;
    }
}


FSTATIC fbool_t MENU_InformationLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_INFORMATION_INST_S* inst = (MENU_INFORMATION_INST_S*)h;

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MAIN_SignalBoxClose((&appDat->sigBox));

    appDat->osdUpdateCb = NULL;

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_INFORMATION);

    inst->pnlHdl = NULL;
    return FTRUE;
}


FSTATIC fbool_t MENU_InformationOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENUSTACK_ENTRY     newEntry;
    FAPI_SYS_HandleT    arg = NULL;

    switch ( itemId )
    {
        case MENU_INFORMATION_ID_STREAM:
            newEntry = MENU_StreamInfoEntry;
            break;

        default:
            return FFALSE;
    }

    /* store last focus */
    MENUSTACK_SetData((uint32_t)itemId);

    return MENUSTACK_Up(newEntry, arg);
}


FSTATIC void MENU_InformationGetDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item)
{
    uint32_t        strIdx;
    SYS_SERVLOC_S   servLoc;
    SLIST_ITEM_S    listItem;

    item->item.str.font       = MAIN_FONT;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->type                = FGS_ITEM_TYPE_STRING;
    item->item.str.lineHeight = 0;

    memset(&servLoc,   0, sizeof(servLoc));
    memset(&listItem,  0, sizeof(listItem));

    (void)AV_GetRequestedService(&servLoc);
    (void)SLIST_EntryGet(&servLoc,&listItem);

    switch ( id )
    {
        case MENU_INFORMATION_ID_FRONTEND:
            strIdx = STR_ENTRY_INFORMATION_FRONTEND;
            break;

        #ifdef APPL_CI_ENABLED

        case MENU_INFORMATION_ID_CI:
            strIdx = STR_ENTRY_INFORMATION_CI;
            break;

        #endif /* APPL_CI_ENABLED */

        case MENU_INFORMATION_ID_MEMAVAIL:
            strIdx = STR_ENTRY_INFORMATION_MEMAVAIL;
            break;

        #ifdef APPL_HDD_ENABLED

        case MENU_INFORMATION_ID_HDD:
            strIdx = STR_ENTRY_INFORMATION_HDD;
            break;

        #endif /* APPL_HDD_ENABLED */

        #ifdef APPL_USB_ENABLED

        case MENU_INFORMATION_ID_USB:
            strIdx = STR_ENTRY_INFORMATION_USB;
            break;

        #endif /* APPL_USB_ENABLED */

        case MENU_INFORMATION_ID_INFOTXT:
            strIdx = STR_ENTRY_INFORMATION_INFOTXT;
            break;

        case MENU_INFORMATION_ID_SYSTEM:

            switch ( listItem.network.type )
            {
                #ifdef APPL_FESAT_ENABLED
                case FE_SYSTEM_TYPE_SATELLITE:
                    strIdx = STR_ENTRY_INFORMATION_CURRSAT;
                    break;
                #endif /* APPL_FESAT_ENABLED */

                #ifdef APPL_FETERR_ENABLED
                case FE_SYSTEM_TYPE_TERRESTRIAL:
                    strIdx = STR_ENTRY_INFORMATION_CURRSYS;
                    break;
                #endif /* APPL_FETERR_ENABLED */

                #ifdef APPL_FECABLE_ENABLED
                case FE_SYSTEM_TYPE_CABLE:
                    strIdx = STR_ENTRY_INFORMATION_CURRSYS;
                    break;
                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FEDIRECT_ENABLED
                case FE_SYSTEM_TYPE_DIRECT:
                    strIdx = STR_ENTRY_INFORMATION_CURRSYS;
                    break;
                #endif /* APPL_FEDIRECT_ENABLED */

                default:
                    strIdx = STR_EMPTYSTRING;
                    break;
            }
            break;

        case MENU_INFORMATION_ID_TP:
            switch ( listItem.network.type )
            {
                #ifdef APPL_FESAT_ENABLED
                case FE_SYSTEM_TYPE_SATELLITE:
                    strIdx = STR_ENTRY_INFORMATION_CURRTP;
                    break;
                #endif /* APPL_FESAT_ENABLED */

                #ifdef APPL_FETERR_ENABLED
                case FE_SYSTEM_TYPE_TERRESTRIAL:
                    strIdx = STR_ENTRY_INFORMATION_CURRCH;
                    break;
                #endif /* APPL_FETERR_ENABLED */

                #ifdef APPL_FECABLE_ENABLED
                case FE_SYSTEM_TYPE_CABLE:
                    strIdx = STR_ENTRY_INFORMATION_CURRCH;
                    break;
                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FEDIRECT_ENABLED
                case FE_SYSTEM_TYPE_DIRECT:
                    strIdx = STR_ENTRY_INFORMATION_CURRCH;
                    break;
                #endif /* APPL_FEDIRECT_ENABLED */

                default:
                    strIdx = STR_EMPTYSTRING;
                    break;
            }
            break;

        case MENU_INFORMATION_ID_STRENGTH:
            strIdx = STR_ENTRY_INFORMATION_STRENGTH;
            break;

        case MENU_INFORMATION_ID_QUALITY:
            strIdx = STR_ENTRY_INFORMATION_QUALITY;
            break;

        case MENU_INFORMATION_ID_STREAM:
            strIdx = STR_ENTRY_INFORMATION_STREAM;
            break;

        case MENU_INFORMATION_HELP_ID_STREAM:
            strIdx = STR_ENTRY_OK;
            item->align = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
            item->item.str.font = SYMBOL_FONT;
            break;

        default:
            return;
    }

    item->item.str.chr = OSD_GetStr(strIdx);

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus;
                                                break;
        case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;
                                                break;
        case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled;
                                                break;
    }
}


FSTATIC void MENU_InformationGetItem (FAPI_SYS_HandleT h, int32_t id,
                                      uint32_t index, FGS_ITEMSTATE_E state,
                                      FGS_ITEM_S* item)
{
    #if (defined APPL_HDD_ENABLED) || (defined APPL_USB_ENABLED)
    MENU_INFORMATION_INST_S* inst = (MENU_INFORMATION_INST_S*)h;
    #endif /* (defined APPL_HDD_ENABLED) || (defined APPL_USB_ENABLED) */

    TPLIST_ITEM_S     tp;
    SYS_SERVLOC_S     servLoc;
    SLIST_ITEM_S      listItem;
    uint8_t           feInUse;
    char_t            strBuf[64];
    SYS_STRING_S      string;
    uint32_t          bufPos = 0;
    #ifdef APPL_CI_ENABLED
    RSRC_STATUS_S*    rsrcStatus;
    int32_t           ciInUse;
    int32_t           cnt;
    #endif
    #ifdef APPL_FESAT_ENABLED
    CFGSAT_SAT_S      satInf;
    #endif /* APPL_FESAT_ENABLED */

    memset(&servLoc,   0, sizeof(servLoc));
    memset(&listItem,  0, sizeof(listItem));

    strBuf[0] = 0;

    (void)AV_GetRequestedService(&servLoc);
    (void)SLIST_EntryGet(&servLoc,&listItem);

    (void)STR_BufClear(NULL);
    STR_Clear(&string);

    switch ( id )
    {
        case MENU_INFORMATION_ID_FRONTEND:
            feInUse = 0;
            if(appDat->numSatFe > 0)
            {
                strcat(strBuf,"SAT: %d");
                feInUse |= 0x1;
            }
            if(appDat->numTerrFe > 0)
            {
                if(strBuf[0] == '\0')
                    strcat(strBuf,"Terr: %d");
                else
                    strcat(strBuf,", Terr: %d");
                feInUse |= 0x2;
            }
            if(appDat->numCableFe > 0)
            {
                if(strBuf[0] == '\0')
                    strcat(strBuf,"Cable: %d");
                else
                    strcat(strBuf,", Cable: %d");
                feInUse |= 0x4;
            }


            switch(feInUse)
            {
                case 0: appDat->asciiBuf[0] = 0;                              break;
                case 1: sprintf(appDat->asciiBuf, strBuf,appDat->numSatFe);   break;
                case 2: sprintf(appDat->asciiBuf, strBuf,appDat->numTerrFe);  break;
                case 3: sprintf(appDat->asciiBuf, strBuf,appDat->numSatFe,
                                                         appDat->numTerrFe);  break;
                case 4: sprintf(appDat->asciiBuf, strBuf,appDat->numCableFe); break;
                case 5: sprintf(appDat->asciiBuf, strBuf,appDat->numSatFe,
                                                         appDat->numCableFe); break;
                case 6: sprintf(appDat->asciiBuf, strBuf,appDat->numCableFe,
                                                         appDat->numTerrFe);  break;
                case 7: sprintf(appDat->asciiBuf, strBuf,appDat->numSatFe,
                                                         appDat->numCableFe,
                                                         appDat->numTerrFe);  break;
                default: break;
            }

            string = STR_IntASCIIStr();
            break;

        #ifdef APPL_CI_ENABLED

        case MENU_INFORMATION_ID_CI:
            ciInUse = 0;
            rsrcStatus = RSRC_GetStatus();

            for ( cnt = 0; cnt < RSRC_TS_NUMS; cnt++ )
            {
                if(rsrcStatus->hw.ci[cnt].use != 0)
                {
                    ciInUse++;
                }
            }
            sprintf(appDat->asciiBuf, "%ld",ciInUse);
            string = STR_IntASCIIStr();
            break;

        #endif /* APPL_CI_ENABLED */

        #ifdef APPL_HDD_ENABLED

        case MENU_INFORMATION_ID_HDD:
            string = MENU_InformationGetDevInfoString (inst, SYS_DEV_HDD);
            break;

        #endif /* APPL_HDD_ENABLED */

        #ifdef APPL_USB_ENABLED

        case MENU_INFORMATION_ID_USB :
            string = MENU_InformationGetDevInfoString (inst, SYS_DEV_USB);
            break;

        #endif /* APPL_USB_ENABLED */

        case MENU_INFORMATION_ID_SYSTEM:

            switch ( listItem.network.type )
            {
                #ifdef APPL_FESAT_ENABLED

                case FE_SYSTEM_TYPE_SATELLITE:
                    if ( CFGSAT_GetDataByIndex (listItem.network.satIdx,
                                                &satInf) == FAPI_OK )
                    {
                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 STR_Build(satInf.nameCharSet,
                                                           satInf.name),
                                                 MAIN_LINEBUFCHARSET);

                        string = STR_IntLineStr();
                    }
                    break;

                #endif /* APPL_FESAT_ENABLED */

                #ifdef APPL_FETERR_ENABLED

                case FE_SYSTEM_TYPE_TERRESTRIAL:
                    sprintf(appDat->asciiBuf, "DVB-T");
                    string = STR_IntASCIIStr();
                    break;

                #endif /* APPL_FETERR_ENABLED */

                #ifdef APPL_FECABLE_ENABLED

                case FE_SYSTEM_TYPE_CABLE:
                    sprintf(appDat->asciiBuf, "DVB-C");
                    string = STR_IntASCIIStr();
                    break;

                #endif /* APPL_FEDIRECT_ENABLED */

                default:
                    break;
            }
            break;

        case MENU_INFORMATION_ID_TP :
            memset(&tp, 0, sizeof(tp));

            (void)TPLIST_GetDataByIndex (&(listItem.network), listItem.tpIdx,
                                         &tp);

            switch ( listItem.network.type )
            {
                #ifdef APPL_FESAT_ENABLED
                case FE_SYSTEM_TYPE_SATELLITE:
                    sprintf (appDat->asciiBuf, "%li ",
                             tp.pars.sat.frequency / 1000);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                            OSD_GetStr(STR_UNIT_MHZ),
                                             MAIN_LINEBUFCHARSET);

                    if ( tp.pars.sat.pol == FE_POL_HOR )
                    {
                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 STR_GetASCII(" H"),
                                                 MAIN_LINEBUFCHARSET);
                    }
                    else
                    {
                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 STR_GetASCII(" V"),
                                                 MAIN_LINEBUFCHARSET);
                    }
                    break;
                #endif /* APPL_FESAT_ENABLED */

                #ifdef APPL_FETERR_ENABLED
                case FE_SYSTEM_TYPE_TERRESTRIAL:
                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_Build(tp.nameCharSet, tp.name),
                                             MAIN_LINEBUFCHARSET);

                    sprintf (appDat->asciiBuf, ": %li ",
                             tp.pars.terr.frequency / 1000000);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             OSD_GetStr(STR_UNIT_MHZ),
                                             MAIN_LINEBUFCHARSET);
                    break;

                #endif /* APPL_FETERR_ENABLED */

                #ifdef APPL_FECABLE_ENABLED

                case FE_SYSTEM_TYPE_CABLE:
                    break;

                #endif /* APPL_FEDIRECT_ENABLED */

                default:
                    break;
            }

            string = STR_IntLineStr();
            break;

        default:
            return;
    }

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item->item.str.lineHeight = 0;
    item->item.str.chr        = string;
    item->item.str.font       = MAIN_FONT;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS:  item->item.str.col = appDat->txtAttr.colNonFocus;
                            break;
        case FGS_FOCUS:     item->item.str.col = appDat->txtAttr.colFocus;
                            break;
        case FGS_DISABLED:  item->item.str.col = appDat->txtAttr.colDisabled;
                            break;
    }
}


FSTATIC SYS_STRING_S MENU_InformationGetDevInfoString (
                                    MENU_INFORMATION_INST_S* inst,
                                    SYS_DEVICE_E device)
{
    SYS_DEVICE_INFO_S*  pDev   = inst->devInf + device;
    uint32_t            bufPos;
    SYS_STRING_S        string;

    if ( pDev->status.mountStatus == SYS_DEV_FS_MOUNTED )
    {
        bufPos = 0;

        sprintf (appDat->asciiBuf, "%li ",
                 pDev->freeSpace.kbytes / 1024);

        bufPos += STR_BufAppend (NULL, bufPos,
                                 STR_IntASCIIStr(),
                                 MAIN_LINEBUFCHARSET);

        bufPos += STR_BufAppend (NULL, bufPos,
                                 OSD_GetStr(STR_UNIT_MBYTE),
                                 MAIN_LINEBUFCHARSET);

        string = STR_IntLineStr();
    }
    else if ( pDev->status.avlStatus == SYS_DEVAVL_EXT_ACCESS )
    {
        string = OSD_GetStr(STR_ENTRY_INFORMATION_DEV_EXT_ACCESS);
    }
    else if (pDev->status.mountStatus == SYS_DEV_FS_UNFORMATTED)
    {
        string = OSD_GetStr(STR_ENTRY_INFORMATION_DEV_NOT_FORMAT);
    }
    else
    {
        string = OSD_GetStr(STR_ENTRY_INFORMATION_DEV_NOT_AVL);
    }

    return string;
}

FSTATIC void MENU_InformationUpdate (void)
{
    MENU_INFORMATION_INST_S*    inst = menuInformationDat;
    SYS_DEVICE_INFO_S           devInf;
    SYS_DEVICE_E                devCnt;
    fbool_t                     refresh;
    SYS_DEVICE_INFO_S*          pDev;

    if ( (inst         == NULL)
      || (inst->pnlHdl == NULL) ) return;

    (void)MAIN_SignalBoxUpdate(&(appDat->sigBox));

    for ( devCnt = (SYS_DEVICE_E)0; devCnt < SYS_DEV_NUMS; devCnt++ )
    {
        refresh = FFALSE;
        pDev    = inst->devInf + devCnt;

        if ( SYS_DeviceGetInfo(devCnt, &devInf) == FAPI_OK )
        {
            if ( (devInf.status.avlStatus   != pDev->status.avlStatus)
              || (devInf.status.mountStatus != pDev->status.mountStatus)
              || ((devInf.freeSpace.kbytes / 1024)
               != (pDev->freeSpace.kbytes / 1024)) )
            {
                refresh = FTRUE;
            }
        }
        else
        {
            memset(&devInf, 0, sizeof(devInf));
        }

        *pDev = devInf;

        if ( refresh )
        {
            switch ( devCnt )
            {
                #ifdef APPL_HDD_ENABLED

                case SYS_DEV_HDD:
                    (void)FGS_LIST_RefreshMember (inst->infoListHdl,
                                                  MENU_INFORMATION_ID_HDD);
                    break;

                #endif /* APPL_HDD_ENABLED */

                #ifdef APPL_USB_ENABLED

                case SYS_DEV_USB:
                    (void)FGS_LIST_RefreshMember (inst->infoListHdl,
                                                  MENU_INFORMATION_ID_USB);
                    break;

                #endif /* APPL_USB_ENABLED */

                default:
                    /* do nothing */
                    break;
            }
        }
    }
}


FSTATIC fbool_t MENU_InformationBuild(FAPI_SYS_HandleT h)
{
    MENU_INFORMATION_INST_S* inst = (MENU_INFORMATION_INST_S*)h;
    FGS_PANEL_OPEN_S         pnlPars;
    FGS_LIST_OPEN_S          listPars;
    FAPI_SYS_HandleT         itemHdl = NULL;
    FGS_ITEM_TXT_OPEN_S      txtPars;
    FGS_POS_S                itemPos;
    FGS_POS_S                valuePos;
    FGS_POS_S                descrPos;
    MAIN_SIGBOX_OPEN_S       sigPars;
    int32_t                  retVal = FAPI_OK;
    uint16_t                 helpWidth = 0;
    uint16_t                 stackedFrameWidth = 0;
    uint16_t                 innerHeight;
    uint32_t                 numLines    = MENU_INFORMATION_ID_NUMS;

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_OK),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth,
                                        1, &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_InformationOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_INFORMATION, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_INFORMATION_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_INFORMATION,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_InformationGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_InformationGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_InformationOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_InformationOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;

        inst->infoListHdl = MENU_CommonAddList (&listPars,
                                            MENU_INFORMATION_ID_INFOLIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->infoListHdl == NULL ) break;

        /* ----- menu lines -------------------------------------------------*/

        memset(&txtPars,  0, sizeof(txtPars));

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;

        /* --- Frontend --------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_ID_FRONTEND,
                                    inst->infoListHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- CI ------------------------------------------------*/

        #ifdef APPL_CI_ENABLED

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_ID_CI,
                                    inst->infoListHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_CI_ENABLED */

        /* --- Memory available ----------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_ID_MEMAVAIL,
                                    inst->infoListHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- HDD FS --------------------------------------------*/

        #ifdef APPL_HDD_ENABLED

        txtPars.descrPos.x      = 2*SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_ID_HDD,
                                    inst->infoListHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_HDD_ENABLED */

        /* --- USB FS ----------------------------------------------------*/

        #ifdef APPL_USB_ENABLED

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_ID_USB,
                                    inst->infoListHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_USB_ENABLED */

        /* --- Info text --------------------------------------------*/
        txtPars.descrPos        = descrPos;
        txtPars.pos = itemPos;
        txtPars.descrPos.width += valuePos.width;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_ID_INFOTXT,
                                    inst->infoListHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        txtPars.descrPos.width -= valuePos.width;

        /* --- Current satellite --------------------------------------------*/
        txtPars.descrPos.x      = 2*SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_ID_SYSTEM,
                                    inst->infoListHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Current TP --------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_ID_TP,
                                    inst->infoListHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Signal strength -------------------------------------------- */
        /* --- Signal quality --------------------------------------------- */
        /* --- Signal bar ------------------------------------------------- */

        txtPars.descrPos   = descrPos;

        memset(&sigPars,  0, sizeof(sigPars));

        sigPars.pnlHdl     = inst->pnlHdl;
        sigPars.pos.x      = itemPos.x + listPars.pos.x + SCALER_GetCurrVal(MAIN_RIGHT_PADDING)+ (SCALER_GetCurrVal(MAIN_LEFT_PADDING));
        sigPars.pos.y      = itemPos.y + listPars.pos.y + SCALER_GetCurrVal(LIST_LINE_PADDING);
        sigPars.descrWidth = ((uint16_t)(itemPos.width- SCALER_GetCurrVal(MAIN_LEFT_PADDING))/(2))-(SCALER_GetCurrVal(MAIN_LEFT_PADDING));
        sigPars.barWidth   = ((uint16_t)(itemPos.width- SCALER_GetCurrVal(MAIN_LEFT_PADDING)-SCALER_GetCurrVal(MAIN_RIGHT_PADDING))/(2));
        sigPars.feIdx      = -1;

        retVal = MAIN_SignalBoxOpen(&(appDat->sigBox), &sigPars);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += MAIN_SignalBoxGetHeight();
        itemPos.y += (2*SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* --- 2nd List -----------------------------------------------------*/

        listPars.pos.y  = listPars.pos.y + itemPos.y;
        listPars.pInst  = &(inst->listInst);

        retVal = FGS_LIST_Open (&listPars, MENU_INFORMATION_ID_LIST,
                                inst->pnlHdl, &(inst->listHdl));
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----------------------------------------------------------------- */

        itemPos.y = 0;

        /* --- Stream info ------------------------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_ID_STREAM,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Create help list ------------------------------------------ */

        listPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                   (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                   SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                   stackedFrameWidth;
        listPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(MAIN_WIDTH) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   stackedFrameWidth;
        listPars.pos.width       = helpWidth;
        listPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        listPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                 + SCALER_GetCurrVal(LIST_LINE_PADDING);
        listPars.pInst           = &(appDat->helpListInst);

        retVal = FGS_LIST_Open (&listPars, MENU_INFORMATION_ID_HELPLIST,
                                inst->pnlHdl, &(inst->listHelpHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        itemPos.x          = SCALER_GetCurrVal(HELP_LR_PADDING);
        itemPos.y          = 0;
        itemPos.width      = listPars.pos.width-2*SCALER_GetCurrVal(HELP_LR_PADDING);
        itemPos.height     = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        itemPos.alignment  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* position of list item descriptors in list entry */
        descrPos.x         = 0;
        descrPos.y         = 0;
        descrPos.width     = itemPos.width;
        descrPos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----- help lines ------------------------------------------------ */

        itemPos.y += (numLines-1) *
                     (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                    + SCALER_GetCurrVal(LIST_LINE_PADDING));

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_HELP;
        txtPars.descrPos           = descrPos;
        txtPars.pos                = itemPos;

        /* --- Stream help --------------------------------------------*/

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_INFORMATION_HELP_ID_STREAM,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)(
                                                 (SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2)
                                                + SCALER_GetCurrVal(HELP_TB_PADDING)
                                                + SCALER_GetCurrVal(HELP_LINE_HEIGHT)),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);

            inst->pnlHdl = NULL;
        }

        return FFALSE;
    }

    return FTRUE;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
