/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_jpeg.c
**
**              $Id: app_menu_jpeg.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       JPEG viewer menu.
**
** Detail description. [\ref app_smartgo_app_ref_jpeg]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
**
*****************************************************************************
*/
/*lint -save -e641 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define MENU_JPEG_MAXDIRDEPTH   8   //!< depth for recursive directroy search
#define MENU_JPEG_PATH_MAX      256 //!< Max. path length supported

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum
{
    MENU_JPEG_ID_TITLE,
    MENU_JPEG_ID_DIMENSION,
    MENU_JPEG_ID_DEGREES,
    MENU_JPEG_ID_ZOOMRATE,
    MENU_JPEG_ID_ZOOM,
    MENU_JPEG_ID_NEXT,
    MENU_JPEG_ID_PREVIOUS,
    MENU_JPEG_ID_ROTATE,
    MENU_JPEG_ID_ZOOMIN,
    MENU_JPEG_ID_ZOOMOUT,
    MENU_JPEG_ID_BACK,
    MENU_JPEG_ID_BACK_DESC,
    MENU_JPEG_ID_PLAY,
    MENU_JPEG_ID_PLAY_DESC,
    MENU_JPEG_ID_RED,
    MENU_JPEG_ID_GREEN,
    MENU_JPEG_ID_BLUE,
    MENU_JPEG_ID_INFO,
    MENU_JPEG_ID_INFO_DESC,
    MENU_JPEG_ID_CONTEXT
};


enum
{
    MENU_JPEG_PLAYMODE_LINEAR_THIS = 0,
    MENU_JPEG_PLAYMODE_SHUFFLE_THIS,
    MENU_JPEG_PLAYMODE_LINEAR_ALL,
    MENU_JPEG_PLAYMODE_SHUFFLE_ALL,
    MENU_JPEG_PLAYMODE_NONE
};


typedef struct
{
    FAPI_SYS_HandleT      pnlHdl;

    MENU_JPEG_PARAMS_S    params;

    FGS_PANEL_MEMBER_S*   ZoomBorderP;
    FGS_POS_S             zoomBorderPos;
    FGS_POS_S             zoomPos;

    JPEG_orientation_t    rot;
    uint16_t              zoom;
    uint16_t              xpos;
    uint16_t              ypos;

    uint16_t              picSizeX;     // 0: unknown
    uint16_t              picSizeY;     // 0: unknown

    fbool_t               hidden;

    uint32_t              rootcount;
    uint32_t              allcount;

    uint16_t              playmode;
    uint16_t              playtime;

    uint32_t              random;
    uint32_t              currentfile;

    char_t                filePath[MENU_JPEG_PATH_MAX];
} MENU_JPEG_INST_S;


FSTATIC MENU_JPEG_INST_S* menuJPEGDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_JPEGLeave           (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_JPEGKeys            (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_JPEGBuild           (FAPI_SYS_HandleT h, fbool_t isjpeg);
FSTATIC int32_t MENU_JPEGPaintZoom       (FAPI_SYS_HandleT h);
FSTATIC void    MENU_JPEGUpdateImage     (MENU_JPEG_INST_S* inst);
FSTATIC void    MENU_JPEGContextGetItem  (FAPI_SYS_HandleT h, int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item);
FSTATIC fbool_t MENU_JPEGContextSelect   (FAPI_SYS_HandleT h, int32_t id,
                                          uint16_t index);
FSTATIC void    MENU_JPEGCountImages     (const char_t* path, fbool_t root,
                                          uint32_t* countroot, uint32_t* count,
                                          uint32_t maxDirDepth);
FSTATIC void    MENU_JPEGFindCurrentImage(MENU_JPEG_INST_S* inst);
FSTATIC fbool_t MENU_JPEGFindImage       (MENU_JPEG_INST_S* inst,
                                          fbool_t recurse);
FSTATIC void    MENU_JPEGFindRecursive   (MENU_JPEG_INST_S* inst,
                                          const char_t* pathPtr,
                                          int32_t* countP,
                                          uint32_t maxDirDepth);
FSTATIC void    MENU_JPEGUpdateZoomBox   (MENU_JPEG_INST_S* inst);
FSTATIC FINLINE fbool_t MENU_JPEGFileIsJpeg  (const char_t* fileName);
FSTATIC FINLINE fbool_t MENU_JPEGZoomIsSupported (const char_t* fileName);
FSTATIC void    MENU_JPEGOnFocusSet      (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_JPEGGetItem         (FAPI_SYS_HandleT h, int32_t id,
                                          uint32_t index, FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item);
FSTATIC void    MENU_JPEGSwitchMenu      (MENU_JPEG_INST_S* inst,
                                          fbool_t zoomIsSupported);

/*!
*******************************************************************************
**
** \brief  Initial function for the OSD Service list Menu
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_JPEGInit(void)
{
    if(menuJPEGDat != NULL)
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    menuJPEGDat= UTILS_Malloc(sizeof(MENU_JPEG_INST_S));
    if(menuJPEGDat == NULL)
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(menuJPEGDat, 0, sizeof(MENU_JPEG_INST_S));

    menuJPEGDat->random= 0xA427;
    menuJPEGDat->playmode= MENU_JPEG_PLAYMODE_NONE;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function.
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_JPEGEntry(FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_JPEG_INST_S*    inst   = menuJPEGDat;
    FGS_POS_S            framePos;
    JPEG_SCREEN_PARAMS_S screenParams;
    SYS_DEVICE_E         device;

    if ( h == NULL )
    {
        return FFALSE;
    }

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    inst->params = *((MENU_JPEG_PARAMS_S*)h);

    inst->rot      = ROT_0;
    inst->zoom     = 100;
    inst->xpos     = 50;
    inst->ypos     = 50;
    inst->playmode = MENU_JPEG_PLAYMODE_NONE;

    inst->picSizeX = 0;
    inst->picSizeY = 0;

    MENU_CommonAppendpath (inst->params.filename, inst->params.path,
                           inst->filePath, FFALSE, MENU_JPEG_PATH_MAX);

    #ifdef JPEG_DBG
    FAPI_SYS_PRINT_DEBUG(4,"[ MENU_JPEGEntry] file path for JPEG is %s\n", inst->filePath);
    #endif

    // TODO: replace MENU_JPEGFileIsJpeg by MENU_JPEGFileIsDecodable !!

    (void)MENU_JPEGBuild((FAPI_SYS_HandleT)inst,
                            MENU_JPEGZoomIsSupported(inst->params.filename));

    (void)MENUSTACK_FocusClaim ((int32_t)MENUID_JPEG);
    (void)FGS_PANEL_Show       (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup ((MENUSTACK_CLEANUP)MENU_JPEGLeave,
                          (FAPI_SYS_HandleT)inst);

    /* configure JPEG module */
    memset(&framePos, 0, sizeof(framePos));

    FGS_GetFrameDimensions(&framePos);

    screenParams.pos.left  = (uint16_t)framePos.x;
    screenParams.pos.top   = (uint16_t)framePos.y;
    screenParams.pos.hsize = (uint16_t)framePos.width;
    screenParams.pos.vsize = (uint16_t)framePos.height;

    (void)JPEG_Open (&screenParams);

    /* show first image */
    MENU_JPEGUpdateImage (inst);

    inst->rootcount = 0;
    inst->allcount  = 0;

    if ( SYS_DeviceGetByPath (inst->params.path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    MENU_JPEGCountImages(inst->params.path, FTRUE, &inst->rootcount,
                         &inst->allcount, MENU_JPEG_MAXDIRDEPTH);
    MENU_JPEGFindCurrentImage(inst);

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_NEXT);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_PREVIOUS);

    return FTRUE;
}

void MENU_JPEGHandleEvent (SYS_EVTTYPE_E type, const void* data)
{
    MENU_JPEG_INST_S*     inst = menuJPEGDat;

    if ( inst->pnlHdl == NULL ) return;

    switch ( type )
    {
        case EVT_JPEG_FILEINFO:
            inst->picSizeX = ((JPEG_EVT_INFO_S*)data)->sizeX;
            inst->picSizeY = ((JPEG_EVT_INFO_S*)data)->sizeY;

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_DIMENSION);

            break;
        case EVT_JPEG_DECODEINFO:
            if (((JPEG_EVT_DECODEINFO_S*)data)->infoType != JPEG_DECODEINFO_OK)
            {
                MENU_CommonMsgShow(STR_MSG_PIC_NOT_SUPPORT, FFALSE);
            }
            break;
        default:
            break;
    }
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_JPEGOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    MAIN_HandleExitKeys(hasFocus);

    if(hasFocus)
    {
        FGS_SetAppKeyHandler(FGS_KEY_UP,     MENU_JPEGKeys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_DOWN,   MENU_JPEGKeys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_LEFT,   MENU_JPEGKeys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_RIGHT,  MENU_JPEGKeys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_RED,    MENU_JPEGKeys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_BLUE,   MENU_JPEGKeys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_GREEN,  MENU_JPEGKeys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_INFO,   MENU_JPEGKeys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_PLAY,   MENU_JPEGKeys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_STOP,   MENU_JPEGKeys, h, FTRUE, FTRUE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_UP);
        FGS_ClearAppKeyHandler(FGS_KEY_DOWN);
        FGS_ClearAppKeyHandler(FGS_KEY_LEFT);
        FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_BLUE);
        FGS_ClearAppKeyHandler(FGS_KEY_GREEN);
        FGS_ClearAppKeyHandler(FGS_KEY_INFO);
        FGS_ClearAppKeyHandler(FGS_KEY_PLAY);
        FGS_ClearAppKeyHandler(FGS_KEY_STOP);
    }
}


FSTATIC fbool_t MENU_JPEGLeave(FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_JPEG_INST_S*       inst= (MENU_JPEG_INST_S*)h;
    force = force;
    (void)JPEG_Close();

    /* hide the unformatted message if necessary */
    (void)MAIN_MsgBoxHide(&(appDat->msgBox));

    //disable JPEG slide show
    inst->playmode= MENU_JPEG_PLAYMODE_NONE;

    if(inst->pnlHdl != NULL)
        (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_JPEG);

    inst->pnlHdl= NULL;
    return FTRUE;
}

FSTATIC void MENU_JPEGGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_JPEG_INST_S*   inst = (MENU_JPEG_INST_S*)h;
    uint16_t            i;
    uint16_t            filenamepos;

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = (int32_t)MAIN_FONT;
    item->item.str.col        = appDat->txtAttr.colNonFocus;
    item->item.str.lineHeight = 0;

    index = index;
    state = state;

    switch ( id )
    {
        default:
        case MENU_JPEG_ID_TITLE:
            filenamepos = 0;

            for ( i = 0; i < MENU_JPEG_PATH_MAX; i++ )
            {
                if ( inst->filePath[i] == 0 )
                {
                    break;
                }

                if ( inst->filePath[i] == MENU_COMMON_PATH_SEPARATOR )
                {
                    filenamepos= i+1;
                }
            }

            item->item.str.font = (int32_t)TITLE_FONT;
            item->item.str.col  = OSD_COL_TITLE_TEXT;
            item->item.str.chr  = STR_GetASCII (inst->filePath + filenamepos);
            break;

        case MENU_JPEG_ID_DIMENSION:
            item->item.str.font = (int32_t)TITLE_FONT;
            item->item.str.col  = OSD_COL_TITLE_TEXT;

            if ( inst->picSizeX == 0 )
            {
                item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            }
            else
            {
                sprintf(appDat->asciiBuf, "%dx%d", inst->picSizeX,
                                                   inst->picSizeY);

                item->item.str.chr = STR_IntASCIIStr();
            }
            break;

        case MENU_JPEG_ID_ZOOMRATE:
            item->item.str.font = (int32_t)TITLE_FONT;
            item->item.str.col  = OSD_COL_TITLE_TEXT;

            sprintf(appDat->asciiBuf, "%d%%", inst->zoom);
            item->item.str.chr = STR_IntASCIIStr();
            break;

        case MENU_JPEG_ID_DEGREES:
            item->item.str.font = (int32_t)TITLE_FONT;
            item->item.str.col  = OSD_COL_TITLE_TEXT;

            switch ( inst->rot )
            {
                case ROT_0:
                    item->item.str.chr = STR_GetASCII("0");
                    break;

                case ROT_90:
                    item->item.str.chr = STR_GetASCII("90");
                    break;

                case ROT_180:
                    item->item.str.chr = STR_GetASCII("180");
                    break;

                case ROT_270:
                    item->item.str.chr = STR_GetASCII("270");
                    break;
            }
            break;

        case MENU_JPEG_ID_NEXT:
            if ( (inst->zoom != 100)                         ||
                 (inst->playmode != MENU_JPEG_PLAYMODE_NONE) ||
                 (inst->currentfile >= inst->rootcount) )
            {
                item->item.str.col    = OSD_COL_TEXT_DISABLED;
            }

            item->item.str.chr        = OSD_GetStr(STR_ENTRY_JPEG_NEXT);
            break;

        case MENU_JPEG_ID_PREVIOUS:
            if ( (inst->zoom != 100)                         ||
                 (inst->playmode != MENU_JPEG_PLAYMODE_NONE) ||
                 (inst->currentfile == 1) )
            {
                item->item.str.col    = OSD_COL_TEXT_DISABLED;
            }
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_JPEG_PREVIOUS);
            break;

        case MENU_JPEG_ID_BACK:
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_JPEG_MENU);
            break;

        case MENU_JPEG_ID_BACK_DESC:
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_JPEG_VIEWER);
            break;

        case MENU_JPEG_ID_ROTATE:
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_JPEG_ROTATE);
            break;

        case MENU_JPEG_ID_ZOOMIN:
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_JPEG_ZOOMIN);
            break;

        case MENU_JPEG_ID_ZOOMOUT:
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_JPEG_ZOOMOUT);
            break;

        case MENU_JPEG_ID_INFO:
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_JPEG_INFO);
            break;

        case MENU_JPEG_ID_INFO_DESC:
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_JPEG_HIDESHOW);
            break;

        case MENU_JPEG_ID_PLAY:
            if(inst->playmode != MENU_JPEG_PLAYMODE_NONE)
                item->item.str.chr    = OSD_GetStr(STR_ENTRY_JPEG_STOP);
            else
                item->item.str.chr    = OSD_GetStr(STR_ENTRY_JPEG_PLAY);
            break;

        case MENU_JPEG_ID_PLAY_DESC:
            if ( inst->playmode != MENU_JPEG_PLAYMODE_NONE )
            {
                item->item.str.chr = OSD_GetStr(
                                     STR_ENTRY_MEDIA_HELP_PIC_SLIDESHOW_STOP);
            }
            else
            {
                item->item.str.chr = OSD_GetStr(
                                     STR_ENTRY_MEDIA_HELP_PIC_SLIDESHOW_START);
            }
            break;

        case MENU_JPEG_ID_RED:
            MENU_COMMON_ITEM_BUTTON(item, ICON_BTNREDSMALL);
            break;

        case MENU_JPEG_ID_GREEN:
            MENU_COMMON_ITEM_BUTTON(item, ICON_BTNGREENSMALL);
            break;

        case MENU_JPEG_ID_BLUE:
            MENU_COMMON_ITEM_BUTTON(item, ICON_BTNBLUESMALL);
            break;
    }
}


FSTATIC fbool_t MENU_JPEGKeys(FAPI_SYS_HandleT h)
{
    MENU_JPEG_INST_S*       inst= (MENU_JPEG_INST_S*)h;
    fbool_t                 keyUsed= FTRUE;
    fbool_t                 update= FTRUE;
    int16_t                 temp;
    MENU_SELECTLIST_OPEN_S  selectLst;

    switch(FGS_GetCurrentKeyCode())
    {
        case FGS_KEY_LEFT:
            if(inst->playmode==MENU_JPEG_PLAYMODE_NONE)
            {
                if(inst->zoom == 100)
                {   if(inst->currentfile > 1)
                    {
                        inst->currentfile--;
                        (void)MENU_JPEGFindImage(inst, FFALSE);
                        (void)FGS_PANEL_RefreshMember(
                                            inst->pnlHdl, MENU_JPEG_ID_TITLE);
                        (void)FGS_PANEL_RefreshMember(
                                            inst->pnlHdl, MENU_JPEG_ID_PREVIOUS);
                        (void)FGS_PANEL_RefreshMember(
                                            inst->pnlHdl, MENU_JPEG_ID_NEXT);
                    }
                    else
                    {
                        keyUsed= FFALSE;
                        update = FFALSE;
                    }
                }
                else
                {   if(inst->xpos == 0)
                    {   keyUsed= FFALSE;
                        update = FFALSE;
                    }
                    else
                    {   temp= (int16_t)(inst->xpos - (10000 / inst->zoom));
                        if(temp < 0)
                            temp= 0;
                        inst->xpos= (uint16_t)temp;
                        MENU_JPEGUpdateZoomBox(inst);
                    }
                }
            }
            else
            {
                update = FFALSE;
            }
            break;

        case FGS_KEY_RIGHT:
            if(inst->playmode==MENU_JPEG_PLAYMODE_NONE)
            {
                if(inst->zoom == 100)
                {
                    if(inst->currentfile < inst->rootcount )
                    {
                        inst->currentfile++;
                        (void)MENU_JPEGFindImage(inst, FFALSE);
                        (void)FGS_PANEL_RefreshMember(
                                            inst->pnlHdl, MENU_JPEG_ID_TITLE);
                        (void)FGS_PANEL_RefreshMember(
                                            inst->pnlHdl, MENU_JPEG_ID_PREVIOUS);
                        (void)FGS_PANEL_RefreshMember(
                                            inst->pnlHdl, MENU_JPEG_ID_NEXT);
                    }
                    else
                    {   keyUsed= FFALSE;
                        update = FFALSE;
                    }
                }
                else
                {   if(inst->xpos == 100)
                    {   keyUsed= FFALSE;
                        update = FFALSE;
                    }
                    else
                    {   temp= (int16_t)(inst->xpos + (10000 / inst->zoom));
                        if(temp > 100)
                            temp= 100;
                        inst->xpos= (uint16_t)temp;
                        MENU_JPEGUpdateZoomBox(inst);
                    }
                }
            }
            else
            {
                update = FFALSE;
            }
            break;

        case FGS_KEY_UP:
            if((inst->zoom == 100) || (inst->ypos == 0))
            {   keyUsed= FFALSE;
                update = FFALSE;
            }
            else
            {   temp= (int16_t)(inst->ypos - (10000 / inst->zoom));
                if(temp < 0)
                    temp= 0;
                inst->ypos= (uint16_t)temp;
                MENU_JPEGUpdateZoomBox(inst);
            }
            break;

        case FGS_KEY_DOWN:
            if((inst->zoom == 100) || (inst->ypos == 100))
            {   keyUsed= FFALSE;
                update = FFALSE;
            }
            else
            {   temp= (int16_t)(inst->ypos + (10000 / inst->zoom));
                if(temp > 100)
                    temp= 100;
                inst->ypos= (uint16_t)temp;
                MENU_JPEGUpdateZoomBox(inst);
            }
            break;

        case FGS_KEY_RED:
            if (!MENU_JPEGZoomIsSupported(inst->filePath))
            {
                keyUsed= FFALSE;
                update = FFALSE;
                break;
            }
            if(inst->zoom < 800)
            {   inst->zoom= inst->zoom + 100;
                (void)FGS_PANEL_RefreshMember(
                                    inst->pnlHdl, MENU_JPEG_ID_ZOOMRATE);
                      MENU_JPEGUpdateZoomBox(inst);
                (void)FGS_PANEL_RefreshMember(
                                    inst->pnlHdl, MENU_JPEG_ID_ZOOM);
                (void)FGS_PANEL_RefreshMember(
                                    inst->pnlHdl, MENU_JPEG_ID_NEXT);
                (void)FGS_PANEL_RefreshMember(
                                    inst->pnlHdl, MENU_JPEG_ID_PREVIOUS);
            }
            else
            {   keyUsed= FFALSE;
                update = FFALSE;
            }
            break;

        case FGS_KEY_GREEN:
            if (!MENU_JPEGZoomIsSupported(inst->filePath))
            {
                keyUsed= FFALSE;
                update = FFALSE;
                break;
            }
            if(inst->zoom > 100)
            {   inst->zoom= inst->zoom - 100;
                if(inst->zoom == 100)
                {   inst->xpos= 50;
                    inst->ypos= 50;
                }
                (void)FGS_PANEL_RefreshMember(
                                    inst->pnlHdl, MENU_JPEG_ID_ZOOMRATE);
                      MENU_JPEGUpdateZoomBox(inst);
                (void)FGS_PANEL_RefreshMember(
                                    inst->pnlHdl, MENU_JPEG_ID_ZOOM);
                (void)FGS_PANEL_RefreshMember(
                                    inst->pnlHdl, MENU_JPEG_ID_NEXT);
                (void)FGS_PANEL_RefreshMember(
                                    inst->pnlHdl, MENU_JPEG_ID_PREVIOUS);
            }
            else
            {   keyUsed= FFALSE;
                update = FFALSE;
            }
            break;

        case FGS_KEY_BLUE:
            if (!MENU_JPEGZoomIsSupported(inst->filePath))
            {
                keyUsed= FFALSE;
                update = FFALSE;
                break;
            }
            switch ( inst->rot )
            {
                case ROT_0:   inst->rot= ROT_90;  break;
                case ROT_90:  inst->rot= ROT_180; break;
                case ROT_180: inst->rot= ROT_270; break;
                default:
                case ROT_270: inst->rot= ROT_0;  break;
            }

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_DEGREES);
            break;

        case FGS_KEY_INFO:
            if(inst->hidden)
            {
                (void)FGS_PANEL_Show(inst->pnlHdl, FFALSE);
                inst->hidden= FFALSE;
            }
            else
            {   (void)FGS_PANEL_Hide(inst->pnlHdl);
                inst->hidden= FTRUE;
            }
            update= FFALSE;
            break;

        case FGS_KEY_PLAY:
            if (!MENU_JPEGFileIsJpeg(inst->filePath))
            {
                update= FFALSE;
                break;
            }
            if(inst->playmode == MENU_JPEG_PLAYMODE_NONE)
            {
                memset(&selectLst, 0, sizeof(selectLst));

                selectLst.pos.width     = SCALER_GetCurrVal(MAIN_WIDTH);
                selectLst.pos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT)*4;
                selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT|FGS_ALIGN_VER_TOP;
                selectLst.id            = MENU_JPEG_ID_CONTEXT;
                selectLst.numVisLines   = 4;
                selectLst.numItems      = 4;
                selectLst.index         = 0;
                selectLst.getItem       = MENU_JPEGContextGetItem;
                selectLst.getItemArg    = h;
                selectLst.onSelect      = (FGS_LNAVI_SELECT_CB)
                                                MENU_JPEGContextSelect;
                selectLst.onSelectArg   = h;

                MENU_CommonSetPanelPos(&selectLst.pos);
                (void)MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
            }
            else
                keyUsed= FFALSE;

            update= FFALSE;
            break;

        case FGS_KEY_STOP:
            if (!MENU_JPEGFileIsJpeg(inst->filePath))
            {
                update= FFALSE;
                break;
            }
            if(inst->playmode == MENU_JPEG_PLAYMODE_NONE)
                keyUsed= FFALSE;
            else
            {
                inst->playmode= MENU_JPEG_PLAYMODE_NONE;
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_PLAY);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                        MENU_JPEG_ID_PLAY_DESC);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_NEXT);
                (void)FGS_PANEL_RefreshMember(
                                        inst->pnlHdl, MENU_JPEG_ID_PREVIOUS);
            }
            update= FFALSE;
            break;

        default:
            keyUsed= FFALSE;
            update = FFALSE;
            break;
    }

    if(update)
    {
        MENU_JPEGSwitchMenu(inst, MENU_JPEGZoomIsSupported(inst->filePath));
        MENU_JPEGUpdateImage(inst);
    }

    return keyUsed;
}


FSTATIC fbool_t MENU_JPEGBuild(FAPI_SYS_HandleT h, fbool_t isjpeg)
{
    MENU_JPEG_INST_S*     inst= (MENU_JPEG_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_BOX_OPEN_S        boxPars;
    int32_t               retVal;
    uint16_t              menuHeight;
    uint32_t              numLines= 3;
    FGS_PANEL_MEMBER_S    member;
    FAPI_SYS_HandleT      itemHdl;
    int32_t               jpegTitleWidth;
    int32_t               ii;

    memset(&boxPars,  0, sizeof(boxPars));
    memset(&member,   0, sizeof(member));

    menuHeight= (2*SCALER_GetCurrVal(OUTER_FRAME_ALL)) +
                (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                 SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                 SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                 SCALER_GetCurrVal(MAIN_BOT_PADDING) +
                 (numLines * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                 ((numLines- 1) * SCALER_GetCurrVal(LIST_LINE_PADDING));

    for(ii=1; ii>0;ii--)
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        pnlPars.pos.width       = SCALER_GetCurrVal(JPEG_WIDTH) +
                                  (2*SCALER_GetCurrVal(OUTER_FRAME_ALL)) +
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        pnlPars.pos.height      = menuHeight;
        pnlPars.pos.y           = (-(int16_t)(SCALER_GetCurrVal(JPEG_POS_Y) +
                                    pnlPars.pos.height));
        pnlPars.pos.alignment   = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_BOTTOM;
        pnlPars.bkgrType        = OSD_PNLTYPE_MENUWTITLE;
        pnlPars.onFocusSet      = MENU_JPEGOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;
        pnlPars.pInst           = &(appDat->pnlInst);

        retVal= FGS_PANEL_Open(&pnlPars, (int32_t)MENUID_JPEG, &(inst->pnlHdl));
        if(retVal != FAPI_OK)
            break;

        /* ------Title --------------------------------------------- */
        jpegTitleWidth = (int32_t)(SCALER_GetCurrVal(JPEG_WIDTH) -
                                (2*(SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  SCALER_GetCurrVal(MAIN_LEFT_PADDING))));
        boxPars.pos.x           = (int16_t)(SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  SCALER_GetCurrVal(MAIN_LEFT_PADDING));
        boxPars.pos.y           = (int16_t)
                            (((SCALER_GetCurrVal(OUTER_FRAME_ALL)/ 2)&~1)+1);
        boxPars.pos.width       = (uint16_t)(jpegTitleWidth / 3);
        boxPars.pos.height      = SCALER_GetCurrVal(TITLE_LINE_HEIGHT)-3;
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_TITLE;
        boxPars.getItem         = (FGS_GETITEM_CB)MENU_JPEGGetItem;
        boxPars.getItemArg      = h;

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_TITLE,
                                                inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.x           = boxPars.pos.x+ (int16_t)boxPars.pos.width;
        boxPars.pos.width       = (uint16_t)(jpegTitleWidth/3);

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_DIMENSION,
                                                    inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.x           = boxPars.pos.x+ (int16_t)boxPars.pos.width;
        boxPars.pos.width       = (uint16_t)(jpegTitleWidth/6);

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_DEGREES,
                                                     inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.x           = boxPars.pos.x+ (int16_t)boxPars.pos.width;
        boxPars.pos.width       = (uint16_t)(jpegTitleWidth/6);

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_ZOOMRATE,
                                                    inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        /* ------Zoom box --------------------------------------------- */
        inst->zoomBorderPos.x         = (int16_t)(SCALER_GetCurrVal(JPEG_WIDTH)
                                - (SCALER_GetCurrVal(LIST_LINE_HEIGHT)*3));
        inst->zoomBorderPos.y        = (int16_t)
                               (SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                               (2*SCALER_GetCurrVal(TITLE_TB_PADDING))+
                                SCALER_GetCurrVal(TITLE_LINE_HEIGHT));
        inst->zoomBorderPos.width     = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+
                                SCALER_GetCurrVal(LIST_LINE_PADDING))*3;
        inst->zoomBorderPos.height    = inst->zoomBorderPos.width;
        inst->zoomBorderPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h      = (FAPI_SYS_HandleT)inst;
        member.pos    = &(inst->zoomBorderPos);
        member.id     = MENU_JPEG_ID_ZOOM;
        member.show   = MENU_JPEGPaintZoom;

        inst->zoomPos.x     = 2;
        inst->zoomPos.y     = 2;
        inst->zoomPos.width = inst->zoomBorderPos.width - 4;
        inst->zoomPos.height= inst->zoomPos.width;

        retVal= FGS_PANEL_AddMember(inst->pnlHdl,&member,&(inst->ZoomBorderP));
        if(retVal != FAPI_OK)
            break;

        /* -----Info objects ------------------------------------------ */
        /* 1st column*/
        boxPars.pos.x          = (int16_t)
                               (SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                SCALER_GetCurrVal(OUTER_FRAME_ALL));
        boxPars.pos.y          = (int16_t)
                               (SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                               (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                SCALER_GetCurrVal(TITLE_LINE_HEIGHT));
        boxPars.pos.width       = (uint16_t)
                                ((inst->zoomBorderPos.x - boxPars.pos.x) / 3)/4;
        boxPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_PLAY,
                                                inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = (boxPars.pos.y +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal = FGS_BOX_Open(&boxPars, MENU_JPEG_ID_INFO,
                                                inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = (boxPars.pos.y +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_BACK,
                                                    inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        // description of 1st column
        boxPars.pos.x          += (int16_t)boxPars.pos.width;
        boxPars.pos.y           = (int16_t)
                               (SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                               (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                SCALER_GetCurrVal(TITLE_LINE_HEIGHT));
        boxPars.pos.width       = boxPars.pos.width*3;
        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_PLAY_DESC,
                                                inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = (boxPars.pos.y +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal = FGS_BOX_Open(&boxPars, MENU_JPEG_ID_INFO_DESC,
                                                inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = (boxPars.pos.y +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_BACK_DESC,
                                                    inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        /* 2nd column */
        boxPars.pos.x           = boxPars.pos.x +(int16_t)boxPars.pos.width;
        boxPars.pos.y           = (int16_t)
                                (SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                 SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                 SCALER_GetCurrVal(TITLE_LINE_HEIGHT));
        boxPars.pos.width       = (uint16_t)
                                ((inst->zoomBorderPos.x - boxPars.pos.x) / 3);
        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_NEXT,
                                                    inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = (boxPars.pos.y +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_PREVIOUS,
                                                    inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;


        boxPars.pos.x           = (boxPars.pos.x +
                                (int16_t)boxPars.pos.width +
                                (int16_t)SCALER_GetCurrVal(MAIN_ICON_WIDTH));
        boxPars.pos.y           = (int16_t)
                                (SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                 SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                 SCALER_GetCurrVal(TITLE_LINE_HEIGHT));
        boxPars.pos.width       = boxPars.pos.width -
                                 SCALER_GetCurrVal(MAIN_ICON_WIDTH);

        /* 3rd column */
        /* On/off of this column depends on MENU_JPEGSwitchMenu */

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_ZOOMIN,
                                            inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = (boxPars.pos.y +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_ZOOMOUT,
                                                    inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = (boxPars.pos.y +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_ROTATE,
                                                    inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;


        boxPars.pos.x           = boxPars.pos.x -
                                (int16_t)SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        boxPars.pos.y           = (int16_t)
                                (SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                 SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                 SCALER_GetCurrVal(TITLE_LINE_HEIGHT));
        boxPars.pos.width       = SCALER_GetCurrVal(MAIN_ICON_WIDTH);

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_RED,
                                                inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = (boxPars.pos.y +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_GREEN,
                                                inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = (boxPars.pos.y +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal= FGS_BOX_Open(&boxPars, MENU_JPEG_ID_BLUE,
                                                inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        MENU_JPEGSwitchMenu(inst, isjpeg);
    }

    if(retVal != FAPI_OK)
    {
        if(inst->pnlHdl != NULL)
            (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}


FSTATIC int32_t MENU_JPEGPaintZoom(FAPI_SYS_HandleT h)
{
    MENU_JPEG_INST_S*   inst= (MENU_JPEG_INST_S*)h;
    uint32_t                 x1, x2, y1, y2;

    x1= (uint32_t)inst->ZoomBorderP->abspos.x;
    y1= (uint32_t)inst->ZoomBorderP->abspos.y;
    x2= x1 + (uint32_t)inst->zoomBorderPos.width;
    y2= y1 + (uint32_t)inst->zoomBorderPos.height;
    OSD_DrawFilledRectangle(x1, y1, x2, y2,
                            OSD_GetSchemeColor(OSD_SCHEMECOL_HELP_BKGR));

    x1= x1 + (uint32_t)inst->zoomPos.x;
    y1= y1 + (uint32_t)inst->zoomPos.y;
    x2= x1 + inst->zoomPos.width;
    y2= y1 + inst->zoomPos.height;
    OSD_DrawFilledRectangle(x1, y1, x2, y2, OSD_COL_TEXT);

    return FAPI_OK;
}


FSTATIC fbool_t MENU_JPEGFindImage(MENU_JPEG_INST_S* inst, fbool_t recurse)
{
    int32_t             count = 0;//-1;
    uint32_t             maxDirDepth;
    SYS_DEVICE_E    device;

    if ( recurse ) maxDirDepth = MENU_JPEG_MAXDIRDEPTH;
    else           maxDirDepth = 0;

    if ( SYS_DeviceGetByPath (inst->params.path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    MENU_JPEGFindRecursive (inst, inst->params.path, &count, maxDirDepth);

    return (fbool_t)(count == (int32_t)inst->currentfile);
}

/**
**************************************************************************
**\brief Find JPEG pictures recursively
**
** This function searchs JPEG pictures through directroy and updates
**   the JPEG picture paths until the depth is reached. The maximum
**   depth of directory depond on MENU_JPEG_MAXDIRDEPTH.
**
** \param inst handler to the local data structure
** \param pathPtr a path pointer, form which the function starts.
** \param countP a point of the file count
** \param maxDirDepth maximum directroy depth.
** \return void
**************************************************************************
**/
FSTATIC void MENU_JPEGFindRecursive (MENU_JPEG_INST_S* inst,
                                     const char_t* pathPtr,
                                     int32_t* countP, uint32_t maxDirDepth)
{
    char_t          searchPath[MENU_JPEG_PATH_MAX + 4];
    char_t          newPath[MENU_JPEG_PATH_MAX];
    uint32_t        strPos;
    uint32_t        searchPos;
    uint32_t        newStrPos;
    F_FIND          entry;

    searchPos = UTILS_strncpy(searchPath, pathPtr, MENU_JPEG_PATH_MAX);
    strPos = UTILS_strncpy(newPath, pathPtr, MENU_JPEG_PATH_MAX);
    newStrPos = strPos;
    /* append separator and wildcard to the searchpath */
    (void)UTILS_strncpy (searchPath + searchPos, "*.*",
                         (MENU_JPEG_PATH_MAX + 4) - searchPos);

    if (!f_findfirst(searchPath, &entry))
    {
        do
        {
            if(entry.filename[0] == '.') continue;
            newStrPos = strPos+UTILS_strncpy(newPath + strPos,
                            entry.filename, MENU_JPEG_PATH_MAX-newStrPos);
            if((entry.filename[0] != 0) && !(entry.attr & F_ATTR_HIDDEN))
            {
                if(!(entry.attr & F_ATTR_DIR))
                {
                    if ( MENU_JPEGFileIsJpeg(entry.filename) )
                    {
                        (*countP)++;

                        if ( (*countP) == (int32_t)inst->currentfile )
                        {
                            (void)UTILS_strncpy(inst->filePath, newPath,
                                                MENU_JPEG_PATH_MAX);
                            return;
                        }
                    }
                }
                else if ( (entry.attr & F_ATTR_DIR) && (maxDirDepth > 0) )
                {
                    newStrPos += UTILS_strncpy (newPath + newStrPos, "/", 2);
                    MENU_JPEGFindRecursive (inst, newPath, countP,
                                           (uint32_t)(maxDirDepth - 1));
                    if(*countP == (int32_t)inst->currentfile)
                        return;
                }
            }
        } while (!f_findnext(&entry));
    }
}


FSTATIC void MENU_JPEGFindCurrentImage(MENU_JPEG_INST_S* inst)
{
    uint16_t            nn;
    uint16_t            filenamepos= 0;
    fbool_t             foundcurrent;
    F_FIND              entry;
    char_t              path[MENU_JPEG_PATH_MAX];
    uint32_t            length, ii;

    inst->currentfile= 1;
    memset (path, 0, MENU_JPEG_PATH_MAX);
    for ( nn = 0; nn < MENU_JPEG_PATH_MAX; nn++ )
    {
        if (!inst->filePath[nn]) break;

        if ( inst->filePath[nn] == MENU_COMMON_PATH_SEPARATOR )
        {
            filenamepos= nn+1;
        }
    }

    length = UTILS_strlen(inst->params.path);
    if((inst->params.path[length-1]=='*')&&
       (inst->params.path[length-2]=='.')&&
       (inst->params.path[length-3]=='*'))
    {
        for(ii=0;ii<length;ii++)
        {
            path[ii] = inst->params.path[ii];
        }
    }
    else
    {
        MENU_CommonAppendpath ("*.*", inst->params.path, path, FFALSE,
                               MENU_JPEG_PATH_MAX);
    }
    if (!f_findfirst(path, &entry))
    {
        do
        {
            if(entry.filename[0] == '.') continue;
            if((entry.filename[0] != 0) && !(entry.attr & F_ATTR_HIDDEN))
            {
                if ( !(entry.attr & F_ATTR_DIR) &&
                     (MENU_JPEGFileIsJpeg(entry.filename)) )
                {
                    foundcurrent= FTRUE;

                    for(nn= 0; nn<30; nn++)
                    {
                        if(inst->filePath[nn+filenamepos] != entry.filename[nn])
                        {
                            foundcurrent= FFALSE;
                            break;
                        }

                        if ( (entry.filename[nn]           == 0) ||
                             (inst->filePath[nn+filenamepos] == 0) )
                        {
                            break;
                        }
                    }
                    if(entry.filename[nn] || inst->filePath[nn+filenamepos])
                        foundcurrent= FFALSE;

                    if(foundcurrent)
                        return;

                    inst->currentfile++;
                }
            }
        } while(!f_findnext(&entry));
    }
}


FSTATIC void MENU_JPEGUpdateImage (MENU_JPEG_INST_S* inst)
{
    JPEG_PIC_PARAMS_S   params;

    /* hide the unformatted message if necessary */
    (void)MAIN_MsgBoxHide(&(appDat->msgBox));

    memset(&params, 0, sizeof(params));

    params.path      = inst->filePath;
    params.filepos   = 0;
    params.orient    = inst->rot;
    params.zoom      = inst->zoom;
    params.xpos      = inst->xpos;
    params.ypos      = inst->ypos;
    params.embedded  = FFALSE;

    (void)JPEG_Show(&params, NULL);

}


FSTATIC void MENU_JPEGCountImages (const char_t* pathPtr, fbool_t root,
                                   uint32_t* countroot, uint32_t* count,
                                   uint32_t maxDirDepth)
{
    char_t          searchPath[MENU_JPEG_PATH_MAX + 4];
    char_t          newPath[MENU_JPEG_PATH_MAX];
    uint32_t        strPos;
    uint32_t        searchPos;
    uint32_t        newStrPos;
    F_FIND          entry;

    searchPos = UTILS_strncpy(searchPath, pathPtr, MENU_JPEG_PATH_MAX);
    strPos = UTILS_strncpy(newPath, pathPtr, MENU_JPEG_PATH_MAX);
    newStrPos = strPos;
    /* append separator and wildcard to the search path */
    (void)UTILS_strncpy (searchPath + searchPos, "*.*",
                         (MENU_JPEG_PATH_MAX + 4) - searchPos);

    if (!f_findfirst(searchPath, &entry))
    {
        do
        {
            if(entry.filename[0] == '.') continue;
            newStrPos = strPos+UTILS_strncpy(newPath + strPos, entry.filename,
                                         MENU_JPEG_PATH_MAX-newStrPos);
            if((entry.filename[0] != 0) && !(entry.attr & F_ATTR_HIDDEN) )
            {
                if(!(entry.attr & F_ATTR_DIR ))
                {
                    if ( MENU_JPEGFileIsJpeg(entry.filename) )
                    {
                        if(root) (*countroot)++;

                        (*count)++;
                    }
                }
                else if ( (entry.attr & F_ATTR_DIR ) && (maxDirDepth > 0) )
                {
                    newStrPos += UTILS_strncpy (newPath + newStrPos, "/", 2);
                    MENU_JPEGCountImages  (newPath, FFALSE, countroot, count,
                                          (uint32_t)(maxDirDepth - 1));
                }
            }
        } while(!f_findnext(&entry));
    }
}


FSTATIC void MENU_JPEGContextGetItem(FAPI_SYS_HandleT h, int32_t id,
                       uint32_t index, FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_JPEG_INST_S*   inst   = (MENU_JPEG_INST_S*)h;
    uint32_t            bufPos = 0;
    id=id;
    if ( state == FGS_FOCUS )
    {
        item[0].item.str.col= appDat->txtAttr.colFocus;
    }
    else
    {
        item[0].item.str.col= appDat->txtAttr.colNonFocus;
    }

    switch ( index )
    {
        case MENU_JPEG_PLAYMODE_LINEAR_THIS:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_ENTRY_JPEG_FOLDER),
                                     MAIN_LINEBUFCHARSET);

            sprintf (appDat->asciiBuf, " (%ld ", inst->rootcount);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_IntASCIIStr(), MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_ENTRY_JPEG_PICTURES),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_GetASCII(")"), MAIN_LINEBUFCHARSET);
            break;

        case MENU_JPEG_PLAYMODE_SHUFFLE_THIS:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_ENTRY_JPEG_FOLDER),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_GetASCII(", "),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_ENTRY_JPEG_SHUFFLE),
                                     MAIN_LINEBUFCHARSET);
            break;

        case MENU_JPEG_PLAYMODE_LINEAR_ALL:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_ENTRY_JPEG_SUBFOLDERS),
                                     MAIN_LINEBUFCHARSET);

            sprintf (appDat->asciiBuf, " (%ld ", inst->allcount);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_IntASCIIStr(), MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_ENTRY_JPEG_PICTURES),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_GetASCII(")"), MAIN_LINEBUFCHARSET);
            break;

        case MENU_JPEG_PLAYMODE_SHUFFLE_ALL:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_ENTRY_JPEG_SUBFOLDERS),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_GetASCII(", "),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_ENTRY_JPEG_SHUFFLE),
                                     MAIN_LINEBUFCHARSET);
            break;

        default:
            (void)STR_BufClear(NULL);
            break;
    }

    item[0].type                = FGS_ITEM_TYPE_STRING;
    item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[0].item.str.font       = (int32_t)MAIN_FONT;
    item[0].item.str.lineHeight = 0;
    item[0].item.str.chr        = STR_IntLineStr();
}


FSTATIC fbool_t MENU_JPEGContextSelect(FAPI_SYS_HandleT h,
                                            int32_t id, uint16_t index)
{
    MENU_JPEG_INST_S*       inst= (MENU_JPEG_INST_S*)h;
    id = id;
    inst->playmode= index;
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_PLAY);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_PLAY_DESC);
    inst->playtime= 0;
    inst->currentfile= 0;
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_NEXT);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_PREVIOUS);

    return FTRUE;
}


/**
**************************************************************************
**\brief Time upadte event handle
**
** This function is called when EVT_TIME_UPDATE events occur and performs
**   picture slide show depending on playmode.
**
** \return void
**************************************************************************
**/
void MENU_JPEGHandleTime(void)
{
    fbool_t       found= FFALSE;
    uint32_t      newFileNum;

    if(menuJPEGDat->playmode == MENU_JPEG_PLAYMODE_NONE)
        return;

    if(((menuJPEGDat->random & 0x2080) == 0x0080) ||
                            ((menuJPEGDat->random & 0x2080) == 0x2000))
        menuJPEGDat->random= (menuJPEGDat->random << 1) | 1;
    else
        menuJPEGDat->random= menuJPEGDat->random << 1;

    menuJPEGDat->playtime++;
    if(menuJPEGDat->playtime < 5)
        return;

    if ( JPEG_IsBusy() )
        return;

    menuJPEGDat->playtime= 0;

    switch(menuJPEGDat->playmode)
    {
        case MENU_JPEG_PLAYMODE_LINEAR_THIS:
            if(menuJPEGDat->currentfile < (menuJPEGDat->rootcount))
            {
                menuJPEGDat->currentfile++;
                found= MENU_JPEGFindImage(menuJPEGDat, FFALSE);
            }
            if(menuJPEGDat->currentfile >= menuJPEGDat->rootcount)
            {
                menuJPEGDat->currentfile = 0;
            }
            break;

        case MENU_JPEG_PLAYMODE_SHUFFLE_THIS:
            newFileNum = (((menuJPEGDat->random & 0x0FFFF) *
                                        menuJPEGDat->rootcount) >> 16)+1;
            //avoid the same file number, show a different picture each time
            if (menuJPEGDat->currentfile == newFileNum) return;
            else menuJPEGDat->currentfile = newFileNum;
            found= MENU_JPEGFindImage(menuJPEGDat, FFALSE);
            break;

        case MENU_JPEG_PLAYMODE_LINEAR_ALL:
            if(menuJPEGDat->currentfile < (menuJPEGDat->allcount))
            {
                menuJPEGDat->currentfile++;
                found= MENU_JPEGFindImage(menuJPEGDat, FTRUE);
            }
            if(menuJPEGDat->currentfile >= menuJPEGDat->allcount)
            {
                menuJPEGDat->currentfile = 0;
            }
            break;

        case MENU_JPEG_PLAYMODE_SHUFFLE_ALL:
            newFileNum = (((menuJPEGDat->random & 0x0FFFF) *
                                        menuJPEGDat->allcount) >> 16)+1;
            //avoid the same file number, show a different picture each time
            if (menuJPEGDat->currentfile == newFileNum) return;
            else menuJPEGDat->currentfile = newFileNum;
            found= MENU_JPEGFindImage(menuJPEGDat, FTRUE);
            break;
        default:
            break;
    }

    if(found)
    {
        FAPI_SYS_PRINT_DEBUG(4,"[JPEG] %d of %d, %s\n",
                  menuJPEGDat->allcount,
                  menuJPEGDat->currentfile,
                  menuJPEGDat->filePath);

        if(menuJPEGDat->pnlHdl)
            (void)FGS_PANEL_RefreshMember(menuJPEGDat->pnlHdl,
                                                    MENU_JPEG_ID_TITLE);

        MENU_JPEGSwitchMenu(menuJPEGDat,
                        MENU_JPEGZoomIsSupported(menuJPEGDat->filePath));
        MENU_JPEGUpdateImage(menuJPEGDat);
    }
    else
    {
        menuJPEGDat->playmode= MENU_JPEG_PLAYMODE_NONE;
        if(menuJPEGDat->pnlHdl)
        {
            (void)FGS_PANEL_RefreshMember(menuJPEGDat->pnlHdl,
                                                     MENU_JPEG_ID_PLAY);
            (void)FGS_PANEL_RefreshMember(menuJPEGDat->pnlHdl,
                                                     MENU_JPEG_ID_PLAY_DESC);
            (void)FGS_PANEL_RefreshMember(menuJPEGDat->pnlHdl,
                                                     MENU_JPEG_ID_NEXT);
            (void)FGS_PANEL_RefreshMember(menuJPEGDat->pnlHdl,
                                                     MENU_JPEG_ID_PREVIOUS);
        }
    }
}


FSTATIC void MENU_JPEGUpdateZoomBox (MENU_JPEG_INST_S* inst)
{
    uint16_t         fullsize= ((SCALER_GetCurrVal(LIST_LINE_HEIGHT)+
                                SCALER_GetCurrVal(LIST_LINE_PADDING))*3) - 4;
    uint16_t         factor= inst->zoom / 100;
    uint32_t         scrollsize;

    inst->zoomPos.width= fullsize / factor;
    inst->zoomPos.height= inst->zoomPos.width;
    scrollsize= fullsize - inst->zoomPos.width;
    inst->zoomPos.x= 2 + (((int32_t)scrollsize * (int32_t)inst->xpos) / 100);
    inst->zoomPos.y= 2 + (((int32_t)scrollsize * (int32_t)inst->ypos) / 100);

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_JPEG_ID_ZOOM);
}


FSTATIC FINLINE fbool_t MENU_JPEGFileIsJpeg (const char_t* fileName)
{
    uint16_t n;

    for ( n = 0; n < 30; n++ )
    {
        if ( fileName[n] == 0 ) break;
    }

    if (( (n > 4)                &&
         (fileName[n-3] == 'J') &&
         (fileName[n-2] == 'P') &&
         (fileName[n-1] == 'G') )
         ||
        ( (n > 4)                &&
         (fileName[n-3] == 'P') &&
         (fileName[n-2] == 'N') &&
         (fileName[n-1] == 'G') )
         ||
        ( (n > 4)                &&
         (fileName[n-3] == 'B') &&
         (fileName[n-2] == 'M') &&
         (fileName[n-1] == 'P') )

         )
    {
        return FTRUE;
    }
    else
    {
        return FFALSE;
    }
}

/* FIXME: workaround of non-support 'zoom' and 'rotate' picture format,
   e.g. BMP and PNG
   usage: disable 'zoom' and 'rotate' function for some picture format */
FSTATIC FINLINE fbool_t MENU_JPEGZoomIsSupported (const char_t* fileName)
{
    uint16_t n;

    for ( n = 0; n < 30; n++ )
    {
        if ( fileName[n] == 0 ) break;
    }

    if ( (n > 4)                &&
         (fileName[n-3] == 'J') &&
         (fileName[n-2] == 'P') &&
         (fileName[n-1] == 'G') )
    {
        return FTRUE;
    }
    else
    {
        return FFALSE;
    }
}

/* Switch help menu for different picture formats: "zoom" and "rotate"
   supported (e.g. JPEG) and non-supported (e.g. BMP, PNG)             */

FSTATIC void MENU_JPEGSwitchMenu(MENU_JPEG_INST_S* inst, fbool_t zoomIsSupported)
{
    if(zoomIsSupported)
    {
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_DEGREES,  FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ZOOMRATE, FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ZOOM,     FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ZOOMIN,   FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ZOOMOUT,  FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ROTATE,   FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_RED,      FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_GREEN,    FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_BLUE,     FFALSE);
    }
    else
    {
        (void)FGS_PANEL_SetRefreshMode (inst->pnlHdl, FFALSE);

        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_DEGREES,  FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ZOOMRATE, FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ZOOM,     FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ZOOMIN,   FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ZOOMOUT,  FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_ROTATE,   FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_RED,      FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MENU_JPEG_ID_GREEN,    FTRUE);

        (void)FGS_PANEL_SetRefreshMode (inst->pnlHdl, FTRUE);

        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl, MENU_JPEG_ID_BLUE,     FTRUE);
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
