/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_main.c
**
**              $Id: app_menu_main.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD Main menu.
**
** Detail description. [\ref app_smartgo_app_ref_main]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info 713: Loss of precision (assignment) (unsigned int to short).
** - Info 715: Symbol not referenced.
** - Info 732: Loss of sign (assignment) (int to unsigned short).
** - Info 737: Loss of sign in promotion from int to unsigned int.
** - Note 961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**   C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_MAIN_ID_TITLE,
    MENU_MAIN_ID_LIST,
    MENU_MAIN_ID_HELPLIST
};

enum
{
    #ifdef APPL_FILEOP_ENABLED
    MENU_MAIN_ID_ENTRY_VIEWER,
    MENU_MAIN_ID_ENTRY_MEDIA,
    #endif /* APPL_FILEOP_ENABLED */

    #ifdef APPL_PVR_ENABLED
    MENU_MAIN_ID_ENTRY_PLIST,
    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_RECTIMER_ENABLED
    MENU_MAIN_ID_ENTRY_TIMER,
    #endif /* APPL_RECTIMER_ENABLED */

    #ifdef APPL_MOSAIC_ENABLED
    MENU_MAIN_ID_ENTRY_MOSAIC,
    #endif /* APPL_MOSAIC_ENABLED */

    MENU_MAIN_ID_ENTRY_STRSELECT,
    MENU_MAIN_ID_ENTRY_SETUP,
    MENU_MAIN_ID_ENTRY_INFO,

    MENU_MAIN_ID_ENTRY_NUMS
};

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    listHdl;
    FAPI_SYS_HandleT    listHelpHdl;

    MENU_TIME_OPEN_S timePars;
}MENU_MAIN_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
/*FSTATIC*/ MENU_MAIN_INST_S* menuMainDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_MainLeave        (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_MainOnSelect     (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void    MENU_MainGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                       FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC fbool_t MENU_MainBuild        (FAPI_SYS_HandleT h);
FSTATIC void    MENU_MainOnFocusChg   (FAPI_SYS_HandleT h, uint8_t index, int32_t id);
FSTATIC fbool_t MENU_MainSetupReq     (MENU_MAIN_INST_S* inst);
FSTATIC fbool_t MENU_MainSetupAuthorized (MENU_MAIN_INST_S* inst);
FSTATIC void    MENU_MainOnFocusSet   (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_MainGetHelpDescrItem (FAPI_SYS_HandleT h,
                                           int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);

#ifdef APPL_PARENTAL_ENABLED
FSTATIC void  MENU_MainSetupOnPinEnter (void* arg, const char_t* pin, uint8_t numChars);
#endif /* APPL_PARENTAL_ENABLED */

int32_t MENU_MainInit (void)
{
    if ( menuMainDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuMainDat = UTILS_Malloc(sizeof(MENU_MAIN_INST_S));
    if ( menuMainDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuMainDat, 0, sizeof(MENU_MAIN_INST_S));

    return FAPI_OK;
}

fbool_t MENU_MainEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_MAIN_INST_S* inst = menuMainDat;
    uint32_t               lastFocusItem;

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    (void)MENU_MainBuild((FAPI_SYS_HandleT)inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = 0;
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_MAIN_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_MAIN);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_MainLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

FSTATIC void MENU_MainOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_MAIN_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);
}

FSTATIC fbool_t MENU_MainLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_MAIN_INST_S*   inst = (MENU_MAIN_INST_S*)h;

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_MAIN);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC void MENU_MainOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_MAIN_INST_S*   inst = (MENU_MAIN_INST_S*)h;
    FGS_LNAVI_PARAMS_S  naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}


FSTATIC fbool_t MENU_MainOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_MAIN_INST_S*       inst = (MENU_MAIN_INST_S*)h;
    MENUSTACK_ENTRY         newEntry;
    FAPI_SYS_HandleT          arg = NULL;

    switch ( itemId )
    {
        #ifdef APPL_FILEOP_ENABLED

        case MENU_MAIN_ID_ENTRY_VIEWER:
            arg      = (FAPI_SYS_HandleT)1; /* any non-zero value! */
            newEntry = MENU_ViewerEntry;
            break;

        case MENU_MAIN_ID_ENTRY_MEDIA:
            newEntry = MENU_MediaEntry;
            break;

        #endif /* APPL_FILEOP_ENABLED */

        #ifdef APPL_PVR_ENABLED

        case MENU_MAIN_ID_ENTRY_PLIST:
            newEntry = MENU_PlayListEntry;
            break;

        #endif /* APPL_PVR_ENABLED */

        #ifdef APPL_RECTIMER_ENABLED

        case MENU_MAIN_ID_ENTRY_TIMER:
            if ( TIME_IsSync() )
            {
                    newEntry = MENU_TimerProgEntry;
            }
            else
            {
                inst->timePars.strIdx = STR_TIME_MSG_TIMERPROG;
                inst->timePars.done   = FFALSE;
                inst->timePars.next   = MENU_TIME_NEXT_TIMERPROG_MENU;
                arg = (FAPI_SYS_HandleT)(&(inst->timePars));
                newEntry = MENU_TimeEntry;
            }
            break;

        #endif /* APPL_RECTIMER_ENABLED */

        #ifdef APPL_MOSAIC_ENABLED

        case MENU_MAIN_ID_ENTRY_MOSAIC:
            newEntry = MENU_MosaicEntry;
            break;

        #endif /* APPL_MOSAIC_ENABLED */

        case MENU_MAIN_ID_ENTRY_STRSELECT:
            newEntry = MENU_StrSelEntry ;
            break;

        case MENU_MAIN_ID_ENTRY_SETUP:
            return MENU_MainSetupReq (inst);

        case MENU_MAIN_ID_ENTRY_INFO:
            newEntry = MENU_InformationEntry;
            break;

        default:
            return FFALSE;
    }

    /* store last focus */
    MENUSTACK_SetData(itemId);

    return MENUSTACK_Up (newEntry, arg);
}

FSTATIC void MENU_MainGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                    FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx = 0;

    switch ( id )
    {
        #ifdef APPL_FILEOP_ENABLED

        case MENU_MAIN_ID_ENTRY_VIEWER:
            strIdx = STR_ENTRY_MAIN_VIEWER;
            break;

        case MENU_MAIN_ID_ENTRY_MEDIA:
            strIdx = STR_ENTRY_MAIN_MEDIA;
            break;

        #endif /* APPL_FILEOP_ENABLED */

        #ifdef APPL_PVR_ENABLED

        case MENU_MAIN_ID_ENTRY_PLIST:
            strIdx = STR_ENTRY_MAIN_RECORD;
            break;

        #endif /* APPL_PVR_ENABLED */

        #ifdef APPL_RECTIMER_ENABLED

        case MENU_MAIN_ID_ENTRY_TIMER:
            strIdx = STR_ENTRY_MAIN_TIMER;
            break;

        #endif /* APPL_RECTIMER_ENABLED */

        #ifdef APPL_MOSAIC_ENABLED

        case MENU_MAIN_ID_ENTRY_MOSAIC:
            strIdx = STR_ENTRY_MAIN_MOSAIC;
            break;

        #endif /* APPL_MOSAIC_ENABLED */

        case MENU_MAIN_ID_ENTRY_STRSELECT:
            strIdx = STR_ENTRY_MAIN_STRSEL;
            break;

        case MENU_MAIN_ID_ENTRY_SETUP:
            strIdx = STR_ENTRY_MAIN_SETUP;
            break;

        case MENU_MAIN_ID_ENTRY_INFO:
            strIdx = STR_ENTRY_MAIN_INFO;
            break;

        default:
            return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_MainGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                        FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        #ifdef APPL_FILEOP_ENABLED
        case MENU_MAIN_ID_ENTRY_VIEWER:
        case MENU_MAIN_ID_ENTRY_MEDIA:
        #endif /* APPL_FILEOP_ENABLED */

        #ifdef APPL_PVR_ENABLED
        case MENU_MAIN_ID_ENTRY_PLIST:
        #endif /* APPL_PVR_ENABLED */

        #ifdef APPL_RECTIMER_ENABLED
        case MENU_MAIN_ID_ENTRY_TIMER:
        #endif /* APPL_RECTIMER_ENABLED */

        #ifdef APPL_MOSAIC_ENABLED
        case MENU_MAIN_ID_ENTRY_MOSAIC:
        #endif /* APPL_MOSAIC_ENABLED */

        case MENU_MAIN_ID_ENTRY_STRSELECT:
        case MENU_MAIN_ID_ENTRY_SETUP:
        case MENU_MAIN_ID_ENTRY_INFO:
            strIdx = STR_ENTRY_OK;
            break;

        default:
            return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC fbool_t MENU_MainBuild(FAPI_SYS_HandleT h)
{
    MENU_MAIN_INST_S*   inst = (MENU_MAIN_INST_S*)h;
    FGS_PANEL_OPEN_S    pnlPars;
    FGS_LIST_OPEN_S     listPars;
    FAPI_SYS_HandleT    itemHdl;
    FGS_ITEM_TXT_OPEN_S txtPars;
    FGS_POS_S           itemPos;
    FGS_POS_S           descrPos;
    int32_t             retVal;
    uint16_t            helpWidth;
    uint16_t            stackedFrameWidth;
    uint16_t            innerHeight;
    uint32_t            numLines = MENU_MAIN_ID_ENTRY_NUMS;

    memset(&txtPars,  0, sizeof(txtPars));

    helpWidth         = (2 * SCALER_GetCurrVal(HELP_LR_PADDING))
                      + OSD_GetStrWidth (OSD_GetStr(STR_ENTRY_OK),
                                         -1, SYMBOL_FONT);
    innerHeight       = numLines *
                       (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                      + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth,
                                        1, &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_MainOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_MAIN, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_MAIN_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_MAIN,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_MainGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_MainOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_MainOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_MAIN_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, NULL, 100,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- menu lines ------------------------------------------------ */

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.pos                = itemPos;

        /* --- add VIEWER and MEDIA Menu ----------------------------------- */

        #ifdef APPL_FILEOP_ENABLED

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_MAIN_ID_ENTRY_VIEWER,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                  + SCALER_GetCurrVal(LIST_LINE_PADDING);

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_MAIN_ID_ENTRY_MEDIA,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                  + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_FILEOP_ENABLED */

        /* --- add PVR Menu -------------------------------------------------*/

        #ifdef APPL_PVR_ENABLED

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_MAIN_ID_ENTRY_PLIST,
                                inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_PVR_ENABLED */

        /* --- add Timer Menu -----------------------------------------------*/

        #ifdef APPL_RECTIMER_ENABLED

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_MAIN_ID_ENTRY_TIMER,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_RECTIMER_ENABLED */

        /* --- add Mosaic Menu ------------------------------------------------*/

        #ifdef APPL_MOSAIC_ENABLED

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_MAIN_ID_ENTRY_MOSAIC,
                                inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_MOSAIC_ENABLED */

        /* --- add Stream Select Menu ----------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_MAIN_ID_ENTRY_STRSELECT,
                                inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- add Config Menu ---------------------------------------------- */

        txtPars.pos                        = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_MAIN_ID_ENTRY_SETUP,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* -----add Information -------------------------------------------- */

        txtPars.pos                        = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_MAIN_ID_ENTRY_INFO,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_MAIN_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH),
                                        listPars.pos.y, helpWidth, numLines,
                                        MENU_MainGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine (stackedFrameWidth
                          + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                          + SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                          - ((int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2)
                          - SCALER_GetCurrVal(HELP_TB_PADDING)
                          - SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                            pnlPars.pos.width
                          - stackedFrameWidth
                          - (2 * SCALER_GetCurrVal(OUTER_FRAME_ALL))
                          - SCALER_GetCurrVal(MAIN_LEFT_PADDING)
                          - SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                            inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

        FAPI_SYS_FREE(inst);
        return FFALSE;
    }

    return FTRUE;
}

FSTATIC fbool_t MENU_MainSetupReq (MENU_MAIN_INST_S* inst)
{
    fbool_t retVal;

    #ifdef APPL_PARENTAL_ENABLED

    MENU_PINDIALOG_S openParams;
    PCTRL_PARAMS_S   pctrlParams;

    if ( PCTRL_GetParams(&pctrlParams) != FAPI_OK ) return FFALSE;

    /* test if setup area is protected */
    if ( (pctrlParams.protectionArea & APP_PROTECTION_SETUP) != 0 )
    {
        memset(&openParams, 0, sizeof(openParams));

        openParams.titleStr   = OSD_GetStr(STR_MSG_PASSWORD_ENTER_PIN);
        openParams.numChars   = PCTRL_PIN_LEN;
        openParams.onPinEnter = MENU_MainSetupOnPinEnter;
        openParams.arg        = inst;

        if ( MENU_PinDialogOpen(&(appDat->pinDialog), &openParams) == FAPI_OK )
        {
            retVal = FTRUE;
        }
        else
        {
            retVal = FFALSE;
        }
    }
    else
    {
    #endif /* APPL_PARENTAL_ENABLED */

        retVal = MENU_MainSetupAuthorized(inst);

    #ifdef APPL_PARENTAL_ENABLED
    }
    #endif /* APPL_PARENTAL_ENABLED */

    return retVal;
}

#ifdef APPL_PARENTAL_ENABLED

FSTATIC void MENU_MainSetupOnPinEnter (void* arg, const char_t* pin, uint8_t numChars)
{
    MENU_MAIN_INST_S* inst       = (MENU_MAIN_INST_S*)arg;
    fbool_t             authorized = FFALSE;

    if ( PCTRL_CheckPin (PCTRL_PIN_MASTER, pin) == FAPI_OK )
    {
        authorized = FTRUE;
    }
    else
    {
        (void)MENU_PinDialogReset(&(appDat->pinDialog));
    }

    (void)MENU_PinDialogClose(&(appDat->pinDialog));

    if ( authorized ) (void)MENU_MainSetupAuthorized(inst);
}

#endif /* APPL_PARENTAL_ENABLED */

FSTATIC fbool_t MENU_MainSetupAuthorized (MENU_MAIN_INST_S* inst)
{
    /* store last focus */
    MENUSTACK_SetData(MENU_MAIN_ID_ENTRY_SETUP);

    return MENUSTACK_Up(MENU_SystemSetupEntry, NULL);
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
