/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_media.c
**
**              $Id: app_menu_media.c 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       Media menu implementation.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision.
** - Info    715: Symbol 'anySymb' not referenced.
** - Info    737: loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence.
** - Note    970: Use of modifier or type 'float' outside of a typedef.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e737 -e961 -e970 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include "app_menu.h"
#include "app_menu_viewer.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define MENU_MEDIA_SHUFFLE_LW_MAX           8       //!< Results in max. number
                                                    //!< of shuffle entries of
                                                    //!< 2048.
#define MENU_MEDIA_BITS_PER_LW              32
#define MENU_MEDIA_SHUFFLE_ITEMS_MAX        ((MENU_MEDIA_SHUFFLE_LW_MAX) \
                                           * (MENU_MEDIA_BITS_PER_LW))

#define MENU_MEDIA_SMALLINFO_DISPLAYTIME    5000    //!< in milliseconds.

/* ------------------------------------------------------------------------- */

#ifdef APPL_JPEG_ENABLED

#define MENU_MEDIA_SLIDESHOW_DISPLAYTIME    5000    //!< in milliseconds.

#define MENU_MEDIA_ZOOM_MIN                 100     //!< 100%, no zoom.
#define MENU_MEDIA_ZOOM_MAX                 800     //!< Maximum zoom factor in
                                                    //!< per cent.
#define MENU_MEDIA_ZOOM_STEP                100     //!< Zoom steps in per cent.

#define MENU_MEDIA_PIC_DEFAULT_SINGLE       FTRUE
#define MENU_MEDIA_PIC_DEFAULT_SHUFFLE      FFALSE

#define MENU_MEDIA_PICINFO_NUMLINES         3

#define MENU_MEDIA_PIC_ZOOMBOX_SIZE     ((MENU_MEDIA_PICINFO_NUMLINES         \
                                       * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) \
                                  - (2 * SCALER_GetCurrVal(MAIN_TOP_PADDING)))

#endif /* APPL_JPEG_ENABLED */

/* ------------------------------------------------------------------------- */

#ifdef APPL_MP3_ENABLED

#define MENU_MEDIA_MUSIC_DEFAULT_SINGLE         FFALSE
#define MENU_MEDIA_MUSIC_DEFAULT_SHUFFLE        FFALSE

#define MENU_MEDIA_MUSICINFO_NUMLINES           2
#define MENU_MEDIA_MUSICINFO_LEFTWIDTH_PERCENT  40  //!< Width of ID3
                                                    //!< title/album area in
                                                    //!< percent of panel width.

#endif /* APPL_MP3_ENABLED */

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_MEDIA_DIALOG_ID_TITLE = 0,
    MENU_MEDIA_DIALOG_ID_LIST,
    MENU_MEDIA_DIALOG_ID_HELPLIST
};

enum
{
    MENU_MEDIA_SMALLINFO_ID_PICNAME = 0,
    MENU_MEDIA_SMALLINFO_ID_MUSICNAME
};

#ifdef APPL_JPEG_ENABLED

enum
{
    MENU_MEDIA_PICINFO_ID_REST = 0,
    MENU_MEDIA_PICINFO_ID_FILENAME,
    MENU_MEDIA_PICINFO_ID_SLIDESHOWSTAT,
    MENU_MEDIA_PICINFO_ID_SHUFFLESTAT,
    MENU_MEDIA_PICINFO_ID_RESOLUTION,
    MENU_MEDIA_PICINFO_ID_ROTSTAT,
    MENU_MEDIA_PICINFO_ID_ZOOMSTAT,
    MENU_MEDIA_PICINFO_ID_ZOOMBOX,

    MENU_MEDIA_PICINFO_ID_HELP_NEXT_KEY,
    MENU_MEDIA_PICINFO_ID_HELP_NEXT_DESC,
    MENU_MEDIA_PICINFO_ID_HELP_PREV_KEY,
    MENU_MEDIA_PICINFO_ID_HELP_PREV_DESC,
    MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_KEY,
    MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_DESC,
    MENU_MEDIA_PICINFO_ID_HELP_ZOOMIN_KEY,
    MENU_MEDIA_PICINFO_ID_HELP_ZOOMIN_DESC,
    MENU_MEDIA_PICINFO_ID_HELP_ZOOMOUT_KEY,
    MENU_MEDIA_PICINFO_ID_HELP_ZOOMOUT_DESC,
    MENU_MEDIA_PICINFO_ID_HELP_SHUFFLE_KEY,
    MENU_MEDIA_PICINFO_ID_HELP_SHUFFLE_DESC,
    MENU_MEDIA_PICINFO_ID_HELP_ROTATE_KEY,
    MENU_MEDIA_PICINFO_ID_HELP_ROTATE_DESC,

    MENU_MEDIA_PICINFO_ID_NUMS
};

#endif /* APPL_JPEG_ENABLED */

#ifdef APPL_MP3_ENABLED

enum
{
    MENU_MEDIA_MUSICINFO_ID_REST = 0,
    MENU_MEDIA_MUSICINFO_ID_FILENAME,
    MENU_MEDIA_MUSICINFO_ID_SHUFFLESTAT,
    MENU_MEDIA_MUSICINFO_ID_PLAYTIME,
    MENU_MEDIA_MUSICINFO_ID_PROGRESS,
    MENU_MEDIA_MUSICINFO_ID_STATUS,
    MENU_MEDIA_MUSICINFO_ID_TITLE,
    MENU_MEDIA_MUSICINFO_ID_ALBUM,

    MENU_MEDIA_MUSICINFO_ID_HELP_NEXT_KEY,
    MENU_MEDIA_MUSICINFO_ID_HELP_NEXT_DESC,
    MENU_MEDIA_MUSICINFO_ID_HELP_PREV_KEY,
    MENU_MEDIA_MUSICINFO_ID_HELP_PREV_DESC,
    MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_KEY,
    MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_DESC,
    MENU_MEDIA_MUSICINFO_ID_HELP_STOP_KEY,
    MENU_MEDIA_MUSICINFO_ID_HELP_STOP_DESC,
    MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_KEY,
    MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_DESC,

    MENU_MEDIA_MUSICINFO_ID_NUMS
};

typedef enum
{
    MENU_MEDIA_PLAY_STATUS_NONE = 0,
    MENU_MEDIA_PLAY_STATUS_STOP,
    MENU_MEDIA_PLAY_STATUS_PLAY,
    MENU_MEDIA_PLAY_STATUS_PAUSE

} MENU_MEDIA_PLAY_STATUS_E;

#endif /* APPL_MP3_ENABLED */

typedef enum
{
    MENU_MEDIA_TYPE_PICTURE = 0,
    MENU_MEDIA_TYPE_MUSIC,
    MENU_MEDIA_TYPE_NUMS

} MENU_MEDIA_TYPE_E;

enum
{
    #ifdef APPL_JPEG_ENABLED
    MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_PIC = 0,
    #endif /* APPL_JPEG_ENABLED */

    #ifdef APPL_MP3_ENABLED
    MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_MUSIC,
    #endif /* APPL_MP3_ENABLED */

    MENU_MEDIA_DIALOG_ID_ENTRY_LEAVE,
    MENU_MEDIA_DIALOG_ID_ENTRY_NUMS
};

struct menu_media_inst_s;

typedef struct
{
    struct menu_media_inst_s*   pRoot;

    FAPI_SYS_HandleT            pnlHdl;
    FAPI_SYS_HandleT            listHdl;
    FAPI_SYS_HandleT            listHelpHdl;

} MENU_MEDIA_DIALOG_INST_S;

typedef struct
{
    struct menu_media_inst_s*   pRoot;

    FAPI_SYS_HandleT            pnlHdl;
    FGS_PANEL_INST_S            pnlInst;
    FGS_BOX_INST_S              boxInst[MENU_MEDIA_TYPE_NUMS];

    uint64_t                    displayTime;    //!< 0: permanent view, others:
                                                //!< start display time.

} MENU_MEDIA_SMALLINFO_INST_S;

#ifdef APPL_JPEG_ENABLED

typedef struct
{
    FAPI_SYS_HandleT            pnlHdl;
    FGS_PANEL_INST_S            pnlInst;
    FGS_BOX_INST_S              boxInst[MENU_MEDIA_PICINFO_ID_NUMS];

} MENU_MEDIA_PICINFO_INST_S;

typedef struct
{
    MENU_MEDIA_PICINFO_INST_S   info;           //!< Picture info panel.

    JPEG_orientation_t          orientation;    //!< Rotation status
    uint16_t                    zoom;           //!< Zoom in per cent (100...)
    uint8_t                     rasterPosX;
    uint8_t                     rasterPosY;
    uint8_t                     rasterEntries;

    /* picture information */
    PIC_TYPE_E                  format;         //!< Coding format of picture.
    uint16_t                    sizeX;          //!< Horizontal size in pixels,
                                                //!< 0: unknown.
    uint16_t                    sizeY;          //!< Horizontal size in pixels,
                                                //!< 0: unknown.

    uint64_t                    displayTime;    //!< Start display time of
                                                //!< current picture.
    fbool_t                     zoomRotateAvl;  //!< Feature availability flag.

} MENU_MEDIA_PIC_S;

#endif /* APPL_JPEG_ENABLED */

#ifdef APPL_MP3_ENABLED

typedef struct
{
    FAPI_SYS_HandleT            pnlHdl;
    FGS_PANEL_INST_S            pnlInst;
    FGS_BOX_INST_S              boxInst[MENU_MEDIA_MUSICINFO_ID_NUMS];

} MENU_MEDIA_MUSICINFO_INST_S;

typedef struct
{
    MENU_MEDIA_MUSICINFO_INST_S info;           //!< Music info panel.

    MENU_MEDIA_PLAY_STATUS_E    playStatus;
    uint32_t                    timeCurr;       //!< in seconds.
    uint32_t                    timeTotal;      //!< in seconds.
    MP3_FILE_INFO_S             fileInfo;
    fbool_t                     waitForStopEvt; //!< Used during event handling.

} MENU_MEDIA_MUSIC_S;

#endif /* APPL_MP3_ENABLED */

typedef struct
{
    struct menu_media_inst_s* pRoot;    //!< Pointer to root instance.
    void*               pParams;    //!< Pointer to type-specific parameters.
    void*               pInfo;      //!< Pointer to info panel structure.

    /* general parameters */
    fbool_t             active;     //!< A file is currently being presented.
    fbool_t             single;     //!< FTRUE: Presentation is stopped after
                                    //!< this file, FFALSE: next file is
                                    //!< presented.
    fbool_t             shuffle;    //!< FTRUE: Next file is selected in
                                    //!< shuffle mode, FFALSE: selection is
                                    //!< linear.
    uint32_t            shufBM[MENU_MEDIA_SHUFFLE_LW_MAX];
    uint16_t            shufItems;  //!< Number of items already marked in
                                    //!< shuffle array.

    /* parameters of currently active file */
    char_t              path    [MAIN_VIEWER_PATHLEN];
    char_t              name    [MAIN_VIEWER_NAMELEN];
    uint16_t            currIndex;
    uint16_t            numItems;
    char_t              fullPath[MAIN_VIEWER_PATHLEN];

    /* control callbacks */
    int32_t             (*fileOpen)       (void* pType);
    int32_t             (*fileClose)      (void* pType);
    fbool_t             (*fileIsDecodable)(void* pType, const char_t* name,
                                           uint16_t nameLenMax);
    int32_t             (*fileStart)      (void* pType,
                                           const char_t* path,
                                           const char_t* name, fbool_t newFile);
    void                (*handleTime)     (void* pType);
    void                (*handleEvent)    (void* pType, SYS_EVTTYPE_E type,
                                           const void* data);

    /* info menu callbacks */
    int32_t             (*infoShow)       (void* pType, fbool_t showOrHide);
    fbool_t             (*infoIsVisible)  (void* pType);
    int32_t             (*infoUpdate)     (void* pType);

} MENU_MEDIA_DATA_S;

typedef struct menu_media_inst_s
{
    FGS_POS_S                   pos;
    FGS_MEMBER_S*               pRootMem;

    MENU_MEDIA_DIALOG_INST_S    dialog;     //!< Parameters for dialogue.
    MENU_MEDIA_SMALLINFO_INST_S smallInfo;  //!< Parameters for "small info".
    MENU_MEDIA_DATA_S           type[MENU_MEDIA_TYPE_NUMS];

    uint16_t                    infoHeight; //!< Used by info panels for
                                            //!< positioning.

    #ifdef APPL_JPEG_ENABLED

    MENU_MEDIA_PIC_S            pic;        //!< Specific parameters.

    #endif /* APPL_JPEG_ENABLED */

    #ifdef APPL_MP3_ENABLED

    MENU_MEDIA_MUSIC_S          music;      //!< Specific parameters.

    #endif /* APPL_MP3_ENABLED */

} MENU_MEDIA_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

MENU_MEDIA_INST_S* menuMediaDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t MENU_MediaLeave            (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC int32_t MENU_MediaSetFocus         (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC fbool_t MENU_MediaKeys             (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_MediaHandleInfoPanels (MENU_MEDIA_INST_S* inst);
FSTATIC uint32_t MENU_MediaGetFileExtension (const char_t* name,
                                            uint16_t nameLenMax);

FSTATIC fbool_t MENU_MediaDialogEnter      (MENU_MEDIA_INST_S* rootInst,
                                            MENU_MEDIA_DIALOG_INST_S* inst);
FSTATIC fbool_t MENU_MediaDialogLeave      (MENU_MEDIA_DIALOG_INST_S* inst);
FSTATIC int32_t MENU_MediaDialogBuild      (MENU_MEDIA_DIALOG_INST_S* inst);
FSTATIC fbool_t MENU_MediaDialogKeys       (FAPI_SYS_HandleT h);
FSTATIC void    MENU_MediaDialogOnFocusSet (FAPI_SYS_HandleT h,
                                            fbool_t hasFocus);
FSTATIC void    MENU_MediaDialogOnFocusChg (FAPI_SYS_HandleT h, uint8_t index,
                                            int32_t id);
FSTATIC fbool_t MENU_MediaDialogOnSelect   (FAPI_SYS_HandleT h,
                                            int32_t itemId);
FSTATIC void    MENU_MediaDialogGetDescrItem(FAPI_SYS_HandleT h, int32_t id,
                                             uint32_t index,
                                             FGS_ITEMSTATE_E state,
                                             FGS_ITEM_S* item);
FSTATIC void    MENU_MediaDialogGetHelpDescrItem (FAPI_SYS_HandleT h,
                                             int32_t id, uint32_t index,
                                             FGS_ITEMSTATE_E state,
                                             FGS_ITEM_S* item);

FSTATIC int32_t MENU_MediaSmallInfoOpen     (MENU_MEDIA_INST_S* rootInst,
                                             MENU_MEDIA_SMALLINFO_INST_S* inst);
FSTATIC int32_t MENU_MediaSmallInfoClose    (MENU_MEDIA_SMALLINFO_INST_S* inst);
FSTATIC int32_t MENU_MediaSmallInfoShow     (MENU_MEDIA_SMALLINFO_INST_S* inst,
                                             fbool_t showOrHide,
                                             fbool_t permanent);
FSTATIC void    MENU_MediaSmallInfoHandleTime (MENU_MEDIA_SMALLINFO_INST_S*
                                             inst);
FSTATIC void    MENU_MediaSmallInfoGetItem  (FAPI_SYS_HandleT h, int32_t id,
                                             uint32_t index,
                                             FGS_ITEMSTATE_E state,
                                             FGS_ITEM_S* item);

FSTATIC fbool_t MENU_MediaViewerOnBack      (void* arg);


FSTATIC fbool_t MENU_MediaFileStart         (MENU_MEDIA_DATA_S* pType,
                                             const char_t* name,
                                             uint16_t index);
FSTATIC fbool_t MENU_MediaOnFileSelect      (MENU_MEDIA_DATA_S* pType,
                                             const char_t* path,
                                             const char_t* name,
                                             uint16_t nameLenMax,
                                             uint16_t index, uint16_t numItems);
FSTATIC fbool_t MENU_MediaFileNext          (MENU_MEDIA_INST_S* inst,
                                             MENU_MEDIA_TYPE_E type);
FSTATIC fbool_t MENU_MediaFilePrevious      (MENU_MEDIA_INST_S* inst,
                                             MENU_MEDIA_TYPE_E type);
FSTATIC fbool_t MENU_MediaGetFileByShuffle  (MENU_MEDIA_DATA_S* pType,
                                             uint16_t* pNewIndex,
                                             char_t* pNewName,
                                             uint16_t nameLenMax);

FSTATIC int32_t MENU_MediaShuffleClear     (MENU_MEDIA_DATA_S* pType);
FSTATIC int32_t MENU_MediaShuffleSet       (MENU_MEDIA_DATA_S* pType,
                                            int32_t index);
FSTATIC int32_t MENU_MediaShuffleGetNextIdx(MENU_MEDIA_DATA_S* pType);

#ifdef APPL_JPEG_ENABLED

FSTATIC fbool_t MENU_MediaPicOnSelect       (void* arg,
                                             MAIN_VIEWER_FILETYPE type,
                                             const char_t* path,
                                             uint16_t pathLenMax,
                                             const char_t* name,
                                             uint16_t nameLenMax,
                                             uint16_t index, uint16_t numItems);
FSTATIC int32_t MENU_MediaPicInit           (MENU_MEDIA_INST_S* inst);
FSTATIC int32_t MENU_MediaPicOpen           (void* arg);
FSTATIC int32_t MENU_MediaPicClose          (void* arg);
FSTATIC int32_t MENU_MediaPicShow           (void* arg, const char_t* path,
                                             const char_t* name,
                                             fbool_t newFile);
FSTATIC fbool_t MENU_MediaPicIsDecodable    (void* arg,
                                             const char_t* name,
                                             uint16_t nameLenMax);
FSTATIC PIC_TYPE_E MENU_MediaPicGetType     (const char_t* name,
                                             uint16_t nameLenMax);
FSTATIC void    MENU_MediaPicHandleTime     (void* arg);
FSTATIC void    MENU_MediaPicHandleEvent    (void* arg, SYS_EVTTYPE_E type,
                                             const void* data);
FSTATIC fbool_t MENU_MediaPicSlideshowOnOff (MENU_MEDIA_INST_S* inst);
FSTATIC fbool_t MENU_MediaPicShuffleOnOff   (MENU_MEDIA_INST_S* inst);
FSTATIC fbool_t MENU_MediaPicZoomWindowMove (MENU_MEDIA_INST_S* inst,
                                             FGS_KEY_E key);
FSTATIC fbool_t MENU_MediaPicRotate         (MENU_MEDIA_INST_S* inst);
FSTATIC fbool_t MENU_MediaPicZoom           (MENU_MEDIA_INST_S* inst,
                                             fbool_t zoomIn);

FSTATIC int32_t MENU_MediaPicInfoOpen       (MENU_MEDIA_INST_S* rootInst,
                                             MENU_MEDIA_DATA_S* pType);
FSTATIC int32_t MENU_MediaPicInfoClose      (MENU_MEDIA_DATA_S* pType);
FSTATIC int32_t MENU_MediaPicInfoShow       (void* arg, fbool_t showOrHide);
FSTATIC fbool_t MENU_MediaPicInfoIsVisible  (void* arg);
FSTATIC int32_t MENU_MediaPicInfoUpdate     (void* arg);
FSTATIC void    MENU_MediaPicInfoGetItem    (FAPI_SYS_HandleT h, int32_t id,
                                             uint32_t index,
                                             FGS_ITEMSTATE_E state,
                                             FGS_ITEM_S* item);
FSTATIC void    MENU_MediaPicInfoDrawZoomBox(FAPI_SYS_HandleT h, uint8_t align,
                                             uint16_t x, uint16_t y,
                                             uint16_t width, uint16_t height);
FSTATIC void    MENU_MediaPicInfoDrawRest   (FAPI_SYS_HandleT h, uint8_t align,
                                             uint16_t x, uint16_t y,
                                             uint16_t width, uint16_t height);
FSTATIC void    MENU_MediaPicInfoRefreshHelp(MENU_MEDIA_PICINFO_INST_S* inst);

#endif /* APPL_JPEG_ENABLED */

#ifdef APPL_MP3_ENABLED

FSTATIC fbool_t MENU_MediaMusicOnSelect     (void* arg,
                                             MAIN_VIEWER_FILETYPE type,
                                             const char_t* path,
                                             uint16_t pathLenMax,
                                             const char_t* name,
                                             uint16_t nameLenMax,
                                             uint16_t index, uint16_t numItems);
FSTATIC int32_t MENU_MediaMusicInit         (MENU_MEDIA_INST_S* inst);
FSTATIC int32_t MENU_MediaMusicOpen         (void* arg);
FSTATIC int32_t MENU_MediaMusicClose        (void* arg);
FSTATIC fbool_t MENU_MediaMusicIsDecodable  (void* arg, const char_t* name,
                                             uint16_t nameLenMax);
FSTATIC int32_t MENU_MediaMusicStart        (void* arg, const char_t* path,
                                             const char_t* name,
                                             fbool_t newFile);
FSTATIC fbool_t MENU_MediaMusicPause        (MENU_MEDIA_DATA_S*  pType,
                                             fbool_t pauseOn);
FSTATIC fbool_t MENU_MediaMusicStop         (MENU_MEDIA_DATA_S*  pType);
FSTATIC void    MENU_MediaMusicHandleEvent  (void* arg, SYS_EVTTYPE_E type,
                                             const void* data);
FSTATIC fbool_t MENU_MediaMusicHandleKey    (MENU_MEDIA_INST_S* inst,
                                             FGS_KEY_E key);

FSTATIC int32_t MENU_MediaMusicInfoOpen     (MENU_MEDIA_INST_S* rootInst,
                                             MENU_MEDIA_DATA_S* pType);
FSTATIC int32_t MENU_MediaMusicInfoClose    (MENU_MEDIA_DATA_S* pType);
FSTATIC int32_t MENU_MediaMusicInfoShow     (void* arg, fbool_t showOrHide);
FSTATIC fbool_t MENU_MediaMusicInfoIsVisible(void* arg);
FSTATIC int32_t MENU_MediaMusicInfoUpdate   (void* arg);
FSTATIC void    MENU_MediaMusicInfoGetItem  (FAPI_SYS_HandleT h, int32_t id,
                                             uint32_t index,
                                             FGS_ITEMSTATE_E state,
                                             FGS_ITEM_S* item);
FSTATIC void    MENU_MediaMusicInfoDrawRest (FAPI_SYS_HandleT h, uint8_t align,
                                             uint16_t x, uint16_t y,
                                             uint16_t width, uint16_t height);
FSTATIC void    MENU_MediaMusicInfoDrawProgBar (FAPI_SYS_HandleT h,
                                             uint8_t align,
                                             uint16_t x, uint16_t y,
                                             uint16_t width, uint16_t height);
FSTATIC void    MENU_MediaMusicInfoRefreshHelp (MENU_MEDIA_MUSICINFO_INST_S*
                                             inst);

#endif /* APPL_MP3_ENABLED */

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/

/*!
******************************************************************************
** \brief   This function initializes the media menu.
**
** \return  one of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_ALREADY_INITIALIZED
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
******************************************************************************
*/
int32_t MENU_MediaInit (void)
{
    MENU_MEDIA_INST_S* inst;
    int32_t            retVal = FAPI_OK;

    do
    {
        if ( menuMediaDat != NULL )
        {
            retVal = APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
            break;
        }

        menuMediaDat = UTILS_Malloc (sizeof(MENU_MEDIA_INST_S));
        if ( menuMediaDat == NULL )
        {
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
            break;
        }

        memset (menuMediaDat, 0, sizeof(MENU_MEDIA_INST_S));

        inst = menuMediaDat;

        #ifdef APPL_JPEG_ENABLED

        retVal = MENU_MediaPicInit (inst);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        retVal = MENU_MediaMusicInit (inst);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        #endif /* APPL_MP3_ENABLED */
    }
    while ( 0 );

    return retVal;
}

/*!
******************************************************************************
** \brief   With this function you enter the media menu.
**
** This function is called twice. First with \p test flag #FTRUE to check if
** the menu can be opend. If the first call was successful and return #FTRUE
** the second call will open the menu (\p test flag #FFALSE).
**
** \param   h       Handle of the media menu instance.
** \param   test    A flag to test if opening this menu is possible.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
fbool_t MENU_MediaEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_MEDIA_INST_S*  inst = menuMediaDat;
    FGS_MEMBER_S        member;
    fbool_t             retVal;
    uint8_t             cnt;

    if ( inst == NULL)
    {
        retVal = FFALSE;
    }
    else if ( test == FTRUE )
    {
        retVal = FTRUE;
    }
    else
    {
        #ifdef APPL_MHEG_ENABLED
        MHEG_Enable (FFALSE, SYS_SUSPENDSRC_USER);
        #endif

        inst->infoHeight = 0;

        memset (&member,      0, sizeof(member));
        memset (&(inst->pos), 0, sizeof(inst->pos));

        member.h         = (FAPI_SYS_HandleT)inst;
        member.pos       = &(inst->pos); // position is all zeros!
        member.id        = (int32_t)MENUID_MEDIA_ROOT;
        member.isVisible = FFALSE;
        member.inFront   = FFALSE;
        member.setFocus  = MENU_MediaSetFocus;
        member.refresh   = NULL;

        if ( FGS_AddMember (&member, &(inst->pRootMem)) != FAPI_OK )
        {
            retVal = FFALSE;
        }
        else
        {
            (void)MAIN_StopPresentation(); /* stop running presentation */

            /* TODO: Add background-I-Frame that will be overlayed by first
               picture. */

            for ( cnt = 0; cnt < MENU_MEDIA_TYPE_NUMS; cnt++ )
            {
                if ( inst->type[cnt].fileOpen != NULL )
                {
                    (void)(inst->type[cnt].fileOpen (inst->type + cnt));
                }
            }

            /* move focus to media root menu */
            (void)MENUSTACK_FocusClaim (MENUID_MEDIA_ROOT);

            MENUSTACK_SetCleanup (MENU_MediaLeave, (FAPI_SYS_HandleT)inst);

            (void)MENU_MediaSmallInfoOpen (inst, &(inst->smallInfo));

            /* enter file selection dialogue */
            (void)MENU_MediaDialogEnter (inst, &(inst->dialog));

            retVal = FTRUE;
        }
    }

    return retVal;
}

void MENU_MediaHandleEvent (SYS_EVTTYPE_E type, const void* data)
{
    MENU_MEDIA_INST_S*  inst = menuMediaDat;
    uint8_t             cnt;

    if ( inst->pRootMem != NULL )
    {
        for ( cnt = 0; cnt < MENU_MEDIA_TYPE_NUMS; cnt++ )
        {
            if ( inst->type[cnt].handleEvent != NULL )
            {
                inst->type[cnt].handleEvent (inst->type + cnt, type, data);
            }
        }
    }
}

void MENU_MediaHandleTime (void)
{
    MENU_MEDIA_INST_S*  inst  = menuMediaDat;
    uint8_t             cnt;

    if ( inst->pRootMem != NULL )
    {
        for ( cnt = 0; cnt < MENU_MEDIA_TYPE_NUMS; cnt++ )
        {
            if ( inst->type[cnt].handleTime != NULL )
            {
                inst->type[cnt].handleTime (inst->type + cnt);
            }
        }

        MENU_MediaSmallInfoHandleTime (&(inst->smallInfo));
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the media menu instance.
** \param   force   TODO
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_MediaLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_MEDIA_INST_S*  inst = (MENU_MEDIA_INST_S*)h;
    uint8_t             cnt;

    /* stop presentations */
    for ( cnt = 0; cnt < MENU_MEDIA_TYPE_NUMS; cnt++ )
    {
        if ( inst->type[cnt].fileClose != NULL )
        {
            (void)inst->type[cnt].fileClose (inst->type + cnt);
        }
    }

    /* close file viewer if currently open */
    (void)MAIN_ViewerClose();

    /* close dialogue if currently open */
    (void)MENU_MediaDialogLeave (&(inst->dialog));

    /* close information panels */
    (void)MENU_MediaSmallInfoClose (&(inst->smallInfo));

    (void)MAIN_RestartPresentation();

    (void)FGS_RemoveMember       (inst->pRootMem);
    (void)MENUSTACK_FocusRelease (MENUID_MEDIA_ROOT);

    inst->pRootMem = NULL;

    return FTRUE;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h           Handle of the media menu instance.
** \param   hasFocus    TODO
**
******************************************************************************
*/
FSTATIC int32_t MENU_MediaSetFocus (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_OK,     MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_INFO,   MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_LEFT,   MENU_MediaKeys, h, FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RIGHT,  MENU_MediaKeys, h, FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_UP,     MENU_MediaKeys, h, FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_DOWN,   MENU_MediaKeys, h, FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_F2,     MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_F3,     MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_F4,     MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_PLAY,   MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_STOP,   MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_PAUSE,  MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_FF,     MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_REW,    MENU_MediaKeys, h, FTRUE,  FFALSE);

        FGS_SetAppKeyHandler(FGS_KEY_RED,    MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_YELLOW, MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BLUE,   MENU_MediaKeys, h, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_GREEN,  MENU_MediaKeys, h, FTRUE,  FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_OK);
        FGS_ClearAppKeyHandler(FGS_KEY_INFO);
        FGS_ClearAppKeyHandler(FGS_KEY_LEFT);
        FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
        FGS_ClearAppKeyHandler(FGS_KEY_UP);
        FGS_ClearAppKeyHandler(FGS_KEY_DOWN);
        FGS_ClearAppKeyHandler(FGS_KEY_F2);
        FGS_ClearAppKeyHandler(FGS_KEY_F3);
        FGS_ClearAppKeyHandler(FGS_KEY_F4);
        FGS_ClearAppKeyHandler(FGS_KEY_PLAY);
        FGS_ClearAppKeyHandler(FGS_KEY_STOP);
        FGS_ClearAppKeyHandler(FGS_KEY_PAUSE);
        FGS_ClearAppKeyHandler(FGS_KEY_FF);
        FGS_ClearAppKeyHandler(FGS_KEY_REW);

        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_YELLOW);
        FGS_ClearAppKeyHandler(FGS_KEY_BLUE);
        FGS_ClearAppKeyHandler(FGS_KEY_GREEN);
    }

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h   Handle of the media menu instance.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_MediaKeys (FAPI_SYS_HandleT h)
{
    MENU_MEDIA_INST_S*   inst    = (MENU_MEDIA_INST_S*)h;
    fbool_t              keyUsed = FFALSE;
    FGS_KEY_E            currKey;

    currKey = FGS_GetCurrentKeyCode();

    switch ( currKey )
    {
        case FGS_KEY_OK:
            keyUsed = MENU_MediaDialogEnter (inst, &(inst->dialog));
            break;

        case FGS_KEY_INFO:
            keyUsed = MENU_MediaHandleInfoPanels (inst);
            break;

        case FGS_KEY_LEFT:
            #ifdef APPL_JPEG_ENABLED

            keyUsed = MENU_MediaPicZoomWindowMove (inst, currKey);

            if ( !keyUsed )
            {
                /* If not in zoom mode, select previous picture */
                keyUsed = MENU_MediaFilePrevious (inst,
                                                  MENU_MEDIA_TYPE_PICTURE);
            }

            #endif /* APPL_JPEG_ENABLED */
            break;

        case FGS_KEY_RIGHT:
            #ifdef APPL_JPEG_ENABLED

            keyUsed = MENU_MediaPicZoomWindowMove (inst, currKey);

            if ( !keyUsed )
            {
                /* If not in zoom mode, select previous picture */
                keyUsed = MENU_MediaFileNext (inst, MENU_MEDIA_TYPE_PICTURE);
            }

            #endif /* APPL_JPEG_ENABLED */
            break;

        #ifdef APPL_JPEG_ENABLED

        case FGS_KEY_UP:
        case FGS_KEY_DOWN:
            keyUsed = MENU_MediaPicZoomWindowMove (inst, currKey);
            break;

        case FGS_KEY_RED:
            keyUsed = MENU_MediaPicSlideshowOnOff (inst);
            break;

        case FGS_KEY_GREEN:
            keyUsed = MENU_MediaPicShuffleOnOff (inst);
            break;

        case FGS_KEY_YELLOW:
            keyUsed = MENU_MediaFilePrevious (inst, MENU_MEDIA_TYPE_PICTURE);
            break;

        case FGS_KEY_BLUE:
            keyUsed = MENU_MediaFileNext (inst, MENU_MEDIA_TYPE_PICTURE);
            break;

        case FGS_KEY_F2:
            keyUsed = MENU_MediaPicRotate (inst);
            break;

        case FGS_KEY_F3:
            keyUsed = MENU_MediaPicZoom (inst, FFALSE);
            break;

        case FGS_KEY_F4:
            keyUsed = MENU_MediaPicZoom (inst, FTRUE);
            break;

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        case FGS_KEY_PLAY:
        case FGS_KEY_STOP:
        case FGS_KEY_PAUSE:
            keyUsed = MENU_MediaMusicHandleKey (inst, currKey);
            break;

        case FGS_KEY_FF:
            keyUsed = MENU_MediaFileNext (inst, MENU_MEDIA_TYPE_MUSIC);
            break;

        case FGS_KEY_REW:
            keyUsed = MENU_MediaFilePrevious (inst, MENU_MEDIA_TYPE_MUSIC);
            break;

        #endif /* APPL_MP3_ENABLED */

        default:
            /* do nothing */
            break;
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_MediaHandleInfoPanels (MENU_MEDIA_INST_S* inst)
{
    MENU_MEDIA_DATA_S*  pTypePic      = inst->type + MENU_MEDIA_TYPE_PICTURE;
    MENU_MEDIA_DATA_S*  pTypeMusic    = inst->type + MENU_MEDIA_TYPE_MUSIC;
    fbool_t             picInfoOpen   = FFALSE;
    fbool_t             musicInfoOpen = FFALSE;

    if ( pTypePic->infoIsVisible != NULL )
    {
        picInfoOpen = pTypePic->infoIsVisible  (pTypePic);
    }

    if ( pTypeMusic->infoIsVisible != NULL )
    {
        musicInfoOpen = pTypeMusic->infoIsVisible (pTypeMusic);
    }

    if ( !picInfoOpen )
    {
        if ( musicInfoOpen )
        {
            if ( pTypeMusic->infoShow != NULL )
            {
                (void)pTypeMusic->infoShow (pTypeMusic, FFALSE);
            }
            else if ( pTypePic->infoShow != NULL )
            {
                (void)pTypePic->infoShow (pTypePic, FTRUE);
            }
        }
        else
        {
            if ( pTypePic->infoShow != NULL )
            {
                (void)pTypePic->infoShow (pTypePic, FTRUE);
            }
            else if ( pTypeMusic->infoShow != NULL )
            {
                (void)pTypeMusic->infoShow (pTypeMusic, FTRUE);
            }
        }
    }
    else
    {
        if ( !musicInfoOpen )
        {
            if ( pTypeMusic->infoShow != NULL )
            {
                (void)pTypeMusic->infoShow (pTypeMusic, FTRUE);
            }
            else if ( pTypePic->infoShow != NULL )
            {
                (void)pTypePic->infoShow (pTypePic, FFALSE);
            }
        }
        else
        {
            if ( pTypePic->infoShow != NULL )
            {
                (void)pTypePic->infoShow (pTypePic, FFALSE);
            }
            else if ( pTypeMusic->infoShow != NULL )
            {
                (void)pTypeMusic->infoShow (pTypePic, FFALSE);
            }
        }
    }

    return FTRUE;
}

/* ------------------------------------------------------------------------- */

FSTATIC fbool_t MENU_MediaFileStart (MENU_MEDIA_DATA_S* pType,
                                     const char_t* name, uint16_t index)
{
    (void)UTILS_strncpy (pType->name, name, MAIN_VIEWER_NAMELEN);

    pType->currIndex = index;

    if ( pType->fileStart != NULL )
    {
        pType->active = FTRUE;

        (void)pType->fileStart (pType, pType->path, pType->name, FTRUE);

        if ( pType->shuffle )
        {
            /* mark current item in shuffle array */
            (void)MENU_MediaShuffleSet (pType, pType->currIndex);
        }

        if ( pType->infoUpdate != NULL )
        {
            (void)pType->infoUpdate (pType);
        }

        /* show info menu */
        (void)MENU_MediaSmallInfoShow (&(pType->pRoot->smallInfo), FTRUE,
                                       FFALSE);
    }

    return FTRUE;
}

FSTATIC fbool_t MENU_MediaOnFileSelect (MENU_MEDIA_DATA_S* pType,
                                        const char_t* path,
                                        const char_t* name, uint16_t nameLenMax,
                                        uint16_t index, uint16_t numItems)
{
    fbool_t keyUsed = FFALSE;

    if ( pType->fileIsDecodable (pType, name, nameLenMax) )
    {
        if ( (pType->active)
          && (pType->shuffle) )
        {
            if ( UTILS_strncmpi(pType->path, path, -1) != 0 )
            {
                /* Reset shuffle array if path has changed since last file. */
                (void)MENU_MediaShuffleClear(pType);
            }
        }

        (void)UTILS_strncpy (pType->path, path, MAIN_VIEWER_PATHLEN);

        pType->numItems = numItems;

        /* close file viewer */
        (void)MAIN_ViewerClose();

        /* reclaim focus for media root */
        (void)MENUSTACK_FocusClaim (MENUID_MEDIA_ROOT);

        (void)MENU_MediaFileStart (pType, name, index);

        keyUsed = FTRUE;
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_MediaFileNext (MENU_MEDIA_INST_S* inst,
                                    MENU_MEDIA_TYPE_E type)
{
    MENU_MEDIA_DATA_S*      pType = inst->type + type;
    fbool_t                 keyUsed = FFALSE;
    int32_t                 retVal;
    uint16_t                newIndex = 0;
    char_t                  newName[MAIN_VIEWER_NAMELEN];

    newName[0] = 0; /* satisfy MISRA */

    if ( (pType->active)
      && (pType->numItems > 1) )
    {
        if ( pType->shuffle )
        {
            keyUsed = MENU_MediaGetFileByShuffle (pType, &newIndex, newName,
                                                  MAIN_VIEWER_NAMELEN);
        }
        else
        {
            /* search for next item in current directory if not already at end
               of list */
            if ( (pType->currIndex + 1) < pType->numItems )
            {
                retVal = MAIN_ViewerGetFile (pType->path, pType->currIndex,
                                             MAIN_VIEWER_GETFILE_NEXT,
                                             pType->fileIsDecodable,
                                             pType, &newIndex,
                                             newName, MAIN_VIEWER_NAMELEN);

                if ( retVal == FAPI_OK )
                {
                    keyUsed = FTRUE;
                }
            }

            if ( !keyUsed )
            {
                /* If next item can not be found, search for first item in list. */
                retVal = MAIN_ViewerGetFile (pType->path, 0,
                                             MAIN_VIEWER_GETFILE_FIRST,
                                             pType->fileIsDecodable,
                                             pType, &newIndex,
                                             newName, MAIN_VIEWER_NAMELEN);

                /* Ensure that result is not current item! */
                if ( (retVal   == FAPI_OK)
                  && (newIndex != pType->currIndex) )
                {
                    keyUsed = FTRUE;
                }
            }
        }
    }

    if ( keyUsed )
    {
        (void)MENU_MediaFileStart (pType, newName, newIndex);
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_MediaFilePrevious (MENU_MEDIA_INST_S* inst,
                                        MENU_MEDIA_TYPE_E type)
{
    MENU_MEDIA_DATA_S*      pType = inst->type + type;
    fbool_t                 keyUsed = FFALSE;
    int32_t                 retVal;
    uint16_t                newIndex = 0;
    char_t                  newName[MAIN_VIEWER_NAMELEN];

    newName[0] = 0; /* satisfy MISRA */

    if ( (pType->active)
      && (pType->numItems > 1) )
    {
        if ( pType->shuffle )
        {
            keyUsed = MENU_MediaGetFileByShuffle (pType, &newIndex, newName,
                                                  MAIN_VIEWER_NAMELEN);
        }
        else
        {
            /* search for previous item in current directory if not already at
               beginning of list */
            if ( pType->currIndex > 0 )
            {
                retVal = MAIN_ViewerGetFile (pType->path, pType->currIndex,
                                             MAIN_VIEWER_GETFILE_PREVIOUS,
                                             pType->fileIsDecodable,
                                             pType, &newIndex,
                                             newName, MAIN_VIEWER_NAMELEN);

                if ( retVal == FAPI_OK )
                {
                    keyUsed = FTRUE;
                }
            }

            if ( !keyUsed )
            {
                /* If previous item can not be found, search for last item in
                   list. */
                retVal = MAIN_ViewerGetFile (pType->path, 0,
                                             MAIN_VIEWER_GETFILE_LAST,
                                             pType->fileIsDecodable,
                                             pType, &newIndex,
                                             newName, MAIN_VIEWER_NAMELEN);

                /* Ensure that result is not current item! */
                if ( (retVal   == FAPI_OK)
                  && (newIndex != pType->currIndex) )
                {
                    keyUsed = FTRUE;
                }
            }
        }
    }

    if ( keyUsed )
    {
        (void)MENU_MediaFileStart (pType, newName, newIndex);
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_MediaGetFileByShuffle (MENU_MEDIA_DATA_S* pType,
                                            uint16_t* pNewIndex,
                                            char_t* pNewName,
                                            uint16_t nameLenMax)
{
    int32_t newIndex;
    int32_t retVal;
    fbool_t keyUsed = FFALSE;

    if ( pType->active )
    {
        do
        {
            newIndex = MENU_MediaShuffleGetNextIdx (pType);

            if ( newIndex >= 0 )
            {
                retVal = MAIN_ViewerGetFile (pType->path, (uint16_t)newIndex,
                                             MAIN_VIEWER_GETFILE_THIS,
                                             pType->fileIsDecodable,
                                             pType, NULL,
                                             pNewName, nameLenMax);

                if ( retVal == FAPI_OK )
                {
                    *pNewIndex = (uint16_t)newIndex;
                    keyUsed    = FTRUE;
                    break;
                }
                else
                {
                    /* mark non-decodable files so they are not selected
                       again */
                    (void)MENU_MediaShuffleSet (pType, newIndex);
                }
            }
        }
        while ( pType->shufItems < pType->numItems );
    }

    return keyUsed;
}

FSTATIC int32_t MENU_MediaShuffleClear (MENU_MEDIA_DATA_S* pType)
{
    uint16_t cnt;

    pType->shufItems = 0;

    for ( cnt = 0; cnt < MENU_MEDIA_SHUFFLE_LW_MAX; cnt++ )
    {
        pType->shufBM[cnt] = 0;
    }

    return FAPI_OK;
}

FSTATIC int32_t MENU_MediaShuffleSet (MENU_MEDIA_DATA_S* pType, int32_t index)
{
    uint16_t lwIdx;
    int32_t  retVal = SMARTGO_APPL_ERR_PROHIBITED;

    if ( (index >= 0)
      && ((uint16_t)index < pType->numItems) )
    {
        lwIdx = (uint16_t)(index / MENU_MEDIA_BITS_PER_LW);

        if ( lwIdx < MENU_MEDIA_SHUFFLE_LW_MAX )
        {
            if ( (pType->shufBM[lwIdx] & (1 << (index % MENU_MEDIA_BITS_PER_LW)))
                 == 0 )
            {
                pType->shufBM[lwIdx] |= (1 << (index % MENU_MEDIA_BITS_PER_LW));

                pType->shufItems++;
                retVal = FAPI_OK;
            }
        }
    }

    return retVal;
}

FSTATIC int32_t MENU_MediaShuffleGetNextIdx (MENU_MEDIA_DATA_S* pType)
{
    int32_t     retVal = -1;
    uint16_t    randVal;
    uint16_t    lwIdx;
    uint16_t    cnt;
    uint16_t    freeCnt;
    uint16_t    maxItems;

    if ( pType->shufItems < pType->numItems )
    {
        maxItems = pType->numItems - pType->shufItems;

        if ( maxItems > MENU_MEDIA_SHUFFLE_ITEMS_MAX )
        {
            maxItems = MENU_MEDIA_SHUFFLE_ITEMS_MAX;
        }

        /*lint -save -e571 */
        randVal = (uint16_t)((((uint64_t)rand())
                            * ((uint64_t)maxItems))
                            / (((uint64_t)RAND_MAX) + 1));
        /*lint -restore */

        /* should never happen. Note that maxItems is ensured to be >= 1. */
        if ( randVal >= maxItems )
        {
            randVal = maxItems - 1;
        }

        lwIdx   = 0;
        freeCnt = 0;

        for ( cnt = 0; cnt < pType->numItems; cnt++ )
        {
            /* is item free? */
            if ( (pType->shufBM[lwIdx] & (1 << (cnt % MENU_MEDIA_BITS_PER_LW)))
                  == 0 )
            {
                if ( (freeCnt == randVal) )
                {
                    retVal = (int32_t)cnt;
                    break;
                }
                else
                {
                    freeCnt++;
                }
            }

            if ( ((cnt + 1) % MENU_MEDIA_BITS_PER_LW) == 0 )
            {
                lwIdx++;
            }
        }
    }

    return retVal;
}

FSTATIC uint32_t MENU_MediaGetFileExtension (const char_t* name,
                                             uint16_t nameLenMax)
{
    uint16_t    cnt;
    uint32_t    extBin = 0;

    for ( cnt = 0; cnt < nameLenMax; cnt++ )
    {
        if ( name[cnt] == 0 )
        {
            break;
        }
    }

    if ( (cnt < nameLenMax)
      && (cnt >= 4) )
    {
        if ( name[cnt - 4] == '.' )
        {
            extBin = (extBin << 8) | name[cnt - 3];
            extBin = (extBin << 8) | name[cnt - 2];
            extBin = (extBin << 8) | name[cnt - 1];
        }
    }

    return extBin;
}

/* ------------------------------------------------------------------------- */

FSTATIC fbool_t MENU_MediaDialogEnter (MENU_MEDIA_INST_S* rootInst,
                                       MENU_MEDIA_DIALOG_INST_S* inst)
{
    fbool_t retVal = FFALSE;

    if ( inst->pnlHdl == NULL )
    {
        inst->pRoot = rootInst;

        if ( MENU_MediaDialogBuild (inst) == FAPI_OK )
        {
            (void)FGS_LIST_SetMemberFocus  (inst->listHdl, 0);
            (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl,
                                            MENU_MEDIA_DIALOG_ID_LIST);

            (void)MENUSTACK_FocusClaim     (MENUID_MEDIA_DIALOG);
            (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);

            retVal = FTRUE;
        }
    }

    return retVal;
}

FSTATIC fbool_t MENU_MediaDialogLeave (MENU_MEDIA_DIALOG_INST_S* inst)
{
    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);

        inst->pnlHdl = NULL;

        (void)MENUSTACK_FocusRelease (MENUID_MEDIA_DIALOG);
    }

    return FTRUE;
}

FSTATIC int32_t MENU_MediaDialogBuild (MENU_MEDIA_DIALOG_INST_S* inst)
{
    FGS_PANEL_OPEN_S    pnlPars;
    FGS_LIST_OPEN_S     listPars;
    FAPI_SYS_HandleT    itemHdl;
    FGS_ITEM_TXT_OPEN_S txtPars;
    FGS_POS_S           itemPos;
    FGS_POS_S           descrPos;
    int32_t             retVal;
    uint16_t            helpWidth;
    uint16_t            stackedFrameWidth = 0;
    uint16_t            innerHeight;
    uint32_t            numLines = MENU_MEDIA_DIALOG_ID_ENTRY_NUMS;

    helpWidth   = (2 * SCALER_GetCurrVal(HELP_LR_PADDING))
                + OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_OK), -1, SYMBOL_FONT);
    innerHeight = numLines *
                  (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                 + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth,
                                        1, NULL);

        pnlPars.onFocusSet      = MENU_MediaDialogOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;
        pnlPars.pos.alignment   = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
        pnlPars.pos.x           = 0;
        pnlPars.pos.y           = 0;

        retVal = FGS_PANEL_Open (&pnlPars, MENUID_MEDIA_DIALOG,
                                 &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle (MENU_MEDIA_DIALOG_ID_TITLE,
                                      inst->pnlHdl, pnlPars.pos.width,
                                      stackedFrameWidth, STR_TITLE_MEDIA_DIALOG,
                                      NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset(&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_MediaDialogGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_MediaDialogOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_MediaDialogOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars,
                                            MENU_MEDIA_DIALOG_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, NULL, 100,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        memset (&txtPars,  0, sizeof(txtPars));

        txtPars.bkgrType   = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos   = descrPos;

        /* ----- add menu lines -------------------------------------------- */

        #ifdef APPL_JPEG_ENABLED

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars,
                                    MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_PIC,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_JPEG_ENABLED */

        /* ----------------------------------------------------------------- */

        #ifdef APPL_MP3_ENABLED

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars,
                                    MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_MUSIC,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_MP3_ENABLED */

        /* ----------------------------------------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_MEDIA_DIALOG_ID_ENTRY_LEAVE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Create Help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList (
                                        MENU_MEDIA_DIALOG_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH),
                                        listPars.pos.y, helpWidth, numLines,
                                        MENU_MediaDialogGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine (stackedFrameWidth
                          + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                          + SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                          - ((int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2)
                          - SCALER_GetCurrVal(HELP_TB_PADDING)
                          - SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                            pnlPars.pos.width
                          - stackedFrameWidth
                          - (2 * SCALER_GetCurrVal(OUTER_FRAME_ALL))
                          - SCALER_GetCurrVal(MAIN_LEFT_PADDING)
                          - SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                            inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);

            inst->pnlHdl = NULL;
        }
    }

    return retVal;
}

FSTATIC void MENU_MediaDialogOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    if ( hasFocus )
    {
        FGS_SetAppKeyHandler (FGS_KEY_MENU, MENU_MediaDialogKeys, h, FTRUE,
                              FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_BACK, MENU_MediaDialogKeys, h, FTRUE,
                              FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_EXIT, MENU_MediaDialogKeys, h, FTRUE,
                              FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
    }
}

FSTATIC fbool_t MENU_MediaDialogKeys (FAPI_SYS_HandleT h)
{
    MENU_MEDIA_DIALOG_INST_S*   inst    = (MENU_MEDIA_DIALOG_INST_S*)h;
    fbool_t                     keyUsed;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
        case FGS_KEY_BACK:
        case FGS_KEY_EXIT:
            keyUsed = MENU_MediaDialogLeave (inst);

            /* reclaim focus for media root */
            (void)MENUSTACK_FocusClaim (MENUID_MEDIA_ROOT);
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

FSTATIC void MENU_MediaDialogOnFocusChg (FAPI_SYS_HandleT h, uint8_t index,
                                           int32_t id)
{
    MENU_MEDIA_DIALOG_INST_S*   inst = (MENU_MEDIA_DIALOG_INST_S*)h;
    FGS_LNAVI_PARAMS_S          naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = (uint16_t)id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}

FSTATIC fbool_t MENU_MediaDialogOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_MEDIA_DIALOG_INST_S*   inst     = (MENU_MEDIA_DIALOG_INST_S*)h;
    fbool_t                     retVal   = FFALSE;
    MAIN_VIEWER_CONFIG_S        viewerCfg;

    switch ( itemId )
    {
        #ifdef APPL_JPEG_ENABLED

        case MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_PIC:
            memset (&viewerCfg, 0, sizeof(viewerCfg));

            viewerCfg.titleString       = OSD_GetStr(STR_TITLE_VIEWER);
            viewerCfg.allowStackedView  = FFALSE;
            viewerCfg.allowedFileOps    = 0;
            viewerCfg.allowedPicOps     = 0;
            viewerCfg.allowedMusicOps   = 0;
            viewerCfg.allowedMovieOps   = 0;

            (void)UTILS_strncpy (viewerCfg.initialPath,
                                 inst->pRoot->type[MENU_MEDIA_TYPE_PICTURE].
                                 path, MAIN_VIEWER_PATHLEN);

            // TODO: set type according to pic/music choice ?
            viewerCfg.filterType[0]     = MAIN_VIEWER_FILTYPE_ALL;

            viewerCfg.onBack            = MENU_MediaViewerOnBack;
            viewerCfg.onExit            = MENU_MediaViewerOnBack;
            viewerCfg.onSelect          = MENU_MediaPicOnSelect;
            viewerCfg.cbArg             = inst->pRoot;

            /* close media dialogue */
            (void)MENU_MediaDialogLeave (inst);

            /* open file viewer */
            retVal = MAIN_ViewerOpen (&viewerCfg);

            if ( !retVal )
            {
                /* reclaim focus for media root */
                (void)MENUSTACK_FocusClaim (MENUID_MEDIA_ROOT);
            }
            break;

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        case MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_MUSIC:
            memset (&viewerCfg, 0, sizeof(viewerCfg));

            viewerCfg.titleString       = OSD_GetStr(STR_TITLE_VIEWER);
            viewerCfg.allowStackedView  = FFALSE;
            viewerCfg.allowedFileOps    = 0;
            viewerCfg.allowedPicOps     = 0;
            viewerCfg.allowedMusicOps   = 0;
            viewerCfg.allowedMovieOps   = 0;

            (void)UTILS_strncpy (viewerCfg.initialPath,
                                 inst->pRoot->type[MENU_MEDIA_TYPE_MUSIC].
                                 path, MAIN_VIEWER_PATHLEN);

            // TODO: set type according to pic/music choice ?
            viewerCfg.filterType[0]     = MAIN_VIEWER_FILTYPE_ALL;

            viewerCfg.onBack            = MENU_MediaViewerOnBack;
            viewerCfg.onExit            = MENU_MediaViewerOnBack;
            viewerCfg.onSelect          = MENU_MediaMusicOnSelect;
            viewerCfg.cbArg             = inst->pRoot;

            /* close media dialogue */
            (void)MENU_MediaDialogLeave (inst);

            /* open file viewer */
            retVal = MAIN_ViewerOpen (&viewerCfg);

            if ( !retVal )
            {
                /* reclaim focus for media root */
                (void)MENUSTACK_FocusClaim (MENUID_MEDIA_ROOT);
            }
            break;

        #endif /* APPL_MP3_ENABLED */

        case MENU_MEDIA_DIALOG_ID_ENTRY_LEAVE:
            retVal = MENUSTACK_Back (NULL);
            break;

        default:
            retVal = FFALSE;
            break;
    }

    return retVal;
}

FSTATIC void MENU_MediaDialogGetDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item)
{
    uint32_t strIdx = 0;

    switch ( id )
    {
        #ifdef APPL_JPEG_ENABLED

        case MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_PIC:
            strIdx = STR_ENTRY_MEDIA_DIALOG_OPENPIC;
            break;

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        case MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_MUSIC:
            strIdx = STR_ENTRY_MEDIA_DIALOG_OPENMUSIC;
            break;

        #endif /* APPL_MP3_ENABLED */

        case MENU_MEDIA_DIALOG_ID_ENTRY_LEAVE:
            strIdx = STR_ENTRY_MEDIA_DIALOG_LEAVE;
            break;

        default:
            return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_MediaDialogGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                               uint32_t index,
                                               FGS_ITEMSTATE_E state,
                                               FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        #ifdef APPL_JPEG_ENABLED
        case MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_PIC:
        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED
        case MENU_MEDIA_DIALOG_ID_ENTRY_OPEN_MUSIC:
        #endif /* APPL_MP3_ENABLED */

        case MENU_MEDIA_DIALOG_ID_ENTRY_LEAVE:
            strIdx = STR_ENTRY_OK;
            break;

        default:
            return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

/* ------------------------------------------------------------------------- */

FSTATIC fbool_t MENU_MediaViewerOnBack (void* arg)
{
    /* close file viewer */
    (void)MAIN_ViewerClose ();

    /* reclaim focus for media root */
    (void)MENUSTACK_FocusClaim (MENUID_MEDIA_ROOT);

    return FTRUE;
}

/* ------------------------------------------------------------------------- */

FSTATIC int32_t MENU_MediaSmallInfoOpen (MENU_MEDIA_INST_S* rootInst,
                                         MENU_MEDIA_SMALLINFO_INST_S* inst)
{
    FGS_PANEL_OPEN_S    pnlPars;
    FGS_BOX_OPEN_S      boxPars;
    FAPI_SYS_HandleT    itemHdl;
    int32_t             retVal   = FAPI_OK;
    uint32_t            numLines = 0;

    do
    {
        #ifdef APPL_JPEG_ENABLED
        numLines++;
        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED
        numLines++;
        #endif /* APPL_MP3_ENABLED */

        if ( inst->pnlHdl != NULL )
        {
            retVal = SMARTGO_APPL_ERR_ALREADY_OPEN;
            break;
        }
        else if ( numLines < 1 )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        memset (inst, 0, sizeof(MENU_MEDIA_SMALLINFO_INST_S));

        inst->pRoot = rootInst;

        memset (&pnlPars, 0, sizeof(pnlPars));

        pnlPars.pos.x         = SCALER_GetCurrVal(MAIN_HOR_OFFSET);
        pnlPars.pos.y         = SCALER_GetCurrVal(MAIN_VER_OFFSET);
        pnlPars.pos.width     = SCALER_GetCurrVal(MEDIA_SMALLINFO_WIDTH)
                              + SCALER_GetCurrVal(MAIN_LEFT_PADDING)
                              + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        pnlPars.pos.height    = (numLines
                                 * SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                              + ((numLines - 1)
                                 * SCALER_GetCurrVal(LIST_LINE_PADDING))
                              + (2 * SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* NOTE: LIST_LINE_PADDING is used as top/bottom padding */

        pnlPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        pnlPars.bkgrType      = OSD_PNLTYPE_BLACKRECTANGLE;
        pnlPars.inFront       = FFALSE;
        pnlPars.pInst         = &(inst->pnlInst);

        retVal = FGS_PANEL_Open (&pnlPars, MENUID_MEDIA_SMALLINFO,
                                 &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        memset (&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x               = SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = SCALER_GetCurrVal(LIST_LINE_PADDING);
        boxPars.pos.width           = SCALER_GetCurrVal(MEDIA_SMALLINFO_WIDTH);
        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_BLACK;
        boxPars.getItem             = MENU_MediaSmallInfoGetItem;
        boxPars.getItemArg          = (FAPI_SYS_HandleT)inst;

        #ifdef APPL_JPEG_ENABLED

        boxPars.pInst = inst->boxInst + MENU_MEDIA_TYPE_PICTURE;

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_SMALLINFO_ID_PICNAME,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                       + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        boxPars.pInst = inst->boxInst + MENU_MEDIA_TYPE_MUSIC;

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_SMALLINFO_ID_MUSICNAME,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        #endif /* APPL_MP3_ENABLED */

        /* ----------------------------------------------------------------- */
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);

            inst->pnlHdl = NULL;
        }
    }

    return retVal;
}

FSTATIC int32_t MENU_MediaSmallInfoClose (MENU_MEDIA_SMALLINFO_INST_S* inst)
{
    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close (inst->pnlHdl);

        inst->pnlHdl = NULL;
    }

    return FAPI_OK;
}

FSTATIC int32_t MENU_MediaSmallInfoShow (MENU_MEDIA_SMALLINFO_INST_S* inst,
                                         fbool_t showOrHide,
                                         fbool_t permanent)
{
    int32_t retVal = FAPI_OK;

    if ( inst->pnlHdl != NULL )
    {
        if ( showOrHide )
        {
            if ( FGS_PANEL_IsVisible(inst->pnlHdl) != 0 )
            {
                /* already visible, refresh file display */

                #ifdef APPL_JPEG_ENABLED

                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MENU_MEDIA_SMALLINFO_ID_PICNAME);

                #endif /* APPL_JPEG_ENABLED */

                #ifdef APPL_MP3_ENABLED

                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                             MENU_MEDIA_SMALLINFO_ID_MUSICNAME);

                #endif /* APPL_MP3_ENABLED */

                if ( inst->displayTime != 0 )
                {
                    if ( permanent )
                    {
                        /* store request for permanent view */
                        inst->displayTime = 0;
                    }
                    else
                    {
                        /* refresh display time */
                        inst->displayTime = UTILS_GetTimerStamp();
                    }
                }
            }
            else
            {
                retVal = FGS_PANEL_Show (inst->pnlHdl, FFALSE);

                if ( permanent )
                {
                    inst->displayTime = 0;
                }
                else
                {
                    inst->displayTime = UTILS_GetTimerStamp();
                }
            }
        }
        else
        {
            retVal = FGS_PANEL_Hide (inst->pnlHdl);

            inst->displayTime = 0;
        }
    }
    else
    {
        retVal = SMARTGO_APPL_ERR_PROHIBITED;
    }

    return retVal;
}

FSTATIC void MENU_MediaSmallInfoHandleTime (MENU_MEDIA_SMALLINFO_INST_S* inst)
{
    if ( inst->pnlHdl != NULL )
    {
        if ( inst->displayTime != 0 )
        {
            if ( UTILS_GetTimerStampInterval (inst->displayTime,
                                              UTILS_GetTimerStamp())
                 >= MENU_MEDIA_SMALLINFO_DISPLAYTIME )
            {
                /* hide panel if it opened in non-permanent mode */
                (void)MENU_MediaSmallInfoShow (inst, FFALSE, FFALSE);
            }
        }
    }
}

FSTATIC void MENU_MediaSmallInfoGetItem (FAPI_SYS_HandleT h, int32_t id,
                                         uint32_t index,
                                         FGS_ITEMSTATE_E state,
                                         FGS_ITEM_S* item)
{
    MENU_MEDIA_SMALLINFO_INST_S* inst     = (MENU_MEDIA_SMALLINFO_INST_S*)h;
    const char_t*                filename = NULL;

    switch ( id )
    {
        #ifdef APPL_JPEG_ENABLED

        case MENU_MEDIA_SMALLINFO_ID_PICNAME:
            if ( inst->pRoot->type[MENU_MEDIA_TYPE_PICTURE].active )
            {
                filename = inst->pRoot->type[MENU_MEDIA_TYPE_PICTURE].name;
            }
            break;

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        case MENU_MEDIA_SMALLINFO_ID_MUSICNAME:
            if ( inst->pRoot->type[MENU_MEDIA_TYPE_MUSIC].active )
            {
                filename = inst->pRoot->type[MENU_MEDIA_TYPE_MUSIC].name;
            }
            break;

        #endif /* APPL_MP3_ENABLED */

        default:
            /* do nothing */
            break;
    }

    if ( filename != NULL )
    {
        item->type                = FGS_ITEM_TYPE_STRING;
        item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item->item.str.chr        = STR_GetASCII(filename);
        item->item.str.font       = MAIN_FONT;
        item->item.str.lineHeight = 0;
        item->item.str.col        = OSD_COL_WHITE;
    }
}

/* ------------------------------------------------------------------------- */

#ifdef APPL_JPEG_ENABLED

FSTATIC fbool_t MENU_MediaPicOnSelect (void* arg, MAIN_VIEWER_FILETYPE type,
                                       const char_t* path, uint16_t pathLenMax,
                                       const char_t* name, uint16_t nameLenMax,
                                       uint16_t index, uint16_t numItems)
{
    MENU_MEDIA_INST_S*  inst    = (MENU_MEDIA_INST_S*)arg;
    fbool_t             keyUsed = FFALSE;

    if ( type == MAIN_VIEWER_FILETYPE_FILTERED )
    {
        keyUsed = MENU_MediaOnFileSelect (inst->type + MENU_MEDIA_TYPE_PICTURE,
                                          path, name, nameLenMax,
                                          index, numItems);
    }

    return keyUsed;
}

FSTATIC int32_t MENU_MediaPicInit (MENU_MEDIA_INST_S* inst)
{
    MENU_MEDIA_DATA_S* pType = inst->type + MENU_MEDIA_TYPE_PICTURE;

    pType->pRoot     = inst;
    pType->pParams   = &(inst->pic);
    pType->pInfo     = &(inst->pic.info);
    pType->active    = FFALSE;
    pType->single    = MENU_MEDIA_PIC_DEFAULT_SINGLE;
    pType->shuffle   = MENU_MEDIA_PIC_DEFAULT_SHUFFLE;
    pType->path[0]   = 0;
    pType->name[0]   = 0;
    pType->currIndex = 0;
    pType->numItems  = 0;

    pType->fileOpen        = MENU_MediaPicOpen;
    pType->fileClose       = MENU_MediaPicClose;
    pType->fileIsDecodable = MENU_MediaPicIsDecodable;
    pType->fileStart       = MENU_MediaPicShow;
    pType->handleTime      = MENU_MediaPicHandleTime;
    pType->handleEvent     = MENU_MediaPicHandleEvent;
    pType->infoShow        = MENU_MediaPicInfoShow;
    pType->infoIsVisible   = MENU_MediaPicInfoIsVisible;
    pType->infoUpdate      = MENU_MediaPicInfoUpdate;

    return FAPI_OK;
}

FSTATIC int32_t MENU_MediaPicOpen (void* arg)
{
    MENU_MEDIA_DATA_S*      pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_PIC_S*       pPic  = (MENU_MEDIA_PIC_S*)(pType->pParams);
    FGS_POS_S               framePos;
    JPEG_SCREEN_PARAMS_S    screenParams;
    int32_t                 retVal = FAPI_OK;

    do
    {
        pType->active       = FFALSE;
        pType->single       = MENU_MEDIA_PIC_DEFAULT_SINGLE;
        pType->shuffle      = MENU_MEDIA_PIC_DEFAULT_SHUFFLE;

        pPic->orientation   = ROT_0;
        pPic->zoom          = MENU_MEDIA_ZOOM_MIN;
        pPic->rasterPosX    = 0;
        pPic->rasterPosY    = 0;
        pPic->rasterEntries = 1;

        pPic->format        = PIC_TYPE_UNKNOWN;
        pPic->sizeX         = 0;
        pPic->sizeY         = 0;
        pPic->zoomRotateAvl = FFALSE;

        (void)MENU_MediaShuffleClear (pType);

        /* configure JPEG module */
        memset (&framePos, 0, sizeof(framePos));

        FGS_GetFrameDimensions(&framePos);

        memset (&screenParams, 0, sizeof(screenParams));

        screenParams.pos.left  = (uint16_t)framePos.x;
        screenParams.pos.top   = (uint16_t)framePos.y;
        screenParams.pos.hsize = (uint16_t)framePos.width;
        screenParams.pos.vsize = (uint16_t)framePos.height;

        retVal = JPEG_Open (&screenParams);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        retVal = MENU_MediaPicInfoOpen (pType->pRoot, pType);
        if ( retVal != FAPI_OK )
        {
            break;
        }
    }
    while ( 0 );

    return retVal;
}

FSTATIC int32_t MENU_MediaPicClose (void* arg)
{
    MENU_MEDIA_DATA_S* pType = (MENU_MEDIA_DATA_S*)arg;

    (void)MENU_MediaPicInfoClose (pType);

    (void)JPEG_Close();

    return FAPI_OK;
}

FSTATIC int32_t MENU_MediaPicShow (void* arg, const char_t* path,
                                   const char_t* name, fbool_t newFile)
{
    MENU_MEDIA_DATA_S*  pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_PIC_S*   pPic  = (MENU_MEDIA_PIC_S*)(pType->pParams);
    JPEG_PIC_PARAMS_S   params;

    if ( newFile )
    {
        /* Clear size information, will be given by event after start
           of decoding. */
        pPic->sizeX = 0;
        pPic->sizeY = 0;

        /* Reset zoom  and rotation status. */
        pPic->orientation   = ROT_0;
        pPic->zoom          = MENU_MEDIA_ZOOM_MIN;
        pPic->rasterPosX    = 0;
        pPic->rasterPosY    = 0;
        pPic->rasterEntries = 1;

        /* get picture type */

        // TODO: get type via PIC API before decoding actually starts!

        pPic->format = MENU_MediaPicGetType (name, MAIN_VIEWER_NAMELEN);

        /* NOTE: This reflects the current implementation status of picture
                 decoders. In the future, zoom and rotation features shall
                 be available for all picture types. */
        switch ( pPic->format )
        {
            case PIC_TYPE_JPEG:
                pPic->zoomRotateAvl = FTRUE;
                break;

            default:
                pPic->zoomRotateAvl = FFALSE;
                break;
        }
    }

    /* build complete path string by appending 'name' to 'path' */
    MENU_CommonAppendpath (name, path, pType->fullPath, FFALSE,
                           MAIN_VIEWER_PATHLEN);

    memset (&params, 0, sizeof(params));

    params.path      = pType->fullPath;
    params.filepos   = 0;
    params.orient    = pPic->orientation;
    params.zoom      = pPic->zoom;

    if ( pPic->rasterEntries > 1 )
    {
        params.xpos = (pPic->rasterPosX * MENU_MEDIA_ZOOM_MIN)
                    / (pPic->rasterEntries - 1);
        params.ypos = (pPic->rasterPosY * MENU_MEDIA_ZOOM_MIN)
                    / (pPic->rasterEntries - 1);
    }
    else
    {
        params.xpos = 0;
        params.ypos = 0;
    }

    (void)JPEG_Show (&params, NULL);

    pPic->displayTime = UTILS_GetTimerStamp();

    return FAPI_OK;
}

FSTATIC fbool_t MENU_MediaPicIsDecodable (void* arg,
                                          const char_t* name,
                                          uint16_t nameLenMax)
{
    uint32_t    extBin;
    fbool_t     retVal;

    extBin = MENU_MediaGetFileExtension (name, nameLenMax);

    switch ( extBin )
    {
        case MAIN_VIEWER_FILTYPE_JPG_LC:
        case MAIN_VIEWER_FILTYPE_JPG_UC:
        case MAIN_VIEWER_FILTYPE_BMP_LC:
        case MAIN_VIEWER_FILTYPE_BMP_UC:
        case MAIN_VIEWER_FILTYPE_PNG_LC:
        case MAIN_VIEWER_FILTYPE_PNG_UC:
            retVal = FTRUE;
            break;

        default:
            retVal = FFALSE;
            break;
    }

    return retVal;
}

FSTATIC PIC_TYPE_E MENU_MediaPicGetType (const char_t* name,
                                         uint16_t nameLenMax)
{
    uint32_t    extBin;
    PIC_TYPE_E  picType;

    extBin = MENU_MediaGetFileExtension (name, nameLenMax);

    switch ( extBin )
    {
        case MAIN_VIEWER_FILTYPE_JPG_LC:
        case MAIN_VIEWER_FILTYPE_JPG_UC:
            picType = PIC_TYPE_JPEG;
            break;

        case MAIN_VIEWER_FILTYPE_BMP_LC:
        case MAIN_VIEWER_FILTYPE_BMP_UC:
            picType = PIC_TYPE_BMP;
            break;

        case MAIN_VIEWER_FILTYPE_PNG_LC:
        case MAIN_VIEWER_FILTYPE_PNG_UC:
            picType = PIC_TYPE_PNG;
            break;

        default:
            picType = PIC_TYPE_UNKNOWN;
            break;
    }

    return picType;
}

FSTATIC void MENU_MediaPicHandleTime (void* arg)
{
    MENU_MEDIA_DATA_S*  pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_PIC_S*   pPic  = (MENU_MEDIA_PIC_S*)(pType->pParams);

    /* slideshow runtime handling */
    if ( (pType->active)
     && !(pType->single) )
    {
        if ( UTILS_GetTimerStampInterval (pPic->displayTime,
                                          UTILS_GetTimerStamp())
             >= MENU_MEDIA_SLIDESHOW_DISPLAYTIME )
        {
            (void)MENU_MediaFileNext (pType->pRoot, MENU_MEDIA_TYPE_PICTURE);
        }
    }
}

FSTATIC void MENU_MediaPicHandleEvent (void* arg, SYS_EVTTYPE_E type,
                                       const void* data)
{
    MENU_MEDIA_DATA_S*         pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_PIC_S*          pPic  = (MENU_MEDIA_PIC_S*)(pType->pParams);
    MENU_MEDIA_PICINFO_INST_S* pInfo = (MENU_MEDIA_PICINFO_INST_S*)
                                       (pType->pInfo);

    switch ( type )
    {
        case EVT_JPEG_FILEINFO:
            if ( pType->active )
            {
                if ( pPic->sizeX == 0 )
                {
                    pPic->sizeX = ((JPEG_EVT_INFO_S*)data)->sizeX;
                    pPic->sizeY = ((JPEG_EVT_INFO_S*)data)->sizeY;

                    if ( pInfo->pnlHdl != NULL )
                    {
                        (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                              MENU_MEDIA_PICINFO_ID_RESOLUTION);
                    }
                }
            }
            break;

        default:
            /* do nothing */
            break;
    }
}

FSTATIC fbool_t MENU_MediaPicSlideshowOnOff (MENU_MEDIA_INST_S* inst)
{
    MENU_MEDIA_DATA_S*          pType = inst->type + MENU_MEDIA_TYPE_PICTURE;
    MENU_MEDIA_PIC_S*           pPic  = (MENU_MEDIA_PIC_S*)(pType->pParams);
    MENU_MEDIA_PICINFO_INST_S*  pInfo = (MENU_MEDIA_PICINFO_INST_S*)
                                        (pType->pInfo);
    fbool_t                     keyUsed;

    if ( pType->active )
    {
        if ( pType->single ) /* start slideshow */
        {
            /* refresh display time in case picture is already active */
            pPic->displayTime = UTILS_GetTimerStamp ();

            pType->single = FFALSE;
        }
        else /* stop slideshow */
        {
            pType->single = FTRUE;
        }

        if ( pInfo->pnlHdl != NULL )
        {
            (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_SLIDESHOWSTAT);
            (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_DESC);
        }

        keyUsed = FTRUE;
    }
    else
    {
        keyUsed = FFALSE;
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_MediaPicShuffleOnOff (MENU_MEDIA_INST_S* inst)
{
    MENU_MEDIA_DATA_S*          pType = inst->type + MENU_MEDIA_TYPE_PICTURE;
    MENU_MEDIA_PICINFO_INST_S*  pInfo = (MENU_MEDIA_PICINFO_INST_S*)
                                        (pType->pInfo);
    fbool_t                     keyUsed;

    if ( pType->active )
    {
        if ( !(pType->shuffle) ) /* start shuffle */
        {
            (void)MENU_MediaShuffleClear (pType);

            /* mark current picture as already set */
            (void)MENU_MediaShuffleSet (pType, pType->currIndex);

            pType->shuffle = FTRUE;
        }
        else /* stop shuffle */
        {
            pType->shuffle = FFALSE;
        }

        if ( pInfo->pnlHdl != NULL )
        {
            (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_SHUFFLESTAT);
            (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_HELP_SHUFFLE_DESC);
        }

        keyUsed = FTRUE;
    }
    else
    {
        keyUsed = FFALSE;
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_MediaPicZoomWindowMove (MENU_MEDIA_INST_S* inst,
                                             FGS_KEY_E key)
{
    MENU_MEDIA_DATA_S*          pType   = inst->type + MENU_MEDIA_TYPE_PICTURE;
    MENU_MEDIA_PIC_S*           pPic    = (MENU_MEDIA_PIC_S*)(pType->pParams);
    MENU_MEDIA_PICINFO_INST_S*  pInfo;
    fbool_t                     update  = FFALSE;

    if ( pPic->zoom > MENU_MEDIA_ZOOM_MIN )
    {
        switch ( key )
        {
            case FGS_KEY_LEFT:
                if ( pPic->rasterPosX > 0 )
                {
                    pPic->rasterPosX--;
                    update = FTRUE;
                }
                break;

            case FGS_KEY_RIGHT:
                if ( (pPic->rasterPosX + 1) < pPic->rasterEntries )
                {
                    pPic->rasterPosX++;
                    update = FTRUE;
                }
                break;

            case FGS_KEY_UP:
                if ( pPic->rasterPosY > 0 )
                {
                    pPic->rasterPosY--;
                    update = FTRUE;
                }
                break;

            case FGS_KEY_DOWN:
                if ( (pPic->rasterPosY + 1) < pPic->rasterEntries )
                {
                    pPic->rasterPosY++;
                    update = FTRUE;
                }
                break;

            default:
                /* do nothing */
                break;
        }
    }

    if ( update )
    {
        /* stop slideshow if currently active */
        pType->single = FTRUE;

        pInfo = (MENU_MEDIA_PICINFO_INST_S*)(pType->pInfo);

        if ( pInfo->pnlHdl != NULL )
        {
            (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                           MENU_MEDIA_PICINFO_ID_ZOOMBOX);
            (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                           MENU_MEDIA_PICINFO_ID_SLIDESHOWSTAT);
        }

        /* update picture */
        if ( pType->fileStart != NULL )
        {
            (void)pType->fileStart (pType, pType->path, pType->name, FFALSE);
        }
    }

    return update;
}

FSTATIC fbool_t MENU_MediaPicRotate (MENU_MEDIA_INST_S* inst)
{
    MENU_MEDIA_DATA_S*          pType  = inst->type + MENU_MEDIA_TYPE_PICTURE;
    MENU_MEDIA_PIC_S*           pPic   = (MENU_MEDIA_PIC_S*)(pType->pParams);
    MENU_MEDIA_PICINFO_INST_S*  pInfo;
    fbool_t                     keyUsed;

    if ( !(pType->active) )
    {
        keyUsed = FFALSE;
    }
    else if ( !(pPic->zoomRotateAvl) )
    {
        keyUsed = FFALSE;
    }
    else
    {
        switch ( pPic->orientation )
        {
            case ROT_0:   pPic->orientation = ROT_90;  break;
            case ROT_90:  pPic->orientation = ROT_180; break;
            case ROT_180: pPic->orientation = ROT_270; break;
            default:
            case ROT_270: pPic->orientation = ROT_0;   break;
        }

        /* stop slideshow if currently active */
        pType->single = FTRUE;

        pInfo = (MENU_MEDIA_PICINFO_INST_S*)(pType->pInfo);

        if ( pInfo->pnlHdl != NULL )
        {
            (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_ROTSTAT);
            (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_SLIDESHOWSTAT);
            (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_DESC);
        }

        /* update picture */
        if ( pType->fileStart != NULL )
        {
            (void)pType->fileStart (pType, pType->path, pType->name,
                                    FFALSE);
        }

        keyUsed = FTRUE;
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_MediaPicZoom (MENU_MEDIA_INST_S* inst, fbool_t zoomIn)
{
    MENU_MEDIA_DATA_S*          pType  = inst->type + MENU_MEDIA_TYPE_PICTURE;
    MENU_MEDIA_PIC_S*           pPic   = (MENU_MEDIA_PIC_S*)(pType->pParams);
    MENU_MEDIA_PICINFO_INST_S*  pInfo;
    fbool_t                     update = FFALSE;
    fbool_t                     keyUsed;
    uint8_t                     rasterPosX;
    uint8_t                     rasterPosY;
    uint8_t                     rasterEntries;

    if ( !(pType->active) )
    {
        keyUsed = FFALSE;
    }
    else if ( !(pPic->zoomRotateAvl) )
    {
        /* NOTE: Currently zoom and rotate feature is only available for JPEG
                 pictures. */
        keyUsed = FFALSE;
    }
    else
    {
        if ( zoomIn )
        {
            if ( (pPic->zoom + MENU_MEDIA_ZOOM_STEP) <= MENU_MEDIA_ZOOM_MAX )
            {
                pPic->zoom += MENU_MEDIA_ZOOM_STEP;
                update      = FTRUE;
            }
        }
        else
        {
            if ( pPic->zoom >= (MENU_MEDIA_ZOOM_MIN + MENU_MEDIA_ZOOM_STEP) )
            {
                pPic->zoom -= MENU_MEDIA_ZOOM_STEP;

                update = FTRUE;
            }
        }

        if ( update )
        {
            /* NOTE: Result will never be less than 0. */
            rasterEntries = (uint8_t)(((pPic->zoom / MENU_MEDIA_ZOOM_MIN) * 2)
                                      - 1);

            if ( pPic->rasterEntries > 1 )
            {
                if ( rasterEntries > 1 )
                {
                    rasterPosX = (uint8_t)
                                ((pPic->rasterPosX * (rasterEntries - 1))
                               / (pPic->rasterEntries - 1));
                    rasterPosY = (uint8_t)
                                ((pPic->rasterPosY * (rasterEntries - 1))
                               / (pPic->rasterEntries - 1));
                }
                else
                {
                    rasterPosX = 0;
                    rasterPosY = 0;
                }
            }
            else
            {
                if ( rasterEntries > 1 )
                {
                    rasterPosX = (rasterEntries - 1) / 2;
                    rasterPosY = (rasterEntries - 1) / 2;
                }
                else
                {
                    rasterPosX = 0;
                    rasterPosY = 0;
                }
            }

            pPic->rasterPosX    = rasterPosX;
            pPic->rasterPosY    = rasterPosY;
            pPic->rasterEntries = rasterEntries;

            /* stop slideshow if currently active */
            pType->single = FTRUE;

            pInfo = (MENU_MEDIA_PICINFO_INST_S*)(pType->pInfo);

            if ( pInfo->pnlHdl != NULL )
            {
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_ZOOMSTAT);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_ZOOMBOX);

                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_HELP_ZOOMIN_DESC);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_HELP_ZOOMOUT_DESC);

                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_SLIDESHOWSTAT);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_DESC);
            }

            /* update picture */
            if ( pType->fileStart != NULL )
            {
                (void)pType->fileStart (pType, pType->path, pType->name,
                                        FFALSE);
            }

            keyUsed = FTRUE;
        }
        else
        {
            keyUsed = FFALSE;
        }
    }

    return keyUsed;
}

/* ------------------------------------------------------------------------- */

FSTATIC int32_t MENU_MediaPicInfoOpen (MENU_MEDIA_INST_S* rootInst,
                                       MENU_MEDIA_DATA_S* pType)
{
    MENU_MEDIA_PICINFO_INST_S*  inst = (MENU_MEDIA_PICINFO_INST_S*)
                                       (pType->pInfo);
    FGS_PANEL_OPEN_S            pnlPars;
    FGS_BOX_OPEN_S              boxPars;
    FAPI_SYS_HandleT            itemHdl;
    int32_t                     retVal   = FAPI_OK;
    uint8_t                     instCnt  = 0;
    uint8_t                     numLines = MENU_MEDIA_PICINFO_NUMLINES;
    uint16_t                    tempPos;
    uint16_t                    helpColWidth;
    uint16_t                    helpKeyWidth;
    uint16_t                    helpDescWidth;
    uint16_t                    posXStart;
    uint16_t                    posYStart;

    do
    {
        if ( inst->pnlHdl != NULL )
        {
            retVal = SMARTGO_APPL_ERR_ALREADY_OPEN;
            break;
        }

        memset (inst, 0, sizeof(MENU_MEDIA_PICINFO_INST_S));

        memset (&pnlPars, 0, sizeof(pnlPars));

        pnlPars.bkgrType      = OSD_PNLTYPE_MENUWTITLE;
        pnlPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_WIDTH)
                              + (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        pnlPars.pos.height    = (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH))
                              + OSD_GetPanelTitleHeight(pnlPars.bkgrType)
                              + (numLines
                               * SCALER_GetCurrVal(LIST_LINE_HEIGHT));
        pnlPars.pos.x         = 0;
        pnlPars.pos.y         = -(int16_t)
                                 (SCALER_GetCurrVal(MAIN_PAD_Y)
                                + pnlPars.pos.height
                                + rootInst->infoHeight);
        pnlPars.pos.alignment = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_BOTTOM;
        pnlPars.pInst         = &(inst->pnlInst);

        retVal = FGS_PANEL_Open (&pnlPars, MENUID_MEDIA_PICINFO,
                                 &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* Store height information for next menu to be build. Add
           OUTER_FRAME_WIDTH padding distance to adjacent panel. */
        rootInst->infoHeight += pnlPars.pos.height
                              + SCALER_GetCurrVal(OUTER_FRAME_WIDTH);

        /* ----- add box for drawing additional items ---------------------- */

        memset (&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x         = 0;
        boxPars.pos.y         = 0;
        boxPars.pos.width     = pnlPars.pos.width;
        boxPars.pos.height    = pnlPars.pos.height;
        boxPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType      = OSD_BOX_BKGR_NONE;
        boxPars.getItem       = MENU_MediaPicInfoGetItem;
        boxPars.getItemArg    = (FAPI_SYS_HandleT)pType;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_REST,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----- Add title line items -------------------------------------- */

        /* right-side-orientiered items first */

        boxPars.pos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        boxPars.pos.height    = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType      = OSD_BOX_BKGR_TITLE;

        boxPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_PIC_ZOOM_WIDTH);
        boxPars.pos.x         = -(int16_t)
                                 (SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                + SCALER_GetCurrVal(MAIN_RIGHT_PADDING)
                                + boxPars.pos.width);
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_ZOOMSTAT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_PIC_ROT_WIDTH);
        boxPars.pos.x        -= (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X))
                              + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + boxPars.pos.width;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_ROTSTAT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_PIC_RES_WIDTH);
        boxPars.pos.x        -= (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X))
                              + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + boxPars.pos.width;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_RESOLUTION,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_PIC_SLIDE_WIDTH);
        boxPars.pos.x        -= (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X))
                              + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + boxPars.pos.width;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_SLIDESHOWSTAT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_SHUFFLE_WIDTH);
        boxPars.pos.x        -= (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X))
                              + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + boxPars.pos.width;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_SHUFFLESTAT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        tempPos               = -(boxPars.pos.x)
                              + (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X))
                              + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

        /* reserve remaining space in title line for filename */
        boxPars.pos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                              + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width     = pnlPars.pos.width
                              - boxPars.pos.x
                              - tempPos;
        boxPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_FILENAME,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----- add inner items ------------------------------------------- */

        /* NOTE: MAIN_TOP_PADDING is used as top/bottom/right padding for zoom
                 box, being positioned in right corner of picture info
                 display. */

        boxPars.pos.height          = MENU_MEDIA_PIC_ZOOMBOX_SIZE;
        boxPars.pos.width           = boxPars.pos.height;
        boxPars.pos.x               = -(int16_t)
                                       (SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                                      + SCALER_GetCurrVal(MAIN_TOP_PADDING)
                                      + boxPars.pos.width);
        boxPars.pos.y               = SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                                    + OSD_GetPanelTitleHeight(pnlPars.bkgrType)
                                    + SCALER_GetCurrVal(MAIN_TOP_PADDING);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_RIGHT
                                    | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.pInst               = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_ZOOMBOX,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----- add help items -------------------------------------------- */

        /* Calculate total width usable for help items. */
        helpColWidth =
              pnlPars.pos.width
            - MENU_MEDIA_PIC_ZOOMBOX_SIZE           /* zoom box width */
            - SCALER_GetCurrVal(OUTER_FRAME_WIDTH)  /* right of zoom box 1 */
            - SCALER_GetCurrVal(MAIN_TOP_PADDING)   /* right of zoom box 2 */
            - SCALER_GetCurrVal(OUTER_FRAME_ALL)    /* left padding 1 */
            - SCALER_GetCurrVal(MAIN_LEFT_PADDING); /* left padding 2 */

        helpColWidth /= 3;                  /* split into three columns */
        helpKeyWidth  = helpColWidth / 4;   /* split column into key and
                                               description area */
        helpDescWidth = helpColWidth
                      - helpKeyWidth
                      - SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

        /* store top/left position */
        posXStart    = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                     + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        posYStart    = SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                     + OSD_GetPanelTitleHeight(pnlPars.bkgrType);

        /* 1st column */

        memset (&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x         = posXStart;
        boxPars.pos.y         = posYStart;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType      = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem       = MENU_MediaPicInfoGetItem;
        boxPars.getItemArg    = (FAPI_SYS_HandleT)pType;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_NEXT_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_NEXT_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x         = posXStart;
        boxPars.pos.y        += boxPars.pos.height;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_PREV_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_PREV_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x         = posXStart;
        boxPars.pos.y        += boxPars.pos.height;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars,
                               MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars,
                               MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* 2nd column */

        boxPars.pos.x         = posXStart + helpColWidth;
        boxPars.pos.y         = posYStart;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_ZOOMIN_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_ZOOMIN_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x         = posXStart + helpColWidth;
        boxPars.pos.y        += boxPars.pos.height;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_ZOOMOUT_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars,
                               MENU_MEDIA_PICINFO_ID_HELP_ZOOMOUT_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x         = posXStart + helpColWidth;
        boxPars.pos.y        += boxPars.pos.height;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_SHUFFLE_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars,
                               MENU_MEDIA_PICINFO_ID_HELP_SHUFFLE_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* 3rd column */

        boxPars.pos.x         = posXStart + (2 * helpColWidth);
        boxPars.pos.y         = posYStart;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_ROTATE_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_PICINFO_ID_HELP_ROTATE_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /*
        BLUE    next pic        F4      Zoom in       F2      Rotate
        YELLOW  prev pic        F3      Zoom out
        RED     slideshow       GREEN   shuffle
        */
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);

            inst->pnlHdl = NULL;
        }
    }

    return retVal;
}

FSTATIC int32_t MENU_MediaPicInfoClose (MENU_MEDIA_DATA_S* pType)
{
    MENU_MEDIA_PICINFO_INST_S*  inst = (MENU_MEDIA_PICINFO_INST_S*)
                                       (pType->pInfo);

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close (inst->pnlHdl);

        inst->pnlHdl = NULL;
    }

    return FAPI_OK;
}

FSTATIC int32_t MENU_MediaPicInfoShow (void* arg,
                                       fbool_t showOrHide)
{
    MENU_MEDIA_DATA_S*          pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_PICINFO_INST_S*  inst  = (MENU_MEDIA_PICINFO_INST_S*)
                                        (pType->pInfo);
    int32_t                     retVal;

    if ( inst->pnlHdl != NULL )
    {
        if ( showOrHide )
        {
            retVal = FGS_PANEL_Show (inst->pnlHdl, FFALSE);
        }
        else
        {
            retVal = FGS_PANEL_Hide (inst->pnlHdl);
        }
    }
    else
    {
        retVal = SMARTGO_APPL_ERR_PROHIBITED;
    }

    return retVal;
}

FSTATIC fbool_t MENU_MediaPicInfoIsVisible (void* arg)
{
    MENU_MEDIA_DATA_S*          pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_PICINFO_INST_S*  inst  = (MENU_MEDIA_PICINFO_INST_S*)
                                        (pType->pInfo);
    fbool_t                     isVisible;

    if ( inst->pnlHdl != NULL )
    {
        isVisible = FGS_PANEL_IsVisible (inst->pnlHdl);
    }
    else
    {
        isVisible = FFALSE;
    }

    return isVisible;
}

FSTATIC int32_t MENU_MediaPicInfoUpdate (void* arg)
{
    MENU_MEDIA_DATA_S*          pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_PICINFO_INST_S*  inst  = (MENU_MEDIA_PICINFO_INST_S*)
                                        (pType->pInfo);

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_FILENAME);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_RESOLUTION);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_ROTSTAT);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_ZOOMSTAT);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_ZOOMBOX);

        MENU_MediaPicInfoRefreshHelp (inst);
    }

    return FAPI_OK;
}

FSTATIC void MENU_MediaPicInfoRefreshHelp (MENU_MEDIA_PICINFO_INST_S* inst)
{
    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_NEXT_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_NEXT_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_PREV_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_PREV_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                      MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                     MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_ZOOMIN_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_ZOOMIN_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_ZOOMOUT_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_ZOOMOUT_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_SHUFFLE_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_SHUFFLE_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_ROTATE_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_PICINFO_ID_HELP_ROTATE_DESC);
    }
}

FSTATIC void MENU_MediaPicInfoGetItem (FAPI_SYS_HandleT h, int32_t id,
                                       uint32_t index,
                                       FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item)
{
    MENU_MEDIA_DATA_S*         pType    = (MENU_MEDIA_DATA_S*)h;
    MENU_MEDIA_PIC_S*          pPic     = (MENU_MEDIA_PIC_S*)(pType->pParams);
    const char_t*              pStr     = NULL;
    fbool_t                    titleStr = FFALSE;
    uint32_t                   strIdx;

    switch ( id )
    {
        case MENU_MEDIA_PICINFO_ID_REST:
            item->type            = FGS_ITEM_TYPE_GENERIC;
            item->align           = FGS_ALIGN_HOR_CENTER
                                  | FGS_ALIGN_VER_CENTER;
            item->item.gnr.cb     = MENU_MediaPicInfoDrawRest;
            item->item.gnr.cbArg  = NULL;
            break;

        case MENU_MEDIA_PICINFO_ID_FILENAME:
            if ( pType->active )
            {
                item->item.str.chr = STR_GetASCII(pType->name);
            }
            else
            {
                item->item.str.chr = OSD_GetStr(
                                     STR_ENTRY_MEDIA_PIC_NONESELECTED);
            }

            item->align        = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            titleStr           = FTRUE;
            break;

        case MENU_MEDIA_PICINFO_ID_SLIDESHOWSTAT:

            if ( (pType->active)
            && (!(pType->single)) )
            {
                item->align        = FGS_ALIGN_HOR_CENTER
                                   | FGS_ALIGN_VER_CENTER;
                item->item.str.chr = OSD_GetStr(
                                     STR_ENTRY_MEDIA_SLIDESHOW_ACTIVE);
                titleStr           = FTRUE;
            }
            break;

        case MENU_MEDIA_PICINFO_ID_SHUFFLESTAT:

            if ( (pType->active)
              && (pType->shuffle) )
            {
                item->align        = FGS_ALIGN_HOR_CENTER
                                   | FGS_ALIGN_VER_CENTER;
                item->item.str.chr = OSD_GetStr(STR_ENTRY_MEDIA_SHUFFLE_ACTIVE);
                titleStr           = FTRUE;
            }
            break;

        case MENU_MEDIA_PICINFO_ID_RESOLUTION:

            if ( (!(pType->active))
                || (pPic->sizeX == 0) )
            {
                item->item.str.chr = OSD_GetStr (STR_EMPTYSTRING);
            }
            else
            {
                sprintf (appDat->asciiBuf, "%dx%d", pPic->sizeX,
                                                    pPic->sizeY);

                item->item.str.chr = STR_IntASCIIStr();
            }

            item->align = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;

            titleStr = FTRUE;
            break;

        case MENU_MEDIA_PICINFO_ID_ROTSTAT:

            if ( pType->active )
            {
                switch ( pPic->orientation )
                {
                    default:
                    case ROT_0:   pStr = "0";
                                  break;

                    case ROT_90:  pStr = "90";
                                  break;

                    case ROT_180: pStr = "180";
                                  break;

                    case ROT_270: pStr = "270";
                                  break;
                }

                item->align        = FGS_ALIGN_HOR_CENTER
                                   | FGS_ALIGN_VER_CENTER;
                item->item.str.chr = STR_GetASCII(pStr);
                titleStr           = FTRUE;
            }
            break;

        case MENU_MEDIA_PICINFO_ID_ZOOMSTAT:

            if ( pType->active )
            {
                item->type                      = FGS_ITEM_TYPE_NUMBER;
                item->align                     = FGS_ALIGN_HOR_RIGHT
                                                | FGS_ALIGN_VER_CENTER;
                item->item.number.value         = (int32_t)(pPic->zoom);
                item->item.number.format        = FGS_FORMAT_NUM_NORM;
                item->item.number.decimalPlaces = 0;
                item->item.number.unitStr       = OSD_GetStr (STR_UNIT_PERCENT);
                item->item.number.font          = TITLE_FONT;
                item->item.number.col           = OSD_COL_TITLE_TEXT;
            }
            break;

        case MENU_MEDIA_PICINFO_ID_ZOOMBOX:

            if ( (pType->active)
              && (pPic->zoomRotateAvl) )
            {
                item->type                      = FGS_ITEM_TYPE_GENERIC;
                item->align                     = FGS_ALIGN_HOR_CENTER
                                                | FGS_ALIGN_VER_CENTER;
                item->item.gnr.cb               = MENU_MediaPicInfoDrawZoomBox;
                item->item.gnr.cbArg            = pPic;
            }
            break;

        /* ----- help items ------------------------------------------------ */

        case MENU_MEDIA_PICINFO_ID_HELP_NEXT_KEY:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_BUTTON(item, ICON_BTNBLUESMALL);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_NEXT_DESC:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_MEDIA_HELP_PIC_NEXT);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_PREV_KEY:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_BUTTON(item, ICON_BTNYELLOWSMALL);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_PREV_DESC:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_MEDIA_HELP_PIC_PREV);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_KEY:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_BUTTON(item, ICON_BTNREDSMALL);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_SLIDESHOW_DESC:

            if ( pType->active )
            {
                if ( pType->single )
                {
                    strIdx = STR_ENTRY_MEDIA_HELP_PIC_SLIDESHOW_START;
                }
                else
                {
                    strIdx = STR_ENTRY_MEDIA_HELP_PIC_SLIDESHOW_STOP;
                }

                MENU_COMMON_ITEM_HELPSTR(item, strIdx);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_ZOOMIN_KEY:

            if ( (pType->active)
              && (pPic->zoomRotateAvl) )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_F4);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_ZOOMIN_DESC:

            if ( (pType->active)
              && (pPic->zoomRotateAvl) )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_JPEG_ZOOMIN);

                /* disable if max. zoom factor is reached */
                if ( pPic->zoom >= MENU_MEDIA_ZOOM_MAX )
                {
                    item->item.str.col = appDat->txtAttr.colDisabled;
                }
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_ZOOMOUT_KEY:

            if ( (pType->active)
              && (pPic->zoomRotateAvl) )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_F3);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_ZOOMOUT_DESC:

            if ( (pType->active)
              && (pPic->zoomRotateAvl) )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_JPEG_ZOOMOUT);

                /* disable if min. zoom factor is reached */
                if ( pPic->zoom <= MENU_MEDIA_ZOOM_MIN )
                {
                    item->item.str.col = appDat->txtAttr.colDisabled;
                }
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_SHUFFLE_KEY:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_BUTTON(item, ICON_BTNGREENSMALL);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_SHUFFLE_DESC:

            if ( pType->active )
            {
                if ( !(pType->shuffle) )
                {
                    strIdx = STR_ENTRY_MEDIA_HELP_SHUFFLE_ON;
                }
                else
                {
                    strIdx = STR_ENTRY_MEDIA_HELP_SHUFFLE_OFF;
                }

                MENU_COMMON_ITEM_HELPSTR(item, strIdx);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_ROTATE_KEY:

            if ( (pType->active)
              && (pPic->zoomRotateAvl) )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_F2);
            }
            break;

        case MENU_MEDIA_PICINFO_ID_HELP_ROTATE_DESC:

            if ( (pType->active)
              && (pPic->zoomRotateAvl) )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_JPEG_ROTATE);
            }
            break;

        default:
            /* do nothing */
            break;
    }

    if ( titleStr )
    {
        item->type                = FGS_ITEM_TYPE_STRING;
        item->item.str.font       = TITLE_FONT;
        item->item.str.lineHeight = 0;
        item->item.str.col        = OSD_COL_TITLE_TEXT;
    }
}

FSTATIC void MENU_MediaPicInfoDrawZoomBox (FAPI_SYS_HandleT h, uint8_t align,
                                           uint16_t x, uint16_t y,
                                           uint16_t width, uint16_t height)
{
    MENU_MEDIA_PIC_S*   pPic = (MENU_MEDIA_PIC_S*)h;
    float               zoomX;
    float               zoomY;
    float               zoomWidth;
    uint16_t            borderWidth = 2; // TODO: add scalable parameter
    uint16_t            zoomXAbs;
    uint16_t            zoomYAbs;

    /* draw background */
    OSD_DrawFilledRectangle (x, y, x + width - 1, y + height - 1,
                             OSD_COL_BLACK);

    /*lint -save -e790 */
    zoomWidth = ((float)((width - (2 * borderWidth)) * MENU_MEDIA_ZOOM_MIN))
              / ((float)(pPic->zoom));
    /*lint -restore */

    zoomX     = (pPic->rasterPosX * (zoomWidth / 2));
    zoomY     = (pPic->rasterPosY * (zoomWidth / 2));

    zoomXAbs = x + borderWidth + (uint16_t)zoomX;
    zoomYAbs = y + borderWidth + (uint16_t)zoomY;

    /* snap grid on zoom box frame */
    if ( (zoomXAbs + (uint16_t)zoomWidth + 1) == (x + width - borderWidth) )
    {
        zoomXAbs++;
    }

    if ( (zoomYAbs + (uint16_t)zoomWidth + 1) == (y + width - borderWidth) )
    {
        zoomYAbs++;
    }

    /* draw zoom window */
    OSD_DrawFilledRectangle (zoomXAbs, zoomYAbs,
                             zoomXAbs + (uint16_t)zoomWidth - 1,
                             zoomYAbs + (uint16_t)zoomWidth - 1,
                             OSD_COL_WHITE);
}

FSTATIC void MENU_MediaPicInfoDrawRest (FAPI_SYS_HandleT h, uint8_t align,
                                        uint16_t x, uint16_t y,
                                        uint16_t width, uint16_t height)
{
    uint32_t lineHeight = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
    uint16_t posY       = y + SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
    uint16_t posX       = x + width;

    /* Draw vertical separators in top line */

    posX -= SCALER_GetCurrVal(OUTER_FRAME_ALL)
          + SCALER_GetCurrVal(MAIN_RIGHT_PADDING)
          + SCALER_GetCurrVal(MEDIA_INFO_PIC_ZOOM_WIDTH)
          + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(posX, posY, lineHeight);

    posX -= SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(MEDIA_INFO_PIC_ROT_WIDTH)
          + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(posX, posY, lineHeight);

    posX -= SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(MEDIA_INFO_PIC_RES_WIDTH)
          + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(posX, posY, lineHeight);

    posX -= SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(MEDIA_INFO_PIC_SLIDE_WIDTH)
          + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(posX, posY, lineHeight);

    posX -= SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(MEDIA_INFO_SHUFFLE_WIDTH)
          + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(posX, posY, lineHeight);
}

#endif /* APPL_JPEG_ENABLED */

/* ------------------------------------------------------------------------- */

#ifdef APPL_MP3_ENABLED

FSTATIC fbool_t MENU_MediaMusicOnSelect (void* arg, MAIN_VIEWER_FILETYPE type,
                                         const char_t* path,
                                         uint16_t pathLenMax,
                                         const char_t* name,
                                         uint16_t nameLenMax,
                                         uint16_t index, uint16_t numItems)
{
    MENU_MEDIA_INST_S*  inst    = (MENU_MEDIA_INST_S*)arg;
    fbool_t             keyUsed = FFALSE;

    if ( type == MAIN_VIEWER_FILETYPE_FILTERED )
    {
        keyUsed = MENU_MediaOnFileSelect (inst->type + MENU_MEDIA_TYPE_MUSIC,
                                          path, name, nameLenMax,
                                          index, numItems);
    }

    return keyUsed;
}

FSTATIC int32_t MENU_MediaMusicInit (MENU_MEDIA_INST_S* inst)
{
    MENU_MEDIA_DATA_S* pType = inst->type + MENU_MEDIA_TYPE_MUSIC;

    pType->pRoot     = inst;
    pType->pParams   = &(inst->music);
    pType->pInfo     = &(inst->music.info);
    pType->active    = FFALSE;
    pType->single    = MENU_MEDIA_MUSIC_DEFAULT_SINGLE;
    pType->shuffle   = MENU_MEDIA_MUSIC_DEFAULT_SHUFFLE;
    pType->path[0]   = 0;
    pType->name[0]   = 0;
    pType->currIndex = 0;
    pType->numItems  = 0;

    pType->fileOpen        = MENU_MediaMusicOpen;
    pType->fileClose       = MENU_MediaMusicClose;
    pType->fileIsDecodable = MENU_MediaMusicIsDecodable;
    pType->fileStart       = MENU_MediaMusicStart;
    pType->handleEvent     = MENU_MediaMusicHandleEvent;
    pType->infoShow        = MENU_MediaMusicInfoShow;
    pType->infoIsVisible   = MENU_MediaMusicInfoIsVisible;
    pType->infoUpdate      = MENU_MediaMusicInfoUpdate;

    return FAPI_OK;
}

FSTATIC int32_t MENU_MediaMusicOpen (void* arg)
{
    MENU_MEDIA_DATA_S*      pType  = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_MUSIC_S*     pMusic = (MENU_MEDIA_MUSIC_S*)(pType->pParams);
    int32_t                 retVal = FAPI_OK;

    do
    {
        pType->active          = FFALSE;
        pType->single          = MENU_MEDIA_MUSIC_DEFAULT_SINGLE;
        pType->shuffle         = MENU_MEDIA_MUSIC_DEFAULT_SHUFFLE;

        pMusic->playStatus     = MENU_MEDIA_PLAY_STATUS_NONE;
        pMusic->timeCurr       = 0;
        pMusic->timeTotal      = 0;
        pMusic->waitForStopEvt = FFALSE;

        memset (&(pMusic->fileInfo), 0, sizeof(pMusic->fileInfo));

        retVal = MENU_MediaMusicInfoOpen (pType->pRoot, pType);
        if ( retVal != FAPI_OK )
        {
            break;
        }
    }
    while ( 0 );

    return retVal;
}

FSTATIC int32_t MENU_MediaMusicClose (void* arg)
{
    MENU_MEDIA_DATA_S* pType = (MENU_MEDIA_DATA_S*)arg;

    (void)MENU_MediaMusicInfoClose (pType);

    /* stop MP3 playback if currently active */
    (void)MENU_MediaMusicStop (pType);

    return FAPI_OK;
}

FSTATIC fbool_t MENU_MediaMusicIsDecodable (void* arg,
                                            const char_t* name,
                                            uint16_t nameLenMax)
{
    uint32_t    extBin;
    fbool_t     retVal;

    extBin = MENU_MediaGetFileExtension (name, nameLenMax);

    switch ( extBin )
    {
        case MAIN_VIEWER_FILTYPE_MP3_LC:
        case MAIN_VIEWER_FILTYPE_MP3_UC:
            retVal = FTRUE;
            break;

        default:
            retVal = FFALSE;
            break;
    }

    return retVal;
}

FSTATIC int32_t MENU_MediaMusicStart (void* arg, const char_t* path,
                                      const char_t* name, fbool_t newFile)
{
    MENU_MEDIA_DATA_S*  pType  = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_MUSIC_S* pMusic = (MENU_MEDIA_MUSIC_S*)(pType->pParams);
    int32_t             retVal;

    /* build complete path string by appending 'name' to 'path' */
    MENU_CommonAppendpath (name, path, pType->fullPath, FFALSE,
                           MAIN_VIEWER_PATHLEN);

    (void)MENU_MediaMusicStop (pType);

    if ( newFile )
    {
        pMusic->timeTotal  = 0;

        memset (&(pMusic->fileInfo), 0, sizeof(pMusic->fileInfo));

        (void)MP3_GetInfo (pType->fullPath, &(pMusic->fileInfo));
    }

    retVal = MP3_Start (pType->fullPath);

    if ( retVal == FAPI_OK )
    {
        pMusic->playStatus = MENU_MEDIA_PLAY_STATUS_PLAY;
    }

    return retVal;
}

FSTATIC fbool_t MENU_MediaMusicPause (MENU_MEDIA_DATA_S* pType,
                                      fbool_t pauseOn)
{
    fbool_t             keyUsed = FFALSE;
    MENU_MEDIA_MUSIC_S* pMusic = (MENU_MEDIA_MUSIC_S*)(pType->pParams);

    if ( pType->active )
    {
        if ( MP3_Pause (pauseOn) == FAPI_OK )
        {
            if ( pauseOn )
            {
                pMusic->playStatus = MENU_MEDIA_PLAY_STATUS_PAUSE;
            }
            else
            {
                pMusic->playStatus = MENU_MEDIA_PLAY_STATUS_PLAY;
            }

            keyUsed = FTRUE;
        }
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_MediaMusicStop (MENU_MEDIA_DATA_S* pType)
{
    fbool_t             keyUsed = FFALSE;
    MENU_MEDIA_MUSIC_S* pMusic = (MENU_MEDIA_MUSIC_S*)(pType->pParams);

    if ( pType->active )
    {
        if ( (pMusic->playStatus != MENU_MEDIA_PLAY_STATUS_NONE)
          && (pMusic->playStatus != MENU_MEDIA_PLAY_STATUS_STOP) )
        {
            if ( MP3_Stop () == FAPI_OK )
            {
                pMusic->playStatus     = MENU_MEDIA_PLAY_STATUS_STOP;
                pMusic->timeCurr       = 0;
                pMusic->waitForStopEvt = FTRUE;

                keyUsed = FTRUE;
            }
        }
    }

    return keyUsed;
}

FSTATIC void MENU_MediaMusicHandleEvent (void* arg, SYS_EVTTYPE_E type,
                                         const void* data)
{
    MENU_MEDIA_DATA_S*           pType  = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_MUSIC_S*          pMusic = (MENU_MEDIA_MUSIC_S*)(pType->pParams);
    MENU_MEDIA_MUSICINFO_INST_S* pInfo  = (MENU_MEDIA_MUSICINFO_INST_S*)
                                          (pType->pInfo);
    MP3_EVT_STATUS_S*            pEvent;

    switch ( type )
    {
        case EVT_MP3_STATUS:
            if ( pType->active )
            {
                pEvent = (MP3_EVT_STATUS_S*)data;

                if ( (pEvent->status == MP3_STATUS_END)
                  || (pEvent->status == MP3_STATUS_PLAY)
                  || (pEvent->status == MP3_STATUS_PAUSE) )
                {
                    /* Ignore event if related to previous file. */
                    if ( !(pMusic->waitForStopEvt) )
                    {
                        switch ( pMusic->playStatus )
                        {
                            case MENU_MEDIA_PLAY_STATUS_PLAY:
                            case MENU_MEDIA_PLAY_STATUS_PAUSE:
                                pMusic->timeCurr   = pEvent->timeCurr;
                                pMusic->timeTotal  = pEvent->timeTotal;

                                /* refresh playback time display */
                                if ( pInfo->pnlHdl != NULL )
                                {
                                    (void)FGS_PANEL_RefreshMember (
                                        pInfo->pnlHdl,
                                        MENU_MEDIA_MUSICINFO_ID_PLAYTIME);
                                    (void)FGS_PANEL_RefreshMember (
                                        pInfo->pnlHdl,
                                        MENU_MEDIA_MUSICINFO_ID_PROGRESS);
                                }
                                break;

                            default:
                                /* do nothing */
                                break;
                        }
                    }
                }
                else if ( pEvent->status == MP3_STATUS_READY )
                {
                    pMusic->waitForStopEvt = FFALSE;
                }

                if ( (pEvent->status == MP3_STATUS_END)
                  && (!(pType->single)) )
                {
                    (void)MENU_MediaFileNext (pType->pRoot,
                                              MENU_MEDIA_TYPE_MUSIC);
                }
            }
            break;

        default:
            /* do nothing */
            break;
    }
}

FSTATIC fbool_t MENU_MediaMusicHandleKey (MENU_MEDIA_INST_S* inst,
                                          FGS_KEY_E key)
{
    MENU_MEDIA_DATA_S*           pType   = inst->type + MENU_MEDIA_TYPE_MUSIC;
    MENU_MEDIA_MUSIC_S*          pMusic  = (MENU_MEDIA_MUSIC_S*)(pType->pParams);
    MENU_MEDIA_MUSICINFO_INST_S* pInfo   = (MENU_MEDIA_MUSICINFO_INST_S*)
                                           (pType->pInfo);
    fbool_t                      keyUsed = FTRUE;
    int32_t                      retVal;

    if ( pType->active )
    {
        switch ( key )
        {
            case FGS_KEY_PLAY:
                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_STOP:
                        /* restart playback */
                        retVal = MENU_MediaMusicStart (pType, pType->path,
                                                       pType->name, FFALSE);

                        if ( retVal != FAPI_OK )
                        {
                            keyUsed = FFALSE;
                        }
                        break;

                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        /* resume playback */
                        keyUsed = MENU_MediaMusicPause(pType, FFALSE);
                        break;

                    default:
                        keyUsed = FFALSE;
                        break;
                }
                break;

            case FGS_KEY_STOP:
                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_PLAY:
                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        /* stop playback */
                        keyUsed = MENU_MediaMusicStop (pType);
                        break;

                    default:
                        keyUsed = FFALSE;
                        break;
                }
                break;

            case FGS_KEY_PAUSE:
                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_PLAY:
                        /* pause playback */
                        keyUsed = MENU_MediaMusicPause (pType, FTRUE);
                        break;

                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        /* resume playback */
                        keyUsed = MENU_MediaMusicPause (pType, FFALSE);
                        break;

                    default:
                        keyUsed = FFALSE;
                        break;
                }
                break;

            default:
                keyUsed = FFALSE;
                break;
        }

        if ( keyUsed )
        {
            /* refresh playback status display */
            if ( pInfo->pnlHdl != NULL )
            {
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_MUSICINFO_ID_STATUS);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_MUSICINFO_ID_PLAYTIME);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_MUSICINFO_ID_PROGRESS);

                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_KEY);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_DESC);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_MUSICINFO_ID_HELP_STOP_KEY);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_MUSICINFO_ID_HELP_STOP_DESC);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_KEY);
                (void)FGS_PANEL_RefreshMember (pInfo->pnlHdl,
                                    MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_DESC);
            }
        }
    }

    return keyUsed;
}

/* ------------------------------------------------------------------------- */

FSTATIC int32_t MENU_MediaMusicInfoOpen (MENU_MEDIA_INST_S* rootInst,
                                         MENU_MEDIA_DATA_S* pType)
{
    MENU_MEDIA_MUSICINFO_INST_S*    inst = (MENU_MEDIA_MUSICINFO_INST_S*)
                                           (pType->pInfo);
    FGS_PANEL_OPEN_S                pnlPars;
    FGS_BOX_OPEN_S                  boxPars;
    FAPI_SYS_HandleT                itemHdl;
    int32_t                         retVal   = FAPI_OK;
    uint8_t                         instCnt  = 0;
    uint16_t                        tempPos;
    uint8_t                         numLines = MENU_MEDIA_MUSICINFO_NUMLINES;
    uint16_t                        helpColWidth;
    uint16_t                        helpKeyWidth;
    uint16_t                        helpDescWidth;
    uint16_t                        posXStart;
    uint16_t                        posYStart;

    do
    {
        if ( inst->pnlHdl != NULL )
        {
            retVal = SMARTGO_APPL_ERR_ALREADY_OPEN;
            break;
        }

        memset (inst, 0, sizeof(MENU_MEDIA_MUSICINFO_INST_S));

        memset (&pnlPars, 0, sizeof(pnlPars));

        pnlPars.bkgrType      = OSD_PNLTYPE_MENUWTITLE;
        pnlPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_WIDTH)
                              + (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        pnlPars.pos.height    = (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH))
                              + OSD_GetPanelTitleHeight(pnlPars.bkgrType)
                              + (numLines
                               * SCALER_GetCurrVal(LIST_LINE_HEIGHT));
        pnlPars.pos.x         = 0;
        pnlPars.pos.y         = -(int16_t)
                                 (SCALER_GetCurrVal(MAIN_PAD_Y)
                                  + (int16_t)pnlPars.pos.height
                                  + (int16_t)rootInst->infoHeight);
        pnlPars.pos.alignment = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_BOTTOM;
        pnlPars.pInst         = &(inst->pnlInst);

        retVal = FGS_PANEL_Open (&pnlPars, MENUID_MEDIA_MUSICINFO,
                                 &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* Store height information for next menu to be build. Add
           OUTER_FRAME_WIDTH padding distance to adjacent panel. */
        rootInst->infoHeight += pnlPars.pos.height
                              + SCALER_GetCurrVal(OUTER_FRAME_WIDTH);

        /* ----- add box for drawing additional items ---------------------- */

        memset (&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x         = 0;
        boxPars.pos.y         = 0;
        boxPars.pos.width     = pnlPars.pos.width;
        boxPars.pos.height    = pnlPars.pos.height;
        boxPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType      = OSD_BOX_BKGR_NONE;
        boxPars.getItem       = MENU_MediaMusicInfoGetItem;
        boxPars.getItemArg    = (FAPI_SYS_HandleT)pType;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_REST,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----- Add title line items -------------------------------------- */

        /* right-side-orientiered items first */

        boxPars.pos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        boxPars.pos.height    = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType      = OSD_BOX_BKGR_TITLE;

        boxPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_MUSIC_TIME_WIDTH);
        boxPars.pos.x         = -(int16_t)(SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                + SCALER_GetCurrVal(MAIN_RIGHT_PADDING)
                                + boxPars.pos.width);
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_PLAYTIME,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_MUSIC_PROG_WIDTH);
        boxPars.pos.height    = SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                              - (2 * SCALER_GetCurrVal(INFO_PROGBAR_PAD));
        boxPars.pos.x        -= (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X))
                              + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + boxPars.pos.width;
        boxPars.pos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                              + SCALER_GetCurrVal(INFO_PROGBAR_PAD);
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_PROGRESS,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.width     = SCALER_GetCurrVal(INFO_PLAYICON_WIDTH);
        boxPars.pos.height    = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.x        -= (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X))
                              + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + boxPars.pos.width;
        boxPars.pos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_STATUS,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.width     = SCALER_GetCurrVal(MEDIA_INFO_SHUFFLE_WIDTH);
        boxPars.pos.x        -= (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X))
                              + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                              + boxPars.pos.width;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_SHUFFLESTAT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        tempPos               = -(boxPars.pos.x)
                              + (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X))
                              + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

        /* reserve remaining space in title line for filename */
        boxPars.pos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                              + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width     = pnlPars.pos.width
                              - boxPars.pos.x
                              - tempPos;
        boxPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_FILENAME,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----- add inner items ------------------------------------------- */

        boxPars.pos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                              + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                OSD_GetPanelTitleHeight(pnlPars.bkgrType);
        boxPars.pos.width     = ((pnlPars.pos.width
                               * MENU_MEDIA_MUSICINFO_LEFTWIDTH_PERCENT)
                               / 100)
                              - boxPars.pos.x
                              - SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        boxPars.pos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType      = OSD_BOX_BKGR_NORMAL;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_TITLE,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.y        += SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_ALBUM,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----- add help items -------------------------------------------- */

        /* Calculate total width usable for help items. */
        helpColWidth = pnlPars.pos.width
                     - ((pnlPars.pos.width
                       * MENU_MEDIA_MUSICINFO_LEFTWIDTH_PERCENT) / 100)
                     - SCALER_GetCurrVal(MAIN_TEXT_PAD_X)    /* left padding */
                     - SCALER_GetCurrVal(OUTER_FRAME_WIDTH); /* right frame  */

        helpColWidth /= 3;                  /* split into three columns */
        helpKeyWidth  = helpColWidth / 4;   /* split column into key and
                                               description area */
        helpDescWidth = helpColWidth
                      - helpKeyWidth
                      - SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

        /* store top/left position */
        posXStart    = ((pnlPars.pos.width
                       * MENU_MEDIA_MUSICINFO_LEFTWIDTH_PERCENT) / 100)
                     + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                     + SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        posYStart    = SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                     + OSD_GetPanelTitleHeight(pnlPars.bkgrType);

        /* 1st column */

        memset (&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x         = posXStart;
        boxPars.pos.y         = posYStart;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType      = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem       = MENU_MediaMusicInfoGetItem;
        boxPars.getItemArg    = (FAPI_SYS_HandleT)pType;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_HELP_NEXT_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_HELP_NEXT_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x         = posXStart;
        boxPars.pos.y        += boxPars.pos.height;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_HELP_PREV_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_HELP_PREV_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* 2nd column */

        boxPars.pos.x         = posXStart + helpColWidth;
        boxPars.pos.y         = posYStart;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x         = posXStart + helpColWidth;
        boxPars.pos.y        += boxPars.pos.height;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_HELP_STOP_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars,
                               MENU_MEDIA_MUSICINFO_ID_HELP_STOP_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* 3rd column */

        boxPars.pos.x         = posXStart + (2 * helpColWidth);
        boxPars.pos.y         = posYStart;
        boxPars.pos.width     = helpKeyWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars, MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_KEY,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        boxPars.pos.x        += helpKeyWidth;
        boxPars.pos.width     = helpDescWidth;
        boxPars.pInst         = inst->boxInst + (instCnt++);

        retVal = FGS_BOX_Open (&boxPars,
                               MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_DESC,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /*
        FF      next music      PLAY    Play    PAUSE   Pause
        REW     prev music      STOP    Stop
        */
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);

            inst->pnlHdl = NULL;
        }
    }

    return retVal;
}

FSTATIC int32_t MENU_MediaMusicInfoClose (MENU_MEDIA_DATA_S* pType)
{
    MENU_MEDIA_MUSICINFO_INST_S* inst = (MENU_MEDIA_MUSICINFO_INST_S*)
                                        (pType->pInfo);

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close (inst->pnlHdl);

        inst->pnlHdl = NULL;
    }

    return FAPI_OK;
}

FSTATIC int32_t MENU_MediaMusicInfoShow (void* arg, fbool_t showOrHide)
{
    MENU_MEDIA_DATA_S*           pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_MUSICINFO_INST_S* inst  = (MENU_MEDIA_MUSICINFO_INST_S*)
                                         (pType->pInfo);
    int32_t                      retVal;

    if ( inst->pnlHdl != NULL )
    {
        if ( showOrHide )
        {
            retVal = FGS_PANEL_Show (inst->pnlHdl, FFALSE);
        }
        else
        {
            retVal = FGS_PANEL_Hide (inst->pnlHdl);
        }
    }
    else
    {
        retVal = SMARTGO_APPL_ERR_PROHIBITED;
    }

    return retVal;
}

FSTATIC fbool_t MENU_MediaMusicInfoIsVisible (void* arg)
{
    MENU_MEDIA_DATA_S*           pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_MUSICINFO_INST_S* inst  = (MENU_MEDIA_MUSICINFO_INST_S*)
                                         (pType->pInfo);
    fbool_t                      isVisible;

    if ( inst->pnlHdl != NULL )
    {
        isVisible = FGS_PANEL_IsVisible (inst->pnlHdl);
    }
    else
    {
        isVisible = FFALSE;
    }

    return isVisible;
}

FSTATIC int32_t MENU_MediaMusicInfoUpdate (void* arg)
{
    MENU_MEDIA_DATA_S*           pType = (MENU_MEDIA_DATA_S*)arg;
    MENU_MEDIA_MUSICINFO_INST_S* inst  = (MENU_MEDIA_MUSICINFO_INST_S*)
                                         (pType->pInfo);

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_FILENAME);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_PLAYTIME);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_PROGRESS);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_STATUS);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_TITLE);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_ALBUM);

        MENU_MediaMusicInfoRefreshHelp (inst);
    }

    return FAPI_OK;
}

FSTATIC void MENU_MediaMusicInfoRefreshHelp (MENU_MEDIA_MUSICINFO_INST_S* inst)
{
    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_NEXT_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_NEXT_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_PREV_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_PREV_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_STOP_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_STOP_DESC);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_KEY);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_DESC);
    }
}

FSTATIC void MENU_MediaMusicInfoGetItem (FAPI_SYS_HandleT h, int32_t id,
                                         uint32_t index,
                                         FGS_ITEMSTATE_E state,
                                         FGS_ITEM_S* item)
{
    MENU_MEDIA_DATA_S*  pType    = (MENU_MEDIA_DATA_S*)h;
    MENU_MEDIA_MUSIC_S* pMusic   = (MENU_MEDIA_MUSIC_S*)(pType->pParams);
    fbool_t             titleStr = FFALSE;
    fbool_t             bodyStr  = FFALSE;
    SYS_STRING_S        str;
    uint32_t            strIdx;
    uint32_t            m1;
    uint32_t            m2;
    uint32_t            bufPos = 0;

    switch ( id )
    {
        case MENU_MEDIA_MUSICINFO_ID_REST:

            item->type                      = FGS_ITEM_TYPE_GENERIC;
            item->align                     = FGS_ALIGN_HOR_CENTER
                                            | FGS_ALIGN_VER_CENTER;
            item->item.gnr.cb               = MENU_MediaMusicInfoDrawRest;
            item->item.gnr.cbArg            = NULL;
            break;

        case MENU_MEDIA_MUSICINFO_ID_FILENAME:

            if ( pType->active )
            {
                item->item.str.chr = STR_GetASCII(pType->name);
            }
            else
            {
                item->item.str.chr = OSD_GetStr(
                                     STR_ENTRY_MEDIA_MUSIC_NONESELECTED);
            }

            item->align = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            titleStr = FTRUE;
            break;

        case MENU_MEDIA_MUSICINFO_ID_SHUFFLESTAT:

            if ( (pType->active)
              && (pType->shuffle) )
            {
                item->align        = FGS_ALIGN_HOR_CENTER
                                   | FGS_ALIGN_VER_CENTER;
                item->item.str.chr = OSD_GetStr(STR_ENTRY_MEDIA_SHUFFLE_ACTIVE);
                titleStr           = FTRUE;
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_PLAYTIME:

            if ( pType->active )
            {
                m1 = pMusic->timeCurr  / 60;
                m2 = pMusic->timeTotal / 60;

                sprintf(appDat->asciiBuf, "%ld:%02ld / %ld:%02ld",
                        m1, pMusic->timeCurr  - (m1 * 60),
                        m2, pMusic->timeTotal - (m2 * 60));

                item->align        = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
                item->item.str.chr = STR_IntASCIIStr();

                titleStr = FTRUE;
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_PROGRESS:

            item->type                      = FGS_ITEM_TYPE_GENERIC;
            item->align                     = FGS_ALIGN_HOR_CENTER
                                            | FGS_ALIGN_VER_CENTER;
            item->item.gnr.cb               = MENU_MediaMusicInfoDrawProgBar;
            item->item.gnr.cbArg            = pMusic;
            break;

        case MENU_MEDIA_MUSICINFO_ID_STATUS:

            if ( pType->active )
            {
                item->type                   = FGS_ITEM_TYPE_STRING;
                item->align                  = FGS_ALIGN_HOR_CENTER
                                             | FGS_ALIGN_VER_CENTER;
                item->item.str.font          = SYMBOL_TITLE;
                item->item.str.lineHeight    = 0;
                item->item.str.col           = OSD_COL_TITLE_TEXT;

                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_PLAY:
                        strIdx = STR_SYMBOL_PLAY;
                        break;

                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        strIdx = STR_SYMBOL_PAUSE;
                        break;

                    case MENU_MEDIA_PLAY_STATUS_STOP:
                        strIdx = STR_SYMBOL_STOP;
                        break;

                    default:
                        strIdx = STR_EMPTYSTRING;
                        break;
                }

                item->item.str.chr = OSD_GetStr(strIdx);
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_TITLE:

            if ( pType->active )
            {
                str = STR_Build (pMusic->fileInfo.titleCs,
                                 pMusic->fileInfo.title);

                if ( STR_IsEmpty(str) )
                {
                    str = OSD_GetStr (STR_ENTRY_MP3_UNKNOWN_TITLE);
                }

                bufPos += STR_BufAppend (NULL, bufPos, str,
                                         MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend (NULL, bufPos, STR_GetASCII(" - "),
                                         MAIN_LINEBUFCHARSET);

                str = STR_Build (pMusic->fileInfo.artistCs,
                                 pMusic->fileInfo.artist);

                if ( STR_IsEmpty(str) )
                {
                    str = OSD_GetStr (STR_ENTRY_MP3_UNKNOWN_ARTIST);
                }

                bufPos += STR_BufAppend (NULL, bufPos, str,
                                         MAIN_LINEBUFCHARSET);

                item->align               = FGS_ALIGN_HOR_LEFT
                                          | FGS_ALIGN_VER_CENTER;
                item->item.str.chr        = STR_IntLineStr();
                bodyStr = FTRUE;
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_ALBUM:

            if ( pType->active )
            {
                str = STR_Build (pMusic->fileInfo.albumCs,
                                 pMusic->fileInfo.album);

                if ( STR_IsEmpty(str) )
                {
                    str = OSD_GetStr (STR_ENTRY_MP3_UNKNOWN_ALBUM);
                }

                item->align               = FGS_ALIGN_HOR_LEFT
                                          | FGS_ALIGN_VER_CENTER;
                item->item.str.chr        = str;
                bodyStr = FTRUE;
            }
            break;

        /* ----- help items ------------------------------------------------ */

        case MENU_MEDIA_MUSICINFO_ID_HELP_NEXT_KEY:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_FF);
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_HELP_NEXT_DESC:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_MEDIA_HELP_MUSIC_NEXT);
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_HELP_PREV_KEY:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_REW);
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_HELP_PREV_DESC:

            if ( pType->active )
            {
                MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_MEDIA_HELP_MUSIC_PREV);
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_KEY:

            if ( pType->active )
            {
                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_STOP:
                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_PLAY);
                        break;

                    default:
                        break;
                }
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_HELP_PLAY_DESC:

            if ( pType->active )
            {
                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_STOP:
                        strIdx = STR_ENTRY_MEDIA_HELP_MUSIC_PLAY_START;
                        break;

                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        strIdx = STR_ENTRY_MEDIA_HELP_MUSIC_PLAY_RESUME;
                        break;

                    default:
                        strIdx = STR_EMPTYSTRING;
                        break;
                }

                MENU_COMMON_ITEM_HELPSTR(item, strIdx);
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_HELP_STOP_KEY:

            if ( pType->active )
            {
                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_PLAY:
                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_STOP);
                        break;

                    default:
                        break;
                }
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_HELP_STOP_DESC:

            if ( pType->active )
            {
                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_PLAY:
                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        MENU_COMMON_ITEM_HELPSTR(item,
                                               STR_ENTRY_MEDIA_HELP_MUSIC_STOP);
                        break;

                    default:
                        break;
                }
            }
            break;

        case MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_KEY:
            if ( pType->active )
            {
                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_PLAY:
                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_PAUSE);
                        break;

                    default:
                        break;
                }
            }

            break;

        case MENU_MEDIA_MUSICINFO_ID_HELP_PAUSE_DESC:
            if ( pType->active )
            {
                switch ( pMusic->playStatus )
                {
                    case MENU_MEDIA_PLAY_STATUS_PLAY:
                        strIdx = STR_ENTRY_MEDIA_HELP_MUSIC_PAUSE;
                        break;

                    case MENU_MEDIA_PLAY_STATUS_PAUSE:
                        strIdx = STR_ENTRY_MEDIA_HELP_MUSIC_PLAY_RESUME;
                        break;

                    default:
                        strIdx = STR_EMPTYSTRING;
                        break;
                }

                MENU_COMMON_ITEM_HELPSTR(item, strIdx);
            }
            break;

        default:
            /* do nothing */
            break;
    }

    if ( titleStr )
    {
        item->type                = FGS_ITEM_TYPE_STRING;
        item->item.str.font       = TITLE_FONT;
        item->item.str.lineHeight = 0;
        item->item.str.col        = OSD_COL_TITLE_TEXT;
    }
    else if ( bodyStr )
    {
        item->type                = FGS_ITEM_TYPE_STRING;
        item->item.str.font       = MAIN_FONT;
        item->item.str.lineHeight = 0;
        item->item.str.col        = OSD_COL_TEXT;
    }
}

FSTATIC void MENU_MediaMusicInfoDrawRest (FAPI_SYS_HandleT h, uint8_t align,
                                          uint16_t x, uint16_t y,
                                          uint16_t width, uint16_t height)
{
    uint32_t lineHeight = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
    uint16_t posY       = y + SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
    uint16_t posX       = x + width;

    /* Draw vertical separators in top line */

    posX -= SCALER_GetCurrVal(OUTER_FRAME_ALL)
          + SCALER_GetCurrVal(MAIN_RIGHT_PADDING)
          + SCALER_GetCurrVal(MEDIA_INFO_MUSIC_TIME_WIDTH)
          + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(posX, posY, lineHeight);

    posX -= SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(MEDIA_INFO_MUSIC_PROG_WIDTH)
          + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(posX, posY, lineHeight);

    posX -= SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INFO_PLAYICON_WIDTH)
          + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(posX, posY, lineHeight);

    posX -= SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(MEDIA_INFO_SHUFFLE_WIDTH)
          + SCALER_GetCurrVal(MAIN_TEXT_PAD_X)
          + SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    OSD_DrawSeparatorVertical(posX, posY, lineHeight);

    /* draw vertical separator in inner line */

    OSD_DrawSeparatorVertical(x + ((width
                              * MENU_MEDIA_MUSICINFO_LEFTWIDTH_PERCENT) / 100),
                              y + SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                              + OSD_GetPanelTitleHeight(OSD_PNLTYPE_MENUWTITLE),
                              MENU_MEDIA_MUSICINFO_NUMLINES
                            * SCALER_GetCurrVal(LIST_LINE_HEIGHT));
}

FSTATIC void MENU_MediaMusicInfoDrawProgBar (FAPI_SYS_HandleT h, uint8_t align,
                                             uint16_t x, uint16_t y,
                                             uint16_t width, uint16_t height)
{
    MENU_MEDIA_MUSIC_S* pMusic = (MENU_MEDIA_MUSIC_S*)h;
    uint32_t            lenCurr;

    if ( pMusic->timeTotal == 0 )
    {
        lenCurr = 0;
    }
    else if ( pMusic->timeCurr >= pMusic->timeTotal )
    {
        lenCurr = width;
    }
    else
    {
        lenCurr = (pMusic->timeCurr * width) / pMusic->timeTotal;
    }

    if ( lenCurr > 0 )
    {
        OSD_DrawFilledRectangle (x, y,
                                 x + lenCurr - 1,
                                 y + height - 1,
                                 OSD_COL_BAR_ACTIVE);
    }

    if ( lenCurr < width )
    {
        OSD_DrawFilledRectangle (x + lenCurr, y,
                                 x + width - 1,
                                 y + height - 1,
                                 OSD_GetSchemeColor(
                                 OSD_SCHEMECOL_PROGRESSBAR_INACT));
    }
}

#endif /* APPL_MP3_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
