/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_mp3.c
**
**              $Id: app_menu_mp3.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       MP3 playback menu.
**
** Detail description. [\ref app_smartgo_app_ref_mp3]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info 713: Loss of precision (assignment) (unsigned int to short).
** - Info 715: Symbol not referenced.
** - Info 732: Loss of sign (assignment) (int to unsigned short).
** - Info 737: Loss of sign in promotion from int to unsigned int.
** - Note 961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**   C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include "app_menu.h"
#include "app_menu_viewer.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define MP3_MAXDIRDEPTH  0

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum
{
    MENU_MP3_ID_TITLE,
    MENU_MP3_ID_COVER,
    MENU_MP3_ID_NOWPLAYING,
    MENU_MP3_ID_CURTITLE,
    MENU_MP3_ID_CURARTIST,
    MENU_MP3_ID_CURALBUM,
    MENU_MP3_ID_PROGRESS,
    MENU_MP3_ID_TIME,
    MENU_MP3_ID_NEXT,
    MENU_MP3_ID_NEXTTITLE,
    MENU_MP3_ID_NEXTARTIST,
    MENU_MP3_ID_NEXTALBUM,
    MENU_MP3_ID_HELP_KEY_MENU,
    MENU_MP3_ID_HELP_ITEM_BACK,
    MENU_MP3_ID_HELP_KEY_EXIT,
    MENU_MP3_ID_HELP_ITEM_CLOSE,
    MENU_MP3_ID_HELP_KEY_SKIP,
    MENU_MP3_ID_HELP_ITEM_SKIPTHIS,
    MENU_MP3_ID_HELP_KEY_RED,
    MENU_MP3_ID_HELP_ITEM_SKIPNEXT,
    MENU_MP3_ID_HELP_KEY_BLUE,
    MENU_MP3_ID_HELP_REPEAT,
    MENU_MP3_ID_HELP_KEY_YELLOW,
    MENU_MP3_ID_HELP_SHUFFLE,
    MENU_MP3_ID_SEPARATOR,
    MENU_MP3_ID_PLAY,
    MENU_MP3_ID_SHUFFLE
};


typedef struct
{
    FAPI_SYS_HandleT      pnlHdl;

    MENU_MP3_PARAMS_S     params;

    uint32_t              allcount;

    fbool_t               shuffle;
    fbool_t               run;

    uint32_t              random;
    uint32_t              currentfile;
    uint32_t              nextfile;

    FGS_POS_S             progresspos;
    FGS_PANEL_MEMBER_S*   progressP;
    FGS_POS_S             separatorpos;
    FGS_PANEL_MEMBER_S*   separatorP;
    uint16_t              coversize;

    char_t                thisStr[MAIN_VIEWER_PATHLEN];
    char_t                nextStr[MAIN_VIEWER_PATHLEN];

    char_t                thisFilename[MAIN_VIEWER_NAMELEN];
    char_t                nextFilename[MAIN_VIEWER_NAMELEN];

    MP3_FILE_INFO_S       tagsThis;
    MP3_FILE_INFO_S       tagsNext;
    MP3_EVT_STATUS_S      playStat;

}MENU_MP3_INST_S;


FSTATIC MENU_MP3_INST_S* menuMP3Dat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_MP3Leave             (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_MP3Keys              (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_MP3Build             (FAPI_SYS_HandleT h);
FSTATIC void    MENU_MP3CountSongs        (const char_t* path, uint32_t* count,
                                           uint32_t maxDirDepth);
FSTATIC void    MENU_MP3FindCurrentSong   (MENU_MP3_INST_S* inst);
FSTATIC fbool_t MENU_MP3FindSong          (const char_t* root,
                                           uint32_t reqIndex, char_t* resultStrP,
                                           char_t* resultNameP);
FSTATIC void    MENU_MP3FindRecursive     (const char_t* path,
                                           uint32_t reqIndex, int32_t* countP,
                                           char_t* resultStrP,
                                           char_t* resultNameP,
                                           uint32_t maxDirDepth);
FSTATIC void    MENU_MP3GetNextFile       (MENU_MP3_INST_S* inst);
FSTATIC int32_t MENU_MP3PaintProgress     (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_MP3PaintSeparator    (FAPI_SYS_HandleT h);
FSTATIC void    MENU_MP3StartPlay         (MENU_MP3_INST_S* inst);
FSTATIC void    MENU_MP3SetThisTrackInfo  (MENU_MP3_INST_S* inst);
FSTATIC void    MENU_MP3SetNextTrackInfo  (MENU_MP3_INST_S* inst);
FSTATIC void    MENU_MP3GetTrackInfo      (MENU_MP3_INST_S* inst,
                                           const char_t* fileStr,
                                           MP3_FILE_INFO_S* tags);
FSTATIC void    MENU_MP3OnFocusSet        (FAPI_SYS_HandleT h,
                                           fbool_t hasFocus);
FSTATIC void    MENU_MP3GetItem           (FAPI_SYS_HandleT h,
                                           int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);

FSTATIC FINLINE fbool_t MENU_MP3FileIsMp3 (const char_t* fileName);

/*!
*******************************************************************************
**
** \brief  Initial function for the MP3 Menu
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_MP3Init()
{
    if(menuMP3Dat != NULL)
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuMP3Dat= UTILS_Malloc(sizeof(MENU_MP3_INST_S));
    if(menuMP3Dat == NULL)
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuMP3Dat, 0, sizeof(MENU_MP3_INST_S));

    menuMP3Dat->random= 0x138E;
    menuMP3Dat->shuffle= FFALSE;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function.
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_MP3Entry(FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_MP3_INST_S*    inst = menuMP3Dat;
    SYS_DEVICE_E        device;

    #ifdef APPL_JPEG_ENABLED
    JPEG_SCREEN_PARAMS_S screenParams;
    FGS_LOC_S            memberPos;
    #endif /* APPL_JPEG_ENABLED */

    if ( h == NULL ) return FFALSE;

    if ( test == FTRUE ) return FTRUE;

    /* MP3 panel is started from file viewer who passes open params */
    inst->params = *((MENU_MP3_PARAMS_S*)h);

    memset(&(inst->tagsThis), 0, sizeof(inst->tagsThis));
    memset(&(inst->tagsNext), 0, sizeof(inst->tagsNext));

    inst->playStat.timeTotal = 0;
    inst->playStat.timeCurr  = 0;
    inst->run= FTRUE;

    MENU_CommonAppendpath(inst->params.filename, inst->params.path,
                          inst->thisStr, FFALSE, MAIN_VIEWER_PATHLEN);
    (void)UTILS_strncpy (inst->thisFilename, inst->params.filename,
                   MAIN_VIEWER_NAMELEN);

    (void)MENU_MP3Build((FAPI_SYS_HandleT)inst);

    (void)MENUSTACK_FocusClaim ((int32_t)MENUID_MP3);
    (void)FGS_PANEL_Show       (inst->pnlHdl, FFALSE);           /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_MP3Leave, (FAPI_SYS_HandleT)inst);

    /* stop current presentation */
    (void)MAIN_StopPresentation();

    #ifdef APPL_JPEG_ENABLED

    (void)FGS_PANEL_GetMemberAbsPos(inst->pnlHdl,
                              MENU_MP3_ID_COVER,
                              &memberPos);

    screenParams.pos.left  = memberPos.x;
    screenParams.pos.top   = memberPos.y;
    screenParams.pos.hsize = inst->coversize;
    screenParams.pos.vsize = inst->coversize;

    (void)JPEG_Open (&screenParams);

    #endif /* APPL_JPEG_ENABLED */

    inst->allcount = 0;

    if ( SYS_DeviceGetByPath (inst->params.path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    MENU_MP3CountSongs (inst->params.path, &inst->allcount, MP3_MAXDIRDEPTH);
    MENU_MP3FindCurrentSong (inst);

    inst->nextfile = inst->currentfile;

    FAPI_SYS_PRINT_DEBUG(4,"[MP3] %d files, this is no.%d\n",
               inst->allcount, inst->currentfile);

    MENU_MP3GetNextFile(inst);
    (void)MENU_MP3FindSong(inst->params.path, inst->nextfile,
                           inst->nextStr, inst->nextFilename);

    MENU_MP3SetThisTrackInfo(inst);
    MENU_MP3SetNextTrackInfo(inst);

    MENU_MP3StartPlay(inst);

    return FTRUE;
}

void MENU_MP3HandleEvent (const MP3_EVT_STATUS_S* data)
{
    MENU_MP3_INST_S* inst = menuMP3Dat;

    if ( inst->pnlHdl == NULL ) return;

    inst->playStat = *data;

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_TIME);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_PROGRESS);

    /* on stop: start next playback */
    if ( inst->playStat.status == MP3_STATUS_END )
    {
        (void)MP3_Stop();

        inst->currentfile= inst->nextfile;
        (void)MENU_MP3FindSong(inst->params.path, inst->currentfile, inst->thisStr,
                               inst->thisFilename);
        MENU_MP3SetThisTrackInfo(inst);

        MENU_MP3HandleTime();
        MENU_MP3GetNextFile(inst);
        (void)MENU_MP3FindSong(inst->params.path, inst->nextfile, inst->nextStr,
                               inst->nextFilename);
        MENU_MP3SetNextTrackInfo(inst);

        FAPI_SYS_PRINT_DEBUG(4,"[MP3] Next is no %d, %s\n",
                   inst->nextfile, inst->nextStr);

        MENU_MP3StartPlay(inst);
    }
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_MP3OnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    MAIN_HandleExitKeys(hasFocus);

    if(hasFocus)
    {
        FGS_SetAppKeyHandler(FGS_KEY_RIGHT,  MENU_MP3Keys, h, FTRUE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_RED,    MENU_MP3Keys, h, FFALSE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_BLUE,   MENU_MP3Keys, h, FFALSE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_YELLOW, MENU_MP3Keys, h, FFALSE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_PLAY,   MENU_MP3Keys, h, FFALSE, FTRUE);
        //FGS_SetAppKeyHandler(FGS_KEY_STOP,   MENU_MP3Keys, h, FFALSE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_PAUSE,  MENU_MP3Keys, h, FFALSE, FTRUE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_BLUE);
        FGS_ClearAppKeyHandler(FGS_KEY_YELLOW);
        FGS_ClearAppKeyHandler(FGS_KEY_PLAY);
        //FGS_ClearAppKeyHandler(FGS_KEY_STOP);
        FGS_ClearAppKeyHandler(FGS_KEY_PAUSE);
    }
}

FSTATIC fbool_t MENU_MP3Leave(FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_MP3_INST_S*        inst= (MENU_MP3_INST_S*)h;

    (void)MP3_Stop();
    (void)MAIN_MsgBoxHide(&(appDat->msgBox));

    #ifdef APPL_JPEG_ENABLED

    (void)JPEG_Close();

    #endif /* APPL_JPEG_ENABLED */

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    inst->pnlHdl= NULL;
    (void)MAIN_RestartPresentation();

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_MP3);

    return FTRUE;
}

FSTATIC void MENU_MP3GetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                              FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_MP3_INST_S*    inst= (MENU_MP3_INST_S*)h;
    uint32_t            m1, m2;
    uint32_t            strIdx;

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.col        = appDat->txtAttr.colNonFocus;
    item->item.str.lineHeight = 0;

    switch(id)
    {
        case MENU_MP3_ID_TITLE:
            MENU_COMMON_ITEM_TITLESTR(item, STR_TITLE_MP3);
            break;

        case MENU_MP3_ID_NOWPLAYING:
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_MP3_NOW);
            break;

        case MENU_MP3_ID_NEXT:
            item->item.str.chr        = OSD_GetStr(STR_ENTRY_MP3_NEXT);
            break;

        case MENU_MP3_ID_TIME:
            item->align               = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;

            m1 = inst->playStat.timeCurr / 60;
            m2 = inst->playStat.timeTotal / 60;

            sprintf(appDat->asciiBuf, "%ld'%02ld'' Of %ld'%02ld''",
                    m1, inst->playStat.timeCurr - m1 * 60,
                    m2, inst->playStat.timeTotal - m2 * 60);
            item->item.str.chr        = STR_IntASCIIStr();

            break;

        case MENU_MP3_ID_CURTITLE:
            // in case title field is missing from file, display the file name instead
            if ( inst->tagsThis.title[0] == 0 )
            {
                item->item.str.chr     = STR_GetASCII(inst->thisFilename);
            }
            else item->item.str.chr = STR_Build(inst->tagsThis.titleCs, inst->tagsThis.title);
            break;

        case MENU_MP3_ID_CURARTIST:
            if ( inst->tagsThis.artist[0] == 0 )
                 item->item.str.chr = OSD_GetStr(STR_ENTRY_MP3_UNKNOWN_ARTIST);
            else item->item.str.chr = STR_Build(inst->tagsThis.artistCs, inst->tagsThis.artist);
            break;

        case MENU_MP3_ID_CURALBUM:
            if ( inst->tagsThis.album[0] == 0 )
                 item->item.str.chr = OSD_GetStr(STR_ENTRY_MP3_UNKNOWN_ALBUM);
            else item->item.str.chr = STR_Build(inst->tagsThis.albumCs, inst->tagsThis.album);
            break;

        case MENU_MP3_ID_NEXTTITLE:
            // in case title field is missing from file, display the file name instead
            if ( inst->tagsNext.title[0] == 0 )
            {
               item->item.str.chr     = STR_GetASCII(inst->nextFilename);
            }
            else item->item.str.chr = STR_Build(inst->tagsNext.titleCs, inst->tagsNext.title);
            break;

        case MENU_MP3_ID_NEXTARTIST:
            if ( inst->tagsNext.artist[0] == 0 )
                 item->item.str.chr = OSD_GetStr(STR_ENTRY_MP3_UNKNOWN_ARTIST);
            else item->item.str.chr = STR_Build(inst->tagsNext.artistCs, inst->tagsNext.artist);
            break;

        case MENU_MP3_ID_NEXTALBUM:
            if ( inst->tagsNext.album[0] == 0 )
                 item->item.str.chr = OSD_GetStr(STR_ENTRY_MP3_UNKNOWN_ALBUM);
            else item->item.str.chr = STR_Build(inst->tagsNext.albumCs, inst->tagsNext.album);
            break;

        case MENU_MP3_ID_HELP_KEY_MENU:
            MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_MENU);
            break;

        case MENU_MP3_ID_HELP_ITEM_BACK:
            MENU_COMMON_ITEM_HELPSTR(item, STR_HELP_MENU_BACK);
            break;

        case MENU_MP3_ID_HELP_KEY_EXIT:
            MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_EXIT);
            break;

        case MENU_MP3_ID_HELP_ITEM_CLOSE:
            MENU_COMMON_ITEM_HELPSTR(item, STR_HELP_MENU_EXIT);
            break;

        case MENU_MP3_ID_HELP_KEY_SKIP:
            MENU_COMMON_ITEM_HELPSTR(item, STR_KEY_RIGHT);
            break;

        case MENU_MP3_ID_HELP_ITEM_SKIPTHIS:
            MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_MP3_SKIPTHIS);
            break;

        case MENU_MP3_ID_HELP_KEY_RED:
            MENU_COMMON_ITEM_BUTTON(item, ICON_BTNREDSMALL);
            break;

        case MENU_MP3_ID_HELP_ITEM_SKIPNEXT:
            MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_MP3_SKIPNEXT);
            break;

        case MENU_MP3_ID_HELP_KEY_BLUE:
            MENU_COMMON_ITEM_BUTTON(item, ICON_BTNBLUESMALL);
            break;

        case MENU_MP3_ID_HELP_REPEAT:
            MENU_COMMON_ITEM_HELPSTR(item, STR_ENTRY_MP3_REPEAT);
            break;

        case MENU_MP3_ID_HELP_KEY_YELLOW:
            MENU_COMMON_ITEM_BUTTON(item, ICON_BTNYELLOWSMALL);
            break;

        case MENU_MP3_ID_HELP_SHUFFLE:
            if ( inst->shuffle )
            {
                strIdx = STR_ENTRY_MP3_SHUFFLEOFF;
            }
            else
            {
                strIdx = STR_ENTRY_MP3_SHUFFLEON;
            }

            MENU_COMMON_ITEM_HELPSTR(item, strIdx);
            break;

         case MENU_MP3_ID_PLAY:
            if(inst->run)
                item->item.str.chr  = OSD_GetStr(STR_SYMBOL_PLAY);
            else
                item->item.str.chr  = OSD_GetStr(STR_SYMBOL_PAUSE);

            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
            item->item.str.font          = SYMBOL_TITLE;
            item->item.str.lineHeight    = 0;
            item->item.str.col           = OSD_COL_TEXT;
            break;

        case MENU_MP3_ID_SHUFFLE:
            if(inst->shuffle)
            {
                item->item.icon.addr       = OSD_IconGet(ICON_SHUFFLE);
                item->type                 = FGS_ITEM_TYPE_ICON;
                item->align                = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
                item->item.icon.drawMode   = FGS_ICONMODE_OVRLAYPLAIN;
                item->item.icon.bkgrCol    = OSD_COL_BLANK;
                item->item.icon.altBkgrCol = OSD_GetSchemeColor(OSD_SCHEMECOL_TITLE_BKGR);
            }
            break;

         default:
             break;
    }
}

FSTATIC fbool_t MENU_MP3Keys(FAPI_SYS_HandleT h)
{
    MENU_MP3_INST_S*    inst= (MENU_MP3_INST_S*)h;
    fbool_t             keyUsed= FTRUE;

    switch(FGS_GetCurrentKeyCode())
    {
        case FGS_KEY_RIGHT:
            (void)MP3_Stop();

            inst->currentfile= inst->nextfile;
            (void)MENU_MP3FindSong(inst->params.path, inst->currentfile,
                                   inst->thisStr, inst->thisFilename);
            MENU_MP3SetThisTrackInfo(inst);

            MENU_MP3HandleTime();
            MENU_MP3GetNextFile(inst);
            (void)MENU_MP3FindSong(inst->params.path, inst->nextfile,
                                   inst->nextStr, inst->nextFilename);
            MENU_MP3SetNextTrackInfo(inst);

            FAPI_SYS_PRINT_DEBUG(4,"[MP3] Next is no %d, %s\n",
                       inst->nextfile, inst->nextStr);

            MENU_MP3StartPlay(inst);
            inst->run = FTRUE;
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_PLAY);
            break;

        case FGS_KEY_RED:
            MENU_MP3HandleTime();
            MENU_MP3GetNextFile(inst);
            (void)MENU_MP3FindSong(inst->params.path, inst->nextfile,
                                   inst->nextStr, inst->nextFilename);

            FAPI_SYS_PRINT_DEBUG(4,"[MP3] Next is no %d, %s\n",
                       inst->nextfile, inst->nextStr);

            MENU_MP3SetNextTrackInfo(inst);
            break;

        case FGS_KEY_YELLOW:
            if ( inst->shuffle )
                inst->shuffle = FFALSE;
            else
                inst->shuffle = FTRUE;

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_HELP_SHUFFLE);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_SHUFFLE);
            break;

        case FGS_KEY_BLUE:
            inst->nextfile= inst->currentfile;
            (void)MENU_MP3FindSong(inst->params.path, inst->nextfile,
                                   inst->nextStr, inst->nextFilename);

            // FIXME: is this necessary?
            //MENU_MP3SetThisTrackInfo(inst);
            MENU_MP3SetNextTrackInfo(inst);
            break;

        case FGS_KEY_PLAY:
            if ( !(inst->run) )
            {
                inst->run = FTRUE;
                (void)MP3_Pause(FFALSE);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_PLAY);

            }
            else
            {
                keyUsed= FFALSE;
            }
            break;

//        case FGS_KEY_STOP:
//            inst->run= FFALSE;
//            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_PLAY);
//            break;
        case FGS_KEY_PAUSE:
            if ( inst->run )
            {
                inst->run = FFALSE;
                (void)MP3_Pause(FTRUE);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_PLAY);
            }
            else
            {
                keyUsed= FFALSE;
            }
            break;

        default:
            keyUsed= FFALSE;
            break;
    }

    return keyUsed;
}


FSTATIC fbool_t MENU_MP3Build(FAPI_SYS_HandleT h)
{
    MENU_MP3_INST_S*      inst= (MENU_MP3_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_BOX_OPEN_S        boxPars;
    int32_t               retVal;
    uint16_t              innerHeight;
    uint32_t              numLines= 12;
    FGS_PANEL_MEMBER_S    member;
    FAPI_SYS_HandleT      itemHdl;
    uint16_t              stackedFrameWidth;
    uint8_t               numHelpLines = 2;
    uint16_t              mainPanelWidth;
    uint16_t              bottomHelpKeyWidth;
    uint16_t              bottomHelpItemWidth;

    memset(&boxPars,  0, sizeof(boxPars));
    memset(&member,   0, sizeof(member));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        innerHeight = (numLines       * SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                    + ((numLines - 1) * SCALER_GetCurrVal(LIST_LINE_PADDING));

        MENU_CommonSetParamsTitleWBotHelp (&pnlPars,
                                           SCALER_GetCurrVal(MP3_WIDTH),
                                           innerHeight,
                                           numHelpLines,
                                           &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_MP3OnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal= FGS_PANEL_Open(&pnlPars, MENUID_MP3, &(inst->pnlHdl));
        if(retVal != FAPI_OK)
            break;

        /* ------Title --------------------------------------------- */

        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                  stackedFrameWidth;
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                  (SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                  SCALER_GetCurrVal(OUTER_FRAME_WIDTH))/2 + /* done for centering */
                                  SCALER_GetCurrVal(TITLE_TB_PADDING) +
                                  stackedFrameWidth;
        boxPars.pos.width       = SCALER_GetCurrVal(MP3_WIDTH) -
                                  (2*SCALER_GetCurrVal(OUTER_FRAME_ALL)) -
                                  SCALER_GetCurrVal(MAIN_RIGHT_PADDING) -
                                  stackedFrameWidth;
        boxPars.pos.height      = (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                  SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_TITLE;
        boxPars.getItem         = (FGS_GETITEM_CB)MENU_MP3GetItem;
        boxPars.getItemArg      = h;

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_TITLE, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        /* -----Info objects ------------------------------------------ */
        /* -----Now playing ------------------------------------------- */

        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                  SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                  (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                  SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                  stackedFrameWidth;
        boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;
        boxPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.width       = ((SCALER_GetCurrVal(MP3_WIDTH) / 5) * 3) -
                                  SCALER_GetCurrVal(MP3_INFO_PADDING);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_NOWPLAYING,
                             inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y          += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(LIST_LINE_PADDING);
        boxPars.pos.x          += SCALER_GetCurrVal(MP3_INFO_PADDING);
        boxPars.pos.width      -= SCALER_GetCurrVal(MP3_INFO_PADDING);

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_CURTITLE,
                             inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y          += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(LIST_LINE_PADDING);

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_CURARTIST,
                             inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y          += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(LIST_LINE_PADDING);

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_CURALBUM,
                             inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        inst->progresspos.x     = boxPars.pos.x;
        inst->progresspos.y     = boxPars.pos.y +
                                  2 * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(LIST_LINE_PADDING));
        inst->progresspos.height= SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        inst->progresspos.width = boxPars.pos.width;
        inst->progresspos.alignment= FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h                = (FAPI_SYS_HandleT)inst;
        member.pos              = &inst->progresspos;
        member.id               = MENU_MP3_ID_PROGRESS;
        member.show             = MENU_MP3PaintProgress;

        retVal= FGS_PANEL_AddMember(inst->pnlHdl, &member, &inst->progressP);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = boxPars.pos.y +
                                  3 * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(LIST_LINE_PADDING));

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_TIME, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.x           = stackedFrameWidth +
                                  SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width       = SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        boxPars.pos.height      = SCALER_GetCurrVal(MAIN_ICON_HEIGHT);
        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_SHUFFLE, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        inst->separatorpos.x    = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  stackedFrameWidth;
        inst->separatorpos.y    = boxPars.pos.y +
                                  SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(LIST_LINE_PADDING) +
                                  (SCALER_GetCurrVal(LIST_LINE_HEIGHT)>>1);
        inst->separatorpos.height= 4;
        inst->separatorpos.width= SCALER_GetCurrVal(MP3_WIDTH);
        inst->separatorpos.alignment= FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h                = (FAPI_SYS_HandleT)inst;
        member.pos              = &inst->separatorpos;
        member.id               = MENU_MP3_ID_SEPARATOR;
        member.show             = MENU_MP3PaintSeparator;

        retVal= FGS_PANEL_AddMember(inst->pnlHdl, &member, &inst->separatorP);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = inst->progresspos.y;

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_PLAY, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        /* -----Next --------------------------------------------------- */

        boxPars.pos.y           = inst->progresspos.y +
                                  3 * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(LIST_LINE_PADDING));
        boxPars.pos.width       = ((SCALER_GetCurrVal(MP3_WIDTH) / 5) * 3) - SCALER_GetCurrVal(MP3_INFO_PADDING);
        boxPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT);

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_NEXT, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y          += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(LIST_LINE_PADDING);

        boxPars.pos.x           = boxPars.pos.x + SCALER_GetCurrVal(MP3_INFO_PADDING);

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_NEXTTITLE, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = boxPars.pos.y + SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_NEXTARTIST, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.y           = boxPars.pos.y + SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_NEXTALBUM, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        /* ------Cover box --------------------------------------------- */

        inst->coversize         = 7 * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING));

        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  stackedFrameWidth +
                                  SCALER_GetCurrVal(MP3_WIDTH) -
                                  SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                  inst->coversize;
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                  SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                  (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                  SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                  stackedFrameWidth;
        boxPars.bkgrType        = OSD_BOX_BKGR_TRANSPARENT;
        boxPars.pos.height      = inst->coversize;
        boxPars.pos.width       = inst->coversize;
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        retVal= FGS_BOX_Open(&boxPars, MENU_MP3_ID_COVER, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        /* -----Help objects ------------------------------------------ */
        mainPanelWidth          = SCALER_GetCurrVal(MP3_WIDTH) -
                                  (2*SCALER_GetCurrVal(OUTER_FRAME_ALL));
        bottomHelpKeyWidth      = OSD_GetStrWidth(OSD_GetStr(STR_KEY_MENU),-1,HELP_FONT) +
                                  SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        bottomHelpItemWidth     = (mainPanelWidth/3) - bottomHelpKeyWidth;

        /* -- 1st Bottom line -- */
        /* menu icon*/
        boxPars.pos.x               = stackedFrameWidth +
                                      SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - (numHelpLines-1)*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_HELP;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_KEY_MENU, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_ITEM_BACK, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* exit icon */
        boxPars.pos.x              += boxPars.pos.width +
                                      SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.width           = bottomHelpKeyWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_KEY_EXIT, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_ITEM_CLOSE, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* skip icon */
        boxPars.pos.x              += boxPars.pos.width +
                                      SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.width           = bottomHelpKeyWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_KEY_SKIP, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_ITEM_SKIPTHIS, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 2nd Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                         SCALER_GetCurrVal(HELP_LINE_PADDING);

        /* red icon (skip next)*/
        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = bottomHelpKeyWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_KEY_RED, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_ITEM_SKIPNEXT, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* blue icon (repeat current)*/
        boxPars.pos.x              += boxPars.pos.width +
                                      SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.width           = bottomHelpKeyWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_KEY_BLUE, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_REPEAT, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* yellow icon (shuffle on)*/
        boxPars.pos.x              += boxPars.pos.width +
                                      SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.width           = bottomHelpKeyWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_KEY_YELLOW, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_MP3_ID_HELP_SHUFFLE, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

    }while(0);

    if(retVal != FAPI_OK)
    {
        if(inst->pnlHdl != NULL)
            (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}

// FIXME: Buffersize of resultP should be passed as additional argument.
FSTATIC fbool_t MENU_MP3FindSong (const char_t* root, uint32_t reqIndex,
                                  char_t* resultStrP, char_t* resultNameP)
{
    int32_t         count= -1;
    SYS_DEVICE_E    device;

    resultStrP[0] = 0;
    resultNameP[0] = 0;

    if ( SYS_DeviceGetByPath (root, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    MENU_MP3FindRecursive (root, reqIndex, &count, resultStrP,
                           resultNameP, MP3_MAXDIRDEPTH);

    return (fbool_t)((uint32_t)count == reqIndex);
}

FSTATIC void MENU_MP3FindRecursive (const char_t* path, uint32_t reqIndex,
                                    int32_t* countP, char_t* resultStrP,
                                    char_t* resultNameP,
                                    uint32_t maxDirDepth)
{
    char_t              currpath[MAIN_VIEWER_PATHLEN] = {0};
    char_t              newpath [MAIN_VIEWER_PATHLEN] = {0};
    uint32_t            length, ii;
    F_FIND              entry;

    length = UTILS_strlen(path);
    if((path[length-1]=='*')&&
       (path[length-2]=='.')&&
       (path[length-3]=='*'))
    {
        for(ii=0;ii<length;ii++)
        {
            currpath[ii] = path[ii];
        }
    }
    else
    {
        MENU_CommonAppendpath ("*.*", path, currpath, FFALSE,
                               MAIN_VIEWER_PATHLEN);
    }

    if(!f_findfirst(currpath, &entry))
    {
        do
        {
            if((entry.filename[0] != 0) && !(entry.attr & F_ATTR_HIDDEN))
            {
                if(!(entry.attr & F_ATTR_DIR))
                {
                    if ( MENU_MP3FileIsMp3(entry.filename) )
                    {
                        (*countP)++;

                        if(*countP == reqIndex)
                        {
                            (void)UTILS_strncpy (resultNameP, entry.filename,
                                           MAIN_VIEWER_NAMELEN);
                            MENU_CommonAppendpath (entry.filename, path,
                                                   resultStrP, FFALSE,
                                                   MAIN_VIEWER_PATHLEN);
                            return;
                        }
                    }
                }

                else if((entry.attr & F_ATTR_DIR) && (maxDirDepth > 0) )
                {
                    MENU_CommonAppendpath (entry.filename, currpath, newpath,
                                           FTRUE, MAIN_VIEWER_PATHLEN);

                    MENU_MP3FindRecursive (newpath, reqIndex, countP, resultStrP,
                                          resultNameP, (uint32_t)(maxDirDepth - 1));

                    if(*countP == reqIndex)
                        return;
                }
            }
        } while (!f_findnext(&entry));
    }
}

FSTATIC void MENU_MP3FindCurrentSong(MENU_MP3_INST_S* inst)
{
    uint16_t            nn, ii, length;
    uint16_t            filenamepos= 0;
    fbool_t             foundcurrent;
    char_t              path[MAIN_VIEWER_PATHLEN] = {0};
    F_FIND              entry;

    inst->currentfile= 0;

    for ( nn= 0; nn < MAIN_VIEWER_PATHLEN; nn++ )
    {
        if(!inst->thisStr[nn]) break;

        if(inst->thisStr[nn] == '/')
            filenamepos= nn+1;
    }

    length = UTILS_strlen(inst->params.path);
    if((inst->params.path[length-1]=='*')&&
       (inst->params.path[length-2]=='.')&&
       (inst->params.path[length-3]=='*'))
    {
        for(ii=0;ii<length;ii++)
        {
            path[ii] = inst->params.path[ii];
        }
    }
    else
    {
        MENU_CommonAppendpath ("*.*", inst->params.path, path, FFALSE,
                               MAIN_VIEWER_PATHLEN);
    }

    if (!f_findfirst(path, &entry))
    {
        do
        {
            if((entry.filename[0] != 0) && !(entry.attr & F_ATTR_HIDDEN))
            {
                if ( !(entry.attr & F_ATTR_DIR) &&
                     (MENU_MP3FileIsMp3(entry.filename)) )
                {
                    foundcurrent= FTRUE;

                    for(nn= 0; nn<MAIN_VIEWER_NAMELEN; nn++)
                    {
                        if(inst->thisStr[nn+filenamepos] != entry.filename[nn])
                        {
                            foundcurrent= FFALSE;
                            break;
                        }

                        if ( (entry.filename[nn]           == 0) ||
                             (inst->thisStr[nn+filenamepos] == 0) )
                        {
                            break;
                        }
                    }
                    if(entry.filename[nn] || inst->thisStr[nn+filenamepos])
                        foundcurrent= FFALSE;

                    if(foundcurrent)
                        return;

                    inst->currentfile++;
                }
            }
        } while(!f_findnext(&entry));
    }
}

FSTATIC void MENU_MP3CountSongs (const char_t* path, uint32_t* count,
                                 uint32_t maxDirDepth)
{
    char_t              currpath[MAIN_VIEWER_PATHLEN] = {0};
    char_t              newpath [MAIN_VIEWER_PATHLEN] = {0};
    uint32_t            length, ii;
    F_FIND              entry;

    length = UTILS_strlen(path);
    if((path[length-1]=='*')&&
       (path[length-2]=='.')&&
       (path[length-3]=='*'))
    {
        for(ii=0;ii<length;ii++)
        {
            currpath[ii] = path[ii];
        }
    }
    else
    {
        MENU_CommonAppendpath ("*.*", path, currpath, FFALSE,
                               MAIN_VIEWER_PATHLEN);
    }

    if (!f_findfirst(currpath, &entry))
    {
        do
        {
            if( (entry.filename[0] != 0) &&
               !(entry.attr & F_ATTR_HIDDEN) )
            {
                if (!(entry.attr & F_ATTR_DIR ))
                {
                    if ( MENU_MP3FileIsMp3(entry.filename) )
                    {
                        (*count)++;
                    }
                }

                else if ( (entry.attr & F_ATTR_DIR ) &&
                          (maxDirDepth > 0) )
                {
                    MENU_CommonAppendpath (entry.filename, currpath, newpath,
                                           FTRUE, MAIN_VIEWER_PATHLEN);
                    MENU_MP3CountSongs    (newpath, count,
                                           (uint32_t)(maxDirDepth - 1));
                }
            }
        } while(!f_findnext(&entry));
    }

}

FSTATIC void MENU_MP3GetNextFile(MENU_MP3_INST_S* inst)
{
    if(inst->shuffle)
    {
        inst->nextfile= (inst->allcount * (inst->random & 0xFFFF)) >> 16;
    }
    else
    {
        inst->nextfile++;
        if ( inst->nextfile >= inst->allcount ) inst->nextfile = 0;
    }
}

FSTATIC void MENU_MP3SetThisTrackInfo (MENU_MP3_INST_S* inst)
{
    #ifdef APPL_JPEG_ENABLED
    JPEG_PIC_PARAMS_S jpegParams;
    #endif /* APPL_JPEG_ENABLED */

    MENU_MP3GetTrackInfo(inst, inst->thisStr, &(inst->tagsThis));

    inst->playStat.timeTotal = 0;
    inst->playStat.timeCurr  = 0;

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_CURTITLE);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_CURARTIST);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_CURALBUM);

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_PROGRESS);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_TIME);

    #ifdef APPL_JPEG_ENABLED

    if ( inst->tagsThis.jpegAvl )
    {
        memset(&jpegParams, 0, sizeof(jpegParams));

        jpegParams.path      = inst->thisStr;
        jpegParams.filepos   = inst->tagsThis.picPos;
        jpegParams.orient    = ROT_0;
        jpegParams.zoom      = 100;
        jpegParams.xpos      = 0;
        jpegParams.ypos      = 0;
        jpegParams.embedded  = FTRUE;

        (void)JPEG_Show(&jpegParams, NULL);
    }
    else
    {
        (void)JPEG_Clear();
    }

    #endif /* APPL_JPEG_ENABLED */
}

FSTATIC void MENU_MP3SetNextTrackInfo(MENU_MP3_INST_S* inst)
{
    MENU_MP3GetTrackInfo(inst, inst->nextStr, &(inst->tagsNext));

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_NEXTTITLE);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_NEXTARTIST);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MP3_ID_NEXTALBUM);
}

FSTATIC void MENU_MP3GetTrackInfo (MENU_MP3_INST_S* inst,
                                   const char_t* fileStr,
                                   MP3_FILE_INFO_S* tags)
{
    memset (tags, 0, sizeof(MP3_FILE_INFO_S));

    (void)MP3_GetInfo (fileStr, tags);
}

void MENU_MP3HandleTime()
{
    if(((menuMP3Dat->random & 0x2080) == 0x0080) || ((menuMP3Dat->random & 0x2080) == 0x2000))
        menuMP3Dat->random= (menuMP3Dat->random << 1) | 1;
    else
        menuMP3Dat->random= menuMP3Dat->random << 1;
}


FSTATIC void MENU_MP3StartPlay(MENU_MP3_INST_S* inst)
{
    int32_t                 retVal;

    (void)MAIN_MsgBoxHide(&(appDat->msgBox));

    retVal = MP3_Start(inst->thisStr);
    if( retVal != FAPI_OK)
    {
        MENU_CommonMsgShow(STR_MSG_MP3_FORMAT_NOT_SUPPORT, FTRUE);
    }
}

FSTATIC int32_t MENU_MP3PaintProgress(FAPI_SYS_HandleT h)
{
    MENU_MP3_INST_S*   inst= (MENU_MP3_INST_S*)h;

    OSD_DrawStatusBarHorizontal (inst->progressP->abspos.x,
                                 inst->progressP->abspos.y,
                                 inst->progresspos.width,
                                 inst->progresspos.height,
                                 inst->playStat.timeCurr,
                                 inst->playStat.timeTotal,
                                 OSD_COL_YELLOW,
                                 OSD_GetSchemeColor(OSD_SCHEMECOL_PROGRESSBAR_INACT),
                                 OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME)
                                 );

    return FAPI_OK;
}

FSTATIC int32_t MENU_MP3PaintSeparator(FAPI_SYS_HandleT h)
{
    MENU_MP3_INST_S*   inst= (MENU_MP3_INST_S*)h;

    OSD_DrawSeparatorHorizontal(inst->separatorP->abspos.x, inst->separatorP->abspos.y, inst->separatorpos.width);

    return FAPI_OK;
}

FSTATIC FINLINE fbool_t MENU_MP3FileIsMp3 (const char_t* fileName)
{
    uint16_t n;

    for ( n = 0; n < MAIN_VIEWER_NAMELEN; n++ )
    {
        if ( fileName[n] == 0 ) break;
    }

    if ( (n > 4)                &&
         (fileName[n-3] == 'M') &&
         (fileName[n-2] == 'P') &&
         (fileName[n-1] == '3') )
    {
        return FTRUE;
    }
    else
    {
        return FFALSE;
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
