/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_oad_scan.c
**
**              $Id: app_menu_oad_scan.c 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       OAD scan menu functions for Sat.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-20010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 408: Type mismatch with switch expression.
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"
#include "app_menu_scan.h"


/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_OAD_ID_TITLE = 0,
    MENU_OAD_ID_LIST,
    MENU_OAD_ID_HELPLIST
};

enum
{
    #ifdef APPL_FESAT_ENABLED
    MENU_OAD_ID_ENTRY_SATELLITE,
    #endif /* APPL_FESAT_ENABLED */

    MENU_OAD_ID_ENTRY_EDITTP,
    MENU_OAD_ID_ENTRY_STARTSCAN
};

typedef struct
{
    FAPI_SYS_HandleT            pnlHdl;
    FAPI_SYS_HandleT            listHdl;
    FAPI_SYS_HandleT            listHelpHdl;

    #ifdef APPL_FESAT_ENABLED
    MENU_SCAN_CFG_S             cfgSat;
    CFGSAT_SAT_S                tmpSatDat;
    #endif /* APPL_FESAT_ENABLED */

    MENU_SCAN_CFG_S*            pCfg;

    int8_t                      currFe;

    #ifdef APPL_FESAT_ENABLED

    int16_t                     currSatLstPos;
    CFGSAT_SAT_S                satInf;      //!< parameters of current satellite
    fbool_t                     satInfValid;

    #endif /* APPL_FESAT_ENABLED */

    TPLIST_ITEM_S               carrierInf;  //!< parameters of current carrier

} MENU_OAD_INST_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

MENU_OAD_INST_S* menuOadScanDat = NULL;

#ifdef APPL_FESAT_ENABLED
/*---------------------------------------------------------------------------*/
/* local functions for Satellite                                             */
/*---------------------------------------------------------------------------*/
FSTATIC void  MENU_OadSatGetDescrItem    (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item);
FSTATIC fbool_t MENU_OadSatBuild         (FAPI_SYS_HandleT h);
#endif

/*---------------------------------------------------------------------------*/
/* local functions for serving all front ends                                */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_OadLeave            (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC void    MENU_OadOnFocusSet       (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_OadOnFocusChg       (FAPI_SYS_HandleT h, uint8_t index,
                                          int32_t id);
FSTATIC fbool_t MENU_OadKeys             (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_OadUpdateTuner      (MENU_OAD_INST_S* inst);
FSTATIC fbool_t MENU_OadOnSelect         (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void    MENU_OadValChg           (FAPI_SYS_HandleT h,
                                          int32_t itemId, FGS_VAL_S newVal);
FSTATIC void    MENU_OadGetItem          (FAPI_SYS_HandleT h, int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item);
FSTATIC void    MENU_OadGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state, FGS_ITEM_S* item);

FSTATIC void    MENU_OadEntryStart       (MENU_OAD_INST_S* inst, FAPI_SYS_HandleT h);
FSTATIC void    MENU_OadEntryEnd         (MENU_OAD_INST_S* inst);
FSTATIC void    MENU_OadTpCallback       (int16_t satLstPos, int16_t carrierLstPos);
FSTATIC fbool_t MENU_OadStart     (MENU_OAD_INST_S* inst);
/*---------------------------------------------------------------------------*/
/* common used Init function serving all front ends                          */
/*---------------------------------------------------------------------------*/

/*!
*******************************************************************************
**
** \brief  Initial function for the OAD Menu
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_OadInit (void)
{
    MENU_OAD_INST_S* inst;
    MENU_SCAN_CFG_S     scanCfg;

    if ( menuOadScanDat != NULL )
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuOadScanDat = UTILS_Malloc(sizeof(MENU_OAD_INST_S));
    if ( menuOadScanDat == NULL )
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuOadScanDat, 0, sizeof(MENU_OAD_INST_S));

    inst = menuOadScanDat;

    /* set default scan menu parameters */

    memset(&scanCfg, 0, sizeof(scanCfg));

    scanCfg.opts.scanNIT = FTRUE;
    scanCfg.opts.choice  = AV_SCANCHOICE_ALL;
    scanCfg.opts.filter  = AV_SCANFILTER_ALL;
    scanCfg.tpLstPos     = 0;

    #ifdef APPL_FESAT_ENABLED

    inst->cfgSat                  = scanCfg;
    inst->cfgSat.system           = FE_SYSTEM_TYPE_SATELLITE;
    inst->cfgSat.network.type     = FE_SYSTEM_TYPE_SATELLITE;
    inst->cfgSat.network.satIdx   = -1; /* mark as invalid */

    inst->pCfg        = &(inst->cfgSat);
    inst->satInfValid = FFALSE;

    #endif /* APPL_FESAT_ENABLED */

    return FAPI_OK;
}

FSTATIC void MENU_OadEntryStart (MENU_OAD_INST_S* inst, FAPI_SYS_HandleT h)
{
    inst->currFe        = -1;

    if ( h != NULL )
    {
        /* reset default configuration */
        inst->pCfg->tpLstPos       = 0;
    }
}

FSTATIC void MENU_OadEntryEnd (MENU_OAD_INST_S* inst)
{
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_OAD_ID_LIST);

    /* set global focus on menu panel */
    (void)MENUSTACK_FocusClaim ((int32_t)MENUID_SCAN);

    /* turn visibility on */
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);

    (void)MENU_OadUpdateTuner(inst); /* start tuning mode */

    MENUSTACK_SetCleanup (MENU_OadLeave, (FAPI_SYS_HandleT)inst);
}

/*---------------------------------------------------------------------------*/
/* functions for serving satellite front ends                                */
/*---------------------------------------------------------------------------*/

#ifdef APPL_FESAT_ENABLED

/************** Satellite Scan Menu *******************************************/

/*!
*******************************************************************************
**
** \brief  Entry function.
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_OadSatEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_OAD_INST_S*   inst = menuOadScanDat;
    uint32_t                lastFocusItem;
    CFGSAT_IDX              newIdx;
    fbool_t                 satPosValid = FFALSE;
    int16_t                 lstPos;

    DBG_Assert(inst != 0);
    if ( inst == NULL ) return FFALSE; /* to satisfy MISRA */

    if ( test == FTRUE ) return FTRUE;

    (void)MAIN_StopPresentation(); /* stop running presentation */

    MENU_OadEntryStart(inst, h);

    inst->currSatLstPos = 0;

    /* ----- get initial satellite selection ------------------------------- */

    if ( appDat->currSatIdx >= 0 )
    {
        if ( CFGSAT_GetListPos (appDat->currSatIdx, &lstPos) == FAPI_OK )
        {
            inst->currSatLstPos = lstPos;
            satPosValid         = FTRUE;
        }
    }

    /* If satellite is invalid/not in list, choose first satellite in list */
    if ( !satPosValid )
    {
        inst->currSatLstPos = 0;

        if ( CFGSAT_GetIndex (inst->currSatLstPos, &newIdx) == FAPI_OK )
        {
            appDat->currSatIdx = newIdx;
            satPosValid        = FTRUE;
        }
    }

    if ( satPosValid )
    {
        if ( CFGSAT_GetSatData (inst->currSatLstPos, &(inst->satInf))
                                == FAPI_OK )
        {
            inst->satInfValid = FTRUE;
        }
    }

    inst->pCfg->network.satIdx = appDat->currSatIdx;

    (void)MENU_OadSatBuild((FAPI_SYS_HandleT)inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {

        lastFocusItem = MENU_OAD_ID_ENTRY_SATELLITE;
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);

    MENU_OadEntryEnd (inst);

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  Set the descriptions for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_OadSatGetDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                      uint32_t index,
                                      FGS_ITEMSTATE_E state,
                                      FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_OAD_ID_ENTRY_SATELLITE:
            strIdx = STR_ENTRY_OAD_SATELLITE;
            break;

        case MENU_OAD_ID_ENTRY_EDITTP:
            strIdx = STR_ENTRY_OAD_EDITTP;
            break;

        case MENU_OAD_ID_ENTRY_STARTSCAN:
            strIdx = STR_ENTRY_OAD_STARTUPDATE;
            break;

        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

/*!
*******************************************************************************
**
** \brief  Builds the menu pannel and its members.
**
** \param h        Address of menu handle.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_OadSatBuild(FAPI_SYS_HandleT h)
{
    MENU_OAD_INST_S*     inst = (MENU_OAD_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_LIST_OPEN_S       listPars;
    FGS_LIST_OPEN_S       helpListPars;
    FAPI_SYS_HandleT      itemHdl;
    FGS_BOX_OPEN_S        boxPars;
    FGS_ITEM_TXT_OPEN_S   txtPars;
    FGS_ITEM_TXT_OPEN_S   helpPars;
    MAIN_SIGBOX_OPEN_S    sigPars;
    FGS_ITEM_SEL_OPEN_S   selPars;
    FGS_POS_S             itemPos;
    FGS_ITEM_S            item;
    FGS_POS_S             valuePos;
    FGS_POS_S             descrPos;
    int32_t               retVal;
    uint16_t              helpWidth;
    uint16_t              stackedFrameWidth;
    uint16_t              innerHeight;
    uint32_t              numLines = 5;
    int16_t               numCarriers = 0;
    int16_t               numSats = 0;

    memset(&helpListPars, 0, sizeof(helpListPars));
    memset(&boxPars,      0, sizeof(boxPars));
    memset(&txtPars,      0, sizeof(txtPars));
    memset(&helpPars,     0, sizeof(helpPars));
    memset(&sigPars,      0, sizeof(sigPars));
    memset(&selPars,      0, sizeof(selPars));
    memset(&item,         0, sizeof(item));

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_LOKR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                        + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_OadOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SCAN, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_OAD_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_OAD_SELTP,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_OadSatGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_OadGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_OadOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_OadOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_OadValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_OAD_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- create help list ------------------------------------------ */

        /* NOTE: Common help list method not used here because of non-linear
                 item position. */

        helpListPars = listPars;

        helpListPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                       SCALER_GetCurrVal(MAIN_WIDTH) +
                                       SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                       SCALER_GetCurrVal(HELP_LR_PADDING) +
                                       stackedFrameWidth;
        helpListPars.pos.width       = helpWidth
                                   - 2 * SCALER_GetCurrVal(HELP_LR_PADDING);
        helpListPars.bkgrType        = 0;
        helpListPars.getDescrItem    = MENU_OadGetHelpDescrItem;
        helpListPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        helpListPars.getItem         = NULL;
        helpListPars.getItemArg      = NULL;
        helpListPars.onFocusChg      = NULL;
        helpListPars.onFocusChgArg   = NULL;
        helpListPars.onSelect        = NULL;
        helpListPars.onSelectArg     = NULL;
        helpListPars.onValChg        = NULL;
        helpListPars.onValChgArg     = NULL;
        helpListPars.pInst           = &(appDat->helpListInst);

        retVal = FGS_LIST_Open (&helpListPars, MENU_OAD_ID_HELPLIST,
                                inst->pnlHdl, &(inst->listHelpHdl));
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.x              = 0;
        helpPars.pos.y              = 0;
        helpPars.pos.width          = helpListPars.pos.width;
        helpPars.pos.height         = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        helpPars.pos.alignment      = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        helpPars.bkgrType           = OSD_LSTITEM_BKGR_HELP;
        helpPars.descrPos.x         = 0;
        helpPars.descrPos.y         = 0;
        helpPars.descrPos.width     = helpPars.pos.width;
        helpPars.descrPos.height    = helpPars.pos.height;
        helpPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----- menu lines -------------------------------------------------*/

        memset(&txtPars, 0, sizeof(FGS_ITEM_TXT_OPEN_S));

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;

        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;

        /* ----- Scanned Satellite ----------------------------------------- */

        (void)CFGSAT_GetNumSats(&numSats);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = numSats;
        selPars.params.index        = inst->currSatLstPos;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_OAD_ID_ENTRY_SATELLITE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_OAD_ID_ENTRY_SATELLITE,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                          + SCALER_GetCurrVal(LIST_LINE_PADDING);

        itemPos.y      += 2 * SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Carrier --------------------------------------------------- */

        (void)TPLIST_GetNumCarriers (&(inst->pCfg->network), &numCarriers);

        if ( (int16_t)inst->pCfg->tpLstPos >= numCarriers )
        {
            inst->pCfg->tpLstPos = 0;
        }

        selPars.pos                 = itemPos;
        selPars.params.numItems     = numCarriers;
        selPars.params.index        = inst->pCfg->tpLstPos;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                     MENU_OAD_ID_ENTRY_EDITTP,
                                     inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_OAD_ID_ENTRY_EDITTP,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                          + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Signal bar ------------------------------------------------- */

        sigPars.pnlHdl     = inst->pnlHdl;
        sigPars.pos.x      = itemPos.x + listPars.pos.x
                             + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        sigPars.pos.y      = itemPos.y + listPars.pos.y
                             + SCALER_GetCurrVal(LIST_LINE_PADDING);
        sigPars.descrWidth = ((uint16_t)(itemPos.width -
                              SCALER_GetCurrVal(MAIN_LEFT_PADDING))/(2));
        sigPars.barWidth   = ((uint16_t)(itemPos.width-
                              SCALER_GetCurrVal(MAIN_LEFT_PADDING)-
                              SCALER_GetCurrVal(MAIN_RIGHT_PADDING))/(2));
        sigPars.feIdx      = -1;

        retVal = MAIN_SignalBoxOpen(&(appDat->sigBox), &sigPars);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += MAIN_SignalBoxGetHeight();
        itemPos.y      += (2*SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ---start scan --------------------------------------------------- */

        helpPars.pos.y  = itemPos.y;
        txtPars.pos     = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_OAD_ID_ENTRY_STARTSCAN,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_OAD_ID_ENTRY_STARTSCAN,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);

        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}
#endif /* APPL_FESAT_ENABLED */

/*---------------------------------------------------------------------------*/
/* local functions for serving all front ends                                */
/*---------------------------------------------------------------------------*/

/*!
*******************************************************************************
**
** \brief  Leave the oad Menu.
**
** More detailed description
**
** \param h        Address of menu handle.
** \param force    .
**
** \return FTRUE
******************************************************************************/
FSTATIC fbool_t MENU_OadLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_OAD_INST_S*   inst = (MENU_OAD_INST_S*)h;

    if ( inst->currFe >= 0 )
    {
        (void)AV_TuneModeStop(inst->currFe);
    }

    (void)MAIN_SignalBoxClose((&appDat->sigBox));

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
        case FGS_KEY_EXIT:
            (void)MAIN_RestartPresentation();
            break;

        default:
            break;
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_SCAN);

    inst->pnlHdl = NULL;
    return FTRUE;
}


/*!
*******************************************************************************
**
** \brief  Common used function called when the menu gets/looses the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_OadOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_OAD_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_MENU,  MENU_OadKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK,  MENU_OadKeys, h, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
    }
}

/*!
*******************************************************************************
**
** \brief  Activates the help icons for the current menu list item.
**
** \param h        Address of menu handle.
** \param index    unused.
** \param id       List id gets focus.
**
******************************************************************************/
FSTATIC void MENU_OadOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_OAD_INST_S*   inst = (MENU_OAD_INST_S*)h;

    (void)FGS_LIST_SetMemberFocus(inst->listHelpHdl, id);
}

/*!
*******************************************************************************
**
** \brief  Common used function to handle the OK-key for the selected listitem.
**
** \param h        Address of menu handle.
** \param itemId   Listitem Id.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_OadOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_OAD_INST_S*   inst = (MENU_OAD_INST_S*)h;
    MENUSTACK_ENTRY         newEntry = NULL;
    FAPI_SYS_HandleT        arg      = NULL;
    uint8_t                 subLines = 0;
    MENU_SELECTLIST_OPEN_S  selectLst;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    FGS_LOC_S               memberLoc;
    fbool_t                 openSelectLst = FFALSE;
    MENU_TPLIST_PARAMS_S    tplistPars;

    switch ( itemId )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_OAD_ID_ENTRY_SATELLITE :
            openSelectLst = FTRUE;
            subLines = 5;
            break;

        #endif /* APPL_FESAT_ENABLED */

        case MENU_OAD_ID_ENTRY_EDITTP:
            /* currently only available for satellite scan menu */
            if( inst->pCfg->system == FE_SYSTEM_TYPE_SATELLITE )
            {
                #ifdef APPL_FESAT_ENABLED
                if(inst->pCfg->system == FE_SYSTEM_TYPE_SATELLITE)
                {
                    tplistPars.satLstPos     = inst->currSatLstPos;
                }
                #endif
                tplistPars.system        = inst->pCfg->system;
                tplistPars.carrierLstPos = inst->pCfg->tpLstPos;
                tplistPars.lockSat       = FFALSE;
                tplistPars.leaveCb       = MENU_OadTpCallback;

                arg      = (FAPI_SYS_HandleT)(&tplistPars);
                newEntry = MENU_TpListEntry;
            }
            else
            {
                openSelectLst = FTRUE;
                subLines      = 4;
            }
            break;

        case MENU_OAD_ID_ENTRY_STARTSCAN:
            return MENU_OadStart(inst);

        default:
            MENU_CommonMsgShow(STR_MSG_NOTIMPLEMENTEDYET, FFALSE);
            return FFALSE;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.width     = (4 * SCALER_GetCurrVal(SUB_LIST_WIDTH)) / 3;
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.x         = memberLoc.x + SCALER_GetCurrVal(MAIN_WIDTH) -
                                  selectLst.pos.width;
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_OadGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;
        selectLst.onExit        = NULL;
        selectLst.onExitArg     = h;

        return MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }
    else
    {
        /* store last focus */
        MENUSTACK_SetData(itemId);

        if(newEntry == NULL) return FFALSE;
        else                 return MENUSTACK_Up(newEntry, arg);
    }
}

FSTATIC void MENU_OadTpCallback (int16_t satLstPos, int16_t carrierLstPos)
{
    MENU_OAD_INST_S*   inst = menuOadScanDat;

    #ifdef APPL_FESAT_ENABLED
    CFGSAT_IDX newIdx;

    if( inst->pCfg == &inst->cfgSat )
    {
        if ( CFGSAT_GetIndex (satLstPos, &newIdx) == FAPI_OK )
        {
            appDat->currSatIdx = newIdx;
        }

    }
    #endif
    inst->pCfg->tpLstPos = carrierLstPos;
}
/*!
*******************************************************************************
**
** \brief  Common used function to handles the Value change for the selected Listitem.
**
** \param h        Address of menu handle.
** \param itemId   Listitem Id.
** \param newVal   New Value for Listitem.
**
******************************************************************************/
FSTATIC void MENU_OadValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    MENU_OAD_INST_S*     inst = (MENU_OAD_INST_S*)h;
    int16_t               numCarriers = 0;
    FGS_ITEM_SEL_PARAMS_S selParams;

    #ifdef APPL_FESAT_ENABLED
    CFGSAT_IDX            newIdx;
    #endif /* APPL_FESAT_ENABLED */

    switch ( itemId )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_OAD_ID_ENTRY_SATELLITE: /* satellite has changed */

            inst->satInfValid = FFALSE;

            if ( CFGSAT_GetIndex (newVal.number, &newIdx) == FAPI_OK )
            {
                appDat->currSatIdx         = newIdx;

                inst->pCfg->network.satIdx = newIdx;
                inst->currSatLstPos        = newVal.number;
                inst->pCfg->tpLstPos       = 0;

                if ( CFGSAT_GetSatData (inst->currSatLstPos,
                                        &(inst->satInf)) == FAPI_OK )
                {
                    inst->satInfValid = FTRUE;
                }

                (void)TPLIST_GetNumCarriers (&(inst->pCfg->network), &numCarriers);

                selParams.numItems = numCarriers;
                selParams.index    = inst->pCfg->tpLstPos;

                /* update carrier display */
                (void)FGS_LIST_SetMemberParams (inst->listHdl,
                                          MENU_OAD_ID_ENTRY_EDITTP, &selParams);
                (void)MENU_OadUpdateTuner(inst);
            }
            break;

        #endif /* APPL_FESAT_ENABLED */

        case MENU_OAD_ID_ENTRY_EDITTP:
            inst->pCfg->tpLstPos = (uint16_t)newVal.number;

            (void)MENU_OadUpdateTuner(inst);
            break;

        default:
            break;
    }
}

/*!
*******************************************************************************
**
** \brief  Set the value (e.g. text, number, icon, ...) for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_OadGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_OAD_INST_S*     inst = (MENU_OAD_INST_S*)h;
    uint32_t                 bufPos = 0;

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    switch ( id )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_OAD_ID_ENTRY_SATELLITE :
            if ( CFGSAT_GetSatData(index, &(inst->tmpSatDat)) == FAPI_OK )
            {
                item->type            = FGS_ITEM_TYPE_STRING;
                item->item.str.chr    = STR_Build(inst->tmpSatDat.nameCharSet,
                                                  inst->tmpSatDat.name);
            }
            break;

        #endif /* APPL_FESAT_ENABLED */

        case MENU_OAD_ID_ENTRY_EDITTP :
            item->type                = FGS_ITEM_TYPE_STRING;
            if ( TPLIST_GetCarrierData (&(inst->pCfg->network), index,
                                        &(inst->carrierInf)) != FAPI_OK )
            {
                item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                break;
            }

            switch ( inst->pCfg->system )
            {
                #ifdef APPL_FESAT_ENABLED
                case FE_SYSTEM_TYPE_SATELLITE:
                    sprintf (appDat->asciiBuf, "%li: %li ", index + 1,
                             inst->carrierInf.pars.sat.frequency / 1000);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                            OSD_GetStr(STR_UNIT_MHZ),
                                             MAIN_LINEBUFCHARSET);

                    if ( inst->carrierInf.pars.sat.pol == FE_POL_HOR)
                    {
                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 OSD_GetStr(STR_UNIT_HOR),
                                                 MAIN_LINEBUFCHARSET);
                    }
                    else
                    {
                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 OSD_GetStr(STR_UNIT_VER),
                                                 MAIN_LINEBUFCHARSET);
                    }
                    break;
                #endif /* APPL_FESAT_ENABLED */

                default:
                    (void)STR_BufClear(NULL);
                    break;
            }

            item->item.str.chr = STR_IntLineStr();
            break;

        default:
            item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            break;
    }

    switch ( item->type )
    {
        case FGS_ITEM_TYPE_STRING:
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;

            switch ( state )
            {
                default:
                case FGS_NONFOCUS:
                    item->item.str.col = appDat->txtAttr.colNonFocus;
                    break;
                case FGS_FOCUS:
                    item->item.str.col = appDat->txtAttr.colFocus;
                    break;
                case FGS_DISABLED:
                    item->item.str.col = appDat->txtAttr.colDisabled;
                    break;
            }
            break;

        case FGS_ITEM_TYPE_NUMBER:
            item->item.number.font          = MAIN_FONT;
            switch ( state )
            {
                default:
                case FGS_NONFOCUS:
                    item->item.number.col = appDat->txtAttr.colNonFocus;
                    break;
                case FGS_FOCUS:
                    item->item.number.col = appDat->txtAttr.colFocus;
                    break;
                case FGS_DISABLED:
                    item->item.number.col = appDat->txtAttr.colDisabled;
                    break;
            }
            break;

        default:
            break;
    }
}

/*!
*******************************************************************************
**
** \brief  Common used function to get the help description item for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_OadGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                           FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        #ifdef APPL_FESAT_ENABLED
        case MENU_OAD_ID_ENTRY_SATELLITE:
        #endif /* APPL_FESAT_ENABLED */

        case MENU_OAD_ID_ENTRY_EDITTP   : strIdx = STR_ENTRY_LOKR;
                                           break;

        case MENU_OAD_ID_ENTRY_STARTSCAN: strIdx = STR_ENTRY_OK;
                                           break;

        default:  return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}


/*!
*******************************************************************************
**
** \brief  Common used function to handle the Keys adapted by this menu.
**
** \param h        Address of menu handle.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_OadKeys (FAPI_SYS_HandleT h)
{
    fbool_t             keyUsed = FTRUE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
            /* FALLTHHROUGHT */
        case FGS_KEY_BACK:
            keyUsed = MENUSTACK_Back (NULL);
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

int32_t MENU_OadUpdateTuner (MENU_OAD_INST_S* inst)
{
    int32_t retVal;

    if ( inst->currFe >= 0 )
    {
        (void)AV_TuneModeStop(inst->currFe);
        inst->currFe = -1;
    }

    retVal = AV_TuneModeStart (-1, &(inst->pCfg->network),
                               inst->pCfg->tpLstPos,
                               NULL, &(inst->currFe));

    /* update signal bar */
    (void)MAIN_SignalBoxSetFeIdx (&(appDat->sigBox), inst->currFe);
    (void)MAIN_SignalBoxUpdate   (&(appDat->sigBox));

    return retVal;
}


FSTATIC fbool_t MENU_OadStart (MENU_OAD_INST_S* inst)
{
    MENU_OADUPDATE_UpdateInfo(&(inst->pCfg->network),&(inst->carrierInf));
    return MENUSTACK_Back(NULL);
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
