/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_osd.c
**
**              $Id: app_menu_osd.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD osd menu.
**
** Detail description. [\ref app_smartgo_app_ref_osd]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info 713: Loss of precision (assignment) (unsigned int to short).
** - Info 715: Symbol not referenced.
** - Info 732: Loss of sign (assignment) (int to unsigned short).
** - Info 737: Loss of sign in promotion from int to unsigned int.
** - Info 774: Boolean within 'if' always evaluates to True.
** - Note 961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**   C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_OSD_ID_TITLE = 0,
    MENU_OSD_ID_LIST,
    MENU_OSD_ID_HELPLIST
};

enum
{
    MENU_OSD_ID_ENTRY_DISPLAYTIME = 0,
    MENU_OSD_ID_ENTRY_TRANSPARENCY,
    MENU_OSD_ID_ENTRY_COLORSCHEME,
    MENU_OSD_ID_ENTRY_HORALIGN,
    MENU_OSD_ID_ENTRY_VERALIGN,
    MENU_OSD_ID_ENTRY_NUMS
};

enum
{
    MENU_OSD_HOR_LEFT = 0,
    MENU_OSD_HOR_CENTER,
    MENU_OSD_HOR_RIGHT,
    MENU_OSD_HOR_NUMS
};

enum
{
    MENU_OSD_VER_TOP = 0,
    MENU_OSD_VER_CENTER,
    MENU_OSD_VER_BOTTOM,
    MENU_OSD_VER_NUMS
};

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    listHdl;
    FAPI_SYS_HandleT    listHelpHdl;
    fbool_t        transparencyChg;
    uint16_t          panelWidth;
    uint16_t          panelHeight;

} MENU_OSD_INST_S;

FSTATIC MENU_OSD_INST_S* menuOsdDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_OsdLeave        (FAPI_SYS_HandleT h,
                                      fbool_t force);
FSTATIC fbool_t MENU_OsdOnSelect     (FAPI_SYS_HandleT h,
                                      int32_t itemId);
FSTATIC void    MENU_OsdValChg       (FAPI_SYS_HandleT h,
                                      int32_t itemId,
                                      FGS_VAL_S newVal);
FSTATIC void    MENU_OsdGetDescrItem (FAPI_SYS_HandleT h,
                                      int32_t id,
                                      uint32_t index,
                                      FGS_ITEMSTATE_E state,
                                      FGS_ITEM_S* item);
FSTATIC void    MENU_OsdGetItem      (FAPI_SYS_HandleT h,
                                      int32_t id,
                                      uint32_t index,
                                      FGS_ITEMSTATE_E state,
                                      FGS_ITEM_S* item);
FSTATIC fbool_t MENU_OsdBuild        (FAPI_SYS_HandleT h);
FSTATIC void    MENU_OsdReposition   (MENU_OSD_INST_S* inst);
FSTATIC void    MENU_OsdOnFocusChg   (FAPI_SYS_HandleT h,
                                      uint8_t index,
                                      int32_t id);
FSTATIC void    MENU_OsdOnFocusSet   (FAPI_SYS_HandleT h,
                                      fbool_t hasFocus);
FSTATIC void    MENU_OsdGetHelpDescrItem (FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item);

int32_t MENU_OsdInit (void)
{
    if ( menuOsdDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuOsdDat = UTILS_Malloc(sizeof(MENU_OSD_INST_S));
    if ( menuOsdDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuOsdDat, 0, sizeof(MENU_OSD_INST_S));

    return FAPI_OK;
}

fbool_t MENU_OsdEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_OSD_INST_S* inst = menuOsdDat;
    uint32_t              lastFocusItem;

    if ( test == FTRUE ) return FTRUE;

    inst->transparencyChg = FFALSE;

    (void)MENU_OsdBuild((FAPI_SYS_HandleT)inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = MENU_OSD_ID_ENTRY_DISPLAYTIME;
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_OSD_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_OSD);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_OsdLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

FSTATIC void MENU_OsdOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_OSD_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);
}

FSTATIC void MENU_OsdOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_OSD_INST_S*   inst = (MENU_OSD_INST_S*)h;
    FGS_LNAVI_PARAMS_S naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}

FSTATIC fbool_t MENU_OsdLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_OSD_INST_S* inst = (MENU_OSD_INST_S*)h;

    if ( force == FFALSE )
    {
        MAIN_SaveSettingsIfChanged();
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_OSD);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC fbool_t MENU_OsdOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_OSD_INST_S*         inst     = (MENU_OSD_INST_S*)h;
    MENUSTACK_ENTRY          newEntry = NULL;
    FAPI_SYS_HandleT                arg      = NULL;
    uint8_t                       subLines;
    MENU_SELECTLIST_OPEN_S   selectLst;
    FGS_ITEM_SEL_PARAMS_S    selParams;
    FGS_LOC_S                memberLoc;
    fbool_t                    openSelectLst = FFALSE;

    switch ( itemId )
    {
        case MENU_OSD_ID_ENTRY_COLORSCHEME:
            openSelectLst  = FTRUE;
            subLines       = 3;
            break;

        case MENU_OSD_ID_ENTRY_HORALIGN:
            openSelectLst  = FTRUE;
            subLines       = 3;
            break;

        case MENU_OSD_ID_ENTRY_VERALIGN:
            openSelectLst  = FTRUE;
            subLines       = 3;
            break;

        default:
            return FFALSE;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.x         = memberLoc.x +
                                  SCALER_GetCurrVal(MAIN_WIDTH) -
                                  SCALER_GetCurrVal(SUB_LIST_WIDTH) -
                                  SCALER_GetCurrVal(OSD_POS_X);
        selectLst.pos.y         = memberLoc.y -
                                  SCALER_GetCurrVal(OSD_POS_Y);
        selectLst.pos.width     = SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_OsdGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;
        selectLst.onExit        = NULL;
        selectLst.onExitArg     = NULL;

        return MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }
    else
    {
        /* store last focus */
        MENUSTACK_SetData(itemId);

        if(newEntry == NULL) return FFALSE;
        else                 return MENUSTACK_Up(newEntry, arg);
    }
}

FSTATIC void MENU_OsdValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    MENU_OSD_INST_S* inst  = (MENU_OSD_INST_S*)h;
    uint32_t         align = 0;

    switch ( itemId )
    {
        case MENU_OSD_ID_ENTRY_DISPLAYTIME:
            (void)SETTINGS_DisplayTimeSet(newVal.number);
            break;

        case MENU_OSD_ID_ENTRY_TRANSPARENCY:
            (void)OSD_SetTransparency((OSD_TRANSPARENCY_E)newVal.number);
            break;

        case MENU_OSD_ID_ENTRY_COLORSCHEME:
            (void)OSD_SetColorScheme((OSD_COLSCHEME_E)newVal.number);
            (void)FGS_PANEL_Refresh(inst->pnlHdl,NULL);
            break;

        case MENU_OSD_ID_ENTRY_HORALIGN:

            (void)MAIN_UserDataGet(USERDAT_MENUALIGNMENT, &align);

            align &= (~FGS_ALIGN_MASK_HOR);

            switch ( newVal.number )
            {
                default:
                case MENU_OSD_HOR_LEFT:   align |= FGS_ALIGN_HOR_LEFT;
                                          break;

                case MENU_OSD_HOR_CENTER: align |= FGS_ALIGN_HOR_CENTER;
                                          break;

                case MENU_OSD_HOR_RIGHT:  align |= FGS_ALIGN_HOR_RIGHT;
                                          break;
            }

            (void)MAIN_UserDataSet(USERDAT_MENUALIGNMENT, align);

            MENU_OsdReposition(inst);
            break;

        case MENU_OSD_ID_ENTRY_VERALIGN:

            (void)MAIN_UserDataGet(USERDAT_MENUALIGNMENT, &align);

            align &= (~FGS_ALIGN_MASK_VER);

            switch ( newVal.number )
            {
                default:
                case MENU_OSD_VER_TOP:    align |= FGS_ALIGN_VER_TOP;
                                          break;

                case MENU_OSD_VER_CENTER: align |= FGS_ALIGN_VER_CENTER;
                                          break;

                case MENU_OSD_VER_BOTTOM: align |= FGS_ALIGN_VER_BOTTOM;
                                          break;
            }

            (void)MAIN_UserDataSet(USERDAT_MENUALIGNMENT, align);

            MENU_OsdReposition(inst);
            break;

        default:
            break;
    }
}

FSTATIC void MENU_OsdGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_OSD_ID_ENTRY_DISPLAYTIME  : strIdx = STR_ENTRY_OSD_DISPLAYTIME;  break;
        case MENU_OSD_ID_ENTRY_TRANSPARENCY : strIdx = STR_ENTRY_OSD_TRANSPARENCY; break;
        case MENU_OSD_ID_ENTRY_COLORSCHEME  : strIdx = STR_ENTRY_OSD_COLORSCHEME;  break;
        case MENU_OSD_ID_ENTRY_HORALIGN     : strIdx = STR_ENTRY_OSD_HORALIGN;     break;
        case MENU_OSD_ID_ENTRY_VERALIGN     : strIdx = STR_ENTRY_OSD_VERALIGN;     break;
        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_OsdGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                       FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        case MENU_OSD_ID_ENTRY_DISPLAYTIME:
            strIdx = STR_ENTRY_09LR;
            break;
        case MENU_OSD_ID_ENTRY_TRANSPARENCY:
        case MENU_OSD_ID_ENTRY_COLORSCHEME:
        case MENU_OSD_ID_ENTRY_HORALIGN:
        case MENU_OSD_ID_ENTRY_VERALIGN:
            strIdx = STR_ENTRY_LOKR;
            break;

        default:  return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}


FSTATIC void MENU_OsdGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                              FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t                strIdx;
    uint16_t                dispTime = SETTINGS_DEFAULT_DISPTIME;
    OSD_TRANSPARENCY_E transp   = OSD_TRANSPARENCY_100;
    uint32_t                col;

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: col = appDat->txtAttr.colDisabled; break;
    }

    switch ( id )
    {
        case MENU_OSD_ID_ENTRY_DISPLAYTIME:
            (void)SETTINGS_DisplayTimeGet(&dispTime);

            item->type                      = FGS_ITEM_TYPE_NUMBER;
            item->item.number.value         = dispTime;
            item->item.number.format        = FGS_FORMAT_NUM_NORM;
            item->item.number.decimalPlaces = 0;
            item->item.number.unitStr       = OSD_GetStr(STR_UNIT_SECS);
            item->item.number.col           = col;
            item->item.number.font          = MAIN_FONT;
            break;

        case MENU_OSD_ID_ENTRY_TRANSPARENCY:
            (void)OSD_GetTransparency(&transp);

            item->type                      = FGS_ITEM_TYPE_NUMBER;
            item->item.number.value         = (int32_t)transp * 10;
            item->item.number.format        = FGS_FORMAT_NUM_NORM;
            item->item.number.decimalPlaces = 0;
            item->item.number.unitStr       = OSD_GetStr(STR_UNIT_PERCENT);
            item->item.number.col           = col;
            item->item.number.font          = MAIN_FONT;
            break;

        case MENU_OSD_ID_ENTRY_COLORSCHEME:
            switch ( index )
            {
                case 0 : strIdx = STR_VALUE_OSD_COLOR_BLUE; break;
                case 1 : strIdx = STR_VALUE_OSD_COLOR_GREEN; break;
                case 2 : strIdx = STR_VALUE_OSD_COLOR_RED; break;
                case 3 : strIdx = STR_VALUE_OSD_COLOR_BLACK; break;
                default: strIdx = STR_EMPTYSTRING; break;
            }

            item->item.str.chr        = OSD_GetStr(strIdx);
            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.col        = col;
            break;

        case MENU_OSD_ID_ENTRY_HORALIGN:
            switch ( index )
            {
                case MENU_OSD_HOR_LEFT:   strIdx = STR_VALUE_OSD_ALIGN_LEFT;
                                          break;

                case MENU_OSD_HOR_RIGHT:  strIdx = STR_VALUE_OSD_ALIGN_RIGHT;
                                          break;

                case MENU_OSD_HOR_CENTER: strIdx = STR_VALUE_OSD_ALIGN_CENTER;
                                          break;

                default:                  strIdx = STR_EMPTYSTRING;
                                          break;
            }

            item->item.str.chr        = OSD_GetStr(strIdx);
            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.col        = col;
            break;

        case MENU_OSD_ID_ENTRY_VERALIGN:
            switch ( index )
            {
                case MENU_OSD_VER_TOP:    strIdx = STR_VALUE_OSD_ALIGN_TOP;
                                          break;

                case MENU_OSD_VER_BOTTOM: strIdx = STR_VALUE_OSD_ALIGN_BOTTOM;
                                          break;

                case MENU_OSD_VER_CENTER: strIdx = STR_VALUE_OSD_ALIGN_CENTER;
                                          break;

                default:                  strIdx = STR_EMPTYSTRING;
                                          break;
            }

            item->item.str.chr        = OSD_GetStr(strIdx);
            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.col        = col;
            break;

        default:  return;
    }
}

FSTATIC fbool_t MENU_OsdBuild(FAPI_SYS_HandleT h)
{
    MENU_OSD_INST_S*        inst = (MENU_OSD_INST_S*)h;
    FGS_PANEL_OPEN_S        pnlPars;
    FGS_LIST_OPEN_S         listPars;
    FAPI_SYS_HandleT        itemHdl;
    FGS_ITEM_SEL_OPEN_S     selPars;
    FGS_ITEM_NUM_OPEN_S     numPars;
    FGS_POS_S               itemPos;
    FGS_POS_S               valuePos;
    FGS_POS_S               descrPos;
    int32_t                 retVal;
    uint16_t                helpWidth=0;
    uint16_t                stackedFrameWidth=0;
    uint32_t                numLines = MENU_OSD_ID_ENTRY_NUMS;
    uint16_t                innerHeight;
    uint32_t                menuAlign = 0;
    OSD_TRANSPARENCY_E      transparency = SETTINGS_DEFAULT_TRANSPARENCY;
    OSD_COLSCHEME_E         colScheme    = SETTINGS_DEFAULT_COLSCHEME;

    (void)MAIN_UserDataGet(USERDAT_MENUALIGNMENT, &menuAlign);

    memset(&numPars,  0, sizeof(numPars));
    memset(&selPars,  0, sizeof(selPars));

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_09LR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth,
                                        1, &stackedFrameWidth);

        /* store panel width and height to be reused for repositioning. */
        inst->panelWidth  = pnlPars.pos.width;
        inst->panelHeight = pnlPars.pos.height;

        pnlPars.onFocusSet      = MENU_OsdOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_OSD, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_OSD_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth,STR_TITLE_OSD,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_OsdGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_OsdGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_OsdOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_OsdOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_OsdValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_OSD_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 75,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* --- Display time --------------------------------------------*/

        numPars.pos                 = itemPos;
        numPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        numPars.descrPos            = descrPos;
        numPars.valuePos            = valuePos;
        numPars.type                = FGS_NITEM_TYPE_NUMBER;
        numPars.wraparound          = FTRUE;
        numPars.params.minValue     = SETTINGS_DISPTIME_MIN;
        numPars.params.maxValue     = SETTINGS_DISPTIME_MAX;
        numPars.params.maxDigits    = 2;
        numPars.params.step         = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_OSD_ID_ENTRY_DISPLAYTIME,
                                    inst->listHdl, &itemHdl);
        if (retVal != FAPI_OK )break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Transparency ------------------------------------------------*/

        (void)OSD_GetTransparency(&transparency);

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = OSD_TRANSPARENCY_NUMS;
        selPars.params.index        = transparency;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_OSD_ID_ENTRY_TRANSPARENCY,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Color scheme ------------------------------------*/

        (void)OSD_GetColorScheme(&colScheme);

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = OSD_COLSCHEME_NUMS;
        selPars.params.index        = colScheme;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_OSD_ID_ENTRY_COLORSCHEME,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Hor align ------------------------------------*/

        selPars.pos                 = itemPos;
        selPars.params.numItems     = MENU_OSD_HOR_NUMS;

        switch ( menuAlign & FGS_ALIGN_MASK_HOR )
        {
            default:
            case FGS_ALIGN_HOR_LEFT:   selPars.params.index = MENU_OSD_HOR_LEFT;   break;
            case FGS_ALIGN_HOR_RIGHT:  selPars.params.index = MENU_OSD_HOR_RIGHT;  break;
            case FGS_ALIGN_HOR_CENTER: selPars.params.index = MENU_OSD_HOR_CENTER; break;
        }

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_OSD_ID_ENTRY_HORALIGN,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Ver align ------------------------------------*/

        selPars.pos                 = itemPos;
        selPars.params.numItems     = MENU_OSD_VER_NUMS;

        switch ( menuAlign & FGS_ALIGN_MASK_VER )
        {
            default:
            case FGS_ALIGN_VER_TOP:    selPars.params.index = MENU_OSD_VER_TOP;    break;
            case FGS_ALIGN_VER_BOTTOM: selPars.params.index = MENU_OSD_VER_BOTTOM; break;
            case FGS_ALIGN_VER_CENTER: selPars.params.index = MENU_OSD_VER_CENTER; break;
        }

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_OSD_ID_ENTRY_VERALIGN,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_OSD_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH), listPars.pos.y,
                                        helpWidth, numLines,
                                        MENU_OsdGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}

FSTATIC void MENU_OsdReposition (MENU_OSD_INST_S* inst)
{
    FGS_POS_S newpos;

    memset(&newpos, 0, sizeof(newpos));

    newpos.width     = inst->panelWidth;
    newpos.height    = inst->panelHeight;

    MENU_CommonSetPanelPos(&newpos);

    (void)FGS_SetMemberPosition((int32_t)MENUID_OSD, &newpos);
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
