/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_password.c
**
**              $Id: app_menu_password.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD password menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info 713: Loss of precision (assignment) (unsigned int to short).
** - Info 715: Symbol not referenced.
** - Info 732: Loss of sign (assignment) (int to unsigned short).
** - Info 737: Loss of sign in promotion from int to unsigned int.
** - Note 961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**   C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum
{
    MENU_PASSWORD_ID_TITLE = 0,
    MENU_PASSWORD_ID_LIST,
    MENU_PASSWORD_ID_HELPLIST
};

enum
{
    MENU_PW_ID_ENTRY_SETUPLOCK = 0,
    MENU_PW_ID_ENTRY_SETCHANPW,
    MENU_PW_ID_ENTRY_SETMASTERPW,
    MENU_PW_ID_ENTRY_VIEWAGE,
    MENU_PW_ID_ENTRY_USEPARENTALINFO,
    MENU_PW_ID_ENTRY_NUMS
};

typedef enum
{
    MENU_PW_CHG_START = 0,
    MENU_PW_CHG_OLD_ENTERED,
    MENU_PW_CHG_NEW_ENTERED

} MENU_PW_CHG_STATE;

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    listHdl;
    FAPI_SYS_HandleT    listHelpHdl;

    fbool_t             dataChg;

    PCTRL_PIN_TYPE_E    pwChgType;
    MENU_PW_CHG_STATE   pwChgState;
    char_t              pwChgPin[PCTRL_PIN_LEN];

} MENU_PW_INST_S;

FSTATIC MENU_PW_INST_S* menuPwDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_PasswordLeave      ( FAPI_SYS_HandleT h,
                                          fbool_t force );
FSTATIC fbool_t MENU_PasswordOnSelect   ( FAPI_SYS_HandleT h,
                                          int32_t itemId );
FSTATIC void    MENU_PasswordValChg     ( FAPI_SYS_HandleT h,
                                          int32_t itemId,
                                          FGS_VAL_S newVal );
FSTATIC void    MENU_PasswordOnFocusChg ( FAPI_SYS_HandleT h,
                                          uint8_t index,
                                          int32_t id );
FSTATIC fbool_t MENU_PasswordBuild      ( MENU_PW_INST_S* inst );
FSTATIC fbool_t MENU_PasswordStartPwChg ( MENU_PW_INST_S* inst,
                                          PCTRL_PIN_TYPE_E type );
FSTATIC void    MENU_PasswordOnPinEnter ( void* arg,
                                          const char_t* pin,
                                          uint8_t numChars );
FSTATIC void   MENU_PasswordGetItem     ( FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item );
FSTATIC void    MENU_PasswordOnFocusSet  (FAPI_SYS_HandleT h,
                                          fbool_t hasFocus);
FSTATIC void    MENU_PasswordGetDescrItem(FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item);
FSTATIC void    MENU_PasswordGetHelpDescrItem(FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);

int32_t MENU_PasswordInit (void)
{
    if ( menuPwDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuPwDat = UTILS_Malloc(sizeof(MENU_PW_INST_S));
    if ( menuPwDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuPwDat, 0, sizeof(MENU_PW_INST_S));

    return FAPI_OK;
}


fbool_t MENU_PasswordEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_PW_INST_S* inst = menuPwDat;
    uint32_t             lastFocusItem;

    if ( test == FTRUE ) return FTRUE;

    inst->dataChg = FFALSE;

    if ( !(MENU_PasswordBuild(inst)) ) return FFALSE;

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = 0;
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_PASSWORD_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_PASSWORD);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_PasswordLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

FSTATIC void MENU_PasswordOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_PW_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);
}

FSTATIC void MENU_PasswordOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_PW_INST_S*    inst = (MENU_PW_INST_S*)h;
    FGS_LNAVI_PARAMS_S naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}


FSTATIC fbool_t MENU_PasswordLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_PW_INST_S*    inst = (MENU_PW_INST_S*)h;

    if ( inst->dataChg )
    {
        if ( force )
        {
            /* restore data on forced exit */
            (void)PCTRL_Load();
        }
        else
        {
            MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

            (void)PCTRL_Save(FFALSE);

            (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        }

        (void)PCTRL_Restart();
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_PASSWORD);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC fbool_t MENU_PasswordOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    fbool_t                 keyUsed = FFALSE;
    fbool_t                 openSelectLst = FFALSE;
    uint8_t                 subLines = 0;
    MENU_SELECTLIST_OPEN_S  selectLst;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    FGS_LOC_S               memberLoc;
    MENU_PW_INST_S*         inst = (MENU_PW_INST_S*)h;

    switch ( itemId )
    {
        case MENU_PW_ID_ENTRY_SETUPLOCK:
            openSelectLst  = FTRUE;
            subLines       = 2;
            break;
        case MENU_PW_ID_ENTRY_SETCHANPW:
            keyUsed = MENU_PasswordStartPwChg (inst, PCTRL_PIN_SINGLE);
            break;

        case MENU_PW_ID_ENTRY_SETMASTERPW:
            keyUsed = MENU_PasswordStartPwChg (inst, PCTRL_PIN_MASTER);
            break;
        case MENU_PW_ID_ENTRY_USEPARENTALINFO:
            openSelectLst  = FTRUE;
            subLines       = 2;
            break;

        default:
            break;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.x         = memberLoc.x +
                                  SCALER_GetCurrVal(MAIN_WIDTH) -
                                  SCALER_GetCurrVal(SUB_LIST_WIDTH) -
                                  SCALER_GetCurrVal(OSD_POS_X);
        selectLst.pos.y         = memberLoc.y -
                                  SCALER_GetCurrVal(OSD_POS_Y);
        selectLst.pos.width     = SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_PasswordGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;
        selectLst.onExit        = NULL;
        selectLst.onExitArg     = NULL;

        return MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }
    return keyUsed;
}

FSTATIC void MENU_PasswordValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    MENU_PW_INST_S* inst   = (MENU_PW_INST_S*)h;
    fbool_t           valChg = FFALSE;
    PCTRL_PARAMS_S  pctrlParams;
    uint32_t             newProtArea;
    fbool_t           newBoolVal;

    if ( PCTRL_GetParams(&pctrlParams) != FAPI_OK ) return;

    switch ( itemId )
    {
        case MENU_PW_ID_ENTRY_SETUPLOCK:

            newProtArea = pctrlParams.protectionArea;

            if ( newVal.number == 0 ) newProtArea &= (~APP_PROTECTION_SETUP);
            else                      newProtArea |=   APP_PROTECTION_SETUP;

            if ( newProtArea != pctrlParams.protectionArea )
            {
                pctrlParams.protectionArea = newProtArea;
                valChg = FTRUE;
            }
            break;

        case MENU_PW_ID_ENTRY_VIEWAGE:
            if ( pctrlParams.viewerAge != newVal.number )
            {
                pctrlParams.viewerAge = (uint8_t)newVal.number;
                valChg = FTRUE;
            }
            break;

        case MENU_PW_ID_ENTRY_USEPARENTALINFO:
            newBoolVal = (newVal.number > 0) ? FTRUE : FFALSE;

            if ( pctrlParams.useStreamRating != newBoolVal )
            {
                pctrlParams.useStreamRating = newBoolVal;
                valChg = FTRUE;
            }
            break;

        default:
            break;
    }

    if ( valChg )
    {
        if ( PCTRL_SetParams(&pctrlParams) == FAPI_OK )
        {
            inst->dataChg = FTRUE;
        }
    }
}

FSTATIC void MENU_PasswordGetDescrItem (FAPI_SYS_HandleT h,
                                        int32_t id,
                                        uint32_t index,
                                        FGS_ITEMSTATE_E state,
                                        FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_PW_ID_ENTRY_SETUPLOCK:
            strIdx = STR_ENTRY_PASSWORD_SETUP;
            break;

        case MENU_PW_ID_ENTRY_SETCHANPW:
            strIdx = STR_ENTRY_PASSWORD_LOCKEDCH;
            break;

        case MENU_PW_ID_ENTRY_SETMASTERPW:
            strIdx = STR_ENTRY_PASSWORD_MASTERPW;
            break;

        case MENU_PW_ID_ENTRY_VIEWAGE:
            strIdx = STR_ENTRY_PASSWORD_AGE;
            break;

        case MENU_PW_ID_ENTRY_USEPARENTALINFO:
            strIdx = STR_ENTRY_PASSWORD_INFOSTREAM;
            break;

        default:
            return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_PasswordGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                            FGS_ITEMSTATE_E state,
                                            FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        case MENU_PW_ID_ENTRY_SETUPLOCK:
        case MENU_PW_ID_ENTRY_USEPARENTALINFO: strIdx = STR_ENTRY_LOKR; break;

        case MENU_PW_ID_ENTRY_VIEWAGE:         strIdx = STR_ENTRY_09LR; break;

        case MENU_PW_ID_ENTRY_SETCHANPW:
        case MENU_PW_ID_ENTRY_SETMASTERPW:     strIdx = STR_ENTRY_OK;   break;

        default:                               return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC void MENU_PasswordGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    PCTRL_PARAMS_S pctrlParams;
    uint32_t            strIdx;
    uint32_t            col;

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: col = appDat->txtAttr.colDisabled; break;
    }

    switch ( id )
    {
        case MENU_PW_ID_ENTRY_VIEWAGE:
            pctrlParams.viewerAge = 0;

            (void)PCTRL_GetParams(&pctrlParams);

            item->type                      = FGS_ITEM_TYPE_NUMBER;

            item->item.number.value         = pctrlParams.viewerAge;
            item->item.number.format        = FGS_FORMAT_NUM_NORM;
            item->item.number.decimalPlaces = 0;
            item->item.number.font          = MAIN_FONT;
            item->item.number.col           = col;
            break;

        case MENU_PW_ID_ENTRY_SETUPLOCK:
        case MENU_PW_ID_ENTRY_USEPARENTALINFO:
            strIdx = (index == 0) ? STR_GNR_OFF : STR_GNR_ON;

            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.chr        = OSD_GetStr(strIdx);
            item->item.str.col        = col;
            break;

        default:
            break;
    }
}

FSTATIC fbool_t MENU_PasswordBuild (MENU_PW_INST_S* inst)
{
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_LIST_OPEN_S       listPars;
    FGS_ITEM_SEL_OPEN_S   selPars;
    FGS_ITEM_TXT_OPEN_S   txtPars;
    FGS_ITEM_NUM_OPEN_S   numPars;
    FAPI_SYS_HandleT      itemHdl;
    FGS_POS_S             itemPos;
    FGS_POS_S             valuePos;
    FGS_POS_S             descrPos;
    int32_t               retVal;
    uint16_t              helpWidth;
    uint16_t              stackedFrameWidth;
    uint16_t              innerHeight;
    uint32_t              numLines     = MENU_PW_ID_ENTRY_NUMS;
    PCTRL_PARAMS_S        pctrlParams;

    memset(&selPars,  0, sizeof(selPars));
    memset(&txtPars,  0, sizeof(txtPars));
    memset(&numPars,  0, sizeof(numPars));

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_09LR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        retVal = PCTRL_GetParams(&pctrlParams);
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_PasswordOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_PASSWORD, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_PASSWORD_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_PASSWORD,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_PasswordGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_PasswordGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_PasswordOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_PasswordOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_PasswordValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_PASSWORD_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 80,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- menu lines -------------------------------------------------*/

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;

        selPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos           = descrPos;
        selPars.valuePos           = valuePos;

        numPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        numPars.descrPos           = descrPos;
        numPars.valuePos           = valuePos;

        /* --- Setup lock status --------------------------------------------*/

        selPars.pos                = itemPos;
        selPars.params.numItems    = 2;
        selPars.params.index       = ((pctrlParams.protectionArea &
                                      APP_PROTECTION_SETUP) != 0) ? 1 : 0;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_PW_ID_ENTRY_SETUPLOCK,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Change channel password ------------------------------------- */

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_PW_ID_ENTRY_SETCHANPW,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Change Master password ---------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_PW_ID_ENTRY_SETMASTERPW,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Viewing age --------------------------------------------------*/

        numPars.pos              = itemPos;
        numPars.type             = FGS_NITEM_TYPE_NUMBER;
        numPars.wraparound       = FTRUE;
        numPars.params.minValue  = 1;
        numPars.params.maxValue  = 99;
        numPars.params.maxDigits = 2;
        numPars.params.step      = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_PW_ID_ENTRY_VIEWAGE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Use parental info from stream? ------------------------------ */

        selPars.pos                = itemPos;
        selPars.params.numItems    = 2;
        selPars.params.index       = pctrlParams.useStreamRating ? 1 : 0;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_PW_ID_ENTRY_USEPARENTALINFO,
                                    inst->listHdl, &itemHdl);

        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_PASSWORD_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH), listPars.pos.y,
                                        helpWidth, numLines,
                                        MENU_PasswordGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;
        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}


FSTATIC fbool_t MENU_PasswordStartPwChg (MENU_PW_INST_S* inst,
                                       PCTRL_PIN_TYPE_E type)
{
    MENU_PINDIALOG_S pinParams;

    inst->pwChgState  = MENU_PW_CHG_START;
    inst->pwChgType   = type;

    memset (&pinParams, 0, sizeof(pinParams));

    pinParams.titleStr   = OSD_GetStr(STR_MSG_PASSWORD_CURR_PIN);
    pinParams.numChars   = PCTRL_PIN_LEN;
    pinParams.onPinEnter = MENU_PasswordOnPinEnter;
    pinParams.arg        = inst;

    if ( MENU_PinDialogOpen(&(appDat->pinDialog), &pinParams) == FAPI_OK )
    {
        return FTRUE;
    }
    else
    {
        return FFALSE;
    }
}

FSTATIC void MENU_PasswordOnPinEnter (void* arg, const char_t* pin, uint8_t numChars)
{
    MENU_PW_INST_S*    inst = (MENU_PW_INST_S*)arg;
    MENU_PINDIALOG_S   pinParams;
    uint8_t            cnt;

    memset(&pinParams, 0, sizeof(pinParams));

    pinParams.numChars   = PCTRL_PIN_LEN;
    pinParams.onPinEnter = MENU_PasswordOnPinEnter;
    pinParams.arg        = inst;

    switch ( inst->pwChgState )
    {
        case MENU_PW_CHG_START:

            /* current PIN entered: check for match */
            if ( PCTRL_CheckPin(inst->pwChgType, pin) == FAPI_OK )
            {
                inst->pwChgState = MENU_PW_CHG_OLD_ENTERED;

                (void)MENU_PinDialogClose(&(appDat->pinDialog));

                pinParams.titleStr = OSD_GetStr(STR_MSG_PASSWORD_NEW_PIN);

                (void)MENU_PinDialogOpen (&(appDat->pinDialog), &pinParams);
            }
            else
            {
                (void)MENU_PinDialogReset(&(appDat->pinDialog));
            }
            break;

        case MENU_PW_CHG_OLD_ENTERED:

            /* new PIN has been entered for the first time: store it
               temporarily. */
            memcpy(inst->pwChgPin, (char_t*)pin, PCTRL_PIN_LEN);

            inst->pwChgState = MENU_PW_CHG_NEW_ENTERED;

            (void)MENU_PinDialogClose(&(appDat->pinDialog));

            pinParams.titleStr = OSD_GetStr(STR_MSG_PASSWORD_CONFIRM_PIN);

            (void)MENU_PinDialogOpen (&(appDat->pinDialog), &pinParams);
            break;

        case MENU_PW_CHG_NEW_ENTERED:

            /* compare new PIN with the one previously stored. */
            for ( cnt = 0; cnt < PCTRL_PIN_LEN; cnt++ )
            {
                if ( inst->pwChgPin[cnt] != pin[cnt] ) break;
            }

            if ( cnt >= PCTRL_PIN_LEN ) /* PINs match */
            {
                if ( PCTRL_SetPin(inst->pwChgType, pin) == FAPI_OK )
                {
                    inst->dataChg = FTRUE;

                    MENU_CommonMsgShow(STR_MSG_PASSWORD_PIN_CHANGED, FFALSE);
                }

                (void)MENU_PinDialogClose(&(appDat->pinDialog));
            }
            else
            {
                (void)MENU_PinDialogReset(&(appDat->pinDialog));
            }
            break;
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
