/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_playlist.c
**
**              $Id: app_menu_playlist.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Playlist menu implementation.
**
** Detail description. [\ref app_smartgo_app_ref_recordings]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define MENU_PLAYLIST_BOT_HLP_LINES 3
#define MENU_PLAYLIST_LINES         8

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum
{
    MENU_PLIST_ID_TITLE = 0,
    MENU_PLIST_ID_LIST,
    MENU_PLIST_ID_REST,
    MENU_PLIST_ID_HELPBOTTOM
};

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    lstHdl;

    FGS_PANEL_MEMBER_S* pItem;
    FGS_POS_S           itemPos;

    PVR_LIST_ITEM_S     tmpItem;
    uint16_t            tmpIdx;
    int32_t             tmpLastStop;

} MENU_PLIST_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC MENU_PLIST_INST_S*   playListDat   = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_PlayListLeave                  (FAPI_SYS_HandleT h,
                                                     fbool_t force);
FSTATIC void    MENU_PlayListOnFocusSet             (FAPI_SYS_HandleT h,
                                                     fbool_t hasFocus);
FSTATIC void    MENU_PlayListOnSelect               (FAPI_SYS_HandleT h,
                                                     int32_t id,
                                                     uint16_t index);
FSTATIC fbool_t MENU_PlayListKeys                   (FAPI_SYS_HandleT h);
FSTATIC void    MENU_PlayListItemGet                (FAPI_SYS_HandleT hdl,
                                                     int32_t id,
                                                     uint32_t index,
                                                     FGS_ITEMSTATE_E state,
                                                     FGS_ITEM_S* item);
FSTATIC fbool_t MENU_PlayListBuild                  (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_PlayListPaint                  (FAPI_SYS_HandleT h);
FSTATIC void    MENU_PlayListConfirm                (void* arg);

FSTATIC void    MENU_PlayListPlayFromLastStop       (void* arg);
FSTATIC void    MENU_PlayListPlayFromBeginning      (void* arg);
FSTATIC void    MENU_PlayListPlayBkgrFromLastStop   (void* arg);
FSTATIC void    MENU_PlayListPlayBkgrFromBeginning  (void* arg);
FSTATIC void    MENU_PlayListEditStringExit         (MENU_EDITSTRING_DATA_S*
                                                     editData, int32_t id);

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
/*!
******************************************************************************
** \brief   Initialize the Playlist Menu
**
** This function allocat the memory for the menu handle \p playListDat and
** clears the handle structure.
**
** \return  one of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_ALREADY_INITIALIZED
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
******************************************************************************
*/
int32_t MENU_PlayListInit ( void )
{
    if (playListDat != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    playListDat = UTILS_Malloc (sizeof(MENU_PLIST_INST_S));
    if ( playListDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }
    memset (playListDat, 0, sizeof(MENU_PLIST_INST_S));

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   With this function you enter the Playlist menu.
**
** This function is called twice. First with \p test flag #FTRUE to check if
** the menu can be opend. If the first call was successful and return #FTRUE
** the second call will open the menu (\p test flag #FFALSE).
**
** \param   h       Handle of the Menu.
** \param   test    A flag to test if opening this menu is possible.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
fbool_t MENU_PlayListEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_PLIST_INST_S*  inst = playListDat;
    uint32_t            lastFocusItem;

    DBG_Assert(inst != 0);

    #ifdef APPL_MHEG_ENABLED
    MHEG_Enable(FFALSE,SYS_SUSPENDSRC_USER);
    #endif
    if ( inst == NULL )
    {
        return FFALSE; /* to satisfy MISRA */
    }

    if ( inst->pnlHdl != NULL )
    {
        FAPI_SYS_PRINT_DEBUG(2,"MENU_PlayListEntry: already open!\n");
        return FFALSE;
    }

    if ( test == FTRUE ) return FTRUE;

    inst->pnlHdl = NULL;

    (void) MENU_PlayListBuild((FAPI_SYS_HandleT)inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = 0;
    }

    (void) FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_PLIST_ID_LIST);
    (void) MENUSTACK_FocusClaim     ((int32_t)MENUID_PLIST);
    (void) FGS_PANEL_Show           (inst->pnlHdl, FFALSE);   /* turn visibility on */

    MENUSTACK_SetCleanup (MENU_PlayListLeave, (FAPI_SYS_HandleT)inst);
    return FTRUE;
}

/*!
*******************************************************************************
** \brief   This function updates the Playlist e.g. A recoding starts or
**          stopps.
**
*******************************************************************************
*/
void MENU_PlayListUpdate ( void )
{
    MENU_PLIST_INST_S* inst          = playListDat;
    int16_t            playlistItems = 0;
    FGS_LNAVI_PARAMS_S naviPars;

    DBG_Assert(inst != 0);
    if(inst == NULL)
    {
        return; /* to satisfy MISRA */
    }

    if ( (inst->pnlHdl != NULL) &&
         (FGS_LNAVI_GetParams(inst->lstHdl, &naviPars) == FAPI_OK) )
    {
        (void) PVR_LIST_GetNumItems(&playlistItems);

        naviPars.numItems = playlistItems;
        (void) FGS_LNAVI_SetParams(inst->lstHdl, &naviPars);
    }
}

/*!
******************************************************************************
** \brief   This function is a callback for the menustack. It close the
**          Playlist Menu.
**
** \param   h       Handle of this Menu.
** \param   force   (unused)
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_PlayListLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_PLIST_INST_S* inst = (MENU_PLIST_INST_S*)h;

    if ( inst->pnlHdl != NULL )
    {
        (void) FGS_PANEL_Close(inst->pnlHdl);
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_PLIST);

    inst->pnlHdl = NULL;
    return FTRUE;
}

/*!
******************************************************************************
** \brief   This function is a callback for the menustack. It sets or remove
**          the focus to the extended EPG and allocatets the keys.
**
** The following Keys are used by this menu:
** - #FGS_KEY_EXIT      -> To close all opend menus.
** - #FGS_KEY_BACK      -> Go back to the previous menu.
** - #FGS_KEY_MENU      -> Go back to the previous menu.
** - #FGS_KEY_RED       ->
** - #FGS_KEY_YELLOW    ->
** - #FGS_KEY_BLUE      -> only if APPL_EDITMARKER_ENABLED is set in
**                         Makefile.app to YES
**
** \param   h           Handle (unused).
** \param   hasFocus    #FTRUE set the focus to this menu #FFALSE remove the
**                      focus.
**
******************************************************************************
*/
FSTATIC void MENU_PlayListOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_PLIST_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_RED,    MENU_PlayListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_YELLOW, MENU_PlayListKeys, h, FTRUE, FFALSE);

        #ifdef APPL_EDITMARKER_ENABLED
        FGS_SetAppKeyHandler(FGS_KEY_BLUE,   MENU_PlayListKeys, h, FTRUE, FFALSE);
        #endif /* APPL_EDITMARKER_ENABLED */
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_YELLOW);

        #ifdef APPL_EDITMARKER_ENABLED
        FGS_ClearAppKeyHandler(FGS_KEY_BLUE);
        #endif /* APPL_EDITMARKER_ENABLED */
    }
}

/*!
******************************************************************************
** \brief   This function handles the #FGS_KEY_OK for list entries.
**
** By pressing the OK button 2 possible events could occour:
** 1. The playback starts directly, if the file was never started before or the
**    last playback stopps at the end of the recoding.
** 2. The playback was stopped while the file was played, a confirm dialog will
**    be opened and ask the user wether he wants to start the playback from the
**    last stopp point or from the beginning.
**
** \param   h       Handle of this Menu.
** \param   id
** \param   index   Currently selected focused list entry.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
void MENU_PlayListOnSelect (FAPI_SYS_HandleT h, int32_t id, uint16_t index)
{
    MENU_PLIST_INST_S*      inst = (MENU_PLIST_INST_S*)h;
    MENU_CONFDIALOG_S       confPars;
    PVR_LIST_ITEM_S         playlistItem;
    PVR_RecordInfoT         pParams;
    int8_t                  recIdx;

    #ifdef APPL_PIP_ENABLED

    PIP_STATUS_S pipStatus;

    if ( (PIP_GetStatus(&pipStatus) == FAPI_OK) &&
         (pipStatus.active) )
    {
        //FIXME: not necessary here.
        PIP_Hide();
    }

    #endif /* APPL_PIP_ENABLED */

    memset (&confPars,     0, sizeof(confPars));
    memset (&playlistItem, 0, sizeof(playlistItem));
    memset (&pParams,      0, sizeof(pParams));

    /* read list item data to retrieve last stop point */
    (void)PVR_LIST_GetItem(index, &(playlistItem), &recIdx);

    if (recIdx >= 0)
    {
        switch ( playlistItem.type )
        {
            default:
            case REC_TYPE_NORMAL:
                /* set confirm dialog CB functions here */
                confPars.onConfirm = MENU_PlayListPlayFromLastStop;
                confPars.onDiscard = MENU_PlayListPlayFromBeginning;
                break;

            case REC_TYPE_BACKGROUND:
                (void)PVRSYS_RecordGetInfo(appDat->pvrStatus.bkgrRecIdx,
                                           NULL, &pParams);

                /* set confirm dialog CB functions here */
                confPars.onConfirm = MENU_PlayListPlayBkgrFromLastStop;
                confPars.onDiscard = MENU_PlayListPlayBkgrFromBeginning;
                break;
        } /* end of switch ( playlistItem.type ) */
    } /* end of if (recIdx >= 0) */
    else
    {
        /* set confirm dialog CB functions here */
        confPars.onConfirm = MENU_PlayListPlayFromLastStop;
        confPars.onDiscard = MENU_PlayListPlayFromBeginning;
    }

    /*
        now read block marker data and retrieve the last stop point except
        for circular recordings that have already looped, because last stop point
        makes probably no sense anymore. then just go to the beginning of the circular
        recording.
    */
    if ( ( playlistItem.type == REC_TYPE_BACKGROUND) &&
         ( pParams.hasLooped == FTRUE ) )
    {
        MENU_PlayListPlayBkgrFromBeginning(inst);
    }
    else
    {
        /* only open the confirm dialog if there is already a stored stop point */
        if ( playlistItem.lastStop > 0 )
        {
            inst->tmpLastStop = playlistItem.lastStop;
            inst->tmpIdx = index;
            /*
                open a dialogue to ask wether to start from last stop position or
                to restart from the beginning.
            */

            confPars.numDescLines = 1;
            confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_PVR_PLAY_START_INFO);
            confPars.infoStr      = OSD_GetStr(STR_CONFDIALOG_PVR_PLAY_START_ACTION);
            confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_PVR_PLAY_LASTSTOP);
            confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_PVR_PLAY_BEGINNING);
            confPars.onDecline    = NULL;
            confPars.arg          = inst;

            (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
        }
        else
        {
            if ( PVRSYS_PlayStart (index, 0, NULL, NULL) != FAPI_OK )
            {
                (void)MAIN_ProhibitedShow(&(appDat->prohib));
            }

            /* leave playback menu */
            (void)MENUSTACK_Exit(NULL);
        }
    }
}

/*!
******************************************************************************
** \brief   This function is a callback for the confirm dialog box.
**
** It starts the selected playback from the last stop position.
**
** \param   arg     Menu Handle.
**
** \sa
**          - MENU_PlayListOnSelect()
**          - MENU_PlayListPlayFromBeginning()
**          - MENU_PlayListPlayBkgrFromLastStop()
**          - MENU_PlayListPlayBkgrFromBeginning()
**
******************************************************************************
*/
FSTATIC void MENU_PlayListPlayFromLastStop (void* arg)
{
    MENU_PLIST_INST_S* inst = (MENU_PLIST_INST_S*)arg;
    int32_t            retVal;

    retVal = PVRSYS_PlayStart (inst->tmpIdx, inst->tmpLastStop, NULL, NULL);
    if ( retVal != FAPI_OK )
    {
        (void)MAIN_ProhibitedShow(&(appDat->prohib));
    }

    /* leave playback menu */
    (void)MENUSTACK_Exit(NULL);
}

/*!
******************************************************************************
** \brief   This function is a callback for the confirm dialog box.
**
** It starts the selected playback from the beginning.
**
** \param   arg     Menu Handle.
**
** \sa
**          - MENU_PlayListOnSelect()
**          - MENU_PlayListPlayFromLastStop()
**          - MENU_PlayListPlayBkgrFromLastStop()
**          - MENU_PlayListPlayBkgrFromBeginning()
**
******************************************************************************
*/
FSTATIC void MENU_PlayListPlayFromBeginning (void* arg)
{
    MENU_PLIST_INST_S* inst = (MENU_PLIST_INST_S*)arg;
    int32_t            retVal;

    retVal = PVRSYS_PlayStart (inst->tmpIdx, 0, NULL, NULL);
    if ( retVal != FAPI_OK )
    {
        (void)MAIN_ProhibitedShow(&(appDat->prohib));
    }

    /* leave playback menu */
    (void)MENUSTACK_Exit(NULL);
}

/*!
******************************************************************************
** \brief   This function is a callback for the confirm dialog box.
**
** It starts the selected background recording from the last stop position.
**
** \param   arg     Menu Handle.
**
** \sa
**          - MENU_PlayListOnSelect()
**          - MENU_PlayListPlayFromLastStop()
**          - MENU_PlayListPlayFromBeginning()
**          - MENU_PlayListPlayBkgrFromBeginning()
**
******************************************************************************
*/
FSTATIC void MENU_PlayListPlayBkgrFromLastStop (void* arg)
{
    MENU_PLIST_INST_S* inst = (MENU_PLIST_INST_S*)arg;
    int32_t            retVal;

    retVal =  MAIN_PvrStartTimeShiftPlayback (appDat->pvrStatus.bkgrRecIdx,
                                              PVRSYS_TIMESHIFT_BEGIN,
                                              PLAY_STATUS_PAUSE,
                                              0);
    if ( retVal == FAPI_OK )
    {
        /* do a jump to last stop position and change to play mode*/
        (void)PVRSYS_PlayJump (inst->tmpLastStop, PVR_JUMP_DATA_ABS);
        (void)PVRSYS_PlaySetMode(PLAY_STATUS_NORMAL, 0);
    }
    else
    {
        (void)MAIN_ProhibitedShow(&(appDat->prohib));
    }
    /* leave playback menu */
    (void)MENUSTACK_Exit(NULL);
}

/*!
******************************************************************************
** \brief   This function is a callback for the confirm dialog box.
**
** It starts the selected background from the beginning.
**
** \param   arg     Menu Handle.
**
** \sa
**          - MENU_PlayListOnSelect()
**          - MENU_PlayListPlayFromLastStop()
**          - MENU_PlayListPlayFromBeginning()
**          - MENU_PlayListPlayBkgrFromLastStop()
**
******************************************************************************
*/
FSTATIC void MENU_PlayListPlayBkgrFromBeginning (void* arg)
{
    int32_t retVal;

    retVal = MAIN_PvrStartTimeShiftPlayback (appDat->pvrStatus.bkgrRecIdx,
                                             PVRSYS_TIMESHIFT_BEGIN,
                                             PLAY_STATUS_NORMAL,
                                             0);
    if( retVal != FAPI_OK )
    {
        (void)MAIN_ProhibitedShow(&(appDat->prohib));
    }

    /* leave playback menu */
    (void)MENUSTACK_Exit(NULL);
}

/*!
*******************************************************************************
**
** \brief   Handles the Keys adapted by this menu.
**
** This function is used as callback for the FGS_SetAppKeyHandler(). and
** handles the following Keys adapted by this Menu:
** - #FGS_KEY_RED
** - #FGS_KEY_YELLOW
** - #FGS_KEY_BLUE
**
** \param   h   Menu Handle.
**
** \return  One of the following status codes:
**          - #FTRUE    If the key was used by this menu.
**          - #FFALSE   If the key wasn't used.
**
** \sa
**          - FGS_SetAppKeyHandler()
**          - MENU_PlayListOnFocusSet()
**
*******************************************************************************-
*/
FSTATIC fbool_t MENU_PlayListKeys (FAPI_SYS_HandleT h)
{
    MENU_PLIST_INST_S*      inst = (MENU_PLIST_INST_S*)h;
    fbool_t                 keyUsed = FTRUE;
    MENU_CONFDIALOG_S       confPars;
    int8_t                  recIdx = -1;
    FGS_LNAVI_PARAMS_S      naviParams;
    MENU_EDITSTRING_OPEN_S  editParams;
    uint16_t                stackedFrameWidth;
    #ifdef APPL_EDITMARKER_ENABLED
    // TODO: to be ported!
    MENU_EDITREC_PARAMS_S   editRecParams;
    #endif

    stackedFrameWidth   = (MENUSTACK_GetLevel() - 1)
                          * (SCALER_GetCurrVal(STACKED_FRAME_DIST)
                            + SCALER_GetCurrVal(OUTER_FRAME_ALL));

    memset (&editParams,  0, sizeof(editParams));

    if ( inst == NULL ) return FFALSE;

    if ( FGS_LNAVI_GetParams(inst->lstHdl, &naviParams) != FAPI_OK )
    {
        return FFALSE;
    }

    switch ( FGS_GetCurrentKeyCode() )
    {
        /* ----- Delete focussed item --------------------------------------- */

        case FGS_KEY_RED:

            /* find out if file is currently being recorded */
            if ( PVR_LIST_GetItem(naviParams.index, NULL, &recIdx) != FAPI_OK )
            {
                keyUsed = FFALSE;
                break;
            }

            if ( recIdx >= 0 )
            {
                keyUsed = FFALSE;
                break;
            }


            /* open a confirm dialog to re-confirm the user's action */
            memset (&confPars, 0, sizeof(confPars));

            confPars.numDescLines = 1;
            confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_REM_REC);
            confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_REM_REC_OK);
            confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_REM_REC_ABORT);
            confPars.onConfirm    = MENU_PlayListConfirm;
            confPars.onDecline    = NULL;
            confPars.onDiscard    = NULL;
            confPars.arg          = inst;

            (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
            break;

        case FGS_KEY_YELLOW:
            if ( PVR_LIST_GetItem(naviParams.index,
                                  &(inst->tmpItem), &recIdx) != FAPI_OK )
            {
                keyUsed = FFALSE;
                break;
            }
            if ( &(appDat->pnlInst) != NULL)
            {
                (void)FGS_PANEL_GetPosition (inst->pnlHdl, &(editParams.pos));
                editParams.pos.y  += stackedFrameWidth
                                   + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                   + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                   + SCALER_GetCurrVal(MAIN_TOP_PADDING)
                                   + (2*SCALER_GetCurrVal(TITLE_TB_PADDING))
                                   + SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                                   + SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                   + SCALER_GetCurrVal(LIST_LINE_PADDING);
            }

            editParams.editTitle         = NULL;
            editParams.editTitleArg      = NULL;
            editParams.onExit            = MENU_PlayListEditStringExit;
            editParams.editData.string   = STR_Build(inst->tmpItem.nameCharSet,
                                                     inst->tmpItem.name);
            editParams.editData.maxChars = PVR_NAME_LENGTH - 1;
            editParams.id                = MENU_PLIST_ID_LIST;

            keyUsed = MENU_EditStringOpen (&(appDat->editStringBox),
                                           &editParams);
            break;

        #ifdef APPL_EDITMARKER_ENABLED

        case FGS_KEY_BLUE:

            editRecParams.lstPos = naviParams.index;

            keyUsed = MENUSTACK_Up (MENU_EditRecEntry,
                                   (FAPI_SYS_HandleT)(&editRecParams));
            break;

        #endif /* APPL_EDITMARKER_ENABLED */

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

/*!
******************************************************************************
** \brief   This function reads the list entries.
**
** \param   hdl     Menu Handle.
** \param   id      The item identifier.
** \param   index   TODO
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_PlayListItemGet (FAPI_SYS_HandleT hdl, int32_t id,
                                   uint32_t index, FGS_ITEMSTATE_E state,
                                   FGS_ITEM_S* item)
{
    MENU_PLIST_INST_S* inst = (MENU_PLIST_INST_S*)hdl;
    uint32_t           col;
    int8_t             recIdx;

    if ( PVR_LIST_GetItem(index, &(inst->tmpItem), &recIdx) != FAPI_OK )
    {
        return;
    }

    (void)TIME_AddUtcOffset (&(inst->tmpItem.startTime),&(inst->tmpItem.startDate));

    /* --------------------------------------------------------------------- */

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: col = OSD_COL_TEXT;             break;
        case FGS_FOCUS:    col = OSD_COL_TEXT_HIGHLIGHTED; break;
        case FGS_DISABLED: col = OSD_COL_TEXT_DISABLED;    break;
    }

    /* ----- Start date ---------------------------------------------------- */

    item[0].type                = FGS_ITEM_TYPE_DATE;
    item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[0].item.date.value     = inst->tmpItem.startDate;
    item[0].item.date.format    = appDat->dataFormatShort;
    item[0].item.date.font      = MAIN_FONT;
    item[0].item.date.col       = col;

    /* ----- Start time ---------------------------------------------------- */

    item[1].type                = FGS_ITEM_TYPE_TIME;
    item[1].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[1].item.time.value     = inst->tmpItem.startTime;
    item[1].item.time.format    = appDat->timeFormat;
    item[1].item.time.font      = MAIN_FONT;
    item[1].item.time.col       = col;

    /* ----- Title --------------------------------------------------------- */

    item[2].type                = FGS_ITEM_TYPE_STRING;
    item[2].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[2].item.str.chr        = STR_Build(inst->tmpItem.nameCharSet,
                                            inst->tmpItem.name);
    item[2].item.str.font       = MAIN_FONT;
    item[2].item.str.lineHeight = 0;
    item[2].item.str.col        = col;

    /* ----- Recording content --------------------------------------------- */

    item[3].type                = FGS_ITEM_TYPE_STRING;
    item[3].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[3].item.str.font       = MAIN_FONT;
    item[3].item.str.lineHeight = 0;
    item[3].item.str.col        = col;

    switch(inst->tmpItem.service.serviceType)
    {
        case SERVICE_TYPE_DIGITAL_TV:               /* FALLTHROUGH */
        case SERVICE_TYPE_DIGITAL_TV_ADVANCED:      /* FALLTHROUGH */
        case SERVICE_TYPE_DIGITAL_TV_HD:            /* FALLTHROUGH */
        case SERVICE_TYPE_DIGITAL_TV_HD_ADVANCED:   /* FALLTHROUGH */
        case SERVICE_TYPE_DATA_BROADCAST:
            item[3].item.str.chr = OSD_GetStr(STR_ENTRY_PLIST_TV);
            break;
        case SERVICE_TYPE_DIGITAL_RADIO: /* FALLTHROUGH */
        case SERVICE_TYPE_DIGITAL_RADIO_ADVANCED:
            item[3].item.str.chr = OSD_GetStr(STR_ENTRY_PLIST_RADIO);
            break;
        default:
            item[3].item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            break;
    }

    /* ----- Duration ------------------------------------------------------ */

    item[4].align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    if ( recIdx < 0 )
    {
        item[4].type                = FGS_ITEM_TYPE_TIME;
        item[4].item.time.value     = inst->tmpItem.duration;
        item[4].item.time.format    = FGS_FORMAT_TIME_PLAYTIME;
        item[4].item.time.font      = MAIN_FONT;
        item[4].item.time.col       = col;
    }
    else
    {
        item[4].type                = FGS_ITEM_TYPE_STRING;
        item[4].item.str.chr        = OSD_GetStr(STR_ENTRY_PLIST_REC);
        item[4].item.str.font       = MAIN_FONT;
        item[4].item.str.lineHeight = 0;
        item[4].item.str.col        = col;
    }

    /* ----- Drive --------------------------------------------------------- */

    #ifdef APPL_USB_ENABLED

    /* to do: should also handle USB-only or HDD-only configurations! */
    item[5].type                = FGS_ITEM_TYPE_STRING;
    item[5].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[5].item.str.font       = MAIN_FONT;
    item[5].item.str.lineHeight = 0;
    item[5].item.str.col        = col;

    switch ( inst->tmpItem.device )
    {
        #ifdef APPL_HDD_ENABLED

        case SYS_DEV_HDD:
            item[5].item.str.chr = OSD_GetStr(STR_VALUE_TIMEREDIT_RECTO_INTHD);
            break;

        #endif /* APPL_HDD_ENABLED */

        case SYS_DEV_USB:
            item[5].item.str.chr = OSD_GetStr(STR_VALUE_TIMEREDIT_RECTO_USB);
            break;

        default:
            item[5].item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            break;
    }

    #endif /* APPL_USB_ENABLED */
}

/*!
******************************************************************************
** \brief   This function sets up the menu structure and creates the FGS items.
**
** \param   h   Menu handle.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
** \sa
**          - MENU_PlayListEntry()
**
******************************************************************************
*/
FSTATIC fbool_t MENU_PlayListBuild (FAPI_SYS_HandleT h)
{
    MENU_PLIST_INST_S*  inst = (MENU_PLIST_INST_S*)h;
    FGS_PANEL_OPEN_S    pnlPars;
    FGS_LNAVI_OPEN_S    naviPars;
    FGS_PANEL_MEMBER_S  member;
    int32_t             retVal;
    int16_t             playlistItems = 0;
    FGS_BOX_OPEN_S      boxPars;
    FGS_ITEM_S          item;
    uint16_t            idCnt;
    FAPI_SYS_HandleT    itemHdl;
    uint16_t            stackedFrameWidth;
    uint16_t            mainPanelWidth;
    uint16_t            bottomHelpKeyWidth;
    uint16_t            bottomHelpItemWidth;
    uint16_t            innerHeight;
    uint32_t            numLines     = 1 + MENU_PLAYLIST_LINES;
    uint8_t             numHelpLines = MENU_PLAYLIST_BOT_HLP_LINES;

    memset (&naviPars, 0, sizeof(naviPars));
    memset (&member,   0, sizeof(member));
    memset (&item,     0, sizeof(item));
    memset (&boxPars,  0, sizeof(boxPars));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        innerHeight = (numLines       * SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                    + ((numLines - 1) * SCALER_GetCurrVal(LIST_LINE_PADDING));

        MENU_CommonSetParamsTitleWBotHelp (&pnlPars,
                                           SCALER_GetCurrVal(PLAYLIST_WIDTH),
                                           innerHeight,
                                           numHelpLines,
                                           &stackedFrameWidth);

        mainPanelWidth          = pnlPars.pos.width - stackedFrameWidth -
                                  (2*SCALER_GetCurrVal(OUTER_FRAME_ALL));
        bottomHelpKeyWidth      = OSD_GetStrWidth(OSD_GetStr(STR_KEY_MENU),-1,
                                  HELP_FONT) +
                                  SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        bottomHelpItemWidth     = mainPanelWidth/2 - bottomHelpKeyWidth -
                                  SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

        pnlPars.onFocusSet      = MENU_PlayListOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_PLIST, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_PLIST_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_PBLIST,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */
        (void)PVR_LIST_GetNumItems(&playlistItems);

        naviPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   stackedFrameWidth;
        naviPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                   (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                   SCALER_GetCurrVal(TITLE_LINE_HEIGHT)+
                                   SCALER_GetCurrVal(LIST_LINE_HEIGHT)  +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   stackedFrameWidth;
        naviPars.pos.width       = SCALER_GetCurrVal(PLAYLIST_WIDTH);
        naviPars.pos.height      = (numLines-1) * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        naviPars.bkgrType        = OSD_NAVIBKGR_NORMAL;
        naviPars.type            = FGS_LNAVI_TYPE_SCROLL;
        naviPars.rowHeight       = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.scrollbarWidth  = SCALER_GetCurrVal(SCROLLBAR_WIDTH);
        naviPars.numVisLines     = numLines  - 1;
        naviPars.numColumns      = 5;
        #ifdef APPL_USB_ENABLED
            naviPars.numColumns++;
        #endif /* APPL_USB_ENABLED */

        naviPars.params.numItems = playlistItems;
        naviPars.params.index    = 0;
        naviPars.params.posInVis = 0;

        naviPars.colStart[0]     = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[0]     = SCALER_GetCurrVal(PLAYLIST_COL0_WIDTH);  /* date */

        naviPars.colStart[1]     = naviPars.colStart[0] + naviPars.colWidth[0] +
                                   SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[1]     = SCALER_GetCurrVal(PLAYLIST_COL1_WIDTH);   /* time */

        naviPars.colStart[2]     = naviPars.colStart[1] + naviPars.colWidth[1] +
                                   SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[2]     = SCALER_GetCurrVal(PLAYLIST_COL2_WIDTH);   /* title */

        naviPars.colStart[3]     = naviPars.colStart[2] + naviPars.colWidth[2] +
                                   SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[3]     = SCALER_GetCurrVal(PLAYLIST_COL3_WIDTH);    /* type */

        naviPars.colStart[4]     = naviPars.colStart[3] + naviPars.colWidth[3] +
                                   SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[4]     = SCALER_GetCurrVal(PLAYLIST_COL4_WIDTH);    /* duration */

        #ifdef APPL_USB_ENABLED

        naviPars.colStart[5]     = naviPars.colStart[4] + naviPars.colWidth[4] +
                                   SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[5]     = SCALER_GetCurrVal(PLAYLIST_COL5_WIDTH);    /* drive */

        #endif /* APPL_USB_ENABLED */

        naviPars.topDownOrient   = FTRUE;
        naviPars.wraparound      = FTRUE;
        naviPars.getItem         = MENU_PlayListItemGet;
        naviPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        naviPars.onSelect        = MENU_PlayListOnSelect;
        naviPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        naviPars.pInst           = &(appDat->lnaviInst);

        retVal = FGS_LNAVI_Open (&naviPars, MENU_PLIST_ID_LIST,
                                 inst->pnlHdl,
                                &(inst->lstHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        inst->itemPos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   stackedFrameWidth;
        inst->itemPos.y         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                   (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                   SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                   stackedFrameWidth;
        inst->itemPos.width     = SCALER_GetCurrVal(PLAYLIST_WIDTH);
        inst->itemPos.height    = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(INNER_FRAME_WIDTH));
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemPos);
        member.id            = MENU_PLIST_ID_REST;
        member.isHidden      = FFALSE;
        member.show          = MENU_PlayListPaint;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pItem));
        if ( retVal != FAPI_OK ) break;

        /* -----Bottom help lines ------------------------------------------ */

        idCnt = MENU_PLIST_ID_HELPBOTTOM;

        /* -----OK - String ------------------------------------------ */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_OK);

        boxPars.pos.x               = stackedFrameWidth
                                      + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                      + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - (numHelpLines-1)*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Colon ------------------------------------------ */
        item.item.str.chr           = OSD_GetStr(STR_GNR_COLON);
        boxPars.pos.x               += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr, -1,
                                      HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Playback String ------------------------------------------ */
        item.item.str.chr           = OSD_GetStr(STR_HELP_PLIST_PLAYBACK);
        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Yellow button ------------------------------------------ */

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNYELLOWSMALL);

        boxPars.pos.x               = stackedFrameWidth +
                                      SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      mainPanelWidth/2;
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Colon ------------------------------------------ */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);

        boxPars.pos.x               += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr, -1,
                                      HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Rename Recordings String ----------------------------------- */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_PLIST_RENAME);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 2nd Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        /* -----Red button ------------------------------------------ */

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNREDSMALL);

        boxPars.pos.x               = stackedFrameWidth
                                      + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                      + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;
        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Colon ------------------------------------------ */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);

        boxPars.pos.x               += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1,
                                      HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;
        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Remove recording String ----------------------------------- */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_PLIST_REMOVE);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_EDITMARKER_ENABLED

        /* ----- Blue button ----------------------------------------------- */

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNBLUESMALL);

        boxPars.pos.x               = stackedFrameWidth +
                                      SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      mainPanelWidth/2;
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;
        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- Colon ----------------------------------------------------- */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);

        boxPars.pos.x               += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr, -1,
                                      HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;
        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Edit Rec. String ----------------------------------- */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_PLIST_EDIT);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        #endif /* APPL_EDITMARKER_ENABLED */

        /* -- 3rd Bottom line -- */
        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    /* ---------------------------------------------------------------------- */
    return FTRUE;
}

FSTATIC int32_t MENU_PlayListPaint (FAPI_SYS_HandleT h)
{
    MENU_PLIST_INST_S* inst = (MENU_PLIST_INST_S*)h;
    uint16_t           yCurr;
    uint16_t           posX;
    uint32_t           strWidth;
    uint16_t           posOffset;

    posX  = inst->pItem->abspos.x + SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
    yCurr = inst->pItem->abspos.y;

    /* ----- print help lines ---------------------------------------------- */

    OSD_DrawStringLtd (posX,
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(PLAYLIST_COL0_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_PLIST_DATE),
                       FTRUE);
    posX += SCALER_GetCurrVal(PLAYLIST_COL0_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX,
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(PLAYLIST_COL1_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_PLIST_TIME),
                       FTRUE);
    posX += SCALER_GetCurrVal(PLAYLIST_COL1_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX,
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(PLAYLIST_COL2_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_PLIST_TITLE),
                       FTRUE);
    posX += SCALER_GetCurrVal(PLAYLIST_COL2_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX,
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(PLAYLIST_COL3_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_PLIST_TYPE),
                       FTRUE);

    posX += SCALER_GetCurrVal(PLAYLIST_COL3_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    /* String "duration" is displayed right aligned */

    strWidth = OSD_GetStrWidth (OSD_GetStr(STR_ENTRY_PLIST_DURATION), -1,
                                MAIN_FONT);

    if ( strWidth < SCALER_GetCurrVal(PLAYLIST_COL4_WIDTH) )
         posOffset = SCALER_GetCurrVal(PLAYLIST_COL4_WIDTH) - strWidth;
    else posOffset = 0;

    OSD_DrawStringLtd (posX  + posOffset,
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(PLAYLIST_COL4_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_PLIST_DURATION),
                       FTRUE);

    posX += SCALER_GetCurrVal(PLAYLIST_COL4_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    #ifdef APPL_USB_ENABLED
    OSD_DrawStringLtd (posX,
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(PLAYLIST_COL5_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_PLIST_DRIVE),
                       FTRUE);
    #endif /* APPL_USB_ENABLED */

    posX  = inst->pItem->abspos.x;
    yCurr += SCALER_GetCurrVal(LIST_LINE_HEIGHT);
    OSD_DrawSeparatorHorizontal (posX, yCurr, inst->itemPos.width);

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   This function implements the confimation to delete a recoding by
**          pressing the #FGS_KEY_RED.
**
** \param   arg   Menu handle.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
** \sa
**          - MENU_PlayListKeys()
**
******************************************************************************
*/
FSTATIC void MENU_PlayListConfirm (void* arg)
{
    MENU_PLIST_INST_S* inst = (MENU_PLIST_INST_S*)arg;
    int8_t             recIdx       = -1;
    int16_t            numListItems = 0;
    PVRSYS_STATUS_S    pvrStatus;
    PVRLST_IDX         lstIdx;
    FGS_LNAVI_PARAMS_S naviParams;

    if ( FGS_LNAVI_GetParams(inst->lstHdl, &naviParams) != FAPI_OK )
    {
        return;
    }
    if ( PVR_LIST_GetItem(naviParams.index, NULL, &recIdx) != FAPI_OK )
    {
        return;
    }

    if ( recIdx >= 0 ) /* is item currently being recorded? */
    {
        return;
    }
    if ( PVR_LIST_GetIndex(naviParams.index, &lstIdx) != FAPI_OK )
    {
        return;
    }

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return;
    }
    /* stop playback if file is currently played */
    if ( (pvrStatus.play.type   != PLAY_TYPE_NONE) &&
         (pvrStatus.play.lstIdx == lstIdx) )
    {
        (void)MAIN_PvrResumeLiveTv(&pvrStatus);
    }

    if ( PVR_LIST_RemoveItem (naviParams.index) == FAPI_OK )
    {
        (void)PVR_LIST_GetNumItems (&numListItems);

        naviParams.numItems = numListItems;

        (void)FGS_LNAVI_SetParams(inst->lstHdl, &naviParams);
    }
}

/*!
******************************************************************************
** \brief   This function handle the string editor return.
**
** \param   editData    String edit parameter.
** \param   id          The list entry id.
**
** \sa
**          - MENU_PlayListKeys()
**
******************************************************************************
*/
FSTATIC void MENU_PlayListEditStringExit(MENU_EDITSTRING_DATA_S* editData,
                                         int32_t id)
{
    MENU_PLIST_INST_S*      inst = playListDat;
    FGS_LNAVI_PARAMS_S      naviPars;
    int8_t                  recIdx;

    memset (&naviPars, 0, sizeof(naviPars));

    (void)FGS_LNAVI_GetParams (inst->lstHdl, &naviPars);

    if ( editData->updated )
    {
        if ( PVR_LIST_GetItem(naviPars.index,
                              &(inst->tmpItem), &recIdx) != FAPI_OK )
        {
            return;
        }

        (void)STR_Copy (editData->string, inst->tmpItem.name,
                  &(inst->tmpItem.nameCharSet), editData->string.cs,
                  PVR_NAME_LENGTH);

        (void)PVR_LIST_ReplaceItem (naviPars.index, &(inst->tmpItem));

        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_PLIST_ID_LIST);
    }

}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
