/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_satinsttype.c
**
**              $Id: app_menu_satinsttype.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD Type of satellit installtion menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define NUMLINES_1TSGL          2
#define NUMLINES_1TMUX          2
#ifdef APPL_DISEQC11_SMATV_ENABLED
#define NUMLINES_SMATV          2
#endif
#ifdef APPL_DISEQC12_ENABLED
#define NUMLINES_1TMOT          2
#endif
#define NUMLINES_LOOPTHROUGH    2
#define NUMLINES_SAMESAT        4
#define NUMLINES_INDEPENDENT2   3
#define NUMLINES_INFO           2
/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_SATINSTTYPE_ID_TITLE = 0,
    MENU_SATINSTTYPE_ID_HEADER,
    MENU_SATINSTTYPE_ID_INFO,
    MENU_SATINSTTYPE_ID_INFOTXT,
    MENU_SATINSTTYPE_ID_LIST,

    #ifdef APPL_FESAT_ENABLED
    MENU_SATINSTTYPE_ID_PICTURE,
    #endif

    MENU_SATINSTTYPE_ID_HELPLIST
};

enum
{
    MENU_SATINSTTYPE_ID_ENTRY_1TSGL = 0,
    MENU_SATINSTTYPE_ID_ENTRY_1TMUX,
    #ifdef APPL_DISEQC11_SMATV_ENABLED
    MENU_SATINSTTYPE_ID_ENTRY_SMATV,
    #endif
    #ifdef APPL_DISEQC12_ENABLED
    MENU_SATINSTTYPE_ID_ENTRY_1TMOT,
    #endif
    MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP,
    MENU_SATINSTTYPE_ID_ENTRY_2LNB1DISH,
    MENU_SATINSTTYPE_ID_ENTRY_2DISH,
    MENU_SATINSTTYPE_ID_ENTRY_NOSAT
};


typedef struct
{
    FAPI_SYS_HandleT             pnlHdl;
    FAPI_SYS_HandleT             headerHdl;
    FAPI_SYS_HandleT             listHdl;
    FAPI_SYS_HandleT             listHelpHdl;

    fbool_t                      guidedMode;

} MENU_SATINSTTYPE_INST_S;

FSTATIC MENU_SATINSTTYPE_INST_S* menuSatInstTypeDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_SatInstTypeLeave      (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_SatInstTypeOnSelect   (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC fbool_t MENU_SatInstTypeBuild      (FAPI_SYS_HandleT h);
FSTATIC void    MENU_SatInstTypeOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id);
FSTATIC void    MENU_SatInstTypeGetItem    (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                            FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC void    MENU_SatInstTypeGetBoxItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                            FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC void    MENU_SatInstTypeOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_SatInstTypeGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                              FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC void MENU_SatInstTypeGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                               FGS_ITEMSTATE_E state,
                                               FGS_ITEM_S* item);

int32_t MENU_SatInstTypeInit (void)
{
    MENU_SATINSTTYPE_INST_S* inst;

    if ( menuSatInstTypeDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuSatInstTypeDat = UTILS_Malloc(sizeof(MENU_SATINSTTYPE_INST_S));
    if ( menuSatInstTypeDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuSatInstTypeDat, 0, sizeof(MENU_SATINSTTYPE_INST_S));

    inst = menuSatInstTypeDat;

    inst->guidedMode = FFALSE;

    return FAPI_OK;
}

fbool_t MENU_SatInstTypeEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_SATINSTTYPE_INST_S* inst = menuSatInstTypeDat;
    uint32_t                 lastFocusItem = 0;
    CFGSAT_FEPARS_S          feCfg;

    if ( test == FTRUE ) return FTRUE;

    if ( h != NULL )
    {
        inst->guidedMode = ((MENU_GUIDED_PARAMS_S*)h)->guided;
    }

    (void)MENU_SatInstTypeBuild((FAPI_SYS_HandleT)inst);

    /* dependant upon number of available tuners select OSD item list and first focus */

    if ( CFGSAT_GetFrontendSetup(&feCfg) != FAPI_OK ) return FFALSE;

    switch ( feCfg.installType )
    {
        case CFGSAT_INSTALLTYPE_INDEPENDENT:
            if(appDat->numSatFe == 1)
            {
                if( feCfg.controlMode[0] == CFGSAT_FEMODE_CLASSIC )
                    lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_1TSGL;
                else if( (feCfg.controlMode[0] == CFGSAT_FEMODE_DISEQC10 ) ||
                         (feCfg.controlMode[0] == CFGSAT_FEMODE_DISEQC11_DTH ) )
                    lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_1TMUX;
                #ifdef APPL_DISEQC11_SMATV_ENABLED
                else if(feCfg.controlMode[0] == CFGSAT_FEMODE_DISEQC11_SMATV )
                    lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_SMATV;
                #endif
                #ifdef APPL_DISEQC12_ENABLED
                else if (feCfg.controlMode[0] == CFGSAT_FEMODE_DISEQC12 )
                    lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_1TMOT;
                #endif
            }
            else
                lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_2DISH;
            break;

        case CFGSAT_INSTALLTYPE_LOOPTHROUGH:
            lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP;
            break;

        case CFGSAT_INSTALLTYPE_SAMESAT:
            lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_2LNB1DISH;
            break;

        default:
            if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
            {
                if ( appDat->numSatFe > 1 )
                {
                    lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP;
                }
                else if ( appDat->numSatFe == 1 )
                {
                    lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_1TSGL;
                }
                else
                {
                    lastFocusItem = MENU_SATINSTTYPE_ID_ENTRY_NOSAT;
                }
            }
            break;
    }
    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_SATINSTTYPE_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_SATINSTTYPE);       /* set global focus on menu panel */
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_SatInstTypeLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

FSTATIC void MENU_SatInstTypeOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_SATINSTTYPE_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);
}

FSTATIC void MENU_SatInstTypeOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_SATINSTTYPE_INST_S*   inst = (MENU_SATINSTTYPE_INST_S*)h;

    (void)FGS_LIST_SetMemberFocus(inst->listHelpHdl, id);

    #ifdef APPL_FESAT_ENABLED
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_SATINSTTYPE_ID_PICTURE);
    #endif
}

FSTATIC fbool_t MENU_SatInstTypeLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_SATINSTTYPE_INST_S* inst = (MENU_SATINSTTYPE_INST_S*)h;

    if ( CFGSAT_SavingIsNecessary() )
    {
        if ( force == FTRUE )
        {
            (void)CFGSAT_Load();
        }
        else
        {
            MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

            (void)CFGSAT_Save (FFALSE);

            (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        }
    }

    /* start last known service and clear AntProgParameters if antenna menu is left */
    if(FGS_GetCurrentKeyCode() == FGS_KEY_EXIT)
    {
        (void)MAIN_RestartPresentation();
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_SATINSTTYPE);

    inst->pnlHdl = NULL;
    return FTRUE;
}


FSTATIC fbool_t MENU_SatInstTypeOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_SATINSTTYPE_INST_S* inst = (MENU_SATINSTTYPE_INST_S*)h;
    CFGSAT_FEPARS_S          feCfg;
    uint8_t                  cnt;
    uint8_t                  feNums = SYS_GetHWConfig()->feNums;
    if ( CFGSAT_GetFrontendSetup(&feCfg) != FAPI_OK ) return FFALSE;

    switch ( itemId )
    {
        case MENU_SATINSTTYPE_ID_ENTRY_1TSGL:
            feCfg.installType = CFGSAT_INSTALLTYPE_INDEPENDENT;
            for(cnt=0;cnt < feNums; cnt++)
            {
                feCfg.controlMode[cnt] = CFGSAT_FEMODE_CLASSIC;
            }
            break;

        case MENU_SATINSTTYPE_ID_ENTRY_1TMUX:
            feCfg.installType = CFGSAT_INSTALLTYPE_INDEPENDENT;
            for(cnt=0;cnt < feNums; cnt++)
            {
                feCfg.controlMode[cnt] = CFGSAT_FEMODE_DISEQC10;
            }
            break;

        #ifdef APPL_DISEQC11_SMATV_ENABLED
        case MENU_SATINSTTYPE_ID_ENTRY_SMATV:
            feCfg.installType = CFGSAT_INSTALLTYPE_INDEPENDENT;
            for(cnt=0;cnt < feNums; cnt++)
            {
                feCfg.controlMode[cnt] = CFGSAT_FEMODE_DISEQC11_SMATV;
            }
            break;
        #endif

        #ifdef APPL_DISEQC12_ENABLED
        case MENU_SATINSTTYPE_ID_ENTRY_1TMOT:
            feCfg.installType = CFGSAT_INSTALLTYPE_INDEPENDENT;
            for(cnt=0;cnt < feNums; cnt++)
            {
                feCfg.controlMode[cnt] = CFGSAT_FEMODE_DISEQC12;
            }
            break;
        #endif

        case MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP:
            feCfg.installType = CFGSAT_INSTALLTYPE_LOOPTHROUGH;
            for(cnt=0;cnt < feNums; cnt++)
            {
                feCfg.controlMode[cnt] = CFGSAT_FEMODE_DISEQC10;
            }
            break;

        case MENU_SATINSTTYPE_ID_ENTRY_2LNB1DISH:
            feCfg.installType = CFGSAT_INSTALLTYPE_SAMESAT;
            for(cnt=0;cnt < feNums; cnt++)
            {
                feCfg.controlMode[cnt] = CFGSAT_FEMODE_DISEQC10;
            }
            break;

        case MENU_SATINSTTYPE_ID_ENTRY_2DISH:
            feCfg.installType = CFGSAT_INSTALLTYPE_INDEPENDENT;
            for(cnt=0;cnt < feNums; cnt++)
            {
                feCfg.controlMode[cnt] = CFGSAT_FEMODE_DISEQC10;
            }
            break;

        case MENU_SATINSTTYPE_ID_ENTRY_NOSAT:
            /* right now do nothing */
            break;

        default:
            MENU_CommonMsgShow(STR_MSG_NOTIMPLEMENTEDYET, FFALSE);
            return FFALSE;
    }

    (void)CFGSAT_SetFrontendSetup(&feCfg);

    /* store last focus */
    MENUSTACK_SetData(itemId);

    if ( inst->guidedMode ) return MENU_GuidedGotoNext();
    else                    return MENUSTACK_Back(NULL);
}


FSTATIC void MENU_SatInstTypeGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_SATINSTTYPE_ID_ENTRY_1TSGL: strIdx = STR_ENTRY_SATINSTTYPE_PRE1; break;
        case MENU_SATINSTTYPE_ID_ENTRY_1TMUX: strIdx = STR_ENTRY_SATINSTTYPE_PRE2; break;
        #ifdef APPL_DISEQC11_SMATV_ENABLED
        case MENU_SATINSTTYPE_ID_ENTRY_SMATV: strIdx = STR_ENTRY_SATINSTTYPE_PRE3; break;
        #endif
        #if defined (APPL_DISEQC12_ENABLED) && defined(APPL_DISEQC11_SMATV_ENABLED)
        case MENU_SATINSTTYPE_ID_ENTRY_1TMOT: strIdx = STR_ENTRY_SATINSTTYPE_PRE4; break;
        #elif defined (APPL_DISEQC12_ENABLED)
        case MENU_SATINSTTYPE_ID_ENTRY_1TMOT: strIdx = STR_ENTRY_SATINSTTYPE_PRE3; break;
        #endif
        case MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP: strIdx = STR_ENTRY_SATINSTTYPE_PRE1;
                                                  break;
        case MENU_SATINSTTYPE_ID_ENTRY_2LNB1DISH: strIdx = STR_ENTRY_SATINSTTYPE_PRE2;
                                                  break;
        case MENU_SATINSTTYPE_ID_ENTRY_2DISH    : strIdx = STR_ENTRY_SATINSTTYPE_PRE3;
                                                  break;
        case MENU_SATINSTTYPE_ID_ENTRY_NOSAT: strIdx = STR_ENTRY_SATINSTTYPE_PRE1; break;
        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_SatInstTypeGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                               FGS_ITEMSTATE_E state,
                                               FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_SATINSTTYPE_ID_ENTRY_1TSGL:
        case MENU_SATINSTTYPE_ID_ENTRY_1TMUX:
        #ifdef APPL_DISEQC11_SMATV_ENABLED
        case MENU_SATINSTTYPE_ID_ENTRY_SMATV:
        #endif
        #ifdef APPL_DISEQC12_ENABLED
        case MENU_SATINSTTYPE_ID_ENTRY_1TMOT:
        #endif
        case MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP:
        case MENU_SATINSTTYPE_ID_ENTRY_2LNB1DISH:
        case MENU_SATINSTTYPE_ID_ENTRY_2DISH:
        case MENU_SATINSTTYPE_ID_ENTRY_NOSAT:
            strIdx = STR_ENTRY_OK;
            break;

        default:  return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC void MENU_SatInstTypeGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t               strIdx;
    item->item.str.font       = MAIN_FONT;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->type                = FGS_ITEM_TYPE_STRING;
    item->item.str.lineHeight = SCALER_GetCurrVal(LIST_LINE_HEIGHT);

    switch ( id )
    {
        case MENU_SATINSTTYPE_ID_ENTRY_1TSGL: strIdx = STR_ENTRY_SATINSTTYPE_1TSGL; break;
        case MENU_SATINSTTYPE_ID_ENTRY_1TMUX: strIdx = STR_ENTRY_SATINSTTYPE_1TMUX; break;
        #ifdef APPL_DISEQC11_SMATV_ENABLED
        case MENU_SATINSTTYPE_ID_ENTRY_SMATV: strIdx = STR_ENTRY_SATINSTTYPE_SAMTV; break;
        #endif
        #ifdef APPL_DISEQC12_ENABLED
        case MENU_SATINSTTYPE_ID_ENTRY_1TMOT: strIdx = STR_ENTRY_SATINSTTYPE_1TMOT; break;
        #endif
        case MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP: strIdx = STR_ENTRY_SATINSTTYPE_TUNERLOOP;
                                                  break;
        case MENU_SATINSTTYPE_ID_ENTRY_2LNB1DISH: strIdx = STR_ENTRY_SATINSTTYPE_2LNB1DISH;
                                                  break;
        case MENU_SATINSTTYPE_ID_ENTRY_2DISH    : strIdx = STR_ENTRY_SATINSTTYPE_2DISH;
                                                  break;
        case MENU_SATINSTTYPE_ID_ENTRY_NOSAT: strIdx = STR_ENTRY_SATINSTTYPE_NOSAT; break;
        default:  return;
    }

    item->item.str.chr        = OSD_GetStr(strIdx);

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
    }

}

void MENU_SatInstTypeGetBoxItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                 FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    #ifdef USE_SATINST_GFX
        MENU_SATINSTTYPE_INST_S* inst = (MENU_SATINSTTYPE_INST_S*)h;
    #endif
    item->type                   = FGS_ITEM_TYPE_STRING;
    item->align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
    item->item.str.font          = MAIN_FONT;
    item->item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
    item->item.str.col           = OSD_COL_LIST_TEXT;

    switch(id)
    {
        case MENU_SATINSTTYPE_ID_INFO:
            item->item.str.chr = OSD_GetStr(STR_ENTRY_SATINSTTYPE_INFO);
            break;
        case MENU_SATINSTTYPE_ID_INFOTXT:
            item->item.str.chr = OSD_GetStr(STR_ENTRY_SATINSTTYPE_INFOTXT);
            break;
        case MENU_SATINSTTYPE_ID_HEADER:
            item->item.str.chr = OSD_GetStr(STR_ENTRY_SATINSTTYPE_HEADER);
            break;

        #ifdef APPL_FESAT_ENABLED

        case MENU_SATINSTTYPE_ID_PICTURE:
            item->type                   = FGS_ITEM_TYPE_ICON;
            item->item.icon.drawMode     = FGS_ICONMODE_OVRLAY;
            item->item.icon.bkgrCol      = OSD_COL_BLANK;
            item->item.icon.altBkgrCol   = OSD_COL_BLANK;
            item->item.icon.addr         = NULL;
            #ifdef USE_SATINST_GFX
                switch ( FGS_LIST_GetMemberFocusID ((FAPI_SYS_HandleT)inst->listHdl) )
                {

                    case MENU_SATINSTTYPE_ID_ENTRY_1TSGL:
                        item->item.icon.addr     = (uint32_t*)OSD_IconGet(PICTURE_ONE_SAT_SGLLNB);
                        break;

                    case MENU_SATINSTTYPE_ID_ENTRY_1TMUX:
                        item->item.icon.addr     = (uint32_t*)OSD_IconGet(PICTURE_ONE_SAT_MULTLNB);
                        break;

                    #ifdef APPL_DISEQC11_SMATV_ENABLED
                    case MENU_SATINSTTYPE_ID_ENTRY_SMATV:
                        item->item.icon.addr     = NULL;//(uint32_t*)OSD_IconGet(PICTURE_ONE_SAT_MOTLNB);
                        break;
                    #endif

                    #ifdef APPL_DISEQC12_ENABLED
                    case MENU_SATINSTTYPE_ID_ENTRY_1TMOT:
                        item->item.icon.addr     = (uint32_t*)OSD_IconGet(PICTURE_ONE_SAT_MOTLNB);
                        break;
                    #endif

                    case MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP:
                        item->item.icon.addr     = (uint32_t*)OSD_IconGet(PICTURE_TWO_SAT_SGLLNB);
                        break;

                    case MENU_SATINSTTYPE_ID_ENTRY_2LNB1DISH:
                        item->item.icon.addr     = (uint32_t*)OSD_IconGet(PICTURE_TWO_SAT_MULTLNB);
                        break;

                    case MENU_SATINSTTYPE_ID_ENTRY_2DISH:
                        item->item.icon.addr     = (uint32_t*)OSD_IconGet(PICTURE_TWO_SAT_MOTLNB);
                        break;


                    default:

                        break;

                } /* end of switch(listItemId) */
            #endif /* USE_SATINST_GFX */
            break;

        #endif /* APPL_FESAT_ENABLED */

        default:
            break;
    } /* end of switch(id) */
}

FSTATIC fbool_t MENU_SatInstTypeBuild(FAPI_SYS_HandleT h)
{
    MENU_SATINSTTYPE_INST_S*    inst = (MENU_SATINSTTYPE_INST_S*)h;
    FGS_PANEL_OPEN_S            pnlPars;
    FGS_LIST_OPEN_S             listPars;
    FAPI_SYS_HandleT            itemHdl;
    FGS_BOX_OPEN_S              boxPars;
    FGS_ITEM_TXT_OPEN_S         txtPars;
    FGS_ITEM_TXT_OPEN_S         helpTxtPars;
    FGS_POS_S                   descrPos;
    FGS_POS_S                   valuePos;
    FGS_POS_S                   helpPos;
    int32_t                     retVal;
    uint16_t                    helpWidth;
    uint16_t                    stackedFrameWidth;
    uint16_t                    innerHeight;
    uint8_t                     numLines, numListLines;
    uint8_t                     numListItems;
    fbool_t                     addInfoBox   = FFALSE;
    fbool_t                     addHeaderBox = FFALSE;

    memset(&boxPars,      0, sizeof(boxPars));
    memset(&txtPars,      0, sizeof(txtPars));
    memset(&helpTxtPars,  0, sizeof(helpTxtPars));
    memset(&helpPos,      0, sizeof(helpPos));
    memset(&descrPos,     0, sizeof(descrPos));
    memset(&valuePos,     0, sizeof(valuePos));

    /* set number of list lines according to detected front end types */
    numLines = numListLines = numListItems = 0;

    if ( appDat->numSatFe > 1 )
    {
        numLines = numListLines = NUMLINES_LOOPTHROUGH + NUMLINES_SAMESAT + NUMLINES_INDEPENDENT2;
        numListItems = 3;
        #ifdef APPL_DISEQC11_SMATV_ENABLED
        numListItems++;
        numLines     += NUMLINES_SMATV;
        numListLines += NUMLINES_SMATV;
        #endif
        addHeaderBox = FTRUE;
    }
    else if ( appDat->numSatFe == 1 )
    {
        numLines = numListLines = NUMLINES_1TSGL + NUMLINES_1TMUX;
        numListItems = 2;
        #ifdef APPL_DISEQC12_ENABLED
        numListItems++;
        numLines     += NUMLINES_1TMOT;
        numListItems += NUMLINES_1TMOT;
        #endif
        #ifdef APPL_DISEQC11_SMATV_ENABLED
        numListItems++;
        numLines     += NUMLINES_SMATV;
        numListLines += NUMLINES_SMATV;
        #endif

        addHeaderBox = FTRUE;
    }

    if ( (appDat->numTerrFe  > 0) ||
         (appDat->numCableFe > 0) )
    {
        numLines += NUMLINES_INFO;
        if(numListItems == 0)
            numListItems = 1;

        addInfoBox = FTRUE;
    }

    if(numLines == 0)
    {
        numLines = NUMLINES_INFO;
    }

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_OK),
                                   -1,SYMBOL_FONT);
    innerHeight       = (numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                        SCALER_GetCurrVal(LIST_LINE_PADDING))) +
                        SCALER_GetCurrVal(SATINSTYPE_PIC_HEIGHT) + /* Sat Picture */
                        SCALER_GetCurrVal(LIST_LINE_PADDING);

    if ( addHeaderBox )
    {
        innerHeight += (1*SCALER_GetCurrVal(LIST_LINE_HEIGHT));
    }

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_SatInstTypeOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SATINSTTYPE, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create box holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_SATINSTTYPE_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_SATINSTTYPE,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create box to inform about terrestrial or cable front end - */

        if ( addInfoBox )
        {
            boxPars.pos.x           = stackedFrameWidth
                                    + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                    + SCALER_GetCurrVal(MAIN_LEFT_PADDING);

            boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                    + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                    + SCALER_GetCurrVal(MAIN_TOP_PADDING)
                                    + (2 * SCALER_GetCurrVal(TITLE_TB_PADDING))
                                    + SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                                    + stackedFrameWidth;

            boxPars.pos.height      = NUMLINES_INFO
                                    * SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            boxPars.pos.width       = OSD_GetStrWidth (
                                        OSD_GetStr(STR_ENTRY_SATINSTTYPE_INFO),
                                        -1, SYMBOL_FONT)
                                    + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
            boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;
            boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;
            boxPars.getItem         = MENU_SatInstTypeGetBoxItem;

            retVal = FGS_BOX_Open (&boxPars, MENU_SATINSTTYPE_ID_INFO,
                                   inst->pnlHdl, &(inst->headerHdl));
            if ( retVal != FAPI_OK )break;

            boxPars.pos.x          += boxPars.pos.width;
            boxPars.pos.width       = SCALER_GetCurrVal(MAIN_WIDTH)
                                    - SCALER_GetCurrVal(MAIN_LEFT_PADDING)
                                    - SCALER_GetCurrVal(MAIN_RIGHT_PADDING)
                                    - boxPars.pos.width;

            retVal = FGS_BOX_Open (&boxPars, MENU_SATINSTTYPE_ID_INFOTXT,
                                   inst->pnlHdl, &(inst->headerHdl));
            if ( retVal != FAPI_OK )break;
        }

        /* ----- create box holding header ------------------------------ */

        if ( addHeaderBox )
        {
            boxPars.pos.x           = stackedFrameWidth
                                    + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                    + SCALER_GetCurrVal(MAIN_LEFT_PADDING);

            /* line position depends upon availability of Terr or Cable
               frontend */
            if ( addInfoBox )
            {
                boxPars.pos.y      += (NUMLINES_INFO
                                    * SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                                    + (NUMLINES_INFO
                                    * SCALER_GetCurrVal(LIST_LINE_PADDING));
            }
            else
            {
                boxPars.pos.y       = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                    + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                    + SCALER_GetCurrVal(MAIN_TOP_PADDING)
                                    + (2*SCALER_GetCurrVal(TITLE_TB_PADDING))
                                    + SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                                    + stackedFrameWidth;
            }

            boxPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            boxPars.pos.width       = SCALER_GetCurrVal(MAIN_WIDTH)
                                    - SCALER_GetCurrVal(MAIN_LEFT_PADDING)
                                    - SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
            boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            boxPars.bkgrType        = OSD_BOX_BKGR_NORMAL;
            boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;
            boxPars.getItem         = MENU_SatInstTypeGetBoxItem;

            retVal = FGS_BOX_Open (&boxPars, MENU_SATINSTTYPE_ID_HEADER,
                                   inst->pnlHdl, &(inst->headerHdl));
            if ( retVal != FAPI_OK )break;
        }
        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   stackedFrameWidth;
        if ( boxPars.pos.y == 0 )
        {
            listPars.pos.y       = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                   (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                   SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                   stackedFrameWidth;
        }
        else
        {
            listPars.pos.y       = boxPars.pos.y +
                                   boxPars.pos.height +
                                   SCALER_GetCurrVal(MAIN_TOP_PADDING)+
                                   SCALER_GetCurrVal(MAIN_BOT_PADDING);
        }

        listPars.pos.width       = SCALER_GetCurrVal(MAIN_WIDTH);
        listPars.pos.height      = (numListLines * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                                   (numListItems * SCALER_GetCurrVal(LIST_LINE_PADDING));
        listPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        listPars.topDownOriented = FTRUE;
        listPars.wraparound      = FTRUE;
        listPars.getItem         = MENU_SatInstTypeGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.getDescrItem    = MENU_SatInstTypeGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_SatInstTypeOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_SatInstTypeOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = NULL;
        listPars.onValChgArg     = NULL;
        listPars.pInst           = &(appDat->listInst);

        retVal = FGS_LIST_Open (&listPars, MENU_SATINSTTYPE_ID_LIST, inst->pnlHdl,
                                &(inst->listHdl));
        if ( retVal != FAPI_OK ) break;

        /* position of list item preambels and descriptors in list entry */
        txtPars.pos.x         = 0;
        txtPars.pos.y         = 0;
        txtPars.pos.width     = listPars.pos.width;
        txtPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        txtPars.bkgrType      = OSD_LSTITEM_BKGR_NORMAL;

        /* position of list item descriptors holding the Preambel in list entry */
        txtPars.descrPos.x         = SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        txtPars.descrPos.y         = 0;
        txtPars.descrPos.width     = SCALER_GetCurrVal(SATINSTYPE_PREAMBEL_WIDTH);
        txtPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        /* position of list item values holding the installation description in list entry */
        txtPars.valuePos.x         = SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                     SCALER_GetCurrVal(SATINSTYPE_PREAMBEL_WIDTH);
        txtPars.valuePos.y         = 0;
        txtPars.valuePos.width     = txtPars.pos.width -
                                     SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                     SCALER_GetCurrVal(MAIN_RIGHT_PADDING) -
                                     SCALER_GetCurrVal(SATINSTYPE_PREAMBEL_WIDTH) ;
        txtPars.valuePos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----- create help list ------------------------------------------ */

        /* NOTE: Common help list method not used here because of varying item
                 height */

        listPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(MAIN_WIDTH) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(HELP_LR_PADDING) +
                                   stackedFrameWidth;
        listPars.pos.width       = helpWidth - 2 * SCALER_GetCurrVal(HELP_LR_PADDING);
        listPars.getDescrItem    = MENU_SatInstTypeGetHelpDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = NULL;
        listPars.onFocusChgArg   = NULL;
        listPars.onSelect        = NULL;
        listPars.onSelectArg     = NULL;
        listPars.onValChg        = NULL;
        listPars.onValChgArg     = NULL;
        listPars.pInst           = &(appDat->helpListInst);

        retVal = FGS_LIST_Open (&listPars, MENU_SATINSTTYPE_ID_HELPLIST,
                                inst->pnlHdl, &(inst->listHelpHdl));
        if ( retVal != FAPI_OK ) break;

        helpTxtPars                = txtPars;
        helpTxtPars.pos.width      = listPars.pos.width;
        helpTxtPars.bkgrType       = OSD_LSTITEM_BKGR_HELP;
        helpTxtPars.descrPos.x     = 0;
        helpTxtPars.descrPos.width = helpTxtPars.pos.width;

        if(  (appDat->numSatFe == 0) &&
            ((appDat->numTerrFe > 0) || (appDat->numCableFe > 0)) )
        {
            /* ----- no Sat but Terr or Cable ----------------------------*/
            txtPars.pos.height      = NUMLINES_INFO * SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            txtPars.descrPos.height = txtPars.valuePos.height = txtPars.pos.height;

            helpTxtPars.pos.height      = txtPars.pos.height;
            helpTxtPars.descrPos.height = txtPars.pos.height;

            /* open list item */
            retVal = FGS_ITEM_TXT_Open (&txtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_NOSAT,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;
            /* open help item */
            retVal = FGS_ITEM_TXT_Open (&helpTxtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_NOSAT,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            txtPars.pos.y += (NUMLINES_INFO * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                             SCALER_GetCurrVal(LIST_LINE_PADDING);
        }
        if ( appDat->numSatFe == 1 )
        {
            /* ----- 1TSGL ------------------------------------------------*/
            txtPars.pos.height      = NUMLINES_1TSGL * SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            txtPars.descrPos.height = txtPars.valuePos.height = txtPars.pos.height;

            helpTxtPars.pos.height      = txtPars.pos.height;
            helpTxtPars.descrPos.height = txtPars.pos.height;

            /* open list item */
            retVal = FGS_ITEM_TXT_Open (&txtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_1TSGL,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;
            /* open help item */
            retVal = FGS_ITEM_TXT_Open (&helpTxtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_1TSGL,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            txtPars.pos.y += (NUMLINES_1TSGL * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                             SCALER_GetCurrVal(LIST_LINE_PADDING);
            /* ----- 1TMUX ---------------------------------------------*/

            txtPars.pos.height      = NUMLINES_1TMUX * SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            txtPars.descrPos.height = txtPars.valuePos.height = txtPars.pos.height;

            helpTxtPars.pos.y           = txtPars.pos.y;
            helpTxtPars.pos.height      = txtPars.pos.height;
            helpTxtPars.descrPos.height = txtPars.pos.height;

            retVal = FGS_ITEM_TXT_Open (&txtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_1TMUX,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpTxtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_1TMUX,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            txtPars.pos.y += (NUMLINES_1TMUX * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                             SCALER_GetCurrVal(LIST_LINE_PADDING);

            /* ----- SMATV ------------------------------------------------*/
            #ifdef APPL_DISEQC11_SMATV_ENABLED

            txtPars.pos.height      = NUMLINES_SMATV * SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            txtPars.descrPos.height = txtPars.valuePos.height = txtPars.pos.height;

            helpTxtPars.pos.y           = txtPars.pos.y;
            helpTxtPars.pos.height      = txtPars.pos.height;
            helpTxtPars.descrPos.height = txtPars.pos.height;

            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SATINSTTYPE_ID_ENTRY_SMATV,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpTxtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_SMATV,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            txtPars.pos.y += (NUMLINES_SMATV * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                             SCALER_GetCurrVal(LIST_LINE_PADDING);

            #endif /* APPL_DISEQC11_SMATV_ENABLED */
            /* ----- 1TMOT ------------------------------------------------*/
            #ifdef APPL_DISEQC12_ENABLED

            txtPars.pos.height      = NUMLINES_1TMOT * SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            txtPars.descrPos.height = txtPars.valuePos.height = txtPars.pos.height;

            helpTxtPars.pos.y           = txtPars.pos.y;
            helpTxtPars.pos.height      = txtPars.pos.height;
            helpTxtPars.descrPos.height = txtPars.pos.height;

            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SATINSTTYPE_ID_ENTRY_1TMOT,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpTxtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_1TMOT,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            #endif /* APPL_DISEQC12_ENABLED */
        }

        else if ( appDat->numSatFe == 2 )
        {
        /* ----- Loopthrough ------------------------------------------------*/
            txtPars.pos.height      = NUMLINES_LOOPTHROUGH * SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            txtPars.descrPos.height = txtPars.valuePos.height = txtPars.pos.height;

            helpTxtPars.pos.height      = txtPars.pos.height;
            helpTxtPars.descrPos.height = txtPars.pos.height;

            retVal = FGS_ITEM_TXT_Open (&txtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpTxtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_TUNERLOOP,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            txtPars.pos.y += (NUMLINES_LOOPTHROUGH * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                             SCALER_GetCurrVal(LIST_LINE_PADDING);
            /* ----- Same satellite ---------------------------------------------*/

            txtPars.pos.height      = NUMLINES_SAMESAT * SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            txtPars.descrPos.height = txtPars.valuePos.height = txtPars.pos.height;

            helpTxtPars.pos.y           = txtPars.pos.y;
            helpTxtPars.pos.height      = txtPars.pos.height;
            helpTxtPars.descrPos.height = txtPars.pos.height;

            retVal = FGS_ITEM_TXT_Open (&txtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_2LNB1DISH,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpTxtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_2LNB1DISH,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            txtPars.pos.y += (NUMLINES_SAMESAT * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                             SCALER_GetCurrVal(LIST_LINE_PADDING);

            /* ----- Independent ------------------------------------------------*/

            txtPars.pos.height      = NUMLINES_INDEPENDENT2 * SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            txtPars.descrPos.height = txtPars.valuePos.height = txtPars.pos.height;

            helpTxtPars.pos.y           = txtPars.pos.y;
            helpTxtPars.pos.height      = txtPars.pos.height;
            helpTxtPars.descrPos.height = txtPars.pos.height;

            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SATINSTTYPE_ID_ENTRY_2DISH,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpTxtPars,
                                        MENU_SATINSTTYPE_ID_ENTRY_2DISH,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

        }
        //txtPars.pos.y += (NUMLINES_INDEPENDENT2 * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
        //                 SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- create box holding SatPic ------------------------------ */

        #ifdef APPL_FESAT_ENABLED

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL);
        boxPars.pos.y               = listPars.pos.y + listPars.pos.height;
        boxPars.pos.height          = SCALER_GetCurrVal(SATINSTYPE_PIC_HEIGHT);
        boxPars.pos.width           = SCALER_GetCurrVal(SATINSTYPE_PIC_WIDTH);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;

        retVal = FGS_BOX_Open (&boxPars, MENU_SATINSTTYPE_ID_PICTURE, inst->pnlHdl,
                                &(inst->headerHdl));
        if ( retVal != FAPI_OK )break;

        #endif /* APPL_FESAT_ENABLED */


        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    return FTRUE;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
