/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_satlist.c
**
**              $Id: app_menu_satlist.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD Satellite List menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"
#include "app_satellites.h"


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define MENU_SATLIST_DEFAULT_LO_FREQ    9750000  // kHz
#define MENU_SATLIST_DEFAULT_HI_FREQ   10600000  // kHz

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_SATLIST_ID_HEADER = 0,
    MENU_SATLIST_ID_TITLE,
    MENU_SATLIST_ID_LIST,
    MENU_SATLIST_ID_HELPLIST,
    MENU_SATLIST_ID_HELPBOTTOM
};

enum
{
    MENU_SATLIST_ID_ENTRY_SELSAT = 0,
 // MENU_SATLIST_ID_ENTRY_EMPTY1, // empty line
    MENU_SATLIST_ID_ENTRY_NAME     = (MENU_SATLIST_ID_ENTRY_SELSAT+2),
    MENU_SATLIST_ID_ENTRY_ORBPOS,
    MENU_SATLIST_ID_ENTRY_WEFLAG,
 // MENU_SATLIST_ID_ENTRY_EMPTY2, // empty line.
    MENU_SATLIST_ID_ENTRY_GOTONEXT = (MENU_SATLIST_ID_ENTRY_WEFLAG+2),
    MENU_SATLIST_ID_ENTRY_NUMS
};

typedef struct
{
    FAPI_SYS_HandleT             pnlHdl;
    FAPI_SYS_HandleT             headerHdl;
    FAPI_SYS_HandleT             listHdl;
    FAPI_SYS_HandleT             listHelpHdl;

    fbool_t                      guidedMode;
    int16_t                      satLstPos;     //!< in sync with
                                                //!< appDat->currSatIdx!
    CFGSAT_SAT_S                 currSatDat;
    fbool_t                      satDatValid;
    CFGSAT_SAT_S                 tmpSatDat;

    fbool_t                      satChg;

}MENU_SATLIST_INST_S;

FSTATIC MENU_SATLIST_INST_S* menuSatListDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_SatListLeave              (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_SatListOnSelect           (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void    MENU_SatListValChg             (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal);
FSTATIC void    MENU_SatListGetItem            (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                                FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC fbool_t MENU_SatListKeys               (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_SatListBuild              (FAPI_SYS_HandleT h);
FSTATIC void    MENU_SatListOnFocusChg         (FAPI_SYS_HandleT h, uint8_t index, int32_t id);
FSTATIC void    MENU_SatListConfirmRemoval     (void* arg);
FSTATIC void    MENU_SatListEditStringExit     (MENU_EDITSTRING_DATA_S* editData, int32_t id);
FSTATIC void    MENU_SatListOnFocusSet         (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_SatListGetDescrItem       (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                                FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC void    MENU_SatListGetHelpDescrItem   (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                                FGS_ITEMSTATE_E state,
                                                FGS_ITEM_S* item);
/*!
*******************************************************************************
**
** \brief  Initial function for the OSD Satellite list Menu
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_SatListInit (void)
{
    MENU_SATLIST_INST_S* inst;

    if ( menuSatListDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuSatListDat = UTILS_Malloc(sizeof(MENU_SATLIST_INST_S));
    if ( menuSatListDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuSatListDat, 0, sizeof(MENU_SATLIST_INST_S));

    inst = menuSatListDat;

    inst->guidedMode = FFALSE;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function.
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_SatListEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_SATLIST_INST_S* inst = menuSatListDat;
    uint32_t             lastFocusItem;
    fbool_t              retVal;
    int16_t              lstPos;
    CFGSAT_IDX           newIdx;
    fbool_t              satPosValid = FFALSE;

    if ( test == FTRUE ) return FTRUE;

    if ( h != NULL )
    {
        inst->guidedMode = ((MENU_GUIDED_PARAMS_S*)h)->guided;
    }

    inst->satDatValid = FFALSE;
    inst->satChg      = FFALSE;
    inst->satLstPos   = 0;

    memset(&(inst->currSatDat),  0, sizeof(inst->currSatDat));

    /* ----- get initial satellite selection ------------------------------- */

    if ( appDat->currSatIdx >= 0 )
    {
        if ( CFGSAT_GetListPos (appDat->currSatIdx, &lstPos) == FAPI_OK )
        {
            inst->satLstPos = lstPos;
            satPosValid     = FTRUE;
        }
    }

    /* If satellite is invalid/not in list, choose first satellite in list */
    if ( !satPosValid )
    {
        inst->satLstPos = 0;

        if ( CFGSAT_GetIndex (inst->satLstPos, &newIdx) == FAPI_OK )
        {
            appDat->currSatIdx = newIdx;
            satPosValid        = FTRUE;
        }
    }

    if ( satPosValid )
    {
        if ( CFGSAT_GetSatData(inst->satLstPos, &(inst->currSatDat)) == FAPI_OK )
        {
            inst->satDatValid = FTRUE;
        }
    }

    /* --------------------------------------------------------------------- */

    retVal = MENU_SatListBuild((FAPI_SYS_HandleT)inst);
    if ( !retVal ) return FFALSE;

    if ( !(inst->satDatValid) )
    {
        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                              MENU_SATLIST_ID_ENTRY_NAME,
                                              FFALSE);
        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                              MENU_SATLIST_ID_ENTRY_ORBPOS,
                                              FFALSE);
        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                              MENU_SATLIST_ID_ENTRY_WEFLAG,
                                              FFALSE);
    }

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        if ( inst->guidedMode ) lastFocusItem = MENU_SATLIST_ID_ENTRY_GOTONEXT;
        else                    lastFocusItem = 0;
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_SATLIST_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_SATLIST);       /* set global focus on menu panel */
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_SatListLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_SatListOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_SATLIST_INST_S*)h)->pnlHdl != NULL);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_MENU,  MENU_SatListKeys,  h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK,  MENU_SatListKeys,  h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_EXIT,  MENUSTACK_Exit, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_GREEN, MENU_SatListKeys,  h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RED,   MENU_SatListKeys,  h, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler(FGS_KEY_GREEN);
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
    }
}

/*!
*******************************************************************************
**
** \brief  Activates the help icons for the current menu list item.
**
** \param h        Address of menu handle.
** \param index    unused.
** \param id       List id gets focus.
**
******************************************************************************/
FSTATIC void MENU_SatListOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_SATLIST_INST_S*   inst = (MENU_SATLIST_INST_S*)h;
    FGS_LNAVI_PARAMS_S     naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}

FSTATIC fbool_t MENU_SatListLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_SATLIST_INST_S* inst = (MENU_SATLIST_INST_S*)h;

    if ( inst->satChg )
    {
        if ( force )
        {
            /* restore data on forced exit */
            (void)CFGSAT_Load     ();
            (void)TPLIST_Load     (FE_SYSTEM_TYPE_SATELLITE);
            (void)SLIST_LoadLists ();

            #ifdef APPL_RECTIMER_ENABLED
            (void)RECTIMER_Load();
            #endif /* APPL_RECTIMER_ENABLED */
        }
        else
        {
            MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

            (void)CFGSAT_Save (FFALSE);
            (void)TPLIST_Save (FE_SYSTEM_TYPE_SATELLITE, FFALSE);
            (void)SLIST_Save  (FFALSE);

            #ifdef APPL_RECTIMER_ENABLED
            (void)RECTIMER_Save (FFALSE);
            #endif /* APPL_RECTIMER_ENABLED */

            /* NOTE: Service/carrier/timer lists might have been modified due to
                     removal of satellites. This is why they are written here
                     as well. */

            (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        }

        inst->satChg = FFALSE;
    }

    /* start last known service and clear AntProgParameters if antenna menu is left */
    if(FGS_GetCurrentKeyCode() == FGS_KEY_EXIT)
    {
        (void)MAIN_RestartPresentation();
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_SATLIST);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC fbool_t MENU_SatListOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_SATLIST_INST_S*    inst     = (MENU_SATLIST_INST_S*)h;
    uint8_t                 subLines = 0;
    MENU_SELECTLIST_OPEN_S  selectLst;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    fbool_t                 keyUsed = FTRUE;
    FGS_LOC_S               memberLoc;
    MENU_EDITSTRING_OPEN_S  editParams;
    uint32_t                stackedFrameWidth =0;

    switch ( itemId )
    {
        case MENU_SATLIST_ID_ENTRY_NAME:
            if ( &(appDat->pnlInst) != NULL)
            {

                stackedFrameWidth   = (MENUSTACK_GetLevel() - 1)
                                      * (SCALER_GetCurrVal(STACKED_FRAME_DIST)
                                        + SCALER_GetCurrVal(OUTER_FRAME_ALL));
                (void)FGS_PANEL_GetPosition (inst->pnlHdl, &(editParams.pos));
                editParams.pos.y  += stackedFrameWidth
                                   + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                   + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                   + SCALER_GetCurrVal(MAIN_TOP_PADDING)
                                   + (2*SCALER_GetCurrVal(TITLE_TB_PADDING))
                                   + SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                                   + SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                   + SCALER_GetCurrVal(LIST_LINE_PADDING);
            }
            editParams.editTitle = NULL;
            editParams.editTitleArg = NULL;
            editParams.onExit = MENU_SatListEditStringExit;
            editParams.editData.string   = STR_Build(
                                            inst->currSatDat.nameCharSet,
                                            inst->currSatDat.name);
            editParams.editData.maxChars = CFGSAT_SATNAME_SZ_MAX - 1;
            editParams.id               = MENU_SATLIST_ID_LIST;

            keyUsed = MENU_EditStringOpen (&(appDat->editStringBox),&editParams);
            return keyUsed;
        case MENU_SATLIST_ID_ENTRY_SELSAT:
            subLines = 5;
            break;

        case MENU_SATLIST_ID_ENTRY_WEFLAG:
            subLines = 2;
            break;

        case MENU_SATLIST_ID_ENTRY_GOTONEXT:
            keyUsed = MENU_GuidedGotoNext();
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    if ( subLines > 0 )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.width     = 2 * SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.x         = memberLoc.x + SCALER_GetCurrVal(MAIN_WIDTH) -
                                  selectLst.pos.width;
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_SatListGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;
        selectLst.onExit        = NULL;
        selectLst.onExitArg     = h;

        keyUsed = MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }

    return keyUsed;
}


FSTATIC void MENU_SatListValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    MENU_SATLIST_INST_S*  inst = (MENU_SATLIST_INST_S*)h;
    FGS_VAL_S             nextVal;
    FGS_ITEM_SEL_PARAMS_S selParams;

    memset(&nextVal, 0, sizeof(nextVal));

    switch ( itemId )
    {
        case MENU_SATLIST_ID_ENTRY_SELSAT:
            inst->satLstPos = newVal.number;

            (void)CFGSAT_GetIndex (inst->satLstPos, &(appDat->currSatIdx));

            if ( CFGSAT_GetSatData(inst->satLstPos, &(inst->currSatDat))
                  == FAPI_OK )
            {
                inst->satDatValid = FTRUE;
            }
            else
            {
                inst->satDatValid = FFALSE;
            }

            (void)FGS_LIST_SetRefreshMode       (inst->listHdl, FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_SATLIST_ID_ENTRY_NAME,   inst->satDatValid);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_SATLIST_ID_ENTRY_ORBPOS, inst->satDatValid);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_SATLIST_ID_ENTRY_WEFLAG, inst->satDatValid);
            (void)FGS_LIST_SetRefreshMode       (inst->listHdl, FTRUE);

            if ( FGS_LIST_GetMemberParams(inst->listHdl,
                 MENU_SATLIST_ID_ENTRY_WEFLAG, &selParams) == FAPI_OK )
            {
                if ( inst->satDatValid )
                {
                    selParams.index = inst->currSatDat.orbitalPos.westEastFlag
                                      - 1;
                }
                else
                {
                    selParams.index = 0;
                }

                (void)FGS_LIST_SetMemberParams(inst->listHdl,
                                         MENU_SATLIST_ID_ENTRY_WEFLAG, &selParams);
            }

            (void)FGS_LIST_RefreshMember(inst->listHdl,MENU_SATLIST_ID_ENTRY_NAME);
            (void)FGS_LIST_RefreshMember(inst->listHdl,MENU_SATLIST_ID_ENTRY_ORBPOS);
            break;

        case MENU_SATLIST_ID_ENTRY_NAME:
            if ( inst->satDatValid )
            {
                inst->satChg = FTRUE;

                (void)STR_Copy (newVal.string, inst->currSatDat.name,
                                &(inst->currSatDat.nameCharSet),
                                newVal.string.cs, CFGSAT_SATNAME_SZ_MAX);

                (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                                         SYS_OWNER_USER);

                (void)FGS_LIST_RefreshMember (inst->listHdl,
                                              MENU_SATLIST_ID_ENTRY_SELSAT);
            }
            break;

        case MENU_SATLIST_ID_ENTRY_ORBPOS:
            if ( inst->satDatValid )
            {
                if ( inst->currSatDat.orbitalPos.degrees != newVal.number )
                {
                    inst->currSatDat.orbitalPos.degrees = newVal.number;

                    inst->satChg = FTRUE;

                    (void)CFGSAT_ReplaceSat (inst->satLstPos,
                                             &(inst->currSatDat),
                                             SYS_OWNER_USER);
                }
            }
            break;

        case MENU_SATLIST_ID_ENTRY_WEFLAG:
            if ( inst->satDatValid )
            {
                if ( inst->currSatDat.orbitalPos.westEastFlag !=
                     (newVal.number + 1) )
                {
                    inst->satChg = FTRUE;

                    inst->currSatDat.orbitalPos.westEastFlag =
                        (FE_WE_FLAG_E)(newVal.number + 1);

                    (void)CFGSAT_ReplaceSat (inst->satLstPos,
                                             &(inst->currSatDat),
                                             SYS_OWNER_USER);
                }
            }
            break;

        default:
            break;
    }
}

FSTATIC void MENU_SatListGetDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                       uint32_t index, FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_SATLIST_ID_ENTRY_SELSAT  : strIdx = STR_ENTRY_SATLIST_SELSAT;  break;
        case MENU_SATLIST_ID_ENTRY_NAME    : strIdx = STR_ENTRY_SATLIST_NAME;    break;
        case MENU_SATLIST_ID_ENTRY_ORBPOS  : strIdx = STR_ENTRY_SATLIST_ORBPOS;  break;
        case MENU_SATLIST_ID_ENTRY_WEFLAG  : strIdx = STR_ENTRY_SATLIST_WEFLAG;  break;

        case MENU_SATLIST_ID_ENTRY_GOTONEXT:
            strIdx = MENU_GuidedGetNextStr();
            break;

        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_SatListGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        case MENU_SATLIST_ID_ENTRY_SELSAT:
            strIdx = STR_ENTRY_LOKR;
            break;

        case MENU_SATLIST_ID_ENTRY_ORBPOS:
            strIdx = STR_ENTRY_09LR;
            break;

        case MENU_SATLIST_ID_ENTRY_WEFLAG:
            strIdx = STR_ENTRY_LOKR;
            break;

        case MENU_SATLIST_ID_ENTRY_NAME:
        case MENU_SATLIST_ID_ENTRY_GOTONEXT:
            strIdx = STR_ENTRY_OK;
            break;

        default:  return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC void MENU_SatListGetItem (FAPI_SYS_HandleT h, int32_t id,
                                  uint32_t index, FGS_ITEMSTATE_E state,
                                  FGS_ITEM_S* item)
{
    MENU_SATLIST_INST_S* inst = (MENU_SATLIST_INST_S*)h;
    uint32_t             col;
    int32_t              strIdx = -1;

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: col = appDat->txtAttr.colDisabled; break;
    }

    switch ( id )
    {
        case MENU_SATLIST_ID_ENTRY_SELSAT:
            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.col        = col;

            if ( CFGSAT_GetSatData(index, &(inst->tmpSatDat)) == FAPI_OK )
            {
                item->item.str.chr    = STR_Build(inst->tmpSatDat.nameCharSet,
                                                  inst->tmpSatDat.name);
            }
            else
            {
                item->item.str.chr    = OSD_GetStr(STR_VALUE_SATLIST_NOSAT);
            }
            break;

        case MENU_SATLIST_ID_ENTRY_NAME:
            if ( inst->satDatValid )
            {
                item->type                = FGS_ITEM_TYPE_STRING;
                item->item.str.chr        = STR_Build(
                                            inst->currSatDat.nameCharSet,
                                            inst->currSatDat.name);
                item->item.str.font       = MAIN_FONT;
                item->item.str.lineHeight = 0;
                item->item.str.col        = col;
            }
            break;

        case MENU_SATLIST_ID_ENTRY_ORBPOS:
            if ( inst->satDatValid )
            {
                item->type                      = FGS_ITEM_TYPE_NUMBER;
                item->item.number.value         = inst->currSatDat.orbitalPos.
                                                  degrees;
                item->item.number.format        = FGS_FORMAT_NUM_NORM;
                item->item.number.decimalPlaces = 1;
                item->item.number.unitStr       = OSD_GetStr(STR_UNIT_DEG);
                item->item.number.font          = MAIN_FONT;
                item->item.number.col           = col;
            }
            break;

        case MENU_SATLIST_ID_ENTRY_WEFLAG:
            if ( inst->satDatValid )
            {
                switch ( index + 1 )
                {
                    case FE_WE_FLAG_WEST:
                        strIdx = STR_VALUE_SATLIST_WEFLAG_WEST;
                        break;

                    case FE_WE_FLAG_EAST:
                        strIdx = STR_VALUE_SATLIST_WEFLAG_EAST;
                        break;

                    default:
                        break;
                }

                if ( strIdx >= 0 )
                {
                    item->item.str.chr        = OSD_GetStr(strIdx);
                    item->type                = FGS_ITEM_TYPE_STRING;
                    item->item.str.font       = MAIN_FONT;
                    item->item.str.lineHeight = 0;
                    item->item.str.col        = col;
                }
            }
            break;

        default:
            break;
    }
}

FSTATIC fbool_t MENU_SatListKeys (FAPI_SYS_HandleT h)
{
    MENU_SATLIST_INST_S*  inst    = (MENU_SATLIST_INST_S*)h;
    fbool_t               keyUsed = FTRUE;
    FGS_ITEM_SEL_PARAMS_S selParams;
    CFGSAT_SAT_S          satDat;
    int16_t               numSat = 0;
    MENU_CONFDIALOG_S     confPars;
    int32_t               retVal;
    SYS_STRING_S          string;
    uint8_t               cnt;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
            /* FALLTHROUGHT */
        case FGS_KEY_BACK:
            if ( inst->guidedMode ) keyUsed = MENU_GuidedGotoPrevious();
            else                    keyUsed = MENUSTACK_Back (NULL);
            break;

        case FGS_KEY_GREEN: /* add a satellite */

            memset(&satDat, 0, sizeof(satDat));

            /* NOTE: ID of user-defined satellites must be out of range of
                     IDs of pre-defined satellites. */

            satDat.id                      = SATID_NUMS;
            satDat.active                  = FTRUE;
            satDat.orbitalPos.westEastFlag = FE_WE_FLAG_WEST;
            for(cnt=0;cnt<RSRC_FE_MAX;cnt++)
            {
                satDat.lnbCfg[cnt].freqLo = MENU_SATLIST_DEFAULT_LO_FREQ;
                satDat.lnbCfg[cnt].freqHi = MENU_SATLIST_DEFAULT_HI_FREQ;
            }

            string = OSD_GetStr(STR_VALUE_SATLIST_NEWSAT);

            (void)STR_BufAppend(NULL,0,string,SYS_CHARSET_ASCII);
            string = STR_IntLineStr();

            (void)STR_Copy (string, satDat.name, &(satDat.nameCharSet),
                      string.cs, CFGSAT_SATNAME_SZ_MAX);

            retVal = CFGSAT_AddSat (&satDat, SYS_OWNER_USER, &(inst->satLstPos));

            switch ( retVal )
            {
                case SMARTGO_SYS_ERR_CAPACITY:
                    MENU_CommonMsgShow(STR_MSG_NOMORESATPOSSIBLE, FFALSE);
                    break;

                case FAPI_OK:
                    inst->satChg = FTRUE;

                    (void)CFGSAT_GetNumSats(&numSat);

                    selParams.numItems = numSat;
                    selParams.index    = inst->satLstPos;

                    (void)FGS_LIST_SetMemberParams (inst->listHdl,
                                              MENU_SATLIST_ID_ENTRY_SELSAT,
                                              &selParams);

                    (void)FGS_LIST_RefreshMember (inst->listHdl,
                                                  MENU_SATLIST_ID_ENTRY_NAME);
                    (void)FGS_LIST_RefreshMember (inst->listHdl,
                                                  MENU_SATLIST_ID_ENTRY_ORBPOS);
                    (void)FGS_LIST_RefreshMember (inst->listHdl,
                                                  MENU_SATLIST_ID_ENTRY_WEFLAG);
                    break;

                default:
                    MAIN_MsgFunctionReturnedFailure(retVal);
                    keyUsed = FFALSE;
                    break;
            }
            break;

        case FGS_KEY_RED: /* remove a satellite */

            if ( inst->satDatValid )
            {
                /* open a confirm dialog to re-confirm the user's action */
                memset(&confPars, 0, sizeof(confPars));

                confPars.numDescLines = 1;
                confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_REM_SAT);
                confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_REM_SAT_OK);
                confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_REM_SAT_ABORT);
                confPars.onConfirm    = MENU_SatListConfirmRemoval;
                confPars.onDecline    = NULL;
                confPars.onDiscard    = NULL;
                confPars.arg          = inst;

                (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_SatListBuild(FAPI_SYS_HandleT h)
{
    MENU_SATLIST_INST_S*  inst = (MENU_SATLIST_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_LIST_OPEN_S       listPars;
    FAPI_SYS_HandleT      itemHdl;
    FGS_BOX_OPEN_S        boxPars;
    FGS_ITEM_SEL_OPEN_S   selPars;
    FGS_ITEM_TXT_OPEN_S   txtPars;
    FGS_ITEM_NUM_OPEN_S   numPars;
    FGS_POS_S             itemPos;
    FGS_ITEM_S            item;
    FGS_POS_S             valuePos;
    FGS_POS_S             descrPos;
    int32_t               retVal;
    uint16_t              helpWidth;
    uint16_t              mainPanelWidth;
    uint16_t              bottomHelpKeyWidth;
    uint16_t              bottomHelpItemWidth;
    uint16_t              stackedFrameWidth;
    uint16_t              innerHeight;
    uint32_t              numLines  = MENU_SATLIST_ID_ENTRY_NUMS;
    uint8_t               numHelpLines=2;
    int16_t               satNums  = 0;
    uint16_t              idCnt;
    uint16_t              offsetY;
    uint16_t              listHeight;

    memset(&boxPars,  0, sizeof(boxPars));
    memset(&selPars,  0, sizeof(selPars));
    memset(&txtPars,  0, sizeof(txtPars));
    memset(&numPars,  0, sizeof(numPars));
    memset(&item,     0, sizeof(item));

    if ( !(inst->guidedMode) ) numLines -= 2;

    helpWidth   = (2 * SCALER_GetCurrVal(HELP_LR_PADDING))
                + OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_09LR), -1, SYMBOL_FONT);
    innerHeight = ((numLines + 2) * (SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                + SCALER_GetCurrVal(LIST_LINE_PADDING))
                + SCALER_GetCurrVal(MAIN_TOP_PADDING)
                + SCALER_GetCurrVal(MAIN_BOT_PADDING);

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, numHelpLines,
                                        &stackedFrameWidth);

        mainPanelWidth          = pnlPars.pos.width -
                                  stackedFrameWidth -
                                  2*SCALER_GetCurrVal(OUTER_FRAME_ALL);
        bottomHelpKeyWidth      = SCALER_GetCurrVal(HELP_BOT_XOFF) -
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        bottomHelpItemWidth     = mainPanelWidth/2 -
                                  bottomHelpKeyWidth -
                                  SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

        pnlPars.onFocusSet      = MENU_SatListOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SATLIST, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_SATLIST_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_SATLIST,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create box holding header ------------------------------ */

        item.type                   = FGS_ITEM_TYPE_STRING;
        item.align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        item.item.str.font          = MAIN_FONT;
        item.item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        item.item.str.col           = OSD_COL_LIST_TEXT;
        item.item.str.chr           = OSD_GetStr(STR_ENTRY_SATLIST_HEADER);

        boxPars.pos.x               = stackedFrameWidth
                                    + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                    + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                      SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                      (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                      SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                      stackedFrameWidth;
        boxPars.pos.height          = (2*SCALER_GetCurrVal(LIST_LINE_HEIGHT));
        boxPars.pos.width           = SCALER_GetCurrVal(MAIN_WIDTH)
                                    - SCALER_GetCurrVal(MAIN_LEFT_PADDING)
                                    - SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_SATLIST_ID_HEADER, inst->pnlHdl,
                                &(inst->headerHdl));
        if ( retVal != FAPI_OK )break;

        /* ----- create list ----------------------------------------------- */

        listHeight = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                               + SCALER_GetCurrVal(LIST_LINE_PADDING));
        offsetY    = boxPars.pos.height
                   + SCALER_GetCurrVal(LIST_LINE_PADDING)
                   + SCALER_GetCurrVal(MAIN_TOP_PADDING);

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_SatListGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_SatListGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_SatListOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_SatListOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_SatListValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_SATLIST_ID_LIST,
                                            stackedFrameWidth, offsetY,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            listHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- menu lines -------------------------------------------------*/

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;

        /* --- Select Sat --------------------------------------------*/

        (void)CFGSAT_GetNumSats(&satNums);

        selPars.pos                 = itemPos;
        selPars.bkgrType            = txtPars.bkgrType;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = satNums;
        selPars.params.index        = inst->satLstPos;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_SATLIST_ID_ENTRY_SELSAT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += 2 *
                     (SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                      SCALER_GetCurrVal(LIST_LINE_PADDING));
        //itemPos.y += (SCALER_GetCurrVal(MAIN_TOP_PADDING)+SCALER_GetCurrVal(MAIN_BOT_PADDING));

        /* --- SATLIST Name ----------------------------------------*/

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = 1;
        selPars.params.index        = 0;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_SATLIST_ID_ENTRY_NAME,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- SATLIST orb pos------------------------------------*/

        numPars.pos               = itemPos;
        numPars.bkgrType          = OSD_LSTITEM_BKGR_NORMAL;
        numPars.descrPos          = descrPos;
        numPars.valuePos          = valuePos;
        numPars.type              = FGS_NITEM_TYPE_NUMBER;
        numPars.wraparound        = FTRUE;
        numPars.params.minValue   = 0;
        numPars.params.maxValue   = 1800;
        numPars.params.maxDigits  = 4;
        numPars.params.step       = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_SATLIST_ID_ENTRY_ORBPOS,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- SATLIST west east flag------------------------------------*/

        selPars.pos               = itemPos;
        selPars.bkgrType          = txtPars.bkgrType;
        selPars.descrPos          = descrPos;
        selPars.valuePos          = valuePos;
        selPars.params.numItems   = 2;

        if ( inst->satDatValid )
        {
            selPars.params.index = inst->currSatDat.orbitalPos.westEastFlag - 1;
        }
        else
        {
            selPars.params.index = 0;
        }

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_SATLIST_ID_ENTRY_WEFLAG,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- SATLIST Goto next -------------------------------*/

        if ( inst->guidedMode )
        {
            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                       + SCALER_GetCurrVal(LIST_LINE_PADDING);

            txtPars.pos = itemPos;
            txtPars.descrPos.width      = SCALER_GetCurrVal(MAIN_WIDTH) -
                                          SCALER_GetCurrVal(MAIN_LEFT_PADDING);
            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SATLIST_ID_ENTRY_GOTONEXT,
                                    inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                       + SCALER_GetCurrVal(LIST_LINE_PADDING);
        }

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_SATLIST_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH), listPars.pos.y,
                                        helpWidth, numLines,
                                        MENU_SatListGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        idCnt = MENU_SATLIST_ID_HELPBOTTOM;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNREDSMALL);

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - (numHelpLines-1)*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- Colon ----------------------------------------------------- */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);

        boxPars.pos.x               += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SATLIST_REMOVESAT);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNGREENSMALL);

        boxPars.pos.x               = stackedFrameWidth
                                    + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                    + mainPanelWidth / 2;
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Colon ------------------------------------------ */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);

        boxPars.pos.x               += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SATLIST_ADDSAT);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 2nd Bottom line -- */
        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    return FTRUE;
}

FSTATIC void MENU_SatListConfirmRemoval (void* arg)
{
    MENU_SATLIST_INST_S*  inst = (MENU_SATLIST_INST_S*)arg;
    FGS_ITEM_SEL_PARAMS_S selParams;
    int16_t               numSat = 0;

    inst->satChg = FTRUE;

    (void)CFGSAT_RemoveSat (inst->satLstPos);
    (void)CFGSAT_GetNumSats(&numSat);

    if ( numSat <  1 ) /* no more satellites available */
    {
        inst->satLstPos    = 0;
        appDat->currSatIdx = -1;
    }
    else if ( inst->satLstPos >= numSat )
    {
        inst->satLstPos = numSat - 1;

        (void)CFGSAT_GetIndex (inst->satLstPos, &(appDat->currSatIdx));
    }

    selParams.numItems = numSat;
    selParams.index    = inst->satLstPos;

    (void)FGS_LIST_SetMemberParams(inst->listHdl, MENU_SATLIST_ID_ENTRY_SELSAT,
                             &selParams);

    (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SATLIST_ID_ENTRY_NAME);
    (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SATLIST_ID_ENTRY_ORBPOS);
    (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SATLIST_ID_ENTRY_WEFLAG);
}

FSTATIC void MENU_SatListEditStringExit(MENU_EDITSTRING_DATA_S* editData, int32_t id)
{
    MENU_SATLIST_INST_S*  inst = menuSatListDat;

    if ( editData->updated )
    {
        if ( inst->satDatValid )
        {
            inst->satChg = FTRUE;

            (void)STR_Copy (editData->string, inst->currSatDat.name,
                      &(inst->currSatDat.nameCharSet),
                      editData->string.cs, CFGSAT_SATNAME_SZ_MAX);

            (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                               SYS_OWNER_USER);

            (void)FGS_LIST_RefreshMember (inst->listHdl, MENU_SATLIST_ID_ENTRY_SELSAT);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_SATLIST_ID_LIST);
        }
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
