/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_scan.c
**
**              $Id: app_menu_scan.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Osd scan menu functions for Sat, Terrestrial and Cable.
**
** Detail description. [\ref app_smartgo_app_ref_scan]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 408: Type mismatch with switch expression.
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"
#include "app_menu_scan.h"


/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_SCAN_ID_TITLE = 0,
    MENU_SCAN_ID_LIST,
    MENU_SCAN_ID_HELPLIST
};

enum
{
    #ifdef APPL_FESAT_ENABLED
    MENU_SCAN_ID_ENTRY_SATELLITE,
    #endif /* APPL_FESAT_ENABLED */

    #ifdef APPL_FETERR_ENABLED
    MENU_SCAN_ID_ENTRY_REGION_TERR,
    #endif /* APPL_FETERR_ENABLED */

    MENU_SCAN_ID_ENTRY_METHOD,
    MENU_SCAN_ID_ENTRY_MODE,
    MENU_SCAN_ID_ENTRY_TVRADIO,
    MENU_SCAN_ID_ENTRY_EDITTP,
    MENU_SCAN_ID_ENTRY_STARTSCAN,
    MENU_SCAN_ID_ENTRY_GOTONEXT
};

typedef struct
{
    FAPI_SYS_HandleT            pnlHdl;
    FAPI_SYS_HandleT            listHdl;
    FAPI_SYS_HandleT            listHelpHdl;

    #ifdef APPL_FESAT_ENABLED
    MENU_SCAN_CFG_S             cfgSat;
    CFGSAT_SAT_S                tmpSatDat;
    #endif /* APPL_FESAT_ENABLED */

    #ifdef APPL_FETERR_ENABLED
    MENU_SCAN_CFG_S             cfgTerr;
    #endif /* APPL_FETERR_ENABLED */

    #ifdef APPL_FECABLE_ENABLED
    MENU_SCAN_CFG_S             cfgCable;
    #endif /* APPL_FECABLE_ENABLED*/

    #ifdef APPL_FEDIRECT_ENABLED
    MENU_SCAN_CFG_S             cfgDirect;
    #endif /* APPL_FEDIRECT_ENABLED */

    MENU_SCAN_CFG_S*            pCfg;

    fbool_t                     guidedMode;
    int8_t                      currFe;

    #ifdef APPL_FESAT_ENABLED

    int16_t                     currSatLstPos;
    CFGSAT_SAT_S                satInf;      //!< parameters of current satellite
    fbool_t                     satInfValid;

    #endif /* APPL_FESAT_ENABLED */

    TPLIST_ITEM_S               carrierInf;  //!< parameters of current carrier

} MENU_SCAN_INST_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

MENU_SCAN_INST_S* menuScanDat = NULL;

#ifdef APPL_FESAT_ENABLED
/*---------------------------------------------------------------------------*/
/* local functions for Satellite                                             */
/*---------------------------------------------------------------------------*/
FSTATIC void  MENU_ScanSatGetDescrItem    (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);
FSTATIC fbool_t MENU_ScanSatBuild              (FAPI_SYS_HandleT h);
#endif

#ifdef APPL_FETERR_ENABLED
/*---------------------------------------------------------------------------*/
/* local functions for Terrestrial                                           */
/*---------------------------------------------------------------------------*/
FSTATIC void  MENU_ScanTerrGetDescrItem       (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                               FGS_ITEMSTATE_E state,
                                               FGS_ITEM_S* item);
FSTATIC fbool_t MENU_ScanTerrBuild              (FAPI_SYS_HandleT h);
#endif

#ifdef APPL_FECABLE_ENABLED
/*---------------------------------------------------------------------------*/
/* local functions for Cable                                                 */
/*---------------------------------------------------------------------------*/
FSTATIC void  MENU_ScanCableGetDescrItem       (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                               FGS_ITEMSTATE_E state,
                                               FGS_ITEM_S* item);
FSTATIC fbool_t MENU_ScanCableBuild              (FAPI_SYS_HandleT h);
#endif

#ifdef APPL_FEDIRECT_ENABLED
/*---------------------------------------------------------------------------*/
/* local functions for direct ts                                             */
/*---------------------------------------------------------------------------*/
FSTATIC void  MENU_ScanDirectGetDescrItem       (FAPI_SYS_HandleT h,
                                                 int32_t id,
                                                 uint32_t index,
                                                 FGS_ITEMSTATE_E state,
                                                 FGS_ITEM_S* item);
FSTATIC fbool_t MENU_ScanDirectBuild              (FAPI_SYS_HandleT h);

#endif
/*---------------------------------------------------------------------------*/
/* local functions for serving all front ends                                */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_ScanLeave              (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC void  MENU_ScanOnFocusSet         (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void  MENU_ScanOnFocusChg         (FAPI_SYS_HandleT h, uint8_t index, int32_t id);
FSTATIC fbool_t MENU_ScanKeys               (FAPI_SYS_HandleT h);
FSTATIC int32_t  MENU_ScanUpdateTuner        (MENU_SCAN_INST_S* inst);
FSTATIC fbool_t MENU_ScanOnSelect           (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void  MENU_ScanValChg             (FAPI_SYS_HandleT h,
                                           int32_t itemId, FGS_VAL_S newVal);
FSTATIC void  MENU_ScanMethodHasChanged   (MENU_SCAN_INST_S* inst);
FSTATIC void  MENU_ScanGetItem            (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);
FSTATIC void  MENU_ScanGetHelpDescrItem   (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                           FGS_ITEMSTATE_E state, FGS_ITEM_S* item);

FSTATIC void MENU_ScanEntryStart (MENU_SCAN_INST_S* inst, FAPI_SYS_HandleT h);
FSTATIC void MENU_ScanEntryEnd   (MENU_SCAN_INST_S* inst);
FSTATIC void MENU_ScanTpCallback (int16_t satLstPos, int16_t carrierLstPos);

/*---------------------------------------------------------------------------*/
/* common used Init function serving all front ends                          */
/*---------------------------------------------------------------------------*/

/*!
*******************************************************************************
**
** \brief  Initial function for the OSD Scan Menu
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_ScanInit (void)
{
    MENU_SCAN_INST_S* inst;
    MENU_SCAN_CFG_S   scanCfg;

    if ( menuScanDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuScanDat = UTILS_Malloc(sizeof(MENU_SCAN_INST_S));
    if ( menuScanDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuScanDat, 0, sizeof(MENU_SCAN_INST_S));

    inst = menuScanDat;

    /* set default scan menu parameters */

    inst->guidedMode = FFALSE;

    memset(&scanCfg, 0, sizeof(scanCfg));

    scanCfg.method       = MENU_SCANMETHOD_AUTO;
    scanCfg.opts.scanNIT = FTRUE;
    scanCfg.opts.choice  = AV_SCANCHOICE_ALL;
    scanCfg.opts.filter  = AV_SCANFILTER_ALL;
    scanCfg.tpLstPos     = 0;

    #ifdef APPL_FESAT_ENABLED

    inst->cfgSat                  = scanCfg;
    inst->cfgSat.system           = FE_SYSTEM_TYPE_SATELLITE;
    inst->cfgSat.network.type     = FE_SYSTEM_TYPE_SATELLITE;
    inst->cfgSat.network.satIdx   = -1; /* mark as invalid */

    #endif /* APPL_FESAT_ENABLED */

    #ifdef APPL_FETERR_ENABLED

    inst->cfgTerr                 = scanCfg;
    inst->cfgTerr.system          = FE_SYSTEM_TYPE_TERRESTRIAL;
    inst->cfgTerr.network.type    = FE_SYSTEM_TYPE_TERRESTRIAL;
    inst->cfgTerr.network.satIdx  = 0;

    #endif /* APPL_FETERR_ENABLED */

    #ifdef APPL_FECABLE_ENABLED

    scanCfg.method                = MENU_SCANMETHOD_SINGLE;

    inst->cfgCable                = scanCfg;
    inst->cfgCable.system         = FE_SYSTEM_TYPE_CABLE;
    inst->cfgCable.network.type   = FE_SYSTEM_TYPE_CABLE;
    inst->cfgCable.network.satIdx = 0;

    #endif /* APPL_FECABLE_ENABLED */

    #ifdef APPL_FEDIRECT_ENABLED

    scanCfg.method                = MENU_SCANMETHOD_SINGLE;

    inst->cfgDirect                = scanCfg;
    inst->cfgDirect.system         = FE_SYSTEM_TYPE_DIRECT;
    inst->cfgDirect.network.type   = FE_SYSTEM_TYPE_DIRECT;
    inst->cfgDirect.network.satIdx = 0;

    #endif /* APPL_FEDIRECT_ENABLED */

    return FAPI_OK;
}

FSTATIC void MENU_ScanEntryStart (MENU_SCAN_INST_S* inst, FAPI_SYS_HandleT h)
{
    inst->currFe        = -1;

    if ( h != NULL )
    {
        inst->guidedMode = ((MENU_GUIDED_PARAMS_S*)h)->guided;

        /* reset default configuration */
        inst->pCfg->tpLstPos       = 0;
    }
}

FSTATIC void MENU_ScanEntryEnd (MENU_SCAN_INST_S* inst)
{
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_SCAN_ID_LIST);

    /* set global focus on menu panel */
    (void)MENUSTACK_FocusClaim ((int32_t)MENUID_SCAN);

    /* turn visibility on */
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);

    (void)MENU_ScanUpdateTuner(inst); /* start tuning mode */

    MENUSTACK_SetCleanup (MENU_ScanLeave, (FAPI_SYS_HandleT)inst);
}

/*---------------------------------------------------------------------------*/
/* functions for serving satellite front ends                                */
/*---------------------------------------------------------------------------*/

#ifdef APPL_FESAT_ENABLED

/************** Satellite Scan Menu *******************************************/

/*!
*******************************************************************************
**
** \brief  Entry function.
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_ScanSatEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_SCAN_INST_S*   inst = menuScanDat;
    uint32_t            lastFocusItem;
    CFGSAT_IDX          newIdx;
    fbool_t             satPosValid = FFALSE;
    int16_t             lstPos;

    DBG_Assert(inst != 0);
    if ( inst == NULL ) return FFALSE; /* to satisfy MISRA */

    if ( test == FTRUE ) return FTRUE;

    inst->pCfg        = &(inst->cfgSat);
    inst->satInfValid = FFALSE;

    MENU_ScanEntryStart (inst, h);

    inst->currSatLstPos = 0;

    /* ----- get initial satellite selection ------------------------------- */

    if ( appDat->currSatIdx >= 0 )
    {
        if ( CFGSAT_GetListPos (appDat->currSatIdx, &lstPos) == FAPI_OK )
        {
            inst->currSatLstPos = lstPos;
            satPosValid         = FTRUE;
        }
    }

    /* If satellite is invalid/not in list, choose first satellite in list */
    if ( !satPosValid )
    {
        inst->currSatLstPos = 0;

        if ( CFGSAT_GetIndex (inst->currSatLstPos, &newIdx) == FAPI_OK )
        {
            appDat->currSatIdx = newIdx;
            satPosValid        = FTRUE;
        }
    }

    if ( satPosValid )
    {
        if ( CFGSAT_GetSatData (inst->currSatLstPos, &(inst->satInf))
                                == FAPI_OK )
        {
            inst->satInfValid = FTRUE;
        }
    }

    inst->pCfg->network.satIdx = appDat->currSatIdx;

    (void)MENU_ScanSatBuild((FAPI_SYS_HandleT)inst);

    /* set values according to current carrier selection */
    MENU_ScanMethodHasChanged(inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        if ( inst->guidedMode )
        {
            lastFocusItem = MENU_SCAN_ID_ENTRY_STARTSCAN;
        }
        else
        {
            lastFocusItem = MENU_SCAN_ID_ENTRY_SATELLITE;
        }
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);

    MENU_ScanEntryEnd (inst);

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  Set the descriptions for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_ScanSatGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                    FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_SCAN_ID_ENTRY_SATELLITE:
            strIdx = STR_ENTRY_SCAN_SATELLITE;
            break;

        case MENU_SCAN_ID_ENTRY_METHOD:
            strIdx = STR_ENTRY_SCAN_METHOD;
            break;

        case MENU_SCAN_ID_ENTRY_MODE:
            strIdx = STR_ENTRY_SCAN_MODE;
            break;

        case MENU_SCAN_ID_ENTRY_TVRADIO:
            strIdx = STR_ENTRY_SCAN_TVRADIO;
            break;

        case MENU_SCAN_ID_ENTRY_EDITTP:
            strIdx = STR_ENTRY_SCAN_EDITTP;
            break;

        case MENU_SCAN_ID_ENTRY_STARTSCAN:
            strIdx = STR_ENTRY_SCAN_STARTSCAN;
            break;

        case MENU_SCAN_ID_ENTRY_GOTONEXT:
            strIdx = MENU_GuidedGetNextStr();
            break;

        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

/*!
*******************************************************************************
**
** \brief  Builds the menu pannel and its members.
**
** \param h        Address of menu handle.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_ScanSatBuild(FAPI_SYS_HandleT h)
{
    MENU_SCAN_INST_S*     inst = (MENU_SCAN_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_LIST_OPEN_S       listPars;
    FGS_LIST_OPEN_S       helpListPars;
    FAPI_SYS_HandleT      itemHdl;
    FGS_BOX_OPEN_S        boxPars;
    FGS_ITEM_TXT_OPEN_S   txtPars;
    FGS_ITEM_TXT_OPEN_S   helpPars;
    MAIN_SIGBOX_OPEN_S    sigPars;
    FGS_ITEM_SEL_OPEN_S   selPars;
    FGS_POS_S             itemPos;
    FGS_ITEM_S            item;
    FGS_POS_S             valuePos;
    FGS_POS_S             descrPos;
    int32_t               retVal;
    uint16_t              helpWidth;
    uint16_t              stackedFrameWidth;
    uint16_t              innerHeight;
    uint32_t              numLines = 8;
    int16_t               numCarriers = 0;
    int16_t               numSats = 0;

    memset(&helpListPars, 0, sizeof(helpListPars));
    memset(&boxPars,      0, sizeof(boxPars));
    memset(&txtPars,      0, sizeof(txtPars));
    memset(&helpPars,     0, sizeof(helpPars));
    memset(&sigPars,      0, sizeof(sigPars));
    memset(&selPars,      0, sizeof(selPars));
    memset(&item,         0, sizeof(item));

    if ( inst->guidedMode ) numLines++;

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_LOKR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_ScanOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SCAN, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_SCAN_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_SCAN_SAT,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_ScanSatGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_ScanGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_ScanOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_ScanOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_ScanValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_SCAN_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- create help list ------------------------------------------ */

        /* NOTE: Common help list method not used here because of non-linear
                 item position. */

        helpListPars = listPars;

        helpListPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                       SCALER_GetCurrVal(MAIN_WIDTH) +
                                       SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                       SCALER_GetCurrVal(HELP_LR_PADDING) +
                                       stackedFrameWidth;
        helpListPars.pos.width       = helpWidth - 2 * SCALER_GetCurrVal(HELP_LR_PADDING);
        helpListPars.bkgrType        = 0;
        helpListPars.getDescrItem    = MENU_ScanGetHelpDescrItem;
        helpListPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        helpListPars.getItem         = NULL;
        helpListPars.getItemArg      = NULL;
        helpListPars.onFocusChg      = NULL;
        helpListPars.onFocusChgArg   = NULL;
        helpListPars.onSelect        = NULL;
        helpListPars.onSelectArg     = NULL;
        helpListPars.onValChg        = NULL;
        helpListPars.onValChgArg     = NULL;
        helpListPars.pInst           = &(appDat->helpListInst);

        retVal = FGS_LIST_Open (&helpListPars, MENU_SCAN_ID_HELPLIST,
                                inst->pnlHdl, &(inst->listHelpHdl));
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.x              = 0;
        helpPars.pos.y              = 0;
        helpPars.pos.width          = helpListPars.pos.width;
        helpPars.pos.height         = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        helpPars.pos.alignment      = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        helpPars.bkgrType           = OSD_LSTITEM_BKGR_HELP;
        helpPars.descrPos.x         = 0;
        helpPars.descrPos.y         = 0;
        helpPars.descrPos.width     = helpPars.pos.width;
        helpPars.descrPos.height    = helpPars.pos.height;
        helpPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----- menu lines -------------------------------------------------*/

        memset(&txtPars, 0, sizeof(FGS_ITEM_TXT_OPEN_S));

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;

        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;

        /* ----- Scanned Satellite ----------------------------------------- */

        (void)CFGSAT_GetNumSats(&numSats);

        selPars.pos                 = itemPos;
        selPars.params.numItems     = numSats;
        selPars.params.index        = inst->currSatLstPos;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_SCAN_ID_ENTRY_SATELLITE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_SATELLITE,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        itemPos.y      += 2 * SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Scan Method ----------------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.params.numItems     = MENU_SCANMETHOD_NUMS;
        selPars.params.index        = inst->pCfg->method;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                    MENU_SCAN_ID_ENTRY_METHOD,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_METHOD,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Scan Mode ------------------------------------------------- */

        selPars.pos              = itemPos;
        selPars.params.numItems  = AV_SCANFILTER_NUMS;
        selPars.params.index     = inst->pCfg->opts.filter;

        helpPars.pos.y           = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                    MENU_SCAN_ID_ENTRY_MODE,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_MODE,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- TV/Radio option ------------------------------------------- */

        selPars.pos              = itemPos;
        selPars.params.numItems  = AV_SCANCHOICE_NUMS;
        selPars.params.index     = inst->pCfg->opts.choice;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                    MENU_SCAN_ID_ENTRY_TVRADIO,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_TVRADIO,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Carrier --------------------------------------------------- */

        (void)TPLIST_GetNumCarriers (&(inst->pCfg->network), &numCarriers);

        if ( (int16_t)inst->pCfg->tpLstPos >= numCarriers )
        {
            inst->pCfg->tpLstPos = 0;
        }

        selPars.pos                 = itemPos;
        selPars.params.numItems     = numCarriers;
        selPars.params.index        = inst->pCfg->tpLstPos;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                     MENU_SCAN_ID_ENTRY_EDITTP,
                                     inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_EDITTP,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Signal bar ------------------------------------------------- */

        sigPars.pnlHdl     = inst->pnlHdl;
        sigPars.pos.x      = itemPos.x + listPars.pos.x + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        sigPars.pos.y      = itemPos.y + listPars.pos.y + SCALER_GetCurrVal(LIST_LINE_PADDING);
        sigPars.descrWidth = ((uint16_t)(itemPos.width- SCALER_GetCurrVal(MAIN_LEFT_PADDING))/(2));
        sigPars.barWidth   = ((uint16_t)(itemPos.width-
                                    SCALER_GetCurrVal(MAIN_LEFT_PADDING)-
                                    SCALER_GetCurrVal(MAIN_RIGHT_PADDING))/(2));
        sigPars.feIdx      = -1;

        retVal = MAIN_SignalBoxOpen(&(appDat->sigBox), &sigPars);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += MAIN_SignalBoxGetHeight();
        itemPos.y      += (2*SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ---start scan --------------------------------------------------- */

        helpPars.pos.y  = itemPos.y;
        txtPars.pos     = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SCAN_ID_ENTRY_STARTSCAN,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_STARTSCAN,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- GOTO next --------------------------------------------------- */

        if ( inst->guidedMode )
        {
            helpPars.pos.y  = itemPos.y;
            txtPars.pos     = itemPos;
            /* use full width for description since there is no value */
            txtPars.descrPos.width += txtPars.valuePos.width;
            txtPars.valuePos.width = 0;

            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SCAN_ID_ENTRY_GOTONEXT,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpPars,
                                        MENU_SCAN_ID_ENTRY_GOTONEXT,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);

        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}
#endif /* APPL_FESAT_ENABLED */




/************** Terrestrial Scan Menu ****************************************/

#ifdef APPL_FETERR_ENABLED
/*!
*******************************************************************************
**
** \brief  Entry function for terrestrial scan menu
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_ScanTerrEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_SCAN_INST_S* inst = menuScanDat;
    uint32_t               lastFocusItem;

    DBG_Assert(inst != 0);
    if(inst == NULL)return FFALSE; /* to satisfy MISRA */

    if ( test == FTRUE ) return FTRUE;

    inst->pCfg = &(inst->cfgTerr);

    MENU_ScanEntryStart (inst, h);

    (void)MENU_ScanTerrBuild((FAPI_SYS_HandleT)inst);

    /* set values according to current channel selection */
    MENU_ScanMethodHasChanged(inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        if ( inst->guidedMode )
        {
            lastFocusItem = MENU_SCAN_ID_ENTRY_STARTSCAN;
        }
        else
        {
            lastFocusItem = MENU_SCAN_ID_ENTRY_REGION_TERR;
        }
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);

    MENU_ScanEntryEnd (inst);

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  Set the descriptions for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_ScanTerrGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                        FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_SCAN_ID_ENTRY_REGION_TERR:
            strIdx = STR_ENTRY_SCAN_REGION;
            break;

        case MENU_SCAN_ID_ENTRY_METHOD:
            strIdx = STR_ENTRY_SCAN_METHOD;
            break;

        case MENU_SCAN_ID_ENTRY_MODE:
            strIdx = STR_ENTRY_SCAN_MODE;
            break;

        case MENU_SCAN_ID_ENTRY_EDITTP:
            strIdx = STR_ENTRY_SCAN_EDITCH_TERR;
            break;

        case MENU_SCAN_ID_ENTRY_STARTSCAN:
            strIdx = STR_ENTRY_SCAN_STARTSCAN;
            break;

        case MENU_SCAN_ID_ENTRY_GOTONEXT:
            strIdx = MENU_GuidedGetNextStr();
            break;

        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}


/*!
*******************************************************************************
**
** \brief  Builds the terrestrial menu panel and its members.
**
** \param h        Address of menu handle.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_ScanTerrBuild(FAPI_SYS_HandleT h)
{
    MENU_SCAN_INST_S* inst = (MENU_SCAN_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_LIST_OPEN_S       listPars;
    FGS_LIST_OPEN_S       helpListPars;
    FAPI_SYS_HandleT      itemHdl;
    FGS_BOX_OPEN_S        boxPars;
    FGS_ITEM_TXT_OPEN_S   txtPars;
    FGS_ITEM_TXT_OPEN_S   helpPars;
    MAIN_SIGBOX_OPEN_S    sigPars;
    FGS_ITEM_SEL_OPEN_S   selPars;
    FGS_POS_S             itemPos;
    FGS_ITEM_S            item;
    FGS_POS_S             valuePos;
    FGS_POS_S             descrPos;
    int32_t               retVal;
    uint16_t              helpWidth;
    uint16_t              stackedFrameWidth;
    uint16_t              innerHeight;
    uint32_t              numLines = 7;
    int16_t               numCarriers = 0;

    memset(&helpListPars, 0, sizeof(helpListPars));
    memset(&boxPars,      0, sizeof(boxPars));
    memset(&txtPars,      0, sizeof(txtPars));
    memset(&helpPars,     0, sizeof(helpPars));
    memset(&sigPars,      0, sizeof(sigPars));
    memset(&selPars,      0, sizeof(selPars));
    memset(&item,         0, sizeof(item));

    if ( inst->guidedMode ) numLines++;

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_LOKR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_ScanOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SCAN, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_SCAN_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_SCAN_TERR,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_ScanTerrGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_ScanGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_ScanOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_ScanOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_ScanValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_SCAN_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- create help list ------------------------------------------ */

        /* NOTE: Common help list method not used here because of non-linear
                 item position. */

        helpListPars = listPars;

        helpListPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                       SCALER_GetCurrVal(MAIN_WIDTH) +
                                       SCALER_GetCurrVal(INNER_FRAME_WIDTH) + SCALER_GetCurrVal(HELP_LR_PADDING) +
                                       stackedFrameWidth;
        helpListPars.pos.width       = helpWidth - 2 * SCALER_GetCurrVal(HELP_LR_PADDING);
        helpListPars.bkgrType        = 0;
        helpListPars.getDescrItem    = MENU_ScanGetHelpDescrItem;
        helpListPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        helpListPars.getItem         = NULL;
        helpListPars.getItemArg      = NULL;
        helpListPars.onFocusChg      = NULL;
        helpListPars.onFocusChgArg   = NULL;
        helpListPars.onSelect        = NULL;
        helpListPars.onSelectArg     = NULL;
        helpListPars.onValChg        = NULL;
        helpListPars.onValChgArg     = NULL;
        helpListPars.pInst           = &(appDat->helpListInst);

        retVal = FGS_LIST_Open (&helpListPars, MENU_SCAN_ID_HELPLIST,
                                inst->pnlHdl, &(inst->listHelpHdl));
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.x              = 0;
        helpPars.pos.y              = 0;
        helpPars.pos.width          = helpListPars.pos.width;
        helpPars.pos.height         = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        helpPars.pos.alignment      = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        helpPars.bkgrType           = OSD_LSTITEM_BKGR_HELP;
        helpPars.descrPos.x         = 0;
        helpPars.descrPos.y         = 0;
        helpPars.descrPos.width     = helpPars.pos.width;
        helpPars.descrPos.height    = helpPars.pos.height;
        helpPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----- menu lines -------------------------------------------------*/

        memset(&txtPars, 0, sizeof(FGS_ITEM_TXT_OPEN_S));

        txtPars.bkgrType   = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos   = descrPos;
        txtPars.valuePos   = valuePos;
        txtPars.pos        = itemPos;

        selPars.bkgrType   = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos   = descrPos;
        selPars.valuePos   = valuePos;


        /* ----- Scan Region ----------------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.params.numItems     = CHANREGION_NUMS;
        selPars.params.index        = inst->pCfg->region;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_SCAN_ID_ENTRY_REGION_TERR,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_REGION_TERR,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        itemPos.y      += 2 * SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Scan Method ----------------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.params.numItems     = MENU_SCANMETHOD_NUMS;
        selPars.params.index        = inst->pCfg->method;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                    MENU_SCAN_ID_ENTRY_METHOD,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_METHOD,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Scan Mode ------------------------------------------------- */

        selPars.pos              = itemPos;
        selPars.params.numItems  = AV_SCANFILTER_NUMS;
        selPars.params.index     = inst->pCfg->opts.filter;

        helpPars.pos.y           = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                    MENU_SCAN_ID_ENTRY_MODE,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_MODE,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Channel --------------------------------------------------- */

        (void)TPLIST_GetNumCarriers (&(inst->pCfg->network), &numCarriers);

        if ( (int16_t)inst->pCfg->tpLstPos >= numCarriers )
        {
            inst->pCfg->tpLstPos = 0;
        }

        selPars.pos                 = itemPos;
        selPars.params.numItems     = numCarriers;
        selPars.params.index        = inst->pCfg->tpLstPos;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                     MENU_SCAN_ID_ENTRY_EDITTP,
                                     inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_EDITTP,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Signal bar ------------------------------------------------- */

        sigPars.pnlHdl     = inst->pnlHdl;
        sigPars.pos.x      = itemPos.x + listPars.pos.x + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        sigPars.pos.y      = itemPos.y + listPars.pos.y + SCALER_GetCurrVal(LIST_LINE_PADDING);
        sigPars.descrWidth = ((uint16_t)(itemPos.width- SCALER_GetCurrVal(MAIN_LEFT_PADDING))/(2));
        sigPars.barWidth   = ((uint16_t)(itemPos.width-
                                    SCALER_GetCurrVal(MAIN_LEFT_PADDING)-
                                    SCALER_GetCurrVal(MAIN_RIGHT_PADDING))/(2));
        sigPars.feIdx      = -1;

        retVal = MAIN_SignalBoxOpen(&(appDat->sigBox), &sigPars);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += MAIN_SignalBoxGetHeight();
        itemPos.y      += (2*SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ---start scan --------------------------------------------------- */

        helpPars.pos.y  = itemPos.y;
        txtPars.pos     = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SCAN_ID_ENTRY_STARTSCAN,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_STARTSCAN,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- GOTO next --------------------------------------------------- */

        if ( inst->guidedMode )
        {
            helpPars.pos.y  = itemPos.y;
            txtPars.pos     = itemPos;
            /* use full width for description since there is no value */
            txtPars.descrPos.width += txtPars.valuePos.width;
            txtPars.valuePos.width = 0;

            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SCAN_ID_ENTRY_GOTONEXT,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpPars,
                                        MENU_SCAN_ID_ENTRY_GOTONEXT,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);

        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}


#endif /* APPL_FETERR_ENABLED */

/************** Cable Scan Menu ****************************************/

#ifdef APPL_FECABLE_ENABLED
/*!
*******************************************************************************
**
** \brief  Entry function for terrestrial scan menu
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_ScanCableEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_SCAN_INST_S* inst = menuScanDat;
    uint32_t               lastFocusItem;

    DBG_Assert(inst != 0);
    if(inst == NULL)return FFALSE; /* to satisfy MISRA */

    if ( test == FTRUE ) return FTRUE;

    inst->pCfg = &(inst->cfgCable);

    MENU_ScanEntryStart (inst, h);

    (void)MENU_ScanCableBuild((FAPI_SYS_HandleT)inst);

    /* set values according to current channel selection */
    MENU_ScanMethodHasChanged(inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        if ( inst->guidedMode )
        {
            lastFocusItem = MENU_SCAN_ID_ENTRY_STARTSCAN;
        }
        else
        {
            lastFocusItem = MENU_SCAN_ID_ENTRY_METHOD;
        }
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);

    MENU_ScanEntryEnd (inst);

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  Set the descriptions for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_ScanCableGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                        FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_SCAN_ID_ENTRY_METHOD:
            strIdx = STR_ENTRY_SCAN_METHOD;
            break;

        case MENU_SCAN_ID_ENTRY_MODE:
            strIdx = STR_ENTRY_SCAN_MODE;
            break;

        case MENU_SCAN_ID_ENTRY_EDITTP:
            strIdx = STR_ENTRY_SCAN_EDITCH;
            break;

        case MENU_SCAN_ID_ENTRY_STARTSCAN:
            strIdx = STR_ENTRY_SCAN_STARTSCAN;
            break;

        case MENU_SCAN_ID_ENTRY_GOTONEXT:
            strIdx = MENU_GuidedGetNextStr();
            break;

        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}


/*!
*******************************************************************************
**
** \brief  Builds the terrestrial menu panel and its members.
**
** \param h        Address of menu handle.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_ScanCableBuild(FAPI_SYS_HandleT h)
{
    MENU_SCAN_INST_S*       inst = (MENU_SCAN_INST_S*)h;
    FGS_PANEL_OPEN_S        pnlPars;
    FGS_LIST_OPEN_S         listPars;
    FGS_LIST_OPEN_S         helpListPars;
    FAPI_SYS_HandleT        itemHdl;
    FGS_BOX_OPEN_S          boxPars;
    FGS_ITEM_TXT_OPEN_S     txtPars;
    FGS_ITEM_TXT_OPEN_S     helpPars;
    MAIN_SIGBOX_OPEN_S      sigPars;
    FGS_ITEM_SEL_OPEN_S     selPars;
    FGS_ITEM_NUM_OPEN_S     numPars;
    FGS_POS_S               itemPos;
    FGS_ITEM_S              item;
    FGS_POS_S               valuePos;
    FGS_POS_S               descrPos;
    int32_t                 retVal;
    uint16_t                helpWidth;
    uint16_t                stackedFrameWidth;
    uint16_t                innerHeight;
    uint32_t                numLines = 6;
    int16_t                 numCarriers = 0;

    memset(&helpListPars, 0, sizeof(helpListPars));
    memset(&boxPars,      0, sizeof(boxPars));
    memset(&txtPars,      0, sizeof(txtPars));
    memset(&helpPars,     0, sizeof(helpPars));
    memset(&sigPars,      0, sizeof(sigPars));
    memset(&selPars,      0, sizeof(selPars));
    memset(&numPars,      0, sizeof(numPars));
    memset(&item,         0, sizeof(item));

    if ( inst->guidedMode ) numLines++;

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_LOKR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                 + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_ScanOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SCAN, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_SCAN_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_SCAN_CABLE,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_ScanCableGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_ScanGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_ScanOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_ScanOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_ScanValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_SCAN_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- create help list ------------------------------------------ */

        /* NOTE: Common help list method not used here because of non-linear
                 item position. */

        helpListPars = listPars;

        helpListPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_WIDTH) +
                                      SCALER_GetCurrVal(INNER_FRAME_WIDTH) + SCALER_GetCurrVal(HELP_LR_PADDING) +
                                      stackedFrameWidth;
        helpListPars.pos.width       = helpWidth - 2 * SCALER_GetCurrVal(HELP_LR_PADDING);
        helpListPars.bkgrType        = 0;
        helpListPars.getDescrItem    = MENU_ScanGetHelpDescrItem;
        helpListPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        helpListPars.getItem         = NULL;
        helpListPars.getItemArg      = NULL;
        helpListPars.onFocusChg      = NULL;
        helpListPars.onFocusChgArg   = NULL;
        helpListPars.onSelect        = NULL;
        helpListPars.onSelectArg     = NULL;
        helpListPars.onValChg        = NULL;
        helpListPars.onValChgArg     = NULL;
        helpListPars.pInst           = &(appDat->helpListInst);

        retVal = FGS_LIST_Open (&helpListPars, MENU_SCAN_ID_HELPLIST,
                                inst->pnlHdl, &(inst->listHelpHdl));
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.x              = 0;
        helpPars.pos.y              = 0;
        helpPars.pos.width          = helpListPars.pos.width;
        helpPars.pos.height         = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        helpPars.pos.alignment      = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        helpPars.bkgrType           = OSD_LSTITEM_BKGR_HELP;
        helpPars.descrPos.x         = 0;
        helpPars.descrPos.y         = 0;
        helpPars.descrPos.width     = helpPars.pos.width;
        helpPars.descrPos.height    = helpPars.pos.height;
        helpPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----- menu lines -------------------------------------------------*/

        memset(&txtPars, 0, sizeof(FGS_ITEM_TXT_OPEN_S));

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;

        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;

        /* ----- Scan Method ----------------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.params.numItems     = MENU_SCANMETHOD_NUMS;
        selPars.params.index        = inst->pCfg->method;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                    MENU_SCAN_ID_ENTRY_METHOD,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_METHOD,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Scan Mode ------------------------------------------------- */

        selPars.pos              = itemPos;
        selPars.params.numItems  = AV_SCANFILTER_NUMS;
        selPars.params.index     = inst->pCfg->opts.filter;

        helpPars.pos.y           = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                    MENU_SCAN_ID_ENTRY_MODE,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_MODE,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Channel --------------------------------------------------- */

        (void) TPLIST_GetNumCarriers (&(inst->pCfg->network), &numCarriers);

        if ( (int16_t)inst->pCfg->tpLstPos >= numCarriers )
        {
            inst->pCfg->tpLstPos = 0;
        }

        selPars.pos                 = itemPos;
        selPars.params.numItems     = numCarriers;
        selPars.params.index        = inst->pCfg->tpLstPos;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                     MENU_SCAN_ID_ENTRY_EDITTP,
                                     inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_EDITTP,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Signal bar ------------------------------------------------- */

        sigPars.pnlHdl     = inst->pnlHdl;
        sigPars.pos.x      = itemPos.x + listPars.pos.x + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        sigPars.pos.y      = itemPos.y + listPars.pos.y + SCALER_GetCurrVal(LIST_LINE_PADDING);
        sigPars.descrWidth = ((uint16_t)(itemPos.width- SCALER_GetCurrVal(MAIN_LEFT_PADDING))/(2));
        sigPars.barWidth   = ((uint16_t)(itemPos.width-
                                    SCALER_GetCurrVal(MAIN_LEFT_PADDING)-
                                    SCALER_GetCurrVal(MAIN_RIGHT_PADDING))/(2));
        sigPars.feIdx      = -1;

        retVal = MAIN_SignalBoxOpen(&(appDat->sigBox), &sigPars);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += MAIN_SignalBoxGetHeight();
        itemPos.y      += (2*SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* ---start scan --------------------------------------------------- */

        helpPars.pos.y  = itemPos.y;
        txtPars.pos     = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SCAN_ID_ENTRY_STARTSCAN,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_STARTSCAN,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- GOTO next --------------------------------------------------- */

        if ( inst->guidedMode )
        {
            helpPars.pos.y  = itemPos.y;
            txtPars.pos     = itemPos;
            /* use full width for description since there is no value */
            txtPars.descrPos.width += txtPars.valuePos.width;
            txtPars.valuePos.width = 0;

            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SCAN_ID_ENTRY_GOTONEXT,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpPars,
                                        MENU_SCAN_ID_ENTRY_GOTONEXT,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);

        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}


#endif /* APPL_FECABLE_ENABLED */


#ifdef APPL_FEDIRECT_ENABLED
/*!
*******************************************************************************
**
** \brief  Entry function for direct ts scan menu
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_ScanDirectEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_SCAN_INST_S* inst = menuScanDat;

    DBG_Assert(inst != 0);
    if(inst == NULL)return FFALSE; /* to satisfy MISRA */

    if ( test == FTRUE ) return FTRUE;

    inst->pCfg = &(inst->cfgDirect);

    MENU_ScanEntryStart (inst, h);

    //TPLIST_GetCarrierData (&(inst->pCfg->network),inst->pCfg->tpLstPos ,&(inst->carrierInf));

    (void)MENU_ScanDirectBuild((FAPI_SYS_HandleT)inst);

    /* set values according to current channel selection */
    //MENU_ScanMethodHasChanged(inst);

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, MENU_SCAN_ID_ENTRY_STARTSCAN);

    MENU_ScanEntryEnd (inst);

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  Set the descriptions for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_ScanDirectGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                        FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_SCAN_ID_ENTRY_MODE:
            strIdx = STR_ENTRY_SCAN_MODE;
            break;

        case MENU_SCAN_ID_ENTRY_TVRADIO:
            strIdx = STR_ENTRY_SCAN_TVRADIO;
            break;

        case MENU_SCAN_ID_ENTRY_STARTSCAN:
            strIdx = STR_ENTRY_SCAN_STARTSCAN;
            break;

        case MENU_SCAN_ID_ENTRY_GOTONEXT:
            strIdx = MENU_GuidedGetNextStr();
            break;

        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}


/*!
*******************************************************************************
**
** \brief  Builds the direct ts menu panel and its members.
**
** \param h        Address of menu handle.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_ScanDirectBuild(FAPI_SYS_HandleT h)
{
    MENU_SCAN_INST_S*       inst = (MENU_SCAN_INST_S*)h;
    FGS_PANEL_OPEN_S        pnlPars;
    FGS_LIST_OPEN_S         listPars;
    FGS_LIST_OPEN_S         helpListPars;
    FAPI_SYS_HandleT        itemHdl;
    FGS_ITEM_TXT_OPEN_S     txtPars;
    FGS_ITEM_TXT_OPEN_S     helpPars;
    FGS_ITEM_SEL_OPEN_S     selPars;
    FGS_POS_S               itemPos;
    FGS_ITEM_S              item;
    FGS_POS_S               valuePos;
    FGS_POS_S               descrPos;
    int32_t                 retVal;
    uint16_t                helpWidth;
    uint16_t                stackedFrameWidth;
    uint16_t                innerHeight;
    uint32_t                numLines = 4;

    memset(&helpListPars, 0, sizeof(helpListPars));
    memset(&txtPars,      0, sizeof(txtPars));
    memset(&helpPars,     0, sizeof(helpPars));
    memset(&selPars,      0, sizeof(selPars));
    memset(&item,         0, sizeof(item));

    if ( inst->guidedMode ) numLines++;

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_LOKR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                  + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_ScanOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SCAN, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_SCAN_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth,STR_TITLE_SCAN_DIRECT,
                                     NULL,NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_ScanDirectGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_ScanGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_ScanOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_ScanOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_ScanValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_SCAN_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- create help list ------------------------------------------ */

        /* NOTE: Common help list method not used here because of non-linear
                 item position. */

        helpListPars = listPars;

        helpListPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_WIDTH) +
                                      SCALER_GetCurrVal(INNER_FRAME_WIDTH) + SCALER_GetCurrVal(HELP_LR_PADDING) +
                                      stackedFrameWidth;
        helpListPars.pos.width       = helpWidth - 2 * SCALER_GetCurrVal(HELP_LR_PADDING);
        helpListPars.bkgrType        = 0;
        helpListPars.getDescrItem    = MENU_ScanGetHelpDescrItem;
        helpListPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        helpListPars.getItem         = NULL;
        helpListPars.getItemArg      = NULL;
        helpListPars.onFocusChg      = NULL;
        helpListPars.onFocusChgArg   = NULL;
        helpListPars.onSelect        = NULL;
        helpListPars.onSelectArg     = NULL;
        helpListPars.onValChg        = NULL;
        helpListPars.onValChgArg     = NULL;
        helpListPars.pInst           = &(appDat->helpListInst);

        retVal = FGS_LIST_Open (&helpListPars, MENU_SCAN_ID_HELPLIST,
                                inst->pnlHdl, &(inst->listHelpHdl));
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.x              = 0;
        helpPars.pos.y              = 0;
        helpPars.pos.width          = helpListPars.pos.width;
        helpPars.pos.height         = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        helpPars.pos.alignment      = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        helpPars.bkgrType           = OSD_LSTITEM_BKGR_HELP;
        helpPars.descrPos.x         = 0;
        helpPars.descrPos.y         = 0;
        helpPars.descrPos.width     = helpPars.pos.width;
        helpPars.descrPos.height    = helpPars.pos.height;
        helpPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----- menu lines -------------------------------------------------*/

        /* ----- Scan Mode ------------------------------------------------- */

        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.pos              = itemPos;
        selPars.params.numItems  = AV_SCANFILTER_NUMS;
        selPars.params.index     = inst->pCfg->opts.filter;

        helpPars.pos.y           = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                    MENU_SCAN_ID_ENTRY_MODE,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_MODE,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- TV/Radio option ------------------------------------------- */

        selPars.pos              = itemPos;
        selPars.params.numItems  = AV_SCANCHOICE_NUMS;
        selPars.params.index     = inst->pCfg->opts.choice;

        helpPars.pos.y              = itemPos.y;

        retVal = FGS_ITEM_SEL_Open (&selPars,
                                    MENU_SCAN_ID_ENTRY_TVRADIO,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_TVRADIO,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- empty line ------------------------------------------------- */

        itemPos.y      += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ---start scan --------------------------------------------------- */

        helpPars.pos.y  = itemPos.y;

        memset(&txtPars, 0, sizeof(FGS_ITEM_TXT_OPEN_S));

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;
        txtPars.pos     = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SCAN_ID_ENTRY_STARTSCAN,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_SCAN_ID_ENTRY_STARTSCAN,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- GOTO next --------------------------------------------------- */

        if ( inst->guidedMode )
        {
            helpPars.pos.y  = itemPos.y;
            txtPars.pos     = itemPos;
            /* use full width for description since there is no value */
            txtPars.descrPos.width += txtPars.valuePos.width;
            txtPars.valuePos.width = 0;

            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SCAN_ID_ENTRY_GOTONEXT,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_ITEM_TXT_Open (&helpPars,
                                        MENU_SCAN_ID_ENTRY_GOTONEXT,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);

        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}


#endif /* APPL_FECABLE_ENABLED */
/*---------------------------------------------------------------------------*/
/* local functions for serving all front ends                                */
/*---------------------------------------------------------------------------*/

/*!
*******************************************************************************
**
** \brief  Leave the Scan Menu.
**
** More detailed description
**
** \param h        Address of menu handle.
** \param force    .
**
** \return FTRUE
******************************************************************************/
FSTATIC fbool_t MENU_ScanLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_SCAN_INST_S*   inst = (MENU_SCAN_INST_S*)h;

    if ( inst->currFe >= 0 )
    {
        (void)AV_TuneModeStop(inst->currFe);
    }

    (void)MAIN_SignalBoxClose((&appDat->sigBox));

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_EXIT:
            (void)MAIN_RestartPresentation();
            break;

        default:
            break;
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_SCAN);

    inst->pnlHdl = NULL;
    return FTRUE;
}


/*!
*******************************************************************************
**
** \brief  Common used function called when the menu gets/looses the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_ScanOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_SCAN_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_MENU,  MENU_ScanKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK,  MENU_ScanKeys, h, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
    }
}

/*!
*******************************************************************************
**
** \brief  Activates the help icons for the current menu list item.
**
** \param h        Address of menu handle.
** \param index    unused.
** \param id       List id gets focus.
**
******************************************************************************/
FSTATIC void MENU_ScanOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_SCAN_INST_S*   inst = (MENU_SCAN_INST_S*)h;

    (void)FGS_LIST_SetMemberFocus(inst->listHelpHdl, id);
}

/*!
*******************************************************************************
**
** \brief  Common used function to handle the OK-key for the selected listitem.
**
** \param h        Address of menu handle.
** \param itemId   Listitem Id.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_ScanOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_SCAN_INST_S*       inst = (MENU_SCAN_INST_S*)h;
    MENUSTACK_ENTRY         newEntry = NULL;
    FAPI_SYS_HandleT        arg      = NULL;
    uint8_t                 subLines = 0;
    MENU_SELECTLIST_OPEN_S  selectLst;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    FGS_LOC_S               memberLoc;
    fbool_t                 openSelectLst = FFALSE;
    int16_t                 numCarriers = 0;
    MENU_TPLIST_PARAMS_S    tplistPars;

    switch ( itemId )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_SCAN_ID_ENTRY_SATELLITE :
            openSelectLst = FTRUE;
            subLines = 5;
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case MENU_SCAN_ID_ENTRY_REGION_TERR :
            openSelectLst = FTRUE;
            subLines      = 5;
            break;

        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED
        /*
        case MENU_SCAN_ID_ENTRY_REGION_CABLE :
            openSelectLst = FTRUE;
            subLines      = 5;
            break;
        */
        #endif /* APPL_FECABLE_ENABLED */

        case MENU_SCAN_ID_ENTRY_METHOD :
            openSelectLst = FTRUE;
            subLines      = MENU_SCANMETHOD_NUMS;
            break;

        case MENU_SCAN_ID_ENTRY_MODE :
            openSelectLst = FTRUE;
            subLines      = AV_SCANFILTER_NUMS;
            break;

        case MENU_SCAN_ID_ENTRY_TVRADIO :
            openSelectLst = FTRUE;
            subLines      = AV_SCANCHOICE_NUMS;
            break;

        case MENU_SCAN_ID_ENTRY_EDITTP:

            /* currently only available for satellite scan menu */
            if(( inst->pCfg->system == FE_SYSTEM_TYPE_SATELLITE ) ||
               ( inst->pCfg->system == FE_SYSTEM_TYPE_CABLE ))
            {
                #ifdef APPL_FESAT_ENABLED
                if(inst->pCfg->system == FE_SYSTEM_TYPE_SATELLITE)
                {
                    tplistPars.satLstPos     = inst->currSatLstPos;
                }
                #endif
                tplistPars.system        = inst->pCfg->system;
                tplistPars.carrierLstPos = inst->pCfg->tpLstPos;
                tplistPars.lockSat       = FFALSE;
                tplistPars.leaveCb       = MENU_ScanTpCallback;

                arg      = (FAPI_SYS_HandleT)(&tplistPars);
                newEntry = MENU_TpListEntry;
            }
            else
            {
                openSelectLst = FTRUE;
                subLines      = 4;
            }
            break;

        case MENU_SCAN_ID_ENTRY_STARTSCAN:
            (void)TPLIST_GetNumCarriers (&(inst->pCfg->network), &numCarriers);
            if( (numCarriers != 0)
                #ifdef APPL_BLINDSCAN_ENABLED
                ||(inst->pCfg->method == MENU_SCANMETHOD_BLIND)
                #endif
               )
            {
                arg      = (FAPI_SYS_HandleT)(inst->pCfg);
                newEntry = MENU_ScanningEntry;
            }
            else
            {
                return FFALSE;
            }
            break;

        case MENU_SCAN_ID_ENTRY_GOTONEXT :

            return MENU_GuidedGotoNext();

        default:
            MENU_CommonMsgShow(STR_MSG_NOTIMPLEMENTEDYET, FFALSE);
            return FFALSE;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.width     = (4 * SCALER_GetCurrVal(SUB_LIST_WIDTH)) / 3;
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.x         = memberLoc.x + SCALER_GetCurrVal(MAIN_WIDTH) -
                                  selectLst.pos.width;
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_ScanGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;
        selectLst.onExit        = NULL;
        selectLst.onExitArg     = h;

        return MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }
    else
    {
        /* store last focus */
        MENUSTACK_SetData(itemId);

        if(newEntry == NULL) return FFALSE;
        else                 return MENUSTACK_Up(newEntry, arg);
    }
}

FSTATIC void MENU_ScanTpCallback (int16_t satLstPos, int16_t carrierLstPos)
{
    MENU_SCAN_INST_S*   inst = menuScanDat;

    #ifdef APPL_FESAT_ENABLED
    CFGSAT_IDX newIdx;

    if( inst->pCfg == &inst->cfgSat )
    {
        if ( CFGSAT_GetIndex (satLstPos, &newIdx) == FAPI_OK )
        {
            appDat->currSatIdx = newIdx;
        }

    }
    #endif
    inst->pCfg->tpLstPos = carrierLstPos;
}
/*!
*******************************************************************************
**
** \brief  Common used function to handles the Value change for the selected Listitem.
**
** \param h        Address of menu handle.
** \param itemId   Listitem Id.
** \param newVal   New Value for Listitem.
**
******************************************************************************/
FSTATIC void MENU_ScanValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    MENU_SCAN_INST_S*     inst = (MENU_SCAN_INST_S*)h;
    int16_t               numCarriers = 0;
    FGS_ITEM_SEL_PARAMS_S selParams;

    #ifdef APPL_FESAT_ENABLED
    CFGSAT_IDX            newIdx;
    #endif /* APPL_FESAT_ENABLED */

    switch ( itemId )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_SCAN_ID_ENTRY_SATELLITE: /* satellite has changed */

            inst->satInfValid = FFALSE;

            if ( CFGSAT_GetIndex (newVal.number, &newIdx) == FAPI_OK )
            {
                appDat->currSatIdx         = newIdx;

                inst->pCfg->network.satIdx = newIdx;
                inst->currSatLstPos        = newVal.number;
                inst->pCfg->tpLstPos       = 0;

                if ( CFGSAT_GetSatData (inst->currSatLstPos,
                                        &(inst->satInf)) == FAPI_OK )
                {
                    inst->satInfValid = FTRUE;
                }

                (void)TPLIST_GetNumCarriers (&(inst->pCfg->network), &numCarriers);

                selParams.numItems = numCarriers;
                selParams.index    = inst->pCfg->tpLstPos;

                /* update carrier display */
                (void)FGS_LIST_SetMemberParams (inst->listHdl,
                                          MENU_SCAN_ID_ENTRY_EDITTP, &selParams);
                (void)MENU_ScanUpdateTuner(inst);
            }
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case MENU_SCAN_ID_ENTRY_REGION_TERR:
            inst->pCfg->region = (CHANREGION_E)newVal.number;
//            if(TPLIST_GetIndex(&(inst->pCfg->network),newVal.number,&newTPListIdx) == FAPI_OK)
//            {
//                TPLIST_GetNumCarriers (&(inst->pCfg->network), &numCarriers);

                /* clear all members in current channel list */
                (void)TPLIST_Clear(&(inst->pCfg->network),SYS_OWNER_SYSTEM);
 /*               for (listId = 0; listId < numCarriers; listId++)
                {
                    if(TPLIST_RemoveCarrier(&(inst->pCfg->network), listId) != FAPI_OK)
                        break;
                }
*/                /* now populate TPLIST according to new region setting */
                MAIN_GenerateTerrChannelList ((CHANREGION_E)newVal.number);

                (void)TPLIST_GetNumCarriers (&(inst->pCfg->network), &numCarriers);
                selParams.numItems = numCarriers;
                selParams.index    = 0;
                /* update carrier display */
                (void)FGS_LIST_SetMemberParams(inst->listHdl, MENU_SCAN_ID_ENTRY_EDITTP,
                                         &selParams);
                if(TPLIST_GetCarrierData(&(inst->pCfg->network),0,&(inst->carrierInf)) == FAPI_OK)
//                TPLIST_GetIndex(&(inst->pCfg->network),0,&newTPListIdx);
/*                if(TPLIST_GetDataByIndex(&(inst->pCfg->network),
                                         newTPListIdx,
                                         &(inst->carrierInf)
                                        ) == FAPI_OK)
*/
                {
                    /* update OSD */
                    inst->pCfg->tpLstPos = 0;
                    (void)MENU_ScanUpdateTuner(inst);
                }
//            }
            break;

        #endif /* APPL_FETERR_ENABLED */


        case MENU_SCAN_ID_ENTRY_METHOD:
            inst->pCfg->method = (uint8_t)newVal.number;

            MENU_ScanMethodHasChanged(inst);
            break;

        case MENU_SCAN_ID_ENTRY_MODE:
            inst->pCfg->opts.filter = (AV_SCANFILTER_E)newVal.number;
            break;

        case MENU_SCAN_ID_ENTRY_TVRADIO:
            inst->pCfg->opts.choice = (AV_SCANCHOICE_E)newVal.number;
            break;

        case MENU_SCAN_ID_ENTRY_EDITTP:
            inst->pCfg->tpLstPos = (uint16_t)newVal.number;

            (void)MENU_ScanUpdateTuner(inst);
            break;

        default:
            break;
    }
}

/*!
*******************************************************************************
**
** \brief  Set the value (e.g. text, number, icon, ...) for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_ScanGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_SCAN_INST_S*     inst = (MENU_SCAN_INST_S*)h;
    uint32_t                 bufPos = 0;
    uint32_t strIdx;

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    switch ( id )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_SCAN_ID_ENTRY_SATELLITE :
            if ( CFGSAT_GetSatData(index, &(inst->tmpSatDat)) == FAPI_OK )
            {
                item->type            = FGS_ITEM_TYPE_STRING;
                item->item.str.chr    = STR_Build(inst->tmpSatDat.nameCharSet,
                                                  inst->tmpSatDat.name);
            }
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        case MENU_SCAN_ID_ENTRY_REGION_TERR :
            switch ( (CHANREGION_E)index )
            {
                case CHANREGION_WESTEUROPE:
                    strIdx = STR_VALUE_SCAN_REGION_WESTEUROPE;
                    break;
                case CHANREGION_EASTEUROPE:
                    strIdx = STR_VALUE_SCAN_REGION_EASTEUROPE;
                    break;
                case CHANREGION_UK:
                    strIdx = STR_VALUE_SCAN_REGION_UK;
                    break;
                case CHANREGION_FRANCE:
                    strIdx = STR_VALUE_SCAN_REGION_FRANCE;
                    break;
                case CHANREGION_ITALY:
                    strIdx = STR_VALUE_SCAN_REGION_ITALY;
                    break;
                case CHANREGION_IRELAND:
                    strIdx = STR_VALUE_SCAN_REGION_IRELAND;
                    break;
                case CHANREGION_CHINA:
                    strIdx = STR_VALUE_SCAN_REGION_CHINA;
                    break;
                case CHANREGION_USA:
                    strIdx = STR_VALUE_SCAN_REGION_USA;
                    break;
                case CHANREGION_CANADA:
                    strIdx = STR_VALUE_SCAN_REGION_CANADA;
                    break;
                case CHANREGION_JAPAN:
                    strIdx = STR_VALUE_SCAN_REGION_JAPAN;
                    break;
                case CHANREGION_AUSTRALIA:
                    strIdx = STR_VALUE_SCAN_REGION_AUSTRALIA;
                    break;
                default:
                    strIdx = STR_EMPTYSTRING;
                    break;
            }
            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.chr        = OSD_GetStr(strIdx);
            break;
        #endif

        case MENU_SCAN_ID_ENTRY_METHOD :
            switch ( index )
            {
                case MENU_SCANMETHOD_AUTO:
                    strIdx = STR_VALUE_SCAN_METHOD_AUTO;
                    break;

                case MENU_SCANMETHOD_SINGLE:
                    strIdx = STR_VALUE_SCAN_METHOD_CHANNEL;
                    break;
                #ifdef APPL_BLINDSCAN_ENABLED
                case MENU_SCANMETHOD_BLIND:
                    strIdx = STR_VALUE_SCAN_METHOD_BLIND;
                    break;
                #endif
                default:
                    strIdx = STR_EMPTYSTRING;
                    break;
            }

            item->type         = FGS_ITEM_TYPE_STRING;
            item->item.str.chr = OSD_GetStr(strIdx);
            break;

        case MENU_SCAN_ID_ENTRY_MODE :
            switch ( (AV_SCANFILTER_E)index )
            {
                case AV_SCANFILTER_ALL:
                    strIdx = STR_VALUE_SCAN_MODE_ALL;
                    break;
                case AV_SCANFILTER_FREEONLY:
                    strIdx = STR_VALUE_SCAN_MODE_FREEONLY;
                    break;
                case AV_SCANFILTER_CRYPTEDONLY:
                    strIdx = STR_VALUE_SCAN_MODE_CRYPTONLY;
                    break;
                default:
                    strIdx = STR_EMPTYSTRING;
                    break;
            }

            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.chr        = OSD_GetStr(strIdx);
            break;

        case MENU_SCAN_ID_ENTRY_TVRADIO :
            /*lint -save -e408 */
            switch ( (AV_SCANFILTER_E)index )
            {
                case AV_SCANCHOICE_ALL:
                    strIdx = STR_VALUE_SCAN_CHOICE_ALL;
                    break;
                case AV_SCANCHOICE_TVONLY:
                    strIdx = STR_VALUE_SCAN_CHOICE_TV;
                    break;
                case AV_SCANCHOICE_RADIOONLY:
                    strIdx = STR_VALUE_SCAN_CHOICE_RADIO;
                    break;
                default:
                    strIdx = STR_EMPTYSTRING;
                    break;
            }
            /*lint -restore */

            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.chr        = OSD_GetStr(strIdx);
            break;

        case MENU_SCAN_ID_ENTRY_EDITTP :
            item->type                = FGS_ITEM_TYPE_STRING;
            if ( TPLIST_GetCarrierData (&(inst->pCfg->network), index,
                                        &(inst->carrierInf)) != FAPI_OK )
            {
                item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                break;
            }

            switch ( inst->pCfg->system )
            {
                #ifdef APPL_FESAT_ENABLED
                case FE_SYSTEM_TYPE_SATELLITE:
                    sprintf (appDat->asciiBuf, "%li: %li ", index + 1,
                             inst->carrierInf.pars.sat.frequency / 1000);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                            OSD_GetStr(STR_UNIT_MHZ),
                                             MAIN_LINEBUFCHARSET);

                    if ( inst->carrierInf.pars.sat.pol == FE_POL_HOR)
                    {
                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 OSD_GetStr(STR_UNIT_HOR),
                                                 MAIN_LINEBUFCHARSET);
                    }
                    else
                    {
                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 OSD_GetStr(STR_UNIT_VER),
                                                 MAIN_LINEBUFCHARSET);
                    }
                    break;
                #endif /* APPL_FESAT_ENABLED */

                #ifdef APPL_FETERR_ENABLED

                case FE_SYSTEM_TYPE_TERRESTRIAL:
                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_Build(
                                             inst->carrierInf.nameCharSet,
                                             inst->carrierInf.name),
                                             MAIN_LINEBUFCHARSET);

                    sprintf (appDat->asciiBuf, ": %li ",
                             inst->carrierInf.pars.terr.frequency / 1000000);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             OSD_GetStr(STR_UNIT_MHZ),
                                             MAIN_LINEBUFCHARSET);
                    break;
                #endif /* APPL_FETERR_ENABLED */

                #ifdef APPL_FECABLE_ENABLED
                case FE_SYSTEM_TYPE_CABLE:
                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_Build(
                                             inst->carrierInf.nameCharSet,
                                             inst->carrierInf.name),
                                             MAIN_LINEBUFCHARSET);

                    sprintf (appDat->asciiBuf, ": %li ",
                             inst->carrierInf.pars.cable.frequency / 1000000);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             OSD_GetStr(STR_UNIT_MHZ),
                                             MAIN_LINEBUFCHARSET);
                    break;
                #endif /* APPL_FEDIRECT_ENABLED */

                default:
                    (void)STR_BufClear(NULL);
                    break;
            }

            item->item.str.chr = STR_IntLineStr();
            break;

        default:
            item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            break;
    }

    switch ( item->type )
    {
        case FGS_ITEM_TYPE_STRING:
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;

            switch ( state )
            {
                default:
                case FGS_NONFOCUS:
                    item->item.str.col = appDat->txtAttr.colNonFocus;
                    break;
                case FGS_FOCUS:
                    item->item.str.col = appDat->txtAttr.colFocus;
                    break;
                case FGS_DISABLED:
                    item->item.str.col = appDat->txtAttr.colDisabled;
                    break;
            }
            break;

        case FGS_ITEM_TYPE_NUMBER:
            item->item.number.font          = MAIN_FONT;
            switch ( state )
            {
                default:
                case FGS_NONFOCUS:
                    item->item.number.col = appDat->txtAttr.colNonFocus;
                    break;
                case FGS_FOCUS:
                    item->item.number.col = appDat->txtAttr.colFocus;
                    break;
                case FGS_DISABLED:
                    item->item.number.col = appDat->txtAttr.colDisabled;
                    break;
            }
            break;

        default:
            break;
    }
}

/*!
*******************************************************************************
**
** \brief  Common used function to get the help description item for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_ScanGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                           FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        #ifdef APPL_FESAT_ENABLED
        case MENU_SCAN_ID_ENTRY_SATELLITE:
        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        case MENU_SCAN_ID_ENTRY_REGION_TERR   :
        #endif /* APPL_FETERR_ENABLED */

        case MENU_SCAN_ID_ENTRY_METHOD   :
        case MENU_SCAN_ID_ENTRY_MODE     :
        case MENU_SCAN_ID_ENTRY_TVRADIO  :
        case MENU_SCAN_ID_ENTRY_EDITTP   : strIdx = STR_ENTRY_LOKR;
                                           break;

        case MENU_SCAN_ID_ENTRY_STARTSCAN:
        case MENU_SCAN_ID_ENTRY_GOTONEXT : strIdx = STR_ENTRY_OK;
                                           break;

        default:  return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}


/*!
*******************************************************************************
**
** \brief  Common used function to handle the Keys adapted by this menu.
**
** \param h        Address of menu handle.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_ScanKeys (FAPI_SYS_HandleT h)
{
    MENU_SCAN_INST_S* inst = (MENU_SCAN_INST_S*)h;
    fbool_t             keyUsed = FTRUE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
            /* FALLTHHROUGHT */
        case FGS_KEY_BACK:
            if ( inst->guidedMode ) keyUsed = MENU_GuidedGotoPrevious();
            else                    keyUsed = MENUSTACK_Back (NULL);
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

int32_t MENU_ScanUpdateTuner (MENU_SCAN_INST_S* inst)
{
    int32_t retVal;

    if ( inst->currFe >= 0 )
    {
        (void)AV_TuneModeStop(inst->currFe);
        inst->currFe = -1;
    }

    retVal = AV_TuneModeStart (-1, &(inst->pCfg->network),
                               inst->pCfg->tpLstPos,
                               NULL, &(inst->currFe));

    /* update signal bar */
    (void)MAIN_SignalBoxSetFeIdx (&(appDat->sigBox), inst->currFe);
    (void)MAIN_SignalBoxUpdate   (&(appDat->sigBox));

    return retVal;
}

FSTATIC void MENU_ScanMethodHasChanged (MENU_SCAN_INST_S* inst)
{
    switch ( inst->pCfg->method )
    {
        case MENU_SCANMETHOD_AUTO:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                           MENU_SCAN_ID_ENTRY_EDITTP, FFALSE);
            break;

        case MENU_SCANMETHOD_SINGLE:

            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                           MENU_SCAN_ID_ENTRY_EDITTP, FTRUE);
            break;

        #if 0
        // TODO: to be implemented
        case MENU_SCANMETHOD_CURRTS:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_SCAN_ID_EDITTP,
                                           FFALSE);
            break;
        #endif
        #ifdef APPL_BLINDSCAN_ENABLED
        case MENU_SCANMETHOD_BLIND:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                           MENU_SCAN_ID_ENTRY_EDITTP, FTRUE);

            break;
        #endif
        default:
            break;
    }
}


void MENU_ScanUpdateLockStatus (uint8_t feNum, SYS_FE_LOCK_E status)
{
    MENU_SCAN_INST_S* inst = (MENU_SCAN_INST_S*)menuScanDat;

    if ( inst->pnlHdl == NULL ) return;

    (void)MAIN_SignalBoxUpdate(&(appDat->sigBox));
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
