/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_scanning.c
**
**              $Id: app_menu_scanning.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD Scanning menu.
**
** Detail description. [\ref app_smartgo_app_ref_scanning]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"
#include "app_menu_scan.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_SCANNING_ID_SCANSTATUS = 0,
    MENU_SCANNING_ID_NEWTP,
    MENU_SCANNING_ID_NEWSERV,
    MENU_SCANNING_ID_TVLISTTITLE,
    MENU_SCANNING_ID_RADIOLISTTITLE,
    MENU_SCANNING_ID_TVLIST,
    MENU_SCANNING_ID_RADIOLIST,
    MENU_SCANNING_ID_TITLE,
    MENU_SCANNING_ID_LIST,
    MENU_SCANNING_ID_REST,
    MENU_SCANNING_ID_HELP_LISTCHANGE_BUTTON,
    MENU_SCANNING_ID_HELP_LISTCHANGE_TEXT,
    MENU_SCANNING_ID_HELPBOTTOM
};

typedef struct
{
    MENU_SCAN_CFG_S*    cfg;

    FAPI_SYS_HandleT           pnlHdl;
    FAPI_SYS_HandleT           listHdl;
    FAPI_SYS_HandleT           tvListHdl;
    FAPI_SYS_HandleT           radioListHdl;

    FGS_LNAVI_INST_S    radioLnaviInst;

    FGS_PANEL_MEMBER_S* pItem;
    FGS_POS_S           itemPos;
    FGS_PANEL_MEMBER_S* pItemBorder;
    FGS_POS_S           itemBorderPos;

    /* Scan status parameters */
    int8_t                  currFe;
    fbool_t               listChg;
    fbool_t               tpChg;
    SYS_EVT_SCAN_S      scanStat;
    SLIST_SERVIDX       tvList   [SLIST_MAX_TV];
    SLIST_SERVIDX       radioList[SLIST_MAX_RADIO];
    uint16_t                 tvServFound;     // NEW found services
    uint16_t                 tvAlreadyInList;
    uint16_t                 radioServFound;  // NEW found services
    uint16_t                 radioAlreadyInList;
    uint16_t                 chanFound;     // NEW found channels/transponders
    int16_t                 tpTempAdded;   // index of temporary added transponder
    SLIST_ITEM_S        tmpListItem;

} MENU_SCANNING_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

MENU_SCANNING_INST_S* menuScanningDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void  MENU_ScanningOnFocusSet        (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC fbool_t MENU_ScanningLeave             (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_ScanningExitKeys          (FAPI_SYS_HandleT h);
FSTATIC void  MENU_ScanningExitAndSaveHdl    (void* arg);
FSTATIC void  MENU_ScanningDiscardChangesHdl (void* arg);
FSTATIC void  MENU_ScanningRestoreData       (MENU_SCANNING_INST_S* inst);
FSTATIC void  MENU_ScanningGetDescrItem      (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void  MENU_ScanningGetItem           (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void  MENU_ScanningGetListItem       (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC fbool_t MENU_ScanningKeys              (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_ScanningBuild             (MENU_SCANNING_INST_S* inst);
FSTATIC int32_t  MENU_ScanningProgressPaint     (FAPI_SYS_HandleT h);
FSTATIC int32_t  MENU_ScanningPaintRest         (FAPI_SYS_HandleT h);

FSTATIC fbool_t MENU_ScanningStopScan          (MENU_SCANNING_INST_S* inst);

/*!
*******************************************************************************
**
** \brief  Initial function for the OSD Scanning Menu.
**         For the scanning menu no init is needed.
**
** \return FAPI_OK
**
** \sa     MENU_ScanInit
******************************************************************************/
int32_t MENU_ScanningInit (void)
{
    if ( menuScanningDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuScanningDat = UTILS_Malloc(sizeof(MENU_SCANNING_INST_S));
    if ( menuScanningDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuScanningDat, 0, sizeof(MENU_SCANNING_INST_S));

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function.
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_ScanningEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_SCANNING_INST_S* inst   = menuScanningDat;
    int32_t                  retVal = FAPI_OK;

    if ( h == NULL ) return FFALSE;

    inst->cfg = (MENU_SCAN_CFG_S*)h;

    if ( (inst->cfg->method != MENU_SCANMETHOD_AUTO) &&
         (inst->cfg->method != MENU_SCANMETHOD_SINGLE)
         #ifdef APPL_BLINDSCAN_ENABLED
         && (inst->cfg->method != MENU_SCANMETHOD_BLIND)
        #endif
       )
    {
        return FFALSE;
    }

    if ( test == FTRUE ) return FTRUE;

    inst->listChg            = FFALSE;
    inst->tpChg              = FFALSE;
    inst->scanStat.status    = SCAN_STAT_IDLE;
    inst->scanStat.tpDone    = 0;
    inst->scanStat.tpMax     = 0;
    inst->scanStat.progCurr  = 0;
    inst->scanStat.progMax   = 0;
    inst->tvServFound        = 0;
    inst->tvAlreadyInList    = 0;
    inst->radioServFound     = 0;
    inst->radioAlreadyInList = 0;
    inst->chanFound          = 0;
    inst->tpTempAdded        = -1;
    inst->currFe             = -1;

    (void)MENU_ScanningBuild (inst);

    (void)MENUSTACK_FocusClaim ((int32_t)MENUID_SCANNING);
    (void)FGS_PANEL_Show       (inst->pnlHdl, FFALSE);

    MENUSTACK_SetCleanup(MENU_ScanningLeave, (FAPI_SYS_HandleT)inst);

    /* ----- start scan operation ------------------------------------------ */

    switch ( inst->cfg->method )
    {
        case MENU_SCANMETHOD_AUTO:
            retVal = AV_ScanCarrierList(&(inst->cfg->network),
                                        &(inst->cfg->opts),
                                        &(inst->currFe));
            break;

        case MENU_SCANMETHOD_SINGLE:
            retVal = AV_ScanSingleCarrier(&(inst->cfg->network),
                                          inst->cfg->tpLstPos,
                                          &(inst->cfg->opts),
                                          &(inst->currFe));
            break;
        #ifdef APPL_BLINDSCAN_ENABLED
        case MENU_SCANMETHOD_BLIND:
            retVal = AV_ScanBlind(&(inst->cfg->network),
                                  &(inst->cfg->opts),
                                  &(inst->currFe));
            break;
        #endif
        default:
            /* should never happen */
            break;
    }

    if ( retVal != FAPI_OK )
    {
        inst->currFe = -1;
        inst->scanStat.status = SCAN_STAT_COMPLETED;

        (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SCANNING_ID_TITLE);
    }

    return FTRUE;
}

void MENU_ScanningProgress (SYS_EVT_SCAN_S* stat)
{
    MENU_SCANNING_INST_S* inst = menuScanningDat;

    if ( inst->pnlHdl == NULL ) return;
    inst->scanStat = *stat; /* store new status */

    switch ( stat->status )
    {
        case SCAN_STAT_ABORTED:
        case SCAN_STAT_COMPLETED:
            (void)MENU_ScanningStopScan(inst);
            (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SCANNING_ID_TITLE);
            break;

        #ifdef APPL_FESAT_ENABLED

        case SCAN_STAT_SCAN_HOR:
            break;

        case SCAN_STAT_SCAN_VER:
            break;

        #endif /* APPL_FESAT_ENABLED */

        case SCAN_STAT_TPSCAN:
            switch ( stat->mode )
            {
                case SCAN_MODE_SINGLE:
                    break;

                case SCAN_MODE_CURRENTTS:
                    break;

                #ifdef APPL_BLINDSCAN_ENABLED
                case SCAN_MODE_BLIND:
                #endif
                case SCAN_MODE_LIST:
                    /* display next transponder/channel to be scanned */
                    inst->cfg->tpLstPos = stat->tpNext;
                    break;

                default:
                    break;
            }

            (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SCANNING_ID_TITLE);
            break;

        default:
            break;
    }

    /* update progress bar */
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_SCANNING_ID_SCANSTATUS);
}

void MENU_ScanningServAdded (SYS_EVT_SERVICE_S* pServ)
{
    MENU_SCANNING_INST_S* inst = menuScanningDat;
    SLIST_ITEM_S          newServ;
    SYS_SERVLOC_S         newLoc;
    FGS_LNAVI_PARAMS_S    naviParams;

    if ( inst->pnlHdl == NULL ) return;

    memset(&newServ, 0, sizeof(newServ));

    newLoc.listId = SLIST_NONE;
    newLoc.index  = -1;

    switch ( pServ->addStat )
    {
        case FAPI_OK:
            inst->listChg = FTRUE;  /* set marker for service list storage */

            /* update number of services in OSD */
            (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SCANNING_ID_NEWSERV);
            break;

        case SMARTGO_SYS_ERR_REPLACED_IN_LIST:
            inst->listChg = FTRUE;  /* set marker for service list storage */
            break;

        case SMARTGO_SYS_ERR_ALREADY_IN_LIST:
            break;

        case SMARTGO_SYS_ERR_CAPACITY:
            // to do: show message indicating exceeded list capacity!
            return;

        default:
            return;
    }

    /* ----- update scan status window ------------------------------------- */

    if ( SLIST_EntryGet(&(pServ->servLoc), &newServ) == FAPI_OK )
    {
        newLoc = pServ->servLoc;
    }

    if ( newLoc.listId == SLIST_TV )
    {
        if ( pServ->addStat == SMARTGO_SYS_ERR_ALREADY_IN_LIST)
        {
            inst->tvAlreadyInList++;
        }
        else if ( inst->tvServFound < SLIST_MAX_TV )
        {
            inst->tvList[inst->tvServFound++] = SLIST_GetServiceIndex(&newLoc);

            if ( FGS_LNAVI_GetParams (inst->tvListHdl, &naviParams) == FAPI_OK )
            {
                naviParams.numItems = inst->tvServFound;
                naviParams.index    = naviParams.numItems - 1;

                (void)FGS_LNAVI_SetParams(inst->tvListHdl, &naviParams);
            }
        }

        (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SCANNING_ID_NEWSERV);
    }
    else if ( newLoc.listId == SLIST_RADIO )
    {
        if ( pServ->addStat == SMARTGO_SYS_ERR_ALREADY_IN_LIST )
        {
            inst->radioAlreadyInList++;
        }
        else if ( inst->radioServFound < SLIST_MAX_RADIO )
        {
            inst->radioList[inst->radioServFound++] = SLIST_GetServiceIndex
                                                      (&newLoc);

            if ( FGS_LNAVI_GetParams (inst->radioListHdl, &naviParams) == FAPI_OK )
            {
                naviParams.numItems = inst->radioServFound;
                naviParams.index    = naviParams.numItems - 1;

                (void)FGS_LNAVI_SetParams(inst->radioListHdl, &naviParams);
            }
        }

        (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SCANNING_ID_NEWSERV);
    }
}

void MENU_ScanningCarrierAdded (SYS_EVT_CARRIERADD_S* chanadd)
{
    MENU_SCANNING_INST_S* inst = menuScanningDat;

    if ( inst->pnlHdl == NULL ) return;

    switch ( chanadd->status )
    {
        case FAPI_OK:
            inst->chanFound++;    /* update number of found TP */
            inst->tpChg = FTRUE;  /* set marker for TP list storage */

            /* update number of carriers in OSD */
            (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_SCANNING_ID_NEWTP);
            break;

        case SMARTGO_SYS_ERR_REPLACED_IN_LIST:
            inst->tpChg = FTRUE;  /* set marker for TP list storage */
            break;
        default:
            break;
    }
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_ScanningOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_SCANNING_INST_S*)h)->pnlHdl != NULL);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_MENU,MENU_ScanningExitKeys,h,FTRUE,FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK,MENU_ScanningExitKeys,h,FTRUE,FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RED, MENU_ScanningKeys,    h,FTRUE,FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BLUE,MENU_ScanningKeys,    h,FTRUE,FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_BLUE);
    }
}

/*!
*******************************************************************************
**
** \brief  Leave the Scanning Menu.
**
** More detailed description
**
** \param h        Address of menu handle.
** \param force    .
**
** \return FTRUE
******************************************************************************/
FSTATIC fbool_t MENU_ScanningLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_SCANNING_INST_S* inst = (MENU_SCANNING_INST_S*)h;

    if ( inst->currFe >= 0 ) /* scan currently active? */
    {
        if ( force )
        {
            (void)AV_ScanStop(inst->currFe);
            inst->currFe = -1;
        }
        else
        {
            return FFALSE; /* block closing of menu */
        }
    }

    if ( (inst->listChg) || (inst->tpChg) ) /* data changed? */
    {
        if ( force )
        {
            /* Restore data */
            MENU_ScanningRestoreData (inst);
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1,"Exit from scan menu blocked due to data not saved!\n");
            return FFALSE;
        }
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_SCANNING);

    inst->pnlHdl = NULL;
    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  Handles exit keys for this menu.
**
** \param h        Address of menu handle.
******************************************************************************/
FSTATIC fbool_t MENU_ScanningExitKeys (FAPI_SYS_HandleT h)
{
    MENU_SCANNING_INST_S* inst        = (MENU_SCANNING_INST_S*)h;
    MENU_CONFDIALOG_S     confPars;

    memset(&confPars,  0, sizeof(confPars));
    /* do not exit if a scan is running */
    if ( (inst->scanStat.status != SCAN_STAT_ABORTED) &&
         (inst->scanStat.status != SCAN_STAT_COMPLETED) )
    {
        return FFALSE;
    }

    if ( inst->listChg || inst->tpChg )
    {
        confPars.numDescLines = 1;
        confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_SCANNING);
        confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_SAVEOK);
        //confPars.declineStr   = OSD_GetStr(STR_CONFDIALOG_SAVEABORT);
        confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_SAVEDISCARD);
        confPars.onConfirm    = MENU_ScanningExitAndSaveHdl;
        confPars.onDecline    = NULL;
        confPars.onDiscard    = MENU_ScanningDiscardChangesHdl;
        confPars.arg          = inst;

        return MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
     }
     else
     {
         (void)MENUSTACK_Back(NULL);
         return FTRUE;
     }
}

/*!
*******************************************************************************
**
** \brief  Handles exit and save changes.
**
** \param arg     Pointer on argument.
******************************************************************************/
FSTATIC void MENU_ScanningExitAndSaveHdl (void* arg)
{
    MENU_SCANNING_INST_S*  inst = (MENU_SCANNING_INST_S*)arg;

    MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

    if ( inst->listChg ) (void)SLIST_Save (FFALSE);
    if ( inst->tpChg )   (void)TPLIST_Save(inst->cfg->system, FFALSE);

    (void)MAIN_MsgBoxHide(&(appDat->msgBox));

    inst->listChg = FFALSE;
    inst->tpChg   = FFALSE;

    (void)MENUSTACK_Back(NULL);
}

/*!
*******************************************************************************
**
** \brief  Handles discard changes.
**
** \param arg     Pointer on argument.
******************************************************************************/
FSTATIC void MENU_ScanningDiscardChangesHdl (void* arg)
{
    MENU_SCANNING_INST_S* inst = (MENU_SCANNING_INST_S*)arg;

    /* Overwrite changes with stored data from Flash */
    MENU_ScanningRestoreData (inst);

    (void)MENUSTACK_Back(NULL);
}

/*!
*******************************************************************************
**
** \brief  Restore old data, if changes were rejected.
**
** \param inst     Pointer on global menu parameters.
******************************************************************************/
FSTATIC void MENU_ScanningRestoreData (MENU_SCANNING_INST_S* inst)
{
    if ( inst->listChg ) (void)SLIST_LoadLists();
    if ( inst->tpChg )   (void)TPLIST_Load(inst->cfg->system);

    inst->listChg = FFALSE;
    inst->tpChg   = FFALSE;
}

/*!
*******************************************************************************
**
** \brief  Set the descriptions for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_ScanningGetDescrItem (FAPI_SYS_HandleT h,
                                        int32_t id,
                                        uint32_t index,
                                        FGS_ITEMSTATE_E state,
                                        FGS_ITEM_S* item)
{
    uint32_t               strIdx;

    item->item.str.font       = MAIN_FONT;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->type                = FGS_ITEM_TYPE_STRING;
    item->item.str.lineHeight = 0;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS:
            item->item.str.col = appDat->txtAttr.colNonFocus;
            break;
        case FGS_FOCUS:
            item->item.str.col = appDat->txtAttr.colFocus;
            break;
        case FGS_DISABLED:
            item->item.str.col = appDat->txtAttr.colDisabled;
            break;
    }

    switch ( id )
    {
        case MENU_SCANNING_ID_TITLE:
            strIdx = STR_TITLE_SCANNING;
            item->item.str.col = OSD_COL_TITLE_TEXT;
            item->item.str.font = TITLE_FONT;
            break;

        case MENU_SCANNING_ID_SCANSTATUS    :
            strIdx = STR_ENTRY_SCANNING_SCANSTATUS;
            break;

        case MENU_SCANNING_ID_NEWTP         :
            strIdx = STR_ENTRY_SCANNING_NEWTP;
            break;

        case MENU_SCANNING_ID_NEWSERV       :
            strIdx = STR_ENTRY_SCANNING_NEWSERV;
            break;

        case MENU_SCANNING_ID_TVLISTTITLE   :
            strIdx = STR_ENTRY_SCANNING_TVLIST;
            break;

        case MENU_SCANNING_ID_RADIOLISTTITLE:
            strIdx = STR_ENTRY_SCANNING_RADIOLIST;
            item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            break;

        default:  return;
    }

    item->item.str.chr        = OSD_GetStr(strIdx);
}

/*!
*******************************************************************************
**
** \brief  Set the value (e.g. text, number, icon, ...) for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_ScanningGetItem (FAPI_SYS_HandleT h,
                                   int32_t id,
                                   uint32_t index,
                                   FGS_ITEMSTATE_E state,
                                   FGS_ITEM_S* item)
{
    MENU_SCANNING_INST_S* inst = (MENU_SCANNING_INST_S*)h;
    TPLIST_ITEM_S         tpDat;
    uint32_t                     bufPos = 0;

    #ifdef APPL_FESAT_ENABLED
    CFGSAT_SAT_S          satDat;
    #endif /* APPL_FESAT_ENABLED */

    inst->tmpListItem.nameCharSet = SYS_CHARSET_UNDEFINED;
    inst->tmpListItem.name[0]     = 0;

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS:
            item->item.str.col = appDat->txtAttr.colNonFocus;
            break;
        case FGS_FOCUS:
            item->item.str.col = appDat->txtAttr.colFocus;
            break;
        case FGS_DISABLED:
            item->item.str.col = appDat->txtAttr.colDisabled;
            break;
    }

    switch ( id )
    {
        case MENU_SCANNING_ID_TITLE:
            item->item.str.col  = OSD_COL_TITLE_TEXT;
            item->item.str.font = TITLE_FONT;

            switch ( inst->scanStat.status )
            {
                case SCAN_STAT_COMPLETED:
                    item->item.str.chr = OSD_GetStr(STR_TITLE_SCAN_FINISHED);
                    break;

                case SCAN_STAT_ABORTED:
                    item->item.str.chr = OSD_GetStr(STR_TITLE_SCAN_ABORTED);
                    break;

                case SCAN_STAT_TPSCAN:

                    (void)STR_BufClear(NULL);

                    if ( TPLIST_GetCarrierData (&(inst->cfg->network),
                              inst->cfg->tpLstPos, &tpDat) == FAPI_OK )
                    {
                        switch ( inst->cfg->system )
                        {
                            #ifdef APPL_FESAT_ENABLED
                            case FE_SYSTEM_TYPE_SATELLITE:
                                if ( CFGSAT_GetDataByIndex (inst->cfg->network.satIdx,
                                                            &satDat) == FAPI_OK )
                                {
                                    bufPos += STR_BufAppend (NULL, bufPos,
                                                STR_Build(satDat.nameCharSet,
                                                          satDat.name),
                                                MAIN_LINEBUFCHARSET);

                                    sprintf (appDat->asciiBuf,", TP%i, %ld ",
                                             inst->scanStat.tpNext + 1,
                                             (tpDat.pars.sat.frequency / 1000));

                                    bufPos += STR_BufAppend (NULL,
                                                bufPos, STR_IntASCIIStr(),
                                                MAIN_LINEBUFCHARSET);

                                    bufPos += STR_BufAppend (NULL, bufPos,
                                                OSD_GetStr(STR_UNIT_MHZ),
                                                MAIN_LINEBUFCHARSET);

                                    if ( tpDat.pars.sat.pol == FE_POL_HOR)
                                    {
                                        bufPos += STR_BufAppend (NULL, bufPos,
                                                    STR_GetASCII(", H"),
                                                    MAIN_LINEBUFCHARSET);
                                    }
                                    else
                                    {
                                        bufPos += STR_BufAppend (NULL, bufPos,
                                                    STR_GetASCII(", V"),
                                                    MAIN_LINEBUFCHARSET);
                                    }
                                }
                                break;

                            #endif /* APPL_FESAT_ENABLED */

                            #ifdef APPL_FETERR_ENABLED

                            case FE_SYSTEM_TYPE_TERRESTRIAL:
                                bufPos += STR_BufAppend (NULL, bufPos,
                                            STR_Build (tpDat.nameCharSet,
                                                       tpDat.name),
                                            MAIN_LINEBUFCHARSET);

                                sprintf (appDat->asciiBuf, ": %li ",
                                         tpDat.pars.terr.frequency / 1000000);

                                bufPos += STR_BufAppend (NULL, bufPos,
                                            STR_IntASCIIStr(),
                                            MAIN_LINEBUFCHARSET);

                                bufPos += STR_BufAppend (NULL, bufPos,
                                            OSD_GetStr(STR_UNIT_MHZ),
                                            MAIN_LINEBUFCHARSET);
                                break;

                            #endif /* APPL_FETERR_ENABLED */

                            #ifdef APPL_FECABLE_ENABLED
                            case FE_SYSTEM_TYPE_CABLE:
                                /* FALLTHROUGH */
                            #endif /* APPL_FECABLE_ENABLED */

                            default:
                                break;
                        }
                    }

                    item->item.str.chr  = STR_IntLineStr();
                    break;

                default:
                    item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                    break;
            }
            break;

        case MENU_SCANNING_ID_NEWTP :
            sprintf(appDat->asciiBuf, "%4d", inst->chanFound);
            item->item.str.chr = STR_IntASCIIStr();
            break;

        case MENU_SCANNING_ID_NEWSERV :
            bufPos += STR_BufAppend (NULL, bufPos,
                    OSD_GetStr(STR_VALUE_SCANNING_NEWSERV_TV),
                                     MAIN_LINEBUFCHARSET);

            sprintf (appDat->asciiBuf, ": %d/%d   ", inst->tvServFound,
                                                     inst->tvAlreadyInList);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                    OSD_GetStr(STR_VALUE_SCANNING_NEWSERV_RADIO),
                                     MAIN_LINEBUFCHARSET);

            sprintf (appDat->asciiBuf, ": %d/%d", inst->radioServFound,
                                                  inst->radioAlreadyInList);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);

            item->item.str.chr = STR_IntLineStr();
            break;

        default:
            item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            break;
    }
}

/*!
*******************************************************************************
**
** \brief  Set the value (e.g. text, number, icon, ...) for each listitem.
**
** \param h        Address of menu handle.
** \param id       Listitem Id.
** \param index    Current index of listitem.
** \param state    Status of listitem.
** \param item     Pointer on listitem.
******************************************************************************/
FSTATIC void MENU_ScanningGetListItem (FAPI_SYS_HandleT h,
                                       int32_t id,
                                       uint32_t index,
                                       FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item)
{
    MENU_SCANNING_INST_S* inst = (MENU_SCANNING_INST_S*)h;

    inst->tmpListItem.nameCharSet = SYS_CHARSET_UNDEFINED;
    inst->tmpListItem.name[0]     = 0;

    item[0].type     = FGS_ITEM_TYPE_ICON;
    item[0].align    = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[0].item.icon.drawMode   = FGS_ICONMODE_OVRLAY;
    item[0].item.icon.altBkgrCol = OSD_GetSchemeColor(OSD_SCHEMECOL_MAIN_BKGR);
    item[0].item.icon.bkgrCol    = OSD_COL_BLANK;

    item[1].type                = FGS_ITEM_TYPE_STRING;
    item[1].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[1].item.str.font       = MAIN_FONT;
    item[1].item.str.lineHeight = 0;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item[1].item.str.col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    item[1].item.str.col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: item[1].item.str.col = appDat->txtAttr.colDisabled; break;
    }

    switch ( id )
    {
        case MENU_SCANNING_ID_TVLIST :
            (void)SLIST_GetServByIndex (inst->tvList[index], &(inst->tmpListItem));

            item[1].item.str.chr = STR_Build(inst->tmpListItem.nameCharSet,
                                           inst->tmpListItem.name);
            break;

        case MENU_SCANNING_ID_RADIOLIST :
            (void)SLIST_GetServByIndex (inst->radioList[index], &(inst->tmpListItem));

            item[1].item.str.chr = STR_Build(inst->tmpListItem.nameCharSet,
                                           inst->tmpListItem.name);
            break;

        default:
            item[1].item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            break;
    }

    if( !SLIST_FlagIsSet(&inst->tmpListItem, SLIST_FLAG_ALLFREE) )
    {
        item[0].item.icon.addr = OSD_IconGet(ICON_CRYPTED);
    }
    else
    {
        item[0].item.icon.addr = NULL;
    }
}

/*!
*******************************************************************************
**
** \brief  Handles the Keys adapted by this menu.
**
** \param h        Address of menu handle.
**
** \return FTRUE
**         FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_ScanningKeys (FAPI_SYS_HandleT h)
{
    MENU_SCANNING_INST_S* inst    = (MENU_SCANNING_INST_S*)h;
    fbool_t                 keyUsed = FTRUE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_RED:
            (void)MENU_ScanningStopScan (inst);
            break;

        case FGS_KEY_BLUE: /* toggle lists */
            if ( inst->currFe >= 0 ) /* scan currently active? */
            {
                keyUsed = FFALSE;
            }
            else
            {
                if ( FGS_PANEL_GetMemberFocusID (inst->pnlHdl) ==
                                               MENU_SCANNING_ID_TVLIST )
                {
                    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl,
                                              MENU_SCANNING_ID_RADIOLIST);
                }
                else
                {
                    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl,
                                              MENU_SCANNING_ID_TVLIST);
                }
            }
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

/*!
*******************************************************************************
**
** \brief   Builds the menu pannel and its members.
**
** \param   inst    Menu handle pointer.
**
** \return
**          - #FTRUE
**          - #FFALSE
******************************************************************************/
FSTATIC fbool_t MENU_ScanningBuild(MENU_SCANNING_INST_S* inst)
{
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_LIST_OPEN_S       listPars;
    FAPI_SYS_HandleT      itemHdl;
    FGS_LNAVI_OPEN_S      naviPars;
    FGS_BOX_OPEN_S        boxPars;
    FGS_ITEM_TXT_OPEN_S   txtPars;
    FGS_POS_S             itemPos;
    FGS_ITEM_S            item;
    FGS_POS_S             valuePos;
    FGS_POS_S             descrPos;
    int32_t               retVal;
    uint16_t              stackedFrameWidth;
    uint16_t              innerHeight;
    uint32_t              numLines = 10;
    uint8_t               numHelpLines=2;
    FGS_PANEL_MEMBER_S    member;
    uint16_t              idCnt;

    memset(&naviPars, 0, sizeof(naviPars));
    memset(&listPars, 0, sizeof(listPars));
    memset(&boxPars,  0, sizeof(boxPars));
    memset(&txtPars,  0, sizeof(txtPars));
    memset(&item,     0, sizeof(item));
    memset(&itemPos,  0, sizeof(itemPos));
    memset(&descrPos, 0, sizeof(descrPos));
    memset(&valuePos, 0, sizeof(valuePos));
    memset(&member,   0, sizeof(member));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        innerHeight = (numLines       * SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                    + ((numLines - 1) * SCALER_GetCurrVal(LIST_LINE_PADDING));

        MENU_CommonSetParamsTitleWBotHelp (&pnlPars,
                                           SCALER_GetCurrVal(SCANNING_WIDTH),
                                           innerHeight,
                                           numHelpLines,
                                           &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_ScanningOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SCANNING, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        listPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   stackedFrameWidth;
        listPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(TITLE_TB_PADDING) +
                                   stackedFrameWidth;
        listPars.pos.width       = SCALER_GetCurrVal(SCANNING_WIDTH);
        listPars.pos.height      = numLines *
                                   (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING))+
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                   SCALER_GetCurrVal(TITLE_TB_PADDING) +
                                   SCALER_GetCurrVal(TITLE_LINE_HEIGHT);

        listPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        listPars.topDownOriented = FTRUE;
        listPars.wraparound      = FTRUE;
        listPars.getDescrItem    = MENU_ScanningGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_ScanningGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.pInst           = &(appDat->listInst);

        retVal = FGS_LIST_Open (&listPars, MENU_SCANNING_ID_LIST,
                                inst->pnlHdl, &(inst->listHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        itemPos.x          = 0;
        itemPos.y          = 0;
        itemPos.width      = listPars.pos.width;
        itemPos.height     = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        itemPos.alignment  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* position of list item descriptors in list entry */
        descrPos.x         = SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        descrPos.y         = 0;
        descrPos.width     = (uint16_t)(itemPos.width -
                                   SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                   SCALER_GetCurrVal(MAIN_RIGHT_PADDING)) / 2;
        descrPos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* position of list item values in list entry */
        valuePos.x         = descrPos.x + descrPos.width;
        valuePos.y         = descrPos.y;
        valuePos.width     = listPars.pos.width - descrPos.width -
                             SCALER_GetCurrVal(MAIN_LEFT_PADDING)  - SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        valuePos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        valuePos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----- menu lines ------------------------------------------------ */

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_TITLE;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;

        /* --- Title  ------------------------------------------------- */

        retVal = FGS_ITEM_TXT_Open (&txtPars,
                                    MENU_SCANNING_ID_TITLE, inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y +=  SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                      SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                      SCALER_GetCurrVal(TITLE_TB_PADDING) +
                      SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        /* --- Scan Status ------------------------------------------------- */
        descrPos.width     = ((uint16_t)((itemPos.width -
                                     SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                     SCALER_GetCurrVal(MAIN_RIGHT_PADDING))  * 1 )/2);
        valuePos.x         = descrPos.x + descrPos.width;
        valuePos.width     = listPars.pos.width - descrPos.width -
                             SCALER_GetCurrVal(MAIN_LEFT_PADDING)  - SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars,
                                    MENU_SCANNING_ID_SCANSTATUS,
                                    inst->listHdl,
                                    &itemHdl);
        if ( retVal != FAPI_OK ) break;
        (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                       MENU_SCANNING_ID_SCANSTATUS,
                                       FFALSE);

        /* ----- Scan Status Bar ------------------------------------------- */

        inst->itemPos.x         = listPars.pos.x +
                                  descrPos.width +
                                  SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                  4;
        inst->itemPos.y         = listPars.pos.y + itemPos.y + 2;
        inst->itemPos.width     = valuePos.width;
        inst->itemPos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING);
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemPos);
        member.id            = MENU_SCANNING_ID_SCANSTATUS;
        member.isHidden      = FFALSE;
        member.show          = MENU_ScanningProgressPaint;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl,
                                     &member,
                                     &(inst->pItem));
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- SCANNING NEWTP ---------------------------------------------- */

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars,
                                    MENU_SCANNING_ID_NEWTP,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                       MENU_SCANNING_ID_NEWTP,
                                       FFALSE);
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- SCANNING NEWSERV -------------------------------------------- */

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars,
                                    MENU_SCANNING_ID_NEWSERV,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                       MENU_SCANNING_ID_NEWSERV, FFALSE);
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- create TV Service list ------------------------------------- */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_ENTRY_SCANNING_TVLIST);

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                      stackedFrameWidth;
        boxPars.pos.y               = listPars.pos.y + itemPos.y;
        boxPars.pos.width           = descrPos.width;
        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_SCANNING_ID_TVLISTTITLE,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- create Radio Service list ---------------------------------- */
        item.align                  = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_TOP;
        item.item.str.chr           = OSD_GetStr(STR_ENTRY_SCANNING_RADIOLIST);
        boxPars.pos.x              += descrPos.width;
        boxPars.pos.width           = valuePos.width;
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars,MENU_SCANNING_ID_RADIOLISTTITLE,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- List Borders ---------------------------------------------- */

        inst->itemBorderPos.x         = listPars.pos.x +
                                        itemPos.x -
                                        SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                        SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        inst->itemBorderPos.y         = listPars.pos.y +
                                        itemPos.y -
                                        SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        inst->itemBorderPos.width     = descrPos.width -
                                        SCALER_GetCurrVal(MAIN_RIGHT_PADDING) +
                                        2*SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        inst->itemBorderPos.height    = (numLines-4) *
                                        (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING)) +
                                        2*SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        inst->itemBorderPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemBorderPos);
        member.id            = MENU_SCANNING_ID_REST;//MENU_SCAN_ID_REST;
        member.isHidden      = FFALSE;
        member.show          = MENU_ScanningPaintRest;

         retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                      &(inst->pItemBorder));

        if ( retVal != FAPI_OK ) break;

        naviPars.pos.x           = listPars.pos.x +
                                   itemPos.x +
                                   SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        naviPars.pos.y           = listPars.pos.y + itemPos.y;
        naviPars.pos.width       = descrPos.width -SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        naviPars.pos.height      = (numLines-4) *
                                   (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        naviPars.bkgrType        = OSD_NAVIBKGR_NORMAL;
        naviPars.type            = FGS_LNAVI_TYPE_SCROLL;
        naviPars.rowHeight       = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.scrollbarWidth  = SCALER_GetCurrVal(SCROLLBAR_WIDTH);
        naviPars.numVisLines     = numLines  - 4;
        naviPars.numColumns      = 2;
        naviPars.params.numItems = 0;
        naviPars.params.index    = 0;
        naviPars.params.posInVis = 0;

        naviPars.colStart[0]     = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[0]     = SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        naviPars.colStart[1]     = naviPars.colStart[0] + naviPars.colWidth[0];
        naviPars.colWidth[1]     = (SCALER_GetCurrVal(SCANNING_WIDTH)>>1)-
                                   SCALER_GetCurrVal(SCROLLBAR_WIDTH)-
                                   SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                   naviPars.colStart[1];

        naviPars.topDownOrient   = FTRUE;
        naviPars.wraparound      = FTRUE;
        naviPars.getItem         = MENU_ScanningGetListItem;
        naviPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        naviPars.pInst           = &(appDat->lnaviInst);

        retVal = FGS_LNAVI_Open (&naviPars, MENU_SCANNING_ID_TVLIST,
                                 inst->pnlHdl, &(inst->tvListHdl));
        if ( retVal != FAPI_OK ) break;

        naviPars.pos.x          += (SCALER_GetCurrVal(SCANNING_WIDTH)>>1);
        naviPars.pInst           = &(inst->radioLnaviInst);

        retVal = FGS_LNAVI_Open (&naviPars, MENU_SCANNING_ID_RADIOLIST,
                                 inst->pnlHdl, &(inst->radioListHdl));
        if ( retVal != FAPI_OK ) break;

        /* -----Bottom help lines ------------------------------------------ */

        idCnt = MENU_SCANNING_ID_HELPBOTTOM;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNREDSMALL);

        boxPars.pos.x               = stackedFrameWidth +
                                      SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - (numHelpLines-1)*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = SCALER_GetCurrVal(HELP_LINE_WIDTH);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SCANNING_STOP);

        boxPars.pos.x              += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,
                                                      -1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNBLUESMALL);

        boxPars.pos.x               = ((pnlPars.pos.width/2)+SCALER_GetCurrVal(HELP_BOT_XOFF));
        boxPars.pos.width           = SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_SCANNING_ID_HELP_LISTCHANGE_BUTTON, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SCANNING_TOGGLETVRA);

        boxPars.pos.x              += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,
                                                      -1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_SCANNING_ID_HELP_LISTCHANGE_TEXT, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;


        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                      MENU_SCANNING_ID_HELP_LISTCHANGE_BUTTON,
                                      FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                      MENU_SCANNING_ID_HELP_LISTCHANGE_TEXT,
                                      FTRUE);
        /* -- 2. Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        item.item.str.chr           = OSD_GetStr(STR_KEY_MENU);
        boxPars.pos.x               = stackedFrameWidth +
                                      SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,
                                                      -1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_MENU_BACK);
        boxPars.pos.x              += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,
                                                      -1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  Builds the scanning progressbar.
**
** \param h        Address of menu handle.
**
** \return FAPI_OK
******************************************************************************/
int32_t MENU_ScanningProgressPaint (FAPI_SYS_HandleT h)
{
    MENU_SCANNING_INST_S* inst = (MENU_SCANNING_INST_S*)h;

    OSD_DrawStatusBarHorizontal(
                inst->pItem->abspos.x,
                inst->pItem->abspos.y,
                inst->itemPos.width,
                SCALER_GetCurrVal(LIST_LINE_HEIGHT) - (2 * SCALER_GetCurrVal(INNER_FRAME_WIDTH)),
                inst->scanStat.progCurr,
                inst->scanStat.progMax,
                OSD_COL_YELLOW,
                OSD_GetSchemeColor(OSD_SCHEMECOL_PROGRESSBAR_INACT),
                OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME));

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Builds the borders for the TV and radio lists.
**
** \param h        Address of menu handle.
**
** \return FAPI_OK
******************************************************************************/
int32_t MENU_ScanningPaintRest(FAPI_SYS_HandleT h)
{
    MENU_SCANNING_INST_S* inst = (MENU_SCANNING_INST_S*)h;
    uint32_t    x1,x2,y1,y2;

    x1 = inst->pItemBorder->abspos.x;
    y1 = inst->pItemBorder->abspos.y;
    x2 = x1 + inst->itemBorderPos.width;
    y2 = y1 + inst->itemBorderPos.height-1;

    /* Border TV list */
    OSD_DrawFilledRectangle(x1,y1,x2,y2,OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME));

    x1 += (SCALER_GetCurrVal(SCANNING_WIDTH)>>1);
    x2 = x1 + inst->itemBorderPos.width;

    /* Border radio List */
    OSD_DrawFilledRectangle(x1,y1,x2,y2,OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME));

    return FAPI_OK;
}

FSTATIC fbool_t MENU_ScanningStopScan (MENU_SCANNING_INST_S* inst)
{
    FGS_LNAVI_PARAMS_S naviParams;

    if ( inst->currFe >= 0 )
    {
        /* abort scan */
        (void)AV_ScanStop(inst->currFe);
        inst->currFe = -1;

        /* set focus on TV list */
        (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_SCANNING_ID_TVLIST);

        /* set focus on last list item */
        if ( FGS_LNAVI_GetParams (inst->tvListHdl, &naviParams) == FAPI_OK )
        {
            if ( naviParams.numItems > 0 )
            {
                naviParams.index = naviParams.numItems - 1;

                (void)FGS_LNAVI_SetParams(inst->tvListHdl, &naviParams);
                (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                              MENU_SCANNING_ID_HELP_LISTCHANGE_BUTTON,
                                              FFALSE);
                (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                              MENU_SCANNING_ID_HELP_LISTCHANGE_TEXT,
                                              FFALSE);
            }
        }
    }

    return FTRUE;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
