/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_servlist.c
**
**              $Id: app_menu_servlist.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD Service List menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: loss of precision
** - Info    715: symbol 'anySymb' not referenced
** - Info    737: loss of sign in promotion from int to unsigned int
** - Info    778: Constant expression evaluates to 0 in operation.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define MENU_SERVLIST_NUM_GLOB   2 /* TV + Radio */
#define MENU_SERVLIST_NUM_LISTS  (MENU_SERVLIST_NUM_GLOB + SLIST_USR_NUMLISTS)

#define MENU_SERVLIST_NUM_LINES  7

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum
{
    MENU_SERVLIST_ID_TVRADIOTITLE = 0,
    MENU_SERVLIST_ID_FAVTITLE,
    MENU_SERVLIST_ID_TVRADIO,
    MENU_SERVLIST_ID_FAV,
    MENU_SERVLIST_ID_GOTONEXT,
    MENU_SERVLIST_ID_TITLE,
    MENU_SERVLIST_ID_LIST,
    MENU_SERVLIST_ID_REST,
    MENU_SERVLIST_ID_HELPBOTTOM
};

typedef struct
{
    SLIST_SORTMODE_E            sortMode;
    fbool_t                     ascending;
    uint16_t                    size;
    int16_t*                    entry;

} MENU_SERVLIST_LIST_S;

typedef struct
{
    FAPI_SYS_HandleT            pnlHdl;
    FAPI_SYS_HandleT            listHdl;

    fbool_t                     guidedMode;

    FAPI_SYS_HandleT            tvRadioListHdl;
    FGS_LNAVI_INST_S            tvRadioLnaviInst;
    FAPI_SYS_HandleT            tvRadioTitleItemHdl;

    FAPI_SYS_HandleT            favListHdl;
    FGS_LNAVI_INST_S            favLnaviInst;
    FAPI_SYS_HandleT            favTitleHdl;

    FGS_PANEL_MEMBER_S*         pItemBorder;
    FGS_POS_S                   itemBorderPos;
    uint16_t                    serviceBoxWidth;

    SLIST_TYPE                  currGlobList;
    SLIST_TYPE                  currFavList;
    fbool_t                     listChg;
    fbool_t                     exitFlag;

    #ifdef APPL_LCN_ENABLED
    fbool_t                     showLcn;
    #endif /* APPL_LCN_ENABLED */

    MENU_SERVLIST_LIST_S        list[MENU_SERVLIST_NUM_LISTS];
    SYS_STRBUF_S                tmpBuf;

} MENU_SERVLIST_INST_S;

FSTATIC MENU_SERVLIST_INST_S* menuServListDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_ServListLeave             (FAPI_SYS_HandleT h,
                                                fbool_t force);
FSTATIC fbool_t MENU_ServListExitKeys          (FAPI_SYS_HandleT h);
FSTATIC void    MENU_ServListExitAndSaveHdl    (void* arg);
FSTATIC void    MENU_ServListDiscardChangesHdl (void* arg);
FSTATIC void    MENU_ServListRestoreData       (MENU_SERVLIST_INST_S* inst);
FSTATIC fbool_t MENU_ServListOnSelect          (FAPI_SYS_HandleT h,
                                                int32_t itemId);
FSTATIC void    MENU_ServListOnFocusSet        (FAPI_SYS_HandleT h,
                                                fbool_t hasFocus);
FSTATIC fbool_t MENU_ServListKeys              (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_ServListBuild             (MENU_SERVLIST_INST_S* inst);
FSTATIC int32_t MENU_ServListPaintRest         (FAPI_SYS_HandleT h);
FSTATIC void    MENU_ServListSort              (MENU_SERVLIST_INST_S* inst,
                                                uint16_t idx);
FSTATIC void    MENU_ServListSortAll           (MENU_SERVLIST_INST_S* inst);
FSTATIC MENU_SERVLIST_LIST_S* MENU_ServListGetList (MENU_SERVLIST_INST_S* inst,
                                                SLIST_TYPE listId);
FSTATIC void    MENU_ServListGetItem           (FAPI_SYS_HandleT h, int32_t id,
                                                uint32_t index,
                                                FGS_ITEMSTATE_E state,
                                                FGS_ITEM_S* item);
FSTATIC void    MENU_ServListGetTitleItem      (FAPI_SYS_HandleT h, int32_t id,
                                                uint32_t index,
                                                FGS_ITEMSTATE_E state,
                                                FGS_ITEM_S* item);
FSTATIC void    MENU_ServListGetDescrItem      (FAPI_SYS_HandleT h, int32_t id,
                                                uint32_t index,
                                                FGS_ITEMSTATE_E state,
                                                FGS_ITEM_S* item);
FSTATIC void    MENU_ServListEditStringExit    (MENU_EDITSTRING_DATA_S*
                                                editData, int32_t id);

/*!
*******************************************************************************
**
** \brief  Initial function for the OSD Service list Menu
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_ServListInit (void)
{
    uint16_t                cnt;
    MENU_SERVLIST_LIST_S*   pList;
    MENU_SERVLIST_INST_S*   inst;
    int32_t                 retVal = FAPI_OK;

    if ( menuServListDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuServListDat = UTILS_Malloc(sizeof(MENU_SERVLIST_INST_S));
    if ( menuServListDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuServListDat, 0, sizeof(MENU_SERVLIST_INST_S));

    inst = menuServListDat;

    retVal = STR_BufOpen (&(inst->tmpBuf), MAIN_LINEBUFSIZE * 2, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    inst->guidedMode   = FFALSE;

    pList = menuServListDat->list;

    for ( cnt = 0; cnt < MENU_SERVLIST_NUM_LISTS; cnt++, pList++ )
    {
        pList->sortMode  = SLIST_SORT_BYINDEX;
        pList->ascending = FTRUE;

        switch ( cnt )
        {
            case 0:   pList->size = SLIST_MAX_TV;    break;
            case 1:   pList->size = SLIST_MAX_RADIO; break;
            default:  pList->size = SLIST_USR_SIZE;  break;
        }

        pList->entry = UTILS_Malloc (pList->size * sizeof(*(pList->entry)));
        if ( pList->entry == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function.
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_ServListEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_SERVLIST_INST_S* inst = menuServListDat;
    uint8_t               cnt;

    #ifdef APPL_LCN_ENABLED
    SLIST_PARAMS_S        slistParams;
    #endif /* APPL_LCN_ENABLED */

    if ( test == FTRUE ) return FTRUE;

    if ( h != NULL )
    {
        inst->guidedMode = ((MENU_GUIDED_PARAMS_S*)h)->guided;
    }

    inst->currGlobList       = SLIST_TV;
    inst->currFavList        = SLIST_OFFSET_USR;
    inst->listChg            = FFALSE;

    for ( cnt = 0; cnt < MENU_SERVLIST_NUM_LISTS; cnt++ )
    {
        inst->list[cnt].sortMode  = SLIST_SORT_BYINDEX;
        inst->list[cnt].ascending = FTRUE;
    }

    #ifdef APPL_LCN_ENABLED

    if ( SLIST_GetParams(&slistParams) == FAPI_OK )
    {
        inst->showLcn = slistParams.sortByLcn;
    }
    else
    {
        inst->showLcn = FFALSE;
    }

    #endif /* APPL_LCN_ENABLED */

    MENU_ServListSortAll(inst);

    if ( MENU_ServListBuild(inst) == FFALSE )
    {
        return FFALSE;
    }

    /* set list focus on first item */
    if ( inst->guidedMode )
    {
        (void)FGS_LIST_SetMemberFocus  (inst->listHdl,
                                        MENU_SERVLIST_ID_GOTONEXT);
        (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl,
                                        MENU_SERVLIST_ID_LIST);
    }
    else
    {
        (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_SERVLIST_ID_TVRADIO);
    }

    (void)MENUSTACK_FocusClaim ((int32_t)MENUID_SERVLIST);
    (void)FGS_PANEL_Show       (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_ServListLeave, (FAPI_SYS_HandleT)inst);
    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_ServListOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_SERVLIST_INST_S*)h)->pnlHdl != NULL);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_MENU,   MENU_ServListExitKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK,   MENU_ServListExitKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_EXIT,   MENU_ServListExitKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_NAVI,   MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_TV,     MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_REW,    MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_FF,     MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_OK,     MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_PLAY,   MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RED,    MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_GREEN,  MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_YELLOW, MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BLUE,   MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_APPL,   MENU_ServListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_TEXT,   MENU_ServListKeys, h, FTRUE, FFALSE);

        #ifdef APPL_PARENTAL_ENABLED
        FGS_SetAppKeyHandler(FGS_KEY_STOP,   MENU_ServListKeys, h, FTRUE, FFALSE);
        #endif /* APPL_PARENTAL_ENABLED */
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler(FGS_KEY_NAVI);
        FGS_ClearAppKeyHandler(FGS_KEY_TV);
        FGS_ClearAppKeyHandler(FGS_KEY_REW);
        FGS_ClearAppKeyHandler(FGS_KEY_FF);
        FGS_ClearAppKeyHandler(FGS_KEY_OK);
        FGS_ClearAppKeyHandler(FGS_KEY_PLAY);
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_GREEN);
        FGS_ClearAppKeyHandler(FGS_KEY_YELLOW);
        FGS_ClearAppKeyHandler(FGS_KEY_BLUE);
        FGS_ClearAppKeyHandler(FGS_KEY_APPL);
        FGS_ClearAppKeyHandler(FGS_KEY_TEXT);

        #ifdef APPL_PARENTAL_ENABLED
        FGS_ClearAppKeyHandler(FGS_KEY_STOP);
        #endif /* APPL_PARENTAL_ENABLED */
    }
}


FSTATIC fbool_t MENU_ServListLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_SERVLIST_INST_S* inst = (MENU_SERVLIST_INST_S*)h;

    /* Has data changed? */
    if ( inst->listChg == FTRUE )
    {
        if ( force == FTRUE )
        {
            MENU_ServListRestoreData(inst);
        }
        else
        {
            return FFALSE;
        }
    }

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_SERVLIST);

    inst->pnlHdl = NULL;

    return FTRUE;
}

FSTATIC fbool_t MENU_ServListExitKeys (FAPI_SYS_HandleT h)
{
    MENU_SERVLIST_INST_S*   inst    = (MENU_SERVLIST_INST_S*)h;
    MENU_CONFDIALOG_S       confPars;
    fbool_t                 keyUsed      = FFALSE;
    fbool_t                 openDialogue = FFALSE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
            /* FALLTHROUGHT */
        case FGS_KEY_BACK:
            if ( inst->listChg )
            {
                openDialogue   = FTRUE;
                inst->exitFlag = FFALSE;
            }
            else
            {
                if ( inst->guidedMode ) keyUsed = MENU_GuidedGotoPrevious();
                else                    keyUsed = MENUSTACK_Back(NULL);
            }
            break;

        case FGS_KEY_EXIT:
            if ( inst->listChg )
            {
                openDialogue   = FTRUE;
                inst->exitFlag = FTRUE;
            }
            else
            {
                (void)MAIN_RestartPresentation();

                keyUsed = MENUSTACK_Exit(NULL);
            }
            break;

        default:
            break;
    }

    if ( openDialogue )
    {
        memset(&confPars,  0, sizeof(confPars));

        confPars.numDescLines = 1;
        confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_SERVLIST);
        confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_SAVEOK);
        //confPars.declineStr   = OSD_GetStr(STR_CONFDIALOG_SAVEABORT);
        confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_SAVEDISCARD);
        confPars.onConfirm    = MENU_ServListExitAndSaveHdl;
        confPars.onDecline    = NULL;
        confPars.onDiscard    = MENU_ServListDiscardChangesHdl;
        confPars.arg          = inst;

        keyUsed = MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
    }

    return keyUsed;
}

FSTATIC void MENU_ServListExitAndSaveHdl (void* arg)
{
    MENU_SERVLIST_INST_S*  inst = (MENU_SERVLIST_INST_S*)arg;

    MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

    if ( inst->listChg )
    {
        (void)SLIST_Save (FFALSE);

        /* NOTE: Removal of services could lead to removal of timers: Save timer
                 list as well. */

        #ifdef APPL_RECTIMER_ENABLED
        (void)RECTIMER_Save (FFALSE);
        #endif /* APPL_RECTIMER_ENABLED */
    }

    (void)MAIN_MsgBoxHide(&(appDat->msgBox));

    inst->listChg = FFALSE;

    if ( inst->exitFlag )
    {
        (void)MAIN_RestartPresentation();
        (void)MENUSTACK_Exit(NULL);
    }
    else
    {
        if ( inst->guidedMode ) (void)MENU_GuidedGotoPrevious();
        else                    (void)MENUSTACK_Back(NULL);
    }
}

FSTATIC void MENU_ServListDiscardChangesHdl (void* arg)
{
    MENU_SERVLIST_INST_S* inst = (MENU_SERVLIST_INST_S*)arg;

    /* Overwrite changes with stored data from Flash */
    MENU_ServListRestoreData (inst);

    if ( inst->exitFlag )
    {
        (void)MAIN_RestartPresentation();
        (void)MENUSTACK_Exit(NULL);
    }
    else
    {
        if ( inst->guidedMode ) (void)MENU_GuidedGotoPrevious();
        else                    (void)MENUSTACK_Back(NULL);
    }
}

FSTATIC void MENU_ServListRestoreData (MENU_SERVLIST_INST_S* inst)
{
    if ( inst->listChg )
    {
        (void)SLIST_LoadLists();

        #ifdef APPL_RECTIMER_ENABLED
        (void)RECTIMER_Load();
        #endif /* APPL_RECTIMER_ENABLED */
    }

    inst->listChg = FFALSE;
}

FSTATIC fbool_t MENU_ServListOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    fbool_t retVal;

    switch ( itemId )
    {
        case MENU_SERVLIST_ID_GOTONEXT:
            retVal = MENU_GuidedGotoNext();
            break;

        default:
            retVal = FFALSE;
            break;
    }

    return retVal;
}


FSTATIC void MENU_ServListGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_SERVLIST_ID_GOTONEXT:
            strIdx = MENU_GuidedGetNextStr();
            break;

        default:
            return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_ServListGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                   FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_SERVLIST_INST_S*   inst = (MENU_SERVLIST_INST_S*)h;
    SLIST_ITEM_S            listItem;
    SYS_SERVLOC_S           servLoc;
    uint16_t                servNum;
    int32_t                 retVal;
    uint32_t                bufPos = 0;


    memset(&listItem, 0 , sizeof(listItem));
    memset(&servLoc,  0 , sizeof(servLoc));

    switch ( id )
    {
        case MENU_SERVLIST_ID_TVRADIO:
            servLoc.listId = inst->currGlobList;

            /*lint -save -e778 */

            if ( inst->currGlobList == SLIST_TV )
            {
                servLoc.index = inst->list[SLIST_TV - SLIST_OFFSET_GLOB].
                                entry[index];
            }
            else
            {
                servLoc.index = inst->list[SLIST_RADIO - SLIST_OFFSET_GLOB].
                                entry[index];
            }

            /*lint -restore */

            retVal = SLIST_EntryGet (&servLoc, &listItem);

            servNum = (uint16_t)(servLoc.index + 1);

            #ifdef APPL_LCN_ENABLED

            if ( inst->showLcn )
            {
                if ( SLIST_GetLcn (&servLoc, &servNum) != FAPI_OK )
                {
                    servNum = 0;
                }
            }

            #endif /* APPL_LCN_ENABLED */

            sprintf (appDat->asciiBuf, "%04i ", servNum);

            bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);

            if ( retVal == FAPI_OK )
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         STR_Build(listItem.nameCharSet,
                                                   listItem.name),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_VALUE_SERVLIST_DELETED),
                                         MAIN_LINEBUFCHARSET);

                state = FGS_DISABLED;
            }
            break;

        case MENU_SERVLIST_ID_FAV:
            servLoc.listId = inst->currFavList;
            servLoc.index  = inst->list[inst->currFavList - SLIST_OFFSET_USR +
                                        MENU_SERVLIST_NUM_GLOB].entry[index];

            retVal = SLIST_EntryGet (&servLoc, &listItem);

            servNum = (uint16_t)(servLoc.index + 1);

            #ifdef APPL_LCN_ENABLED

            if ( inst->showLcn )
            {
                if ( SLIST_GetLcn (&servLoc, &servNum) != FAPI_OK )
                {
                    servNum = 0;
                }
            }

            #endif /* APPL_LCN_ENABLED */

            sprintf (appDat->asciiBuf, "%03i ", servNum);

            bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);

            if ( retVal == FAPI_OK )
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         STR_Build(listItem.nameCharSet,
                                                   listItem.name),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_VALUE_SERVLIST_DELETED),
                                         MAIN_LINEBUFCHARSET);

                state = FGS_DISABLED;
            }
            break;

        default:
            return;
    }

    item[0].type     = FGS_ITEM_TYPE_ICON;
    item[0].align    = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[0].item.icon.drawMode   = FGS_ICONMODE_OVRLAY;
    item[0].item.icon.altBkgrCol = OSD_GetSchemeColor(OSD_SCHEMECOL_MAIN_BKGR);
    item[0].item.icon.bkgrCol    = OSD_COL_BLANK;

    if( !SLIST_FlagIsSet(&listItem, SLIST_FLAG_ALLFREE) )
    {
        item[0].item.icon.addr = OSD_IconGet(ICON_CRYPTED);
    }
    else
    {
        item[0].item.icon.addr = NULL;
    }

    item[1].item.str.chr        = STR_IntLineStr();
    item[1].type                = FGS_ITEM_TYPE_STRING;
    item[1].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[1].item.str.font       = MAIN_FONT;
    item[1].item.str.lineHeight = 0;

    if ( listItem.minAge != PCTRL_MINAGE_UNLIMITED )
    {
        item[1].item.str.col = OSD_COL_RED;
    }
    else
    {
        switch ( state )
        {
            default:
            case FGS_NONFOCUS: item[1].item.str.col = appDat->txtAttr.colNonFocus; break;
            case FGS_FOCUS:    item[1].item.str.col = appDat->txtAttr.colFocus;    break;
            case FGS_DISABLED: item[1].item.str.col = appDat->txtAttr.colDisabled; break;
        }
    }
}

FSTATIC void MENU_ServListGetTitleItem (FAPI_SYS_HandleT h, int32_t id,
                                        uint32_t index, FGS_ITEMSTATE_E state,
                                        FGS_ITEM_S* item)
{
    MENU_SERVLIST_INST_S*   inst = (MENU_SERVLIST_INST_S*)h;

    switch ( id )
    {
        case MENU_SERVLIST_ID_TVRADIOTITLE:
            if(inst->currGlobList == SLIST_TV)
                item->item.str.chr = OSD_GetStr(STR_ENTRY_SERVLIST_GLOBTV);
            else
                item->item.str.chr = OSD_GetStr(STR_ENTRY_SERVLIST_GLOBRADIO);
            break;

        case MENU_SERVLIST_ID_FAVTITLE:
            item->item.str.chr = SLIST_ListNameGet(inst->currFavList);
            break;

        default:
            return;
    }

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
    item->item.str.font       = HELP_FONT;
    item->item.str.lineHeight = 0;
    item->item.str.col        = OSD_COL_TEXT;
}

FSTATIC fbool_t MENU_ServListKeys (FAPI_SYS_HandleT h)
{
    MENU_SERVLIST_INST_S*   inst    = (MENU_SERVLIST_INST_S*)h;
    fbool_t                 keyUsed = FTRUE;
    SYS_SERVLOC_S           servLoc;
    SLIST_REF_S             servRef;
    SLIST_ITEM_S            listItem;
    int32_t                 pnlFocus;
    FGS_LNAVI_PARAMS_S      naviParsGlob;
    FGS_LNAVI_PARAMS_S      naviParsFav;
    int32_t                 newFocus;
    MENU_SERVLIST_LIST_S*   pList;
    MENU_EDITSTRING_OPEN_S  editParams;
    uint32_t                stackedFrameWidth =0;
    SYS_STRING_S            str;

    #ifdef APPL_PARENTAL_ENABLED
    uint8_t                 minAgeCurr;
    #endif /* APPL_PARENTAL_ENABLED */

    if ( inst->pnlHdl == NULL )
    {
        return FFALSE;
    }

    memset(&servLoc, 0, sizeof(servLoc));
    memset(&servRef, 0, sizeof(servRef));

    pnlFocus = FGS_PANEL_GetMemberFocusID(inst->pnlHdl);

    /* ----- identify currently focussed service --------------------------- */

    memset(&naviParsGlob, 0, sizeof(naviParsGlob));

    (void)FGS_LNAVI_GetParams (inst->tvRadioListHdl, &naviParsGlob);

    memset(&naviParsFav,  0, sizeof(naviParsFav));

    (void)FGS_LNAVI_GetParams (inst->favListHdl,     &naviParsFav);

    switch ( pnlFocus )
    {
        case MENU_SERVLIST_ID_TVRADIO:

            servLoc.listId = inst->currGlobList;
            servLoc.index  = inst->list[inst->currGlobList - SLIST_OFFSET_GLOB].
                             entry[naviParsGlob.index];
            break;

        case MENU_SERVLIST_ID_FAV:
            servLoc.listId = inst->currFavList;
            servLoc.index  = inst->list[inst->currFavList - SLIST_OFFSET_USR +
                                        MENU_SERVLIST_NUM_GLOB].
                                        entry[naviParsFav.index];
            break;

        case MENU_SERVLIST_ID_LIST:
            break;

        default : return FFALSE;
    }

    if ( servLoc.index <= 0 ) servLoc.index = 0;

    /* --------------------------------------------------------------------- */

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_NAVI: /* "EPG" switch active list */
            switch ( pnlFocus )
            {
                case MENU_SERVLIST_ID_TVRADIO:
                    newFocus = MENU_SERVLIST_ID_FAV;
                    break;

                case MENU_SERVLIST_ID_FAV:
                    if ( inst->guidedMode ) newFocus = MENU_SERVLIST_ID_LIST;
                    else                    newFocus = MENU_SERVLIST_ID_TVRADIO;
                    break;

                default:
                case MENU_SERVLIST_ID_LIST:
                    newFocus = MENU_SERVLIST_ID_TVRADIO;
                    break;
            }

            (void)FGS_PANEL_SetMemberFocus(inst->pnlHdl, newFocus);
            break;

        case FGS_KEY_TV: /* Toggle TV/Radio list */
            if( inst->currGlobList == SLIST_TV)
            {
                inst->currGlobList = SLIST_RADIO;
            }
            else
            {
                inst->currGlobList = SLIST_TV;
            }

            naviParsGlob.numItems = SLIST_ListSizeGet(inst->currGlobList);

            (void)FGS_LNAVI_SetParams(inst->tvRadioListHdl, &naviParsGlob);

            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                           MENU_SERVLIST_ID_TVRADIOTITLE);
            break;

        case FGS_KEY_OK : /* add to favorites */
            if ( pnlFocus == MENU_SERVLIST_ID_TVRADIO )
            {
                servRef.listNum = (uint8_t)(servLoc.listId - SLIST_OFFSET_GLOB);
                servRef.index   = servLoc.index;

                servLoc.listId  = inst->currFavList;
                servLoc.index   = naviParsFav.index;

                if(((uint16_t)(servLoc.index+1)) <= naviParsFav.numItems)
                {
                    servLoc.index += 1;
                    naviParsFav.index += 1;
                }

                if ( SLIST_EntryInsert(&servLoc, NULL, &servRef) == FAPI_OK )
                {
                    inst->listChg = FTRUE;

                    MENU_ServListSort(inst, inst->currFavList);

                    naviParsFav.numItems = SLIST_ListSizeGet(inst->currFavList);

                    (void)FGS_LNAVI_SetParams(inst->favListHdl, &naviParsFav);
                }
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case FGS_KEY_BLUE : /* select favorite list */
            inst->currFavList++;
            if ( inst->currFavList >= (SLIST_OFFSET_USR + SLIST_USR_NUMLISTS) )
            {
                inst->currFavList = SLIST_OFFSET_USR;
            }

            naviParsFav.numItems = SLIST_ListSizeGet(inst->currFavList);

            (void)FGS_LNAVI_SetParams(inst->favListHdl, &naviParsFav);

            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                           MENU_SERVLIST_ID_FAVTITLE);
            break;

        case FGS_KEY_RED: /* remove item */
            if ( SLIST_EntryRemove (&servLoc, FTRUE) == FAPI_OK )
            {
                inst->listChg = FTRUE;

                /* Removal of an item potentially affects all lists, this is why
                   all lists are re-sorted */
                MENU_ServListSortAll(inst);

                if ( pnlFocus == MENU_SERVLIST_ID_TVRADIO )
                {
                    naviParsGlob.numItems = SLIST_ListSizeGet(inst->
                                                              currGlobList);

                    (void)FGS_LNAVI_SetParams(inst->tvRadioListHdl,
                                              &naviParsGlob);
                }

                naviParsFav.numItems = SLIST_ListSizeGet(inst->currFavList);

                (void)FGS_LNAVI_SetParams(inst->favListHdl, &naviParsFav);
            }
            break;

        case FGS_KEY_GREEN: /* sort list by name */

            pList = MENU_ServListGetList(inst, servLoc.listId);

            if ( pList->sortMode == SLIST_SORT_BYNAME )
            {
                pList->ascending = pList->ascending ? FFALSE : FTRUE;
            }
            else
            {
                pList->sortMode  = SLIST_SORT_BYNAME;
                pList->ascending = FTRUE;
            }

            MENU_ServListSort(inst, servLoc.listId);

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, pnlFocus);
            break;

        case FGS_KEY_YELLOW: /* sort list by index */

            pList = MENU_ServListGetList(inst, servLoc.listId);

            if ( pList->sortMode == SLIST_SORT_BYINDEX )
            {
                pList->ascending = pList->ascending ? FFALSE : FTRUE;
            }
            else
            {
                pList->sortMode  = SLIST_SORT_BYINDEX;
                pList->ascending = FTRUE;
            }

            MENU_ServListSort(inst, servLoc.listId);

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, pnlFocus);
            break;

        case FGS_KEY_PLAY :
            (void) MAIN_SelectService (&servLoc, FFALSE);
            break;

        case FGS_KEY_REW : /* move item up */

            if ( servLoc.index < 1 )
            {
                break;
            }

            if ( SLIST_EntryMove(servLoc.listId,
                                 (uint16_t)(servLoc.index),
                                 (uint16_t)(servLoc.index - 1)) == FAPI_OK )
            {
                inst->listChg = FTRUE;

                switch ( pnlFocus )
                {
                    default:
                    case MENU_SERVLIST_ID_TVRADIO:

                        MENU_ServListSort(inst, inst->currGlobList );

                        pList = MENU_ServListGetList(inst, inst->currGlobList);

                        if ( pList->sortMode == SLIST_SORT_BYINDEX )
                        {
                            /* move index */
                            if ( !(pList->ascending) )
                            {
                                naviParsGlob.index++;
                                naviParsGlob.posInVis++;
                            }
                            else if ( naviParsGlob.index > 0 )
                            {
                                naviParsGlob.index--;

                                if ( naviParsGlob.posInVis > 0 )
                                {
                                    naviParsGlob.posInVis--;
                                }
                            }

                            (void)FGS_LNAVI_SetParams(inst->tvRadioListHdl,
                                                      &naviParsGlob);
                        }
                        else
                        {
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                      MENU_SERVLIST_ID_TVRADIO);
                        }
                        break;

                    case MENU_SERVLIST_ID_FAV:

                        MENU_ServListSort(inst, inst->currFavList );

                        pList = MENU_ServListGetList(inst, inst->currFavList);

                        if ( pList->sortMode == SLIST_SORT_BYINDEX )
                        {
                            /* move index */
                            if ( !(pList->ascending) )
                            {
                                naviParsFav.index++;
                                naviParsFav.posInVis++;
                            }
                            else if ( naviParsFav.index > 0 )
                            {
                                naviParsFav.index--;

                                if ( naviParsFav.posInVis > 0 )
                                {
                                    naviParsFav.posInVis--;
                                }
                            }

                            (void)FGS_LNAVI_SetParams(inst->favListHdl,
                                                      &naviParsFav);
                        }
                        else
                        {
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                          MENU_SERVLIST_ID_FAV);
                        }
                        break;
                }
            }
            break;

        case FGS_KEY_FF :  /* move item down */
            if ( SLIST_EntryMove(servLoc.listId, (uint16_t)(servLoc.index),
                                 (uint16_t)(servLoc.index + 1)) == FAPI_OK )
            {
                inst->listChg = FTRUE;

                switch ( pnlFocus )
                {
                    default:
                    case MENU_SERVLIST_ID_TVRADIO:

                        MENU_ServListSort(inst, inst->currGlobList );

                        pList = MENU_ServListGetList(inst, inst->currGlobList);

                        if ( pList->sortMode == SLIST_SORT_BYINDEX )
                        {
                            /* move index */
                            if ( pList->ascending )
                            {
                                naviParsGlob.index++;
                                naviParsGlob.posInVis++;
                            }
                            else if ( naviParsGlob.index > 0 )
                            {
                                naviParsGlob.index--;

                                if ( naviParsGlob.posInVis > 0 )
                                {
                                    naviParsGlob.posInVis--;
                                }
                            }

                            (void)FGS_LNAVI_SetParams(inst->tvRadioListHdl,
                                                      &naviParsGlob);
                        }
                        else
                        {
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                      MENU_SERVLIST_ID_TVRADIO);
                        }
                        break;

                    case MENU_SERVLIST_ID_FAV:

                        MENU_ServListSort(inst, inst->currFavList );

                        pList = MENU_ServListGetList(inst, inst->currFavList);

                        if ( pList->sortMode == SLIST_SORT_BYINDEX )
                        {
                            /* move index */
                            if ( pList->ascending )
                            {
                                naviParsFav.index++;
                                naviParsFav.posInVis++;
                            }
                            else if ( naviParsFav.index > 0 )
                            {
                                naviParsFav.index--;

                                if ( naviParsFav.posInVis > 0 )
                                {
                                    naviParsFav.posInVis--;
                                }
                            }

                            (void)FGS_LNAVI_SetParams(inst->favListHdl,
                                                      &naviParsFav);
                        }
                        else
                        {
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                          MENU_SERVLIST_ID_FAV);
                        }
                        break;
                }
            }
            break;

        case FGS_KEY_APPL: /* Clear all service lists */
            if ( SLIST_ClearAllLists() == FAPI_OK )
            {
                inst->listChg = FTRUE;

                /* update both lists */
                naviParsGlob.numItems = SLIST_ListSizeGet(inst->currGlobList);

                (void)FGS_LNAVI_SetParams(inst->tvRadioListHdl, &naviParsGlob);

                naviParsFav.numItems = SLIST_ListSizeGet(inst->currFavList);

                (void)FGS_LNAVI_SetParams(inst->favListHdl, &naviParsFav);
            }
            break;

        case FGS_KEY_TEXT :
            if ( (pnlFocus == MENU_SERVLIST_ID_TVRADIO)
              || (pnlFocus == MENU_SERVLIST_ID_FAV) )
            {
                stackedFrameWidth   = (MENUSTACK_GetLevel() - 1)
                                      * (SCALER_GetCurrVal(STACKED_FRAME_DIST)
                                        + SCALER_GetCurrVal(OUTER_FRAME_ALL));

                (void)FGS_PANEL_GetPosition (inst->pnlHdl, &(editParams.pos));

                editParams.pos.y  += stackedFrameWidth
                                   + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                   + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                   + SCALER_GetCurrVal(MAIN_TOP_PADDING)
                                   + (2*SCALER_GetCurrVal(TITLE_TB_PADDING))
                                   + SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                                   + SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

                (void)STR_BufClear(&(inst->tmpBuf));

                if ( pnlFocus == MENU_SERVLIST_ID_TVRADIO )
                {
                    /* focus on global list -> rename service */

                    if ( SLIST_EntryGet (&servLoc, &listItem) != FAPI_OK )
                    {
                        keyUsed = FFALSE;
                        break;
                    }

                    (void)STR_BufAppend(&(inst->tmpBuf),0,
                                        STR_Build(listItem.nameCharSet,
                                                  listItem.name),
                                                  listItem.nameCharSet);
                }
                else
                {
                    /* focus on favorite list -> rename list name */

                    str = SLIST_ListNameGet(inst->currFavList);

                    (void)STR_BufAppend(&inst->tmpBuf,0, str, str.cs);
                }

                editParams.editTitle         = NULL;
                editParams.editTitleArg      = NULL;
                editParams.onExit            = MENU_ServListEditStringExit;
                editParams.editData.string   = inst->tmpBuf.string;
                editParams.editData.maxChars = MAIN_LINEBUFSIZE - 1;
                editParams.id                = pnlFocus;

                keyUsed = MENU_EditStringOpen (&(appDat->editStringBox),
                                               &editParams);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        #ifdef APPL_PARENTAL_ENABLED

        case FGS_KEY_STOP: /* Toggle service lock status */

            if ( PCTRL_GetServiceProtection (&servLoc, &minAgeCurr) == FAPI_OK )
            {
                if ( minAgeCurr == PCTRL_MINAGE_UNLIMITED )
                {
                    minAgeCurr = PCTRL_MINAGE_LOCKED;
                }
                else
                {
                    minAgeCurr = PCTRL_MINAGE_UNLIMITED;
                }

                if ( PCTRL_SetServiceProtection (&servLoc, minAgeCurr) == FAPI_OK )
                {
                    inst->listChg = FTRUE;
                }

                (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                              MENU_SERVLIST_ID_TVRADIO);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                              MENU_SERVLIST_ID_FAV);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        #endif /* APPL_PARENTAL_ENABLED */

        default:
            break;
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_ServListBuild (MENU_SERVLIST_INST_S* inst)
{
    FGS_PANEL_OPEN_S    pnlPars;
    FGS_LIST_OPEN_S     listPars;
    FAPI_SYS_HandleT    itemHdl;
    FGS_BOX_OPEN_S      boxPars;
    FGS_ITEM_TXT_OPEN_S txtPars;
    FGS_LNAVI_OPEN_S    naviPars;
    FGS_ITEM_S          item;
    int32_t             retVal;
    uint16_t            stackedFrameWidth;
    uint16_t            innerHeight;
    uint32_t            numLines     = MENU_SERVLIST_NUM_LINES;
    uint8_t             numHelpLines = 7;
    FGS_PANEL_MEMBER_S  member;
    int32_t             idCnt;
    int16_t             midPos;
    uint16_t            titleHeight;
    uint16_t            startPosX;

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        innerHeight = ((numLines + 1)   * SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                     + (numLines        * SCALER_GetCurrVal(LIST_LINE_PADDING));

        if ( inst->guidedMode )
        {
            innerHeight += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                         + SCALER_GetCurrVal(LIST_LINE_PADDING)
                         + SCALER_GetCurrVal(MAIN_BOT_PADDING);
        }

        MENU_CommonSetParamsTitleWBotHelp (&pnlPars,
                                           SCALER_GetCurrVal(SERVLIST_WIDTH),
                                           innerHeight,
                                           numHelpLines,
                                           &stackedFrameWidth);

        titleHeight       = stackedFrameWidth
                          + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                          + SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                          + (2 * SCALER_GetCurrVal(TITLE_TB_PADDING))
                          + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                          + SCALER_GetCurrVal(MAIN_TOP_PADDING);

        pnlPars.onFocusSet      = MENU_ServListOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SERVLIST, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_SERVLIST_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_SERVLIST,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        /* Calculate width of one service list box.
           NOTE: The space between the two boxes is MAIN_LEFT_PADDING. */
        inst->serviceBoxWidth = (SCALER_GetCurrVal(SERVLIST_WIDTH)
                              - (SCALER_GetCurrVal(MAIN_LEFT_PADDING) * 2)
                              -  SCALER_GetCurrVal(MAIN_RIGHT_PADDING)) / 2;

        /* ----- create list box title lines ------------------------------- */

        startPosX = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                  + SCALER_GetCurrVal(MAIN_LEFT_PADDING)
                  + stackedFrameWidth;

        memset(&boxPars,  0, sizeof(boxPars));

        boxPars.pos.x               = startPosX;
        boxPars.pos.y               = titleHeight;
        boxPars.pos.width           = inst->serviceBoxWidth;
        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem             = MENU_ServListGetTitleItem;
        boxPars.getItemArg          = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars, MENU_SERVLIST_ID_TVRADIOTITLE,
                               inst->pnlHdl, &(inst->tvRadioTitleItemHdl));
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x              += boxPars.pos.width
                                   +  SCALER_GetCurrVal(MAIN_LEFT_PADDING);

        retVal = FGS_BOX_Open (&boxPars, MENU_SERVLIST_ID_FAVTITLE,
                               inst->pnlHdl, &inst->favTitleHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- List Borders ---------------------------------------------- */

        inst->itemBorderPos.x         = startPosX
                                      - SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        inst->itemBorderPos.y         = boxPars.pos.y
                                      + boxPars.pos.height;
        inst->itemBorderPos.width     = (2 * inst->serviceBoxWidth)
                                      + SCALER_GetCurrVal(MAIN_LEFT_PADDING)
                                      + (2 * SCALER_GetCurrVal(INNER_FRAME_WIDTH));
        inst->itemBorderPos.height    = (numLines *
                                        (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                       + SCALER_GetCurrVal(LIST_LINE_PADDING)))
                                       + (2 * SCALER_GetCurrVal(INNER_FRAME_WIDTH));
        inst->itemBorderPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        memset(&member,   0, sizeof(member));

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemBorderPos);
        member.id            = MENU_SERVLIST_ID_REST;
        member.isHidden      = FFALSE;
        member.show          = MENU_ServListPaintRest;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pItemBorder));

        if ( retVal != FAPI_OK ) break;

        memset(&naviPars, 0, sizeof(naviPars));

        naviPars.pos.x           = startPosX;
        naviPars.pos.y           = boxPars.pos.y
                                 + boxPars.pos.height
                                 + SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        naviPars.pos.width       = inst->serviceBoxWidth;
        naviPars.pos.height      = numLines *
                                   (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                  + SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        naviPars.bkgrType        = OSD_NAVIBKGR_NORMAL;
        naviPars.type            = FGS_LNAVI_TYPE_SCROLL;
        naviPars.rowHeight       = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                 +  SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.scrollbarWidth  = SCALER_GetCurrVal(SCROLLBAR_WIDTH);
        naviPars.numVisLines     = numLines;
        naviPars.numColumns      = 2;
        naviPars.params.numItems = SLIST_ListSizeGet(SLIST_TV);
        naviPars.params.index    = 0;
        naviPars.params.posInVis = 0;

        naviPars.colStart[0]     = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[0]     = SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        naviPars.colStart[1]     = naviPars.colStart[0] + naviPars.colWidth[0];
        naviPars.colWidth[1]     = inst->serviceBoxWidth
                                 - naviPars.colStart[1]
                                 - SCALER_GetCurrVal(SCROLLBAR_WIDTH)
                                 - SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.topDownOrient   = FTRUE;
        naviPars.wraparound      = FTRUE;
        naviPars.getItem         = MENU_ServListGetItem;
        naviPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        naviPars.pInst           = &(inst->tvRadioLnaviInst);

        retVal = FGS_LNAVI_Open (&naviPars, MENU_SERVLIST_ID_TVRADIO,
                                 inst->pnlHdl,
                                &(inst->tvRadioListHdl));
        if ( retVal != FAPI_OK ) break;

        naviPars.pos.x          += naviPars.pos.width
                                 + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        naviPars.params.numItems = SLIST_ListSizeGet(SLIST_OFFSET_USR);
        naviPars.pInst           = &(inst->favLnaviInst);

        retVal = FGS_LNAVI_Open (&naviPars, MENU_SERVLIST_ID_FAV,
                                 inst->pnlHdl,
                                &(inst->favListHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create navigation button for guided installation mode ----- */

        if ( inst->guidedMode )
        {
            memset (&listPars, 0, sizeof(listPars));

            listPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                     + stackedFrameWidth;
            listPars.pos.y           = naviPars.pos.y
                                     + naviPars.pos.height
                                     + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                     + SCALER_GetCurrVal(MAIN_BOT_PADDING);
            listPars.pos.width       = SCALER_GetCurrVal(SERVLIST_WIDTH);
            listPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                     + SCALER_GetCurrVal(LIST_LINE_PADDING);

            listPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            listPars.topDownOriented = FTRUE;
            listPars.wraparound      = FTRUE;
            listPars.getDescrItem    = MENU_ServListGetDescrItem;
            listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
            listPars.onSelect        = MENU_ServListOnSelect;
            listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
            listPars.pInst           = &(appDat->listInst);

            retVal = FGS_LIST_Open (&listPars, MENU_SERVLIST_ID_LIST,
                                    inst->pnlHdl, &(inst->listHdl));
            if ( retVal != FAPI_OK ) break;

            memset(&txtPars,  0, sizeof(txtPars));

            txtPars.pos.x           = 0;
            txtPars.pos.y           = 0;
            txtPars.pos.width       = listPars.pos.width;
            txtPars.pos.height      = listPars.pos.height;
            txtPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            txtPars.bkgrType        = OSD_LSTITEM_BKGR_NORMAL;
            txtPars.descrPos        = txtPars.pos;
            txtPars.descrPos.x     += SCALER_GetCurrVal(MAIN_LEFT_PADDING);
            txtPars.descrPos.width -= SCALER_GetCurrVal(MAIN_LEFT_PADDING)
                                    + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SERVLIST_ID_GOTONEXT,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        midPos = ((pnlPars.pos.width / 2) + 30);
        idCnt  = MENU_SERVLIST_ID_HELPBOTTOM;

        memset(&item, 0, sizeof(item));

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_EPG);

        memset(&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = 0 - SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - numHelpLines * SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - (numHelpLines - 1) * SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_SERVLIST_SWITCHLIST);
        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.width           = (uint16_t)(midPos - boxPars.pos.x + 1);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNBLUESMALL);

        boxPars.pos.x               = midPos - 1;
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SERVLIST_SELECTLIST);

        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 2. Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_TV);

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);

        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_SERVLIST_TOGGLETVRA);
        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.width           = (uint16_t)(midPos - boxPars.pos.x + 1);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNREDSMALL);

        boxPars.pos.x               = midPos - 1;
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SERVLIST_REMOVEITEM);

        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* --3. Bottom line -- */

        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        item.item.str.chr           = OSD_GetStr(STR_KEYS_REWFF);

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);

        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_SERVLIST_IDXUPDOWN);
        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.width           = (uint16_t)(midPos - boxPars.pos.x + 1);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNGREENSMALL);

        boxPars.pos.x               = midPos - 1;
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SERVLIST_SORTBYNAME);

        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 4. Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        item.item.str.chr           = OSD_GetStr(STR_KEY_OK);

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);

        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_SERVLIST_ADDTOFAV);
        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.width           = (uint16_t)(midPos - boxPars.pos.x + 1);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNYELLOWSMALL);

        boxPars.pos.x               = midPos - 1;
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SERVLIST_SORTBYIDX);

        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 5. Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        item.item.str.chr           = OSD_GetStr(STR_KEY_PLAY);

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);

        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_SERVLIST_TUNE);
        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.width           = (uint16_t)(midPos - boxPars.pos.x + 1);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_APPL);

        boxPars.pos.x               = midPos;
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SERVLIST_CLEARLISTS);

        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 6. Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        item.item.str.chr           = OSD_GetStr(STR_KEY_MENU);
        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_MENU_BACK);
        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.width           = (uint16_t)(midPos - boxPars.pos.x + 1);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_KEY_EXIT);
        boxPars.pos.x               = midPos;
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_MENU_EXIT);
        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 7. Bottom line ----------------------------------------------- */

        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        item.item.str.chr           = OSD_GetStr(STR_KEY_TEXT);
        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_SERVLIST_RENAME_SERVICE);
        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_LEFT);
        boxPars.pos.width           = (uint16_t)(midPos - boxPars.pos.x + 1);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_PARENTAL_ENABLED

        item.item.str.chr           = OSD_GetStr(STR_KEY_STOP);
        boxPars.pos.x               = midPos;
        boxPars.pos.width           = SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_SERVLIST_LOCK);
        boxPars.pos.x              += SCALER_GetCurrVal(SERVLIST_WIDTH_HELPKEY_RIGHT);
        boxPars.pos.width           = (uint16_t)(midPos - boxPars.pos.x + 1);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_PARENTAL_ENABLED */

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);
        }

        return FFALSE;
    }

    return FTRUE;
}

FSTATIC int32_t MENU_ServListPaintRest(FAPI_SYS_HandleT h)
{
    MENU_SERVLIST_INST_S*   inst = (MENU_SERVLIST_INST_S*)h;
    uint32_t                x1,x2,y1,y2;
    uint16_t                frameWidth = SCALER_GetCurrVal(INNER_FRAME_WIDTH);

    x1 = (uint32_t)inst->pItemBorder->abspos.x;
    x2 = x1 + inst->serviceBoxWidth + (2 * frameWidth) - 1;
    y1 = (uint32_t)inst->pItemBorder->abspos.y;
    y2 = y1 + inst->itemBorderPos.height - 1;

    /* Draw frame of left list */
    OSD_DrawFilledRectangle (x1, y1, x2, y2,
                             OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME));

    x1 = x2 + 1 + SCALER_GetCurrVal(MAIN_LEFT_PADDING) - (2 * frameWidth);
    x2 = x1 + inst->serviceBoxWidth + (2 * frameWidth) - 1;

    /* Draw frame of right box */
    OSD_DrawFilledRectangle (x1, y1, x2, y2,
                             OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME));

    return FAPI_OK;
}


FSTATIC void MENU_ServListSort (MENU_SERVLIST_INST_S* inst, SLIST_TYPE type)
{
    uint16_t idx;

    switch ( type )
    {
        case SLIST_TV:
        case SLIST_RADIO:
            idx = type - SLIST_OFFSET_GLOB;
            break;

        default:
            idx = type - SLIST_OFFSET_USR + MENU_SERVLIST_NUM_GLOB;
            break;
    }

    (void)SLIST_Sort(type,
                     inst->list[idx].sortMode,
                     inst->list[idx].ascending,
                     inst->list[idx].entry,
                     inst->list[idx].size);
}

FSTATIC void MENU_ServListSortAll (MENU_SERVLIST_INST_S* inst)
{
    uint8_t     idx;
    SLIST_TYPE  type;

    for ( idx = 0; idx < MENU_SERVLIST_NUM_LISTS; idx++ )
    {
        switch ( idx )
        {
            case 0:     type = SLIST_TV;                break;
            case 1:     type = SLIST_RADIO;             break;
            default:    type = SLIST_OFFSET_USR + idx -
                               MENU_SERVLIST_NUM_GLOB;  break;
        }

        (void)SLIST_Sort(type,
                         inst->list[idx].sortMode,
                         inst->list[idx].ascending,
                         inst->list[idx].entry,
                         inst->list[idx].size);
    }
}

FSTATIC MENU_SERVLIST_LIST_S* MENU_ServListGetList (MENU_SERVLIST_INST_S* inst,
                                                    SLIST_TYPE listId)
{
    MENU_SERVLIST_LIST_S* pList;

    switch ( listId )
    {
        case SLIST_RADIO:
        case SLIST_TV:
            pList = &inst->list[listId - SLIST_OFFSET_GLOB];
            break;

        default:
            pList = inst->list + listId - SLIST_OFFSET_USR +
                    MENU_SERVLIST_NUM_GLOB;
            break;
    }

    return pList;
}

FSTATIC void MENU_ServListEditStringExit (MENU_EDITSTRING_DATA_S* editData,
                                          int32_t id)
{
    MENU_SERVLIST_INST_S*   inst = menuServListDat;
    FGS_LNAVI_PARAMS_S      naviParsGlob;
    SYS_SERVLOC_S           servLoc;
    SLIST_ITEM_S            listItem;
    int32_t                 retVal = FAPI_OK;

    if ( editData->updated )
    {
        switch ( FGS_PANEL_GetMemberFocusID(inst->pnlHdl) )
        {
            case MENU_SERVLIST_ID_TVRADIO:
                memset(&naviParsGlob, 0, sizeof(FGS_LNAVI_PARAMS_S));
                memset(&servLoc,      0, sizeof(SYS_SERVLOC_S));
                memset(&listItem,     0, sizeof(SLIST_ITEM_S));

                (void)FGS_LNAVI_GetParams (inst->tvRadioListHdl, &naviParsGlob);

                servLoc.listId = inst->currGlobList;
                servLoc.index  = inst->list[inst->currGlobList
                                 - SLIST_OFFSET_GLOB].entry[naviParsGlob.index];

                if ( servLoc.index < 0 )
                {
                    return;
                }

                retVal = SLIST_SetServNameByIndex(
                                                SLIST_GetServiceIndex(&servLoc),
                                                editData->string);
                if(retVal != FAPI_OK) return;

                inst->listChg = FTRUE;

                (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                              MENU_SERVLIST_ID_TVRADIO);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                              MENU_SERVLIST_ID_FAV);
                break;

            case MENU_SERVLIST_ID_FAV:
                (void)SLIST_ListNameSet(inst->currFavList, editData->string);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                              MENU_SERVLIST_ID_FAVTITLE);
                inst->listChg = FTRUE;
                break;

            default:
                /* do nothing */
                break;
        }
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
