/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_servselect.c
**
**              $Id: app_menu_servselect.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Service selection module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
*****************************************************************************
*/
/*lint -save -e713 -e715 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

fbool_t opened = FFALSE;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t MENU_ServSelectLeave           (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC void  MENU_ServSelectOnFocusSet      (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC int16_t   MENU_ServSelectGetInitialIndex (SLIST_TYPE listId);
FSTATIC fbool_t MENU_ServSelectServiceAvailable(const SYS_SERVLOC_S* pServLoc);
FSTATIC void  MENU_ServSelectOnServSelect    (const SYS_SERVLOC_S* pServLoc);

#ifdef APPL_PVR_ENABLED

FSTATIC void  MENU_ServSelectOnRecSelect     (uint8_t recIdx);
FSTATIC fbool_t MENU_ServSelectRecAvailable    (uint8_t recIdx);

#endif /* APPL_PVR_ENABLED */


int32_t MENU_ServSelectInit (void)
{
    /* currently nothing to do... */

    return FAPI_OK;
}

fbool_t MENU_ServSelectEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    SYS_SERVLOC_S        servLoc;
    MAIN_SERVBOX_OPEN_S  servBoxParams;

    if ( test == FTRUE ) return FTRUE;

    servLoc.listId = SLIST_NONE;

    (void)AV_GetRequestedService(&servLoc);

    if ( (servLoc.listId == SLIST_NONE) ||
         (servLoc.listId == SLIST_INVISIBLE) )
    {
        servLoc.listId = SLIST_TV;
    }

    memset(&servBoxParams, 0, sizeof(servBoxParams));

    servBoxParams.initialListId          = servLoc.listId;
    servBoxParams.getInitialIndexForList = MENU_ServSelectGetInitialIndex;
    servBoxParams.serviceIsAvailable     = MENU_ServSelectServiceAvailable;
    servBoxParams.onServiceSelect        = MENU_ServSelectOnServSelect;
    servBoxParams.onFocusSet             = MENU_ServSelectOnFocusSet;
    servBoxParams.onFocusSetArg          = NULL;

    #ifdef APPL_PVR_ENABLED

    servBoxParams.recordIsAvailable      = MENU_ServSelectRecAvailable;
    servBoxParams.onRecordSelect         = MENU_ServSelectOnRecSelect;

    #endif /* APPL_PVR_ENABLED */

    if ( MAIN_ServBoxOpen(&(appDat->servBox), &servBoxParams) != FAPI_OK )
    {
        return FFALSE;
    }

    MENUSTACK_SetCleanup(MENU_ServSelectLeave, NULL);

    opened = FTRUE;
    return FTRUE;
}

void MENU_ServSelectServiceUpdate (const SYS_SERVLOC_S* pServLoc)
{
    SLIST_TYPE currList = SLIST_NONE;

    if ( !opened ) return;

    (void)MAIN_ServBoxGetCurrList(&(appDat->servBox), &currList);

    if ( currList == pServLoc->listId )
    {
        (void)MAIN_ServBoxSetServLoc(&(appDat->servBox), pServLoc);
    }
}

FSTATIC fbool_t MENU_ServSelectLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    (void)MAIN_ServBoxClose(&(appDat->servBox));

    opened = FFALSE;
    return FTRUE;
}

FSTATIC void MENU_ServSelectOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    MAIN_HandleZappingKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_MENU, MAIN_DirectMenuTrans, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK, MENUSTACK_Exit,       NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_EXIT, MENUSTACK_Exit,       NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_INFO, MAIN_DirectMenuTrans, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_NAVI, MAIN_DirectMenuTrans, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK, MAIN_KeyPIP,          NULL, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler(FGS_KEY_INFO);
        FGS_ClearAppKeyHandler(FGS_KEY_NAVI);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
    }
}

FSTATIC int16_t MENU_ServSelectGetInitialIndex (SLIST_TYPE listId)
{
    SYS_SERVLOC_S        activeServ;
    int16_t                  foundIndex;

    activeServ.listId = SLIST_NONE;
    activeServ.index  = -1;

    (void)AV_GetRequestedService (&activeServ);

    if ( SLIST_EntryFindInList(&activeServ, listId, &foundIndex) == FAPI_OK )
    {
        return foundIndex;
    }
    else
    {
        return 0;
    }
}

FSTATIC fbool_t MENU_ServSelectServiceAvailable (const SYS_SERVLOC_S* pServLoc)
{
    return (MAIN_ServiceSwitchGetAvailability (pServLoc) == FAPI_OK) ? FTRUE
                                                                   : FFALSE;
}

FSTATIC void MENU_ServSelectOnServSelect (const SYS_SERVLOC_S* pServLoc)
{
    SYS_SERVLOC_S        activeServ;
    int32_t                 retVal = FAPI_OK;

    activeServ.listId = SLIST_NONE;
    activeServ.index  = -1;

    (void)AV_GetRequestedService (&activeServ);

    /* do not select service if already selected */
    if ( (pServLoc->listId != activeServ.listId) ||
         (pServLoc->index  != activeServ.index) )
    {
        retVal = MAIN_SelectService (pServLoc, FFALSE);
    }

    if ( retVal == FAPI_OK )
    {
        (void)MENUSTACK_Exit(NULL);
    }
}

#ifdef APPL_PVR_ENABLED

FSTATIC fbool_t MENU_ServSelectRecAvailable (uint8_t recIdx)
{
    /* will be called only if record is active. */
    return FTRUE;
}

FSTATIC void MENU_ServSelectOnRecSelect (uint8_t recIdx)
{
    int32_t retVal;

    retVal = PVRSYS_PlayRecStart (recIdx, PVRSYS_TIMESHIFT_BEGIN,
                                  PLAY_STATUS_NORMAL, 0, NULL, NULL);

    if ( retVal == FAPI_OK )
    {
        (void)MENUSTACK_Exit(NULL);
    }
}

#endif /* APPL_PVR_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
