/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_systemsetup.c
**
**              $Id: app_menu_systemsetup.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD system setup menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_SYSTEMSETUP_ID_TITLE = 0,
    MENU_SYSTEMSETUP_ID_LIST,
    MENU_SYSTEMSETUP_ID_HELPLIST
};

enum
{
    MENU_SYSTEMSETUP_ID_ENTRY_LANGUAGE = 0,
    MENU_SYSTEMSETUP_ID_ENTRY_TIME,
    MENU_SYSTEMSETUP_ID_ENTRY_ANTENNA,
    MENU_SYSTEMSETUP_ID_ENTRY_ADVD,

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)
    MENU_SYSTEMSETUP_ID_ENTRY_CI,
    #endif /* APPL_CI_ENABLED */

    MENU_SYSTEMSETUP_ID_ENTRY_OSD,

    #if defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED)
        MENU_SYSTEMSETUP_ID_ENTRY_UTIL,
    #endif /* defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED) */

    #ifdef APPL_PARENTAL_ENABLED
        MENU_SYSTEMSETUP_ID_ENTRY_PASSWORD,
    #endif /* APPL_PARENTAL_ENABLED */

    #ifdef APPL_ETHERNET_ENABLED
        MENU_SYSTEMSETUP_ID_ENTRY_NETWORK,
    #endif /* APPL_ETHERNET_ENABLED */

    #ifdef APPL_UPDATE_ENABLED
        MENU_SYSTEMSETUP_ID_ENTRY_UPDATE,
    #endif /* APPL_UPDATE_ENABLED */

    MENU_SYSTEMSETUP_ID_ENTRY_FACTORY,
    MENU_SYSTEMSETUP_ID_ENTRY_POWER,
    MENU_SYSTEMSETUP_ID_ENTRY_NUMS
};

typedef struct
{
    FAPI_SYS_HandleT     pnlHdl;
    FAPI_SYS_HandleT     listHdl;
    FAPI_SYS_HandleT     listHelpHdl;
    SYS_PWRMODE_E        powerMode;
    fbool_t              powerChg;
    fbool_t              exitFlag;

}MENU_SYSTEMSETUP_INST_S;

FSTATIC MENU_SYSTEMSETUP_INST_S* menuSystemSetupDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_SystemSetupLeave                 (FAPI_SYS_HandleT h,
                                                       fbool_t force);
FSTATIC fbool_t MENU_SystemSetupOnSelect              (FAPI_SYS_HandleT h,
                                                       int32_t itemId);
FSTATIC void    MENU_SystemSetupValChg                (FAPI_SYS_HandleT h,
                                                       int32_t itemId,
                                                       FGS_VAL_S newVal);
FSTATIC void    MENU_SystemSetupGetItem               (FAPI_SYS_HandleT h,
                                                       int32_t id,
                                                       uint32_t index,
                                                       FGS_ITEMSTATE_E state,
                                                       FGS_ITEM_S* item);
FSTATIC fbool_t MENU_SystemSetupBuild                 (FAPI_SYS_HandleT h);
FSTATIC void    MENU_SystemSetupOnFocusChg            (FAPI_SYS_HandleT h,
                                                       uint8_t index,
                                                       int32_t id);
FSTATIC void    MENU_SystemSetupConfirmFactoryRestore (void* arg);
FSTATIC void    MENU_SystemSetupOnFocusSet            (FAPI_SYS_HandleT h,
                                                       fbool_t hasFocus);
FSTATIC void    MENU_SystemSetupGetDescrItem          (FAPI_SYS_HandleT h,
                                                       int32_t id,
                                                       uint32_t index,
                                                       FGS_ITEMSTATE_E state,
                                                       FGS_ITEM_S* item);
FSTATIC void    MENU_SystemSetupGetHelpDescrItem      (FAPI_SYS_HandleT h,
                                                       int32_t id,
                                                       uint32_t index,
                                                       FGS_ITEMSTATE_E state,
                                                       FGS_ITEM_S* item);

/*!
*******************************************************************************
**
** \brief  Initial function for the OSD System setup Menu
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_SystemSetupInit(void)
{
    if ( menuSystemSetupDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuSystemSetupDat = UTILS_Malloc(sizeof(MENU_SYSTEMSETUP_INST_S));
    if ( menuSystemSetupDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuSystemSetupDat, 0, sizeof(MENU_SYSTEMSETUP_INST_S));

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function
**
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_SystemSetupEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_SYSTEMSETUP_INST_S* inst = menuSystemSetupDat;
    uint32_t                      lastFocusItem;

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    #ifdef APPL_MHEG_ENABLED
    MHEG_Enable (FFALSE, SYS_SUSPENDSRC_USER);
    #endif

    inst->powerMode        = SYS_PWRMODE_DOWN;
    inst->powerChg         = FFALSE;
    inst->exitFlag         = FFALSE;

    (void)MENU_SystemSetupBuild((FAPI_SYS_HandleT)inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = MENU_SYSTEMSETUP_ID_ENTRY_LANGUAGE;
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_SYSTEMSETUP_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_SYSTEMSETUP);   /* set global focus on menu panel */
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_SystemSetupLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_SystemSetupOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_SYSTEMSETUP_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);
}

/*!
*******************************************************************************
**
** \brief  Activates the help icons for the current menu list item.
**
** \param h        Address of menu handle.
** \param index    unused.
** \param id       List id gets focus.
**
******************************************************************************/
FSTATIC void MENU_SystemSetupOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_SYSTEMSETUP_INST_S* inst = (MENU_SYSTEMSETUP_INST_S*)h;
    FGS_LNAVI_PARAMS_S       naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}

FSTATIC fbool_t MENU_SystemSetupLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_SYSTEMSETUP_INST_S* inst = (MENU_SYSTEMSETUP_INST_S*)h;

    if ( inst->powerChg )
    {
        MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);
        (void)SETTINGS_SetPowerUpMode(inst->powerMode);
        (void)SETTINGS_Save ();
        (void)MAIN_MsgBoxHide(&(appDat->msgBox));

        inst->powerChg = FFALSE;
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_SYSTEMSETUP);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC fbool_t MENU_SystemSetupOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_SYSTEMSETUP_INST_S* inst = (MENU_SYSTEMSETUP_INST_S*)h;
    MENUSTACK_ENTRY          newEntry = NULL;
    FAPI_SYS_HandleT         arg = NULL;
    uint8_t                  subLines = 0;
    MENU_SELECTLIST_OPEN_S   selectLst;
    FGS_ITEM_SEL_PARAMS_S    selParams;
    FGS_LOC_S                memberLoc;
    fbool_t                  openSelectLst = FFALSE;
    MENU_CONFDIALOG_S        confPars;

    memset(&confPars,  0, sizeof(confPars));

    switch ( itemId )
    {
        case MENU_SYSTEMSETUP_ID_ENTRY_LANGUAGE:
            newEntry = MENU_LanguageEntry;
            break;

        case MENU_SYSTEMSETUP_ID_ENTRY_TIME:
            newEntry = MENU_TimeEntry;
            break;

        case MENU_SYSTEMSETUP_ID_ENTRY_ANTENNA:
            newEntry = MENU_AntProgEntry;
            break;

        case MENU_SYSTEMSETUP_ID_ENTRY_ADVD:
            newEntry = MENU_AudioVideoEntry;
            break;

        #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

        case MENU_SYSTEMSETUP_ID_ENTRY_CI:
            newEntry = MENU_CiEntry;
            break;

        #endif /* APPL_CI_ENABLED */

        case MENU_SYSTEMSETUP_ID_ENTRY_OSD:
            newEntry = MENU_OsdEntry;
            break;

        #if defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED)

        case MENU_SYSTEMSETUP_ID_ENTRY_UTIL:
            newEntry = MENU_HddEntry;
            break;

        #endif /* defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED) */

        #ifdef APPL_PARENTAL_ENABLED

        case MENU_SYSTEMSETUP_ID_ENTRY_PASSWORD:
            newEntry = MENU_PasswordEntry;
            break;

        #endif /* APPL_PARENTAL_ENABLED */

        #ifdef APPL_ETHERNET_ENABLED

        case MENU_SYSTEMSETUP_ID_ENTRY_NETWORK:
            newEntry = MENU_NetworkEntry;
            break;

        #endif /* APPL_ETHERNET_ENABLED */

        #ifdef APPL_UPDATE_ENABLED

        case MENU_SYSTEMSETUP_ID_ENTRY_UPDATE:
            newEntry = MENU_UpdateEntry;
            break;

        #endif /* APPL_UPDATE_ENABLED */

        case MENU_SYSTEMSETUP_ID_ENTRY_FACTORY:
            //  restore Factory settings and goto Welcome Screen;

            memset(&confPars, 0, sizeof(confPars));

            confPars.numDescLines = 1;
            confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_RESTORFACTORY);
            confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_SAVEOK);
            confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_SAVEABORT);
            confPars.onConfirm    = MENU_SystemSetupConfirmFactoryRestore;
            confPars.onDecline    = NULL;
            confPars.onDiscard    = NULL;
            confPars.arg          = NULL;

            return MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);

        case MENU_SYSTEMSETUP_ID_ENTRY_POWER :
            openSelectLst = FTRUE;
            subLines = 2;
            inst->powerChg = FTRUE;
            break;

        default:
            MENU_CommonMsgShow(STR_MSG_NOTIMPLEMENTEDYET, FFALSE);
            return FFALSE;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.x         = memberLoc.x + SCALER_GetCurrVal(MAIN_WIDTH) -
                                  SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.width     = SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_SystemSetupGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;
        selectLst.onExit        = NULL;
        selectLst.onExitArg     = h;

        return MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }
    else
    {
        /* store last focus */
        MENUSTACK_SetData(itemId);

        if(newEntry == NULL) return FFALSE;
        else                 return MENUSTACK_Up(newEntry, arg);
    }
}

FSTATIC void MENU_SystemSetupValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    MENU_SYSTEMSETUP_INST_S* inst = (MENU_SYSTEMSETUP_INST_S*)h;

    switch ( itemId )
    {
        case MENU_SYSTEMSETUP_ID_ENTRY_POWER:
            inst->powerMode = (SYS_PWRMODE_E)newVal.number;
            inst->powerChg  = FTRUE;
            break;

        default:
            break;
    }
}

FSTATIC void MENU_SystemSetupGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_SYSTEMSETUP_ID_ENTRY_LANGUAGE      : strIdx = STR_ENTRY_SYSTEMSETUP_LANG;     break;
        case MENU_SYSTEMSETUP_ID_ENTRY_TIME          : strIdx = STR_ENTRY_SYSTEMSETUP_TIME;     break;
        case MENU_SYSTEMSETUP_ID_ENTRY_ANTENNA       : strIdx = STR_ENTRY_SYSTEMSETUP_ANTENNA;  break;
        case MENU_SYSTEMSETUP_ID_ENTRY_ADVD          : strIdx = STR_ENTRY_SYSTEMSETUP_ADVD;     break;

        #ifdef APPL_CI_ENABLED
        #ifdef APPL_CA_ENABLED
        case MENU_SYSTEMSETUP_ID_ENTRY_CI            : strIdx = STR_ENTRY_SYSTEMSETUP_CICA;       break;
        #else
        case MENU_SYSTEMSETUP_ID_ENTRY_CI            : strIdx = STR_ENTRY_SYSTEMSETUP_CI;       break;
        #endif
        #else
        #ifdef APPL_CA_ENABLED
        case MENU_SYSTEMSETUP_ID_ENTRY_CI            : strIdx = STR_ENTRY_SYSTEMSETUP_CA;       break;
        #endif
        #endif /* APPL_CI_ENABLED */

        case MENU_SYSTEMSETUP_ID_ENTRY_OSD           : strIdx = STR_ENTRY_SYSTEMSETUP_OSD;      break;

        #if defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED)

        case MENU_SYSTEMSETUP_ID_ENTRY_UTIL          : strIdx = STR_ENTRY_SYSTEMSETUP_UTILITIES;break;

        #endif /* defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED) */

        #ifdef APPL_PARENTAL_ENABLED

        case MENU_SYSTEMSETUP_ID_ENTRY_PASSWORD      : strIdx = STR_ENTRY_SYSTEMSETUP_PASSWORD; break;

        #endif /* APPL_PARENTAL_ENABLED */

        #ifdef APPL_ETHERNET_ENABLED

        case MENU_SYSTEMSETUP_ID_ENTRY_NETWORK       : strIdx = STR_ENTRY_SYSTEMSETUP_NETWORK; break;

        #endif /* APPL_ETHERNET_ENABLED */

        #ifdef APPL_UPDATE_ENABLED

        case MENU_SYSTEMSETUP_ID_ENTRY_UPDATE         : strIdx = STR_ENTRY_SYSTEMSETUP_UPDATE;  break;

        #endif /* APPL_UPDATE_ENABLED */

        case MENU_SYSTEMSETUP_ID_ENTRY_FACTORY       : strIdx = STR_ENTRY_SYSTEMSETUP_FACTORY;  break;
        case MENU_SYSTEMSETUP_ID_ENTRY_POWER         : strIdx = STR_ENTRY_SYSTEMSETUP_POWER;    break;
        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_SystemSetupGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                               FGS_ITEMSTATE_E state,
                                               FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        case MENU_SYSTEMSETUP_ID_ENTRY_LANGUAGE :
        case MENU_SYSTEMSETUP_ID_ENTRY_TIME     :
        case MENU_SYSTEMSETUP_ID_ENTRY_ANTENNA  :
        case MENU_SYSTEMSETUP_ID_ENTRY_ADVD     :
        case MENU_SYSTEMSETUP_ID_ENTRY_OSD      :

        #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)
        case MENU_SYSTEMSETUP_ID_ENTRY_CI:
        #endif /* APPL_CI_ENABLED */

        #if defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED)
        case MENU_SYSTEMSETUP_ID_ENTRY_UTIL :
        #endif /* defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED) */

        #ifdef APPL_PARENTAL_ENABLED
        case MENU_SYSTEMSETUP_ID_ENTRY_PASSWORD :
        #endif /*APPL_PARENTAL_ENABLED */
        #ifdef APPL_ETHERNET_ENABLED
        case MENU_SYSTEMSETUP_ID_ENTRY_NETWORK :
        #endif

        #ifdef APPL_UPDATE_ENABLED
        case MENU_SYSTEMSETUP_ID_ENTRY_UPDATE    :
        #endif /* APPL_UPDATE_ENABLED */

        case MENU_SYSTEMSETUP_ID_ENTRY_FACTORY  : strIdx = STR_ENTRY_OK;
                                                  break;
        case MENU_SYSTEMSETUP_ID_ENTRY_POWER    : strIdx = STR_ENTRY_LOKR;
                                                  break;
        default:  return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC void MENU_SystemSetupGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id ){

    case MENU_SYSTEMSETUP_ID_ENTRY_POWER:
        switch ( index )
        {
            case 0 : strIdx = STR_VALUE_SYSTEMSETUP_STANDBY; break;
            case 1 : strIdx = STR_VALUE_SYSTEMSETUP_UP;      break;
            default: strIdx = STR_EMPTYSTRING;               break;
        }
        break;
    default:  return;
    }

    item->item.str.chr        = OSD_GetStr(strIdx);
    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
    }

}

FSTATIC fbool_t MENU_SystemSetupBuild(FAPI_SYS_HandleT h)
{
    MENU_SYSTEMSETUP_INST_S* inst = (MENU_SYSTEMSETUP_INST_S*)h;
    FGS_PANEL_OPEN_S         pnlPars;
    FGS_LIST_OPEN_S          listPars;
    FAPI_SYS_HandleT         itemHdl;
    FGS_ITEM_SEL_OPEN_S      selPars;
    FGS_ITEM_TXT_OPEN_S      txtPars;
    FGS_POS_S                itemPos;
    FGS_POS_S                valuePos;
    FGS_POS_S                descrPos;
    int32_t                  retVal;
    uint16_t                 helpWidth;
    uint16_t                 stackedFrameWidth;
    uint16_t                 innerHeight;
    uint32_t                 numLines = MENU_SYSTEMSETUP_ID_ENTRY_NUMS;

    memset(&txtPars,  0, sizeof(txtPars));
    memset(&selPars,  0, sizeof(selPars));

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_LOKR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_SystemSetupOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_SYSTEMSETUP, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_SYSTEMSETUP_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_SYSTEMSETUP,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_SystemSetupGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_SystemSetupGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_SystemSetupOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_SystemSetupOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_SystemSetupValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_SYSTEMSETUP_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 66,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- menu lines -------------------------------------------------*/

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;

        /* --- Language settings --------------------------------------------*/

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_LANGUAGE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);


        /* --- Time settings ------------------------------------------------*/

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_TIME,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Antenna and program setup ------------------------------------*/

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_ANTENNA,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Audio/Video setup --------------------------------------------*/

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_ADVD,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- CI Setup -------------------------------------------------- */

        #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_CI,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_CI_ENABLED */

        /* --- OSD Setup ----------------------------------------------------*/

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_OSD,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Harddisk/USB utilities -------------------------------------- */

        #if defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED)

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_UTIL,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* defined(APPL_HDD_ENABLED) || defined(APPL_USB_ENABLED) */

        /* --- Password ---------------------------------------------------*/

        #ifdef APPL_PARENTAL_ENABLED

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_PASSWORD,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_PARENTAL_ENABLED */

        /* --- Network ---------------------------------------------------*/

        #ifdef APPL_ETHERNET_ENABLED

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_NETWORK,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_ETHERNET_ENABLED */

        /* --- Software update -------------------------------------------*/

        #ifdef APPL_UPDATE_ENABLED

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_UPDATE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_UPDATE_ENABLED */

        /* --- Restore settings -------------------------------------------*/

        txtPars.pos = itemPos;
        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_SYSTEMSETUP_ID_ENTRY_FACTORY,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Power up state ----------------------------------------------*/

        (void)SETTINGS_GetPowerUpMode (&(inst->powerMode));
        selPars.pos                 = itemPos;
        selPars.bkgrType            = txtPars.bkgrType;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = SYS_PWRMODE_NUMS;
        selPars.params.index        = inst->powerMode;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_SYSTEMSETUP_ID_ENTRY_POWER, inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_SYSTEMSETUP_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH),
                                        listPars.pos.y, helpWidth, numLines,
                                        MENU_SystemSetupGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}

FSTATIC void MENU_SystemSetupConfirmFactoryRestore (void* arg)
{
    MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

    (void)SYS_RestoreDefaultState(FTRUE);

    (void)MAIN_UserDataSet(USERDAT_FACTORYFLAG, 1);

    /* Save all */
    #ifdef APPL_FESAT_ENABLED
    (void)CFGSAT_Save(FFALSE);
    #endif /* APPL_FESAT_ENABLED */

    (void)TPLIST_Save(FE_SYSTEM_TYPE_SATELLITE,   FFALSE);
    (void)TPLIST_Save(FE_SYSTEM_TYPE_TERRESTRIAL, FFALSE);
    (void)TPLIST_Save(FE_SYSTEM_TYPE_CABLE,       FFALSE);
    (void)SLIST_Save(FFALSE);
    (void)SETTINGS_Save();

    (void) AV_ResetService();

    #ifdef APPL_RECTIMER_ENABLED
    (void)RECTIMER_Save (FFALSE);
    #endif /* APPL_RECTIMER_ENABLED */

    #ifdef APPL_PARENTAL_ENABLED

    (void)PCTRL_Save(FFALSE);

    #endif /* APPL_PARENTAL_ENABLED */

    (void)MAIN_MsgBoxHide(&(appDat->msgBox));

    (void)OSD_SetColorScheme(OSD_COLSCHEME_CURRENT);

    (void)MENUSTACK_Exit(NULL);
    (void)MENUSTACK_Up(MENU_WelcomeEntry,NULL);
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
