/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_time.c
**
**              $Id: app_menu_time.c 5890 2010-03-03 15:39:03Z jmorha $
**
** \brief       Time-menu implementation.
**
** Detail description. [\ref app_smartgo_app_ref_time]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_TIME_ID_TITLE = 0,
    MENU_TIME_ID_LIST,
    MENU_TIME_ID_HELPLIST

};

enum
{
    MENU_TIME_ID_ENTRY_MSG1 = 0,
    MENU_TIME_ID_ENTRY_MSG2,
    MENU_TIME_ID_ENTRY_AUTO_TIME,
    MENU_TIME_ID_ENTRY_DATE,
    MENU_TIME_ID_ENTRY_TIME,
    MENU_TIME_ID_ENTRY_DST,
    MENU_TIME_ID_ENTRY_AUTO_OFFSET,
    MENU_TIME_ID_ENTRY_REGION,
    MENU_TIME_ID_ENTRY_GMT,
    MENU_TIME_ID_ENTRY_NUMS

};

typedef enum
{
    TIME_REGION_IDX_BAKER_ISLAND = 0,
    TIME_REGION_IDX_SAMOA,
    TIME_REGION_IDX_HAWAIIAN,
    TIME_REGION_IDX_ILES_MARQUISES,
    TIME_REGION_IDX_ANCHORAGE,
    TIME_REGION_IDX_LOSANGLES,
    TIME_REGION_IDX_DENVER,
    TIME_REGION_IDX_MEXICO,
    TIME_REGION_IDX_NEWYORK,
    TIME_REGION_IDX_VENEZUELA,
    TIME_REGION_IDX_MANAUS,
    TIME_REGION_IDX_NEWFOUNDLAND,
    TIME_REGION_IDX_BRASILIA,
    TIME_REGION_IDX_SOUTHGEORGIA,
    TIME_REGION_IDX_AZORES,
    TIME_REGION_IDX_LONDON,
    TIME_REGION_IDX_BERLIN,
    TIME_REGION_IDX_CAIRO,
    TIME_REGION_IDX_MOSCOW,
    TIME_REGION_IDX_TEHRAN,
    TIME_REGION_IDX_DUBAI,
    TIME_REGION_IDX_KABUL,
    TIME_REGION_IDX_PERM,
    TIME_REGION_IDX_BOMBAY,
    TIME_REGION_IDX_NEPAL,
    TIME_REGION_IDX_OMSK,
    TIME_REGION_IDX_BURMA,
    TIME_REGION_IDX_LAOS,
    TIME_REGION_IDX_SHANGHAI,
    TIME_REGION_IDX_CAIGUNA_EUCLA,
    TIME_REGION_IDX_TOKYO,
    TIME_REGION_IDX_ALICE_SPRINGS,
    TIME_REGION_IDX_SYDNEY,
    TIME_REGION_IDX_LORD_HOWE,
    TIME_REGION_IDX_MAGADAN,
    TIME_REGION_IDX_NORFOLK,
    TIME_REGION_IDX_FIJI,
    TIME_REGION_IDX_CHATHAM_ISLAND,
    TIME_REGION_IDX_TONGA,
    TIME_REGION_IDX_KIRIBATI,

    TIME_REGION_IDX_NUMS

}TIME_REGION_IDX_E;

enum
{
    TIME_AUTO_OFFSET_DISABLED = 0,
    TIME_AUTO_OFFSET_ENABLED

};

enum
{
    TIME_AUTO_DISABLED = 0,
    TIME_AUTO_ENABLED

};


typedef struct
{
    FAPI_SYS_HandleT       pnlHdl;
    FAPI_SYS_HandleT       listHdl;
    FAPI_SYS_HandleT       itemHdl;
    FAPI_SYS_HandleT       listHelpHdl;
    fbool_t                hasFocus;
    MENU_TIME_OPEN_S       params;

}MENU_TIME_INST_S;

typedef struct
{
    TIME_REGION_IDX_E       regionIdx;
    int16_t                 gmtOffset;
    uint32_t                regionStrIdx;

} OSDTIME_ENTRY_S;

FSTATIC MENU_TIME_INST_S* menuTimeDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC OSDTIME_ENTRY_S menuTimeZoneArray[TIME_REGION_IDX_NUMS] =
{
  { TIME_REGION_IDX_BAKER_ISLAND,  -720, STR_TIMEZONE_BAKER_ISLAND  },
  { TIME_REGION_IDX_SAMOA,         -660, STR_TIMEZONE_SAMOA         },
  { TIME_REGION_IDX_HAWAIIAN,      -600, STR_TIMEZONE_HAWAIIAN      },
  { TIME_REGION_IDX_ILES_MARQUISES,-570, STR_TIMEZONE_ILES_MARQUISES},
  { TIME_REGION_IDX_ANCHORAGE,     -540, STR_TIMEZONE_ANCHORAGE     },
  { TIME_REGION_IDX_LOSANGLES,     -480, STR_TIMEZONE_LOSANGLES     },
  { TIME_REGION_IDX_DENVER,        -420, STR_TIMEZONE_DENVER        },
  { TIME_REGION_IDX_MEXICO,        -360, STR_TIMEZONE_MEXICO        },
  { TIME_REGION_IDX_NEWYORK,       -300, STR_TIMEZONE_NEWYORK       },
  { TIME_REGION_IDX_VENEZUELA,     -270, STR_TIMEZONE_VENEZUELA     },
  { TIME_REGION_IDX_MANAUS,        -240, STR_TIMEZONE_MANAUS        },
  { TIME_REGION_IDX_NEWFOUNDLAND,  -210, STR_TIMEZONE_NEWFOUNDLAND  },
  { TIME_REGION_IDX_BRASILIA,      -180, STR_TIMEZONE_BRASILIA      },
  { TIME_REGION_IDX_SOUTHGEORGIA,  -120, STR_TIMEZONE_SOUTHGEORGIA  },
  { TIME_REGION_IDX_AZORES,         -60, STR_TIMEZONE_AZORES        },
  { TIME_REGION_IDX_LONDON,           0, STR_TIMEZONE_LONDON        },
  { TIME_REGION_IDX_BERLIN,          60, STR_TIMEZONE_BERLIN        },
  { TIME_REGION_IDX_CAIRO,          120, STR_TIMEZONE_CAIRO         },
  { TIME_REGION_IDX_MOSCOW,         180, STR_TIMEZONE_MOSCOW        },
  { TIME_REGION_IDX_TEHRAN,         210, STR_TIMEZONE_TEHRAN        },
  { TIME_REGION_IDX_DUBAI,          240, STR_TIMEZONE_DUBAI         },
  { TIME_REGION_IDX_KABUL,          270, STR_TIMEZONE_KABUL         },
  { TIME_REGION_IDX_PERM,           300, STR_TIMEZONE_PERM          },
  { TIME_REGION_IDX_BOMBAY,         330, STR_TIMEZONE_BOMBAY        },
  { TIME_REGION_IDX_NEPAL,          345, STR_TIMEZONE_NEPAL         },
  { TIME_REGION_IDX_OMSK,           360, STR_TIMEZONE_OMSK          },
  { TIME_REGION_IDX_BURMA,          390, STR_TIMEZONE_BURMA         },
  { TIME_REGION_IDX_LAOS,           420, STR_TIMEZONE_LAOS          },
  { TIME_REGION_IDX_SHANGHAI,       480, STR_TIMEZONE_SHANGHAI      },
  { TIME_REGION_IDX_CAIGUNA_EUCLA,  525, STR_TIMEZONE_CAIGUNA_EUCLA },
  { TIME_REGION_IDX_TOKYO,          540, STR_TIMEZONE_TOKYO         },
  { TIME_REGION_IDX_ALICE_SPRINGS,  570, STR_TIMEZONE_ALICE_SPRINGS },
  { TIME_REGION_IDX_SYDNEY,         600, STR_TIMEZONE_SYDNEY        },
  { TIME_REGION_IDX_LORD_HOWE,      630, STR_TIMEZONE_LORD_HOWE     },
  { TIME_REGION_IDX_MAGADAN,        660, STR_TIMEZONE_MAGADAN       },
  { TIME_REGION_IDX_NORFOLK,        690, STR_TIMEZONE_NORFOLK       },
  { TIME_REGION_IDX_FIJI,           720, STR_TIMEZONE_FIJI          },
  { TIME_REGION_IDX_CHATHAM_ISLAND, 765, STR_TIMEZOME_CHATHAM_ISLAND},
  { TIME_REGION_IDX_TONGA,          780, STR_TIMEZONE_TONGA         },
  { TIME_REGION_IDX_KIRIBATI,       840, STR_TIMEZONE_KIRIBATI      },
};

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t          MENU_TimeLeave               (FAPI_SYS_HandleT h,
                                                       fbool_t force);
FSTATIC fbool_t          MENU_TimeOnSelect            (FAPI_SYS_HandleT h,
                                                       int32_t itemId);
FSTATIC void             MENU_TimeValChg              (FAPI_SYS_HandleT h,
                                                       int32_t itemId,
                                                       FGS_VAL_S newVal);
FSTATIC void             MENU_TimeGetDescrItem        (FAPI_SYS_HandleT h,
                                                       int32_t id,
                                                       uint32_t index,
                                                       FGS_ITEMSTATE_E state,
                                                       FGS_ITEM_S* item);
FSTATIC void             MENU_TimeGetItem             (FAPI_SYS_HandleT h,
                                                       int32_t id,
                                                       uint32_t index,
                                                       FGS_ITEMSTATE_E state,
                                                       FGS_ITEM_S* item);
FSTATIC fbool_t          MENU_TimeBuild               (FAPI_SYS_HandleT h);
FSTATIC void             MENU_TimeOnFocusChg          (FAPI_SYS_HandleT h,
                                                       uint8_t index,
                                                       int32_t id);
FSTATIC fbool_t          MENU_TimeKeys                (FAPI_SYS_HandleT h);

FSTATIC OSDTIME_ENTRY_S* MENU_TimeGetArrayByGmtOffset (int16_t gmtOffset);
FSTATIC void             MENU_TimeOnFocusSet          (FAPI_SYS_HandleT h,
                                                       fbool_t hasFocus);
FSTATIC void             MENU_TimeGetHelpDescrItem    (FAPI_SYS_HandleT h,
                                                       int32_t id,
                                                       uint32_t index,
                                                       FGS_ITEMSTATE_E state,
                                                       FGS_ITEM_S* item);

/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************
** \brief   TODO
**
** \return  One off the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_ALREADY_INITIALIZED
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
*******************************************************************************
*/
int32_t MENU_TimeInit (void)
{
    if ( menuTimeDat != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    menuTimeDat = UTILS_Zalloc(sizeof(MENU_TIME_INST_S));
    if ( menuTimeDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h       TODO
** \param   test    TODO
**
** \return  One off the following status codes:
**          - #FTRUE
**          - #FFALSE
**
*******************************************************************************
*/
fbool_t MENU_TimeEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_TIME_INST_S* inst = menuTimeDat;
    MENU_TIME_OPEN_S* params = (MENU_TIME_OPEN_S*) h;
    uint32_t          lastFocusItem;

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    if ( MENUSTACK_GetData((uint32_t*)(&lastFocusItem)) == FFALSE )
    {
        lastFocusItem = MENU_TIME_ID_ENTRY_AUTO_TIME;
    }
    if(params!=NULL)
    {
        /* system request time from user. Preset all values for user time edit */
        inst->params = *params;
        lastFocusItem = MENU_TIME_ID_ENTRY_DATE;

        TIME_SetAutoMode(FFALSE);
    }

    (void)MENU_TimeBuild((FAPI_SYS_HandleT)inst);

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_TIME_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_TIME);          /* set global focus on menu panel */
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_TimeLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}


/*!
*******************************************************************************
** \brief   TODO
**
** \param   h       TODO
**
** \return  One off the following status codes:
**          - #FTRUE
**          - #FFALSE
**
*******************************************************************************
*/
FSTATIC fbool_t MENU_TimeKeys (FAPI_SYS_HandleT h)
{
    MENU_TIME_INST_S*   inst = (MENU_TIME_INST_S*)h;
    fbool_t             keyUsed = FTRUE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
            /* FALLTHROUGHT */
        case FGS_KEY_BACK:
            inst->params.done    = FTRUE;
            (void)MENU_TimeHandleEvt(&(inst->params));
            break;
        case FGS_KEY_EXIT:
            keyUsed = MENUSTACK_Exit(NULL);
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h           TODO
** \param   hasFocus    TODO
**
*******************************************************************************
*/
FSTATIC void MENU_TimeOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    MENU_TIME_INST_S*   inst = (MENU_TIME_INST_S*)h;

    DBG_Assert(((MENU_TIME_INST_S*)h)->pnlHdl != NULL);

    inst->hasFocus = hasFocus;

    if(inst->params.strIdx == 0)
    {
        MAIN_HandleExitKeys(hasFocus);
    }
    else
    {
        if ( hasFocus )
        {
            FGS_SetAppKeyHandler(FGS_KEY_MENU, MENU_TimeKeys, h, FTRUE, FFALSE);
            FGS_SetAppKeyHandler(FGS_KEY_BACK, MENU_TimeKeys, h, FTRUE, FFALSE);
            FGS_SetAppKeyHandler(FGS_KEY_EXIT, MENU_TimeKeys, h, FTRUE, FFALSE);
            MENU_TimeUpdate();
        }
        else
        {
            FGS_ClearAppKeyHandler(FGS_KEY_MENU);
            FGS_ClearAppKeyHandler(FGS_KEY_BACK);
            FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        }
    }
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h       TODO
** \param   index   TODO
** \param   id      TODO
**
*******************************************************************************
*/
FSTATIC void MENU_TimeOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_TIME_INST_S*   inst = (MENU_TIME_INST_S*)h;

    (void)FGS_LIST_SetMemberFocus(inst->listHelpHdl, id);
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h       TODO
** \param   force   TODO
**
** \return  One off the following status codes:
**          - #FTRUE
**          - #FFALSE
**
*******************************************************************************
*/
FSTATIC fbool_t MENU_TimeLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_TIME_INST_S* inst = (MENU_TIME_INST_S*)h;
    fbool_t           enabled;

    if ( force == FFALSE )
    {
        MAIN_SaveSettingsIfChanged();
    }

    (void)SETTINGS_AutoTimeGet(&enabled);
    TIME_SetAutoMode(enabled);

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_TIME);

    inst->pnlHdl = NULL;
    inst->params.strIdx = 0;

    return FTRUE;
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h       TODO
** \param   itemId  TODO
**
** \return  One off the following status codes:
**          - #FTRUE
**          - #FFALSE
**
*******************************************************************************
*/
FSTATIC fbool_t MENU_TimeOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_TIME_INST_S*       inst = (MENU_TIME_INST_S*)h;
    MENUSTACK_ENTRY         newEntry = NULL;
    FAPI_SYS_HandleT        arg = NULL;
    uint8_t                 subLines;
    MENU_SELECTLIST_OPEN_S  selectLst;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    FGS_LOC_S               memberLoc;
    fbool_t                 openSelectLst = FFALSE;

    switch ( itemId )
    {
        case MENU_TIME_ID_ENTRY_AUTO_TIME :
            openSelectLst = FTRUE;
            subLines = 2;
        break;

        case MENU_TIME_ID_ENTRY_AUTO_OFFSET :
            openSelectLst = FTRUE;
            subLines = 2;
        break;

        case MENU_TIME_ID_ENTRY_DST :
            openSelectLst = FTRUE;
            subLines = 2;
        break;

        case MENU_TIME_ID_ENTRY_REGION :
            openSelectLst = FTRUE;
            subLines = 5;
        break;

        default:
            return FFALSE;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.x         = memberLoc.x +
                                  SCALER_GetCurrVal(MAIN_WIDTH) -
                                  SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.width     = SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_TimeGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;
        selectLst.onExit        = NULL;
        selectLst.onExitArg     = h;

        return MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }
    else
    {
        /* store last focus */
        MENUSTACK_SetData(itemId);

        return MENUSTACK_Up(newEntry, arg);
    }
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h       TODO
** \param   itemId  TODO
** \param   newVal  TODO
**
*******************************************************************************
*/
FSTATIC void MENU_TimeValChg (FAPI_SYS_HandleT h, int32_t itemId,
                              FGS_VAL_S newVal)
{
    MENU_TIME_INST_S*     inst = (MENU_TIME_INST_S*)h;
    FGS_ITEM_SEL_PARAMS_S selParams;
    int16_t               utcOffset = 0;
    fbool_t               dstActive = FFALSE;
    fbool_t               enabled;
    fbool_t               settingEnabled;
    SYS_CTRYCODE          ctryCode;
    SYS_TIME_S            time;
    SYS_DATE_S            date;

    switch ( itemId )
    {
        case MENU_TIME_ID_ENTRY_AUTO_TIME:
            enabled = (newVal.number == TIME_AUTO_OFFSET_DISABLED) ?
                      FFALSE : FTRUE;

            (void)SETTINGS_AutoTimeGet(&settingEnabled);
            if(enabled != settingEnabled)
                (void)SETTINGS_AutoTimeSet(enabled);

            TIME_SetAutoMode(enabled);

            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_TIME_ID_ENTRY_DATE,
                                            (fbool_t)!enabled);
            (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                            MENU_TIME_ID_ENTRY_TIME,
                                            (fbool_t)!enabled);
            break;

        case MENU_TIME_ID_ENTRY_DATE:
            (void)TIME_GetWeekdayByDate(&(newVal.date));
            (void)TIME_TimeAndDateSet(NULL,&(newVal.date));
            break;

        case MENU_TIME_ID_ENTRY_TIME:
            (void)SETTINGS_AutoTimeSet(FFALSE);
            TIME_SetAutoMode(FFALSE);
            (void)SETTINGS_TimeOffsetGet (&utcOffset, &dstActive);
            (void)TIME_TimeAndDateGet(&time, &date, NULL);
            time = newVal.time;
            if(dstActive) utcOffset += 60;
            TIME_AddSeconds (&time, &date, -((int32_t)utcOffset * 60) );
            (void)TIME_TimeAndDateSet(&time, &date);
            break;

        case MENU_TIME_ID_ENTRY_DST:
            (void)SETTINGS_TimeOffsetGet (&utcOffset, &dstActive);

            dstActive = ((newVal.number == 0) ? FFALSE : FTRUE);

            (void)SETTINGS_TimeOffsetSet (utcOffset, dstActive );
            MENU_TimeUpdate();
            break;

        case MENU_TIME_ID_ENTRY_AUTO_OFFSET:
            if ( SETTINGS_AutoTimeOffsetGet (&enabled, &ctryCode) == FAPI_OK )
            {
                enabled = (newVal.number == TIME_AUTO_OFFSET_DISABLED) ?
                          FFALSE : FTRUE;

                (void)SETTINGS_AutoTimeOffsetSet (enabled, ctryCode);

                (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                                MENU_TIME_ID_ENTRY_REGION,
                                                (fbool_t)!enabled);
            }
            break;

        case MENU_TIME_ID_ENTRY_REGION:
            (void)SETTINGS_TimeOffsetGet (&utcOffset, &dstActive);

            utcOffset = menuTimeZoneArray[newVal.number].gmtOffset;

            (void)SETTINGS_TimeOffsetSet (utcOffset, dstActive);

            selParams.numItems = TIME_REGION_IDX_NUMS;
            selParams.index    = newVal.number;

            (void)FGS_LIST_SetMemberParams(inst->listHdl, MENU_TIME_ID_ENTRY_GMT,
                                     &selParams);
            break;

        case MENU_TIME_ID_ENTRY_GMT:
            break;

        default:
            break;
    }
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h       TODO
** \param   id      TODO
** \param   index   TODO
** \param   state   TODO
** \param   item    TODO
**
*******************************************************************************
*/
FSTATIC void MENU_TimeGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                    FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_TIME_INST_S*    inst = (MENU_TIME_INST_S*)h;
    uint32_t             strIdx;

    item->item.str.font       = MAIN_FONT;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->type                = FGS_ITEM_TYPE_STRING;
    item->item.str.lineHeight = 0;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
    }
    switch ( id )
    {
        case MENU_TIME_ID_ENTRY_MSG1:
            strIdx = inst->params.strIdx;
            item->item.str.col = appDat->txtAttr.colNonFocus;
        break;
        case MENU_TIME_ID_ENTRY_MSG2:
            strIdx = STR_TIME_MSG_INFO;
            item->item.str.col = appDat->txtAttr.colNonFocus;
            break;
        case MENU_TIME_ID_ENTRY_AUTO_TIME:
            strIdx = STR_ENTRY_TIME_AUTOTIME;
            break;
        case MENU_TIME_ID_ENTRY_DATE:
            strIdx = STR_ENTRY_TIME_DATE;
            break;
        case MENU_TIME_ID_ENTRY_TIME:
            strIdx = STR_ENTRY_TIME_TIME;
            break;
        case MENU_TIME_ID_ENTRY_DST:
            strIdx = STR_ENTRY_TIME_DST;
            break;
        case MENU_TIME_ID_ENTRY_AUTO_OFFSET:
            strIdx = STR_ENTRY_TIME_AUTOZONE;
            break;
        case MENU_TIME_ID_ENTRY_REGION:
            strIdx = STR_ENTRY_TIME_REGION;
            break;
        case MENU_TIME_ID_ENTRY_GMT:
            strIdx = STR_ENTRY_TIME_GMT;
            break;
        default:  return;
    }

    item->item.str.chr = OSD_GetStr(strIdx);

}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h       TODO
** \param   id      TODO
** \param   index   TODO
** \param   state   TODO
** \param   item    TODO
**
*******************************************************************************
*/
FSTATIC void MENU_TimeGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                        FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_TIME_ID_ENTRY_DATE:
        case MENU_TIME_ID_ENTRY_TIME:
            strIdx = STR_ENTRY_09LR;
            break;
        case MENU_TIME_ID_ENTRY_AUTO_TIME:
        case MENU_TIME_ID_ENTRY_AUTO_OFFSET:
        case MENU_TIME_ID_ENTRY_REGION:
        case MENU_TIME_ID_ENTRY_DST:
            strIdx = STR_ENTRY_LOKR;
            break;
        default:
            return;
    }
    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h       TODO
** \param   id      TODO
** \param   index   TODO
** \param   state   TODO
** \param   item    TODO
**
*******************************************************************************
*/
FSTATIC void MENU_TimeGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    SYS_DATE_S          currSysDate;
    SYS_TIME_S          currSysTime;
    uint32_t            bufPos    = 0;

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    (void)TIME_TimeAndDateGet(&currSysTime, &currSysDate, NULL);

    switch ( id )
    {
        case MENU_TIME_ID_ENTRY_AUTO_TIME:
            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            switch (index)
            {
                default:
                case TIME_AUTO_DISABLED:
                    item->item.str.chr = OSD_GetStr(STR_VALUE_TIME_AUTOTIME_DISABLE);
                    break;
                case TIME_AUTO_ENABLED:
                    item->item.str.chr = OSD_GetStr(STR_VALUE_TIME_AUTOTIME_ENABLE);
                    break;
            }
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
            }
            break;
        case MENU_TIME_ID_ENTRY_DATE:
            item->type             = FGS_ITEM_TYPE_DATE;
            item->align            = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.date.format = appDat->dataFormatNorm;
            item->item.date.font   = MAIN_FONT;
            item->item.date.value  = currSysDate;
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.date.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.date.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.date.col = appDat->txtAttr.colDisabled; break;
            }
            break;

        case MENU_TIME_ID_ENTRY_TIME:
            item->type             = FGS_ITEM_TYPE_TIME;
            item->align            = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.time.format = appDat->timeFormat;
            item->item.time.font   = MAIN_FONT;
            item->item.time.value  = currSysTime;
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.time.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.time.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.time.col = appDat->txtAttr.colDisabled; break;
            }
            break;

        case MENU_TIME_ID_ENTRY_DST:
            switch(index)
            {
                case 0:
                item->item.str.chr = OSD_GetStr(STR_VALUE_DAYLIGHT_SAVINGS_OFF);
                break;
                case 1:
                item->item.str.chr = OSD_GetStr(STR_VALUE_DAYLIGHT_SAVINGS_ON);
                break;
                default:
                item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                break;
            }
            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
            }
            break;
        case MENU_TIME_ID_ENTRY_AUTO_OFFSET:
            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            switch (index)
            {
                default:
                case TIME_AUTO_DISABLED:
                    item->item.str.chr = OSD_GetStr(STR_VALUE_TIME_AUTOZONE_DISABLE);
                    break;
                case TIME_AUTO_ENABLED:
                    item->item.str.chr = OSD_GetStr(STR_VALUE_TIME_AUTOZONE_ENABLE);
                    break;
            }
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
            }
            break;
        case MENU_TIME_ID_ENTRY_REGION:
            if ( index >= TIME_REGION_IDX_NUMS )
            {
                item->item.str.chr        = OSD_GetStr(STR_EMPTYSTRING);
            }
            else
            {
                item->item.str.chr        = OSD_GetStr(menuTimeZoneArray[index].regionStrIdx);
            }

            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
            }
            break;
        case MENU_TIME_ID_ENTRY_GMT:
            if(menuTimeZoneArray[index].gmtOffset < 0)
            {
                sprintf (appDat->asciiBuf, "- %02i:%02i ",
                        ((menuTimeZoneArray[index].gmtOffset/60)*(-1)),
                        ((menuTimeZoneArray[index].gmtOffset%60)*(-1)));
            }
            else if(menuTimeZoneArray[index].gmtOffset > 0)
            {
                sprintf (appDat->asciiBuf, "+ %02i:%02i ",
                         menuTimeZoneArray[index].gmtOffset/60,
                         menuTimeZoneArray[index].gmtOffset % 60);
            }
            else
            {
                sprintf (appDat->asciiBuf, "00:00 ");
            }

            bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(STR_UNIT_HOUR),
                                     MAIN_LINEBUFCHARSET);

            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.chr        = STR_IntLineStr();

            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
            }
            break;
        default:
            item->type                 = FGS_ITEM_TYPE_NONE;
            break;
    }
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   h   TODO
**
** \return  One of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
*******************************************************************************
*/
FSTATIC fbool_t MENU_TimeBuild(FAPI_SYS_HandleT h)
{
    MENU_TIME_INST_S*       inst = (MENU_TIME_INST_S*)h;
    FGS_PANEL_OPEN_S        pnlPars;
    FGS_LIST_OPEN_S         listPars;
    FGS_LIST_OPEN_S         helpListPars;
    FGS_ITEM_TXT_OPEN_S     txtPars;
    FGS_ITEM_NUM_OPEN_S     numPars;
    FGS_ITEM_SEL_OPEN_S     selPars;
    FGS_ITEM_TXT_OPEN_S     helpPars;
    FGS_POS_S               itemPos;
    FGS_POS_S               valuePos;
    FGS_POS_S               descrPos;
    int32_t                 retVal;
    uint16_t                helpWidth;
    uint16_t                stackedFrameWidth;
    uint16_t                innerHeight;
    uint32_t                numLines = MENU_TIME_ID_ENTRY_NUMS;
    fbool_t                 dstActive = SETTINGS_DEFAULT_DSTACTIVE;
    int16_t                 utcOffset = 0;
    SYS_CTRYCODE            ctryCode;
    fbool_t                 enabled;
    OSDTIME_ENTRY_S*        timeArray;
    FAPI_SYS_HandleT        itemHdl;

    memset(&txtPars,      0, sizeof(txtPars));
    memset(&numPars,      0, sizeof(numPars));
    memset(&selPars,      0, sizeof(selPars));
    memset(&helpListPars, 0, sizeof(helpListPars));
    memset(&helpPars,     0, sizeof(helpPars));

    if(inst->params.strIdx == 0)
    {
        numLines -= 2;  /* No msg lines */
    }

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_09LR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_TimeOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_TIME, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */
        retVal = MENU_CommonAddTitle(MENU_TIME_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_TIME,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_TimeGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_TimeGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_TimeOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_TimeOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_TimeValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_TIME_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 66,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* NOTE: Common help list method not used here because of non-linear
                 item position. */

        helpListPars = listPars;

        helpListPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                       SCALER_GetCurrVal(MAIN_WIDTH) +
                                       SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                       SCALER_GetCurrVal(HELP_LR_PADDING) +
                                       stackedFrameWidth;
        helpListPars.pos.width       = helpWidth - 2 * SCALER_GetCurrVal(HELP_LR_PADDING);
        helpListPars.bkgrType        = 0;
        helpListPars.getDescrItem    = MENU_TimeGetHelpDescrItem;
        helpListPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        helpListPars.getItem         = NULL;
        helpListPars.getItemArg      = NULL;
        helpListPars.onFocusChg      = NULL;
        helpListPars.onFocusChgArg   = NULL;
        helpListPars.onSelect        = NULL;
        helpListPars.onSelectArg     = NULL;
        helpListPars.onValChg        = NULL;
        helpListPars.onValChgArg     = NULL;
        helpListPars.pInst           = &(appDat->helpListInst);

        retVal = FGS_LIST_Open (&helpListPars, MENU_TIME_ID_HELPLIST,
                                inst->pnlHdl, &(inst->listHelpHdl));
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.x              = 0;
        helpPars.pos.y              = 0;
        helpPars.pos.width          = helpListPars.pos.width;
        helpPars.pos.height         = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        helpPars.pos.alignment      = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        helpPars.bkgrType           = OSD_LSTITEM_BKGR_HELP;
        helpPars.descrPos.x         = 0;
        helpPars.descrPos.y         = 0;
        helpPars.descrPos.width     = helpPars.pos.width;
        helpPars.descrPos.height    = helpPars.pos.height;
        helpPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----- menu lines ------------------------------------------------ */

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.descrPos.width     = itemPos.width - MAIN_LEFT_PADDING -
                                     MAIN_RIGHT_PADDING;
        txtPars.pos                = itemPos;

        /* ----- msg line -------------------------------------------------- */
        if ( inst->params.strIdx != 0 )
        {
            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TIME_ID_ENTRY_MSG1,
                                    inst->listHdl, &(inst->itemHdl));
            if ( retVal != FAPI_OK ) break;

            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                           MENU_TIME_ID_ENTRY_MSG1,
                                           FFALSE);

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                         SCALER_GetCurrVal(LIST_LINE_PADDING);

            txtPars.pos = itemPos;
            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TIME_ID_ENTRY_MSG2,
                                    inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                            MENU_TIME_ID_ENTRY_MSG2,
                                           FFALSE);

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                         SCALER_GetCurrVal(LIST_LINE_PADDING);
        }
        /* --- Automatic Time ---------------------------------------------*/

        enabled = TIME_IsAutoMode();

        selPars.pos             = itemPos;
        selPars.bkgrType        = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos        = descrPos;
        selPars.valuePos        = valuePos;
        selPars.params.numItems = 2;
        selPars.params.index    = ( (enabled == FTRUE) ?
                                    TIME_AUTO_ENABLED :
                                    TIME_AUTO_DISABLED);

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIME_ID_ENTRY_AUTO_TIME,
                                    inst->listHdl,&(inst->itemHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }
        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TIME_ID_ENTRY_AUTO_TIME,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);
        /* --- user set Date ------------------------------------------------*/
        numPars.pos                 = itemPos;
        numPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        numPars.descrPos            = descrPos;
        numPars.valuePos            = valuePos;
        numPars.type                = FGS_NITEM_TYPE_DATE;
        numPars.wraparound          = FFALSE;
        numPars.params.minValue  = 0;
        numPars.params.maxValue  = 23;
        numPars.params.maxDigits = 2;
        numPars.params.step      = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TIME_ID_ENTRY_DATE,
                                    inst->listHdl,&(inst->itemHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }
        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TIME_ID_ENTRY_DATE,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* if AUTO_TIME is enabled, disable manual settings */
        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                        MENU_TIME_ID_ENTRY_DATE,
                                        (fbool_t)!enabled);

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);
        /* --- user set TIME ------------------------------------------------*/
        numPars.pos                 = itemPos;
        numPars.type                = FGS_NITEM_TYPE_TIME;
        numPars.params.minValue  = 0;
        numPars.params.maxValue  = 23;
        numPars.params.maxDigits = 2;
        numPars.params.step      = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TIME_ID_ENTRY_TIME,
                                    inst->listHdl,&(inst->itemHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }
        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TIME_ID_ENTRY_TIME,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* if AUTO_TIME is enabled, disable manual settings */
        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                        MENU_TIME_ID_ENTRY_TIME,
                                        (fbool_t)!enabled);

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Daylight saving time ------------------------------------*/

        (void)SETTINGS_TimeOffsetGet (&utcOffset, &dstActive);
        selPars.pos             = itemPos;
        selPars.params.numItems = 2;
        selPars.params.index    = dstActive;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIME_ID_ENTRY_DST,
                                    inst->listHdl,&(inst->itemHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }
        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TIME_ID_ENTRY_DST,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* if AUTO_TIME is enabled, disable manual settings */

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Allow Automatic Offset -----------------------------------*/

        (void)SETTINGS_AutoTimeOffsetGet(&enabled, &ctryCode);

        selPars.pos             = itemPos;
        selPars.bkgrType        = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos        = descrPos;
        selPars.valuePos        = valuePos;
        selPars.params.numItems = 2;
        selPars.params.index    = ( (enabled == FTRUE) ?
                                    TIME_AUTO_OFFSET_ENABLED :
                                    TIME_AUTO_OFFSET_DISABLED);

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIME_ID_ENTRY_AUTO_OFFSET,
                                    inst->listHdl,&(inst->itemHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }
        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TIME_ID_ENTRY_AUTO_OFFSET,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Region --------------------------------------------*/

        timeArray = MENU_TimeGetArrayByGmtOffset(utcOffset);
        if(timeArray == 0) break;

        selPars.pos             = itemPos;
        selPars.bkgrType        = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos        = descrPos;
        selPars.valuePos        = valuePos;
        selPars.params.numItems = TIME_REGION_IDX_NUMS;
        selPars.params.index    = timeArray->regionIdx;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIME_ID_ENTRY_REGION,
                                    inst->listHdl,&(inst->itemHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }
        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TIME_ID_ENTRY_REGION,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* if AUTO_TIME_OFFSET is enabled, disable manual settings */
        (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                       MENU_TIME_ID_ENTRY_REGION,
                                       (fbool_t)!enabled);

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- GMT offset ------------------------------------------------*/
        selPars.pos             = itemPos;
        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIME_ID_ENTRY_GMT,
                                    inst->listHdl,&(inst->itemHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }

        (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl,
                                       MENU_TIME_ID_ENTRY_GMT,
                                       FFALSE);

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* -----Bottom help lines ------------------------------------------ */
        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    return FTRUE;
}

/*!
*******************************************************************************
** \brief   TODO
**
*******************************************************************************
*/
void MENU_TimeUpdate(void)
{
    MENU_TIME_INST_S*     inst = menuTimeDat;

    if( ( inst           == NULL ) ||
        ( inst->pnlHdl   == NULL ) ||
        ( inst->listHdl  == NULL ) ||
        ( inst->hasFocus == FFALSE))
    {
        return;
    }

    if ( ( inst->params.strIdx != 0     ) &&
         ( TIME_IsAutoMode()   == FTRUE ) &&
         ( TIME_IsSync()       == FTRUE ) )
    {
        inst->params.done = FTRUE;
        (void)MENU_TimeHandleEvt(&(inst->params));
    }
    else
    {
        (void)FGS_LIST_RefreshMember (inst->listHdl, MENU_TIME_ID_ENTRY_TIME);
        (void)FGS_LIST_RefreshMember (inst->listHdl, MENU_TIME_ID_ENTRY_DATE);
    }
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   params  TODO
**
** \return  One of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
*******************************************************************************
*/
fbool_t MENU_TimeHandleEvt(MENU_TIME_OPEN_S* params)
{
    MENU_TIME_INST_S*     inst = menuTimeDat;
    fbool_t retVal = FTRUE;

    if(inst == NULL )
    {
        return FFALSE;
    }

    if(params == NULL)
    {
        return MENUSTACK_Exit(NULL);
    }

    if( ( params->done  == FFALSE ) ||
        ( inst->pnlHdl  == NULL ) )
    {
        if( inst->pnlHdl == NULL)
            retVal = MENUSTACK_Up(MENU_TimeEntry,(FAPI_SYS_HandleT)params);
    }
    else
    {
        retVal = FFALSE;
        if(TIME_IsSync())
        {
            switch(params->next)
            {
                case MENU_TIME_NEXT_EPG:
                    retVal = MENUSTACK_Switch(MENU_EpgGridEntry,NULL,-1);
                    break;
                case MENU_TIME_NEXT_START_TIMER_REC:
                    retVal = MENUSTACK_Back(NULL);
                    break;
                case MENU_TIME_NEXT_START_INSTANT_REC:
                    (void)MENUSTACK_Back(NULL);
                    #ifdef APPL_PVR_ENABLED
                    retVal = MAIN_PvrStartInstantRecording();
                    #endif
                    break;
                case MENU_TIME_NEXT_TIMERPROG_MENU:
                    #ifdef APPL_PVR_ENABLED
                    retVal = MENUSTACK_Switch(MENU_TimerProgEntry,NULL,-1);
                    #endif
                    break;
            }
        }
    }
    return retVal;
}

/*!
*******************************************************************************
** \brief   TODO
**
** \param   gmtOffset   TODO
**
** \return  TODO
**
*******************************************************************************
*/
FSTATIC OSDTIME_ENTRY_S* MENU_TimeGetArrayByGmtOffset(int16_t gmtOffset)
{
    uint8_t i;

    for(i=0; i<TIME_REGION_IDX_NUMS; i++)
    {
        if(menuTimeZoneArray[i].gmtOffset == gmtOffset)
            return &(menuTimeZoneArray[i]);
    }
    return 0;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
