/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_timeredit.c
**
**              $Id: app_menu_timeredit.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Osd Timer edit menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#if (defined APPL_HDD_ENABLED) || (defined APPL_USB_ENABLED)

    #define MENU_TIMEREDIT_RECTO_AVL

#endif /* (defined APPL_HDD_ENABLED) || (defined APPL_USB_ENABLED) */

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_TIMEREDIT_ID_TITLE,
    MENU_TIMEREDIT_ID_LIST,
    MENU_TIMEREDIT_ID_HELPLIST,
    MENU_TIMEREDIT_ID_HELPBOTTOM
};

enum
{
    MENU_TIMEREDIT_ID_ENTRY_NAME = 0,
    MENU_TIMEREDIT_ID_ENTRY_DATE,
    MENU_TIMEREDIT_ID_ENTRY_STIME,
    MENU_TIMEREDIT_ID_ENTRY_ETIME,
    MENU_TIMEREDIT_ID_ENTRY_PROG,
    MENU_TIMEREDIT_ID_ENTRY_MODE,
    MENU_TIMEREDIT_ID_ENTRY_PWRMODE,

    #ifdef MENU_TIMEREDIT_RECTO_AVL
    MENU_TIMEREDIT_ID_ENTRY_RECTO,
    #endif /* MENU_TIMEREDIT_RECTO_AVL */

    MENU_TIMEREDIT_ID_ENTRY_NUMS
};


typedef struct
{
    SYS_DEVICE_E        device;
    fbool_t             available;

} MENU_TIMEREDIT_DEV_S;

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    listHdl;
    FAPI_SYS_HandleT    listHelpHdl;

    int16_t             timerIndex;
    RECTIMER_ITEM_S     timerData;
    SYS_SERVLOC_S       timerService;
    int16_t             utcOffset;      // UTC offset + DST in minutes
    fbool_t             valChanged;     // indication that a value has beeen changed
    fbool_t             dataChanged;    // indication that timer data was saved
    fbool_t             forceSave;      // indication that timer date must be saved
    //fbool_t             directCall;     // indication that menu was called directly
                                        // by instant record request

    #ifdef MENU_TIMEREDIT_RECTO_AVL

    MENU_TIMEREDIT_DEV_S devList[SYS_DEV_NUMS]; //!< List of (currently)
                                                //!< available devices.
    uint32_t            devNums;                //!< Number of devices in list.
    uint32_t            devFirstIdx;            //!< Initial list index.

    #endif /* MENU_TIMEREDIT_RECTO_AVL */


} MENU_TIMEREDIT_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC MENU_TIMEREDIT_INST_S* menuTimerEditDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t MENU_TimerEditLeave       (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC void    MENU_TimerEditOnFocusSet  (FAPI_SYS_HandleT h,
                                           fbool_t hasFocus);
FSTATIC void    MENU_TimerEditGetDescrItem(FAPI_SYS_HandleT h, int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);
FSTATIC void    MENU_TimerEditGetItem     (FAPI_SYS_HandleT h, int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);
FSTATIC void    MENU_TimerEditGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);
FSTATIC void    MENU_TimerEditStore       (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_TimerEditOnSelect    (FAPI_SYS_HandleT h, int32_t id);
FSTATIC void    MENU_TimerEditOnValChg    (FAPI_SYS_HandleT h, int32_t itemId,
                                           FGS_VAL_S newVal);
FSTATIC void    MENU_TimerEditGetRepMode  (FAPI_SYS_HandleT h, int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);
FSTATIC void    MENU_TimerEditGetService  (FAPI_SYS_HandleT h, int32_t id,
                                           uint32_t index,
                                           FGS_ITEMSTATE_E state,
                                           FGS_ITEM_S* item);
FSTATIC fbool_t MENU_TimerEditBuild       (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_TimerEditKeys        (FAPI_SYS_HandleT h);
FSTATIC void    MENU_TimerEditOnFocusChg  (FAPI_SYS_HandleT h, uint8_t index,
                                           int32_t id);
FSTATIC void    MENU_TimerEditConfirm     (void* arg);
FSTATIC void    MENU_TimerEditAbort       (void* arg);
FSTATIC void    MENU_TimerEditStringExit  (MENU_EDITSTRING_DATA_S* editData,
                                           int32_t id);


int32_t MENU_TimerEditInit (void)
{
    if ( menuTimerEditDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuTimerEditDat = UTILS_Malloc(sizeof(MENU_TIMEREDIT_INST_S));
    if ( menuTimerEditDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuTimerEditDat, 0, sizeof(MENU_TIMEREDIT_INST_S));

    return FAPI_OK;
}

fbool_t MENU_TimerEditEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_TIMEREDIT_INST_S*    inst = menuTimerEditDat;
    uint32_t                  lastFocusItem;
    MENU_TIMEREDIT_PARAMS_S*  pParams = (MENU_TIMEREDIT_PARAMS_S*)h;
    SLIST_ITEM_S              serviceItem;
    fbool_t                   dstActive;
    SYS_DEVICE_E              devCnt;

    #ifdef MENU_TIMEREDIT_RECTO_AVL
    uint32_t                  cnt;
    SYS_DEV_STAT_S            devStat;
    #endif /* MENU_TIMEREDIT_RECTO_AVL */

    inst->pnlHdl      = NULL;

    inst->timerIndex  = pParams->timerIndex;
    inst->dataChanged = FFALSE;
    inst->forceSave   = FFALSE;

    if ( inst->timerIndex >= 0 )
    {
        /* edit existing timer: get timer data */
        if ( RECTIMER_Get (inst->timerIndex, &(inst->timerData)) != FAPI_OK )
        {
            return FFALSE;
        }

        inst->timerService = SLIST_GetListReference(inst->timerData.
                                                    serviceIndex);
        inst->valChanged   = FFALSE;
    }
    else
    {
        /* edit new timer: store passed timer data */
        inst->timerData    = *(pParams->newDat);
        inst->timerService = pParams->newService;
        inst->valChanged   = FTRUE;
    }

    if ( SLIST_EntryGet (&(inst->timerService), &serviceItem) != FAPI_OK )
    {
        MENU_CommonMsgShow(STR_MSG_NOSERVICES, FTRUE);

        (void)RECTIMER_Save(FFALSE);

        (void)MAIN_MsgBoxHide(&(appDat->msgBox));

        return FFALSE;
    }

    if ( test == FTRUE ) return FTRUE;

    /* ----- Create list of available devices ------------------------------ */

    #ifdef MENU_TIMEREDIT_RECTO_AVL

    inst->devNums = 0;

    /* Populate device list with available devices. In order to also show a
       target device which is currently not available (but has been selected as
       target) it is added to the list in any case, but with a flag
       indicating its availability. */

    for ( devCnt = (SYS_DEVICE_E)0; devCnt < SYS_DEV_NUMS; devCnt++ )
    {
        if ( SYS_DeviceGetStatus (devCnt, &devStat) == FAPI_OK )
        {
            if ( devStat.mountStatus == SYS_DEV_FS_MOUNTED )
            {
                inst->devList[inst->devNums].device    = devCnt;
                inst->devList[inst->devNums].available = FTRUE;

                inst->devNums++;
            }
        }
    }

    /* choose initial selection index from requested device */

    for ( cnt = 0; cnt < inst->devNums; cnt++ )
    {
        if ( inst->devList[cnt].device == inst->timerData.targetDevice )
        {
            inst->devFirstIdx = cnt;
            break;
        }
    }

    if ( cnt >= inst->devNums )
    {
        /* requested device is not available, add it to the list! */

        inst->devList[inst->devNums].device    = inst->timerData.targetDevice;
        inst->devList[inst->devNums].available = FFALSE;

        inst->devFirstIdx = inst->devNums;

        inst->devNums++;
    }

    #endif /* MENU_TIMEREDIT_RECTO_AVL */

    /* --------------------------------------------------------------------- */

    /* convert from UTC to local time */
    (void)SETTINGS_TimeOffsetGet (&(inst->utcOffset), &dstActive);

    if ( dstActive ) inst->utcOffset += 60;

    TIME_AddSeconds (&(inst->timerData.startTime),
                     &(inst->timerData.startDate), (int32_t)inst->utcOffset * 60);
    TIME_AddSeconds (&(inst->timerData.endTime),
                     &(inst->timerData.endDate), (int32_t)inst->utcOffset * 60);

    (void)MENU_TimerEditBuild((FAPI_SYS_HandleT)inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = MENU_TIMEREDIT_ID_ENTRY_NAME;
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_TIMEREDIT_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_TIMEREDIT);       /* set global focus on menu panel */
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_TimerEditLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

FSTATIC void MENU_TimerEditOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)h;

    DBG_Assert(inst->pnlHdl != NULL);

    if ( ( inst == NULL ) ||
         ( inst->pnlHdl == NULL ) )
    {
        return;
    }
    if ( hasFocus )
    {
        FGS_SetAppKeyHandler (FGS_KEY_MENU, MENU_TimerEditKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_BACK, MENU_TimerEditKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler (FGS_KEY_EXIT, MENUSTACK_Exit,  NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RECORD, MENU_TimerEditKeys, h, FTRUE,FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler(FGS_KEY_RECORD);
    }
}

FSTATIC void MENU_TimerEditOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)h;
    FGS_LNAVI_PARAMS_S     naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}

FSTATIC fbool_t MENU_TimerEditLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)h;

    if ( inst->forceSave )
    {
        MENU_TimerEditStore((FAPI_SYS_HandleT)inst);
    }

    if ( !force )
    {
        if ( inst->dataChanged && (inst->forceSave == FTRUE) )
        {
            MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

            (void)RECTIMER_Save(FFALSE);

            (void)MAIN_MsgBoxHide(&(appDat->msgBox));

            inst->dataChanged = FFALSE;
        }
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_TIMEREDIT);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC fbool_t MENU_TimerEditOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_TIMEREDIT_INST_S*  inst = (MENU_TIMEREDIT_INST_S*)h;
    MENUSTACK_ENTRY         newEntry = NULL;
    FAPI_SYS_HandleT        arg      = NULL;
    uint8_t                 subLines = 0;
    MENU_SELECTLIST_OPEN_S  selectLst;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    fbool_t                 keyUsed = FTRUE;
    FGS_LOC_S               memberLoc;
    fbool_t                 openSelectLst = FFALSE;
    MENU_EDITSTRING_OPEN_S  editParams;
    uint16_t                stackedFrameWidth;

    stackedFrameWidth   = (MENUSTACK_GetLevel() - 1)
                          * (SCALER_GetCurrVal(STACKED_FRAME_DIST)
                            + SCALER_GetCurrVal(OUTER_FRAME_ALL));
    switch ( itemId )
    {
        case MENU_TIMEREDIT_ID_ENTRY_NAME:
            if ( &(appDat->pnlInst) != NULL)
            {
                (void)FGS_PANEL_GetPosition (inst->pnlHdl, &(editParams.pos));

                editParams.pos.y  += stackedFrameWidth
                                   + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                   + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                   + SCALER_GetCurrVal(MAIN_TOP_PADDING)
                                   + (2*SCALER_GetCurrVal(TITLE_TB_PADDING))
                                   + SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                                   + SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                   + SCALER_GetCurrVal(LIST_LINE_PADDING);
            }
            editParams.editTitle = NULL;
            editParams.editTitleArg = NULL;
            editParams.onExit = MENU_TimerEditStringExit;
            if(inst->timerData.descCharSet == SYS_CHARSET_UNDEFINED)
            {
                inst->timerData.descCharSet = SYS_CHARSET_ISO_10646;
            }
            editParams.editData.string   = STR_Build(inst->timerData.descCharSet,
                                                     inst->timerData.desc);
            editParams.editData.maxChars = RECTIMER_DESC_LENGTH - 1;
            editParams.id               = MENU_TIMEREDIT_ID_LIST;

            keyUsed = MENU_EditStringOpen (&(appDat->editStringBox),&editParams);
            return keyUsed;

        case MENU_TIMEREDIT_ID_ENTRY_MODE :
            openSelectLst = FTRUE;
            subLines      = 3;
            break;

        case MENU_TIMEREDIT_ID_ENTRY_PWRMODE :
            openSelectLst = FTRUE;
            subLines      = 2;
            break;

        #ifdef MENU_TIMEREDIT_RECTO_AVL

        case MENU_TIMEREDIT_ID_ENTRY_RECTO :
            openSelectLst = FTRUE;
            subLines      = 2;
            break;

        #endif /* MENU_TIMEREDIT_RECTO_AVL */

        default:
            return FFALSE;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.x         = memberLoc.x + SCALER_GetCurrVal(MAIN_WIDTH) -
                                  SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.width     = SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_TimerEditGetItem;
        selectLst.getItemArg    = (FAPI_SYS_HandleT)inst;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;

        keyUsed = MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);

        if ( keyUsed )
        {
            (void)MAIN_SignalBoxPause(&(appDat->sigBox));
        }

        return keyUsed;
    }
    else
    {
        /* store last focus */
        MENUSTACK_SetData(itemId);
        if(newEntry == NULL)
            return FFALSE;
        else
            return MENUSTACK_Up(newEntry, arg);
    }
}


FSTATIC void MENU_TimerEditOnValChg (FAPI_SYS_HandleT h, int32_t itemId,
                                     FGS_VAL_S newVal)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)h;

    switch ( itemId )
    {
        case MENU_TIMEREDIT_ID_ENTRY_DATE:
            inst->timerData.startDate = newVal.date;
            inst->valChanged = FTRUE;
            break;

        case MENU_TIMEREDIT_ID_ENTRY_STIME:
            inst->timerData.startTime = newVal.time;
            inst->valChanged = FTRUE;
            break;

        case MENU_TIMEREDIT_ID_ENTRY_ETIME:
            inst->timerData.endTime = newVal.time;
            inst->valChanged = FTRUE;
            break;

        case MENU_TIMEREDIT_ID_ENTRY_PROG:
            if ( inst->timerService.listId != SLIST_INVISIBLE )
            {
                inst->timerService.index = newVal.number;
            }

            inst->valChanged = FTRUE;
            break;

        case MENU_TIMEREDIT_ID_ENTRY_MODE:
            inst->timerData.type = (RECTIMER_TYPE_E)newVal.number;
            inst->valChanged = FTRUE;
            break;

        case MENU_TIMEREDIT_ID_ENTRY_PWRMODE:
            inst->timerData.autoStandby = ((newVal.number == 0) ? FFALSE
                                                                : FTRUE);
            inst->valChanged = FTRUE;
            break;

        #ifdef MENU_TIMEREDIT_RECTO_AVL

        case MENU_TIMEREDIT_ID_ENTRY_RECTO:
            if ( (newVal.number < (int32_t)inst->devNums)
              && (inst->devList[newVal.number].available) )
            {
                inst->timerData.targetDevice = inst->devList[newVal.number].
                                               device;

                inst->valChanged = FTRUE;
            }
            break;

        #endif /* MENU_TIMEREDIT_RECTO_AVL */

        default:
            break;
    }
}

FSTATIC void MENU_TimerEditGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_TIMEREDIT_ID_ENTRY_NAME   : strIdx = STR_ENTRY_TIMEREDIT_NAME;    break;
        case MENU_TIMEREDIT_ID_ENTRY_DATE   : strIdx = STR_ENTRY_TIMEREDIT_DATE;    break;
        case MENU_TIMEREDIT_ID_ENTRY_STIME  : strIdx = STR_ENTRY_TIMEREDIT_STIME;   break;
        case MENU_TIMEREDIT_ID_ENTRY_ETIME  : strIdx = STR_ENTRY_TIMEREDIT_ETIME;   break;
        case MENU_TIMEREDIT_ID_ENTRY_PROG   : strIdx = STR_ENTRY_TIMEREDIT_PROG;    break;
        case MENU_TIMEREDIT_ID_ENTRY_MODE   : strIdx = STR_ENTRY_TIMEREDIT_MODE;    break;
        case MENU_TIMEREDIT_ID_ENTRY_PWRMODE: strIdx = STR_ENTRY_TIMEREDIT_PWRMODE; break;

        #ifdef MENU_TIMEREDIT_RECTO_AVL
        case MENU_TIMEREDIT_ID_ENTRY_RECTO  : strIdx = STR_ENTRY_TIMEREDIT_RECTO;   break;
        #endif /* MENU_TIMEREDIT_RECTO_AVL */
        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_TimerEditGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                             uint32_t index,
                                             FGS_ITEMSTATE_E state,
                                             FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        case MENU_TIMEREDIT_ID_ENTRY_NAME:    strIdx = STR_ENTRY_OK;   break;
        case MENU_TIMEREDIT_ID_ENTRY_DATE:
        case MENU_TIMEREDIT_ID_ENTRY_STIME:
        case MENU_TIMEREDIT_ID_ENTRY_ETIME:   strIdx = STR_ENTRY_09LR; break;
        case MENU_TIMEREDIT_ID_ENTRY_PROG:    strIdx = STR_ENTRY_LR;   break;
        case MENU_TIMEREDIT_ID_ENTRY_MODE:

        #ifdef MENU_TIMEREDIT_RECTO_AVL
        case MENU_TIMEREDIT_ID_ENTRY_RECTO:
        #endif /* MENU_TIMEREDIT_RECTO_AVL */

        case MENU_TIMEREDIT_ID_ENTRY_PWRMODE: strIdx = STR_ENTRY_LOKR; break;
        default:  return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC void MENU_TimerEditGetItem (FAPI_SYS_HandleT h, int32_t id,
                                    uint32_t index, FGS_ITEMSTATE_E state,
                                    FGS_ITEM_S* item)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)h;
    uint32_t               itemCol;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: itemCol = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    itemCol = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: itemCol = appDat->txtAttr.colDisabled; break;
    }

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    switch ( id )
    {
        case MENU_TIMEREDIT_ID_ENTRY_NAME:
            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.col        = itemCol;
            item->item.str.chr        = STR_Build (inst->timerData.descCharSet,
                                                   inst->timerData.desc);
            break;

        case MENU_TIMEREDIT_ID_ENTRY_DATE:
            item->type             = FGS_ITEM_TYPE_DATE;
            item->item.date.value  = inst->timerData.startDate;
            item->item.date.format = appDat->dataFormatNorm;
            item->item.date.font   = MAIN_FONT;
            item->item.date.col    = itemCol;
            break;

        case MENU_TIMEREDIT_ID_ENTRY_STIME:
            item->type             = FGS_ITEM_TYPE_TIME;
            item->item.time.value  = inst->timerData.startTime;
            item->item.time.format = appDat->timeFormat;
            item->item.time.font   = MAIN_FONT;
            item->item.time.col    = itemCol;
            break;

        case MENU_TIMEREDIT_ID_ENTRY_ETIME:
            item->type             = FGS_ITEM_TYPE_TIME;
            item->item.time.value  = inst->timerData.endTime;
            item->item.time.format = appDat->timeFormat;
            item->item.time.font   = MAIN_FONT;
            item->item.time.col    = itemCol;
            break;

        case MENU_TIMEREDIT_ID_ENTRY_PROG:
            MENU_TimerEditGetService(h, 0, index, state, item);
            break;

        case MENU_TIMEREDIT_ID_ENTRY_MODE:
            MENU_TimerEditGetRepMode(h, 0, index, state, item);
            break;

        case MENU_TIMEREDIT_ID_ENTRY_PWRMODE:
            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.col        = itemCol;

            if ( index == 0 )
            {
                item->item.str.chr = OSD_GetStr(
                                        STR_VALUE_TIMEREDIT_PWRMODE_PWRUP);
            }
            else
            {
                item->item.str.chr = OSD_GetStr(
                                        STR_VALUE_TIMEREDIT_PWRMODE_STDBY);
            }
            break;

        #ifdef MENU_TIMEREDIT_RECTO_AVL

        case MENU_TIMEREDIT_ID_ENTRY_RECTO:
            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;

            if ( index >= inst->devNums )
            {
                item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            }
            else
            {
                switch ( inst->devList[index].device )
                {
                    #ifdef APPL_HDD_ENABLED

                    case SYS_DEV_HDD:
                        item->item.str.chr = OSD_GetStr(
                                             STR_VALUE_TIMEREDIT_RECTO_INTHD);
                        break;

                    #endif /* APPL_HDD_ENABLED */

                    #ifdef APPL_USB_ENABLED

                    case SYS_DEV_USB:
                        item->item.str.chr = OSD_GetStr(
                                             STR_VALUE_TIMEREDIT_RECTO_USB);
                        break;

                    #endif /* APPL_USB_ENABLED */

                    default:
                        item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                        break;
                }

                if  ( !(inst->devList[index].available) )
                {
                    itemCol = appDat->txtAttr.colDisabled;
                }
            }

            item->item.str.col = itemCol;
            break;

        #endif /* MENU_TIMEREDIT_RECTO_AVL */

        default:
            break;
    }
}

FSTATIC fbool_t MENU_TimerEditBuild(FAPI_SYS_HandleT h)
{
    MENU_TIMEREDIT_INST_S*  inst = (MENU_TIMEREDIT_INST_S*)h;
    FGS_PANEL_OPEN_S        pnlPars;
    FGS_LIST_OPEN_S         listPars;
    FAPI_SYS_HandleT        itemHdl;
    FGS_BOX_OPEN_S          boxPars;
    FGS_ITEM_NUM_OPEN_S     numPars;
    FGS_ITEM_SEL_OPEN_S     selPars;
    FGS_POS_S               itemPos;
    FGS_ITEM_S              item;
    FGS_POS_S               valuePos;
    FGS_POS_S               descrPos;
    int32_t                 retVal;
    uint16_t                helpWidth;
    uint16_t                mainPanelWidth;
    uint16_t                bottomHelpKeyWidth;
    uint16_t                bottomHelpItemWidth;
    uint16_t                stackedFrameWidth;
    uint16_t                innerHeight;
    uint32_t                numLines     = MENU_TIMEREDIT_ID_ENTRY_NUMS;
    uint8_t                 numHelpLines = 2;
    uint16_t                idCnt;

    memset(&boxPars,  0, sizeof(boxPars));
    memset(&numPars,  0, sizeof(numPars));
    memset(&selPars,  0, sizeof(selPars));
    memset(&item,     0, sizeof(item));

    helpWidth         = (2 * SCALER_GetCurrVal(HELP_LR_PADDING))
                        + OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_09LR),
                                          -1, SYMBOL_FONT);
    innerHeight = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, numHelpLines,
                                        &stackedFrameWidth);

        mainPanelWidth          = pnlPars.pos.width
                                - stackedFrameWidth
                                - (2 * SCALER_GetCurrVal(OUTER_FRAME_ALL));
        bottomHelpKeyWidth      = OSD_GetStrWidth(OSD_GetStr(STR_KEY_MENU),-1,HELP_FONT)
                                + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        bottomHelpItemWidth     = (mainPanelWidth / 2)
                                - bottomHelpKeyWidth
                                - SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

        pnlPars.onFocusSet      = MENU_TimerEditOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_TIMEREDIT, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle (MENU_TIMEREDIT_ID_TITLE,
                                      inst->pnlHdl, pnlPars.pos.width,
                                      stackedFrameWidth, STR_TITLE_TIMEREDIT,
                                      NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_TimerEditGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_TimerEditGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_TimerEditOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_TimerEditOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_TimerEditOnValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_TIMEREDIT_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* --- TIMEREDIT Name ------------------------------------------------*/

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = 1;
        selPars.params.index        = 0;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIMEREDIT_ID_ENTRY_NAME,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Start Date --------------------------------------------*/

        numPars.pos                 = itemPos;
        numPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        numPars.descrPos            = descrPos;
        numPars.valuePos            = valuePos;
        numPars.type                = FGS_NITEM_TYPE_DATE;
        numPars.wraparound          = FFALSE;
        numPars.params.minValue  = 0;
        numPars.params.maxValue  = 23;
        numPars.params.maxDigits = 2;
        numPars.params.step      = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TIMEREDIT_ID_ENTRY_DATE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Start Time ------------------------------------------------ */

        numPars.pos              = itemPos;
        numPars.type             = FGS_NITEM_TYPE_TIME;
        numPars.wraparound       = FFALSE;
        numPars.params.minValue  = 0;
        numPars.params.maxValue  = 23;
        numPars.params.maxDigits = 2;
        numPars.params.step      = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TIMEREDIT_ID_ENTRY_STIME,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- End Time -------------------------------------------------- */

        numPars.pos              = itemPos;
        numPars.wraparound       = FFALSE;
        numPars.params.minValue  = 0;
        numPars.params.maxValue  = 59;
        numPars.params.maxDigits = 2;
        numPars.params.step      = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TIMEREDIT_ID_ENTRY_ETIME,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Service --------------------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;

        if ( inst->timerService.listId != SLIST_INVISIBLE )
        {
            selPars.params.numItems = SLIST_ListSizeGet(inst->timerService.
                                                        listId);
            selPars.params.index    = inst->timerService.index;
        }
        else
        {
            selPars.params.numItems = 1;
            selPars.params.index    = 0;
        }

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIMEREDIT_ID_ENTRY_PROG,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Timer Mode/Type ------------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = RECTIMER_TYPE_NUMS;
        selPars.params.index        = inst->timerData.type;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIMEREDIT_ID_ENTRY_MODE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Auto-Standby ---------------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = 2;
        selPars.params.index        = inst->timerData.autoStandby ? 1 : 0;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIMEREDIT_ID_ENTRY_PWRMODE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- TIMEREDIT Record to------------------------------------*/

        #ifdef MENU_TIMEREDIT_RECTO_AVL

        selPars.pos                 = itemPos;
        selPars.params.numItems     = inst->devNums;
        selPars.params.index        = inst->devFirstIdx;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TIMEREDIT_ID_ENTRY_RECTO,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* MENU_TIMEREDIT_RECTO_AVL */

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_TIMEREDIT_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH),
                                        listPars.pos.y, helpWidth, numLines,
                                        MENU_TimerEditGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }
        /* REC */
        idCnt     = MENU_TIMEREDIT_ID_HELPBOTTOM;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_REC);

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = - (int16_t)(SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2)
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - (numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT))
                                      - ((numHelpLines-1)*SCALER_GetCurrVal(HELP_LINE_PADDING));
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Colon ------------------------------------------ */
        item.item.str.chr           = OSD_GetStr(STR_GNR_COLON);
        boxPars.pos.x              += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_TIMEREDIT_SAVEANDEXIT);
        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 2nd Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                boxPars.pos.y,
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                (2 * SCALER_GetCurrVal(OUTER_FRAME_ALL)) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    return FTRUE;
}

FSTATIC fbool_t MENU_TimerEditKeys (FAPI_SYS_HandleT h)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)h;
    fbool_t                  keyUsed  = FTRUE;
    MENU_CONFDIALOG_S      confPars;

    if ( inst == NULL ) return FFALSE;

    switch ( FGS_GetCurrentKeyCode() )
    {

        case FGS_KEY_RECORD:
            inst->forceSave = FTRUE;
            keyUsed = MENUSTACK_Back(NULL);
            break;

        case FGS_KEY_MENU:
            /* FALLTHROUGHT */
        case FGS_KEY_BACK:
            if(inst->valChanged)
            {
                /* open a confirm dialog to re-confirm the user's action */
                memset(&confPars,  0, sizeof(confPars));

                confPars.numDescLines = 1;
                confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_SAVE_TIMER);
                confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_SAVE_TIMER_OK);
                confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_SAVE_TIMER_ABORT);
                confPars.onConfirm    = MENU_TimerEditConfirm;
                confPars.onDecline    = NULL;
                confPars.onDiscard    = MENU_TimerEditAbort;
                confPars.arg          = inst;

                (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
            }
            else
            {
                inst->forceSave = FFALSE;
                return MENUSTACK_Back(NULL);
            }
            break;
        default:
            keyUsed = FFALSE;
            break;
    }
    return keyUsed;
}


FSTATIC void MENU_TimerEditStore (FAPI_SYS_HandleT h)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)h;
    int16_t newPos;

    if ( !(inst->valChanged) ) return;

    /* ----- add or replace timer ------------------------------------------ */

    /* NOTE: Maximum recording time with this implementation is 24 hours! */

    inst->timerData.endDate = inst->timerData.startDate;

    /* test for date wrapround */
    if ( TIME_IsLaterThan (&(inst->timerData.startTime),
                           &(inst->timerData.startDate),
                           &(inst->timerData.endTime),
                           &(inst->timerData.endDate)) == FTRUE )
    {
        /* add 24 hours to end time */
        TIME_AddDays (&(inst->timerData.endDate), 1);
    }

    /* convert from local time to UTC */
    TIME_AddSeconds (&(inst->timerData.startTime),
                     &(inst->timerData.startDate), -((int32_t)inst->utcOffset * 60));
    TIME_AddSeconds (&(inst->timerData.endTime),
                     &(inst->timerData.endDate), -((int32_t)inst->utcOffset * 60));

    inst->timerData.serviceIndex = SLIST_GetServiceIndex(&(inst->timerService));

    if ( inst->timerIndex >= 0 )
    {
        if ( RECTIMER_Replace (inst->timerIndex, &(inst->timerData)) == FAPI_OK )
        {
            inst->dataChanged = FTRUE;
        }
    }
    else
    {
        if ( RECTIMER_Add (&(inst->timerData), &newPos) == FAPI_OK )
        {
            inst->dataChanged = FTRUE;
        }
    }
}

FSTATIC void MENU_TimerEditGetRepMode (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                   FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
    }

    switch ( (RECTIMER_TYPE_E)index )
    {
        case RECTIMER_TYPE_SINGLE:   strIdx = STR_VALUE_TIMEREDIT_MODE_SINGLE;   break;
        case RECTIMER_TYPE_WEEKDAYS: strIdx = STR_VALUE_TIMEREDIT_MODE_WEEKDAYS; break;
        case RECTIMER_TYPE_WEEKLY:   strIdx = STR_VALUE_TIMEREDIT_MODE_WEEKLY;   break;
        case RECTIMER_TYPE_WEEKENDS: strIdx = STR_VALUE_TIMEREDIT_MODE_WEEKENDS; break;
        case RECTIMER_TYPE_DAILY:    strIdx = STR_VALUE_TIMEREDIT_MODE_DAILY;    break;
        default:                     strIdx = STR_EMPTYSTRING;    break;
    }

    item->item.str.chr = OSD_GetStr(strIdx);
}

FSTATIC void MENU_TimerEditGetService (FAPI_SYS_HandleT h, int32_t id,
                                       uint32_t index, FGS_ITEMSTATE_E state,
                                       FGS_ITEM_S* item)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)h;
    SLIST_ITEM_S           service;
    SYS_SERVLOC_S          servLoc;
    uint16_t               servNum;
    uint32_t               bufPos = 0;

    #ifdef APPL_LCN_ENABLED
    SLIST_PARAMS_S         slistParams;
    #endif /* APPL_LCN_ENABLED */

    servLoc = inst->timerService;

    if ( inst->timerService.listId != SLIST_INVISIBLE )
    {
        servLoc.index = (int16_t)index;
    }

    if ( SLIST_EntryGet (&servLoc, &service) != FAPI_OK )
    {
        return;
    }

    servNum = index + 1;

    #ifdef APPL_LCN_ENABLED

    if ( (SLIST_GetParams(&slistParams) == FAPI_OK) && slistParams.sortByLcn )
    {
        servNum = 0;

        (void)SLIST_GetLcn (&servLoc, &servNum);
    }

    #endif /* APPL_LCN_ENABLED */

    sprintf (appDat->asciiBuf, "%04d - ", servNum);

    bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                             MAIN_LINEBUFCHARSET);

    bufPos += STR_BufAppend (NULL, bufPos,
                             STR_Build(service.nameCharSet, service.name),
                             MAIN_LINEBUFCHARSET);

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;
    item->item.str.chr        = STR_IntLineStr();

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
    }
}

FSTATIC void MENU_TimerEditConfirm (void* arg)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)arg;

    inst->forceSave = FTRUE;

    (void)MENUSTACK_Back(NULL);
}

FSTATIC void MENU_TimerEditAbort (void* arg)
{
    MENU_TIMEREDIT_INST_S* inst = (MENU_TIMEREDIT_INST_S*)arg;

    inst->forceSave = FFALSE;

    (void)MENUSTACK_Back(NULL);
}

FSTATIC void MENU_TimerEditStringExit(MENU_EDITSTRING_DATA_S* editData, int32_t id)
{
    MENU_TIMEREDIT_INST_S*  inst = menuTimerEditDat;

    if ( editData->updated )
    {
        (void)STR_Copy (editData->string, inst->timerData.desc,
                  &(inst->timerData.descCharSet), editData->string.cs,
                  RECTIMER_DESC_LENGTH);

        inst->valChanged = FTRUE;
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_TIMEREDIT_ID_LIST);
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
