/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_timerprog.c
**
**              $Id: app_menu_timerprog.c 5903 2010-03-04 10:51:23Z jmorha $
**
** \brief       Osd Timer programming menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum
{
    MENU_TIMERPROG_ID_TITLE = 0,
    MENU_TIMERPROG_ID_LIST,
    MENU_TIMERPROG_ID_REST,
    MENU_TIMERPROG_ID_HELPBOTTOM
};

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    listHdl;

    FGS_PANEL_MEMBER_S* pItem;
    FGS_POS_S           itemPos;

    RECTIMER_ITEM_S     tmpTimer;
    SLIST_ITEM_S        tmpService;
    fbool_t             dataChanged;

}MENU_TIMERPROG_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC MENU_TIMERPROG_INST_S* menuTimerProgDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_TimerProgLeave      (FAPI_SYS_HandleT h,
                                          fbool_t force);
FSTATIC void    MENU_TimerProgOnSelect   (FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint16_t index);
FSTATIC fbool_t MENU_TimerProgKeys       (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_TimerProgBuild      (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_TimerProgDraw       (FAPI_SYS_HandleT h);
FSTATIC void    MENU_TimerProgOnFocusSet (FAPI_SYS_HandleT h,
                                          fbool_t hasFocus);
FSTATIC void    MENU_TimerProgGetItem    (FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item);


int32_t MENU_TimerProgInit (void)
{
    if ( menuTimerProgDat != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    menuTimerProgDat = UTILS_Zalloc(sizeof(MENU_TIMERPROG_INST_S));
    if ( menuTimerProgDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    return FAPI_OK;
}

fbool_t MENU_TimerProgEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_TIMERPROG_INST_S* inst = menuTimerProgDat;
    uint32_t               lastFocusItem;

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    if ( !(MENU_TimerProgBuild((FAPI_SYS_HandleT)inst)) )
    {
        return FFALSE;
    }

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        lastFocusItem = 0;//MENU_TIMERPROG_ID_;
    }

    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_TIMERPROG_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_TIMERPROG);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */
    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_TimerProgLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

FSTATIC void MENU_TimerProgOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_TIMERPROG_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_RED ,   MENU_TimerProgKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BLUE,   MENU_TimerProgKeys, h, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_BLUE);
    }
}

FSTATIC fbool_t MENU_TimerProgLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_TIMERPROG_INST_S* inst = (MENU_TIMERPROG_INST_S*)h;

    DBG_Assert(inst->pnlHdl != 0);

    if ( !force )
    {
        if ( inst->dataChanged )
        {
            MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

            (void)RECTIMER_Save(FFALSE);

            (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        }
    }

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_TIMERPROG);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC void MENU_TimerProgOnSelect (FAPI_SYS_HandleT h, int32_t id,
                                     uint16_t index)
{
    MENU_TIMERPROG_INST_S*  inst = (MENU_TIMERPROG_INST_S*)h;
    MENU_TIMEREDIT_PARAMS_S editPars;
    RECTIMER_ITEM_S         recTmr;
    FGS_LNAVI_PARAMS_S      naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHdl, &naviParams) != FAPI_OK )
    {
        return;
    }

    if ( (uint16_t)(index + 1) >= naviParams.numItems )
    {
        /* create new timer */
        memset(&recTmr, 0, sizeof(RECTIMER_ITEM_S));

        #ifdef APPL_PVR_ENABLED

            recTmr.operation    = RECTIMER_OP_RECORD;
            recTmr.targetDevice = MAIN_PvrGetDefaultRecDevice ();

            if ( recTmr.targetDevice == SYS_DEV_NULL )
            {
                /* no recording device found */
                MENU_CommonMsgShow(STR_MSG_NORECDEVICE, FFALSE);
                return;
            }

        #else

            recTmr.operation    = RECTIMER_OP_SWITCH_SERVICE;
            recTmr.targetDevice = SYS_DEV_NULL;

        #endif /* APPL_PVR_ENABLED */

        (void)TIME_TimeAndDateGetUtc (&(recTmr.startTime),
                                      &(recTmr.startDate), NULL);

        /* clear seconds: record at full minute only! */
        recTmr.startTime.second = 0;

        recTmr.endTime = recTmr.startTime;
        recTmr.endDate = recTmr.startDate;

        /* set standard 'Tatort' length: 90 minutes */
        TIME_AddSeconds(&(recTmr.endTime),
                        &(recTmr.endDate), 60 * 90);

        (void)AV_GetRequestedService (&(editPars.newService));

        if ( (editPars.newService.listId == SLIST_NONE)      ||
             (editPars.newService.listId == SLIST_INVISIBLE) ||
             (editPars.newService.index < 0) )
        {
            editPars.newService.listId  = SLIST_TV;
            editPars.newService.index   = 0;
        }

        recTmr.type         = RECTIMER_TYPE_SINGLE;
        recTmr.enabled      = FTRUE;
        recTmr.serviceIndex = -1;
        recTmr.eventId      = 0;
        recTmr.autoStandby  = FFALSE;

        editPars.timerIndex = -1;
        editPars.newDat     = &recTmr;
    }
    else
    {
        /* edit existing timer */
        editPars.timerIndex = index;
        editPars.newDat     = NULL;
    }

    (void)MENUSTACK_Up(MENU_TimerEditEntry, (FAPI_SYS_HandleT)(&editPars));
}

FSTATIC void MENU_TimerProgGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                    FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_TIMERPROG_INST_S* inst = (MENU_TIMERPROG_INST_S*)h;
    uint32_t               col;
    FGS_LNAVI_PARAMS_S     naviParams;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: col = OSD_COL_TEXT;             break;
        case FGS_FOCUS:    col = OSD_COL_TEXT_HIGHLIGHTED; break;
        case FGS_DISABLED: col = OSD_COL_TEXT_DISABLED;    break;
    }

    if ( FGS_LNAVI_GetParams(inst->listHdl, &naviParams) != FAPI_OK )
    {
        return;
    }

    /* last item in list is always empty... */
    if ( (uint16_t)(index + 1) >= naviParams.numItems )
    {
        memset(item + 1, 0, sizeof(FGS_ITEM_S));

        item[1].type                = FGS_ITEM_TYPE_TIME;
        item[1].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item[1].item.time.format    = appDat->timeFormat;
        item[1].item.time.font      = MAIN_FONT;
        item[1].item.time.col       = col;

        item[4].type                = FGS_ITEM_TYPE_STRING;
        item[4].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item[4].item.str.chr        = OSD_GetStr(STR_GNR_EMPTY);
        item[4].item.str.font       = MAIN_FONT;
        item[4].item.str.lineHeight = 0;
        item[4].item.str.col        = col;
    }
    else
    {
        if ( RECTIMER_Get(index, &(inst->tmpTimer)) != FAPI_OK )
        {
            return;
        }

        if ( SLIST_GetServByIndex (inst->tmpTimer.serviceIndex,
                                   &(inst->tmpService)) != FAPI_OK )
        {
            return;
        }

        (void)TIME_AddUtcOffset (&(inst->tmpTimer.startTime),&(inst->tmpTimer.startDate));
        (void)TIME_AddUtcOffset (&(inst->tmpTimer.endTime),  &(inst->tmpTimer.endDate));

        /* ----- Start date ------------------------------------------------ */

        item[0].type                = FGS_ITEM_TYPE_DATE;
        item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item[0].item.date.value     = inst->tmpTimer.startDate;
        item[0].item.date.format    = appDat->dataFormatShort;
        item[0].item.date.font      = MAIN_FONT;
        item[0].item.date.col       = col;

        /* ----- Start time ------------------------------------------------ */

        item[1].type                = FGS_ITEM_TYPE_TIME;
        item[1].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item[1].item.time.value     = inst->tmpTimer.startTime;
        item[1].item.time.format    = appDat->timeFormat;
        item[1].item.time.font      = MAIN_FONT;
        item[1].item.time.col       = col;

        /* ----- End time -------------------------------------------------- */

        item[2]                     = item[1];
        item[2].item.time.value     = inst->tmpTimer.endTime;

        /* ----- Activation state ------------------------------------------ */

        item[3].type                = FGS_ITEM_TYPE_STRING;
        item[3].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item[3].item.str.font       = MAIN_FONT;
        item[3].item.str.lineHeight = 0;
        item[3].item.str.col        = col;

        if ( inst->tmpTimer.enabled )
        {
            item[3].item.str.chr = OSD_GetStr(STR_VALUE_TIMERPROG_ACTIVE_YES);
        }
        else
        {
            item[3].item.str.chr = OSD_GetStr(STR_VALUE_TIMERPROG_ACTIVE_NO);
        }

        /* ----- Service --------------------------------------------------- */

        item[4].type                = FGS_ITEM_TYPE_STRING;
        item[4].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item[4].item.str.chr        = STR_Build(inst->tmpService.nameCharSet,
                                                inst->tmpService.name);
        item[4].item.str.font       = MAIN_FONT;
        item[4].item.str.lineHeight = 0;
        item[4].item.str.col        = col;

        /* ----- Event ----------------------------------------------------- */

        item[5]                     = item[4];
        item[5].item.str.chr        = STR_Build (inst->tmpTimer.descCharSet,
                                                 inst->tmpTimer.desc);

        /* ----- Mode ------------------------------------------------------ */

        item[6]                     = item[5];

        switch ( inst->tmpTimer.type )
        {
            case RECTIMER_TYPE_SINGLE:
                item[6].item.str.chr = OSD_GetStr(STR_VALUE_TIMERPROG_MODE_SINGLE);
                break;

            case RECTIMER_TYPE_WEEKDAYS:
                item[6].item.str.chr = OSD_GetStr(STR_VALUE_TIMERPROG_MODE_WEEKDAYS);
                break;

            case RECTIMER_TYPE_WEEKLY:
                item[6].item.str.chr = OSD_GetStr(STR_VALUE_TIMERPROG_MODE_WEEKLY);
                break;

            case RECTIMER_TYPE_WEEKENDS:
                item[6].item.str.chr = OSD_GetStr(STR_VALUE_TIMERPROG_MODE_WEEKENDS);
                break;

            case RECTIMER_TYPE_DAILY:
                item[6].item.str.chr = OSD_GetStr(STR_VALUE_TIMERPROG_MODE_DAILY);
                break;

            default:
                break;
        }

        /* ----- Auto-Standby ---------------------------------------------- */

        item[7]                     = item[6];
        if ( inst->tmpTimer.autoStandby )
        {
            item[7].item.str.chr = OSD_GetStr(STR_VALUE_TIMERPROG_AFTEREC_STANDBY);
        }
        else
        {
            item[7].item.str.chr = OSD_GetStr(STR_VALUE_TIMERPROG_AFTEREC_PWRUP);
        }

        /* ----- Target device for recording ------------------------------- */

        item[8]                     = item[7];

        switch ( inst->tmpTimer.targetDevice )
        {
            #ifdef APPL_HDD_ENABLED

            case SYS_DEV_HDD:
                item[8].item.str.chr = OSD_GetStr(STR_VALUE_TIMEREDIT_RECTO_INTHD);
                break;

            #endif /* APPL_HDD_ENABLED */

            #ifdef APPL_USB_ENABLED

            case SYS_DEV_USB:
                item[8].item.str.chr = OSD_GetStr(STR_VALUE_TIMEREDIT_RECTO_USB);
                break;

            #endif /* APPL_USB_ENABLED */

            default:
                item[8].item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                break;
        }
    }
}

FSTATIC fbool_t MENU_TimerProgKeys (FAPI_SYS_HandleT h)
{
    MENU_TIMERPROG_INST_S*  inst = (MENU_TIMERPROG_INST_S*)h;
    RECTIMER_ITEM_S         recTmr;
    fbool_t                 keyUsed = FTRUE;
    FGS_LNAVI_PARAMS_S      naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHdl, &naviParams) != FAPI_OK )
    {
        return FFALSE;
    }

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_BLUE: /* activate/deactivate timer */
            if ( RECTIMER_Get (naviParams.index, &recTmr) == FAPI_OK )
            {
                /* toggle "enabled" status*/
                recTmr.enabled = (recTmr.enabled == FTRUE) ? FFALSE : FTRUE;

                (void)RECTIMER_Replace(naviParams.index, &recTmr);

                inst->dataChanged = FTRUE;

                MENU_TimerProgUpdate();
            }
            break;

        case FGS_KEY_RED:
            if ( RECTIMER_Remove (naviParams.index) == FAPI_OK )
            {
                inst->dataChanged = FTRUE;
                MENU_TimerProgUpdate();
            }
            break;

        default:
          keyUsed = FFALSE;
          break;
    }

    return keyUsed;
}

void MENU_TimerProgUpdate (void)
{
    MENU_TIMERPROG_INST_S* inst = menuTimerProgDat;
    uint16_t               numTimers;
    FGS_LNAVI_PARAMS_S     naviParams;

    if ( inst->pnlHdl == NULL )
    {
        return;
    }

    if ( (RECTIMER_GetNumTimers(&numTimers) == FAPI_OK) &&
         (FGS_LNAVI_GetParams(inst->listHdl, &naviParams) == FAPI_OK) )
    {
        naviParams.numItems = numTimers + 1;

        (void)FGS_LNAVI_SetParams(inst->listHdl, &naviParams);
    }
}

FSTATIC fbool_t MENU_TimerProgBuild(FAPI_SYS_HandleT h)
{
    MENU_TIMERPROG_INST_S* inst = (MENU_TIMERPROG_INST_S*)h;
    FGS_PANEL_OPEN_S       pnlPars;
    FGS_LNAVI_OPEN_S       naviPars;
    FAPI_SYS_HandleT       itemHdl;
    FGS_BOX_OPEN_S         boxPars;
    FGS_ITEM_TXT_OPEN_S    txtPars;
    FGS_PANEL_MEMBER_S     member;
    FGS_ITEM_S             item;
    int32_t                retVal;
    uint16_t               stackedFrameWidth;
    uint16_t               innerHeight;
    uint16_t               mainPanelWidth;
    uint16_t               bottomHelpKeyWidth;
    uint16_t               bottomHelpItemWidth;
    uint32_t               numLines = 8;
    uint8_t                numHelpLines = 3;
    uint16_t               numTimers;
    uint16_t               idCnt;

    memset(&naviPars, 0, sizeof(naviPars));
    memset(&boxPars,  0, sizeof(boxPars));
    memset(&txtPars,  0, sizeof(txtPars));
    memset(&item,     0, sizeof(item));
    memset(&member,   0, sizeof(member));
    memset(&pnlPars,  0, sizeof(pnlPars));

    do
    {
        innerHeight = (numLines       * SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                    + ((numLines - 1) * SCALER_GetCurrVal(LIST_LINE_PADDING));

        MENU_CommonSetParamsTitleWBotHelp (&pnlPars,
                                           SCALER_GetCurrVal(TIMERPROG_WIDTH),
                                           innerHeight,
                                           numHelpLines,
                                           &stackedFrameWidth);

        mainPanelWidth          = pnlPars.pos.width
                                - stackedFrameWidth
                                - 2 * SCALER_GetCurrVal(OUTER_FRAME_ALL);
        bottomHelpKeyWidth      = OSD_GetStrWidth(OSD_GetStr(STR_KEY_MENU),-1,HELP_FONT) + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        bottomHelpItemWidth     = mainPanelWidth/2 - bottomHelpKeyWidth - SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

        pnlPars.onFocusSet      = MENU_TimerProgOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_TIMERPROG, &(inst->pnlHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_TIMERPROG_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_TIMERPROG,
                                     NULL, NULL);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----- create navilist ----------------------------------------------- */
        retVal = RECTIMER_GetNumTimers(&numTimers);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        naviPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   stackedFrameWidth;
        naviPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                   (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                   SCALER_GetCurrVal(TITLE_LINE_HEIGHT)+
                                   SCALER_GetCurrVal(LIST_LINE_HEIGHT)  +
                                   (2*SCALER_GetCurrVal(LIST_LINE_PADDING)) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   stackedFrameWidth;
        naviPars.pos.width       = SCALER_GetCurrVal(TIMERPROG_WIDTH);
        naviPars.pos.height      = (numLines-1) * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        naviPars.bkgrType        = OSD_NAVIBKGR_NORMAL;
        naviPars.type            = FGS_LNAVI_TYPE_SCROLL;
        naviPars.rowHeight       = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.scrollbarWidth  = SCALER_GetCurrVal(SCROLLBAR_WIDTH);
        naviPars.numVisLines     = numLines  - 1;
        naviPars.numColumns      = 9;
        naviPars.params.numItems = numTimers + 1;
        naviPars.params.index    = 0;
        naviPars.params.posInVis = 0;

        naviPars.colStart[0]     = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[0]     = SCALER_GetCurrVal(TIMERPROG_COL0_WIDTH);  /* date*/

        naviPars.colStart[1]     = naviPars.colStart[0] + naviPars.colWidth[0] +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[1]     = SCALER_GetCurrVal(TIMERPROG_COL1_WIDTH);   /* time start */

        naviPars.colStart[2]     = naviPars.colStart[1] + naviPars.colWidth[1] +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[2]     = SCALER_GetCurrVal(TIMERPROG_COL2_WIDTH);   /* time end */

        naviPars.colStart[3]     = naviPars.colStart[2] + naviPars.colWidth[2] +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[3]     = SCALER_GetCurrVal(TIMERPROG_COL3_WIDTH);    /* active */

        naviPars.colStart[4]     = naviPars.colStart[3] + naviPars.colWidth[3] +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[4]     = SCALER_GetCurrVal(TIMERPROG_COL4_WIDTH);    /* service */

        naviPars.colStart[5]     = naviPars.colStart[4] + naviPars.colWidth[4] +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[5]     = SCALER_GetCurrVal(TIMERPROG_COL5_WIDTH);    /* event */

        naviPars.colStart[6]     = naviPars.colStart[5] + naviPars.colWidth[5] +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[6]     = SCALER_GetCurrVal(TIMERPROG_COL6_WIDTH);    /* mode */

        naviPars.colStart[7]     = naviPars.colStart[6] + naviPars.colWidth[6] +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[7]     = SCALER_GetCurrVal(TIMERPROG_COL7_WIDTH);    /* auto-standby */

        naviPars.colStart[8]     = naviPars.colStart[7] + naviPars.colWidth[7] +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[8]     = SCALER_GetCurrVal(TIMERPROG_COL8_WIDTH);    /* record to  */

        naviPars.topDownOrient   = FTRUE;
        naviPars.wraparound      = FTRUE;
        naviPars.getItem         = MENU_TimerProgGetItem;
        naviPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        naviPars.onSelect        = MENU_TimerProgOnSelect;
        naviPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        naviPars.pInst           = &(appDat->lnaviInst);

        retVal = FGS_LNAVI_Open (&naviPars, MENU_TIMERPROG_ID_LIST,
                                 inst->pnlHdl,
                                &(inst->listHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----------------------------------------------------------------- */

        inst->itemPos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   stackedFrameWidth;
        inst->itemPos.y         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                   (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                   SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                   stackedFrameWidth;
        inst->itemPos.width     = SCALER_GetCurrVal(TIMERPROG_WIDTH);
        inst->itemPos.height    = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(INNER_FRAME_WIDTH));
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemPos);
        member.id            = MENU_TIMERPROG_ID_REST;
        member.isHidden      = FFALSE;
        member.show          = MENU_TimerProgDraw;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pItem));
        if ( retVal != FAPI_OK )
        {
            break;
        }
        /* -----Bottom help lines ------------------------------------------ */
        /* Red icon */
        idCnt = MENU_TIMERPROG_ID_HELPBOTTOM;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNREDSMALL);

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - 2*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* -----Colon ------------------------------------------ */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);

        boxPars.pos.x              += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_TIMERPROG_REMOVE);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* Blue icon */
        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNBLUESMALL);

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + mainPanelWidth/2;
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* -----Colon ------------------------------------------ */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);

        boxPars.pos.x              += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_TIMERPROG_ACTIVATE);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

       /* -- 2nd Bottom line -- */
       /* OK icon */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_OK);

        boxPars.pos.x               = stackedFrameWidth + SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* -----Colon ------------------------------------------ */

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);

        boxPars.pos.x              += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        item.item.str.chr           = OSD_GetStr(STR_HELP_TIMERPROG_EDIT);
        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

       /* -- 3rd Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                boxPars.pos.y,
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK )
        {
            break;
        }

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);

            inst->pnlHdl = NULL;
        }

        return FFALSE;
    }

    return FTRUE;
}

FSTATIC int32_t MENU_TimerProgDraw (FAPI_SYS_HandleT h)
{
    MENU_TIMERPROG_INST_S* inst = (MENU_TIMERPROG_INST_S*)h;
    uint16_t               yCurr;
    uint16_t               posX;

    posX  = inst->pItem->abspos.x;
    yCurr = inst->pItem->abspos.y;

    /* ----- print help lines ---------------------------------------------- */

    OSD_DrawStringLtd (posX  + SCALER_GetCurrVal(MAIN_TEXT_PAD_X),
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(TIMERPROG_COL0_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_TIMERPROG_DATE),
                       FTRUE);
    posX += SCALER_GetCurrVal(TIMERPROG_COL0_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX  + SCALER_GetCurrVal(MAIN_TEXT_PAD_X),
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(TIMERPROG_COL1_WIDTH) +
                       SCALER_GetCurrVal(TIMERPROG_COL2_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X),
                       -1, OSD_GetStr(STR_ENTRY_TIMERPROG_TIME),
                       FTRUE);

    posX += SCALER_GetCurrVal(TIMERPROG_COL1_WIDTH) +
            SCALER_GetCurrVal(TIMERPROG_COL2_WIDTH) +
            2*SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX  + SCALER_GetCurrVal(MAIN_TEXT_PAD_X),
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(TIMERPROG_COL3_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_TIMERPROG_ACTIVE),
                       FTRUE);

    posX += SCALER_GetCurrVal(TIMERPROG_COL3_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX  + SCALER_GetCurrVal(MAIN_TEXT_PAD_X),
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(TIMERPROG_COL4_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_TIMERPROG_PROG),
                       FTRUE);

    posX += SCALER_GetCurrVal(TIMERPROG_COL4_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX  + SCALER_GetCurrVal(MAIN_TEXT_PAD_X),
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(TIMERPROG_COL5_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_TIMERPROG_TITLE),
                       FTRUE);

    posX += SCALER_GetCurrVal(TIMERPROG_COL5_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX  + SCALER_GetCurrVal(MAIN_TEXT_PAD_X),
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(TIMERPROG_COL6_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_TIMERPROG_MODE),
                       FTRUE);

    posX += SCALER_GetCurrVal(TIMERPROG_COL6_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX  + SCALER_GetCurrVal(MAIN_TEXT_PAD_X),
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(TIMERPROG_COL7_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_TIMERPROG_AFTER),
                       FTRUE);

    posX += SCALER_GetCurrVal(TIMERPROG_COL7_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    OSD_DrawStringLtd (posX  + SCALER_GetCurrVal(MAIN_TEXT_PAD_X),
                       yCurr,
                       OSD_COL_TEXT, MAIN_FONT, -1,
                       SCALER_GetCurrVal(TIMERPROG_COL8_WIDTH) -
                       SCALER_GetCurrVal(MAIN_TEXT_PAD_X), -1,
                       OSD_GetStr(STR_ENTRY_TIMERPROG_RECTO),
                       FTRUE);

    posX += SCALER_GetCurrVal(TIMERPROG_COL8_WIDTH) +
            SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    posX  = inst->pItem->abspos.x;
    yCurr += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
             (2*SCALER_GetCurrVal(LIST_LINE_PADDING));
    OSD_DrawSeparatorHorizontal (posX, yCurr, inst->itemPos.width);

    return FAPI_OK;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
