/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_tplist.c
**
**              $Id: app_menu_tplist.c 5890 2010-03-03 15:39:03Z jmorha $
**
** \brief       Carrier list editor menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define TP_SAT_MIN_FREQ         1000 // MHz
#define TP_SAT_MAX_FREQ        19999 // MHz
#define TP_CABLE_MIN_FREQ         47 // MHz
#define TP_CABLE_MAX_FREQ        862 // MHz

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum
{
    MENU_TPLIST_ID_TITLE = 0,
    MENU_TPLIST_ID_LIST,
    MENU_TPLIST_ID_HELPLIST,
    MENU_TPLIST_ID_HELPBOTTOM
};

enum
{
    MENU_TPLIST_ID_ENTRY_SELTP = 0,
    MENU_TPLIST_ID_ENTRY_FREQ,
    MENU_TPLIST_ID_ENTRY_SYMBRATE,
    #ifdef APPL_FESAT_ENABLED
    MENU_TPLIST_ID_ENTRY_SELSAT,
    MENU_TPLIST_ID_ENTRY_POL,
    #endif
    MENU_TPLIST_ID_ENTRY_MODULATION
};

enum
{
    CAB_FE_MOD_QAM_16,  //!< QAM 16.
    CAB_FE_MOD_QAM_32,  //!< QAM 32.
    CAB_FE_MOD_QAM_64,  //!< QAM 64.
    CAB_FE_MOD_QAM_128, //!< QAM 129.
    CAB_FE_MOD_QAM_256, //!< QAM 256.
    CAB_FE_MOD_NUMS
};

enum
{
    SAT_FE_MOD_QPSK = 0,
    SAT_FE_MOD_QPSK_S2,
    SAT_FE_MOD_8PSK,
    SAT_FE_MOD_NUMS
};

typedef struct
{
    FAPI_SYS_HandleT        pnlHdl;
    FAPI_SYS_HandleT        listHdl;
    FAPI_SYS_HandleT        listHelpHdl;
    fbool_t                 newCarrier;

    /* Status parameters */
    MENU_TPLIST_PARAMS_S    params;
    SYS_NETWORK_S           currNetwork;
    fbool_t                 networkValid;

    TPLIST_ITEM_S           currCarrierDat;
    fbool_t                 carrierDatValid;

    TPLIST_ITEM_S           tmpCarrierDat;

    #ifdef APPL_FESAT_ENABLED
    CFGSAT_SAT_S            tmpSatDat;
    #endif

    int8_t                  currFe;

    fbool_t                 tpChg;
    fbool_t                 exitFlag;

} MENU_TPLIST_INST_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC MENU_TPLIST_INST_S* menuTpListDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_TpListLeave             (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_TpListOnSelect          (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void    MENU_TpListValChg            (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal);
FSTATIC void    MENU_TpListGetItem           (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                              FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC fbool_t MENU_TpListKeys              (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_TpListBuild             (FAPI_SYS_HandleT h);
FSTATIC void    MENU_TpListOnFocusChg        (FAPI_SYS_HandleT h, uint8_t index, int32_t id);
FSTATIC void    MENU_TpListConfirm           (void* arg);
FSTATIC void    MENU_TpListOnFocusSet        (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_TpListGetDescrItem      (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                              FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC void    MENU_TpListGetHelpDescrItem  (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);

/*!
*******************************************************************************
**
** \brief  Initial function for the OSD Transponder list settings Menu.
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_TpListInit (void)
{
    MENU_TPLIST_INST_S* inst;

    if ( menuTpListDat != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    menuTpListDat = UTILS_Zalloc(sizeof(MENU_TPLIST_INST_S));
    if ( menuTpListDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }
    inst = menuTpListDat;

    inst->params.system        = FE_SYSTEM_TYPE_SATELLITE;
    inst->params.satLstPos     = 0;
    inst->params.carrierLstPos = 0;
    inst->params.lockSat       = FFALSE;
    inst->params.leaveCb       = NULL;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function.
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_TpListEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_TPLIST_INST_S* inst = menuTpListDat;
    uint32_t                 lastFocusItem;
    int32_t                retVal;

    if ( test == FTRUE ) return FTRUE;

    memset(&(inst->currNetwork),    0, sizeof(inst->currNetwork));
    memset(&(inst->currCarrierDat), 0, sizeof(inst->currCarrierDat));

    inst->tpChg         = FFALSE;
    inst->exitFlag      = FFALSE;
    inst->currFe        = -1;
    inst->newCarrier    = FFALSE;

    if ( h != NULL )
    {
        inst->params = *((MENU_TPLIST_PARAMS_S*)h);
    }
    else
    {
        inst->params.leaveCb = NULL;
    }

    inst->currNetwork.type = inst->params.system;
    inst->networkValid     = FTRUE;

    #ifdef APPL_FESAT_ENABLED

    if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
    {
        if ( CFGSAT_GetIndex (inst->params.satLstPos,
                              &(inst->currNetwork.satIdx)) != FAPI_OK )
        {
            inst->params.satLstPos = 0;

            if ( CFGSAT_GetIndex (inst->params.satLstPos,
                                  &(inst->currNetwork.satIdx)) != FAPI_OK )
            {
                inst->networkValid = FFALSE;
            }
        }
    }

    #endif /* APPL_FESAT_ENABLED */

    inst->carrierDatValid = FFALSE;

    if ( inst->networkValid )
    {
        /* Store information on currently selected carrier */
        if ( TPLIST_GetCarrierData (&(inst->currNetwork),
                                   inst->params.carrierLstPos,
                                   &(inst->currCarrierDat)) == FAPI_OK )
        {
            inst->carrierDatValid = FTRUE;
        }
    }

    (void)MENU_TpListBuild((FAPI_SYS_HandleT)inst);

    if ( !(inst->carrierDatValid) )
    {
        (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_TPLIST_ID_ENTRY_FREQ,       FFALSE);
        (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_TPLIST_ID_ENTRY_SYMBRATE,   FFALSE);
        (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_TPLIST_ID_ENTRY_MODULATION, FFALSE);
        #ifdef APPL_FESAT_ENABLED
        (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_TPLIST_ID_ENTRY_POL,        FFALSE);
        #endif
    }

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        if ( inst->params.system == FE_SYSTEM_TYPE_CABLE )
        {
            lastFocusItem = MENU_TPLIST_ID_ENTRY_SELTP;
        }
        #ifdef APPL_FESAT_ENABLED
        if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE)
        {
            if ( inst->params.lockSat ) lastFocusItem = MENU_TPLIST_ID_ENTRY_SELTP;
            else                        lastFocusItem = MENU_TPLIST_ID_ENTRY_SELSAT;
        }
        #endif
    }

    /* set list focus on first item */
    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_TPLIST_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_TPLIST);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    if ( inst->networkValid && inst->carrierDatValid )
    {
        retVal = AV_TuneModeStart (-1, &(inst->currNetwork),
                                  inst->params.carrierLstPos,
                                  NULL, &(inst->currFe));
        if ( retVal != FAPI_OK )
        {
            inst->currFe = -1;
        }
    }

    (void)MAIN_SignalBoxSetFeIdx(&(appDat->sigBox), inst->currFe);
    (void)MAIN_SignalBoxUpdate  (&(appDat->sigBox));

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_TpListLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_TpListOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_TPLIST_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_GREEN, MENU_TpListKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RED,   MENU_TpListKeys, h, FTRUE, FFALSE);

        /* start signal bar update */
        (void)MAIN_SignalBoxResume(&(appDat->sigBox));
    }
    else
    {
        /* halt signal bar update */
        (void)MAIN_SignalBoxPause(&(appDat->sigBox));

        FGS_ClearAppKeyHandler(FGS_KEY_GREEN);
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
    }
}

/*!
*******************************************************************************
**
** \brief  Activates the help icons for the current menu list item.
**
** \param h        Address of menu handle.
** \param index    unused.
** \param id       List id gets focus.
**
******************************************************************************/
FSTATIC void MENU_TpListOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_TPLIST_INST_S*   inst = (MENU_TPLIST_INST_S*)h;

    (void)FGS_LIST_SetMemberFocus(inst->listHelpHdl, id);
}

FSTATIC fbool_t MENU_TpListLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_TPLIST_INST_S* inst = (MENU_TPLIST_INST_S*)h;

    if ( inst->tpChg )
    {
        if ( force )
        {
            /* restore data on forced exit */
            (void)TPLIST_Load    (inst->params.system);
            (void)SLIST_LoadLists();

            #ifdef APPL_RECTIMER_ENABLED
            (void)RECTIMER_Load();
            #endif /* APPL_RECTIMER_ENABLED */
        }
        else
        {
            MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

            (void)TPLIST_Save (inst->params.system, FFALSE);
            (void)SLIST_Save  (FFALSE);

            #ifdef APPL_RECTIMER_ENABLED
            (void)RECTIMER_Save (FFALSE);
            #endif /* APPL_RECTIMER_ENABLED */

            /* NOTE: Service and timer lists might have been modified due to
                     removal of carriers. This is why they are written here
                     as well. */

            (void)TPLIST_ResortLists(inst->params.system);

            (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        }

        inst->tpChg = FFALSE;
    }

    if ( inst->currFe >= 0 )
    {
        (void)AV_TuneModeStop(inst->currFe);
        inst->currFe = -1;
    }

    if ( FGS_GetCurrentKeyCode() == FGS_KEY_EXIT )
    {
        (void)MAIN_RestartPresentation();
    }

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MAIN_SignalBoxClose((&appDat->sigBox));
    inst->pnlHdl = NULL;

    if ( inst->params.leaveCb != NULL )
    {
        inst->params.leaveCb (inst->params.satLstPos,
                              inst->params.carrierLstPos);
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_TPLIST);

    return FTRUE;
}

FSTATIC fbool_t MENU_TpListOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_TPLIST_INST_S*      inst     = (MENU_TPLIST_INST_S*)h;
    uint8_t                       subLines = 0;
    MENU_SELECTLIST_OPEN_S   selectLst;
    FGS_ITEM_SEL_PARAMS_S    selParams;
    fbool_t                    keyUsed       = FFALSE;
    FGS_LOC_S                memberLoc;

    switch ( itemId )
    {
        #ifdef APPL_FESAT_ENABLED
        case MENU_TPLIST_ID_ENTRY_SELSAT:
            subLines = 5;
            break;
        #endif
        case MENU_TPLIST_ID_ENTRY_SELTP:
            if ( !(inst->newCarrier) ) subLines = 6;
            break;

        case MENU_TPLIST_ID_ENTRY_MODULATION:
            subLines = 3;
            break;

        default:
            break;
    }

    if ( subLines > 0 )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.width     = 2 * SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.x         = memberLoc.x + SCALER_GetCurrVal(MAIN_WIDTH) -
                                  selectLst.pos.width;
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_TpListGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;

        keyUsed = MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }

    return keyUsed;
}


FSTATIC void MENU_TpListValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    MENU_TPLIST_INST_S*     inst = (MENU_TPLIST_INST_S*)h;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    int16_t                 numTp = 0;
    int32_t                 retVal;
    uint32_t                currModulation;

    memset(&selParams, 0, sizeof(selParams));

    switch ( itemId )
    {
        #ifdef APPL_FESAT_ENABLED
        case MENU_TPLIST_ID_ENTRY_SELSAT: /* satellite has changed */
            inst->newCarrier = FFALSE;

            inst->params.satLstPos     = newVal.number;
            inst->params.carrierLstPos = 0;


            if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
            {
                if ( CFGSAT_GetIndex (inst->params.satLstPos,
                                    &(inst->currNetwork.satIdx)) == FAPI_OK )
                {
                    inst->networkValid = FTRUE;
                }
                else
                {
                    inst->networkValid = FFALSE;
                }
            }

            if ( inst->networkValid )
            {
                if ( TPLIST_GetCarrierData (&(inst->currNetwork),
                                           inst->params.carrierLstPos,
                                           &(inst->currCarrierDat)) == FAPI_OK )
                {
                    inst->carrierDatValid = FTRUE;
                }
                else
                {
                    inst->carrierDatValid = FFALSE;
                }

                (void)TPLIST_GetNumCarriers(&(inst->currNetwork), &numTp);

                /* update carrier selection */
                selParams.numItems = numTp;
                selParams.index    = inst->params.carrierLstPos;

                (void)FGS_LIST_SetMemberParams (inst->listHdl,
                                          MENU_TPLIST_ID_ENTRY_SELTP,
                                          &selParams);
            }
            else
            {
                inst->carrierDatValid = FFALSE;
            }
            break;
        #endif /* APPL_FESAT_ENABLED */

        case MENU_TPLIST_ID_ENTRY_SELTP: /* carrier has changed */
            if ( !(inst->newCarrier) )
            {
                inst->params.carrierLstPos = newVal.number;

                if ( inst->networkValid )
                {
                    if ( TPLIST_GetCarrierData(&(inst->currNetwork),
                                               inst->params.carrierLstPos,
                                               &(inst->currCarrierDat)) == FAPI_OK )
                    {
                        inst->carrierDatValid = FTRUE;
                    }
                    else
                    {
                        inst->carrierDatValid = FFALSE;
                    }
                }
                else
                {
                    inst->carrierDatValid = FFALSE;
                }
            }

            (void)FGS_LIST_SetRefreshMode       (inst->listHdl, FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_TPLIST_ID_ENTRY_FREQ,     inst->carrierDatValid);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_TPLIST_ID_ENTRY_SYMBRATE, inst->carrierDatValid);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_TPLIST_ID_ENTRY_MODULATION, inst->carrierDatValid);
            #ifdef APPL_FESAT_ENABLED
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_TPLIST_ID_ENTRY_POL,      inst->carrierDatValid);
            #endif
            (void)FGS_LIST_SetRefreshMode       (inst->listHdl, FTRUE);

            /* update carrier parameters */
            (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TPLIST_ID_ENTRY_FREQ);
            (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TPLIST_ID_ENTRY_SYMBRATE);

            if ( FGS_LIST_GetMemberParams(inst->listHdl,
                 MENU_TPLIST_ID_ENTRY_MODULATION, &selParams) == FAPI_OK )
            {
                if ( inst->carrierDatValid )
                {
                    #ifdef APPL_FECABLE_ENABLED
                    if (inst->params.system == FE_SYSTEM_TYPE_CABLE)
                    {
                        switch( inst->currCarrierDat.pars.cable.modulation )
                        {
                            default:
                            case FE_MOD_QAM_16 : selParams.index = CAB_FE_MOD_QAM_16; break;
                            case FE_MOD_QAM_32 : selParams.index = CAB_FE_MOD_QAM_32; break;
                            case FE_MOD_QAM_64 : selParams.index = CAB_FE_MOD_QAM_64; break;
                            case FE_MOD_QAM_128: selParams.index = CAB_FE_MOD_QAM_128;break;
                            case FE_MOD_QAM_256: selParams.index = CAB_FE_MOD_QAM_256;break;
                        }
                    }
                    #endif
                    #ifdef APPL_FESAT_ENABLED
                    if (inst->params.system == FE_SYSTEM_TYPE_SATELLITE)
                    {
                        switch( inst->currCarrierDat.pars.sat.modulation )
                        {
                            default:
                            case FE_MOD_QPSK   : selParams.index = SAT_FE_MOD_QPSK;    break;
                            case FE_MOD_QPSK_S2: selParams.index = SAT_FE_MOD_QPSK_S2; break;
                            case FE_MOD_8PSK   : selParams.index = SAT_FE_MOD_8PSK;    break;
                        }
                    }
                    #endif
                }
                (void)FGS_LIST_SetMemberParams(inst->listHdl, MENU_TPLIST_ID_ENTRY_MODULATION,
                                        &selParams);
            }

            #ifdef APPL_FESAT_ENABLED
            if (inst->params.system == FE_SYSTEM_TYPE_SATELLITE)
            {
                if ( FGS_LIST_GetMemberParams(inst->listHdl,
                     MENU_TPLIST_ID_ENTRY_POL, &selParams) == FAPI_OK )
                {
                    if ( inst->carrierDatValid )
                         selParams.index = inst->currCarrierDat.pars.sat.pol;
                    else selParams.index = 0;

                    (void)FGS_LIST_SetMemberParams(inst->listHdl, MENU_TPLIST_ID_ENTRY_POL,
                                             &selParams);
                }
            }
            #endif
            break;

        case MENU_TPLIST_ID_ENTRY_FREQ:

            if ( inst->carrierDatValid )
            {
                #ifdef APPL_FECABLE_ENABLED

                if (inst->params.system == FE_SYSTEM_TYPE_CABLE)
                {
                   if ( inst->currCarrierDat.pars.cable.frequency !=
                        (newVal.number * 1000000) )
                   {
                       inst->currCarrierDat.pars.cable.frequency = newVal.number * 1000000;
                       inst->tpChg      = FTRUE;
                   }
                }

                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FESAT_ENABLED
                if (inst->params.system == FE_SYSTEM_TYPE_SATELLITE)
                {
                    if ( inst->currCarrierDat.pars.sat.frequency !=
                         (newVal.number * 1000) )
                    {
                        inst->currCarrierDat.pars.sat.frequency = newVal.number * 1000;
                        inst->tpChg      = FTRUE;
                    }
                }
                #endif /* APPL_FESAT_ENABLED */

                if(inst->tpChg == FTRUE)
                {
                    inst->newCarrier = FFALSE;

                    (void)TPLIST_ReplaceCarrier (&(inst->currNetwork),
                                                 inst->params.carrierLstPos,
                                                 &(inst->currCarrierDat), SYS_OWNER_USER);
                }
                (void)FGS_LIST_RefreshMember (inst->listHdl,
                                        MENU_TPLIST_ID_ENTRY_SELTP);
            }
            break;

        case MENU_TPLIST_ID_ENTRY_SYMBRATE:

            if ( inst->carrierDatValid )
            {
                #ifdef APPL_FECABLE_ENABLED

                if (inst->params.system == FE_SYSTEM_TYPE_CABLE)
                {
                   if ( inst->currCarrierDat.pars.cable.symbolRate != newVal.number )
                   {
                       inst->currCarrierDat.pars.cable.symbolRate = newVal.number;
                       inst->tpChg = FTRUE;
                   }
                }

                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FESAT_ENABLED
                if (inst->params.system == FE_SYSTEM_TYPE_SATELLITE)
                {
                    if ( inst->currCarrierDat.pars.sat.symbolRate != newVal.number )
                    {
                        inst->currCarrierDat.pars.sat.symbolRate = newVal.number;
                        inst->tpChg = FTRUE;
                    }
                }

                #endif /* APPL_FESAT_ENABLED */

                if(inst->tpChg == FTRUE)
                {
                    inst->newCarrier = FFALSE;

                    (void)TPLIST_ReplaceCarrier (&(inst->currNetwork),
                                                 inst->params.carrierLstPos,
                                                 &(inst->currCarrierDat), SYS_OWNER_USER);
                }
                (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TPLIST_ID_ENTRY_SELTP);
            }
            break;

        case MENU_TPLIST_ID_ENTRY_MODULATION:
            if ( inst->carrierDatValid )
            {
                #ifdef APPL_FECABLE_ENABLED

                if ( inst->params.system == FE_SYSTEM_TYPE_CABLE )
                {
                    switch( inst->currCarrierDat.pars.cable.modulation )
                    {
                        default:
                        case FE_MOD_QAM_16 : currModulation = CAB_FE_MOD_QAM_16; break;
                        case FE_MOD_QAM_32 : currModulation = CAB_FE_MOD_QAM_32; break;
                        case FE_MOD_QAM_64 : currModulation = CAB_FE_MOD_QAM_64; break;
                        case FE_MOD_QAM_128: currModulation = CAB_FE_MOD_QAM_128;break;
                        case FE_MOD_QAM_256: currModulation = CAB_FE_MOD_QAM_256;break;
                    }
                    if ( currModulation != newVal.number )
                    {
                        switch( newVal.number )
                        {
                            default:
                            case CAB_FE_MOD_QAM_16:
                                inst->currCarrierDat.pars.cable.modulation = FE_MOD_QAM_16;
                                break;
                            case CAB_FE_MOD_QAM_32:
                                inst->currCarrierDat.pars.cable.modulation = FE_MOD_QAM_32;
                                break;
                            case CAB_FE_MOD_QAM_64:
                                inst->currCarrierDat.pars.cable.modulation = FE_MOD_QAM_64;
                                break;
                            case CAB_FE_MOD_QAM_128:
                                inst->currCarrierDat.pars.cable.modulation = FE_MOD_QAM_128;
                                break;
                            case CAB_FE_MOD_QAM_256:
                                inst->currCarrierDat.pars.cable.modulation = FE_MOD_QAM_256;
                                break;
                        }
                        inst->tpChg = FTRUE;
                    }
                }
                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FESAT_ENABLED
                if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
                {
                    switch( inst->currCarrierDat.pars.sat.modulation )
                    {
                        default:
                        case FE_MOD_QPSK   : currModulation = SAT_FE_MOD_QPSK;    break;
                        case FE_MOD_QPSK_S2: currModulation = SAT_FE_MOD_QPSK_S2; break;
                        case FE_MOD_8PSK   : currModulation = SAT_FE_MOD_8PSK;    break;
                    }
                    if ( currModulation != newVal.number )
                    {
                        switch( newVal.number )
                        {
                            default:
                            case SAT_FE_MOD_QPSK   :
                                inst->currCarrierDat.pars.sat.modulation = FE_MOD_QPSK;
                                break;
                            case SAT_FE_MOD_QPSK_S2:
                                inst->currCarrierDat.pars.sat.modulation = FE_MOD_QPSK_S2;
                                break;
                            case SAT_FE_MOD_8PSK   :
                                inst->currCarrierDat.pars.sat.modulation = FE_MOD_8PSK;
                                break;
                        }
                        inst->tpChg = FTRUE;
                    }
                }
                #endif /* APPL_FESAT_ENABLED */
                if(inst->tpChg == FTRUE )
                {
                    inst->newCarrier = FFALSE;

                    (void)TPLIST_ReplaceCarrier (&(inst->currNetwork),
                                                 inst->params.carrierLstPos,
                                                 &(inst->currCarrierDat), SYS_OWNER_USER);

                }
                (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TPLIST_ID_ENTRY_SELTP);
            }
            break;

        #ifdef APPL_FESAT_ENABLED
        case MENU_TPLIST_ID_ENTRY_POL:
            if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
            {
                if ( inst->carrierDatValid )
                {
                    if ( inst->currCarrierDat.pars.sat.pol != newVal.number )
                    {
                        inst->currCarrierDat.pars.sat.pol = (FE_POL_E)newVal.number;
                        inst->tpChg      = FTRUE;
                        inst->newCarrier = FFALSE;

                        (void)TPLIST_ReplaceCarrier (&(inst->currNetwork),
                                                     inst->params.carrierLstPos,
                                                     &(inst->currCarrierDat), SYS_OWNER_USER);
                    }

                    (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TPLIST_ID_ENTRY_SELTP);
                }
            }
            break;
        #endif
        default:
            return;
    }

    /* ----- update tune mode ---------------------------------------------- */

    if ( inst->currFe >= 0 )
    {
        (void)AV_TuneModeStop(inst->currFe);
        inst->currFe = -1;
    }

    if ( inst->networkValid && inst->carrierDatValid )
    {
        retVal = AV_TuneModeStart (-1, &(inst->currNetwork),
                                  inst->params.carrierLstPos,
                                  NULL, &(inst->currFe));
        if ( retVal != FAPI_OK )
        {
            inst->currFe = -1;
        }
    }

    (void)MAIN_SignalBoxSetFeIdx(&(appDat->sigBox), inst->currFe);
    (void)MAIN_SignalBoxUpdate  (&(appDat->sigBox));
}

FSTATIC void MENU_TpListGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t    strIdx;

    switch ( id )
    {
        case MENU_TPLIST_ID_ENTRY_SELTP     : strIdx = STR_ENTRY_TPLIST_SELTP;      break;
        case MENU_TPLIST_ID_ENTRY_FREQ      : strIdx = STR_ENTRY_TPLIST_FREQ;       break;
        case MENU_TPLIST_ID_ENTRY_SYMBRATE  : strIdx = STR_ENTRY_TPLIST_SYMBRATE;   break;
        case MENU_TPLIST_ID_ENTRY_MODULATION: strIdx = STR_ENTRY_TPLIST_MODULATION; break;
        #ifdef APPL_FESAT_ENABLED
        case MENU_TPLIST_ID_ENTRY_SELSAT    : strIdx = STR_ENTRY_TPLIST_SELSAT;     break;
        case MENU_TPLIST_ID_ENTRY_POL       : strIdx = STR_ENTRY_TPLIST_POL;        break;
        #endif
        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_TpListGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_TPLIST_ID_ENTRY_MODULATION:
        #ifdef APPL_FESAT_ENABLED
        case MENU_TPLIST_ID_ENTRY_SELSAT    :
        #endif
        case MENU_TPLIST_ID_ENTRY_SELTP     : strIdx = STR_ENTRY_LOKR;
                                              break;
        case MENU_TPLIST_ID_ENTRY_FREQ      :
        case MENU_TPLIST_ID_ENTRY_SYMBRATE  : strIdx = STR_ENTRY_09LR;
                                              break;
        #ifdef APPL_FESAT_ENABLED
        case MENU_TPLIST_ID_ENTRY_POL       : strIdx = STR_ENTRY_LR;
                                              break;
        #endif
        default:  return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

FSTATIC void MENU_TpListGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_TPLIST_INST_S* inst = (MENU_TPLIST_INST_S*)h;
    uint32_t            bufPos = 0;
    uint32_t            strIdx = STR_EMPTYSTRING;

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    switch ( id )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_TPLIST_ID_ENTRY_SELSAT:

            if ( CFGSAT_GetSatData(index, &(inst->tmpSatDat)) == FAPI_OK )
            {
                item->type                = FGS_ITEM_TYPE_STRING;
                    item->item.str.chr        = STR_Build(
                                                inst->tmpSatDat.nameCharSet,
                                                inst->tmpSatDat.name);
            }
            break;

        #endif /* APPL_FESAT_ENABLED */

        case MENU_TPLIST_ID_ENTRY_SELTP:

            (void)STR_BufClear(NULL);

            if ( inst->newCarrier )
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         STR_GetASCII("add new Transponder"), // TODO move to Language file
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                #ifdef APPL_FECABLE_ENABLED
                if ( inst->params.system == FE_SYSTEM_TYPE_CABLE )
                {
                    if ( inst->networkValid &&
                       (TPLIST_GetCarrierData (&(inst->currNetwork), index,
                                               &(inst->tmpCarrierDat)) == FAPI_OK) )
                    {
                        switch ( inst->tmpCarrierDat.pars.cable.modulation )
                        {
                            default:
                            case FE_MOD_QAM_16   : strIdx = STR_GNR_MOD_QAM16;  break;
                            case FE_MOD_QAM_32   : strIdx = STR_GNR_MOD_QAM32;  break;
                            case FE_MOD_QAM_64   : strIdx = STR_GNR_MOD_QAM64;  break;
                            case FE_MOD_QAM_128  : strIdx = STR_GNR_MOD_QAM128; break;
                            case FE_MOD_QAM_256  : strIdx = STR_GNR_MOD_QAM256; break;
                        }

                        sprintf (appDat->asciiBuf, "%ld ",
                                 ((uint32_t)inst->tmpCarrierDat.pars.cable.frequency/1000000));

                        bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                                 MAIN_LINEBUFCHARSET);

                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 OSD_GetStr(STR_UNIT_MHZ),
                                                 MAIN_LINEBUFCHARSET);

                        sprintf (appDat->asciiBuf, ", %ld.%ld ",
                                ((uint32_t)inst->tmpCarrierDat.pars.cable.symbolRate/1000),
                                ((uint32_t)inst->tmpCarrierDat.pars.cable.symbolRate/100) -
                                 10 * ((uint32_t)inst->tmpCarrierDat.pars.cable.symbolRate/1000));

                        bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                                 MAIN_LINEBUFCHARSET);

                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 OSD_GetStr(STR_UNIT_MBAUD),
                                                 MAIN_LINEBUFCHARSET);

                        bufPos += STR_BufAppend (NULL, bufPos, STR_GetASCII(" , "),
                                                 MAIN_LINEBUFCHARSET);

                        bufPos += STR_BufAppend (NULL, bufPos,
                                                 OSD_GetStr(strIdx),
                                                 MAIN_LINEBUFCHARSET);
                    }
                }

                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FESAT_ENABLED

                if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
                {
                    if ( inst->networkValid &&
                       (TPLIST_GetCarrierData (&(inst->currNetwork), index,
                                               &(inst->tmpCarrierDat)) == FAPI_OK) )
                    {
                        sprintf (appDat->asciiBuf, "%ld ",
                                 ((uint32_t)inst->tmpCarrierDat.pars.sat.frequency/1000));

                        bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                                 MAIN_LINEBUFCHARSET);

                        bufPos += STR_BufAppend (NULL, bufPos,
                           OSD_GetStr(STR_UNIT_MHZ),
                                                 MAIN_LINEBUFCHARSET);

                        sprintf (appDat->asciiBuf, ", %ld.%ld ",
                               ((uint32_t)inst->tmpCarrierDat.pars.sat.symbolRate/1000),
                               ((uint32_t)inst->tmpCarrierDat.pars.sat.symbolRate/100) -
                                10 * ((uint32_t)inst->tmpCarrierDat.pars.sat.symbolRate/1000));

                        bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                                 MAIN_LINEBUFCHARSET);

                        bufPos += STR_BufAppend (NULL, bufPos,
                               OSD_GetStr(STR_UNIT_MBAUD),
                                                 MAIN_LINEBUFCHARSET);

                        if ( inst->tmpCarrierDat.pars.sat.pol == FE_POL_HOR )
                        {
                            bufPos += STR_BufAppend (NULL, bufPos,
                                                     STR_GetASCII(" , H"),
                                                     MAIN_LINEBUFCHARSET);
                        }
                        else
                        {
                                    bufPos += STR_BufAppend (NULL, bufPos,
                                                             STR_GetASCII(" , V"),
                                                             MAIN_LINEBUFCHARSET);
                        }
                    }
                }
                #endif /* APPL_FESAT_ENABLED */
            }

            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.chr        = STR_IntLineStr();
            break;

        case MENU_TPLIST_ID_ENTRY_FREQ:
            if ( inst->carrierDatValid )
            {
                item->type                      = FGS_ITEM_TYPE_NUMBER;

                #ifdef APPL_FECABLE_ENABLED

                if ( inst->params.system == FE_SYSTEM_TYPE_CABLE )
                {
                    item->item.number.value = inst->currCarrierDat.pars.cable.frequency / 1000000;
                }

                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FESAT_ENABLED

                if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
                {
                    item->item.number.value  = inst->currCarrierDat.pars.sat.frequency / 1000;
                }

                #endif /* APPL_FESAT_ENABLED */

                item->item.number.format        = FGS_FORMAT_NUM_NORM;
                item->item.number.decimalPlaces = 0;
                item->item.number.unitStr       = OSD_GetStr(STR_UNIT_MHZ);
            }
            break;

        case MENU_TPLIST_ID_ENTRY_SYMBRATE:
            if ( inst->carrierDatValid )
            {
                item->type  = FGS_ITEM_TYPE_NUMBER;
                if ( inst->params.system == FE_SYSTEM_TYPE_CABLE )
                {
                    item->item.number.value = inst->currCarrierDat.pars.cable.symbolRate;
                }
                else
                {
                    item->item.number.value = inst->currCarrierDat.pars.sat.symbolRate;
                }

                item->item.number.format        = FGS_FORMAT_NUM_NORM;
                item->item.number.decimalPlaces = 0;
                item->item.number.unitStr       = OSD_GetStr(STR_UNIT_KBAUD);
            }
            break;

        case MENU_TPLIST_ID_ENTRY_MODULATION:

            if ( inst->carrierDatValid )
            {
                #ifdef APPL_FECABLE_ENABLED

                if ( inst->params.system == FE_SYSTEM_TYPE_CABLE )
                {
                    switch ( index )
                    {
                        default:
                        case CAB_FE_MOD_QAM_16:  strIdx = STR_GNR_MOD_QAM16;  break;
                        case CAB_FE_MOD_QAM_32:  strIdx = STR_GNR_MOD_QAM32;  break;
                        case CAB_FE_MOD_QAM_64:  strIdx = STR_GNR_MOD_QAM64;  break;
                        case CAB_FE_MOD_QAM_128: strIdx = STR_GNR_MOD_QAM128; break;
                        case CAB_FE_MOD_QAM_256: strIdx = STR_GNR_MOD_QAM256; break;
                    }
                }

                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FESAT_ENABLED
                if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
                {
                    switch ( index )
                    {
                        default:
                        case SAT_FE_MOD_QPSK   : strIdx = STR_VALUE_TPLIST_MOD_QPSK;    break;
                        case SAT_FE_MOD_QPSK_S2: strIdx = STR_VALUE_TPLIST_MOD_QPSK_S2; break;
                        case SAT_FE_MOD_8PSK   : strIdx = STR_VALUE_TPLIST_MOD_8PSK;    break;
                    }
                }
                #endif /* APPL_FESAT_ENABLED */
            }
            item->item.str.chr        = OSD_GetStr(strIdx);
            item->type                = FGS_ITEM_TYPE_STRING;
            item->item.str.lineHeight = 0;
            break;
        #ifdef APPL_FESAT_ENABLED
        case MENU_TPLIST_ID_ENTRY_POL:
            if ( inst->carrierDatValid )
            {
                switch ( index )
                {
                    default:
                    case FE_POL_HOR  : strIdx = STR_VALUE_TPLIST_POL_HOR; break;
                    case FE_POL_VER  : strIdx = STR_VALUE_TPLIST_POL_VER; break;
                }

                item->item.str.chr        = OSD_GetStr(strIdx);
                item->type                = FGS_ITEM_TYPE_STRING;
            }
            break;
        #endif
        default:  return;
    }

    switch ( item->type )
    {
        case FGS_ITEM_TYPE_STRING:
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
            }
            break;

        case FGS_ITEM_TYPE_NUMBER:
            item->item.number.font = MAIN_FONT;
            switch ( state )
            {
                default:
                case FGS_NONFOCUS: item->item.number.col = appDat->txtAttr.colNonFocus; break;
                case FGS_FOCUS:    item->item.number.col = appDat->txtAttr.colFocus;    break;
                case FGS_DISABLED: item->item.number.col = appDat->txtAttr.colDisabled; break;
            }
            break;

        default:
            break;
    }
}

FSTATIC fbool_t MENU_TpListKeys (FAPI_SYS_HandleT h)
{
    MENU_TPLIST_INST_S*   inst = (MENU_TPLIST_INST_S*)h;
    fbool_t                 keyUsed = FTRUE;
    TPLIST_ITEM_S         tpDat;
    MENU_CONFDIALOG_S     confPars;
    int16_t                   numTp = 0;
    int32_t                   retVal;
    FGS_ITEM_SEL_PARAMS_S selParams;

    memset(&selParams, 0, sizeof(selParams));

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_GREEN:  /* add carrier */
            if ( inst->networkValid && (!(inst->newCarrier)) )
            {
                memset (&tpDat, 0, sizeof(tpDat));

                tpDat.tsId       = -1;
                tpDat.networkId  = -1;
                tpDat.onId       = -1;
                tpDat.sigQuality = 0;

                retVal = (int32_t)TPLIST_AddCarrier(&(inst->currNetwork),
                                                &tpDat, SYS_OWNER_USER,
                                                &(inst->params.carrierLstPos));

                if ( retVal == FAPI_OK )
                {
                    inst->currCarrierDat  = tpDat;
                    #ifdef APPL_FESAT_ENABLED
                    if(inst->params.system == FE_SYSTEM_TYPE_CABLE)
                    {
                        inst->currCarrierDat.pars.cable.modulation = FE_MOD_QAM_16;
                    }
                    #endif
                    #ifdef APPL_FESAT_ENABLED
                    if(inst->params.system == FE_SYSTEM_TYPE_SATELLITE)
                    {
                        inst->currCarrierDat.pars.sat.modulation = FE_MOD_QPSK;
                    }
                    #endif
                    inst->carrierDatValid = FTRUE;

                    inst->newCarrier      = FTRUE;

                    (void)TPLIST_GetNumCarriers(&(inst->currNetwork), &numTp);

                    selParams.numItems = numTp;
                    selParams.index    = inst->params.carrierLstPos;

                    (void)FGS_LIST_SetMemberParams (inst->listHdl,
                                              MENU_TPLIST_ID_ENTRY_SELTP,
                                              &selParams);
                }
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case FGS_KEY_RED:
            if ( inst->networkValid &&
                 inst->carrierDatValid )
            {
                /* open a confirm dialog to re-confirm the user's action */
                memset(&confPars, 0, sizeof(confPars));
                confPars.numDescLines = 1;
                confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_REM_TP);
                confPars.confirmStr   = OSD_GetStr(STR_CONFDIALOG_REM_TP_OK);
                confPars.discardStr   = OSD_GetStr(STR_CONFDIALOG_REM_TP_ABORT);
                confPars.onConfirm    = MENU_TpListConfirm;
                confPars.onDecline    = NULL;
                confPars.onDiscard    = NULL;
                confPars.arg          = inst;

                (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;

}

FSTATIC fbool_t MENU_TpListBuild(FAPI_SYS_HandleT h)
{
    MENU_TPLIST_INST_S*   inst = (MENU_TPLIST_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_LIST_OPEN_S       listPars;
    FGS_LIST_OPEN_S       helpListPars;
    FAPI_SYS_HandleT      itemHdl;
    FGS_BOX_OPEN_S        boxPars;
    FGS_ITEM_NUM_OPEN_S   numPars;
    MAIN_SIGBOX_OPEN_S    sigPars;
    FGS_ITEM_SEL_OPEN_S   selPars;
    FGS_ITEM_TXT_OPEN_S   helpPars;
    FGS_POS_S             itemPos;
    FGS_ITEM_S            item;
    FGS_POS_S             valuePos;
    FGS_POS_S             descrPos;
    int32_t               retVal;
    uint16_t              helpWidth;
    uint16_t              stackedFrameWidth;
    uint16_t              menuHeight;
    uint16_t              menuWidth;
    int16_t               satNums = 0;
    int16_t               numTp   = 0;
    uint32_t              numLines;
    uint16_t              idCnt;
    /* Bottom Help Box Parameters */
    uint16_t              mainPanelWidth;
    uint16_t              bottomHelpKeyWidth;
    uint16_t              bottomHelpItemWidth;
    uint8_t               botHelpLinesC1;
    uint8_t               botHelpLinesC2;
    uint8_t               bottomHelpNumLines1;
    uint8_t               bottomHelpNumLinesTotal;
    uint8_t               stackLevel;
    uint16_t              listHeight;

    memset(&boxPars,      0, sizeof(boxPars));
    memset(&selPars,      0, sizeof(selPars));
    memset(&numPars,      0, sizeof(numPars));
    memset(&sigPars,      0, sizeof(sigPars));
    memset(&item,         0, sizeof(item));
    memset(&helpListPars, 0, sizeof(helpListPars));
    memset(&helpPars,     0, sizeof(helpPars));

    if( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
    {
        numLines = 6;
    }
    else
    {
        numLines = 4;
    }
    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_09LR),
                                   -1,SYMBOL_FONT);

    stackLevel        = (uint8_t)MENUSTACK_GetLevel();
    if ( stackLevel > 0 )
    {
        stackLevel--;
    }

    stackedFrameWidth = stackLevel *
                        (SCALER_GetCurrVal(STACKED_FRAME_DIST) + SCALER_GetCurrVal(OUTER_FRAME_ALL));
    menuHeight        = stackedFrameWidth + (2*SCALER_GetCurrVal(OUTER_FRAME_ALL)) +
                        (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) + SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                        SCALER_GetCurrVal(MAIN_TOP_PADDING) + SCALER_GetCurrVal(MAIN_BOT_PADDING) +
                        ( numLines        * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                        ((numLines - 1 )  * SCALER_GetCurrVal(LIST_LINE_PADDING)) +
                        (2*SCALER_GetCurrVal(HELP_TB_PADDING)) + (2*SCALER_GetCurrVal(INNER_FRAME_WIDTH)) +
                        MAIN_SignalBoxGetHeight();
    menuWidth         = SCALER_GetCurrVal(MAIN_WIDTH) + helpWidth +
                        2*SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                        SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                        stackedFrameWidth;

    /* -----Calculate bottom help parameter ---------------------------- */
    mainPanelWidth      = menuWidth - stackedFrameWidth - 2*SCALER_GetCurrVal(OUTER_FRAME_ALL) ;
    bottomHelpKeyWidth  = SCALER_GetCurrVal(HELP_BOT_XOFF) -
                          SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
    bottomHelpItemWidth = mainPanelWidth/2 -
                          bottomHelpKeyWidth -
                          OSD_GetStrWidth(OSD_GetStr(STR_GNR_COLON),-1, HELP_FONT) -
                          SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

    botHelpLinesC1 = (uint8_t)FGS_GetLinesFromString (OSD_GetStr(STR_HELP_TPLIST_ADDTP),
                                             bottomHelpItemWidth, HELP_FONT,
                                             NULL);

    botHelpLinesC2 =(uint8_t) FGS_GetLinesFromString (OSD_GetStr(STR_HELP_TPLIST_REMOVETP),
                                             bottomHelpItemWidth, HELP_FONT, NULL);

    bottomHelpNumLinesTotal = bottomHelpNumLines1
                            = ( botHelpLinesC1 >= botHelpLinesC2 ?
                                                  botHelpLinesC1 :
                                                  botHelpLinesC2);
    botHelpLinesC1 = (uint8_t)FGS_GetLinesFromString (OSD_GetStr(STR_HELP_MENU_BACK),
                                             bottomHelpItemWidth, HELP_FONT,
                                             NULL);
    botHelpLinesC2 = (uint8_t)FGS_GetLinesFromString (OSD_GetStr(STR_HELP_MENU_EXIT),
                                             bottomHelpItemWidth, HELP_FONT,
                                             NULL);

    bottomHelpNumLinesTotal += (botHelpLinesC1 >= botHelpLinesC2 ?
                                                      botHelpLinesC1 :
                                                      botHelpLinesC2);

    do
    {
        memset(&pnlPars,  0, sizeof(pnlPars));

        pnlPars.pos.width       = menuWidth;
        pnlPars.pos.height      = menuHeight +
                                  ( bottomHelpNumLinesTotal    * SCALER_GetCurrVal(HELP_LINE_HEIGHT)) +
                                  ((bottomHelpNumLinesTotal -1)* SCALER_GetCurrVal(HELP_LINE_PADDING));
        pnlPars.bkgrType        = (OSD_PNLTYPE_MENUWTITLEWHELP0 + bottomHelpNumLinesTotal)
                                | (((uint16_t)stackLevel) << 8)
                                | (((uint32_t)helpWidth) << 16);
        pnlPars.onFocusSet      = MENU_TpListOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;
        pnlPars.pInst           = &(appDat->pnlInst);

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_TPLIST, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_TPLIST_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_TPLIST,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        listHeight = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                               + SCALER_GetCurrVal(LIST_LINE_PADDING));

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_TpListGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_TpListGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_TpListOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_TpListOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_TpListValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_TPLIST_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            listHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;

        /* NOTE: Common help list method not used here because of non-linear
                 item position. */

        helpListPars = listPars;

        helpListPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                       SCALER_GetCurrVal(MAIN_WIDTH) +
                                       SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                       SCALER_GetCurrVal(HELP_LR_PADDING) +
                                       stackedFrameWidth;
        helpListPars.pos.width       = helpWidth - 2 * SCALER_GetCurrVal(HELP_LR_PADDING);
        helpListPars.bkgrType        = 0;
        helpListPars.getDescrItem    = MENU_TpListGetHelpDescrItem;
        helpListPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        helpListPars.getItem         = NULL;
        helpListPars.getItemArg      = NULL;
        helpListPars.onFocusChg      = NULL;
        helpListPars.onFocusChgArg   = NULL;
        helpListPars.onSelect        = NULL;
        helpListPars.onSelectArg     = NULL;
        helpListPars.onValChg        = NULL;
        helpListPars.onValChgArg     = NULL;
        helpListPars.pInst           = &(appDat->helpListInst);

        retVal = FGS_LIST_Open (&helpListPars, MENU_TPLIST_ID_HELPLIST,
                                inst->pnlHdl, &(inst->listHelpHdl));
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.x              = 0;
        helpPars.pos.y              = 0;
        helpPars.pos.width          = helpListPars.pos.width;
        helpPars.pos.height         = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        helpPars.pos.alignment      = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        helpPars.bkgrType           = OSD_LSTITEM_BKGR_HELP;
        helpPars.descrPos.x         = 0;
        helpPars.descrPos.y         = 0;
        helpPars.descrPos.width     = helpPars.pos.width;
        helpPars.descrPos.height    = helpPars.pos.height;
        helpPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* --- Select Sat ----------------------------------------------------*/
        #ifdef APPL_FESAT_ENABLED

        (void)CFGSAT_GetNumSats(&satNums);
        if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
        {
            (void) CFGSAT_GetNumSats(&satNums);
            selPars.params.numItems     = satNums;
            selPars.params.index        = inst->params.satLstPos;

            retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TPLIST_ID_ENTRY_SELSAT,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            helpPars.pos.y = itemPos.y;
            retVal = FGS_ITEM_TXT_Open (&helpPars,
                                        MENU_TPLIST_ID_ENTRY_SELSAT,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            /* disable satellite selection if requested by caller */
            if ( inst->params.lockSat )
            {
                (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                                MENU_TPLIST_ID_ENTRY_SELSAT,
                                                FFALSE);
            }

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        }
        #endif /* APPL_FESAT_ENABLED */

        /* --- TPLIST Select TP ----------------------------------------*/

        (void)TPLIST_GetNumCarriers (&(inst->currNetwork), &numTp);

        selPars.pos                 = itemPos;
        selPars.descrPos.width      = OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_TPLIST_SELTP), -1,MAIN_FONT);
        selPars.valuePos.width      = itemPos.width -
                                      selPars.descrPos.width -
                                      SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        selPars.valuePos.x          = itemPos.x + selPars.descrPos.width;
        selPars.params.numItems     = numTp;
        selPars.params.index        = inst->params.carrierLstPos;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TPLIST_ID_ENTRY_SELTP,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TPLIST_ID_ENTRY_SELTP,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- TPLIST frequency ------------------------------------*/

        numPars.pos              = itemPos;
        numPars.type             = FGS_NITEM_TYPE_NUMBER;
        numPars.descrPos         = descrPos;
        numPars.valuePos         = valuePos;
        numPars.bkgrType         = OSD_LSTITEM_BKGR_NORMAL;
        numPars.wraparound       = FTRUE;
        if ( inst->params.system == FE_SYSTEM_TYPE_CABLE )
        {
            numPars.params.minValue  = TP_CABLE_MIN_FREQ;
            numPars.params.maxValue  = TP_CABLE_MAX_FREQ;
        }
        else
        {
            numPars.params.minValue  = TP_SAT_MIN_FREQ;
            numPars.params.maxValue  = TP_SAT_MAX_FREQ;
        }
        numPars.params.maxDigits = 5;
        numPars.params.step      = 1;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TPLIST_ID_ENTRY_FREQ,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TPLIST_ID_ENTRY_FREQ,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- TPLIST Symbol rate------------------------------------*/

        numPars.pos              = itemPos;
        numPars.bkgrType         = FGS_NITEM_TYPE_NUMBER;
        numPars.descrPos         = descrPos;
        numPars.valuePos         = valuePos;
        numPars.type             = (FGS_ITEM_NUM_TYPE_E)OSD_LSTITEM_BKGR_NORMAL;
        numPars.wraparound       = FTRUE;
        numPars.params.minValue  = 0;
        numPars.params.maxValue  = 50000;
        numPars.params.maxDigits = 5;
        numPars.params.step      = 100;

        retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TPLIST_ID_ENTRY_SYMBRATE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TPLIST_ID_ENTRY_SYMBRATE,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- TPLIST Modulation -------------------------------*/

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        #ifdef APPL_FECABLE_ENABLED
        if ( inst->params.system == FE_SYSTEM_TYPE_CABLE )
        {
            selPars.params.numItems     = CAB_FE_MOD_NUMS;
            switch( inst->currCarrierDat.pars.cable.modulation )
            {
                default:
                case FE_MOD_QAM_16 : selPars.params.index = CAB_FE_MOD_QAM_16;  break;
                case FE_MOD_QAM_32 : selPars.params.index = CAB_FE_MOD_QAM_32;  break;
                case FE_MOD_QAM_64 : selPars.params.index = CAB_FE_MOD_QAM_64;  break;
                case FE_MOD_QAM_128: selPars.params.index = CAB_FE_MOD_QAM_128; break;
                case FE_MOD_QAM_256: selPars.params.index = CAB_FE_MOD_QAM_256; break;
            }
        }
        #endif
        #ifdef APPL_FESAT_ENABLED
        if ( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
        {
            selPars.params.numItems     = SAT_FE_MOD_NUMS;
            switch( inst->currCarrierDat.pars.sat.modulation )
            {
                default:
                case FE_MOD_QPSK   : selPars.params.index = SAT_FE_MOD_QPSK;    break;
                case FE_MOD_QPSK_S2: selPars.params.index = SAT_FE_MOD_QPSK_S2; break;
                case FE_MOD_8PSK   : selPars.params.index = SAT_FE_MOD_8PSK;    break;
            }
        }
        #endif
        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TPLIST_ID_ENTRY_MODULATION,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        helpPars.pos.y = itemPos.y;
        retVal = FGS_ITEM_TXT_Open (&helpPars,
                                    MENU_TPLIST_ID_ENTRY_MODULATION,
                                    inst->listHelpHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- TPLIST Polarisation -------------------------------*/
        #ifdef APPL_FESAT_ENABLED
        if( inst->params.system == FE_SYSTEM_TYPE_SATELLITE )
        {
            selPars.pos                 = itemPos;
            selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
            selPars.descrPos            = descrPos;
            selPars.valuePos            = valuePos;
            selPars.params.numItems     = 2;
            selPars.params.index        = inst->currCarrierDat.pars.sat.pol;

            retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TPLIST_ID_ENTRY_POL,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            helpPars.pos.y = itemPos.y;
            retVal = FGS_ITEM_TXT_Open (&helpPars,
                                        MENU_TPLIST_ID_ENTRY_POL,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        }
        #endif
        /* --- Signal strength -------------------------------------------- */
        /* --- Signal quality --------------------------------------------- */
        /* --- Signal bar ------------------------------------------------- */

        sigPars.pnlHdl     = inst->pnlHdl;
        sigPars.pos.x      = itemPos.x + listPars.pos.x + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        sigPars.pos.y      = itemPos.y + listPars.pos.y + SCALER_GetCurrVal(LIST_LINE_PADDING);
        sigPars.descrWidth = ((uint16_t)(itemPos.width- SCALER_GetCurrVal(MAIN_LEFT_PADDING))/(2));
        sigPars.barWidth   = ((uint16_t)(itemPos.width- SCALER_GetCurrVal(MAIN_LEFT_PADDING)-SCALER_GetCurrVal(MAIN_RIGHT_PADDING))/(2));
        sigPars.feIdx      = -1;

        retVal = MAIN_SignalBoxOpen(&(appDat->sigBox), &sigPars);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += MAIN_SignalBoxGetHeight();
        itemPos.y += (2*SCALER_GetCurrVal(LIST_LINE_PADDING));

        /* -----Bottom help lines ------------------------------------------ */

        idCnt = MENU_TPLIST_ID_HELPBOTTOM;

        item.type                = FGS_ITEM_TYPE_ICON;
        item.align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        item.item.icon.addr      = (uint32_t*)OSD_IconGet(ICON_BTNREDSMALL);
        item.item.icon.drawMode  = FGS_ICONMODE_OVRLAY;
        item.item.icon.bkgrCol   = OSD_COL_BLANK;
        item.item.icon.altBkgrCol= OSD_COL_BLANK;

        boxPars.pos.x               = stackedFrameWidth +
                                      SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - bottomHelpNumLinesTotal*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - (bottomHelpNumLinesTotal-1)*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.height          = bottomHelpNumLines1 * SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                                      (bottomHelpNumLines1 - 1) * SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;


        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);
        item.align = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        boxPars.pos.x               += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_TPLIST_REMOVETP);
        item.item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;


        item.type                = FGS_ITEM_TYPE_ICON;
        item.item.icon.addr      = (uint32_t*)OSD_IconGet(ICON_BTNGREENSMALL);
        item.item.icon.drawMode  = FGS_ICONMODE_OVRLAY;
        item.item.icon.bkgrCol   = OSD_COL_BLANK;
        item.item.icon.altBkgrCol= OSD_COL_BLANK;
        boxPars.pos.x               = stackedFrameWidth +
                                      SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      mainPanelWidth/2;
        boxPars.pos.width           = bottomHelpKeyWidth; // 20;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_GNR_COLON);
        item.align = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        boxPars.pos.x               += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_TPLIST_ADDTP);
        item.item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -- 2nd Bottom line -- */

        boxPars.pos.y += bottomHelpNumLines1*SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                         (bottomHelpNumLines1-1)*SCALER_GetCurrVal(HELP_LINE_PADDING);

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                boxPars.pos.y,
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

        /*-------------------------------------------------------------------*/

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    return FTRUE;
}


FSTATIC void MENU_TpListConfirm(void* arg)
{
    MENU_TPLIST_INST_S*   inst  = (MENU_TPLIST_INST_S*)arg;
    int16_t                   numTp = 0;
    FGS_ITEM_SEL_PARAMS_S selParams;

    memset(&selParams, 0, sizeof(selParams));

    if ( TPLIST_RemoveCarrier(&(inst->currNetwork), inst->params.carrierLstPos)
         == FAPI_OK )
    {
        inst->tpChg      = FTRUE;
        inst->newCarrier = FFALSE;

        (void)TPLIST_GetNumCarriers(&(inst->currNetwork), &numTp);

        if ( numTp < 1 )
        {
            inst->params.carrierLstPos = 0;
        }
        else if ( inst->params.carrierLstPos >= numTp )
        {
            inst->params.carrierLstPos = (numTp - 1);
        }
        else if ( inst->params.carrierLstPos > 0 )
        {
            inst->params.carrierLstPos--;
        }

        selParams.numItems = numTp;
        selParams.index    = inst->params.carrierLstPos;

        (void)FGS_LIST_SetMemberParams(inst->listHdl, MENU_TPLIST_ID_ENTRY_SELTP,
                                 &selParams);
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
