/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_tuner.c
**
**              $Id: app_menu_tuner.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Tuner configuration menu.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:

** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define LNB_MIN_FREQ  1000 // MHz
#define LNB_MAX_FREQ 19999 // MHz

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_TUNER_ID_TITLE = 0,
    MENU_TUNER_ID_LIST,
    MENU_TUNER_ID_REST,
    MENU_TUNER_ID_HELPLIST
};

enum
{
    #ifdef APPL_FESAT_ENABLED

    MENU_TUNER_ID_CONTROLMETHOD,
    MENU_TUNER_ID_SELSAT,
    MENU_TUNER_ID_HIGHFREQ,
    MENU_TUNER_ID_LOWFREQ,
    MENU_TUNER_ID_CLASSICID,
    MENU_TUNER_ID_TONEBURSTID,
    MENU_TUNER_ID_DISEQC10ID,
    MENU_TUNER_ID_DISEQC11ID,
    MENU_TUNER_ID_DISEQC11ID_SMATV,
    MENU_TUNER_ID_DISEQC11REP,
    MENU_TUNER_ID_DISEQC12POS,
    MENU_TUNER_ID_MOVESTYLE,
    MENU_TUNER_ID_MOVELR,
    MENU_TUNER_ID_STORESAT,

    #ifdef APPL_DISEQC11_SMATV_ENABLED

    MENU_TUNER_ID_IF_FREQ,

    #endif

    #ifdef APPL_DISEQC12_ENABLED

    MENU_TUNER_ID_EXPMOTOR,

    #endif

    MENU_TUNER_ID_SELTP,
    MENU_TUNER_ID_LOCK,
    MENU_TUNER_ID_LONGITUDE,
    MENU_TUNER_ID_LATITUDE,
    MENU_TUNER_ID_TURNMOTOR,

    #endif /* APPL_FESAT_ENABLED */

    #ifdef APPL_FETERR_ENABLED

    MENU_TUNER_ID_ANTENNA_POWER,

    #endif /* APPL_FETERR_ENABLED */

    MENU_TUNER_ID_GOTONEXT,

    #ifdef APPL_FESAT_ENABLED

    MENU_TUNER_HELP_ID_CONTROLMETHOD,
    MENU_TUNER_HELP_ID_SELSAT,
    MENU_TUNER_HELP_ID_HIGHFREQ,
    MENU_TUNER_HELP_ID_LOWFREQ,
    MENU_TUNER_HELP_ID_IF_FREQ,
    MENU_TUNER_HELP_ID_CLASSICID,
    MENU_TUNER_HELP_ID_TONEBURSTID,
    MENU_TUNER_HELP_ID_DISEQC10ID,
    MENU_TUNER_HELP_ID_DISEQC11ID,
    MENU_TUNER_HELP_ID_DISEQC11ID_SMATV,
    MENU_TUNER_HELP_ID_DISEQC11REP,
    MENU_TUNER_HELP_ID_DISEQC12POS,
    MENU_TUNER_HELP_ID_APPLY,
    MENU_TUNER_HELP_ID_GOTOPOS,
    MENU_TUNER_HELP_ID_MOVESTYLE,
    MENU_TUNER_HELP_ID_MOVELR,
    MENU_TUNER_HELP_ID_STORESAT,
    MENU_TUNER_HELP_ID_EXPMOTOR,
    MENU_TUNER_HELP_ID_SELTP,
    MENU_TUNER_HELP_ID_LONGITUDE,
    MENU_TUNER_HELP_ID_LATITUDE,
    MENU_TUNER_HELP_ID_TURNMOTOR,

    #endif /* APPL_FESAT_ENABLED */

    #ifdef APPL_FETERR_ENABLED

    MENU_TUNER_HELP_ID_ANTENNA_POWER,

    #endif /* APPL_FETERR_ENABLED */

    MENU_TUNER_HELP_ID_GOTONEXT
};

#if defined(APPL_DISEQC12_ENABLED) || defined(APPL_USALS_ENABLED)
enum
{
    MOVE_MOTOR_WEST=0,
    MOVE_MOTOR_STOP,
    MOVE_MOTOR_EAST,
    MOVE_MOTOR_NUMS
};
#endif


typedef enum
{
    MENU_TUNER_MOVE_STEP = 0,
    MENU_TUNER_MOVE_CONT,
    MENU_TUNER_MOVE_1SEC,
    MENU_TUNER_MOVE_NUMS

} MENU_TUNER_MOVESTYLE;

typedef struct
{
    /* open parameters */
    fbool_t                     guidedMode;
    uint8_t                     feIdx;
    uint8_t                     feLast;

    FE_SYSTEM_TYPE_E            feType;

    FAPI_SYS_HandleT            pnlHdl;
    FAPI_SYS_HandleT            listHdl;
    FAPI_SYS_HandleT            listHelpHdl;


    #ifdef APPL_FESAT_ENABLED

    int16_t                     satLstPos;
    CFGSAT_SAT_S                currSatDat;
    fbool_t                     satDatValid;
    CFGSAT_SAT_S                tmpSatDat;

    CFGSAT_FEPARS_S             currFeDat;

    int16_t                     carrierLstPos; /* reference carrier for signal bar display */
    fbool_t                     sigBarOpen;
    MENU_TUNER_MOVESTYLE        currMoveStyle;

    #endif /* APPL_FESAT_ENABLED */

    #ifdef APPL_FETERR_ENABLED

    fbool_t                     antennaPowerTerr;

    #endif /* APPL_FETERR_ENABLED */

    fbool_t                     dataChg;

    FGS_PANEL_MEMBER_S*         pItem;
    FGS_POS_S                   itemPos;

} MENU_TUNER_INST_S;

FSTATIC MENU_TUNER_INST_S* menuTunerDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_TunerLeave              (FAPI_SYS_HandleT h,
                                              fbool_t force);
FSTATIC fbool_t MENU_TunerOnSelect           (FAPI_SYS_HandleT h,
                                              int32_t itemId);
FSTATIC void    MENU_TunerValChg             (FAPI_SYS_HandleT h,
                                              int32_t itemId,
                                              FGS_VAL_S newVal);
FSTATIC void    MENU_TunerGetDescrItem       (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void    MENU_TunerGetHelpDescrItem   (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void    MENU_TunerGetItemTitle       (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void    MENU_TunerGetItem            (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
#if(0)
FSTATIC void    MENU_TunerGetTitleItem       (FAPI_SYS_HandleT h,
                                              int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
#endif
FSTATIC fbool_t MENU_TunerKeys               (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_TunerBuild              (FAPI_SYS_HandleT h);
FSTATIC void    MENU_TunerOnFocusChg         (FAPI_SYS_HandleT h,
                                              uint8_t index,
                                              int32_t id);

#ifdef APPL_FESAT_ENABLED

FSTATIC int32_t MENU_TunerDraw               (FAPI_SYS_HandleT h);
FSTATIC void    MENU_TunerUpdateSignalDisplay(MENU_TUNER_INST_S* inst);
FSTATIC void    MENU_TunerUpdateCarrierSel   (int16_t satLstPos,
                                              int16_t carrierLstPos);
#ifdef APPL_DISEQC12_ENABLED
FSTATIC void    MENU_TunerMotorHalt          (void);
#endif
#endif /* APPL_FESAT_ENABLED */

FSTATIC void MENU_TunerOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus);

/*!
*******************************************************************************
**
** \brief  Initial function for the OSD Tuner settings Menu.
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_TunerInit (void)
{
    MENU_TUNER_INST_S* inst;

    if ( menuTunerDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuTunerDat = UTILS_Malloc(sizeof(MENU_TUNER_INST_S));
    if ( menuTunerDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(menuTunerDat, 0, sizeof(MENU_TUNER_INST_S));

    inst = menuTunerDat;

    inst->guidedMode    = FFALSE;
    inst->feIdx         = 0;
    inst->feLast        = 0;

    #ifdef APPL_FESAT_ENABLED

    inst->satLstPos     = 0;
    inst->carrierLstPos = 0;

    #endif /* APPL_FESAT_ENABLED */

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_TunerEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_TUNER_INST_S* inst = menuTunerDat;
    uint32_t           lastFocusItem;
    SYS_FE_STATUS_S    feStatus;

    #ifdef APPL_FESAT_ENABLED

    CFGSAT_IDX         newIdx;
    fbool_t            satPosValid = FFALSE;
    int16_t            lstPos;

    #endif /* APPL_FESAT_ENABLED */

    if ( inst == NULL ) return FFALSE;

    if ( test == FTRUE ) return FTRUE;

    if ( h != NULL )
    {
        inst->guidedMode    =          ((MENU_GUIDED_PARAMS_S*)h)->guided;
        inst->feIdx         = (uint8_t)((MENU_GUIDED_PARAMS_S*)h)->arg1;
        inst->feLast        = (uint8_t)((MENU_GUIDED_PARAMS_S*)h)->arg2;

        #ifdef APPL_FESAT_ENABLED

        inst->carrierLstPos = 0;

        #endif /* APPL_FESAT_ENABLED */
    }

    inst->feType = FE_SYSTEM_TYPE_DIRECT;

    if ( AV_GetFeStatus(inst->feIdx, &feStatus) == FAPI_OK )
    {
        inst->feType = feStatus.type;
    }

    switch ( inst->feType )
    {
        #ifdef APPL_FESAT_ENABLED

        case FE_SYSTEM_TYPE_SATELLITE:
            inst->satLstPos     = 0;
            inst->satDatValid   = FFALSE;
            inst->sigBarOpen    = FFALSE;
            inst->currMoveStyle = MENU_TUNER_MOVE_STEP;

            /* ----- get initial satellite selection ----------------------- */

            if ( appDat->currSatIdx >= 0 )
            {
                if ( CFGSAT_GetListPos (appDat->currSatIdx, &lstPos) == FAPI_OK )
                {
                    inst->satLstPos = lstPos;
                    satPosValid     = FTRUE;
                }
            }

            /* If satellite is invalid/not in list,
               choose first satellite in list */
            if ( !satPosValid )
            {
                inst->satLstPos = 0;

                if ( CFGSAT_GetIndex (inst->satLstPos, &newIdx) == FAPI_OK )
                {
                    appDat->currSatIdx = newIdx;
                    satPosValid        = FTRUE;
                }
            }

            if ( satPosValid )
            {
                if ( CFGSAT_GetSatData(inst->satLstPos, &(inst->currSatDat)) == FAPI_OK )
                {
                    inst->satDatValid = FTRUE;
                }
            }

            memset(&(inst->currFeDat), 0, sizeof(inst->currFeDat));

            (void)CFGSAT_GetFrontendSetup(&(inst->currFeDat));
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case FE_SYSTEM_TYPE_TERRESTRIAL:
            inst->antennaPowerTerr = FFALSE;

            (void)SETTINGS_AntennaPowerGet((int8_t)inst->feIdx,
                                     &(inst->antennaPowerTerr));
            break;

        #endif /* APPL_FETERR_ENABLED */

        default:
            break;
    }

    inst->dataChg = FFALSE;

    (void)MENU_TunerBuild((FAPI_SYS_HandleT)inst);

    if ( MENUSTACK_GetData(&lastFocusItem) == FFALSE )
    {
        if( inst->guidedMode )
        {
            lastFocusItem = MENU_TUNER_ID_GOTONEXT;
        }
        else
        {
            switch ( inst->feType )
            {
               #ifdef APPL_FESAT_ENABLED

               case FE_SYSTEM_TYPE_SATELLITE:
                   lastFocusItem = MENU_TUNER_ID_CONTROLMETHOD;
                   break;

               #endif /* APPL_FESAT_ENABLED */

               #ifdef APPL_FETERR_ENABLED

               case FE_SYSTEM_TYPE_TERRESTRIAL:
                   lastFocusItem = MENU_TUNER_ID_ANTENNA_POWER;
                   break;

               #endif /* APPL_FETERR_ENABLED */

               default:
                   lastFocusItem = 0;
                   break;
            }
        }
    }

    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, lastFocusItem);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_TUNER_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_TUNER);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);

    #ifdef APPL_FESAT_ENABLED

    MENU_TunerUpdateSignalDisplay (inst);

    #endif /* APPL_FESAT_ENABLED */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_TunerLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_TunerOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_TUNER_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_MENU,  MENU_TunerKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK,  MENU_TunerKeys, h, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
    }
}

/*!
*******************************************************************************
**
** \brief  Activates the help icons for the current menu list item.
**
** \param h        Address of menu handle.
** \param index    unused.
** \param id       List id gets focus.
**
******************************************************************************/
FSTATIC void MENU_TunerOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_TUNER_INST_S*   inst = (MENU_TUNER_INST_S*)h;

    switch ( inst->feType )
    {
        #ifdef APPL_FESAT_ENABLED

        case FE_SYSTEM_TYPE_SATELLITE:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_CONTROLMETHOD, FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_SELSAT,        FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_HIGHFREQ,      FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_LOWFREQ,       FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_IF_FREQ,       FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_CLASSICID,     FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_TONEBURSTID,   FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC10ID,    FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC11ID,    FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC11ID_SMATV, FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC11REP,   FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC12POS,   FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_APPLY,         FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_GOTOPOS,       FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_MOVESTYLE,     FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_MOVELR,        FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_STORESAT,      FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_EXPMOTOR,      FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_SELTP,         FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_LONGITUDE,     FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_LATITUDE,      FFALSE);
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_TURNMOTOR,     FFALSE);
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case FE_SYSTEM_TYPE_TERRESTRIAL:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_ANTENNA_POWER, FFALSE);
            break;

        #endif /* APPL_FETERR_ENABLED */
        default:
            break;
    }

    (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_GOTONEXT,      FFALSE);

    switch ( id )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_TUNER_ID_CONTROLMETHOD :
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_CONTROLMETHOD, FTRUE);
            break;
        case MENU_TUNER_ID_SELSAT:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_SELSAT,        FTRUE);
            break;
        case MENU_TUNER_ID_HIGHFREQ:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_HIGHFREQ,      FTRUE);
            break;
        case MENU_TUNER_ID_LOWFREQ:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_LOWFREQ,       FTRUE);
            break;
        case MENU_TUNER_ID_CLASSICID:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_CLASSICID,     FTRUE);
            break;
        case MENU_TUNER_ID_TONEBURSTID:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_TONEBURSTID,   FTRUE);
            break;
        case MENU_TUNER_ID_DISEQC10ID:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC10ID,    FTRUE);
            break;
        case MENU_TUNER_ID_DISEQC11ID:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC11ID,    FTRUE);
            break;
        case MENU_TUNER_ID_DISEQC11ID_SMATV:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC11ID_SMATV, FTRUE);
            break;
        case MENU_TUNER_ID_DISEQC11REP:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC11REP,   FTRUE);
            break;
        case MENU_TUNER_ID_DISEQC12POS :
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_DISEQC12POS,   FTRUE);
            break;
        case MENU_TUNER_ID_MOVESTYLE:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_MOVESTYLE,     FTRUE);
            break;
        case MENU_TUNER_ID_MOVELR:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_MOVELR,       FTRUE);
            break;
        case MENU_TUNER_ID_STORESAT:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_STORESAT,      FTRUE);
            break;

        #ifdef APPL_DISEQC11_SMATV_ENABLED

        case MENU_TUNER_ID_IF_FREQ:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_IF_FREQ,      FTRUE);
            break;
        #endif
        #ifdef APPL_DISEQC12_ENABLED

        case MENU_TUNER_ID_EXPMOTOR:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_EXPMOTOR,      FTRUE);
            break;

        #endif

        case MENU_TUNER_ID_SELTP :
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_SELTP,         FTRUE);
            break;
        case MENU_TUNER_ID_LONGITUDE:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_LONGITUDE,     FTRUE);
            break;
        case MENU_TUNER_ID_LATITUDE:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_LATITUDE,      FTRUE);
            break;
        case MENU_TUNER_ID_TURNMOTOR:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_TURNMOTOR,     FTRUE);
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case MENU_TUNER_ID_ANTENNA_POWER:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_ANTENNA_POWER, FTRUE);
            break;

        #endif /* APPL_FETERR_ENABLED */

        case MENU_TUNER_ID_GOTONEXT:
            (void)FGS_LIST_SetMemberEnableStatus(inst->listHelpHdl, MENU_TUNER_HELP_ID_GOTONEXT,      FTRUE);
            break;

        default:
            /* do nothing */
            break;
    }
}

FSTATIC fbool_t MENU_TunerLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_TUNER_INST_S* inst = (MENU_TUNER_INST_S*)h;
    FGS_KEY_E          currKey;

    if ( inst->dataChg )
    {
        if ( force )
        {
            /* restore data on forced exit */
            switch ( inst->feType )
            {
                #ifdef APPL_FESAT_ENABLED

                case FE_SYSTEM_TYPE_SATELLITE:
                    (void)CFGSAT_Load();
                    break;

                #endif /* APPL_FESAT_ENABLED */

                #ifdef APPL_FETERR_ENABLED

                case FE_SYSTEM_TYPE_TERRESTRIAL:
                    /* MBisch: discuss later on forced exit philosophy */
                    //SETTINGS_Load();
                    break;

                #endif /* APPL_FETERR_ENABLED */

                #ifdef APPL_FECABLE_ENABLED

                case FE_SYSTEM_TYPE_CABLE:
                    break;

                #endif /* APPL_FECABLE_ENABLED */

                default:
                    break;
            }
        }
        else
        {
            MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

            switch ( inst->feType )
            {
                #ifdef APPL_FESAT_ENABLED

                case FE_SYSTEM_TYPE_SATELLITE:
                    (void)CFGSAT_Save (FFALSE);
                    break;

                #endif /* APPL_FESAT_ENABLED */

                #ifdef APPL_FETERR_ENABLED

                case FE_SYSTEM_TYPE_TERRESTRIAL:
                    (void)SETTINGS_Save();
                    break;

                #endif /* APPL_FETERR_ENABLED */

                #ifdef APPL_FECABLE_ENABLED

                case FE_SYSTEM_TYPE_CABLE:
                    break;

                #endif /* APPL_FECABLE_ENABLED */

                default:
                    break;
            }

            (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        }
    }

    currKey = FGS_GetCurrentKeyCode ();

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    #ifdef APPL_FESAT_ENABLED

    if ( inst->sigBarOpen ) (void)MAIN_SignalBoxClose((&appDat->sigBox));

    (void)TIME_PeriodicTimerAbort(OSD_PERIODTIMER_GENERAL);
    appDat->osdUpdateCb = NULL;

    #ifdef APPL_DISEQC12_ENABLED

    (void)DIS12_HaltMotor (menuTunerDat->feIdx);

    #endif

    (void)AV_TuneModeStop     (inst->feIdx);

    #ifdef APPL_DISEQC12_ENABLED
    (void)AV_SetAutoMotorMode (inst->feIdx, FTRUE);
    #endif

    #endif /* APPL_FESAT_ENABLED */

    /* Restart previous service */
    if ( currKey == FGS_KEY_EXIT )
    {
        (void)MAIN_RestartPresentation();
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_TUNER);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC fbool_t MENU_TunerOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_TUNER_INST_S*      inst = (MENU_TUNER_INST_S*)h;
    MENUSTACK_ENTRY         newEntry = NULL;
    FAPI_SYS_HandleT        arg      = NULL;
    uint8_t                 subLines = 0;
    MENU_SELECTLIST_OPEN_S  selectLst;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    fbool_t                 keyUsed = FTRUE;
    FGS_LOC_S               memberLoc;
    fbool_t                 openSelectLst = FFALSE;
    uint8_t                 feIdx;
    fbool_t                 nextMenuFound;

    #ifdef APPL_FESAT_ENABLED

    MENU_TPLIST_PARAMS_S    tplistPars;

    #endif /* APPL_FESAT_ENABLED */

    switch ( itemId )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_TUNER_ID_CONTROLMETHOD :
            openSelectLst = FTRUE;
            subLines      = 3;
            break;

        case MENU_TUNER_ID_SELSAT :
            openSelectLst = FTRUE;
            subLines      = 5;
            break;

        case MENU_TUNER_ID_CLASSICID :
            openSelectLst = FTRUE;
            subLines      = 2;
            break;

        case MENU_TUNER_ID_TONEBURSTID :
            openSelectLst = FTRUE;
            subLines      = 3;
            break;

        case MENU_TUNER_ID_DISEQC10ID :
            openSelectLst    = FTRUE;
            subLines         = 5;
            break;

        case MENU_TUNER_ID_DISEQC11ID :
            openSelectLst    = FTRUE;
            subLines         = 5;
            break;
        #ifdef APPL_DISEQC11_SMATV_ENABLED
        case MENU_TUNER_ID_DISEQC11ID_SMATV :
            openSelectLst    = FTRUE;
            subLines         = 3;
            break;
        case MENU_TUNER_ID_IF_FREQ :
            openSelectLst    = FTRUE;
            subLines         = 3;
            break;
        #endif
        case MENU_TUNER_ID_DISEQC11REP :
            openSelectLst    = FTRUE;
            subLines         = 3;
            break;

        #ifdef APPL_DISEQC12_ENABLED

        case MENU_TUNER_ID_DISEQC12POS :
            openSelectLst    = FTRUE;
            subLines         = 5;
            break;

        case MENU_TUNER_ID_MOVESTYLE :
            openSelectLst      = FTRUE;
            subLines           = 3;
            break;

        case MENU_TUNER_ID_MOVELR :
            if ( FGS_LIST_GetMemberParams(inst->listHdl, MENU_TUNER_ID_MOVELR ,
                                          &selParams) == FAPI_OK )
            {
                (void)AV_SetAutoMotorMode (inst->feIdx, FFALSE);

                switch ( inst->currMoveStyle )
                {
                    case MENU_TUNER_MOVE_STEP:
                        switch (selParams.index)
                        {
                            default:
                            case MOVE_MOTOR_STOP:
                                (void)DIS12_HaltMotor (inst->feIdx);
                                break;

                            case MOVE_MOTOR_WEST:
                                (void)DIS12_DriveMotor(inst->feIdx, FE_WE_FLAG_WEST, 0xFF);
                                /* start timer */
                                appDat->osdUpdateCb = MENU_TunerMotorHalt;

                                (void)TIME_PeriodicTimerLaunch(OSD_PERIODTIMER_GENERAL, 1); // min. period: 100 ms
                                break;
                            case MOVE_MOTOR_EAST:
                                (void)DIS12_DriveMotor(inst->feIdx, FE_WE_FLAG_EAST, 0xFF);
                                /* start timer */
                                appDat->osdUpdateCb = MENU_TunerMotorHalt;

                                (void)TIME_PeriodicTimerLaunch(OSD_PERIODTIMER_GENERAL, 1); // min. period: 100 ms
                                break;
                        }
                        break;

                    case MENU_TUNER_MOVE_CONT:// continuous
                        switch ( selParams.index )
                        {
                            default:
                            case MOVE_MOTOR_STOP:
                                (void)DIS12_HaltMotor (inst->feIdx);
                                break;

                            case MOVE_MOTOR_WEST:
                                (void)DIS12_DriveMotor(inst->feIdx, FE_WE_FLAG_WEST, 0);
                                break;

                            case MOVE_MOTOR_EAST:
                                (void)DIS12_DriveMotor(inst->feIdx, FE_WE_FLAG_EAST, 0);
                                 break;
                        }
                        break;

                    case MENU_TUNER_MOVE_1SEC:
                        switch ( selParams.index )
                        {
                            default:
                            case MOVE_MOTOR_STOP:
                                (void)DIS12_HaltMotor (inst->feIdx);
                                break;

                            case MOVE_MOTOR_WEST:
                                (void)DIS12_DriveMotor(inst->feIdx, FE_WE_FLAG_WEST, 1);
                                /* start timer */
                                appDat->osdUpdateCb = MENU_TunerMotorHalt;

                                (void)TIME_PeriodicTimerLaunch(OSD_PERIODTIMER_GENERAL, 10); // min. period: 100 ms
                                break;

                            case MOVE_MOTOR_EAST:
                                (void)DIS12_DriveMotor(inst->feIdx, FE_WE_FLAG_EAST, 1);
                                /* start timer */
                                appDat->osdUpdateCb = MENU_TunerMotorHalt;

                                (void)TIME_PeriodicTimerLaunch(OSD_PERIODTIMER_GENERAL, 10); // min. period: 100 ms
                                break;
                        }
                        break;

                    default:
                        break;
                }
            }

            break;

        case MENU_TUNER_ID_STORESAT :
            if(inst->currSatDat.lnbCfg[inst->feIdx].diseqc12Pos > 0)
            {
                (void)DIS12_StoreSatPosition (inst->feIdx,
                                              inst->currSatDat.lnbCfg[inst->feIdx].
                                              diseqc12Pos);
            }
            break;

        case MENU_TUNER_ID_EXPMOTOR :
            arg      = (FAPI_SYS_HandleT)(&(inst->feIdx));

            newEntry = MENU_ExpMotorEntry;
            break;

        #endif /* APPL_DISEQC12_ENABLED */

        case MENU_TUNER_ID_SELTP:

            tplistPars.system        = FE_SYSTEM_TYPE_SATELLITE;
            tplistPars.satLstPos     = inst->satLstPos;
            tplistPars.carrierLstPos = inst->carrierLstPos;
            tplistPars.lockSat       = FTRUE;
            tplistPars.leaveCb       = MENU_TunerUpdateCarrierSel;

            arg      = (FAPI_SYS_HandleT)(&tplistPars);
            newEntry = MENU_TpListEntry;
            break;

        #ifdef APPL_USALS_ENABLED

        case MENU_TUNER_ID_TURNMOTOR:
            (void)AV_SetAutoMotorMode(inst->feIdx, FFALSE);
            (void)USALS_GotoRefPosition(inst->feIdx);
            break;

        #endif /* APPL_USALS_ENABLED */

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case MENU_TUNER_ID_ANTENNA_POWER :
            openSelectLst      = FTRUE;
            subLines           = 2;
            break;

        #endif /* APPL_FETERR_ENABLED */

        case MENU_TUNER_ID_GOTONEXT:
            if ( inst->guidedMode ) /* next menu in guided mode */
            {
                keyUsed = MENU_GuidedGotoNext ();
            }
            else if ( inst->feIdx < inst->feLast ) /* next tuner menu */
            {
                keyUsed       = FFALSE;
                nextMenuFound = FFALSE;

                for ( feIdx = inst->feIdx+1; feIdx < appDat->hwCfg->feNums; feIdx++ )
                {
                    switch ( inst->feType )
                    {
                        #ifdef APPL_FESAT_ENABLED

                        case FE_SYSTEM_TYPE_SATELLITE:
                            nextMenuFound = FTRUE;
                            break;

                        #endif /* APPL_FESAT_ENABLED */

                        #ifdef APPL_FETERR_ENABLED

                        case FE_SYSTEM_TYPE_TERRESTRIAL:
                            nextMenuFound = FTRUE;
                            break;

                        #endif /* APPL_FETERR_ENABLED */

                        default:
                            break;
                    }

                    if ( nextMenuFound )
                    {
                        inst->feIdx = feIdx;
                        keyUsed = MENUSTACK_Switch(MENU_TunerEntry, NULL, -1);
                        break;
                    }
                }
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        default:
            MENU_CommonMsgShow(STR_MSG_NOTIMPLEMENTEDYET, FFALSE);
            keyUsed = FFALSE;
            break;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.x         = memberLoc.x + SCALER_GetCurrVal(MAIN_WIDTH) -
                                  SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.y         = memberLoc.y;
        selectLst.pos.width     = SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_TunerGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;

        keyUsed = MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }
    else if ( newEntry != NULL )
    {
        /* store last focus */
        MENUSTACK_SetData(itemId);

        keyUsed = MENUSTACK_Up(newEntry, arg);
    }

    return keyUsed;
}

FSTATIC void MENU_TunerValChg (FAPI_SYS_HandleT h, int32_t itemId, FGS_VAL_S newVal)
{
    MENU_TUNER_INST_S*      inst = (MENU_TUNER_INST_S*)h;
    fbool_t                 valChanged = FFALSE;

    #ifdef APPL_FESAT_ENABLED

    MENU_EVT_REFRESH_S      tunerEvt;
    FGS_ITEM_SEL_PARAMS_S   selParams;
    CFGSAT_LNBCFG_S*        pLnb;
    fbool_t                 retune = FFALSE;
    int32_t                 retVal;

    #ifdef APPL_USALS_ENABLED

    SYS_GEO_POS_S           geoPos;

    #endif /* APPL_USALS_ENABLED */

    pLnb = inst->currSatDat.lnbCfg + inst->feIdx;

    #endif /* APPL_FESAT_ENABLED */

    switch ( itemId )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_TUNER_ID_CONTROLMETHOD :

            if ( inst->currFeDat.controlMode[inst->feIdx] != newVal.number )
            {
                inst->currFeDat.controlMode[inst->feIdx] = (CFGSAT_LNBMODE_E)newVal.number;

                (void)CFGSAT_SetFrontendSetup(&(inst->currFeDat));
                inst->dataChg = FTRUE;
            }
            #ifdef APPL_DISEQC12_ENABLED
            (void)AV_SetAutoMotorMode (inst->feIdx, FTRUE);
            #endif
            tunerEvt.hdr.type   = APP_EVT_MENU_REFRESH;
            tunerEvt.hdr.length = sizeof(tunerEvt);
            tunerEvt.cb         = MENU_TunerRefresh;
            retVal = EVT_Send (&tunerEvt,"Refresh Tuner Menu");
            break;

        case MENU_TUNER_ID_SELSAT:
            inst->satLstPos     = newVal.number;
            inst->carrierLstPos = 0;

            (void)CFGSAT_GetIndex (inst->satLstPos, &(appDat->currSatIdx));

            retVal = CFGSAT_GetSatData (inst->satLstPos, &(inst->currSatDat));

            if ( retVal == FAPI_OK ) inst->satDatValid = FTRUE;
            else                     inst->satDatValid = FFALSE;

            #ifdef APPL_DISEQC12_ENABLED
            (void)AV_SetAutoMotorMode (inst->feIdx, FTRUE);
            #endif
            (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TUNER_ID_HIGHFREQ);
            (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TUNER_ID_LOWFREQ );

            switch ( inst->currFeDat.controlMode[inst->feIdx] )
            {
                case CFGSAT_FEMODE_CLASSIC:
                    if ( FGS_LIST_GetMemberParams(inst->listHdl,
                                                  MENU_TUNER_ID_CLASSICID,
                                                  &selParams) == FAPI_OK )
                    {
                        selParams.index = pLnb->active;
                        (void)FGS_LIST_SetMemberParams(inst->listHdl,
                                                 MENU_TUNER_ID_CLASSICID,
                                                 &selParams);
                    }
                    break;

                case CFGSAT_FEMODE_TONEBURST:
                    if ( FGS_LIST_GetMemberParams(inst->listHdl,
                                                  MENU_TUNER_ID_TONEBURSTID,
                                                  &selParams) == FAPI_OK )
                    {
                        if ( !(pLnb->active) ) selParams.index = 0;
                        else                   selParams.index = pLnb->toneburstId + 1;

                        (void)FGS_LIST_SetMemberParams(inst->listHdl,
                                                 MENU_TUNER_ID_TONEBURSTID,
                                                 &selParams);
                    }
                    break;

                case CFGSAT_FEMODE_DISEQC10:
                    if ( FGS_LIST_GetMemberParams(inst->listHdl,
                                                  MENU_TUNER_ID_DISEQC10ID,
                                                  &selParams) == FAPI_OK )
                    {
                        if ( !(pLnb->active) ) selParams.index = 0;
                        else                   selParams.index = pLnb->diseqc10Id + 1;

                        (void)FGS_LIST_SetMemberParams(inst->listHdl,
                                                 MENU_TUNER_ID_DISEQC10ID,
                                                 &selParams);
                    }
                    break;
                case CFGSAT_FEMODE_DISEQC11_DTH:
                    if ( FGS_LIST_GetMemberParams(inst->listHdl,
                                                  MENU_TUNER_ID_DISEQC11ID,
                                                  &selParams) == FAPI_OK )
                    {
                        if ( !(pLnb->active) ) selParams.index = 0;
                        else                   selParams.index = pLnb->diseqc11Id + 1;

                        (void)FGS_LIST_SetMemberParams(inst->listHdl,
                                                 MENU_TUNER_ID_DISEQC11ID,
                                                 &selParams);
                    }
                    (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TUNER_ID_DISEQC11REP );
                    break;
                #ifdef APPL_DISEQC11_SMATV_ENABLED
                case CFGSAT_FEMODE_DISEQC11_SMATV:
                    if ( FGS_LIST_GetMemberParams(inst->listHdl,
                                                  MENU_TUNER_ID_DISEQC11ID_SMATV,
                                                  &selParams) == FAPI_OK )
                    {
                        if (!pLnb->active)
                        {
                            selParams.index = 0;
                        }
                        else
                        {
                            selParams.index = pLnb->diseqc11Id_SMATV + 1;
                        }

                        (void)FGS_LIST_SetMemberParams(inst->listHdl,
                                                 MENU_TUNER_ID_DISEQC11ID_SMATV,
                                                 &selParams);
                    }
                    (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TUNER_ID_DISEQC11REP );
                    (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TUNER_ID_IF_FREQ );
                    break;
                #endif
                #ifdef APPL_DISEQC12_ENABLED

                case CFGSAT_FEMODE_DISEQC12:
                    if ( FGS_LIST_GetMemberParams(inst->listHdl,
                                                  MENU_TUNER_ID_DISEQC12POS,
                                                  &selParams) == FAPI_OK )
                    {
                        if ( !(pLnb->active) ) selParams.index = 0;
                        else                   selParams.index = pLnb->diseqc12Pos;

                        (void)FGS_LIST_SetMemberParams(inst->listHdl,
                                                 MENU_TUNER_ID_DISEQC12POS,
                                                 &selParams);
                    }
                    break;

                #endif /* APPL_DISEQC12_ENABLED */

                #ifdef APPL_USALS_ENABLED

                    case CFGSAT_FEMODE_USALS:
                        retune  = FTRUE;
                        break;

                #endif /* APPL_USALS_ENABLED */
                default:
                    break;
            }

            retune = FTRUE;
            break;

        case MENU_TUNER_ID_HIGHFREQ :

            if ( pLnb->freqHi != newVal.number * 1000 )
            {
                pLnb->freqHi = newVal.number * 1000;

                (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                                  SYS_OWNER_USER);

                inst->dataChg = FTRUE;
                retune        = FTRUE;
            }
            break;

        case MENU_TUNER_ID_LOWFREQ :

            if ( pLnb->freqLo != newVal.number * 1000 )
            {
                pLnb->freqLo = newVal.number * 1000;

                (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                                  SYS_OWNER_USER);

                inst->dataChg = FTRUE;
                retune        = FTRUE;
            }
            break;

        case MENU_TUNER_ID_CLASSICID:
            if ( pLnb->active != newVal.number )
            {
                pLnb->active = (fbool_t)newVal.number;

                (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                                   SYS_OWNER_USER);

                inst->dataChg = FTRUE;
                retune        = FTRUE;
            }
            break;

        case MENU_TUNER_ID_TONEBURSTID:
            if ( newVal.number > 0 )
            {
                if ( !(pLnb->active) ||
                      (pLnb->toneburstId != newVal.number - 1) )
                {
                    pLnb->active      = FTRUE;
                    pLnb->toneburstId = (uint8_t)newVal.number - 1;
                    valChanged        = FTRUE;
                }
            }
            else if ( pLnb->active )
            {
                pLnb->active = FFALSE;
                valChanged   = FTRUE;
            }

            if ( valChanged )
            {
                (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                                  SYS_OWNER_USER);

                inst->dataChg = FTRUE;
                retune        = FTRUE;
            }
            break;

        case MENU_TUNER_ID_DISEQC10ID:
            if ( newVal.number > 0 )
            {
                if ( !(pLnb->active) ||
                      (pLnb->diseqc10Id != newVal.number - 1) )
                {
                    pLnb->active     = FTRUE;
                    pLnb->diseqc10Id = (uint8_t)newVal.number - 1;
                    valChanged       = FTRUE;
                }
            }
            else if ( pLnb->active )
            {
                pLnb->active = FFALSE;
                valChanged   = FTRUE;
            }

            if ( valChanged )
            {
                (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                                  SYS_OWNER_USER);

                inst->dataChg = FTRUE;
                retune        = FTRUE;
            }
            break;

        case MENU_TUNER_ID_DISEQC11ID:
            if ( newVal.number > 0 )
            {
                if ( !(pLnb->active) ||
                      (pLnb->diseqc11Id != newVal.number - 1) )
                {
                    pLnb->active     = FTRUE;
                    pLnb->diseqc11Id = (uint8_t)newVal.number - 1;
                    valChanged       = FTRUE;
                }
            }
            else if ( pLnb->active )
            {
                pLnb->active = FFALSE;
                valChanged   = FTRUE;
            }

            if ( valChanged )
            {
                (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                                  SYS_OWNER_USER);

                inst->dataChg = FTRUE;
                retune        = FTRUE;
            }
            break;

        case MENU_TUNER_ID_DISEQC11ID_SMATV:
            if ( newVal.number > 0 )
            {
                if ( !(pLnb->active) ||
                      (pLnb->diseqc11Id_SMATV != newVal.number - 1) )
                {
                    pLnb->active     = FTRUE;
                    pLnb->diseqc11Id_SMATV = (uint8_t)newVal.number - 1;
                    valChanged       = FTRUE;
                }
            }
            else if ( pLnb->active )
            {
                pLnb->active = FFALSE;
                valChanged   = FTRUE;
            }

            if ( valChanged )
            {
                (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                                  SYS_OWNER_USER);

                inst->dataChg = FTRUE;
                retune        = FTRUE;
            }
            break;

        case MENU_TUNER_ID_DISEQC11REP:
             pLnb->diseqc11Rep = (uint8_t)newVal.number;
            (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                              SYS_OWNER_USER);

            inst->dataChg = FTRUE;
            retune        = FTRUE;
            break;

        #ifdef APPL_DISEQC11_SMATV_ENABLED

        case MENU_TUNER_ID_IF_FREQ:
           inst->currFeDat.ifFreq = (uint32_t)1284+(newVal.number*116); /* store MHz value in kHz */

           (void)CFGSAT_SetFrontendSetup(&(inst->currFeDat));
           inst->dataChg = FTRUE;
           retune        = FTRUE;
            break;
        #endif
        case MENU_TUNER_ID_DISEQC12POS:
            if ( newVal.number > 0 )
            {
                if ( !(pLnb->active) ||
                      (pLnb->diseqc12Pos != newVal.number) )
                {
                    pLnb->active      = FTRUE;
                    pLnb->diseqc12Pos = (uint8_t)newVal.number;
                    valChanged        = FTRUE;
                }
            }
            else if ( pLnb->active )
            {
                pLnb->active = FFALSE;
                valChanged   = FTRUE;
            }

            if ( valChanged )
            {
                (void)CFGSAT_ReplaceSat (inst->satLstPos, &(inst->currSatDat),
                                  SYS_OWNER_USER);

                inst->dataChg = FTRUE;
                retune        = FTRUE;
            }
            break;

        case MENU_TUNER_ID_MOVESTYLE :
            inst->currMoveStyle = (MENU_TUNER_MOVESTYLE)newVal.number;
            break;

        #ifdef APPL_USALS_ENABLED

        case MENU_TUNER_ID_LONGITUDE:
            (void)AV_SetAutoMotorMode (inst->feIdx, FTRUE);

            if ( USALS_GetGeoPosition(&geoPos) == FAPI_OK )
            {
                if ( geoPos.longitude != newVal.number )
                {
                    geoPos.longitude = newVal.number;

                    (void)USALS_SetGeoPosition(&geoPos);

                    inst->dataChg = FTRUE;
                    retune        = FTRUE;
                }
            }
            break;

        case MENU_TUNER_ID_LATITUDE:
            (void)AV_SetAutoMotorMode (inst->feIdx, FTRUE);

            if ( USALS_GetGeoPosition(&geoPos) == FAPI_OK )
            {
                if ( geoPos.latitude != newVal.number )
                {
                    geoPos.latitude = newVal.number;

                    (void)USALS_SetGeoPosition(&geoPos);

                    inst->dataChg = FTRUE;
                    retune        = FTRUE;
                }
            }
            break;

        #endif /* APPL_USALS_ENABLED */

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case MENU_TUNER_ID_ANTENNA_POWER:
            if ( (newVal.number == 0 && inst->antennaPowerTerr == FTRUE) ||
                 (newVal.number == 1 && inst->antennaPowerTerr == FFALSE) )
                inst->dataChg = FTRUE;

            inst->antennaPowerTerr = (newVal.number == 0) ? FFALSE : FTRUE;

            (void)SETTINGS_AntennaPowerSet((int8_t)inst->feIdx, inst->antennaPowerTerr);
            break;

        #endif /* APPL_FETERR_ENABLED */

        default:
            break;
    }

    #ifdef APPL_FESAT_ENABLED

    if( retune )
    {
        MENU_TunerUpdateSignalDisplay(inst);
    }

    #endif /* APPL_FESAT_ENABLED */
}

FSTATIC void MENU_TunerGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                     FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_TUNER_INST_S* inst   = (MENU_TUNER_INST_S*)h;
    uint32_t           strIdx = STR_EMPTYSTRING;

    switch ( id )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_TUNER_ID_CONTROLMETHOD       : strIdx = STR_ENTRY_TUNER_CONTROLMETHOD;     break;
        case MENU_TUNER_ID_SELSAT              : strIdx = STR_ENTRY_TUNER_SELSAT       ;     break;
        case MENU_TUNER_ID_HIGHFREQ            : strIdx = STR_ENTRY_TUNER_HIGHFREQ     ;     break;
        case MENU_TUNER_ID_LOWFREQ             : strIdx = STR_ENTRY_TUNER_LOWFREQ      ;     break;
        case MENU_TUNER_ID_TONEBURSTID         : strIdx = STR_ENTRY_TUNER_TONEBURSTID  ;     break;
        case MENU_TUNER_ID_CLASSICID           : strIdx = STR_ENTRY_TUNER_CLASSICID    ;     break;
        case MENU_TUNER_ID_DISEQC10ID          : strIdx = STR_ENTRY_TUNER_DISEQC10ID   ;     break;
        case MENU_TUNER_ID_DISEQC11ID          : strIdx = STR_ENTRY_TUNER_DISEQC11ID   ;     break;
        case MENU_TUNER_ID_DISEQC11ID_SMATV    : strIdx = STR_ENTRY_TUNER_DISEQC11ID_SMATV;  break;
        case MENU_TUNER_ID_DISEQC11REP         : strIdx = STR_ENTRY_TUNER_DISEQC11REP  ;     break;
        case MENU_TUNER_ID_DISEQC12POS         : strIdx = STR_ENTRY_TUNER_DISEQC12POS  ;     break;
        case MENU_TUNER_ID_MOVESTYLE           : strIdx = STR_ENTRY_TUNER_MOVESTYLE    ;     break;
        case MENU_TUNER_ID_MOVELR              : strIdx = STR_ENTRY_TUNER_MOVELR       ;     break;
        case MENU_TUNER_ID_STORESAT            : strIdx = STR_ENTRY_TUNER_STORESAT     ;     break;
        #ifdef APPL_DISEQC11_SMATV_ENABLED
        case MENU_TUNER_ID_IF_FREQ             : strIdx = STR_ENTRY_TUNER_IF_FREQUNCY  ;     break;
        #endif
        #ifdef APPL_DISEQC12_ENABLED
        case MENU_TUNER_ID_EXPMOTOR            : strIdx = STR_ENTRY_TUNER_EXPMOTOR     ;     break;
        #endif
        case MENU_TUNER_ID_SELTP               : strIdx = STR_ENTRY_TUNER_SELTP        ;     break;
        case MENU_TUNER_ID_LOCK                : strIdx = STR_ENTRY_TUNER_LOCK         ;     break;
        case MENU_TUNER_ID_LONGITUDE           : strIdx = STR_ENTRY_TUNER_LONGITUDE    ;     break;
        case MENU_TUNER_ID_LATITUDE            : strIdx = STR_ENTRY_TUNER_LATITUDE     ;     break;
        case MENU_TUNER_ID_TURNMOTOR           : strIdx = STR_ENTRY_TUNER_TURNMOTOR    ;     break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case MENU_TUNER_ID_ANTENNA_POWER       : strIdx = STR_ENTRY_TUNER_ANTENNA_POWER;     break;

        #endif

        case MENU_TUNER_ID_GOTONEXT            :
            if ( inst->guidedMode ) strIdx = MENU_GuidedGetNextStr();
            else                    strIdx = STR_ENTRY_TUNER_GOTO_NEXT;
            break;

        default:  return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_TunerGetItemTitle (FAPI_SYS_HandleT h,
                                     int32_t id,
                                     uint32_t index,
                                     FGS_ITEMSTATE_E state,
                                     FGS_ITEM_S* item)
{
    MENU_TUNER_INST_S*  inst = (MENU_TUNER_INST_S*)menuTunerDat;
    uint32_t            bufPos = 0;

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = TITLE_FONT;
    item->item.str.lineHeight = 0;
    item->item.str.col        = OSD_COL_TITLE_TEXT;

    switch ( inst->feType )
    {

        #ifdef APPL_FESAT_ENABLED
        case FE_SYSTEM_TYPE_SATELLITE:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr((uint32_t)STR_TITLE_TUNER_SAT),
                                     MAIN_LINEBUFCHARSET);
            break;
        #endif
        #ifdef APPL_FETERR_ENABLED
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr((uint32_t)STR_TITLE_TUNER_TERR),
                                     MAIN_LINEBUFCHARSET);
            break;
        #endif
        #ifdef APPL_FECAB_ENABLED
        case FE_SYSTEM_TYPE_CABLE:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr((uint32_t)STR_TITLE_TUNER_CAB),
                                     MAIN_LINEBUFCHARSET);
            break;
        #endif

        default:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr((uint32_t)STR_EMPTYSTRING),
                                     MAIN_LINEBUFCHARSET);
    }

    sprintf (appDat->asciiBuf, " %d ", (inst->feIdx+1));

    bufPos += STR_BufAppend (NULL, bufPos,
                             STR_IntASCIIStr(),
                             MAIN_LINEBUFCHARSET);

    bufPos += STR_BufAppend (NULL, bufPos,
                             OSD_GetStr((uint32_t)STR_TITLE_TUNER),
                             MAIN_LINEBUFCHARSET);

    item->item.str.chr = STR_IntLineStr();
}

FSTATIC void MENU_TunerGetHelpDescrItem (FAPI_SYS_HandleT h,
                                         int32_t id,
                                         uint32_t index,
                                         FGS_ITEMSTATE_E state,
                                         FGS_ITEM_S* item)
{
    uint32_t           strIdx = STR_EMPTYSTRING;

    item->item.str.font       = MAIN_FONT;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->type                = FGS_ITEM_TYPE_STRING;
    item->item.str.lineHeight = 0;

    switch ( id )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_TUNER_HELP_ID_MOVESTYLE      :
        case MENU_TUNER_HELP_ID_CONTROLMETHOD  :
        case MENU_TUNER_HELP_ID_SELSAT         :
        case MENU_TUNER_HELP_ID_CLASSICID      :
        case MENU_TUNER_HELP_ID_TONEBURSTID    :
        case MENU_TUNER_HELP_ID_DISEQC10ID     :
        case MENU_TUNER_HELP_ID_DISEQC11ID     :
        case MENU_TUNER_HELP_ID_DISEQC11REP    :
        #ifdef APPL_DISEQC11_SMATV_ENABLED
        case MENU_TUNER_HELP_ID_DISEQC11ID_SMATV:
        case MENU_TUNER_HELP_ID_IF_FREQ        :
        #endif
        case MENU_TUNER_HELP_ID_DISEQC12POS    : strIdx = STR_ENTRY_LOKR;
                                                 item->align = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
                                                 item->item.str.font = SYMBOL_FONT;        break;
        case MENU_TUNER_HELP_ID_HIGHFREQ       :
        case MENU_TUNER_HELP_ID_LOWFREQ        :strIdx = STR_ENTRY_09LR;
                                                 item->align = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
                                                 item->item.str.font = SYMBOL_FONT;        break;
        case MENU_TUNER_HELP_ID_APPLY          :
        case MENU_TUNER_HELP_ID_GOTOPOS        :
        case MENU_TUNER_HELP_ID_EXPMOTOR       :
        case MENU_TUNER_HELP_ID_SELTP          :
        case MENU_TUNER_HELP_ID_STORESAT       :
        case MENU_TUNER_HELP_ID_TURNMOTOR      : strIdx = STR_ENTRY_OK;
                                                 item->align = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
                                                 item->item.str.font = SYMBOL_FONT;        break;
        case MENU_TUNER_HELP_ID_MOVELR         : strIdx = STR_ENTRY_LOKR;
                                                 item->align = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
                                                 item->item.str.font = SYMBOL_FONT;        break;
        case MENU_TUNER_HELP_ID_LONGITUDE      :
        case MENU_TUNER_HELP_ID_LATITUDE       : strIdx = STR_ENTRY_09LR;
                                                 item->align = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
                                                 item->item.str.font = SYMBOL_FONT;        break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case MENU_TUNER_HELP_ID_ANTENNA_POWER  : strIdx = STR_ENTRY_LOKR;
                                                 item->align = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
                                                 item->item.str.font = SYMBOL_FONT;
                                                 break;

        #endif /* APPL_FETERR_ENABLED */

        case MENU_TUNER_HELP_ID_GOTONEXT       : strIdx = STR_ENTRY_OK;
                                                 item->align = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
                                                 item->item.str.font = SYMBOL_FONT;
                                                 break;

        default:  return;
    }

    item->item.str.chr = OSD_GetStr(strIdx);

    switch ( state )
    {
        default:
        case FGS_DISABLED:
            item->item.str.col = OSD_GetSchemeColor(OSD_SCHEMECOL_HELP_BKGR);
            break;
        case FGS_NONFOCUS:
        case FGS_FOCUS:
            item->item.str.col = appDat->txtAttr.colFocus;
            break;
    }
}

FSTATIC void MENU_TunerGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                               FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_TUNER_INST_S*  inst = (MENU_TUNER_INST_S*)h;
    uint32_t            col;

    #ifdef APPL_FESAT_ENABLED

    uint32_t            strIdx;
    SYS_FE_STATUS_S     feStatus;
    CFGSAT_LNBCFG_S*    pLnb;

    #ifdef APPL_USALS_ENABLED

    SYS_GEO_POS_S       geoPos;

    #endif /* APPL_USALS_ENABLED */

    pLnb = inst->currSatDat.lnbCfg + inst->feIdx;

    #endif /* APPL_FESAT_ENABLED */

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    switch ( id )
    {
        #ifdef APPL_FESAT_ENABLED

        case MENU_TUNER_ID_CONTROLMETHOD :
            switch( index )
            {
                default:
                case CFGSAT_FEMODE_CLASSIC:
                    strIdx = STR_VALUE_TUNER_CONFMODE_CLASSIC;
                    break;
                case CFGSAT_FEMODE_TONEBURST:
                    strIdx = STR_VALUE_TUNER_CONFMODE_TONEBURST;
                    break;
                case CFGSAT_FEMODE_DISEQC10:
                    strIdx = STR_VALUE_TUNER_CONFMODE_DISEQC10;
                    break;

                case CFGSAT_FEMODE_DISEQC11_DTH:
                    strIdx = STR_VALUE_TUNER_CONFMODE_DISEQC11_DTH;
                    break;

                #ifdef APPL_DISEQC11_SMATV_ENABLED
                case CFGSAT_FEMODE_DISEQC11_SMATV:
                    strIdx = STR_VALUE_TUNER_CONFMODE_DISEQC11_SMATV;
                    break;
                #endif /* APPL_DISEQC11_SMATV_ENABLED */

                #ifdef APPL_DISEQC12_ENABLED
                case CFGSAT_FEMODE_DISEQC12:
                    strIdx = STR_VALUE_TUNER_CONFMODE_DISEQC12;
                    break;
                #endif /* APPL_DISEQC12_ENABLED */

                #ifdef APPL_USALS_ENABLED
                case CFGSAT_FEMODE_USALS:
                    strIdx = STR_VALUE_TUNER_CONFMODE_USALS;
                    break;
                #endif /* APPL_USALS_ENABLED */
            }

            item->item.str.chr = OSD_GetStr(strIdx);
            item->type         = FGS_ITEM_TYPE_STRING;
            break;

        case MENU_TUNER_ID_SELSAT :

            if ( CFGSAT_GetSatData(index, &(inst->tmpSatDat)) == FAPI_OK )
            {
                item->type            = FGS_ITEM_TYPE_STRING;
                item->item.str.chr    = STR_Build(inst->tmpSatDat.nameCharSet,
                                                  inst->tmpSatDat.name);
            }
            break;

        case MENU_TUNER_ID_HIGHFREQ:
            item->type                      = FGS_ITEM_TYPE_NUMBER;
            item->item.number.value         = pLnb->freqHi / 1000;
            item->item.number.format        = FGS_FORMAT_NUM_NORM;
            item->item.number.decimalPlaces = 0;
            item->item.number.unitStr       = OSD_GetStr(STR_UNIT_MHZ);
            break;

        case MENU_TUNER_ID_LOWFREQ:
            item->type                      = FGS_ITEM_TYPE_NUMBER;
            item->item.number.value         = pLnb->freqLo / 1000;
            item->item.number.format        = FGS_FORMAT_NUM_NORM;
            item->item.number.decimalPlaces = 0;
            item->item.number.unitStr       = OSD_GetStr(STR_UNIT_MHZ);
            break;

        case MENU_TUNER_ID_CLASSICID:
            switch ( index )
            {
                case FTRUE : strIdx = STR_VALUE_TUNER_CLASSICID_ON;  break;
                case FFALSE: strIdx = STR_VALUE_TUNER_CLASSICID_OFF; break;
                default    : strIdx = STR_EMPTYSTRING;               break;
            }

            item->item.str.chr        = OSD_GetStr(strIdx);
            item->type                = FGS_ITEM_TYPE_STRING;
            break;

        case MENU_TUNER_ID_TONEBURSTID:
            if ( index > 0 )
            {
                sprintf(appDat->asciiBuf,"%ld", index - 1);
                item->item.str.chr        = STR_IntASCIIStr();
            }
            else
            {
                item->item.str.chr = OSD_GetStr(STR_GNR_OFF);
            }

            item->type = FGS_ITEM_TYPE_STRING;
            break;

        case MENU_TUNER_ID_DISEQC10ID:
            if ( index > 0 )
            {
                sprintf(appDat->asciiBuf,"%ld", index - 1);
                item->item.str.chr = STR_IntASCIIStr();
            }
            else
            {
                item->item.str.chr = OSD_GetStr(STR_GNR_OFF);
            }

            item->type = FGS_ITEM_TYPE_STRING;
            break;

        case MENU_TUNER_ID_DISEQC11ID:
            if ( index > 0 )
            {
                sprintf(appDat->asciiBuf,"%ld", index - 1);
                item->item.str.chr = STR_IntASCIIStr();
            }
            else
            {
                item->item.str.chr = OSD_GetStr(STR_GNR_OFF);
            }

            item->type = FGS_ITEM_TYPE_STRING;
            break;

        case MENU_TUNER_ID_DISEQC11ID_SMATV:
            if ( index > 0 )
            {
                sprintf(appDat->asciiBuf,"%ld", index - 1);
                item->item.str.chr = STR_IntASCIIStr();
            }
            else
            {
                item->item.str.chr = OSD_GetStr(STR_GNR_OFF);
            }

            item->type = FGS_ITEM_TYPE_STRING;
            break;

        case MENU_TUNER_ID_DISEQC11REP:
            switch(index)
            {
                case 0: item->item.str.chr = OSD_GetStr(STR_VALUE_TUNER_DISEQC11REP_NONE);
                    break;
                default:
                case 1: item->item.str.chr = OSD_GetStr(STR_VALUE_TUNER_DISEQC11REP_1);
                    break;
                case 2: item->item.str.chr = OSD_GetStr(STR_VALUE_TUNER_DISEQC11REP_2);
                    break;
            }
            item->type = FGS_ITEM_TYPE_STRING;
            break;
        #ifdef APPL_DISEQC11_SMATV_ENABLED

        case MENU_TUNER_ID_IF_FREQ:
            sprintf(appDat->asciiBuf,"%ld - %ldMHz",index,(1284+(index*116)));
            item->item.str.chr = STR_IntASCIIStr();
            item->type = FGS_ITEM_TYPE_STRING;
            break;
        #endif

        #ifdef APPL_DISEQC12_ENABLED
        case MENU_TUNER_ID_DISEQC12POS:
            if ( index > 0 )
            {
                sprintf(appDat->asciiBuf,"%ld", index);
                item->item.str.chr = STR_IntASCIIStr();
            }
            else
            {
                item->item.str.chr = OSD_GetStr(STR_GNR_OFF);
            }
            item->type = FGS_ITEM_TYPE_STRING;
            break;
        #endif
        case MENU_TUNER_ID_MOVESTYLE :
            switch ( index )
            {
                case MENU_TUNER_MOVE_STEP:
                    strIdx = STR_VALUE_TUNER_MOVSTYLE_STEP;
                    break;
                case MENU_TUNER_MOVE_CONT:
                    strIdx = STR_VALUE_TUNER_MOVSTYLE_CONT;
                    break;
                case MENU_TUNER_MOVE_1SEC:
                    strIdx = STR_VALUE_TUNER_MOVSTYLE_1SEC;
                    break;
                default:
                case MENU_TUNER_MOVE_NUMS:
                    strIdx = STR_EMPTYSTRING;
                    break;
            }

            item->item.str.chr        = OSD_GetStr(strIdx);
            item->type                = FGS_ITEM_TYPE_STRING;
            break;

        #if defined(APPL_DISEQC12_ENABLED) || defined(APPL_USALS_ENABLED)
            case MENU_TUNER_ID_MOVELR :
                switch ( index )
                {
                    case MOVE_MOTOR_STOP:
                        strIdx = STR_VALUE_TUNER_MOTLR_STOP;
                        break;
                    case MOVE_MOTOR_WEST:
                        strIdx = STR_VALUE_TUNER_MOTLR_LEFT;
                        break;
                    case MOVE_MOTOR_EAST:
                        strIdx = STR_VALUE_TUNER_MOTLR_RIGHT;
                        break;
                    default:
                        strIdx = STR_EMPTYSTRING;
                        break;
                }

                item->item.str.chr = OSD_GetStr(strIdx);
                item->type         = FGS_ITEM_TYPE_STRING;
                break;
        #endif

        #ifdef APPL_USALS_ENABLED

        case MENU_TUNER_ID_LONGITUDE:
            if ( USALS_GetGeoPosition(&geoPos) == FAPI_OK )
            {
                item->type                      = FGS_ITEM_TYPE_NUMBER;
                item->item.number.value         = geoPos.longitude;
                item->item.number.format        = FGS_FORMAT_NUM_NORM;
                item->item.number.decimalPlaces = 1;

                if ( geoPos.longitude >= 0 )
                {
                    item->item.number.unitStr   = OSD_GetStr(STR_UNIT_EAST);
                }
                else
                {
                    item->item.number.unitStr   = OSD_GetStr(STR_UNIT_WEST);
                }
            }
            break;

        case MENU_TUNER_ID_LATITUDE:
            if ( USALS_GetGeoPosition(&geoPos) == FAPI_OK )
            {
                item->type                      = FGS_ITEM_TYPE_NUMBER;
                item->item.number.value         = geoPos.latitude;
                item->item.number.format        = FGS_FORMAT_NUM_NORM;
                item->item.number.decimalPlaces = 1;

                if ( geoPos.latitude >= 0 )
                {
                    item->item.number.unitStr   = OSD_GetStr(STR_UNIT_NORTH);
                }
                else
                {
                    item->item.number.unitStr   = OSD_GetStr(STR_UNIT_SOUTH);
                }
            }
            break;

        #endif /* APPL_USALS_ENABLED */

        case MENU_TUNER_ID_LOCK :
            memset(&feStatus, 0, sizeof(feStatus));
            (void)AV_GetFeStatus(inst->feIdx, &feStatus);

            switch ( feStatus.lock )
            {
                case SYS_FELOCK_LOCKED:
                    strIdx = STR_VALUE_TUNER_LOCK_LOCKED;
                    break;
                default:
                    strIdx = STR_VALUE_TUNER_LOCK_UNLOCKED;
                    break;
            }
            item->item.str.chr = OSD_GetStr(strIdx);
            item->type         = FGS_ITEM_TYPE_STRING;
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case MENU_TUNER_ID_ANTENNA_POWER:
            switch(index)
            {
                case 0:
                    item->item.str.chr = OSD_GetStr(STR_GNR_OFF);
                    break;
                case 1:
                    item->item.str.chr = OSD_GetStr(STR_GNR_ON);
                    break;

                default: item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                    break;
            }

            item->type                = FGS_ITEM_TYPE_STRING;
            break;

        #endif /* APPL_FETERR_ENABLED */

        default:  return;
    }

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: col = appDat->txtAttr.colNonFocus;     break;
        case FGS_FOCUS:    col = appDat->txtAttr.colFocus;        break;
        case FGS_DISABLED: col = appDat->txtAttr.colDisabled;     break;
    }

    switch ( item->type )
    {
        case FGS_ITEM_TYPE_STRING:
            item->item.str.font       = MAIN_FONT;
            item->item.str.col        = col;
            item->item.str.lineHeight = 0;
            break;

        case FGS_ITEM_TYPE_NUMBER:
            item->item.number.font = MAIN_FONT;
            item->item.number.col  = col;
            break;

        default:
            break;
    }
}
#if(0)
FSTATIC void MENU_TunerGetTitleItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                     FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_TUNER_INST_S*  inst   = (MENU_TUNER_INST_S*)h;
    uint32_t                 bufPos = 0;

    switch ( inst->feType )
    {
        #ifdef APPL_FESAT_ENABLED

        case FE_SYSTEM_TYPE_SATELLITE:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_TITLE_TUNER),
                                     MAIN_LINEBUFCHARSET);

            sprintf (appDat->asciiBuf, " %d: ", (inst->feIdx + 1));

            bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_TITLE_TUNER_LNB),
                                     MAIN_LINEBUFCHARSET);
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case FE_SYSTEM_TYPE_TERRESTRIAL:
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_TITLE_TUNER),
                                     MAIN_LINEBUFCHARSET);

            sprintf (appDat->asciiBuf, " %d: ", (inst->feIdx + 1));

            bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_TITLE_TUNER_DVBT),
                                     MAIN_LINEBUFCHARSET);
            break;

        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED

        case FE_SYSTEM_TYPE_CABLE:
            /* to do... */

        #endif /* APPL_FECABLE_ENABLED */

        default:
            STR_BufClear(NULL);
            break;
    }

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.chr        = STR_IntLineStr();
    item->item.str.font       = TITLE_FONT;
    item->item.str.lineHeight = 0;
    item->item.str.col        = OSD_COL_TITLE_TEXT;
}
#endif
FSTATIC fbool_t MENU_TunerKeys (FAPI_SYS_HandleT h)
{
    MENU_TUNER_INST_S*  inst    = (MENU_TUNER_INST_S*)h;
    fbool_t               keyUsed = FFALSE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
            /* FALLTHROUGHT */
        case FGS_KEY_BACK:
            if ( inst->guidedMode ) keyUsed = MENU_GuidedGotoPrevious();
            else                    keyUsed = MENUSTACK_Back(NULL);
            break;

        default:
            break;
    }

    return keyUsed;
}

FSTATIC fbool_t MENU_TunerBuild(FAPI_SYS_HandleT h)
{
    MENU_TUNER_INST_S*    inst = (MENU_TUNER_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FGS_LIST_OPEN_S       listPars;
    FAPI_SYS_HandleT      itemHdl;
    MAIN_SIGBOX_OPEN_S    sigPars;
    FGS_ITEM_TXT_OPEN_S   txtPars;
    FGS_ITEM_NUM_OPEN_S   numPars;
    FGS_ITEM_SEL_OPEN_S   selPars;
    FGS_BOX_OPEN_S        boxPars;
    FGS_POS_S             itemPos;
    FGS_POS_S             valuePos;
    FGS_POS_S             descrPos;
    int32_t               retVal;
    uint16_t              helpWidth;
    uint16_t              stackedFrameWidth;
    uint16_t              innerHeight;
    uint32_t              numLines     = 1;
    FGS_PANEL_MEMBER_S    member;
    fbool_t               skipLine    = FFALSE;

    #ifdef APPL_FESAT_ENABLED

    CFGSAT_LNBCFG_S*      pLnb = NULL;
    CFGSAT_LNBMODE_E      controlMode;
    fbool_t               openSignalBar;
    int16_t               satNums;

    #endif /* APPL_FESAT_ENABLED */

    if(inst == NULL) return FFALSE;

    #ifdef APPL_FESAT_ENABLED
    controlMode = inst->currFeDat.controlMode[inst->feIdx];
    #endif /* APPL_FESAT_ENABLED */

    switch ( inst->feType )
    {
        #ifdef APPL_FESAT_ENABLED

        case FE_SYSTEM_TYPE_SATELLITE:
            pLnb = inst->currSatDat.lnbCfg + inst->feIdx;

            switch ( controlMode )
            {
                default:
                case CFGSAT_FEMODE_CLASSIC:
                case CFGSAT_FEMODE_TONEBURST:
                case CFGSAT_FEMODE_DISEQC10: numLines =  5; break;
                case CFGSAT_FEMODE_DISEQC11_DTH: numLines = 6; break;

                #ifdef APPL_DISEQC11_SMATV_ENABLED
                case CFGSAT_FEMODE_DISEQC11_SMATV: numLines = 5; break;
                #endif /* APPL_DISEQC12_SMATV_ENABLED */

                #ifdef APPL_DISEQC12_ENABLED
                case CFGSAT_FEMODE_DISEQC12: numLines = 13; break;
                #endif /* APPL_DISEQC12_ENABLED */

                #ifdef APPL_USALS_ENABLED
                case CFGSAT_FEMODE_USALS:    numLines = 13; break;
                #endif /* APPL_USALS_ENABLED */
            }
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case FE_SYSTEM_TYPE_TERRESTRIAL:
            numLines = 1;
            break;

        #endif /* APPL_FETERR_ENABLED */

        default:
            break;
    }

    if ( inst->guidedMode ||
        (inst->feIdx < inst->feLast) )
    {
        numLines += 1;
    }

    #ifdef APPL_FESAT_ENABLED

    switch ( controlMode )
    {
        #ifdef APPL_DISEQC12_ENABLED
        case CFGSAT_FEMODE_DISEQC12:
            openSignalBar = FTRUE;
            break;
        #endif /* APPL_DISEQC12_ENABLED */

        #ifdef APPL_USALS_ENABLED
        case CFGSAT_FEMODE_USALS:
            openSignalBar = FTRUE;
            break;
        #endif /* APPL_USALS_ENABLED */

        default:
            openSignalBar = FFALSE;
            break;
    }

    #endif /* APPL_FESAT_ENABLED */

    memset(&txtPars,  0, sizeof(txtPars));
    memset(&numPars,  0, sizeof(numPars));
    memset(&sigPars,  0, sizeof(sigPars));
    memset(&selPars,  0, sizeof(selPars));
    memset(&member,   0, sizeof(member));
    memset(&boxPars,  0, sizeof(boxPars));

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_09LR),
                                   -1,SYMBOL_FONT);
    innerHeight        = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_TunerOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_TUNER, &(inst->pnlHdl));
        if ( retVal != FAPI_OK )     break;

        /* ----------------------------------------------------------------- */

        inst->itemPos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   stackedFrameWidth;
        inst->itemPos.y         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                   (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                   SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                   stackedFrameWidth;
        inst->itemPos.width     = SCALER_GetCurrVal(MAIN_WIDTH);
        inst->itemPos.height    = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(INNER_FRAME_WIDTH));
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* ----------------------------------------------------------------- */

        #ifdef APPL_FESAT_ENABLED

        if ( inst->feType == FE_SYSTEM_TYPE_SATELLITE )
        {
            member.h             = (FAPI_SYS_HandleT)inst;
            member.pos           = &(inst->itemPos);
            member.id            = MENU_TUNER_ID_REST;
            member.isHidden      = FFALSE;
            member.show          = MENU_TunerDraw;

            retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                         &(inst->pItem));
            if ( retVal != FAPI_OK ) break;
        }

        #endif /* APPL_FESAT_ENABLED */

        /* ----- create textbox holding title ------------------------------ */

        boxPars.pos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                 SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                 stackedFrameWidth;
        boxPars.pos.y         = SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                (SCALER_GetCurrVal(OUTER_FRAME_ALL) - SCALER_GetCurrVal(OUTER_FRAME_WIDTH))/2 + /* done for centering */
                                 SCALER_GetCurrVal(TITLE_TB_PADDING) +
                                 stackedFrameWidth;
        boxPars.pos.width     = pnlPars.pos.width - (2*SCALER_GetCurrVal(OUTER_FRAME_ALL)) -
                                 SCALER_GetCurrVal(MAIN_RIGHT_PADDING) - stackedFrameWidth;
        boxPars.pos.height    = (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) + SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType      = OSD_BOX_BKGR_NONE;
        boxPars.getItem       = MENU_TunerGetItemTitle;

        retVal = FGS_BOX_Open (&boxPars, MENU_TUNER_ID_TITLE,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_TunerGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_TunerGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_TunerOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_TunerOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_TunerValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_TUNER_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 66,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- menu lines -------------------------------------------------*/

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;

        switch ( inst->feType )
        {
            #ifdef APPL_FESAT_ENABLED

            case FE_SYSTEM_TYPE_SATELLITE:

                /* ----- control method ---------------------------------------------*/

                selPars.pos                 = itemPos;
                selPars.bkgrType            = txtPars.bkgrType;
                selPars.descrPos            = descrPos;
                selPars.valuePos            = valuePos;
                selPars.params.numItems     = CFGSAT_FEMODE_NUMS;
                selPars.params.index        = controlMode;

                retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_CONTROLMETHOD,
                                            inst->listHdl, &itemHdl);
                if ( retVal != FAPI_OK ) break;

                itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
                itemPos.y += 2*SCALER_GetCurrVal(LIST_LINE_PADDING);

                /* ----- select satellite -------------------------------------------*/

                (void)CFGSAT_GetNumSats(&satNums);

                selPars.pos                 = itemPos;
                selPars.params.numItems     = satNums;
                selPars.params.index        = inst->satLstPos;

                retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_SELSAT,
                                            inst->listHdl, &itemHdl);
                if ( retVal != FAPI_OK ) break;

                itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

                /* ----- System IDs ------------------------------------------------ */

                selPars.pos                 = itemPos;

                if(pLnb != NULL)
                {
                    switch ( controlMode )
                    {
                        case CFGSAT_FEMODE_CLASSIC:
                            selPars.params.numItems = 2;
                            selPars.params.index    = pLnb->active;

                            retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_CLASSICID,
                                                        inst->listHdl, &itemHdl);
                            break;

                        case CFGSAT_FEMODE_TONEBURST:
                            selPars.params.numItems = CFGSAT_TONEBURST_ID_NUMS + 1;

                            if ( !(pLnb->active) ) selPars.params.index = 0;
                            else                   selPars.params.index = pLnb->toneburstId + 1;

                            retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_TONEBURSTID,
                                                        inst->listHdl, &itemHdl);
                            break;

                        case CFGSAT_FEMODE_DISEQC10:
                            selPars.params.numItems = CFGSAT_DISEQC10_ID_NUMS + 1;

                            if ( !(pLnb->active) ) selPars.params.index = 0;
                            else                   selPars.params.index = pLnb->diseqc10Id + 1;

                            retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_DISEQC10ID,
                                                        inst->listHdl, &itemHdl);
                            break;

                        case CFGSAT_FEMODE_DISEQC11_DTH:
                            selPars.params.numItems = CFGSAT_DISEQC11_DTH_ID_NUMS + 1;

                            if ( !(pLnb->active) ) selPars.params.index = 0;
                            else                   selPars.params.index = pLnb->diseqc11Id + 1;

                            retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_DISEQC11ID,
                                                        inst->listHdl, &itemHdl);
                            break;

                        #ifdef APPL_DISEQC11_SMATV_ENABLED

                        case CFGSAT_FEMODE_DISEQC11_SMATV:
                            selPars.params.numItems = CFGSAT_DISEQC11_SMATV_ID_NUMS + 1;

                            if ( !(pLnb->active) )
                            {
                                selPars.params.index = 0;
                            }
                            else
                            {
                                selPars.params.index = pLnb->diseqc11Id_SMATV + 1;
                            }

                            retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_DISEQC11ID_SMATV,
                                                        inst->listHdl, &itemHdl);
                            break;

                        #endif /* APPL_DISEQC12_ENABLED */

                        #ifdef APPL_DISEQC12_ENABLED

                        case CFGSAT_FEMODE_DISEQC12:
                            selPars.params.numItems = CFGSAT_DISEQC12_POS_NUMS + 1;

                            if ( !(pLnb->active) ) selPars.params.index = 0;
                            else                   selPars.params.index = pLnb->diseqc12Pos;

                            retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_DISEQC12POS,
                                                        inst->listHdl, &itemHdl);
                            break;

                        #endif /* APPL_DISEQC12_ENABLED */

                        default:
                            skipLine = FTRUE;
                            break;
                    }
                }
                else
                {
                    skipLine = FTRUE;
                }

                if ( retVal != FAPI_OK ) break;

                if ( !skipLine )
                {
                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }
                /* --- DiSEcQ 1.1 repeat ----------------------------*/
                if( ( controlMode == CFGSAT_FEMODE_DISEQC11_DTH )
                    #ifdef APPL_DISEQC11_SMATV_ENABLED
                    ||
                    ( controlMode == CFGSAT_FEMODE_DISEQC11_SMATV )
                    #endif
                    )
                {
                    selPars.pos             = itemPos;
                    selPars.params.numItems = 3;
                    if(pLnb != NULL)
                    {
                        if(pLnb->diseqc11Rep>2)pLnb->diseqc11Rep=2;
                        selPars.params.index = pLnb->diseqc11Rep;
                    }
                    else
                    {
                        selPars.params.index = 0;
                    }

                    retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_DISEQC11REP,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }
                /* --- DiSEcQ 1.1 SMATV IF frequncy ------------------------*/
                #ifdef APPL_DISEQC11_SMATV_ENABLED
                if( controlMode == CFGSAT_FEMODE_DISEQC11_SMATV)
                {
                    selPars.pos             = itemPos;
                    selPars.params.numItems = 8;
                    switch(inst->currFeDat.ifFreq)
                    {
                        default:
                        case 1284: selPars.params.index = 0; break;
                        case 1400: selPars.params.index = 1; break;
                        case 1516: selPars.params.index = 2; break;
                        case 1632: selPars.params.index = 3; break;
                        case 1748: selPars.params.index = 4; break;
                        case 1864: selPars.params.index = 5; break;
                        case 1980: selPars.params.index = 6; break;
                        case 2196: selPars.params.index = 7; break;
                    };
                    retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_IF_FREQ,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }
                /* --- High freq ------------------------------------*/
                if( controlMode != CFGSAT_FEMODE_DISEQC11_SMATV)
                #endif
                {
                    numPars.pos              = itemPos;
                    numPars.bkgrType         = OSD_LSTITEM_BKGR_NORMAL;
                    numPars.descrPos         = descrPos;
                    numPars.valuePos         = valuePos;
                    numPars.type             = FGS_NITEM_TYPE_NUMBER;
                    numPars.wraparound       = FTRUE;
                    numPars.params.minValue  = LNB_MIN_FREQ;
                    numPars.params.maxValue  = LNB_MAX_FREQ;
                    numPars.params.maxDigits = 5;
                    numPars.params.step      = 1;

                    retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TUNER_ID_HIGHFREQ,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                    /* --- Low freq ------------------------------------*/

                    numPars.pos              = itemPos;
                    numPars.params.minValue  = LNB_MIN_FREQ;
                    numPars.params.maxValue  = LNB_MAX_FREQ;
                    numPars.params.maxDigits = 5;
                    numPars.params.step      = 1;

                    retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TUNER_ID_LOWFREQ,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }
                #ifdef APPL_DISEQC12_ENABLED

                if ( controlMode == CFGSAT_FEMODE_DISEQC12 )
                {
                    /* --- STORESAT ------------------------------------------------*/

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TUNER_ID_STORESAT,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- EXPMOTOR ------------------------------------------------ */

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TUNER_ID_EXPMOTOR,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }

                #endif /* APPL_DISEQC12_ENABLED */

                /* --- LONGITUDE settings ------------------------------------------------*/

                #ifdef APPL_USALS_ENABLED

                if ( controlMode == CFGSAT_FEMODE_USALS )
                {
                    numPars.pos              = itemPos;
                    numPars.bkgrType         = OSD_LSTITEM_BKGR_NORMAL;
                    numPars.descrPos         = descrPos;
                    numPars.valuePos         = valuePos;
                    numPars.type             = FGS_NITEM_TYPE_NUMBER;
                    numPars.wraparound       = FTRUE;
                    numPars.params.minValue  = -900;
                    numPars.params.maxValue  = 900;
                    numPars.params.maxDigits = 5;
                    numPars.params.step      = 1;

                    retVal = FGS_ITEM_NUM_Open (&numPars,
                                                MENU_TUNER_ID_LONGITUDE,
                                                inst->listHdl,
                                                &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- LATITUDE settings --------------------------------------- */

                    numPars.pos              = itemPos;
                    numPars.bkgrType         = OSD_LSTITEM_BKGR_NORMAL;
                    numPars.descrPos         = descrPos;
                    numPars.valuePos         = valuePos;
                    numPars.type             = FGS_NITEM_TYPE_NUMBER;
                    numPars.wraparound       = FTRUE;
                    numPars.params.minValue  = -1800;
                    numPars.params.maxValue  = 1800;
                    numPars.params.maxDigits = 5;
                    numPars.params.step      = 1;

                    retVal = FGS_ITEM_NUM_Open (&numPars, MENU_TUNER_ID_LATITUDE,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- TURNMOTOR ------------------------------------------------*/

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TUNER_ID_TURNMOTOR,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }

                #endif /* APPL_USALS_ENABLED */

                #if defined(APPL_DISEQC12_ENABLED) || defined(APPL_USALS_ENABLED)

                if (
                    #ifdef APPL_DISEQC12_ENABLED
                    ( controlMode == CFGSAT_FEMODE_DISEQC12 )
                    #endif
                    #if defined(APPL_DISEQC12_ENABLED) && defined(APPL_USALS_ENABLED)
                    ||
                    #endif
                    #ifdef APPL_USALS_ENABLED
                     ( controlMode == CFGSAT_FEMODE_USALS )
                    #endif
                   )
                {
                    /* --- MOVESTYLE ------------------------------------------------*/

                    selPars.pos                 = itemPos;
                    selPars.params.numItems     = MENU_TUNER_MOVE_NUMS;
                    selPars.params.index        = inst->currMoveStyle;

                    retVal = FGS_ITEM_SEL_Open (&selPars, MENU_TUNER_ID_MOVESTYLE,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- MOVELR -------------------------------------------------- */

                    selPars.pos                 = itemPos;
                    selPars.params.numItems     = MOVE_MOTOR_NUMS;
                    selPars.params.index        = MOVE_MOTOR_STOP;

                    retVal = FGS_ITEM_SEL_Open (&selPars,  MENU_TUNER_ID_MOVELR,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }
                #endif /* #if define(APPL_DISEQC12_ENABLED) || defined(APPL_USALS_ENABLED) */

                if ( openSignalBar )
                {
                    /* ----- SELTP ------------------------------------------------- */

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TUNER_ID_SELTP,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* ----- Signal box -------------------------------------------- */

                    sigPars.pnlHdl     = inst->pnlHdl;
                    sigPars.pos.x      = itemPos.x + listPars.pos.x + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
                    sigPars.pos.y      = itemPos.y + listPars.pos.y + SCALER_GetCurrVal(LIST_LINE_PADDING);
                    sigPars.descrWidth = ((uint16_t)(itemPos.width- SCALER_GetCurrVal(MAIN_LEFT_PADDING))/(2));
                    sigPars.barWidth   = ((uint16_t)(itemPos.width- SCALER_GetCurrVal(MAIN_LEFT_PADDING)-SCALER_GetCurrVal(MAIN_RIGHT_PADDING))/(2));
                    sigPars.feIdx      = -1;

                    retVal = MAIN_SignalBoxOpen(&(appDat->sigBox), &sigPars);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += MAIN_SignalBoxGetHeight();

                    itemPos.y += (2*SCALER_GetCurrVal(LIST_LINE_PADDING));

                    /* ----- LOCK -------------------------------------------------- */

                    txtPars.pos                 = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TUNER_ID_LOCK,
                                                inst->listHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_TUNER_ID_LOCK, FFALSE);

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }
                break;

            #endif /* APPL_FESAT_ENABLED */

            #ifdef APPL_FETERR_ENABLED

            case FE_SYSTEM_TYPE_TERRESTRIAL:
                /* ----- Antenna Power ---------------------------------------------*/

                selPars.pos                 = itemPos;
                selPars.bkgrType            = txtPars.bkgrType;
                selPars.descrPos            = descrPos;
                selPars.valuePos            = valuePos;
                selPars.params.numItems     = 2;
                selPars.params.index        = inst->antennaPowerTerr;

                retVal = FGS_ITEM_SEL_Open (&selPars,
                                            MENU_TUNER_ID_ANTENNA_POWER,
                                            inst->listHdl, &itemHdl);
                if ( retVal != FAPI_OK ) break;

                itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                             SCALER_GetCurrVal(LIST_LINE_PADDING);
                itemPos.y += 2*SCALER_GetCurrVal(LIST_LINE_PADDING);
                break;

            #endif /* APPL_FETERR_ENABLED */

            #ifdef APPL_FECABLE_ENABLED

            case FE_SYSTEM_TYPE_CABLE:
                break;

            #endif /* APPL_FECABLE_ENABLED */

            default:
                break;
        }

        /* ----- Goto next ------------------------------------------------- */

        if ( inst->guidedMode || (inst->feIdx < inst->feLast) )
        {
            txtPars.pos            = itemPos;
            txtPars.descrPos.width = (itemPos.width -
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                      SCALER_GetCurrVal(MAIN_RIGHT_PADDING));

            retVal = FGS_ITEM_TXT_Open (&txtPars,
                                        MENU_TUNER_ID_GOTONEXT,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                         SCALER_GetCurrVal(LIST_LINE_PADDING);
        }

        /* ----- Create help list ------------------------------------------ */

        listPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   SCALER_GetCurrVal(MAIN_WIDTH) +
                                   SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                   stackedFrameWidth;
        listPars.pos.width       = helpWidth;
        listPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        listPars.pInst           = &(appDat->helpListInst);
        listPars.getDescrItem    = MENU_TunerGetHelpDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;

        retVal = FGS_LIST_Open (&listPars, MENU_TUNER_ID_HELPLIST, inst->pnlHdl,
                                &(inst->listHelpHdl));
        if ( retVal != FAPI_OK )     break;

        /* ----------------------------------------------------------------- */

        itemPos.x          = SCALER_GetCurrVal(HELP_LR_PADDING);
        itemPos.y          = 0;
        itemPos.width      = listPars.pos.width-
                             (2*SCALER_GetCurrVal(HELP_LR_PADDING));
        itemPos.height     = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        itemPos.alignment  = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_TOP;

        /* position of list item descriptors in list entry */
        descrPos.x         = 0;
        descrPos.y         = 0;
        descrPos.width     = itemPos.width;
        descrPos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        descrPos.alignment = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;

        /* ----- help lines ------------------------------------------------ */

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_HELP;
        txtPars.descrPos           = descrPos;
        txtPars.pos                = itemPos;

        switch ( inst->feType )
        {
            #ifdef APPL_FESAT_ENABLED

            case FE_SYSTEM_TYPE_SATELLITE:
                /* --- Control method ----------------------------------------*/

                retVal = FGS_ITEM_TXT_Open (&txtPars,
                                            MENU_TUNER_HELP_ID_CONTROLMETHOD,
                                            inst->listHelpHdl,
                                            &itemHdl);
                if ( retVal != FAPI_OK ) break;

                itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                             SCALER_GetCurrVal(LIST_LINE_PADDING);

                /* ----- select satellite  ---------------------------------- */

                itemPos.y += 2*SCALER_GetCurrVal(LIST_LINE_PADDING);

                txtPars.pos = itemPos;
                retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TUNER_HELP_ID_SELSAT,
                                            inst->listHelpHdl, &itemHdl);
                if ( retVal != FAPI_OK ) break;

                itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                             SCALER_GetCurrVal(LIST_LINE_PADDING);

                /* ----- Control Mode IDs ----------------------------------- */

                txtPars.pos = itemPos;

                switch ( controlMode )
                {
                    default:
                    case CFGSAT_FEMODE_CLASSIC:
                        retVal = FGS_ITEM_TXT_Open (
                                                &txtPars,
                                                MENU_TUNER_HELP_ID_CLASSICID,
                                                inst->listHelpHdl, &itemHdl);
                        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                     SCALER_GetCurrVal(LIST_LINE_PADDING);
                        break;

                    case CFGSAT_FEMODE_TONEBURST:
                        retVal = FGS_ITEM_TXT_Open (
                                                &txtPars,
                                                MENU_TUNER_HELP_ID_TONEBURSTID,
                                                inst->listHelpHdl, &itemHdl);
                        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                     SCALER_GetCurrVal(LIST_LINE_PADDING);
                        break;

                    case CFGSAT_FEMODE_DISEQC10:
                        retVal = FGS_ITEM_TXT_Open (
                                                &txtPars,
                                                MENU_TUNER_HELP_ID_DISEQC10ID,
                                                inst->listHelpHdl, &itemHdl);
                        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                     SCALER_GetCurrVal(LIST_LINE_PADDING);
                        break;
                    case CFGSAT_FEMODE_DISEQC11_DTH:
                        retVal = FGS_ITEM_TXT_Open (
                                                &txtPars,
                                                MENU_TUNER_HELP_ID_DISEQC11ID,
                                                inst->listHelpHdl, &itemHdl);
                        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                     SCALER_GetCurrVal(LIST_LINE_PADDING);
                        break;

                    #ifdef APPL_DISEQC11_SMATV_ENABLED
                    case CFGSAT_FEMODE_DISEQC11_SMATV:
                        retVal = FGS_ITEM_TXT_Open (
                                                &txtPars,
                                                MENU_TUNER_HELP_ID_DISEQC11ID_SMATV,
                                                inst->listHelpHdl, &itemHdl);
                        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                     SCALER_GetCurrVal(LIST_LINE_PADDING);
                        break;
                    #endif
                    #ifdef APPL_DISEQC12_ENABLED

                    case CFGSAT_FEMODE_DISEQC12:
                        retVal = FGS_ITEM_TXT_Open (
                                                &txtPars,
                                                MENU_TUNER_HELP_ID_DISEQC12POS,
                                                inst->listHelpHdl, &itemHdl);
                        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                     SCALER_GetCurrVal(LIST_LINE_PADDING);
                        break;

                    #endif /* APPL_DISEQC12_ENABLED */

                    #ifdef APPL_USALS_ENABLED

                    case CFGSAT_FEMODE_USALS:
                        /* Do nothing */
                        break;

                    #endif /* APPL_DISEQC12_ENABLED */
                }
                if ( retVal != FAPI_OK ) break;

                /* ----- DiSEqC 1.1 repeat ---------------------------------- */
                if( ( controlMode == CFGSAT_FEMODE_DISEQC11_DTH )
                    #ifdef APPL_DISEQC11_SMATV_ENABLED
                    ||
                    ( controlMode == CFGSAT_FEMODE_DISEQC11_SMATV )
                    #endif
                    )
                {
                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_DISEQC11REP,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }
                /* ----- DiSEqC 1.1 SMATV IF frequency ---------------------- */
                #ifdef APPL_DISEQC11_SMATV_ENABLED
                if ( controlMode == CFGSAT_FEMODE_DISEQC11_SMATV )
                {
                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (
                                            &txtPars,
                                            MENU_TUNER_HELP_ID_IF_FREQ,
                                            inst->listHelpHdl, &itemHdl);
                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }

                /* ----- High freq ------------------------------------------ */
                if( controlMode != CFGSAT_FEMODE_DISEQC11_SMATV)
                #endif /* APPL_DISEQC11_SMATV_ENABLED */
                {
                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_HIGHFREQ,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* ----- Low freq ------------------------------------------- */

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_LOWFREQ,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }
                #ifdef APPL_DISEQC12_ENABLED

                if ( controlMode == CFGSAT_FEMODE_DISEQC12 )
                {
                    /* --- STORESAT ------------------------------------------*/

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_STORESAT,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* ----- EXPMOTOR --------------------------------------- */

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_EXPMOTOR,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- MOVESTYLE -----------------------------------------*/

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_MOVESTYLE,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- MOVELR --------------------------------------------*/

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_MOVELR,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }

                #endif /* APPL_DISEQC12_ENABLED */

                #ifdef APPL_USALS_ENABLED

                if ( controlMode == CFGSAT_FEMODE_USALS )
                {
                    /* ----- LONGITUDE settings ----------------------------- */

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_LONGITUDE,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- LATITUDE settings -------------------------------- */

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_LATITUDE,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- TURNMOTOR ---------------------------------------- */

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_TURNMOTOR,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- TURNMOTOR style ---------------------------------- */

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_MOVESTYLE,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);

                    /* --- TURNMOTOR left/right ----------------------------- */

                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_MOVELR,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK ) break;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }

                #endif /* APPL_USALS_ENABLED */

                /* --- SELTP ------------------------------------------------*/

                if ( openSignalBar )
                {
                    txtPars.pos = itemPos;
                    retVal = FGS_ITEM_TXT_Open (&txtPars,
                                                MENU_TUNER_HELP_ID_SELTP,
                                                inst->listHelpHdl, &itemHdl);
                    if ( retVal != FAPI_OK )     break;

                    inst->sigBarOpen = FTRUE;

                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                    itemPos.y += MAIN_SignalBoxGetHeight();
                    itemPos.y += (2*SCALER_GetCurrVal(LIST_LINE_PADDING));
                     /* lock line */
                    itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                 SCALER_GetCurrVal(LIST_LINE_PADDING);
                }
                break;

            #endif /* APPL_FESAT_ENABLED */

            #ifdef APPL_FETERR_ENABLED

            case FE_SYSTEM_TYPE_TERRESTRIAL:
                /* ----- Antenna Power ---------------------------------------------*/

                retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TUNER_HELP_ID_ANTENNA_POWER,
                                            inst->listHelpHdl, &itemHdl);
                if ( retVal != FAPI_OK ) break;

                itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
                itemPos.y += 2*SCALER_GetCurrVal(LIST_LINE_PADDING);
                break;

            #endif /* APPL_FETERR_ENABLED */

            default:
                break;
        }

        /* ----- Goto next ------------------------------------------------- */

        if ( inst->guidedMode || (inst->feIdx < inst->feLast) )
        {
            txtPars.pos = itemPos;
            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_TUNER_HELP_ID_GOTONEXT,
                                        inst->listHelpHdl, &itemHdl);
            if ( retVal != FAPI_OK )     break;

            itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    return FTRUE;
}

void MENU_TunerUpdateLockStatus (uint8_t feNum, SYS_FE_LOCK_E status)
{
    MENU_TUNER_INST_S* inst = (MENU_TUNER_INST_S*)menuTunerDat;

    if ( inst->pnlHdl == NULL ) return;

    #ifdef APPL_FESAT_ENABLED

    (void)FGS_LIST_RefreshMember(inst->listHdl, MENU_TUNER_ID_LOCK);

    #endif /* APPL_FESAT_ENABLED */
}

#ifdef APPL_FESAT_ENABLED

FSTATIC int32_t MENU_TunerDraw (FAPI_SYS_HandleT h)
{
    MENU_TUNER_INST_S* inst = (MENU_TUNER_INST_S*)h;
    FGS_LIST_INST_S *       listInst = (FGS_LIST_INST_S*)inst->listHdl;
    FGS_LOC_S               absPos;

    memset(&absPos, 0, sizeof(absPos));

    absPos = listInst->member[0].abspos;
    OSD_DrawSeparatorHorizontal(absPos.x,
                                absPos.y + SCALER_GetCurrVal(LIST_LINE_HEIGHT) + 2*SCALER_GetCurrVal(LIST_LINE_PADDING),
                                SCALER_GetCurrVal(MAIN_WIDTH));

    return FAPI_OK;
}

FSTATIC void MENU_TunerUpdateSignalDisplay (MENU_TUNER_INST_S* inst)
{
    #if (defined APPL_USALS_ENABLED) || (defined APPL_DISEQC12_ENABLED)

    SYS_NETWORK_S network;
    int32_t       retVal;

    if ( !inst->sigBarOpen )
    {
        return;
    }

    (void)AV_TuneModeStop(inst->feIdx);

    memset(&network, 0, sizeof(network));

    network.type   = FE_SYSTEM_TYPE_SATELLITE;
    network.satIdx = appDat->currSatIdx;

    retVal = AV_TuneModeStart (inst->feIdx, &network,
                               inst->carrierLstPos,
                               NULL, NULL);
    if ( retVal == FAPI_OK )
    {
        (void)MAIN_SignalBoxSetFeIdx(&(appDat->sigBox), inst->feIdx);
    }
    else
    {
        (void)MAIN_SignalBoxSetFeIdx(&(appDat->sigBox), -1);
    }

    (void)MAIN_SignalBoxUpdate  (&(appDat->sigBox));

    #endif
}

FSTATIC void MENU_TunerUpdateCarrierSel (int16_t satLstPos, int16_t carrierLstPos)
{
    MENU_TUNER_INST_S* inst = (MENU_TUNER_INST_S*)menuTunerDat;

    DBG_Assert(inst != NULL);
    if(inst == NULL) return;

    inst->satLstPos     = satLstPos;
    inst->carrierLstPos = carrierLstPos;
}

#endif /* APPL_FESAT_ENABLED */

void MENU_TunerRefresh(void)
{
    MENU_TUNER_INST_S* inst = (MENU_TUNER_INST_S*)menuTunerDat;

    /* check if Menu is open */
    if ( inst->pnlHdl == NULL ) return;

    #ifdef APPL_FESAT_ENABLED

    if ( inst->feType == FE_SYSTEM_TYPE_SATELLITE )
    {
        /* ----- close menu panel ------------------------------------------ */

        (void)FGS_PANEL_Close(inst->pnlHdl);

        if ( inst->sigBarOpen ) (void)MAIN_SignalBoxClose((&appDat->sigBox));

        inst->pnlHdl     = NULL;
        inst->sigBarOpen = FFALSE;

        (void)MENUSTACK_FocusRelease ((int32_t)MENUID_TUNER);

        /* ----- re-open menu panel ---------------------------------------- */

        inst->currMoveStyle = MENU_TUNER_MOVE_STEP;

        (void)MENU_TunerBuild((FAPI_SYS_HandleT)inst);

        (void)FGS_LIST_SetMemberFocus  (inst->listHdl, MENU_TUNER_ID_CONTROLMETHOD);
        (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_TUNER_ID_LIST);
        (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_TUNER);
        (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);

        MENU_TunerUpdateSignalDisplay (inst);
    }

    #endif /* APPL_FESAT_ENABLED */
}

#ifdef APPL_DISEQC12_ENABLED
FSTATIC void MENU_TunerMotorHalt(void)
{
    if ( menuTunerDat->pnlHdl == NULL )
    {
        return; /* menu visible? */
    }

    (void)TIME_PeriodicTimerAbort(OSD_PERIODTIMER_GENERAL);
    appDat->osdUpdateCb = NULL;

    (void)DIS12_HaltMotor (menuTunerDat->feIdx);
}
#endif /* APPL_DISEQC12_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
