/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_update.c
**
**              $Id: app_menu_update.c 5963 2010-03-09 12:02:13Z ychen $
**
** \brief       Software update menu.
**
** Detail description. [\ref app_smartgo_app_ref_update]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

#ifdef APPL_FILEUPDATE_ENABLED
#include "app_menu_viewer.h"
#endif /* APPL_FILEUPDATE_ENABLED */

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define MENU_UPDATE_REBOOT_DELAY  3000 //!< Sleep between update and reset in ms

#ifdef APPL_FILEUPDATE_ENABLED

#define FILEUPDATE_BUFSIZE        (16*1024) //!< in bytes

#endif /* APPL_FILEUPDATE_ENABLED */

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum /* panel */
{
    MENU_UPDATE_ID_TITLE = 0,
    MENU_UPDATE_ID_LIST,
    MENU_UPDATE_ID_CURRVERSION,
    MENU_UPDATE_ID_PREVVERSION,
    MENU_UPDATE_ID_REST,
    MENU_UPDATE_ID_PROGRESS,
    MENU_UPDATE_ID_STATUS,
    MENU_UPDATE_REDBUTTON_KEY,
    MENU_UPDATE_REDBUTTON_DESC
};

enum /* list */
{
    #ifdef APPL_FILEUPDATE_ENABLED
    MENU_UPDATE_ID_ENTRY_FROM_FILE,
    #endif /* APPL_FILEUPDATE_ENABLED */

    #ifdef APPL_OAD_ENABLED
    MENU_UPDATE_ID_ENTRY_OVER_AIR,
    #endif /* APPL_OAD_ENABLED */

    MENU_UPDATE_ID_ENTRY_REVERT,

    #ifdef APPL_USB_ENABLED
    MENU_UPDATE_ID_ENTRY_SAVESSU,
    #endif /* APPL_USB_ENABLED */

    MENU_UPDATE_ID_ENTRY_NUMS
};

typedef enum
{
    MENU_UPDATE_TYPE_NONE =0,

    #ifdef APPL_OAD_ENABLED
    MENU_UPDATE_TYPE_OAD,
    #endif

    MENU_UPDATE_TYPE_FILE

} MENU_UPDATE_TYPE_E;


typedef enum
{
    MENU_UPDATE_STAT_NONE,
    MENU_UPDATE_STAT_SYSTEMBUSY,
    MENU_UPDATE_STAT_INVALIDDATA,
    MENU_UPDATE_STAT_GETDATA,
    MENU_UPDATE_STAT_PROGRAMMING,
    MENU_UPDATE_STAT_ABORTED,
    MENU_UPDATE_STAT_ERROR,
    MENU_UPDATE_STAT_SUCCESS

} MENU_UPDATE_STATUS_E;

typedef struct
{
    FGS_BOX_INST_S          boxInst[5];

    FAPI_SYS_HandleT        pnlHdl;
    FAPI_SYS_HandleT        listHdl;
    FGS_PANEL_MEMBER_S*     pItem;
    FGS_POS_S               itemPos;

    FGS_PANEL_MEMBER_S*     pProg;
    FGS_POS_S               progPos;

    FAPI_BOOT_InfoT         bootInfo;
    fbool_t                 bootInfoValid;
    BOOTER_FlashSlotHeaderT bootHdr[BOOTER_NUM_SLOTS];
    fbool_t                 bootHdrValid[BOOTER_NUM_SLOTS];

    #ifdef APPL_FILEUPDATE_ENABLED

    char_t                  path[MAIN_VIEWER_PATHLEN];
    uint8_t                 buffer[FILEUPDATE_BUFSIZE];

    #endif /* APPL_FILEUPDATE_ENABLED */

    #ifdef APPL_OAD_ENABLED

    fbool_t                 oadProgressRq;
    TPLIST_ITEM_S           carrierInf;  //!< parameters of current carrier
    SYS_NETWORK_S           network;

    #endif /* APPL_OAD_ENABLED */

    MENU_UPDATE_TYPE_E      type;
    MENU_UPDATE_STATUS_E    updateStatus;
    uint32_t                progresscurrent;

} MENU_UPDATE_INST_S;


FSTATIC MENU_UPDATE_INST_S* menuUpdateDat = NULL;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_UpdateLeave        (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_UpdateOnSelect     (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void    MENU_UpdateOnFocusSet   (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    MENU_UpdateGetItem      (FAPI_SYS_HandleT h, int32_t id,
                                         uint32_t index, FGS_ITEMSTATE_E state,
                                         FGS_ITEM_S* item);
FSTATIC void    MENU_UpdateGetBoxItem   (FAPI_SYS_HandleT h, int32_t id,
                                         uint32_t index, FGS_ITEMSTATE_E state,
                                         FGS_ITEM_S* item);
FSTATIC void    MENU_UpdateGetListItem  (FAPI_SYS_HandleT h, int32_t id,
                                         uint32_t index,
                                         FGS_ITEMSTATE_E state,
                                         FGS_ITEM_S* item);
FSTATIC fbool_t MENU_UpdateBuild        (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_UpdateDrawRest     (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_UpdateDrawProgress (FAPI_SYS_HandleT h);
FSTATIC void    MENU_UpdateReadBootInfo (MENU_UPDATE_INST_S* inst);
FSTATIC void    MENU_UpdateRevert       (void* h);
FSTATIC fbool_t MENU_UpdateKeys         (FAPI_SYS_HandleT h);
FSTATIC void    MENU_UpdateAbort        (void* h);

FSTATIC void    MENU_UpdateEnableList   (MENU_UPDATE_INST_S* inst);
FSTATIC void    MENU_UpdateDisableList  (MENU_UPDATE_INST_S* inst);
FSTATIC fbool_t MENU_UpdateSetType      (MENU_UPDATE_INST_S* inst,
                                         MENU_UPDATE_TYPE_E type);

#ifdef APPL_FILEUPDATE_ENABLED

FSTATIC uint32_t MENU_FILEUPDATE_GetChosenVersion   (char_t* path);
FSTATIC FINLINE fbool_t MENU_FILEUPDATE_FileIsValid (const char_t* fileName);
FSTATIC void    MENU_FILEUPDATE_Do                  (void* h);
FSTATIC void    MENU_FILEUPDATE_Decline             (void* h);

FSTATIC fbool_t MENU_UpdateViewerOnBack             (void* arg);
FSTATIC fbool_t MENU_UpdateViewerOnSelect           (void* arg,
                                                     MAIN_VIEWER_FILETYPE type,
                                                     const char_t* path,
                                                     uint16_t pathLenMax,
                                                     const char_t* name,
                                                     uint16_t nameLenMax,
                                                     uint16_t index,
                                                     uint16_t numFiles);

#endif /* APPL_FILEUPDATE_ENABLED */

#ifdef APPL_OAD_ENABLED

FSTATIC void    MENU_UPDATE_OadStart                (void* h);

#endif /* APPL_OAD_ENABLED */

/*!
*******************************************************************************
**
** \brief  Initial function for the OSD System setup Menu
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MENU_UpdateInit(void)
{
    if ( menuUpdateDat != NULL ) return FAPI_ERR_ALREADY_INITIALIZED;

    menuUpdateDat = UTILS_Malloc(sizeof(MENU_UPDATE_INST_S));
    if ( menuUpdateDat == NULL ) return FAPI_ERR_OUT_OF_MEMORY;

    memset(menuUpdateDat, 0, sizeof(MENU_UPDATE_INST_S));

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function
**
**
** \param h      Address of menu handle.
** \param test   entry test.
**
** \return FTRUE
**
******************************************************************************/
fbool_t MENU_UpdateEntry(FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_UPDATE_INST_S*  inst = menuUpdateDat;
    int32_t              focusItem;
    fbool_t              enabled;

    if ( test == FTRUE ) return FTRUE;

    inst->updateStatus    = MENU_UPDATE_STAT_NONE;
    inst->progresscurrent = 0;

    MENU_UpdateReadBootInfo(inst);

    (void)MENU_UpdateBuild((FAPI_SYS_HandleT)inst);
    
    MENU_UpdateEnableList (inst);
    (void)MENU_UpdateSetType    (inst, MENU_UPDATE_TYPE_NONE);

    if ( MENUSTACK_GetData((uint32_t*)(&focusItem)) == FFALSE )
    {
        /* search non-disabled item */
        focusItem = 0;

        while ( focusItem < MENU_UPDATE_ID_ENTRY_NUMS )
        {
            if ( (FGS_LIST_GetMemberEnableStatus (inst->listHdl, focusItem,
                                                 &enabled) == FAPI_OK) &&
                  enabled )
            {
                break;
            }

            focusItem++;
        }
    }

    (void)FGS_PANEL_SetMemberFocus(inst->pnlHdl, MENU_UPDATE_ID_LIST);
    (void)FGS_LIST_SetMemberFocus (inst->listHdl, focusItem);
    (void)MENUSTACK_FocusClaim    ((int32_t)MENUID_UPDATE);
    (void)FGS_PANEL_Show          (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_UpdateLeave, (FAPI_SYS_HandleT)inst);

    #ifdef APPL_OAD_ENABLED

    if ( inst->oadProgressRq )
    {
        MENU_UPDATE_OadStart(inst);
    }

    #endif /* APPL_OAD_ENABLED */

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MENU_UpdateOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_UPDATE_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_RED,  MENU_UpdateKeys, h, FTRUE, FTRUE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
    }

}

FSTATIC fbool_t MENU_UpdateKeys (FAPI_SYS_HandleT h)
{
    MENU_UPDATE_INST_S* inst = (MENU_UPDATE_INST_S*)h;
    MENU_CONFDIALOG_S   confPars;
    fbool_t             keyUsed  = FTRUE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_RED:
            if ( inst->updateStatus == MENU_UPDATE_STAT_GETDATA )
            {
                memset(&confPars, 0, sizeof(MENU_CONFDIALOG_S));

                confPars.numDescLines   = 1;
                confPars.titleStr       = OSD_GetStr(STR_ENTRY_UPDATE_ABORT);
                confPars.infoStr        = OSD_GetStr(STR_ENTRY_UPDATE_ABORT_INFO);
                confPars.confirmStr     = OSD_GetStr(STR_ENTRY_UPDATE_ABORT_OK);
                confPars.declineStr.chr = NULL;
                confPars.discardStr     = OSD_GetStr(STR_ENTRY_UPDATE_ABORT_DECLINE);
                confPars.onConfirm      = MENU_UpdateAbort;
                confPars.onDecline      = NULL;
                confPars.onDiscard      = NULL;
                confPars.arg            = (void*)inst;
                (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

FSTATIC void MENU_UpdateAbort(void* h)
{
    MENU_UPDATE_INST_S*      inst = (MENU_UPDATE_INST_S*)h;

    switch ( inst->type )
    {
        #ifdef APPL_OAD_ENABLED

        case MENU_UPDATE_TYPE_OAD:
            (void)OAD_Stop();
            (void)MAIN_RestartPresentation();
            break;

        #endif /* APPL_OAD_ENABLED */

        default:
            break;
    }

    (void)UPDATE_Abort();
    (void)UPDATE_Close();

    MENU_UpdateEnableList (inst);
    (void)MENU_UpdateSetType    (inst, MENU_UPDATE_TYPE_NONE);
}

FSTATIC fbool_t MENU_UpdateLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_UPDATE_INST_S*      inst = (MENU_UPDATE_INST_S*)h;

    // TODO: block if update process is currently active!
    if (( inst->updateStatus == MENU_UPDATE_STAT_GETDATA ) ||
       ( inst->updateStatus == MENU_UPDATE_STAT_PROGRAMMING ))
    {
        return FFALSE;
    }

    /* close file viewer if currently open */
    (void)MAIN_ViewerClose();

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_UPDATE);

    inst->pnlHdl = NULL;

    #ifdef APPL_OAD_ENABLED

    inst->oadProgressRq = FFALSE;

    #endif /* APPL_OAD_ENABLED */

    return FTRUE;
}

FSTATIC fbool_t MENU_UpdateOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_UPDATE_INST_S*     inst   = (MENU_UPDATE_INST_S*)h;
    fbool_t                 retVal = FTRUE;
    MENU_CONFDIALOG_S       confPars;

    #ifdef APPL_FILEUPDATE_ENABLED

    MAIN_VIEWER_CONFIG_S    viewerCfg;

    #endif /* APPL_FILEUPDATE_ENABLED */

    #ifdef APPL_OAD_ENABLED

    MENUSTACK_ENTRY         newEntry = NULL;

    #endif /* APPL_OAD_ENABLED */

    switch ( itemId )
    {
        #ifdef APPL_FILEUPDATE_ENABLED

        case MENU_UPDATE_ID_ENTRY_FROM_FILE:
            memset (&viewerCfg, 0, sizeof(viewerCfg));

            viewerCfg.titleString       = OSD_GetStr(STR_ENTRY_UPDATE_CHOOSE);
            viewerCfg.allowStackedView  = FTRUE;
            viewerCfg.allowedFileOps    = MAIN_VIEWER_FILEOP_RENAME;
            viewerCfg.allowedPicOps     = 0;
            viewerCfg.allowedMusicOps   = 0;
            viewerCfg.allowedMovieOps   = 0;
            viewerCfg.initialPath[0]    = 0;
            viewerCfg.filterType[0]     = MAIN_VIEWER_FILTYPE_SSU_LC;
            viewerCfg.filterType[1]     = MAIN_VIEWER_FILTYPE_SSU_UC;
            viewerCfg.onBack            = MENU_UpdateViewerOnBack;
            viewerCfg.onExit            = MENU_UpdateViewerOnBack;
            viewerCfg.onSelect          = MENU_UpdateViewerOnSelect;
            viewerCfg.cbArg             = inst;

            (void)FGS_PANEL_Hide (inst->pnlHdl);

            /* open file viewer */
            retVal = MAIN_ViewerOpen (&viewerCfg);

            if ( !retVal )
            {
                (void)FGS_PANEL_Show (inst->pnlHdl, FFALSE);
            }
            break;

        #endif /* APPL_FILEUPDATE_ENABLED */

        #ifdef APPL_OAD_ENABLED

        case MENU_UPDATE_ID_ENTRY_OVER_AIR:

            newEntry     = MENU_OadSatEntry;
            /* store last focus */
            MENUSTACK_SetData(itemId);

            if ( newEntry == NULL ) return retVal;
            else                    return MENUSTACK_Up(newEntry, NULL);

        #endif /* APPL_OAD_ENABLED */

        case MENU_UPDATE_ID_ENTRY_REVERT:
            memset(&confPars, 0, sizeof(MENU_CONFDIALOG_S));

            confPars.numDescLines   = 1;
            confPars.titleStr       = OSD_GetStr(STR_ENTRY_UPDATE_DELETE);
            confPars.infoStr        = OSD_GetStr(STR_ENTRY_UPDATE_HINT);
            confPars.confirmStr     = OSD_GetStr(STR_ENTRY_VIEWER_DELETE_OK);
            confPars.declineStr.chr = NULL;
            confPars.discardStr     = OSD_GetStr(STR_ENTRY_VIEWER_DELETE_CANCEL);
            confPars.onConfirm      = MENU_UpdateRevert;
            confPars.onDecline      = NULL;
            confPars.onDiscard      = NULL;
            confPars.arg            = (void*)inst;
            (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
            break;

        #ifdef APPL_USB_ENABLED

        case MENU_UPDATE_ID_ENTRY_SAVESSU:

            memset(&confPars, 0, sizeof(MENU_CONFDIALOG_S));

            confPars.numDescLines = 1;
            confPars.confirmStr   = OSD_GetStr(STR_ENTRY_UPDATE_CONFIRM);

            if ( SYS_WriteBootImage (SYS_DEV_USB) != FAPI_OK )
            {
                confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_SSS_PROBLEM);
                confPars.infoStr      = OSD_GetStr(STR_CONFDIALOG_SSS_PROBLEMEX);
            }
            else
            {
                confPars.titleStr     = OSD_GetStr(STR_CONFDIALOG_SSS_REPORT);
                confPars.infoStr      = OSD_GetStr(STR_CONFDIALOG_SSS_REPORTEX);
            }

            (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
            break;

        #endif /* APPL_USB_ENABLED */

        default:
            retVal = FFALSE;
            break;
    }

    return retVal;
}

FSTATIC void MENU_UpdateGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                 FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_UPDATE_INST_S* inst= (MENU_UPDATE_INST_S*)h;
    uint8_t             otherSlot;
    uint32_t            bufPos;

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;
    item->item.str.col        = appDat->txtAttr.colNonFocus;

    switch ( id )
    {
        case MENU_UPDATE_ID_CURRVERSION:
            bufPos = STR_BufAppend(NULL, 0,
                                   OSD_GetStr(STR_ENTRY_UPDATE_CURRENT_VERSION),
                                   MAIN_LINEBUFCHARSET);
            if ( inst->bootInfoValid &&
                 inst->bootHdrValid[inst->bootInfo.slotIndex] )
            {

                sprintf (appDat->asciiBuf, ": %s %d.%02d",
                        inst->bootHdr[inst->bootInfo.slotIndex].name,
                        (int16_t)(inst->bootHdr[inst->bootInfo.slotIndex].version>>8),
                        (int16_t)(inst->bootHdr[inst->bootInfo.slotIndex].version & 0xFF));
            }
            else
            {
                sprintf (appDat->asciiBuf, ": --");
            }
            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);
            item->item.str.chr = STR_IntLineStr();
            break;

        case MENU_UPDATE_ID_PREVVERSION:
            otherSlot = (inst->bootInfo.slotIndex == 0) ? 1 : 0;
            bufPos = STR_BufAppend(NULL, 0,
                                   OSD_GetStr(STR_ENTRY_UPDATE_PREV_VERSION),
                                   MAIN_LINEBUFCHARSET);
            /* previous revision */
            if ( inst->bootInfoValid &&
                 inst->bootHdrValid[otherSlot] )
            {
                sprintf (appDat->asciiBuf,": %s %d.%02d",
                         inst->bootHdr[otherSlot].name,
                         (int16_t)(inst->bootHdr[otherSlot].version>>8),
                         (int16_t)(inst->bootHdr[otherSlot].version & 0xFF));
            }
            else
            {
                sprintf (appDat->asciiBuf,": --");
            }

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);
            item->item.str.chr = STR_IntLineStr();
            break;

        case MENU_UPDATE_ID_STATUS:
            switch ( inst->updateStatus )
            {
                default:
                case MENU_UPDATE_STAT_NONE:
                    item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                    break;

                case MENU_UPDATE_STAT_SYSTEMBUSY:
                    item->item.str.chr = OSD_GetStr(STR_INFO_UPDATE_SYSBUSY);
                    break;

                case MENU_UPDATE_STAT_INVALIDDATA:
                    item->item.str.chr = OSD_GetStr(STR_INFO_UPDATE_INVALID_DATA);
                    break;

                case MENU_UPDATE_STAT_GETDATA:
                    item->item.str.chr = OSD_GetStr(STR_INFO_UPDATE_GET_DATA);
                    break;

                case MENU_UPDATE_STAT_PROGRAMMING:
                    item->item.str.chr = OSD_GetStr(STR_INFO_UPDATE_PROGRAMMING);
                    break;

                case MENU_UPDATE_STAT_ABORTED:
                    item->item.str.chr = OSD_GetStr(STR_INFO_UPDATE_ABORT);
                    break;

                case MENU_UPDATE_STAT_ERROR:
                    item->item.str.chr = OSD_GetStr(STR_INFO_UPDATE_ERROR);
                    break;

                case MENU_UPDATE_STAT_SUCCESS:
                    item->item.str.chr = OSD_GetStr(STR_INFO_UPDATE_SUCCESS);
                    break;
            }

            break;

        default:
            item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            break;
    }
}

FSTATIC void MENU_UpdateGetListItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                     FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    item->item.str.font       = MAIN_FONT;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->type                = FGS_ITEM_TYPE_STRING;
    item->item.str.lineHeight = 0;

    switch ( id )
    {
        #ifdef APPL_FILEUPDATE_ENABLED
        case MENU_UPDATE_ID_ENTRY_FROM_FILE:
            strIdx = STR_ENTRY_UPDATE_FROM_FILE;
            break;
        #endif /* APPL_FILEUPDATE_ENABLED */

        #ifdef APPL_OAD_ENABLED
        case MENU_UPDATE_ID_ENTRY_OVER_AIR:
            strIdx = STR_ENTRY_UPDATE_FROM_AIR;
            break;

        #endif /* APPL_OAD_ENABLED */

        case MENU_UPDATE_ID_ENTRY_REVERT:
            strIdx = STR_ENTRY_UPDATE_REVERT;
            break;

        #ifdef APPL_USB_ENABLED
        case MENU_UPDATE_ID_ENTRY_SAVESSU:
            strIdx = STR_ENTRY_UPDATE_SAVESSU;
            break;
        #endif /* APPL_USB_ENABLED */

        default:
            return;
    }

    item->item.str.chr = OSD_GetStr(strIdx);

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
    }
}

FSTATIC void MENU_UpdateGetBoxItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                            FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    //MENU_UPDATE_INST_S*  inst = (MENU_UPDATE_INST_S*)h;

    switch(id)
    {
        case MENU_UPDATE_REDBUTTON_KEY:
            MENU_COMMON_ITEM_BUTTON(item, ICON_BTNREDSMALL);
            break;

        case MENU_UPDATE_REDBUTTON_DESC:
            item->type                   = FGS_ITEM_TYPE_STRING;
            item->align                  = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
            item->item.str.font          = MAIN_FONT;
            item->item.str.lineHeight    = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
            item->item.str.col           = OSD_COL_LIST_TEXT;
            item->item.str.chr           = OSD_GetStr(STR_HELP_UPDATE_CANCEL);
            break;
        default:
        break;
    }
}


FSTATIC fbool_t MENU_UpdateBuild(FAPI_SYS_HandleT h)
{
    MENU_UPDATE_INST_S* inst = (MENU_UPDATE_INST_S*)h;
    FGS_PANEL_OPEN_S    pnlPars;
    FGS_LIST_OPEN_S     listPars;
    FGS_BOX_OPEN_S      boxPars;
    FAPI_SYS_HandleT    itemHdl;
    FGS_ITEM_TXT_OPEN_S txtPars;
    FGS_ITEM_S          item;
    int32_t             retVal;
    uint16_t            bottomHelpKeyWidth;
    uint16_t            bottomHelpItemWidth;
    uint16_t            boxPositionLeft;
    uint16_t            mainPanelWidth;
    uint16_t            stackedFrameWidth;
    uint16_t            innerHeight;
    uint32_t            numLines = MENU_UPDATE_ID_ENTRY_NUMS;
    uint8_t             numHelpLines=2;
    FGS_PANEL_MEMBER_S  member;
    uint16_t            listHeight;

    memset(&member,   0, sizeof(member));
    memset(&txtPars,  0, sizeof(txtPars));
    memset(&boxPars,  0, sizeof(boxPars));
    memset(&item,     0, sizeof(item));

    mainPanelWidth          = SCALER_GetCurrVal(MAIN_WIDTH) - 2*SCALER_GetCurrVal(OUTER_FRAME_ALL);
    bottomHelpKeyWidth      = OSD_GetStrWidth(OSD_GetStr(STR_KEY_MENU),-1,HELP_FONT) + SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
    bottomHelpItemWidth     = mainPanelWidth/2 - bottomHelpKeyWidth - SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

    do
    {
        innerHeight = ( numLines        * SCALER_GetCurrVal(LIST_LINE_HEIGHT))  /* list */
                    + ((numLines - 1 )  * SCALER_GetCurrVal(LIST_LINE_PADDING))
                    + ( 2               * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) /*version info*/
                    + ( 1               * SCALER_GetCurrVal(LIST_LINE_PADDING))
                    + SCALER_GetCurrVal(UPDATE_PROGRESSBAR_HEIGHT) /* progress bar */
                    + SCALER_GetCurrVal(LIST_LINE_HEIGHT) /* status line */
                    + (2 * SCALER_GetCurrVal(INNER_FRAME_WIDTH)); /* hor. delimiters */

        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWBotHelp (&pnlPars,
                                           SCALER_GetCurrVal(MAIN_WIDTH),
                                           innerHeight,
                                           numHelpLines,
                                           &stackedFrameWidth);

        boxPositionLeft   = stackedFrameWidth
                          + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                          + SCALER_GetCurrVal(MAIN_LEFT_PADDING);

        pnlPars.onFocusSet      = MENU_UpdateOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, (int32_t)MENUID_UPDATE, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle (MENU_UPDATE_ID_TITLE, inst->pnlHdl,
                                      pnlPars.pos.width, stackedFrameWidth,
                                      STR_TITLE_UPDATE, NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        listHeight  = (numLines * SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                    + ((numLines - 1) * SCALER_GetCurrVal(LIST_LINE_PADDING));

        memset (&listPars, 0, sizeof(listPars));

        listPars.getItem         = MENU_UpdateGetListItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_UpdateOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_UPDATE_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            listHeight, inst->pnlHdl,
                                            NULL, NULL, NULL, 0, &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- menu lines -------------------------------------------------*/

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.valuePos.x         = SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        txtPars.valuePos.y         = 0;
        txtPars.valuePos.width     = listPars.pos.width - SCALER_GetCurrVal(MAIN_LEFT_PADDING)  -
                                     SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        txtPars.valuePos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        txtPars.valuePos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        txtPars.pos.x              = 0;
        txtPars.pos.y              = 0;
        txtPars.pos.width          = listPars.pos.width;
        txtPars.pos.height         = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        txtPars.pos.alignment      = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        /* --- Update by file    --------------------------------------------*/

        #ifdef APPL_FILEUPDATE_ENABLED

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_UPDATE_ID_ENTRY_FROM_FILE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        txtPars.pos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_FILEUPDATE_ENABLED */

        /* --- Update via air -----------------------------------------------*/

        #ifdef APPL_OAD_ENABLED

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_UPDATE_ID_ENTRY_OVER_AIR,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        txtPars.pos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_OAD_ENABLED */

        /* --- Revert update ------------------------------------------------*/

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_UPDATE_ID_ENTRY_REVERT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        txtPars.pos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- Save software to USB -----------------------------------------*/

        #ifdef APPL_USB_ENABLED

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_UPDATE_ID_ENTRY_SAVESSU,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        txtPars.pos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* APPL_USB_ENABLED */

        /* --- Version info  ------------------------------------------------*/

        boxPars.pos.x               = listPars.pos.x + txtPars.valuePos.x;
        boxPars.pos.y               = listPars.pos.y +
                                      listPars.pos.height + SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        boxPars.pos.width           = txtPars.valuePos.width;
        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem             = MENU_UpdateGetItem;
        boxPars.getItemArg          = (FAPI_SYS_HandleT)inst;
        boxPars.pInst               = &(inst->boxInst[0]);

        retVal = FGS_BOX_Open (&boxPars, MENU_UPDATE_ID_CURRVERSION,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING);
        boxPars.pInst  = &(inst->boxInst[1]);

        retVal = FGS_BOX_Open (&boxPars, MENU_UPDATE_ID_PREVVERSION,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* --- Horizontal lines -------------------------------------------- */

        inst->itemPos.x         = listPars.pos.x + txtPars.valuePos.x;
        inst->itemPos.y         = listPars.pos.y + txtPars.pos.y;
        inst->itemPos.width     = txtPars.valuePos.width;
        inst->itemPos.height    = 2 * SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                  2 * SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                  SCALER_GetCurrVal(LIST_LINE_PADDING);
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h      = (FAPI_SYS_HandleT)inst;
        member.pos    = &(inst->itemPos);
        member.id     = MENU_UPDATE_ID_REST;
        member.show   = MENU_UpdateDrawRest;

        retVal= FGS_PANEL_AddMember(inst->pnlHdl, &member, &(inst->pItem));
        if ( retVal != FAPI_OK ) break;

        /* ----- Progress bar ---------------------------------------------- */

        inst->progPos.x         = listPars.pos.x + txtPars.valuePos.x;
        inst->progPos.y         = inst->itemPos.y + inst->itemPos.height;
        inst->progPos.width     = inst->itemPos.width;
        inst->progPos.height    = SCALER_GetCurrVal(UPDATE_PROGRESSBAR_HEIGHT);
        inst->progPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h      = (FAPI_SYS_HandleT)inst;
        member.pos    = &(inst->progPos);
        member.id     = MENU_UPDATE_ID_PROGRESS;
        member.show   = MENU_UpdateDrawProgress;

        retVal= FGS_PANEL_AddMember(inst->pnlHdl, &member, &(inst->pProg));
        if ( retVal != FAPI_OK ) break;

        /* ----- Status line ----------------------------------------------- */

        boxPars.pos.x               = inst->progPos.x;
        boxPars.pos.y               = inst->progPos.y +
                                      inst->progPos.height;
        boxPars.pos.width           = inst->progPos.width;
        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem             = MENU_UpdateGetItem;
        boxPars.getItemArg          = (FAPI_SYS_HandleT)inst;
        boxPars.pInst               = &(inst->boxInst[2]);

        retVal = FGS_BOX_Open (&boxPars, MENU_UPDATE_ID_STATUS,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Bottom help lines ------------------------------------------ */
        boxPars.pos.y            = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                   - SCALER_GetCurrVal(HELP_TB_PADDING)
                                   - numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                   - (numHelpLines-1)*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.height       = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.alignment    = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType         = OSD_BOX_BKGR_HELP;
        boxPars.getItem          = MENU_UpdateGetBoxItem;
        boxPars.getItemArg       = (FAPI_SYS_HandleT)inst;

        /* -----1st bottom help line---------------------------------------- */
        /* -----Red button ------------------------------------------------- */
        boxPars.pInst               = &(inst->boxInst[3]);
        boxPars.pos.x               = boxPositionLeft;
        boxPars.pos.width           = bottomHelpKeyWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_UPDATE_REDBUTTON_KEY, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Delete Index String ----------------------------------- */
        boxPars.pInst               = &(inst->boxInst[4]);
        boxPars.pos.x              += SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = bottomHelpItemWidth;

        retVal = FGS_BOX_Open (&boxPars, MENU_UPDATE_REDBUTTON_DESC, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /*----------2nd bottom help line---------------------------------------- */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;
    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }
    return FTRUE;
}


FSTATIC void MENU_UpdateReadBootInfo (MENU_UPDATE_INST_S* inst)
{
    int32_t   retVal = FAPI_OK;
    uint8_t   cnt;

    inst->bootInfoValid = FFALSE;

    for ( cnt = 0; cnt < BOOTER_NUM_SLOTS; cnt++ )
    {
        inst->bootHdrValid[cnt] = FFALSE;
    }

    do
    {
        retVal = SYS_GetBootInfo(&(inst->bootInfo));
        if ( retVal != FAPI_OK ) break;

        for ( cnt = 0; cnt < BOOTER_NUM_SLOTS; cnt++ )
        {
            retVal = SYS_GetBootSlotInfo (cnt, inst->bootHdr + cnt);
            if ( retVal != FAPI_OK ) break;
        }
    }
    while ( 0 );

    if ( retVal == FAPI_OK )
    {
        inst->bootInfoValid = FTRUE;

        /* test for boot header validity */
        for ( cnt = 0; cnt < BOOTER_NUM_SLOTS; cnt++ )
        {
            if (  (inst->bootHdr[cnt].signature == BOOTER_SLOTHDR_SIG) &&
                 ((inst->bootHdr[cnt].status & 0x1) == 0) )
            {
                inst->bootHdrValid[cnt] = FTRUE;
            }
        }
    }
}

FSTATIC int32_t MENU_UpdateDrawRest (FAPI_SYS_HandleT h)
{
    MENU_UPDATE_INST_S*   inst= (MENU_UPDATE_INST_S*)h;

    OSD_DrawSeparatorHorizontal ((uint16_t)inst->pItem->abspos.x,
                                 (uint16_t)inst->pItem->abspos.y,
                                 inst->itemPos.width);

    OSD_DrawSeparatorHorizontal ((uint16_t)inst->pItem->abspos.x,
                                 (uint16_t)inst->pItem->abspos.y +
                                 (uint16_t)SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                 ((uint16_t)2 * (uint16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                                 (uint16_t)SCALER_GetCurrVal(LIST_LINE_PADDING),
                                 inst->itemPos.width);

    return FAPI_OK;
}

FSTATIC int32_t MENU_UpdateDrawProgress (FAPI_SYS_HandleT h)
{
    MENU_UPDATE_INST_S*   inst= (MENU_UPDATE_INST_S*)h;
    OSD_DrawStatusBarHorizontal((uint16_t) inst->pProg->abspos.x,
                                (uint16_t)inst->pProg->abspos.y + (uint16_t)SCALER_GetCurrVal(UPDATE_PROGRESSBAR_PAD),
                                inst->progPos.width,
                                SCALER_GetCurrVal(UPDATE_PROGRESSBAR_HEIGHT) -
                                (2 * SCALER_GetCurrVal(UPDATE_PROGRESSBAR_PAD)),
                                inst->progresscurrent, 1000,
                                OSD_COL_GREEN, OSD_COL_BAR_INACTIVE,
                                OSD_COL_FRAME);
    return FAPI_OK;
}

FSTATIC void MENU_UpdateRevert (void* h)
{
    MENU_UPDATE_INST_S* inst= (MENU_UPDATE_INST_S*)h;
    uint32_t            slotAddr;
    uint8_t             cnt;

    /* both images must be valid when reverting to previous version. */
    for ( cnt = 0; cnt < BOOTER_NUM_SLOTS; cnt++ )
    {
        if ( !(inst->bootHdrValid[cnt]) ) return;
    }

    (void)MAIN_StopPresentation();

    /* Invalidate image of currently running application. */
    slotAddr = (inst->bootInfo.slotIndex == 0) ? inst->bootInfo.slotAddress0
                                               : inst->bootInfo.slotAddress1;

    /* invalidate current boot image */
    if ( UPDATE_InvalidateImage(slotAddr) == FAPI_OK )
    {
        /* Hard-reset the system */
        (void)SYS_Reboot();
    }
}

void MENU_UpdateProgress (const SYS_EVT_UPDATE_S* pData)
{
    MENU_UPDATE_INST_S*   inst      = menuUpdateDat;
    MENU_UPDATE_STATUS_E  newStatus = inst->updateStatus;

    if ( inst->pnlHdl == NULL ) return;

    switch ( pData->status )
    {
        case UPDATE_STAT_DATACOLLECT:
            inst->progresscurrent = (pData->progCurr * 1000) / pData->progMax;

            newStatus = MENU_UPDATE_STAT_GETDATA;

            (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_UPDATE_ID_PROGRESS);
            break;

        case UPDATE_STAT_PROGRAMMING:
            switch ( pData->programStat )
            {
                case UPDATE_PROGRAM_ERASE:
                    inst->progresscurrent = (pData->progCurr * 300) /
                                             pData->progMax;
                    break;

                case UPDATE_PROGRAM_WRITE:
                    inst->progresscurrent = 300 + ((pData->progCurr * 700) /
                                                    pData->progMax);
                    break;

                default:
                    break;
            }

            newStatus = MENU_UPDATE_STAT_PROGRAMMING;

            (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_UPDATE_ID_PROGRESS);
            break;

        case UPDATE_STAT_FINISHED:
            newStatus = MENU_UPDATE_STAT_SUCCESS;
            break;

        case UPDATE_STAT_ABORTED:
            
            (void)UPDATE_Close();

            newStatus = MENU_UPDATE_STAT_ABORTED;

            /* re-enable list */
            (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_UPDATE_ID_LIST);
            break;

        case UPDATE_STAT_ERROR:
            
            #ifdef APPL_OAD_ENABLED
            
            if (inst->type == MENU_UPDATE_TYPE_OAD)
            {
                (void)OAD_Stop();
                (void)MAIN_RestartPresentation();
            }
            
            #endif /* APPL_OAD_ENABLED */
            
            (void)UPDATE_Close();

            newStatus = MENU_UPDATE_STAT_ERROR;

            /* re-enable list */
            (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_UPDATE_ID_LIST);
            break;

        default:
            break;
    }

    if ( inst->updateStatus != newStatus )
    {
        inst->updateStatus = newStatus;

        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_UPDATE_ID_STATUS);

        if ( inst->updateStatus == MENU_UPDATE_STAT_SUCCESS )
        {
            (void)RTOS_Sleep(MENU_UPDATE_REBOOT_DELAY);

            (void)SYS_Reboot(); /* Hard-reset the system */
        }
    }
}

FSTATIC void MENU_UpdateDisableList (MENU_UPDATE_INST_S* inst)
{
    #ifdef APPL_FILEUPDATE_ENABLED
    (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                    MENU_UPDATE_ID_ENTRY_FROM_FILE, FFALSE);
    #endif /* APPL_FILEUPDATE_ENABLED */

    #ifdef APPL_OAD_ENABLED
    (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                    MENU_UPDATE_ID_ENTRY_OVER_AIR, FFALSE);
    #endif /* APPL_OAD_ENABLED */

    (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                    MENU_UPDATE_ID_ENTRY_REVERT, FFALSE);

    #ifdef APPL_USB_ENABLED
    (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                    MENU_UPDATE_ID_ENTRY_SAVESSU, FFALSE);
    #endif /* APPL_USB_ENABLED */

}

FSTATIC void MENU_UpdateEnableList (MENU_UPDATE_INST_S* inst)
{
    uint8_t    cnt;
    fbool_t status;
    
    /* the update related features in this menu is enabled only when
       the boot slot information is available, i.e. the current application
       is requried to be started by the 'booter'. */
    if ( inst->bootInfoValid ) status = FTRUE;
    else                       status = FFALSE;

    #ifdef APPL_FILEUPDATE_ENABLED

    (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                  MENU_UPDATE_ID_ENTRY_FROM_FILE, status);

    #endif /* APPL_FILEUPDATE_ENABLED */

    #ifdef APPL_OAD_ENABLED
    
    #ifdef APPL_USB_ENABLED
    (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                    MENU_UPDATE_ID_ENTRY_SAVESSU, status);
    #endif /* APPL_USB_ENABLED */
    
    /* disable MENU_UPDATE_ID_ENTRY_OVER_AIR in case front-end is not
       supported */
    if ( status && (appDat->numSatFe == 0) )
    {
           status = FFALSE;
    }

    (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                  MENU_UPDATE_ID_ENTRY_OVER_AIR, status);
    #endif /* APPL_OAD_ENABLED */

    status = FTRUE;

    /* both images must be valid when reverting to previous version. */
    for ( cnt = 0; cnt < BOOTER_NUM_SLOTS; cnt++ )
    {
        if ( !(inst->bootHdrValid[cnt]) )
        {
            status = FFALSE;
            break;
        }
    }

    (void)FGS_LIST_SetMemberEnableStatus(inst->listHdl, MENU_UPDATE_ID_ENTRY_REVERT,
                                         status);
}

FSTATIC fbool_t MENU_UpdateSetType (MENU_UPDATE_INST_S* inst,
                                  MENU_UPDATE_TYPE_E type)
{
    if(inst->type == MENU_UPDATE_TYPE_NONE)
    {
        inst->type = type;
        return FTRUE;
    }
    else if( (inst->type != MENU_UPDATE_TYPE_NONE) &&
             (type       != MENU_UPDATE_TYPE_NONE) )
    {
        return FFALSE;
    }

    return FFALSE;
}

#ifdef APPL_FILEUPDATE_ENABLED

FSTATIC fbool_t MENU_UpdateViewerOnBack (void* arg)
{
    MENU_UPDATE_INST_S* inst = (MENU_UPDATE_INST_S*)arg;
    fbool_t             retVal;

    /* close file viewer */
    retVal = MAIN_ViewerClose();

    if ( retVal )
    {
        (void)MENUSTACK_FocusClaim ((int32_t)MENUID_UPDATE);
        (void)FGS_PANEL_Show        (inst->pnlHdl, FFALSE);
    }

    return retVal;
}

FSTATIC fbool_t MENU_UpdateViewerOnSelect (void* arg,
                                           MAIN_VIEWER_FILETYPE type,
                                           const char_t* path,
                                           uint16_t pathLenMax,
                                           const char_t* name,
                                           uint16_t nameLenMax,
                                           uint16_t index, uint16_t numFiles)
{
    MENU_UPDATE_INST_S* inst = (MENU_UPDATE_INST_S*)arg;
    fbool_t             keyUsed = FFALSE;
    uint32_t            version;
    MENU_CONFDIALOG_S   confPars;

    if ( type == MAIN_VIEWER_FILETYPE_FILTERED )
    {
        /* close file viewer */
        (void)MAIN_ViewerClose();

        (void)MENUSTACK_FocusClaim ((int32_t)MENUID_UPDATE);
        (void)FGS_PANEL_Show       (inst->pnlHdl, FFALSE);

        /* store path */
        MENU_CommonAppendpath (name, path, inst->path, FFALSE,
                               MAIN_VIEWER_PATHLEN);

        version = MENU_FILEUPDATE_GetChosenVersion(inst->path);
        if ( version != 0 )
        {
            uint32_t bufPos = 0;

            memset(&confPars, 0, sizeof(MENU_CONFDIALOG_S));

            bufPos = STR_BufAppend(NULL, bufPos,
                                   OSD_GetStr(STR_ENTRY_UPDATE_REPLACE),
                                   MAIN_LINEBUFCHARSET);

            sprintf (appDat->asciiBuf, " %d.%02d ",
                     (int16_t)(inst->bootHdr[inst->bootInfo.slotIndex].version >> 8),
                     (int16_t)(inst->bootHdr[inst->bootInfo.slotIndex].version & 0xFF));
            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend(NULL, bufPos,
                                   OSD_GetStr(STR_ENTRY_UPDATE_BY),
                                   MAIN_LINEBUFCHARSET);

            sprintf (appDat->asciiBuf, " %d.%02d?", (int16_t)(version >> 8),
                                                    (int16_t)(version & 0xFF));
            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);

            confPars.infoStr        = STR_IntLineStr();
            confPars.titleStr       = OSD_GetStr(STR_ENTRY_UPDATE_UPDATE);
            confPars.confirmStr     = OSD_GetStr(STR_ENTRY_VIEWER_DELETE_OK);
            confPars.discardStr     = OSD_GetStr(STR_ENTRY_VIEWER_DELETE_CANCEL);
            confPars.numDescLines   = 1;
            confPars.declineStr.chr = NULL;
            confPars.onConfirm      = MENU_FILEUPDATE_Do;
            confPars.onDecline      = MENU_FILEUPDATE_Decline;
            confPars.onDiscard      = MENU_FILEUPDATE_Decline;
            confPars.arg            = (void*)inst;

            (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
        }

        keyUsed = FTRUE;
    }

    return keyUsed;
}

FSTATIC void MENU_FILEUPDATE_Do (void* h)
{
    MENU_UPDATE_INST_S* inst= (MENU_UPDATE_INST_S*)h;
    F_FILE*             fileP;
    uint32_t            slotAddr;
    int32_t             bytes;
    int32_t             retVal = FAPI_OK;
    SYS_DEVICE_E        device;

    #ifdef APPL_FILEOP_ENABLED
    FILEOP_INFO_S       fileStatus;
    #endif /* APPL_FILEOP_ENABLED */

    (void)MAIN_StopPresentation();

    do
    {
        (void)MENU_ConfDialogClose(&(appDat->confDialog));

        MENU_UpdateDisableList (inst);
        (void)MENU_UpdateSetType     (inst, MENU_UPDATE_TYPE_FILE);

        inst->updateStatus = MENU_UPDATE_STAT_NONE;
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_UPDATE_ID_STATUS);

        retVal = UPDATE_Open();
        if ( retVal != FAPI_OK )
        {
            inst->updateStatus = MENU_UPDATE_STAT_SYSTEMBUSY;
            break;
        }

        #ifdef APPL_FILEOP_ENABLED

        if ( (FILEOP_GetInfo(&fileStatus) == FAPI_OK) && fileStatus.active )
        {
            retVal = FAPI_ERR_DEVICE_BUSY;

            inst->updateStatus = MENU_UPDATE_STAT_SYSTEMBUSY;
            break;
        }

        #endif /* APPL_FILEOP_ENABLED */

        inst->updateStatus = MENU_UPDATE_STAT_GETDATA;
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_UPDATE_ID_STATUS);

        if ( SYS_DeviceGetByPath (inst->path, &device, NULL) == FAPI_OK )
        {
            (void)SYS_DevicePreAccess (device);
        }

        fileP = f_open(inst->path, "r");
        if ( fileP == NULL )
        {
            inst->updateStatus = MENU_UPDATE_STAT_INVALIDDATA;

            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        do
        {
            bytes = f_read(inst->buffer, 1, FILEUPDATE_BUFSIZE, fileP);
            if ( !bytes )
            {
                break;
            }
            else if ( bytes < 0 )
            {
                inst->updateStatus = MENU_UPDATE_STAT_INVALIDDATA;

                retVal = SMARTGO_APPL_ERR_INTERNAL;
                break;
            }

            retVal = UPDATE_AddData (inst->buffer, (uint32_t)bytes);
            if ( retVal != FAPI_OK )
            {
                inst->updateStatus = MENU_UPDATE_STAT_INVALIDDATA;

                break;
            }
        }
        while ( bytes > 0 );

        (void)f_close(fileP);

        if ( retVal != FAPI_OK ) break;

        /* always update image that is not currently active. */
        slotAddr = (inst->bootInfo.slotIndex == 0) ? inst->bootInfo.slotAddress1
                                                   : inst->bootInfo.slotAddress0;

        retVal = UPDATE_ProgramImage (slotAddr);
        if ( retVal != FAPI_OK )
        {
            inst->updateStatus = MENU_UPDATE_STAT_INVALIDDATA;
            break;
        }

        /* Update started -> disable list */
        (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_UPDATE_ID_PROGRESS);
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        (void)UPDATE_Close();

        (void)MAIN_RestartPresentation();

        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_UPDATE_ID_STATUS);

        MENU_UpdateEnableList (inst);
        (void)MENU_UpdateSetType    (inst, MENU_UPDATE_TYPE_NONE);
    }

    inst->path[0]= 0;
}


FSTATIC void MENU_FILEUPDATE_Decline (void* h)
{
    MENU_UPDATE_INST_S*     inst= (MENU_UPDATE_INST_S*)h;

    inst->path[0]= 0;
}

FSTATIC uint32_t MENU_FILEUPDATE_GetChosenVersion (char_t* path)
{
    F_FILE*             fileP;
    BOOTER_FlashSlotHeaderT  header;
    SYS_DEVICE_E        device;

    if ( !(MENU_FILEUPDATE_FileIsValid(path)) ) return 0;

    if ( SYS_DeviceGetByPath (path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    fileP = f_open(path, "r");
    if( fileP == NULL ) return 0;

    memset(&header, 0, sizeof(header));

    (void)f_read(&header, 1, sizeof(BOOTER_FlashSlotHeaderT), fileP);
    (void)f_close(fileP);

    if(header.signature != BOOTER_SLOTHDR_SIG)
        return 0;

    return header.version;
}

FSTATIC FINLINE fbool_t MENU_FILEUPDATE_FileIsValid (const char_t* fileName)
{
    uint16_t n;

    for ( n = 0; n < MAIN_VIEWER_PATHLEN; n++ )
    {
        if ( fileName[n] == 0 ) break;
    }

    if ( (n > 4)                &&
         (fileName[n-3] == 'S') &&
         (fileName[n-2] == 'S') &&
         (fileName[n-1] == 'U') )
    {
        return FTRUE;
    }
    else
    {
        return FFALSE;
    }
}

#endif /* APPL_FILEUPDATE_ENABLED */

#ifdef APPL_OAD_ENABLED

FSTATIC void MENU_OADUPDATE_UpdateInfo (SYS_NETWORK_S* pnetwork,
                                      TPLIST_ITEM_S* pcarrierInf)
{
    MENU_UPDATE_INST_S*     inst = menuUpdateDat;

    memcpy(&(inst->network),pnetwork, sizeof(SYS_NETWORK_S));
    memcpy(&(inst->carrierInf),pcarrierInf, sizeof(TPLIST_ITEM_S));

    inst->oadProgressRq = FTRUE;

}

FSTATIC void MENU_UPDATE_OadStart (void* h)
{
    MENU_UPDATE_INST_S* inst= (MENU_UPDATE_INST_S*)h;
    uint32_t            slotAddr;

    (void)MAIN_StopPresentation();

    MENU_UpdateDisableList (inst);
    (void)MENU_UpdateSetType     (inst, MENU_UPDATE_TYPE_OAD);

    inst->updateStatus = MENU_UPDATE_STAT_GETDATA;
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_UPDATE_ID_STATUS);

    /* always update image that is not currently active. */
    slotAddr = (inst->bootInfo.slotIndex == 0) ? inst->bootInfo.slotAddress1
                                               : inst->bootInfo.slotAddress0;

    (void)OAD_Start (&(inst->network), &(inst->carrierInf), NULL, slotAddr);
}

#endif /* APPL_OAD_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
