/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_viewer.c
**
**              $Id: app_menu_viewer.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       File viewer menu.
**
** Detail description. [\ref app_smartgo_app_ref_viewer]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info 713: Loss of precision (assignment) (unsigned int to short).
** - Info 715: Symbol not referenced.
** - Info 732: Loss of sign (assignment) (short to unsigned int).
** - Info 737: Loss of sign in promotion from int to unsigned int.
**
** - Note 961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on C's
**   operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include "app_menu.h"
#include "app_menu_viewer.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define MAIN_VIEWER_DISPITEMS         11

#define MAIN_VIEWER_INFOLEN_MAX       128
#define MAIN_VIEWER_PARTLEN_MAX       16    // maximum length of partition path
#define MAIN_VIEWER_FOLDERLVL_MAX     20

#define MAIN_VIEWER_FILE_EXIST        F_NO_ERROR

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MAIN_VIEWER_ID_LEFTTITLE = 0,
    MAIN_VIEWER_ID_RIGHTTITLE,
    MAIN_VIEWER_ID_LEFT,
    MAIN_VIEWER_ID_RIGHT,
    MAIN_VIEWER_ID_TITLE,
    MAIN_VIEWER_ID_REST,
    MAIN_VIEWER_ID_HELP1,
    MAIN_VIEWER_ID_HELP3,
    MAIN_VIEWER_ID_HELP4,
    MAIN_VIEWER_ID_HELPCOLOR,
    MAIN_VIEWER_ID_INFO1,
    MAIN_VIEWER_ID_INFO2,
    MAIN_VIEWER_ID_INFO3,
    MAIN_VIEWER_ID_PREVIEW,
    MAIN_VIEWER_ID_CONTEXT,
    MAIN_VIEWER_ID_ACTION,
    MAIN_VIEWER_ID_PROGRESS
};

enum
{
    MAIN_VIEWER_ID_CONTEXT_NEW_FOLDER = 0,
    MAIN_VIEWER_ID_CONTEXT_COPY,
    MAIN_VIEWER_ID_CONTEXT_MOVE,
    MAIN_VIEWER_ID_CONTEXT_RENAME,
    MAIN_VIEWER_ID_CONTEXT_DELETE
};

enum
{
    MAIN_VIEWER_OP_NORMAL = 0,
    MAIN_VIEWER_OP_MOVE_SELECTED,
    MAIN_VIEWER_OP_COPY_SELECTED,
    MAIN_VIEWER_OP_MOVE_PROGRESS,
    MAIN_VIEWER_OP_COPY_PROGRESS,
    MAIN_VIEWER_OP_DELETE_PROGRESS,
    MAIN_VIEWER_OP_RENAME
};

typedef struct
{
    SCALER_PIC_ID_E     icon;
    fbool_t             isDir;          //!< FTRUE: Directory, FFALSE: File
    fbool_t             selectionMatch; //!< FTRUE: Matches user-defined
                                        //!< selection pattern.
    fbool_t             enabled;        //!< FFALSE: Shown in 'disabled' color.
    SYS_SIZE_S          size;
    char_t              name[MAIN_VIEWER_NAMELEN];

} MAIN_VIEWER_ITEM;

typedef struct
{
    SYS_DEVICE_E        device;
    SCALER_PIC_ID_E     icon;
    char_t              name[MAIN_VIEWER_NAMELEN];
    char_t              path[MAIN_VIEWER_PARTLEN_MAX];
    SYS_DEV_FS_E        status;

} MAIN_VIEWER_PARTITION;

typedef struct
{
    uint16_t            currentitem;
    uint16_t            currenttop;

} MAIN_VIEWER_LEVEL;

typedef struct
{
    FAPI_SYS_HandleT      pnlHdl;
    FGS_PANEL_INST_S      pnlInst;

    FAPI_SYS_HandleT      leftListHdl;
    FGS_LNAVI_INST_S      leftLnaviInst;

    FAPI_SYS_HandleT      rightListHdl;
    FGS_LNAVI_INST_S      rightLnaviInst;

    FGS_PANEL_MEMBER_S*   pItemBorder;
    FGS_POS_S             itemBorderPos;

    FGS_PANEL_MEMBER_S*   progressBorderP;
    FGS_POS_S             progressPos;

    uint32_t              maxprogress;
    uint32_t              currentprogress;

    uint16_t              currentdepth;
    MAIN_VIEWER_LEVEL     level[MAIN_VIEWER_FOLDERLVL_MAX];

    uint16_t              previewSizeX;
    uint16_t              previewSizeY;

    MAIN_VIEWER_ITEM      leftitem  [MAIN_VIEWER_DISPITEMS];
    MAIN_VIEWER_ITEM      rightitem [MAIN_VIEWER_DISPITEMS];
    MAIN_VIEWER_PARTITION partitions[MAIN_VIEWER_DISPITEMS];
    uint16_t              partitioncount;
    int16_t               leftbaseitem;

    SCALER_PIC_ID_E       currenticon;

    char_t                path    [MAIN_VIEWER_PATHLEN];
    char_t                copypath[MAIN_VIEWER_PATHLEN];
    SCALER_PIC_ID_E       copyIconType;

    uint16_t              operation;
    uint16_t              renamepos;
    char_t                renamestr[MAIN_VIEWER_NAMELEN];

    MAIN_VIEWER_CONFIG_S  params;

    SYS_STRBUF_S          infoLine1;
    SYS_STRBUF_S          infoLine2;
    SYS_STRBUF_S          infoLine3;

    fbool_t               jpegActive;
    fbool_t               devAccessed[SYS_DEV_NUMS];

} MAIN_VIEWER_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC MAIN_VIEWER_INST_S* viewerDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void     MAIN_ViewerOnFocusSet       (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC fbool_t  MAIN_ViewerKeys             (FAPI_SYS_HandleT h);
FSTATIC fbool_t  MAIN_ViewerBuild            (FAPI_SYS_HandleT h);
FSTATIC int32_t  MAIN_ViewerPaintRest        (FAPI_SYS_HandleT h);
FSTATIC int32_t  MAIN_ViewerPaintProgress    (FAPI_SYS_HandleT h);
FSTATIC void     MAIN_ViewerConfirmDelete    (MAIN_VIEWER_INST_S* inst);
FSTATIC void     MAIN_ViewerDeleteOK         (void* arg);
FSTATIC fbool_t  MAIN_ViewerFocusChange      (FAPI_SYS_HandleT h, int32_t itemId);
FSTATIC void     MAIN_ViewerGetItem          (FAPI_SYS_HandleT h, int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC fbool_t  MAIN_ViewerSelect           (FAPI_SYS_HandleT h, int32_t id, uint16_t index);
FSTATIC fbool_t  MAIN_ViewerFolderBack       (MAIN_VIEWER_INST_S* inst);
FSTATIC fbool_t  MAIN_ViewerFolderDown       (MAIN_VIEWER_INST_S* inst,
                                              const MAIN_VIEWER_ITEM* pCurrItem,
                                              const FGS_LNAVI_PARAMS_S*
                                              pCurrNaviPos);
FSTATIC fbool_t  MAIN_ViewerContextSelect    (FAPI_SYS_HandleT h, int32_t id, uint16_t index);
FSTATIC void     MAIN_ViewerContextGetItem   (FAPI_SYS_HandleT h, int32_t id, uint32_t index, FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC char_t   MAIN_ViewerNextLetter       (char_t letter, fbool_t up);
FSTATIC void     MAIN_ViewerRename           (MAIN_VIEWER_INST_S* inst, uint16_t pos);
FSTATIC void     MAIN_ViewerValidate         (MAIN_VIEWER_INST_S* inst,
                                              char_t* initialPath);
FSTATIC void     MAIN_ViewerPrepareName      (char_t* sourcepath, char_t* destpath,
                                              char_t* resultpath);
FSTATIC void     MAIN_ViewerUpdate           (MAIN_VIEWER_INST_S* inst);
FSTATIC void     MAIN_ViewerUpdateRight      (MAIN_VIEWER_INST_S* inst, uint16_t newleft);
FSTATIC void     MAIN_ViewerCopyName         (const char_t* path, char_t* name,
                                              uint16_t nameBufSize);
FSTATIC void     MAIN_ViewerEditStringExit   (MENU_EDITSTRING_DATA_S* editData,
                                              int32_t id);
FSTATIC fbool_t  MAIN_ViewerCheckFileType    (uint32_t* filterType,
                                              uint32_t type);
FSTATIC uint16_t MAIN_ViewerCheckPartitions  (MAIN_VIEWER_INST_S* inst,
                                              MAIN_VIEWER_PARTITION*
                                              partitionP);
FSTATIC void     MAIN_ViewerCopyVolumes      (MAIN_VIEWER_PARTITION* partitionP,
                                              uint16_t count,
                                              MAIN_VIEWER_ITEM* itemP);
FSTATIC uint16_t MAIN_ViewerCheckContent     (char_t* path,
                                              MAIN_VIEWER_ITEM* itemP,
                                              uint16_t base,
                                              uint32_t* filterType);
FSTATIC void     MAIN_ViewerSwitchInfo       (MAIN_VIEWER_INST_S * inst,
                                              fbool_t info);

FSTATIC void     MAIN_ViewerConfirmOverwrite (MAIN_VIEWER_INST_S * inst);
FSTATIC void     MAIN_ViewerOverwriteOK      (void* arg);

#ifdef APPL_PVR_ENABLED
FSTATIC int32_t  MAIN_ViewerResyncPvrList    (SYS_DEVICE_E device);
#endif /* APPL_PVR_ENABLED */

/* ------------------------------------------------------------------------- */

FSTATIC fbool_t MENU_ViewerLeave    (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC fbool_t MENU_ViewerOnBack   (void* arg);
FSTATIC fbool_t MENU_ViewerOnExit   (void* arg);
FSTATIC fbool_t MENU_ViewerOnSelect (void* arg,
                                     MAIN_VIEWER_FILETYPE type,
                                     const char_t* path,
                                     uint16_t pathLenMax,
                                     const char_t* name,
                                     uint16_t nameLenMax,
                                     uint16_t index, uint16_t numItems);

int32_t MENU_ViewerInit  (void)
{
    /* currently nothing to be done! */

    return FAPI_OK;
}

fbool_t MENU_ViewerEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MAIN_VIEWER_CONFIG_S    viewerCfg;
    fbool_t                 retVal;
    MAIN_VIEWER_CONFIG_S*   pParams = NULL;

    if ( test == FTRUE ) return FTRUE;

    #ifdef APPL_MHEG_ENABLED
    MHEG_Enable (FFALSE, SYS_SUSPENDSRC_USER);
    #endif
    if ( h != NULL )
    {
        memset (&viewerCfg, 0, sizeof(viewerCfg));

        viewerCfg.titleString       = OSD_GetStr(STR_TITLE_VIEWER);
        viewerCfg.allowStackedView  = FTRUE;
        viewerCfg.allowedFileOps    = MAIN_VIEWER_FILEOP_ALL;
        viewerCfg.allowedPicOps     = MAIN_VIEWER_OP_ALL;
        viewerCfg.allowedMusicOps   = MAIN_VIEWER_OP_ALL;
        viewerCfg.allowedMovieOps   = MAIN_VIEWER_OP_ALL;
        viewerCfg.initialPath[0]    = 0;
        viewerCfg.filterType[0]     = MAIN_VIEWER_FILTYPE_ALL;
        viewerCfg.onBack            = MENU_ViewerOnBack;
        viewerCfg.onExit            = MENU_ViewerOnExit;
        viewerCfg.onSelect          = MENU_ViewerOnSelect;

        pParams = &viewerCfg;
    }

    retVal = MAIN_ViewerOpen (pParams);

    if ( retVal )
    {
        MENUSTACK_SetCleanup (MENU_ViewerLeave, NULL);
    }

    return retVal;
}

FSTATIC fbool_t MENU_ViewerLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    (void)MAIN_ViewerClose ();

    return FTRUE;
}

FSTATIC fbool_t MENU_ViewerOnBack (void* arg)
{
    return MENUSTACK_Back (NULL);
}

FSTATIC fbool_t MENU_ViewerOnExit (void* arg)
{
    return MENUSTACK_Exit (NULL);
}

FSTATIC fbool_t MENU_ViewerOnSelect (void* arg,
                                     MAIN_VIEWER_FILETYPE type,
                                     const char_t* path,
                                     uint16_t pathLenMax,
                                     const char_t* name,
                                     uint16_t nameLenMax,
                                     uint16_t index, uint16_t numItems)
{
    fbool_t             keyUsed = FFALSE;

    #ifdef APPL_MP3_ENABLED
    MENU_MP3_PARAMS_S   mp3Params;
    #endif /* APPL_MP3_ENABLED */

    #ifdef APPL_PVR_ENABLED
    char_t              fullPath[MAIN_VIEWER_PATHLEN] = {0};
    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_JPEG_ENABLED
    MENU_JPEG_PARAMS_S  jpegParams;
    #endif /* APPL_JPEG_ENABLED */

    switch ( type )
    {
        #ifdef APPL_JPEG_ENABLED

        case MAIN_VIEWER_FILETYPE_PICTURE:
            jpegParams.path     = path;
            jpegParams.filename = name;

            keyUsed = MENUSTACK_Up (MENU_JPEGEntry,
                                   (FAPI_SYS_HandleT)&jpegParams);
            break;

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        case MAIN_VIEWER_FILETYPE_MUSIC:
            mp3Params.path     = path;
            mp3Params.filename = name;

            keyUsed = MENUSTACK_Up (MENU_MP3Entry,
                                   (FAPI_SYS_HandleT)&mp3Params);
            break;

        #endif /* APPL_MP3_ENABLED */

        #ifdef APPL_PVR_ENABLED

        case MAIN_VIEWER_FILETYPE_MOVIE:
            MENU_CommonAppendpath (name, path, fullPath, FFALSE,
                                   MAIN_VIEWER_PATHLEN);

            if ( PVRSYS_PlayStartByPath (fullPath, 0, NULL, NULL) != FAPI_OK )
            {
                keyUsed = FFALSE;
            }
            else
            {
                keyUsed = MENUSTACK_Exit (NULL);
            }
            break;

        #endif /* APPL_PVR_ENABLED */

        default:
            /* do nothing */
            break;
    }

    return keyUsed;
}

/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************
**
** \brief  Initial function for the OSD Service list Menu
**
** \return FAPI_OK
**
******************************************************************************/
int32_t MAIN_ViewerInit (void)
{
    MAIN_VIEWER_INST_S* inst;
    int32_t             retVal;

    if ( viewerDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    viewerDat = UTILS_Malloc(sizeof(MAIN_VIEWER_INST_S));
    if ( viewerDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset (viewerDat, 0, sizeof(MAIN_VIEWER_INST_S));

    inst = viewerDat;

    retVal = STR_BufOpen (&(inst->infoLine1), MAIN_VIEWER_INFOLEN_MAX, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = STR_BufOpen (&(inst->infoLine2), MAIN_VIEWER_INFOLEN_MAX, NULL);
    if ( retVal != FAPI_OK ) return retVal;
    retVal = STR_BufOpen (&(inst->infoLine3), MAIN_VIEWER_INFOLEN_MAX, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    inst->jpegActive = FFALSE;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Entry function.
**
** \param params      TODO
**
** \return FTRUE
**
******************************************************************************/
fbool_t MAIN_ViewerOpen (MAIN_VIEWER_CONFIG_S* params)
{
    MAIN_VIEWER_INST_S* inst = viewerDat;
    uint16_t            cnt;

    #ifdef APPL_HDD_ENABLED
    fbool_t             msgShow = FFALSE;
    int32_t             retVal;
    SYS_DEV_PWRMODE_E   hddPwrState;

    retVal = SYS_DeviceGetPowerMode (SYS_DEV_HDD, &hddPwrState);
    if ( (retVal      == FAPI_OK )
      && (hddPwrState == SYS_DEV_PWRMODE_STANDBY) )
    {
        msgShow = FTRUE;
        MENU_CommonMsgShow (STR_MSG_HDD_SPINUP,FTRUE);
    }

    #endif /* APPL_HDD_ENABLED */

    /* reset access status of all potential devices */
    for ( cnt = 0; cnt < SYS_DEV_NUMS; cnt++ )
    {
        inst->devAccessed[cnt] = FFALSE;
    }

    if ( !(MAIN_ViewerBuild((FAPI_SYS_HandleT)inst)) )
    {
        return FFALSE;
    }

    inst->partitioncount = MAIN_ViewerCheckPartitions (inst, inst->partitions);
    inst->jpegActive     = FFALSE;

    /* NOTE: params are set to NULL if viewer is entered from a higher-level
             menu. In this case, previous settings are kept! */

    if ( params != NULL )
    {
        inst->params = *params;

        for ( cnt = 0; cnt < MAIN_VIEWER_FOLDERLVL_MAX; cnt++ )
        {
            inst->level[cnt].currentitem = 0;
            inst->level[cnt].currenttop  = 0;
        }

        inst->currentdepth = 0;
        inst->path[0]      = 0;

        MAIN_ViewerValidate (inst, inst->params.initialPath);
    }

    MAIN_ViewerUpdate(inst);

    #ifdef APPL_HDD_ENABLED

    if ( msgShow )
    {
        (void)MAIN_MsgBoxHide(&(appDat->msgBox));
    }

    #endif /* APPL_HDD_ENABLED */

    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MAIN_VIEWER_ID_LEFT);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_VIEWER);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);

    /* update left and especially right window because now all index params are valid */
    MAIN_ViewerUpdate(inst);

    return FTRUE;
}

int32_t MAIN_ViewerGetFile (const char_t* path, uint16_t index,
                            MAIN_VIEWER_GETFILE_E mode,
                            fbool_t (*fileMatch)(void* arg, const char_t* name,
                                                 uint16_t nameLenMax),
                            void* fileMatchArg,
                            uint16_t* pNewIndex,
                            char_t* fileNameBuf,
                            uint16_t fileNameLength)
{
    uint16_t     currIndex = 0;
    F_FIND       find;
    char_t       fileNameRef[F_MAXPATH];
    uint32_t     strPos;
    SYS_DEVICE_E device;
    char_t       searchPath[MAIN_VIEWER_PATHLEN + 3];
    int32_t      status;
    fbool_t      currMatch;
    fbool_t      abort    = FFALSE;
    int32_t      retVal   = SMARTGO_APPL_ERR_DATA_NOT_AVL;
    int32_t      refIndex = -1;

    fileNameRef[0] = 0;

    strPos = UTILS_strncpy (searchPath, path, MAIN_VIEWER_PATHLEN);

    /* append wildcard characters */
    (void)UTILS_strncpy (searchPath + strPos, "*.*",
                         MAIN_VIEWER_PATHLEN + 3 - strPos);

    if ( SYS_DeviceGetByPath (path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    status = f_findfirst(searchPath, &find);

    while ( status == F_NO_ERROR )
    {
        if ( (find.filename[0] != 0)
          && (find.filename[0] != '.')
          && ((find.attr & F_ATTR_HIDDEN) == 0) )
        {
            currMatch = FTRUE;

            if ( fileMatch != NULL )
            {
                currMatch = fileMatch (fileMatchArg, find.filename, F_MAXPATH);
            }

            if ( (mode      == MAIN_VIEWER_GETFILE_THIS)
              && (currIndex == index) )
            {
                if ( currMatch )
                {
                    (void)UTILS_strncpy (fileNameRef, find.filename, F_MAXPATH);

                    refIndex = currIndex;
                }

                break;
            }

            if ( currMatch )
            {
                switch ( mode )
                {
                    case MAIN_VIEWER_GETFILE_FIRST:
                        (void)UTILS_strncpy (fileNameRef, find.filename,
                                             F_MAXPATH);
                        refIndex = currIndex;
                        abort    = FTRUE;
                        break;

                    case MAIN_VIEWER_GETFILE_LAST:
                        (void)UTILS_strncpy (fileNameRef, find.filename,
                                             F_MAXPATH);
                        refIndex = currIndex;
                        break;

                    case MAIN_VIEWER_GETFILE_NEXT:
                        if ( currIndex > index )
                        {
                            (void)UTILS_strncpy (fileNameRef, find.filename,
                                                 F_MAXPATH);
                            refIndex = currIndex;
                            abort    = FTRUE;
                        }
                        break;

                    case MAIN_VIEWER_GETFILE_PREVIOUS:
                        if ( currIndex < index )
                        {
                            (void)UTILS_strncpy (fileNameRef, find.filename,
                                                 F_MAXPATH);
                            refIndex = currIndex;
                        }
                        else
                        {
                            abort = FTRUE;
                        }
                        break;

                    default:
                        /* do nothing */
                        break;
                }

                if ( abort )
                {
                    break;
                }
            }

            currIndex++;
        }

        status = f_findnext(&find);
    }

    if ( refIndex >= 0 )
    {
        if ( pNewIndex != NULL )
        {
            *pNewIndex = refIndex;
        }

        if ( fileNameBuf != NULL )
        {
            (void)UTILS_strncpy (fileNameBuf, fileNameRef, fileNameLength);
        }

        retVal = FAPI_OK;
    }

    return retVal;
}

void MAIN_ViewerHandleEvent (SYS_EVTTYPE_E type, const void* data)
{
    MAIN_VIEWER_INST_S* inst = viewerDat;
    uint32_t            bufPos;

    #ifdef APPL_JPEG_ENABLED
    FGS_LNAVI_PARAMS_S  navipar;
    #endif /* APPL_JPEG_ENABLED */

    if ( inst->pnlHdl == NULL ) return;

    #ifdef APPL_JPEG_ENABLED

    switch ( type )
    {
        case EVT_JPEG_FILEINFO:

            switch ( inst->currenticon )
            {
                case ICON_PICTURESMALL:

                    /* line 1: dimensions */

                    bufPos = STR_BufAppend(&(inst->infoLine1), 0,
                                          OSD_GetStr(STR_ENTRY_VIEWER_JPEG_DIM),
                                          MAIN_LINEBUFCHARSET);

                    sprintf (appDat->asciiBuf, " %d x %d",
                            ((JPEG_EVT_INFO_S*)data)->sizeX,
                            ((JPEG_EVT_INFO_S*)data)->sizeY);

                    (void)STR_BufAppend(&(inst->infoLine1), bufPos,
                                  STR_IntASCIIStr(),
                                  MAIN_LINEBUFCHARSET);

                    (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                                   MAIN_VIEWER_ID_INFO1);

                    /* line 2: file size */

                    (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);

                    bufPos = STR_BufAppend(&(inst->infoLine2), 0,
                                          OSD_GetStr(STR_ENTRY_VIEWER_JPEG_SIZE),
                                          MAIN_LINEBUFCHARSET);

                    sprintf (appDat->asciiBuf, " %ldk",
                             inst->leftitem[navipar.index - inst->leftbaseitem].
                             size.kbytes);

                    (void)STR_BufAppend(&(inst->infoLine2), bufPos,
                                  STR_IntASCIIStr(),
                                  MAIN_LINEBUFCHARSET);

                    (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                                   MAIN_VIEWER_ID_INFO2);

                    /* line 3: viewer information */

                    if ( ((JPEG_EVT_INFO_S*)data)->fileType == PIC_TYPE_BMP )
                    {
                        if ((((JPEG_EVT_INFO_S*)data)->sizeX >OSDHANDLER_MAX_HRES)||
                            (((JPEG_EVT_INFO_S*)data)->sizeY > OSDHANDLER_MAX_VRES))
                        {
                            bufPos = STR_BufAppend(&(inst->infoLine3), 0,
                                          OSD_GetStr(STR_ENTRY_VIEWER_INVALID_DIMENSION),
                                          MAIN_LINEBUFCHARSET);
                            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                                           MAIN_VIEWER_ID_INFO3);
                        }
                        if (((JPEG_EVT_INFO_S*)data)->bitsperpix!=24)
                        {
                            bufPos = STR_BufAppend(&(inst->infoLine3), 0,
                                          OSD_GetStr(STR_ENTRY_VIEWER_INVALID_FORMAT),
                                          MAIN_LINEBUFCHARSET);
                            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                                           MAIN_VIEWER_ID_INFO3);
                        }
                    }

                    (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MAIN_VIEWER_ID_PREVIEW, FFALSE);
                    break;

                #ifdef APPL_MP3_ENABLED

                case ICON_MUSICSMALL:
                    (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                MAIN_VIEWER_ID_PREVIEW, FFALSE);
                    break;

                #endif /* APPL_MP3_ENABLED */

                default:
                    break;
            }
            break;
        case EVT_JPEG_OSDINFO:
            if (((JPEG_EVT_OSDINFO_S*)data)->osdIsReady)
            {
                (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                    MAIN_VIEWER_ID_PREVIEW,
                                                    FFALSE);
            }
            else
            {
                (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl,
                                                    MAIN_VIEWER_ID_PREVIEW,
                                                    FTRUE);
            }
            break;
        case EVT_JPEG_DECODEINFO:
            if (((JPEG_EVT_DECODEINFO_S*)data)->infoType != JPEG_DECODEINFO_OK)
            {
                MENU_CommonMsgShow(STR_MSG_PIC_NOT_SUPPORT, FFALSE);
            }
            break;
        default:
            break;
    }

    #endif /* APPL_JPEG_ENABLED */
}

/*!
*******************************************************************************
**
** \brief  This function is call when the menu gets/lose the panel focus.
**
** \param h        Address of menu handle.
** \param hasFocus If the menu has focus or not.
**
******************************************************************************/
FSTATIC void MAIN_ViewerOnFocusSet(FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    //MAIN_VIEWER_INST_S*     inst= (MAIN_VIEWER_INST_S*)h;

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_MENU,   MAIN_ViewerKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BACK,   MAIN_ViewerKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_EXIT,   MAIN_ViewerKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_YELLOW, MAIN_ViewerKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_GREEN,  MAIN_ViewerKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RED,    MAIN_ViewerKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_UP,     MAIN_ViewerKeys, h, FFALSE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_DOWN,   MAIN_ViewerKeys, h, FFALSE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_LEFT,   MAIN_ViewerKeys, h, FFALSE, FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_RIGHT,  MAIN_ViewerKeys, h, FFALSE, FTRUE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler(FGS_KEY_YELLOW);
        FGS_ClearAppKeyHandler(FGS_KEY_GREEN);
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_UP);
        FGS_ClearAppKeyHandler(FGS_KEY_DOWN);
        FGS_ClearAppKeyHandler(FGS_KEY_LEFT);
        FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
    }
}

FSTATIC fbool_t MAIN_ViewerClose (void)
{
    MAIN_VIEWER_INST_S* inst = viewerDat;
    SYS_DEVICE_E        device;

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_SetMemberHideStatus (inst->pnlHdl,
                                             MAIN_VIEWER_ID_PREVIEW,
                                             FTRUE);

        /* hide the unformatted message if necessary */
        (void)MAIN_MsgBoxHide(&(appDat->msgBox));

        #ifdef APPL_JPEG_ENABLED

        if ( inst->jpegActive )
        {
            (void)JPEG_Close();

            inst->jpegActive = FFALSE;
        }

        #endif /* APPL_JPEG_ENABLED */

        (void)FGS_PANEL_Close(inst->pnlHdl);

        inst->pnlHdl = NULL;

        (void)MENUSTACK_FocusRelease((int32_t)MENUID_VIEWER);

        /* release access status of all potential devices */
        for ( device = (SYS_DEVICE_E)0; device < SYS_DEV_NUMS; device++ )
        {
            if ( inst->devAccessed[device] )
            {
                (void)SYS_DeviceSetAccessStatus (device, FFALSE);
            }
        }
    }

    return FTRUE;
}


FSTATIC void MAIN_ViewerGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                 FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MAIN_VIEWER_INST_S* inst = (MAIN_VIEWER_INST_S*)h;
    uint16_t            y1, x1;
    uint32_t            prepos, lettersize;
    FGS_LNAVI_PARAMS_S  navipar;
    uint32_t            bufPos = 0;
    uint32_t            strIdx;

    switch ( id )
    {
        case MAIN_VIEWER_ID_TITLE:
            item[0].type                = FGS_ITEM_TYPE_STRING;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.str.font       = TITLE_FONT;
            item[0].item.str.col        = OSD_COL_TITLE_TEXT;
            item[0].item.str.lineHeight = 0;

            switch(inst->operation)
            {
                case MAIN_VIEWER_OP_MOVE_SELECTED:
                case MAIN_VIEWER_OP_COPY_SELECTED:
                    item[0].item.str.chr= OSD_GetStr(STR_ENTRY_VIEWER_DESTINATION);
                    break;

                default:
                    item[0].item.str.chr = inst->params.titleString;
                    break;
            }
            break;

        case MAIN_VIEWER_ID_ACTION:
            item[0].type                = FGS_ITEM_TYPE_STRING;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.str.font       = MAIN_FONT;
            item[0].item.str.col        = OSD_COL_TITLE_TEXT;
            item[0].item.str.lineHeight = 0;

            switch(inst->operation)
            {
                case MAIN_VIEWER_OP_MOVE_PROGRESS:
                    strIdx = STR_ENTRY_VIEWER_MOVE;
                    break;
                case MAIN_VIEWER_OP_COPY_PROGRESS:
                    strIdx = STR_ENTRY_VIEWER_COPY;
                    break;
                case MAIN_VIEWER_OP_DELETE_PROGRESS:
                    strIdx = STR_ENTRY_VIEWER_DELETE;
                    break;
                default:
                    strIdx = STR_EMPTYSTRING;
                    break;
            }

            if ( strIdx == STR_EMPTYSTRING )
            {
                (void)STR_BufClear(NULL);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(strIdx),
                                         MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_GNR_COLON),
                                         MAIN_LINEBUFCHARSET);
            }

            item[0].item.str.chr = STR_IntLineStr();
            break;

        case MAIN_VIEWER_ID_LEFT:
            if ( (((int16_t)index) <   inst->leftbaseitem) ||
                 (((int16_t)index) >= (inst->leftbaseitem
                                     + MAIN_VIEWER_DISPITEMS)) )
            {
                if ( FGS_LNAVI_GetParams(inst->leftListHdl, &navipar) == FAPI_OK )
                {
                    inst->leftbaseitem = navipar.index - navipar.posInVis;

                    if ( inst->currentdepth == 0 )
                    {
                        MAIN_ViewerCopyVolumes (inst->partitions,
                                                inst->partitioncount,
                                                inst->leftitem);
                    }
                    else
                    {
                        (void)MAIN_ViewerCheckContent (inst->path,
                                                       inst->leftitem,
                                                       inst->leftbaseitem,
                                                       inst->params.
                                                       filterType);
                    }
                }
            }

            /* ----- icon -------------------------------------------------- */

            item[0].type                = FGS_ITEM_TYPE_ICON;
            item[0].align               = FGS_ALIGN_HOR_LEFT
                                        | FGS_ALIGN_VER_CENTER;
            item[0].item.icon.drawMode  = FGS_ICONMODE_OVRLAY;
            item[0].item.icon.bkgrCol   = OSD_COL_BLANK;
            item[0].item.icon.altBkgrCol= OSD_COL_BLANK;
            item[0].item.icon.addr      = OSD_IconGet(inst->leftitem[
                                          index-inst->leftbaseitem].icon);

            /* ----- file name --------------------------------------------- */

            if ( !(inst->leftitem[index-inst->leftbaseitem].enabled) )
            {
                item[1].item.str.col    = appDat->txtAttr.colDisabled;
            }
            else if ( state == FGS_FOCUS )
            {
                item[1].item.str.col    = appDat->txtAttr.colFocus;
            }
            else
            {
                item[1].item.str.col    = appDat->txtAttr.colNonFocus;
            }

            item[1].type                = FGS_ITEM_TYPE_STRING;
            item[1].align               = FGS_ALIGN_HOR_LEFT
                                        | FGS_ALIGN_VER_CENTER;
            item[1].item.str.font       = MAIN_FONT;
            item[1].item.str.lineHeight = 0;

            if ( (inst->operation == MAIN_VIEWER_OP_RENAME)
              && (state == FGS_FOCUS) )
            {
                item[1].item.str.chr = OSD_GetStr(STR_EMPTYSTRING);

                (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);

                prepos = OSD_GetStrWidth (STR_GetASCII(inst->renamestr),
                                          inst->renamepos,
                                          MAIN_FONT);
                lettersize = OSD_GetStrWidth (STR_GetASCII(inst->renamestr +
                                              inst->renamepos), 1,
                                              MAIN_FONT);

                y1 = inst->leftLnaviInst.pMem->abspos.y
                   + (navipar.posInVis * inst->leftLnaviInst.data.rowHeight);
                x1 = inst->leftLnaviInst.pMem->abspos.x
                   + inst->leftLnaviInst.data.colStart[1];

                OSD_DrawFilledRectangle (x1 + prepos, y1,
                                         x1 + prepos + lettersize,
                                         y1 + inst->leftLnaviInst.data.
                                              rowHeight,
                                         OSD_COL_BLACK);

                OSD_DrawStringLtd (x1, y1 + 2,
                                   appDat->txtAttr.colFocus,
                                   MAIN_FONT, -1,
                                   inst->leftLnaviInst.data.colWidth[1], -1,
                                   STR_GetASCII(inst->renamestr), FFALSE);
            }
            else
            {
                if ( inst->partitions[index-inst->leftbaseitem].status
                   == SYS_DEV_FS_UNFORMATTED)
                {
                    item[1].item.str.chr = OSD_GetStr(
                                           STR_MSG_FS_DEVICE_NOT_FORMATTED);
                }
                else
                {
                    item[1].item.str.chr = STR_GetASCII(inst->leftitem[
                                           index-inst->leftbaseitem].name);
                }
            }
            break;

        case MAIN_VIEWER_ID_RIGHT:
            item[0].type                = FGS_ITEM_TYPE_ICON;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.icon.drawMode  = FGS_ICONMODE_OVRLAY;
            item[0].item.icon.bkgrCol   = OSD_COL_BLANK;
            item[0].item.icon.altBkgrCol= OSD_COL_BLANK;
            item[0].item.icon.addr      = OSD_IconGet(inst->rightitem[index].icon);
            item[1].type                = FGS_ITEM_TYPE_STRING;
            item[1].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[1].item.str.font       = MAIN_FONT;
            item[1].item.str.lineHeight = 0;

            if ( !(inst->rightitem[index].enabled) )
            {
                item[1].item.str.col    = appDat->txtAttr.colDisabled;
            }
            else
            {
                item[1].item.str.col    = appDat->txtAttr.colNonFocus;
            }

            item[1].item.str.chr = STR_GetASCII(inst->rightitem[index].name);
            break;

        case MAIN_VIEWER_ID_LEFTTITLE:
            item[0].type                = FGS_ITEM_TYPE_STRING;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.str.font       = MAIN_FONT;
            item[0].item.str.col        = appDat->txtAttr.colNonFocus;
            item[0].item.str.lineHeight = 0;

            switch ( inst->currentdepth )
            {
                case 0: /* Show general volume string */
                    item[0].item.str.chr = OSD_GetStr(STR_ENTRY_VIEWER_VOLUME);
                    break;

                case 1: /* Show volume/partition name */
                    item[0].item.str.chr = STR_GetASCII(inst->partitions[
                                                        inst->level[0].
                                                        currentitem].name);
                    break;

                default: /* Show folder name */
                    MAIN_ViewerCopyName (inst->path, appDat->asciiBuf,
                                         MAIN_LINEBUFCHARS);
                    item[0].item.str.chr = STR_IntASCIIStr();
                    break;
            }
            break;

        case MAIN_VIEWER_ID_RIGHTTITLE:
            (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);
            item[0].type                = FGS_ITEM_TYPE_STRING;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.str.font       = MAIN_FONT;
            item[0].item.str.col        = appDat->txtAttr.colNonFocus;
            item[0].item.str.lineHeight = 0;
            item[0].item.str.chr        = STR_GetASCII(inst->leftitem[
                                          navipar.index-inst->leftbaseitem].
                                          name);
            break;

        case MAIN_VIEWER_ID_HELP1:

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_KEY_MENU),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_HELP_MENU_BACK),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_GetASCII("    "),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_KEY_EXIT),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_HELP_MENU_EXIT),
                                     MAIN_LINEBUFCHARSET);

            item[0].type                = FGS_ITEM_TYPE_STRING;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.str.font       = MAIN_FONT;
            item[0].item.str.col        = appDat->txtAttr.colNonFocus;
            item[0].item.str.lineHeight = 0;
            item[0].item.str.chr        = STR_IntLineStr();
            break;

        case MAIN_VIEWER_ID_HELP3:

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_GetASCII("  "), MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_KEY_OK),
                                     MAIN_LINEBUFCHARSET);

            if ( inst->operation == MAIN_VIEWER_OP_RENAME )
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_GNR_COLON),
                                         MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_ENTRY_VIEWER_RENAME),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_HELP_MENU_OPEN),
                                         MAIN_LINEBUFCHARSET);
            }

            item[0].type                = FGS_ITEM_TYPE_STRING;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.str.font       = MAIN_FONT;
            item[0].item.str.col        = appDat->txtAttr.colNonFocus;
            item[0].item.str.lineHeight = 0;
            item[0].item.str.chr        = STR_IntLineStr();
            break;

        case MAIN_VIEWER_ID_HELP4:
            if ( inst->currentdepth == 0 )
            {
                item[0].type            = FGS_ITEM_TYPE_NONE;
            }
            else
            {   item[0].type            = FGS_ITEM_TYPE_STRING;
                item[0].align           = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
                item[0].item.str.font   = MAIN_FONT;
                item[0].item.str.col    = appDat->txtAttr.colNonFocus;
                item[0].item.str.lineHeight= 0;
                switch ( inst->operation )
                {
                    case MAIN_VIEWER_OP_NORMAL:
                        item[0].item.str.chr= OSD_GetStr(STR_ENTRY_VIEWER_OPTIONS);
                        break;
                    case MAIN_VIEWER_OP_MOVE_SELECTED:
                        item[0].item.str.chr= OSD_GetStr(STR_ENTRY_VIEWER_MOVETO);
                        break;
                    case MAIN_VIEWER_OP_COPY_SELECTED:
                        item[0].item.str.chr= OSD_GetStr(STR_ENTRY_VIEWER_COPYTO);
                        break;
                    case MAIN_VIEWER_OP_MOVE_PROGRESS:
                        item[0].item.str.chr= OSD_GetStr(STR_ENTRY_VIEWER_ABORTMOVE);
                        break;
                    case MAIN_VIEWER_OP_DELETE_PROGRESS:
                        item[0].item.str.chr= OSD_GetStr(STR_ENTRY_VIEWER_ABORTDELETE);
                        break;
                    case MAIN_VIEWER_OP_COPY_PROGRESS:
                        item[0].item.str.chr= OSD_GetStr(STR_ENTRY_VIEWER_ABORTCOPY);
                        break;
                    case MAIN_VIEWER_OP_RENAME:
                        item[0].item.str.chr= OSD_GetStr(STR_ENTRY_VIEWER_CANCELRENAME);
                        break;
                    default: item[0].item.str.chr=OSD_GetStr(STR_EMPTYSTRING);
                }
            }
            break;

        case MAIN_VIEWER_ID_HELPCOLOR:
            if (  inst->currentdepth > 0 )
            {
                switch ( inst->operation )
                {
                    case MAIN_VIEWER_OP_NORMAL:
                        MENU_COMMON_ITEM_BUTTON(item, ICON_BTNYELLOWSMALL);
                        break;

                    case MAIN_VIEWER_OP_MOVE_SELECTED:
                    case MAIN_VIEWER_OP_COPY_SELECTED:
                        MENU_COMMON_ITEM_BUTTON(item, ICON_BTNGREENSMALL);
                        break;

                    case MAIN_VIEWER_OP_MOVE_PROGRESS:
                    case MAIN_VIEWER_OP_DELETE_PROGRESS:
                    case MAIN_VIEWER_OP_RENAME:
                    case MAIN_VIEWER_OP_COPY_PROGRESS:
                        MENU_COMMON_ITEM_BUTTON(item, ICON_BTNREDSMALL);
                        break;

                    default:
                        /* do nothing */
                        break;
                }
            }
            break;

        case MAIN_VIEWER_ID_INFO1:
            item[0].type                = FGS_ITEM_TYPE_STRING;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.str.font       = MAIN_FONT;
            item[0].item.str.col        = appDat->txtAttr.colNonFocus;
            item[0].item.str.lineHeight = 0;
            item[0].item.str.chr        = inst->infoLine1.string;
            break;

        case MAIN_VIEWER_ID_INFO2:
            item[0].type                = FGS_ITEM_TYPE_STRING;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.str.font       = MAIN_FONT;
            item[0].item.str.col        = appDat->txtAttr.colNonFocus;
            item[0].item.str.lineHeight = 0;
            item[0].item.str.chr        = inst->infoLine2.string;
            break;

        case MAIN_VIEWER_ID_INFO3:
            item[0].type                = FGS_ITEM_TYPE_STRING;
            item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
            item[0].item.str.font       = MAIN_FONT;
            item[0].item.str.col        = appDat->txtAttr.colNonFocus;
            item[0].item.str.lineHeight = 0;
            item[0].item.str.chr        = inst->infoLine3.string;
            break;

        case MAIN_VIEWER_ID_PREVIEW:
            item[0].type                = FGS_ITEM_TYPE_NONE;
            break;

        default:
            /* do nothing */
            break;
    }
}

FSTATIC void MAIN_ViewerCopyName (const char_t* path, char_t* name,
                                  uint16_t nameBufSize)
{
    uint16_t    prepos = 0;
    uint16_t    i;

    if ( nameBufSize > 0 )
    {
        for ( i = 0; i < MAIN_VIEWER_PATHLEN; i++ )
        {
            if ( path[i] == 0 )
            {
                break;
            }
            else if ( path[i] == MENU_COMMON_PATH_SEPARATOR )
            {
                prepos = 0;
            }
            else if ( (prepos + 1) < nameBufSize )
            {
                name[prepos++] = path[i];
            }
        }

        name[prepos] = 0; /* always terminate string */
    }
}


FSTATIC char_t MAIN_ViewerNextLetter (char_t letter, fbool_t up)
{                 // Order:  0-9 A-Z_.
    if(up)
    {
        if((letter >= 'A') && (letter < 'Z'))  return letter + 1;
        if((letter >= 'a') && (letter < 'z'))  return letter-'a'+'A' + 1;
        if((letter == 'Z') || (letter == 'z')) return '_';
        if(letter == '_')                      return '.';
        if(letter == '.')                      return '0';
        if((letter >= '0') && (letter < '9'))  return letter + 1;

        return 'A';
    }
    if((letter > 'A') && (letter <= 'Z'))      return letter - 1;
    if((letter > 'a') && (letter <= 'z'))      return letter-'a'+'A' - 1;
    if((letter == 'A') || (letter == 'a'))     return '9';
    if(letter == '_')                          return 'Z';
    if(letter == '.')                          return '_';
    if((letter > '0') && (letter <= '9'))      return letter - 1;
    return '.';
}

FSTATIC fbool_t MAIN_ViewerKeys (FAPI_SYS_HandleT h)
{
    MAIN_VIEWER_INST_S*     inst    = (MAIN_VIEWER_INST_S*)h;
    fbool_t                 keyUsed = FTRUE;
    MENU_SELECTLIST_OPEN_S  selectLst;
    FGS_LOC_S               abspos;
    char_t                  path[MAIN_VIEWER_PATHLEN];
    int32_t                 retVal;
    uint8_t                 attr;
    SYS_DEVICE_E            device;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_MENU:
            if ( inst->params.onBack != NULL )
            {
                keyUsed = inst->params.onBack (inst->params.cbArg);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case FGS_KEY_BACK:
            if ( inst->currentdepth > 0 )
            {
                keyUsed = MAIN_ViewerFolderBack (inst);
            }
            else if ( inst->params.onBack != NULL )
            {
                keyUsed = inst->params.onBack (inst->params.cbArg);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case FGS_KEY_EXIT:
            if ( inst->params.onExit != NULL )
            {
                keyUsed = inst->params.onExit (inst->params.cbArg);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case FGS_KEY_YELLOW:
            if ( (inst->operation == MAIN_VIEWER_OP_NORMAL)
              && (inst->currentdepth > 0) )
            {
                memset(&selectLst, 0, sizeof(selectLst));

                (void)FGS_PANEL_GetMemberAbsPos(inst->pnlHdl,
                                                MAIN_VIEWER_ID_HELPCOLOR,
                                                &abspos);

                selectLst.pos.x         = abspos.x -
                                          (SCALER_GetCurrVal(SUB_LIST_WIDTH)/5);
                selectLst.pos.y         = abspos.y -
                                          (6*SCALER_GetCurrVal(LIST_LINE_HEIGHT));
                selectLst.pos.width     = SCALER_GetCurrVal(SUB_LIST_WIDTH);
                selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
                selectLst.id            = MAIN_VIEWER_ID_CONTEXT;
                selectLst.numVisLines   = 5;
                selectLst.numItems      = 5;
                selectLst.index         = 0;
                selectLst.getItem       = MAIN_ViewerContextGetItem;
                selectLst.getItemArg    = h;
                selectLst.onSelect      = (FGS_LNAVI_SELECT_CB)
                                          MAIN_ViewerContextSelect;
                selectLst.onSelectArg   = h;

                (void)MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case FGS_KEY_GREEN:
            if( (inst->operation == MAIN_VIEWER_OP_MOVE_SELECTED) ||
                (inst->operation == MAIN_VIEWER_OP_COPY_SELECTED) )
            {
                retVal = SYS_DeviceGetByPath(inst->copypath,&device,NULL);
                if ( (retVal == FAPI_OK) &&
                     (SYS_DeviceGetFreekBytes(device) == 0) )
                {
                    #ifdef APPL_HDD_ENABLED
                    if ( device == SYS_DEV_HDD )
                    {
                        MENU_CommonMsgShow(STR_MSG_HDD_DISKFULL,FFALSE);
                    }
                    #endif /* APPL_HDD_ENABLED */

                    #ifdef APPL_USB_ENABLED
                    if ( device == SYS_DEV_USB )
                    {
                        MENU_CommonMsgShow(STR_MSG_USB_DISKFULL,FFALSE);
                    }
                    #endif /* APPL_USB_ENABLED */

                    inst->operation = MAIN_VIEWER_OP_NORMAL;

                    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
                    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
                    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
                    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_ACTION);
                    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_PROGRESS);
                    break;
                }
                switch(inst->operation)
                {
                    case MAIN_VIEWER_OP_MOVE_SELECTED:
                        MAIN_ViewerPrepareName(inst->copypath, inst->path, path);

                        retVal = f_getattr(path,&attr);

                        if ( retVal == MAIN_VIEWER_FILE_EXIST )
                        {
                            MAIN_ViewerConfirmOverwrite(inst);
                        }
                        else
                        {
                            inst->operation= MAIN_VIEWER_OP_MOVE_PROGRESS;

                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_ACTION);
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_PROGRESS);
                            FAPI_SYS_PRINT_DEBUG(1,"Move to: %s\n", path);
                            (void)FILEOP_Move(inst->copypath, path);
                        }
                        break;

                    case MAIN_VIEWER_OP_COPY_SELECTED:
                        MAIN_ViewerPrepareName(inst->copypath, inst->path, path);

                        retVal = f_getattr(path,&attr);

                        if (retVal == MAIN_VIEWER_FILE_EXIST )
                        {
                            MAIN_ViewerConfirmOverwrite(inst);
                        }
                        else
                        {
                            inst->operation = MAIN_VIEWER_OP_COPY_PROGRESS;

                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_ACTION);
                            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_PROGRESS);
                            FAPI_SYS_PRINT_DEBUG(1,"Copy to: %s\n", path);
                            (void)FILEOP_Copy(inst->copypath, path);
                        }
                        break;

                    default:
                        break;
                }
            }
            else
            {
                keyUsed= FFALSE;
            }
            break;

        case FGS_KEY_RED:
            switch(inst->operation)
            {
                case MAIN_VIEWER_OP_MOVE_SELECTED:
                case MAIN_VIEWER_OP_COPY_SELECTED:
                    FAPI_SYS_PRINT_DEBUG(1,"Cancel select\n");
                    break;

                case MAIN_VIEWER_OP_MOVE_PROGRESS:
                    (void)FILEOP_Abort();
                    FAPI_SYS_PRINT_DEBUG(1,"Cancel move\n");
                    break;

                case MAIN_VIEWER_OP_COPY_PROGRESS:
                    (void)FILEOP_Abort();
                    FAPI_SYS_PRINT_DEBUG(1,"Cancel copy\n");
                    break;

                case MAIN_VIEWER_OP_DELETE_PROGRESS:
                    (void)FILEOP_Abort();
                    FAPI_SYS_PRINT_DEBUG(1,"Cancel delete\n");
                    break;

                case MAIN_VIEWER_OP_RENAME:
                    FAPI_SYS_PRINT_DEBUG(1,"Cancel rename\n");
                    inst->operation= MAIN_VIEWER_OP_NORMAL;
                    (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                                   MAIN_VIEWER_ID_LEFT);
                    (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                                   MAIN_VIEWER_ID_HELP3);
                    break;

                default:
                    keyUsed= FFALSE;
            }

            inst->operation = MAIN_VIEWER_OP_NORMAL;

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_ACTION);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_PROGRESS);
            break;

        case FGS_KEY_LEFT:
            if ( inst->operation == MAIN_VIEWER_OP_RENAME )
            {
                if ( inst->renamepos > 0 )
                {
                    inst->renamepos--;
                    (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                  MAIN_VIEWER_ID_LEFT);
                }
                else
                {
                    keyUsed= FFALSE;
                }
            }
            else
            {
                keyUsed= FGS_LNAVI_PageUp(inst->leftListHdl);
            }
            break;

        case FGS_KEY_RIGHT:
            if ( inst->operation == MAIN_VIEWER_OP_RENAME )
            {
                if ( inst->renamepos < 30 )
                {
                    inst->renamepos++;
                    (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                  MAIN_VIEWER_ID_LEFT);
                }
                else
                {
                    keyUsed= FFALSE;
                }
                break;
            }
            else
            {
                keyUsed= FGS_LNAVI_PageDown(inst->leftListHdl);
            }
            break;

        case FGS_KEY_UP:
            if ( inst->operation == MAIN_VIEWER_OP_RENAME )
            {
                inst->renamestr[inst->renamepos] = MAIN_ViewerNextLetter(inst->renamestr[inst->renamepos], FTRUE);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_LEFT);
            }
            else
            {
                keyUsed = FGS_LNAVI_Up(inst->leftListHdl);
            }
            break;

        case FGS_KEY_DOWN:
            if ( inst->operation == MAIN_VIEWER_OP_RENAME )
            {
                inst->renamestr[inst->renamepos]= MAIN_ViewerNextLetter(inst->renamestr[inst->renamepos], FFALSE);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_LEFT);
            }
            else
            {
                keyUsed= FGS_LNAVI_Down(inst->leftListHdl);
            }
            break;

        default:
            keyUsed= FFALSE;
            break;
    }
    return keyUsed;
}


FSTATIC fbool_t MAIN_ViewerBuild (FAPI_SYS_HandleT h)
{
    MAIN_VIEWER_INST_S*   inst = (MAIN_VIEWER_INST_S*)h;
    FGS_PANEL_OPEN_S      pnlPars;
    FAPI_SYS_HandleT      itemHdl;
    FGS_BOX_OPEN_S        boxPars;
    FGS_LNAVI_OPEN_S      naviPars;
    int32_t               retVal;
    uint16_t              stackedFrameWidth = 0;
    uint16_t*             pStacked;
    uint16_t              innerHeight;
    uint32_t              numLines= 12;
    uint8_t               numHelpLines=1;
    FGS_PANEL_MEMBER_S    member;
    uint32_t              column;

    memset(&naviPars, 0, sizeof(naviPars));
    memset(&boxPars,  0, sizeof(boxPars));
    memset(&member,   0, sizeof(member));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        innerHeight = (numLines       * SCALER_GetCurrVal(LIST_LINE_HEIGHT))
                    + ((numLines - 1) * SCALER_GetCurrVal(LIST_LINE_PADDING));

        if ( inst->params.allowStackedView )
        {
            pStacked = &stackedFrameWidth;
        }
        else
        {
            pStacked = NULL;
        }

        MENU_CommonSetParamsTitleWBotHelp (&pnlPars,
                                           SCALER_GetCurrVal(VIEWER_WIDTH),
                                           innerHeight, numHelpLines, pStacked);

        pnlPars.onFocusSet      = MAIN_ViewerOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        /* overwrite default params! */
        pnlPars.pInst           = &(inst->pnlInst);

        MENU_CommonSetPanelPos(&pnlPars.pos);

        retVal= FGS_PANEL_Open(&pnlPars, MENUID_VIEWER, &(inst->pnlHdl));
        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* ----- create textbox holding title ------------------------------ */

        boxPars.pos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                stackedFrameWidth;
        boxPars.pos.y         = SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                (SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                 SCALER_GetCurrVal(OUTER_FRAME_WIDTH))/2 + /* done for centering */
                                SCALER_GetCurrVal(TITLE_TB_PADDING) +
                                stackedFrameWidth;
        boxPars.pos.width     = pnlPars.pos.width / 2;
        boxPars.pos.height    = (2*SCALER_GetCurrVal(TITLE_TB_PADDING))
                              + SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        boxPars.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType      = OSD_BOX_BKGR_TITLE;
        boxPars.getItem       = MAIN_ViewerGetItem;
        boxPars.getItemArg    = (FAPI_SYS_HandleT)inst;

        retVal=FGS_BOX_Open(&boxPars, MAIN_VIEWER_ID_TITLE, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        /* ----- create action & progress bar ------------------------------ */

        column = (SCALER_GetCurrVal(VIEWER_WIDTH)/10); // 10% of panel width
        boxPars.pos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                stackedFrameWidth +
                                SCALER_GetCurrVal(VIEWER_WIDTH) - (5 * column);
        boxPars.pos.width     = column + (column/2); // 10% of panel width
        boxPars.bkgrType      = OSD_BOX_BKGR_TITLE;

        retVal=FGS_BOX_Open(&boxPars, MAIN_VIEWER_ID_ACTION, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        inst->progressPos.x         = boxPars.pos.x + column;
        inst->progressPos.y         = boxPars.pos.y;
        inst->progressPos.width     = (column * 3) - (column/2);// 25% of panel width
        inst->progressPos.height    = SCALER_GetCurrVal(TITLE_LINE_HEIGHT)-5;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->progressPos);
        member.id            = MAIN_VIEWER_ID_PROGRESS;
        member.show          = MAIN_ViewerPaintProgress;

        retVal= FGS_PANEL_AddMember(inst->pnlHdl, &member, &(inst->progressBorderP));
        if(retVal != FAPI_OK)
            break;

        /* ----- create viewer list ---------------------------------------- */

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                      stackedFrameWidth;
        boxPars.pos.y               = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                      SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                      (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                      SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                      stackedFrameWidth;
        boxPars.pos.width           = (SCALER_GetCurrVal(VIEWER_WIDTH) -
                                       SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                       SCALER_GetCurrVal(MAIN_RIGHT_PADDING))>>1;
        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;

        retVal= FGS_BOX_Open(&boxPars, MAIN_VIEWER_ID_LEFTTITLE, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        boxPars.pos.x              += boxPars.pos.width +
                                      SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

        retVal=FGS_BOX_Open(&boxPars, MAIN_VIEWER_ID_RIGHTTITLE, inst->pnlHdl, &itemHdl);
        if(retVal != FAPI_OK)
            break;

        /* ----- List Borders ---------------------------------------------- */

        inst->itemBorderPos.x         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                        stackedFrameWidth -
                                        SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                        SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        inst->itemBorderPos.y         = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                        SCALER_GetCurrVal(INNER_FRAME_WIDTH) +
                                        SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                        (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                        SCALER_GetCurrVal(TITLE_LINE_HEIGHT) +
                                        stackedFrameWidth +
                                        SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                                        (2*SCALER_GetCurrVal(LIST_LINE_PADDING)) -
                                        SCALER_GetCurrVal(INNER_FRAME_WIDTH);
        inst->itemBorderPos.width     = SCALER_GetCurrVal(VIEWER_WIDTH) +
                                        2*SCALER_GetCurrVal(INNER_FRAME_WIDTH) -
                                        SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                        SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        inst->itemBorderPos.height    = (numLines - 1) *
                                        (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING)) +
                                        SCALER_GetCurrVal(INNER_FRAME_WIDTH) + 1;
        inst->itemBorderPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemBorderPos);
        member.id            = MAIN_VIEWER_ID_REST;
        member.show          = MAIN_ViewerPaintRest;

        retVal= FGS_PANEL_AddMember(inst->pnlHdl, &member, &(inst->pItemBorder));
        if(retVal != FAPI_OK)
            break;

        naviPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                   stackedFrameWidth + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        naviPars.pos.y           = inst->itemBorderPos.y+2;
        naviPars.pos.width       = boxPars.pos.width -SCALER_GetCurrVal(MAIN_RIGHT_PADDING);
        naviPars.pos.height      = (numLines-1) * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        naviPars.bkgrType        = OSD_NAVIBKGR_NORMAL;
        naviPars.type            = FGS_LNAVI_TYPE_SCROLL;
        naviPars.rowHeight       = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)+SCALER_GetCurrVal(LIST_LINE_PADDING));
        naviPars.scrollbarWidth  = SCALER_GetCurrVal(SCROLLBAR_WIDTH);
        naviPars.numVisLines     = numLines - 1;

        naviPars.numColumns      = 2;

        naviPars.colStart[0]     = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviPars.colWidth[0]     = SCALER_GetCurrVal(MAIN_ICON_WIDTH);

        naviPars.colStart[1]     = SCALER_GetCurrVal(MAIN_TEXT_PAD_X) * 2 + SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        naviPars.colWidth[1]     = (SCALER_GetCurrVal(VIEWER_WIDTH)/2)- SCALER_GetCurrVal(SCROLLBAR_WIDTH)-SCALER_GetCurrVal(MAIN_LEFT_PADDING) - SCALER_GetCurrVal(MAIN_ICON_WIDTH) - SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

        naviPars.topDownOrient   = FTRUE;
        naviPars.wraparound      = FTRUE;
        naviPars.getItem         = MAIN_ViewerGetItem;
        naviPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        naviPars.pInst           = &(inst->leftLnaviInst);

        naviPars.onFocusChg      = (FGS_LNAVI_FOCUSCHG_CB)MAIN_ViewerFocusChange;
        naviPars.onFocusChgArg   = h;
        naviPars.onSelect        = (FGS_LNAVI_SELECT_CB)MAIN_ViewerSelect;
        naviPars.onSelectArg     = h;

        retVal = FGS_LNAVI_Open (&naviPars, MAIN_VIEWER_ID_LEFT, inst->pnlHdl, &(inst->leftListHdl));
        if ( retVal != FAPI_OK ) break;

        naviPars.pos.x          += SCALER_GetCurrVal(VIEWER_WIDTH)>>1;
        naviPars.scrollbarWidth  = 0;
        naviPars.pInst           = &(inst->rightLnaviInst);
        naviPars.bkgrType        = OSD_NAVIBKGR_NOSELECTION;
        naviPars.onFocusChg      = NULL;
        naviPars.onSelect        = NULL;

        retVal = FGS_LNAVI_Open (&naviPars, MAIN_VIEWER_ID_RIGHT, inst->pnlHdl, &(inst->rightListHdl));
        if ( retVal != FAPI_OK ) break;

        /* -----Bottom help lines ------------------------------------------ */

        boxPars.pos.x               = inst->itemBorderPos.x;
        boxPars.pos.y               = - (int16_t)((SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2)
                                      + SCALER_GetCurrVal(HELP_TB_PADDING)
                                      + SCALER_GetCurrVal(HELP_LINE_HEIGHT));
        boxPars.pos.width           = inst->itemBorderPos.width / 2;
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;

        retVal = FGS_BOX_Open (&boxPars, MAIN_VIEWER_ID_HELP1, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x               = boxPars.pos.x + inst->itemBorderPos.width / 2;
        boxPars.bkgrType            = OSD_BOX_BKGR_HELP;
        boxPars.pos.width           = inst->itemBorderPos.width / 4 - SCALER_GetCurrVal(MAIN_ICON_WIDTH);

        retVal = FGS_BOX_Open (&boxPars, MAIN_VIEWER_ID_HELP3, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x               = boxPars.pos.x + pnlPars.pos.width/4 - SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        boxPars.pos.width           = SCALER_GetCurrVal(MAIN_ICON_WIDTH);

        retVal = FGS_BOX_Open (&boxPars, MAIN_VIEWER_ID_HELPCOLOR, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x               = boxPars.pos.x + SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        boxPars.pos.width           = inst->itemBorderPos.x +
                                      inst->itemBorderPos.width -
                                      boxPars.pos.x;

        retVal = FGS_BOX_Open (&boxPars, MAIN_VIEWER_ID_HELP4, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Preview objects ------------------------------------------ */

        boxPars.pos.x               = naviPars.pos.x + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = naviPars.pos.y;
        boxPars.pos.width           = naviPars.pos.width - 2*SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;
        boxPars.isHidden            = FTRUE;

        retVal = FGS_BOX_Open (&boxPars, MAIN_VIEWER_ID_INFO1, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.y               = boxPars.pos.y + SCALER_GetCurrVal(LIST_LINE_HEIGHT);

        retVal = FGS_BOX_Open (&boxPars, MAIN_VIEWER_ID_INFO2, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.y               = boxPars.pos.y + SCALER_GetCurrVal(LIST_LINE_HEIGHT);

        retVal = FGS_BOX_Open (&boxPars, MAIN_VIEWER_ID_INFO3, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.y               = boxPars.pos.y + SCALER_GetCurrVal(LIST_LINE_HEIGHT) + (SCALER_GetCurrVal(LIST_LINE_HEIGHT)>>1);
        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT) * (MAIN_VIEWER_DISPITEMS - 3);//(boxPars.pos.width * 3) >> 2;
        boxPars.bkgrType            = OSD_BOX_BKGR_TRANSPARENT;

        inst->previewSizeX= boxPars.pos.width;
        inst->previewSizeY= boxPars.pos.height;

        retVal = FGS_BOX_Open (&boxPars, MAIN_VIEWER_ID_PREVIEW, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

    }while(0);

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        inst->pnlHdl = NULL;

        return FFALSE;
    }

    return FTRUE;
}


FSTATIC fbool_t MAIN_ViewerFocusChange (FAPI_SYS_HandleT h, int32_t item)
{
    MAIN_VIEWER_INST_S* inst = (MAIN_VIEWER_INST_S*)h;
    FGS_LNAVI_PARAMS_S  navipar;

    (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);

    inst->level[inst->currentdepth].currentitem = navipar.index;

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);

    MAIN_ViewerUpdateRight (inst, navipar.index-inst->leftbaseitem);

    return FTRUE;
}


FSTATIC void MAIN_ViewerUpdateRight (MAIN_VIEWER_INST_S* inst, uint16_t newleft)
{
    char_t                  filePath[MAIN_VIEWER_PATHLEN];
    uint16_t                itemcount;
    FGS_LNAVI_PARAMS_S      navipar;
    fbool_t                 isHidden;
    uint32_t                bufPos;

    #if defined(APPL_FILEUPDATE_ENABLED)
    F_FILE*                 fileP;
    SYS_DEVICE_E            device;
    #endif /* defined(APPL_FILEUPDATE_ENABLED) */

    #ifdef APPL_JPEG_ENABLED
    JPEG_SCREEN_PARAMS_S    jpegScreen;
    JPEG_PIC_PARAMS_S       jpegPic;
    FGS_LOC_S               memberPos;
    #endif /* APPL_JPEG_ENABLED */

    #ifdef APPL_MP3_ENABLED
    SYS_STRING_S            string;
    MP3_FILE_INFO_S         mp3Info;
    #endif /* APPL_MP3_ENABLED */

    #ifdef APPL_PVR_ENABLED
    PVR_LIST_ITEM_S         pvritem;
    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_FILEUPDATE_ENABLED
    BOOTER_FlashSlotHeaderT      updateHeader;
    #endif /* APPL_FILEUPDATE_ENABLED */

    if ( FGS_PANEL_GetMemberHideStatus (inst->pnlHdl, MAIN_VIEWER_ID_PREVIEW,
                                         &isHidden) != FAPI_OK )
    {
        return;
    }

    /* hide the unformatted message if necessary */
    (void)MAIN_MsgBoxHide(&(appDat->msgBox));

    if ( !isHidden )
    {
        (void)FGS_PANEL_SetMemberHideStatus (inst->pnlHdl,
                                             MAIN_VIEWER_ID_PREVIEW, FTRUE);
    }

    /* ----- stop potentially active preview ------------------------------- */

    if ( inst->leftitem[newleft].icon != inst->currenticon )
    {
        switch ( inst->currenticon )
        {
            #ifdef APPL_JPEG_ENABLED

            case ICON_PICTURESMALL:
                if ( inst->jpegActive )
                {
                    (void)JPEG_Close();

                    inst->jpegActive = FFALSE;
                }
                break;

            #endif /* APPL_JPEG_ENABLED */

            #ifdef APPL_MP3_ENABLED

            case ICON_MUSICSMALL:

                #ifdef APPL_JPEG_ENABLED

                if ( inst->jpegActive )
                {
                    /* kill optional embedded jpeg picture */
                    (void)JPEG_Close();

                    inst->jpegActive = FFALSE;
                }

                #endif /* APPL_JPEG_ENABLED */

                break;

            #endif /* APPL_MP3_ENABLED */

            #ifdef APPL_PVR_ENABLED

            case ICON_MOVIESMALL:
                break;

            #endif /* APPL_PVR_ENABLED */

            default:
                break;
        }

        inst->currenticon = inst->leftitem[newleft].icon;
    }

    switch ( inst->currenticon )
    {
        #ifdef APPL_MP3_ENABLED

        case ICON_MUSICSMALL:
            MAIN_ViewerSwitchInfo (inst, FTRUE);

            /* show MP3 information */
            if ( (inst->params.allowedMusicOps & MAIN_VIEWER_OP_SHOWINFO) != 0 )
            {
                memset(&mp3Info, 0, sizeof(mp3Info));

                MENU_CommonAppendpath (inst->leftitem[newleft].name,
                                       inst->path,
                                       filePath, FFALSE, MAIN_VIEWER_PATHLEN);

                (void)MP3_GetInfo (filePath, &mp3Info);

                /* line 1: MP3 title */

                string = STR_Build (mp3Info.titleCs, mp3Info.title);

                bufPos = STR_BufAppend(&(inst->infoLine1), 0,
                                      OSD_GetStr(STR_ENTRY_VIEWER_MP3_TITLE),
                                      MAIN_LINEBUFCHARSET);

                if ( !(STR_IsEmpty(string)) )
                {
                    (void)STR_BufAppend(&(inst->infoLine1), bufPos, string,
                                  MAIN_LINEBUFCHARSET);
                }
                else
                {
                    (void)STR_BufAppend(&(inst->infoLine1), bufPos,
                                  OSD_GetStr(STR_ENTRY_VIEWER_MP3_UNKNOWN),
                                  MAIN_LINEBUFCHARSET);
                }

                /* line 2: MP3 artist */

                string = STR_Build (mp3Info.artistCs, mp3Info.artist);

                bufPos = STR_BufAppend(&(inst->infoLine2), 0,
                                      OSD_GetStr(STR_ENTRY_VIEWER_MP3_ARTIST),
                                      MAIN_LINEBUFCHARSET);

                if ( !(STR_IsEmpty(string)) )
                {
                    (void)STR_BufAppend(&(inst->infoLine2), bufPos, string,
                                  MAIN_LINEBUFCHARSET);
                }
                else
                {
                    (void)STR_BufAppend(&(inst->infoLine2), bufPos,
                                  OSD_GetStr(STR_ENTRY_VIEWER_MP3_UNKNOWN),
                                  MAIN_LINEBUFCHARSET);
                }

                /* line 3: album */

                string = STR_Build (mp3Info.albumCs, mp3Info.album);

                bufPos = STR_BufAppend(&(inst->infoLine3), 0,
                                      OSD_GetStr(STR_ENTRY_VIEWER_MP3_ALBUM),
                                      MAIN_LINEBUFCHARSET);

                if ( !(STR_IsEmpty(string)) )
                {
                    (void)STR_BufAppend(&(inst->infoLine3), bufPos, string,
                                  MAIN_LINEBUFCHARSET);
                }
                else
                {
                    (void)STR_BufAppend(&(inst->infoLine3), bufPos,
                                  OSD_GetStr(STR_ENTRY_VIEWER_MP3_UNKNOWN),
                                  MAIN_LINEBUFCHARSET);
                }

                #ifdef APPL_JPEG_ENABLED

                if (( inst->jpegActive ) && ( !mp3Info.jpegAvl))
                {
                    /* kill optional embedded jpeg picture in case current MP3
                    file does not contain embedded picture, but previous MP3 file does*/
                    (void)JPEG_Close();

                    inst->jpegActive = FFALSE;
                }

                /* show embedded picture */
                if ( (mp3Info.jpegAvl)
                  && (inst->params.allowedMusicOps & MAIN_VIEWER_OP_PREVIEW)
                      != 0 )
                {
                    memset (&memberPos, 0, sizeof(memberPos));

                    (void)FGS_PANEL_GetMemberAbsPos(inst->pnlHdl,
                                                    MAIN_VIEWER_ID_PREVIEW,
                                                    &memberPos);

                    jpegPic.path      = filePath;
                    jpegPic.filepos   = mp3Info.picPos;
                    jpegPic.orient    = ROT_0;
                    jpegPic.zoom      = 100;
                    jpegPic.xpos      = 0;
                    jpegPic.ypos      = 0;
                    jpegPic.embedded  = FTRUE;

                    jpegScreen.pos.left  = memberPos.x;
                    jpegScreen.pos.top   = memberPos.y;
                    jpegScreen.pos.hsize = inst->previewSizeX+5;
                    jpegScreen.pos.vsize = inst->previewSizeY+5;

                    (void)JPEG_Show(&jpegPic, &jpegScreen);

                    inst->jpegActive = FTRUE;
                }

                #endif /* APPL_JPEG_ENABLED */

                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MAIN_VIEWER_ID_INFO1);
                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MAIN_VIEWER_ID_INFO2);
                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MAIN_VIEWER_ID_INFO3);
            }
            break;

        #endif /* APPL_MP3_ENABLED */

        #ifdef APPL_PVR_ENABLED

        case ICON_MOVIESMALL:
            MAIN_ViewerSwitchInfo (inst, FTRUE);

            if ( (inst->params.allowedMovieOps & MAIN_VIEWER_OP_SHOWINFO) != 0 )
            {
                MENU_CommonAppendpath (inst->leftitem[newleft].name, inst->path,
                                       filePath, FFALSE, MAIN_VIEWER_PATHLEN);

                if ( PVR_LIST_GetItemByPath(filePath, &pvritem) == FAPI_OK )
                {
                    /* line 1: Recording name */

                    (void)STR_BufAppend(&(inst->infoLine1), 0,
                                  STR_Build(pvritem.nameCharSet, pvritem.name),
                                  MAIN_LINEBUFCHARSET);

                    /* line 2: Recording date */

                    sprintf(appDat->asciiBuf, "%02d.%02d.%04d   %02d:%02d",
                            pvritem.startDate.day,
                            pvritem.startDate.month,
                            pvritem.startDate.year,
                            pvritem.startTime.hour,
                            pvritem.startTime.minute);

                    (void)STR_BufAppend(&(inst->infoLine2), 0, STR_IntASCIIStr(),
                                  MAIN_LINEBUFCHARSET);

                    /* line 3: Recording duration */

                    if ( pvritem.duration.hour == 0 )
                    {
                        sprintf(appDat->asciiBuf, "%2i:%02d ",
                                pvritem.duration.minute,
                                pvritem.duration.second);
                    }
                    else
                    {
                        sprintf(appDat->asciiBuf, "%2i:%02d:%02d",
                                pvritem.duration.hour,
                                pvritem.duration.minute,
                                pvritem.duration.second);
                    }

                    bufPos = STR_BufAppend(&(inst->infoLine3), 0,
                                          STR_IntASCIIStr(),
                                          MAIN_LINEBUFCHARSET);

                    (void)STR_BufAppend(&(inst->infoLine3), bufPos,
                                  OSD_GetStr(STR_UNIT_MIN),
                                  MAIN_LINEBUFCHARSET);

                    (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                                   MAIN_VIEWER_ID_INFO1);
                    (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                                   MAIN_VIEWER_ID_INFO2);
                    (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                                   MAIN_VIEWER_ID_INFO3);
                }
            }
            break;

        #endif /* APPL_PVR_ENABLED */

        #ifdef APPL_JPEG_ENABLED

        case ICON_PICTURESMALL:
            MAIN_ViewerSwitchInfo (inst, FTRUE);

            if ( (inst->params.allowedPicOps & MAIN_VIEWER_OP_PREVIEW) != 0 )
            {
                memset (&memberPos, 0, sizeof(memberPos));

                (void)FGS_PANEL_GetMemberAbsPos(inst->pnlHdl,
                                                MAIN_VIEWER_ID_PREVIEW,
                                                &memberPos);

                MENU_CommonAppendpath(inst->leftitem[newleft].name,
                                      inst->path,
                                      filePath, FFALSE, MAIN_VIEWER_PATHLEN);

                jpegPic.path      = filePath;
                jpegPic.filepos   = 0;
                jpegPic.orient    = ROT_0;
                jpegPic.zoom      = 100;
                jpegPic.xpos      = 0;
                jpegPic.ypos      = 0;
                jpegPic.embedded  = FFALSE;

                // to compensate the round of absolute value calcualtion -x
                jpegScreen.pos.left  = memberPos.x-2;
                jpegScreen.pos.top   = memberPos.y-2;
                jpegScreen.pos.hsize = inst->previewSizeX+5;
                jpegScreen.pos.vsize = inst->previewSizeY+5;

                (void)JPEG_Show(&jpegPic, &jpegScreen);

                inst->jpegActive = FTRUE;
            }
            break;

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_FILEUPDATE_ENABLED

        case ICON_UPDATESMALL:
            MAIN_ViewerSwitchInfo (inst, FTRUE);

            // TODO: Add this as configuration option as well?

            MENU_CommonAppendpath(inst->leftitem[newleft].name, inst->path,
                                  filePath, FFALSE, MAIN_VIEWER_PATHLEN);

            if ( SYS_DeviceGetByPath (filePath, &device, NULL) == FAPI_OK )
            {
                (void)SYS_DevicePreAccess (device);
            }

            fileP = f_open(filePath,"r");
            if( fileP != NULL )
            {
                memset(&updateHeader, 0, sizeof(updateHeader));

                (void)f_read  (&updateHeader, 1, sizeof(BOOTER_FlashSlotHeaderT),
                               fileP);
                (void)f_close (fileP);


                bufPos = 0;
                bufPos += STR_BufAppend(&(inst->infoLine1), bufPos,
                                      OSD_GetStr(STR_ENTRY_VIEWER_UPDATENAME),
                                      MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend(&(inst->infoLine1),
                                        bufPos,
                                        OSD_GetStr(STR_GNR_COLON),
                                        MAIN_LINEBUFCHARSET);

                if ( updateHeader.signature == BOOTER_SLOTHDR_SIG )
                {
                    bufPos += STR_BufAppend(&(inst->infoLine1),
                                           bufPos,
                                           STR_GetASCII(updateHeader.name),
                                           MAIN_LINEBUFCHARSET);
                }
                else
                {
                    bufPos += STR_BufAppend(&(inst->infoLine1), bufPos,
                                           OSD_GetStr(STR_GNR_2_MINUS),
                                           MAIN_LINEBUFCHARSET);
                }

                bufPos = 0;
                bufPos += STR_BufAppend(&(inst->infoLine2), bufPos,
                                      OSD_GetStr(STR_ENTRY_VIEWER_UPDATESUPPLIER),
                                      MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend(&(inst->infoLine2),
                                        bufPos,
                                        OSD_GetStr(STR_GNR_COLON),
                                        MAIN_LINEBUFCHARSET);
                /* if ( updateHeader.signature == BOOTER_SLOTHDR_SIG )
                {
                    bufPos += STR_BufAppend(&(inst->infoLine2),
                                            bufPos,
                                            STR_GetASCII(updateHeader.manID),
                                            MAIN_LINEBUFCHARSET);
                }
                else */
                {
                    bufPos += STR_BufAppend(&(inst->infoLine2), bufPos,
                                           OSD_GetStr(STR_GNR_2_MINUS),
                                           MAIN_LINEBUFCHARSET);
                }

                /* line 3 */

                bufPos = 0;
                bufPos += STR_BufAppend(&(inst->infoLine3), bufPos,
                                      OSD_GetStr(STR_ENTRY_VIEWER_UPDATEVERSION),
                                      MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend(&(inst->infoLine3),
                                        bufPos,
                                        OSD_GetStr(STR_GNR_COLON),
                                        MAIN_LINEBUFCHARSET);
                if ( updateHeader.signature == BOOTER_SLOTHDR_SIG )
                {
                    sprintf(appDat->asciiBuf, "%ld.%02ld",
                             updateHeader.version>>8,
                             updateHeader.version & 0xFF);

                    bufPos += STR_BufAppend(&(inst->infoLine3), bufPos, STR_IntASCIIStr(),
                                  MAIN_LINEBUFCHARSET);
                }
                else
                {
                    bufPos += STR_BufAppend(&(inst->infoLine3), bufPos,
                                           OSD_GetStr(STR_GNR_2_MINUS),
                                           MAIN_LINEBUFCHARSET);
                }

                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MAIN_VIEWER_ID_INFO1);
                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MAIN_VIEWER_ID_INFO2);
                (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                               MAIN_VIEWER_ID_INFO3);
            }
            break;

        #endif /* APPL_FILEUPDATE_ENABLED */

        case ICON_FILESMALL:
            MAIN_ViewerSwitchInfo (inst, FFALSE);

            navipar.index    = 0;
            navipar.numItems = 0;

            (void)FGS_LNAVI_SetParams(inst->rightListHdl, &navipar);
            break;

        default:
            if ( inst->currentdepth > 0 )
            {
                MENU_CommonAppendpath (inst->leftitem[newleft].name, inst->path,
                                       filePath, FTRUE, MAIN_VIEWER_PATHLEN);
            }
            else
            {
                (void)UTILS_strncpy(filePath, inst->partitions[newleft].path,
                                    MAIN_VIEWER_PARTLEN_MAX);
            }

            itemcount = MAIN_ViewerCheckContent (filePath, inst->rightitem, 0,
                                                 inst->params.filterType);

            MAIN_ViewerSwitchInfo (inst, FFALSE);

            navipar.index    = 0;
            navipar.numItems = itemcount;

            (void)FGS_LNAVI_SetParams(inst->rightListHdl, &navipar);
            break;
    }

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_RIGHTTITLE);
}


FSTATIC void MAIN_ViewerRename (MAIN_VIEWER_INST_S* inst, uint16_t pos)
{
    uint16_t    i;
    int32_t     retVal = F_NO_ERROR;
    char_t      path[MAIN_VIEWER_PATHLEN];

    MENU_CommonAppendpath (inst->leftitem[pos-inst->leftbaseitem].name,
                           inst->path, path, FFALSE, MAIN_VIEWER_PATHLEN);

    for(i= 28; i>0; i--)
        if(inst->renamestr[i] != '_')
            break;

    inst->renamestr[i+1]= 0;

    retVal = f_rename((const char_t*)path , (const char_t*)inst->renamestr);
    if( (retVal == F_ERR_INVALIDNAME) ||
        (retVal == F_ERR_INVALIDDIR)  ||
        (retVal == F_ERR_NOTFOUND) )
    {
        MENU_CommonMsgShow(STR_MSG_INVALIDSTRING, FFALSE);
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1,"Rename %s to %s\n", path, inst->renamestr);
    }

    inst->operation    = MAIN_VIEWER_OP_NORMAL;
    inst->leftbaseitem = -20;

    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MAIN_VIEWER_ID_LEFT);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MAIN_VIEWER_ID_HELP3);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
}


FSTATIC fbool_t MAIN_ViewerSelect (FAPI_SYS_HandleT h, int32_t id,
                                   uint16_t index)
{
    fbool_t             keyUsed = FFALSE;
    MAIN_VIEWER_INST_S* inst    = (MAIN_VIEWER_INST_S*)h;
    FGS_LNAVI_PARAMS_S  navipar;
    MAIN_VIEWER_ITEM*   pCurrItem;
    uint16_t            currIndex = 0;
    uint16_t            numItems  = 0;

    if ( FGS_LNAVI_GetParams(inst->leftListHdl, &navipar) != FAPI_OK )
    {
        return FFALSE;
    }
    else if ( inst->operation == MAIN_VIEWER_OP_RENAME )
    {
        MAIN_ViewerRename (inst, navipar.index);
        return FTRUE;
    }
    else if ( navipar.numItems < 1 )
    {
        return FFALSE;
    }

    if ( navipar.index > 0 )
    {
        currIndex = navipar.index - 1;
    }

    numItems = navipar.numItems - 1;

    pCurrItem = inst->leftitem + (navipar.index - inst->leftbaseitem);

    if ( (pCurrItem->selectionMatch)
      && (inst->params.onSelect != NULL) )
    {
        keyUsed = inst->params.onSelect (inst->params.cbArg,
                                         MAIN_VIEWER_FILETYPE_FILTERED,
                                         inst->path, MAIN_VIEWER_PATHLEN,
                                         pCurrItem->name,
                                         MAIN_VIEWER_NAMELEN,
                                         currIndex, numItems);
    }

    if ( !keyUsed )
    {
        switch ( pCurrItem->icon )
        {
            case ICON_FILESMALL:
                /* do nothing */
                break;

            #ifdef APPL_MP3_ENABLED

            case ICON_MUSICSMALL:

                if ( inst->params.onSelect != NULL )
                {
                    keyUsed = inst->params.onSelect (inst->params.cbArg,
                                                     MAIN_VIEWER_FILETYPE_MUSIC,
                                                     inst->path,
                                                     MAIN_VIEWER_PATHLEN,
                                                     pCurrItem->name,
                                                     MAIN_VIEWER_NAMELEN,
                                                     currIndex, numItems);
                }
                break;

            #endif /* APPL_MP3_ENABLED */

            #ifdef APPL_PVR_ENABLED

            case ICON_MOVIESMALL:
                if ( inst->params.onSelect != NULL )
                {
                    keyUsed = inst->params.onSelect (inst->params.cbArg,
                                                     MAIN_VIEWER_FILETYPE_MOVIE,
                                                     inst->path,
                                                     MAIN_VIEWER_PATHLEN,
                                                     pCurrItem->name,
                                                     MAIN_VIEWER_NAMELEN,
                                                     currIndex, numItems);
                }
                break;

            #endif /* APPL_PVR_ENABLED */

            #ifdef APPL_JPEG_ENABLED

            case ICON_PICTURESMALL:
                if ( inst->params.onSelect != NULL )
                {
                    keyUsed = inst->params.onSelect (inst->params.cbArg,
                                                MAIN_VIEWER_FILETYPE_PICTURE,
                                                inst->path,
                                                MAIN_VIEWER_PATHLEN,
                                                pCurrItem->name,
                                                MAIN_VIEWER_NAMELEN,
                                                currIndex, numItems);
                }
                break;

            #endif /* APPL_JPEG_ENABLED */

            default:
                if ( pCurrItem->isDir )
                {
                    if ( (inst->currentdepth > 0)
                      && ((navipar.index + inst->leftbaseitem) == 0) )
                    {
                        keyUsed = MAIN_ViewerFolderBack (inst);
                    }
                    else
                    {
                        keyUsed = MAIN_ViewerFolderDown (inst, pCurrItem,
                                                         &navipar);
                    }
                }
                break;
        }
    }

    return keyUsed;
}


FSTATIC void MAIN_ViewerContextGetItem (FAPI_SYS_HandleT h, int32_t id,
                                        uint32_t index, FGS_ITEMSTATE_E state,
                                        FGS_ITEM_S* item)
{
    MAIN_VIEWER_INST_S* inst = (MAIN_VIEWER_INST_S*)h;
    FGS_LNAVI_PARAMS_S  navipar;
    uint8_t             fileOpReq = 0;

    if ( FGS_LNAVI_GetParams(inst->leftListHdl, &navipar) != FAPI_OK )
    {
        return;
    }

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;

    switch ( index )
    {
        case MAIN_VIEWER_ID_CONTEXT_COPY:
            item->item.str.chr = OSD_GetStr(STR_ENTRY_VIEWER_COPY);
            fileOpReq          = MAIN_VIEWER_FILEOP_COPY;
            break;

        case MAIN_VIEWER_ID_CONTEXT_MOVE:
            item->item.str.chr = OSD_GetStr(STR_ENTRY_VIEWER_MOVE);
            fileOpReq          = MAIN_VIEWER_FILEOP_MOVE;
            break;

        case MAIN_VIEWER_ID_CONTEXT_RENAME:
            item->item.str.chr = OSD_GetStr(STR_ENTRY_VIEWER_RENAME);
            fileOpReq          = MAIN_VIEWER_FILEOP_RENAME;
            break;

        case MAIN_VIEWER_ID_CONTEXT_DELETE:
            item->item.str.chr = OSD_GetStr(STR_ENTRY_VIEWER_DELETE);
            fileOpReq          = MAIN_VIEWER_FILEOP_DELETE;
            break;

        case MAIN_VIEWER_ID_CONTEXT_NEW_FOLDER:
            item->item.str.chr = OSD_GetStr(STR_ENTRY_VIEWER_NEW_FOLDER);
            break;

        default:
            item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
            break;
    }

    if ( (fileOpReq != 0)
      && (((inst->params.allowedFileOps & fileOpReq) == 0)
       || ((navipar.index + inst->leftbaseitem)      == 0)) )
    {
        item->item.str.col = appDat->txtAttr.colDisabled;
    }
    else if ( state == FGS_FOCUS )
    {
        item->item.str.col = appDat->txtAttr.colFocus;
    }
    else
    {
        item->item.str.col = appDat->txtAttr.colNonFocus;
    }
}


FSTATIC fbool_t MAIN_ViewerContextSelect(FAPI_SYS_HandleT h, int32_t id,
                                         uint16_t index)
{
    MAIN_VIEWER_INST_S*     inst = (MAIN_VIEWER_INST_S*)h;
    char_t                  path[MAIN_VIEWER_PATHLEN];
    char_t                  foldername[10];
    uint16_t                foldernumber;
    int32_t                 error;
    fbool_t                 keyUsed = FTRUE;
    FGS_LNAVI_PARAMS_S      navipar;
    SYS_DEVICE_E            device;
    MENU_EDITSTRING_OPEN_S  editParams;
    uint16_t                stackedFrameWidth;
    F_FIND                  find;
    MAIN_VIEWER_ITEM*       pCurrItem;

    if ( FGS_LNAVI_GetParams(inst->leftListHdl, &navipar) != FAPI_OK )
    {
        return FFALSE;
    }

    pCurrItem = inst->leftitem + (navipar.index - inst->leftbaseitem);

    switch ( index )
    {
        case MAIN_VIEWER_ID_CONTEXT_COPY:
            if ( (inst->params.allowedFileOps & MAIN_VIEWER_FILEOP_COPY) != 0 )
            {
                inst->operation= MAIN_VIEWER_OP_COPY_SELECTED;

                MENU_CommonAppendpath (pCurrItem->name, inst->path,
                                       inst->copypath, FFALSE,
                                       MAIN_VIEWER_PATHLEN);

                inst->copyIconType = inst->currenticon;
                FAPI_SYS_PRINT_DEBUG(1,"Copy from: %s\n", inst->copypath);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case MAIN_VIEWER_ID_CONTEXT_MOVE:
            if ( (inst->params.allowedFileOps & MAIN_VIEWER_FILEOP_MOVE) != 0 )
            {
                inst->operation= MAIN_VIEWER_OP_MOVE_SELECTED;

                MENU_CommonAppendpath (pCurrItem->name, inst->path,
                                       inst->copypath, FFALSE,
                                       MAIN_VIEWER_PATHLEN);

                inst->copyIconType = inst->currenticon;
                FAPI_SYS_PRINT_DEBUG(1,"Move from: %s\n", inst->copypath);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case MAIN_VIEWER_ID_CONTEXT_RENAME:
            if ( (inst->params.allowedFileOps & MAIN_VIEWER_FILEOP_RENAME)
                 != 0 )
            {
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);

                /* initialise string editor parameter */
                (void)FGS_PANEL_GetPosition (inst->pnlHdl, &(editParams.pos));

                /* FIXME: This is relying on 'allowStackedView'-option
                          being FTRUE. */
                stackedFrameWidth   = (MENUSTACK_GetLevel() - 1)
                                      * (SCALER_GetCurrVal(STACKED_FRAME_DIST)
                                      + SCALER_GetCurrVal(OUTER_FRAME_ALL));
                editParams.pos.x  += stackedFrameWidth
                                     + SCALER_GetCurrVal(OUTER_FRAME_ALL);
                editParams.pos.y  += stackedFrameWidth
                                     + SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                     + SCALER_GetCurrVal(INNER_FRAME_WIDTH)
                                     + SCALER_GetCurrVal(MAIN_TOP_PADDING)
                                     + (2*SCALER_GetCurrVal(TITLE_TB_PADDING))
                                     + SCALER_GetCurrVal(TITLE_LINE_HEIGHT)
                                     + SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                     + SCALER_GetCurrVal(LIST_LINE_PADDING);
                editParams.editTitle = NULL;
                editParams.editTitleArg = NULL;
                editParams.onExit = MAIN_ViewerEditStringExit;

                editParams.editData.string   = STR_GetASCII(pCurrItem->name);
                editParams.editData.maxChars = MAIN_VIEWER_NAMELEN - 1;
                editParams.id               = (int32_t)navipar.index;
                if( !MENU_EditStringOpen (&(appDat->editStringBox),&editParams) )
                {
                    keyUsed = FFALSE;
                }
                else
                {
                    inst->operation = MAIN_VIEWER_OP_RENAME;
                }

                (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                              MAIN_VIEWER_ID_LEFT);
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                              MAIN_VIEWER_ID_HELP3);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case MAIN_VIEWER_ID_CONTEXT_DELETE:
            if ( (inst->params.allowedFileOps & MAIN_VIEWER_FILEOP_DELETE)
                 != 0 )
            {
                MAIN_ViewerConfirmDelete(inst);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;

        case MAIN_VIEWER_ID_CONTEXT_NEW_FOLDER:
            if ( SYS_DeviceGetByPath (inst->path, &device, NULL) == FAPI_OK )
            {
                (void)SYS_DevicePreAccess (device);
            }
            if( SYS_DeviceGetFreekBytes(device) == 0)
            {
                #ifdef APPL_HDD_ENABLED
                if ( device == SYS_DEV_HDD )
                {
                    MENU_CommonMsgShow(STR_MSG_HDD_DISKFULL,FFALSE);
                }
                #endif /* APPL_HDD_ENABLED */

                #ifdef APPL_USB_ENABLED
                if ( device == SYS_DEV_USB )
                {
                    MENU_CommonMsgShow(STR_MSG_USB_DISKFULL,FFALSE);
                }
                #endif /* APPL_USB_ENABLED */
                break;
            }
            foldernumber = 1;

            do
            {   sprintf(foldername,"FOLDER%02d", foldernumber++);

                MENU_CommonAppendpath(foldername, inst->path, path,
                                      FFALSE, MAIN_VIEWER_PATHLEN);

                FAPI_SYS_PRINT_DEBUG(1,"New folder: %s\n", path);

                error= f_findfirst(path,&find);

            } while ( !error && (foldernumber < 99) );

            (void)SYS_DeviceMkdirByPath(path);

            MAIN_ViewerUpdate(inst);
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}


FSTATIC int32_t MAIN_ViewerPaintRest (FAPI_SYS_HandleT h)
{
    MAIN_VIEWER_INST_S* inst = (MAIN_VIEWER_INST_S*)h;
    uint32_t            x1,x2,y1,y2;

    x1 = inst->pItemBorder->abspos.x;
    y1 = inst->pItemBorder->abspos.y;
    x2 = x1 + (inst->itemBorderPos.width/2)- SCALER_GetCurrVal(MAIN_LEFT_PADDING);
    y2 = y1 + inst->itemBorderPos.height-1;

    OSD_DrawRectangle(x1,y1,x2,y2,OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME));
    OSD_DrawRectangle(x1+1,y1+1,x2-1,y2-1,OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME));

    x1 += x2 + 2*SCALER_GetCurrVal(MAIN_LEFT_PADDING) - x1 -2;
    x2 += (inst->itemBorderPos.width/2) + SCALER_GetCurrVal(MAIN_LEFT_PADDING)-2;

    OSD_DrawRectangle(x1,y1,x2,y2,OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME));
    OSD_DrawRectangle(x1+1,y1+1,x2+1,y2-1,OSD_GetSchemeColor(OSD_SCHEMECOL_FRAME));

    return FAPI_OK;
}

void MAIN_ViewerUpdateProgress (FILEOP_INFO_S* infoP)
{
    MAIN_VIEWER_INST_S* inst = viewerDat;

    if ( inst == NULL )
    {
        return;
    }
    else if ( infoP == NULL )
    {
        return;
    }

    if ( infoP->status == FILEOP_STATUS_ERROR )
    {
        switch ( infoP->error )
        {
            case SMARTGO_SYS_ERR_FILEWRITE:

                switch ( infoP->dstDevice )
                {
                    #ifdef APPL_HDD_ENABLED

                    case SYS_DEV_HDD:
                        MENU_CommonMsgShow (STR_MSG_HDD_NOT_ENOUGHT_SPACE,
                                            FFALSE);
                        break;

                    #endif /* APPL_HDD_ENABLED */

                    #ifdef APPL_USB_ENABLED

                    case SYS_DEV_USB:
                        MENU_CommonMsgShow (STR_MSG_USB_NOT_ENOUGHT_SPACE,
                                            FFALSE);
                        break;

                    #endif /* APPL_USB_ENABLED */

                    default:
                        /* do nothing */
                        break;
                }
                break;

            default:
                /* do nothing */
                break;
        }
    }

    if( !(infoP->active) )
    {
        inst->operation = MAIN_VIEWER_OP_NORMAL;

        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                          MAIN_VIEWER_ID_TITLE);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                          MAIN_VIEWER_ID_HELP4);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                          MAIN_VIEWER_ID_HELPCOLOR);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                          MAIN_VIEWER_ID_ACTION);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                          MAIN_VIEWER_ID_PROGRESS);

            MAIN_ViewerUpdate (inst);

            #ifdef APPL_PVR_ENABLED

            /* Resync PVR lists if a directory has been involved that PVR is
               aware of. */

            (void)MAIN_ViewerResyncPvrList (infoP->srcDevice);
            (void)MAIN_ViewerResyncPvrList (infoP->dstDevice);

            #endif /* APPL_PVR_ENABLED */
        }
        return;
    }

    inst->maxprogress     = infoP->progress.allDataMax.kbytes;
    inst->currentprogress = infoP->progress.allDataCurr.kbytes;

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                      MAIN_VIEWER_ID_PROGRESS);
    }
}

FSTATIC int32_t MAIN_ViewerPaintProgress (FAPI_SYS_HandleT h)
{
    MAIN_VIEWER_INST_S* inst = (MAIN_VIEWER_INST_S*)h;
    uint32_t            x1, x2, y1, y2;

    x1= inst->progressBorderP->abspos.x;
    y1= inst->progressBorderP->abspos.y;
    y2= y1 + inst->progressPos.height;

    if((inst->operation != MAIN_VIEWER_OP_MOVE_PROGRESS) &&
       (inst->operation != MAIN_VIEWER_OP_COPY_PROGRESS) &&
       (inst->operation != MAIN_VIEWER_OP_DELETE_PROGRESS))
    {
        x2= x1 + (inst->progressPos.width);
        OSD_DrawFilledRectangle(x1, y1, x2, y2, OSD_GetSchemeColor(OSD_SCHEMECOL_TITLE_BKGR));
        return FAPI_OK;
    }

    if(inst->maxprogress)
        x2= x1 + inst->progressPos.width * inst->currentprogress / inst->maxprogress;
    else
        x2= x1;
    OSD_DrawFilledRectangle(x1, y1, x2, y2, OSD_COL_BAR_ACTIVE);

    x1= x2;
    x2= inst->progressBorderP->abspos.x + inst->progressPos.width;
    OSD_DrawFilledRectangle(x1, y1, x2, y2, OSD_GetSchemeColor(OSD_SCHEMECOL_PROGRESSBAR_INACT));

    return FAPI_OK;
}

FSTATIC fbool_t MAIN_ViewerFolderDown (MAIN_VIEWER_INST_S* inst,
                                       const MAIN_VIEWER_ITEM* pCurrItem,
                                       const FGS_LNAVI_PARAMS_S* pCurrNaviPos)
{
    fbool_t             keyUsed = FFALSE;
    FGS_LNAVI_PARAMS_S  navipar;

    if ( inst->currentdepth < MAIN_VIEWER_FOLDERLVL_MAX )
    {
        inst->level[inst->currentdepth].currenttop = pCurrNaviPos->posInVis;

        if ( inst->currentdepth > 0 )
        {
            MENU_CommonAppendpath (pCurrItem->name,
                                   inst->path, inst->path, FTRUE,
                                   MAIN_VIEWER_PATHLEN);
        }
        else
        {
            (void)UTILS_strncpy (inst->path, inst->partitions[
                                 pCurrNaviPos->index].path,
                                 MAIN_VIEWER_PARTLEN_MAX);

            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                           MAIN_VIEWER_ID_HELP4);
            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                            MAIN_VIEWER_ID_HELPCOLOR);
        }

        inst->currentdepth++;

        inst->level[inst->currentdepth].currentitem = 0;

        memset (&navipar, 0, sizeof(navipar));

        navipar.index    = 0;
        navipar.posInVis = 0;
        navipar.numItems = MAIN_ViewerCheckContent (inst->path, inst->leftitem,
                                                    0, inst->params.filterType);

        (void)FGS_LNAVI_SetParams (inst->leftListHdl, &navipar);

        inst->leftbaseitem = 0;

        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MAIN_VIEWER_ID_LEFTTITLE);

        keyUsed = FTRUE;
    }

    return keyUsed;
}

FSTATIC fbool_t MAIN_ViewerFolderBack (MAIN_VIEWER_INST_S* inst)
{
    uint16_t            itemcount;
    uint16_t            i;
    int16_t             slashpos1= -1;
    int16_t             slashpos2= -1;
    FGS_LNAVI_PARAMS_S  navipar;

    for ( i = 0; i < MAIN_VIEWER_PATHLEN; i++ )
    {
        if ( inst->path[i] == MENU_COMMON_PATH_SEPARATOR )
        {
            slashpos2 = slashpos1;
            slashpos1 = i;
        }

        if ( inst->path[i] == 0 )
        {
            break;
        }
    }

    if ( slashpos2 != -1 )
    {
        inst->path[slashpos2+1]= 0;
    }
    else
    {
        inst->path[0]= 0;
    }

    inst->currentdepth--;
    inst->leftbaseitem = 0;
    if ( inst->currentdepth == 0 )
    {
        MAIN_ViewerCopyVolumes (inst->partitions, inst->partitioncount,
                                inst->leftitem);
        itemcount = inst->partitioncount;
    }
    else
    {
        itemcount = MAIN_ViewerCheckContent (inst->path, inst->leftitem, 0,
                                             inst->params.filterType);
    }

    memset (&navipar, 0, sizeof(navipar));

    navipar.index    = inst->level[inst->currentdepth].currentitem;
    navipar.numItems = itemcount;
    navipar.posInVis = inst->level[inst->currentdepth].currenttop;

    (void)FGS_LNAVI_SetParams (inst->leftListHdl, &navipar);

    if ( inst->currentdepth == 0 )
    {
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
    }

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_LEFTTITLE);

    return FTRUE;
}

FSTATIC void MAIN_ViewerValidate (MAIN_VIEWER_INST_S* inst, char_t* initialPath)
{
    int32_t         retVal;
    SYS_DEVICE_E    device;
    uint16_t        pathCnt = 0;
    F_FIND          find;
    char_t          tmpPath[MAIN_VIEWER_PATHLEN + 3]; /* add space for '*.*' */
    uint32_t        strPos;
    uint16_t        partCnt;

    inst->path[0]      = 0;
    inst->currentdepth = 0;

    do
    {
        strPos = UTILS_strncpy (inst->path, initialPath, MAIN_VIEWER_PATHLEN);

        if ( strPos < 1 )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* append separator to end of path if not existing yet */
        if ( inst->path[strPos - 1] != MENU_COMMON_PATH_SEPARATOR )
        {
            if ( (strPos + 1) < MAIN_VIEWER_PATHLEN )
            {
                inst->path[strPos++] = MENU_COMMON_PATH_SEPARATOR;
                inst->path[strPos]   = 0;
            }
            else
            {
                retVal = SMARTGO_APPL_ERR_INTERNAL;
                break;
            }
        }

        /* ----- determine device corresponding to given path -------------- */

        retVal = SYS_DeviceGetByPath (inst->path, &device, &pathCnt);

        if ( retVal != FAPI_OK )
        {
            break;
        }

        /* search current device in partition list */

        for ( partCnt = 0; partCnt < inst->partitioncount; partCnt++ )
        {
            if ( inst->partitions[partCnt].device == device )
            {
                break;
            }
        }

        if ( partCnt >= inst->partitioncount )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        inst->level[inst->currentdepth].currentitem = partCnt;

        inst->currentdepth++;

        /* ----- check if path is valid ------------------------------------ */

        /* create temporary path with *.* appended */
        strPos = UTILS_strncpy (tmpPath, inst->path, MAIN_VIEWER_PATHLEN);

        (void)UTILS_strncpy (tmpPath + strPos, "*.*", 4);

        retVal = f_findfirst (tmpPath, &find);

        if ( retVal != F_NO_ERROR )
        {
            break;
        }

        /* ----- scan remaining path in order to determine depth ----------- */

        for ( ; pathCnt < MAIN_VIEWER_PATHLEN; pathCnt++ )
        {
            if ( inst->path[pathCnt] == 0 )
            {
                break;
            }
            else if ( inst->path[pathCnt] == MENU_COMMON_PATH_SEPARATOR )
            {
                if ( inst->currentdepth < MAIN_VIEWER_FOLDERLVL_MAX )
                {
                    inst->currentdepth++;
                }
                else
                {
                    retVal = SMARTGO_APPL_ERR_INTERNAL;
                    break;
                }
            }
        }
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        inst->path[0]      = 0;
        inst->currentdepth = 0;
    }
}


FSTATIC void MAIN_ViewerUpdate (MAIN_VIEWER_INST_S* inst)
{
    FGS_LNAVI_PARAMS_S      navipar;

    (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);

    inst->leftbaseitem= 0;

    if ( inst->currentdepth > 0 )
    {
        navipar.numItems = MAIN_ViewerCheckContent (inst->path, inst->leftitem,
                                                    0,
                                                    inst->params.filterType);
    }
    else
    {
        MAIN_ViewerCopyVolumes (inst->partitions, inst->partitioncount,
                                inst->leftitem);
        navipar.numItems= inst->partitioncount;
    }

    navipar.posInVis = inst->level[inst->currentdepth].currenttop;
    navipar.index    = inst->level[inst->currentdepth].currentitem;

    (void)FGS_LNAVI_SetParams(inst->leftListHdl, &navipar);
    (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_LEFTTITLE);

    MAIN_ViewerUpdateRight(inst, navipar.index - inst->leftbaseitem);
}

FSTATIC void MAIN_ViewerPrepareName (char_t* sourcepath, char_t* destpath,
                                     char_t* resultpath)
{
    uint16_t     i;
    uint16_t     seppos= 0;

    for ( i = 0; i < MAIN_VIEWER_PATHLEN; i++ )
    {
        if ( sourcepath[i] == 0 )
        {
            break;
        }
        else if ( sourcepath[i] == MENU_COMMON_PATH_SEPARATOR )
        {
            seppos = i;
        }
    }

    MENU_CommonAppendpath (&sourcepath[seppos+1], destpath, resultpath,
                           FFALSE, MAIN_VIEWER_PATHLEN);
}

void MAIN_ViewerDeviceUpdate (void)
{
    int32_t             error;
    char_t              path[MAIN_VIEWER_PATHLEN];
    FGS_LNAVI_PARAMS_S  navipar;
    SYS_DEVICE_E        device;
    F_FIND              find;
    MAIN_VIEWER_INST_S* inst = viewerDat;

    if ( inst->pnlHdl == NULL ) return;

    inst->partitioncount = MAIN_ViewerCheckPartitions (inst, inst->partitions);

    memset (&navipar, 0, sizeof(navipar));

    (void)FGS_LNAVI_GetParams (inst->leftListHdl, &navipar);

    MENU_CommonAppendpath (inst->leftitem[navipar.index-inst->leftbaseitem].
                           name, inst->path, path, FFALSE,
                           MAIN_VIEWER_PATHLEN);

    if ( SYS_DeviceGetByPath (path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    if((inst->operation == MAIN_VIEWER_OP_MOVE_SELECTED) ||
       (inst->operation == MAIN_VIEWER_OP_COPY_SELECTED))
    {
        /* Operation source path still exists ? */
        error = f_findfirst(inst->copypath,&find);
        if ( error )
        {
            /* abort operation and update title and help line */
            inst->operation = MAIN_VIEWER_OP_NORMAL;
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_ACTION);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_PROGRESS);
        }
    }

    error = f_findfirst(path,&find);           /* Current path still exists ? */
    if ( error )
    {
        (void)UTILS_strncpy(inst->path,
                            inst->partitions[0].path,
                            MAIN_VIEWER_PARTLEN_MAX);

        inst->currentdepth = 0;
    }

    if ( inst->currentdepth == 0 )
    {
        MAIN_ViewerUpdate (inst);
    }
}

#ifdef APPL_PVR_ENABLED

FSTATIC int32_t MAIN_ViewerResyncPvrList (SYS_DEVICE_E device)
{
    int32_t retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;

    switch ( device )
    {
        case SYS_DEV_HDD:
            retVal = PVRSYS_DeviceSyncDirectory (device, MAIN_DIR_HDD_PVR);
            break;

        default:
            retVal = PVRSYS_DeviceSyncDirectory (device, MAIN_DIR_OTH_PVR);
            break;
    }

    return retVal;
}

#endif /* APPL_PVR_ENABLED */

FSTATIC void MAIN_ViewerEditStringExit(MENU_EDITSTRING_DATA_S* editData, int32_t id)
{
    MAIN_VIEWER_INST_S* inst = viewerDat;

    if ( editData->updated )
    {
        (void)STR_Copy (editData->string, inst->renamestr, NULL,
                        SYS_CHARSET_ASCII, MAIN_VIEWER_NAMELEN);

        MAIN_ViewerRename(inst, id);
    }
    else
    {
        inst->operation= MAIN_VIEWER_OP_NORMAL;
    }
}

FSTATIC uint16_t MAIN_ViewerCheckPartitions (MAIN_VIEWER_INST_S* inst,
                                             MAIN_VIEWER_PARTITION* partitionP)
{
    uint16_t        count = 0;
    SYS_DEVICE_E    device;
    SYS_DEV_STAT_S  devStatus;
    int32_t         error;
    fbool_t         addDevice;

    for ( device = (SYS_DEVICE_E)0; device < SYS_DEV_NUMS; device++ )
    {
        addDevice = FFALSE;

        if ( SYS_DeviceGetStatus (device, &devStatus) == FAPI_OK )
        {
            partitionP[count].device = device;

            if ( devStatus.mountStatus == SYS_DEV_FS_MOUNTED )
            {
                /* set access status for available device if not already done */
                if ( !(inst->devAccessed[device]) )
                {
                    if ( SYS_DeviceSetAccessStatus (device, FTRUE) == FAPI_OK )
                    {
                        inst->devAccessed[device] = FTRUE;
                    }
                }

                (void)SYS_DeviceGetMountPath (device, partitionP[count].path,
                                              MAIN_VIEWER_PARTLEN_MAX);

                MENU_CommonAppendpath("", partitionP[count].path,
                                          partitionP[count].path, FTRUE,
                                          MAIN_VIEWER_PATHLEN);

                error = SYS_DeviceGetLabel (device, partitionP[count].name,
                                            MAIN_VIEWER_NAMELEN);

                if ( (error != FAPI_OK) || (partitionP[count].name[0] == 0) )
                {
                    MAIN_ViewerCopyName (partitionP[count].path,
                                         partitionP[count].name,
                                         MAIN_VIEWER_NAMELEN);
                }

                addDevice = FTRUE;
            }
            else
            {
                if ( devStatus.mountStatus == SYS_DEV_FS_UNFORMATTED )
                {
                    /* neverthelessly add device */
                    addDevice = FTRUE;
                }

                /* release access status for device if accessed before */
                if ( inst->devAccessed[device] )
                {
                    if ( SYS_DeviceSetAccessStatus (device, FFALSE) == FAPI_OK )
                    {
                        inst->devAccessed[device] = FFALSE;
                    }
                }
            }

            if ( addDevice )
            {
                switch ( device )
                {
                    #ifdef APPL_HDD_ENABLED

                    case SYS_DEV_HDD:
                        partitionP[count].icon = ICON_HARDDISKSMALL;
                        break;

                    #endif /* APPL_HDD_ENABLED */

                    #ifdef APPL_USB_ENABLED

                    case SYS_DEV_USB:
                        partitionP[count].icon = ICON_USBSMALL;
                        break;

                    #endif /* APPL_USB_ENABLED */

                    default:
                        continue;
                }

                partitionP[count].status = devStatus.mountStatus;

                count++;
            }
        }
    }

    return count;
}


FSTATIC void MAIN_ViewerCopyVolumes (MAIN_VIEWER_PARTITION* partitionP,
                                     uint16_t count, MAIN_VIEWER_ITEM* itemP)
{
    uint16_t     i;

    for ( i = 0; i < count; i++ )
    {
        itemP[i].icon           = partitionP[i].icon;
        itemP[i].isDir          = FTRUE;
        itemP[i].selectionMatch = FFALSE;
        itemP[i].enabled        = FTRUE;
        itemP[i].size.kbytes    = 0;
        itemP[i].size.bytes     = 0;

        (void)UTILS_strncpy(itemP[i].name, partitionP[i].name,
                            MAIN_VIEWER_NAMELEN);
    }
}


FSTATIC void MAIN_ViewerSwitchInfo (MAIN_VIEWER_INST_S* inst, fbool_t info)
{
    if ( info )
    {
        (void)STR_BufClear(&(inst->infoLine1));
        (void)STR_BufClear(&(inst->infoLine2));
        (void)STR_BufClear(&(inst->infoLine3));

        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_INFO1);
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_INFO2);
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_INFO3);

        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl, MAIN_VIEWER_ID_RIGHT,
                                            FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl, MAIN_VIEWER_ID_INFO1,
                                            FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl, MAIN_VIEWER_ID_INFO2,
                                            FFALSE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl, MAIN_VIEWER_ID_INFO3,
                                            FFALSE);
    }
    else
    {
        (void)FGS_PANEL_SetRefreshMode (inst->pnlHdl, FFALSE);

        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl, MAIN_VIEWER_ID_INFO1,
                                            FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl, MAIN_VIEWER_ID_INFO2,
                                            FTRUE);
        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl, MAIN_VIEWER_ID_RIGHT,
                                            FFALSE);

        (void)FGS_PANEL_SetRefreshMode (inst->pnlHdl, FTRUE);

        (void)FGS_PANEL_SetMemberHideStatus(inst->pnlHdl, MAIN_VIEWER_ID_INFO3,
                                            FTRUE);
    }
}


FSTATIC fbool_t MAIN_ViewerCheckFileType (uint32_t* filterType,
                                          uint32_t type)
{
    uint16_t    cnt;
    fbool_t     retVal = FFALSE;

    for ( cnt = 0; cnt < MAIN_VIEWER_NUM_FILTER_TYPES; cnt++ )
    {
        if ( filterType[cnt] == 0 )
        {
            break;
        }
        else if ( (filterType[cnt] == type)
               || (filterType[cnt] == MAIN_VIEWER_FILTYPE_ALL) )
        {
            retVal = FTRUE;
            break;
        }
    }

    return retVal;
}

FSTATIC uint16_t MAIN_ViewerCheckContent (char_t* path,
                                          MAIN_VIEWER_ITEM* itemP,
                                          uint16_t base,
                                          uint32_t* filterType)
{
    uint16_t                 count    = 1;
    uint16_t                 countabs = 1;
    uint16_t                 n, i;
    uint32_t                 type = 0;
    F_FIND                   find;
    SYS_DEVICE_E             device;
    int32_t                  retVal;
    uint32_t                 strPos;
    char_t                   searchPath[MAIN_VIEWER_PATHLEN + 3];

    #ifdef APPL_PVR_ENABLED
    char_t                   fullpath[MAIN_VIEWER_PATHLEN];
    #endif /* APPL_PVR_ENABLED */

    strPos = UTILS_strncpy (searchPath, path, MAIN_VIEWER_PATHLEN);

    /* append wildcard characters */
    (void)UTILS_strncpy (searchPath + strPos, "*.*",
                         MAIN_VIEWER_PATHLEN + 3 - strPos);

    if ( base == 0 )
    {
        itemP[0].name[0]        = '.';
        itemP[0].name[1]        = '.';
        itemP[0].name[2]        = 0;
        itemP[0].icon           = ICON_FOLDERSMALL;
        itemP[0].isDir          = FTRUE;
        itemP[0].selectionMatch = FFALSE;
        itemP[0].enabled        = FTRUE;
        itemP[0].size.kbytes    = 0;
        itemP[0].size.bytes     = 0;
    }

    if ( SYS_DeviceGetByPath (path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    retVal = f_findfirst(searchPath, &find);

    while ( retVal == F_NO_ERROR )
    {
        if ( (find.filename[0] != 0)
          && (find.filename[0] != '.')
          && (!(find.attr & F_ATTR_HIDDEN)) )
        {
            if( (count >= base) &&
                (count < (uint16_t)(base + MAIN_VIEWER_DISPITEMS)) )
            {
                n = UTILS_strncpy (itemP[count-base].name, find.filename,
                                   MAIN_VIEWER_NAMELEN);

                SYS_SizeClear(&(itemP[count-base].size));
                SYS_SizeAdd  (&(itemP[count-base].size), find.filesize / 1024,
                                                         find.filesize % 1024);

                if ( find.attr & F_ATTR_DIR ) /* is directory */
                {
                    #ifdef APPL_PVR_ENABLED

                    MENU_CommonAppendpath(find.filename, path, fullpath,
                                          FFALSE, MAIN_VIEWER_PATHLEN);

                    if ( PVR_LIST_GetItemByPath(fullpath, NULL) == FAPI_OK )
                    {
                        itemP[count-base].icon           = ICON_MOVIESMALL;
                        itemP[count-base].selectionMatch =
                            MAIN_ViewerCheckFileType(filterType,
                                                     MAIN_VIEWER_FILTYPE_ALL);
                        itemP[count-base].enabled        = itemP[count-base].
                                                           selectionMatch;
                    }
                    else
                    {

                    #endif /* APPL_PVR_ENABLED */

                        itemP[count-base].icon           = ICON_FOLDERSMALL;
                        itemP[count-base].selectionMatch = FFALSE;
                        itemP[count-base].enabled        = FTRUE;

                    #ifdef APPL_PVR_ENABLED
                    }
                    #endif /* APPL_PVR_ENABLED */

                    itemP[count-base].isDir          = FTRUE;
                }
                else /* is file */
                {
                    type = 0;
                    if ( n >= 4 )
                    {
                        for(i= 0; i<3; i++)
                        {
                            type = (type << 8) | find.ext[i];
                        }
                    }

                    switch ( type )
                    {
                        #ifdef APPL_JPEG_ENABLED

                        case MAIN_VIEWER_FILTYPE_JPG_LC: //'jpg':
                        case MAIN_VIEWER_FILTYPE_JPG_UC: //'JPG':
                            itemP[count-base].icon = ICON_PICTURESMALL;
                            break;

                        case MAIN_VIEWER_FILTYPE_BMP_LC: //'bmp':
                        case MAIN_VIEWER_FILTYPE_BMP_UC: //'BMP':
                            itemP[count-base].icon = ICON_PICTURESMALL;
                            break;

                        case MAIN_VIEWER_FILTYPE_PNG_LC: //'png':
                        case MAIN_VIEWER_FILTYPE_PNG_UC: //'PNG':
                            itemP[count-base].icon= ICON_PICTURESMALL;
                            break;

                        #endif /* APPL_JPEG_ENABLED */

                        #ifdef APPL_MP3_ENABLED

                        case MAIN_VIEWER_FILTYPE_MP3_LC: //'mp3':
                        case MAIN_VIEWER_FILTYPE_MP3_UC: //'MP3':
                            itemP[count-base].icon = ICON_MUSICSMALL;
                            break;

                        #endif /* APPL_MP3_ENABLED */

                        #ifdef APPL_FILEUPDATE_ENABLED

                        case MAIN_VIEWER_FILTYPE_SSU_LC: //'ssu':
                        case MAIN_VIEWER_FILTYPE_SSU_UC: //'SSU':
                            itemP[count-base].icon = ICON_UPDATESMALL;
                            break;

                        #endif /* APPL_FILEUPDATE_ENABLED */

                        default:
                            itemP[count-base].icon = ICON_FILESMALL;
                            break;
                    }

                    itemP[count-base].isDir          = FFALSE;
                    itemP[count-base].selectionMatch =
                        MAIN_ViewerCheckFileType(filterType, type);
                    itemP[count-base].enabled        = itemP[count-base].
                                                       selectionMatch;

                }
                #if 0
                /* NOTE: might be used in the future for invalid file types */
                else
                {
                    itemP[count-base].icon           = ICON_BTNREDSMALL;
                    itemP[count-base].isDir          = FFALSE;
                    itemP[count-base].selectionMatch = FFALSE;
                    itemP[count-base].enabled        = FFALSE;
                }
                #endif
            }

            count++;
            countabs++;
        }

        retVal = f_findnext(&find);
    }
    return countabs;
}


FSTATIC void MAIN_ViewerConfirmDelete(MAIN_VIEWER_INST_S * inst)
{
    MENU_CONFDIALOG_S       confPars;
    FGS_LNAVI_PARAMS_S      navipar;

    (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);

    memset(&confPars, 0, sizeof(MENU_CONFDIALOG_S));

    confPars.numDescLines = 1;

    switch ( inst->leftitem[navipar.index-inst->leftbaseitem].icon )
    {
        case ICON_FOLDERSMALL:
            confPars.titleStr = OSD_GetStr(STR_ENTRY_VIEWER_DELETE_FOLDER);
            break;

        case ICON_MOVIESMALL:
            confPars.titleStr = OSD_GetStr(STR_ENTRY_VIEWER_DELETE_RECORDING);
            break;

        default:
            confPars.titleStr = OSD_GetStr(STR_ENTRY_VIEWER_DELETE_FILE);
            break;
    }

    confPars.confirmStr   = OSD_GetStr(STR_ENTRY_VIEWER_DELETE_OK);
    confPars.discardStr   = OSD_GetStr(STR_ENTRY_VIEWER_DELETE_CANCEL);
    confPars.onConfirm    = MAIN_ViewerDeleteOK;
    confPars.onDecline    = NULL;
    confPars.onDiscard    = NULL;
    confPars.arg          = (void*)inst;

    (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
}


FSTATIC void MAIN_ViewerDeleteOK(void* arg)
{
    MAIN_VIEWER_INST_S*     inst= arg;
    char_t                  path[MAIN_VIEWER_PATHLEN];
    FGS_LNAVI_PARAMS_S      navipar;

    (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);

    MENU_CommonAppendpath(inst->leftitem[navipar.index-inst->leftbaseitem].name,
                          inst->path, path, FFALSE, MAIN_VIEWER_PATHLEN);

    FAPI_SYS_PRINT_DEBUG(1,"Delete: %s\n", path);

    inst->operation= MAIN_VIEWER_OP_DELETE_PROGRESS;

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_ACTION);
    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_PROGRESS);

    (void)FILEOP_Delete(path);
}

FSTATIC void MAIN_ViewerConfirmOverwrite(MAIN_VIEWER_INST_S * inst)
{
    MENU_CONFDIALOG_S       confPars;
    FGS_LNAVI_PARAMS_S      navipar;

    (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);

    memset(&confPars, 0, sizeof(MENU_CONFDIALOG_S));

    confPars.numDescLines = 1;

    switch ( inst->copyIconType)
    {
        case ICON_FOLDERSMALL:
            confPars.titleStr = OSD_GetStr(STR_ENTRY_VIEWER_OVERWRITE_FOLDER);
            break;
        #ifdef APPL_MP3_ENABLED
        case ICON_MUSICSMALL:
        #endif /* APPL_MP3_ENABLED */
        case ICON_FILESMALL:
        #ifdef APPL_JPEG_ENABLED
        case ICON_PICTURESMALL:
        #endif /*APPL_JPEG_ENABLED*/
        #ifdef APPL_FILEUPDATE_ENABLED
        case ICON_UPDATESMALL:
        #endif /* APPL_FILEUPDATE_ENABLED */
        default:
            confPars.titleStr = OSD_GetStr(STR_ENTRY_VIEWER_OVERWRITE_FILE);
            break;
    }

    confPars.confirmStr   = OSD_GetStr(STR_ENTRY_VIEWER_OVERWRITE_OK);
    confPars.discardStr   = OSD_GetStr(STR_ENTRY_VIEWER_OVERWRITE_ABORT);
    confPars.onConfirm    = MAIN_ViewerOverwriteOK;
    confPars.onDecline    = NULL;
    confPars.onDiscard    = NULL;
    confPars.arg          = (void*)inst;

    (void)MENU_ConfDialogOpen(&(appDat->confDialog), &confPars);
}


FSTATIC void MAIN_ViewerOverwriteOK(void* arg)
{
    MAIN_VIEWER_INST_S*     inst= arg;
    char_t                  path[MAIN_VIEWER_PATHLEN];
    FGS_LNAVI_PARAMS_S      navipar;

    (void)FGS_LNAVI_GetParams(inst->leftListHdl, &navipar);

    switch(inst->operation)
    {
        case MAIN_VIEWER_OP_MOVE_SELECTED:
            inst->operation= MAIN_VIEWER_OP_MOVE_PROGRESS;

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_ACTION);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_PROGRESS);

            MAIN_ViewerPrepareName(inst->copypath,inst->path, path);

            FAPI_SYS_PRINT_DEBUG(1,"Move to: %s\n", path);
            (void)FILEOP_Move(inst->copypath, path);
            break;

        case MAIN_VIEWER_OP_COPY_SELECTED:
            inst->operation= MAIN_VIEWER_OP_COPY_PROGRESS;

            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_TITLE);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELP4);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_HELPCOLOR);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_ACTION);
            (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MAIN_VIEWER_ID_PROGRESS);

            MAIN_ViewerPrepareName(inst->copypath,inst->path, path);

            FAPI_SYS_PRINT_DEBUG(1,"Copy to: %s\n", path);
            (void)FILEOP_Copy(inst->copypath, path);
            break;

        default:
            break;
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
