/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_satellites.c
**
**              $Id: app_satellites.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Default satellite/carrier definitions.
**
** Detail description. [\ref app_smartgo_app_ref_satellites]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */
//#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "sys_api.h"
#include "app_satellites.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

#ifdef APPL_FESAT_ENABLED

FSTATIC TUNE_ITEM_SAT_S crAstra[] =
{
    { 10773000, 22000, FE_POL_HOR, FE_MOD_QPSK   }, /* TVP-HD */
    { 11303000, 22000, FE_POL_HOR, FE_MOD_8PSK   }, /* ARTE-HD */
    { 11836500, 27500, FE_POL_HOR, FE_MOD_QPSK   }, /* ARD I */
    { 12110000, 27500, FE_POL_HOR, FE_MOD_QPSK   }, /* ARD   */
  //{ 12109500, 27500, FE_POL_HOR, FE_MOD_QPSK   }, /* ARD II */
    { 11954000, 27500, FE_POL_HOR, FE_MOD_QPSK   }, /* ZDF */
    { 12187500, 27500, FE_POL_HOR, FE_MOD_QPSK   }, /* RTL */
    { 12545000, 22000, FE_POL_HOR, FE_MOD_QPSK   }, /* Pro7, N24 */
    { 12551000, 22000, FE_POL_VER, FE_MOD_QPSK   }, /* Bloomberg */
    { 11597000, 22000, FE_POL_VER, FE_MOD_QPSK   }, /* Eurosport, DW-TV */
    { 11720000, 27500, FE_POL_HOR, FE_MOD_QPSK   }, /* Premiere */
    { 11758000, 27500, FE_POL_HOR, FE_MOD_QPSK   }, /* Premiere */
    { 12285000, 27500, FE_POL_VER, FE_MOD_QPSK   }, /* CanalSatellite France */
    { 12246000, 27500, FE_POL_VER, FE_MOD_QPSK   }, /* APS */
    { 12692000, 22000, FE_POL_HOR, FE_MOD_QPSK   }, /* ORF */
    { 11914000, 27500, FE_POL_HOR, FE_MOD_QPSK_S2}, /* Premiere HD / HD promo */
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK   }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crHotbird[] =
{
    { 10723000, 29900, FE_POL_HOR, FE_MOD_QPSK },
    { 10775000, 28000, FE_POL_HOR, FE_MOD_QPSK },
    { 10853000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    { 10892000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    { 10949000, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 10992000, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 11034000, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 11075000, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 11137000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    { 11604000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    { 11747000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    { 11804000, 27500, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crEutelsatW2[] =
{
    { 11293000, 13333, FE_POL_HOR, FE_MOD_QPSK },
    { 11304000, 30000, FE_POL_VER, FE_MOD_QPSK },
    { 11634000, 17578, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crEutelsatW1[] =
{
    { 11126000,  4340, FE_POL_HOR, FE_MOD_QPSK },
    { 12629000,  5632, FE_POL_VER, FE_MOD_QPSK },
    { 12717000,  7036, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crEutelsatW3A[] =
{
    { 11175000, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 11128300, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 11324000, 27500, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crSirius[] =
{
    { 12380000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    { 12680000,  9766, FE_POL_HOR, FE_MOD_QPSK },
    { 12697000, 14685, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crTelecom2C[] =
{
    { 12508000,  6285, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crIntelsat1002[] =
{
    { 12527000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crAtlanticBird3[] =
{
    { 11054000, 29950, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crAtlanticBird2[] =
{
    { 11010000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crIntelsat907[] =
{
    { 11555000, 30000, FE_POL_HOR, FE_MOD_QPSK_S2 },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crHispasat1C1D[] =
{
    { 11682000,  3800, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crAstra1D3A[] =
{
    { 11515000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crArabsat[] =
{
    { 11919000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crEurobird1[] =
{
    { 11426000, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 11474000,  3079, FE_POL_VER, FE_MOD_QPSK },
    { 11681000, 27500, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crAstra2[] =
{
    { 10714000, 22000, FE_POL_HOR, FE_MOD_QPSK }, /* freesat */
    { 10729000, 22000, FE_POL_VER, FE_MOD_QPSK }, /* freesat */
    { 10758000, 22000, FE_POL_VER, FE_MOD_QPSK }, /* freesat */
    { 10773000, 22000, FE_POL_HOR, FE_MOD_QPSK }, /* freesat */
    { 10788000, 22000, FE_POL_VER, FE_MOD_QPSK }, /* freesat */
    { 10803000, 22000, FE_POL_HOR, FE_MOD_QPSK }, /* freesat */
    { 10818000, 22000, FE_POL_VER, FE_MOD_QPSK }, /* freesat */
    { 10832000, 22000, FE_POL_HOR, FE_MOD_QPSK }, /* freesat */
    { 10847000, 22000, FE_POL_VER, FE_MOD_QPSK }, /* freesat */
    { 10891000, 22000, FE_POL_HOR, FE_MOD_QPSK }, /* freesat */
    { 10906000, 22000, FE_POL_VER, FE_MOD_QPSK }, /* freesat */

    { 10744000, 22000, FE_POL_HOR, FE_MOD_QPSK }, /* skydigital */
    { 10862000, 22000, FE_POL_HOR, FE_MOD_QPSK }, /* skydigital */
    { 10876000, 22000, FE_POL_HOR, FE_MOD_QPSK }, /* skydigital */
    { 12480000, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 12032000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crArabsat2B[] =
{
    { 12740000,  5084, FE_POL_VER, FE_MOD_QPSK },
    { 12520000,  3000, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crEurobird3[] =
{
    { 11104000,  1785, FE_POL_HOR, FE_MOD_QPSK },
    { 11098000,  2441, FE_POL_HOR, FE_MOD_QPSK },
    { 11110000,  3124, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crTurksat1C[] =
{
    { 11694000,  4444, FE_POL_HOR, FE_MOD_QPSK },
    { 11162000,  2222, FE_POL_VER, FE_MOD_QPSK },
    { 11129000,  4268, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crEutelsatSesat[] =
{
    { 11647000,  5000, FE_POL_HOR, FE_MOD_QPSK_S2 },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crPas1R[] =
{
    { 11510000, 26693, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crThor[] =
{
    { 11372000, 24500, FE_POL_VER, FE_MOD_QPSK },
    { 11823000, 28000, FE_POL_HOR, FE_MOD_QPSK },
    { 12399000, 28000, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crNilesat[] =
{
    { 11747000, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 11823000, 27500, FE_POL_VER, FE_MOD_QPSK },
    { 12034000, 27500, FE_POL_HOR, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

FSTATIC TUNE_ITEM_SAT_S crAtlanticBird1[] =
{
    { 11595000, 27500, FE_POL_VER, FE_MOD_QPSK },
    {        0,     0, FE_POL_HOR, FE_MOD_QPSK }  /* End of list: mandatory! */
};

APP_SATPAR_S satLst[SATID_NUMS] =
{
  { SATID_HOTBIRD,         "Hotbird"        , -1, 130, FE_WE_FLAG_EAST, crHotbird       },
  { SATID_ASTRA,           "ASTRA"          , -1, 192, FE_WE_FLAG_EAST, crAstra         },
  { SATID_EUTELSAT_W2,     "Eutelsat W2"    , -1, 160, FE_WE_FLAG_EAST, crEutelsatW2    },
  { SATID_EUTELSAT_W1,     "Eutelsat W1"    , -1, 100, FE_WE_FLAG_EAST, crEutelsatW1    },
  { SATID_EUTELSAT_W3A,    "Eutelsat W3A"   , -1,  70, FE_WE_FLAG_EAST, crEutelsatW3A   },
  { SATID_SIRIUS,          "Sirius"         , -1,  50, FE_WE_FLAG_EAST, crSirius        },
  { SATID_TELECOM_2C,      "Telecom 2C"     , -1,  30, FE_WE_FLAG_EAST, crTelecom2C     },
  { SATID_INTELSAT_10_02,  "Intelsat 10-02" , -1,  10, FE_WE_FLAG_WEST, crIntelsat1002  },
  { SATID_ATLANTIC_BIRD_3, "Atlantic Bird 3", -1,  50, FE_WE_FLAG_WEST, crAtlanticBird3 },
  { SATID_ATLANTIC_BIRD_2, "Atlantic Bird 2", -1,  80, FE_WE_FLAG_WEST, crAtlanticBird2 },
  { SATID_INTELSAT_907,    "Intelsat 907"   , -1, 275, FE_WE_FLAG_WEST, crIntelsat907   },
  { SATID_HISPASAT_1C_1D,  "Hispasat 1C/1D" , -1, 300, FE_WE_FLAG_WEST, crHispasat1C1D  },
  { SATID_ASTRA_1D_3A,     "ASTRA 1D/3A"    , -1, 235, FE_WE_FLAG_EAST, crAstra1D3A     },
  { SATID_ARABSAT,         "Arabsat"        , -1, 260, FE_WE_FLAG_EAST, crArabsat       },
  { SATID_EUROBIRD_1,      "Eurobird 1"     , -1, 285, FE_WE_FLAG_EAST, crEurobird1     },
  { SATID_ASTRA_2,         "ASTRA 2"        , -1, 282, FE_WE_FLAG_EAST, crAstra2        },
  { SATID_ARABSAT_2B,      "Arabsat 2B"     , -1, 305, FE_WE_FLAG_EAST, crArabsat2B     },
  { SATID_EUROBIRD_3,      "Eurobird 3"     , -1, 330, FE_WE_FLAG_EAST, crEurobird3     },
  { SATID_TURKSAT_1C,      "Trksat 1C"     , -1, 420, FE_WE_FLAG_EAST, crTurksat1C     },
  { SATID_EUTELSAT_SESAT,  "Eutelsat Sesat" , -1, 360, FE_WE_FLAG_EAST, crEutelsatSesat },
  { SATID_PAS_1_R,         "Pas 1 R"        , -1, 450, FE_WE_FLAG_WEST, crPas1R         },
  { SATID_THOR,            "Thor"           , -1,   8, FE_WE_FLAG_WEST, crThor          },
  { SATID_NILESAT,         "Nilesat"        , -1,  70, FE_WE_FLAG_WEST, crNilesat       },
  { SATID_ATLANTIC_BIRD_1, "Atlantic Bird 1", -1, 125, FE_WE_FLAG_WEST, crAtlanticBird1 },
};

#endif /* APPL_FESAT_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
