/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_servbox.c
**
**              $Id: app_servbox.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Service selection item.
**
** Detail description. [\ref app_smartgo_app_ref_servbox]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define CHANLIST_NUMLINES          9

enum
{
    OSD_CHANLST_ID_NAVI = 0,
    OSD_CHANLST_ID_TITLE,
    OSD_CHANLST_ID_HELPBOTTOM

};

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void  MAIN_ServBoxOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC int32_t  MAIN_ServBoxDrawTitle  (FAPI_SYS_HandleT h);
FSTATIC fbool_t MAIN_ServBoxKeys       (FAPI_SYS_HandleT h);
FSTATIC fbool_t MAIN_ServBoxShow       (MENU_SERVBOX_INST_S* inst, int16_t centerIdx);
FSTATIC void  MAIN_ServBoxHide       (MENU_SERVBOX_INST_S* inst);
FSTATIC void  MAIN_ServBoxItemGet    (FAPI_SYS_HandleT hdl, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item);
FSTATIC void MAIN_ServBoxOnSelect    (FAPI_SYS_HandleT h, int32_t id, uint16_t index);

/*---------------------------------------------------------------------------*/
/* main function                                                             */
/*---------------------------------------------------------------------------*/

int32_t MAIN_ServBoxInit (MENU_SERVBOX_INST_S* inst)
{
    memset(inst, 0, sizeof(MENU_SERVBOX_INST_S));

    return FAPI_OK;
}

int32_t MAIN_ServBoxOpen (MENU_SERVBOX_INST_S* inst,
                       const MAIN_SERVBOX_OPEN_S* openParams)
{
    int16_t index;

    #ifdef APPL_LCN_ENABLED
    SLIST_PARAMS_S slistParams;
    #endif /* APPL_LCN_ENABLED */

    if ( (inst->pnlHdl != NULL) ||
         (openParams   == NULL) ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    inst->params      = *openParams;
    inst->naviHdl     = NULL;
    inst->pTitleItem  = NULL;
    inst->currServLst = inst->params.initialListId;

    #ifdef APPL_PVR_ENABLED
    inst->numRecords  = (inst->params.recordIsAvailable != NULL) ? RSRC_REC_MAX
                                                                 : 0;
    #else
    inst->numRecords = 0;
    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_LCN_ENABLED

    if ( SLIST_GetParams(&slistParams) == FAPI_OK )
    {
        inst->showLcn = slistParams.sortByLcn;
    }
    else
    {
        inst->showLcn = FFALSE;
    }

    #endif /* APPL_LCN_ENABLED */

    /* Set default service list if no list has been passed */
    if ( (inst->currServLst == SLIST_NONE) ||
         (inst->currServLst == SLIST_INVISIBLE) )
    {
        inst->currServLst = SLIST_TV;
    }

    index = inst->params.getInitialIndexForList(inst->currServLst);

    if ( MAIN_ServBoxShow(inst, index) == FFALSE )
    {
        return SMARTGO_APPL_ERR_INTERNAL;
    }

    return FAPI_OK;
}

int32_t MAIN_ServBoxClose (MENU_SERVBOX_INST_S* inst)
{
    if ( inst->pnlHdl == 0 ) return FAPI_OK;

    MAIN_ServBoxHide(inst);

    return FAPI_OK;
}

int32_t MAIN_ServBoxSetServLoc (MENU_SERVBOX_INST_S* inst,
                             const SYS_SERVLOC_S* pServLoc)
{
    int32_t               retVal;
    FGS_LNAVI_PARAMS_S naviParams;

    if ( inst->pnlHdl == 0 ) return SMARTGO_APPL_ERR_INTERNAL;

    if ( pServLoc->listId != inst->currServLst )
    {
        MAIN_ServBoxHide(inst);
        inst->currServLst = pServLoc->listId;
        (void)MAIN_ServBoxShow(inst, pServLoc->index);

        retVal = FAPI_OK;
    }
    else
    {
        retVal = FGS_LNAVI_GetParams(inst->naviHdl, &naviParams);

        if ( retVal == FAPI_OK )
        {
            naviParams.index = pServLoc->index + inst->numRecords;

            retVal = FGS_LNAVI_SetParams (inst->naviHdl, &naviParams);
        }
    }

    return retVal;
}

int32_t MAIN_ServBoxGetCurrList (MENU_SERVBOX_INST_S* inst, SLIST_TYPE* listId)
{
    if ( inst->pnlHdl == 0 ) return SMARTGO_APPL_ERR_INTERNAL;

    *listId = inst->currServLst;
    return FAPI_OK;
}

int32_t MAIN_ServBoxGetCurrIndex (MENU_SERVBOX_INST_S* inst, int16_t* index)
{
    FGS_LNAVI_PARAMS_S naviParams;
    int32_t               retVal;

    if ( inst->pnlHdl == 0 ) return SMARTGO_APPL_ERR_INTERNAL;

    retVal = FGS_LNAVI_GetParams(inst->naviHdl, &naviParams);

    if ( retVal == FAPI_OK )
    {
        if ( naviParams.index < inst->numRecords )
        {
            *index = -1 - naviParams.index;
        }
        else
        {
            *index = (uint16_t)(naviParams.index - inst->numRecords);
        }
    }

    return retVal;
}

int32_t MAIN_ServBoxUpdate (MENU_SERVBOX_INST_S* inst)
{
    if ( inst->pnlHdl == 0 ) return FAPI_OK;

    (void)FGS_PANEL_RefreshMember(inst->pnlHdl, OSD_CHANLST_ID_NAVI);

    return FAPI_OK;
}

FSTATIC fbool_t MAIN_ServBoxKeys (FAPI_SYS_HandleT h)
{
    MENU_SERVBOX_INST_S* inst = (MENU_SERVBOX_INST_S*)h;
    int16_t                  centerIdx;
    fbool_t                keyUsed  = FTRUE;

    MAIN_ServBoxHide(inst);
    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_RED:
            inst->currServLst = SLIST_TV;
            break;

        case FGS_KEY_GREEN:
            inst->currServLst = SLIST_RADIO;
            break;

        case FGS_KEY_YELLOW:
            if ( ((int16_t)inst->currServLst >= SLIST_OFFSET_USR) &&
                 (inst->currServLst < (SLIST_OFFSET_USR +
                                       SLIST_USR_NUMLISTS))  )
            {
                inst->currServLst++;

                if ( inst->currServLst >= (SLIST_OFFSET_USR +
                                           SLIST_USR_NUMLISTS) )
                {
                    inst->currServLst = SLIST_OFFSET_USR;
                }
            }
            else
            {
                inst->currServLst = SLIST_OFFSET_USR;
            }
            break;

        default:
            keyUsed = FFALSE;
            break;
    }
    centerIdx = inst->params.getInitialIndexForList(inst->currServLst);
    (void)MAIN_ServBoxShow(inst, centerIdx);

    return keyUsed;
}

/*
    centerIdx: 0...(n-1): service index
                      -1: Rec1
                      -2: Rec2
*/
FSTATIC fbool_t MAIN_ServBoxShow (MENU_SERVBOX_INST_S* inst, int16_t centerIdx)
{
    int32_t            retVal;
    uint16_t           numServ;
    FGS_PANEL_OPEN_S   pnlPars;
    FGS_LNAVI_OPEN_S   naviDat;
    FGS_LNAVI_PARAMS_S naviParams;
    FGS_PANEL_MEMBER_S member;
    uint16_t           visLines;
    uint8_t            numHelpLines =3;
    FAPI_SYS_HandleT   itemHdl;
    FGS_BOX_OPEN_S     boxPars;
    FGS_ITEM_S         item;
    uint16_t           idCnt;
    uint16_t           mainPanelWidth;
    uint16_t           bottomHelpKeyWidth;
    uint16_t           bottomHelpItemWidth;

    if ( inst->pnlHdl != NULL ) return FFALSE; /* test if already open */

    memset(&pnlPars,  0, sizeof(pnlPars));
    memset(&naviDat,  0, sizeof(naviDat));
    memset(&member,   0, sizeof(member));
    memset(&boxPars,  0, sizeof(boxPars));
    memset(&item,     0, sizeof(item));
    do
    {
        /* ----- Create Channel list items --------------------------------- */

        numServ = SLIST_ListSizeGet(inst->currServLst) + inst->numRecords;

        if      ( numServ == 0 )                visLines = 1;
        else if ( numServ < CHANLIST_NUMLINES ) visLines = numServ;
        else                                    visLines = CHANLIST_NUMLINES;

        pnlPars.pos.x          = -(int16_t)(SCALER_GetCurrVal(MAIN_PAD_X) +
                                  SCALER_GetCurrVal(CHANLIST_WIDTH));
        pnlPars.pos.y          = 0; // is ignored due to centered position!
        pnlPars.pos.width      = SCALER_GetCurrVal(CHANLIST_WIDTH);
        pnlPars.pos.height     = OSD_GetListHeight (visLines) +
                                (numHelpLines * SCALER_GetCurrVal(HELP_LINE_HEIGHT)) +
                                (numHelpLines * SCALER_GetCurrVal(HELP_LINE_PADDING)) +
                                (2*SCALER_GetCurrVal(INNER_FRAME_WIDTH)) +
                                (2*SCALER_GetCurrVal(OUTER_FRAME_ALL)) +
                                (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        pnlPars.pos.alignment  = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
        pnlPars.bkgrType       = OSD_PNLTYPE_MENUWTITLEWBOTHELP3;
        pnlPars.onFocusSet     = MAIN_ServBoxOnFocusSet;
        pnlPars.onFocusSetArg  = (FAPI_SYS_HandleT)inst;
        pnlPars.pInst          = &(appDat->pnlInst);

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_CHANLIST, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        naviDat.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        naviDat.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH)+
                                  (2*SCALER_GetCurrVal(TITLE_TB_PADDING)) +
                                  SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        naviDat.pos.width       = pnlPars.pos.width -
                                  2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        naviDat.pos.height      = OSD_GetListHeight (visLines);
        naviDat.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        naviDat.bkgrType        = OSD_NAVIBKGR_NORMAL |
                                  OSD_CORNER_BOT_LEFT | OSD_CORNER_BOT_RIGHT;
        naviDat.type            = FGS_LNAVI_TYPE_FIXEDPOS;
        naviDat.rowHeight       = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        naviDat.scrollbarWidth  = 0;
        naviDat.numVisLines     = visLines;
        naviDat.numColumns      = 3;
        naviDat.params.numItems = (numServ < 1) ? 1 : numServ;
        naviDat.params.index    = 0;
        naviDat.params.posInVis = visLines / 2;

        naviDat.colStart[0]     = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        naviDat.colWidth[0]     = SCALER_GetCurrVal(INFO_NUM_WIDTH);
        naviDat.colStart[1]     = naviDat.colStart[0] + naviDat.colWidth[0];
        naviDat.colWidth[1]     = SCALER_GetCurrVal(MAIN_ICON_WIDTH);
        naviDat.colStart[2]     = naviDat.colStart[1] + naviDat.colWidth[1];
        naviDat.colWidth[2]     = naviDat.pos.width - naviDat.colStart[2] -
                                  naviDat.scrollbarWidth -
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

        naviDat.topDownOrient   = FFALSE;
        naviDat.wraparound      = FTRUE;
        naviDat.getItem         = MAIN_ServBoxItemGet;
        naviDat.getItemArg      = (FAPI_SYS_HandleT)inst;
        naviDat.onSelect        = MAIN_ServBoxOnSelect;
        naviDat.onSelectArg     = (FAPI_SYS_HandleT)inst;
        naviDat.pInst           = &(appDat->lnaviInst);

        retVal = FGS_LNAVI_Open (&naviDat, OSD_CHANLST_ID_NAVI,
                                 inst->pnlHdl,
                                &(inst->naviHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        inst->titlePos.x         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->titlePos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->titlePos.width     = pnlPars.pos.width -
                                   2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->titlePos.height    = SCALER_GetCurrVal(TITLE_LINE_HEIGHT)+
                                   (2*SCALER_GetCurrVal(TITLE_TB_PADDING))+
                                   SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->titlePos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->titlePos);
        member.id            = OSD_CHANLST_ID_TITLE;
        member.isHidden      = FFALSE;
        member.show          = MAIN_ServBoxDrawTitle;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pTitleItem));
        if ( retVal != FAPI_OK ) break;

        /* -----Bottom help lines ------------------------------------------ */
        mainPanelWidth          = pnlPars.pos.width - 2*SCALER_GetCurrVal(OUTER_FRAME_ALL);
        bottomHelpKeyWidth      = OSD_GetStrWidth(OSD_GetStr(STR_KEY_MENU),-1,HELP_FONT) +
                                  SCALER_GetCurrVal(MAIN_RIGHT_PADDING);;
        bottomHelpItemWidth     = mainPanelWidth/2 - bottomHelpKeyWidth - SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

        /* Red icon */
        idCnt = OSD_CHANLST_ID_HELPBOTTOM;

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNREDSMALL);

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = - (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL)/2
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - numHelpLines*SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                                      - 2*SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SERVBOX_TV);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* green icon */
        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNGREENSMALL);

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) + mainPanelWidth/2;
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SERVBOX_RADIO);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

       /* -- 2nd Bottom line -- */
       /* yellow icon */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT)
                       + SCALER_GetCurrVal(HELP_LINE_PADDING);

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNYELLOWSMALL);

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_SERVBOX_FAV);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr,-1, HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

       /* -- 3rd Bottom line -- */
        boxPars.pos.y += SCALER_GetCurrVal(HELP_LINE_HEIGHT) + SCALER_GetCurrVal(HELP_LINE_PADDING);

        /* menu icon */
        item.item.str.chr           = OSD_GetStr(STR_KEY_MENU);

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_SERVBOX_OPEN_MENU);
        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* exit icon */
        item.item.str.chr           = OSD_GetStr(STR_KEY_EXIT);
        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) + mainPanelWidth/2;
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_MENU_EXIT);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- Show Channel List ----------------------------------------- */

        /* calculate list index */
        if ( (uint16_t)(numServ + inst->numRecords) > 0 )
        {
            centerIdx += inst->numRecords;

            if ( centerIdx < 0 ) centerIdx = 0;

            else if ( centerIdx < (int16_t)inst->numRecords )
            {
                centerIdx = (inst->numRecords - centerIdx- 1);
            }

            /* set navi focus */
            if ( FGS_LNAVI_GetParams(inst->naviHdl, &naviParams) == FAPI_OK )
            {
                naviParams.index = centerIdx;

                (void)FGS_LNAVI_SetParams(inst->naviHdl, &naviParams);
            }

            (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, OSD_CHANLST_ID_NAVI); /* set panel focus */
        }

        (void)MENUSTACK_FocusClaim ((int32_t)MENUID_CHANLIST);
        (void)FGS_PANEL_Show       (inst->pnlHdl, FFALSE); /* turn visibility on */
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        MAIN_ServBoxHide(inst);
        return FFALSE;
    }

    return FTRUE;
}

FSTATIC void MAIN_ServBoxOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    MENU_SERVBOX_INST_S* inst = (MENU_SERVBOX_INST_S*)h;

    DBG_Assert(((MENU_SERVBOX_INST_S*)h)->pnlHdl != NULL);

    if ( inst->params.onFocusSet != NULL )
    {
        inst->params.onFocusSet(inst->params.onFocusSetArg, hasFocus);
    }

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_RED,    MAIN_ServBoxKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_GREEN,  MAIN_ServBoxKeys, h, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_YELLOW, MAIN_ServBoxKeys, h, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_RED);
        FGS_ClearAppKeyHandler(FGS_KEY_GREEN);
        FGS_ClearAppKeyHandler(FGS_KEY_YELLOW);
    }
}

FSTATIC int32_t MAIN_ServBoxDrawTitle (FAPI_SYS_HandleT h)
{
    MENU_SERVBOX_INST_S*    inst = (MENU_SERVBOX_INST_S*)h;
    SCALER_PIC_ID_E         icon = ICON_NONE;
    const DRAW_PicItemT*    pIcon = NULL;
    SYS_STRING_S            title;
    uint32_t                posX, posY;

    posX = inst->pTitleItem->abspos.x;
    posX += SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    posY = inst->pTitleItem->abspos.y +
           SCALER_GetCurrVal(TITLE_TB_PADDING);

    /* print custom title line */

    switch ( inst->currServLst )
    {
        case SLIST_TV:
            icon = ICON_TV;
            break;

        case SLIST_RADIO:
            icon = ICON_RADIO;
            break;

        default:
            if ( ((int16_t)inst->currServLst >= SLIST_OFFSET_USR) &&
                 (inst->currServLst < (SLIST_OFFSET_USR +
                                       SLIST_USR_NUMLISTS)) )
            {
                 icon = ICON_USR;
            }
            else
            {
                icon = ICON_NONE;
            }
            break;
    }

    pIcon  = SCALER_GetPic(icon);
    title = SLIST_ListNameGet(inst->currServLst);


    if ( pIcon != NULL )
    {
        /* paint list symbol... */
        (void)DRAW_Fbm_Overlay(OSD_LAYER_MENU,
                          posX,
                          posY + SCALER_GetCurrVal(OUTER_FRAME_WIDTH),
                          pIcon->fbmHeaderPtr,
                          pIcon->dataPtr,
                          OSD_COL_BLANK);
    }

    posX += SCALER_GetCurrVal(MAIN_ICON_WIDTH);
    posX += SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
    /* vertical separator */
    OSD_DrawSeparatorVertical(posX,
                              inst->pTitleItem->abspos.y,
                              inst->titlePos.height);

    posX += SCALER_GetCurrVal(INNER_FRAME_WIDTH);
    posX += SCALER_GetCurrVal(MAIN_TEXT_PAD_X);

    if ( title.chr != NULL )
    {
        OSD_DrawStringLtd(posX,
                          posY,
                          OSD_COL_TITLE_TEXT,
                          TITLE_FONT, -1,
                          inst->titlePos.width - (posX - inst->pTitleItem->abspos.x) , - 1,
                          title, FTRUE);
    }

    return FAPI_OK;
}

FSTATIC void MAIN_ServBoxHide (MENU_SERVBOX_INST_S* inst)
{
    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_CHANLIST);

    inst->naviHdl    = NULL;
    inst->pnlHdl     = NULL;
    inst->pTitleItem = NULL;
}


FSTATIC void MAIN_ServBoxItemGet (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                  FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_SERVBOX_INST_S* inst = (MENU_SERVBOX_INST_S*)h;
    SYS_SERVLOC_S        servLoc;
    FGS_STRING_S         str;
    uint16_t             servNum;
    int32_t              retVal;

    memset(&str,0,sizeof(FGS_STRING_S));
    str.font       = MAIN_FONT;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: str.col = OSD_COL_TEXT;             break;
        case FGS_FOCUS:    str.col = OSD_COL_TEXT_HIGHLIGHTED; break;
        case FGS_DISABLED: str.col = OSD_COL_TEXT_DISABLED;    break;
    }
    item[0].type     = FGS_ITEM_TYPE_STRING;
    item[0].align    = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[0].item.str = str;

    item[1].type     = FGS_ITEM_TYPE_ICON;
    item[1].align    = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[1].item.icon.drawMode   = FGS_ICONMODE_OVRLAY;
    item[1].item.icon.altBkgrCol = OSD_GetSchemeColor(OSD_SCHEMECOL_MAIN_BKGR);;
    item[1].item.icon.bkgrCol    = OSD_COL_BLANK;

    item[2].type     = FGS_ITEM_TYPE_STRING;
    item[2].align    = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item[2].item.str = str;

    /* test for empty list */
    if ( (uint16_t)(SLIST_ListSizeGet(inst->currServLst) + inst->numRecords) < 1 )
    {
        item[2].item.str.chr = OSD_GetStr(STR_GNR_EMPTY);
    }
    else
    {
        #ifdef APPL_PVR_ENABLED

        if ( index < inst->numRecords )
        {
            /* left column is record index */
            item[0].item.str.chr = STR_IntASCIIStr();

            sprintf (appDat->asciiBuf, "REC %ld", index + 1);

            if ( (appDat->pvrStatus.rec[index].status == REC_STATUS_INACTIVE) ||
                 (!(inst->params.recordIsAvailable((uint8_t)index))) )
            {
                /* Mark record as non-selectable if necessary */
                item[0].item.str.col = OSD_COL_TEXT_DISABLED;
                item[2].item.str.col = OSD_COL_TEXT_DISABLED;
            }

            /* right column is service currently recorded */
            if ( (appDat->pvrStatus.rec[index].status == REC_STATUS_ACTIVE) &&
                 (SLIST_GetServByIndex(appDat->pvrStatus.rec[index].servIdx,
                                       &(inst->currItem)) == FAPI_OK) )
            {
                item[2].item.str.chr = STR_Build (inst->currItem.nameCharSet,
                                                  inst->currItem.name);
            }
        }
        else
        {

        #endif /* APPL_PVR_ENABLED */

            servLoc.listId = inst->currServLst;
            servLoc.index  = index - inst->numRecords;

            retVal = SLIST_EntryGet (&servLoc, &(inst->currItem));

            switch ( retVal )
            {
                case FAPI_OK:
                    /* left column is service number */
                    item[0].item.str.chr = STR_IntASCIIStr();

                    servNum = servLoc.index + 1;

                    #ifdef APPL_LCN_ENABLED

                    if ( inst->showLcn )
                    {
                        if ( SLIST_GetLcn (&servLoc, &servNum) != FAPI_OK )
                        {
                            servNum = 0;
                        }
                    }

                    #endif /* APPL_LCN_ENABLED */

                    sprintf (appDat->asciiBuf, "%04i", servNum);

                    /* Mark service as non-selectable if necessary */
                    if ( !(inst->params.serviceIsAvailable(&servLoc)) )
                    {
                        item[0].item.str.col = OSD_COL_TEXT_DISABLED;
                        item[2].item.str.col = OSD_COL_TEXT_DISABLED;
                    }

                    item[2].item.str.chr = STR_Build(
                                           inst->currItem.nameCharSet,
                                           inst->currItem.name);
                    if( !SLIST_FlagIsSet(&inst->currItem, SLIST_FLAG_ALLFREE) )
                    {
                        item[1].item.icon.addr = OSD_IconGet(ICON_CRYPTED);
                    }
                    else
                    {
                        item[1].item.icon.addr = NULL;
                    }
                    break;

                case SMARTGO_SYS_ERR_ITEM_EMPTY:
                    item[1].item.icon.addr = NULL;
                    item[2].item.str.chr = OSD_GetStr(STR_VALUE_SERVLIST_DELETED);

                    break;

                default:
                    item[1].item.icon.addr = NULL;
                    item[2].item.str.chr = OSD_GetStr(STR_GNR_EMPTY);
                    break;

            }

        #ifdef APPL_PVR_ENABLED
        }
        #endif /* APPL_PVR_ENABLED */
    }
}

FSTATIC void MAIN_ServBoxOnSelect (FAPI_SYS_HandleT h, int32_t id, uint16_t index)
{
    MENU_SERVBOX_INST_S* inst = (MENU_SERVBOX_INST_S*)h;
    SYS_SERVLOC_S        reqServ;

    #ifdef APPL_PVR_ENABLED

    if ( index < inst->numRecords )
    {
        if ( inst->params.onRecordSelect != NULL )
        {
            inst->params.onRecordSelect((uint8_t)index);
        }
    }
    else
    {

    #endif /* APPL_PVR_ENABLED */

        if ( inst->params.onServiceSelect != NULL )
        {
            reqServ.listId = inst->currServLst;
            reqServ.index  = index - inst->numRecords;

            inst->params.onServiceSelect(&reqServ);
        }

    #ifdef APPL_PVR_ENABLED
    }
    #endif /* APPL_PVR_ENABLED */
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
