/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_streamselect.c
**
**              $Id: app_streamselect.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Audio/SPDIF/Subtitle stream selection menu.
**
** Detail description. [\ref app_smartgo_app_ref_streamsel]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)

    #define STRSEL_SUPPORT_SUBTT

    #define MENU_STRSEL_ID_SUBTT_MAX  (SUBTT_STREAMS_MAX + TTXSUBTT_STREAMS_MAX)

    #define OFFSET_FOR_FGS 1

#endif


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    #ifdef STRSEL_SUPPORT_SUBTT
    MENU_STRSEL_ID_ENTRY_SUBTT,
    #endif /* STRSEL_SUPPORT_SUBTT */

    MENU_STRSEL_ID_ENTRY_AUDIO,
    MENU_STRSEL_ID_ENTRY_SPDIF,
    MENU_STRSEL_ID_ENTRY_NUMS
};

enum
{
    MENU_STRSEL_ID_TITLE = 0,
    MENU_STRSEL_ID_LIST,
    MENU_STRSEL_ID_HELPLIST
};

#ifdef STRSEL_SUPPORT_SUBTT
enum
{
    SUBTT_SEL_OFF = -1
};


typedef struct
{
    fbool_t   isDvb;   //!< FTRUE: DVB subtitle, FFALSE: TTX subtitle
    uint16_t     index;   //!< index in corresponding subtitle list (DVB or TTX)

    union
    {
        #ifdef APPL_SUBTT_ENABLED
        SUBTT_STREAMINF_S dvb;
        #endif /* APPL_SUBTT_ENABLED */

        #ifdef APPL_TTXT_ENABLED
        TTXSUBTT_INF_S    ttx;
        #endif /* APPL_TTXT_ENABLED */
    } par;

} MENU_STRSEL_ID_SUBTT_S;

#endif /* STRSEL_SUPPORT_SUBTT */


typedef struct
{
    FAPI_SYS_HandleT            pnlHdl;
    FAPI_SYS_HandleT            titleHdl;
    FAPI_SYS_HandleT            listHdl;
    FAPI_SYS_HandleT            listHelpHdl;

    #ifdef STRSEL_SUPPORT_SUBTT

    uint8_t                     numSubttStreams;
    int8_t                      subttStrCurr;
    MENU_STRSEL_ID_SUBTT_S      subtt[MENU_STRSEL_ID_SUBTT_MAX];

    #endif /* STRSEL_SUPPORT_SUBTT */

    SYS_STREAMINF_AUDIO_S       audioInf;

    uint8_t                     numMainAudStreams;
                                //!< Num. of available main audio audio streams.
    int8_t                      mainAudStrCurr;
                                //!< Current main audio stream index.
    uint8_t                     mainAudStrRef[SYS_AUDIO_STR_MAX];
                                //!< References of main audio streams in
                                //!< service list.
    uint8_t                     numSpdifStreams;
                                //!< Number of available SPDIF passthrough
                                //!< streams.
    int8_t                      spdifStrCurr;
                                //!< Current SPDIF stream index.
    uint8_t                     spdifStrRef[SYS_AUDIO_STR_MAX];
                                //!< References of SPDIF streams in service
                                //!< list.
    AV_SPDIF_STATUS_E           spdifStatusCurr;

} MENU_STRSEL_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC MENU_STRSEL_INST_S* smDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t MENU_StrSelLeave             (FAPI_SYS_HandleT h,
                                              fbool_t force);
FSTATIC void    MENU_StrSelOnFocusSet        (FAPI_SYS_HandleT h,
                                              fbool_t hasFocus);
FSTATIC void    MENU_StrSelGetDescrItem      (FAPI_SYS_HandleT h, int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void    MENU_StrSelGetHelpDescrItem  (FAPI_SYS_HandleT h, int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void    MENU_StrSelOnValChg          (FAPI_SYS_HandleT h, int32_t id,
                                              FGS_VAL_S val);
FSTATIC void    MENU_StrSel_UpdateAudioList  (MENU_STRSEL_INST_S* inst);
FSTATIC int32_t MENU_StrSelUpdateAudioChoice (MENU_STRSEL_INST_S* inst);
FSTATIC int32_t MENU_StrSelUpdateSpdifChoice (MENU_STRSEL_INST_S* inst);
FSTATIC int32_t MENU_StrSelSelectAudioStream (MENU_STRSEL_INST_S* inst,
                                              uint8_t index);
FSTATIC int32_t MENU_StrSelSelectSpdifStream (MENU_STRSEL_INST_S* inst,
                                              AV_SPDIF_STATUS_E type,
                                              int8_t index);

FSTATIC void    MENU_StrSelGetItem           (FAPI_SYS_HandleT h, int32_t id,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void    MENU_StrSelGetAudioItem      (FAPI_SYS_HandleT h,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void    MENU_StrSelGetSpdifItem      (FAPI_SYS_HandleT h,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void    MENU_StrSelSetItemPars       (FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
FSTATIC void    MENU_StrSelReset             (MENU_STRSEL_INST_S* inst);
FSTATIC void    MENU_StrSelOnFocusChg        (FAPI_SYS_HandleT h, uint8_t index,
                                              int32_t id);
FSTATIC fbool_t MENU_StrSelOnSelect          (FAPI_SYS_HandleT h,
                                              int32_t itemId);

#ifdef STRSEL_SUPPORT_SUBTT

FSTATIC int32_t MENU_StrSelSelectSubtitle    (MENU_STRSEL_INST_S* inst,
                                              int8_t index);
FSTATIC int32_t MENU_StrSelUpdateSubttChoice (MENU_STRSEL_INST_S* inst);
FSTATIC void    MENU_StrSel_UpdateSubttList  (MENU_STRSEL_INST_S* inst);
FSTATIC void    MENU_StrSelGetSubttItem      (FAPI_SYS_HandleT h,
                                              uint32_t index,
                                              FGS_ITEMSTATE_E state,
                                              FGS_ITEM_S* item);
#endif /* STRSEL_SUPPORT_SUBTT */

/* ------------------------------------------------------------------------- */

int32_t MENU_StrSelInit (void)
{
    if ( smDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    smDat = UTILS_Malloc(sizeof(MENU_STRSEL_INST_S));
    if ( smDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(smDat, 0, sizeof(MENU_STRSEL_INST_S));

    /* Init variables with non-zero default value only */
    #ifdef STRSEL_SUPPORT_SUBTT

    smDat->subttStrCurr     = -1;

    #endif /* STRSEL_SUPPORT_SUBTT */

    smDat->mainAudStrCurr = -1;
    smDat->spdifStrCurr   = -1;
    return FAPI_OK;
}

fbool_t MENU_StrSelEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_STRSEL_INST_S*    inst = smDat;
    FGS_PANEL_OPEN_S       pnlPars;
    FGS_LIST_OPEN_S        listPars;
    FGS_ITEM_SEL_OPEN_S    selPars;
    int32_t                retVal;
    FGS_POS_S              itemPos;
    FGS_POS_S              descrPos;
    FGS_POS_S              valuePos;
    FAPI_SYS_HandleT       itemHdl;
    uint16_t               helpWidth;
    uint16_t               stackedFrameWidth;
    uint16_t               innerHeight;
    uint32_t               numLines     = MENU_STRSEL_ID_ENTRY_NUMS;

    DBG_Assert(inst != 0);
    if(inst == NULL)return FFALSE; /* to satisfy MISRA */

    if ( inst->pnlHdl != NULL )
    {
        FAPI_SYS_PRINT_DEBUG(2,"MENU_StrSelEntry: already open!\n");
        return FFALSE;
    }

    if ( test == FTRUE ) return FTRUE;

    inst->titleHdl        = NULL;
    inst->listHdl        = NULL;

    #ifdef STRSEL_SUPPORT_SUBTT
    MENU_StrSel_UpdateSubttList(inst);
    #endif /* STRSEL_SUPPORT_SUBTT */

    MENU_StrSel_UpdateAudioList(inst);

    memset(&selPars,   0, sizeof(selPars));

    helpWidth         = (2*SCALER_GetCurrVal(HELP_LR_PADDING)) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_LOKR),
                                   -1,SYMBOL_FONT);
    innerHeight       = numLines * (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                  + SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_StrSelOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&pnlPars.pos);

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_STRSEL, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_STRSEL_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_STRSEL,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_StrSelGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_StrSelGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;
        listPars.onFocusChg      = MENU_StrSelOnFocusChg;
        listPars.onFocusChgArg   = (FAPI_SYS_HandleT)inst;
        listPars.onSelect        = MENU_StrSelOnSelect;
        listPars.onSelectArg     = (FAPI_SYS_HandleT)inst;
        listPars.onValChg        = MENU_StrSelOnValChg;
        listPars.onValChgArg     = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_STRSEL_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 50,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- Subtitle choice ------------------------------------------- */

        #ifdef STRSEL_SUPPORT_SUBTT

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = 0;
        selPars.params.index        = 0;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_STRSEL_ID_ENTRY_SUBTT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        (void)MENU_StrSelUpdateSubttChoice(inst);

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        #endif /* STRSEL_SUPPORT_SUBTT */

        /* ----- Analogue audio choice ------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = 0;
        selPars.params.index        = 0;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_STRSEL_ID_ENTRY_AUDIO,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        (void)MENU_StrSelUpdateAudioChoice(inst);

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- SPDIF audio choice ---------------------------------------- */

        selPars.pos                 = itemPos;
        selPars.bkgrType            = OSD_LSTITEM_BKGR_NORMAL;
        selPars.descrPos            = descrPos;
        selPars.valuePos            = valuePos;
        selPars.params.numItems     = 0;
        selPars.params.index        = 0;

        retVal = FGS_ITEM_SEL_Open (&selPars, MENU_STRSEL_ID_ENTRY_SPDIF,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        (void)MENU_StrSelUpdateSpdifChoice(inst);

        itemPos.y += SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Create help list ------------------------------------------ */

        inst->listHelpHdl = MENU_CommonAddHelpList(MENU_STRSEL_ID_HELPLIST,
                                        inst->pnlHdl, stackedFrameWidth,
                                        SCALER_GetCurrVal(MAIN_WIDTH),
                                        listPars.pos.y,
                                        helpWidth, numLines,
                                        MENU_StrSelGetHelpDescrItem,
                                        (FAPI_SYS_HandleT)inst);
        if ( inst->listHelpHdl == NULL )
        {
            retVal = SMARTGO_APPL_ERR_INTERNAL;
            break;
        }

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(stackedFrameWidth +
                                                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                                - (int32_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                                                SCALER_GetCurrVal(HELP_TB_PADDING) -
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT),
                                                pnlPars.pos.width -
                                                stackedFrameWidth -
                                                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;
        /*-------------------------------------------------------------------*/
        (void)FGS_LIST_SetMemberFocus  (inst->listHdl, MENU_STRSEL_ID_ENTRY_AUDIO);
        (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_STRSEL_ID_LIST);
        (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_STRSEL);
        (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

        inst->pnlHdl = NULL;
        return FFALSE;
    }

    MENUSTACK_SetCleanup (MENU_StrSelLeave, (FAPI_SYS_HandleT)inst);
    return FTRUE;
}


FSTATIC fbool_t MENU_StrSelOnSelect (FAPI_SYS_HandleT h, int32_t itemId)
{
    MENU_STRSEL_INST_S*      inst     = (MENU_STRSEL_INST_S*)h;
    MENUSTACK_ENTRY          newEntry = NULL;
    FAPI_SYS_HandleT         arg      = NULL;
    uint8_t                  subLines;
    MENU_SELECTLIST_OPEN_S   selectLst;
    FGS_ITEM_SEL_PARAMS_S    selParams;
    FGS_LOC_S                memberLoc;
    fbool_t                  openSelectLst = FFALSE;

    switch ( itemId )
    {
        #ifdef STRSEL_SUPPORT_SUBTT
        case MENU_STRSEL_ID_ENTRY_SUBTT:
            openSelectLst  = FTRUE;
            subLines       = 3;
            break;
        #endif /* STRSEL_SUPPORT_SUBTT */
        case MENU_STRSEL_ID_ENTRY_AUDIO:
            openSelectLst  = FTRUE;
            subLines       = 3;
            break;
        case MENU_STRSEL_ID_ENTRY_SPDIF:
            openSelectLst  = FTRUE;
            subLines       = 3;
            break;

        default:
            return FFALSE;
    }

    if ( openSelectLst )
    {
        memset(&memberLoc, 0, sizeof(memberLoc));
        memset(&selectLst, 0, sizeof(selectLst));
        memset(&selParams, 0, sizeof(selParams));

        (void)FGS_LIST_GetMemberAbsPos (inst->listHdl, itemId, &(memberLoc));
        (void)FGS_LIST_GetMemberParams (inst->listHdl, itemId, &selParams);

        selectLst.pos.x         = memberLoc.x +
                                  SCALER_GetCurrVal(MAIN_WIDTH) -
                                  SCALER_GetCurrVal(SUB_LIST_WIDTH) -
                                  SCALER_GetCurrVal(OSD_POS_X);
        selectLst.pos.y         = memberLoc.y -
                                  SCALER_GetCurrVal(OSD_POS_Y);
        selectLst.pos.width     = SCALER_GetCurrVal(SUB_LIST_WIDTH);
        selectLst.pos.height    = 0; /* is calculated internally */
        selectLst.pos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        selectLst.id            = itemId;
        selectLst.numVisLines   = subLines;
        selectLst.numItems      = selParams.numItems;
        selectLst.index         = selParams.index;
        selectLst.getItem       = MENU_StrSelGetItem;
        selectLst.getItemArg    = h;
        selectLst.onSelect      = MENU_CommonSelectListSelected;
        selectLst.onSelectArg   = inst->listHdl;
        selectLst.onExit        = NULL;
        selectLst.onExitArg     = NULL;

        return MENU_SelectListOpen(&(appDat->selectLstInst), &selectLst);
    }
    else
    {
        /* store last focus */
        MENUSTACK_SetData(itemId);

        if(newEntry == NULL) return FFALSE;
        else                 return MENUSTACK_Up(newEntry, arg);
    }
}

FSTATIC void MENU_StrSelOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_STRSEL_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleZappingKeys(hasFocus);
    MAIN_HandleExitKeys   (hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_INFO, MAIN_DirectMenuTrans, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_NAVI, MAIN_DirectMenuTrans, NULL, FTRUE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_INFO);
        FGS_ClearAppKeyHandler(FGS_KEY_NAVI);
    }
}

FSTATIC void MENU_StrSelOnFocusChg (FAPI_SYS_HandleT h, uint8_t index, int32_t id)
{
    MENU_STRSEL_INST_S*   inst = (MENU_STRSEL_INST_S*)h;
    FGS_LNAVI_PARAMS_S    naviParams;

    if ( FGS_LNAVI_GetParams(inst->listHelpHdl, &naviParams) == FAPI_OK )
    {
        naviParams.index = id;

        (void)FGS_LNAVI_SetParams(inst->listHelpHdl, &naviParams);
    }
}

FSTATIC void MENU_StrSelGetDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        #ifdef STRSEL_SUPPORT_SUBTT

        case MENU_STRSEL_ID_ENTRY_SUBTT:
            strIdx = STR_ENTRY_STREAMSEL_SUBTTSTR;
            break;

        #endif /* STRSEL_SUPPORT_SUBTT */

        case MENU_STRSEL_ID_ENTRY_AUDIO:
            strIdx = STR_ENTRY_STREAMSEL_AUDIOSTR;
            break;

        case MENU_STRSEL_ID_ENTRY_SPDIF:
            strIdx = STR_ENTRY_STREAMSEL_SPDIFSTR;
            break;

        default:
            return;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

FSTATIC void MENU_StrSelGetHelpDescrItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( index )
    {
        #ifdef STRSEL_SUPPORT_SUBTT
        case MENU_STRSEL_ID_ENTRY_SUBTT:
        #endif /* STRSEL_SUPPORT_SUBTT */

        case MENU_STRSEL_ID_ENTRY_AUDIO:
        case MENU_STRSEL_ID_ENTRY_SPDIF:
            strIdx = STR_ENTRY_LOKR;
            break;

        default: return;
    }

    MENU_CommonSetHelpDescrParams(item, state, strIdx);
}

void MENU_StrSelServSelEvent (const SYS_EVT_SERVICE_S* pEvent)
{
    MENU_STRSEL_INST_S* inst = smDat;

    DBG_Assert(inst != 0);
    if(inst == NULL)return; /* to satisfy MISRA */

    switch ( pEvent->status )
    {
        case SERVICE_STAT_SUCCESS:
            MENU_StrSel_UpdateAudioList(inst);

            (void)MENU_StrSelUpdateAudioChoice(inst);
            (void)MENU_StrSelUpdateSpdifChoice(inst);
            break;

        case SERVICE_STAT_START:
        case SERVICE_STAT_STOPPED:
            MENU_StrSelReset(inst);
            break;

        default:
            /* do nothing */
            break;
    }
}

#ifdef APPL_PVR_ENABLED

void MENU_StrSelPlayEvent (const SYS_EVT_PVRPLAY_S* pEvent)
{
    MENU_STRSEL_INST_S* inst = smDat;

    DBG_Assert(inst != 0);
    if(inst == NULL)return; /* to satisfy MISRA */

    switch ( pEvent->action )
    {
        case PVR_PLAY_START:
            /* NOTE: Streams are not available unless the first PMT has
                     been received. */
        case PVR_PLAY_STOP:
            MENU_StrSelReset(inst);
            break;

        default:
            /* do nothing */
            break;
    }
}

#endif /* APPL_PVR_ENABLED */

void MENU_StrSelPmtUpdate (void)
{
    MENU_STRSEL_INST_S* inst = smDat;

    DBG_Assert(inst != 0);
    if(inst == NULL)return; /* to satisfy MISRA */

    /* ----- Update audio -------------------------------------------------- */

    MENU_StrSel_UpdateAudioList(inst);

    (void)MENU_StrSelUpdateAudioChoice(inst);
    (void)MENU_StrSelUpdateSpdifChoice(inst);

    /* ----- Update subtitles ---------------------------------------------- */

    #ifdef STRSEL_SUPPORT_SUBTT

    MENU_StrSel_UpdateSubttList(inst);

    (void)MENU_StrSelUpdateSubttChoice(inst);

    #endif /* STRSEL_SUPPORT_SUBTT */
}

FSTATIC fbool_t MENU_StrSelLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_STRSEL_INST_S* inst = (MENU_STRSEL_INST_S*)h;

    if(inst == NULL)return FFALSE;

    DBG_Assert(inst->pnlHdl != 0);

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_STRSEL);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC void MENU_StrSelOnValChg (FAPI_SYS_HandleT h, int32_t id, FGS_VAL_S val)
{
    MENU_STRSEL_INST_S*  inst = (MENU_STRSEL_INST_S*)h;

    if ( inst == NULL )
    {
        return;
    }

    switch ( id )
    {
        #ifdef STRSEL_SUPPORT_SUBTT

        case MENU_STRSEL_ID_ENTRY_SUBTT:
            DBG_Assert(val.number <= MENU_STRSEL_ID_SUBTT_MAX);
            (void)MENU_StrSelSelectSubtitle(inst, (int8_t)(val.number - 1));

            break;

        #endif /* STRSEL_SUPPORT_SUBTT */

        case MENU_STRSEL_ID_ENTRY_AUDIO:
            (void)MENU_StrSelSelectAudioStream(inst, (uint8_t)val.number);
            break;

        case MENU_STRSEL_ID_ENTRY_SPDIF:
            if ( val.number == 0 ) /* PCM */
            {
                (void)MENU_StrSelSelectSpdifStream(inst, SPDIFSTATUS_CURRPCM,
                                                   -1);
            }
            else if ( val.number >= 1 )
            {
                (void)MENU_StrSelSelectSpdifStream(inst,
                                                   SPDIFSTATUS_PASSTHROUGH,
                                                   (int8_t)(val.number - 1));
            }
            break;

        default:
            break;
    }
}

FSTATIC int32_t MENU_StrSelSelectAudioStream (MENU_STRSEL_INST_S* inst, uint8_t index)
{
    int32_t         retVal    = FAPI_OK;
    SYS_PRESSTATE_E presState = SYS_PRESSTATE_NONE;

    if ( inst == NULL )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    if ( index >= inst->numMainAudStreams )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    /* already selected? */
    else if ( index == inst->mainAudStrCurr ) return FAPI_OK;

    (void)SYS_GetPresentationState(&presState);

    switch ( presState )
    {
        default:
        case SYS_PRESSTATE_NONE:
            /* do nothing */
            break;

        case SYS_PRESSTATE_LIVETV:
            retVal = AV_SetAudioStream(inst->mainAudStrRef[index]);
            break;

        #ifdef APPL_PVR_ENABLED

        case SYS_PRESSTATE_PLAYBACK:
            retVal = PVRSYS_PlaySetAudioStream(inst->mainAudStrRef[index]);
            break;

        #endif /* APPL_PVR_ENABLED */
    }

    inst->mainAudStrCurr = index;

    return retVal;
}

FSTATIC int32_t  MENU_StrSelSelectSpdifStream (MENU_STRSEL_INST_S* inst,
                                               AV_SPDIF_STATUS_E type,
                                               int8_t index)
{
    int32_t         retVal    = FAPI_OK;
    SYS_PRESSTATE_E presState = SYS_PRESSTATE_NONE;
    int8_t          strIndex;

    if ( inst == NULL )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if ( type == SPDIFSTATUS_PASSTHROUGH )
    {
        if ( index >= (int8_t)inst->numSpdifStreams )
        {
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
        else if ( index == inst->spdifStrCurr )     /* already selected? */
        {
            return FAPI_OK;
        }
    }

    (void)SYS_GetPresentationState(&presState);

    if ( index < 0 ) strIndex = -1;
    else             strIndex = inst->spdifStrRef[index];

    switch ( presState )
    {
        default:
        case SYS_PRESSTATE_NONE:
            /* do nothing */
            break;

        case SYS_PRESSTATE_LIVETV:
            retVal = AV_SetSPDIFStatus(type, strIndex);
            break;

        #ifdef APPL_PVR_ENABLED

        case SYS_PRESSTATE_PLAYBACK:
            retVal = PVRSYS_PlaySetSPDIFStatus(type, strIndex);
            break;

        #endif /* APPL_PVR_ENABLED */
    }

    if ( type == SPDIFSTATUS_PASSTHROUGH ) inst->spdifStrCurr = index;
    else                                   inst->spdifStrCurr = -1;

    return retVal;
}

#ifdef STRSEL_SUPPORT_SUBTT


FSTATIC int32_t MENU_StrSelSelectSubtitle (MENU_STRSEL_INST_S* inst, int8_t index)
{
    int32_t                    retVal = FAPI_OK;
    MENU_STRSEL_ID_SUBTT_S*    pSubtt;

    int8_t                     subttIdx = -1;
    fbool_t                    isDvb = FTRUE;

    if ( inst == NULL )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if (((uint8_t)index < inst->numSubttStreams )&&
        (index >= 0))
    {
        pSubtt = inst->subtt + index;
        subttIdx = (int8_t)pSubtt->index;
        isDvb    = pSubtt->isDvb;
    }

    if(index != inst->subttStrCurr)
    {
        #if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)
        (void)AV_SetSubttStream(subttIdx, isDvb);
        #endif /* (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED) */
        inst->subttStrCurr = index;
    }


    return retVal;
}
// update list and give inst->subttStrCurr
FSTATIC void MENU_StrSel_UpdateSubttList (MENU_STRSEL_INST_S* inst)
{
    MENU_STRSEL_ID_SUBTT_S* pSubtt = inst->subtt;
    SYS_PRESSTATE_E         presState = SYS_PRESSTATE_NONE;
    uint8_t                 cnt;
    uint8_t                 subttMax;
    int8_t                  subttCurr;

    #ifdef APPL_SUBTT_ENABLED
    SUBTT_STREAMINF_S    dvbInf;
    #endif /* APPL_SUBTT_ENABLED */

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_INF_S       ttxInf;
    #endif /* APPL_TTXT_ENABLED */

    if ( inst == NULL )
    {
        return;
    }

    inst->numSubttStreams   =  0;
    inst->subttStrCurr      = -1;

    /* --------------------------------------------------------------------- */

    (void)SYS_GetPresentationState(&presState);

    switch ( presState )
    {
        default:
        case SYS_PRESSTATE_NONE:
            return;

        case SYS_PRESSTATE_PLAYBACK:
        case SYS_PRESSTATE_LIVETV:
            break;
    }

    /* ----- add DVB subtitles to list ------------------------------------- */

    #ifdef APPL_SUBTT_ENABLED


    subttMax  = SUBTT_NumStreamsGet();
    subttCurr = SUBTT_CurrStreamGet();

    if ( subttMax > SUBTT_STREAMS_MAX ) subttMax = SUBTT_STREAMS_MAX;



    for ( cnt = 0; cnt < subttMax; cnt++ )
    {
        if ( SUBTT_StreamInfoGet (cnt, &dvbInf) == FAPI_OK )
        {
            pSubtt->isDvb   = FTRUE;
            pSubtt->index   = cnt;
            pSubtt->par.dvb = dvbInf;

            if ( (inst->subttStrCurr == SUBTT_SEL_OFF ) &&
                 (cnt                == subttCurr) )
            {
                inst->subttStrCurr = inst->numSubttStreams;
            }

            inst->numSubttStreams++;
            pSubtt++;
        }
    }

    #endif /* APPL_SUBTT_ENABLED */

    /* ----- add TTX subtitles to list ------------------------------------- */

    #ifdef APPL_TTXT_ENABLED

    subttMax  = TTXSUBTT_NumStreamsGet();
    subttCurr = TTXSUBTT_CurrStreamGet();

    if ( subttMax > TTXSUBTT_STREAMS_MAX ) subttMax = TTXSUBTT_STREAMS_MAX;

    for ( cnt = 0; cnt < subttMax; cnt++ )
    {
        if ( TTXSUBTT_StreamInfoGet(cnt, &ttxInf) == FAPI_OK )
        {
            pSubtt->isDvb   = FFALSE;
            pSubtt->index   = cnt;
            pSubtt->par.ttx = ttxInf;


            if ( (inst->subttStrCurr == SUBTT_SEL_OFF ) &&
                 (cnt                == subttCurr) )
            {
                inst->subttStrCurr = inst->numSubttStreams;
            }


            inst->numSubttStreams++;
            pSubtt++;
        }
    }

    #endif /* APPL_TTXT_ENABLED */
}

#endif /* STRSEL_SUPPORT_SUBTT */

FSTATIC void MENU_StrSel_UpdateAudioList (MENU_STRSEL_INST_S* inst)
{
    uint8_t         cnt;
    SLIST_ITEM_S    service;
    SYS_SERVLOC_S   servLoc;
    int8_t          strIdxCurr   = -1;
    int8_t          spdifIdxCurr = -1;
    SYS_PRESSTATE_E presState    = SYS_PRESSTATE_NONE;
    fbool_t         spdifCompliant;

    #ifdef APPL_PVR_ENABLED

    SYS_STREAM_INFO_S playStreamInf;

    #endif /* APPL_PVR_ENABLED */

    if ( inst == NULL )
    {
        return;
    }

    inst->numMainAudStreams = 0;
    inst->mainAudStrCurr    = -1;
    inst->numSpdifStreams   = 0;
    inst->spdifStrCurr      = -1;
    inst->spdifStatusCurr   = SPDIFSTATUS_DISABLED;

    memset(&(inst->audioInf), 0, sizeof(inst->audioInf));

    (void)SYS_GetPresentationState(&presState);

    switch ( presState )
    {
        default:
        case SYS_PRESSTATE_NONE:
            return;

        case SYS_PRESSTATE_LIVETV:
            /* Get audio information from current live service */
            (void)AV_GetRequestedService (&servLoc);

            if ( SLIST_EntryGet (&servLoc, &service) != FAPI_OK )
            {
                return;
            }

            if ( AV_GetAudioStream (&strIdxCurr) != FAPI_OK ) return;

            inst->audioInf = service.strPid.audio;

            (void)AV_GetSPDIFStatus(&(inst->spdifStatusCurr), &spdifIdxCurr);
            break;

        #ifdef APPL_PVR_ENABLED

        case SYS_PRESSTATE_PLAYBACK:
            /* Get audio information from current playback */
            if ( PVRSYS_PlayGetStreamInfo(&playStreamInf) != FAPI_OK )
            {
                return;
            }

            if ( PVRSYS_PlayGetAudioStream (&strIdxCurr) != FAPI_OK ) return;

            inst->audioInf = playStreamInf.audio;

            (void)PVRSYS_PlayGetSPDIFStatus(&(inst->spdifStatusCurr),
                                            &spdifIdxCurr);
            break;

        #endif /* APPL_PVR_ENABLED */
    }

    /* ----- populate internal audio lists --------------------------------- */

    /* main audio via I2S */
    for ( cnt = 0; cnt < inst->audioInf.numStreams; cnt++ )
    {
        /* NOTE: For demo purposes all audio streams are added here.
                 Use AV_AudioFormatIsDecodable in order to filter out
                 those that are not decodable with current audio firmware. */

        if ( cnt == strIdxCurr )
        {
            inst->mainAudStrCurr = inst->numMainAudStreams;
        }

        inst->mainAudStrRef[inst->numMainAudStreams++] = cnt;
    }

    /* SPDIF passthrough audio */
    for ( cnt = 0; cnt < inst->audioInf.numStreams; cnt++ )
    {
        /* is this stream currently selected? */
        if ( (inst->spdifStatusCurr == SPDIFSTATUS_PASSTHROUGH)
          && ((int8_t)cnt == spdifIdxCurr) )
        {
            inst->spdifStrCurr = inst->numSpdifStreams;
            inst->spdifStrRef[inst->numSpdifStreams++] = cnt;
        }
        else if ( AV_AudioFormatIsSpdifCompliant (inst->audioInf.stream[cnt].
                  format, &spdifCompliant) == FAPI_OK )
        {
            /* add all SPDIF-passthrough-compliant streams to list */
            if ( spdifCompliant )
            {
                inst->spdifStrRef[inst->numSpdifStreams++] = cnt;
            }
        }
    }
}

FSTATIC void MENU_StrSelGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                                 FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    switch ( id )
    {
        #ifdef STRSEL_SUPPORT_SUBTT

        case MENU_STRSEL_ID_ENTRY_SUBTT:
           MENU_StrSelGetSubttItem(h, index, state, item);
           break;

        #endif /* STRSEL_SUPPORT_SUBTT */

        case MENU_STRSEL_ID_ENTRY_AUDIO:
            MENU_StrSelGetAudioItem(h, index, state, item);
            break;

        case MENU_STRSEL_ID_ENTRY_SPDIF:
            MENU_StrSelGetSpdifItem(h, index, state, item);
            break;

        default:
            break;
    }
}

#ifdef STRSEL_SUPPORT_SUBTT

FSTATIC void MENU_StrSelGetSubttItem (FAPI_SYS_HandleT h, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_STRSEL_INST_S* inst = (MENU_STRSEL_INST_S*)h;
    uint32_t            bufPos = 0;
    SYS_STRING_S        langStr;

    if ( inst == NULL )
    {
        return;
    }
    MENU_StrSelSetItemPars(state, item);

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    if ( index == 0 ) //Off
    {
        item->item.str.chr = OSD_GetStr(STR_GNR_STREAMDISABLED);
    }
    else
    {
        if ( (uint32_t)(index - OFFSET_FOR_FGS) < inst->numSubttStreams )
        {
            switch ( inst->subtt[index - OFFSET_FOR_FGS].isDvb )
            {
                #ifdef APPL_SUBTT_ENABLED

                case FTRUE:
                    langStr = OSD_LANG_GetLanguageString(
                              inst->subtt[index - OFFSET_FOR_FGS].par.dvb.langCode);

                    sprintf (appDat->asciiBuf, "%li/%i ", index,
                             inst->numSubttStreams);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos, langStr,
                                   MAIN_LINEBUFCHARSET);

                    if(inst->subtt[index - OFFSET_FOR_FGS].par.dvb.displayType!=
                       SUBTT_DISPLAY_NONE)
                    {
                        /*lint -save -e787 */
                        switch(inst->subtt[index - OFFSET_FOR_FGS].par.dvb.displayType)
                        {
                            case SUBTT_DISPLAY_4_3:
                                bufPos += STR_BufAppend (NULL, bufPos,
                                               OSD_GetStr(STR_ENTRY_STREAMSEL_DISP_4_3),
                                               MAIN_LINEBUFCHARSET);
                                break;
                            case SUBTT_DISPLAY_16_9:
                                bufPos += STR_BufAppend (NULL, bufPos,
                                               OSD_GetStr(STR_ENTRY_STREAMSEL_DISP_16_9),
                                               MAIN_LINEBUFCHARSET);
                                break;
                            case SUBTT_DISPLAY_221_1:
                                bufPos += STR_BufAppend (NULL, bufPos,
                                               OSD_GetStr(STR_ENTRY_STREAMSEL_DISP_2_21_1),
                                               MAIN_LINEBUFCHARSET);
                                break;
                            case SUBTT_DISPLAY_HD:
                                bufPos += STR_BufAppend (NULL, bufPos,
                                               OSD_GetStr(STR_ENTRY_STREAMSEL_DISP_HD),
                                               MAIN_LINEBUFCHARSET);
                                break;
                            default:
                                break;
                        }
                        /*lint -restore */
                    }
                    if(inst->subtt[index - OFFSET_FOR_FGS].par.dvb.type ==
                        SYS_SUBTITLE_TYPE_HEARINGIMPAIRED)
                    {
                        bufPos += STR_BufAppendChar(NULL, bufPos,
                                                    ' ', MAIN_LINEBUFCHARSET);
                        bufPos += STR_BufAppendChar(NULL, bufPos,
                                      (FONTSERVER_CHAR_CODE_SET_FONT+SCALER_GetFont(SYMBOL_FONT)),
                                                 MAIN_LINEBUFCHARSET);
                        bufPos += STR_BufAppend (NULL, bufPos,
                                    OSD_GetStr(STR_ENTRY_STREAMSEL_HEARINGIMPAIRED),
                                    MAIN_LINEBUFCHARSET);

                        bufPos += STR_BufAppendChar(NULL, bufPos,
                                      (FONTSERVER_CHAR_CODE_SET_FONT+SCALER_GetFont(MAIN_FONT)),
                                                 MAIN_LINEBUFCHARSET);
                    }
                    item->item.str.chr = STR_IntLineStr();
                    break;

                #endif /* APPL_SUBTT_ENABLED */

                #ifdef APPL_TTXT_ENABLED

                case FFALSE:
                    langStr = OSD_LANG_GetLanguageString(
                              inst->subtt[index - OFFSET_FOR_FGS].par.ttx.langCode);

                    sprintf (appDat->asciiBuf, "%li/%i ", index,
                             inst->numSubttStreams);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos, langStr,
                                             MAIN_LINEBUFCHARSET);

                    (void)STR_BufAppend (NULL, bufPos,
                                   OSD_GetStr(STR_ENTRY_STREAMSEL_TTX_ADDON),
                                   MAIN_LINEBUFCHARSET);

                    item->item.str.chr = STR_IntLineStr();
                    break;

                #endif /* APPL_TTXT_ENABLED */

                default:
                    item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
                    break;
            }
        }
    }
}

#endif /* STRSEL_SUPPORT_SUBTT */

FSTATIC void MENU_StrSelGetAudioItem (FAPI_SYS_HandleT h, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_STRSEL_INST_S* inst = (MENU_STRSEL_INST_S*)h;
    uint32_t            bufPos = 0;
    SYS_STRING_S        langStr;
    SYS_STR_AUDIO_S*    pStream;
    fbool_t             decodable;

    if ( inst == NULL )
    {
        return;
    }

    MENU_StrSelSetItemPars(state, item);

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    if ( inst->numMainAudStreams == 0 )
    {
        if ( index == 0 )
        {
            item->item.str.chr = OSD_GetStr(STR_GNR_STREAMDISABLED);
        }
        else
        {
            item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
        }
    }
    else if ( index < inst->numMainAudStreams )
    {
        pStream = inst->audioInf.stream + (inst->mainAudStrRef[index]);

        langStr = OSD_LANG_GetLanguageString(pStream->langCode);

        sprintf (appDat->asciiBuf, "%li/%i ", index + 1,
                                              inst->numMainAudStreams);

        bufPos += STR_BufAppend (NULL, bufPos,
                                 STR_IntASCIIStr(),
                                 MAIN_LINEBUFCHARSET);

        bufPos += STR_BufAppend (NULL, bufPos, langStr,
                       MAIN_LINEBUFCHARSET);

        switch(pStream->type)
        {
            case SYS_AUDIO_TYPE_EFFECTS:
                bufPos += STR_BufAppend (NULL, bufPos,
                               OSD_GetStr(STR_ENTRY_STREAMSEL_EFFECTS),
                               MAIN_LINEBUFCHARSET);
                break;
            case SYS_AUDIO_TYPE_HEARING_IMPAIRED:
                bufPos += STR_BufAppendChar(NULL, bufPos,
                             (FONTSERVER_CHAR_CODE_SET_FONT+SCALER_GetFont(SYMBOL_FONT)),
                                         MAIN_LINEBUFCHARSET);
                bufPos += STR_BufAppend (NULL, bufPos,
                            OSD_GetStr(STR_ENTRY_STREAMSEL_HEARINGIMPAIRED),
                            MAIN_LINEBUFCHARSET);
                bufPos += STR_BufAppendChar(NULL, bufPos,
                              (FONTSERVER_CHAR_CODE_SET_FONT+SCALER_GetFont(MAIN_FONT)),
                                         MAIN_LINEBUFCHARSET);
                break;
            case SYS_AUDIO_TYPE_VISUAL_IMPAIRED:
                bufPos += STR_BufAppendChar(NULL, bufPos,
                             (FONTSERVER_CHAR_CODE_SET_FONT+SCALER_GetFont(SYMBOL_FONT)),
                                         MAIN_LINEBUFCHARSET);
                bufPos += STR_BufAppend (NULL, bufPos,
                            OSD_GetStr(STR_ENTRY_STREAMSEL_VISUALIMPAIRED),
                            MAIN_LINEBUFCHARSET);
                bufPos += STR_BufAppendChar(NULL, bufPos,
                              (FONTSERVER_CHAR_CODE_SET_FONT+SCALER_GetFont(MAIN_FONT)),
                                         MAIN_LINEBUFCHARSET);
                break;
            default:
                /* add no addional info to string */
                /* Noraml and unknown audio streams */
                break;
        }
        switch(pStream->format)
        {
            case SYS_AUDIO_AC3:
                bufPos += STR_BufAppend (NULL, bufPos,
                               OSD_GetStr(STR_ENTRY_STREAMSEL_AUDIOFORMAT_AC3),
                               MAIN_LINEBUFCHARSET);
                break;
            case SYS_AUDIO_AAC:
                bufPos += STR_BufAppend (NULL, bufPos,
                               OSD_GetStr(STR_ENTRY_STREAMSEL_AUDIOFORMAT_AAC),
                               MAIN_LINEBUFCHARSET);
                break;
            case SYS_AUDIO_HEAAC:
                bufPos += STR_BufAppend (NULL, bufPos,
                               OSD_GetStr(STR_ENTRY_STREAMSEL_AUDIOFORMAT_HEAAC),
                               MAIN_LINEBUFCHARSET);
                break;
            case SYS_AUDIO_DDPDCV:
                bufPos += STR_BufAppend (NULL, bufPos,
                               OSD_GetStr(STR_ENTRY_STREAMSEL_AUDIOFORMAT_DDPDCV),
                               MAIN_LINEBUFCHARSET);
                break;
            default:
            case SYS_AUDIO_MPEG:
                break;
        }

        item->item.str.chr = STR_IntLineStr();

        if ( AV_AudioFormatIsDecodable (pStream->format, &decodable)
             == FAPI_OK )
        {
            if ( !decodable )
            {
                /* draw audio stream as disabled if not decodable */
                item->item.str.col = appDat->txtAttr.colDisabled;
            }
        }
    }
    else
    {
        item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
    }
}

FSTATIC void MENU_StrSelGetSpdifItem (FAPI_SYS_HandleT h, uint32_t index,
                                      FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_STRSEL_INST_S* inst = (MENU_STRSEL_INST_S*)h;
    uint32_t            bufPos = 0;
    SYS_STRING_S        langStr;

    if ( inst == NULL )
    {
        return;
    }

    MENU_StrSelSetItemPars(state, item);

    item->align = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;

    if ( inst->numSpdifStreams == 0 )  /* no SPDIF streams available */
    {
        if ( index == 0 )
        {
            if ( inst->mainAudStrCurr >= 0 ) /* A main stream is selected */
            {
                item->item.str.chr = OSD_GetStr(STR_GNR_SPDIFPCM);
            }
            else /* no main streams available either */
            {
                item->item.str.chr = OSD_GetStr(STR_GNR_STREAMDISABLED);
            }
        }
        else
        {
            item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
        }
    }
    else if ( index == 0 )
    {
        item->item.str.chr = OSD_GetStr(STR_GNR_SPDIFPCM);
    }
    else if ( (uint32_t)(index - 1) < inst->numSpdifStreams )
    {
        langStr = OSD_LANG_GetLanguageString(
                  inst->audioInf.stream[inst->spdifStrRef[index - 1]].langCode);

        if ( inst->audioInf.stream[inst->spdifStrRef[index - 1]].format
             == SYS_AUDIO_AC3 )
        {
            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_GNR_DOLBYDIGITAL),
                                     MAIN_LINEBUFCHARSET);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     OSD_GetStr(STR_GNR_COLON),
                                     MAIN_LINEBUFCHARSET);
        }

        bufPos += STR_BufAppend (NULL, bufPos, langStr,
                                 MAIN_LINEBUFCHARSET);

        item->item.str.chr = STR_IntLineStr();
    }
    else
    {
        item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
    }
}

FSTATIC void MENU_StrSelSetItemPars (FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    if ( item == NULL )
    {
        return;
    }
    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = appDat->txtAttr.colNonFocus; break;
        case FGS_FOCUS:    item->item.str.col = appDat->txtAttr.colFocus;    break;
        case FGS_DISABLED: item->item.str.col = appDat->txtAttr.colDisabled; break;
    }
}

#ifdef STRSEL_SUPPORT_SUBTT

FSTATIC int32_t MENU_StrSelUpdateSubttChoice (MENU_STRSEL_INST_S* inst)
{
    FGS_ITEM_SEL_PARAMS_S selParams;

    if (inst == NULL)   return APPL_SMARTGO_ERR_BAD_PARAMETER;
    if ( inst->pnlHdl == NULL ) return FAPI_OK;

    (void)FGS_LIST_SetRefreshMode(inst->listHdl, FFALSE);

    if ( inst->numSubttStreams == 0 )
    {
        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl, MENU_STRSEL_ID_ENTRY_SUBTT, FFALSE);
    }
    else
    {
        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl, MENU_STRSEL_ID_ENTRY_SUBTT, FTRUE);
    }

    selParams.numItems = inst->numSubttStreams + OFFSET_FOR_FGS; // for "Off" and "Auto"
    selParams.index = (inst->subttStrCurr + 1);

    (void)FGS_LIST_SetRefreshMode(inst->listHdl, FTRUE);

    (void)FGS_LIST_SetMemberParams(inst->listHdl, MENU_STRSEL_ID_ENTRY_SUBTT, &selParams);
    return FAPI_OK;
}

#endif /* STRSEL_SUPPORT_SUBTT */

FSTATIC int32_t MENU_StrSelUpdateAudioChoice (MENU_STRSEL_INST_S* inst)
{
    FGS_ITEM_SEL_PARAMS_S selParams;

    if ( inst == NULL )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    if ( inst->pnlHdl == NULL ) return FAPI_OK;

    selParams.index = 0;

    if ( inst->numMainAudStreams == 0 )
    {
        selParams.numItems = 1;
    }
    else
    {
        selParams.numItems = inst->numMainAudStreams;

        if ( inst->mainAudStrCurr >= 0 ) selParams.index = inst->mainAudStrCurr;
    }

    (void)FGS_LIST_SetMemberParams(inst->listHdl, MENU_STRSEL_ID_ENTRY_AUDIO, &selParams);
    return FAPI_OK;
}

FSTATIC int32_t MENU_StrSelUpdateSpdifChoice (MENU_STRSEL_INST_S* inst)
{
    FGS_ITEM_SEL_PARAMS_S selParams;

    if ( inst == NULL )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    if ( inst->pnlHdl == NULL ) return FAPI_OK;

    (void)FGS_LIST_SetRefreshMode(inst->listHdl, FFALSE);

    if ( inst->numSpdifStreams == 0 ) /* no streams for SPDIF passthr. avl. */
    {
        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                              MENU_STRSEL_ID_ENTRY_SPDIF,
                                              FFALSE);

        selParams.numItems = 1;
        selParams.index    = 0;
    }
    else
    {
        /* NOTE: Index 0 is used for PCM output of current I2S audio. */
        selParams.numItems = inst->numSpdifStreams + 1;

        switch ( inst->spdifStatusCurr )
        {
            default:
            case SPDIFSTATUS_DISABLED:    /* this should not happen */
            case SPDIFSTATUS_CURRPCM:
                selParams.index = 0;
                break;

            case SPDIFSTATUS_PASSTHROUGH:
                if ( inst->spdifStrCurr >= 0 )
                {
                    selParams.index = inst->spdifStrCurr + 1;
                }
                else
                {
                    selParams.index = 0;
                }
                break;
        }

        (void)FGS_LIST_SetMemberEnableStatus (inst->listHdl,
                                              MENU_STRSEL_ID_ENTRY_SPDIF,
                                              FTRUE);
    }

    (void)FGS_LIST_SetRefreshMode(inst->listHdl, FTRUE);
    (void)FGS_LIST_SetMemberParams(inst->listHdl, MENU_STRSEL_ID_ENTRY_SPDIF,
                                   &selParams);
    return FAPI_OK;
}

FSTATIC void MENU_StrSelReset (MENU_STRSEL_INST_S* inst)
{
    if ( inst == NULL )
    {
        return;
    }

    inst->numMainAudStreams = 0;
    inst->mainAudStrCurr    = -1;
    inst->numSpdifStreams   = 0;
    inst->spdifStrCurr      = -1;

    #ifdef STRSEL_SUPPORT_SUBTT

    inst->numSubttStreams  = 0;
    inst->subttStrCurr      = -1;

    #endif /* STRSEL_SUPPORT_SUBTT */

    inst->spdifStatusCurr  = SPDIFSTATUS_DISABLED;

    (void)MENU_StrSelUpdateAudioChoice(inst);
    (void)MENU_StrSelUpdateSpdifChoice(inst);

    #ifdef STRSEL_SUPPORT_SUBTT

    /* Subtitle list is always empty at this time: subtitle information of
       current service is received with PMT update. */
    (void)MENU_StrSelUpdateSubttChoice(inst);


    #endif /* STRSEL_SUPPORT_SUBTT */

    if ( inst->pnlHdl != NULL )
    {
        /* always set focus on audio selection, the other two could be
           disabled. */
        (void)FGS_LIST_SetMemberFocus(inst->listHdl, MENU_STRSEL_ID_ENTRY_AUDIO);
    }
}


/******************************************************************************/
/*lint -restore */
/* END OF FILE */
