/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/fgs/src/fgs_box.c
**
**              $Id: fgs_box.c 5866 2010-03-02 12:09:31Z jmorha $ 
**
** \brief       content description.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
**                
*****************************************************************************
*/
/*lint -save -e715 -e732 */

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "fgs_box.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

FSTATIC int32_t FGS_BOX_SetFocus   (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC int32_t FGS_BOX_Show       (FAPI_SYS_HandleT h);
FSTATIC int32_t FGS_BOX_Hide       (FAPI_SYS_HandleT h);
FSTATIC int32_t FGS_BOX_Close      (FAPI_SYS_HandleT h);

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC FGS_BOX_INIT_S*    boxGlobal  = NULL;
FSTATIC FGS_BOX_DRAWIF_S*  boxDraw    = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

int32_t FGS_BOX_Init (FGS_BOX_INIT_S* pInitParams)
{
    if ( pInitParams == NULL ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    boxGlobal = (FGS_BOX_INIT_S*)
                UTILS_Malloc(sizeof(FGS_BOX_INIT_S));
    if ( boxGlobal == NULL ) return SMARTGO_FGS_ERR_OUT_OF_MEMORY;
    
    (*boxGlobal) = (*pInitParams);
    boxDraw      = &(boxGlobal->draw);
    
    return FAPI_OK;
}

int32_t FGS_BOX_Open (FGS_BOX_OPEN_S* pData, int32_t id, FAPI_SYS_HandleT parent,
                   FAPI_SYS_HandleT* ph)
{
    FGS_BOX_INST_S*     inst;
    FGS_PANEL_MEMBER_S  member;
    int32_t                retVal;
    fbool_t               memAllocated = FFALSE;
    
    if ( (pData  == NULL) ||  
         (parent == NULL) ||
         (ph     == NULL) ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    if ( pData->pInst != NULL )
    {
        inst = pData->pInst;
    }
    else
    {
        inst = (FGS_BOX_INST_S*)
               UTILS_Malloc(sizeof(FGS_BOX_INST_S));
        if ( inst == NULL ) return SMARTGO_FGS_ERR_OUT_OF_MEMORY;
        
        memAllocated = FTRUE;
    }
    
    memset(inst, 0, sizeof(FGS_BOX_INST_S));
    
    inst->memAllocated = memAllocated;
    inst->data         = *pData;
    inst->pMem         = NULL;
    inst->pPanel       = (FGS_PANEL_INST_S*)parent;
    
    *ph = (FAPI_SYS_HandleT)inst;
    
    /* add as panel member */
    
    memset(&member, 0, sizeof(FGS_PANEL_MEMBER_S));
    
    member.h         = (FAPI_SYS_HandleT)inst;
    member.pos       = &(inst->data.pos);
    member.id        = id;
    member.isHidden  = inst->data.isHidden;
    member.setFocus  = FGS_BOX_SetFocus;
    member.show      = FGS_BOX_Show;
    member.hide      = FGS_BOX_Hide;
    member.close     = FGS_BOX_Close;
    
    retVal = FGS_PANEL_AddMember(parent, &member, &(inst->pMem));
    
    if ( (retVal             != FAPI_OK) &&
         (inst->memAllocated == FTRUE) )
    {
        UTILS_Free(inst);
    }
    
    return retVal;
}

FSTATIC int32_t FGS_BOX_Close (FAPI_SYS_HandleT h)
{
    FGS_BOX_INST_S*  inst = (FGS_BOX_INST_S*)h;
    
    if (inst == NULL) return SMARTGO_FGS_ERR_INVALID_HANDLE;
    
    if ( inst->memAllocated == FTRUE )
    {
        UTILS_Free(inst);
    }
    
    return FAPI_OK;
}

FSTATIC int32_t FGS_BOX_Show (FAPI_SYS_HandleT h)
{
    FGS_BOX_INST_S*   inst = (FGS_BOX_INST_S*)h;
    FGS_ITEM_S        item;
    FGS_ITEMSTATE_E   state;
    
    memset(&item, 0, sizeof(item));
    item.type = FGS_ITEM_TYPE_NONE;
    
    if ( inst->data.getItem != NULL )
    {
        if ( inst->pMem->hasFocus ) state = FGS_FOCUS;
        else                        state = FGS_NONFOCUS;
        
        inst->data.getItem (inst->data.getItemArg,
                             inst->pMem->id, 0, state, &item);
    }
    else // deprecated, to be removed!
    {
        if ( !(inst->pMem->hasFocus) )
        {
            item = inst->data.item[FGS_BOX_IDX_NONFOCUS];
        }
        else 
        {
            item = inst->data.item[FGS_BOX_IDX_FOCUS];
        }
    }
    
    if ( boxDraw->drawBkgr != NULL )
    {
        boxDraw->drawBkgr(h, inst->pPanel);
    }
    
    /* draw value item */
    if ( item.type != FGS_ITEM_TYPE_NONE )
    {
        (void)FGS_DrawItem (&item, inst->pMem->abspos.x,
                             inst->pMem->abspos.y,
                             inst->data.pos.width,
                             inst->data.pos.height);
    }
    
    return FAPI_OK;
}

FSTATIC int32_t FGS_BOX_Hide (FAPI_SYS_HandleT h)
{
    //FGS_BOX_INST_S*   inst = (FGS_BOX_INST_S*)h;
    
    return FAPI_OK;
}

// deprecated, to be removed
int32_t FGS_BOX_SetNonFocusItem (FAPI_SYS_HandleT h, FGS_ITEM_S* pItem)
{
    FGS_BOX_INST_S* inst   = (FGS_BOX_INST_S*)h;
    uint8_t              idx    = FGS_BOX_IDX_NONFOCUS;
    
    if ( (h     == NULL) ||
         (pItem == NULL) ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    if ( inst->data.getItem == NULL )
    {
        inst->data.item[idx] = *pItem;
        
        /* redraw item if currently visible */
        if (    inst->pMem->isVisible && 
             (!(inst->pMem->hasFocus)) )
        {
            (void)FGS_PANEL_RefreshMember((FAPI_SYS_HandleT)inst->pPanel, inst->pMem->id);
        }
    }
    
    return FAPI_OK;
}

// deprecated, to be removed
int32_t FGS_BOX_SetFocusItem (FAPI_SYS_HandleT h, FGS_ITEM_S* pItem)
{
    FGS_BOX_INST_S* inst   = (FGS_BOX_INST_S*)h;
    uint8_t              idx    = FGS_BOX_IDX_FOCUS;
    
    if ( (h     == NULL) ||
         (pItem == NULL) ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    if ( inst->data.getItem == NULL )
    {
        inst->data.item[idx] = *pItem;
        
        /* redraw item if currently visible */
        if ( inst->pMem->isVisible && 
             inst->pMem->hasFocus )
        {
            (void)FGS_PANEL_RefreshMember((FAPI_SYS_HandleT)inst->pPanel, inst->pMem->id);
        }
    }
    
    return FAPI_OK;
}

// deprecated, to be removed
int32_t FGS_BOX_Clear (FAPI_SYS_HandleT h)
{
    FGS_BOX_INST_S* inst   = (FGS_BOX_INST_S*)h;
    uint8_t              cnt;
    
    if ( inst->data.getItem == NULL )
    {
        for ( cnt = 0; cnt < FGS_BOX_IDX_MAX; cnt++ )
        {
            inst->data.item[cnt].type = FGS_ITEM_TYPE_NONE;
        }
        
        /* redraw item if currently visible */
        if ( inst->pMem->isVisible )
        {
            (void)FGS_PANEL_RefreshMember((FAPI_SYS_HandleT)inst->pPanel, inst->pMem->id);
        }
    }
    
    return FAPI_OK;
}

FSTATIC int32_t FGS_BOX_SetFocus (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    FGS_BOX_INST_S* inst   = (FGS_BOX_INST_S*)h;
    
    /* redraw item if currently visible */
    if ( inst->pMem->isVisible )
    {
        (void)FGS_PANEL_RefreshMember((FAPI_SYS_HandleT)inst->pPanel, inst->pMem->id);
    }
    
    return FAPI_OK;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
