/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/fgs/src/fgs_item_text.c
**
**              $Id: fgs_item_text.c 5866 2010-03-02 12:09:31Z jmorha $ 
**
** \brief       content description.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** 
*****************************************************************************
*/
/*lint -save */

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "fgs_item_text.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC FGS_ITEM_TXT_INIT_S*     fgsItemTxtGlobal  = NULL;
FSTATIC FGS_ITEM_DRAWIF_S*       fgsTxtDraw = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC int32_t  FGS_ITEM_TXT_Close       (FAPI_SYS_HandleT h);
FSTATIC int32_t  FGS_ITEM_TXT_Draw        (FAPI_SYS_HandleT h);
//FSTATIC int32_t  FGS_ITEM_TXT_SetValue    (FAPI_SYS_HandleT h, FGS_VAL_S* pValue);
FSTATIC int32_t  FGS_ITEM_TXT_SetFocus    (FAPI_SYS_HandleT h, fbool_t hasFocus);

int32_t FGS_ITEM_TXT_Init (FGS_ITEM_TXT_INIT_S* pInitParams)
{
    if ( pInitParams == NULL ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    fgsItemTxtGlobal = (FGS_ITEM_TXT_INIT_S*)
                UTILS_Malloc(sizeof(FGS_ITEM_TXT_INIT_S));
    if ( fgsItemTxtGlobal == NULL ) return SMARTGO_FGS_ERR_OUT_OF_MEMORY;
    
    (*fgsItemTxtGlobal) = (*pInitParams);
    
    fgsTxtDraw = &(fgsItemTxtGlobal->gfx);
    
    return FAPI_OK;
}

int32_t FGS_ITEM_TXT_Open (FGS_ITEM_TXT_OPEN_S* pData, int32_t id,
                        FAPI_SYS_HandleT parent, FAPI_SYS_HandleT* ph)
{
    FGS_ITEM_TXT_INST_S*  inst;
    FGS_LIST_MEMBER_S     member;
    int32_t                  retVal;
    
    if ( (pData  == NULL) ||  
         (parent == NULL) ||
         (ph     == NULL) ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    inst = (FGS_ITEM_TXT_INST_S*)
           UTILS_Malloc(sizeof(FGS_ITEM_TXT_INST_S));
    if ( inst == NULL ) return SMARTGO_FGS_ERR_OUT_OF_MEMORY;
    
    inst->data         = *pData;
    inst->pMem         = NULL;
    
    *ph = (FAPI_SYS_HandleT)inst;
    
    /* add as list member */
    
    memset(&member, 0, sizeof(FGS_LIST_MEMBER_S));
    
    member.h           = (FAPI_SYS_HandleT)inst;
    member.pos         = &(inst->data.pos);
    member.id          = id;
    member.isVisible   = FFALSE;
    member.hasFocus    = FFALSE;
    member.isActivated = FFALSE;
    member.descrPos    = inst->data.descrPos;
    member.valuePos    = inst->data.valuePos;
    member.setFocus    = FGS_ITEM_TXT_SetFocus;
    member.draw        = FGS_ITEM_TXT_Draw;
    member.close       = FGS_ITEM_TXT_Close;
    
    retVal = FGS_LIST_AddMember (parent, &member, &(inst->pMem));
    
    if ( retVal != FAPI_OK )
    {
        UTILS_Free(inst);
    }
    
    return retVal;
}

FSTATIC int32_t FGS_ITEM_TXT_Close (FAPI_SYS_HandleT h)
{
    FGS_ITEM_TXT_INST_S* inst = (FGS_ITEM_TXT_INST_S*)h;
    
    if (inst == NULL) return SMARTGO_FGS_ERR_INVALID_HANDLE;
    
    UTILS_Free(inst);
    
    return FAPI_OK;

}

FSTATIC int32_t FGS_ITEM_TXT_Draw (FAPI_SYS_HandleT h)
{
    FGS_ITEM_TXT_INST_S* inst = (FGS_ITEM_TXT_INST_S*)h;
    FGS_ITEM_S           item;
    FGS_ITEMSTATE_E      state;
    
    if ( !(inst->pMem->isVisible) ) return FAPI_OK;
    
    memset(&item, 0, sizeof(item));
    item.type = FGS_ITEM_TYPE_NONE;
    
    if ( inst->pMem->pParent->getItem != NULL )
    {
        if      (!(inst->pMem->isEnabled)) state = FGS_DISABLED;
        else if   (inst->pMem->hasFocus)   state = FGS_FOCUS;
        else                               state = FGS_NONFOCUS;
        
        inst->pMem->pParent->getItem (inst->pMem->pParent->getItemArg,
                                      inst->pMem->id, 0, state, &item);
    }
    
    /* paint background */
    if ( fgsTxtDraw->drawBkgr != NULL )
    {
        fgsTxtDraw->drawBkgr (h, inst->pMem->pPanel, &item);
    }
    
    /* draw description item */
    FGS_LIST_DrawItemDescr(inst->pMem);
    
    /* draw value item */
    if ( item.type != FGS_ITEM_TYPE_NONE )
    {
        /* draw value item */
        (void)FGS_DrawItem (&item,
                      (uint16_t)(inst->pMem->abspos.x + inst->pMem->valuePos.x),
                      (uint16_t)(inst->pMem->abspos.y + inst->pMem->valuePos.y),
                      inst->pMem->valuePos.width,
                      inst->pMem->valuePos.height);
    }
    
    /* paint rest */
    if ( fgsTxtDraw->drawRest != NULL )
    {
        fgsTxtDraw->drawRest(h);
    }
    
    return FAPI_OK;
}

FSTATIC int32_t FGS_ITEM_TXT_SetFocus (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    FGS_ITEM_TXT_INST_S* inst = (FGS_ITEM_TXT_INST_S*)h;
    
    if ( inst->pMem->hasFocus == hasFocus ) return FAPI_OK;
    
    inst->pMem->hasFocus = hasFocus;
    
    /* redraw item if currently visible */
    if ( inst->pMem->isVisible == FTRUE ) (void)FGS_ITEM_TXT_Draw(h);
    return FAPI_OK;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
