/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/fgs/src/fgs_tbox.c
**
**              $Id: fgs_tbox.c 5866 2010-03-02 12:09:31Z jmorha $ 
**
** \brief       content description.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    715: Symbol not referenced.
*****************************************************************************
*/
/*lint -save -e715 */

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "fgs_tbox.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

FSTATIC int32_t FGS_TBOX_SetFocus  (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC void    FGS_TBOX_PrintLine (FGS_TBOX_INST_S* inst, uint16_t lineIdx);
FSTATIC int32_t FGS_TBOX_Show      (FAPI_SYS_HandleT h);
FSTATIC int32_t FGS_TBOX_Hide      (FAPI_SYS_HandleT h);
FSTATIC int32_t FGS_TBOX_Close     (FAPI_SYS_HandleT h);

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC FGS_TBOX_INIT_S*   tboxGlobal = NULL;
FSTATIC FGS_TBOX_DRAWIF_S* tboxDraw   = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

int32_t FGS_TBOX_Init (FGS_TBOX_INIT_S* pInitParams)
{
    if ( pInitParams == NULL ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    tboxGlobal = (FGS_TBOX_INIT_S*)
                 UTILS_Malloc(sizeof(FGS_TBOX_INIT_S));
    if ( tboxGlobal == NULL ) return SMARTGO_FGS_ERR_OUT_OF_MEMORY;
    
    (*tboxGlobal) = (*pInitParams);
    tboxDraw      = &(tboxGlobal->draw);
    
    return FAPI_OK;
}

int32_t FGS_TBOX_Open (FGS_TBOX_OPEN_S* pData, int32_t id, FAPI_SYS_HandleT parent,
                    FAPI_SYS_HandleT* ph)
{
    FGS_TBOX_INST_S*    inst;
    FGS_PANEL_MEMBER_S  member;
    int32_t                retVal;
    uint16_t                 cnt;
    
    if ( (pData  == NULL) ||  
         (parent == NULL) ||
         (ph     == NULL) ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    if ( pData->numLines > FGS_TBOX_LINES_MAX )
    {
        return SMARTGO_FGS_ERR_BAD_PARAMETER;
    }
    
    inst = (FGS_TBOX_INST_S*)
           UTILS_Malloc(sizeof(FGS_TBOX_INST_S));
    if ( inst == NULL ) return SMARTGO_FGS_ERR_OUT_OF_MEMORY;
    
    memset(inst, 0, sizeof(FGS_TBOX_INST_S));
    
    inst->data         = *pData;
    inst->pMem         = NULL;
    inst->pPanel       = (FGS_PANEL_INST_S*)parent;
    
    /* initialise line items */
    for ( cnt = 0; cnt < FGS_TBOX_LINES_MAX; cnt++ )
    {
        inst->line[cnt].text[0]      = 0;
        inst->line[cnt].item.type    = FGS_ITEM_TYPE_NONE;
    }
    
    *ph = (FAPI_SYS_HandleT)inst;
    
    /* add as panel member */
    
    memset(&member, 0, sizeof(FGS_PANEL_MEMBER_S));
    
    member.h        = (FAPI_SYS_HandleT)inst;
    member.pos      = &(inst->data.pos);
    member.id       = id;
    member.isHidden = FFALSE;
    member.setFocus = FGS_TBOX_SetFocus;
    member.show     = FGS_TBOX_Show;
    member.hide     = FGS_TBOX_Hide;
    member.close    = FGS_TBOX_Close;
    
    retVal = FGS_PANEL_AddMember(parent, &member, &(inst->pMem));
    
    if ( retVal != FAPI_OK )
    {
        UTILS_Free(inst);
    }
    
    return retVal;
}

FSTATIC int32_t FGS_TBOX_Show (FAPI_SYS_HandleT h)
{
    FGS_TBOX_INST_S*  inst = (FGS_TBOX_INST_S*)h;
    uint16_t               cnt;
    
    for ( cnt = 0; cnt < inst->data.numLines; cnt++ )
    {
        FGS_TBOX_PrintLine(inst, cnt);
    }
    
    return FAPI_OK;
}

FSTATIC int32_t FGS_TBOX_Hide (FAPI_SYS_HandleT h)
{
    //FGS_TBOX_INST_S*  inst = (FGS_TBOX_INST_S*)h;
    
    /* nothing to do... */
    
    return FAPI_OK;
}

int32_t FGS_TBOX_SetText (FAPI_SYS_HandleT h, uint16_t lineIdx, FGS_STRING_S* txt,
                       uint8_t alignment)
{
    FGS_TBOX_INST_S* inst   = (FGS_TBOX_INST_S*)h;
    FGS_TBOX_LINE_S* pLine;
    
    if ( lineIdx >= inst->data.numLines ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    pLine = &(inst->line[lineIdx]);
    
    (void)UTILS_strncpy(pLine->text, txt->chr.chr, FGS_TBOX_MAX_CHAR_PER_LINE);
    
    pLine->item.type         = FGS_ITEM_TYPE_STRING;
    pLine->item.align        = alignment;
    pLine->item.item.str     = *txt;
    pLine->item.item.str.chr.chr = pLine->text;
    
    /* redraw line if box is currently visible */
    if ( inst->pMem->isVisible )
    {
        if ( FGS_GetOverlapStatus(inst->pPanel->pMem, &(inst->pMem->abspos),
                                  inst->data.pos.width,
                                  inst->data.pos.height) == FFALSE )
        {
            FGS_TBOX_PrintLine(inst, lineIdx);
        }
    }
    
    return FAPI_OK;
}

int32_t FGS_TBOX_Clear (FAPI_SYS_HandleT h)
{
    FGS_TBOX_INST_S* inst   = (FGS_TBOX_INST_S*)h;
    uint16_t              cnt;
    
    /* clear all lines */
    for ( cnt = 0; cnt < inst->data.numLines; cnt++ )
    {
        inst->line[cnt].text[0]   = 0;
        inst->line[cnt].item.type = FGS_ITEM_TYPE_NONE;
    }
    
    /* redraw box if currently visible */
    if ( inst->pMem->isVisible )
    {
        (void)FGS_PANEL_RefreshMember((FAPI_SYS_HandleT)inst->pPanel, inst->pMem->id);
    }
    
    return FAPI_OK;
}

int32_t FGS_TBOX_ClearLine (FAPI_SYS_HandleT h, uint16_t lineIdx)
{
    FGS_TBOX_INST_S* inst   = (FGS_TBOX_INST_S*)h;
    
    if ( lineIdx >= inst->data.numLines ) return SMARTGO_FGS_ERR_BAD_PARAMETER;
    
    inst->line[lineIdx].text[0]   = 0;
    inst->line[lineIdx].item.type = FGS_ITEM_TYPE_NONE;
    
    /* redraw line if currently visible */
    if ( inst->pMem->isVisible )
    {
        if ( FGS_GetOverlapStatus(inst->pPanel->pMem, &(inst->pMem->abspos),
                                  inst->data.pos.width,
                                  inst->data.pos.height) == FFALSE )
        {
            FGS_TBOX_PrintLine(inst, lineIdx);
        }
    }
    
    return FAPI_OK;

}

FSTATIC int32_t FGS_TBOX_Close (FAPI_SYS_HandleT h)
{
    if ( h == NULL ) return SMARTGO_FGS_ERR_INVALID_HANDLE;
    
    UTILS_Free(h);
    return FAPI_OK;
}

FSTATIC int32_t FGS_TBOX_SetFocus (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    FGS_TBOX_INST_S* inst   = (FGS_TBOX_INST_S*)h;
    
    /* redraw box if currently visible */
    if ( inst->pMem->isVisible )
    {
        (void)FGS_PANEL_RefreshMember((FAPI_SYS_HandleT)inst->pPanel, inst->pMem->id);
    }
    
    return FAPI_OK;
}

FSTATIC void FGS_TBOX_PrintLine (FGS_TBOX_INST_S* inst, uint16_t lineIdx)
{
    uint16_t posY;
    
    posY = (uint16_t)inst->pMem->abspos.y + (lineIdx * inst->data.lineHeight);
    
    if ( tboxDraw->drawLineBkgr != NULL )
    {
        tboxDraw->drawLineBkgr((FAPI_SYS_HandleT)inst,
                               inst->pPanel,
                               (uint16_t)inst->pMem->abspos.x,
                               posY,
                               inst->data.pos.width,
                               inst->data.lineHeight,
                               lineIdx);
    }
    
    (void)FGS_DrawItem (&(inst->line[lineIdx].item),
                  (uint16_t)inst->pMem->abspos.x, posY,
                  inst->data.pos.width,
                  inst->data.lineHeight);
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
