/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/inc/smartgo_types.h
**
**              $Id: smartgo_types.h 5866 2010-03-02 12:09:31Z jmorha $ 
**
** \brief       SmartGo-HD internal defined types.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef __SMARTGO_TYPES_H__
#define __SMARTGO_TYPES_H__

#include <fapi/sys_error.h>

/*!
********************************************************************************
** Standard error codes for SmartGo-HD
********************************************************************************
*/

/*! Bad parameter passed. */
#define APPL_SMARTGO_ERR_BAD_PARAMETER         (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define APPL_SMARTGO_ERR_OUT_OF_MEMORY         (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_OUT_OF_MEMORY)

/*! Device already initialised. */
#define APPL_SMARTGO_ERR_ALREADY_INITIALIZED   (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_ALREADY_INITIALIZED)

/*! Device not initialised. */
#define APPL_SMARTGO_ERR_NOT_INITIALIZED       (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_NOT_INITIALIZED)

/*! Feature or function is not available. */
#define APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_FEATURE_NOT_SUPPORTED)

/*! Timeout occured. */
#define APPL_SMARTGO_ERR_TIMEOUT               (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_TIMEOUT)

/*! The device is busy, try again later. */
#define APPL_SMARTGO_ERR_DEVICE_BUSY           (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_DEVICE_BUSY)

/*! Invalid handle was passed. */
#define APPL_SMARTGO_ERR_INVALID_HANDLE        (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define APPL_SMARTGO_ERR_SEMAPHORE_CREATE      (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_SEMAPHORE_CREATE)

/*! The driver's used version is not supported. */
#define APPL_SMARTGO_ERR_UNSUPPORTED_VERSION   (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_UNSUPPORTED_VERSION)

/*! Device/module is not open. */
#define APPL_SMARTGO_ERR_NOT_OPEN              (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_NOT_OPEN)

/*! Device/module is already open. */
#define APPL_SMARTGO_ERR_ALREADY_OPEN          (int32_t)(APPLICATION_SMARTGO_MODULE_BASE + FAPI_ERR_ALREADY_OPEN)

/*!
********************************************************************************
** Additional base values for specific SmartGo-HD modules 
********************************************************************************
*/
/*! SmartGo-HD System Layer specific error code base. */
#define APPL_SMARTGO_SYS_ERR_BASE               (int32_t)(APPLICATION_SMARTGO_MODULE_BASE - 1000)

/*! SmartGo-HD GUI System specific error code base. */
#define APPL_SMARTGO_FGS_ERR_BASE               (int32_t)(APPLICATION_SMARTGO_MODULE_BASE - 2000)

/*! SmartGo-HD Application Layer specific error code base. 
    Error codes are defined in the application itself! */
#define APPL_SMARTGO_APPL_ERR_BASE              (int32_t)(APPLICATION_SMARTGO_MODULE_BASE - 3000)

/*!
********************************************************************************
** SmartGo-HD system layer error codes
**
** Each error code shall be decreased in steps of 1.
********************************************************************************
*/

#define SMARTGO_SYS_ERR_PROHIBITED              (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-1)
//!< Requested operation currently not possible

#define SMARTGO_SYS_ERR_INTERNAL                (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-2)
//!< Internal error, should not occur!

#define SMARTGO_SYS_ERR_ABORTED                 (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-3)
//!< Internal use: action has been aborted

#define SMARTGO_SYS_ERR_CONFIG_ERROR            (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-4)
//!< Requested operation not available for current application configuration

#define SMARTGO_SYS_ERR_CAPACITY                (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-5)
//!< Requested operation requires space that is currently not available

#define SMARTGO_SYS_ERR_DATA_NOT_AVL            (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-6)
//!< Requested data is not available

#define SMARTGO_SYS_ERR_ALREADY_IN_LIST         (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-7)
//!< Item to be added is already in list

#define SMARTGO_SYS_ERR_REPLACED_IN_LIST        (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-8)
//!< Item to be added is already in list but has been replaced

#define SMARTGO_SYS_ERR_INVALID_DATA            (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-9)
//!< Invalid data has been processed

#define SMARTGO_SYS_ERR_RESOURCES               (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-10)
//!< System resources exhausted (no more threads/semaphores available, ...)

#define SMARTGO_SYS_ERR_ITEM_EMPTY              (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-11)
//!< Requested item (e.g. service) is empty.

#define SMARTGO_SYS_ERR_RTOS_ERROR              (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-13)
//!< An unspecified RTOS error has occured.

#define SMARTGO_SYS_ERR_DATA_NOT_PROVIDED       (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-14)
//!< Requested data is not provided (e.g. no EIT "Present" data provided by broadcaster)

#define SMARTGO_SYS_ERR_RSRC                    (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-15)
//!< Requesteed resource not available (general resource error message)

#define SMARTGO_SYS_ERR_RSRC_LIVE               (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-16)
//!< Resource not available due to live-TV currently running

#define SMARTGO_SYS_ERR_RSRC_PLAY               (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-17)
//!< Resource not available due to playback currently running

#define SMARTGO_SYS_ERR_RSRC_RECORD             (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-18)
//!< Resource not available due to record currently running

#define SMARTGO_SYS_ERR_RSRC_SCAN               (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-19)
//!< Resource not available due to scan currently running

#define SMARTGO_SYS_ERR_RSRC_CI                 (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-20)
//!< CI module is needed for decoding but is occupied or not available

#define SMARTGO_SYS_ERR_RSRC_NETWORK            (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-21)
//!< Requested network (e.g. satellite) cannot be accessed

#define SMARTGO_SYS_ERR_RSRC_PIP                (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-22)
//!< Resource not available due to PIP currently being active

#define SMARTGO_SYS_ERR_RSRC_TUNEMODE           (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-23)
//!< Resource not available due to tune mode currently running

#define SMARTGO_SYS_ERR_RSRC_UPDATE             (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-24)
//!< Resource not available due to software update currently active

#define SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE         (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-25)
//!< Operation requested for termination is not active.

#define SMARTGO_SYS_ERR_UPDATE_CRC              (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-26)
//!< The CRC check on input data for update failed

#define SMARTGO_SYS_ERR_UPDATE_CAPACITY         (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-27)
//!< The size check on input data for update failed

#define SMARTGO_SYS_ERR_FILEREAD                (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-28)
//!< File reading error 

#define SMARTGO_SYS_ERR_FILEWRITE               (int32_t)(APPL_SMARTGO_SYS_ERR_BASE-29)
//!< File writing error

/*!
********************************************************************************
** SmartGo-HD FGS (GUI) layer error codes
**
** Each error code shall be decreased in steps of 1.
********************************************************************************
*/

/*! Bad parameter passed. */
#define SMARTGO_FGS_ERR_BAD_PARAMETER           (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define SMARTGO_FGS_ERR_OUT_OF_MEMORY           (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_OUT_OF_MEMORY)

/*! Device already initialised. */
#define SMARTGO_FGS_ERR_ALREADY_INITIALIZED     (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_ALREADY_INITIALIZED)

/*! Device not initialised. */
#define SMARTGO_FGS_ERR_NOT_INITIALIZED         (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_NOT_INITIALIZED)

/*! Feature or function is not available. */
#define SMARTGO_FGS_ERR_FEATURE_NOT_SUPPORTED   (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_FEATURE_NOT_SUPPORTED)

/*! Timeout occured. */
#define SMARTGO_FGS_ERR_TIMEOUT                 (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_TIMEOUT)

/*! The device is busy, try again later. */
#define SMARTGO_FGS_ERR_DEVICE_BUSY             (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_DEVICE_BUSY)

/*! Invalid handle was passed. */
#define SMARTGO_FGS_ERR_INVALID_HANDLE          (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define SMARTGO_FGS_ERR_SEMAPHORE_CREATE        (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_SEMAPHORE_CREATE)

/*! The driver's used version is not supported. */
#define SMARTGO_FGS_ERR_UNSUPPORTED_VERSION     (int32_t)(APPL_SMARTGO_FGS_ERR_BASE + FAPI_ERR_UNSUPPORTED_VERSION)


#define SMARTGO_FGS_ERR_PROHIBITED              (int32_t)(APPL_SMARTGO_FGS_ERR_BASE-101)
//!< Requested operation currently not possible

#define SMARTGO_FGS_ERR_RESOURCES               (int32_t)(APPL_SMARTGO_FGS_ERR_BASE-102)
//!< System resources exhausted (no more threads/semaphores available, ...)

#ifndef FFALSE

typedef enum 
{
    FFALSE = 0,
    FTRUE
    
} fbool_t;

#define FFALSE FFALSE

#endif

#ifdef APPL_DEBUG_MODE
    #define FSTATIC
#else
    #define FSTATIC static
#endif

#define FINLINE

#endif /* __SMARTGO_TYPES_H__ */
/******************************************************************************/
/* END OF FILE */
