/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/inc/sys_languages.h
**
**              $Id: sys_languages.h 5866 2010-03-02 12:09:31Z jmorha $ 
**
** \brief       Language code definitions according to ISO 639-2
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _SYS_LANGUAGES_H_
#define _SYS_LANGUAGES_H_

#include <fapi/sys_driver.h>

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

#define TRIPLEASCII(a,b,c)  ((((uint32_t)(a)) << 16) | (((uint32_t)(b)) << 8) | (c))

typedef enum
{
    LANG_NOTSPECIFIED                         = 0xFFFFFF,
    LANG_NONE                                 = 0xFFFFFE,
    
    LANG_AFAR                                 = TRIPLEASCII('a', 'a', 'r'),
    LANG_ABKHAZIAN                            = TRIPLEASCII('a', 'b', 'k'),
    LANG_AFRIKAANS                            = TRIPLEASCII('a', 'f', 'r'),
    LANG_AMHARIC                              = TRIPLEASCII('a', 'm', 'h'),
    LANG_ARABIC                               = TRIPLEASCII('a', 'r', 'a'),
    LANG_ASSAMESE                             = TRIPLEASCII('a', 's', 'm'),
    LANG_AYMARA                               = TRIPLEASCII('a', 'y', 'm'),
    LANG_AZERBAIJANI                          = TRIPLEASCII('a', 'z', 'e'),
    LANG_BASHKIR                              = TRIPLEASCII('b', 'a', 'k'),
    LANG_BYELORUSSIAN                         = TRIPLEASCII('b', 'e', 'l'),
    LANG_BULGARIAN                            = TRIPLEASCII('b', 'u', 'l'),
    LANG_BIHARI                               = TRIPLEASCII('b', 'h', 'h'),
    LANG_BISLAMA                              = TRIPLEASCII('b', 'i', 's'),
    LANG_BENGALI                              = TRIPLEASCII('b', 'e', 'n'),
    LANG_TIBETAN                              = TRIPLEASCII('b', 'o', 'd'),
    LANG_BRETON                               = TRIPLEASCII('b', 'r', 'e'),
    LANG_CATALAN                              = TRIPLEASCII('c', 'a', 't'),
    LANG_CORSICAN                             = TRIPLEASCII('c', 'o', 's'),
    LANG_CZECH                                = TRIPLEASCII('c', 'z', 'e'),
    LANG_WELSH                                = TRIPLEASCII('w', 'e', 'l'),
    LANG_DANISH                               = TRIPLEASCII('d', 'a', 'n'),
    LANG_GERMAN                               = TRIPLEASCII('g', 'e', 'r'),
    LANG_GREEK                                = TRIPLEASCII('g', 'r', 'e'),
    LANG_ENGLISH                              = TRIPLEASCII('e', 'n', 'g'),
    LANG_ESPERANTO                            = TRIPLEASCII('e', 'p', 'o'),
    LANG_SPANISH                              = TRIPLEASCII('s', 'p', 'a'),
    LANG_ESTONIAN                             = TRIPLEASCII('e', 's', 't'),
    LANG_BASQUE                               = TRIPLEASCII('b', 'a', 'q'),
    LANG_PERSIAN                              = TRIPLEASCII('p', 'e', 'r'),
    LANG_FINNISH                              = TRIPLEASCII('f', 'i', 'n'),
    LANG_FIJI                                 = TRIPLEASCII('f', 'i', 'j'),
    LANG_FAROESE                              = TRIPLEASCII('f', 'a', 'o'),
    LANG_FRENCH                               = TRIPLEASCII('f', 'r', 'e'),
    LANG_FRISIAN                              = TRIPLEASCII('f', 'r', 'y'),
    LANG_IRISH                                = TRIPLEASCII('g', 'l', 'e'),
    LANG_SCOTS_GAELIC                         = TRIPLEASCII('g', 'l', 'a'),
    LANG_GALICIAN                             = TRIPLEASCII('g', 'l', 'g'),
    LANG_GUARANI                              = TRIPLEASCII('g', 'r', 'n'),
    LANG_GUJARATI                             = TRIPLEASCII('g', 'u', 'j'),
    LANG_HAUSA                                = TRIPLEASCII('h', 'a', 'u'),
    LANG_HINDI                                = TRIPLEASCII('h', 'i', 'n'),
    LANG_CROATIAN                             = TRIPLEASCII('s', 'c', 'r'),
    LANG_HUNGARIAN                            = TRIPLEASCII('h', 'u', 'n'),
    LANG_ARMENIAN                             = TRIPLEASCII('a', 'r', 'm'),
    LANG_INTERLINGUA                          = TRIPLEASCII('i', 'n', 'a'),
    LANG_INTERLINGUE                          = TRIPLEASCII('i', 'l', 'e'),
    LANG_INUPIAK                              = TRIPLEASCII('i', 'p', 'k'),
    LANG_INDONESIAN                           = TRIPLEASCII('i', 'n', 'd'),
    LANG_ICELANDIC                            = TRIPLEASCII('i', 'c', 'e'),
    LANG_ITALIAN                              = TRIPLEASCII('i', 't', 'a'),
    LANG_HEBREW                               = TRIPLEASCII('h', 'e', 'b'),
    LANG_JAPANESE                             = TRIPLEASCII('j', 'p', 'n'),
    LANG_YIDDISH                              = TRIPLEASCII('y', 'i', 'd'),
    LANG_JAVANESE                             = TRIPLEASCII('j', 'a', 'v'),
    LANG_GEORGIAN                             = TRIPLEASCII('g', 'e', 'o'),
    LANG_KAZAKH                               = TRIPLEASCII('k', 'a', 'z'),
    LANG_GREENLANDIC                          = TRIPLEASCII('k', 'a', 'l'),
    LANG_CAMBODIAN                            = TRIPLEASCII('k', 'h', 'm'),
    LANG_KANNADA                              = TRIPLEASCII('k', 'a', 'n'),
    LANG_KOREAN                               = TRIPLEASCII('k', 'o', 'r'),
    LANG_KASHMIRI                             = TRIPLEASCII('k', 'a', 's'),
    LANG_KURDISH                              = TRIPLEASCII('k', 'u', 'r'),
    LANG_KIRGHIZ                              = TRIPLEASCII('k', 'i', 'r'),
    LANG_LATIN                                = TRIPLEASCII('l', 'a', 't'),
    LANG_LINGALA                              = TRIPLEASCII('l', 'i', 'n'),
    LANG_LAOTHIAN                             = TRIPLEASCII('l', 'a', 'o'),
    LANG_LITHUANIAN                           = TRIPLEASCII('l', 'i', 't'),
    LANG_LATVIAN_LETTISH                      = TRIPLEASCII('l', 'a', 'v'),
    LANG_MALAGASY                             = TRIPLEASCII('m', 'l', 'g'),
    LANG_MAORI                                = TRIPLEASCII('m', 'a', 'o'),
    LANG_MACEDONIAN                           = TRIPLEASCII('m', 'a', 'c'),
    LANG_MALAYALAM                            = TRIPLEASCII('m', 'a', 'l'),
    LANG_MONGOLIAN                            = TRIPLEASCII('m', 'o', 'n'),
    LANG_MOLDAVIAN                            = TRIPLEASCII('m', 'o', 'l'),
    LANG_MARATHI                              = TRIPLEASCII('m', 'a', 'r'),
    LANG_MALAY                                = TRIPLEASCII('m', 'a', 'y'),
    LANG_MALTESE                              = TRIPLEASCII('m', 'l', 't'),
    LANG_BURMESE                              = TRIPLEASCII('b', 'u', 'r'),
    LANG_NAURU                                = TRIPLEASCII('n', 'a', 'u'),
    LANG_NEPALI                               = TRIPLEASCII('n', 'e', 'p'),
    LANG_DUTCH                                = TRIPLEASCII('d', 'u', 't'),
    LANG_NORWEGIAN                            = TRIPLEASCII('n', 'o', 'r'),
    LANG_OCCITAN                              = TRIPLEASCII('o', 'c', 'i'),
    LANG_AFAN_OROMO                           = TRIPLEASCII('o', 'r', 'm'),
    LANG_ORIYA                                = TRIPLEASCII('o', 'r', 'i'),
    LANG_PUNJABI                              = TRIPLEASCII('p', 'a', 'n'),
    LANG_POLISH                               = TRIPLEASCII('p', 'o', 'l'),
    LANG_PASHTO                               = TRIPLEASCII('p', 'u', 's'),
    LANG_PORTUGUESE                           = TRIPLEASCII('p', 'o', 'r'),
    LANG_QUECHUA                              = TRIPLEASCII('q', 'u', 'e'),
    LANG_RHAETO_ROMANCE                       = TRIPLEASCII('r', 'o', 'h'),
    LANG_KURUNDI                              = TRIPLEASCII('r', 'u', 'n'),
    LANG_ROMANIAN                             = TRIPLEASCII('r', 'u', 'm'),
    LANG_RUSSIAN                              = TRIPLEASCII('r', 'u', 's'),
    LANG_KINYARWANDA                          = TRIPLEASCII('k', 'i', 'n'),
    LANG_SANSKRIT                             = TRIPLEASCII('s', 'a', 'n'),
    LANG_SINDHI                               = TRIPLEASCII('s', 'n', 'd'),
    LANG_SANGHO                               = TRIPLEASCII('s', 'a', 'g'),
    LANG_SINGHALESE                           = TRIPLEASCII('s', 'i', 'n'),
    LANG_SLOVAK                               = TRIPLEASCII('s', 'l', 'o'),
    LANG_SLOVENIAN                            = TRIPLEASCII('s', 'l', 'v'),
    LANG_SAMOAN                               = TRIPLEASCII('s', 'm', 'o'),
    LANG_SHONA                                = TRIPLEASCII('s', 'n', 'a'),
    LANG_SOMALI                               = TRIPLEASCII('s', 'o', 'm'),
    LANG_ALBANIAN                             = TRIPLEASCII('a', 'l', 'b'),
    LANG_SERBIAN                              = TRIPLEASCII('s', 'c', 'c'),
    LANG_SISWATI                              = TRIPLEASCII('s', 's', 'w'),
    LANG_SESOTHO                              = TRIPLEASCII('s', 'o', 't'),
    LANG_SUNDANESE                            = TRIPLEASCII('s', 'u', 'n'),
    LANG_SWEDISH                              = TRIPLEASCII('s', 'w', 'e'),
    LANG_SWAHILI                              = TRIPLEASCII('s', 'w', 'a'),
    LANG_TAMIL                                = TRIPLEASCII('t', 'a', 'm'),
    LANG_TELUGU                               = TRIPLEASCII('t', 'e', 'l'),
    LANG_TAJIK                                = TRIPLEASCII('t', 'g', 'k'),
    LANG_THAI                                 = TRIPLEASCII('t', 'h', 'a'),
    LANG_TIGRINYA                             = TRIPLEASCII('t', 'i', 'r'),
    LANG_TURKMEN                              = TRIPLEASCII('t', 'u', 'k'),
    LANG_TAGALOG                              = TRIPLEASCII('t', 'g', 'l'),
    LANG_SETSWANA                             = TRIPLEASCII('t', 's', 'n'),
    LANG_TONGA                                = TRIPLEASCII('t', 'o', 'n'),
    LANG_TURKISH                              = TRIPLEASCII('t', 'u', 'r'),
    LANG_TSONGA                               = TRIPLEASCII('t', 's', 'o'),
    LANG_TATAR                                = TRIPLEASCII('t', 'a', 't'),
    LANG_TWI                                  = TRIPLEASCII('t', 'w', 'i'),
    LANG_UKRAINIAN                            = TRIPLEASCII('u', 'k', 'r'),
    LANG_URDU                                 = TRIPLEASCII('u', 'r', 'd'),
    LANG_UZBEK                                = TRIPLEASCII('u', 'z', 'b'),
    LANG_VIETNAMESE                           = TRIPLEASCII('v', 'i', 'e'),
    LANG_VOLAPUK                              = TRIPLEASCII('v', 'o', 'l'),
    LANG_WOLOF                                = TRIPLEASCII('w', 'o', 'l'),
    LANG_XHOSA                                = TRIPLEASCII('x', 'h', 'o'),
    LANG_YORUBA                               = TRIPLEASCII('y', 'o', 'r'),
    LANG_CHINESE                              = TRIPLEASCII('c', 'h', 'i'),
    LANG_ZULU                                 = TRIPLEASCII('z', 'u', 'l'),
    LANG_ORIGINAL                             = TRIPLEASCII('q', 'a', 'a'),
    
    LANG_UNDETERMINED                         = TRIPLEASCII('u', 'n', 'd')
    
} SYS_LANGCODE;

typedef enum
{
    CTRY_NOTSPECIFIED                         = 0xFFFFFF,
    
    /* ----- Country codes according to ISO-3166 --------------------------- */
    
    CTRY_ARUBA                                = TRIPLEASCII('A', 'B', 'W'),
    CTRY_AFGHANISTAN                          = TRIPLEASCII('A', 'F', 'G'),
    CTRY_ANGOLA                               = TRIPLEASCII('A', 'G', 'O'),
    CTRY_ANGUILLA                             = TRIPLEASCII('A', 'I', 'A'),
    CTRY_ALBANIA                              = TRIPLEASCII('A', 'L', 'B'),
    CTRY_ANDORRA                              = TRIPLEASCII('A', 'N', 'D'),
    CTRY_NETHERLANDS_ANTILLES                 = TRIPLEASCII('A', 'N', 'T'),
    CTRY_UNITED_ARAB_EMIRATES                 = TRIPLEASCII('A', 'R', 'E'),
    CTRY_ARGENTINA                            = TRIPLEASCII('A', 'R', 'G'),
    CTRY_ARMENIA                              = TRIPLEASCII('A', 'R', 'M'),
    CTRY_AMERICAN_SAMOA                       = TRIPLEASCII('A', 'S', 'M'),
    CTRY_ANTARCTICA                           = TRIPLEASCII('A', 'T', 'A'),
    CTRY_FRENCH_SOUTHERN_TERRITORIES          = TRIPLEASCII('A', 'T', 'F'),
    CTRY_ANTIGUA_AND_BARBUDA                  = TRIPLEASCII('A', 'T', 'G'),
    CTRY_AUSTRALIA                            = TRIPLEASCII('A', 'U', 'S'),
    CTRY_AUSTRIA                              = TRIPLEASCII('A', 'U', 'T'),
    CTRY_AZERBAIJAN                           = TRIPLEASCII('A', 'Z', 'E'),
    CTRY_BURUNDI                              = TRIPLEASCII('B', 'D', 'I'),
    CTRY_BELGIUM                              = TRIPLEASCII('B', 'E', 'L'),
    CTRY_BENIN                                = TRIPLEASCII('B', 'E', 'N'),
    CTRY_BURKINA_FASO                         = TRIPLEASCII('B', 'F', 'A'),
    CTRY_BANGLADESH                           = TRIPLEASCII('B', 'G', 'D'),
    CTRY_BULGARIA                             = TRIPLEASCII('B', 'G', 'R'),
    CTRY_BAHRAIN                              = TRIPLEASCII('B', 'H', 'R'),
    CTRY_BAHAMAS                              = TRIPLEASCII('B', 'H', 'S'),
    CTRY_BOSNIA_AND_HERZEGOVINA               = TRIPLEASCII('B', 'I', 'H'),
    CTRY_BELARUS                              = TRIPLEASCII('B', 'L', 'R'),
    CTRY_BELIZE                               = TRIPLEASCII('B', 'L', 'Z'),
    CTRY_BERMUDA                              = TRIPLEASCII('B', 'M', 'U'),
    CTRY_BOLIVIA                              = TRIPLEASCII('B', 'O', 'L'),
    CTRY_BRAZIL                               = TRIPLEASCII('B', 'R', 'A'),
    CTRY_BARBADOS                             = TRIPLEASCII('B', 'R', 'B'),
    CTRY_BRUNEI_DARUSSALAM                    = TRIPLEASCII('B', 'R', 'N'),
    CTRY_BHUTAN                               = TRIPLEASCII('B', 'T', 'N'),
    CTRY_BOUVET_ISLAND                        = TRIPLEASCII('B', 'V', 'T'),
    CTRY_BOTSWANA                             = TRIPLEASCII('B', 'W', 'A'),
    CTRY_CENTRAL_AFRICAN_REPUBLIC             = TRIPLEASCII('C', 'A', 'F'),
    CTRY_CANADA                               = TRIPLEASCII('C', 'A', 'N'),
    CTRY_COCOS_KEELING_ISLANDS                = TRIPLEASCII('C', 'C', 'K'),
    CTRY_SWITZERLAND                          = TRIPLEASCII('C', 'H', 'E'),
    CTRY_CHILE                                = TRIPLEASCII('C', 'H', 'L'),
    CTRY_CHINA                                = TRIPLEASCII('C', 'H', 'N'),
    CTRY_COTE_DIVOIRE                         = TRIPLEASCII('C', 'I', 'V'),
    CTRY_CAMEROON                             = TRIPLEASCII('C', 'M', 'R'),
    CTRY_CONGO_THE_DEMOCRATIC_REPUBLIC_OF_THE = TRIPLEASCII('C', 'O', 'D'),
    CTRY_CONGO                                = TRIPLEASCII('C', 'O', 'G'),
    CTRY_COOK_ISLANDS                         = TRIPLEASCII('C', 'O', 'K'),
    CTRY_COLOMBIA                             = TRIPLEASCII('C', 'O', 'L'),
    CTRY_COMOROS                              = TRIPLEASCII('C', 'O', 'M'),
    CTRY_CAPE_VERDE                           = TRIPLEASCII('C', 'P', 'V'),
    CTRY_COSTA_RICA                           = TRIPLEASCII('C', 'R', 'I'),
    CTRY_CUBA                                 = TRIPLEASCII('C', 'U', 'B'),
    CTRY_CHRISTMAS_ISLAND                     = TRIPLEASCII('C', 'X', 'R'),
    CTRY_CAYMAN_ISLANDS                       = TRIPLEASCII('C', 'Y', 'M'),
    CTRY_CYPRUS                               = TRIPLEASCII('C', 'Y', 'P'),
    CTRY_CZECH_REPUBLIC                       = TRIPLEASCII('C', 'Z', 'E'),
    CTRY_GERMANY                              = TRIPLEASCII('D', 'E', 'U'),
    CTRY_DJIBOUTI                             = TRIPLEASCII('D', 'J', 'I'),
    CTRY_DOMINICA                             = TRIPLEASCII('D', 'M', 'A'),
    CTRY_DENMARK                              = TRIPLEASCII('D', 'N', 'K'),
    CTRY_DOMINICAN_REPUBLIC                   = TRIPLEASCII('D', 'O', 'M'),
    CTRY_ALGERIA                              = TRIPLEASCII('D', 'Z', 'A'),
    CTRY_ECUADOR                              = TRIPLEASCII('E', 'C', 'U'),
    CTRY_EGYPT                                = TRIPLEASCII('E', 'G', 'Y'),
    CTRY_ERITREA                              = TRIPLEASCII('E', 'R', 'I'),
    CTRY_WESTERN_SARARA                       = TRIPLEASCII('E', 'S', 'H'),
    CTRY_SPAIN                                = TRIPLEASCII('E', 'S', 'P'),
    CTRY_ESTONIA                              = TRIPLEASCII('E', 'S', 'T'),
    CTRY_ETHIOPIA                             = TRIPLEASCII('E', 'T', 'H'),
    CTRY_FINLAND                              = TRIPLEASCII('F', 'I', 'N'),
    CTRY_FIJI                                 = TRIPLEASCII('F', 'J', 'I'),
    CTRY_FALKLAND_ISLANDS                     = TRIPLEASCII('F', 'L', 'K'),
    CTRY_FRANCE                               = TRIPLEASCII('F', 'R', 'A'),
    CTRY_FAROE_ISLANDS                        = TRIPLEASCII('F', 'R', 'O'),
    CTRY_MICRONESIA_FEDERATED_STATES_OF       = TRIPLEASCII('F', 'S', 'M'),
    CTRY_GABON                                = TRIPLEASCII('G', 'A', 'B'),
    CTRY_UNITED_KINGDOM                       = TRIPLEASCII('G', 'B', 'R'),
    CTRY_GEORGIA                              = TRIPLEASCII('G', 'E', 'O'),
    CTRY_GUERNSEY                             = TRIPLEASCII('G', 'G', 'Y'),
    CTRY_GHANA                                = TRIPLEASCII('G', 'H', 'A'),
    CTRY_GIBRALTAR                            = TRIPLEASCII('G', 'I', 'B'),
    CTRY_GUINEA                               = TRIPLEASCII('G', 'I', 'N'),
    CTRY_GUADELOUPE                           = TRIPLEASCII('G', 'L', 'P'),
    CTRY_GAMBIA                               = TRIPLEASCII('G', 'M', 'B'),
    CTRY_GUINEA_BISSAU                        = TRIPLEASCII('G', 'N', 'B'),
    CTRY_EQUATORIAL_GUINEA                    = TRIPLEASCII('G', 'N', 'Q'),
    CTRY_GREECE                               = TRIPLEASCII('G', 'R', 'C'),
    CTRY_GRENADA                              = TRIPLEASCII('G', 'R', 'D'),
    CTRY_GREENLAND                            = TRIPLEASCII('G', 'R', 'L'),
    CTRY_GUATEMALA                            = TRIPLEASCII('G', 'T', 'M'),
    CTRY_FRENCH_GUIANA                        = TRIPLEASCII('G', 'U', 'F'),
    CTRY_GUAM                                 = TRIPLEASCII('G', 'U', 'M'),
    CTRY_GUYANA                               = TRIPLEASCII('G', 'U', 'Y'),
    CTRY_HONG_KONG                            = TRIPLEASCII('H', 'K', 'G'),
    CTRY_HEARD_ISLAND_AND_MCDONALD_ISLANDS    = TRIPLEASCII('H', 'M', 'D'),
    CTRY_HONDURAS                             = TRIPLEASCII('H', 'N', 'D'),
    CTRY_CROATIA                              = TRIPLEASCII('H', 'R', 'V'),
    CTRY_HAITI                                = TRIPLEASCII('H', 'T', 'I'),
    CTRY_HUNGARY                              = TRIPLEASCII('H', 'U', 'N'),
    CTRY_INDONESIA                            = TRIPLEASCII('I', 'D', 'N'),
    CTRY_ISLE_OF_MAN                          = TRIPLEASCII('I', 'M', 'N'),
    CTRY_INDIA                                = TRIPLEASCII('I', 'N', 'D'),
    CTRY_BRITISH_INDIAN_OCEAN_TERRITORY       = TRIPLEASCII('I', 'O', 'T'),
    CTRY_IRELAND                              = TRIPLEASCII('I', 'R', 'L'),
    CTRY_IRAN_ISLAMIC_REPUBLIC_OF             = TRIPLEASCII('I', 'R', 'N'),
    CTRY_IRAQ                                 = TRIPLEASCII('I', 'R', 'Q'),
    CTRY_ICELAND                              = TRIPLEASCII('I', 'S', 'L'),
    CTRY_ISRAEL                               = TRIPLEASCII('I', 'S', 'R'),
    CTRY_ITALY                                = TRIPLEASCII('I', 'T', 'A'),
    CTRY_JAMAICA                              = TRIPLEASCII('J', 'A', 'M'),
    CTRY_JERSEY                               = TRIPLEASCII('J', 'E', 'Y'),
    CTRY_JORDAN                               = TRIPLEASCII('J', 'O', 'R'),
    CTRY_JAPAN                                = TRIPLEASCII('J', 'P', 'N'),
    CTRY_KAZAKHSTAN                           = TRIPLEASCII('K', 'A', 'Z'),
    CTRY_KENYA                                = TRIPLEASCII('K', 'E', 'N'),
    CTRY_KYRGYZSTAN                           = TRIPLEASCII('K', 'G', 'Z'),
    CTRY_CAMBODIA                             = TRIPLEASCII('K', 'H', 'M'),
    CTRY_KIRIBATI                             = TRIPLEASCII('K', 'I', 'R'),
    CTRY_SAINT_KITTS_AND_NEVIS                = TRIPLEASCII('K', 'N', 'A'),
    CTRY_KOREA_REPUBLIC_OF                    = TRIPLEASCII('K', 'O', 'R'),
    CTRY_KUWAIT                               = TRIPLEASCII('K', 'W', 'T'),
    CTRY_LAO_PEOPLES_DEMOCRATIC_REPUBLIC      = TRIPLEASCII('L', 'A', 'O'),
    CTRY_LEBANON                              = TRIPLEASCII('L', 'B', 'N'),
    CTRY_LIBERIA                              = TRIPLEASCII('L', 'B', 'R'),
    CTRY_LIBYAN_ARAB_JAMABIRIYA               = TRIPLEASCII('L', 'B', 'Y'),
    CTRY_SAINT_LUCIA                          = TRIPLEASCII('L', 'C', 'A'),
    CTRY_LIECHTENSTEIN                        = TRIPLEASCII('L', 'I', 'E'),
    CTRY_SRI_LANKA                            = TRIPLEASCII('L', 'K', 'A'),
    CTRY_LESOTHO                              = TRIPLEASCII('L', 'S', 'O'),
    CTRY_LITHUANIA                            = TRIPLEASCII('L', 'T', 'U'),
    CTRY_LUXEMBOURG                           = TRIPLEASCII('L', 'U', 'X'),
    CTRY_LATVIA                               = TRIPLEASCII('L', 'V', 'A'),
    CTRY_MACAU                                = TRIPLEASCII('M', 'A', 'C'),
    CTRY_MOROCCO                              = TRIPLEASCII('M', 'A', 'R'),
    CTRY_MONACO                               = TRIPLEASCII('M', 'C', 'O'),
    CTRY_MOLDOVA                              = TRIPLEASCII('M', 'D', 'A'),
    CTRY_MADAGASCAR                           = TRIPLEASCII('M', 'D', 'G'),
    CTRY_MALDIVES                             = TRIPLEASCII('M', 'D', 'V'),
    CTRY_MEXICO                               = TRIPLEASCII('M', 'E', 'X'),
    CTRY_MARSHALL_ISLANDS                     = TRIPLEASCII('M', 'H', 'L'),
    CTRY_MACEDONIA                            = TRIPLEASCII('M', 'K', 'D'),
    CTRY_MALI                                 = TRIPLEASCII('M', 'L', 'I'),
    CTRY_MALTA                                = TRIPLEASCII('M', 'L', 'T'),
    CTRY_MYANMAR                              = TRIPLEASCII('M', 'M', 'R'),
    CTRY_MONTENEGRO                           = TRIPLEASCII('M', 'N', 'E'),
    CTRY_MONGOLIA                             = TRIPLEASCII('M', 'N', 'G'),
    CTRY_NORTHERN_MARIANA_ISLANDS             = TRIPLEASCII('M', 'N', 'P'),
    CTRY_MOZAMBIQUE                           = TRIPLEASCII('M', 'O', 'Z'),
    CTRY_MAURITANIA                           = TRIPLEASCII('M', 'R', 'T'),
    CTRY_MONTSERRAT                           = TRIPLEASCII('M', 'S', 'R'),
    CTRY_MARTINIQUE                           = TRIPLEASCII('M', 'T', 'Q'),
    CTRY_MAURITIUS                            = TRIPLEASCII('M', 'U', 'S'),
    CTRY_MALAWI                               = TRIPLEASCII('M', 'W', 'I'),
    CTRY_MALAYSIA                             = TRIPLEASCII('M', 'Y', 'S'),
    CTRY_MAYOTTE                              = TRIPLEASCII('M', 'Y', 'T'),
    CTRY_NAMIBIA                              = TRIPLEASCII('N', 'A', 'M'),
    CTRY_NEW_CALEDONIA                        = TRIPLEASCII('N', 'C', 'L'),
    CTRY_NIGER                                = TRIPLEASCII('N', 'E', 'R'),
    CTRY_NORFOLK_ISLAND                       = TRIPLEASCII('N', 'F', 'K'),
    CTRY_NIGERIA                              = TRIPLEASCII('N', 'G', 'A'),
    CTRY_NICARAGUA                            = TRIPLEASCII('N', 'I', 'C'),
    CTRY_NIUE                                 = TRIPLEASCII('N', 'I', 'U'),
    CTRY_NETHERLANDS                          = TRIPLEASCII('N', 'L', 'D'),
    CTRY_NORWAY                               = TRIPLEASCII('N', 'O', 'R'),
    CTRY_NEPAL                                = TRIPLEASCII('N', 'P', 'L'),
    CTRY_NAURU                                = TRIPLEASCII('N', 'R', 'U'),
    CTRY_NEW_ZEALAND                          = TRIPLEASCII('N', 'Z', 'L'),
    CTRY_OMAN                                 = TRIPLEASCII('O', 'M', 'N'),
    CTRY_PAKISTAN                             = TRIPLEASCII('P', 'A', 'K'),
    CTRY_PANAMA                               = TRIPLEASCII('P', 'A', 'N'),
    CTRY_PITCAIRN                             = TRIPLEASCII('P', 'C', 'N'),
    CTRY_PERU                                 = TRIPLEASCII('P', 'E', 'R'),
    CTRY_PHILIPPINES                          = TRIPLEASCII('P', 'H', 'L'),
    CTRY_PALAU                                = TRIPLEASCII('P', 'L', 'W'),
    CTRY_PAPUA_NEW_GUINEA                     = TRIPLEASCII('P', 'N', 'G'),
    CTRY_POLAND                               = TRIPLEASCII('P', 'O', 'L'),
    CTRY_PUERTO_RICO                          = TRIPLEASCII('P', 'R', 'I'),
    CTRY_KOREA_DEMOCRATIC_PEOPLES_REPUBLIC_OF = TRIPLEASCII('P', 'R', 'K'),
    CTRY_PORTUGAL                             = TRIPLEASCII('P', 'R', 'T'),
    CTRY_PARAGUAY                             = TRIPLEASCII('P', 'R', 'Y'),
    CTRY_PALESTINIA                           = TRIPLEASCII('P', 'S', 'E'),
    CTRY_FRENCH_POLYNESIA                     = TRIPLEASCII('P', 'Y', 'F'),
    CTRY_QATAR                                = TRIPLEASCII('Q', 'A', 'T'),
    CTRY_REUNION                              = TRIPLEASCII('R', 'E', 'U'),
    CTRY_ROMANIA                              = TRIPLEASCII('R', 'O', 'U'),
    CTRY_RUSSIAN_FEDERATION                   = TRIPLEASCII('R', 'U', 'S'),
    CTRY_RWANDA                               = TRIPLEASCII('R', 'W', 'A'),
    CTRY_SAUDI_ARABIA                         = TRIPLEASCII('S', 'A', 'U'),
    CTRY_SUDAN                                = TRIPLEASCII('S', 'D', 'N'),
    CTRY_SENEGAL                              = TRIPLEASCII('S', 'E', 'N'),
    CTRY_SINGAPORE                            = TRIPLEASCII('S', 'G', 'P'),
    CTRY_SOUTH_GEORGIA                        = TRIPLEASCII('S', 'G', 'S'),
    CTRY_SAINT_HELENA                         = TRIPLEASCII('S', 'H', 'N'),
    CTRY_SVALBARD_AND_JAN_MAYEN               = TRIPLEASCII('S', 'J', 'M'),
    CTRY_SOLOMON_ISLANDS                      = TRIPLEASCII('S', 'L', 'B'),
    CTRY_SIERRA_LEONE                         = TRIPLEASCII('S', 'L', 'E'),
    CTRY_EL_SALVADOR                          = TRIPLEASCII('S', 'L', 'V'),
    CTRY_SAN_MARINO                           = TRIPLEASCII('S', 'M', 'R'),
    CTRY_SOMALIA                              = TRIPLEASCII('S', 'O', 'M'),
    CTRY_SAINT_PIERRE_AND_MIQUELON            = TRIPLEASCII('S', 'P', 'M'),
    CTRY_SERBIA                               = TRIPLEASCII('S', 'R', 'B'),
    CTRY_SAO_TOME_AND_PRINCIPE                = TRIPLEASCII('S', 'T', 'P'),
    CTRY_SURINAME                             = TRIPLEASCII('S', 'U', 'R'),
    CTRY_SLOVAKIA                             = TRIPLEASCII('S', 'V', 'K'),
    CTRY_SLOVENIA                             = TRIPLEASCII('S', 'V', 'N'),
    CTRY_SWEDEN                               = TRIPLEASCII('S', 'W', 'E'),
    CTRY_SWAZILAND                            = TRIPLEASCII('S', 'W', 'Z'),
    CTRY_SEYCHELLES                           = TRIPLEASCII('S', 'Y', 'C'),
    CTRY_SYRIAN_ARAB_REPUBLIC                 = TRIPLEASCII('S', 'Y', 'R'),
    CTRY_TURKS_AND_CAICOS_ISLANDS             = TRIPLEASCII('T', 'C', 'A'),
    CTRY_CHAD                                 = TRIPLEASCII('T', 'C', 'D'),
    CTRY_TOGO                                 = TRIPLEASCII('T', 'G', 'O'),
    CTRY_THAILAND                             = TRIPLEASCII('T', 'H', 'A'),
    CTRY_TAJIKISTAN                           = TRIPLEASCII('T', 'J', 'K'),
    CTRY_TOKELAU                              = TRIPLEASCII('T', 'K', 'L'),
    CTRY_TURKMENISTAN                         = TRIPLEASCII('T', 'K', 'M'),
    CTRY_EAST_TIMOR                           = TRIPLEASCII('T', 'L', 'S'),
    CTRY_TONGA                                = TRIPLEASCII('T', 'O', 'N'),
    CTRY_TRINIDAD_TOBAGO                      = TRIPLEASCII('T', 'T', 'O'),
    CTRY_TUNISIA                              = TRIPLEASCII('T', 'U', 'N'),
    CTRY_TURKEY                               = TRIPLEASCII('T', 'U', 'R'),
    CTRY_TUVALU                               = TRIPLEASCII('T', 'U', 'V'),
    CTRY_TAIWAN                               = TRIPLEASCII('T', 'W', 'N'),
    CTRY_TANZANIA                             = TRIPLEASCII('T', 'Z', 'A'),
    CTRY_UGANDA                               = TRIPLEASCII('U', 'G', 'A'),
    CTRY_UKRAINE                              = TRIPLEASCII('U', 'K', 'R'),
    CTRY_UNITED_STATES_MINOR_OUTLYING_ISLANDS = TRIPLEASCII('U', 'M', 'I'),
    CTRY_URUGUAY                              = TRIPLEASCII('U', 'R', 'Y'),
    CTRY_UNITED_STATES                        = TRIPLEASCII('U', 'S', 'A'),
    CTRY_UZBEKISTAN                           = TRIPLEASCII('U', 'Z', 'B'),
    CTRY_VATICAN                              = TRIPLEASCII('V', 'A', 'T'),
    CTRY_SAINT_VINCENT_GRENADINES             = TRIPLEASCII('V', 'C', 'T'),
    CTRY_VENEZUELA                            = TRIPLEASCII('V', 'E', 'N'),
    CTRY_VIRGIN_ISLANDS_BRITISH               = TRIPLEASCII('V', 'G', 'B'),
    CTRY_VIRGIN_ISLANDS_US                    = TRIPLEASCII('V', 'I', 'R'),
    CTRY_VIETNAM                              = TRIPLEASCII('V', 'N', 'M'),
    CTRY_VANUATU                              = TRIPLEASCII('V', 'U', 'T'),
    CTRY_WALLIS_AND_FUTUNA                    = TRIPLEASCII('W', 'L', 'F'),
    CTRY_SAMOA                                = TRIPLEASCII('W', 'S', 'M'),
    CTRY_YEMEN                                = TRIPLEASCII('Y', 'E', 'M'),
    CTRY_SOUTH_AFRICA                         = TRIPLEASCII('Z', 'A', 'F'),
    CTRY_ZAMBIA                               = TRIPLEASCII('Z', 'M', 'B'),
    CTRY_ZIMBABWE                             = TRIPLEASCII('Z', 'W', 'E'),
    
    /* ----- Supplementary country codes defined by ETSI (ETR 162) --------- */
    
    CTRY_ETR_SCANDINAVIA                      = TRIPLEASCII('9', '0', '0'),
    CTRY_ETR_NORTH_AMERICA                    = TRIPLEASCII('9', '0', '1'),
    CTRY_ETR_ALL                              = TRIPLEASCII('9', '0', '2')
    
} SYS_CTRYCODE;

#endif /* _SYS_LANGUAGES_H_ */
/******************************************************************************/
/* END OF FILE */
