/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/inc/init.h
**
**              $Id: init.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       System initialisation and internally used utility functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _INIT_H_
#define _INIT_H_

#include "config.h"
//#include <fapi/sys_init_mb86h60_devkit.h>

/*---------------------------------------------------------------------------*/
/* buffer manager configuration                                              */
/*---------------------------------------------------------------------------*/
#ifdef APPL_PVR_ENABLED
    #define PSI_USED_TSDS     FAPI_TSD_MAX
#else
    #define PSI_USED_TSDS     1
#endif /* APPL_PVR_ENABLED */

#if (defined APPL_CA_ENABLED) || (defined APPL_CI_ENABLED)
    #define PSI_BUFS_PER_TSD     4   //!< PAT/PMT/CAT/NIT
#else
    #define PSI_BUFS_PER_TSD     3   //!< PAT/PMT/NIT
#endif /* (defined APPL_CA_ENABLED) || (defined APPL_CI_ENABLED) */

#define INIT_BM_OFFSET     (FAPI_SYS_MMAP_BM_SECTION_START \
                          - FAPI_SYS_MMAP_DDR_START)

#define INIT_PAT_SIZE      0x00001000
#define INIT_PMT_SIZE      0x00001000
#define INIT_PCR_SIZE      0

#if (defined APPL_CA_ENABLED) || (defined APPL_CI_ENABLED)
    #define INIT_CAT_SIZE      0x00001000
#else
    #define INIT_CAT_SIZE      0
#endif

#define INIT_NIT_SIZE      0x00001000
#define INIT_SDT_SIZE      0x00001000
#define INIT_BAT_SIZE      0x00001000
#define INIT_TDTTOT_SIZE   0x00002000

#define INIT_EIT_BUFSIZE   0x00002000
// TODO: use a constant here (instead of value 4)
#define INIT_EIT_SIZE      (4 * INIT_EIT_BUFSIZE)

#define INIT_PCR_ADDR        INIT_BM_OFFSET
#define INIT_ECM_ADDR       (INIT_PCR_ADDR      + INIT_PCR_SIZE)
#define INIT_EMM_ADDR       (INIT_ECM_ADDR      + INIT_ECM_SIZE)
#define INIT_PAT_ADDR       (INIT_EMM_ADDR      + INIT_EMM_SIZE)
#define INIT_PMT_ADDR       (INIT_PAT_ADDR      + (INIT_PAT_SIZE \
                                                 * PSI_USED_TSDS))
#define INIT_CAT_ADDR       (INIT_PMT_ADDR      + (INIT_PMT_SIZE \
                                                 * PSI_USED_TSDS))
#define INIT_NIT_ADDR       (INIT_CAT_ADDR      + (INIT_CAT_SIZE \
                                                 * PSI_USED_TSDS))
#define INIT_SDT_ADDR       (INIT_NIT_ADDR      + (INIT_NIT_SIZE \
                                                 * PSI_USED_TSDS))
#define INIT_BAT_ADDR       (INIT_SDT_ADDR      + INIT_SDT_SIZE)
#define INIT_TDTTOT_ADDR    (INIT_BAT_ADDR      + INIT_BAT_SIZE)
#define INIT_EIT_ADDR       (INIT_TDTTOT_ADDR   + INIT_TDTTOT_SIZE)
#define INIT_PIP_ADDR       (INIT_EIT_ADDR      + INIT_EIT_SIZE)
#define INIT_PIPSRC_ADDR    (INIT_PIP_ADDR      + INIT_PIP_SIZE)
#define INIT_OAD_ADDR       (INIT_PIPSRC_ADDR   + INIT_PIPSRC_SIZE)

#define INIT_END_ADDR       (INIT_OAD_ADDR      + INIT_OAD_SIZE)

#define INIT_ALL_SIZE       (INIT_END_ADDR      - INIT_BM_OFFSET)

#if ( INIT_ALL_SIZE > FAPEX_SYS_MMAP_BM_SECTION_BYTES )
    //#error Application start address collides with buffer reservations
#endif

/* ------------------------------------------------------------------------- */

#ifdef __cplusplus
extern "C" {
#endif

void        InitAll      (void);

void        SYS_MemoryException   (void);
void        SYS_InstructionError  (void);

#ifdef __cplusplus
}
#endif

#endif /* _INIT_H_ */

/******************************************************************************/
/* END OF FILE */
