/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/inc/linklist.h
**
**              $Id: linklist.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       General, double-linked list for internal use.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _LINKLIST_H_
#define _LINKLIST_H_


/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

struct linklst_hdr_s
{
    struct linklst_hdr_s *pPrev;        //!< points to next list entry (NULL if none)
    struct linklst_hdr_s *pNext;        //!< points to prev. list entry (NULL if none)
    void               *listSig;        //!< list identification
};

typedef struct linklst_hdr_s LINKLST_HDR_S;

typedef struct
{
    LINKLST_HDR_S      *pFirst; //!< points to first item in list (NULL if empty)
    LINKLST_HDR_S      *pLast;  //!< points to last item in list (NULL if empty)
    uint32_t            numItems;       //!< number of list items

} LINKLST_S;




/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern              "C"
{
#endif

    int32_t             LINKLST_Init(LINKLST_S * pLst, uint32_t numItems,
                                     uint32_t itemSize, void *pItem);
    int32_t             LINKLST_PushFront(LINKLST_S * pLst,
                                          LINKLST_HDR_S * pItem);
    int32_t             LINKLST_PushBack(LINKLST_S * pLst,
                                         LINKLST_HDR_S * pItem);
    int32_t             LINKLST_Insert(LINKLST_S * pLst, LINKLST_HDR_S * pItem,
                                       LINKLST_HDR_S * pRef,
                                       fbool_t insertBefore);
    int32_t             LINKLST_Remove(LINKLST_S * pLst, LINKLST_HDR_S * pItem);
    LINKLST_HDR_S      *LINKLST_GetFirst(LINKLST_S * pLst);
    LINKLST_HDR_S      *LINKLST_GetLast(LINKLST_S * pLst);
    LINKLST_HDR_S      *LINKLST_GetByIndex(LINKLST_S * pLst, uint32_t index);
    LINKLST_HDR_S      *LINKLST_GetNext(LINKLST_S * pLst,
                                        LINKLST_HDR_S * pItem);
    LINKLST_HDR_S      *LINKLST_GetPrevious(LINKLST_S * pLst,
                                            LINKLST_HDR_S * pItem);
    uint32_t            LINKLST_GetListSize(LINKLST_S * pLst);

#ifdef __cplusplus
}
#endif

#endif /* _LINKLIST_H_ */

/******************************************************************************/
/* END OF FILE */
