/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/inc/mheg.h
**
**              $Id: mheg.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       TODO
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _MHEG_H_
#define _MHEG_H_

#include <sfal/sfal.h>
#include <mheg/itk_types.h>
#include <mheg/itk.h>
#include <mheg/itk_nvs.h>

#include "sys_priv.h"

/******************************************************************************
** @section MHEG-5
**
** MHEG-5 Engine
**
** @{
******************************************************************************/

/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

typedef enum
{
    MHEG_CMD_VD_FREEZE = 0,
    MHEG_CMD_VD_PLAY,
    MHEG_CMD_VD_BLANK,
    MHEG_CMD_VD_STOP,
    MHEG_CMD_VD_UNUSED,

    MHEG_CMD_AD_PAUSE,
    MHEG_CMD_AD_PLAY,
    MHEG_CMD_AD_STOP,
    MHEG_CMD_AD_CLOSE,
    MHEG_CMD_AD_OPEN,
    MHEG_CMD_AD_UNUSED,

    MHEG_CMD_AC_PLAY,
    MHEG_CMD_AC_PAUSE,
    MHEG_CMD_AC_STOP,

    MHEG_CMD_DX_ALLOC_FILTERSET,
    MHEG_CMD_DX_FREE_FILTERSET,
    MHEG_CMD_DX_ALLOC_FILTER,
    MHEG_CMD_DX_FREE_FILTER,
    MHEG_CMD_DX_CONTROL_FILTER,
    MHEG_CMD_DX_REQUEST_PAT,
    MHEG_CMD_DX_REQUEST_PMT,

    MHEG_CMD_NUMS

} MHEG_CMD_TYPE_E;

typedef struct
{
    SFAL_FILTERSET_S    set;
    FAPI_SYS_HandleT    hdl;

} MHEG_CMD_DX_ALLOCSET_S;

typedef struct
{
    FAPI_SYS_HandleT    filterSetHdl;

} MHEG_CMD_DX_FREESET_S;

typedef struct
{
    FAPI_SYS_HandleT    chHdl;
    uint8_t             size;
    uint16_t            chPid;
    itk_uint8_t*        filterMask;
    itk_uint8_t*        equalMask;
    itk_uint8_t*        notEqualMask;
    fbool_t             isRunning;
    FAPI_SYS_HandleT    filtHdl;

} MHEG_CMD_DX_ALLOC_S;

typedef struct
{
    FAPI_SYS_HandleT    filtHdl;
    fbool_t             isRunning;

} MHEG_CMD_DX_FREE_S;

typedef struct
{
    FAPI_SYS_HandleT    filtHdl;
    fbool_t             isRunning;

} MHEG_CMD_DX_SET_S;

typedef struct
{
    FAPI_SYS_HandleT    filtHdl;
    fbool_t             start;
    void*               data;

} MHEG_CMD_DX_CONTROL_S;

typedef struct
{
    FAPI_SYS_HandleT    filtHdl;
    fbool_t             isRunning;
    int16_t             pid;

} MHEG_CMD_DX_CHANGE_S;

typedef struct
{
    int16_t             tsId;
    fbool_t             enable;

} MHEG_CMD_DX_REQUEST_PAT_S;

typedef struct
{
    uint16_t            pid;
    uint16_t            programNumber;
    fbool_t             enable;

} MHEG_CMD_DX_REQUEST_PMT_S;

typedef struct
{
    fbool_t   freeze;

} MHEG_CMD_VD_FREEZE_S;

typedef struct
{
    fbool_t   freeze;

} MHEG_CMD_VD_STOP_S;

typedef struct
{
    uint16_t    videoPID;
    uint8_t     videoType;

} MHEG_CMD_VD_SELECT_S;

typedef struct
{
    fbool_t     enable;

} MHEG_CMD_VD_BLANK_S;

typedef struct
{
    fbool_t   pause;

} MHEG_CMD_AD_PAUSE_S;

typedef struct
{
    uint16_t    audioPID;
    int32_t     audioFormat;
    int32_t     audioChannel;

} MHEG_CMD_AD_SELECT_S;

typedef struct
{
    MHEG_CMD_TYPE_E type;

    union
    {
        MHEG_CMD_VD_FREEZE_S    freeze;
        MHEG_CMD_VD_STOP_S      stop;
        MHEG_CMD_VD_SELECT_S    selVideo;
        MHEG_CMD_VD_BLANK_S     blank;
    } item;

} AV_CMD_MHEG_VD_S;

typedef struct
{
    MHEG_CMD_TYPE_E type;

    union
    {
        MHEG_CMD_DX_ALLOCSET_S      setAlloc;
        MHEG_CMD_DX_FREESET_S       setFree;
        MHEG_CMD_DX_ALLOC_S         alloc;
        MHEG_CMD_DX_FREE_S          free;
        MHEG_CMD_DX_SET_S           set;
        MHEG_CMD_DX_CONTROL_S       control;
        MHEG_CMD_DX_CHANGE_S        change;
        MHEG_CMD_DX_REQUEST_PAT_S   pat;
        MHEG_CMD_DX_REQUEST_PMT_S   pmt;
    } item;

} AV_CMD_MHEG_DX_S;

typedef struct
{
    MHEG_CMD_TYPE_E type;

    union
    {
        MHEG_CMD_AD_PAUSE_S     pause;
        MHEG_CMD_AD_SELECT_S    selAudio;
    } item;

} AV_CMD_MHEG_AD_S;


typedef struct
{
    struct itk_service_t            itkHandle;

    itk_service_event_callback_t    callback;
    void*                           context;

    fbool_t                         started;

    itk_int32_t                     onid;
    itk_int32_t                     tsid;
    itk_int32_t                     sid;
    itk_int32_t                     lcn;
    SLIST_ITEM_S                    servItem;
    TPLIST_ITEM_S                   tpItem;
    PSISI_BAT_S*                    pBat;
    RTOS_SemaphoreT                 mutex;
    SLIST_SERVIDX                   servIndex;

    RTOS_SemaphoreT                 retuneFinish;
    uint32_t                        retuneFlags;

} MHEG_SERVICE_INST_S;

typedef enum
{
    FORMAT_DVB = 0,
    FORMAT_LCN,
    FORMAT_RECEIVER,
    FORMAT_FSI,
    FORMAT_UNKNOWN,

} REF_STRING_FORMAT_E;

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

itk_display_t*      MHEG_InitDisplay        (MHEG_SETKEYMASK_CB keyCallback,
                                             uint32_t layer);
itk_demux_t*        MHEG_InitDemux          (void);
itk_fontdriver_t*   MHEG_InitFont           (void);
itk_misc_t*         MHEG_InitMisc           (const char_t* hostIdStr);
itk_os_t*           MHEG_InitOs             (MHEG_OS_INIT_PARAMS_S* params);
itk_service_t*      MHEG_InitService        (void);
itk_nvs_t*          MHEG_InitNvs            (void);
itk_zlib_t*         MHEG_InitZlib           (void);
itk_appmmi_t*       MHEG_InitAppmmi         (void);
itk_http_t*         MHEG_InitHttp           (void);
itk_crypto_t*       MHEG_InitCrypto         (void);
int32_t             MHEG_InitAudioClip      (void);

fbool_t             MHEG_SetFilterSetHandler(FAPI_SYS_HandleT sessionHdl,
                                             FAPI_SYS_HandleT channelHdl);
fbool_t             MHEG_SetFilterHandle    (FAPI_SYS_HandleT inst,
                                             FAPI_SYS_HandleT sfFilterHdl);
itk_avstream_t*     MHEG_AVStreamCreate     (itk_display_t *thiz,
                                             const char_t *reference,
                                             itk_int32_t presentationCount,
                                             itk_avstream_callback_t callback,
                                             void *callbackContext);
itk_audioclip_t*    MHEG_AudioClipCreate    (itk_display_t *thiz,
                                             const void *content,
                                             size_t contentSize,
                                             itk_int32_t presentationCount,
                                             itk_audioclip_callback_t callback,
                                             void *callbackContext);
int32_t             MHEG_InitConsole        (void);
REF_STRING_FORMAT_E MHEG_ParseReferenceString(MHEG_SERVICE_INST_S *servHandle,
                                              const char_t *service);
SLIST_SERVIDX       MHEG_FindServiceFromID  (MHEG_SERVICE_INST_S *servHandle);
SLIST_SERVIDX       MHEG_FindServiceFromLCN (MHEG_SERVICE_INST_S *servHandle);
SLIST_SERVIDX       MHEG_FindCurrentServIndex(MHEG_SERVICE_INST_S *servHandle);

int32_t     PMT_StreamParseGetNext(const uint8_t* buf,
                               uint32_t bufSize,
                               uint16_t* pmtPos,
                               PSISI_PMT_STREAM_S* pInfo);

int32_t     AVINT_MHEG_Video    ( const AV_CMD_MHEG_VD_S* mhegCmd);
int32_t     AVINT_MHEG_Audio    ( const AV_CMD_MHEG_AD_S* mhegCmd);
int32_t     AVINT_MHEG_Demux    ( const AV_CMD_MHEG_DX_S* mhegCmd);
fbool_t     MHEG_GetVideoState  ( void );
void        MHEG_ResetVideoState( void );
fbool_t     MHEG_GetAudioState  ( void );
void        MHEG_ResetAudioState( void );
fbool_t     MHEG_IsStarted      ( void );

void        MHEG_ImageHide      ( itk_image_t *thiz );
fbool_t     MHEG_ImagePresent   ( void );

void        MHEGINT_Open    ( void );
void        MHEGINT_Close   ( void );
void        MHEGINT_Stop    (fbool_t isHostCall);
void        MHEGINT_Start   (void);
int32_t     MHEGINT_Show    (void);


void        MHEG_PAT_Section_Notify(const uint8_t *sectionBuff,
                                    uint8_t sectNum,
                                    uint16_t sectionSize,
                                    fbool_t  avail);
void        MHEG_PMT_Section_Notify(const uint8_t *sectionBuff,
                                    uint16_t sectionSize);
uint8_t     MHEG_PrivSectFiltered(FAPI_SYS_HandleT handle,
                                  const volatile uint8_t* pSectStart,
                                  const volatile uint8_t* pSectEnd,
                                  const volatile uint8_t* pSect,
                                  uint16_t length, void* pPrivateData);
void        MHEG_Query_Buffer(FAPI_SYS_HandleT handle,
                              uint8_t**        ppBuffer,
                              uint16_t*        pBufferSize,
                              void*            pPrivateData);
void        MHEG_Private_Section_Notify(FAPI_SYS_HandleT handle,
                                        SFAL_SECTION_STATUS_E sectionStatus,
                                        const uint8_t * pSectionBuffer,
                                        uint16_t sectionSize,
                                        void *pPrivateData);

void        MHEG_DisplayDispAspectChanged(void);
void        MHEG_DisplayHasChanged(void);
uint32_t    MHEG_DisplayGetLayerId(void);
fbool_t     MHEG_DisplayGetSubtitle( void );
int32_t     MHEG_DisplayGetPhysicalAspectRatio(itk_ar_factor_t* pysicalAspectRatio);
void        MHEG_DisplayClear( void );
int32_t     MHEG_DisplayHide( void );
int32_t     MHEG_DisplayShow( void );
fbool_t     MHEG_DisplayGetAfd (void);
fbool_t     MHEG_DisplayGetWssScartSignal(FAPI_VISCALE_WssStateEnumT * wssSignal,
                                      uint8_t* scartSignal);

void        MHEG_AVStreamRestartDefault( void );

int32_t     MHEG_ServiceStart(void);
int32_t     MHEG_ServiceStop(fbool_t isHostCall);
uint32_t    MHEG_ServiceGetFlags(void);


#ifdef __cplusplus
}
#endif

#endif /* _MHEG_H_ */

/******************************************************************************/
/* END OF FILE */

