/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/inc/psisi.h
**
**              $Id: psisi.h 5925 2010-03-05 06:43:50Z jmorha $
**
** \brief       Internal section filtering module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _PSISI_H_
#define _PSISI_H_

/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/
#define ONID_PRIVATE_START                     0xFF00
#define PRIVATE_DATA_SPECIFIER_UKDTT       0x0000233A

#define ISO_PAT_PID                            0x0000
#define ISO_CAT_PID                            0x0001
#define ISO_TSDT_PID                           0x0002
#define ISO_NIT_PID                            0x0010
#define ISO_SDT_PID                            0x0011
#define ISO_BAT_PID                            0x0011
#define ISO_EIT_PID                            0x0012
#define ISO_RST_PID                            0x0013
#define ISO_TDT_PID                            0x0014
#define ISO_TOT_PID                            0x0014
#define ISO_DIT_PID                            0x001E
#define ISO_SIT_PID                            0x001F

#define ISO_PAT_TABLE_ID                         0x00
#define ISO_CAT_TABLE_ID                         0x01
#define ISO_PMT_TABLE_ID                         0x02
#define ISO_TSDT_TABLE_ID                        0x03
#define ISO_NIT_TABLE_ID                         0x40
#define ISO_NIT_OTHER_TABLE_ID                   0x41
#define ISO_SDT_TABLE_ID                         0x42
#define ISO_SDT_OTHER_TABLE_ID                   0x46
#define ISO_BAT_TABLE_ID                         0x4A
#define ISO_EIT_TABLE_ID                         0x4E
#define ISO_EIT_OTHER_TABLE_ID                   0x4F
#define ISO_TDT_TABLE_ID                         0x70
#define ISO_RST_TABLE_ID                         0x71
#define ISO_TOT_TABLE_ID                         0x73
#define ISO_DIT_TABLE_ID                         0x7E
#define ISO_SIT_TABLE_ID                         0x7F

#define MAX_FILTER_DEPTH            7
#define PAT_FILTER_DEPTH            7
#define PMT_FILTER_DEPTH            7
#define TDTTOT_FILTER_DEPTH         4
#define SDT_FILTER_DEPTH            7
#define BAT_FILTER_DEPTH            7
#define CAT_FILTER_DEPTH            7
#define NIT_FILTER_DEPTH            7

#define PSISI_VERSION_ANY          0xFF

#define PSISI_PAT_PRG_MAX          2048     //!< Below theoretical limit but
                                            //!< should be sufficient.
#define PSISI_SDT_BUF_MAX          12288    //!< Below theoretical limit but
                                            //!< should be sufficient.
#define PSISI_CAT_STR_MAX          128      //!< Below theoretical limit but
                                            //!< should be sufficient.
#define PSISI_NIT_TS_MAX           256
#define PSISI_BAT_TS_MAX           256
#define PSISI_SECT_MAX             256     //!< Max. sections per PSI/SI table.
#define PSISI_SECT_MAX_LW          ((PSISI_SECT_MAX + 31) >> 5)

/* Control of section availability bitfields */

#define PSISI_SectionAvailable(pSectFound, sectNum)                     \
                     ( (((pSectFound)[(sectNum) >> 5]) &                \
                        (((uint32_t)0x1) << ((sectNum) & 0x1F))) != 0 )

#define PSISI_SectionMark(pSectFound, sectNum)                          \
                     (pSectFound)[(sectNum) >> 5] |=                    \
                     (((uint32_t)0x1) << ((sectNum) & 0x1F))

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

typedef void (*PSISI_SECT_NOTIFY_CB)(uint8_t tsdIdx,
                                     const uint8_t* pSectionBuffer,
                                     uint16_t sectionSize);


typedef struct
{
    uint8_t        data [MAX_FILTER_DEPTH];
    uint8_t        equal[MAX_FILTER_DEPTH];
    uint8_t        nonEq[MAX_FILTER_DEPTH];

} PSISI_FILTER_S;

typedef struct
{
    uint8_t         tag;
    const uint8_t*  start;
    uint16_t        length; //!< Total descriptor length incl. tag and length fields.

} PSISI_DESC_S;

typedef struct
{
    uint8_t         type;
    uint16_t        pid;
    const uint8_t*  esInfoStart;
    uint16_t        esInfoLen;

} PSISI_PMT_STREAM_S;

typedef struct
{
    uint16_t        program_number;
    uint16_t        program_map_pid;

} PSISI_PAT_PRG_S;

typedef struct
{
    fbool_t         nonEmpty;                   //!< FTRUE: non-empy.
    fbool_t         allAvailable;               //!< ftrue: all sections avl.
    uint32_t        avl[PSISI_SECT_MAX_LW];     //!< Section avl. bitfield.

} PSISI_SECT_S;

typedef struct
{
    PSISI_SECT_S    sect;                       //!< Section handling data.
    uint16_t        transport_stream_id;        //!< transport_stream_id.
    uint8_t         version_number;             //!< version_number.
    uint16_t        network_pid;                //!< network_pid.
    uint16_t        numPrograms;                //!< Curr. num. of programs.
    PSISI_PAT_PRG_S program[PSISI_PAT_PRG_MAX]; //!< Array of program info.

} PSISI_PAT_S;

typedef struct
{
    uint8_t         *descr_loop;
    int32_t         tsId;
    int32_t         onId;
    uint16_t        descrLen;

} PSISI_NIT_TS_S;

typedef struct
{
    PSISI_SECT_S    sect;                   //!< Section handling data.
    int32_t         network_id;             //!< network_id.
    uint8_t         version_number;         //!< version_number.
    uint16_t        network_pid;            //!< network_pid.
    uint16_t        numOfTs;                //!< Curr. num. of TS.
    PSISI_NIT_TS_S  ts[PSISI_NIT_TS_MAX];   //!< Array of program info.
} PSISI_NIT_S;

typedef struct
{
    uint16_t serviceId;
    uint16_t lcn;
    uint8_t  service_type;
    fbool_t  invisible;
    fbool_t  unselectable;

} PSISI_NIT_SERVICE_INFO_S;

#define FLAG_GET_LCN 0x01
#define FLAG_GET_ATTR 0x02

typedef struct
{
    uint8_t                     desFlag;
    uint16_t                    serviceCount;
    TPLIST_IDX                  tpIdx;
    TPLIST_ITEM_S               tpItem;
    PSISI_NIT_SERVICE_INFO_S    nitServiceList[CARRIER_SERVICE_MAX];

} PSISI_NIT_TP_INFO_S;

typedef struct
{
    uint8_t  *descr_loop;
    uint16_t tsId;
    uint16_t onId;
    uint16_t descrLen;

} PSISI_BAT_TS_S;

typedef struct
{
    PSISI_SECT_S    sect;                   //!< Section handling data.
    uint16_t        network_id;             //!< network_id.
    uint8_t         version_number;         //!< version_number.
    uint16_t        network_pid;            //!< network_pid.
    uint16_t        numOfTs;                //!< Curr. num. of TS.
    PSISI_NIT_TS_S  ts[PSISI_BAT_TS_MAX];   //!< Array of program info.

} PSISI_BAT_S;

typedef struct
{
    PSISI_PAT_S     curr;
    PSISI_PAT_S     temp;

} PSISI_PAT_DATA_S;

#ifdef APPL_MHEG_ENABLED
typedef struct
{
    uint8_t* data[256];
    uint16_t size[256];
    uint32_t numSect;
    uint8_t* fullData;
    uint32_t fullSize;
    fbool_t  avl;

} PSISI_PAT_BUF_S;
#endif

typedef struct
{
    fbool_t         available;
    uint16_t        pid;
    uint16_t        program_number;
    uint8_t         version_number;
    uint8_t         buf[PMT_SECTSIZE_MAX]; //!< PMT section buffer
    uint16_t        bufSize;

} PSISI_PMT_S;

typedef struct
{
    PSISI_SECT_S    sect;                       //!< Section handling data.
    uint16_t        transport_stream_id;        //!< transport_stream_id.
    uint8_t         version_number;             //!< version_number.
    uint16_t        original_network_id;        //!< original_network_id.
    uint32_t        serviceInfoSize;            //!< Curr. fill level of service
                                                //!< info buffer in bytes.
    uint8_t         serviceInfo[PSISI_SDT_BUF_MAX]; //!< Service info buffer.

} PSISI_SDT_S;

typedef struct
{
    uint16_t        service_id;
    uint8_t         eit_schedule_flag;
    uint8_t         eit_pf_flag;
    uint8_t         running_status;
    uint8_t         free_ca_mode;
    const uint8_t*  descLoopStart;
    uint16_t        descLoopLength;

} PSISI_SDT_SERVICE_S;

typedef struct
{
    uint16_t        ca_system_id;
    uint16_t        pid;

} PSISI_CAT_STR_S;

typedef struct
{
    PSISI_SECT_S    sect;                       //!< Section handling data.
    uint8_t         version_number;             //!< version_number.

    uint16_t        numStreams;                 //!< Current number of CA
                                                //!< descriptor infos.
    PSISI_CAT_STR_S stream[PSISI_CAT_STR_MAX];  //!< CA descriptor info.

} PSISI_CAT_S;

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

int32_t         PSISI_Init                  (void);
void            PSISI_InternalHandling      (void);
int32_t         PSISI_TDTTOT_StartFiltering (void);
void            PSISI_TDTTOT_StopFiltering  (void);

int32_t         PSISI_SetTsdUsage           (RSRC_OP op, uint8_t tsdIdx,
                                             PSISI_PAT_DATA_S* pPat);
int32_t         PSISI_ReleaseTsdUsage       (RSRC_OP op, uint8_t tsdIdx);

fbool_t         PSISI_SectionAllAvailable   (const uint32_t* pSectFound,
                                             uint8_t lastSection);

int32_t         PSISI_PMT_StartFiltering    (uint8_t  tsdIdx,
                                             uint16_t prgNum,
                                             uint16_t pid,
                                             uint8_t version,
                                             fbool_t equal);
int32_t         PSISI_PMT_StopFiltering     (uint8_t tsdIdx);
int32_t         PSISI_PMT_GetFilterParams   (uint8_t tsdIdx,
                                             uint16_t* pid,
                                             uint16_t* prgNum);
int32_t         PSISI_PMT_Get               (uint8_t tsdIdx,
                                             PSISI_PMT_S* pPmt);
const PSISI_PMT_S* PSISI_PMT_GetCurrent     (uint8_t tsdIdx);
int32_t         PSISI_PMT_Invalidate        (uint8_t tsdIdx);
int32_t         PSISI_PMT_NotifyReception   (uint8_t tsdIdx);
int32_t         PSISI_PMT_GetStreamPIDs     (uint8_t tsdIdx,
                                             SYS_STREAM_INFO_S* pPid);
uint8_t         PSISI_PMT_AddAudioStreams   (const PSISI_PMT_STREAM_S* pPmtStream,
                                             uint8_t format,
                                             SYS_STREAMINF_AUDIO_S* pAudio);
int32_t         PSISI_PMT_StreamParseInit   (const PSISI_PMT_S* pPmt, uint16_t* pmtPos);
int32_t         PSISI_PMT_StreamParseGetNext(const PSISI_PMT_S* pPmt, uint16_t* pmtPos,
                                             PSISI_PMT_STREAM_S* pInfo);
int32_t         PSISI_PMT_StreamParseGetDesc(const uint8_t* pDesc, uint16_t* descPos,
                                             uint16_t descLen);
int32_t         PSISI_SDT_StartFiltering    (uint8_t version, fbool_t equal);
int32_t         PSISI_SDT_StopFiltering     (void);
void            PSISI_SDT_Invalidate        (PSISI_SDT_S* pSdt);
int32_t         PSISI_SDT_GetNextService    (const PSISI_SDT_S* pSdt,
                                             uint16_t* serviceInfoPos,
                                             PSISI_SDT_SERVICE_S* pService);
int32_t         PSISI_SDT_GetServiceById    (const PSISI_SDT_S* pSdt,
                                             uint16_t service_id,
                                             PSISI_SDT_SERVICE_S* pService);

#if (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED)

int32_t         PSISI_CAT_StartFiltering    (uint8_t tsdIdx,
                                             uint8_t version,
                                             fbool_t equal,
                                             fbool_t versionUpdate,
                                             PSISI_SECT_NOTIFY_CB sectNotify);
int32_t         PSISI_CAT_StopFiltering     (uint8_t tsdIdx);
int32_t         PSISI_CAT_Invalidate        (uint8_t tsdIdx);
int32_t         PSISI_CAT_Get               (uint8_t tsdIdx, PSISI_CAT_S* pCat);
const PSISI_CAT_S* PSISI_CAT_GetCurrent     (uint8_t tsdIdx);

#endif /* (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED) */

int32_t         PSISI_DescParseGetNext      (const uint8_t* pLoop,
                                             uint16_t* loopPos,
                                             uint16_t loopLen,
                                             PSISI_DESC_S* pInfo);
int32_t         PSISI_DescFind              (const uint8_t* pLoop,
                                             uint16_t loopLen,
                                             uint8_t tagToFind,
                                             PSISI_DESC_S* pInfo);

SYS_LANGCODE    PSISI_GetLanguageCode       (const uint8_t* pBuf);
int32_t         PSISI_GetServiceDescriptor  (const uint8_t* pLoop,
                                             uint16_t loopLen,
                                             char_t* servName,
                                             uint16_t servNameLen,
                                             SYS_CHARSET* servNameCharSet,
                                             uint8_t* serviceType);
#ifdef SLIST_WITH_PROVIDER
int32_t         PSISI_GetServiceProviderName(const uint8_t* pLoop,
                                             uint16_t loopLen,
                                             char_t* provider,
                                             uint16_t providerLen,
                                             SYS_CHARSET* providerCharSet);
#endif
void            PSISI_PAT_Invalidate        (PSISI_PAT_DATA_S* pPat);
int32_t         PSISI_PAT_GetPmtPid         (const PSISI_PAT_S* pPat,
                                             uint16_t prgNum,
                                             uint16_t* pmtPid);
const PSISI_PAT_S* PSISI_PAT_GetCurrent     (uint8_t tsdIdx);

#ifdef APPL_MHEG_ENABLED
fbool_t PSISI_PAT_GetCurrentBuf(PSISI_PAT_BUF_S* pPatBuf,uint8_t tsdIdx);
#endif

int32_t         PSISI_InvalidateByFe        (PSISI_PAT_DATA_S* pPat);
int32_t         PSISI_RestartByFe           (PSISI_PAT_DATA_S* pPat);

int32_t PSISI_NIT_StartFiltering        (uint8_t  version,
                                         fbool_t  equal,
                                         uint32_t timeout);
int32_t PSISI_NIT_StopFiltering         (void);
int32_t PSISI_NIT_Invalidate            (void);
int32_t PSISI_NIT_GetInfo               (uint16_t *numOfTs, int32_t *networkId);
int32_t PSISI_NIT_GetTransportInfo      (uint16_t tsIdx, int32_t *tsId, int32_t *onId);
int32_t PSISI_NIT_GetTransportDescLoop  (uint16_t tsIdx, uint8_t *buf, uint16_t *len);
int32_t PSISI_BAT_StartFiltering        (uint8_t version, fbool_t equal); //Ravi...New declarations for BAT
int32_t PSISI_BAT_StopFiltering         (void);
int32_t PSISI_BAT_Invalidate            (void);
int32_t PSISI_BAT_GetInfo               (uint16_t *numOfTs, uint16_t *networkId);
int32_t PSISI_BAT_GetTransportInfo      (uint16_t tsIdx, uint16_t *tsId, uint16_t *onId);
int32_t PSISI_BAT_GetTransportDescLoop  (uint16_t tsIdx, uint8_t *buf, uint16_t *len);
fbool_t PSISI_NIT_ParseByTP             (FE_SYSTEM_TYPE_E feType, const uint8_t * pDesBuffer, uint16_t len, PSISI_NIT_TP_INFO_S *pNitTPService, int32_t curtsId);
const PSISI_NIT_S* PSISI_NIT_Get        (void);


#ifdef APPL_DTG_SIPSI_T2_ENABLED
fbool_t PSISI_CheckSameLCN(uint16_t lcn, uint8_t *phdlcnBuf, uint16_t hdlcnBuflen, uint16_t *pserviceId);
uint16_t PSISI_GetLCNByServiceID(uint8_t *plcnBuf, uint16_t lcnBuflen, uint16_t GetserviceId);
#endif

#ifdef __cplusplus
}
#endif

#endif /* _PSISI_H_ */

/******************************************************************************/
/* END OF FILE */
