/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/inc/pvr_sys.h
**
**              $Id: pvr_sys.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Internal PVR control functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _PVRSYS_H_
#define _PVRSYS_H_

#include "sys_api.h"

#include <val/val.h>

typedef struct
{
    VAL_PlayModeEnumT   videoMode;
    uint32_t            iFrameDisplayTime;  //!< For PVR_FASTMODETYPE_IFRAMES
                                            //!< and PVR_FASTMODETYPE_FEED_CONTROLLED.
    int32_t             factor;             //!< For PVR_FASTMODETYPE_IFRAMES
                                            //!< and PVR_FASTMODETYPE_FEED_CONTROLLED
                                            //!< and PVR_FASTMODETYPE_FEED_NONE.
    uint16_t            speedFactor;        //!< Informational use only.
    
} PVRSYS_FastModeCfgT;

typedef struct
{
    VAL_PlayModeEnumT   videoMode;
    uint16_t            speedFactor;        //!< Informational use only.
    
} PVRSYS_SlowModeCfgT;

#ifdef __cplusplus
extern "C" {
#endif

int32_t PVRSYS_Init                  (void);
int32_t PVRSYS_Wakeup                (void);
int32_t PVRSYS_Shutdown              (void);
void    PVRSYS_AbortDeviceOperations (SYS_DEVICE_E device);
void    PVRSYS_AbortAllOperations    (void);
void    PVRSYS_PlayEvent             (PVR_PlayNotifyT action);
void    PVRSYS_RecEvent              (uint8_t recIdx, PVR_RecordNotifyT event);
void    PVRSYS_RestoreDefaults       (void);
int32_t PVRSYS_SettingsSave          (void);
int32_t PVRSYS_SetMountPath          (SYS_DEVICE_E device);
void    PVRSYS_PMTReceived           (const SYS_EVT_PMT_S* pEvt);
void    PVRSYS_CATReceived           (const SYS_EVT_CAT_S* pEvt);
fbool_t PVRSYS_PlayPidAvailable      (uint16_t pid);
int32_t PVRSYS_InvalidateService     (SLIST_SERVIDX servIdx);
int32_t PVRSYS_PlayReroute           (const RSRC_CFG_S* cfg);
int32_t PVRSYS_RecordReroute         (const RSRC_CFG_S* cfg);

int32_t PVRSYS_GetFastModeConfig     (uint16_t index,
                                      PVRSYS_FastModeCfgT* cfgPtr);
int32_t PVRSYS_GetSlowModeConfig     (uint16_t index,
                                      PVRSYS_SlowModeCfgT* cfgPtr);

int32_t PVRDEC_Init                  (void);
void    PVRDEC_RegisterFunctions     (PVR_TargetControlT* ctrlPtr);

#ifdef APPL_CI_ENABLED

void  PVRSYS_CAMRemoved              (uint8_t recIdx, RSRC_TS_E ts);

#ifdef APPL_DESCRREC_ENABLED

int32_t PVRSYS_DescrRecReroute       (const RSRC_CFG_S* cfg);

fbool_t PVRSYS_TsFeedData            (uint8_t tsd, const uint32_t* pData,
                                      uint32_t packets,
                                      uint32_t* pMillisNeeded);

#endif /* APPL_DESCRREC_ENABLED */

#endif /* APPL_CI_ENABLED */

#ifdef __cplusplus
}
#endif

#endif /* _PVRSYS_H_ */

/******************************************************************************/
/* END OF FILE */
