/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/inc/statlist.h
**
**              $Id: statlist.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _STATLIST_H_
#define _STATLIST_H_

#include "sys_api.h"
#include "sys_priv.h"

/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/

/* bit assignment of "status" field */
#define STATLST_USED       0x01
#define STATLST_OWNER      ((SYS_OWNER_ALL) << 4)

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

typedef uint8_t     STATLST_ENTRYSTAT;
typedef void        (*STATLST_REMOVE_CB) (void *arg, int32_t index);

typedef struct
{
    uint32_t            entrySize;      //!< Size of one entry in bytes.
    uint32_t            numEntries;     //!< Current number of entries.
    uint32_t            maxEntries;     //!< Max. number of entries.
    int32_t             afterLast;      //!< Index after last used entry.
    int32_t             firstFree;      //!< Index of first free entry.

} STATLST_STATUS_S;

typedef struct
{
    fbool_t             initialized;    //!< initialisation status
    uint8_t            *data;   //!< Entry data.
    STATLST_ENTRYSTAT  *status; //!< Status of each entry.
    STATLST_STATUS_S    stat;   //!< Status parameters

} STATLST_S;

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern              "C"
{
#endif

    int32_t             STATLST_Init(STATLST_S * pLst, uint32_t maxEntries,
                                     uint32_t entrySize);
    int32_t             STATLST_Clear(STATLST_S * pLst, SYS_OWNER_E owners,
                                      STATLST_REMOVE_CB cb, void *cbArg);
    int32_t             STATLST_AddEntry(STATLST_S * pLst, const void *pEntry,
                                         SYS_OWNER_E owner, int32_t * index);
    int32_t             STATLST_SetEntry(STATLST_S * pLst, int32_t index,
                                         const void *pEntry, SYS_OWNER_E owner);
    void               *STATLST_GetEntry(STATLST_S * pLst, int32_t index);
    int32_t             STATLST_RemoveEntry(STATLST_S * pLst, int32_t index);
    int32_t             STATLST_CopyList(STATLST_S * pDes, STATLST_S * pSrc);

    fbool_t             STATLST_LoadFromFS(STATLST_S * pLst,
                                           fbool_t targetIsHdd, void *fileHdl);
    fbool_t             STATLST_WriteToFS(STATLST_S * pLst, fbool_t targetIsHdd,
                                          void *fileHdl);
    int32_t             STATLST_LoadFromFlash(STATLST_S * pLst,
                                              FLASH_FILE_S * fileHdl);
    int32_t             STATLST_WriteToFlash(STATLST_S * pLst,
                                             FLASH_FILE_S * fileHdl,
                                             fbool_t program);

    uint32_t            STATLST_GetMaxDataSize(uint32_t maxEntries,
                                               uint32_t entrySize);

#ifdef __cplusplus
}
#endif

#endif /* _STATLIST_H_ */

/******************************************************************************/
/* END OF FILE */
