/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/av.c
**
**              $Id: av.c 5881 2010-03-03 08:04:49Z jmorha $
**
** \brief       Audio and video control module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
**  TODO add lint warning information and check if flag are usefull !!!
**
*****************************************************************************
*/
/*lint -save -e64 -e574 -e534 -e641 -e712 -e715 -e732 -e734 -e737 -e747 -e762 -e957 */
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <stdint.h>
#include <string.h>
#include "fapex/abst_types.h"
#include "fapex/abst_libtsd.h"
#include "fapex/abst_libaudec.h"
#include "fapex/abst_libvienc.h"
#include "fapex/abst_libviout.h"
#include "val/val.h"
#include "av.h"
#include "fe/fe.h"
#include "sfal/sfal.h"
#include "fcdata.h"
#include "utils.h"
#include "scart.h"


//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#define AV_FE_TUNE_ATTEMPTS         2

/* all sleep constants are in units of milliseconds! */

#define AV_SLEEP_DECODING_NORMAL     10
#define AV_SLEEP_DECODING_IDLE       250
#define AV_SLEEP_DECODING_RECOVERY   500

#define AV_SLEEP_IDLE                100

#define AV_SLEEP_DEFAULT             AV_SLEEP_IDLE
#define AV_SLEEP_MAX                 750

#define AV_SIGNALUPDATE_MIN          200 // min. interval for signal bar update in ms
#define AV_LOCKCHECKINTERVAL_MIN     250 // min. tuner lock test interval in ms
#define AV_MOTOR_ADDRESS             0x31  //!< DiseqC address byte for motor dishes

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** External Functions Declaration
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************

AV_DATA_S*      avData = NULL;
int32_t fd_TSD[4] = {-1,-1,-1,-1};

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

FSTATIC void     AV_ThreadFunction       (void*);
FSTATIC void     AV_ThreadProcessing     (void);
FSTATIC uint32_t AV_TaskNormal           (AV_FEDAT_S* pFe);
FSTATIC uint32_t AV_TaskIdle             (AV_FEDAT_S* pFe);
#ifdef APPL_FECABLE_ENABLED
FSTATIC void     AV_SetSystemParamsCable (FE_SYSTEM_PARAMS_S* pSys);
#endif
#ifdef APPL_FETERR_ENABLED
FSTATIC void     AV_SetSystemParamsTerr  (FE_SYSTEM_PARAMS_S* pSys);
#endif
        fbool_t AV_SleepAndCheckAbortion (int32_t timeInMs, fbool_t abortable);
FSTATIC void    AVINT_SetLockStatus      (AV_FEDAT_S* pFe, SYS_FE_LOCK_E feStatus);
FSTATIC int32_t AVINT_TuneParamsPrepare  (AV_FEDAT_S* pFe,
                                         FE_TUNE_PARAMS_S* pPar,
                                         CFGSAT_IDX satIdx);
FSTATIC int32_t AVINT_Activate           (void);
FSTATIC int32_t AVINT_Deactivate         (void);
FSTATIC int32_t AVINT_GotoIdleState      (AV_FEDAT_S* pFe);
FSTATIC int32_t AVINT_StartWelcomeJingle (void);
FSTATIC void    AVINT_StartTuneMode      (AV_FEDAT_S* pFe, AV_CMD_S* pCmd);



FSTATIC void AVINT_SetAntennaPower (AV_FEDAT_S* pFe, uint8_t power)
{
    if ( pFe->drvHdl != NULL )
    {
        FE_SetAntennaPower(pFe->drvHdl, power);
        pFe->forceTuning = FTRUE;
    }
}

FSTATIC void AVINT_SFAL_ThreadFunction(void* opt)
{

    while (1)
    {
        SFAL_ThreadFunction();
        //printf("[av][AVINT_SFAL_ThreadFunction] sfal thread is alive\n");
        //usleep(1000);
    }
}

int32_t AV_Init (void)
{
    const SYS_HWCONFIG_S*   hwCfg = SYS_GetHWConfig();
    ITC_PARAMS_S            itcParams;
//    SFAL_INIT_S             sfalParams;                        modify by plato
    int32_t                 retVal = FAPI_OK;
    uint32_t                cnt;
    FCDATA_ENTRY_S          fcEntry;

    DBG_Assert(avData == NULL);

    /* --------------------------------------------------------------------- */

    do
    {
        avData = UTILS_Malloc(sizeof(AV_DATA_S));
        if ( avData == NULL )
        {
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
            break;
        }

        /* Inits all entries to 0. Non-zero initialisations are set below */
        memset(avData, 0, sizeof(AV_DATA_S));

        avData->liveStatus          = SERVICE_STAT_STOPPED;
        avData->runningStatus       = SYS_RUNSTAT_UNKNOWN;

        avData->videoOutCurr        = VIDEOOUT_UNKNOWN;
        avData->wssOverride         = FFALSE;
        avData->wssMode             = AV_WSS_4_3_FULLFORMAT;

        avData->sleepMode           = FTRUE;
        avData->threadState         = AV_STATE_SLEEP;
#ifndef _LINUX_FAPI
        avData->scartInputStatus    = FAPI_BOARD_SCART_SB_UNDEFINED;
#endif
        avData->scartInputLastCheck = 0;

        avData->liveTv.pidOverride.adOverride  = FFALSE;
        avData->liveTv.pidOverride.vdOverride  = FFALSE;
        avData->liveTv.pidOverride.pcrOverride = FFALSE;
        avData->liveTv.pidOverride.adPid       = 0x1FFF;
        avData->liveTv.pidOverride.vdPid       = 0x1FFF;
        avData->liveTv.pidOverride.pcrPid      = 0x1FFF;
        avData->liveTv.pidOverride.adType      = 0xFF;
        avData->liveTv.pidOverride.vdType      = 0xFF;

        for ( cnt = 0; cnt < hwCfg->feNums; cnt++ )
        {
            avData->feDat[cnt].feIdx               = cnt;
            avData->feDat[cnt].systemType          = FE_SYSTEM_TYPE_DIRECT;
            avData->feDat[cnt].state               = AV_FESTATE_IDLE;

            avData->feDat[cnt].tuneLockStatus      = SYS_FELOCK_UNUSED;

            #ifdef APPL_FESAT_ENABLED
            avData->feDat[cnt].autoMotorMode       = FTRUE;
            #endif /* APPL_FESAT_ENABLED */

            avData->feDat[cnt].currTunePar.network.type   = FE_SYSTEM_TYPE_DIRECT;
            avData->feDat[cnt].currTunePar.network.satIdx = -1;
            avData->feDat[cnt].currTunePar.manualMode     = FFALSE;
            avData->feDat[cnt].currTunePar.tpIdx          = -1;

            avData->feDat[cnt].reqTunePar = avData->feDat[cnt].currTunePar;

            avData->feDat[cnt].forceLnbSettsUpdate = FTRUE;
            avData->feDat[cnt].forceTuning         = FTRUE;
        }

        /* Initialize TSD inputs */
        for ( cnt = 0; cnt < RSRC_TSD_NUMS; cnt++ )
        {
            AVINT_SetTsInput (cnt, RSRC_TS_A);
        }


        

        /* Initialise section filtering abstraction layer */
//        sfalParams.threadPriority = THREAD_PRIO_SFAL;   modify by plato
//        sfalParams.threadSleep    = 100;
//        retVal = SFAL_Init(&sfalParams);

        retVal = SFAL_Init();

        if ( retVal != FAPI_OK ) break;

        retVal = AVINT_AudioInit();
        if ( retVal != FAPI_OK ) break;

        retVal = AVINT_VideoInit();
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_PVR_ENABLED

        retVal = AVINT_PlayRecInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_PVR_ENABLED */

        /* ----------------------------------------------------------------- */

        retVal = AVINT_LiveTvInit();
        if ( retVal != FAPI_OK ) break;

        itcParams.setAbortReq = NULL;
        retVal = ITC_Init(&(avData->itcInst), &itcParams);
        if ( retVal != FAPI_OK ) break;

        retVal = PSISI_Init ();
        if ( retVal != FAPI_OK ) break;

        /* Attempt to get latest "frequently-changed"-settings and modify some
           default values if available */

        if ( FCDATA_GetEntry(&fcEntry) == FAPI_OK )
        {
            avData->liveTv.reqServLoc   = fcEntry.servLoc;
            avData->liveTv.quietServLoc = fcEntry.servLoc;
        }

        #ifdef APPL_PIP_ENABLED

        retVal = AVINT_PIPInit();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_PIP_ENABLED */

        #ifdef APPL_MHEG_ENABLED

        retVal = AVINT_MHEG_Init();
        if ( retVal != FAPI_OK ) break;

        #endif /* APPL_MHEG_ENABLED */

        SCART_SetOutputSource (FFALSE); /* set default SCART output */

        // FIXME: set SCART to OFF here?

        /* create thread */
        retVal = SYS_ThreadCreate (AV_ThreadFunction, NULL,
                                   THREAD_PRIO_AV,
                                   THREAD_STACKSZ_AV,
                                   THREAD_NAME_AV,
                                   NULL);
        if ( retVal != FAPI_OK ) break;

        /* create PSi/Si thread */

        #if 1
        retVal = SYS_ThreadCreate(AVINT_SFAL_ThreadFunction,NULL,
                                   THREAD_PRIO_SFAL,
                                   THREAD_STACKSZ_APPLICATION,
                                   NULL,
                                   NULL);
        if ( retVal != FAPI_OK ) break;
        #endif

        retVal = FAPI_OK;
    }
    while ( 0 );

    return retVal;
}

FSTATIC int32_t AV_OpenFrontends(void)
{
    const SYS_HWCONFIG_S*  hwCfg = SYS_GetHWConfig();
    FE_OpenParamsT         openParams;
    uint8_t                feCnt;
    AV_FEDAT_S*            pFe;
    fbool_t                antennaPwr;
    FE_SYSTEM_TYPE_E       deliveryType;
    int32_t                errorCode;

    #if defined(APPL_FETERR_ENABLED) || defined(APPL_FECABLE_ENABLED)
    FE_SYSTEM_PARAMS_S     systemParams;
    #endif

    #ifdef TARGET_FUJITSU_DEVKIT
    SYS_VIDEO_OUT         videoOutput;
    #endif /* TARGET_FUJITSU_DEVKIT */

    #ifdef APPL_USALS_ENABLED
    USALS_OPEN_PARAMS_S    usalsOpenParams;
    #endif /* APPL_USALS_ENABLED */

    #if defined(APPL_FETERR_ENABLED) || defined(APPL_FECABLE_ENABLED)
    memset(&systemParams,0,sizeof(FE_SYSTEM_PARAMS_S));
    #endif
    SYS_ControlFpSymbol(FP_SYMB_FELOCK, FFALSE);

    FE_Init();
    /* ----- attempt to open all available frontends ----------------------- */
    for ( feCnt = 0; feCnt < hwCfg->feNums; feCnt++ )
    {
        pFe = avData->feDat + feCnt;

        pFe->tuneLockStatus      = SYS_FELOCK_UNUSED;
        pFe->forceLnbSettsUpdate = FTRUE;
        pFe->forceTuning         = FTRUE;
        pFe->drvHdl              = NULL;

        /* On -N and -E Dks the index does not matter.
           Always both front ends will be resetted.
           On -C DK index is limited to 0 and 1 */
        openParams.slotNum = feCnt;
        openParams.version = FE_VERSION;
        if(FE_DetectSlot(openParams.slotNum) != FAPI_OK)
        {
            pFe->drvHdl = NULL;
            FAPI_SYS_PRINT_DEBUG(1,"[AV][AV_OpenFrontends] FE-%i: Not detected!\n", pFe->feIdx + 1);
        }
        else
        {
            pFe->drvHdl = FE_Open(&openParams, &errorCode);
            if(pFe->drvHdl == NULL)
            {
                return errorCode;
            }
            FE_GetSystemType(pFe->drvHdl, &deliveryType);
            switch ( deliveryType )
            {
                #ifdef APPL_FESAT_ENABLED
                    case FE_SYSTEM_TYPE_SATELLITE:
                        break;
                #endif /* APPL_FESAT_ENABLED */

                #ifdef APPL_FETERR_ENABLED
                    case FE_SYSTEM_TYPE_TERRESTRIAL:
                        AV_SetSystemParamsTerr(&systemParams);
                        break;
                #endif /* APPL_FETERR_ENABLED */

                #ifdef APPL_FECABLE_ENABLED
                    case FE_SYSTEM_TYPE_CABLE:
                        AV_SetSystemParamsCable(&systemParams);
                        break;
                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FEDIRECT_ENABLED
                    case FE_SYSTEM_TYPE_DIRECT:
                        break;
                #endif /* APPL_FECABLE_ENABLED */

                default:
                    /* ignore list entry*/
                    continue;
            }

            pFe->systemType               = deliveryType;
            pFe->currTunePar.network.type = deliveryType;

            RSRC_SetFeStatus (pFe->feIdx, FTRUE, pFe->systemType);

            /* set antenna power status */
            if ( SETTINGS_AntennaPowerGet(pFe->feIdx, &antennaPwr) == FAPI_OK )
            {
                AVINT_SetAntennaPower(pFe, antennaPwr);
            }

            #ifdef APPL_FESAT_ENABLED

            if ( pFe->systemType == FE_SYSTEM_TYPE_SATELLITE )
            {
                #ifdef APPL_USALS_ENABLED

                usalsOpenParams.feHdl        = pFe->drvHdl;
                usalsOpenParams.motorAddress = AV_MOTOR_ADDRESS;

                errorCode = USALS_Open(&usalsOpenParams, &(pFe->usalsHdl));
                if ( errorCode != FAPI_OK )
                {
                    pFe->usalsHdl = NULL;
                }

                #endif /* APPL_USALS_ENABLED */
            }

            #endif /* APPL_FESAT_ENABLED */
        }
    }

    // FIXME: is this handling still needed?
    #ifdef TARGET_FUJITSU_DEVKIT
    /* NOTE: Set Scart output to current value. This is necessary for Fujitsu
       evaluation HW because the GPIO pin that controls the SCART video
       output is used for resetting the frontend as well */
    if ( SETTINGS_VideoOutputGet(&videoOutput) == FAPI_OK )
    {
        (void)SETTINGS_VideoOutputSet(videoOutput);
    }
    #endif /* TARGET_FUJITSU_DEVKIT */

    return FAPI_OK;
}

FSTATIC int32_t AV_CloseFrontends(void)
{
    const SYS_HWCONFIG_S*   hwCfg = SYS_GetHWConfig();
    uint8_t                 feCnt;
    AV_FEDAT_S*             pFe;

    for ( feCnt = 0; feCnt < hwCfg->feNums; feCnt++ )
    {
        pFe = avData->feDat + feCnt;

        #ifdef APPL_USALS_ENABLED

        if ( pFe->usalsHdl != NULL )
        {
            (void)USALS_Close(pFe->usalsHdl);
            pFe->usalsHdl = NULL;
        }

        #endif /* APPL_USALS_ENABLED */

        if ( pFe->drvHdl != NULL )
        {
            /* switch off antenna power */
            (void)FE_SetAntennaPower(pFe->drvHdl, 0);

            (void)FE_Close(pFe->drvHdl);

            pFe->drvHdl = NULL;

            RSRC_SetFeStatus (pFe->feIdx, FFALSE, FE_SYSTEM_TYPE_DIRECT);
        }

        pFe->tuneLockStatus           = SYS_FELOCK_NOTLOCKED;
        pFe->systemType               = FE_SYSTEM_TYPE_DIRECT;
        pFe->currTunePar.network.type = FE_SYSTEM_TYPE_DIRECT;
    }

    SYS_ControlFpSymbol(FP_SYMB_FELOCK, FFALSE);

    return FAPI_OK;
}

#ifdef APPL_FETERR_ENABLED

FSTATIC void AV_SetSystemParamsTerr (FE_SYSTEM_PARAMS_S* pSys)
{
    pSys->terr.autoBandwidth     = 1;
    pSys->terr.autoFEC           = 1;
    pSys->terr.autoModulation    = 1;
    pSys->terr.autoTransMode     = 1;
    pSys->terr.autoGuardInterval = 1;
    pSys->terr.autoHierarchy     = 1;
    pSys->terr.defaultIQ         = FE_IQ_AUTO;
    pSys->terr.custom            = NULL;
}

#endif /* APPL_FETERR_ENABLED */

#ifdef APPL_FECABLE_ENABLED

FSTATIC void AV_SetSystemParamsCable (FE_SYSTEM_PARAMS_S* pSys)
{
    pSys->cable.autoFEC           = 1;
    pSys->cable.autoModulation    = 1;
    pSys->cable.defaultIQ         = FE_IQ_AUTO;
    pSys->cable.custom            = NULL;
}

#endif /* APPL_FECABLE_ENABLED */

#ifdef APPL_FESAT_ENABLED

int32_t AV_SetSystemParamsSat (int8_t feIdx, CFGSAT_IDX satIdx,
                               FE_SYSTEM_PARAMS_S* pSys)
{
    CFGSAT_FEPARS_S  feCfg;
    CFGSAT_SAT_S     satInf;
    int32_t          retVal;
    CFGSAT_LNBCFG_S* pLnb;

    /* get global frontend configuration */
    retVal = CFGSAT_GetFrontendSetup (&feCfg);
    if ( retVal != FAPI_OK ) return retVal;

    /* get sat-specific parameters */
    retVal = CFGSAT_GetDataByIndex(satIdx, &satInf);
    if ( retVal != FAPI_OK ) return retVal;

    pLnb = satInf.lnbCfg + feIdx;

    if ( !(pLnb->active) )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    pSys->sat.diseqc10Rep   = 0;
    pSys->sat.diseqc11Rep   = pLnb->diseqc11Rep;
    pSys->sat.lowFrequency  = pLnb->freqLo;
    pSys->sat.highFrequency = pLnb->freqHi;
    pSys->sat.ifFrequency   = feCfg.ifFreq;
    pSys->sat.autoFEC       = 0;
    pSys->sat.defaultIQ     = FE_IQ_AUTO;
    pSys->sat.custom        = NULL;
    pSys->sat.motorAddress  = AV_MOTOR_ADDRESS;

    switch ( feCfg.controlMode[feIdx] )
    {
        default:
        case CFGSAT_FEMODE_CLASSIC:
            pSys->sat.classic    = 1;
            pSys->sat.posBy22kHz = 0;
            pSys->sat.toneburst  = 0;
            pSys->sat.diseqc     = FE_DISEQC_NONE;
            break;

        #ifdef APPL_SATPOSBY22KHZ_ENABLED
        case CFGSAT_FEMODE_POSBY22KHZ:
            pSys->sat.classic    = 0;
            pSys->sat.posBy22kHz = 1;
            pSys->sat.toneburst  = 0;
            pSys->sat.diseqc     = FE_DISEQC_NONE;
            break;
        #endif

        case CFGSAT_FEMODE_TONEBURST:
            pSys->sat.classic    = 1;
            pSys->sat.posBy22kHz = 0;
            pSys->sat.toneburst  = 1;
            pSys->sat.diseqc     = FE_DISEQC_NONE;
            break;

        case CFGSAT_FEMODE_DISEQC10:
            pSys->sat.classic    = 1;
            pSys->sat.posBy22kHz = 0;
            pSys->sat.toneburst  = 1;
            pSys->sat.diseqc     = FE_DISEQC_1_0;
            break;

        case CFGSAT_FEMODE_DISEQC11_DTH:
            pSys->sat.classic    = 1;
            pSys->sat.posBy22kHz = 0;
            pSys->sat.toneburst  = 1;
            pSys->sat.diseqc     = FE_DISEQC_1_1_DTH;
            break;

        #ifdef APPL_DISEQC11_SMATV_ENABLED
        case CFGSAT_FEMODE_DISEQC11_SMATV:
            pSys->sat.classic     = 1;
            pSys->sat.toneburst   = 1;
            pSys->sat.diseqc      = FE_DISEQC_1_1_SMATV;
            break;
        #endif

        #ifdef APPL_DISEQC12_ENABLED
        case CFGSAT_FEMODE_DISEQC12:
            pSys->sat.classic    = 1;
            pSys->sat.posBy22kHz = 0;
            pSys->sat.toneburst  = 1;
            pSys->sat.diseqc     = FE_DISEQC_1_0;
            break;
        #endif /* APPL_DISEQC12_ENABLED */

        #ifdef APPL_USALS_ENABLED
        case CFGSAT_FEMODE_USALS:
            pSys->sat.classic    = 1;
            pSys->sat.posBy22kHz = 0;
            pSys->sat.toneburst  = 1;
            pSys->sat.diseqc     = FE_DISEQC_1_0;
        #endif /* APPL_USALS_ENABLED */

    }

    return FAPI_OK;
}

FSTATIC int32_t AVINT_MoveMotor (AV_FEDAT_S* pFe, CFGSAT_IDX satIdx)
{
    CFGSAT_FEPARS_S  feCfg;
    CFGSAT_SAT_S     satInf;
    int32_t          retVal = FAPI_OK;
    #if defined(APPL_DISEQC12_ENABLED) || defined(APPL_USALS_ENABLED)
    SYS_EVT_MOTOR_S  ctrlEvent;
    #endif

    /* get global frontend configuration */
    retVal = CFGSAT_GetFrontendSetup (&feCfg);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = CFGSAT_GetDataByIndex(satIdx, &satInf);
    if ( retVal != FAPI_OK ) return retVal;

    #if defined(APPL_DISEQC12_ENABLED) || defined(APPL_USALS_ENABLED)
    ctrlEvent.hdr.type   = EVT_MOTOR_MOVES;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);
    ctrlEvent.moving     = FTRUE;
    #endif

    switch ( feCfg.controlMode[pFe->feIdx] )
    {
        #ifdef APPL_DISEQC12_ENABLED

        case CFGSAT_FEMODE_DISEQC12:
            if ( pFe->drvHdl != NULL )
            {
                retVal = FE_DISEQC12_GotoSat (pFe->drvHdl,
                                              satInf.lnbCfg[pFe->feIdx].
                                              diseqc12Pos);
                if(retVal != FAPI_OK) break;
                EVT_Send(&ctrlEvent,"Move Motor");
            }
            break;

        #endif /* APPL_DISEQC12_ENABLED */

        #ifdef APPL_USALS_ENABLED

        case CFGSAT_FEMODE_USALS:
            if ( pFe->usalsHdl != NULL )
            {
                retVal = USALS_GotoPos (pFe->usalsHdl, &(satInf.orbitalPos));
                if(retVal != FAPI_OK) break;
                EVT_Send(&ctrlEvent,"Move Motor");
            }
            break;

        #endif /* APPL_USALS_ENABLED */

        default:
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;
    }

    return retVal;
}

FSTATIC int32_t AVINT_TuneParamsAddSatInfo (AV_FEDAT_S* pFe, CFGSAT_IDX satIdx,
                                           FE_TUNE_PARAMS_S* pPar)
{
    CFGSAT_FEPARS_S  feCfg;
    CFGSAT_SAT_S     satInf;
    int32_t          retVal;

    /* get global frontend configuration */
    retVal = CFGSAT_GetFrontendSetup (&feCfg);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = CFGSAT_GetDataByIndex(satIdx, &satInf);
    if ( retVal != FAPI_OK ) return retVal;

    if(satInf.lnbCfg[pFe->feIdx].active == FFALSE)
    {
        return SMARTGO_SYS_ERR_RSRC_NETWORK;
    }

    switch ( feCfg.controlMode[pFe->feIdx] )
    {
        default:
        case CFGSAT_FEMODE_CLASSIC:
            pPar->sat.satID = 0;
            break;

        #ifdef APPL_SATPOSBY22KHZ_ENABLED
        case CFGSAT_FEMODE_POSBY22KHZ:
        #endif
        case CFGSAT_FEMODE_TONEBURST:
            pPar->sat.satID = satInf.lnbCfg[pFe->feIdx].toneburstId;
            break;

        case CFGSAT_FEMODE_DISEQC10:
            pPar->sat.satID = satInf.lnbCfg[pFe->feIdx].diseqc10Id;
            break;

        case CFGSAT_FEMODE_DISEQC11_DTH:
            pPar->sat.satID = satInf.lnbCfg[pFe->feIdx].diseqc11Id;
            break;

        #ifdef APPL_DISEQC11_SMATV_ENABLED

        case CFGSAT_FEMODE_DISEQC11_SMATV:
            pPar->sat.satID       = satInf.lnbCfg[pFe->feIdx].diseqc11Id_SMATV;
            pPar->sat.ifFrequency = feCfg.ifFreq;
            break;

        #endif /* APPL_DISEQC11_SMATV_ENABLED */

        #ifdef APPL_DISEQC12_ENABLED

        case CFGSAT_FEMODE_DISEQC12:
            pPar->sat.satID = satInf.lnbCfg[pFe->feIdx].diseqc12Pos;
            break;

        #endif /* APPL_DISEQC12_ENABLED */

        #ifdef APPL_USALS_ENABLED

        case CFGSAT_FEMODE_USALS:
            // TODO!
            pPar->sat.satID = 0;
            break;

        #endif /* APPL_USALS_ENABLED */
    }

    pPar->sat.orbitalPosition = satInf.orbitalPos.degrees;
    pPar->sat.westEastFlag    = satInf.orbitalPos.westEastFlag;
    return FAPI_OK;
}
#endif /* APPL_FESAT_ENABLED */

/*---------------------------------------------------------------------------*/
/* thread function                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC void AV_ThreadFunction(void* opt)
{
    AV_CMD_S*        pCmd;
    AV_CMD_S         avCmd;
    AV_FEDAT_S*      pFe;

    SYS_DevicePrepareThread ();

    while ( 1 )
    {
        FAPI_SYS_PRINT_DEBUG(1,"\n[AV] AV Thread is alive\n");
        if ( avData->threadState != AV_STATE_SLEEP )
        {
            AV_ThreadProcessing();
        }

        /* wait for command */
        pCmd = (AV_CMD_S*) ITC_CmdWait(&(avData->itcInst));
        if ( pCmd == NULL ) continue;

        /* Make a local copy of the command data as passed structure is
            used for next AV-command as soon as reception has been
            acknowledged. */
        avCmd = *pCmd;

        /* acknowledge command reception */
        ITC_AckSend(&(avData->itcInst), pCmd);

        FAPI_SYS_PRINT_DEBUG(1,"[AV] cmd received: %i\n", avCmd.type);

        switch ( avCmd.type )
        {
            case AV_CMD_NOP:
                /* do nothing */
                break;

            case AV_CMD_WAKEUP:
                AVINT_Activate();
                break;

            case AV_CMD_GOTOSLEEP:
                AVINT_Deactivate();
                break;

            case AV_CMD_SET_OUTPUT_SYSTEM:
                AVINT_OutputSystemUpdate (avCmd.arg.outsys.hdSystem,
                                          avCmd.arg.outsys.sdSystem,
                                          FFALSE);
                break;

            case AV_CMD_SET_VIDEOAREA:
                AVINT_SetVideoArea (avCmd.arg.varea.fullScreen,
                                    FFALSE, // don't use DTG scalling Factors
                                  &(avCmd.arg.varea.area));
                break;

            case AV_CMD_ZOOM_VIDEO:
                AVINT_ZoomVideo (avCmd.arg.varea.fullScreen,
                                 FFALSE, // don't use DTG scalling Factors
                               &(avCmd.arg.varea.area));
                break;

            case AV_CMD_VIDEO_FREEZE:
                (void)AVINT_VideoFreeze (avCmd.arg.videoFreeze.freeze);
                break;

            case AV_CMD_VIDEO_IFRAME_SHOW:
                (void)AVINT_VideoIFrameShow(avCmd.arg.videoIframe.data,
                                            avCmd.arg.videoIframe.size,
                                            avCmd.arg.videoIframe.format);
                break;

            case AV_CMD_VIDEO_IFRAME_HIDE:
                (void)AVINT_VideoStop(FFALSE);
                break;

            case AV_CMD_SET_STEREOMODE:
                AVINT_AudioSetStereoMode (avCmd.arg.stereo.mode);
                break;

            /* ------------------------------------------------------------- */

            case AV_CMD_SELECT_SERVICE:
                AVINT_LiveTvSelectService (&(avCmd.arg.servsel.servLoc),
                                           &(avCmd.arg.servsel.cfg),
                                           &(avCmd.arg.servsel.pref),
                                             avCmd.arg.servsel.ctrlInfo);
                break;

            case AV_CMD_RESTART_SERVICE:
                AVINT_LiveTvRestart (&(avCmd.arg.servsel.servLoc),
                                     &(avCmd.arg.servsel.cfg));
                break;

            case AV_CMD_SELECT_AUDIOSTREAM:
                AVINT_LiveTvSelAudioStream (avCmd.arg.audio.index);
                break;

            case AV_CMD_SELECT_SPDIFSTREAM:
                AVINT_LiveTvSelSpdifStream (avCmd.arg.audio.type,
                                            avCmd.arg.audio.index);
                break;
            case AV_CMD_SELECT_SUBTTSTREAM:

                #if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)
                AVINT_SubtSelectStream(avCmd.arg.subtt.index,
                                       avCmd.arg.subtt.isDvb,
                                       FTRUE);
                #endif /*(defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)*/
                break;

            case AV_CMD_STOP_SERVICE:
                AVINT_LiveTvStop (avCmd.arg.servstop.freeze,
                                  avCmd.arg.servstop.sendNotification,
                                  avCmd.arg.servstop.resetFocus,
                                  avCmd.arg.servstop.keepRadioScreen,
                                  avCmd.arg.servstop.ctrlInfo);
                break;

            /* ------------------------------------------------------------- */

            case AV_CMD_SCAN_CARRIER:
            case AV_CMD_SCAN_LIST:
            case AV_CMD_SCAN_INPUT:
            case AV_CMD_SCAN_BLIND:
                pFe = avData->feDat + avCmd.arg.scan.cfg.feIdx;

                AVINT_GotoIdleState(pFe);
                AVINT_StartScanOperation(pFe, &avCmd);
                break;

            case AV_CMD_STOP_SCAN:
                pFe = avData->feDat + avCmd.arg.abort.feIdx;

                AVINT_AbortScanOperation(pFe);
                break;

            /* ------------------------------------------------------------- */

            #ifdef APPL_PVR_ENABLED

            case AV_CMD_REC_START:
                AVINT_RecStart (&(avCmd.arg.record.cfg));
                break;

            case AV_CMD_REC_STOP:
                AVINT_RecStop (avCmd.arg.record.cfg.tsdIdx);
                break;

            case AV_CMD_REC_REROUTE:
                AVINT_RecReroute (&(avCmd.arg.record.cfg));
                break;

            case AV_CMD_PLAY_START:
                AVINT_PlayStart(avCmd.arg.play.lstIdx,
                                avCmd.arg.play.servIdx,
                              &(avCmd.arg.play.cfg));
                break;

            case AV_CMD_PLAY_STOP:
                AVINT_PlayStop();
                break;

            #if defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED)

            case AV_CMD_DESCRREC_START:
                AVINT_DescrRecStart (&(avCmd.arg.record.cfg));
                break;

            case AV_CMD_DESCRREC_STOP:
                AVINT_DescrRecStop ();
                break;

            case AV_CMD_DESCRREC_REROUTE:
                AVINT_DescrRecReroute (&(avCmd.arg.record.cfg));
                break;

            #endif /* defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED) */

            #ifdef APPL_MPEG_ENCODER_ENABLED

            case AV_CMD_REC_MPENC_START:
                AVINT_RecMpencStart (&(avCmd.arg.record.cfg));
                break;

            case AV_CMD_REC_MPENC_STOP:
                AVINT_RecMpencStop (avCmd.arg.record.cfg.tsdIdx);
                break;

            #endif /* APPL_MPEG_ENCODER_ENABLED */

            #endif /* APPL_PVR_ENABLED */

            /* ------------------------------------------------------------- */

            case AV_CMD_TUNEMODE:
                pFe = avData->feDat + avCmd.arg.tune.cfg.feIdx;

                AVINT_GotoIdleState(pFe);
                AVINT_StartTuneMode(pFe, &avCmd);
                break;

            case AV_CMD_STOP_TUNEMODE:
                pFe = avData->feDat + avCmd.arg.abort.feIdx;

                AVINT_GotoIdleState(pFe);
                break;

            /* ------------------------------------------------------------- */

            #ifdef APPL_OAD_ENABLED

            case AV_CMD_START_OAD:
                pFe = avData->feDat + avCmd.arg.oad.cfg.feIdx;

                AVINT_GotoIdleState(pFe);
                AVINT_OAD_Start(pFe, &avCmd);
                break;

            case AV_CMD_STOP_OAD:
                pFe = avData->feDat + avCmd.arg.abort.feIdx;

                AVINT_GotoIdleState(pFe);
                AVINT_OAD_Stop(pFe);
                break;

            #endif /* APPL_OAD_ENABLED */

            /* ------------------------------------------------------------- */

            #ifdef APPL_MPEG_ENCODER_ENABLED

            case AV_CMD_MPENC_SELECT:
                AVINT_MpencMonitorStart (&(avCmd.arg.servsel.cfg));
                break;

            case AV_CMD_MPENC_STOP:
                AVINT_MpencMonitorStop ();
                break;

            #endif /* APPL_MPEG_ENCODER_ENABLED */

            /* ------------------------------------------------------------- */

            #ifdef APPL_FESAT_ENABLED

            case AV_CMD_SET_AUTOMOTORMODE:
                pFe = avData->feDat + avCmd.arg.motor.feIdx;

                pFe->autoMotorMode = avCmd.arg.motor.enable;
                break;

            #endif /* APPL_FESAT_ENABLED */

            /* ------------------------------------------------------------- */

            #ifdef APPL_DISEQC12_ENABLED

            case AV_CMD_DIS12_DRIVEMOTOR:
                pFe = avData->feDat + avCmd.arg.dis12move.feIdx;
                if ( avCmd.arg.dis12move.westEastFlag == FE_WE_FLAG_WEST )
                {
                    FE_DISEQC12_DriveMotorWest (pFe->drvHdl,
                                                avCmd.arg.dis12move.numSteps);
                }
                else
                {
                    FE_DISEQC12_DriveMotorEast (pFe->drvHdl,
                                                avCmd.arg.dis12move.numSteps);
                }
                break;

            case AV_CMD_DIS12_HALTMOTOR:
                pFe = avData->feDat + avCmd.arg.dis12move.feIdx;

                if ( pFe->drvHdl != NULL )
                {
                    FE_DISEQC12_Halt (pFe->drvHdl);
                }
                break;

            case AV_CMD_DIS12_RECALCPOS:
                pFe = avData->feDat + avCmd.arg.dis12move.feIdx;

                if ( pFe->drvHdl != NULL )
                {
                    FE_DISEQC12_ReCalcPos (pFe->drvHdl);
                }
                break;

            case AV_CMD_DIS12_STOREPOS:
                pFe = avData->feDat + avCmd.arg.dis12idx.feIdx;

                if ( pFe->drvHdl != NULL )
                {
                    FE_DISEQC12_StoreSatPos (pFe->drvHdl,
                                            avCmd.arg.dis12idx.satPos);
                }
                break;

            case AV_CMD_DIS12_GOTOPOS:
                pFe = avData->feDat + avCmd.arg.dis12idx.feIdx;

                if ( pFe->drvHdl != NULL )
                {
                    FE_DISEQC12_GotoSat (pFe->drvHdl,
                                            avCmd.arg.dis12idx.satPos);
                }
                break;

            case AV_CMD_DIS12_SETLIMITWEST:
                pFe = avData->feDat + avCmd.arg.dis12move.feIdx;

                if ( pFe->drvHdl != NULL )
                {
                    FE_DISEQC12_SetLimitWest(pFe->drvHdl);
                }
                break;

            case AV_CMD_DIS12_SETLIMITEAST:
                pFe = avData->feDat + avCmd.arg.dis12move.feIdx;

                if ( pFe->drvHdl != NULL )
                {
                    FE_DISEQC12_SetLimitEast(pFe->drvHdl);
                }
                break;

            case AV_CMD_DIS12_CLEARLIMITS:
                pFe = avData->feDat + avCmd.arg.dis12move.feIdx;

                if ( pFe->drvHdl != NULL )
                {
                    FE_DISEQC12_LimitsOff(pFe->drvHdl);
                }
                break;

            case AV_CMD_DIS12_GOTOZEROPOS:
                pFe = avData->feDat + avCmd.arg.dis12move.feIdx;

                if ( pFe->drvHdl != NULL )
                {
                    FE_DISEQC12_Goto00(pFe->drvHdl);
                }
                break;

            #endif /* APPL_DISEQC12_ENABLED */

            #ifdef APPL_USALS_ENABLED

            case AV_CMD_USALS_GOTOPOS:
                pFe = avData->feDat + avCmd.arg.usalspos.feIdx;

                if ( pFe->usalsHdl != NULL )
                {
                    (void)USALS_GotoPos (pFe->usalsHdl,
                                         &(avCmd.arg.usalspos.orbPos));
                }
                break;

            case AV_CMD_USALS_GOTOZERO:
                pFe = avData->feDat + avCmd.arg.usalspos.feIdx;

                if ( pFe->usalsHdl != NULL )
                {
                    (void)USALS_GotoPosZero (pFe->usalsHdl);
                }
                break;
            #endif /* APPL_USALS_ENABLED */

            /* ------------------------------------------------------------- */

            #ifdef APPL_PIP_ENABLED

            case AV_CMD_PIP_SELECT:

                AVINT_PIPSelect (&(avCmd.arg.pip.servLoc),
                                 &(avCmd.arg.pip.cfg));
                break;

            case AV_CMD_PIP_STOP:
                AVINT_PIPStop ();
                break;

            #endif /* APPL_PIP_ENABLED */

            /* ------------------------------------------------------------- */

            #ifdef APPL_MHEG_ENABLED

            case AV_CMD_MHEG_VIDEO:
                AVINT_MHEG_Video(&(avCmd.arg.mhegVd));
                break;

            case AV_CMD_MHEG_AUDIO:
                AVINT_MHEG_Audio(&(avCmd.arg.mhegAd));
                break;

            case AV_CMD_MHEG_DEMUX:
                AVINT_MHEG_Demux(&(avCmd.arg.mhegDx));
                break;

            #endif /* APPL_MHEG_ENABLED */
            /* ------------------------------------------------------------- */

            case AV_CMD_SET_ANTENNAPOWER:
                pFe = avData->feDat + avCmd.arg.antpwr.feIdx;

                AVINT_SetAntennaPower (pFe, avCmd.arg.antpwr.power);
                break;

            default:
                FAPI_SYS_PRINT_DEBUG(1,"[AV] ERROR UNKNOWN COMMAND = %d\n",avCmd.type);
                break;
        }
    }
}

FSTATIC void AV_ThreadProcessing(void)
{
    const SYS_CONFIGDAT_S*  sysCfg = SYS_GetConfig();
    const SYS_HWCONFIG_S*   hwCfg  = SYS_GetHWConfig();
    uint16_t                cnt;
    uint32_t                reqSleepTime;
    uint64_t                currTime;
    int32_t                 sleepTime;
    AV_FEDAT_S*             pFe;
    #ifdef APPL_MPEG_ENCODER_ENABLED
        FD_MPENC_NOTIFY_S   pollNoti;
        FD_MPENC_INTSTAT_S  encStat;
        FD_MPENC_PTS_S      encPts;
        uint8_t             palDec;
    #endif

    /* perform jobs according to current state */
    while ( !(avData->itcInst.abortReq) )
    {
        /* ----- Perform general tasks ------------------------------------- */

        PSISI_InternalHandling();

        AVINT_ScartHandleOutputSource();

        #ifdef APPL_MPEG_ENCODER_ENABLED
        MPENC_Poll(&pollNoti,&encStat,&encPts,&palDec);
        #endif

        if ( avData->jingleReq &&
             (UTILS_GetTimerStampInterval(avData->jingleStartStamp,
                                          SYS_GET_TIMESTAMP_MS)
                                          >= sysCfg->welcomeJingleDelay) )
        {
            AVINT_StartWelcomeJingle();
            avData->jingleReq = FFALSE;
        }

        /* ----- Frontend related tasks ------------------------------------ */

        for ( cnt = 0; cnt < hwCfg->feNums; cnt++ )
        {
            pFe = avData->feDat + cnt;

            if ( pFe->drvHdl == NULL ) continue;

            currTime = SYS_GET_TIMESTAMP_MS;

            /* test if this fe's task is ready for execution */
            if ( avData->nextSlotStamp[cnt] > currTime )
            {
                continue;
            }

            /* NOTE: In case of an overflow, a task could be
                     scheduled earlier than its requested time,
                     which should generally be uncritical
                     -> no action has been taken to detect
                     overflow here. */

            reqSleepTime = AV_SLEEP_DEFAULT; /* set default sleep time */

            switch ( pFe->state )
            {
                case AV_FESTATE_IDLE:
                    reqSleepTime = AV_TaskIdle(pFe);
                    break;

                case AV_FESTATE_NORMAL:
                    reqSleepTime = AV_TaskNormal(pFe);
                    break;

                case AV_FESTATE_SCANNING:
                    reqSleepTime = AV_TaskScan(pFe);
                    break;

                #ifdef APPL_OAD_ENABLED

                case AV_FESTATE_OAD:
                    reqSleepTime = AV_TaskOad(pFe);
                    break;

                #endif /* APPL_OAD_ENABLED */

                default:
                    break;
            }

            avData->nextSlotStamp[cnt] = currTime + reqSleepTime;

            if ( avData->itcInst.abortReq ) break;
        }

        if ( avData->itcInst.abortReq ) break;

        /* ----- PVR playback tasks ---------------------------------------- */

        #ifdef APPL_PVR_ENABLED

        if ( avData->play.active )
        {
            currTime = SYS_GET_TIMESTAMP_MS;

            if ( !(avData->play.internal) )
            {
                avData->nextSlotStamp[hwCfg->feNums] = currTime +
                                                       AVINT_PlayTasks ();
            }

            #if defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED)

            else
            {
                avData->nextSlotStamp[hwCfg->feNums] = currTime +
                                                       AVINT_DescrRecTasks ();
            }
            #endif /* defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED) */
        }

        if ( avData->itcInst.abortReq ) break;

        #endif /* APPL_PVR_ENABLED */

        /* ----- Schedule next loop ---------------------------------------- */

        currTime  = SYS_GET_TIMESTAMP_MS;
        sleepTime = AV_SLEEP_MAX; /* initialise with maximum sleep time */

        /* calculate minimum sleep time until next scheduled job */
        for ( cnt = 0; cnt < hwCfg->feNums; cnt++ )
        {
            pFe = avData->feDat + cnt;

            if ( pFe->drvHdl == NULL ) continue;

            if ( avData->nextSlotStamp[cnt] <= currTime )
            {
                // TODO: test if overflow has occured!!
                sleepTime = 0;
            }
            else if ( (avData->nextSlotStamp[cnt] - currTime) < sleepTime )
            {
                sleepTime = avData->nextSlotStamp[cnt] - currTime;
            }
        }

        #ifdef APPL_PVR_ENABLED

        if ( avData->play.active )
        {
            if ( avData->nextSlotStamp[hwCfg->feNums] <= currTime )
            {
                // TODO: test if overflow has occured!!
                sleepTime = 0;
            }
            else if ( (avData->nextSlotStamp[hwCfg->feNums] - currTime)
                       < sleepTime )
            {
                sleepTime = avData->nextSlotStamp[hwCfg->feNums] - currTime;
            }
        }

        #endif /* APPL_PVR_ENABLED */

        FAPI_SYS_PRINT_DEBUG(4,"curr %i sleepTime %i \n", currTime, sleepTime);

        AV_SleepAndCheckAbortion(sleepTime, FTRUE);
    }
}

/*
returns:
- 1: sleep has not been aborted
- 0: sleep has been aborted
*/
fbool_t AV_SleepAndCheckAbortion (int32_t timeInMs, fbool_t abortable)
{
    if ( timeInMs == 0 ) return FTRUE;

    if ( !abortable )
    {
        FAPI_SYS_SLEEP(timeInMs);
        return FFALSE;
    }
    else
    {
        return ITC_SleepAndCheckAbortion(&(avData->itcInst), timeInMs);
    }
}

FSTATIC uint32_t AV_TaskIdle(AV_FEDAT_S* pFe)
{
    uint64_t currTime;

    currTime = SYS_GET_TIMESTAMP_MS;

    if ( UTILS_GetTimerStampInterval(pFe->sigLastUpdateTime, currTime) >=
         AV_SIGNALUPDATE_MIN )
    {
        pFe->sigLastUpdateTime = currTime;
        AVINT_UpdateSignalParams(pFe);
    }

    return AV_SLEEP_IDLE;
}

void AVINT_UpdateSignalParams (AV_FEDAT_S* pFe)
{
    if ( pFe->drvHdl != NULL )
    {
        if ( pFe->tuneLockStatus != SYS_FELOCK_LOCKED )
        {
            pFe->sigStrengthCurr = 0;
            pFe->sigQualiCurr    = 0;
        }
        else
        {
            FE_GetSignalStrength (pFe->drvHdl, &(pFe->sigStrengthCurr));
            FE_GetSignalQuality  (pFe->drvHdl, &(pFe->sigQualiCurr));
        }
    }
}

FSTATIC uint32_t AV_TaskNormal (AV_FEDAT_S* pFe)
{
    int32_t                  retVal;
    FE_STATUS_E              feStatus;
    uint64_t                 nextSlotTime;
    uint64_t                 currTime;

    currTime = SYS_GET_TIMESTAMP_MS;

    if ( UTILS_GetTimerStampInterval(pFe->sigLastUpdateTime, currTime) >=
         AV_SIGNALUPDATE_MIN )
    {
        pFe->sigLastUpdateTime = currTime;

        AVINT_UpdateSignalParams(pFe);
    }

    do
    {
        /* ----- Tuning is necessary --------------------------------------- */

        if ( AVINT_TuningIsNecessary(pFe, &(pFe->reqTunePar)) )
        {
            nextSlotTime = AV_SLEEP_DECODING_RECOVERY;

            /* try to tune to requested transponder/channel */
            retVal = AVINT_Tune(pFe, &(pFe->reqTunePar), FTRUE);

            if ( avData->itcInst.abortReq ) return 0;

            AVINT_UpdateSignalParams(pFe);

            if ( retVal == FAPI_OK )
            {
                if ( pFe->useStatus == 0 )
                {
                    nextSlotTime = AV_SLEEP_DECODING_IDLE;
                }
                else
                {
                    nextSlotTime = AV_SLEEP_DECODING_NORMAL;
                }

                if ( pFe->useStatus & AV_USESTATUS_LIVETV )
                {
                    AVINT_LiveTvSignalAvailable(pFe);

                    if ( avData->itcInst.abortReq ) return 0;
                }

                #ifdef APPL_PVR_ENABLED

                if ( pFe->useStatus & AV_USESTATUS_REC_TSD2 )
                {
                    AVINT_RecSignalAvailable(FAPI_TSD1);

                    if ( avData->itcInst.abortReq ) return 0;
                }

                if ( pFe->useStatus & AV_USESTATUS_REC_TSD3 )
                {
                    AVINT_RecSignalAvailable(FAPI_TSD2);

                    if ( avData->itcInst.abortReq ) return 0;
                }

                #endif /* APPL_PVR_ENABLED */

                #ifdef APPL_PIP_ENABLED

                if ( pFe->useStatus & AV_USESTATUS_PIP )
                {
                    AVINT_PIPSignalAvailable(pFe);

                    if ( avData->itcInst.abortReq ) return 0;
                }

                #endif /* APPL_PIP_ENABLED */
            }
            else
            {
            }
        }

        /* ----- Tuner is currently locked --------------------------------- */

        else
        {
            if ( pFe->useStatus == 0 )
            {
                nextSlotTime = AV_SLEEP_DECODING_IDLE;
            }
            else
            {
                nextSlotTime = AV_SLEEP_DECODING_NORMAL;
            }

            if ( pFe->useStatus & AV_USESTATUS_LIVETV )
            {
                AVINT_LiveTvSignalAvailable(pFe);
                AVINT_LiveTvTasks          (pFe);
                if ( avData->itcInst.abortReq ) return 0;
            }

            #ifdef APPL_PVR_ENABLED

            if ( pFe->useStatus & AV_USESTATUS_REC_TSD2 )
            {
                AVINT_RecSignalAvailable(FAPI_TSD1);
                AVINT_RecTasks          (FAPI_TSD1);

                if ( avData->itcInst.abortReq ) return 0;
            }

            if ( pFe->useStatus & AV_USESTATUS_REC_TSD3 )
            {
                AVINT_RecSignalAvailable(FAPI_TSD2);
                AVINT_RecTasks          (FAPI_TSD2);

                if ( avData->itcInst.abortReq ) return 0;
            }

            #endif /* APPL_PVR_ENABLED */

            #ifdef APPL_PIP_ENABLED

            if ( pFe->useStatus & AV_USESTATUS_PIP )
            {
                AVINT_PIPSignalAvailable(pFe);
                AVINT_PIPTasks          (pFe);

                if ( avData->itcInst.abortReq ) return 0;
            }

            #endif /* APPL_PIP_ENABLED */

            /* ----- "Perform Lost-Lock" detection ------------------------- */

            if ( UTILS_GetTimerStampInterval(pFe->lastLockCheck, currTime) >=
                 AV_LOCKCHECKINTERVAL_MIN )
            {
                pFe->lastLockCheck = currTime;

                retVal = FE_GetStatus(pFe->drvHdl, &feStatus);

                if ( retVal == FAPI_OK )
                {
                    if (( feStatus != FE_STATUS_LOCKED ) && ( feStatus != FE_STATUS_TUNING ))
                    {
                        FAPI_SYS_PRINT_DEBUG(1,"FE-%i: LOCK LOST \n", pFe->feIdx + 1);

                        AVINT_SetLockStatus(pFe, SYS_FELOCK_NOTLOCKED);
                        AVINT_UpdateSignalParams(pFe);

                        if ( pFe->useStatus & AV_USESTATUS_LIVETV )
                        {
                            AVINT_LiveTvSignalLost(pFe);
                        }

                        #ifdef APPL_PVR_ENABLED

                        if ( pFe->useStatus & AV_USESTATUS_REC_TSD2 )
                        {
                            AVINT_RecSignalLost(FAPI_TSD1);
                        }

                        if ( pFe->useStatus & AV_USESTATUS_REC_TSD3 )
                        {
                            AVINT_RecSignalLost(FAPI_TSD2);
                        }

                        #endif /* APPL_PVR_ENABLED */

                        #ifdef APPL_PIP_ENABLED

                        if ( pFe->useStatus & AV_USESTATUS_PIP )
                        {
                            AVINT_PIPSignalLost(pFe);
                        }

                        #endif /* APPL_PIP_ENABLED */

                        nextSlotTime = AV_SLEEP_DECODING_RECOVERY;
                    }
                }
            }
        }
    }
    while ( 0 );

    return nextSlotTime;
}

fbool_t AVINT_TuningIsNecessary (AV_FEDAT_S* pFe, const AV_TUNEPARS_S* tunePars)
{
    fbool_t retVal = FFALSE;

    if ( (pFe->forceTuning)                                  ||
         (pFe->tuneLockStatus         != SYS_FELOCK_LOCKED)  ||
         (pFe->currTunePar.manualMode != tunePars->manualMode) )
    {
        return FTRUE;
    }

    if ( !(pFe->currTunePar.manualMode) )
    {
        if ( (pFe->currTunePar.network.satIdx != tunePars->network.satIdx) ||
             (pFe->currTunePar.tpIdx          != tunePars->tpIdx) )
        {
            return FTRUE;
        }
    }
    else /* manual mode: compare tune parameters one by one */
    {
        switch ( pFe->currTunePar.network.type )
        {
            #ifdef APPL_FESAT_ENABLED

            case FE_SYSTEM_TYPE_SATELLITE:
                if ( (tunePars->par.sat.frequency  != pFe->currTunePar.par.sat.frequency)  ||
                     (tunePars->par.sat.symbolRate != pFe->currTunePar.par.sat.symbolRate) ||
                     (tunePars->par.sat.fec        != pFe->currTunePar.par.sat.fec)        ||
                     (tunePars->par.sat.modulation != pFe->currTunePar.par.sat.modulation) ||
                     (tunePars->par.sat.pol        != pFe->currTunePar.par.sat.pol) )
                {
                    retVal = FTRUE;
                }
                break;

            #endif /* APPL_FESAT_ENABLED */

            #ifdef APPL_FETERR_ENABLED

            case FE_SYSTEM_TYPE_TERRESTRIAL:
                if ( (tunePars->par.terr.frequency     != pFe->currTunePar.par.terr.frequency)     ||
                     (tunePars->par.terr.bandwidth     != pFe->currTunePar.par.terr.bandwidth)     ||
                     (tunePars->par.terr.codeRateHP    != pFe->currTunePar.par.terr.codeRateHP)    ||
                     (tunePars->par.terr.codeRateLP    != pFe->currTunePar.par.terr.codeRateLP)    ||
                     (tunePars->par.terr.modulation    != pFe->currTunePar.par.terr.modulation)    ||
                     (tunePars->par.terr.transmission  != pFe->currTunePar.par.terr.transmission)  ||
                     (tunePars->par.terr.guardInterval != pFe->currTunePar.par.terr.guardInterval) ||
                     (tunePars->par.terr.hierarchy     != pFe->currTunePar.par.terr.hierarchy) )
                {
                    retVal = FTRUE;
                }
                break;

            #endif /* APPL_FETERR_ENABLED */

            #ifdef APPL_FECABLE_ENABLED

            case FE_SYSTEM_TYPE_CABLE:
                if ( (tunePars->par.cable.frequency     != pFe->currTunePar.par.cable.frequency)     ||
                     (tunePars->par.cable.symbolRate    != pFe->currTunePar.par.cable.symbolRate)    ||
                     (tunePars->par.cable.modulation    != pFe->currTunePar.par.cable.modulation)    ||
                     (tunePars->par.cable.fec           != pFe->currTunePar.par.cable.fec)           ||
                     (tunePars->par.cable.iq            != pFe->currTunePar.par.cable.iq) )
                {
                    retVal = FTRUE;
                }
                break;

            #endif /* APPL_FECABLE_ENABLED */

            #ifdef APPL_FEDIRECT_ENABLED

            case FE_SYSTEM_TYPE_DIRECT:
                retVal = FFALSE;
                break;

            #endif /* APPL_FEDIRECT_ENABLED */

            default:
                retVal = FFALSE;
                break;
        }
    }

    return retVal;
}

int32_t AVINT_Tune (AV_FEDAT_S* pFe, const AV_TUNEPARS_S* tunePars,
                    fbool_t abortable)
{
    uint8_t               attempt;
    FE_STATUS_E           status;
    int32_t               ferr = FAPI_OK;
    TPLIST_ITEM_S         tpItem;
    #ifdef APPL_FESAT_ENABLED

    FE_SYSTEM_PARAMS_S systemParams;

    #endif /* APPL_FESAT_ENABLED */

    /* requested frontend available? */
    if ( pFe->drvHdl == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* ----- Get and prepare tune parameters ------------------------------  */

    /* If no tune parameters are passed, get them from carrier information */
    if ( !(tunePars->manualMode) )
    {
        ferr = TPLIST_GetDataByIndex(&(tunePars->network), tunePars->tpIdx,
                                     &tpItem);
        if ( ferr != FAPI_OK ) return ferr;
    }
    else
    {
        tpItem.pars = tunePars->par;
    }
    ferr = AVINT_TuneParamsPrepare(pFe, &(tpItem.pars),
                                   tunePars->network.satIdx);
    if ( ferr != FAPI_OK ) return ferr;

    /* --------------------------------------------------------------------  */

    #ifdef APPL_FESAT_ENABLED

    if ( tunePars->network.type == FE_SYSTEM_TYPE_SATELLITE )
    {
        /* test if target satellite has changed: in this case, tuner system
           parameters must be updated */
        if ( (tunePars->network.satIdx != pFe->currTunePar.network.satIdx) ||
              pFe->forceLnbSettsUpdate )
        {
            do
            {
                ferr = FE_GetSystemParams(pFe->drvHdl, &systemParams);
                if ( ferr != FAPI_OK ) break;

                ferr = AV_SetSystemParamsSat(pFe->feIdx, tunePars->network.satIdx,
                                             &systemParams);
                if ( ferr != FAPI_OK ) break;

                ferr = FE_SetSystemParams(pFe->drvHdl, &systemParams);
                if ( ferr != FAPI_OK ) break;
            }
            while ( 0 );

            if ( ferr != FAPI_OK ) return ferr;

            pFe->forceLnbSettsUpdate = FFALSE;
        }

        if ( pFe->autoMotorMode )
        {
            AVINT_MoveMotor (pFe, tunePars->network.satIdx);
        }
    }

    #endif /* APPL_FESAT_ENABLED */

    /* --------------------------------------------------------------------  */

    /* Mark corresponding PAT/PMT data as invalid */
    PSISI_InvalidateByFe(&(pFe->pat));

    if ( abortable && avData->itcInst.abortReq ) return SMARTGO_SYS_ERR_ABORTED;

    if ( pFe->tuneLockStatus == SYS_FELOCK_NOTLOCKED )
    {
        AVINT_SetLockStatus(pFe, SYS_FELOCK_RECOVERYTUNING);
    }
    else
    {
        AVINT_SetLockStatus(pFe, SYS_FELOCK_TUNING);
    }

    /* set requested tune parameters as "current" tune parameters */
    pFe->currTunePar     = *tunePars;
    pFe->currTunePar.par = tpItem.pars;

    for ( attempt = 0; attempt < AV_FE_TUNE_ATTEMPTS; attempt++ )
    {
        ferr = FE_Tune(pFe->drvHdl, &(tpItem.pars));

        if ( ferr == FAPI_OK )
        {
            /* poll tuning status */\
            do
            {
                if ( abortable && avData->itcInst.abortReq )
                {
                    return SMARTGO_SYS_ERR_ABORTED;
                }

                if ( AV_SleepAndCheckAbortion(50, abortable) == FFALSE )
                {
                    return SMARTGO_SYS_ERR_ABORTED;
                }
                ferr = FE_GetStatus(pFe->drvHdl, &status);
            }
            while ( (ferr   == FAPI_OK) &&
                    (status == FE_STATUS_TUNING) );

            if ( (ferr == FAPI_OK) &&
                 (status != FE_STATUS_LOCKED) )
            {
                ferr = FE_ERR_FE_NOT_LOCKED;
            }
            break;
        }

        if ( abortable && avData->itcInst.abortReq )
        {
            return SMARTGO_SYS_ERR_ABORTED;
        }

        //AV_ResetFrontend();

        if ( AV_SleepAndCheckAbortion(50, abortable) == FFALSE )
        {
            return SMARTGO_SYS_ERR_ABORTED;
        }
    }

    /* debug output */
    switch ( tunePars->network.type )
    {
        case FE_SYSTEM_TYPE_SATELLITE:
            FAPI_SYS_PRINT_DEBUG(1,"[AV] tune FE-%i: f=%d %s, symb=%d, satId=%d, mod= %d, fec=%d, ferr=%d\n",
                      pFe->feIdx + 1,
                      tpItem.pars.sat.frequency,
                     (tpItem.pars.sat.pol == FE_POL_HOR) ? "H" : "V",
                      tpItem.pars.sat.symbolRate,
                      tpItem.pars.sat.satID,
                      tpItem.pars.sat.modulation,
                      tpItem.pars.sat.fec,
                      ferr);
            break;

        case FE_SYSTEM_TYPE_TERRESTRIAL:
            FAPI_SYS_PRINT_DEBUG(1,"[AV] tune FE-%i: f=%d, bw=%d coderate=%d, ferr=%d\n",
                      pFe->feIdx + 1,
                      tpItem.pars.terr.frequency,
                      tpItem.pars.terr.bandwidth,
                      tpItem.pars.terr.codeRateHP,
                      ferr);
            break;

        case FE_SYSTEM_TYPE_CABLE:
            FAPI_SYS_PRINT_DEBUG(1,"[AV] tune FE-%i: f=%d, sr=%d mod=%d, ferr=%d\n",
                      pFe->feIdx + 1,
                      tpItem.pars.cable.frequency,
                      tpItem.pars.cable.symbolRate,
                      tpItem.pars.cable.modulation,
                      ferr);
            break;


        case FE_SYSTEM_TYPE_DIRECT:
            FAPI_SYS_PRINT_DEBUG(1,"[AV] tune FE-%i: TS direct input ferr=%d\n",
                      pFe->feIdx + 1,
                      ferr);
            break;

        default:
            break;
    }

    if ( ferr == FAPI_OK )
    {
        AVINT_SetLockStatus(pFe, SYS_FELOCK_LOCKED);
        /* Restart/resume PAT filtering */
        PSISI_RestartByFe(&(pFe->pat));
        pFe->forceTuning = FFALSE;
    }
    else
    {
        AVINT_SetLockStatus(pFe, SYS_FELOCK_NOTLOCKED);
    }

    return ferr;
}

FSTATIC void AVINT_SetLockStatus (AV_FEDAT_S* pFe, SYS_FE_LOCK_E feStatus)
{
    const SYS_HWCONFIG_S*   hwCfg      = SYS_GetHWConfig();
    fbool_t                 ledEnabled = FFALSE;
    SYS_EVT_LOCKSTAT_S      ctrlEvent;
    uint8_t                 cnt;

    if ( feStatus != pFe->tuneLockStatus )
    {
        pFe->tuneLockStatus = feStatus;

        /* handle frontend lock LED: LED is switched on as soon and as long
           as at least one frontend is locked. */
        for ( cnt = 0; cnt < hwCfg->feNums; cnt++ )
        {
            if ( avData->feDat[cnt].tuneLockStatus == SYS_FELOCK_LOCKED )
            {
                ledEnabled = FTRUE;
                break;
            }
        }

        SYS_ControlFpSymbol(FP_SYMB_FELOCK, ledEnabled);
    }

    /* send notification */
    ctrlEvent.hdr.type   = EVT_LOCK_STATUS;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);
    ctrlEvent.feNum      = pFe->feIdx;
    ctrlEvent.status     = feStatus;

    (void)EVT_Send(&ctrlEvent, "Lock");
}

FSTATIC int32_t AVINT_TuneParamsGenerate (AV_FEDAT_S* pFe, const TUNE_ITEM_S* pItem,
                                       CFGSAT_IDX satIdx,
                                       FE_TUNE_PARAMS_S* pPar)
{
    #ifdef APPL_FESAT_ENABLED
    int32_t retVal;
    #endif /* APPL_FESAT_ENABLED */

    memset(pPar, 0, sizeof(FE_TUNE_PARAMS_S));

    switch ( pFe->systemType )
    {
        #ifdef APPL_FESAT_ENABLED

        case FE_SYSTEM_TYPE_SATELLITE:
            retVal = AVINT_TuneParamsAddSatInfo(pFe, satIdx, pPar);
            if ( retVal != FAPI_OK ) return retVal;

            pPar->sat.frequency       = pItem->params.sat.frequency;
            pPar->sat.symbolRate      = pItem->params.sat.symbol;
            pPar->sat.pol             = pItem->params.sat.pol;
            pPar->sat.modulation      = pItem->params.sat.modulation;
            pPar->sat.fec             = FE_FEC_AUTO;
            pPar->sat.iq              = FE_IQ_AUTO;
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case FE_SYSTEM_TYPE_TERRESTRIAL:
            pPar->terr.frequency      = pItem->params.terr.frequency;
            pPar->terr.bandwidth      = pItem->params.terr.bandwidth;
            pPar->terr.codeRateHP     = pItem->params.terr.codeRate;
            pPar->terr.codeRateLP     = pItem->params.terr.codeRate;
            pPar->terr.modulation     = FE_MOD_AUTO;
            pPar->terr.transmission   = FE_TRANS_MODE_AUTO;
            pPar->terr.guardInterval  = FE_GUARD_INTERVAL_AUTO;
            pPar->terr.hierarchy      = FE_HIERARCHY_AUTO;
            pPar->terr.iq             = FE_IQ_NORMAL; /* FE_IQ_INVERTED */

            if ( pPar->terr.codeRateHP == FE_FEC_NONE )
                pPar->terr.codeRateHP = FE_FEC_AUTO;

            if ( pPar->terr.codeRateLP == FE_FEC_NONE )
                pPar->terr.codeRateLP = FE_FEC_AUTO;
            break;

        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED

        case FE_SYSTEM_TYPE_CABLE:
            pPar->cable.frequency      = pItem->params.cable.frequency;
            pPar->cable.symbolRate     = pItem->params.cable.symbol;
            pPar->cable.modulation     = pItem->params.cable.modulation;
            pPar->cable.fec            = FE_FEC_AUTO;
            pPar->cable.iq             = FE_IQ_AUTO;
            break;

        #endif /* APPL_FECABLE_ENABLED */

        #ifdef APPL_FEDIRECT_ENABLED

        case FE_SYSTEM_TYPE_DIRECT:
            break;

        #endif /* APPL_FEDIRECT_ENABLED */

        default:
           break;
    }

    return FAPI_OK;
}

FSTATIC int32_t AVINT_TuneParamsPrepare (AV_FEDAT_S* pFe,
                                      FE_TUNE_PARAMS_S* pPar,
                                      CFGSAT_IDX satIdx)
{
    #ifdef APPL_FESAT_ENABLED
    int32_t retVal;
    #endif /* APPL_FESAT_ENABLED */

    switch ( pFe->systemType )
    {
        #ifdef APPL_FESAT_ENABLED

        case FE_SYSTEM_TYPE_SATELLITE:
            retVal = AVINT_TuneParamsAddSatInfo(pFe, satIdx, pPar);
            if ( retVal != FAPI_OK ) return retVal;

            if ( pPar->sat.fec        == FE_FEC_NONE ) pPar->sat.fec        = FE_FEC_AUTO;
            if ( pPar->sat.modulation == FE_MOD_NONE ) pPar->sat.modulation = FE_MOD_AUTO;
            break;

        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED

        case FE_SYSTEM_TYPE_TERRESTRIAL:
            if ( pPar->terr.codeRateHP    == FE_FEC_NONE )            pPar->terr.codeRateHP     = FE_FEC_AUTO;
            if ( pPar->terr.codeRateLP    == FE_FEC_NONE )            pPar->terr.codeRateLP     = FE_FEC_AUTO;
            if ( pPar->terr.modulation    == FE_MOD_NONE )            pPar->terr.modulation     = FE_MOD_AUTO;
            if ( pPar->terr.transmission  == FE_TRANS_MODE_NONE )     pPar->terr.transmission   = FE_TRANS_MODE_AUTO;
            if ( pPar->terr.guardInterval == FE_GUARD_INTERVAL_NONE ) pPar->terr.guardInterval  = FE_GUARD_INTERVAL_AUTO;
            if ( pPar->terr.hierarchy     == FE_HIERARCHY_NONE )      pPar->terr.hierarchy      = FE_HIERARCHY_AUTO;

            pPar->terr.iq             = FE_IQ_NORMAL; /* FE_IQ_INVERTED */
            break;

        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED

        case FE_SYSTEM_TYPE_CABLE:
            if ( pPar->cable.modulation    == FE_MOD_NONE )
                pPar->cable.modulation     = FE_MOD_AUTO;

            pPar->cable.iq             = FE_IQ_NORMAL; /* FE_IQ_INVERTED */
            break;

        #endif /* APPL_FECABLE_ENABLED */

        #ifdef APPL_FEDIRECT_ENABLED

        case FE_SYSTEM_TYPE_DIRECT:
            break;

        #endif /* APPL_FEDIRECT_ENABLED */

        default:
           break;
    }

    return FAPI_OK;
}

FSTATIC int32_t AVINT_Activate (void)
{
    int32_t err_code = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(1, "[AV][AVINT_Activate] enter\n");

    if ( avData->threadState == AV_STATE_ACTIVE )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[AV][AVINT_Activate] quit(already activate)\n");
        return FAPI_OK;
    }

    //(void)AVINT_VideoOpen();

    err_code = AVINT_VideoOpen();
    FAPI_SYS_PRINT_DEBUG(1, "AVINT_VideoOpen() errcode = %d\n", err_code);

    //(void)AVINT_AudioWakeup();
    err_code = AVINT_AudioWakeup();
    FAPI_SYS_PRINT_DEBUG(1, "AVINT_AudioWakeup() errcode = %d\n", err_code);

    AV_OutputFormatUpdate (FTRUE);


    AV_PowerupScreenShow(FFALSE);

    AV_OpenFrontends();  /* open frontend(s) */

    avData->jingleReq        = FFALSE;
    avData->jingleStartStamp = SYS_GET_TIMESTAMP_MS;

    (void)SETTINGS_PlayJingleGet(&(avData->jingleReq));

    avData->threadState = AV_STATE_ACTIVE;
    FAPI_SYS_PRINT_DEBUG(1, "[AV][AVINT_Activate] quit\n");
    return FAPI_OK;
}

FSTATIC int32_t AVINT_Deactivate (void)
{
    uint8_t cnt;

    if ( avData->threadState == AV_STATE_SLEEP ) return FAPI_OK;

    /* stop all operations on all frontends */
    for ( cnt = 0; cnt < SYS_GetHWConfig()->feNums; cnt++ )
    {
        AVINT_GotoIdleState(avData->feDat + cnt);
    }

    (void)AVINT_AudioShutdown ();
    (void)AVINT_VideoClose    ();

    AV_CloseFrontends(); /* Close frontend(s) */

    avData->threadState = AV_STATE_SLEEP;
    return FAPI_OK;
}

FSTATIC int32_t AVINT_GotoIdleState (AV_FEDAT_S* pFe)
{
    switch ( pFe->state )
    {
        case AV_FESTATE_IDLE:
            /* do nothing */
            break;

        case AV_FESTATE_NORMAL:
            if ( pFe->useStatus & AV_USESTATUS_LIVETV )
            {
                AVINT_LiveTvStop (FFALSE, FFALSE, FFALSE, FFALSE, FTRUE);
            }

            #ifdef APPL_PIP_ENABLED

            if ( pFe->useStatus & AV_USESTATUS_PIP )
            {
                AVINT_PIPStop();
            }

            #endif /* APPL_PIP_ENABLED */

            // TODO: abort recordings!!

            break;

        case AV_FESTATE_SCANNING:
            AVINT_AbortScanOperation(pFe);
            break;

        #ifdef APPL_OAD_ENABLED

        case AV_FESTATE_OAD:
            //AVINT_OAD_Stop(pFe);
            break;

        #endif /* APPL_OAD_ENABLED */

        default:
            break;
    }

    pFe->state = AV_FESTATE_IDLE;

    return FAPI_OK;
}

/* NOTE: This is only to be called prior to entering active AV mode! */
int32_t AV_PowerupScreenShow (fbool_t openVideo)
{
    const SYS_CONFIGDAT_S*  sysCfg = SYS_GetConfig();
    int32_t                 retVal = FAPI_OK;
    SYS_VIDEO_OUT           videoOutput;

    FAPI_SYS_PRINT_DEBUG(1, "[av] enter AV_PowerupScreenShow\n");

    if ( (!avData->video.bootScreenActive) &&
         (sysCfg->bootScreen != NULL) )
    {
        if ( openVideo )
        {
            /* open video output */
            retVal = AVINT_VideoOpen();
        }

        if ( retVal == FAPI_OK )
        {
            // TODO: set this according to display format (4:3 display only) !!!
            SCART_SetMode(SCART_MODE_16_9);

            if(SETTINGS_VideoOutputGet(&videoOutput) == FAPI_OK)
            {
                SCART_SetVideoOutput(videoOutput);
            }

            // TODO: to be supported!
            //VAL_SetWssData(VIDEO_GetViencHdHandle(),AV_WSS_4_3_CENTRE);

            // Reset the cropping window
            retVal = AVINT_SetVideoArea(FTRUE, FFALSE, NULL);

            if ( retVal == FAPI_OK )
            {
                /* show I-Frame */
                retVal = AVINT_VideoIFrameShow (sysCfg->bootScreen,
                                                sysCfg->bootScreenSize,
                                                STRTYPE_VIDEO_MPEG2);
                avData->video.bootScreenActive = FTRUE;
            }
        }
    }
    FAPI_SYS_PRINT_DEBUG(1, "[av] quit AV_PowerupScreenShow, errcode = %d\n", retVal);
    return retVal;
}

void AV_UpdateTPParams (AV_FEDAT_S* pFe, const SYS_NETWORK_S* network,
                        TPLIST_IDX carrierIdx)
{
    FE_TUNE_PARAMS_S    tunePars;
    TPLIST_ITEM_S       tpItem;
    uint16_t            sigQuality = 0;
    uint8_t             changed    = 0;
    int32_t             retParams;
    int32_t             retQuality;

    if ( pFe->drvHdl == NULL ) return;

    retParams  = FE_GetTuneParams    (pFe->drvHdl, &tunePars);
    retQuality = FE_GetSignalQuality (pFe->drvHdl, &sigQuality);

    if ( TPLIST_GetDataByIndex(network, carrierIdx, &tpItem) != FAPI_OK ) return;

    /* ----- update signal quality ----------------------------------------- */

    if ( retQuality == FAPI_OK )
    {
        /* Avoid 0 for signal quality as it means 'unknown'. */
        if ( sigQuality < 1 ) sigQuality = 1;

        if ( tpItem.sigQuality != sigQuality )
        {
            tpItem.sigQuality = sigQuality;
            changed = 1;
        }
    }

    /* ----- update stored TP/channel parameters --------------------------- */

    if ( retParams == FAPI_OK )
    {
        switch ( network->type )
        {
            #ifdef APPL_FESAT_ENABLED

            case FE_SYSTEM_TYPE_SATELLITE:
                if ( (tunePars.sat.fec != FE_FEC_NONE) &&
                     (tunePars.sat.fec != FE_FEC_AUTO) &&
                     (tunePars.sat.fec != tpItem.pars.sat.fec) )
                {
                    tpItem.pars.sat.fec = tunePars.sat.fec;
                    changed = 1;
                }

                if ( (tunePars.sat.iq != FE_IQ_NONE) &&
                     (tunePars.sat.iq != FE_IQ_AUTO) &&
                     (tunePars.sat.iq != tpItem.pars.sat.iq) )
                {
                    tpItem.pars.sat.iq = tunePars.sat.iq;
                    changed = 1;
                }
                break;

            #endif /* APPL_FESAT_ENABLED */

            #ifdef APPL_FETERR_ENABLED

            case FE_SYSTEM_TYPE_TERRESTRIAL:
                if ( (tunePars.terr.codeRateHP != FE_FEC_NONE) &&
                     (tunePars.terr.codeRateHP != FE_FEC_AUTO) &&
                     (tunePars.terr.codeRateHP != tpItem.pars.terr.codeRateHP) )
                {
                    tpItem.pars.terr.codeRateHP = tunePars.terr.codeRateHP;
                    changed = 1;
                }

                if ( (tunePars.terr.codeRateLP != FE_FEC_NONE) &&
                     (tunePars.terr.codeRateLP != FE_FEC_AUTO) &&
                     (tunePars.terr.codeRateLP != tpItem.pars.terr.codeRateLP) )
                {
                    tpItem.pars.terr.codeRateLP = tunePars.terr.codeRateLP;
                    changed = 1;
                }

                if ( (tunePars.terr.modulation != FE_MOD_NONE) &&
                     (tunePars.terr.modulation != FE_MOD_AUTO) &&
                     (tunePars.terr.modulation != tpItem.pars.terr.modulation) )
                {
                    tpItem.pars.terr.modulation = tunePars.terr.modulation;
                    changed = 1;
                }

                if ( (tunePars.terr.transmission != FE_TRANS_MODE_NONE) &&
                     (tunePars.terr.transmission != FE_TRANS_MODE_AUTO) &&
                     (tunePars.terr.transmission != tpItem.pars.terr.transmission) )
                {
                    tpItem.pars.terr.transmission = tunePars.terr.transmission;
                    changed = 1;
                }

                if ( (tunePars.terr.guardInterval != FE_GUARD_INTERVAL_NONE) &&
                     (tunePars.terr.guardInterval != FE_GUARD_INTERVAL_AUTO) &&
                     (tunePars.terr.guardInterval != tpItem.pars.terr.guardInterval) )
                {
                    tpItem.pars.terr.guardInterval = tunePars.terr.guardInterval;
                    changed = 1;
                }

                if ( (tunePars.terr.hierarchy != FE_HIERARCHY_NONE) &&
                     (tunePars.terr.hierarchy != FE_HIERARCHY_AUTO) &&
                     (tunePars.terr.hierarchy != tpItem.pars.terr.hierarchy) )
                {
                    tpItem.pars.terr.hierarchy = tunePars.terr.hierarchy;
                    changed = 1;
                }

                if ( (tunePars.terr.iq != FE_IQ_NONE) &&
                     (tunePars.terr.iq != FE_IQ_AUTO) &&
                     (tunePars.terr.iq != tpItem.pars.terr.iq) )
                {
                    tpItem.pars.terr.iq = tunePars.terr.iq;
                    changed = 1;
                }
                break;

            #endif /* APPL_FETERR_ENABLED */

            #ifdef APPL_FECABLE_ENABLED

            case FE_SYSTEM_TYPE_CABLE:
                if ( (tunePars.cable.modulation != FE_MOD_NONE) &&
                     (tunePars.cable.modulation != FE_MOD_AUTO) &&
                     (tunePars.cable.modulation != tpItem.pars.cable.modulation) )
                {
                    tpItem.pars.cable.modulation = tunePars.cable.modulation;
                    changed = 1;
                }

                if ( (tunePars.cable.iq != FE_IQ_NONE) &&
                     (tunePars.cable.iq != FE_IQ_AUTO) &&
                     (tunePars.cable.iq != tpItem.pars.cable.iq) )
                {
                    tpItem.pars.cable.iq = tunePars.cable.iq;
                    changed = 1;
                }
                break;

            #endif /* APPL_FECABLE_ENABLED */

            #ifdef APPL_FEDIRECT_ENABLED

            case FE_SYSTEM_TYPE_DIRECT:
                changed = 0;
                break;

            #endif /* APPL_FECABLE_ENABLED */

            default:
                FAPI_SYS_PRINT_DEBUG(2,"Invalid frontend type: %i\n", pFe->systemType);
               break;
        }
    }

    if ( changed )
    {
        TPLIST_SetDataByIndex (network, carrierIdx, &tpItem, SYS_OWNER_SYSTEM);
    }
}

int32_t AVINT_SetTsInput (uint8_t tsd, RSRC_TS_E ts)
{
    FAPI_SYS_PRINT_MSG("[AV] Come to AVINT_SetTsInput 1...\n");

#ifndef _LINUX_FAPI
    FAPI_TSD_TsInputSettingsT    tsInputSettings;
#else
    FAPEX_TSD_OpenParamStrT     tsdOpenParam;
    FAPEX_TSD_InputParamStrT    tsInputSettings;
#endif
    int32_t                     retVal = FAPI_OK;
    uint32_t                    useStc;
    uint32_t                    tsSelect;

    FAPI_SYS_PRINT_DEBUG(1,"[AV][AVINT_SetTsInput] enter, tsd = %d, input_ts = %d\n", tsd, ts);

    if ( (ts  >= RSRC_TS_NUMS)
      || (tsd >= RSRC_TSD_NUMS) )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
#ifndef _LINUX_FAPI
        memset (&tsInputSettings, 0, sizeof(tsInputSettings));
#else
        memset (&tsdOpenParam, 0, sizeof(FAPEX_TSD_OpenParamStrT));
        memset (&tsInputSettings, 0, sizeof(FAPEX_TSD_InputParamStrT));
#endif
        switch ( ts )
        {
            case RSRC_TS_A:     tsSelect = FAPEX_TSD_INPUT_CH_A;       break;
            case RSRC_TS_B:     tsSelect = FAPEX_TSD_INPUT_CH_B;       break;
            case RSRC_TS_NONE:
            default:            tsSelect = FAPEX_TS_DISABLE; break;
        }

        if ( tsd == RSRC_TSD_LIVE )
        {
            useStc = 1;
        }
        else
        {
            useStc = 0;
        }

#ifndef _LINUX_FAPI
        tsInputSettings.version             = FAPEX_TSD_VERSION;
        tsInputSettings.automaticSync       = 0;
        tsInputSettings.packetStartPolarity = 0;
        tsInputSettings.enablePolarity      = 0;
        tsInputSettings.clockInvert         = 0;
        tsInputSettings.serialParallelMode  = 0x3;
        tsInputSettings.sbz                 = 0;
        tsInputSettings.tsSelect            = tsSelect;
        tsInputSettings.syncByteDistance    = 0;

        retVal = FAPI_TSD_Configure(tsd, useStc, &tsInputSettings, NULL);
#endif

        if ( fd_TSD[tsd]<0 )   //open TSD first time
        {
            /* input settings */
            tsdOpenParam.useStc           = useStc;
            tsdOpenParam.tsSelect         = tsSelect;
            tsdOpenParam.syncByteDistance = 0;
            tsdOpenParam.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
            tsdOpenParam.oflags           = 0;
            fd_TSD[tsd] = FAPEX_TSD_Open( FAPEX_TSD0 + tsd, O_RDWR, &tsdOpenParam );
            if(fd_TSD[tsd] < 0)
            {
                retVal = -1;
            }
            FAPI_SYS_PRINT_DEBUG(1,"[AV] Come to AVINT_SetTsInput FAPEX_TSD_Open...\n");
        }
        else//change TSD inupt
        {
            tsInputSettings.iflags              = FAPEX_TSD_IFLAGS_SP_MODE3;
            tsInputSettings.tsSelect            = tsSelect;
            tsInputSettings.syncByteDistance    = 0;
            retVal = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[tsd],
                                      (FAPEXIOCARG2_T)FAPEX_IOCCOM_TSD_SET_INPUT,
                                      (FAPEXIOCARG3_T)&tsInputSettings );
            FAPI_SYS_PRINT_DEBUG(1,"[AV] Come to AVINT_SetTsInput FAPEX_TSD_Ioctl...\n");

        }


    }

    if ( retVal != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(1,"[AV] AVINT_SetTsInput failed: %i\n", retVal);
    }

    return retVal;
}

FSTATIC int32_t AVINT_StartWelcomeJingle (void)
{
    const SYS_CONFIGDAT_S*     sysCfg = SYS_GetConfig();
    int32_t                    retVal = FAPI_OK;

    if ( sysCfg->welcomeJingle == NULL )
    {
        return FAPI_OK;
    }
#if 0 // TODO
    do
    {
        retVal = FD_AUD_OpenMemory();
        if ( retVal != FAPI_OK ) break;

        if ( FD_AUD_FeedDataBytes (sysCfg->welcomeJingle,
                                   sysCfg->welcomeJingleSize, 1) == 0 )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        retVal = FD_AUD_FlushData();
        if ( retVal != FAPI_OK ) break;

    } while (0);
#endif
    return retVal;
}

FSTATIC void AVINT_StartTuneMode (AV_FEDAT_S* pFe, AV_CMD_S* pCmd)
{
    #ifdef APPL_CI_ENABLED
    CISYS_SetInput(&(pCmd->arg.tune.cfg));
    #endif /* APPL_CI_ENABLED */

    /* set tune mode parameters */
    pFe->reqTunePar.network    = pCmd->arg.tune.network;
    pFe->reqTunePar.tpIdx      = pCmd->arg.tune.tpIdx;

    if ( pCmd->arg.tune.tpIdx < 0 )
    {
        pFe->reqTunePar.manualMode = FTRUE;

        AVINT_TuneParamsGenerate (pFe, &(pCmd->arg.tune.tunePars),
                                  pFe->reqTunePar.network.satIdx,
                                  &(pFe->reqTunePar.par));
    }
    else
    {
        pFe->reqTunePar.manualMode = FFALSE;

        memset(&(pFe->reqTunePar.par), 0, sizeof(FE_TUNE_PARAMS_S));
    }

    pFe->forceTuning = FTRUE;
    pFe->state       = AV_FESTATE_NORMAL;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
