/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/av_live.c
**
**              $Id: av_live.c 5925 2010-03-05 06:43:50Z jmorha $
**
** \brief       Live-TV specific extensions for AV-Module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 506: Constant value Boolean [MISRA 2004 Rules 13.7 and 14.1]
**                do ... while(1);
** - Warning 539: Did not expect positive indentation from
**                "if ( avData->itcInst.abortReq )" this is set externaly.
** - Info 715:    Symbol not referenced.
**
*****************************************************************************
*/
/*lint -save -e506 -e539 -e715 */
#define DEBUG 1

//#define PTS_SEARCH_DEBUG1 1
//#define PTS_SEARCH_DEBUG2 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "fapex/abst_types.h"
#include "val/val.h"

#include "fapex/abst_libtsd.h"
#include "fapex/abst_libvidec.h"
#include "sys_priv.h"
#include "utils.h"

#include "init.h"
#include "av.h"
#include "eit.h"
#include "key_input.h"
#include "teletext.h"
#include "settings.h"

#ifdef APPL_SUBTT_ENABLED
#include "subtitle.h"
#endif /* APPL_SUBTT_ENABLED */

/*---------------------------------------------------------------------------*/
/* Global Data                                                               */
/*---------------------------------------------------------------------------*/
extern int32_t fd_TSD[];

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/



/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/* add by Jim.Kang. 2010.7.19 */
FSTATIC int32_t fd_TSD0_PCR = -1;


/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void         AVINT_LiveTvPmtUpdate(void);
FSTATIC void         AVINT_LiveTvSDTHandling(void);
FSTATIC void         AVINT_LiveTvNITHandling(AV_FEDAT_S* pFe);
FSTATIC void         AVINT_FPLiveSet(const SYS_SERVLOC_S * servLoc,
                                    const SLIST_ITEM_S * service);
FSTATIC void         AVINT_FPLiveClear(void);

/* add by Jim.Kang. 2010.7.19 */
FSTATIC int32_t pcrTsdOpen(void);

/* ------------------------------------------------------------------------- */

int32_t AVINT_LiveTvInit(void)
{
    uint32_t cnt;

    memset(&(avData->liveTv), 0, sizeof(avData->liveTv));

    avData->liveTv.pFe                      = NULL;
    avData->liveTv.streamsStarted           = FFALSE;
    avData->liveTv.videoFreezed             = FFALSE;
    avData->liveTv.serviceDecoded           = FFALSE;
    avData->liveTv.haveVideoStream          = FFALSE;
    avData->liveTv.timeoutSent              = FFALSE;
    avData->liveTv.radioScreenVisible       = FFALSE;

    avData->liveTv.currServLoc.listId       = SLIST_NONE;
    avData->liveTv.currServLoc.index        = -1;
    avData->liveTv.currServIdx              = -1;

    avData->liveTv.reqServLoc               = avData->liveTv.currServLoc;
    avData->liveTv.selServLoc               = avData->liveTv.currServLoc;
    avData->liveTv.prevServLoc              = avData->liveTv.currServLoc;
    avData->liveTv.swapServLoc              = avData->liveTv.currServLoc;
    avData->liveTv.swapFlag                 = FFALSE;

    avData->liveTv.selectedAudioStream      = -1;
    avData->liveTv.currentAudioStream       = -1;
    avData->liveTv.selectedSpdifStream      = -1;
    avData->liveTv.currentSpdifStream       = -1;

    for ( cnt = 0; cnt < SLIST_GLOB_NUMLISTS; cnt++ )
    {
        avData->liveTv.prevIdxGlob[cnt] = -1;
    }

    for ( cnt = 0; cnt < SLIST_USR_NUMLISTS; cnt++ )
    {
        avData->liveTv.prevIdxUsr[cnt] = -1;
    }

    /* add by Jim.Kang. 2010.7.19 */
    pcrTsdOpen();

    return FAPI_OK;
}

int32_t AVINT_LiveTvSelectService(const SYS_SERVLOC_S * servLoc,
                                  const RSRC_CFG_S * cfg,
                                  const SYS_PLAYPREF_S* pref,
                                  uint32_t ctrlInfo)
{
    AV_LIVETV_S        *pLive = &(avData->liveTv);
    int32_t             retVal;
    SLIST_ITEM_S        newService;
    TPLIST_ITEM_S       tpItem;

    for(;;)
    {
        retVal = SLIST_EntryGet(servLoc, &newService);
        if(retVal != FAPI_OK)
            break;

        retVal = TPLIST_GetDataByIndex(&(newService.network), newService.tpIdx,
                                       &tpItem);
        if(retVal != FAPI_OK)
            break;

        pLive->cfg = *cfg;
        pLive->pFe = avData->feDat + pLive->cfg.feIdx;

        if(ctrlInfo == SERVICE_CTRL_NORMAL)
        {
            AVINT_FPLiveSet(servLoc, &newService);
        }
        AVINT_RouteSignal(&(pLive->cfg));

        (void)PSISI_SetTsdUsage (RSRC_OP_LIVE, pLive->cfg.tsdIdx,
                                 &(pLive->pFe->pat));
        PSISI_SDT_Invalidate    (&(avData->sdt));

        #ifdef APPL_CA_ENABLED
        PSISI_CAT_Invalidate (pLive->cfg.tsdIdx);
        #endif /* APPL_CA_ENABLED */

        #ifdef APPL_PARENTAL_ENABLED
        PCTRL_LiveStartHandling(servLoc, &newService);
        #endif /* APPL_PARENTAL_ENABLED */
        /* ----- initialise some live-TV parameters ------------------------ */

        if ( ctrlInfo == SERVICE_CTRL_NORMAL )
        {
            /* invalidate previous service's stream selections */
            pLive->selectedAudioStream = -1;
            pLive->selectedSpdifStream = -1;

            /* update service selection history parameters */

            if((servLoc->listId != pLive->selServLoc.listId) ||
               (servLoc->index != pLive->selServLoc.index))
            {
                /* Store previous service location */
                pLive->prevServLoc = pLive->selServLoc;

                /* Store previous index for each service list available */

                if(pLive->selServLoc.listId == SLIST_NONE)
                {
                    /* no service has been selected before, do nothing */
                }
                else if(pLive->selServLoc.listId >= SLIST_OFFSET_GLOB)
                {
                    if((uint16_t) (pLive->selServLoc.listId & SLIST_MASK_GLOB) <
                       SLIST_GLOB_NUMLISTS)
                    {
                        pLive->prevIdxGlob[pLive->selServLoc.
                                           listId & SLIST_MASK_GLOB] =
                            pLive->selServLoc.index;
                    }
                }
                else
                {
                    if(pLive->selServLoc.listId < SLIST_USR_NUMLISTS)
                    {
                        pLive->prevIdxUsr[pLive->selServLoc.listId] =
                            pLive->selServLoc.index;
                    }
                }
            }
        }

        /* store requested service's parameters */
        pLive->currServIdx = SLIST_GetServiceIndex(servLoc);
        pLive->currServLoc = *servLoc;
        pLive->selServLoc = *servLoc;
        pLive->pref = *pref;
        pLive->currentService = newService;

        pLive->pFe->reqTunePar.network    = newService.network;
        pLive->pFe->reqTunePar.manualMode = FFALSE;
        pLive->pFe->reqTunePar.par        = tpItem.pars;
        pLive->pFe->reqTunePar.tpIdx      = newService.tpIdx;

        pLive->pFe->useStatus |= AV_USESTATUS_LIVETV;
        pLive->pFe->state = AV_FESTATE_NORMAL;

        pLive->currVersionPat = PSISI_VERSION_ANY;
        pLive->currVersionPmt = PSISI_VERSION_ANY;
        pLive->currVersionSdt = PSISI_VERSION_ANY;
        pLive->currVersionNit = PSISI_VERSION_ANY;
        pLive->currRunstat = SYS_RUNSTAT_UNKNOWN;

        #ifdef APPL_CI_ENABLED

        if(pLive->cfg.useCi)
        {
            CISYS_ServiceSelected(pLive->currServIdx, pLive->cfg.ts);
        }

        #endif /* APPL_CI_ENABLED */

        (void)AVINT_AudioOpen();

        /* reset subtitle user selection */
        AVINT_SubtResetUserSelect();
        break; //end of for(;;)
    }

    return retVal;
}

int32_t AVINT_LiveTvRestart(const SYS_SERVLOC_S * servLoc,
                            const RSRC_CFG_S * cfg)
{
    AV_LIVETV_S        *pLive = &(avData->liveTv);
    int32_t             retVal;
    SLIST_ITEM_S        newService;
    TPLIST_ITEM_S       tpItem;

    for(;;)
    {
        retVal = SLIST_EntryGet(servLoc, &newService);
        if(retVal != FAPI_OK)
            break;

        retVal = TPLIST_GetDataByIndex(&(newService.network), newService.tpIdx,
                                       &tpItem);
        if(retVal != FAPI_OK)
            break;

        pLive->cfg = *cfg;
        pLive->pFe = avData->feDat + pLive->cfg.feIdx;

        AVINT_FPLiveSet(servLoc, &newService);

        AVINT_RouteSignal(&(pLive->cfg));

        (void)PSISI_SetTsdUsage (RSRC_OP_LIVE, pLive->cfg.tsdIdx,
                                 &(pLive->pFe->pat));
        PSISI_SDT_Invalidate    (&(avData->sdt));

        #ifdef APPL_CA_ENABLED
        PSISI_CAT_Invalidate (pLive->cfg.tsdIdx);
        #endif /* APPL_CA_ENABLED */

        #ifdef APPL_PARENTAL_ENABLED
        PCTRL_LiveStartHandling(servLoc, &newService);
        #endif /* APPL_PARENTAL_ENABLED */

        /* ----- initialise some live-TV parameters ------------------------ */

        /* store requested service's parameters */
        pLive->currServIdx = SLIST_GetServiceIndex(servLoc);
        pLive->currServLoc = *servLoc;
        pLive->selServLoc  = *servLoc;

        pLive->pref.spdifPref = FFALSE;
        pLive->pref.spdifMode = SPDIFMODE_PREFERPASSTHROUGH;
        pLive->pref.langAudio = LANG_NOTSPECIFIED;
        pLive->pref.langSpdif = LANG_NOTSPECIFIED;

        pLive->currentService = newService;

        pLive->pFe->reqTunePar.network    = newService.network;
        pLive->pFe->reqTunePar.manualMode = FFALSE;
        pLive->pFe->reqTunePar.par        = tpItem.pars;
        pLive->pFe->reqTunePar.tpIdx      = newService.tpIdx;

        pLive->pFe->useStatus |= AV_USESTATUS_LIVETV;
        pLive->pFe->state = AV_FESTATE_NORMAL;

        pLive->currVersionPat = PSISI_VERSION_ANY;
        pLive->currVersionPmt = PSISI_VERSION_ANY;
        pLive->currVersionSdt = PSISI_VERSION_ANY;
        pLive->currVersionNit = PSISI_VERSION_ANY;

        pLive->currRunstat = SYS_RUNSTAT_UNKNOWN;

        #ifdef APPL_CI_ENABLED

        if(pLive->cfg.useCi)
        {
            CISYS_ServiceSelected(pLive->currServIdx, pLive->cfg.ts);
        }

        #endif /* APPL_CI_ENABLED */

        (void)AVINT_AudioOpen();

        break; //end of for(;;)
    }

    return retVal;
}

int32_t AVINT_LiveTvStop (fbool_t freeze, fbool_t sendNotification,
                          fbool_t resetFocus, fbool_t keepRadioScreen,
                          uint32_t ctrlInfo)
{
    AV_LIVETV_S        *pLive = &(avData->liveTv);
    SYS_EVT_SERVICE_S   ctrlEvent;
    int32_t             retVal = FAPI_OK;

    ctrlEvent.servLoc  = pLive->currServLoc;
    ctrlEvent.audioStr = pLive->selectedAudioStream;

    if ( ctrlInfo == SERVICE_CTRL_NORMAL )
    {
        AVINT_FPLiveClear();
    }


    #ifdef APPL_SUBTT_ENABLED
    SUBTT_Clear  ();
    SUBTT_Enable (FTRUE, SYS_SUSPENDSRC_LIVE);
    #endif /* APPL_SUBTT_ENABLED */

    TTXSUBTT_Clear();

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_Enable (FTRUE, SYS_SUSPENDSRC_LIVE);
    #endif /* APPL_TTXT_ENABLED */

    #ifdef APPL_MHEG_ENABLED
    if ( (MHEG_ServiceGetFlags()& (uint32_t)ITK_TUNE_KEEP_CONTEXT) != 0)
    {
        (void)AVINT_StopStreams(freeze, keepRadioScreen, FFALSE, FFALSE);
    }
    else
    #endif
    {
        (void)AVINT_StopStreams(freeze, keepRadioScreen, FTRUE, FTRUE);
    }

    (void)PSISI_ReleaseTsdUsage  (RSRC_OP_LIVE, pLive->cfg.tsdIdx);
    (void)EIT_FilteringStop      ();
    PSISI_TDTTOT_StopFiltering   ();
    (void)PSISI_SDT_StopFiltering();

    #ifdef APPL_CA_ENABLED
    PSISI_CAT_StopFiltering (pLive->cfg.tsdIdx);
    #endif /* APPL_CA_ENABLED */

    #ifdef APPL_PARENTAL_ENABLED

    PCTRL_Stop();

    #endif /* APPL_PARENTAL_ENABLED */

    #ifdef APPL_CI_ENABLED

    if ( pLive->cfg.useCi )
    {
        CISYS_ServiceStop (pLive->currServIdx, pLive->cfg.ts);
    }

    #endif /* APPL_CI_ENABLED */

    pLive->currServLoc.listId = SLIST_NONE;
    pLive->currServLoc.index = -1;
    pLive->currServIdx = -1;

    memset(&(pLive->currentService), 0, sizeof(SLIST_ITEM_S));

    if(pLive->pFe != NULL)
    {
        pLive->pFe->useStatus &= (~AV_USESTATUS_LIVETV);

        if(pLive->pFe->useStatus == 0)
        {
            pLive->pFe->state = AV_FESTATE_IDLE;
        }

        pLive->pFe = NULL;
    }

    #ifdef APPL_MHEG_ENABLED

    avData->dsmccPatReq = FFALSE;
    avData->dsmccPmtReq = FFALSE;
    avData->dsmccLastPmtVersion = 0xFF;

    #endif /* APPL_MHEG_ENABLED */
    #ifdef APPL_MHEG_ENABLED
    if ( (MHEG_ServiceGetFlags()& (uint32_t)ITK_TUNE_KEEP_CONTEXT) == 0)
    #endif
    {
        (void)AVINT_AudioClose();
    }

    if ( sendNotification )
    {
        ctrlEvent.hdr.type = EVT_SERVICE_SELECTED;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.status = SERVICE_STAT_STOPPED;

        /* Notify stopping of service */
        retVal = EVT_Send(&ctrlEvent, "Service stop");
    }

    return retVal;
}

int32_t AVINT_LiveTvSelAudioStream(int8_t index)
{
    int32_t             retVal = FAPI_OK;
    SYS_EVT_AUDIO_S     ctrlEvent;
    SYS_STR_AUDIO_S*    streamPtr;

    if((index < 0) ||
       (index >= (int8_t)(avData->liveTv.currentService.strPid.audio.
                          numStreams)))
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        if ( avData->liveTv.streamsStarted )
        {
            if ( avData->liveTv.currentAudioStream >= 0 )
            {
                /* stop current audio stream */
                AVINT_AudioStop ();
            }

            streamPtr = avData->liveTv.currentService.strPid.audio.stream
                        + index;

            (void)AVINT_AudioStart(streamPtr->pid, streamPtr->format,
                                   streamPtr->channel,
                                   AVINT_LiveTvAudioStartNotify);

            avData->liveTv.currentAudioStream = index;
        }

        avData->liveTv.selectedAudioStream = index;

        ctrlEvent.hdr.type   = EVT_AUDIOSTREAM_CHANGED;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.newIdx     = index;
        ctrlEvent.state      = SYS_PRESSTATE_LIVETV;

        retVal = EVT_Send(&ctrlEvent, "Audio select");
    }

    return retVal;
}

int32_t AVINT_LiveTvSelSpdifStream(AV_SPDIF_STATUS_E type, int8_t index)
{
    SLIST_ITEM_S*           pService = &(avData->liveTv.currentService);
    SYS_STREAMINF_AUDIO_S*  pAudio   = &(pService->strPid.audio);
    int32_t                 retVal   = FAPI_OK;
    SYS_EVT_SPDIF_S         ctrlEvent;

    ctrlEvent.hdr.type   = EVT_SPDIFSTREAM_CHANGED;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);
    ctrlEvent.state      = SYS_PRESSTATE_LIVETV;

    switch ( type )
    {
        default:
        case SPDIFSTATUS_CURRPCM:
            if ( avData->liveTv.currentSpdifStream >= 0 )
            {
                retVal = AVINT_AudioSPDIFSetStatus (type, 0, SYS_AUDIO_NUMS);
                // TODO: add error handling?

                avData->liveTv.currentSpdifStream = -1;
            }

            avData->liveTv.selectedSpdifStream = -1;

            ctrlEvent.newIdx = -1;

            (void)EVT_Send(&ctrlEvent, "SPDIF select");
            break;

        case SPDIFSTATUS_PASSTHROUGH:

            if ( index < 0 )
            {
                /* choose first compliant stream in list */
                index = AVINT_AudioSPDIFFindFirst (pAudio);
            }

            if ( (index < 0)
              || (index >= (int8_t)(pAudio->numStreams))
              || (!(AVINT_AudioIsSPDIFCompliant(pAudio->stream[index].format)))
                 )
            {
                retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            }
            else
            {
                if ( avData->liveTv.streamsStarted )
                {
                    /* select new SPDIF stream */
                    retVal = AVINT_AudioSPDIFSetStatus (type,
                                                pAudio->stream[index].pid,
                                                pAudio->stream[index].format);
                    // TODO: add error handling?

                    avData->liveTv.currentSpdifStream = index;
                }

                avData->liveTv.selectedSpdifStream = index;

                ctrlEvent.newIdx = index;

                (void)EVT_Send(&ctrlEvent, "SPDIF select");
            }
            break;
    }

    return retVal;
}

void AVINT_LiveTvSignalAvailable(AV_FEDAT_S * pFe)
{
    AV_LIVETV_S        *pLive = &(avData->liveTv);
    SYS_EVT_SERVICE_S   ctrlEvent;
    int32_t             retVal;
    fbool_t             handleVideo = FTRUE;
    fbool_t             handleAudio = FTRUE;

    #ifdef APPL_CA_ENABLED
    PSISI_SECT_NOTIFY_CB sectNotify = NULL;
    #endif /* APPL_CA_ENABLED */

    if((!(pLive->streamsStarted)) && (!(SYS_GetConfig()->avPmtControlled)))
    {
        #ifdef APPL_MHEG_ENABLED

        /* Don't handle video Start/Stop if the video is currently freeze or
           an I-Frame is shown by MHEG-5 Engine */
        if(avData->liveTv.videoFreezed || MHEG_GetVideoState())
        {
             handleVideo = FFALSE;
        }
        handleAudio = (fbool_t)!(MHEG_GetAudioState());

        #endif /* APPL_MHEG_ENABLED */

        retVal = AVINT_StartStreams(&(pLive->currentService),
                                    handleVideo, handleAudio);
        if(retVal == FAPI_OK)
        {
            /* Generate event to notify success of service selection */
            ctrlEvent.hdr.type   = EVT_SERVICE_SELECTED;
            ctrlEvent.hdr.length = sizeof(ctrlEvent);
            ctrlEvent.servLoc    = pLive->currServLoc;
            ctrlEvent.audioStr   = pLive->selectedAudioStream;
            ctrlEvent.status     = SERVICE_STAT_SUCCESS;

            (void)EVT_Send(&ctrlEvent, "Service select");
        }
    }

    #ifdef APPL_SUBTT_ENABLED
    SUBTT_Enable (FTRUE, SYS_SUSPENDSRC_LIVE);
    #endif /* APPL_SUBTT_ENABLED */

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_Enable (FTRUE, SYS_SUSPENDSRC_LIVE);
    #endif /* APPL_TTXT_ENABLED */

    /* ----- Open SI-filters ----------------------------------------------- */

    // Martin Comment
    //(void)PSISI_TDTTOT_StartFiltering();
    (void)EIT_FilteringStart(pLive->currServIdx, pLive->currentService.serviceId);
    //(void)PSISI_NIT_StartFiltering(pLive->currVersionNit, FFALSE, 0);
    //(void)PSISI_SDT_StartFiltering(pLive->currVersionSdt, FFALSE);

    #ifdef APPL_CA_ENABLED

    #ifdef APPL_CONAX_ENABLED

    sectNotify = CONAX_CA_CATSectionNotify;

    #endif /* APPL_CONAX_ENABLED */

    PSISI_CAT_StartFiltering (pLive->cfg.tsdIdx, PSISI_VERSION_ANY, FFALSE,
                              FTRUE, sectNotify);

    #endif /* APPL_CA_ENABLED */
}

void AVINT_LiveTvSignalLost(AV_FEDAT_S * pFe)
{
    AV_LIVETV_S        *pLive = &(avData->liveTv);
    fbool_t        handleVideo = FTRUE;
    fbool_t        handleAudio = FTRUE;

    /* ----- Close SI filters ---------------------------------------------- */

    (void)EIT_FilteringStop();
    PSISI_TDTTOT_StopFiltering();
    (void)PSISI_SDT_StopFiltering();

    #ifdef APPL_CA_ENABLED
    PSISI_CAT_StopFiltering (pLive->cfg.tsdIdx);
    #endif /* APPL_CA_ENABLED */

    (void)PSISI_PMT_StopFiltering(pLive->cfg.tsdIdx);

    /* Trigger new filtering of PAT/PMT after signal is available again,
       tables might have been updated during non-signal phase. */
    pLive->currVersionPat = PSISI_VERSION_ANY;

    /* --------------------------------------------------------------------- */

    #ifdef APPL_SUBTT_ENABLED
    SUBTT_Enable (FFALSE, SYS_SUSPENDSRC_LIVE);
    #endif /* APPL_SUBTT_ENABLED */

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_Enable (FFALSE, SYS_SUSPENDSRC_LIVE);
    #endif /* APPL_TTXT_ENABLED */

    #ifdef APPL_MHEG_ENABLED
    /* Don't handle video Start/Stop if the video is currently freeze or
       an I-Frame is shown by MHEG-5 Engine */
     if(avData->liveTv.videoFreezed || MHEG_GetVideoState())
     {
         handleVideo = FFALSE;
     }
     handleAudio = (fbool_t)!(MHEG_GetAudioState());

    #endif /* APPL_MHEG_ENABLED */

    if(pLive->streamsStarted)
    {
        (void)AVINT_StopStreams (FFALSE, FTRUE, handleVideo, handleAudio);
    }
}

void AVINT_LiveTvTasks(AV_FEDAT_S * pFe)
{
    AV_LIVETV_S*        pLive       = &(avData->liveTv);
    const PSISI_PAT_S*  pPat        = PSISI_PAT_GetCurrent(pLive->cfg.tsdIdx);
    const PSISI_PMT_S*  pPmt        = PSISI_PMT_GetCurrent(pLive->cfg.tsdIdx);
    fbool_t             handleVideo = FTRUE;
    fbool_t             handleAudio = FTRUE;
    uint16_t            pmtPid;
    uint32_t            currTime;
    SYS_EVTHDR_S        ctrlEvent;

    #ifdef APPL_MHEG_ENABLED
    PSISI_PAT_BUF_S patBuf;
    uint16_t        pmtPrgNum;
    #endif /* APPL_MHEG_ENABLED */

    if ( avData->itcInst.abortReq )
    {
        return;
    }

    (void)AVINT_AudioTask ();
    (void)AVINT_VideoTask ();

    /* ----- Handle "start-of-decoding" timeout ---------------------------- */

    if((pLive->streamsStarted)    &&
       (!(pLive->timeoutSent))    &&
       (!(pLive->serviceDecoded)) &&
       (TIME_GetSystemTime(&currTime) == FAPI_OK))
    {
        if(TIME_GetSystemTimeInterval(pLive->streamsStartTime, currTime) >=
           AV_SERVICE_DECODING_TIMEOUT)
        {
            /* Decoding timeout: send notification event. */
            ctrlEvent.type   = EVT_SERVICE_NO_DECODING;
            ctrlEvent.length = sizeof(ctrlEvent);

            (void)EVT_Send(&ctrlEvent, "Decoding timeout");

            pLive->timeoutSent = FTRUE;
        }
    }

    if ( avData->itcInst.abortReq )
    {
        return;
    }
    /* --------------------------------------------------------------------- */
    /* handle video/audio sync failure */
    /* --------------------------------------------------------------------- */

    #ifdef APPL_MHEG_ENABLED

    if( (avData->dsmccPatReq) && (pPat->sect.allAvailable))
    {
        if(PSISI_PAT_GetCurrentBuf(&patBuf,0) == FTRUE)
        {
            MHEG_PAT_Section_Notify(patBuf.fullData,((uint8_t)patBuf.numSect-1), patBuf.fullSize, patBuf.avl);
            avData->dsmccPatReq = FFALSE;
        }
    }
    if ( avData->itcInst.abortReq )
    {
        return;
    }
    if ( (avData->liveTv.radioScreenVisible == FTRUE) &&
       (MHEG_GetStatus() != MHEG_STATE_STOPPED))
    {
        (void)AVINT_VideoStop(FFALSE);

        avData->liveTv.radioScreenVisible = FFALSE;
    }
    /* ----- Handle MHEG PMT filtering ------------------------------------- */
    if(avData->dsmccPmtReq)
    {
        if((pPmt->available) &&
           (pPmt->pid == avData->dsmccPmtPid) &&
           (pPmt->program_number == avData->dsmccPmtPrgNum))
        {
            /* pass PMT to MHEG engine */
            avData->dsmccPmtReq         = FFALSE;
            MHEG_PMT_Section_Notify(pPmt->buf, pPmt->bufSize);
            /* resume live PMT handling */
            avData->dsmccLastPmtVersion = pPmt->version_number;
            /* reset live PAT/PMT filtering */
            pLive->currVersionPat = PSISI_VERSION_ANY;

            (void)PSISI_PMT_Invalidate (pLive->cfg.tsdIdx);
        }
        else
        {
            /* Set PMT filter to requested PMT if necessary */
            if ( (PSISI_PMT_GetFilterParams(pLive->cfg.tsdIdx, &pmtPid,
                                            &pmtPrgNum) != FAPI_OK) ||
                 (pmtPrgNum != avData->dsmccPmtPrgNum)              ||
                 (pmtPid    != avData->dsmccPmtPid) )
            {
                (void)PSISI_PMT_StopFiltering (pLive->cfg.tsdIdx);
                (void)PSISI_PMT_Invalidate    (pLive->cfg.tsdIdx);
                (void)PSISI_PMT_StartFiltering(pLive->cfg.tsdIdx,
                                               avData->dsmccPmtPrgNum,
                                               avData->dsmccPmtPid,
                                               PSISI_VERSION_ANY, FTRUE);
            }
        }
    }
    else
    #endif /* APPL_MHEG_ENABLED */
    {
        /* ----- Handle live PMT filtering ------------------------------------- */

        if((pPat != NULL) && (pPat->sect.allAvailable))
        {
            /* PAT update */
            if(pPat->version_number != pLive->currVersionPat)
            {
                FAPI_SYS_PRINT_DEBUG(4,"[LIVE] PAT RECEIVED: ver %i \n",
                           pPat->version_number);

                pLive->currVersionPat = pPat->version_number;
                pLive->currVersionPmt = PSISI_VERSION_ANY;

                /* Invalidate current PMT */
                (void)PSISI_PMT_Invalidate (pLive->cfg.tsdIdx);

                /* Get PMT PID from PAT */
                if(PSISI_PAT_GetPmtPid(pPat,
                                       pLive->currentService.serviceId,
                                       &pmtPid) == FAPI_OK)
                {
                    (void)PSISI_PMT_StartFiltering(pLive->cfg.tsdIdx,
                                                   pLive->currentService.
                                                   serviceId,
                                                   pmtPid, PSISI_VERSION_ANY, FTRUE);
                }
                else
                {
                    /* No PMT reference available for current service. */

                    if((pLive->streamsStarted) &&
                       (SYS_GetConfig()->avPmtControlled))
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[LIVE] No PMT available, stop streams \n");

                        #ifdef APPL_MHEG_ENABLED

                        if(avData->liveTv.videoFreezed || MHEG_GetVideoState())
                        {
                            handleVideo = FFALSE;
                        }
                        handleAudio = (fbool_t)!(MHEG_GetAudioState());

                        #endif /* APPL_MHEG_ENABLED */

                        (void)AVINT_StopStreams (FFALSE, FTRUE, handleVideo, handleAudio);
                    }
                }
            }

            /* PMT update */
            if((pPmt->available) &&
               (pPmt->version_number != pLive->currVersionPmt))
            {
                FAPI_SYS_PRINT_DEBUG(4,"[LIVE] PMT RECEIVED: version %i \n",
                           pPmt->version_number);

                pLive->currVersionPmt = pPmt->version_number;

                #ifdef APPL_SUBTT_ENABLED
                (void)SUBTT_PmtUpdate(pPmt, NULL);
                #endif /* APPL_SUBTT_ENABLED */
                (void)TTXSUBTT_PmtUpdate(pPmt, NULL);
                AVINT_SubtPmtUpdate();

                /* PMT Section update is available */
                AVINT_LiveTvPmtUpdate();

                #ifdef APPL_MHEG_ENABLED
                MHEG_PMT_Section_Notify(pPmt->buf, pPmt->bufSize);
                #endif

                #ifdef APPL_CONAX_ENABLED

                CONAX_CA_PMTNotify (pLive->cfg.tsdIdx, pPmt->buf,
                                    pPmt->bufSize);

                #endif /* APPL_CONAX_ENABLED */

                #ifdef APPL_CI_ENABLED

                if(pLive->cfg.useCi)
                {
                    CISYS_PmtReceived(pLive->currServIdx, pLive->cfg.ts, pPmt);
                }

                #endif /* APPL_CI_ENABLED */

                #ifdef APPL_MHEG_ENABLED
                /* pass PMT to MHEG engine */
                avData->dsmccPmtReq         = FFALSE;
                avData->dsmccLastPmtVersion = pPmt->version_number;

                #endif /* APPL_MHEG_ENABLED */

                (void)PSISI_PMT_NotifyReception (pLive->cfg.tsdIdx);

                /* re-start PMT filtering for non-equal version */
                (void)PSISI_PMT_StartFiltering (pLive->cfg.tsdIdx,
                                                pPmt->program_number,
                                                pPmt->pid, pPmt->version_number,
                                                FFALSE);
            }
            else
            {
                /* re-start PMT filtering for non-equal version */
                /* in case PMT filter is running nothing is done */
                (void)PSISI_PMT_StartFiltering (pLive->cfg.tsdIdx,
                                                pPmt->program_number,
                                                pPmt->pid, pPmt->version_number,
                                                FFALSE);
            }
        }
    }

    if(avData->itcInst.abortReq)
    {
        return;
    }

    /* Handle NIT updates */
    AVINT_LiveTvNITHandling(pLive->pFe);
    /* Handle SDT updates */
    AVINT_LiveTvSDTHandling();

    if(avData->itcInst.abortReq)
    {
        return;
    }
    /* --------------------------------------------------------------------- */

    #ifdef APPL_CI_ENABLED

    /* test if CI stack waits for PMT */
    if(pPmt->available)
    {
        if(pLive->cfg.useCi)
        {
            CISYS_Task(pLive->cfg.ts, pPmt);
        }
    }

    #endif /* APPL_CI_ENABLED */

    if(avData->itcInst.abortReq)
        return;

    /* --------------------------------------------------------------------- */

    #ifdef APPL_PARENTAL_ENABLED

    PCTRL_Task();

    #endif /* APPL_PARENTAL_ENABLED */
}

FSTATIC void AVINT_LiveTvPmtUpdate(void)
{
    AV_LIVETV_S*        pLive = &(avData->liveTv);
    SYS_STREAM_INFO_S   strPids;
    SYS_EVT_SERVICE_S   ctrlEvent;
    fbool_t             handleVideo = FTRUE;
    fbool_t             handleAudio = FTRUE;
    fbool_t             restart     = FFALSE;
    fbool_t             sendEvt     = FFALSE;
    int32_t             retVal;
#ifdef APPL_DTG_SIPSI_T2_ENABLED
    uint8_t             cnt;
    uint8_t             i;
    SYS_LANGCODE        prefLang;
#endif

    /* --------------------------------------------------------------------- */

    /* get stream information from PMT */
    (void)PSISI_PMT_GetStreamPIDs(pLive->cfg.tsdIdx, &strPids);

    /* refresh current service list parameters */
    (void) SLIST_GetServByIndex(pLive->currServIdx, &(pLive->currentService));

    #ifdef APPL_MHEG_ENABLED
    if( strPids.mhegPresent == 1 )
    {
        if ( MHEG_IsOpened() == FFALSE )
        {
            (void) MHEG_Open();// send even to open and start MHEG-5

        }
        else
        {
            if(MHEG_IsStarted() == FFALSE)
            {
                FAPI_SYS_PRINT_DEBUG(2,"[MHEG] started by PMT update \n");
                (void) MHEG_Start();
            }
        }
    }
    else
    {
        if ( MHEG_IsOpened() == FTRUE )
        {
            FAPI_SYS_PRINT_DEBUG(2,"[MHEG] stopped by PMT update \n");
            (void) MHEG_Stop();
            (void) MHEG_Close();
        }
    }
    if(avData->liveTv.videoFreezed || MHEG_GetVideoState())
    {
        handleVideo = FFALSE;
    }
    handleAudio = (fbool_t)!(MHEG_GetAudioState());

    #endif /* APPL_MHEG_ENABLED */

    if ( pLive->streamsStarted )
    {
        /* test for video PID changes */
        if ( strPids.video != pLive->currentService.strPid.video )
        {
            restart = FTRUE;
        }
        /* test for audio PID changes */
        else if ( (pLive->selectedAudioStream >= 0) &&
                  (strPids.audio.stream[pLive->selectedAudioStream].pid !=
                   pLive->currentService.strPid.audio.
                   stream[pLive->selectedAudioStream].pid))
        {
            restart = FTRUE;
        }

        /* test for changes in video stream format */
        else if ( strPids.videoType != pLive->currentService.strPid.videoType )
        {
            restart = FTRUE;
        }
        /* test for changes in audio stream format */
        else if ( (pLive->selectedAudioStream >= 0) &&
                  (strPids.audio.stream[pLive->selectedAudioStream].format !=
                   pLive->currentService.strPid.audio.
                   stream[pLive->selectedAudioStream].format) )
        {
            restart = FTRUE;
        }

    #ifdef APPL_DTG_SIPSI_T2_ENABLED
        else
        {
            //If the audio is playing is not prefer language audio, and there have prefer language audio, we should play prefer audio
            for ( cnt = 0; cnt < SETTINGS_AUDIOLANGPREF_MAX; cnt++ )
            {
                (void) SETTINGS_AudioLangPrefGet(cnt, &prefLang);
                if(prefLang != pLive->currentService.strPid.audio.stream[pLive->selectedAudioStream].langCode)
                {
                    for(i=0;i<strPids.audio.numStreams;i++)
                    {
                        if(prefLang == strPids.audio.stream[i].langCode)
                        {
                            restart = FTRUE;
                            break;
                        }
                    }
                }
                else
                {
                    break;
                }

                if ( restart )
                {
                    break;
                }
            }
        }
    #endif/*#ifdef APPL_DTG_SIPSI_T2_ENABLED*/

        if ( restart )
        {
            FAPI_SYS_PRINT_DEBUG(1,"[LIVE] PMT update -> restart streams \n");
            if(strPids.video > 0)
            {
                (void)AVINT_StopStreams  (FTRUE, FFALSE, handleVideo, handleAudio);
            }
            else
            {
                (void)AVINT_StopStreams  (FFALSE, FFALSE, handleVideo, handleAudio);
            }
        }
    }
    else
    {
        /* streams not yet started, start now! */
        restart = FTRUE;
        sendEvt = FTRUE;
    }

    pLive->currentService.strPid = strPids;     /* update PIDs */

    if ( restart )
    {
        retVal = AVINT_StartStreams (&(pLive->currentService), handleVideo,
                                     handleAudio);

        if ( (retVal == FAPI_OK) && sendEvt )
        {
            /* Generate event to notify success of service selection */
            ctrlEvent.hdr.type   = EVT_SERVICE_SELECTED;
            ctrlEvent.hdr.length = sizeof(ctrlEvent);
            ctrlEvent.servLoc    = pLive->currServLoc;
            ctrlEvent.audioStr   = pLive->selectedAudioStream;
            ctrlEvent.status     = SERVICE_STAT_SUCCESS;

            (void)EVT_Send(&ctrlEvent, "Service select");
        }
    }

    /* update service list with latest PID values */
    (void)SLIST_SetServByIndex(pLive->currServIdx, &(pLive->currentService));
}

int32_t AVINT_StopStreams (fbool_t videoFreeze,
                                   fbool_t keepRadioScreen,
                                   fbool_t handleVideo,
                                   fbool_t handleAudio)
{
    AV_LIVETV_S* pLive = &(avData->liveTv);
    uint32_t enableStcRegulation;
    /*uint32_t pid;*/ // by Jim.Kang
    FAPI_SYS_PRINT_MSG("come to AVINT_StopStreams\n");

    if ( handleAudio )
    {
        AVINT_AudioStop ();

        (void)AVINT_AudioSPDIFSetStatus (SPDIFSTATUS_DISABLED, 0,
                                         SYS_AUDIO_NUMS);

        pLive->currentAudioStream = -1;
        pLive->currentSpdifStream = -1;
    }

    if ( handleVideo )
    {
        if ( (!keepRadioScreen)
          || (!(pLive->radioScreenVisible))
          || videoFreeze )
        {
            (void)AVINT_VideoStop(videoFreeze);

            pLive->radioScreenVisible = FFALSE;
        }
    } /* end handleVideo */

#ifndef _LINUX_FAPI
    (void)FAPI_SYNC_SetStcRegulation(0);
    (void)FAPI_SYNC_SetPcrPid(FAPI_SYNC_INVALIDATE);
#else
    enableStcRegulation = 0;
    /*(void)FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)VIDEO_GetViencHdId(),*/
    /* changed by Jim.Kang 2010.7.19 */
    AV_VIDEO_S* pVideo = &(avData->video);
    (void)FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)VAL_GetIdViencHd(pVideo->valDataPtr),
                             FAPEX_IOCCOM_SYNC_SETSTCREGULATION,
                             (FAPEXIOCARG3_T)&enableStcRegulation );
    /* 
    pid = FAPEX_SYNC_INVALIDATE;
    (void)FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_PCR,
                             FAPEX_IOCCOM_SYNC_SETPCRPID,
                             (FAPEXIOCARG3_T)&pid );
    */
    /* changed by Jim.Kang 2010.7.19 */
    FAPEX_TSD_PidParamStrT  pidParam;
    int32_t result;

    if(fd_TSD0_PCR == -1)
    {
        return 0;
    }

    /* set PCR filtering */
    pidParam.pid        = FAPEX_SYNC_INVALIDATE;
    pidParam.flags      = FAPEX_TSD_FLTFLAGS_PCR;
    pidParam.bufferSize = 0;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_PCR,
                              FAPEX_IOCCOM_TSD_SET_PID_FILTER,
                              (FAPEXIOCARG3_T)&pidParam );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[main.c] Set PCR Pid. %d\n", result);
        return result;
    }

#endif

    pLive->streamsStarted = FFALSE;
    pLive->ptsSynced      = FFALSE;

    return FAPI_OK;
}

int32_t AVINT_StartStreams(const SLIST_ITEM_S* pService,
                                   fbool_t handleVideo,
                                   fbool_t handleAudio)
{
    // TODO: to be supported
    //SYS_VIDSTART_MODE_E        startMode      = VIDEOSTART_FAST;
    int8_t                       indexAudio     = -1;
    int8_t                       indexSpdif     = -1;
    SYS_SPDIF_MODE_E             defSpdifMode   = SPDIFMODE_PCMONLY;
    AV_LIVETV_S*                 pLive          = &(avData->liveTv);
    const SYS_STREAMINF_AUDIO_S* pAudio         = &(pService->strPid.audio);
    const SYS_CONFIGDAT_S*       sysCfg         = SYS_GetConfig();
    uint8_t                      videoType      = STRTYPE_VIDEO_MPEG2;
    uint16_t                     videoPid       = 0;
    uint16_t                     audioPid       = 0;
    uint16_t                     pcrPid         = 0;
    #ifdef APPL_DTG_SIPSI_T2_ENABLED
    fbool_t                     isDecodable;
    #endif
    uint32_t enableStcRegulation;
    uint32_t syncEnvironment;
    /*uint32_t pid;*/ // by Jim.Kang


    FAPI_SYS_PRINT_DEBUG(1, "[av_live][AVINT_StartStreams] enter\n");

    if ( !(pService->strPid.valid) )
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    pLive->haveVideoStream = FFALSE;
    pLive->serviceDecoded  = FFALSE;
    pLive->timeoutSent     = FFALSE;

    AV_OutputFormatUpdate (FFALSE);

    if ( handleAudio )
    {
        /* ----- choose main audio stream (I2S) ---------------------------- */

        /* NOTE: Audio stream selection is performed in the following order:
           1. If a stream had been selected before, it is re-selected
           2. If a language request has been passed, it is attempted to choose
              accordingly.
           3. If a general audio language preference has been set, it is used
              for stream selection.
           4. If all selections fail, language information is ignored and the
              first available stream is chosen.
        */

        /* stream already selected? */
    #ifndef APPL_DTG_SIPSI_T2_ENABLED
        if ( (pLive->selectedAudioStream >= 0)
          && ((uint8_t)pLive->selectedAudioStream < pAudio->numStreams) )
        {
            indexAudio = pLive->selectedAudioStream;
        }
        if ( indexAudio < 0 )
    #else
        {
            if(AV_AudioFormatIsDecodable (SYS_AUDIO_AC3, &isDecodable) == FAPI_OK )
            {
                if(isDecodable)
                {
                    indexAudio = AVINT_AudioChooseStream (pAudio, SYS_AUDIO_AC3,
                                                          pLive->pref.langAudio,
                                                          FTRUE);
                }
            }
        }
    #endif/*#ifdef APPL_DTG_SIPSI_T2_ENABLED*/

        if ( indexAudio < 0 )
        {
            indexAudio = AVINT_AudioChooseStream (pAudio, SYS_AUDIO_ALL,
                                                  pLive->pref.langAudio,
                                                  FTRUE);
        }

        if ( indexAudio >= 0 )
        {
            if ( pLive->pidOverride.adOverride )
            {
                audioPid = pLive->pidOverride.adPid;
            }
            else
            {
                audioPid = pAudio->stream[indexAudio].pid;
            }

            // FIXME: override channel/format as well?

            pLive->selectedAudioStream = indexAudio;
            pLive->currentAudioStream  = indexAudio;
        }
        else if ( pLive->pidOverride.adOverride )
        {
            audioPid = pLive->pidOverride.adPid;
        }
        /* ----- choose SPDIF audio stream --------------------------------- */

        (void)SETTINGS_DefaultSPDIFModeGet(&defSpdifMode);

        if ( (pLive->pref.spdifPref) &&
             (pLive->pref.spdifMode == SPDIFMODE_PCMONLY) )
        {
            /* PCM output is forced: do nothing */
        }
        else if ( (!(pLive->pref.spdifPref)) &&
                  (defSpdifMode == SPDIFMODE_PCMONLY) )
        {
            /* PCM output is set by default: do nothing */
        }
        else
        {
            /* stream already selected? */
            if ( (pLive->selectedSpdifStream >= 0)
              && ((uint8_t)pLive->selectedSpdifStream < pAudio->numStreams)
              && (AVINT_AudioIsSPDIFCompliant(pAudio->stream
                                              [pLive->selectedSpdifStream].
                                              format)) )
            {
                indexSpdif = pLive->selectedSpdifStream;
            }

            if ( indexSpdif < 0 )
            {
                indexSpdif = AVINT_AudioChooseSPDIFStream (pAudio, pLive->pref.
                                                           langSpdif);
            }
        }

        if ( indexSpdif >= 0 )
        {
            pLive->selectedSpdifStream = indexSpdif;
            pLive->currentSpdifStream  = indexSpdif;
        }
        else
        {
            pLive->selectedSpdifStream = -1;
        }

        FAPI_SYS_PRINT_DEBUG(1, "[av_live][AVINT_StartStreams] handle Audio done\n");
    }

    /* ----- choose Video stream ------------------------------------------- */

    if ( handleVideo )
    {
        if ( pLive->pidOverride.vdOverride )
        {
            pLive->haveVideoStream = FTRUE;
            videoType = pLive->pidOverride.vdType;
            videoPid  = pLive->pidOverride.vdPid;
        }
        else if ( pService->strPid.video > 0)
        {
            pLive->haveVideoStream = FTRUE;
            videoType = pService->strPid.videoType;
            videoPid  = pService->strPid.video;
        }
    }

    /* ----- start stream decoding ----------------------------------------- */
#ifndef _LINUX_FAPI
    (void)FAPI_SYNC_SetEnvironment(FAPI_SYNC_PCR_BASED);
#else
    syncEnvironment = FAPEX_SYNC_PCR_BASED;
    FAPEXIOCRET_T retv;
    /*retv = FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)VIDEO_GetVidecId(),*/
    /* changed by Jim.Kang 2010.7.19 */ 
    AV_VIDEO_S* pVideo = &(avData->video);
    retv = FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)VAL_GetIdViencHd(pVideo->valDataPtr),
                             FAPEX_IOCCOM_SYNC_SETENVIRONMENT,
                             (FAPEXIOCARG3_T)&syncEnvironment );
    if ( retv < 0 )
         FAPI_SYS_PRINT_MSG("[av_live][AVINT_StartStreams] FAPEX_IOCCOM_SYNC_SETENVIRONMENT retv =%d\n",retv);
#endif

    // TODO: to be supported
    //(void)SETTINGS_GetServiceStartMode(&startMode);

    // TODO: to be supported
    //if ( startMode == VIDEOSTART_SYNC )
    //{
    //    AVINT_AudioMuteRequestSet(AV_MUTEREQ_SYNC);
    //}

    if ( handleVideo && (videoPid > 0) )
    {
        retv = AVINT_VideoStart (VAL_ENV_LIVE, videoPid, videoType,
                                VAL_PLAYMODE_NORMAL,
                                AVINT_LiveTvVideoStartNotify, NULL);
    }
    else if((sysCfg->radioScreen != NULL) &&
            (!(pLive->radioScreenVisible)) &&
            (pAudio->numStreams > 0)
            #ifdef APPL_MHEG_ENABLED
            && (MHEG_GetStatus() == MHEG_STATE_STOPPED)
            #endif
        )
    {
        (void)AVINT_VideoIFrameShow (sysCfg->radioScreen,
                                     sysCfg->radioScreenSize,
                                     STRTYPE_VIDEO_MPEG2);

        pLive->radioScreenVisible = FTRUE;
    }

    if ( indexSpdif >= 0 )
    {
        (void)AVINT_AudioSPDIFSetStatus (SPDIFSTATUS_PASSTHROUGH,
                                         pAudio->stream[indexSpdif].pid,
                                         pAudio->stream[indexSpdif].format);
    }
    else
    {
        (void)AVINT_AudioSPDIFSetStatus (SPDIFSTATUS_CURRPCM, 0,
                                         SYS_AUDIO_NUMS);
    }
    if( pLive->pidOverride.adOverride )
    {
        indexAudio = 0;
    }
    if ( (indexAudio >= 0) && (audioPid > 0) )
    {
        // TODO: override channel/format as well?

        (void)AVINT_AudioStart (audioPid,
                                pAudio->stream[indexAudio].format,
                                pAudio->stream[indexAudio].channel,
                                AVINT_LiveTvAudioStartNotify);
    }

    /* handle PCR channel start */

    if ( pLive->pidOverride.pcrOverride )
    {
        pcrPid = pLive->pidOverride.pcrPid;
    }
    else
    {
        pcrPid = pService->strPid.pcr;
    }

    if ( pcrPid != 0 )
    {
#ifndef _LINUX_FAPI

        if ( (pcrPid != videoPid) && (pcrPid != audioPid) )
        {
            (void)FAPI_SYNC_SetPcrPid(pcrPid);
        }

        (void)FAPI_SYNC_SetStcRegulation(1);
#else
        FAPEX_TSD_PidParamStrT  pidParam;

        if ( (pcrPid != videoPid) && (pcrPid != audioPid) )
        {
            /* changed by Jim.Kang. 2010.7.19 */
            /*
            pid = (uint32_t)pcrPid;
            (void)FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                             FAPEX_IOCCOM_SYNC_SETPCRPID,
                             (FAPEXIOCARG3_T)&pid );
            */
            pidParam.pid        = (uint32_t)pcrPid;
            pidParam.flags      = FAPEX_TSD_FLTFLAGS_PCR;
            pidParam.bufferSize = 0;
            (void)FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD0_PCR,
                                   FAPEX_IOCCOM_TSD_SET_PID_FILTER,
                                  (FAPEXIOCARG3_T)&pidParam );
        }
        enableStcRegulation = 1;
        /*(void)FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)VIDEO_GetVidecId(),*/
        /* changed by Jim.Kang 2010.7.19 */
        AV_VIDEO_S* pVideo = &(avData->video);
        (void)FAPEX_VIDEC_Ioctl( (FAPEXIOCARG1_T)VAL_GetIdViencHd(pVideo->valDataPtr),
                                 FAPEX_IOCCOM_SYNC_SETSTCREGULATION,
                                (FAPEXIOCARG3_T)&enableStcRegulation);
#endif
    }

    // TODO: to be supported
    //if((startMode == VIDEOSTART_SYNC) && (retVal == FAPI_OK))
    //{
    //    AVINT_AudioMuteRequestClear(AV_MUTEREQ_SYNC);
    //}

    if ( TIME_GetSystemTime(&(pLive->streamsStartTime)) != FAPI_OK )
    {
        pLive->streamsStartTime = 0;
    }

    pLive->streamsStarted = FTRUE;

    // FIXME: why is SETTINGS API used here? Is call needed at all?
    // KCHEN: this overwrite the MHEG dispAR setting
    //(void)SETTINGS_AspectRatioSet();

    return FAPI_OK;
}

void AVINT_LiveTvVideoStartNotify(void* optData)
{
    AV_LIVETV_S        *pLive = &(avData->liveTv);
    SYS_EVT_SERVICE_S   ctrlEvent;

    FAPI_SYS_PRINT_DEBUG(1,"[LIVE] Video starts....\n");

    if(!(pLive->serviceDecoded))
    {
        pLive->serviceDecoded = FTRUE;

        /* Generate event to notify success of service decoding */
        ctrlEvent.hdr.type   = EVT_SERVICE_SELECTED;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.servLoc    = pLive->currServLoc;
        ctrlEvent.audioStr   = pLive->selectedAudioStream;
        ctrlEvent.status     = SERVICE_STAT_DECODED;

        (void)EVT_Send(&ctrlEvent, "Service decoded");
    }
}

void AVINT_LiveTvAudioStartNotify(void)
{
    AV_LIVETV_S        *pLive = &(avData->liveTv);
    SYS_EVT_SERVICE_S   ctrlEvent;

    FAPI_SYS_PRINT_DEBUG(4,"[LIVE] Audio starts....\n");

    if((!(pLive->haveVideoStream)) && (!(pLive->serviceDecoded)))
    {
        pLive->serviceDecoded = FTRUE;

        /* Generate event to notify success of service decoding */
        ctrlEvent.hdr.type = EVT_SERVICE_SELECTED;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.servLoc = pLive->currServLoc;
        ctrlEvent.audioStr = pLive->selectedAudioStream;
        ctrlEvent.status = SERVICE_STAT_DECODED;

        (void)EVT_Send(&ctrlEvent, "Service decoded");
    }
}

FSTATIC void AVINT_LiveTvSDTHandling(void)
{
    AV_LIVETV_S        *pLive = &(avData->liveTv);
    PSISI_SDT_SERVICE_S sdtService;
    SYS_EVT_SDT_S       sdtEvt;
    SYS_EVT_RUNSTAT_S   runstatEvt;
    SYS_RUNSTAT_E       newRunStat;
    int32_t             retVal;
    uint16_t            serviceInfoPos;
    SLIST_ITEM_S        slistItem;

    if(avData->sdt.sect.allAvailable)
    {
        FAPI_SYS_PRINT_DEBUG(4,"[LIVE] SDT Update: version %i\n",
                   avData->sdt.version_number);

        /* ----- Check running status of current live service -------------- */

        if(PSISI_SDT_GetServiceById(&(avData->sdt),
                                    pLive->currentService.serviceId,
                                    &sdtService) == FAPI_OK)
        {
            switch (sdtService.running_status)
            {
                case PSISI_RUNSTAT_STARTSOON:
                case PSISI_RUNSTAT_NOTRUNNING:
                    newRunStat = SYS_RUNSTAT_NOTRUNNING;
                    break;

                default:
                    newRunStat = SYS_RUNSTAT_RUNNING;
                    break;
            }

            if(newRunStat != pLive->currRunstat)
            {
                pLive->currRunstat = newRunStat;

                runstatEvt.hdr.type = EVT_SERVICE_RUNSTAT;
                runstatEvt.hdr.length = sizeof(runstatEvt);
                runstatEvt.status = newRunStat;

                (void)EVT_Send(&runstatEvt, "Running status");
            }
        }

        /* ----- Check service list for changes ---------------------------- */

        serviceInfoPos = 0;

        sdtEvt.hdr.type = EVT_SDT_RECEIVED;
        sdtEvt.hdr.length = sizeof(sdtEvt);
        sdtEvt.version_number = avData->sdt.version_number;
        sdtEvt.servicesChanged = FFALSE;

        do
        {
            retVal = PSISI_SDT_GetNextService(&(avData->sdt), &serviceInfoPos,
                                              &sdtService);
            if(retVal != FAPI_OK)
                break;

            memset(&slistItem, 0, sizeof(SLIST_ITEM_S));

            slistItem.network      = pLive->currentService.network;
            slistItem.tpIdx        = pLive->currentService.tpIdx;
            slistItem.flags       |= SLIST_FLAG_VISIBLESVC;   /* default */
            slistItem.minAge       = PCTRL_MINAGE_UNLIMITED;
            slistItem.lcn          = 0; /* default */
            slistItem.strPid.valid = 0; /* default */

            if ( AV_ScanAddSdtInfos(&sdtService, &slistItem) == FAPI_OK )
            {
                retVal = SLIST_SdtUpdatedService(&slistItem);

                if(retVal == SMARTGO_SYS_ERR_REPLACED_IN_LIST)
                {
                    sdtEvt.servicesChanged = FTRUE;
                }
            }
        }
        while(1);

        /* Update current live service parameters: might have been changed
           by SDT update. */

        (void) SLIST_GetServByIndex(pLive->currServIdx,
                                    &(pLive->currentService));
        (void)EVT_Send(&sdtEvt, "SDT update");

        /* ----------------------------------------------------------------- */

        pLive->currVersionSdt = avData->sdt.version_number;

        /* Start filtering for next version */
        PSISI_SDT_Invalidate(&(avData->sdt));
        (void)PSISI_SDT_StartFiltering (pLive->currVersionSdt, FFALSE);
        /* NOTE: Stopping of filter is done in internal PSISI handling. */
    }
}

FSTATIC void AVINT_LiveTvNITHandling(AV_FEDAT_S* pFe)
{
    AV_LIVETV_S        *pLive = &(avData->liveTv);
    int32_t            retVal;
    PSISI_NIT_S        *pNit = NULL;
    SYS_EVT_NIT_S      nitEvt;
    uint16_t           cnt = 0;
    uint16_t           service_cnt = 0;
    fbool_t            tpFound = FFALSE;
    int32_t            tpIdx = -1;
    int32_t             curtsId = 0;
    TPLIST_ITEM_S       tpItem;

    pNit = (PSISI_NIT_S*)PSISI_NIT_Get();

    if((pNit != NULL) && (pNit->sect.allAvailable))
    {
        FAPI_SYS_PRINT_DEBUG(4,"[LIVE] NIT Update: version %i\n", pNit->version_number);

        retVal = TPLIST_GetDataByIndex(&(pFe->currTunePar.network), pFe->currTunePar.tpIdx, &tpItem);
        if(retVal == FAPI_OK)
        {
            curtsId = tpItem.tsId;
        }
        else
        {
            return;
        }

        /* ----- Check  tp list & service list for changes ---------------------------- */
        nitEvt.hdr.type = EVT_NIT_RECEIVED;
        nitEvt.hdr.length = sizeof(nitEvt);
        nitEvt.version_number = pNit->version_number;
        nitEvt.servicesChanged = FFALSE;

        //FAPI_SYS_PRINT_DEBUG(3,"\n[LIVE] NIT Update: start time %lld\n", FAPEX_SYS_G(FAPI_TIMER_RESOLUTION_1_MSEC));
        //FAPI_SYS_PRINT_DEBUG(3,"\n[LIVE] NIT Update: start time %lld\n", FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC));
        do
        {
            memset(&pFe->nitTpInfo, 0, sizeof(PSISI_NIT_TP_INFO_S));
            pFe->nitTpInfo.tpIdx = -1;

            pFe->nitTpInfo.tpItem.networkId = pNit->network_id;
            pFe->nitTpInfo.tpItem.tsId = pNit->ts[cnt].tsId;
            pFe->nitTpInfo.tpItem.onId = pNit->ts[cnt].onId;

            tpFound = PSISI_NIT_ParseByTP(pFe->systemType, pNit->ts[cnt].descr_loop,
                                        pNit->ts[cnt].descrLen, &pFe->nitTpInfo, curtsId);

            if(tpFound == FFALSE)
            {
                FAPI_SYS_PRINT_DEBUG(2,"[LIVE] NIT Update cannot find TP detail info\n");
            }

            //Compare with tp list
            tpIdx = TPLIST_UpdateCarrier(pFe->systemType, &pFe->nitTpInfo,
                                        pFe->currTunePar.network.satIdx);

            if((tpIdx != -1) && (tpIdx == pFe->currTunePar.tpIdx))
            {
                //Compare with service list for current tp
                FAPI_SYS_PRINT_DEBUG(2,"[LIVE] NIT Update service list\n");
                for(service_cnt=0;service_cnt<pFe->nitTpInfo.serviceCount;service_cnt++)
                {
                    retVal = SLIST_NitUpdatedService(
                            (const PSISI_NIT_SERVICE_INFO_S * )
                            (&(pFe->nitTpInfo.nitServiceList[service_cnt])),
                            pFe->nitTpInfo.tpIdx, &(pLive->currentService.network), pFe->nitTpInfo.desFlag);
                    if(retVal == SMARTGO_SYS_ERR_REPLACED_IN_LIST)
                    {
                        nitEvt.servicesChanged = FTRUE;
                    }
                }
            }
            cnt++;
        }
        while(cnt < pNit->numOfTs);
        #ifdef APPL_LCN_ENABLED
        if(nitEvt.servicesChanged)
        {
            (void) SLIST_LCN_GenerateLists();
            FAPI_SYS_PRINT_DEBUG(2,"[LIVE] NIT Update: SLIST_List TV = %d, Radio = %d\n", SLIST_ListSizeGet(SLIST_TV), SLIST_ListSizeGet(SLIST_RADIO));
        }
        #endif
        (void) SLIST_GetServByIndex(pLive->currServIdx, &(pLive->currentService));
        (void) EVT_Send(&nitEvt, "NIT update");

        //FAPI_SYS_PRINT_DEBUG(3,"[LIVE] NIT Update: end time %lld\n\n", FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC));
        /* ----------------------------------------------------------------- */

        pLive->currVersionNit = pNit->version_number;

        /* Start filtering for next version */
        (void) PSISI_NIT_Invalidate();
        (void) PSISI_NIT_StartFiltering (pLive->currVersionNit, FFALSE, 0);
        /* NOTE: Stopping of filter is done in internal PSISI handling. */
    }
}

/* ------------------------------------------------------------------------- */
FSTATIC void AVINT_FPLiveSet(const SYS_SERVLOC_S * servLoc,
                            const SLIST_ITEM_S * service)
{
#if 0 // TODO
    FD_GHW_FP_SERVICE_S servInfo;
    uint16_t            servNum;

    #ifdef APPL_LCN_ENABLED
    SLIST_PARAMS_S      slistParams;
    #endif /* APPL_LCN_ENABLED */

    memset(&servInfo, 0, sizeof(servInfo));

    switch (servLoc->listId)
    {
        case SLIST_TV:
            servInfo.type = FD_GHW_FP_SERV_TYPE_TV;
        case SLIST_RADIO:
            servInfo.type = FD_GHW_FP_SERV_TYPE_RADIO;
        default:
            servInfo.type = FD_GHW_FP_SERV_TYPE_FAV;
    }

    servNum = servLoc->index + 1;

    #ifdef APPL_LCN_ENABLED

    if((SLIST_GetParams(&slistParams) == FAPI_OK) && slistParams.sortByLcn)
    {
        servNum = 0;

        SLIST_GetLcn(servLoc, &servNum);
    }

    #endif /* APPL_LCN_ENABLED */

    servInfo.number = servNum;
    servInfo.name = service->name;

    FD_GHW_ControlFrontPanel(0, FD_GHW_FP_SET_SERVICEINFO, &servInfo);
#endif
}

FSTATIC void AVINT_FPLiveClear(void)
{
#if 0 // TODO
    FD_GHW_ControlFrontPanel(0, FD_GHW_FP_SET_SERVICEINFO, NULL);
#endif
}

FSTATIC int32_t pcrTsdOpen(void)
{
    FAPEX_TSD_OpenSubIdParamStrT pidChParam;
    int32_t result;

    if(fd_TSD0_PCR != -1) {
        return 0;
    }

    pidChParam.channelNo = 30;
    pidChParam.sub_id    = FAPEXDEV_TSD0_SUB_MINOR_FIRST + pidChParam.channelNo;
    result = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSD[0],
                              FAPEX_IOCCOM_TSD_OPEN_SUBID,
                             (FAPEXIOCARG3_T)&pidChParam);
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[main.c] Open PCR TSD channel. %d\n", result);
    }
    else {
        fd_TSD0_PCR = pidChParam.sub_id;
    }
    return result;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
