/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/fcdata.c
**
**              $Id: fcdata.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Module for flash-sparing storage of data frequently-changed
**              during runtime.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#ifndef _LINUX_FAPI
#include <fapi/sys_driver.h>         /* global type definitions */
#include <fapi/drv_flash.h>
#else
#include <fapex/abst_types.h>
#include <fapex/abst_libvrflash.h>
#endif
#include <flashfs/flashfs.h>

#include "fcdata.h"
#include "sys_priv.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static FLASHFS_HANDLE  flHdl = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/


/*!
********************************************************************************
** \brief   Initialse the frequently change data module of SmartGo-HD.
**
** \note    This function is currently not used.
**
** \return  One of the following status codes:
**          - #FAPI_OK always.
**
********************************************************************************
*/
int32_t FCDATA_Init(void)
{
    int32_t retVal;

    if ( flHdl != NULL )
    {
        retVal = APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }
    else
    {
        retVal = FAPI_OK;

        flHdl = FLASHFS_Open (FILEID_FREQCHGDATA, FLASHFS_OPEN_WRITE, &retVal);
    }

    return retVal;
}

/*!
********************************************************************************
** \brief   Writes the given entry to the frequently changed data file.
**
** \param   pEntry  Pointer to the entry to b stored.
**
** \return  One of the following status codes:
**          - #FAPI_OK if reading was successful.
**          - FLASHFS error code.
**
********************************************************************************
*/
int32_t FCDATA_StoreEntry(const FCDATA_ENTRY_S * pEntry)
{
    int32_t retVal;

    if ( flHdl != NULL )
    {
        retVal = FLASHFS_Write (flHdl, sizeof(FCDATA_ENTRY_S), pEntry, NULL);

        if ( retVal == (int32_t)(sizeof(FCDATA_ENTRY_S)) )
        {
            retVal = FAPI_OK;
        }
        else if ( retVal > 0 )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
        }
    }
    else
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    return retVal;
}

/*!
********************************************************************************
** \brief   Reads the last entry of the frequently changed data file.
**
** \param   pEntry  Pointer to store the entry.
**
** \return  One of the following status codes:
**          - #FAPI_OK if reading was successful.
**          - FLASHFS error code.
**
********************************************************************************
*/
int32_t FCDATA_GetEntry (FCDATA_ENTRY_S* pEntry)
{
    int32_t retVal;

    if ( flHdl != NULL )
    {
        retVal = FLASHFS_Read(flHdl, sizeof(FCDATA_ENTRY_S), pEntry, NULL);

        if ( retVal == (int32_t)(sizeof(FCDATA_ENTRY_S)) )
        {
            retVal = FAPI_OK;
        }
        else if ( retVal > 0 )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
        }
    }
    else
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    return retVal;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
