/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/main.c
**
**              $Id: main.c 5881 2010-03-03 08:04:49Z jmorha $
**
** \brief       System initialisation and common utility functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e732 -e737 -e774 -e961 */
#define DEBUG 1
//#define ALLOW_SMALLER_S_FLASH_FS_START_ADDRESS 0x00240000  // in case of 'smaller S-Flash'

//#define APPL_LOW_POWER_STANDBY 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#ifdef APPL_HDD_ENABLED
    #define FAPI_SYS_INIT_ATA   1
#endif /* APPL_HDD_ENABLED */

#ifdef APPL_ETHERNET_ENABLED
    #define FAPI_SYS_INIT_ETH   1
#endif /* APPL_ETHERNET_ENABLED */


#if (DDR_MEMORY_BYTES_1 == 0x08000000) // 1 GBit
    #define SMARTGO_SYS0_SIZE   30
#elif (DDR_MEMORY_BYTES_1 == 0x04000000) // 512Mbit
        #define SMARTGO_SYS0_SIZE   12
#else
    //#error Unsupported Memory setting
#endif
#define SMARTGO_SYS1_SIZE   FAPI_MMU_MAX_SEGMENT_MBYTES // use the rest of memory

#define APP_SET_BOARD_PARAMS                                            \
    FAPI_MMU_SetSegmentSizeMBytes(FAPI_MMU_HEAP_SEGMENT_INDEX_SYS0,SMARTGO_SYS0_SIZE); \
    FAPI_MMU_SetSegmentSizeMBytes(FAPI_MMU_HEAP_SEGMENT_INDEX_SYS1,SMARTGO_SYS1_SIZE); \
    FAPI_MMU_SetSegmentFunctionMode(FAPI_MMU_HEAP_SEGMENT_INDEX_SYS0,   \
                                    FAPI_MMU_FUNCTION_BGET);            \
    FAPI_MMU_SetSegmentFunctionMode(FAPI_MMU_HEAP_SEGMENT_INDEX_SYS1,   \
                                    FAPI_MMU_FUNCTION_BGET);            \
    FAPI_MMU_SetSegmentStrategyMode(FAPI_MMU_HEAP_SEGMENT_INDEX_SYS0,   \
                                    FAPI_MMU_STRATEGY_BESTFIT);         \
    FAPI_MMU_SetSegmentStrategyMode(FAPI_MMU_HEAP_SEGMENT_INDEX_SYS1,   \
                                    FAPI_MMU_STRATEGY_BESTFIT);         \
    FAPI_AUOUT_SetInternalDac(FAPI_AUOUT_I2S_0);                        \
    /* NOTE: This FAPI_USB_SetKeepVbus(0) should be set to 1 in case */ \
    /* the USB connector is used in host only mode and the ID pin */    \
    /* is set to GND */                                                 \
    FAPI_USB_SetKeepVbus(0);

// avoid 'FAPI_GPIO_SetPinFunction() calls' from 'sys_init_mb86h60_devkit.h'
#define FAPI_SYS_INIT_USE_BOARDS 1

#ifndef _LINUX_FAPI
#include <fapi/sys_init.h>
#endif

#include <ctype.h>

#include <osdhandler/osdhandler.h>
#include <draw/draw.h>
#include <fontserver/fontserver.h>
#include <fontserver/fontserver_engine_fbf.h>
#include <fontserver/fontserver_engine_ft.h>
#include <fbf/fbf_zwo_bold.h>

#ifndef _LINUX_FAPI
#include <rtos/rtos.h>
#include <fapi/board_ifc.h>
#include <boards/boards_common.h>
#endif

#include "sys_api.h"
#include "init.h"
#include "event_queue.h"
#include "settings.h"
#include "av.h"
#include "eit.h"
#include "key_input.h"
#include "sys_time.h"
#include "fcdata.h"
#include "rectimer.h"

#ifndef _LINUX_FAPI
#include <fapi/drv_flash.h>
#else
#include <linux/version.h>
#include "fapex/abst_libvrflash.h"
#include "fapex/abst_libclkpwr.h"
#include <fapex/abst_libboardifc.h>

#endif

#ifdef APPL_SUBTT_ENABLED
#include "subtitle.h"
#endif /* APPL_SUBTT_ENABLED */

#ifdef APPL_TTXT_ENABLED
#include "teletext.h"
#endif /* APPL_TTXT_ENABLED */

#ifdef APPL_RFMOD_ENABLED
#include "rf_modulator.h"
#endif /* APPL_RFMOD_ENABLED */

#ifdef APPL_PVR_ENABLED
#include "pvr_sys.h"
#include "pvr_list.h"
#endif /* APPL_PVR_ENABLED */

#ifdef APPL_PIP_ENABLED
#include "pip.h"
#endif /* APPL_PIP_ENABLED */

#ifdef APPL_UPDATE_ENABLED
#include "update.h"
#endif /* APPL_UPDATE_ENABLED */


#include "utils.h"

#ifndef _LINUX_FAPI
#include BOARD_INCLUDE_FILE         // should be defined in 'Config.mak.user'
#endif
/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

// TODO: to be supported!
//#define ENABLE_BOOTOSD          // define this to enable boot messages on OSD

#ifdef ENABLE_BOOTOSD
#define BOOT_OSD       2
#define BOOTOSD_BACK   0
#define BOOTOSD_FRONT  1

#define BOOTOSD_Y       310
#define BOOTOSD_PAD_IN  10
FSTATIC uint32_t bootOsdFontSizePtr[1];
FSTATIC uint32_t bootOsdFontData[1];
FSTATIC uint32_t bootOsdFontEngine[1];
#endif
/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC SYS_CONFIGDAT_S         sysConfig;

FSTATIC SYS_HWCONFIG_S          hwConfig;

SYS_DATA_S*                     sysData = NULL;

#ifdef ENABLE_BOOTOSD

FSTATIC OSDHANDLER_StartParamsT osdPars;
FSTATIC fbool_t                 osdAvailable = FFALSE;

#endif /* ENABLE_BOOTOSD */

#ifndef _LINUX_FAPI
static FAPI_SYS_HandleT         sysClkpwrHandle = 0;
#else
static int32_t    sysClkpwrHandle = 0;
#endif

#ifdef APPL_LOW_POWER_STANDBY
static uint32_t                 sysStandbyMode  = 0;
#endif

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void            SYS_StartThread         (void*);
FSTATIC void            SYS_UserThread          (void* opt);
FSTATIC fbool_t         SYS_InternalEvtHandling (SYS_EVTTYPE_E type,
                                                const void* data);
FSTATIC int32_t         SYS_GoDown              (void);
FSTATIC int32_t         SYS_GoUp                (void);
#ifdef ENABLE_BOOTOSD
FSTATIC int32_t         SYS_BootOsdInit         (SYS_HDTV_STANDARD hdSys);
FSTATIC void            SYS_BootOsdExit         (void);
#endif
FSTATIC void            SYS_PrintStartupHdr     (void);
#ifdef APPL_DEBUG_MODE
FSTATIC const char_t*   SYS_GetDeviceString     (void);
#endif
FSTATIC int32_t         SYS_Init                (const SYS_CONFIGDAT_S* pSysCfg);

FSTATIC uint32_t        getOsdIntIdx            (uint32_t id);
FSTATIC int32_t         SYS_CheckOsdLayerId     (uint32_t id);
FSTATIC int32_t         SYS_GetFlashDeviceType  (uint32_t *devType);

#ifndef _LINUX_FAPI
FSTATIC void            SYS_SysIrqFunction      (FAPI_INTR_SysIrqEnumT intrType,
                                                 uint32_t atAddress);
#endif
/*---------------------------------------------------------------------------*/
/* start function                                                            */
/*---------------------------------------------------------------------------*/

#define FAPI_BOARD_HWCONFIG_VERSION 0xFABE0003UL    //!< magic to validate 'FAPI_BOARD_HwPlatformInfosT'

int32_t SYS_Start (SYS_CONFIGDAT_S* pConfigDat)
{
    int32_t                      retVal = FAPI_OK;
#ifndef _LINUX_FAPI
    FAPI_BOARD_HwPlatformInfosT* infoPtr;
#else
    FAPEX_BOARD_HwPlatformInfosStrT *infoPtr;
#endif

    do
    {
#ifndef _LINUX_FAPI
        /* Perform necessary platform-specific registrations on boards
           module. */
        (void) BOARDS_INSTALL_PLATFORM_SUPPORT(BOARDS_REGISTER_DEFAULT, 0);
        (void) BOARDS_INSTALL_FRONTEND_SUPPORT(BOARDS_REGISTER_DEFAULT, 0);

        /* store SmartGo system configuration */
        sysConfig = *pConfigDat;

        /* get HW configuration from boards module */
        infoPtr = FAPI_BOARD_GetBoardInfo();

        if ( infoPtr == NULL )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        // TODO: Move this into FAPI_BOARD_GetBoardInfo, provide version
        //       as argument
        if ( infoPtr->MagicAndVersion != FAPI_BOARD_HWCONFIG_VERSION )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        /* store platform HW configuration */
        memset(&hwConfig, 0, sizeof(hwConfig));

        hwConfig.feNums                = infoPtr->numberOfAvlFrontEnds;
        hwConfig.tsInput[RSRC_TS_A]    = infoPtr->feCfg[0].tsPathInputConfig;
        hwConfig.tsInput[RSRC_TS_B]    = infoPtr->feCfg[1].tsPathInputConfig;
        hwConfig.ciAvl  [RSRC_TS_A]    = (fbool_t)(infoPtr->feCfg[0].
                                                   flagCiPathAval);
        hwConfig.ciAvl  [RSRC_TS_B]    = (fbool_t)(infoPtr->feCfg[1].
                                                   flagCiPathAval);
        hwConfig.ciInput[RSRC_TS_A]    = infoPtr->feCfg[0].inputOptionsCiPath;
        hwConfig.ciInput[RSRC_TS_B]    = infoPtr->feCfg[1].inputOptionsCiPath;
        hwConfig.usbHostOnly           = (fbool_t)(infoPtr->flagUsbHostOnly);
        hwConfig.mpencPos              = RSRC_MPENCPOS_NONE; /* not provided */
        // TODO: improved flash handling to be implemented
        retVal = platformInfoPtr->typeOfFlashAvailable;
        if      (retVal & FAPI_BOARD_BITFLAG_PFLASH)
            UseThisFlashType = FAPI_FLASH_DEVICE_PFLASH;
        else if (retVal & FAPI_BOARD_BITFLAG_SFLASH)
            UseThisFlashType = FAPI_FLASH_DEVICE_SFLASH;
        else
            exit(1);

        /* initialise RTOS and FAPI */
        RTOS_InitServices();

        retVal = FAPI_SYS_Init();
        if ( retVal != FAPI_OK )
        {
            break;
        }


        if( ! RTOS_InitKernel() ) /* RTOS returns boolean error code */
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        FAPI_INTR_RegisterSysIrqFunction(SYS_SysIrqFunction);

        /* Init boards module */
        (void)FAPI_BOARD_INIT_DETECT_FUNCTION(BOARDS_INIT_DETECT_DEFAULT, 0);

        break; /* end of for(;;) */
#else
       /* Perform necessary platform-specific registrations on boards   module. */
 //      BOARDS_INSTALL_PLATFORM_SUPPORT_EXT();
       FAPI_BOARD_INIT_DETECT_FUNCTION(0,0);
       /* get HW configuration from boards module */
       FAPI_BOARD_GETBOARDINFO();
       infoPtr = &hwInfo;
       if ( infoPtr == NULL )
       {
           retVal = SMARTGO_SYS_ERR_INTERNAL;
           break;
       }
       // TODO: Move this into FAPI_BOARD_GetBoardInfo, provide version
       //       as argument
       if ( infoPtr->MagicAndVersion != FAPI_BOARD_HWCONFIG_VERSION )
       {
           retVal = SMARTGO_SYS_ERR_INTERNAL;
           break;
       }

       /* store SmartGo system configuration */
        sysConfig = *pConfigDat;

       /* store platform HW configuration */
       memset(&hwConfig, 0, sizeof(hwConfig));

       /* store platform HW configuration */
       memset(&hwConfig, 0, sizeof(hwConfig));
        FAPI_SYS_PRINT_DEBUG(1, "infoPtr->numberOfAvlFrontEnds = %d\n", infoPtr->numberOfAvlFrontEnds);
        FAPI_SYS_PRINT_DEBUG(1, "infoPtr->feCfg[0].tsPathInputConfig = %d\n", infoPtr->feCfg[0].tsPathInputConfig);
        FAPI_SYS_PRINT_DEBUG(1, "infoPtr->feCfg[1].tsPathInputConfig = %d\n", infoPtr->feCfg[1].tsPathInputConfig);
        FAPI_SYS_PRINT_DEBUG(1, "infoPtr->feCfg[0].flagCiPathAval = %d\n", infoPtr->feCfg[0].flagCiPathAval);
        FAPI_SYS_PRINT_DEBUG(1, "infoPtr->feCfg[1].flagCiPathAval = %d\n", infoPtr->feCfg[1].flagCiPathAval);
        FAPI_SYS_PRINT_DEBUG(1, "infoPtr->feCfg[0].inputOptionsCiPath = %d\n", infoPtr->feCfg[0].inputOptionsCiPath);
        FAPI_SYS_PRINT_DEBUG(1, "infoPtr->feCfg[1].inputOptionsCiPath = %d\n", infoPtr->feCfg[1].inputOptionsCiPath);
        FAPI_SYS_PRINT_DEBUG(1, "infoPtr->flagUsbHostOnly = %d\n", infoPtr->flagUsbHostOnly);

        hwConfig.feNums                = infoPtr->numberOfAvlFrontEnds;
        hwConfig.tsInput[RSRC_TS_A]    = infoPtr->feCfg[0].tsPathInputConfig;
        hwConfig.tsInput[RSRC_TS_B]    = infoPtr->feCfg[1].tsPathInputConfig;
        hwConfig.ciAvl  [RSRC_TS_A]    = (fbool_t)(infoPtr->feCfg[0].flagCiPathAval);
        hwConfig.ciAvl  [RSRC_TS_B]    = (fbool_t)(infoPtr->feCfg[1].flagCiPathAval);
        hwConfig.ciInput[RSRC_TS_A]    = infoPtr->feCfg[0].inputOptionsCiPath;
        hwConfig.ciInput[RSRC_TS_B]    = infoPtr->feCfg[1].inputOptionsCiPath;
        hwConfig.usbHostOnly           = (fbool_t)(infoPtr->flagUsbHostOnly);
        hwConfig.mpencPos              = RSRC_MPENCPOS_NONE; /* not provided */
#if 0
        hwConfig.feNums                = 1;//infoPtr->numberOfAvlFrontEnds;
        hwConfig.tsInput[RSRC_TS_A]    = 1;//infoPtr->feCfg[0].tsPathInputConfig;
        hwConfig.tsInput[RSRC_TS_B]    = 0;//infoPtr->feCfg[1].tsPathInputConfig;
        hwConfig.ciAvl  [RSRC_TS_A]    = 0;//(fbool_t)(infoPtr->feCfg[0].
                                             //      flagCiPathAval);
        hwConfig.ciAvl  [RSRC_TS_B]    = 0;//(fbool_t)(infoPtr->feCfg[1].
                                             //      flagCiPathAval);
        hwConfig.ciInput[RSRC_TS_A]    = 0;// infoPtr->feCfg[0].inputOptionsCiPath;
        hwConfig.ciInput[RSRC_TS_B]    = 0;//infoPtr->feCfg[1].inputOptionsCiPath;
        hwConfig.usbHostOnly           = 0;//(fbool_t)(infoPtr->flagUsbHostOnly);
        hwConfig.mpencPos              = RSRC_MPENCPOS_NONE;
#endif

        break;
#endif
    }while(0);

    if ( retVal != FAPI_OK )
    {
        /*lint -save -e586 */
        exit(1);
        /*lint -restore */
    }

    #ifdef APPL_BOOTLDR_ENABLED
    (void)SYS_GetBootloaderInfo();
    #endif /* APPL_BOOTLDR_ENABLED */

    FAPEX_SYS_ThreadT sys_start_thread_id;

    /* create start thread */
    retVal = SYS_ThreadCreate(SYS_StartThread, NULL,
                              THREAD_PRIO_START,
                              THREAD_STACKSZ_START, THREAD_NAME_START, &sys_start_thread_id);

    FAPEX_SYS_WaitEndThread(sys_start_thread_id);

    if(retVal != FAPI_OK)
        return retVal;

    /* enable RTOS scheduler */
#ifndef _LINUX_FAPI
    RTOS_StartMultitasking();
#else

#endif
    return FAPI_OK;
}

const SYS_CONFIGDAT_S* SYS_GetConfig (void)
{
    return &sysConfig;
}
const SYS_HWCONFIG_S* SYS_GetHWConfig (void)
{
    return &hwConfig;
}

int32_t SYS_RestoreDefaultState(fbool_t keepUserData)
{
    #ifdef APPL_RFMOD_ENABLED
    RFMOD_EXCHG_PARAMS_S          rfModParams;
    #endif /* APPL_RFMOD_ENABLED */

    /* Stop any recording, playback ... */
    (void)AV_Stop();                  /* stop running presentation */

    #ifdef APPL_PIP_ENABLED
    PIP_Hide();
    #endif /* APPL_PIP_ENABLED */

    (void)AV_Nop();

    #ifdef APPL_PVR_ENABLED

    PVRSYS_AbortAllOperations();
    PVRSYS_RestoreDefaults();

    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_HDD_ENABLED

    HDD_RestoreDefaults();

    #endif /* APPL_HDD_ENABLED */

    #ifdef APPL_RFMOD_ENABLED

    if(RFMOD_GetParams(&rfModParams) == FAPI_OK)
    {
        // TODO: to be tested...

        rfModParams.rfmod_TVSystem = RF_MOD_OFF;

        RFMOD_SetParams(&rfModParams);
    }

    #endif /* APPL_RFMOD_ENABLED */

    #ifdef APPL_FESAT_ENABLED

    CFGSAT_RestoreDefaults(keepUserData);

    #ifdef APPL_USALS_ENABLED

    (void)USALS_RestoreDefaults();

    #endif /* APPL_USALS_ENABLED */

    #endif /* APPL_FESAT_ENABLED */

    /* Restore transponder/channel list defaults */
    TPLIST_RestoreDefaults(FE_SYSTEM_TYPE_SATELLITE, keepUserData);
    TPLIST_RestoreDefaults(FE_SYSTEM_TYPE_TERRESTRIAL, keepUserData);
    TPLIST_RestoreDefaults(FE_SYSTEM_TYPE_CABLE, keepUserData);

    /* Restore service list defaults */
    (void)SLIST_RestoreDefaults();

    #ifdef APPL_RECTIMER_ENABLED

    (void)RECTIMER_Clear();

    #endif /* APPL_RECTIMER_ENABLED */

    /* Restore default startup settings */
    SETTINGS_StartupDefaultsSet();

    #ifdef APPL_RFMOD_ENABLED

    RFMOD_RestoreDefaults();

    #endif /* APPL_RFMOD_ENABLED */

    #ifdef APPL_PARENTAL_ENABLED

    (void)PCTRL_RestoreDefaults();

    #endif /* APPL_PARENTAL_ENABLED */

    /* Invalidate EIT database contents. */
    (void)EIT_ClearDatabase();

    return FAPI_OK;
}

int32_t SYS_GetVersion(SYS_VERSION_S * pVersion)
{
    if(pVersion == NULL)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    pVersion->major = SYS_VERSION_MAJOR;
    pVersion->minor = SYS_VERSION_MINOR;

    return FAPI_OK;
}

FSTATIC int32_t SYS_GetFlashDeviceType(uint32_t *devType)
{
#ifndef _LINUX_FAPI
    const FAPI_BOARD_HwPlatformInfosT *hwInfoPtr;
    FAPI_SYS_HandleT        bootHdl;
    FAPI_BOOT_OpenParamsT   bootParams = { FAPI_BOOT_VERSION };
    uint32_t                bootMode;
    uint32_t                bootDevice;
#endif
    int32_t                 retVal;
    uint32_t                actFlashType;

    actFlashType = DEFAULT_FLASH_DEVICE;
#ifndef _LINUX_FAPI
    hwInfoPtr = FAPI_BOARD_GetBoardInfo();
    if (  (hwInfoPtr)
        &&(hwInfoPtr->MagicAndVersion == FAPI_BOARD_HWCONFIG_VERSION ) )
    {
        retVal = hwInfoPtr->typeOfFlashAvailable;
        if      (retVal & FAPI_BOARD_BITFLAG_PFLASH)
            actFlashType = FAPI_FLASH_DEVICE_PFLASH;
        else if (retVal & FAPI_BOARD_BITFLAG_SFLASH)
            actFlashType = FAPI_FLASH_DEVICE_SFLASH;
    }
#endif
    *devType = actFlashType;
#ifndef _LINUX_FAPI
    bootHdl = FAPI_BOOT_Open(&bootParams, &retVal);
    if(retVal)
    {
        return retVal;
    }

    retVal = FAPI_BOOT_GetBootMode(bootHdl, &bootMode);
    if(retVal)
    {
        (void)FAPI_BOOT_Close(bootHdl);
        return retVal;
    }

    if( (bootMode == FAPI_BOOT_MODE_ROMBOOTLDR) ||
        (bootMode == FAPI_BOOT_MODE_BOOTER) )
    {
        retVal = FAPI_BOOT_GetBootDevice(bootHdl, &bootDevice);
        if(bootDevice == FAPI_BOOT_DEVICE_SFLASH)
        {
            *devType = FAPI_FLASH_DEVICE_SFLASH;
        }
        else if(bootDevice == FAPI_BOOT_DEVICE_PFLASH)
        {
            *devType = FAPI_FLASH_DEVICE_PFLASH;
        }
    }
    (void)FAPI_BOOT_Close(bootHdl);

    actFlashType = *devType;
    if      (actFlashType == FAPI_FLASH_DEVICE_SFLASH)
        SYS_BootOsdPrint("use S-Flash");
    else if (actFlashType == FAPI_FLASH_DEVICE_PFLASH)
        SYS_BootOsdPrint("use P-Flash");
#endif
    return retVal;
}

FSTATIC int32_t SYS_Init(const SYS_CONFIGDAT_S* pSysCfg)
{
#ifdef ALLOW_SMALLER_S_FLASH_FS_START_ADDRESS       // todo: better solution
    const FAPI_BOARD_HwPlatformInfosT *hwInfoPtr;
#endif
    FAPI_SYS_HandleT              flHdl;
#ifndef _LINUX_FAPI
    FAPI_FLASH_OpenParamsT        flashParams;
#else
    FAPEX_VRFLASH_OpenParamsT     flashParams;
#endif
    int32_t                       retVal;
    uint32_t                      addr;
    uint32_t                      flash_fs_start_offs;
    uint16_t                      cnt;
#ifndef _LINUX_FAPI
    FAPI_CLKPWR_OpenParamsT       clkpwr_open_params = {FAPI_CLKPWR_VERSION};
#else
    //FAPI_CLKPWR_OpenParamsT       clkpwr_open_params = {FAPI_CLKPWR_VERSION};
#endif

    if ( sysData != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    /* allocate system data memory */
    sysData = (SYS_DATA_S*)UTILS_Malloc(sizeof(SYS_DATA_S));
    if ( sysData == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(sysData, 0, sizeof(SYS_DATA_S));

    for(cnt = 0; cnt < OSD_MAX_NUM_LAYER; cnt++)
    {
        sysData->layerDat[cnt].inUse = FFALSE;
#ifndef _LINUX_FAPI
        sysData->layerDat[cnt].layerMutex = RTOS_CreateSemaphore(1);
#else
        FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_Init] cnt = %d\n", cnt);
        sysData->layerDat[cnt].layerMutex =  FAPEX_SYS_CreateSemaphore(1);
#endif
    }

    sysData->pwrMode = SYS_PWRMODE_STARTUP;
    sysData->profile = pSysCfg->appProfile;

    /* --------------------------------------------------------------------- */

    /* create general-purpose semaphore */
#ifndef _LINUX_FAPI
    sysData->mutexId = RTOS_CreateSemaphore(1);
    if(sysData->mutexId == (RTOS_SemaphoreT) NULL)
#else
    sysData->mutexId = FAPEX_SYS_CreateSemaphore(1);
    if(sysData->mutexId == (FAPEX_SYS_SemaphoreT) NULL)
#endif
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

    /* ----- Flash-related operations -------------------------------------- */

    (void)SYS_GetFlashDeviceType(&sysData->flashDeviceType);

    /* get common buffer for flash operations */
    sysData->bufSize = MAX_SECTOR_SIZE;
    sysData->pBuf = UTILS_Malloc(sysData->bufSize + 4);
    if(sysData->pBuf == NULL)
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    /* ensure 4-byte alignment! */
    sysData->pBuf =
        (uint8_t *) ((((uint32_t) (sysData->pBuf)) + 3) & 0xFFFFFFFC);

#ifndef _LINUX_FAPI
    flashParams.version     = FAPI_FLASH_VERSION;
#else
    flashParams.version     = FAPEX_VRFLASH_VERSION;
#endif
    flashParams.deviceType  = sysData->flashDeviceType;
    flashParams.firstSector = 0;
    flashParams.sectorCount = 0;
    flashParams.fsysFlag    = 0;
    flashParams.forceFlush  = 0;

#ifndef _LINUX_FAPI
    flHdl = FAPI_FLASH_Open(&flashParams, &retVal);
#else
    flHdl = FAPEX_VRFLASH_Open(&flashParams, &retVal);
#endif
    if(!flHdl)
    {
        /** TODO error handling */
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

#ifndef _LINUX_FAPI
    retVal = FAPI_FLASH_GetSectorStart(flHdl, 0);
#else
    retVal = FAPEX_VRFLASH_GetSectorStart(flHdl, 0);
#endif
    if(retVal < 0)
        return retVal;
    sysData->flashStartAddr = (uint32_t)retVal;

#ifndef _LINUX_FAPI
    sysData->flashSize = FAPI_FLASH_GetDeviceSize(flHdl);
#else
    sysData->flashSize = FAPEX_VRFLASH_GetDeviceSize(flHdl);
#endif
    if(!sysData->flashSize)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

#ifndef _LINUX_FAPI
    sysData->flashSectors = FAPI_FLASH_GetSectorCount(flHdl);
    if(sysData->flashSectors == FAPI_FLASH_ERR_BAD_PARAMETER)
#else
    sysData->flashSectors = FAPEX_VRFLASH_GetSectorCount(flHdl);
    if(sysData->flashSectors == FAPEX_VRFLASH_ERR_BAD_PARAMETER)
#endif
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    flash_fs_start_offs = FLASHFS_START_ADDRESS;
#ifdef ALLOW_SMALLER_S_FLASH_FS_START_ADDRESS       // todo: better solution
    hwInfoPtr = FAPI_BOARD_GetBoardInfo();
    if (  (hwInfoPtr)
        &&(hwInfoPtr->MagicAndVersion == FAPI_BOARD_HWCONFIG_VERSION ) )
    {
        addr = hwInfoPtr->typeOfFlashAvailable;
        if (addr & FAPI_BOARD_BITFLAG_SFLASH)
        {
            SYS_BootOsdPrint("=>force different SFlash-FS-offset\n");
            flash_fs_start_offs = ALLOW_SMALLER_S_FLASH_FS_START_ADDRESS;
        }
    }
#endif

    /* calculate start sector of user area on Flash */
    for(sysData->flashUsrStartSector = 0;
        sysData->flashUsrStartSector < (uint16_t)sysData->flashSectors;
        sysData->flashUsrStartSector++)
    {
#ifndef _LINUX_FAPI
        retVal = FAPI_FLASH_GetSectorStart(flHdl, sysData->flashUsrStartSector);
#else
        retVal = FAPEX_VRFLASH_GetSectorStart(flHdl, sysData->flashUsrStartSector);
#endif
        if(retVal < 0)
            return retVal;
        addr = (uint32_t)retVal;

        /* test if sector byte address is beyond or equal to requested
           start address */
        if(addr >= flash_fs_start_offs)
            break;
    }
    if(sysData->flashUsrStartSector >= (uint16_t)sysData->flashSectors)
    {
        FAPI_SYS_PRINT_MSG("sysData->flashUsrStartSector(%d),sysData->flashSectors(%d)\n",\
                            sysData->flashUsrStartSector,sysData->flashSectors);
        /* requested start address is beyond Flash memory size */
        //return SMARTGO_SYS_ERR_INTERNAL;
    }

#ifndef _LINUX_FAPI
    retVal = FAPI_FLASH_Close(flHdl);
#else
    retVal = FAPEX_VRFLASH_Close(flHdl);
#endif
    if(retVal != FAPI_OK)
    {
        /** TODO add error handling */
    }
    /* ----- init string operation module ---------------------------------- */

    retVal = STR_Init (pSysCfg);
    if ( retVal != FAPI_OK ) return retVal;

    /* ----- init device management ---------------------------------------- */

    retVal = SYS_DeviceInit();

    if ( retVal != FAPI_OK )
    {
        FAPI_SYS_PRINT_MSG("SYS_DeviceInit failed: %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }

    /* Set initial CI bypass status.
       NOTE: This is necessary to also support a situation where target
       platform has CI HW included but SmartGo-HD's CI option is disabled.
     */
    #if 0
    for(cnt = 0; cnt < 2; cnt++)
    {
        FD_GHW_ControlCiHw(cnt, FD_GHW_CI_BYPASS_EN, NULL, NULL);
    }
    #endif

#ifndef _LINUX_FAPI
    sysClkpwrHandle = FAPI_CLKPWR_Open(&clkpwr_open_params, &retVal);
    if ( !sysClkpwrHandle )
#else
    sysClkpwrHandle = FAPEX_CLKPWR_Open(FAPEXDEV_CLKPWR_MINOR_FIRST, O_RDWR, (FAPEXOPENARG3_T)NULL );
    if(sysClkpwrHandle < 0)
#endif
    {
        //FAPI_SYS_PRINT_DEBUG(1,"[SYS_Init] ERROR clock power open: (%d) - %s/n", retVal, SYS_GetErrorString(retVal) );
        FAPI_SYS_PRINT_DEBUG(1,"[SYS_Init] ERROR clock power open: (%d) \n", retVal);
        return retVal;
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1,"[SYS_Init] Success clock power open: (%d) \n", retVal);
    }

    /* add by Jim.Kang at 2010-7-29 */
    /* power on all 6 video DACs. */
    uint64_t component = FAPEX_CLKPWR_DAC0 | FAPEX_CLKPWR_DAC1 | FAPEX_CLKPWR_DAC2 | FAPEX_CLKPWR_DAC3 | FAPEX_CLKPWR_DAC4 | FAPEX_CLKPWR_DAC5;
    retVal = FAPEX_CLKPWR_Ioctl(sysClkpwrHandle,FAPEX_IOCCOM_CLKPWR_START,(FAPEXIOCARG3_T)&component);
    if(retVal < 0)
        FAPI_SYS_PRINT_MSG("ERROR:[main][SYS_Init] Video DACS power up %d\n",retVal);

    return FAPI_OK;
}

/*---------------------------------------------------------------------------*/
/* start thread function                                                     */
/*---------------------------------------------------------------------------*/
FSTATIC void SYS_StartThread (void* opt)
{
    int32_t             retVal = FAPI_OK;
    fbool_t             showSplash = FFALSE;
    SYS_PWRMODE_E       powerUpMode;

    FAPEX_SYS_ThreadT   sys_user_thread_id;

    /* print application title to console */
    SYS_PrintStartupHdr();

    /* --------------------------------------------------------------------- */

    /* NOTE: Order of initialisations is crucial, due to some dependencies. */

    do
    {
        // TODO: dieser Check ist an der falschen Stelle!
        /* check important configuration arguments */
        /*
        if ( (hwConfig.feNums < 1) || (hwConfig.feNums > RSRC_FE_MAX) )
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;
        }
        */
        /* initialise system parameters */
        retVal = SYS_Init(&sysConfig);
        if(retVal != FAPI_OK)
            break;

        FAPI_SYS_PRINT_MSG("[main] SYS_Init\n");

        retVal = EVT_Init();
        if(retVal != FAPI_OK)
            break;

        FAPI_SYS_PRINT_MSG("[main] EVT_Init\n");

        //retVal = FLASH_Init(THREAD_STACKSZ_FLASHFS, THREAD_PRIO_FLASHFS);
        if(retVal != FAPI_OK)
            break;

        FAPI_SYS_PRINT_MSG("[main] FLASH_Init\n");

       // retVal = FCDATA_Init();
        if(retVal != FAPI_OK)
            break;

        FAPI_SYS_PRINT_MSG("[main] FCDATA_Init\n");

        retVal = RSRC_Init();
        if(retVal != FAPI_OK)
            break;

        FAPI_SYS_PRINT_MSG("[main] RSRC_Init\n");

        /* load basic settings */
        retVal = SETTINGS_StartupInit();
        if(retVal != FAPI_OK)
            break;

        FAPI_SYS_PRINT_MSG("[main] SETTINGS_StartupInit\n");

        /* init AV module */
        retVal = AV_Init();
        if ( retVal != FAPI_OK )
            break;

        FAPI_SYS_PRINT_MSG("[main] AV_Init\n");

        /* get power-up mode requested by application */
        if(sysConfig.appGetStartupOperationMode != NULL)
        {
            powerUpMode = sysConfig.appGetStartupOperationMode();
        }
        else
        {
            powerUpMode = SYS_PWRMODE_DOWN;
        }

        #ifdef APPL_DEBUG_MODE

        /* NOTE: In debug mode, splash screen is shown even if the box starts
              in standby mode. */
        showSplash = FTRUE;

        #else

        if ( powerUpMode == SYS_PWRMODE_UP ) showSplash = FTRUE;

        #endif /* APPL_DEBUG_MODE */

        if ( showSplash )
        {
            //(void)AV_PowerupScreenShow(FTRUE);
        }

        /* --------------------------------------------------------------------- */

        #ifdef ENABLE_BOOTOSD
        // TODO: to be supported!
        SYS_BootOsdInit(hdSys);
        #endif

        retVal = TIME_Init();
        if(retVal != FAPI_OK)
            break;

        FAPI_SYS_PRINT_MSG("TIME_Init\n");

        SYS_BootOsdPrint("Initialise system modules...");

        /* init and load settings */
        retVal = SETTINGS_Init();
        if(retVal != FAPI_OK)
            break;

        #ifdef APPL_FATFS_ENABLED
        (void)f_enterFS();
        #endif /* APPL_FATFS_ENABLED */

        #ifdef APPL_HDD_ENABLED

        retVal = HDD_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_HDD_ENABLED */

        #ifdef APPL_USB_ENABLED

        retVal = USB_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_USB_ENABLED */

        #ifdef APPL_ETHERNET_ENABLED

        retVal = ETH_Init ();
        if ( retVal != FAPI_OK )
        {
            break;
        }

        #endif /* APPL_ETHERNET_ENABLED */

        #ifdef APPL_PVR_ENABLED

        retVal = PVRSYS_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_PVR_ENABLED */

        #ifdef APPL_FESAT_ENABLED

        retVal = CFGSAT_Init();
        if(retVal != FAPI_OK)
            break;

        #ifdef APPL_USALS_ENABLED

        retVal = USALS_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_USALS_ENABLED */

        #endif /* APPL_FESAT_ENABLED */

        retVal = TPLIST_Init();
        if(retVal != FAPI_OK)
            break;

        #ifdef APPL_TTXT_ENABLED

        retVal = TTXSUBTT_Init();
        if(retVal != FAPI_OK)
            break;

        retVal = TTX_Init(&(sysConfig.ttx));
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_TTXT_ENABLED */

        #ifdef APPL_SUBTT_ENABLED

        retVal = SUBTT_Init(&(sysConfig.subtt));
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_SUBTT_ENABLED */

        #ifdef APPL_JPEG_ENABLED

        retVal = JPEG_Init(&(sysConfig.jpeg));
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_JPEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        retVal = MP3_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_MP3_ENABLED */

        /* load/initialise service lists */
        retVal = SLIST_Init();
        if(retVal != FAPI_OK)
            break;

        #ifdef APPL_RECTIMER_ENABLED

        retVal = RECTIMER_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_RECTIMER_ENABLED */

        /* init key handler */
        retVal = KEYIN_Init();
        if(retVal != FAPI_OK)
            break;

        retVal = EIT_Init();
        if(retVal != FAPI_OK)
            break;

        #ifdef APPL_UPDATE_ENABLED

        retVal = UPDATE_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_UPDATE_ENABLED */

        #ifdef APPL_PARENTAL_ENABLED

        retVal = PCTRL_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_PARENTAL_ENABLED */

        #ifdef APPL_FILEOP_ENABLED

        retVal = FILEOP_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_FILEOP_ENABLED */

        #ifdef APPL_MHEG_ENABLED

        retVal = MHEG_Init(&(sysConfig.mheg));
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_MHEG_ENABLED */

        #ifdef APPL_OAD_ENABLED

        retVal = OAD_Init(&(sysConfig.oad));
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_OAD_ENABLED */

        #ifdef APPL_PIP_ENABLED
        retVal = PIP_Init(&(sysConfig.pip));
        if(retVal != FAPI_OK)
            break;
        #endif /* APPL_PIP_ENABLED */

        #ifdef APPL_RFMOD_ENABLED

        retVal = RFMOD_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_RFMOD_ENABLED */

        #ifdef APPL_CI_ENABLED

        retVal = CISYS_Init(&(sysConfig.ci), &hwConfig);
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_CI_ENABLED */

        #ifdef APPL_CONAX_ENABLED

        retVal = CONAX_CA_Init();
        if(retVal != FAPI_OK)
            break;

        #endif /* APPL_CONAX_ENABLED */

        /* ----------------------------------------------------------------- */
        if ( powerUpMode == SYS_PWRMODE_UP )
        {
            retVal = SYS_GoUp();
            if(retVal != FAPI_OK)
                break;
        }
        else
        {
            retVal = SYS_GoDown();
            if(retVal != FAPI_OK)
                break;
            // Hide boot sceen because SYS_GoDown() will not stop it here!
            (void)AV_IFrameHide();
        }

        // TODO: to be supported!
        #ifdef ENABLE_BOOTOSD
        SYS_BootOsdExit();
        #endif
        /* ----- spawn user control thread --------------------------------- */

        retVal = SYS_ThreadCreate(SYS_UserThread, NULL,
                                  THREAD_PRIO_APPLICATION,
                                  THREAD_STACKSZ_APPLICATION,
                                  THREAD_NAME_APPLICATION, &sys_user_thread_id);
        if(retVal != FAPI_OK)
            break;
    }
    while(0);

    if ( retVal != FAPI_OK )
    {
        SYS_BootOsdPrint("System setup failed!");
        FAPI_SYS_PRINT_DEBUG(1,"System setup failed: %s\n", SYS_GetErrorString(retVal));

        /* Stop without exit */
        while ( 1 ) {}
    }

    FAPEX_SYS_WaitEndThread(sys_user_thread_id);

    /* destroy this thread */
#ifndef _LINUX_FAPI
    (void)RTOS_DestroyThread(RTOS_GetThread());
#else
    (void)FAPEX_SYS_DestroyThread (FAPEX_SYS_GetThread());
#endif
}


/*!
*******************************************************************************
**
** \brief  Modify the system's operation mode
**
** Puts the system either in power-down (standby) mode or returns from standby
** to active mode. In standby, all disposable SmartMPEG modules like DACs are
** shut down. Nevertheless, OS operation continues normally, though some
** threads are in sleep mode. A reduced set of system events like remote
** control keys is still provided and can be normally handled in the user
** thread.
**
** \param  reqMode  Requested operation mode: SYS_PWRMODE_DOWN or SYS_PWRMODE_UP
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SYS_GetOperationMode SYS_Reboot
******************************************************************************/
int32_t SYS_SetOperationMode(SYS_PWRMODE_E reqMode)
{
    int32_t                       retVal;

    if(sysData->pwrMode == reqMode)
        return FAPI_OK;

    switch (reqMode)
    {
        case SYS_PWRMODE_DOWN:
            retVal = SYS_GoDown();
            break;

        case SYS_PWRMODE_UP:
            retVal = SYS_GoUp();
            break;

        default:
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;
    }

    return retVal;
}

SYS_PWRMODE_E SYS_GetOperationMode(void)
{
    return sysData->pwrMode;
}

int32_t SYS_GetPresentationState(SYS_PRESSTATE_E * state)
{
    RSRC_STATUS_S* pStatus = RSRC_GetStatus();

    if(state == NULL)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if((pStatus->opStatus & RSRC_OP_LIVE) != 0)
    {
        *state = SYS_PRESSTATE_LIVETV;
    }
    else if((pStatus->opStatus & RSRC_OP_PLAY) != 0)
    {
        *state = SYS_PRESSTATE_PLAYBACK;
    }
    else
    {
        *state = SYS_PRESSTATE_NONE;
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Stop current presentation.
**
** \param  op       Bitfield of potential (presentation) operations to be
**                  stopped if active.
** \param  freeze   Freeze picture if supported by current presentation
**                  operation.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SYS_GetPresentationState
******************************************************************************/
int32_t SYS_StopPresentation (RSRC_OP op, fbool_t freeze)
{
    int32_t         retVal;
    SYS_PRESSTATE_E presState;

    retVal = SYS_GetPresentationState (&presState);
    if ( retVal == FAPI_OK )
    {
        switch ( presState )
        {
            case SYS_PRESSTATE_LIVETV:
                if ( (op & RSRC_OP_LIVE) != 0 )
                {
                    if ( freeze )
                    {
                        retVal = AV_Freeze();
                    }
                    else
                    {
                        retVal = AV_Stop();
                    }
                }
                break;

            #ifdef APPL_PVR_ENABLED

            case SYS_PRESSTATE_PLAYBACK:
                if ( (op & RSRC_OP_PLAY) != 0 )
                {
                    retVal = PVRSYS_PlayStop();
                }
                break;

            #endif /* APPL_PVR_ENABLED */

            #ifdef APPL_MPEG_ENCODER_ENABLED

            case SYS_PRESSTATE_EXTINPUT:
                if ( (op & RSRC_OP_MPENCMONITOR) != 0 )
                {
                    retVal = MPENC_MonitorStop();
                }
                break;

            #endif /* APPL_MPEG_ENCODER_ENABLED */

            default:
                /* do nothing */
                break;
        }
    }

    return retVal;
}

int32_t SYS_SetAppProfile(SYS_APP_PROFILE_E profile)
{
    if(profile >= SYS_APP_PROFILE_NUMS)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    sysData->profile = profile;

    // TODO handle profile specific settings

    return FAPI_OK;
}

SYS_APP_PROFILE_E SYS_GetAppProfile(void)
{
    DBG_Assert( sysData != NULL );

    return sysData->profile;
}

/*!
*******************************************************************************
**
** \brief  Reboot the system.
**
** This function performs a reset on the SmartMPEG.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SYS_Reboot(void)
{
    int32_t                 result;

    // Start the watchdog at the maximum value. Box will reset.
#ifndef _LINUX_FAPI
    result = FAPI_CLKPWR_StartWatchdog(sysClkpwrHandle, FAPI_CLKPWR_WATCHDOG_MAX_VALUE);
    if ( result!= FAPI_OK )
#else
    result = FAPEX_CLKPWR_Ioctl(sysClkpwrHandle, FAPEX_IOCCOM_CLKPWR_WATCHDOG_START, (FAPEXIOCARG3_T)FAPEX_CLKPWR_WATCHDOG_MAX_VALUE);
    if ( result < 0 )
#endif
    {
        FAPI_SYS_PRINT_DEBUG(1,"[SYS_Reboot] ERROR: (%d) - %s\n",result,
                                         SYS_GetErrorString(result));
        return result;
    }

    for (;;)
    {
    }
}


FSTATIC int32_t SYS_GoDown(void)
{
#ifdef APPL_LOW_POWER_STANDBY
    int32_t err_code = FAPI_OK;
#endif

    //FD_POWER_CONFIG_S powerConfig;

    #ifdef APPL_FILEOP_ENABLED
    (void)FILEOP_Abort();
    #endif /* APPL_FILEOP_ENABLED */

    #ifdef APPL_PIP_ENABLED
    PIP_Hide();
    #endif /* APPL_PIP_ENABLED */

    #ifdef APPL_PVR_ENABLED
    (void)PVRSYS_Shutdown();
    #endif /* APPL_PVR_ENABLED */

    /* stop all decoding, clear screen */
    (void)AV_Mute (FTRUE);

    #ifdef APPL_RFMOD_ENABLED
    RFMOD_Close();
    #endif /* APPL_RFMOD_ENABLED */

    (void)AV_Shutdown();
    (void)AV_Nop();         /* sleep until AV-shutdown has been performed */

    #ifdef APPL_CI_ENABLED
    (void)CISYS_Shutdown();
    #endif /* APPL_CI_ENABLED */

    #ifdef APPL_CONAX_ENABLED
    CONAX_CA_Shutdown();
    #endif /* APPL_CONAX_ENABLED */

    #ifdef APPL_PARENTAL_ENABLED
    (void)PCTRL_Shutdown();
    #endif /* APPL_PARENTAL_ENABLED */

    #ifdef APPL_TTXT_ENABLED
    TTX_Shutdown();
    #endif /* APPL_TTXT_ENABLED */

    #ifdef APPL_RECTIMER_ENABLED
    RECTIMER_EnterStandby();
    #endif /* APPL_RECTIMER_ENABLED */

    #ifdef APPL_ETHERNET_ENABLED
    (void)ETH_Shutdown();
    #endif /* APPL_ETHERNET_ENABLED */

    (void)SYS_DeviceShutdown();

    (void)SYS_ControlFpSymbol(FP_SYMB_STDBY, FTRUE);

    /* ----- put device into low power mode -------------------------------- */

#ifdef APPL_LOW_POWER_STANDBY

    /* Disables audio (also processor), tsd, video and reduces ARM speed to 162 MHz. */
#ifndef _LINUX_FAPI
    sysStandbyMode = FAPI_CLKPWR_BASIC_POWER_DOWN;

    sysStandbyMode |= FAPI_CLKPWR_ARM;
    sysStandbyMode |= FAPI_CLKPWR_AUDIO;
    sysStandbyMode |= FAPI_CLKPWR_AUDIO_PROCESSOR;
  //sysStandbyMode |= FAPI_CLKPWR_AXI;                 // on power down - hardware reset required
  //sysStandbyMode |= FAPI_CLKPWR_DDR2_SDRAM;          // on power down - hardware reset required
  //sysStandbyMode |= FAPI_CLKPWR_DMA;                 // on power down - hardware reset required
    sysStandbyMode |= FAPI_CLKPWR_ETH;
    sysStandbyMode |= FAPI_CLKPWR_I2C;
    sysStandbyMode |= FAPI_CLKPWR_ICC_APB_EFUSE;
  //sysStandbyMode |= FAPI_CLKPWR_IRQ_CONTROL;         // on power down - hardware reset required
    sysStandbyMode |= FAPI_CLKPWR_GPIO;
    sysStandbyMode |= FAPI_CLKPWR_SFLASH;
    sysStandbyMode |= FAPI_CLKPWR_SSP;
    sysStandbyMode |= FAPI_CLKPWR_TSD;
  //sysStandbyMode |= FAPI_CLKPWR_UART_FPC;            // Needed for IR module
  //sysStandbyMode |= FAPI_CLKPWR_UPI;
    sysStandbyMode |= FAPI_CLKPWR_USB;
    sysStandbyMode |= FAPI_CLKPWR_VIDEO;
  //sysStandbyMode |= FAPI_CLKPWR_CRITICAL;            // RAM and ARM (system stop)
#else
    sysStandbyMode = FAPEX_CLKPWR_BASIC_POWER_DOWN;

    sysStandbyMode |= FAPEX_CLKPWR_ARM;
    sysStandbyMode |= FAPEX_CLKPWR_AUDIO;
    sysStandbyMode |= FAPEX_CLKPWR_AUDIO_PROCESSOR;
  //sysStandbyMode |= FAPEX_CLKPWR_AXI;                 // on power down - hardware reset required
  //sysStandbyMode |= FAPEX_CLKPWR_DDR2_SDRAM;          // on power down - hardware reset required
  //sysStandbyMode |= FAPEX_CLKPWR_DMA;                 // on power down - hardware reset required
    sysStandbyMode |= FAPEX_CLKPWR_ETH;
    sysStandbyMode |= FAPEX_CLKPWR_I2C;
    sysStandbyMode |= FAPEX_CLKPWR_ICC_APB_EFUSE;
  //sysStandbyMode |= FAPEX_CLKPWR_IRQ_CONTROL;         // on power down - hardware reset required
    sysStandbyMode |= FAPEX_CLKPWR_GPIO;
    sysStandbyMode |= FAPEX_CLKPWR_SFLASH;
    sysStandbyMode |= FAPEX_CLKPWR_SSP;
    sysStandbyMode |= FAPEX_CLKPWR_TSD;
  //sysStandbyMode |= FAPEX_CLKPWR_UART_FPC;            // Needed for IR module
  //sysStandbyMode |= FAPEX_CLKPWR_UPI;
    sysStandbyMode |= FAPEX_CLKPWR_USB;
    sysStandbyMode |= FAPEX_CLKPWR_VIDEO;
  //sysStandbyMode |= FAPEX_CLKPWR_CRITICAL;            // RAM and ARM (system stop)

    /* all 6 video DACs. add by Jim.Kang at 2010-7-28 */
    sysStandbyMode |= FAPEX_CLKPWR_DAC0;
    sysStandbyMode |= FAPEX_CLKPWR_DAC1;
    sysStandbyMode |= FAPEX_CLKPWR_DAC2;
    sysStandbyMode |= FAPEX_CLKPWR_DAC3;
    sysStandbyMode |= FAPEX_CLKPWR_DAC4;
    sysStandbyMode |= FAPEX_CLKPWR_DAC5;
#endif

#ifndef _LINUX_FAPI
    err_code = FAPI_CLKPWR_Standby(sysClkpwrHandle, sysStandbyMode);
    if(err_code != FAPI_OK)
#else
    err_code = FAPEX_CLKPWR_Ioctl(sysClkpwrHandle, FAPEX_IOCCOM_STOP, (FAPEXIOCARG3_T)sysStandbyMode);
    if(err_code < 0)
#endif
    {
        FAPI_SYS_PRINT_DEBUG(1,"[SYS_GoDown] ERROR FAPI_CLKPWR_Standby(): (%d) - %s\n", err_code,
                                                            SYS_GetErrorString(err_code));
    }
#endif
    sysData->pwrMode = SYS_PWRMODE_DOWN;
    return FAPI_OK;
}


FSTATIC int32_t SYS_GoUp(void)
{

#ifdef APPL_LOW_POWER_STANDBY
    int32_t err_code = FAPI_OK;

#ifndef _LINUX_FAPI
    err_code = FAPI_CLKPWR_Wakeup(sysClkpwrHandle, sysStandbyMode);
    if(err_code != FAPI_OK)
#else
    err_code = FAPEX_CLKPWR_Ioctl(sysClkpwrHandle, FAPEX_IOCCOM_CLKPWR_START, (FAPEXIOCARG3_T)&sysStandbyMode);
    if(err_code < 0)
#endif
    {
        FAPI_SYS_PRINT_DEBUG(1,"[SYS_GoUp] ERROR on wakeup: (%d) - %s\n", err_code, SYS_GetErrorString(err_code));
    }

    sysStandbyMode = 0;
#endif

    (void)SYS_ControlFpSymbol(FP_SYMB_STDBY, FFALSE);

    FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GoUp] sending AV_CMD_WAKEUP in SYS_GoUp\n");
    (void)AV_Wakeup();

    FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GoUp] sending AV_CMD_NOP in SYS_GoUp\n");
    (void)AV_Nop();                /* sleep until AV-wakeup has been performed */\

    (void)SYS_DeviceWakeup();

    #ifdef APPL_ETHERNET_ENABLED
    (void)ETH_Wakeup();
    #endif /* APPL_ETHERNET_ENABLED */

    #ifdef APPL_PVR_ENABLED
    (void)PVRSYS_Wakeup();
    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_PARENTAL_ENABLED
    (void)PCTRL_Wakeup();
    #endif /* APPL_PARENTAL_ENABLED */

    #ifdef APPL_CONAX_ENABLED
    CONAX_CA_Init();
    #endif /* APPL_CONAX_ENABLED */

    #ifdef APPL_CI_ENABLED
    (void)CISYS_Wakeup();
    #endif /* APPL_CI_ENABLED */

    #ifdef APPL_RFMOD_ENABLED
    RFMOD_Open();
    #endif /* APPL_RFMOD_ENABLED */

    sysData->pwrMode = SYS_PWRMODE_UP;

    FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GoUp] GoUp Done\n");

    return FAPI_OK;
}

FSTATIC void SYS_UserThread(void *opt)
{
    int32_t                       retVal;
    uint8_t                       evtBuf[EVT_SIZE_MAX];
    void                         *pEvent = evtBuf;

    SYS_DevicePrepareThread();

    /* Initialise application */
    if(sysConfig.appInit != NULL)
    {
        retVal = sysConfig.appInit();

        if(retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1,"Application initialisation failed: %d!\n", retVal);
            while (1) {}
        }
    }

    while ( 1 )
    {
        if ( EVT_Get(pEvent, FTRUE) != FAPI_OK )
        {
            //FAPI_SYS_PRINT_DEBUG(1,"EVT_Get: ERROR \n");
            continue;
        }
        if ( SYS_InternalEvtHandling(((SYS_EVTHDR_S*)pEvent)->type, pEvent) )
        {
            /* Pass event to application if not blocked by internal event
               handling. */
            if ( sysConfig.appEventReceived != NULL )
            {
                sysConfig.appEventReceived(((SYS_EVTHDR_S *) pEvent)->type,
                                           pEvent);
            }
        }
    }
}

FSTATIC fbool_t SYS_InternalEvtHandling(SYS_EVTTYPE_E type, const void *data)
{
    const SYS_EVT_SERVICE_S*    pEvtServ;
    fbool_t                     letEventPass = FTRUE;
    #ifdef APPL_MHEG_ENABLED
    SYS_SERVLOC_S               servLoc;
    #endif
    //SYS_HDTV_STANDARD           hdTvStandard;
    //SYS_SDTV_STANDARD           sdTvStandard;

    /* perform internal event handling */
    switch (type)
    {
        #if 0

        /* NOTE: Not supported yet! */

        case EVT_IN_SYS_CHANGE:
            /* Input TV standard has changed: Trigger new check in order to
               find out if output TV standard has to be changed. */
            if(SETTINGS_OutputTvStandardGet(&hdTvStandard,&sdTvStandard) == FAPI_OK)
            {
                AV_SetTvOutputSystem(hdTvStandard,sdTvStandard);
            }
            break;

        #endif

        case EVT_PMT_RECEIVED:
            #ifdef APPL_PVR_ENABLED
            if((((SYS_EVT_PMT_S*)data)->operation &
                (RSRC_OP_RECANY | RSRC_OP_PLAY)) != 0)
            {
                PVRSYS_PMTReceived(data);
            }
            #endif /* APPL_PVR_ENABLED */
            break;

        case EVT_CAT_RECEIVED:
            #ifdef APPL_PVR_ENABLED
            PVRSYS_CATReceived(data);
            #endif /* APPL_PVR_ENABLED */
            break;

        case EVT_TIME_UPDATE:
            #ifdef APPL_HDD_ENABLED
            HDD_Tasks();
            #endif /* APPL_HDD_ENABLED */

            #ifdef APPL_USB_ENABLED
            USB_Tasks();
            #endif /* APPL_USB_ENABLED */
            break;

        case EVT_TIMEZONE_CHANGE:
            (void)SETTINGS_TimeOffsetSet(((SYS_EVT_TIMEZONE_S*)data)->
                                         utcOffset, FFALSE);
            break;

        case EVT_TIMER_EXPIRED:
            if ( ((SYS_EVT_TIMER_S*)data)->type == TIMER_SINGLE )
            {
                (void)TIME_SingleTimerClose(((SYS_EVT_TIMER_S*)data)->num);
            }
            break;

        case EVT_SERVICE_SELECTED:
            pEvtServ = data;

            AV_UpdateLiveStatus(pEvtServ->status);
            break;

        case EVT_SERVICE_RUNSTAT:
            AV_UpdateRunningStatus(((SYS_EVT_RUNSTAT_S*)data)->status);
            break;

        case EVT_DEVICE_STATUS:

            if ( SYS_DeviceStatusHandling(data) )
            {
                switch (((SYS_EVT_DEVSTAT_S*)data)->device)
                {
                    #ifdef APPL_HDD_ENABLED

                    case SYS_DEV_HDD:
                        HDD_MediaStatusHandling((SYS_EVT_DEVSTAT_S*)data);
                        break;

                    #endif /* APPL_HDD_ENABLED */

                    #ifdef APPL_USB_ENABLED

                    case SYS_DEV_USB:
                        USB_MediaStatusHandling((SYS_EVT_DEVSTAT_S*)data);
                        break;

                    #endif /* APPL_USB_ENABLED */

                    default:
                        break;
                }
            }
            else
            {
                letEventPass = FFALSE;
            }
            break;

        #ifdef APPL_PVR_ENABLED

        case EVT_PVR_PLAYBACK:
        case EVT_PVR_RECORD:
            /* do nothing */
            break;

        case EVT_PVR_PLAYEVENT:
            PVRSYS_PlayEvent (((SYS_EVT_PLAYEVT_S *) data)->event);
            break;

        case EVT_PVR_RECEVENT:
            PVRSYS_RecEvent (((SYS_EVT_RECEVT_S*)data)->recIdx,
                            ((SYS_EVT_RECEVT_S*)data)->event);
            break;

        #endif /* APPL_PVR_ENABLED */

        #ifdef APPL_FILEOP_ENABLED

        case EVT_FILEOP_STATUS:
            FILEOP_HandleEvent((const FILEOP_EVT_S *) data);
            break;

        #endif /* APPL_FILEOP_ENABLED */

        #ifdef APPL_PARENTAL_ENABLED

        case EVT_EIT_UPDATE:
            PCTRL_EitEventHandling(((SYS_EVT_EITEVT_S *) data)->servIdx);
            break;

        #endif /* APPL_PARENTAL_ENABLED */

        #ifdef APPL_CI_ENABLED

        case EVT_CISYS_STATUSCHG:
            CISYS_HandleSlotStatusEvt((const CISYS_EVT_STATUS_S *) data);
            break;

        #endif /* APPL_CI_ENABLED */

        case EVT_AFD_WSS_CHANGE_NOTIFY: // AFD and WSS updated in viscale
            AV_OutputFormatUpdate(FFALSE);

            #ifdef APPL_MHEG_ENABLED
            MHEG_DisplayHasChanged();
            #endif /* APPL_MHEG_ENABLED */

            letEventPass = FFALSE;
            break;

        case EVT_PIC_FORMAT_CHANGE:
            AV_OutputFormatUpdate (FFALSE);

            #ifdef APPL_MHEG_ENABLED
            MHEG_DisplayHasChanged();
            #endif /* APPL_MHEG_ENABLED */
            break;

        #ifdef APPL_MHEG_ENABLED
        case EVT_MHEG_OPEN:
            MHEGINT_Open();
            MHEGINT_Start();
            letEventPass = FFALSE;
            break;
        case EVT_MHEG_CLOSE:
            MHEGINT_Close();
            letEventPass = FFALSE;
            break;
        case EVT_MHEG_START:
            MHEGINT_Start();
            letEventPass = FFALSE;
            break;
            /*
        case EVT_MHEG_APPMMI_MSG:
            MHEG_APPMMI_AckMessage(((MHEG_EVT_APPMMI_MSG_S *) data)->sess,
                                   ((MHEG_EVT_APPMMI_MSG_S *) data)->type,
                                   ((MHEG_EVT_APPMMI_MSG_S *) data)->msg);
            letEventPass = FFALSE;
            break;
            */
        case EVT_MHEG_STOP:
            MHEGINT_Stop(FTRUE); // to be checked, maybe needs to be FTRUE?
            letEventPass = FFALSE;
            break;

        case EVT_MHEG_SHOW:
            (void) MHEGINT_Show();
            letEventPass = FFALSE;
            break;

        case EVT_MHEG_TUNE:
            MHEGINT_Stop(FFALSE);
            servLoc = SLIST_GetListReference ((SLIST_SERVIDX)(((MHEG_EVT_TUNE_S*)data)->index));
            (void) AV_SelectService(&servLoc, NULL,((MHEG_EVT_TUNE_S*)data)->info_ctrl, FTRUE);
            letEventPass = FFALSE;
            break;

        #endif /* APPL_MHEG_ENABLED */

        #ifdef APPL_MP3_ENABLED

        case EVT_MP3_STATUS:
            MP3_EventHandling ((const MP3_EVT_STATUS_S*)data);
            break;

        #endif /* APPL_MP3_ENABLED */

        #ifdef APPL_USB_ENABLED

        case EVT_USB_CONNECT_STATUS:
            USB_ConnectStatusHandling (data);
            break;

        #endif /* APPL_USB_ENABLED */

        #ifdef APPL_ETHERNET_ENABLED

        case EVT_ETH_LINK:
            break;

        case EVT_ETH_DHCP:
            break;

        #endif
        default:
            break;
    }

    return letEventPass;
}

/* ----- utility functions -------------------------------------------------- */

/*
Copies up to size characters from source to target. If source string ends
before, copying is aborted. Resulting string is always terminated, even if
source string size is greater than size.
!!! Watchout: size character includes termination '\0'

Returns number of characters copied, NOT including termination.
*/
uint32_t UTILS_strncpy(char_t *target, const char_t *source, uint32_t size)
{
    uint32_t                      cnt = 0;

    if(size == 0)
        return 0;
    if(!(source && target))
        return 0;

    while((*source) && ((size--) > 1))
    {
        *(target++) = *(source++);
        cnt++;
    }

    *target = '\0';

    return cnt;
}

/*
    counts and returns number of printable characters in source string
    the extended ASCII character set is used as reference
*/
uint32_t UTILS_strlen(const char_t *str)
{
    uint32_t                      cnt = 0;

    while(*str++ >= ' ')
        cnt++;

    return cnt;
}

void UTILS_remWhiteCharAtEnd(char_t *string, uint32_t size)
{
    char_t *strPtr;

    if(size == 0)
        return;
    if(string == NULL)
        return;

    strPtr = string + size - 1;
    do
    {
        /* -e48 Bad Type switched of for isgraph() */
        /*lint -save -e48 -e409 */
        if(isgraph(*strPtr))
        {
            *(strPtr + 1) = 0;
            break;
        }
        /*lint -restore */
        strPtr--;
    }
    /*lint -save -e946 */
    while(strPtr >= string);
    /*lint -restore */

}

int32_t UTILS_strcmp(const char_t *s1, const char_t *s2)
{
    while(1)
    {
        if((*s1) == (*s2))
        {
            if((*s1) == 0)
                return 0;
            else
            {
                s1++;
                s2++;
                continue;
            }
        }
        else
        {
            if((*s1) > (*s2))
                return 1;
            else
                return -1;
        }
    }
}

int32_t UTILS_strncmpi(const char_t *s1, const char_t *s2, int32_t numChars)
{
    uint8_t finite = (numChars >= 0);

    while(1)
    {
        if(finite && (numChars-- <= 0))
            break;

        if((*s1) == (*s2))
        {
            if((*s1) == 0)
                return 0;
            else
            {
                s1++;
                s2++;
                continue;
            }
        }
        else if(((*s1) >= 0x41) && ((*s1) <= 0x5A))
        {
            if((*s2) == ((*s1) + 0x20))
            {
                s1++;
                s2++;
                continue;
            }
        }
        else if(((*s1) >= 0x61) && ((*s1) <= 0x7A))
        {
            if((*s1) == ((*s2) + 0x20))
            {
                s1++;
                s2++;
                continue;
            }
        }

        if((*s1) > (*s2))
            return 1;
        else
            return -1;
    }

    return 0;
}

uint64_t UTILS_GetTimerStampInterval(uint64_t timeStart, uint64_t timeEnd)
{
    if(timeStart <= timeEnd)
        return (timeEnd - timeStart);
    else
        return ((uint64_t)0xFFFFFFFFFFFFFFFF - timeStart + timeEnd + 1);
}

/*!
*******************************************************************************
** \brief   Return the current system time in mSec.
**
** \return  The Timestamp.
**
** \sa
**          - FAPI_TIMER_GetTimeStamp()
**          - #FAPI_TIMER_RESOLUTION_1_MSEC
**
*******************************************************************************
*/
uint64_t UTILS_GetTimerStamp(void)
{
#ifndef _LINUX_FAPI
    return FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC);
#else
    return FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
#endif
}

/*!
*******************************************************************************
** \brief   Gets NON zero'd cached memory of size 'size'.
**
** \param   size    requested memory size.
**
** \return  Pointer to alloced memory, NULL in case of error.
**
** \sa
**          - UTILS_Zalloc()
**          - UTILS_Free()
**
*******************************************************************************
*/
void* UTILS_Malloc(uint32_t size)
{
    void* mem = NULL;

    mem = (void*)FAPI_SYS_MALLOC_CACHED(size);
    if(mem == NULL)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[MALLOC] failed to alloc %d bytes\n",size);
    }
    return mem;
}

/*!
*******************************************************************************
** \brief   Gets zero'd cached memory of size 'size'.
**
** \param   size    requested memory size.
**
** \return  Pointer to alloced memory, NULL in case of error.
**
** \sa
**          - UTILS_Malloc()
**          - UTILS_Free()
**
*******************************************************************************
*/
void* UTILS_Zalloc(uint32_t size)
{
    uint8_t* buf;

    buf = (uint8_t*)UTILS_Malloc(size);
    if ( buf == NULL )
        return NULL;
    memset(buf, 0 ,size);

    return buf;
}

/*!
*******************************************************************************
** \brief   Release the memory.
**
** \param   p   Memory to be released.
**
** \sa
**          - UTILS_Malloc()
**          - UTILS_Zalloc()
**
*******************************************************************************
*/
void UTILS_Free(void *p)
{
    FAPI_SYS_FREE(p);
}

#define DUMP_BYTESPERLINE  8

void UTILS_MemDump(const char_t *str, const uint8_t * pBuf, uint32_t len)
{
    uint32_t                      cnt = 0;
    uint32_t                      lineCnt;
    uint32_t                      bytesPerLine;

    FAPI_SYS_PRINT_MSG("\nMEMORY DUMP: %s\n", str);
    FAPI_SYS_PRINT_MSG("ADDR: 0x%p\n", pBuf);
    FAPI_SYS_PRINT_MSG("LEN:  %i bytes\n", len);

    FAPI_SYS_PRINT_MSG("------------------------------------------------\n");

    while(cnt < len)
    {
        bytesPerLine = ((len - cnt) > DUMP_BYTESPERLINE) ? DUMP_BYTESPERLINE
            : (len - cnt);

        FAPI_SYS_PRINT_MSG("%4i - ", cnt);

        for(lineCnt = 0; lineCnt < bytesPerLine; lineCnt++)
        {
            FAPI_SYS_PRINT_MSG("%02X ", pBuf[lineCnt]);
        }

        for(; lineCnt < DUMP_BYTESPERLINE; lineCnt++)
        {
            FAPI_SYS_PRINT_MSG("   ");
        }

        FAPI_SYS_PRINT_MSG("- ");

        for(lineCnt = 0; lineCnt < bytesPerLine; lineCnt++)
        {
            if(pBuf[lineCnt] >= 0x20)
            {
                FAPI_SYS_PRINT_MSG("%c ", pBuf[lineCnt]);
            }
            else
            {
                FAPI_SYS_PRINT_MSG("_ ");
            }
        }

        FAPI_SYS_PRINT_MSG("\n");

        cnt += bytesPerLine;
        pBuf += bytesPerLine;
    }

    FAPI_SYS_PRINT_MSG("------------------------------------------------\n");
}

/* ------------------------------------------------------------------------- */
void UTILS_WrU16Masked (uint8_t* buffer,uint16_t value, uint16_t mask)
{
   buffer[0] = (uint8_t)((value & mask) >> 8);
   buffer[1] = (uint8_t) (value & mask);
}

uint16_t UTILS_RdU16Masked (const uint8_t* buffer, uint16_t mask)
{
    return ((((uint16_t)(buffer[0])) << 8) | buffer[1] ) & mask;
}

/* ------------------------------------------------------------------------- */

/*
reqCb - optional handler called when layer is requested by other module. If
        FTRUE is returned, layer is given, in case of FFALSE owning module
        does not release layer.
*/
int32_t SYS_GetOsdLayer(const SYS_OSDLAYER_S * params, SYS_OSDLAYERREQ_CB reqCb)
{
    int32_t     retVal = FAPI_OK;
    uint32_t     prevError;
    uint32_t    osdArrayIdx;
    OSDHANDLER_LayerParamsT layerParams;


    FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GetOSDLayer] enter SYS_GetOsdLayer\n");
    /* check arguments */
    if(SYS_CheckOsdLayerId(params->layerIdx) != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[SYS] Invalid OSD Layer 0x%08x\n",params->layerIdx);
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    osdArrayIdx = getOsdIntIdx(params->layerIdx);
    FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GetOSDLayer] osdArrayIdx = %d\n", osdArrayIdx);

#ifndef _LINUX_FAPI
    (void) RTOS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,RTOS_SUSPEND);
#else
    (void) FAPEX_SYS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,FAPEX_SYS_NO_SUSPEND);
#endif
    do
    {
        /* check resource availability */
        if(sysData->layerDat[osdArrayIdx].inUse)
        {
            if((sysData->layerDat[osdArrayIdx].reqCb == NULL) ||
               (sysData->layerDat[osdArrayIdx].reqCb((uint8_t)params->layerIdx) == FFALSE))
            {
                FAPI_SYS_PRINT_DEBUG(1,"[main][SYS_GetOSDLayer] reqCb 0x%x\n",sysData->layerDat[osdArrayIdx].reqCb);
                retVal = SMARTGO_SYS_ERR_PROHIBITED;
                break;
            }
            else
            {
#ifndef _LINUX_FAPI
                (void) RTOS_SetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,RTOS_NO_SUSPEND);
#else
                (void) FAPEX_SYS_SetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,FAPEX_SYS_NO_SUSPEND);
#endif
                retVal = SYS_GiveOsdLayer(params->layerIdx);
                if(retVal != FAPI_OK)
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[SYS] SYS_GiveOsdLayer() failed: %s/n",SYS_GetErrorString(retVal));
                    return retVal;
                }

                FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GetOSDLayer] SYS_GiveOsdLayer success\n");

#ifndef _LINUX_FAPI
                (void) RTOS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,RTOS_SUSPEND);
#else
                (void) FAPEX_SYS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,FAPEX_SYS_SUSPEND);
#endif
            }
        }
        retVal = OSDHANDLER_GetLayerParameters(params->layerIdx,&layerParams);
        if (retVal!=FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1, "[SYS] OSDHANDLER_GetLayerParameters failed %s/n",SYS_GetErrorString(retVal));
            break;
        }

        FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GetOSDLayer] OSDHANDLER_GetLayerParameters success\n");

        if(layerParams.colourMode != params->colourMode)
        {
            retVal = OSDHANDLER_SetColourMode(params->layerIdx,
                                              params->colourMode,
                                              &prevError);
            if (retVal!=FAPI_OK)
            {
                FAPI_SYS_PRINT_DEBUG(1, "[SYS] OSDHANDLER_SetColourMode failed %s/n",SYS_GetErrorString(retVal));
                break;
            }
            FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GetOSDLayer] OSDHANDLER_SetColourMode success\n");
        }
        if(layerParams.layoutMode != params->layoutMode)
        {
            retVal = OSDHANDLER_SetLayoutMode(params->layerIdx,
                                              params->layoutMode,
                                              &prevError);
            if (retVal!=FAPI_OK)
            {
                FAPI_SYS_PRINT_DEBUG(1,"[SYS] OSDHANDLER_SetLayoutMode failed %s/n",SYS_GetErrorString(retVal));
                break;
            }
            FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GetOSDLayer] OSDHANDLER_SetLayoutMode success\n");
        }
        retVal =  OSDHANDLER_SetFillColour(params->layerIdx, params->fillColour);
        if (retVal!=FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[SYS] OSDHANDLER_SetFillColour failed %s/n",SYS_GetErrorString(retVal));
            break;
        }
        FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GetOSDLayer] OSDHANDLER_SetFillColour success\n");
        /* open requested layer */
        retVal = OSDHANDLER_StartLayer(params->layerIdx, &params->params, &prevError);
        if(retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[SYS] OSDHANDLER_StartLayer() failed: %s/n",SYS_GetErrorString(retVal));
            break;
        }

        FAPI_SYS_PRINT_DEBUG(1, "[main][SYS_GetOSDLayer] OSDHANDLER_StartLayer success\n");

        sysData->layerDat[osdArrayIdx].inUse = FTRUE;
        sysData->layerDat[osdArrayIdx].reqCb = reqCb;

    } while(0);

#ifndef _LINUX_FAPI
    (void) RTOS_SetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,RTOS_NO_SUSPEND);
#else
    (void) FAPEX_SYS_SetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,FAPEX_SYS_NO_SUSPEND);
#endif
    return retVal;
}

int32_t SYS_ModifyOsdLayer(const SYS_OSDLAYER_S * params)
{
    uint32_t    prevError;
    uint32_t    osdArrayIdx;

    /* check arguments */
    if(SYS_CheckOsdLayerId(params->layerIdx) != FAPI_OK)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    osdArrayIdx = getOsdIntIdx(params->layerIdx);


    if(!(sysData->layerDat[osdArrayIdx].inUse))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
#ifndef _LINUX_FAPI
    (void) RTOS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,RTOS_SUSPEND);
#else
    (void) FAPEX_SYS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,FAPEX_SYS_SUSPEND);
#endif
    (void)OSDHANDLER_StopLayer(params->layerIdx, &prevError);
    (void)OSDHANDLER_StartLayer(params->layerIdx,
                          (OSDHANDLER_StartParamsT *) & params->params,
                           &prevError);
#ifndef _LINUX_FAPI
    (void) RTOS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,RTOS_NO_SUSPEND);
#else
    (void) FAPEX_SYS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,FAPEX_SYS_NO_SUSPEND);
#endif

    return FAPI_OK;
}

int32_t SYS_GiveOsdLayer(uint32_t index)
{
    uint32_t     prevError;
    uint32_t    osdArrayIdx;

    /* check arguments */
    if(SYS_CheckOsdLayerId(index) != FAPI_OK)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    osdArrayIdx = getOsdIntIdx(index);

#ifndef _LINUX_FAPI
    (void) RTOS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,RTOS_SUSPEND);
#else
    (void) FAPEX_SYS_GetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,FAPEX_SYS_SUSPEND);
#endif
    if(sysData->layerDat[osdArrayIdx].inUse)
    {
        (void)OSDHANDLER_StopLayer(index, &prevError);

        sysData->layerDat[osdArrayIdx].inUse = FFALSE;
    }
#ifndef _LINUX_FAPI
    (void)RTOS_SetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,RTOS_NO_SUSPEND);
#else
    (void) FAPEX_SYS_SetSemaphore(sysData->layerDat[osdArrayIdx].layerMutex,FAPEX_SYS_NO_SUSPEND);
#endif

    return FAPI_OK;
}

fbool_t SYS_OsdLayerAvailable(uint32_t index)
{
    uint32_t    osdArrayIdx;

    /* check arguments */
    if(SYS_CheckOsdLayerId(index) != FAPI_OK)
    {
        return FFALSE;
    }
    osdArrayIdx = getOsdIntIdx(index);

    return (sysData->layerDat[osdArrayIdx].inUse) ? FFALSE : FTRUE;
}

int32_t SYS_GetOsdLayerMode(uint32_t index, uint32_t * layerMode)
{
    OSDHANDLER_LayerParamsT     layerParams;
    uint32_t                    osdArrayIdx;

    /* check arguments */
    if(SYS_CheckOsdLayerId(index) != FAPI_OK)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    osdArrayIdx = getOsdIntIdx(index);

    if(sysData->layerDat[osdArrayIdx].inUse == 0)
    {
        return OSDHANDLER_ERR_LAYER_NOT_STARTED;
    }
    (void)OSDHANDLER_GetLayerParameters(index, &layerParams);
    *layerMode = layerParams.colourMode;

    return FAPI_OK;
}


/*!
*******************************************************************************
**
** \brief   This function allocads uncached memory for an OSD layer.
**
** \param   width       OSD layer width.
** \param   height      OSD layer height.
** \param   bpp         OSD LAyer resolution in BitsPerPixel.
** \param   basePtr     Pointer to base pointer of memory.
**                      (Is needed to free the allocated memory)
** \param   osdPtr      128byte allignd pointer for OSDHANDLER.
** \param   clearMem    FTRUE clears the memory after allocation.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
******************************************************************************/
int32_t SYS_MallocOsdMemory(const uint32_t width,
                            const uint32_t height,
                            const uint32_t bpp,
                            uint8_t** basePtr,
                            uint8_t** osdPtr,
                            fbool_t clearMem)
{
    uint32_t bufSize,w,h;

    if( (width   == 0) ||
        (height  == 0) ||
        (bpp     == 0) ||
        (basePtr == NULL) ||
        (osdPtr  == NULL) )
    {
        FAPI_SYS_PRINT_DEBUG(1,"[%s l=%i] APPL_SMARTGO_ERR_BAD_PARAMETER \n",
                             __FILE__,__LINE__);
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    w = width;
    h = height;
    // one line must allocate a multiple of 128 bytes:
    // go above for the internal picture width:
    if (w % (1024 / bpp))
    {
        w += (1024 / bpp) - (w % (1024 / bpp));
    }
    // height must be a multiple of 2
    if (h % 2)
    {
        h++;
    }
    bufSize = (w * h * bpp)>>3;
    bufSize += 1024;
    *basePtr = FAPI_SYS_MALLOC(bufSize);
    if(*basePtr == NULL)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[%s l=%i] APPL_SMARTGO_ERR_OUT_OF_MEMORY \n",
                             __FILE__,__LINE__);
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }
    if(clearMem == FTRUE)
    {
        memset(*basePtr,0,bufSize);
    }
    *osdPtr = *basePtr;
    if(((uint32_t)(*osdPtr)) % 1024)
    {
        *osdPtr += (1024 - (((uint32_t)*osdPtr) % 1024));
    }

    return FAPI_OK;
}
FSTATIC int32_t SYS_CheckOsdLayerId(uint32_t id)
{
    if ( id < OSDHANDLER_MAX_NUM_HW_LAYER)
    {
        return(FAPI_OK);
    }
    else
    {
        return(APPL_SMARTGO_ERR_BAD_PARAMETER);
    }
}

FSTATIC uint32_t getOsdIntIdx(uint32_t id)
{
    if(id < OSDHANDLER_MAX_NUM_HW_LAYER) /* osd LAYER is hw layer */
        return id;

    return 0xFFFFFFFF;
}

/* ------------------------------------------------------------------------- */
#ifndef _LINUX_FAPI
FSTATIC void SYS_SysIrqFunction (FAPI_INTR_SysIrqEnumT intrType,
                                 uint32_t atAddress)
{
    const char_t* pStr;

    switch ( intrType )
    {
        case FAPI_INTR_UNDEF:       pStr = "UNDEF";     break;
        case FAPI_INTR_PREFETCH:    pStr = "PREFETCH";  break;
        case FAPI_INTR_ABORT:       pStr = "ABORT";     break;
        default:                    pStr = "<UNKOWN>";  break;
    }

    FAPI_SYS_PRINT_DEBUG(1,"[SYS] Exception %s occurred at address 0x%X! Halting system\n",
                       pStr, atAddress);

    while ( 1 ) {}
}
#endif
/* ------------------------------------------------------------------------- */

#ifndef _LINUX_FAPI
int32_t SYS_ThreadCreate(RTOS_ThreadFunctionT function,
                         void *arg,
                         int32_t priority,
                         uint32_t ssize, const char_t *name, RTOS_ThreadT * id)
#else
int32_t SYS_ThreadCreate(FAPEX_SYS_ThreadFunctionT function,
                         void *arg,
                         int32_t priority,
                         uint32_t ssize, const char_t *name, FAPEX_SYS_ThreadT * id)
#endif
{
#ifndef _LINUX_FAPI
    RTOS_ThreadT    threadId;
#else
    FAPEX_SYS_ThreadT threadId;
#endif
    int32_t         retVal;

#ifndef _LINUX_FAPI
    threadId = RTOS_CreateThread(NULL, ssize>>2, priority, function, arg, NULL, name);
#else
    threadId = FAPEX_SYS_CreateThread (NULL, ssize>>2, priority, function, arg, NULL, name);
#endif
    if ( threadId == NULL )
    {
        retVal = SMARTGO_SYS_ERR_RESOURCES;
    }
    else
    {
        if ( id != NULL )
        {
            *id = threadId;
        }

        retVal = FAPI_OK;
    }

    return retVal;
}

/*---------------------------------------------------------------------------*/
/* temporary OSD functions                                                   */
/*---------------------------------------------------------------------------*/

#ifdef ENABLE_BOOTOSD
FSTATIC int32_t SYS_BootOsdInit(SYS_HDTV_STANDARD hdSys)
{

    FONTSERVER_InitParamsT  fontParams;
    SYS_OSDLAYER_S          layerParams;
    int32_t                 retVal;
    uint32_t                prevError;

    if ( osdAvailable )
    {
        return FAPI_OK;
    }

    /* init fonts */;
    bootOsdFontEngine[0]  = (uint32_t)FONTSERVER_ENGINE_FBF_INTERFACE_PTR;
    bootOsdFontData[0]    = (uint32_t) Fujitsu_Zwo_Bold_FBF_32_DATA;
    bootOsdFontSizePtr[0] = Fujitsu_Zwo_Bold_FBF_32_SIZE;
    fontParams.numEngines      = 1;
    fontParams.enginesPtr      = bootOsdFontEngine;
    fontParams.numFonts        = 1;
    fontParams.fontDataPtr     = bootOsdFontData;
    fontParams.fontFileSizePtr = bootOsdFontSizePtr;

    (void)FONTSERVER_Init(&fontParams);

    /* Setup OSD layer */
    osdPars.frameBased    = 0;
    osdPars.outputWidth   = 0;
    osdPars.outputHeight  = 0;
    osdPars.memoryBasePtr = NULL;

    switch(hdSys)
    {
        default:
        case HDSYS_1080I:
            osdPars.xPos          = 150;
            osdPars.yPos          = 300;
            osdPars.inputWidth    = VIDEOSZ_X_1920 - 300;
            osdPars.inputHeight   = 54;
            break;
        case HDSYS_720P:
            osdPars.xPos          = 100;
            osdPars.yPos          = 404;
            osdPars.inputWidth    = VIDEOSZ_X_1280 - 200;
            osdPars.inputHeight   = 54;
            break;
        case HDSYS_576I:
        case HDSYS_576P:
            osdPars.xPos          =  50;
            osdPars.yPos          = 300;
            osdPars.inputWidth    = VIDEOSZ_X_720 - 100;
            osdPars.inputHeight   = 54;
            break;
    }
    layerParams.params   = osdPars;
    layerParams.layerIdx = BOOT_OSD;

    retVal = SYS_GetOsdLayer(&layerParams, NULL);
    if(retVal != FAPI_OK)
        return retVal;

    (void)SYS_SetPaletteColor(BOOT_OSD, BOOTOSD_BACK, 0x000000C0);      // black
    (void)SYS_SetPaletteColor(BOOT_OSD, BOOTOSD_FRONT,0xF0F0F0FF);      // white
    OSDHANDLER_ShowLayer(BOOT_OSD, &prevError);

    osdAvailable = FTRUE;

    /* clear background */
    OSDHANDLER_BitBlitFill(BOOT_OSD, 0, 0, osdPars.inputWidth-1,
                           osdPars.inputHeight-1, BOOTOSD_BACK);

    return FAPI_OK;
}

FSTATIC void SYS_BootOsdExit(void)
{

    uint32_t prevError;

    if ( osdAvailable == FFALSE ) return;

    OSDHANDLER_HideLayer(BOOT_OSD, &prevError);

    SYS_GiveOsdLayer(BOOT_OSD);

    osdAvailable = FFALSE;
    FONTSERVER_Exit();
}
#endif /* ENABLE_BOOTOSD */

void SYS_BootOsdPrint(const char_t *string)
{
    if(string == NULL)
        return;

    #ifdef ENABLE_BOOTOSD

    if ( osdAvailable == FTRUE )
    {
        /* clear box */
        OSDHANDLER_BitBlitFill(BOOT_OSD, 0, 0, osdPars.inputWidth, osdPars.inputHeight, BOOTOSD_BACK);

        (void)FONTSERVER_PrintString((uint8_t*)string,
                               0,
                               BOOT_OSD,
                               BOOTOSD_PAD_IN,
                               BOOTOSD_PAD_IN,
                               BOOTOSD_FRONT,
                               0,
                               0,
                               FONTSERVER_ENCODING_8_BIT,
                               NULL);
    }

    #endif /* ENABLE_BOOTOSD */

    /* print also on terminal */
    FAPI_SYS_PRINT_DEBUG(1,string);
    FAPI_SYS_PRINT_DEBUG(1,"\n");
}

#ifdef APPL_DEBUG_MODE
FSTATIC const char_t* SYS_GetDeviceString (void)
{
    const char_t*   strPtr;
#ifndef _LINUX_FAPI
    uint32_t        device_id = FAPI_SYS_GetDeviceId();

    switch ( device_id )
    {
        case FAPI_SYS_DEVICE_ID_MB86H60:
            strPtr = "MB86H60";
            break;

        case FAPI_SYS_DEVICE_ID_MB86H60B:
            strPtr = "MB86H60B";
            break;

        default:
            strPtr = "Unknown";
            break;
    }

#else
    strPtr = "MB86H60";
#endif

    return strPtr;
}
#endif

FSTATIC void SYS_PrintStartupHdr (void)
{
    FAPI_SYS_HandleT              flHdl;
#ifndef _LINUX_FAPI
    FAPI_FLASH_OpenParamsT        flashParams;
#else
    FAPEX_VRFLASH_OpenParamsT        flashParams;
#endif
    const char_t*                 flashDevice;
    uint32_t                      flashType;
    uint32_t                      flashSize;
    int32_t                       retVal;
#ifndef _LINUX_FAPI
    FAPI_BOARD_HwPlatformInfosT*  infoPtr = FAPI_BOARD_GetBoardInfo();
#endif
    #ifdef APPL_BOOTLDR_ENABLED
    FAPI_BOOT_InfoT               bootInfo;
    BOOT_FLASH_ImageT             bootSlotInfo;
    #endif /* APPL_BOOTLDR_ENABLED */

    /* print application title to console */
    FAPI_SYS_PRINT_DEBUG(1,"\n");
    FAPI_SYS_PRINT_DEBUG(1,"#################################################################\n");
    FAPI_SYS_PRINT_DEBUG(1,"#                                                               #\n");
    FAPI_SYS_PRINT_DEBUG(1,"#                                                               #\n");
    FAPI_SYS_PRINT_DEBUG(1,"#      Fujitsu MB86H60 - SmartGo-HD V%i.%-4i                     #\n",
                         SYS_VERSION_MAJOR, SYS_VERSION_MINOR);
    FAPI_SYS_PRINT_DEBUG(1,"#                                                               #\n");
    FAPI_SYS_PRINT_DEBUG(1,"# (c) Fujitsu Semiconductor Europe 2002 - 2010               #\n");
    FAPI_SYS_PRINT_DEBUG(1,"#                                                               #\n");
    FAPI_SYS_PRINT_DEBUG(1,"# ------------------------------------------------------------- #\n");
    FAPI_SYS_PRINT_DEBUG(1,"#                                                               #\n");
    FAPI_SYS_PRINT_DEBUG(1,"# Build              : %-17s %-22s #\n", __DATE__, __TIME__);
    FAPI_SYS_PRINT_DEBUG(1,"# Device             : %-40s #\n", SYS_GetDeviceString());
    /*
        FAPI_SYS_PRINT_DEBUG(1,"# System Memory Size : %-3d MByte                                #\n",
                         FAPI_SYS_MMAP_DDR_BYTES / 0x100000);

    FAPI_SYS_PRINT_DEBUG(1,"# Video Memory Size  : %-3d MByte                                #\n",
                         FAPI_SYS_MMAP_DDR2_BYTES / 0x100000);
         */
    FAPI_SYS_PRINT_DEBUG(1,"# Flash              : ");

    (void)SYS_GetFlashDeviceType(&flashType);

#ifndef _LINUX_FAPI
    flashParams.version     = FAPI_FLASH_VERSION;
#else
    flashParams.version     = FAPEX_VRFLASH_VERSION;
#endif
    flashParams.deviceType  = flashType;
    flashParams.firstSector = 0;
    flashParams.sectorCount = 0;
    flashParams.fsysFlag    = 0;
    flashParams.forceFlush  = 0;

#ifndef _LINUX_FAPI
    flHdl = FAPI_FLASH_Open( &flashParams, &retVal );
#else
    flHdl = FAPEX_VRFLASH_Open( &flashParams, &retVal );
#endif
    if ( flHdl != NULL )
    {
        /** TODO error handling */
#ifndef _LINUX_FAPI
        flashDevice = FAPI_FLASH_GetDeviceName(flHdl);
#else
        flashDevice = FAPEX_VRFLASH_GetDeviceName(flHdl);
#endif
        if ( flashDevice == NULL )
        {
            flashDevice = "<unknown>";
        }
#ifndef _LINUX_FAPI
        flashSize = FAPI_FLASH_GetDeviceSize(flHdl);
#else
        flashSize = FAPEX_VRFLASH_GetDeviceSize(flHdl);
#endif
    }
    else
    {
        flashSize = 0;
        flashDevice = "<driver opening failure>";
    }

    FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", flashDevice);
    FAPI_SYS_PRINT_DEBUG(1,"# Flashsize          : %-2d MByte                                 #\n", flashSize / 0x100000);
    FAPI_SYS_PRINT_DEBUG(1,"# RTOS               : %-40s #\n", "Linux");
    FAPI_SYS_PRINT_DEBUG(1,"# Target Platform    : %-40s #\n", "MB86H60 Devkit");

#ifndef _LINUX_FAPI
    if ( infoPtr != NULL )
    {
        FAPI_SYS_PRINT_DEBUG(1,"# HW Manufacturer    : %-40s #\n",infoPtr->pHwManufactString);
        FAPI_SYS_PRINT_DEBUG(1,"# HW Model           : %-40s #\n",infoPtr->pHwModelString);
    }
#endif
    FAPI_SYS_PRINT_DEBUG(1,"# Satellite enabled  : ");

    #ifdef APPL_FESAT_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# Terr. enabled      : ");

    #ifdef APPL_FETERR_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# Cable enabled      : ");

    #ifdef APPL_FECABLE_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# Direct TS enabled  : ");

    #ifdef APPL_FEDIRECT_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# TTX enabled        : ");

    #ifdef APPL_TTXT_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# MHEG-5 enabled     : ");

    #ifdef APPL_MHEG_ENABLED
      #if APPL_MHEG_PROFILE == 1
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n","United Kingdom SD profile 1.06");
      #elif APPL_MHEG_PROFILE == 2
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n","United Kingdom HD profile 1.06");
      #elif APPL_MHEG_PROFILE == 3
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n","New Zealand profile");
      #elif APPL_MHEG_PROFILE == 4
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n","Hong Kong profile");
      #elif APPL_MHEG_PROFILE == 5
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n","United Kingdom Freesat profile");
      #elif APPL_MHEG_PROFILE == 6
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n","CI Plus Only profile");
      #elif APPL_MHEG_PROFILE == 7
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n","Australia profile");
      #elif APPL_MHEG_PROFILE == 8
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n","South Africa profile");
      #else
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n","UNKNOWN profile\n");
      #endif
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# PVR enabled        : ");

    #ifdef APPL_PVR_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# HDD enabled        : ");

    #ifdef APPL_HDD_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# USB enabled        : ");

    #ifdef APPL_USB_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# USB function enabl.: ");

    #ifdef APPL_USB_FUNC_MSC_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# DVB Subtt. enabled : ");

    #ifdef APPL_SUBTT_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    FAPI_SYS_PRINT_DEBUG(1,"# CI enabled         : ");

    #ifdef APPL_CI_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif /* APPL_CI_ENABLED */

    FAPI_SYS_PRINT_DEBUG(1,"# Parental c. enabled: ");

    #ifdef APPL_PARENTAL_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif /* APPL_PARENTAL_ENABLED */

    FAPI_SYS_PRINT_DEBUG(1,"# JPEG enabled       : ");

    #ifdef APPL_JPEG_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif /* APPL_JPEG_ENABLED */

    FAPI_SYS_PRINT_DEBUG(1,"# MP3 enabled        : ");

    #ifdef APPL_MP3_ENABLED
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif /* APPL_MP3_ENABLED */

    FAPI_SYS_PRINT_DEBUG(1,"# Debug mode         : ");

    #ifdef APPL_DEBUG_MODE
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "YES" );
    #else
      FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "NO" );
    #endif

    #ifdef APPL_BOOTLDR_ENABLED

    FAPI_SYS_PRINT_DEBUG(1,"# Booter information : ");

    if(SYS_GetBootInfo(&bootInfo) == FAPI_OK)
    {
        if( FAPI_FLASH_GetBootImageInfoByIndex(flHdl,bootInfo.slotIndex,&bootSlotInfo) == FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1,"Ver %2d.%-2d ",
                                 bootSlotInfo.imageVersion >> 8,
                                 bootSlotInfo.imageVersion&0xFF);
        }
        FAPI_SYS_PRINT_DEBUG(1," boot slot %-2d                  #\n",
                  bootInfo.slotIndex);
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1,"%-40s #\n", "Not available" );
    }

    #endif /* APPL_BOOTLDR_ENABLED */

    FAPI_SYS_PRINT_DEBUG(1,"#                                                               #\n");
    FAPI_SYS_PRINT_DEBUG(1,"#################################################################\n");

    if(flHdl != NULL)
    {
#ifndef _LINUX_FAPI
        (void) FAPI_FLASH_Close(flHdl);
#else
        (void) FAPEX_VRFLASH_Close(flHdl);
#endif
    }
}

/* ------------------------------------------------------------------------- */

void SYS_SizeClear(SYS_SIZE_S * size)
{
    memset(size, 0, sizeof(SYS_SIZE_S));
}

void SYS_SizeAdd(SYS_SIZE_S * size, uint32_t kbytes, uint32_t bytes)
{
    size->kbytes += (bytes / 1024) + kbytes;
    size->bytes  +=  bytes % 1024;

    while ( size->bytes > 1024 )
    {
        size->kbytes++;
        size->bytes -= 1024;
    }
}

fbool_t SYS_SizeIsSmaller(const SYS_SIZE_S * size1, const SYS_SIZE_S * size2)
{
    if ( size1->kbytes < size2->kbytes ) return FTRUE;
    else if((size1->kbytes == size2->kbytes) && (size1->bytes < size2->bytes))
    {
        return FTRUE;
    }
    else
    {
        return FFALSE;
    }
}

/* ------------------------------------------------------------------------- */
uint32_t sysPalette[256];
int32_t SYS_SetPaletteColor(uint32_t layer, uint16_t paletteIndex,
                            uint32_t color)
{
    sysPalette[paletteIndex] = color;
    #if 0
    FREG_VO_SetOsd1clut(paletteIndex, color);
    return FAPI_OK;
    #else
    return OSDHANDLER_SetClutPartially(layer, &sysPalette[paletteIndex], paletteIndex,
                                       1, NULL);
    #endif
}

int32_t SYS_GetPaletteColor(uint32_t layer, uint16_t paletteIndex,
                            uint32_t * pColor)
{
    uint32_t    palette[256];
    int32_t     retVal = FAPI_OK;

    retVal = OSDHANDLER_GetClut(layer,palette);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[OSD] OSDHANDLER_GetClut[] failed: %i\n",retVal);
        return retVal;
    }
    *pColor = palette[paletteIndex];

    return retVal;
}

/* ------------------------------------------------------------------------- */

int32_t SYS_ControlFpSymbol(FP_SYMB_E type, fbool_t enable)
{
    #if 0 // TODO
    FP_SYMBOL_S            data;

    data.type = type;
    data.enable = enable;

    return FD_GHW_ControlFrontPanel(0, FP_SET_SYMBOL, &data);
    #endif

    return FAPI_OK;
}

/* ------------------------------------------------------------------------- */
int32_t TSC_Init(TSC_INST_S * inst)
{
    memset(inst, 0, sizeof(TSC_INST_S));

#ifndef _LINUX_FAPI
    inst->wakeupBox = RTOS_CreateMailbox(sizeof(uint32_t));
#else
    inst->wakeupBox = FAPEX_SYS_CreateMailbox(sizeof(uint32_t));
#endif
    if(inst->wakeupBox == NULL)
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

#ifndef _LINUX_FAPI
    inst->ackBox = RTOS_CreateMailbox(sizeof(uint32_t));
#else
    inst->ackBox = FAPEX_SYS_CreateMailbox(sizeof(uint32_t));
#endif
    if(inst->ackBox == NULL)
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

#ifndef _LINUX_FAPI
    inst->mutex = RTOS_CreateSemaphore(1);
#else
    inst->mutex = FAPEX_SYS_CreateSemaphore(1);
#endif
    if(inst->mutex == NULL)
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

    inst->sleepMode = FTRUE;
    inst->sleepReq  = FFALSE;

    return FAPI_OK;
}

int32_t TSC_SendThreadToSleep(TSC_INST_S * inst)
{
    uint32_t dummy;

#ifndef _LINUX_FAPI
    (void) RTOS_GetSemaphore(inst->mutex,RTOS_SUSPEND);
#else
    (void) FAPEX_SYS_GetSemaphore(inst->mutex,FAPEX_SYS_SUSPEND);
#endif
    if ( !(inst->sleepMode) )
    {
        /* issue sleep request */
        inst->sleepReq  = FTRUE;
        inst->sleepMode = FTRUE;

#ifndef _LINUX_FAPI
        (void) RTOS_GetSemaphore(inst->mutex,RTOS_NO_SUSPEND);
#else
        (void) FAPEX_SYS_GetSemaphore(inst->mutex,FAPEX_SYS_NO_SUSPEND);
#endif
        /* wait for acknowledgement */
#ifndef _LINUX_FAPI
        (void)RTOS_GetMailbox(inst->ackBox, &dummy, RTOS_SUSPEND);
#else
        (void)FAPEX_SYS_GetMailbox(inst->ackBox, &dummy, FAPEX_SYS_SUSPEND);
#endif
    }
    else
    {
#ifndef _LINUX_FAPI
        (void) RTOS_GetSemaphore(inst->mutex,RTOS_NO_SUSPEND);
#else
        (void) FAPEX_SYS_GetSemaphore(inst->mutex,FAPEX_SYS_NO_SUSPEND);
#endif
    }

    return FAPI_OK;
}

int32_t TSC_WakeupThread(TSC_INST_S * inst)
{
    uint32_t dummy;

#ifndef _LINUX_FAPI
    (void) RTOS_GetSemaphore(inst->mutex,RTOS_SUSPEND);
#else
    (void) FAPEX_SYS_GetSemaphore(inst->mutex,FAPEX_SYS_SUSPEND);
#endif

    if(inst->sleepMode)
    {
        /* trigger wakeup of pending thread */
#ifndef _LINUX_FAPI
        (void)RTOS_SetMailbox(inst->wakeupBox, &dummy, RTOS_NO_SUSPEND);
#else
        (void)FAPEX_SYS_SetMailbox(inst->wakeupBox, &dummy, FAPEX_SYS_NO_SUSPEND);
#endif
        inst->sleepMode = FFALSE;
    }

#ifndef _LINUX_FAPI
    (void) RTOS_SetSemaphore(inst->mutex,RTOS_NO_SUSPEND);
#else
    (void) FAPEX_SYS_SetSemaphore(inst->mutex,FAPEX_SYS_NO_SUSPEND);
#endif

    return FAPI_OK;
}

void TSC_WaitForWakeup(TSC_INST_S * inst)
{
    uint32_t dummy;

#ifndef _LINUX_FAPI
    (void) RTOS_GetSemaphore(inst->mutex,RTOS_SUSPEND);
#else
    (void) FAPEX_SYS_GetSemaphore(inst->mutex,FAPEX_SYS_SUSPEND);
#endif

    if ( inst->sleepReq )
    {
        inst->sleepReq = FFALSE;

        /* acknowledge sleep request */
#ifndef _LINUX_FAPI
        (void)RTOS_SetMailbox(inst->ackBox, &dummy, RTOS_NO_SUSPEND);
#else
        (void)FAPEX_SYS_SetMailbox(inst->ackBox, &dummy, FAPEX_SYS_NO_SUSPEND);
#endif
    }
    else
    {
        inst->sleepMode = FTRUE;
    }

#ifndef _LINUX_FAPI
    (void) RTOS_SetSemaphore(inst->mutex,RTOS_NO_SUSPEND);
#else
    (void) FAPEX_SYS_SetSemaphore(inst->mutex,FAPEX_SYS_NO_SUSPEND);
#endif

#ifndef _LINUX_FAPI
    (void)RTOS_GetMailbox(inst->wakeupBox, &dummy, RTOS_SUSPEND);
#else
    (void)FAPEX_SYS_GetMailbox(inst->wakeupBox, &dummy, FAPEX_SYS_SUSPEND);
#endif
}

fbool_t TSC_IsSleepRequested (TSC_INST_S* inst)
{
    return inst->sleepReq;
}

/* ------------------------------------------------------------------------- */

int32_t ITC_Init(ITC_INST_S * inst, ITC_PARAMS_S * params)
{
    memset(inst, 0, sizeof(ITC_INST_S));

#ifndef _LINUX_FAPI
    inst->cmdBox = RTOS_CreateMailbox(sizeof(void *));
#else
    inst->cmdBox = FAPEX_SYS_CreateMailbox(sizeof(void *));
#endif
    if(inst->cmdBox == NULL)
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

#ifndef _LINUX_FAPI
    inst->ackBox = RTOS_CreateMailbox(sizeof(void *));
#else
    inst->ackBox = FAPEX_SYS_CreateMailbox(sizeof(void *));
#endif
    if(inst->ackBox == NULL)
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

#ifndef _LINUX_FAPI
    inst->wakeupBox = RTOS_CreateMailbox(sizeof(void *));
#else
    inst->wakeupBox = FAPEX_SYS_CreateMailbox(sizeof(void *));
#endif
    if(inst->wakeupBox == NULL)
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

#ifndef _LINUX_FAPI
    inst->cmdMutex = RTOS_CreateSemaphore(1);
#else
    inst->cmdMutex = FAPEX_SYS_CreateSemaphore(1);
#endif
    if(inst->cmdMutex == NULL)
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }
    inst->osTickPeriod = 1;
    inst->abortReq     = FFALSE;

    return FAPI_OK;
}

int32_t ITC_CmdSend(ITC_INST_S * inst, void* cmd, void ** ack)
{
    void*                  msg = cmd;
    void*                  ackMsg;

    #ifdef ITC_DEBUG
    if((inst->threadRx != NULL)&&
#ifndef _LINUX_FAPI
       (inst->threadRx == RTOS_GetThread()))
#else
       (inst->threadRx == FAPEX_SYS_GetThread()))
#endif
    {
        FAPI_SYS_PRINT_DEBUG(1,"[ITC] ITC_CmdSend() called from RX Thread !!! %s",
#ifndef _LINUX_FAPI
                           RTOS_GetThreadName(RTOS_GetThread()));
#else
                           FAPEX_SYS_GetThreadName(FAPEX_SYS_GetThread()));
#endif
    }
    #endif

#ifndef _LINUX_FAPI
    (void) RTOS_GetSemaphore(inst->cmdMutex,RTOS_SUSPEND);
#else
    (void) FAPEX_SYS_GetSemaphore(inst->cmdMutex,FAPEX_SYS_SUSPEND);
#endif
    /* set abort request to interrupt any thread operation */
    inst->abortReq = FTRUE;

    if(inst->params.setAbortReq != NULL)
    {
        inst->params.setAbortReq();
    }

    /* wakeup potentially sleeping av-thread. Mailbox could be already filled
       which is not an error case! */
#ifndef _LINUX_FAPI
    (void)RTOS_SetMailbox(inst->wakeupBox, &msg, RTOS_NO_SUSPEND);
#else
    (void)FAPEX_SYS_SetMailbox(inst->wakeupBox, &msg, FAPEX_SYS_NO_SUSPEND);
#endif

#ifndef _LINUX_FAPI
    if(!RTOS_SetMailbox(inst->cmdBox, &msg, RTOS_NO_SUSPEND))
#else
    if(FAPEX_SYS_SetMailbox(inst->cmdBox, &msg, FAPEX_SYS_NO_SUSPEND))
#endif
    {
#ifndef _LINUX_FAPI
        (void) RTOS_SetSemaphore(inst->cmdMutex,RTOS_NO_SUSPEND);
#else
        (void) FAPEX_SYS_SetSemaphore(inst->cmdMutex,FAPEX_SYS_NO_SUSPEND);
#endif

        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* wait for acknowledgement */
#ifndef _LINUX_FAPI
    (void)RTOS_GetMailbox(inst->ackBox, &ackMsg, RTOS_SUSPEND);
#else
    (void)FAPEX_SYS_GetMailbox(inst->ackBox, &ackMsg, FAPEX_SYS_SUSPEND);

#endif

    if(ack != NULL)
        *ack = ackMsg;
#ifndef _LINUX_FAPI
    (void) RTOS_SetSemaphore(inst->cmdMutex,RTOS_NO_SUSPEND);
#else
    (void) FAPEX_SYS_SetSemaphore(inst->cmdMutex,FAPEX_SYS_NO_SUSPEND);
#endif

    return FAPI_OK;
}

void* ITC_CmdWait(ITC_INST_S * inst)
{
    void *                  msg;
    #ifdef ITC_DEBUG
    if(inst->threadRx == NULL)
    {
#ifndef _LINUX_FAPI
        inst->threadRx = RTOS_GetThread();
#else
        inst->threadRx = FAPEX_SYS_GetThread();
#endif
    }
    #endif
#ifndef _LINUX_FAPI
    (void)RTOS_GetMailbox(inst->cmdBox, &msg, RTOS_SUSPEND);
#else
    (void)FAPEX_SYS_GetMailbox(inst->cmdBox, &msg, FAPEX_SYS_SUSPEND);
#endif

    if ( msg != NULL ) inst->abortReq = FFALSE;

    return msg;
}

void* ITC_CmdWaitTimeout(ITC_INST_S * inst, uint32_t timeout)
{
    void *                  msg;

    #ifdef ITC_DEBUG
    if(inst->threadRx == NULL)
    {
#ifndef _LINUX_FAPI
        inst->threadRx = RTOS_GetThread();
#else
        inst->threadRx = FAPEX_SYS_GetThread();
#endif
    }
    #endif
#ifndef _LINUX_FAPI
    (void)RTOS_GetMailbox(inst->cmdBox, &msg, timeout);
#else
    (void)FAPEX_SYS_GetMailbox(inst->cmdBox, &msg, timeout);
#endif

    if ( msg != NULL ) inst->abortReq = FFALSE;

    return msg;
}

/*
returns:
- FTRUE: sleep has not been aborted
- FFALSE: sleep has been aborted
*/
fbool_t ITC_SleepAndCheckAbortion(ITC_INST_S * inst, int32_t timeInMs)
{
    uint32_t*   wakeupMsg;
#ifndef _LINUX_FAPI
    uint64_t    startStamp = FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC);
#else
    uint64_t    startStamp = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
#endif
    int32_t     sleepTime = timeInMs;
    uint64_t    timeSlept;

    if ( timeInMs == 0 ) return FTRUE;

    do
    {
        if ( inst->abortReq == FTRUE )
        {
            return FFALSE;
        }

        /* round up to next available sleep time: necessary because of
           FAMOS implementation. */
        if ( (sleepTime % (int32_t)inst->osTickPeriod) > 0 )
        {
            sleepTime += (int32_t)inst->osTickPeriod;
            sleepTime -= (sleepTime % (int32_t)inst->osTickPeriod);
        }

#ifndef _LINUX_FAPI
        (void)RTOS_GetMailbox(inst->wakeupBox, &wakeupMsg, sleepTime);
#else
        (void)FAPEX_SYS_GetMailbox(inst->wakeupBox, &wakeupMsg, sleepTime);
#endif
        if ( inst->abortReq )
        {
            return FFALSE;
        }

        if ( wakeupMsg == NULL )
        {
            /* No message received -> Full timeout is expired */
            break;
        }
        else
        {
            /* check sleep period: Mailbox could have been triggered by
               a deprecated message. */

            timeSlept = UTILS_GetTimerStampInterval(startStamp,
#ifndef _LINUX_FAPI
                         FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC));
#else
                         FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC));
#endif
            sleepTime = timeInMs - (int32_t)timeSlept;
        }
    }
    while ( sleepTime > 0 );

    return FTRUE;
}

int32_t ITC_AckSend(ITC_INST_S * inst, void *ack)
{
#ifndef _LINUX_FAPI
    if(!RTOS_SetMailbox(inst->ackBox, ack, RTOS_NO_SUSPEND))
#else
    if(!FAPEX_SYS_SetMailbox(inst->ackBox, ack, FAPEX_SYS_NO_SUSPEND))
#endif
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }
    else
    {
        return FAPI_OK;
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
