/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/resources.c
**
**              $Id: resources.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Resource management module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    715: Symbol not referenced.
** - Info    774: Boolean within 'if' always evaluates to True.
**
*****************************************************************************
*/
/*lint -save -e641 -e715 -e774 */
#define DEBUG 4

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
//#include <fapex/sys_driver.h>         /* global type definitions */
#include <fapex/abst_types.h>
#include <fapex/abst_libtsd.h>
#include <fapex/abst_libboardifc.h>

#include "utils.h"
#include "sys_api.h"
#include "sys_priv.h"

#ifdef APPL_PVR_ENABLED
#include "pvr_list.h"
#endif /* APPL_PVR_ENABLED */

#ifdef APPL_MPEG_ENCODER_ENABLED
#include "mpenc.h"
#endif /* APPL_MPEG_ENCODER_ENABLED */

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#if ( RSRC_TSD_NUMS < 2 )
    #error Configuration error: TSD
#endif

#if ( RSRC_VD_NUMS < 1 )
    #error Configuration error: VD
#endif

/** switch frequency from low- to high-band in kHz */
#define RSRC_BAND_SW  1950000

#ifdef APPL_PVR_ENABLED
    #define RSRC_REC_1    0
    #define RSRC_REC_2    1
#endif

typedef enum
{
    RSRC_AVL_LEV_NOT_AVL = 0,
    RSRC_AVL_LEV_1,
    //RSRC_AVL_LEV_2, //not used yet
    //RSRC_AVL_LEV_3, //not used yet
    //RSRC_AVL_LEV_4, //not used yet
    //RSRC_AVL_LEV_5, //not used yet

    #ifdef APPL_USALS_ENABLED
    RSRC_AVL_LEV_6,
    #endif /* APPL_USALS_ENABLED */

    #ifdef APPL_DISEQC12_ENABLED
    RSRC_AVL_LEV_7,
    #endif /* APPL_DISEQC12_ENABLED */

    RSRC_AVL_LEV_8,
    RSRC_AVL_LEV_9,
    RSRC_AVL_LEV_10

} RSRC_AVL_LEV_E;

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    fbool_t             camAvl;     //!< FTRUE: CAM available in given slot.
    fbool_t             casMatch;   //!< 1: CA system matches required system.
    fbool_t             usable;     //!< 1: Module is usable for req. operation.

} RSRC_CI_S;

typedef struct
{
    RSRC_STATUS_S           status;
    const SYS_HWCONFIG_S*   hwCfg;

} RSRC_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

RSRC_DATA_S* pRsrc = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC int32_t     RSRC_AssembleHwOccupation   (RSRC_OP op,
                                                 RSRC_STAT_HW_S* hw);
FSTATIC int32_t     RSRC_SetOccupationLive      (const RSRC_CFG_S* pCfg,
                                                 RSRC_STAT_HW_S* hw);
FSTATIC int32_t     RSRC_SetOccupationScan      (const RSRC_CFG_S* pCfg,
                                                 RSRC_STAT_HW_S* hw);
FSTATIC int32_t     RSRC_SetOccupationTune      (const RSRC_CFG_S* pCfg,
                                                 RSRC_STAT_HW_S* hw);

FSTATIC RSRC_AVL_LEV_E RSRC_FrontendAvailableForNetwork(int8_t feIdx,
                                                     const SYS_NETWORK_S *
                                                     network,
                                                     fbool_t* motorNeeded,
                                                     const RSRC_STAT_HW_S * hw);
FSTATIC RSRC_AVL_LEV_E RSRC_TSAvailable(RSRC_TS_E ts, RSRC_CFG_S * cfg,
                                        const RSRC_AVL_LEV_E* feAvl,
                                        fbool_t tryCi,
                                        const RSRC_STAT_HW_S * hw);
FSTATIC fbool_t     RSRC_ServiceAvlOnCurrTs(const SLIST_ITEM_S * servItem,
                                            int8_t feIdx,
                                            const RSRC_STAT_HW_S * hw);
FSTATIC int32_t     RSRC_GetFrontendAvailability(const SYS_HWCONFIG_S* hwCfg,
                                                 const SLIST_ITEM_S * servItem,
                                                 RSRC_AVL_LEV_E* feAvl,
                                                 const RSRC_STAT_HW_S * hw);
FSTATIC int32_t     RSRC_GetExclFeAvailability(int8_t feIdx,
                                               const SYS_HWCONFIG_S* hwCfg,
                                               const SYS_NETWORK_S * network,
                                               RSRC_AVL_LEV_E* feAvl,
                                               const RSRC_STAT_HW_S * hw);
FSTATIC void        RSRC_BuildTSPrefLive(const SYS_HWCONFIG_S* hwCfg,
                                         const RSRC_AVL_LEV_E* tsAvl,
                                         const RSRC_CFG_S * targetCfg,
                                         const RSRC_CI_S * ciSlot,
                                         RSRC_TS_E * tsPref);

#ifdef APPL_FESAT_ENABLED

FSTATIC fbool_t     RSRC_ServiceAvlOnCurrBand(const SLIST_ITEM_S * servItem,
                                              int8_t refFeIdx,
                                              const RSRC_STAT_HW_S * hw);

#endif /* APPL_FESAT_ENABLED */

#ifdef APPL_PVR_ENABLED

FSTATIC int32_t     RSRC_SetOccupationRec(const RSRC_CFG_S * pCfg,
                                          RSRC_STAT_HW_S * hw);
FSTATIC int32_t     RSRC_SetOccupationPlay(const RSRC_CFG_S * pCfg,
                                           RSRC_STAT_HW_S * hw);
#ifdef APPL_CI_ENABLED

FSTATIC int32_t     RSRC_SetOccupationDescrRec(const RSRC_CFG_S * pCfg,
                                               RSRC_STAT_HW_S * hw);
#endif /* APPL_CI_ENABLED */

#endif /* APPL_PVR_ENABLED */

#ifdef APPL_PIP_ENABLED

FSTATIC int32_t  RSRC_SetOccupationPip      (const RSRC_OP_PIP_S* pParams,
                                             RSRC_STAT_HW_S* hw);
FSTATIC int32_t  RSRC_TestForPIPConflict    (const SYS_STREAM_INFO_S* pPid,
                                             uint8_t tsdIdx,
                                             const RSRC_STAT_HW_S* hw);
FSTATIC int32_t  RSRC_PIPConflictCompare    (uint16_t pipVideoPid,
                                             const SYS_STREAM_INFO_S*
                                             compStream);

#endif /* APPL_PIP_ENABLED */

#ifdef APPL_CI_ENABLED

FSTATIC RSRC_CI_S   RSRC_CIUsableForService(RSRC_TS_E ts, SLIST_SERVIDX servIdx,
                                            const SLIST_ITEM_S * pServItem,
                                            const RSRC_STAT_HW_S * hw);

#endif /* APPL_CI_ENABLED */

#ifdef APPL_UPDATE_ENABLED

FSTATIC int32_t     RSRC_SetOccupationUpdate(RSRC_STAT_HW_S * hw);

#endif /* APPL_UPDATE_ENABLED */

#ifdef APPL_CA_ENABLED

FSTATIC RSRC_CAS_E RSRC_CASystemAvailable (const SLIST_CA_INFO_S* pCaInfo);

#endif /* APPL_CA_ENABLED */

#ifdef APPL_MPEG_ENCODER_ENABLED

FSTATIC FERR  RSRC_SetOccupationMpenc     (const RSRC_CFG_S* pCfg,
                                           RSRC_STAT_HW_S* hw);
FSTATIC FERR  RSRC_MpencPathAvailable     (U8 feIdx, RSRC_TS_E ts,
                                           const RSRC_STAT_HW_S* hw);

#endif /* APPL_MPEG_ENCODER_ENABLED */


#if (DEBUG == 0)

#define RSRC_DebugPrintConfig(a, b)

#else

FSTATIC void        RSRC_DebugPrintConfig(const char_t* pStr,
                                          const RSRC_CFG_S* pCfg);

#endif /* not defined DEBUG */

#ifdef APPL_DEBUG_MODE

FSTATIC void RSRC_CheckConsistencyTsInput(const RSRC_CFG_S * pCfg,
                                          RSRC_STAT_HW_S * hw);

FSTATIC void RSRC_CheckConsistencyFeTuning(const RSRC_CFG_S * pCfg,
                                           const SLIST_ITEM_S * servItem,
                                           RSRC_STAT_HW_S * hw);

FSTATIC void RSRC_CheckConsistencyCiUsage(const RSRC_CFG_S * pCfg,
                                          RSRC_STAT_HW_S * hw);

#endif /* APPL_DEBUG_MODE */
/*
Configuration restrictions:

TSD1: live TV / PVR Playback / Scan (service search)
TSD2: Record 1
TSD3: Record 2

CI: only 1 service can be decoded in parallel

*/

#ifdef APPL_DEBUG_MODE

FSTATIC void RSRC_CheckConsistencyTsInput(const RSRC_CFG_S * pCfg,
                                          RSRC_STAT_HW_S * hw)
{
    if(hw->tsd[pCfg->tsdIdx].use != 0)
    {
        DBG_Assert(hw->tsd[pCfg->tsdIdx].tsIn == pCfg->ts);
    }
}

FSTATIC void RSRC_CheckConsistencyFeTuning(const RSRC_CFG_S * pCfg,
                                           const SLIST_ITEM_S * servItem,
                                           RSRC_STAT_HW_S * hw)
{

    /* Martin return here*/
    return;

    if((hw->fe[pCfg->feIdx].use & (~RSRC_STAT_FE_OPENED)) != 0)
    {
        DBG_Assert((hw->fe[pCfg->feIdx].satIdx == servItem->network.satIdx) &&
                   (hw->fe[pCfg->feIdx].tpIdx == servItem->tpIdx));
    }
}

/*lint -save -e715*/
/*  Symbol 'tryCi' an'hw' only referenced if APPL_CI_ENABLED */
FSTATIC void RSRC_CheckConsistencyCiUsage(const RSRC_CFG_S * pCfg,
                                          RSRC_STAT_HW_S * hw)
{
    #ifdef APPL_CI_ENABLED

    if ( pCfg->useCi )
    {
        if((hw->ci[pCfg->ts].use & RSRC_STAT_CI_ANY) != 0)
        {
            DBG_Assert(hw->ci[pCfg->ts].servIdx == pCfg->servIdx);
        }
    }
    #endif /* APPL_CI_ENABLED */
}
/*lint -restore */

#endif /* APPL_DEBUG_MODE */

int32_t RSRC_Init(void)
{
    uint8_t   feCnt;
    RSRC_TS_E tsCnt;

    pRsrc = UTILS_Malloc(sizeof(RSRC_DATA_S));
    if ( pRsrc == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(pRsrc, 0, sizeof(RSRC_DATA_S));

    pRsrc->hwCfg = SYS_GetHWConfig();

    for(feCnt = 0; feCnt < RSRC_FE_MAX; feCnt++)
    {
        pRsrc->status.hw.fe[feCnt].type = FE_SYSTEM_TYPE_DIRECT;
        pRsrc->status.hw.fe[feCnt].satIdx = -1;
        pRsrc->status.hw.fe[feCnt].tpIdx = -1;
    }

    for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
    {
        pRsrc->status.hw.ci[tsCnt].servIdx = SLIST_SERVIDX_NONE;
    }

    return FAPI_OK;
}

RSRC_STATUS_S* RSRC_GetStatus (void)
{
    DBG_Assert(pRsrc != NULL);

    return &(pRsrc->status);
}

void RSRC_ConstraintsSetAll (RSRC_CONSTR_S * pConstr, fbool_t allow)
{
    RSRC_TS_E   tsCnt;
    uint8_t     cnt;

    if(pConstr != NULL)
    {
        for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
        {
            pConstr->omitTs[tsCnt] = (fbool_t)!allow;
        }

        for(cnt = 0; cnt < RSRC_TSD_NUMS; cnt++)
        {
            pConstr->omitTsd[cnt] = (fbool_t)!allow;
        }
    }
}

void RSRC_SetFeStatus(int8_t feIdx, fbool_t open, FE_SYSTEM_TYPE_E type)
{
    DBG_Assert(pRsrc != NULL);

    if((feIdx < 0) || (feIdx >= RSRC_FE_MAX))
        return;

    pRsrc->status.hw.fe[feIdx].type = type;

    if(open)
        pRsrc->status.hw.fe[feIdx].use |= RSRC_STAT_FE_OPENED;
    else
        pRsrc->status.hw.fe[feIdx].use &= (~RSRC_STAT_FE_OPENED);
}

FSTATIC int32_t RSRC_AssembleHwOccupation(RSRC_OP op, RSRC_STAT_HW_S * hw)
{
    int32_t             retVal;
    uint8_t             feCnt;
    RSRC_TS_E           tsCnt;

    /* ----- Initialise structure ------------------------------------------ */

    memset(hw, 0, sizeof(*hw));

    for(feCnt = 0; feCnt < RSRC_FE_MAX; feCnt++)
    {
        hw->fe[feCnt].type = pRsrc->status.hw.fe[feCnt].type;
        hw->fe[feCnt].satIdx = -1;
        hw->fe[feCnt].tpIdx = -1;

        /* set fe's "open" status */
        if((pRsrc->status.hw.fe[feCnt].use & RSRC_STAT_FE_OPENED) != 0)
        {
            hw->fe[feCnt].use |= RSRC_STAT_FE_OPENED;
        }
    }

    for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
    {
        hw->ci[tsCnt].servIdx = SLIST_SERVIDX_NONE;
    }

    /* --------------------------------------------------------------------- */

    if((op & pRsrc->status.opStatus & RSRC_OP_LIVE) != 0)
    {
        retVal = RSRC_SetOccupationLive(&(pRsrc->status.live.cfg), hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    /* --------------------------------------------------------------------- */

    #ifdef APPL_PVR_ENABLED

    if((op & pRsrc->status.opStatus & RSRC_OP_REC1) != 0)
    {
        retVal = RSRC_SetOccupationRec (&(pRsrc->status.rec[RSRC_REC_1].cfg),
                                        hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    #if ( RSRC_REC_MAX > 1 )

    if((op & pRsrc->status.opStatus & RSRC_OP_REC2) != 0)
    {
        retVal = RSRC_SetOccupationRec(&(pRsrc->status.rec[RSRC_REC_2].cfg),
                                       hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    #endif /* ( RSRC_REC_MAX > 1 ) */

    if((op & pRsrc->status.opStatus & RSRC_OP_PLAY) != 0)
    {
        retVal = RSRC_SetOccupationPlay(&(pRsrc->status.play.cfg), hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    #ifdef APPL_CI_ENABLED

    if((op & pRsrc->status.opStatus & RSRC_OP_DESCRREC) != 0)
    {
        retVal = RSRC_SetOccupationDescrRec(&(pRsrc->status.descrRec.cfg), hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    #endif /* APPL_CI_ENABLED */

    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_PIP_ENABLED

    if((op & pRsrc->status.opStatus & RSRC_OP_PIP) != 0)
    {
        retVal = RSRC_SetOccupationPip(&(pRsrc->status.pip), hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    #endif /* APPL_PIP_ENABLED */

    #ifdef APPL_UPDATE_ENABLED

    if((op & pRsrc->status.opStatus & RSRC_OP_UPDATE) != 0)
    {
        retVal = RSRC_SetOccupationUpdate(hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    #endif /* APPL_UPDATE_ENABLED */

    if((op & pRsrc->status.opStatus & RSRC_OP_SCAN) != 0)
    {
        retVal = RSRC_SetOccupationScan(&(pRsrc->status.scan.cfg), hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    if((op & pRsrc->status.opStatus & RSRC_OP_TUNE1) != 0)
    {
        retVal = RSRC_SetOccupationTune(&(pRsrc->status.tune[0].cfg), hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    if((op & pRsrc->status.opStatus & RSRC_OP_TUNE2) != 0)
    {
        retVal = RSRC_SetOccupationTune(&(pRsrc->status.tune[1].cfg), hw);
        if(retVal != FAPI_OK)
            return retVal;
    }

    #ifdef APPL_MPEG_ENCODER_ENABLED

    if ( (op & pRsrc->status.opStatus & RSRC_OP_MPENCMONITOR) != 0 )
    {
        retVal = RSRC_SetOccupationMpenc(&(pRsrc->status.mpencMonitor.cfg), hw);
        if ( retVal != FAPI_OK ) return retVal;
    }

    #endif /* APPL_MPEG_ENCODER_ENABLED */

    return FAPI_OK;
}

// checks if live presentation of requested service is available
// optionally returns necessary configuration if presentation is possible!
int32_t RSRC_ServiceAvlForLive(SLIST_SERVIDX servIdx, RSRC_OP op,
                               const RSRC_CONSTR_S * pConstr, RSRC_CFG_S * pCfg)
{
    SLIST_ITEM_S        servItem;
    RSRC_AVL_LEV_E      feAvl[RSRC_FE_MAX];
    RSRC_CFG_S          targetCfg[RSRC_TS_NUMS];
    RSRC_AVL_LEV_E      tsAvl[RSRC_TS_NUMS];
    RSRC_TS_E           tsPref[RSRC_TS_NUMS];
    RSRC_TS_E           tsCnt;
    RSRC_TS_E           cnt;
    RSRC_CI_S           ciSlot[RSRC_TS_NUMS];
    int32_t             retVal = FAPI_OK;
    RSRC_CAS_E          casFound = RSRC_CAS_NONE;
    RSRC_STAT_HW_S      hw;

    #ifdef APPL_CA_ENABLED
    SLIST_CA_INFO_S     caInfo;
    #endif /* APPL_CA_ENABLED */

    if(servIdx < 0)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* check TSD constraints */
    if((pConstr != NULL) && (pConstr->omitTsd[RSRC_TSD_LIVE]))
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[resource][RSRC_ServiceAvlForLive] RSRC_AssembleHwOccupation() %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }
    /* check operation status: Live operation currently active? */
    if((hw.opStat & RSRC_OP_LIVE) != 0)
        return SMARTGO_SYS_ERR_RSRC_LIVE;

    #ifdef APPL_UPDATE_ENABLED

    if((hw.opStat & RSRC_OP_UPDATE) != 0)
    {
        return SMARTGO_SYS_ERR_RSRC_UPDATE;
    }

    #endif /* APPL_UPDATE_ENABLED */

    for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
    {
        memset(ciSlot + (int32_t)tsCnt, 0, sizeof(RSRC_CI_S));

        tsPref[tsCnt] = RSRC_TS_NONE;
    }

    /* First check if TSD-1 is available. Recording is not intended on TSD-1
       but nevertheless checked. */

    #if 0
    if ( (hw.tsd[RSRC_TSD_LIVE].use & (~RSRC_STAT_TSD_PIP)) != 0 )
    {   
        FAPI_SYS_PRINT_DEBUG(1, "[resource][RSRC_ServiceAvlForLive] hw.tsd[RSRC_TSD_LIVE].use = 0x%02X\n", hw.tsd[RSRC_TSD_LIVE].use);
        FAPI_SYS_PRINT_DEBUG(1, "[resource][RSRC_ServiceAvlForLive] RSRC_STAT_TSD_PIP test %s\n", SYS_GetErrorString(SMARTGO_SYS_ERR_RSRC));
        return SMARTGO_SYS_ERR_RSRC;
    }
    #endif

    retVal = SLIST_GetServByIndex(servIdx, &servItem);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[resource][RSRC_ServiceAvlForLive] SLIST_GetServByIndex() %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }
    #ifdef APPL_PIP_ENABLED

    retVal = RSRC_TestForPIPConflict(&(servItem.strPid), RSRC_TSD_LIVE, &hw);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[resource][RSRC_ServiceAvlForLive] RSRC_TestForPIPConflict() %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }
    #endif /* APPL_PIP_ENABLED */

    /* ----- get available frontends --------------------------------------- */

    /* frontend(s) for requested network available? */
    retVal = RSRC_GetFrontendAvailability(pRsrc->hwCfg, &servItem, feAvl, &hw);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[resource][RSRC_ServiceAvlForLive] RSRC_GetFrontendAvailability() %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }
    /* ----- check if decodable by embedded CA system ---------------------- */

    #ifdef APPL_CA_ENABLED

    if(SLIST_CaInfoGet(servIdx, &caInfo) == FAPI_OK)
    {
        casFound = RSRC_CASystemAvailable(&caInfo);
    }

    #endif /* APPL_CA_ENABLED */

    /* ----- check CI availability and usability --------------------------- */

    #ifdef APPL_CI_ENABLED

    if(casFound == RSRC_CAS_NONE)
    {
        for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
        {
            ciSlot[tsCnt] = RSRC_CIUsableForService(tsCnt, servIdx, &servItem,
                                                    &hw);
        }
    }

    #endif /* APPL_CI_ENABLED */

    /* ----- check available TS inputs ------------------------------------- */

    for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
    {
        targetCfg[tsCnt].op      = RSRC_OP_LIVE;
        targetCfg[tsCnt].servIdx = servIdx;
        targetCfg[tsCnt].pvrIdx  = -1;              //!< Unused.
        targetCfg[tsCnt].tsdIdx  = RSRC_TSD_LIVE;   //!< Fixed parameter.
        targetCfg[tsCnt].useCi   = FFALSE;          //!< Default: no CI usage.
        targetCfg[tsCnt].cas     = casFound;

        for(cnt = (RSRC_TS_E)0; cnt < RSRC_TS_NUMS; cnt++)
        {
            targetCfg[tsCnt].ciMatch[cnt] = ciSlot[cnt].casMatch;
        }

        /* check TS constraints */
        if((pConstr != NULL) && (pConstr->omitTs[tsCnt]))
        {
            tsAvl[tsCnt] = RSRC_AVL_LEV_NOT_AVL;
        }
        /* TSD might be already used (for PIP): test for TS input configuration
           conflicts */
        else if((hw.tsd[RSRC_TSD_LIVE].use != 0) &&
                (hw.tsd[RSRC_TSD_LIVE].tsIn != tsCnt))
        {
            tsAvl[tsCnt] = RSRC_AVL_LEV_NOT_AVL;
        }
        else
        {
            tsAvl[tsCnt] = RSRC_TSAvailable(tsCnt, targetCfg + (int32_t)tsCnt,
                                            feAvl,
                                            ciSlot[tsCnt].usable, &hw);
        }
    }


    /* Martin added for temp pass (20100519)*/
    tsAvl[RSRC_TS_A] = RSRC_AVL_LEV_1;

    /* check if any TS input is available */
    if( (tsAvl[RSRC_TS_A] == RSRC_AVL_LEV_NOT_AVL) &&
        (tsAvl[RSRC_TS_B] == RSRC_AVL_LEV_NOT_AVL))
    {
        FAPI_SYS_PRINT_DEBUG(1, "[resource][RSRC_ServiceAvlForLive] RSRC_AVL_LEV_NOT_AVL test %s\n", SYS_GetErrorString(SMARTGO_SYS_ERR_RSRC));

        return SMARTGO_SYS_ERR_RSRC;
    }

    /* ----- choose TS input by preference --------------------------------- */

    RSRC_BuildTSPrefLive(pRsrc->hwCfg, tsAvl, targetCfg, ciSlot, tsPref);

    if(tsPref[0] == RSRC_TS_NONE)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[resource][RSRC_ServiceAvlForLive] RSRC_BuildTSPrefLive() %s\n", SYS_GetErrorString(SMARTGO_SYS_ERR_INTERNAL));
        return SMARTGO_SYS_ERR_INTERNAL;
    }
    if(pCfg != NULL)
        *pCfg = targetCfg[tsPref[0]];

    return FAPI_OK;
}

int32_t RSRC_SetLiveService(const RSRC_CFG_S * pCfg)
{
    int32_t             retVal = FAPI_OK;

    RSRC_DebugPrintConfig("Set Live", pCfg);

    retVal = RSRC_SetOccupationLive(pCfg, &(pRsrc->status.hw));
    if(retVal != FAPI_OK)
        return retVal;

    pRsrc->status.opStatus |= pCfg->op;
    pRsrc->status.live.cfg  = *pCfg;

    return retVal;
}

int32_t RSRC_LiveServiceStopped(void)
{
    RSRC_CFG_S         *pCfg = &(pRsrc->status.live.cfg);

    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] Live stopped.\n");

    if((pRsrc->status.opStatus & RSRC_OP_LIVE) == 0)
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    pRsrc->status.hw.fe[pCfg->feIdx].use   &= (~RSRC_STAT_FE_LIVE);
    pRsrc->status.hw.tsd[pCfg->tsdIdx].use &= (~RSRC_STAT_TSD_LIVE);
    pRsrc->status.hw.vd[0]                  = RSRC_STAT_VD_UNUSED;
    pRsrc->status.hw.videoout               = RSRC_STAT_VOUT_UNUSED;

    #ifdef APPL_CI_ENABLED

    /* release CI occupation */
    pRsrc->status.hw.ci[pCfg->ts].use      &= (~RSRC_STAT_CI_LIVE);

    #endif /* APPL_CI_ENABLED */

    pRsrc->status.opStatus                 &= (~RSRC_OP_LIVE);
    pRsrc->status.hw.opStat                &= (~RSRC_OP_LIVE);

    return FAPI_OK;
}

/*
- Scan is only performed on TSD-1: parallel scans on multiple frontends
  not allowed

*/
int32_t RSRC_NetworkAvlForScan(const SYS_NETWORK_S * network, RSRC_OP op,
                               RSRC_CFG_S * pCfg)
{
    RSRC_CFG_S          targetCfg[RSRC_TS_NUMS];
    RSRC_AVL_LEV_E      tsAvl[RSRC_TS_NUMS];
    RSRC_AVL_LEV_E      feAvl[RSRC_FE_MAX];
    RSRC_TS_E           tsChosen = RSRC_TS_A;
    RSRC_TS_E           tsCnt;
    int32_t             retVal = FAPI_OK;
    RSRC_STAT_HW_S      hw;

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    /* check operation status: Multiple scan operations in parallel are
       prohibited. */
    if((hw.opStat & RSRC_OP_SCAN) != 0)
        return SMARTGO_SYS_ERR_RSRC_SCAN;

    #ifdef APPL_UPDATE_ENABLED

    if((hw.opStat & RSRC_OP_UPDATE) != 0)
    {
        return SMARTGO_SYS_ERR_RSRC_UPDATE;
    }

    #endif /* APPL_UPDATE_ENABLED */

    /* First check if TSD is available. Due to TSD filtering issues, TSD cannot
       be shared between scan and PIP operations. */

    if ( hw.tsd[RSRC_TSD_SCAN].use != 0 )
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    /* --------------------------------------------------------------------- */

    /* check for frontends available to access requested network */
    retVal = RSRC_GetExclFeAvailability(-1, pRsrc->hwCfg, network, feAvl, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    /* ----- check available TS inputs ------------------------------------- */

    for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
    {
        targetCfg[tsCnt].op                 = RSRC_OP_SCAN;
        targetCfg[tsCnt].servIdx            = 0;               //!< Unused.
        targetCfg[tsCnt].pvrIdx             = -1;              //!< Unused.
        targetCfg[tsCnt].tsdIdx             = RSRC_TSD_SCAN;
        targetCfg[tsCnt].useCi              = FFALSE;          //!< Unused.
        targetCfg[tsCnt].cas                = RSRC_CAS_NONE;   //!< Unused.
        targetCfg[tsCnt].ciMatch[RSRC_TS_A] = FFALSE;          //!< Unused.
        targetCfg[tsCnt].ciMatch[RSRC_TS_B] = FFALSE;          //!< Unused.

        /* TSD might be already used: test for TS input configuration
           conflicts */
        if((hw.tsd[RSRC_TSD_SCAN].use != 0) &&
           (hw.tsd[RSRC_TSD_SCAN].tsIn != tsCnt))
        {
            tsAvl[tsCnt] = RSRC_AVL_LEV_NOT_AVL;
        }
        else
        {
            tsAvl[tsCnt] = RSRC_TSAvailable(tsCnt, targetCfg + (int32_t)tsCnt,
                                            feAvl, FFALSE, &hw);
        }
    }

    /* ----- choose TS input by preference level --------------------------- */

    if(tsAvl[RSRC_TS_B] > tsAvl[RSRC_TS_A])
        tsChosen = RSRC_TS_B;
    else if(tsAvl[RSRC_TS_A] > RSRC_AVL_LEV_NOT_AVL)
        tsChosen = RSRC_TS_A;
    else
    {
        retVal = SMARTGO_SYS_ERR_RSRC;
    }

    if((retVal == FAPI_OK) && (pCfg != NULL))
    {
        *pCfg = targetCfg[tsChosen];
    }

    return retVal;
}

int32_t RSRC_SetScan(const RSRC_CFG_S * pCfg)
{
    int32_t             retVal;

    RSRC_DebugPrintConfig("Set Scan", pCfg);

    retVal = RSRC_SetOccupationScan(pCfg, &(pRsrc->status.hw));
    if(retVal != FAPI_OK)
        return retVal;

    pRsrc->status.opStatus |= pCfg->op;
    pRsrc->status.scan.cfg  = *pCfg;

    return FAPI_OK;
}

int32_t RSRC_ScanStopped(int8_t feIdx)
{
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] Scan stopped: feIdx %i.\n", feIdx);

    if((feIdx < 0) || (feIdx >= RSRC_FE_MAX))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if((pRsrc->status.opStatus & RSRC_OP_SCAN) == 0)
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    pRsrc->status.opStatus                  &= (~RSRC_OP_SCAN);
    pRsrc->status.hw.opStat                 &= (~RSRC_OP_SCAN);
    pRsrc->status.hw.fe[feIdx].use          &= (~RSRC_STAT_FE_SCAN);
    pRsrc->status.hw.tsd[RSRC_TSD_SCAN].use &= (~RSRC_STAT_TSD_SCAN);

    return FAPI_OK;
}

/*
feIdx: >= 0: force frontend, -1: choose automatically
*/
int32_t RSRC_NetworkAvlForTunemode(int8_t feIdx, const SYS_NETWORK_S * network,
                                   RSRC_OP op, RSRC_CFG_S * pCfg)
{
    RSRC_CFG_S          targetCfg[RSRC_TS_NUMS];
    RSRC_AVL_LEV_E      tsAvl[RSRC_TS_NUMS];
    RSRC_AVL_LEV_E      feAvl[RSRC_FE_MAX];
    RSRC_TS_E           tsChosen = RSRC_TS_A;
    RSRC_TS_E           tsCnt;
    int32_t             retVal = FAPI_OK;
    RSRC_STAT_HW_S      hw;
    uint8_t             stat;

    if ( feIdx >= (int8_t)(pRsrc->hwCfg->feNums) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    if(feIdx >= 0)
    {
        if(feIdx == 0)
            stat = RSRC_OP_TUNE1;
        else
            stat = RSRC_OP_TUNE2;

        /* check operation status: Tune mode on requested frontend already
           active? */
        if((hw.opStat & stat) != 0)
            return SMARTGO_SYS_ERR_RSRC_TUNEMODE;
    }

    /* NOTE: TSDs are not occupied by tune mode. */

    /* --------------------------------------------------------------------- */

    /* check for frontends available to access requested network */
    retVal = RSRC_GetExclFeAvailability(feIdx, pRsrc->hwCfg, network, feAvl,
                                        &hw);
    if(retVal != FAPI_OK)
        return retVal;

    /* ----- check available TS inputs ------------------------------------- */

    for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
    {
        targetCfg[tsCnt].servIdx            = 0;               //!< Unused.
        targetCfg[tsCnt].pvrIdx             = -1;              //!< Unused.
        targetCfg[tsCnt].tsdIdx             = RSRC_TSD_LIVE;   //!< Unused.
        targetCfg[tsCnt].useCi              = FFALSE;          //!< Unused.
        targetCfg[tsCnt].cas                = RSRC_CAS_NONE;   //!< Unused.
        targetCfg[tsCnt].ciMatch[RSRC_TS_A] = FFALSE;          //!< Unused.
        targetCfg[tsCnt].ciMatch[RSRC_TS_B] = FFALSE;          //!< Unused.

        tsAvl[tsCnt] = RSRC_TSAvailable (tsCnt, targetCfg + (int32_t)tsCnt,
                                         feAvl, FFALSE, &hw);
    }

    /* ----- choose TS input by preference level --------------------------- */

    if(tsAvl[RSRC_TS_B] > tsAvl[RSRC_TS_A])
        tsChosen = RSRC_TS_B;
    else if(tsAvl[RSRC_TS_A] > RSRC_AVL_LEV_NOT_AVL)
        tsChosen = RSRC_TS_A;
    else
    {
        retVal = SMARTGO_SYS_ERR_RSRC;
    }

    if((retVal == FAPI_OK) && (pCfg != NULL))
    {
        if ( targetCfg[tsChosen].feIdx == 0 )
        {
            targetCfg[tsChosen].op = RSRC_OP_TUNE1;
        }
        else
        {
            targetCfg[tsChosen].op = RSRC_OP_TUNE2;
        }

        *pCfg = targetCfg[tsChosen];
    }

    return retVal;
}

int32_t RSRC_SetTunemode(const RSRC_CFG_S * pCfg)
{
    int32_t             retVal;

    RSRC_DebugPrintConfig("Set Tunemode", pCfg);

    DBG_Assert(pCfg->feIdx < pRsrc->hwCfg->feNums);

    retVal = RSRC_SetOccupationTune(pCfg, &(pRsrc->status.hw));
    if(retVal != FAPI_OK)
        return retVal;

    pRsrc->status.opStatus              |= pCfg->op;
    pRsrc->status.tune[pCfg->feIdx].cfg  = *pCfg;

    return FAPI_OK;
}

int32_t RSRC_TunemodeStopped(int8_t feIdx)
{
    RSRC_OP             stat;

    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] Tunemode stopped: feIdx %i.\n", feIdx);

    if((feIdx < 0) || (feIdx >= RSRC_FE_MAX))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if(feIdx == 0)
        stat = RSRC_OP_TUNE1;
    else
        stat = RSRC_OP_TUNE2;

    if((pRsrc->status.opStatus & stat) == 0)
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    pRsrc->status.opStatus &= (~stat);
    pRsrc->status.hw.opStat &= (~stat);
    pRsrc->status.hw.fe[feIdx].use &= (~RSRC_STAT_FE_TUNEMODE);

    return FAPI_OK;
}


#ifdef APPL_PVR_ENABLED

/* ------------------------------------------------------------------------- */

int32_t RSRC_ServiceAvlForRecord(SLIST_SERVIDX servIdx, RSRC_OP op,
                                 const RSRC_CONSTR_S * pConstr,
                                 RSRC_CFG_S * pCfg)
{
    SLIST_ITEM_S        servItem;
    RSRC_CFG_S          targetCfg[RSRC_TS_NUMS];
    uint8_t             tsdIdx;
    RSRC_AVL_LEV_E      feAvl[RSRC_FE_MAX];
    RSRC_AVL_LEV_E      tsAvl[RSRC_TS_NUMS];
    fbool_t             tsdAvl[RSRC_TSD_NUMS];
    uint8_t             numTsdAvl = 0;
    RSRC_TS_E           tsPref[RSRC_TS_NUMS];
    fbool_t             needCi = FFALSE;
    RSRC_CI_S           ciSlot[RSRC_TS_NUMS];
    RSRC_TS_E           tsChosen = RSRC_TS_A;
    int32_t             retVal = FAPI_OK;
    RSRC_CAS_E          casFound = RSRC_CAS_NONE;
    RSRC_TS_E           cnt;
    RSRC_TS_E           tsCnt;
    uint16_t            index;
    RSRC_STAT_HW_S      hw;


    #ifdef APPL_CA_ENABLED
    SLIST_CA_INFO_S     caInfo;
    #endif /* APPL_CA_ENABLED */

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    #ifdef APPL_UPDATE_ENABLED

    if((hw.opStat & RSRC_OP_UPDATE) != 0)
    {
        return SMARTGO_SYS_ERR_RSRC_UPDATE;
    }

    #endif /* APPL_UPDATE_ENABLED */

    for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
    {
        memset(ciSlot + tsCnt, 0, sizeof(RSRC_CI_S));

        tsPref[tsCnt] = RSRC_TS_NONE;

        targetCfg[tsCnt].servIdx = servIdx;
        targetCfg[tsCnt].pvrIdx  = -1;              //!< Unused.
        targetCfg[tsCnt].ts      = tsCnt;
    }

    for ( index = 0; index < RSRC_TSD_NUMS; index++ )
    {
        tsdAvl[index] = FFALSE;
    }

    switch ( servIdx )
    {
        #ifdef APPL_MPEG_ENCODER_ENABLED

        case SLIST_SERVIDX_MPENC:
            retVal = MPENC_GetStreamParams (&(servItem.strPid));
            break;

        #endif /* APPL_MPEG_ENCODER_ENABLED */

        case SLIST_SERVIDX_NONE:
        case SLIST_SERVIDX_PLAY:
        default:
            retVal = SLIST_GetServByIndex (servIdx, &servItem);
            break;
    }

    if ( retVal != FAPI_OK ) return retVal;

    /* First check available TSDs */
    for(tsdIdx = RSRC_TSD_REC1; tsdIdx < RSRC_TSD_NUMS; tsdIdx++)
    {
        /* check TSD constraints */
        if((pConstr != NULL) && (pConstr->omitTsd[tsdIdx]))
        {
            continue;
        }

        /* NOTE: Conflict with PIP operation is checked separately! */
        if ( (hw.tsd[tsdIdx].use & (~RSRC_STAT_TSD_PIP)) != 0 ) continue;

        #ifdef APPL_PIP_ENABLED

        if ( RSRC_TestForPIPConflict (&(servItem.strPid), tsdIdx, &hw)
             != FAPI_OK )
        {
            continue;
        }

        #endif /* APPL_PIP_ENABLED */

        /* all tests passed */
        tsdAvl[tsdIdx] = FTRUE;
        numTsdAvl++;
    }

    if(numTsdAvl == 0)
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    /* ----- Source is MPEG encoder TS ------------------------------------- */

    /* Determine availability of encoder path. */

    if ( servIdx == SLIST_SERVIDX_MPENC )
    {
        #ifdef APPL_MPEG_ENCODER_ENABLED

        switch ( pRsrc->hwCfg->mpencPos )
        {
            case RSRC_MPENCPOS_FE1:
                tsChosen = RSRC_TS_A;
                targetCfg[tsChosen].feIdx = 0;
                break;

            case RSRC_MPENCPOS_FE2:
                tsChosen = RSRC_TS_B;
                targetCfg[tsChosen].feIdx = 1;
                break;

            default:
            case RSRC_MPENCPOS_NONE:
                return SMARTGO_SYS_ERR_CONFIG_ERROR;
        }

        retVal = RSRC_MpencPathAvailable (targetCfg[tsChosen].feIdx,
                                          targetCfg[tsChosen].ts, &hw);
        if ( retVal != FAPI_OK ) return retVal;

        targetCfg[tsChosen].useCi              = FFALSE;
        targetCfg[tsChosen].ciInput            = FAPI_BOARD_RSRC_CIIN_NONE;
        targetCfg[tsChosen].cas                = RSRC_CAS_NONE;
        targetCfg[tsChosen].ciMatch[RSRC_TS_A] = FFALSE;
        targetCfg[tsChosen].ciMatch[RSRC_TS_B] = FFALSE;

        /* ----- choose TSD ------------------------------------------------ */

        for ( tsdIdx = RSRC_TSD_REC1;
              tsdIdx < RSRC_TSD_NUMS; tsdIdx++ )
        {
            if ( !(tsdAvl[tsdIdx]) ) continue;

            if ( hw.tsd[tsdIdx].use == 0 ) /* TSD is unused */
            {
                break;
            }
            else /* TSD is already occupied by other operation */
            {
                if ( hw.tsd[tsdIdx].tsIn == tsChosen )
                {
                    break;
                }
            }
        }

        if ( tsdIdx >= RSRC_TSD_NUMS )
        {
            return SMARTGO_SYS_ERR_RSRC;
        }

        targetCfg[tsChosen].tsdIdx  = tsdIdx;

        #else

        return SMARTGO_SYS_ERR_INTERNAL;

        #endif /* APPL_MPEG_ENCODER_ENABLED */
    }

    /* ----- Source is live-TV service ------------------------------------- */

    else
    {
        /* frontend(s) for requested delivery system available? */
        retVal = RSRC_GetFrontendAvailability (pRsrc->hwCfg, &servItem,
                                               feAvl, &hw);
        if(retVal != FAPI_OK)
            return retVal;

        /* ----- check if decodable by embedded CA system ------------------ */

        #ifdef APPL_CA_ENABLED

        if ( SLIST_CaInfoGet (servIdx, &caInfo) == FAPI_OK )
        {
            casFound = RSRC_CASystemAvailable (&caInfo);
        }

        #endif /* APPL_CA_ENABLED */

        /* ----- check CI availability and usability ----------------------- */

        #ifdef APPL_CI_ENABLED

        if ( casFound == RSRC_CAS_NONE )
        {
            for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
            {
                ciSlot[tsCnt] = RSRC_CIUsableForService (tsCnt, servIdx,
                                                         &servItem, &hw);
            }
        }

        #endif /* APPL_CI_ENABLED */

        /* ----- check available TS inputs --------------------------------- */

        for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
        {
            targetCfg[tsCnt].cas     = casFound;

            for ( cnt = (RSRC_TS_E)0; cnt < RSRC_TS_NUMS; cnt++)
            {
                targetCfg[tsCnt].ciMatch[cnt] = ciSlot[cnt].casMatch;
            }

            /* check TS constraints */
            if((pConstr != NULL) && (pConstr->omitTs[tsCnt]))
            {
                tsAvl[tsCnt] = RSRC_AVL_LEV_NOT_AVL;
            }
            else
            {
                tsAvl[tsCnt] = RSRC_TSAvailable (tsCnt, targetCfg + tsCnt,
                                                 feAvl, ciSlot[tsCnt].usable,
                                                 &hw);
            }
        }

        /* check if any TS input is available */
        if( (tsAvl[RSRC_TS_A] == RSRC_AVL_LEV_NOT_AVL) &&
            (tsAvl[RSRC_TS_B] == RSRC_AVL_LEV_NOT_AVL) )
        {
            return SMARTGO_SYS_ERR_RSRC;
        }

        /* ----- choose TS input preference order -------------------------- */

        /* TS input preference is chosen as follows:

        a) CI not needed: no input preference

        b) CI needed:

        b1) both CI usable   -> no preference
        b2) no CI usable     -> no preference
        b3) single CI usable -> recording refused if unavailable

        */

        #ifdef APPL_CI_ENABLED

            // TODO: ndern: nicht nur von Flag abhngig machen!!!
        needCi = SLIST_FlagIsSet(&servItem, SLIST_FLAG_ALLFREE) ? FFALSE
                                                                : FTRUE;

        #endif /* APPL_CI_ENABLED */

        // TODO: ndern: auch insertion status beachten, nicht nur slot availability!
        if ( !needCi )
        {
            if ( tsAvl[RSRC_TS_A] >= tsAvl[RSRC_TS_B] )
            {
                tsPref[0] = RSRC_TS_A;
                tsPref[1] = (tsAvl[RSRC_TS_B] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_B : RSRC_TS_NONE;
            }
            else
            {
                tsPref[0] = RSRC_TS_B;
                tsPref[1] = (tsAvl[RSRC_TS_A] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_A : RSRC_TS_NONE;
            }
        }
        /* both CI usable -> no preference */
        else if ( targetCfg[RSRC_TS_A].useCi &&
                  targetCfg[RSRC_TS_B].useCi )
        {
            if ( tsAvl[RSRC_TS_A] >= tsAvl[RSRC_TS_B] )
            {
                tsPref[0] = RSRC_TS_A;
                tsPref[1] = (tsAvl[RSRC_TS_B] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_B : RSRC_TS_NONE;
            }
            else
            {
                tsPref[0] = RSRC_TS_B;
                tsPref[1] = (tsAvl[RSRC_TS_A] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_A : RSRC_TS_NONE;
            }
        }
        /* no CI module usable -> no preference */
        else if ( (!(targetCfg[RSRC_TS_A].useCi)) &&
                  (!(targetCfg[RSRC_TS_B].useCi)) )
        {
            if ( tsAvl[RSRC_TS_A] >= tsAvl[RSRC_TS_B] )
            {
                tsPref[0] = RSRC_TS_A;
                tsPref[1] = (tsAvl[RSRC_TS_B] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_B : RSRC_TS_NONE;
            }
            else
            {
                tsPref[0] = RSRC_TS_B;
                tsPref[1] = (tsAvl[RSRC_TS_A] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_A : RSRC_TS_NONE;
            }
        }
        else if ( targetCfg[RSRC_TS_A].useCi )
        {
            if(tsAvl[RSRC_TS_A] > RSRC_AVL_LEV_NOT_AVL)
            {
                tsPref[0] = RSRC_TS_A;
                tsPref[1] = (tsAvl[RSRC_TS_B] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_B : RSRC_TS_NONE;
            }
            else
            {
                tsPref[0] = RSRC_TS_B;
                tsPref[1] = (tsAvl[RSRC_TS_A] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_A : RSRC_TS_NONE;
            }
        }
        else if ( targetCfg[RSRC_TS_B].useCi )
        {
            if(tsAvl[RSRC_TS_B] > RSRC_AVL_LEV_NOT_AVL)
            {
                tsPref[0] = RSRC_TS_B;
                tsPref[1] = (tsAvl[RSRC_TS_A] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_A : RSRC_TS_NONE;
            }
            else
            {
                tsPref[0] = RSRC_TS_A;
                tsPref[1] = (tsAvl[RSRC_TS_B] > RSRC_AVL_LEV_NOT_AVL)
                            ? RSRC_TS_B : RSRC_TS_NONE;
            }
        }
        else
        {
            retVal = SMARTGO_SYS_ERR_RSRC;
        }

        if(retVal != FAPI_OK)
            return retVal;

        DBG_Assert(tsPref[0] >= 0);

        /* ----------------------------------------------------------------- */

        for(tsdIdx = RSRC_TSD_REC1; tsdIdx < RSRC_TSD_NUMS; tsdIdx++)
        {
            if(!(tsdAvl[tsdIdx]))
                continue;

            if ( hw.tsd[tsdIdx].use == 0 ) /* TSD is unused */
            {
                tsChosen = tsPref[0];
                break;
            }
            else /* TSD is already occupied by other operation */
            {
                for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
                {
                    if(tsPref[tsCnt] == RSRC_TS_NONE)
                        continue;

                    else if((uint8_t) (tsPref[tsCnt]) == hw.tsd[tsdIdx].tsIn)
                    {
                        tsChosen = tsPref[tsCnt];
                        break;
                    }
                }

                if(tsCnt < RSRC_TS_NUMS)
                    break;
            }
        }

        if(tsdIdx >= RSRC_TSD_NUMS)
        {
            return SMARTGO_SYS_ERR_RSRC;
        }

        targetCfg[tsChosen].tsdIdx = tsdIdx;
    }

    /* --------------------------------------------------------------------- */

    if ( pCfg != NULL )
    {
        if ( targetCfg[tsChosen].tsdIdx == RSRC_TSD_REC1 )
        {
            targetCfg[tsChosen].op = RSRC_OP_REC1;
        }
        else
        {
            targetCfg[tsChosen].op = RSRC_OP_REC2;
        }

        *pCfg = targetCfg[tsChosen];
    }

    return FAPI_OK;
}

int32_t RSRC_SetRecord(const RSRC_CFG_S * pCfg)
{
    int32_t             retVal = FAPI_OK;

    RSRC_DebugPrintConfig("Set Record", pCfg);

    retVal = RSRC_SetOccupationRec(pCfg, &(pRsrc->status.hw));
    if(retVal != FAPI_OK)
        return retVal;

    switch (pCfg->tsdIdx)
    {
        case RSRC_TSD_REC1:
            pRsrc->status.rec[RSRC_REC_1].cfg = *pCfg;
            break;

        #if ( RSRC_REC_MAX > 1 )

        case RSRC_TSD_REC2:
            pRsrc->status.rec[RSRC_REC_2].cfg = *pCfg;
            break;

        #endif /* ( RSRC_REC_MAX > 1 ) */

        default:
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;
    }

    pRsrc->status.opStatus |= pCfg->op;

    return retVal;
}

int32_t RSRC_RecordStopped (uint8_t recIdx)
{
    RSRC_OP     stat;
    RSRC_CFG_S* pCfg;

    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] Record stopped: recIdx %i.\n", recIdx);

    if(recIdx >= RSRC_REC_MAX)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    pCfg = &(pRsrc->status.rec[recIdx].cfg);

    if(recIdx == 0)
        stat = RSRC_OP_REC1;
    else
        stat = RSRC_OP_REC2;

    if((pRsrc->status.opStatus & stat) == 0)
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    pRsrc->status.hw.tsd[pCfg->tsdIdx].use &= (~RSRC_STAT_TSD_RECORD);

    if ( pCfg->servIdx != SLIST_SERVIDX_MPENC )
    {
        if ( pCfg->tsdIdx == RSRC_TSD_REC1 )
        {
            pRsrc->status.hw.fe[pCfg->feIdx].use &= (~RSRC_STAT_FE_REC1);
        }
        else
        {
            pRsrc->status.hw.fe[pCfg->feIdx].use &= (~RSRC_STAT_FE_REC2);
        }
    }

    #ifdef APPL_MPEG_ENCODER_ENABLED

    else
    {
        if ( pCfg->tsdIdx == RSRC_TSD_REC1 )
        {
            pRsrc->status.hw.mpenc.use &= (~RSRC_STAT_MPENC_REC1);
        }
        else
        {
            pRsrc->status.hw.mpenc.use &= (~RSRC_STAT_MPENC_REC2);
        }
    }

    #endif /* APPL_MPEG_ENCODER_ENABLED */

    pRsrc->status.opStatus  &= (~stat);
    pRsrc->status.hw.opStat &= (~stat);

    #ifdef APPL_CI_ENABLED

    if ( pCfg->useCi )
    {
        if ( pCfg->tsdIdx == RSRC_TSD_REC1 )
        {
            pRsrc->status.hw.ci[pCfg->ts].use &= (~RSRC_STAT_CI_REC1);
        }
        else
        {
            pRsrc->status.hw.ci[pCfg->ts].use &= (~RSRC_STAT_CI_REC2);
        }
    }

    #endif /* APPL_CI_ENABLED */

    return FAPI_OK;
}

/* ------------------------------------------------------------------------- */

int32_t RSRC_PlaybackAvailable(PVRLST_IDX idx, RSRC_OP op,
                               const RSRC_CONSTR_S * pConstr, RSRC_CFG_S * pCfg)
{
    int32_t             retVal = FAPI_OK;
    RSRC_CAS_E          casFound = RSRC_CAS_NONE;
    RSRC_CFG_S          targetCfg;
    RSRC_STAT_HW_S      hw;
    PVR_LIST_ITEM_S     item;

    #ifdef APPL_CI_ENABLED
    RSRC_TS_E           tsCnt;
    uint8_t             tsdCnt;
    #endif /* APPL_CI_ENABLED */

    memset(&targetCfg,0,sizeof(RSRC_CFG_S));

    /* check TSD constraints */
    if((pConstr != NULL) && (pConstr->omitTsd[RSRC_TSD_PLAY]))
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    retVal = PVR_LIST_GetItemByIndex(idx, &item, NULL);
    if(retVal != FAPI_OK)
        return retVal;

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    /* check operation status */
    if((hw.opStat & RSRC_OP_PLAY) != 0)
    {
        return SMARTGO_SYS_ERR_RSRC_PLAY;
    }

    #ifdef APPL_UPDATE_ENABLED

    if((hw.opStat & RSRC_OP_UPDATE) != 0)
    {
        return SMARTGO_SYS_ERR_RSRC_UPDATE;
    }

    #endif /* APPL_UPDATE_ENABLED */

    /* check if TSD-1 is available */
    if ( hw.tsd[RSRC_TSD_PLAY].use != 0 )
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    /* ----- check if decodable by embedded CA system ---------------------- */

    #ifdef APPL_CA_ENABLED

    casFound = RSRC_CASystemAvailable(&(item.caInfo));

    #endif /* APPL_CA_ENABLED */

    /* --------------------------------------------------------------------- */

    targetCfg.op      = RSRC_OP_PLAY;
    targetCfg.servIdx = SLIST_SERVIDX_PLAY      ;//!< Special value for non-FE streams.
    targetCfg.pvrIdx  = idx;                    //!< Playback source reference.
    targetCfg.feIdx   = 0;                      //!< Entry not used for playback.
    targetCfg.ts      = RSRC_TS_NONE;           //!< Default: TS data is fed internally.
    targetCfg.tsdIdx  = RSRC_TSD_PLAY;          //!< Static parameter!
    targetCfg.useCi   = FFALSE;                 //!< Default: no TS playback via ext. CI.
    targetCfg.ciInput = FAPI_BOARD_RSRC_CIIN_NONE;
                                                //!< Default: no TS playback via ext. CI.
    targetCfg.cas     = casFound;
    targetCfg.ciMatch[RSRC_TS_A] = FFALSE;      //!< Default.
    targetCfg.ciMatch[RSRC_TS_B] = FFALSE;      //!< Default.

    /* ----- check CI availability and usability --------------------------- */

    #ifdef APPL_CI_ENABLED

    /* If recording is potentially scrambled and not decodable by embedded CA,
       attempt to decode by CI via TS playback. The existance of at least one
       ca_system_id in CA information indicates that CA related streams have
       been included in recording.
     */

    if(casFound == RSRC_CAS_NONE)
    {
        for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
        {
            /* Preconditions for CAS match:
               - Matching CAM must be available.
               - TS playback input must be available for given module.
             */

            targetCfg.ciMatch[tsCnt] = FFALSE;

            if ( (pRsrc->hwCfg->ciAvl[tsCnt]) &&
                ((pRsrc->hwCfg->ciInput[tsCnt] & FAPI_BOARD_RSRC_CIIN_TSGEN)
                != 0) )
            {
                /* check if module would match required CA system */

                if(CISYS_ModuleUseable(tsCnt, &(item.caInfo)))
                {
                    targetCfg.ciMatch[tsCnt] = FTRUE;
                }
            }
        }

        for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
        {
            /* Preconditions for TS input:
               - Must not be involved in any operation.
               - CI slot must be available.
               - TS playback input must be available for CI.
               - CI module must not be occupied by any operation.
             */

            /* check TS constraints */
            if((pConstr != NULL) && (pConstr->omitTs[tsCnt]))
            {
                continue;
            }

            if(!(targetCfg.ciMatch[tsCnt]))
                continue;

            /* Check if TS input is used by checking usage and input
               of all TSDs. */
            for(tsdCnt = 0; tsdCnt < RSRC_TSD_NUMS; tsdCnt++)
            {
                if(((hw.tsd[tsdCnt].use & RSRC_STAT_TSD_ANY) != 0) &&
                   (hw.tsd[tsdCnt].tsIn == tsCnt))
                {
                    /* TS input already occupied by a TSD */
                    break;
                }
            }

            if(tsdCnt < RSRC_TSD_NUMS)
                continue;

            if((hw.ci[tsCnt].use & RSRC_STAT_CI_ANY) == 0)
            {
                targetCfg.useCi   = FTRUE;
                targetCfg.ts      = tsCnt;
                targetCfg.ciInput = FAPI_BOARD_RSRC_CIIN_TSGEN;
                break;
            }
        }
    }

    #endif /* APPL_CI_ENABLED */

    if(pCfg != NULL)
        *pCfg = targetCfg;

    return FAPI_OK;
}

int32_t RSRC_SetPlayback(const RSRC_CFG_S * pCfg)
{
    int32_t             retVal;

    RSRC_DebugPrintConfig("Set Playback", pCfg);

    retVal = RSRC_SetOccupationPlay(pCfg, &(pRsrc->status.hw));
    if(retVal != FAPI_OK)
        return retVal;

    pRsrc->status.opStatus |= pCfg->op;
    pRsrc->status.play.cfg  = *pCfg;

    return FAPI_OK;
}

int32_t RSRC_PlaybackStopped(void)
{
    RSRC_CFG_S         *pCfg = &(pRsrc->status.play.cfg);

    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] Playback stopped.\n");

    if((pRsrc->status.opStatus & RSRC_OP_PLAY) == 0)
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    pRsrc->status.opStatus                  &= (~RSRC_OP_PLAY);
    pRsrc->status.hw.opStat                 &= (~RSRC_OP_PLAY);
    pRsrc->status.hw.tsd[pCfg->tsdIdx].use  &= (~RSRC_STAT_TSD_PLAYBACK);
    pRsrc->status.hw.vd[0]                  =    RSRC_STAT_VD_UNUSED;
    pRsrc->status.hw.videoout               =    RSRC_STAT_VOUT_UNUSED;

    #ifdef APPL_CI_ENABLED

    if ( pCfg->useCi )
    {
        pRsrc->status.hw.ci[pCfg->ts].use &= (~RSRC_STAT_CI_PLAY);
    }

    #endif /* APPL_CI_ENABLED */

    return FAPI_OK;
}

/* ------------------------------------------------------------------------- */

#if defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED)

int32_t RSRC_DescrRecordAvailable(PVRLST_IDX idx, RSRC_OP op,
                                  const RSRC_CONSTR_S * pConstr,
                                  RSRC_CFG_S * pCfg)
{
    RSRC_CFG_S          targetCfg;
    uint8_t             tsdIdx;
    uint8_t             tsdCnt;
    RSRC_TS_E           tsCnt;
    int32_t             retVal;
    PVR_LIST_ITEM_S     item;
    RSRC_STAT_HW_S      hw;

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    /* check operation status */
    if((hw.opStat & RSRC_OP_DESCRREC) != 0)
    {
        return SMARTGO_SYS_ERR_RSRC_RECORD;
    }

    #ifdef APPL_UPDATE_ENABLED

    if((hw.opStat & RSRC_OP_UPDATE) != 0)
    {
        return SMARTGO_SYS_ERR_RSRC_UPDATE;
    }

    #endif /* APPL_UPDATE_ENABLED */

    retVal = PVR_LIST_GetItemByIndex(idx, &item, NULL);
    if(retVal != FAPI_OK)
        return retVal;

    /* First check available TSDs */
    for(tsdIdx = RSRC_TSD_REC1; tsdIdx < RSRC_TSD_NUMS; tsdIdx++)
    {
        /* check TSD constraints */
        if ( (pConstr != NULL) && (pConstr->omitTsd[tsdIdx]) )
        {
            continue;
        }

        /* NOTE: Conflict with PIP operation is checked separately! */
        if ( (hw.tsd[tsdIdx].use & (~RSRC_STAT_TSD_PIP)) != 0 ) continue;

        #ifdef APPL_PIP_ENABLED

        if ( RSRC_TestForPIPConflict (&(item.service.strPid), tsdIdx, &hw) !=
                                      FAPI_OK )
        {
            continue;
        }

        #endif /* APPL_PIP_ENABLED */

        /* all tests passed */
        break;
    }

    if(tsdIdx >= RSRC_TSD_NUMS)
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    /* --------------------------------------------------------------------- */

    /* Set default parameters for non-CI recording. If no matching CAM/TS-input
       is available, these settings will apply. */

    targetCfg.op      = RSRC_OP_DESCRREC;
    targetCfg.servIdx = SLIST_SERVIDX_PLAY;//!< Special value for non-FE streams.
    targetCfg.pvrIdx  = idx;               //!< Source reference.
    targetCfg.feIdx   = 0;                 //!< Entry not used.
    targetCfg.ts      = RSRC_TS_NONE;      //!< Default: internal TS input.
    targetCfg.tsdIdx  = tsdIdx;
    targetCfg.useCi   = FFALSE;            //!< Default: no TS play via ext. CI.
    targetCfg.ciInput = FAPI_BOARD_RSRC_CIIN_NONE;
                                           //!< Default: invalid.
    targetCfg.cas     = RSRC_CAS_NONE;     //!< Entry not used.

    /* ----- check CI availability and usability --------------------------- */

    for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
    {
        /* Preconditions for CAS match:
           - Matching CAM must be available.
           - TS playback input must be available for given module.
         */

        targetCfg.ciMatch[tsCnt] = FFALSE;

        if ( (pRsrc->hwCfg->ciAvl[tsCnt]) &&
            ((pRsrc->hwCfg->ciInput[tsCnt] & FAPI_BOARD_RSRC_CIIN_TSGEN) != 0) )
        {
            /* check if module would match required CA system */

            if(CISYS_ModuleUseable(tsCnt, &(item.caInfo)))
            {
                targetCfg.ciMatch[tsCnt] = FTRUE;
            }
        }
    }

    for(tsCnt = (RSRC_TS_E)0; tsCnt < RSRC_TS_NUMS; tsCnt++)
    {
        if(!(targetCfg.ciMatch[tsCnt]))
            continue;

        /* check TS constraints */
        if((pConstr != NULL) && (pConstr->omitTs[tsCnt]))
        {
            continue;
        }

        /* Check if TS input is used by checking usage and input
           of all TSDs. */
        for(tsdCnt = 0; tsdCnt < RSRC_TSD_NUMS; tsdCnt++)
        {
            if(((hw.tsd[tsdCnt].use & RSRC_STAT_TSD_ANY) != 0) &&
               (hw.tsd[tsdCnt].tsIn == tsCnt))
            {
                /* TS input already occupied by a TSD */
                break;
            }
        }

        if(tsdCnt < RSRC_TSD_NUMS)
            continue;

        if((hw.ci[tsCnt].use & RSRC_STAT_CI_ANY) == 0)
        {
            targetCfg.useCi   = FTRUE;
            targetCfg.ts      = tsCnt;
            targetCfg.ciInput = FAPI_BOARD_RSRC_CIIN_TSGEN;
            break;
        }
    }

    if(pCfg != NULL)
        *pCfg = targetCfg;

    return FAPI_OK;
}

int32_t RSRC_SetDescrRecord(const RSRC_CFG_S * pCfg)
{
    int32_t             retVal;

    RSRC_DebugPrintConfig("Set Descr-Record", pCfg);

    retVal = RSRC_SetOccupationDescrRec(pCfg, &(pRsrc->status.hw));
    if(retVal != FAPI_OK)
        return retVal;

    pRsrc->status.opStatus     |= pCfg->op;
    pRsrc->status.descrRec.cfg  = *pCfg;

    return FAPI_OK;
}

int32_t RSRC_DescrRecordStopped(void)
{
    RSRC_CFG_S         *pCfg = &(pRsrc->status.descrRec.cfg);

    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] Descr-Record stopped.\n");

    if((pRsrc->status.opStatus & RSRC_OP_DESCRREC) == 0)
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    pRsrc->status.opStatus                  &= (~RSRC_OP_DESCRREC);
    pRsrc->status.hw.opStat                 &= (~RSRC_OP_DESCRREC);
    pRsrc->status.hw.tsd[pCfg->tsdIdx].use  &= (~RSRC_STAT_TSD_RECORD);

    if ( pCfg->useCi )
    {
        if ( pCfg->tsdIdx == RSRC_TSD_REC1 )
        {
            pRsrc->status.hw.ci[pCfg->ts].use &= (~RSRC_STAT_CI_REC1);
        }
        else
        {
            pRsrc->status.hw.ci[pCfg->ts].use &= (~RSRC_STAT_CI_REC2);
        }
    }

    return FAPI_OK;
}

#endif /* defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED) */

/* ------------------------------------------------------------------------- */

FSTATIC int32_t RSRC_SetOccupationRec(const RSRC_CFG_S * pCfg,
                                      RSRC_STAT_HW_S * hw)
{
    SLIST_ITEM_S        servItem;
    int32_t             retVal;

    DBG_Assert(pCfg->feIdx  < pRsrc->hwCfg->feNums);
    DBG_Assert(pCfg->ts     < RSRC_TS_NUMS);
    DBG_Assert(pCfg->tsdIdx < RSRC_TSD_NUMS);

    #ifdef APPL_DEBUG_MODE

    /* resource management consistency checks */
    RSRC_CheckConsistencyTsInput (pCfg, hw);
    RSRC_CheckConsistencyCiUsage (pCfg, hw);

    #endif /* APPL_DEBUG_MODE */

    if ( pCfg->servIdx != SLIST_SERVIDX_MPENC )
    {
        retVal = SLIST_GetServByIndex (pCfg->servIdx, &servItem);
        if ( retVal != FAPI_OK ) return retVal;

        #ifdef APPL_DEBUG_MODE

        RSRC_CheckConsistencyFeTuning(pCfg, &servItem, hw);

        #endif /* APPL_DEBUG_MODE */

        if ( pCfg->tsdIdx == RSRC_TSD_REC1 )
        {
            hw->fe[pCfg->feIdx].use |= RSRC_STAT_FE_REC1;
        }
        else
        {
            hw->fe[pCfg->feIdx].use |= RSRC_STAT_FE_REC2;
        }

        hw->fe[pCfg->feIdx].satIdx   = servItem.network.satIdx;
        hw->fe[pCfg->feIdx].tpIdx    = servItem.tpIdx;
    }

    #ifdef APPL_MPEG_ENCODER_ENABLED

    else
    {
        if ( pCfg->tsdIdx == RSRC_TSD_REC1 )
        {
            hw->mpenc.use |= RSRC_STAT_MPENC_REC1;
        }
        else
        {
            hw->mpenc.use |= RSRC_STAT_MPENC_REC2;
        }
    }

    #endif /* APPL_MPEG_ENCODER_ENABLED */

    hw->tsd[pCfg->tsdIdx].use  |= RSRC_STAT_TSD_RECORD;
    hw->tsd[pCfg->tsdIdx].tsIn  = pCfg->ts;

    #ifdef APPL_CI_ENABLED

    if ( pCfg->useCi )
    {
        if ( pCfg->tsdIdx == RSRC_TSD_REC1 )
        {
            hw->ci[pCfg->ts].use |= RSRC_STAT_CI_REC1;
        }
        else
        {
            hw->ci[pCfg->ts].use |= RSRC_STAT_CI_REC2;
        }

        hw->ci[pCfg->ts].servIdx = pCfg->servIdx;
        hw->ci[pCfg->ts].input   = pCfg->ciInput;
    }

    #endif /* APPL_CI_ENABLED */

    hw->opStat |= pCfg->op;

    return FAPI_OK;
}

FSTATIC int32_t RSRC_SetOccupationPlay(const RSRC_CFG_S * pCfg,
                                       RSRC_STAT_HW_S * hw)
{
    DBG_Assert(pCfg->ts < RSRC_TS_NUMS);
    DBG_Assert(pCfg->tsdIdx < RSRC_TSD_NUMS);

    #ifdef APPL_DEBUG_MODE

    /* resource management consistency checks */
    RSRC_CheckConsistencyTsInput(pCfg, hw);
    RSRC_CheckConsistencyCiUsage(pCfg, hw);

    #endif /* APPL_DEBUG_MODE */

    hw->tsd[pCfg->tsdIdx].use |= RSRC_STAT_TSD_PLAYBACK;
    hw->tsd[pCfg->tsdIdx].tsIn = pCfg->ts;
    hw->vd[0] = RSRC_STAT_VD_USED;
    hw->videoout = RSRC_STAT_VOUT_PLAYBACK;

    #ifdef APPL_CI_ENABLED

    if ( pCfg->useCi )
    {
        DBG_Assert(pCfg->ts >= RSRC_TS_A);

        hw->ci[pCfg->ts].use    |= RSRC_STAT_CI_PLAY;
        hw->ci[pCfg->ts].servIdx = pCfg->servIdx;
        hw->ci[pCfg->ts].input   = pCfg->ciInput;
    }

    #endif /* APPL_CI_ENABLED */

    hw->opStat                 |= pCfg->op;

    return FAPI_OK;
}

#ifdef APPL_CI_ENABLED

FSTATIC int32_t RSRC_SetOccupationDescrRec(const RSRC_CFG_S * pCfg,
                                           RSRC_STAT_HW_S * hw)
{
    DBG_Assert(pCfg->ts < RSRC_TS_NUMS);
    DBG_Assert(pCfg->tsdIdx < RSRC_TSD_NUMS);

    #ifdef APPL_DEBUG_MODE

    /* resource management consistency checks */
    RSRC_CheckConsistencyTsInput(pCfg, hw);
    RSRC_CheckConsistencyCiUsage(pCfg, hw);

    #endif /* APPL_DEBUG_MODE */

    hw->tsd[pCfg->tsdIdx].use  |= RSRC_STAT_TSD_RECORD;
    hw->tsd[pCfg->tsdIdx].tsIn  = pCfg->ts;

    if ( pCfg->useCi )
    {
        if ( pCfg->tsdIdx == RSRC_TSD_REC1 )
        {
            hw->ci[pCfg->ts].use |= RSRC_STAT_CI_REC1;
        }
        else
        {
            hw->ci[pCfg->ts].use |= RSRC_STAT_CI_REC2;
        }

        hw->ci[pCfg->ts].servIdx = pCfg->servIdx;
        hw->ci[pCfg->ts].input   = pCfg->ciInput;
    }

    hw->opStat |= pCfg->op;

    return FAPI_OK;
}

#endif /* APPL_CI_ENABLED */

#endif /* APPL_PVR_ENABLED */

#ifdef APPL_PIP_ENABLED

int32_t RSRC_ServiceAvlForPIP(SLIST_SERVIDX servIdx, RSRC_OP op,
                              RSRC_OP_PIP_S * pParams)
{
    SLIST_ITEM_S        servItem;
    SLIST_ITEM_S        compServItem;
    SLIST_SERVIDX       compServIdx;
    RSRC_AVL_LEV_E      feAvl[RSRC_FE_MAX];
    RSRC_CFG_S          targetCfg[RSRC_TS_NUMS];
    RSRC_AVL_LEV_E      tsAvl[RSRC_TS_NUMS];
    RSRC_TS_E           tsPref[RSRC_TS_NUMS];
    RSRC_TS_E           tsChosen = RSRC_TS_A;
    RSRC_TS_E           cnt;
    RSRC_CI_S           ciSlot[RSRC_TS_NUMS];
    uint8_t             tsdIdx = 0;
    int32_t             retVal = FAPI_OK;
    RSRC_STAT_HW_S      hw;

    if(servIdx < 0)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    /* check operation status: PIP currently active? */
    if((hw.opStat & RSRC_OP_PIP) != 0)
        return SMARTGO_SYS_ERR_RSRC_PIP;

    #ifdef APPL_UPDATE_ENABLED

    if((hw.opStat & RSRC_OP_UPDATE) != 0)
    {
        return SMARTGO_SYS_ERR_RSRC_UPDATE;
    }

    #endif /* APPL_UPDATE_ENABLED */

    for(cnt = (RSRC_TS_E)0; cnt < RSRC_TS_NUMS; cnt++)
    {
        memset(ciSlot + cnt, 0, sizeof(RSRC_CI_S));

        tsPref[cnt] = RSRC_TS_NONE;
    }

    /* --------------------------------------------------------------------- */

    retVal = SLIST_GetServByIndex(servIdx, &servItem);
    if(retVal != FAPI_OK)
        return retVal;

    /* --------------------------------------------------------------------- */

    /* video stream available ? */
    if(servItem.strPid.video == 0)
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    /* ----- get available frontends --------------------------------------- */

    /* frontend(s) for requested network available? */
    retVal = RSRC_GetFrontendAvailability(pRsrc->hwCfg, &servItem, feAvl, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    /* ----- check CI availability and usability --------------------------- */

    #ifdef APPL_CI_ENABLED

    for(cnt = (RSRC_TS_E)0; cnt < RSRC_TS_NUMS; cnt++)
    {
        ciSlot[cnt] = RSRC_CIUsableForService(cnt, servIdx, &servItem, &hw);
    }

    #endif /* APPL_CI_ENABLED */

    /* ----- check available TS inputs ------------------------------------- */

    for(cnt = (RSRC_TS_E)0; cnt < RSRC_TS_NUMS; cnt++)
    {
        targetCfg[cnt].op      = RSRC_OP_PIP;
        targetCfg[cnt].servIdx = servIdx;
        targetCfg[cnt].pvrIdx  = -1;                 //!< Unused.
        targetCfg[cnt].cas     = RSRC_CAS_NONE;
        targetCfg[cnt].ciMatch[RSRC_TS_A] = FFALSE;  //!< Not supported yet.
        targetCfg[cnt].ciMatch[RSRC_TS_B] = FFALSE;  //!< Not supported yet.

        /* NOTE: Descrambling of video stream for PIP has not been implemented
           yet. */

        tsAvl[cnt] = RSRC_TSAvailable(cnt, targetCfg + cnt, feAvl,
                                      ciSlot[cnt].usable, &hw);
    }

    /* check if any TS input is available */
    if( (tsAvl[RSRC_TS_A] == RSRC_AVL_LEV_NOT_AVL) &&
        (tsAvl[RSRC_TS_B] == RSRC_AVL_LEV_NOT_AVL))
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    /* --------------------------------------------------------------------- */

    /* Set TS preference order */
    RSRC_BuildTSPrefLive(pRsrc->hwCfg, tsAvl, targetCfg, ciSlot, tsPref);

    /* Search for available TSD in order of TS preference */

    for(cnt = (RSRC_TS_E)0; cnt < RSRC_TS_NUMS; cnt++)
    {
        if(tsPref[cnt] < 0)
            continue;

        tsChosen = tsPref[cnt];

        for(tsdIdx = 0; tsdIdx < RSRC_TSD_NUMS; tsdIdx++)
        {
            if ( hw.tsd[tsdIdx].use == 0 ) break; /* TSD unused? */

            #ifdef APPL_CI_ENABLED

            else if ( targetCfg[tsChosen].useCi )
            {
                /* If PIP needs CI, it requires TSD exclusively
                   (PAT/PMT filtering needed). */
                continue;
            }

            #endif /* APPL_CI_ENABLED */

            else if ( (hw.tsd[tsdIdx].use & (RSRC_STAT_TSD_PLAYBACK |
                                             RSRC_STAT_TSD_SCAN     |
                                             RSRC_STAT_TSD_PIP)) != 0 )
            {
                continue;
            }

            /* TSD is in use: test for TS input configuration conflict */
            if ( hw.tsd[tsdIdx].tsIn != tsChosen )
            {
                continue;
            }

            /* Test for PID filtering conflict with live and recording
               operations. */
            compServIdx = -1;

            if ( (hw.tsd[tsdIdx].use & RSRC_STAT_TSD_LIVE) != 0 )
            {
                compServIdx = pRsrc->status.live.cfg.servIdx;
            }

            #ifdef APPL_PVR_ENABLED

            else if ( (hw.tsd[tsdIdx].use & RSRC_STAT_TSD_RECORD) != 0 )
            {
                if ( tsdIdx == RSRC_TSD_REC1 )
                {
                    compServIdx = pRsrc->status.rec[RSRC_REC_1].cfg.servIdx;
                }
                else
                {
                    compServIdx = pRsrc->status.rec[RSRC_REC_2].cfg.servIdx;
                }
            }

            #endif /* APPL_PVR_ENABLED */

            if ( compServIdx >= 0 )
            {
                retVal = SLIST_GetServByIndex (compServIdx, &compServItem);
                if ( retVal != FAPI_OK ) continue;

                if ( RSRC_PIPConflictCompare (servItem.strPid.video,
                                              &(compServItem.strPid)) !=
                                              FAPI_OK )
                {
                    continue;
                }
            }

            #ifdef APPL_MPEG_ENCODER_ENABLED

            /* Test for PID filtering conflict with MPEG encoder input */
            if ( (hw.tsd[tsdIdx].use & RSRC_STAT_TSD_MPENCMONI) != 0 )
            {
                if ( MPENC_GetStreamParams (&(compServItem.strPid)) == FAPI_OK )
                {
                    if ( RSRC_PIPConflictCompare (servItem.strPid.video,
                                                  &(compServItem.strPid)) !=
                                                  FAPI_OK )
                    {
                        continue;
                    }
                }
            }

            #endif /* APPL_MPEG_ENCODER_ENABLED */

            break; /* all tests passed */
        }

        if ( tsdIdx < RSRC_TSD_NUMS )
            break;
    }

    if(cnt >= RSRC_TS_NUMS)
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    targetCfg[tsChosen].tsdIdx = tsdIdx;

    if(pParams != NULL)
    {
        pParams->sourceIsRec = FFALSE;
        pParams->cfg         = targetCfg[tsChosen];
        pParams->recIdx      = -1;
    }

    return FAPI_OK;
}

#ifdef APPL_PVR_ENABLED

int32_t RSRC_RecordAvlForPIP(int8_t recIdx, RSRC_OP op, RSRC_OP_PIP_S * pParams)
{
    RSRC_STAT_HW_S      hw;
    int32_t             retVal;
    RSRC_OP             stat;

    if((recIdx < 0) || (recIdx >= RSRC_REC_MAX))
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    /* check operation status: PIP currently active? */
    if((hw.opStat & RSRC_OP_PIP) != 0)
        return SMARTGO_SYS_ERR_RSRC_PIP;

    #ifdef APPL_UPDATE_ENABLED

    if((hw.opStat & RSRC_OP_UPDATE) != 0)
    {
        return SMARTGO_SYS_ERR_RSRC_UPDATE;
    }

    #endif /* APPL_UPDATE_ENABLED */

    /* test if source record for PIP is active */
    if(recIdx == 0)
        stat = RSRC_OP_REC1;
    else
        stat = RSRC_OP_REC2;

    if((hw.opStat & stat) == 0)
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    if(pParams != NULL)
    {
        pParams->sourceIsRec = FTRUE;
        pParams->recIdx      = recIdx;
    }

    return FAPI_OK;
}

#endif /* APPL_PVR_ENABLED */

int32_t RSRC_SetPIP(const RSRC_OP_PIP_S * pParams)
{
    int32_t             retVal = FAPI_OK;

    RSRC_DebugPrintConfig("Set PIP", &(pParams->cfg));

    retVal = RSRC_SetOccupationPip(pParams, &(pRsrc->status.hw));
    if(retVal != FAPI_OK)
        return retVal;

    pRsrc->status.opStatus |= pParams->cfg.op;
    pRsrc->status.pip       = *pParams;

    return retVal;
}

int32_t RSRC_PIPStopped(void)
{
    RSRC_CFG_S         *pCfg;

    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] PIP stopped.\n");

    if((pRsrc->status.opStatus & RSRC_OP_PIP) == 0)
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    if(pRsrc->status.pip.sourceIsRec)
    {
        /* do nothing */
    }
    else
    {
        pCfg = &(pRsrc->status.pip.cfg);

        pRsrc->status.hw.fe[pCfg->feIdx].use &= (~RSRC_STAT_FE_PIP);
        pRsrc->status.hw.tsd[pCfg->tsdIdx].use &= (~RSRC_STAT_TSD_PIP);

        #ifdef APPL_CI_ENABLED

        /* release CI occupation */
        pRsrc->status.hw.ci[pCfg->ts].use &= (~RSRC_STAT_CI_PIP);

        #endif /* APPL_CI_ENABLED */
    }

    pRsrc->status.hw.vd[1] = RSRC_STAT_VD_UNUSED;

    pRsrc->status.opStatus &= (~RSRC_OP_PIP);
    pRsrc->status.hw.opStat &= (~RSRC_OP_PIP);

    return FAPI_OK;
}

FSTATIC int32_t RSRC_SetOccupationPip(const RSRC_OP_PIP_S * pParams,
                                      RSRC_STAT_HW_S * hw)
{
    int32_t         retVal;
    SLIST_ITEM_S    servItem;

    if(pParams->sourceIsRec)
    {
        /* no additional hardware resources occupied */
    }
    else
    {
        DBG_Assert(pParams->cfg.feIdx < pRsrc->hwCfg->feNums);
        DBG_Assert(pParams->cfg.ts < RSRC_TS_NUMS);
        DBG_Assert(pParams->cfg.tsdIdx < RSRC_TSD_NUMS);

        retVal = SLIST_GetServByIndex(pParams->cfg.servIdx, &servItem);
        if(retVal != FAPI_OK)
            return retVal;

        #ifdef APPL_DEBUG_MODE

        /* resource management consistency checks */
        RSRC_CheckConsistencyTsInput (&(pParams->cfg), hw);
        RSRC_CheckConsistencyFeTuning(&(pParams->cfg), &servItem, hw);
        RSRC_CheckConsistencyCiUsage (&(pParams->cfg), hw);

        #endif /* APPL_DEBUG_MODE */

        hw->fe[pParams->cfg.feIdx].use     |= RSRC_STAT_FE_PIP;
        hw->fe[pParams->cfg.feIdx].satIdx   = servItem.network.satIdx;
        hw->fe[pParams->cfg.feIdx].tpIdx    = servItem.tpIdx;
        hw->tsd[pParams->cfg.tsdIdx].use   |= RSRC_STAT_TSD_PIP;
        hw->tsd[pParams->cfg.tsdIdx].tsIn   = pParams->cfg.ts;

        #ifdef APPL_CI_ENABLED

        if ( pParams->cfg.useCi )
        {
            hw->ci[pParams->cfg.ts].use     |= RSRC_STAT_CI_PIP;
            hw->ci[pParams->cfg.ts].servIdx  = pParams->cfg.servIdx;
            hw->ci[pParams->cfg.ts].input    = pParams->cfg.ciInput;
        }

        #endif /* APPL_CI_ENABLED */
    }

    hw->vd[1]   = RSRC_STAT_VD_USED;

    hw->opStat |= pParams->cfg.op;

    return FAPI_OK;
}

FSTATIC int32_t RSRC_TestForPIPConflict (const SYS_STREAM_INFO_S* pPid,
                                         uint8_t tsdIdx,
                                         const RSRC_STAT_HW_S * hw)
{
    int32_t         retVal;
    RSRC_CFG_S*     pCfg = &(pRsrc->status.pip.cfg);
    SLIST_ITEM_S    pipService;

    if ( ((hw->opStat & RSRC_OP_PIP) == 0) ||
         (pCfg->tsdIdx != tsdIdx) )
    {
        return FAPI_OK;
    }
    else
    {
        /* PIP is active on requested TSD. If PIP uses CI it occupies PAT/PMT
           filtering resources and thus needs TSD exclusively. */

        if ( pCfg->useCi )
        {
            retVal = SMARTGO_SYS_ERR_RSRC_PIP;
        }
        else
        {
            /* Test for TSD filtering conflict: SmartMPEG TSD does not allow to
               filter a single PID multiple times. */

            retVal = SLIST_GetServByIndex (pCfg->servIdx, &pipService);
            if ( retVal == FAPI_OK )
            {
                retVal = RSRC_PIPConflictCompare (pipService.strPid.video,
                                                  pPid);
            }
        }
    }

    return retVal;
}

FSTATIC FERR RSRC_PIPConflictCompare (uint16_t pipVideoPid,
                                      const SYS_STREAM_INFO_S* compStream)
{
    int32_t retVal = FAPI_OK;
    uint8_t cnt;

    if ( (pipVideoPid == compStream->pcr)   ||
         (pipVideoPid == compStream->video) ||
         (pipVideoPid == compStream->ttx) )
    {
        retVal = SMARTGO_SYS_ERR_RSRC_PIP;
    }
    else
    {
        for ( cnt = 0; cnt < compStream->audio.numStreams; cnt++ )
        {
            if ( pipVideoPid == compStream->audio.stream[cnt].pid )
            {
                retVal = SMARTGO_SYS_ERR_RSRC_PIP;
                break;
            }
        }
    }

    return retVal;
}

#endif /* APPL_PIP_ENABLED */

FSTATIC int32_t RSRC_SetOccupationLive(const RSRC_CFG_S * pCfg,
                                       RSRC_STAT_HW_S * hw)
{
    int32_t             retVal;
    SLIST_ITEM_S        servItem;

    DBG_Assert(pCfg->feIdx  < pRsrc->hwCfg->feNums);
    DBG_Assert(pCfg->ts     < RSRC_TS_NUMS);
    DBG_Assert(pCfg->tsdIdx < RSRC_TSD_NUMS);

    retVal = SLIST_GetServByIndex (pCfg->servIdx, &servItem);
    if(retVal != FAPI_OK)
        return retVal;

    #ifdef APPL_DEBUG_MODE

    /* resource management consistency checks */
    RSRC_CheckConsistencyTsInput (pCfg, hw);
    RSRC_CheckConsistencyFeTuning(pCfg, &servItem, hw);
    RSRC_CheckConsistencyCiUsage (pCfg, hw);

    #endif /* APPL_DEBUG_MODE */

    hw->fe[pCfg->feIdx].use         |= RSRC_STAT_FE_LIVE;
    hw->fe[pCfg->feIdx].satIdx       = servItem.network.satIdx;
    hw->fe[pCfg->feIdx].tpIdx        = servItem.tpIdx;
    hw->tsd[pCfg->tsdIdx].use       |= RSRC_STAT_TSD_LIVE;
    hw->tsd[pCfg->tsdIdx].tsIn       = pCfg->ts;
    hw->vd[0]                        = RSRC_STAT_VD_USED;
    hw->videoout                     = RSRC_STAT_VOUT_LIVE;

    #ifdef APPL_CI_ENABLED

    if ( pCfg->useCi )
    {
        hw->ci[pCfg->ts].use        |= RSRC_STAT_CI_LIVE;
        hw->ci[pCfg->ts].servIdx     = pCfg->servIdx;
        hw->ci[pCfg->ts].input       = pCfg->ciInput;
    }

    #endif /* APPL_CI_ENABLED */

    hw->opStat                      |= pCfg->op;

    return FAPI_OK;
}

FSTATIC int32_t RSRC_SetOccupationScan(const RSRC_CFG_S * pCfg,
                                       RSRC_STAT_HW_S * hw)
{
    DBG_Assert(pCfg->feIdx  < pRsrc->hwCfg->feNums);
    DBG_Assert(pCfg->tsdIdx < RSRC_TSD_NUMS);

    #ifdef APPL_DEBUG_MODE

    /* resource management consistency checks */
    RSRC_CheckConsistencyTsInput (pCfg, hw);

    #endif /* APPL_DEBUG_MODE */

    hw->fe[pCfg->feIdx].use    |= RSRC_STAT_FE_SCAN;
    hw->tsd[pCfg->tsdIdx].use  |= RSRC_STAT_TSD_SCAN;
    hw->tsd[pCfg->tsdIdx].tsIn  = pCfg->ts;

    hw->opStat                 |= pCfg->op;
    return FAPI_OK;
}

FSTATIC int32_t RSRC_SetOccupationTune(const RSRC_CFG_S * pCfg,
                                       RSRC_STAT_HW_S * hw)
{
    DBG_Assert(pCfg->feIdx < pRsrc->hwCfg->feNums);

    #ifdef APPL_DEBUG_MODE

    /* resource management consistency checks */

    /* currently empty */

    #endif /* APPL_DEBUG_MODE */

    hw->fe[pCfg->feIdx].use |= RSRC_STAT_FE_TUNEMODE;
    hw->opStat              |= pCfg->op;

    return FAPI_OK;
}

/*
Returns:
0:     Requested network cannot be accessed via given frontend.
1...n: Level of preference

motorNeeded: returns 1 if it is necessary to modify the current motor
             position.
*/
FSTATIC RSRC_AVL_LEV_E RSRC_FrontendAvailableForNetwork(int8_t feIdx,
                                                 const SYS_NETWORK_S * network,
                                                 fbool_t* motorNeeded,
                                                 const RSRC_STAT_HW_S * hw)
{
    RSRC_AVL_LEV_E      prefLvl = RSRC_AVL_LEV_1;

    #ifdef APPL_FESAT_ENABLED

    CFGSAT_FEPARS_S     feSetup;
    CFGSAT_SAT_S        satInf;
    int32_t             retVal;

    #endif /* APPL_FESAT_ENABLED */

    if(motorNeeded != NULL)
        *motorNeeded = FFALSE;

    #ifdef APPL_MPEG_ENCODER_ENABLED

    /* Test if frontend access is locked by MPEG encoder usage. */
    if ( (hw->mpenc.use & RSRC_STAT_MPENC_ANY) != 0 )
    {
        switch ( pRsrc->hwCfg->mpencPos )
        {
            case RSRC_MPENCPOS_FE1:
                if ( feIdx == 0 ) prefLvl = RSRC_AVL_LEV_NOT_AVL;
                break;

            case RSRC_MPENCPOS_FE2:
                if ( feIdx == 1 ) prefLvl = RSRC_AVL_LEV_NOT_AVL;
                break;

            default:
            case RSRC_MPENCPOS_NONE:
                /* do nothing */
                break;
        }

        if ( prefLvl == RSRC_AVL_LEV_NOT_AVL ) return prefLvl;
    }

    #endif /* APPL_MPEG_ENCODER_ENABLED */

    if((hw->fe[feIdx].use & RSRC_STAT_FE_OPENED) == 0)
    {
        return RSRC_AVL_LEV_NOT_AVL;
    }
    else if(hw->fe[feIdx].type != network->type)
    {
        return RSRC_AVL_LEV_NOT_AVL;
    }

    #ifdef APPL_FESAT_ENABLED

    if(network->type == FE_SYSTEM_TYPE_SATELLITE)
    {
        retVal = CFGSAT_GetFrontendSetup(&feSetup);
        if(retVal != FAPI_OK)
            return RSRC_AVL_LEV_NOT_AVL;

        retVal = CFGSAT_GetDataByIndex(network->satIdx, &satInf);
        if(retVal != FAPI_OK)
            return RSRC_AVL_LEV_NOT_AVL;

        /* test if a frontend configuration for this satellite exists */
        if(!(satInf.lnbCfg[feIdx].active))
            return RSRC_AVL_LEV_NOT_AVL;

        /* test if satellite is addressable by this frontend, set preference
           level according to frontend system: Least flexible system is most
           preferred! */
        switch (feSetup.controlMode[feIdx])
        {
            default:
            case CFGSAT_FEMODE_CLASSIC:
                prefLvl = RSRC_AVL_LEV_10;
                break;

            #ifdef APPL_SATPOSBY22KHZ_ENABLED
            case CFGSAT_FEMODE_POSBY22KHZ:
            #endif
            case CFGSAT_FEMODE_TONEBURST:
                prefLvl = RSRC_AVL_LEV_9;
                break;

            case CFGSAT_FEMODE_DISEQC10:
            case CFGSAT_FEMODE_DISEQC11_DTH:
            #ifdef APPL_DISEQC11_SMATV_ENABLED
            case CFGSAT_FEMODE_DISEQC11_SMATV:
            #endif
                prefLvl = RSRC_AVL_LEV_8;
                break;

            #ifdef APPL_DISEQC12_ENABLED

            case CFGSAT_FEMODE_DISEQC12:
                prefLvl = RSRC_AVL_LEV_7;

                if((motorNeeded != NULL) &&
                   (network->satIdx != hw->fe[feIdx].satIdx))
                {
                    /* Indicate that a turning of the motor is necessary */
                    *motorNeeded = FTRUE;
                }
                break;

            #endif /* APPL_DISEQC12_ENABLED */

            #ifdef APPL_USALS_ENABLED

            case CFGSAT_FEMODE_USALS:
                prefLvl = RSRC_AVL_LEV_6;

                if((motorNeeded != NULL) &&
                   (network->satIdx != hw->fe[feIdx].satIdx))
                {
                    /* Indicate that a turning of the motor is necessary */
                    *motorNeeded = FTRUE;
                }
                break;

            #endif /* APPL_USALS_ENABLED */
        }
    }

    #endif /* APPL_FESAT_ENABLED */

    return prefLvl;
}

#ifdef APPL_CI_ENABLED

FSTATIC RSRC_CI_S RSRC_CIUsableForService (RSRC_TS_E ts, SLIST_SERVIDX servIdx,
                                           const SLIST_ITEM_S* pServItem,
                                           const RSRC_STAT_HW_S* hw)
{
    CISYS_SLOTSTATUS_E  slotStatus;
    SLIST_CA_INFO_S     caInfo;
    RSRC_CI_S           res;

    res.camAvl   = FFALSE;
    res.casMatch = FFALSE;
    res.usable   = FFALSE;

    if ( pRsrc->hwCfg->ciAvl[ts] )
    {
        slotStatus = CISYS_GetSlotStatus (ts);

        if ( slotStatus == CISYS_SLOT_MODULE_READY )
        {
            res.camAvl = FTRUE;

            if(SLIST_CaInfoGet(servIdx, &caInfo) == FAPI_OK)
            {
                /* test CI module for availability of required CA system */
                res.casMatch = CISYS_ModuleUseable (ts, &caInfo);

                if ( res.casMatch )
                {
                    /* If CI is already occupied, CI service must
                       match requested service. */
                    if ( ((hw->ci[ts].use & RSRC_STAT_CI_ANY) == 0) ||
                          (hw->ci[ts].servIdx                 == servIdx) )
                    {
                        res.usable = FTRUE;
                    }
                }
            }
        }
    }

    return res;
}

#endif /* APPL_CI_ENABLED */

/*lint -save -e715*/
/* Symbol 'tryCi' an'hw' only referenced if APPL_CI_ENABLED */
FSTATIC RSRC_AVL_LEV_E RSRC_TSAvailable(RSRC_TS_E ts, RSRC_CFG_S * cfg,
                                 const RSRC_AVL_LEV_E* feAvl, fbool_t tryCi,
                                 const RSRC_STAT_HW_S * hw)
{
    fbool_t             setAvl = FTRUE;
    RSRC_AVL_LEV_E      tsAvl;

    #ifdef APPL_CI_ENABLED
    uint8_t             feciAvl[RSRC_FE_MAX]; /* Availability of FEs via CI. */
    #endif /* APPL_CI_ENABLED */

    cfg->ts      = ts;
    cfg->useCi   = FFALSE;
    cfg->ciInput = FAPEX_BOARD_RSRC_CIIN_NONE;

    #ifdef APPL_MPEG_ENCODER_ENABLED

    /* Check for conflicts with MPEG encoder. */
    if ( (hw->mpenc.use & RSRC_STAT_MPENC_ANY) != 0 )
    {
        switch ( pRsrc->hwCfg->mpencPos )
        {
            case RSRC_MPENCPOS_FE1:
                if ( ts == RSRC_TS_A ) setAvl = FFALSE;
                break;

            case RSRC_MPENCPOS_FE2:
                if ( ts == RSRC_TS_B ) setAvl = FFALSE;
                break;

            default:
            case RSRC_MPENCPOS_NONE:
                /* do nothing */
                break;
        }

        if ( !setAvl ) return RSRC_AVL_LEV_NOT_AVL;
    }

    #endif /* APPL_MPEG_ENCODER_ENABLED */

    /* ----- CI requested -------------------------------------------------- */

    #ifdef APPL_CI_ENABLED

    /* NOTE: (tryCi == FTRUE) implies that CAM is usable for requested
       operation! */

    if(tryCi)
    {
        feciAvl[0] = feAvl[0];
        feciAvl[1] = feAvl[1];

        if((pRsrc->hwCfg->ciInput[ts] & FAPI_BOARD_RSRC_CIIN_FE1) == 0)
            feciAvl[0] = 0;
        if((pRsrc->hwCfg->ciInput[ts] & FAPI_BOARD_RSRC_CIIN_FE2) == 0)
            feciAvl[1] = 0;

        /* choose frontend by CI input availability and preference level */
        if((feciAvl[0] > 0) && (feciAvl[1] > 0))
        {
            /* Both frontend available as inputs for CI. Choose by
               preference level. In case of equal preference levels, avoid
               cross-signalling (e.g. TS1-FE2 or TS2-FE1). */

            if(feciAvl[0] == feciAvl[1])
            {
                if(ts == RSRC_TS_A)
                    cfg->ciInput = FAPI_BOARD_RSRC_CIIN_FE1;
                else
                    cfg->ciInput = FAPI_BOARD_RSRC_CIIN_FE2;
            }
            else if(feciAvl[0] > feciAvl[1])
            {
                cfg->ciInput = FAPI_BOARD_RSRC_CIIN_FE1;
            }
            else
            {
                cfg->ciInput = FAPI_BOARD_RSRC_CIIN_FE2;
            }
        }
        else if ( feciAvl[0] > 0 )
        {
            cfg->ciInput = FAPI_BOARD_RSRC_CIIN_FE1;
        }
        else if ( feciAvl[1] > 0 )
        {
            cfg->ciInput = FAPI_BOARD_RSRC_CIIN_FE2;
        }

        /* Any frontend found? */
        if ( cfg->ciInput != FAPI_BOARD_RSRC_CIIN_NONE )
        {
            if(cfg->ciInput == FAPI_BOARD_RSRC_CIIN_FE1)
                cfg->feIdx = 0;
            else
                cfg->feIdx = 1;

            cfg->useCi = FTRUE;
        }
    }

    /* If CI path is not used, ensure that CAM is not used by any other
       operation. This would block the TS input path. */
    if ( !(cfg->useCi) )
    {
        if((pRsrc->hwCfg->ciAvl[ts]) &&
           ((hw->ci[ts].use & RSRC_STAT_CI_ANY) != 0))
        {
            setAvl = FFALSE;
        }
    }

    #endif /* APPL_CI_ENABLED */

    /* ----- Non-CI handling ----------------------------------------------- */

    if ( !(cfg->useCi) )
    {
        switch (pRsrc->hwCfg->tsInput[ts])
        {
            default:
            case FAPEX_BOARD_RSRC_TSIN_NONE:
                setAvl = FFALSE;
                break;

            case FAPEX_BOARD_RSRC_TSIN_FE1:
                cfg->feIdx = 0;
                break;

            case FAPEX_BOARD_RSRC_TSIN_FE2:
                cfg->feIdx = 1;
                break;
        }
    }

    if ( setAvl && (cfg->feIdx < pRsrc->hwCfg->feNums) )
    {
        tsAvl = feAvl[cfg->feIdx];
    }
    else
    {
        tsAvl = RSRC_AVL_LEV_NOT_AVL;
    }

    return tsAvl;
}
/*lint -restore */

FSTATIC int32_t RSRC_GetFrontendAvailability (const SYS_HWCONFIG_S* hwCfg,
                                              const SLIST_ITEM_S* servItem,
                                              RSRC_AVL_LEV_E* feAvl,
                                              const RSRC_STAT_HW_S* hw)
{
    int8_t              cnt;
    fbool_t             motorNeeded = FFALSE;

    #ifdef APPL_FESAT_ENABLED

    CFGSAT_FEPARS_S     feSetup;
    int8_t              refFeIdx;
    int32_t             retVal;

    retVal = CFGSAT_GetFrontendSetup(&feSetup);
    if(retVal != FAPI_OK)
        return retVal;

    #endif /* APPL_FESAT_ENABLED */

    for(cnt = 0; cnt < RSRC_FE_MAX; cnt++)
    {
        if ( cnt >= hwCfg->feNums )
        {
            feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
        }
        else
        {
            feAvl[cnt] = RSRC_FrontendAvailableForNetwork(cnt,
                                                          &(servItem->network),
                                                          &motorNeeded, hw);
        }

        /* test if service is available for target frontend */
        if(feAvl[cnt] > RSRC_AVL_LEV_NOT_AVL)
        {
            #ifdef APPL_FESAT_ENABLED

            if(hw->fe[cnt].type == FE_SYSTEM_TYPE_SATELLITE)
            {
                switch (feSetup.installType)
                {
                    case CFGSAT_INSTALLTYPE_LOOPTHROUGH:
                        /* operation active on this tuner? */
                        if(hw->fe[cnt].use & RSRC_STAT_FE_ANY)
                        {
                            if(!(RSRC_ServiceAvlOnCurrTs(servItem, cnt, hw)))
                            {
                                feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                            }
                        }

                        /* operation active on other tuner? */
                        else if((hwCfg->feNums > 1) &&
                                (((cnt == 0)
                                  && (hw->fe[1].use & RSRC_STAT_FE_ANY))
                                 || ((cnt == 1)
                                     && (hw->fe[0].use & RSRC_STAT_FE_ANY))))
                        {
                            if(cnt == 0)
                                refFeIdx = 1;
                            else
                                refFeIdx = 0;

                            if(!(RSRC_ServiceAvlOnCurrBand(servItem,
                                                           refFeIdx, hw)))
                            {
                                feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                            }
                        }
                        else
                        {
                        }
                        break;

                    case CFGSAT_INSTALLTYPE_SAMESAT:
                        /* operation active on this tuner? */
                        if((hw->fe[cnt].use & RSRC_STAT_FE_ANY))
                        {
                            if(!(RSRC_ServiceAvlOnCurrTs(servItem, cnt, hw)))
                            {
                                feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                            }
                        }

                        /* operation active on other tuner? */
                        else if((hwCfg->feNums > 1) &&
                                (((cnt == 0)
                                  && (hw->fe[1].use & RSRC_STAT_FE_ANY))
                                 || ((cnt == 1)
                                     && (hw->fe[0].use & RSRC_STAT_FE_ANY))))
                        {
                            if(motorNeeded)
                                feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                        }
                        break;

                    default:
                    case CFGSAT_INSTALLTYPE_INDEPENDENT:
                        if(hw->fe[cnt].use & RSRC_STAT_FE_ANY)
                        {
                            if(!(RSRC_ServiceAvlOnCurrTs(servItem, cnt, hw)))
                            {
                                feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                            }
                        }
                        break;
                }
            }
            else
            {

            #endif /* APPL_FESAT_ENABLED */

                if(hw->fe[cnt].use & RSRC_STAT_FE_ANY)
                {
                    /* Frontend is occupied by an operation: only services from
                       tuned TS may be selected. */
                    if(!(RSRC_ServiceAvlOnCurrTs(servItem, cnt, hw)))
                    {
                        feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                    }
                }

            #ifdef APPL_FESAT_ENABLED
            }
            #endif
        }
    }

    return FAPI_OK;
}

/*
feIdx: -1: test all frontends, other: specific frontend to be tested

*/
FSTATIC int32_t RSRC_GetExclFeAvailability(int8_t feIdx,
                                           const SYS_HWCONFIG_S* hwCfg,
                                           const SYS_NETWORK_S* network,
                                           RSRC_AVL_LEV_E* feAvl,
                                           const RSRC_STAT_HW_S* hw)
{
    int8_t              cnt;
    fbool_t             motorNeeded = FFALSE;

    #ifdef APPL_FESAT_ENABLED

    CFGSAT_FEPARS_S     feSetup;
    int32_t             retVal;

    retVal = CFGSAT_GetFrontendSetup(&feSetup);
    if(retVal != FAPI_OK)
        return retVal;

    #endif /* APPL_FESAT_ENABLED */

    /* check for frontends available to access requested network */
    for(cnt = 0; cnt < RSRC_FE_MAX; cnt++)
    {
        if ( cnt >= hwCfg->feNums )
        {
            feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
        }
        else if((feIdx < 0) || (cnt == feIdx))
        {
            feAvl[cnt] = RSRC_FrontendAvailableForNetwork(cnt, network,
                                                          &motorNeeded, hw);
        }
        else
        {
            feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
        }

        /* Frontend must not be occupied by any operation */
        if(feAvl[cnt] > RSRC_AVL_LEV_NOT_AVL)
        {
            #ifdef APPL_FESAT_ENABLED

            if(hw->fe[cnt].type == FE_SYSTEM_TYPE_SATELLITE)
            {
                switch (feSetup.installType)
                {
                    case CFGSAT_INSTALLTYPE_LOOPTHROUGH:

                        /* this tuner occupied ? */
                        if(hw->fe[cnt].use & RSRC_STAT_FE_ANY)
                        {
                            feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                        }

                        /* other tuner occupied ? */
                        else if((hwCfg->feNums > 1) &&
                                (((cnt == 0)
                                  && (hw->fe[1].use & RSRC_STAT_FE_ANY))
                                 || ((cnt == 1)
                                     && (hw->fe[0].use & RSRC_STAT_FE_ANY))))
                        {
                            feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                        }
                        break;

                    case CFGSAT_INSTALLTYPE_SAMESAT:
                        /* this tuner occupied ? */
                        if((hw->fe[cnt].use & RSRC_STAT_FE_ANY))
                        {
                            feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                        }

                        /* other tuner occupied ? */
                        else if((hwCfg->feNums > 1) &&
                                (((cnt == 0)
                                  && (hw->fe[1].use & RSRC_STAT_FE_ANY))
                                 || ((cnt == 1)
                                     && (hw->fe[0].use & RSRC_STAT_FE_ANY))))
                        {
                            if(motorNeeded)
                                feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                        }
                        break;

                    default:
                    case CFGSAT_INSTALLTYPE_INDEPENDENT:
                        if(hw->fe[cnt].use & RSRC_STAT_FE_ANY)
                        {
                            feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                        }
                        break;
                }
            }
            else
            {

            #endif

                if(hw->fe[cnt].use & RSRC_STAT_FE_ANY)
                {
                    feAvl[cnt] = RSRC_AVL_LEV_NOT_AVL;
                }

            #ifdef APPL_FESAT_ENABLED
            }
            #endif
        }
    }

    return FAPI_OK;
}

FSTATIC fbool_t RSRC_ServiceAvlOnCurrTs(const SLIST_ITEM_S * servItem,
                                        int8_t feIdx, const RSRC_STAT_HW_S * hw)
{
    if ( (servItem->network.satIdx != hw->fe[feIdx].satIdx) ||
         (servItem->tpIdx          != hw->fe[feIdx].tpIdx) )
    {
        return FFALSE;
    }
    else return FTRUE;
}

#ifdef APPL_FESAT_ENABLED

FSTATIC fbool_t RSRC_ServiceAvlOnCurrBand(const SLIST_ITEM_S * servItem,
                                          int8_t refFeIdx,
                                          const RSRC_STAT_HW_S * hw)
{
    TPLIST_ITEM_S       currCarrier;
    TPLIST_ITEM_S       newCarrier;
    CFGSAT_SAT_S        satInf;
    int32_t             freqDiff;
    fbool_t             currHiBand;
    fbool_t             newHiBand;

    /* satellite match is mandatory */
    if ( servItem->network.satIdx != hw->fe[refFeIdx].satIdx )
    {
        return FFALSE;
    }

    /* carriers match -> no further tests necessary */
    if ( servItem->tpIdx == hw->fe[refFeIdx].tpIdx ) return FTRUE;

    /* ----- get parameters of both carriers ------------------------------- */

    if(TPLIST_GetDataByIndex(&(servItem->network), servItem->tpIdx,
                             &newCarrier) != FAPI_OK)
    {
        return FFALSE;
    }

    if(TPLIST_GetDataByIndex(&(servItem->network),
                             hw->fe[refFeIdx].tpIdx, &currCarrier) != FAPI_OK)
    {
        return FFALSE;
    }

    /* ----- test for polarity --------------------------------------------- */

    if(currCarrier.pars.sat.pol != newCarrier.pars.sat.pol)
    {
        return FFALSE;
    }

    /* ----- test for Hi/Low band ------------------------------------------ */

    if(CFGSAT_GetDataByIndex(servItem->network.satIdx, &satInf) != FAPI_OK)
    {
        return FFALSE;
    }

    /* test for single-oscillator LNB */
    if(satInf.lnbCfg[refFeIdx].freqLo == satInf.lnbCfg[refFeIdx].freqHi)
    {
        return FTRUE;
    }

    freqDiff = (int32_t)(satInf.lnbCfg[refFeIdx].freqLo) -
               (int32_t)(currCarrier.pars.sat.frequency);

    currHiBand = (fbool_t)((freqDiff > RSRC_BAND_SW) ||
                           (freqDiff < (-RSRC_BAND_SW)));

    freqDiff = (int32_t)(satInf.lnbCfg[refFeIdx].freqLo) -
               (int32_t)(newCarrier.pars.sat.frequency);

    newHiBand = (fbool_t)((freqDiff > RSRC_BAND_SW) ||
                          (freqDiff < (-RSRC_BAND_SW)));

    if ( currHiBand == newHiBand ) return FTRUE;
    else                           return FFALSE;
}

#endif

FSTATIC void RSRC_BuildTSPrefLive(const SYS_HWCONFIG_S* hwCfg,
                                  const RSRC_AVL_LEV_E* tsAvl,
                                  const RSRC_CFG_S* targetCfg,
                                  const RSRC_CI_S* ciSlot,
                                  RSRC_TS_E * tsPref)
{
    RSRC_TS_E   ciPos;
    RSRC_TS_E   nonCiPos;

    /* NOTE: TS input preference is chosen as follows:

       a) no CI slot available: no TS input preference

       b) single CI slot available: if usable, preference is on CI path, else
       preference is on non-CI path

       c) two CI slots available:
       c1) both usable: no preference
       c2) 1 usable: preference is usable CI
       c3) none usable: no preference

     */

    tsPref[0] = RSRC_TS_NONE;
    tsPref[1] = RSRC_TS_NONE;

    if( (tsAvl[RSRC_TS_A] == RSRC_AVL_LEV_NOT_AVL) &&
        (tsAvl[RSRC_TS_B] == RSRC_AVL_LEV_NOT_AVL))
    {
        /* do nothing */
    }
    else if( (tsAvl[RSRC_TS_A] > RSRC_AVL_LEV_NOT_AVL) &&
             (tsAvl[RSRC_TS_B] == RSRC_AVL_LEV_NOT_AVL))
    {
        tsPref[0] = RSRC_TS_A;
    }
    else if((tsAvl[RSRC_TS_B] > RSRC_AVL_LEV_NOT_AVL) &&
            (tsAvl[RSRC_TS_A] == RSRC_AVL_LEV_NOT_AVL))
    {
        tsPref[0] = RSRC_TS_B;
    }
    else /* both TS inputs available */
    {
        /* ----------------------------------------------------------------- */

        /* no CI slot available -> no preference */
        if((!(hwCfg->ciAvl[RSRC_TS_A])) && (!(hwCfg->ciAvl[RSRC_TS_B])))
        {
            if(tsAvl[RSRC_TS_A] >= tsAvl[RSRC_TS_B])
                tsPref[0] = RSRC_TS_A;
            else
                tsPref[0] = RSRC_TS_B;
        }

        /* ----------------------------------------------------------------- */

        /* single CI slot available */
        else if((hwCfg->ciAvl[RSRC_TS_A] && (!(hwCfg->ciAvl[RSRC_TS_B]))) ||
                (hwCfg->ciAvl[RSRC_TS_B] && (!(hwCfg->ciAvl[RSRC_TS_A]))))
        {
            if(hwCfg->ciAvl[RSRC_TS_A])
            {
                ciPos = RSRC_TS_A;
                nonCiPos = RSRC_TS_B;
            }
            else
            {
                ciPos = RSRC_TS_B;
                nonCiPos = RSRC_TS_A;
            }

            /* - CI usable -> Preference is CI path
               - CI not usable -> preference is non-CI path */
            if(targetCfg[ciPos].useCi)
                tsPref[0] = ciPos;
            else
                tsPref[0] = nonCiPos;
        }

        /* ------------------------------------------------------------------ */

        else                    /* two CI slots available */
        {
            if(targetCfg[RSRC_TS_A].useCi &&    /* both usable */
               targetCfg[RSRC_TS_B].useCi)
            {
                if(tsAvl[RSRC_TS_A] >= tsAvl[RSRC_TS_B])
                    tsPref[0] = RSRC_TS_A;
                else
                    tsPref[0] = RSRC_TS_B;
            }
            else if ( targetCfg[RSRC_TS_A].useCi )
            {
                tsPref[0] = RSRC_TS_A;
            }
            else if ( targetCfg[RSRC_TS_B].useCi )
            {
                tsPref[0] = RSRC_TS_B;
            }
            else /* none usable -> prefer empty slot */
            {
                if ( ciSlot[RSRC_TS_A].camAvl )
                {
                    tsPref[0] = RSRC_TS_B;
                }
                else if ( ciSlot[RSRC_TS_B].camAvl )
                {
                    tsPref[0] = RSRC_TS_A;
                }
                else /* none available or usable -> no preference */
                {
                    if(tsAvl[RSRC_TS_A] >= tsAvl[RSRC_TS_B])
                        tsPref[0] = RSRC_TS_A;
                    else
                        tsPref[0] = RSRC_TS_B;
                }
            }
        }

        if(tsPref[0] == RSRC_TS_A)
            tsPref[1] = RSRC_TS_B;
        else
            tsPref[1] = RSRC_TS_A;
    }
}

/* ------------------------------------------------------------------------- */

#ifdef APPL_UPDATE_ENABLED

int32_t RSRC_UpdateAvl(RSRC_OP op)
{
    RSRC_STAT_HW_S      hw;
    int32_t             retVal;

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if(retVal != FAPI_OK)
        return retVal;

    /* Any operation except tunemode is prohibited during update. */
    if((hw.opStat & (~(RSRC_OP_TUNEANY))) != 0)
    {
        if((hw.opStat & RSRC_OP_LIVE) != 0)
        {
            return SMARTGO_SYS_ERR_RSRC_LIVE;
        }

        #ifdef APPL_PVR_ENABLED

        else if((hw.opStat & RSRC_OP_RECANY) != 0)
        {
            return SMARTGO_SYS_ERR_RSRC_RECORD;
        }
        else if((hw.opStat & RSRC_OP_PLAY) != 0)
        {
            return SMARTGO_SYS_ERR_RSRC_PLAY;
        }

        #ifdef APPL_CI_ENABLED

        else if((hw.opStat & RSRC_OP_DESCRREC) != 0)
        {
            return SMARTGO_SYS_ERR_RSRC_RECORD;
        }

        #endif /* APPL_CI_ENABLED */

        #endif /* APPL_PVR_ENABLED */

        #ifdef APPL_PIP_ENABLED

        else if((hw.opStat & RSRC_OP_PIP) != 0)
        {
            return SMARTGO_SYS_ERR_RSRC_PIP;
        }

        #endif /* APPL_PIP_ENABLED */

        else if((hw.opStat & RSRC_OP_SCAN) != 0)
        {
            return SMARTGO_SYS_ERR_RSRC_SCAN;
        }
        else if((hw.opStat & RSRC_OP_UPDATE) != 0)
        {
            return SMARTGO_SYS_ERR_RSRC_UPDATE;
        }
        else
        {
            return SMARTGO_SYS_ERR_RSRC;
        }
    }

    return FAPI_OK;
}

int32_t RSRC_SetUpdate(void)
{
    int32_t             retVal = FAPI_OK;

    retVal = RSRC_SetOccupationUpdate(&(pRsrc->status.hw));
    if(retVal != FAPI_OK)
        return retVal;

    pRsrc->status.opStatus |= RSRC_OP_UPDATE;

    return retVal;
}

int32_t RSRC_UpdateStopped(void)
{
    if((pRsrc->status.opStatus & RSRC_OP_UPDATE) == 0)
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    pRsrc->status.opStatus  &= (~RSRC_OP_UPDATE);
    pRsrc->status.hw.opStat &= (~RSRC_OP_UPDATE);

    return FAPI_OK;
}

FSTATIC int32_t RSRC_SetOccupationUpdate(RSRC_STAT_HW_S * hw)
{
    hw->opStat |= RSRC_OP_UPDATE;

    return FAPI_OK;
}

#endif /* APPL_UPDATE_ENABLED */

#ifdef APPL_CA_ENABLED

FSTATIC RSRC_CAS_E RSRC_CASystemAvailable(const SLIST_CA_INFO_S * pCaInfo)
{
    uint8_t             cnt;

    for(cnt = 0; cnt < pCaInfo->numIds; cnt++)
    {
        #ifdef APPL_CONAX_ENABLED

        if(CONAX_CA_CaSystemIdMatch(pCaInfo->ca_system_id[cnt]))
        {
            return RSRC_CAS_CONAX;
        }

        #endif /* APPL_CONAX_ENABLED */
    }

    return RSRC_CAS_NONE;
}

#endif /* APPL_CA_ENABLED */

#ifdef APPL_MPEG_ENCODER_ENABLED

FERR RSRC_MpencMonitorAvailable (RSRC_OP op, RSRC_CFG_S* pCfg)
{
    FERR                retVal;
    RSRC_CFG_S          targetCfg;
    RSRC_STAT_HW_S      hw;

    #ifdef APPL_PIP_ENABLED

    SYS_STREAM_INFO_S   mpencStreams;

    #endif /* APPL_PIP_ENABLED */

    retVal = RSRC_AssembleHwOccupation(op, &hw);
    if ( retVal != FD_OK ) return retVal;

    /* check operation status */
    if ( (hw.opStat & RSRC_OP_MPENCMONITOR) != 0 )
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    #ifdef APPL_UPDATE_ENABLED

    if ( (hw.opStat & RSRC_OP_UPDATE) != 0 )
    {
        return SMARTGO_SYS_ERR_RSRC_UPDATE;
    }

    #endif /* APPL_UPDATE_ENABLED */

    /* --------------------------------------------------------------------- */

    /* Check if TSD is available. As PIP does not occupy TSD
       exclusively, it is checked separately. */

    if ( (hw.tsd[RSRC_TSD_MPENC].use & (~RSRC_STAT_TSD_PIP)) != 0 )
    {
        return SMARTGO_SYS_ERR_RSRC;
    }

    #ifdef APPL_PIP_ENABLED

    /* Check for PID filtering conflicts with PIP */

    retVal = MPENC_GetStreamParams (&mpencStreams);
    if ( retVal == FD_OK )
    {
        retVal = RSRC_TestForPIPConflict (&mpencStreams, RSRC_TSD_MPENC, &hw);
    }

    if ( retVal != FD_OK )
    {
        return retVal;
    }

    #endif /* APPL_PIP_ENABLED */

    /* --------------------------------------------------------------------- */

    targetCfg.op      = RSRC_OP_MPENCMONITOR;
    targetCfg.servIdx = SLIST_SERVIDX_MPENC;        //!< Static parameter!
    targetCfg.pvrIdx  = -1;                         //!< Not used.
    targetCfg.tsdIdx  = RSRC_TSD_MPENC;             //!< Static parameter!
    targetCfg.useCi   = FFALSE;                     //!< Static parameter!
    targetCfg.ciInput = FAPI_BOARD_RSRC_CIIN_NONE;  //!< Static parameter!
    targetCfg.cas     = RSRC_CAS_NONE;              //!< Static parameter!
    targetCfg.ciMatch[RSRC_TS_A] = FFALSE;          //!< Static parameter!
    targetCfg.ciMatch[RSRC_TS_B] = FFALSE;          //!< Static parameter!

    switch ( pRsrc->hwCfg->mpencPos )
    {
        case RSRC_MPENCPOS_FE1:
            targetCfg.feIdx = 0;
            targetCfg.ts    = RSRC_TS_A;
            break;

        case RSRC_MPENCPOS_FE2:
            targetCfg.feIdx = 1;
            targetCfg.ts    = RSRC_TS_B;
            break;

        default:
        case RSRC_MPENCPOS_NONE:
            return SMARTGO_SYS_ERR_CONFIG_ERROR;
    }

    retVal = RSRC_MpencPathAvailable (targetCfg.feIdx, targetCfg.ts, &hw);
    if ( retVal != FD_OK ) return retVal;

    /* --------------------------------------------------------------------- */

    if ( pCfg != NULL ) *pCfg = targetCfg;

    return FD_OK;
}

FERR RSRC_SetMpencMonitor (const RSRC_CFG_S* pCfg)
{
    FERR retVal;

    RSRC_DebugPrintConfig("Set MPEG encoder", pCfg);

    retVal = RSRC_SetOccupationMpenc(pCfg, &(pRsrc->status.hw));
    if ( retVal != FD_OK ) return retVal;

    pRsrc->status.opStatus        |= pCfg->op;
    pRsrc->status.mpencMonitor.cfg = *pCfg;

    return FD_OK;
}

FERR RSRC_MpencMonitorStopped ()
{
    RSRC_CFG_S* pCfg = &(pRsrc->status.mpencMonitor.cfg);

    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] MPEG encoder stopped.\n");

    if ( (pRsrc->status.opStatus & RSRC_OP_MPENCMONITOR) == 0 )
    {
        return SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE;
    }

    pRsrc->status.opStatus                 &= (~RSRC_OP_MPENCMONITOR);
    pRsrc->status.hw.opStat                &= (~RSRC_OP_MPENCMONITOR);
    pRsrc->status.hw.tsd[pCfg->tsdIdx].use &= (~RSRC_STAT_TSD_MPENCMONI);
    pRsrc->status.hw.vd[0]                  =   RSRC_STAT_VD_UNUSED;
    pRsrc->status.hw.videoout               =   RSRC_STAT_VOUT_UNUSED;

    pRsrc->status.hw.mpenc.use             &= (~RSRC_STAT_MPENC_MONITOR);

    return FD_OK;
}

FSTATIC FERR RSRC_SetOccupationMpenc (const RSRC_CFG_S* pCfg,
                                      RSRC_STAT_HW_S* hw)
{
    DBG_Assert(pCfg->ts     < RSRC_TS_NUMS);
    DBG_Assert(pCfg->tsdIdx < RSRC_TSD_NUMS);

    #ifdef APPL_DEBUG_MODE

    /* resource management consistency checks */
    RSRC_CheckConsistencyTsInput (pCfg, hw);

    #endif /* APPL_DEBUG_MODE */

    hw->mpenc.use              |= RSRC_STAT_MPENC_MONITOR;

    hw->tsd[pCfg->tsdIdx].use  |= RSRC_STAT_TSD_MPENCMONI;
    hw->tsd[pCfg->tsdIdx].tsIn  = pCfg->ts;
    hw->vd[0]                   = RSRC_STAT_VD_USED;
    hw->videoout                = RSRC_STAT_VOUT_MPENC;

    hw->opStat                 |= pCfg->op;

    return FD_OK;
}

FSTATIC FERR RSRC_MpencPathAvailable (U8 feIdx, RSRC_TS_E ts,
                                      const RSRC_STAT_HW_S* hw)
{
    FERR    retVal = FD_OK;
    U8      tsdCnt;

    /* Ensure that frontend muxed with encoder is unused. */
    if ( (hw->fe[feIdx].use & RSRC_STAT_FE_ANY) != 0 )
    {
        retVal = SMARTGO_SYS_ERR_RSRC;
    }
    else
    {
        /* Check if TS input is used by checking usage and input
           of all TSDs. */
        for ( tsdCnt = 0; tsdCnt < RSRC_TSD_NUMS; tsdCnt++ )
        {
            if ( ((hw->tsd[tsdCnt].use & RSRC_STAT_TSD_ANY) != 0) &&
                  (hw->tsd[tsdCnt].tsIn == ts) )
            {
                /* TS input already occupied by a TSD. Ensure that CI path is
                   unused. */

                if ( (hw->ci[ts].use & RSRC_STAT_CI_ANY) != 0 )
                {
                    break;
                }
            }
        }

        if ( tsdCnt < RSRC_TSD_NUMS ) retVal = SMARTGO_SYS_ERR_RSRC;
    }

    return retVal;
}

#endif /* APPL_MPEG_ENCODER_ENABLED */

#if (DEBUG > 0)

FSTATIC void RSRC_DebugPrintConfig(const char_t* pStr, const RSRC_CFG_S * pCfg)
{
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] %s\n", pStr);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - servIdx         %i\n", pCfg->servIdx);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - pvrIdx          %i\n", pCfg->pvrIdx);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - feIdx           %i\n", pCfg->feIdx);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - ts              %i\n", pCfg->ts);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - tsdIdx          %i\n", pCfg->tsdIdx);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - useCi           %i\n", pCfg->useCi);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - ciInput         0x%02X\n", pCfg->ciInput);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - cas             %i\n", pCfg->cas);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - ciMatch[TS-A]   %i\n", pCfg->ciMatch[RSRC_TS_A]);
    FAPI_SYS_PRINT_DEBUG(4,"[RSRC] - ciMatch[TS-B]   %i\n", pCfg->ciMatch[RSRC_TS_B]);
}

#endif /* not defined DEBUG */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
