/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/scart.c
**
**              $Id: scart.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       SCART control functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
//#include <fapi/sys_driver.h>
#include <fapex/abst_types.h>
#include <fapex/abst_libboardifc.h>

#include "utils.h"
#include "scart.h"

FSTATIC SCART_MODE_E  currMode = SCART_MODE_OFF;
FSTATIC SYS_VIDEO_OUT currVideoOut = VIDEOOUT_UNKNOWN;
FSTATIC fbool_t       scartOutputSourceVcr = FFALSE;

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/



/*---------------------------------------------------------------------------*/
/* FAPI_BOARD_CONTROL_SCART is used for hardware independent control              */
/* input    => disabled                                                      */
/* output 0 => 4:3                                                           */
/* output 1 => 16:9                                                          */
/*---------------------------------------------------------------------------*/
void SCART_SetMode(SCART_MODE_E mode)
{
#ifndef _LINUX_FAPI
    switch (mode)
    {
        case SCART_MODE_4_3:
            (void)FAPI_BOARD_CONTROL_SCART(0, FAPI_BOARD_SCART_MODE_4_3, NULL);
            break;
        case SCART_MODE_16_9:
            (void)FAPI_BOARD_CONTROL_SCART(0, FAPI_BOARD_SCART_MODE_16_9, NULL);
            break;
        default:
            (void)FAPI_BOARD_CONTROL_SCART(0, FAPI_BOARD_SCART_MODE_OFF, NULL);
            break;
    }

    currMode = mode;
#endif
}

SCART_MODE_E SCART_GetMode(void)
{
    return currMode;
}

/*---------------------------------------------------------------------------*/
/* FAPI_BOARD_CONTROL_SCART is used to control SCART pin 16 (FBLK): FBAS or RGB   */
/* NOTE: On SmartMPEG Evaluation Board Rev.C (JP151 option) and              */
/*       Fujitsu STB is also used to reset the front-end (see av.c)!         */
/*---------------------------------------------------------------------------*/
void SCART_SetVideoOutput(SYS_VIDEO_OUT mode)
{
#ifndef _LINUX_FAPI

    /* Modify pin 16 only if scart output is not an external source */
    if(!scartOutputSourceVcr)
    {
        switch (mode)
        {
            case VIDEOOUT_RGB:
                (void)FAPI_BOARD_CONTROL_SCART(0, FAPI_BOARD_SCART_VIDEO_OUTPUT_RGB, NULL);
                break;

            //case VIDEOOUT_YCRCB:
            case VIDEOOUT_SVIDEO:
                (void)FAPI_BOARD_CONTROL_SCART(0, FAPI_BOARD_SCART_VIDEO_OUTPUT_CVBS, NULL);
                break;

            default:
            case VIDEOOUT_UNKNOWN:
                /* do nothing */
                break;
        }
    }

    currVideoOut = mode;
#endif
}

void SCART_SetOutputSource(fbool_t vcr)
{
#ifndef _LINUX_FAPI

    scartOutputSourceVcr = vcr;

    if(!vcr)
    {
        FAPI_SYS_PRINT_DEBUG(4,"[SCART] Set output source: DVB \n");

        SCART_SetVideoOutput(currVideoOut);     /* restore pin 16 */
        (void)FAPI_BOARD_CONTROL_SCART(0, FAPI_BOARD_SCART_TV_SOURCE_DVB, NULL);
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(4,"[SCART] Set output source: VCR \n");

        /* Control PIN 16 */
        (void)FAPI_BOARD_CONTROL_SCART(0, FAPI_BOARD_SCART_VIDEO_OUTPUT_CVBS, NULL);
        (void)FAPI_BOARD_CONTROL_SCART(0, FAPI_BOARD_SCART_TV_SOURCE_VCR, NULL);
    }
#endif
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
