/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/ttxvbi.c
**
**              $Id: ttxvbi.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Teletext VBI re-insertion Decoder.
**
** This file contains the implementation of the Teletext VBI . It
** initializes and configures the teletext decoder(s) including all buffers.
** Furthermore it provides a simple API to start/stop and run the teletext
** decoder.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
#include "fapex/abst_types.h"
#include "fapex/abst_libttx.h"
#include "fapex/abst_libtsd.h"

#include "ttxvbi.h"
//#include "fapex/sys_config.h"
#include "fapex/sys_config_mb86h61_devkit.h"


//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************
#define TTX_TSD_CHANNEL     6
#define DRV_ID_TTX_0        (FAPEXDEV_TTX_MINOR_FIRST)

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************
static int32_t fd_TSDx_TTX = -1;
static int32_t fd_TTXVBI   = -1;

//***************************************************************************
//***************************************************************************
//** External Functions Declaration
//***************************************************************************
//***************************************************************************


//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************
extern int32_t fd_TSD[];

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** Opens teletext re-insertion.
**
** This function opens the teletext handle.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t TTXVBI_Open(int32_t *tsdId, int32_t *decId)
{
    FAPEX_TSD_OpenSubIdParamStrT pidChParam;
    FAPEX_TSD_PesParamStrT       pesParam;
    FAPEX_TTX_OpenParamStrT      ttxOpenParam;
    int32_t  fd_TSDx, sub_id;
    int32_t  retVal;


#ifndef _LINUX_FAPI

    if( FAPEX_HCB_BLK_TTX == 0) {
        fd_TSDx = fd_TSD[0];
        sub_id  = FAPEXDEV_TSD0_SUB_MINOR_FIRST + TTX_TSD_CHANNEL;
    }
    else if (FAPEX_HCB_BLK_TTX == 1) {
        fd_TSDx = fd_TSD[1];
        sub_id  = FAPEXDEV_TSD1_SUB_MINOR_FIRST + TTX_TSD_CHANNEL;
    }
    else if (FAPEX_HCB_BLK_TTX == 2) {
        fd_TSDx = fd_TSD[2];
        sub_id  = FAPEXDEV_TSD2_SUB_MINOR_FIRST + TTX_TSD_CHANNEL;
    }
    else {
        fd_TSDx = fd_TSD[3];
        sub_id  = FAPEXDEV_TSD3_SUB_MINOR_FIRST + TTX_TSD_CHANNEL;
    }
#endif
    pidChParam.channelNo = TTX_TSD_CHANNEL;
    pidChParam.sub_id    = sub_id;

#ifndef _LINUX_FAPI
    retVal = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSDx,
                              FAPEX_IOCCOM_OPEN_SUBID,
                              (FAPEXIOCARG3_T)&pidChParam);
#endif
    if(retVal < 0) {
        return retVal;
    }
    *tsdId = fd_TSDx_TTX = pidChParam.sub_id;

    /* set Teletext PES filter */
    pesParam.pid   = 0x1FFF;  /* dummy */
    pesParam.flags = FAPEX_TSD_FLTFLAGS_RM_TS_HEADER |
                     FAPEX_TSD_FLTFLAGS_SWAPWR_SWAP | FAPEX_TSD_FLTFLAGS_SWAPRD_SWAP |
                     FAPEX_TSD_FLTFLAGS_SWAPMODE_4BYTE;
    pesParam.bufferSize       = FAPEX_SYS_MMAP_BM_TELETEXT_BYTES;
    pesParam.bufferThresholdH = pesParam.bufferSize;
    pesParam.bufferThresholdL = 0;
    pesParam.streamType       = FAPEX_TSD_STREAMTYPE_TELETEXT;
    retVal = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSDx_TTX,
                              FAPEX_IOCCOM_TSD_SET_PES_FILTER,
                              (FAPEXIOCARG3_T)&pesParam );
    if(retVal < 0) {
        return retVal;
    }

    ttxOpenParam.inputId = sub_id;

    fd_TTXVBI = FAPEX_TTX_Open( DRV_ID_TTX_0,
                                O_RDWR,
                                (FAPEXOPENARG3_T)&ttxOpenParam );
    *decId = fd_TTXVBI;
    if(fd_TTXVBI < 0) {
        return fd_TTXVBI;
    }
    else {
        return 0;
    }
}


/*!
*******************************************************************************
**
** Closes teletext re-insertion.
**
** This function closes the teletext handle.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t TTXVBI_Close(void)
{
    int32_t retVal = 0;

    if(fd_TTXVBI >= 0) {
        retVal = FAPEX_TTX_Close( fd_TTXVBI );
        if(retVal < 0) {
            return retVal;
        }
        fd_TTXVBI = -1;
    }

    if(fd_TSDx_TTX >= 0) {
        retVal = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSDx_TTX,
                                  FAPEX_IOCCOM_CLOSE_SUBID,
                                  (FAPEXIOCARG3_T)NULL );
        if(retVal < 0) {
        }
        fd_TSDx_TTX = -1;
    }
    return retVal;
}

/*!
*******************************************************************************
**
** Initialises teletext.
**
** This function initialises the teletext by calling the open function.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
void TTXVBI_Init(void)
{

}

/*!
*******************************************************************************
** Starts Teletext re-insertion.
**
** This function starts the teletext re-insertion process for the given PID.
**
** \param pid The teletext PID to decode.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t  TTXVBI_Start(uint32_t pid)
{
    FAPEX_TSD_PidParamStrT  pidParam;
    int32_t retVal;

    /* set TTX PES PID */
    pidParam.pid   = pid;
    pidParam.flags = 0;
    retVal = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSDx_TTX,
                              FAPEX_IOCCOM_TSD_SET_PID_VALUE,
                              (FAPEXIOCARG3_T)&pidParam );
    if(retVal < 0) {
        return retVal;
    }

    /* start TTX driver */
    retVal = FAPEX_TTX_Ioctl( (FAPEXIOCARG1_T)fd_TTXVBI,
                              FAPEX_IOCCOM_START,
                              (FAPEXIOCARG3_T)NULL );
    if(retVal < 0) {
        return retVal;
    }

    /* start TSD channel */
    retVal = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSDx_TTX,
                              FAPEX_IOCCOM_START,
                              (FAPEXIOCARG3_T)NULL );
    if(retVal < 0) {
    }
    return retVal;
}

/*!
*******************************************************************************
** Stops AUDIO Processor.
**
** This function stops the audio decoding process.
*******************************************************************************
*/
int32_t TTXVBI_Stop(void)
{
    int32_t retVal = 0;

    /* stop TTX Driver */
    if(fd_TTXVBI >= 0) {
        retVal = FAPEX_TTX_Ioctl( (FAPEXIOCARG1_T)fd_TTXVBI,
                                  FAPEX_IOCCOM_STOP,
                                  (FAPEXIOCARG3_T)NULL );
        if(retVal < 0) {
            return retVal;
        }
    }

    /* stop TSD channel */
    if(fd_TSDx_TTX >= 0) {
        retVal = FAPEX_TSD_Ioctl( (FAPEXIOCARG1_T)fd_TSDx_TTX,
                                  FAPEX_IOCCOM_STOP,
                                  (FAPEXIOCARG3_T)NULL );
        if(retVal < 0) {
        }
    }
    return retVal;
}


