/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/update.c
**
**              $Id: update.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info 715: function argument not referenced
**
*****************************************************************************
*/
/*lint -save -e715 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/drv_flash.h>
#include <fapi/sys_services.h>

#include "sys_api.h"
#include "av.h"
#include "event_queue.h"
#include "update.h"
#include "utils.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define UPDATE_AREA_0_START          0x40800000
#define UPDATE_AREA_0_SIZE           0x00800000
#define UPDATE_AREA_1_START          0x41000000
#define UPDATE_AREA_1_SIZE           0x00800000

#define UPDATE_WRITE_SIZE            256   //!< Num. of words written in 1 cycle
#define UPDATE_WRITEBYTES_PER_EVENT  2048  //!< Progress event frequency during
                                           //!< flash programming

#define UPDATE_NUM_MEMAREAS          2     //!< Number of memory areas used for
                                           //!< storing image data.

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    uint8_t*              pStart;
    uint32_t              size;

} UPDATE_MEMAREA_S;

typedef struct
{
    TSC_INST_S       tscInst;
    UPDATE_INFO_S    info;

    UPDATE_MEMAREA_S area[UPDATE_NUM_MEMAREAS];

    uint32_t         imgBufWatermark;  //!< Position after greatest byte index written
    uint32_t         targetAddr;       //!< in bytes relative to flash start
    uint32_t         targetSize;       //!< total image size in bytes

    uint16_t         tmpBuf[UPDATE_WRITE_SIZE];

} UPDATE_DATA_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static const uint32_t crcTable[256] = {
    0x00000000, 0x04c11db7, 0x09823b6e, 0x0d4326d9, 0x130476dc, 0x17c56b6b,
    0x1a864db2, 0x1e475005, 0x2608edb8, 0x22c9f00f, 0x2f8ad6d6, 0x2b4bcb61,
    0x350c9b64, 0x31cd86d3, 0x3c8ea00a, 0x384fbdbd, 0x4c11db70, 0x48d0c6c7,
    0x4593e01e, 0x4152fda9, 0x5f15adac, 0x5bd4b01b, 0x569796c2, 0x52568b75,
    0x6a1936c8, 0x6ed82b7f, 0x639b0da6, 0x675a1011, 0x791d4014, 0x7ddc5da3,
    0x709f7b7a, 0x745e66cd, 0x9823b6e0, 0x9ce2ab57, 0x91a18d8e, 0x95609039,
    0x8b27c03c, 0x8fe6dd8b, 0x82a5fb52, 0x8664e6e5, 0xbe2b5b58, 0xbaea46ef,
    0xb7a96036, 0xb3687d81, 0xad2f2d84, 0xa9ee3033, 0xa4ad16ea, 0xa06c0b5d,
    0xd4326d90, 0xd0f37027, 0xddb056fe, 0xd9714b49, 0xc7361b4c, 0xc3f706fb,
    0xceb42022, 0xca753d95, 0xf23a8028, 0xf6fb9d9f, 0xfbb8bb46, 0xff79a6f1,
    0xe13ef6f4, 0xe5ffeb43, 0xe8bccd9a, 0xec7dd02d, 0x34867077, 0x30476dc0,
    0x3d044b19, 0x39c556ae, 0x278206ab, 0x23431b1c, 0x2e003dc5, 0x2ac12072,
    0x128e9dcf, 0x164f8078, 0x1b0ca6a1, 0x1fcdbb16, 0x018aeb13, 0x054bf6a4,
    0x0808d07d, 0x0cc9cdca, 0x7897ab07, 0x7c56b6b0, 0x71159069, 0x75d48dde,
    0x6b93dddb, 0x6f52c06c, 0x6211e6b5, 0x66d0fb02, 0x5e9f46bf, 0x5a5e5b08,
    0x571d7dd1, 0x53dc6066, 0x4d9b3063, 0x495a2dd4, 0x44190b0d, 0x40d816ba,
    0xaca5c697, 0xa864db20, 0xa527fdf9, 0xa1e6e04e, 0xbfa1b04b, 0xbb60adfc,
    0xb6238b25, 0xb2e29692, 0x8aad2b2f, 0x8e6c3698, 0x832f1041, 0x87ee0df6,
    0x99a95df3, 0x9d684044, 0x902b669d, 0x94ea7b2a, 0xe0b41de7, 0xe4750050,
    0xe9362689, 0xedf73b3e, 0xf3b06b3b, 0xf771768c, 0xfa325055, 0xfef34de2,
    0xc6bcf05f, 0xc27dede8, 0xcf3ecb31, 0xcbffd686, 0xd5b88683, 0xd1799b34,
    0xdc3abded, 0xd8fba05a, 0x690ce0ee, 0x6dcdfd59, 0x608edb80, 0x644fc637,
    0x7a089632, 0x7ec98b85, 0x738aad5c, 0x774bb0eb, 0x4f040d56, 0x4bc510e1,
    0x46863638, 0x42472b8f, 0x5c007b8a, 0x58c1663d, 0x558240e4, 0x51435d53,
    0x251d3b9e, 0x21dc2629, 0x2c9f00f0, 0x285e1d47, 0x36194d42, 0x32d850f5,
    0x3f9b762c, 0x3b5a6b9b, 0x0315d626, 0x07d4cb91, 0x0a97ed48, 0x0e56f0ff,
    0x1011a0fa, 0x14d0bd4d, 0x19939b94, 0x1d528623, 0xf12f560e, 0xf5ee4bb9,
    0xf8ad6d60, 0xfc6c70d7, 0xe22b20d2, 0xe6ea3d65, 0xeba91bbc, 0xef68060b,
    0xd727bbb6, 0xd3e6a601, 0xdea580d8, 0xda649d6f, 0xc423cd6a, 0xc0e2d0dd,
    0xcda1f604, 0xc960ebb3, 0xbd3e8d7e, 0xb9ff90c9, 0xb4bcb610, 0xb07daba7,
    0xae3afba2, 0xaafbe615, 0xa7b8c0cc, 0xa379dd7b, 0x9b3660c6, 0x9ff77d71,
    0x92b45ba8, 0x9675461f, 0x8832161a, 0x8cf30bad, 0x81b02d74, 0x857130c3,
    0x5d8a9099, 0x594b8d2e, 0x5408abf7, 0x50c9b640, 0x4e8ee645, 0x4a4ffbf2,
    0x470cdd2b, 0x43cdc09c, 0x7b827d21, 0x7f436096, 0x7200464f, 0x76c15bf8,
    0x68860bfd, 0x6c47164a, 0x61043093, 0x65c52d24, 0x119b4be9, 0x155a565e,
    0x18197087, 0x1cd86d30, 0x029f3d35, 0x065e2082, 0x0b1d065b, 0x0fdc1bec,
    0x3793a651, 0x3352bbe6, 0x3e119d3f, 0x3ad08088, 0x2497d08d, 0x2056cd3a,
    0x2d15ebe3, 0x29d4f654, 0xc5a92679, 0xc1683bce, 0xcc2b1d17, 0xc8ea00a0,
    0xd6ad50a5, 0xd26c4d12, 0xdf2f6bcb, 0xdbee767c, 0xe3a1cbc1, 0xe760d676,
    0xea23f0af, 0xeee2ed18, 0xf0a5bd1d, 0xf464a0aa, 0xf9278673, 0xfde69bc4,
    0x89b8fd09, 0x8d79e0be, 0x803ac667, 0x84fbdbd0, 0x9abc8bd5, 0x9e7d9662,
    0x933eb0bb, 0x97ffad0c, 0xafb010b1, 0xab710d06, 0xa6322bdf, 0xa2f33668,
    0xbcb4666d, 0xb8757bda, 0xb5365d03, 0xb1f740b4
};

UPDATE_DATA_S* udDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void UPDATE_Thread      (void* opt);
FSTATIC void UPDATE_IntReset    (UPDATE_DATA_S* inst);

FSTATIC uint32_t  UPDATE_WriteToMem  (UPDATE_DATA_S* inst, uint32_t bufPos,
                                 const void* pData, uint32_t numBytes);
FSTATIC uint32_t  UPDATE_ReadFromMem (UPDATE_DATA_S* inst, uint32_t bufPos,
                                 void* pData, uint32_t numBytes);


int32_t UPDATE_Init ()
{
    int32_t retVal;
    uint8_t   cnt;

    udDat = UTILS_Malloc(sizeof(UPDATE_DATA_S));
    if ( udDat == NULL ) return FAPI_ERR_OUT_OF_MEMORY;

    memset(udDat, 0, sizeof(UPDATE_DATA_S));

    udDat->area[0].pStart = (uint8_t *)UPDATE_AREA_0_START;
    udDat->area[0].size   =            UPDATE_AREA_0_SIZE;

    udDat->area[1].pStart = (uint8_t *)UPDATE_AREA_1_START;
    udDat->area[1].size   =            UPDATE_AREA_1_SIZE;

    udDat->info.status     = UPDATE_STAT_INACTIVE;
    udDat->info.imgBufPos  = 0;

    for ( cnt = 0; cnt < UPDATE_NUM_MEMAREAS; cnt++ )
    {
        udDat->info.imgBufSize += udDat->area[cnt].size;
    }

    udDat->imgBufWatermark = 0;
    udDat->targetAddr      = 0;
    udDat->targetSize      = 0;

    retVal = TSC_Init(&(udDat->tscInst));
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    return SYS_ThreadCreate (UPDATE_Thread, NULL,
                             THREAD_PRIO_UPDATE,
                             THREAD_STACKSZ_UPDATE,
                             THREAD_NAME_UPDATE,
                             NULL);
}

int32_t UPDATE_Open ()
{
    int32_t retVal;

    if ( udDat->info.status != UPDATE_STAT_INACTIVE )
    {
        return FAPI_ERR_ALREADY_OPEN;
    }

    retVal = RSRC_UpdateAvl(RSRC_OP_ANY);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = RSRC_SetUpdate();
    if ( retVal != FAPI_OK ) return retVal;

    UPDATE_IntReset (udDat);

    return FAPI_OK;
}

int32_t UPDATE_Close ()
{
    switch ( udDat->info.status )
    {
        case UPDATE_STAT_INACTIVE:
            return FAPI_OK;

        case UPDATE_STAT_PROGRAMMING:
            /* not possible while update is running */
            return FAPI_ERR_DEVICE_BUSY;

        default:
            break;
    }

    udDat->info.status = UPDATE_STAT_INACTIVE;

    (void)RSRC_UpdateStopped();

    return FAPI_OK;
}

int32_t UPDATE_GetInfo (UPDATE_INFO_S* pInfo)
{
    if ( udDat->info.status == UPDATE_STAT_INACTIVE )
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    *pInfo = udDat->info;

    return FAPI_OK;
}

int32_t UPDATE_AddData (const uint8_t* pData, uint32_t numBytes)
{
    //FAPI_SYS_PRINT_DEBUG(1,"[UPD]: ADD DATA: %i bytes\n", numBytes);

    if ( udDat->info.status != UPDATE_STAT_DATACOLLECT )
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    if ( (uint32_t)(udDat->info.imgBufPos + numBytes) > udDat->info.imgBufSize )
    {
        return SMARTGO_SYS_ERR_UPDATE_CAPACITY;
    }

    (void)UPDATE_WriteToMem(udDat, udDat->info.imgBufPos, pData, numBytes);

    udDat->info.imgBufPos += numBytes;

    if ( udDat->info.imgBufPos > udDat->imgBufWatermark )
    {
        udDat->imgBufWatermark = udDat->info.imgBufPos;
    }

    return FAPI_OK;
}

int32_t UPDATE_SetData (const uint8_t* pData, uint32_t bytePos, uint32_t numBytes)
{
    //FAPI_SYS_PRINT_DEBUG(1,"[UPD]: SET DATA: pos %i bytes %i\n", bytePos, numBytes);

    if ( udDat->info.status != UPDATE_STAT_DATACOLLECT )
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    if ( (uint32_t)(bytePos + numBytes) > udDat->info.imgBufSize )
    {
        return SMARTGO_SYS_ERR_UPDATE_CAPACITY;
    }

    udDat->info.imgBufPos = bytePos;

    (void)UPDATE_WriteToMem(udDat, udDat->info.imgBufPos, pData, numBytes);

    udDat->info.imgBufPos += numBytes;

    if ( udDat->info.imgBufPos > udDat->imgBufWatermark )
    {
        udDat->imgBufWatermark = udDat->info.imgBufPos;
    }

    return FAPI_OK;
}

int32_t UPDATE_ResetData ()
{
    int32_t retVal = FAPI_OK;

    switch ( udDat->info.status )
    {
        case UPDATE_STAT_INACTIVE:
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
            break;

        case UPDATE_STAT_PROGRAMMING:
            /* not possible while update is running */
            retVal = FAPI_ERR_DEVICE_BUSY;
            break;

        default:
            break;
    }

    if ( retVal == FAPI_OK )
    {
        UPDATE_IntReset (udDat);
    }

    return retVal;
}

/*
slotAddr: target address in bytes from flash start
*/
int32_t UPDATE_ProgramImage (uint32_t slotAddr)
{
    BOOTER_FlashSlotHeaderT  hdr;
    uint32_t            crc;
    uint32_t            crcStart; // start of CRC-checked area in bytes from image start

    if ( udDat->info.status != UPDATE_STAT_DATACOLLECT )
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    /* complete header should be available */
    if ( udDat->imgBufWatermark < sizeof(BOOTER_FlashSlotHeaderT) )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[UPD] Incomplete image header!\n");
        return SMARTGO_SYS_ERR_INVALID_DATA;
    }

    (void)UPDATE_ReadFromMem (udDat, 0, &hdr, sizeof(BOOTER_FlashSlotHeaderT));

    /* ----- Check data integrity ------------------------------------------ */

    if ( hdr.signature != BOOTER_SLOTHDR_SIG )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[UPD] Invalid image header signature: %08X \n",
                   hdr.signature);
        return SMARTGO_SYS_ERR_INVALID_DATA;
    }
    /*else if ( udDat->imgBufWatermark < hdr.slotSize )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[UPD] Incomplete image: %i bytes expected %i available \n",
                 hdr.slotSize, udDat->info.imgBufPos);
        return SMARTGO_SYS_ERR_INVALID_DATA;
    }*/
    else if ( hdr.slotSize > udDat->info.imgBufSize )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[UPD] Invalid image length: %i bytes (buffer: %i bytes)\n",
                 hdr.slotSize, udDat->info.imgBufSize);
        return SMARTGO_SYS_ERR_INVALID_DATA;
    }

    /* Mark header's status field as 'invalid'. Will be updated after image
       has been successfully written to Flash. */
    hdr.status |= 0x00000001;

    (void)UPDATE_WriteToMem (udDat, 0, &hdr, sizeof(BOOTER_FlashSlotHeaderT));

    /* fill up with zeros for 32bit-based CRC calculcation */
    //UPDATE_WriteToMem (udDat, hdr.slotSize, NULL,
      //                 udDat->info.imgBufSize - hdr.slotSize);

    /* Calculate CRC */
    crcStart = (((uint32_t)(&(hdr.crc32)) + sizeof(hdr.crc32)) - (uint32_t)(&hdr));

    /* NOTE: crcStart is assumed to be aligned on four bytes. */

    crc = UPDATE_CalcCrc (crcStart, (hdr.slotSize*4) - crcStart);

    if ( crc != hdr.crc32 )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[UPD] CRC check on image failed!\n");
        return SMARTGO_SYS_ERR_UPDATE_CRC;
    }

    /* always write a multiple of 4 bytes */
    udDat->targetSize = hdr.slotSize*4;

    FAPI_SYS_PRINT_DEBUG(1,"[UPD]: Image size: %i words, size in flash: %i bytes\n",
               hdr.slotSize, udDat->targetSize);

    /* ----- Test target image's position and size in Flash ---------------- */

    if ( (slotAddr + udDat->targetSize) > (uint32_t)sysData->flashSize )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[UPD] Target image exceeds Flash size!\n");
        return SMARTGO_SYS_ERR_UPDATE_CAPACITY;
    }

    /* --------------------------------------------------------------------- */

    udDat->targetAddr  = slotAddr;
    udDat->info.status = UPDATE_STAT_PROGRAMMING;

    /* trigger wakeup of sleeping thread */
    (void)TSC_WakeupThread(&(udDat->tscInst));

    return FAPI_OK;
}

/*
slotAddr: target address in bytes from flash start
*/
int32_t UPDATE_InvalidateImage (uint32_t slotAddr)
{
    FAPI_SYS_HandleT  flHdl;
    FAPI_FLASH_OpenParamsT flParams =
        { FAPI_FLASH_VERSION, 0, 0, 0, 0, 1 };
    uint16_t          null= 0;
    int32_t           retVal;

    flParams.deviceType  = sysData->flashDeviceType;
    flHdl = FAPI_FLASH_Open(&flParams, &retVal);

    if ( retVal == FAPI_OK )
    {
        retVal = FAPI_FLASH_WriteBytes(flHdl, slotAddr, 2, &null);

        (void)FAPI_FLASH_Close(flHdl);
    }

    return retVal;
}

int32_t UPDATE_Abort ()
{
    if ( udDat->info.status == UPDATE_STAT_INACTIVE )
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }
    else
    {
        return TSC_SendThreadToSleep(&(udDat->tscInst));
    }
}

FSTATIC void UPDATE_Thread (void* opt)
{
    SYS_EVT_UPDATE_S         ctrlEvent;
    BOOTER_FlashSlotHeaderT  hdr;
    int32_t                  retVal;
    FAPI_SYS_HandleT         flHdl       = 0;
    FAPI_FLASH_OpenParamsT   flParams =
        { FAPI_FLASH_VERSION, 0, 0, 0, 0, 1 };
    uint16_t                 sectorFirst = 0;
    uint16_t                 sectorNums  = 0;
    uint16_t                 sectorIdx   = 0;
    uint32_t                 sectorAddr;
    uint32_t                 writePos;  // number of words already written
    uint32_t                 bytesToWrite;
    uint16_t                 memData;
    uint16_t                 flashData;
    uint16_t                 writeEvents;

    memset(&ctrlEvent, 0, sizeof(SYS_EVT_UPDATE_S));

    ctrlEvent.hdr.type   = EVT_UPDATE_STATUS;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);

    for(;;)
    {
        retVal = FAPI_OK;

        /* sleep until wakeup call */

        //FAPI_SYS_PRINT_DEBUG(1,"[UPD] Sleep... \n");

        TSC_WaitForWakeup(&(udDat->tscInst));

        //FAPI_SYS_PRINT_DEBUG(1,"[UPD]: wake up...\n");

        do
        {
            /* Initialise progress counters */
            ctrlEvent.status      = UPDATE_STAT_PROGRAMMING;
            ctrlEvent.programStat = UPDATE_PROGRAM_ERASE;
            ctrlEvent.progCurr    = 0;
            ctrlEvent.progMax     = 0;

            flParams.deviceType  = sysData->flashDeviceType;
            flHdl = FAPI_FLASH_Open(&flParams, &retVal);
            if ( retVal != FAPI_OK ) break;

            /* ----- Erase target sectors ---------------------------------- */

            FAPI_SYS_PRINT_DEBUG(1,"[UPD]: Erase sector(s)...\n");

            sectorNums = 0;

            /* get number of first sector */
            sectorFirst = (uint16_t)FAPI_FLASH_GetSectorIndex(flHdl, udDat->targetAddr);

            /* get number of sectors to be erased */
            do
            {
                sectorNums++;

                sectorAddr = (uint32_t)FAPI_FLASH_GetSectorStart(flHdl, sectorFirst + sectorNums);

                if ( udDat->tscInst.sleepReq ) break;
            }
            while ( sectorAddr < (udDat->targetAddr + udDat->targetSize) );

            if ( udDat->tscInst.sleepReq ) break;

            ctrlEvent.progMax = sectorNums;

            /* send initial status event */
            (void)EVT_Send(&ctrlEvent, "Update");

            /* erase sectors */
            for ( sectorIdx = 0; sectorIdx < sectorNums; sectorIdx++ )
            {
                /* Erase sector */
                FAPI_SYS_PRINT_DEBUG(1,"[UPD]: Erasing sector %i ...\n",
                           sectorFirst + sectorIdx);

                retVal = FAPI_FLASH_EraseSector(flHdl, sectorFirst + sectorIdx);

                if ( retVal != FAPI_OK ) break;

                if ( udDat->tscInst.sleepReq ) break;

                ctrlEvent.progCurr    = sectorIdx + 1;

                (void)EVT_Send(&ctrlEvent,"Update");
            }

            if ( (retVal != FAPI_OK) || udDat->tscInst.sleepReq ) break;

            /* ----- Write data -------------------------------------------- */

            FAPI_SYS_PRINT_DEBUG(1,"[UPD]: Write data: %i bytes...\n", udDat->targetSize);

            ctrlEvent.programStat = UPDATE_PROGRAM_WRITE;
            ctrlEvent.progCurr    = 0;
            ctrlEvent.progMax     = udDat->targetSize;

            (void)EVT_Send(&ctrlEvent,"Update");

            writePos    = 0;
            writeEvents = 0;

            while ( writePos < udDat->targetSize )
            {
                bytesToWrite = udDat->targetSize - writePos;

                if ( bytesToWrite > UPDATE_WRITE_SIZE )
                {
                    bytesToWrite = UPDATE_WRITE_SIZE;
                }

                (void)UPDATE_ReadFromMem (udDat, writePos, udDat->tmpBuf,
                                    bytesToWrite);

                //FAPI_SYS_PRINT_DEBUG(1,"[UPD]: Writing: addr %i bytes %i\n",
                //           udDat->targetAddr + writePos, bytesToWrite);

                retVal = FAPI_FLASH_WriteBytes (flHdl,
                                      udDat->targetAddr + writePos,
                                      bytesToWrite, udDat->tmpBuf);

                if ( retVal != FAPI_OK ) break;

                if ( udDat->tscInst.sleepReq ) break;

                writePos += bytesToWrite;

                ctrlEvent.progCurr = writePos;

                if ( (ctrlEvent.progCurr / UPDATE_WRITEBYTES_PER_EVENT) >
                     writeEvents )
                {
                    writeEvents = ctrlEvent.progCurr /
                                  UPDATE_WRITEBYTES_PER_EVENT;

                    (void)EVT_Send(&ctrlEvent,"Update");
                }
            }

            if ( (retVal!= FAPI_OK) || udDat->tscInst.sleepReq ) break;

            ctrlEvent.progCurr    = udDat->targetSize;

            /* send final event */
            (void)EVT_Send(&ctrlEvent,"Update");

            /* ----- Verify data ------------------------------------------- */

            FAPI_SYS_PRINT_DEBUG(1,"[UPD]: Verify data...\n");

            ctrlEvent.programStat = UPDATE_PROGRAM_VERIFY;
            ctrlEvent.progCurr    = 0;
            ctrlEvent.progMax     = udDat->targetSize;

            writePos  = 0;

            (void)EVT_Send(&ctrlEvent,"Update");

            while ( writePos < udDat->targetSize )
            {
                retVal = FAPI_FLASH_ReadBytes (flHdl, udDat->targetAddr + writePos,
                                     2, &flashData);

                if ( retVal != FAPI_OK )
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[UPD]: FAPI_FLASH_Read() failed at word %i\n",
                                writePos);
                    break;
                }

                if ( UPDATE_ReadFromMem (udDat, writePos, &memData, 2)
                     != 2 )
                {
                    retVal = SMARTGO_SYS_ERR_INTERNAL;
                    break;
                }

                if ( memData != flashData )
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[UPD]: Verification failed at word %i\n",
                                writePos);
                    retVal = SMARTGO_SYS_ERR_INTERNAL;
                    break;
                }

                if ( udDat->tscInst.sleepReq ) break;

                writePos+=2;

                ctrlEvent.progCurr = writePos;

                // TODO: send further write notification events?
            }

            if ( (retVal != FAPI_OK) || udDat->tscInst.sleepReq ) break;

            ctrlEvent.progCurr    = udDat->targetSize;

            (void)EVT_Send(&ctrlEvent,"Update");

            /* ----- Set "valid"-status in boot header --------------------- */

            (void)UPDATE_ReadFromMem (udDat, 0, &hdr, sizeof(BOOTER_FlashSlotHeaderT));

            hdr.status &= 0xFFFFFFFE;

            /* overwrite image header */
            retVal = FAPI_FLASH_WriteBytes (flHdl, udDat->targetAddr,
                                            sizeof(BOOTER_FlashSlotHeaderT), &hdr );

            if ( retVal != FAPI_OK ) break;
        }
        while ( 0 );

        if ( flHdl != 0 )
        {
            (void)FAPI_FLASH_Close(flHdl);
        }

        /* ----- Finished, set status -------------------------------------- */

        if ( udDat->tscInst.sleepReq )
        {
            FAPI_SYS_PRINT_DEBUG(1,"[UPD] Update aborted!\n");
            udDat->info.status = UPDATE_STAT_ABORTED;
        }
        else if ( retVal == FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(1,"[UPD] Update finished successfully!\n");
            udDat->info.status = UPDATE_STAT_FINISHED;

            ctrlEvent.programStat = UPDATE_PROGRAM_NONE;
            ctrlEvent.progCurr    = 0;
            ctrlEvent.progMax     = 0;
        }
        else
        {
            // TODO: store error code somewhere?

            FAPI_SYS_PRINT_DEBUG(2,"[UPD] Update stopped with error: %08X!\n", retVal);
            udDat->info.status = UPDATE_STAT_ERROR;

            ctrlEvent.programStat = UPDATE_PROGRAM_NONE;
            ctrlEvent.progCurr    = 0;
            ctrlEvent.progMax     = 0;
        }

        /* Send final notification event */
        ctrlEvent.status = udDat->info.status;

        (void)EVT_Send(&ctrlEvent,"Update");
    }
}

uint32_t UPDATE_CalcCrc (uint32_t bufPos, uint32_t sizeInBytes)
{
    uint32_t cnt, crc;
    uint8_t  data[4];

    crc = 0xFFFFFFFF;

    for ( cnt = 0; cnt < sizeInBytes; cnt += 4 )
    {
        (void)UPDATE_ReadFromMem (udDat, bufPos, data, 4);

        crc = (crc << 8) ^ crcTable[((crc >> 24) ^ data[3]) & 0xFF];
        crc = (crc << 8) ^ crcTable[((crc >> 24) ^ data[2]) & 0xFF];
        crc = (crc << 8) ^ crcTable[((crc >> 24) ^ data[1]) & 0xFF];
        crc = (crc << 8) ^ crcTable[((crc >> 24) ^ data[0]) & 0xFF];

        bufPos += 4;
    }

    return crc;
}

/* ------------------------------------------------------------------------- */

FSTATIC void UPDATE_IntReset (UPDATE_DATA_S* inst)
{
    uint8_t areaCnt;

    inst->info.status     = UPDATE_STAT_DATACOLLECT;
    inst->info.imgBufPos  = 0;

    inst->imgBufWatermark = 0;

    for ( areaCnt = 0; areaCnt < UPDATE_NUM_MEMAREAS; areaCnt++ )
    {
        memset(inst->area[areaCnt].pStart, 0, inst->area[areaCnt].size);
    }
}

/*
if pData is NULL, memory area will be zero'ed.
*/
FSTATIC uint32_t UPDATE_WriteToMem (UPDATE_DATA_S* inst, uint32_t bufPos,
                               const void* pData, uint32_t numBytes)
{
    uint8_t  areaCnt;
    uint32_t areaStartPos;
    uint32_t bytesToCopy;
    uint32_t bytesCopied = 0;
    uint8_t* pSrc        = (uint8_t*)pData;

    while ( numBytes > 0 )
    {
        areaStartPos = 0;

        for ( areaCnt = 0; areaCnt < UPDATE_NUM_MEMAREAS; areaCnt++ )
        {
            if ( (bufPos >= areaStartPos) &&
                 (bufPos <  (areaStartPos + inst->area[areaCnt].size)) )
            {
                bytesToCopy = (areaStartPos + inst->area[areaCnt].size) - bufPos;

                if ( numBytes < bytesToCopy )
                {
                    bytesToCopy = numBytes;
                }

                if ( pSrc != NULL )
                {
                    memcpy(inst->area[areaCnt].pStart+(bufPos-areaStartPos),
                           pSrc, bytesToCopy);
                    pSrc += bytesToCopy;
                }
                else
                {
                    memset (inst->area[areaCnt].pStart +
                                   (bufPos - areaStartPos),
                                   0, bytesToCopy);
                }

                bufPos      += bytesToCopy;
                numBytes    -= bytesToCopy;
                bytesCopied += bytesToCopy;
                break;
            }

            areaStartPos += inst->area[areaCnt].size;
        }

        if ( areaCnt >= UPDATE_NUM_MEMAREAS ) break;
    }

    return bytesCopied;
}

FSTATIC uint32_t UPDATE_ReadFromMem (UPDATE_DATA_S* inst, uint32_t bufPos,
                                void* pData, uint32_t numBytes)
{
    uint8_t  areaCnt;
    uint32_t areaStartPos;
    uint32_t bytesToCopy;
    uint32_t bytesCopied = 0;
    uint8_t* pSrc        = (uint8_t*)pData;

    while ( numBytes > 0 )
    {
        areaStartPos = 0;

        for ( areaCnt = 0; areaCnt < UPDATE_NUM_MEMAREAS; areaCnt++ )
        {
            if ( (bufPos >= areaStartPos) &&
                 (bufPos <  (areaStartPos + inst->area[areaCnt].size)) )
            {
                bytesToCopy = (areaStartPos + inst->area[areaCnt].size) - bufPos;

                if ( numBytes < bytesToCopy )
                {
                    bytesToCopy = numBytes;
                }

                memcpy (pSrc, inst->area[areaCnt].pStart +
                       (bufPos - areaStartPos), bytesToCopy);

                pSrc         += bytesToCopy;
                bufPos       += bytesToCopy;
                numBytes     -= bytesToCopy;
                bytesCopied  += bytesToCopy;
                break;
            }

            areaStartPos += inst->area[areaCnt].size;
        }

        if ( areaCnt >= UPDATE_NUM_MEMAREAS ) break;
    }

    return bytesCopied;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
